/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  This file includes the Flashlite 2.0 Viewer settings stored in
*                the central repository.
*
*/


#ifndef FLASHLITE_2_0_PRIVATECRKEYS_H
#define FLASHLITE_2_0_PRIVATECRKEYS_H

/** Flashlite 2.0 Viewer settings */
//--------------------------------------------------------
//  Viewer related cenrep keys
//--------------------------------------------------------

const TUid KCRUIdFlashliteViewer = { 0x1027506E };

/**
* Initial Static Heap Size for the Flashlite player.
**/
const TUint32 KFlashliteVwrStaticHeapSize = 0x00000001;

/**
* Maximum Dynamic Heap Size for the Flashlite player
**/
const TUint32 KFlashliteVwrDynamicHeapMaxSize = 0x00000002;

/**
* Default IAP ID for access in Flashlite.
**/
const TUint32 KFlashliteVwrIAPId = 0x00000003;

/**
* Engine mode is a bitmap of flags to enable / disable various 
* features like sound, svg, video, etc.
**/
const TUint32 KFlashliteVwrEngineMode = 0x00000004;

/**
* Enable or Disable Debug in Viewer
**/
const TUint32 KFlashliteVwrDebugMode = 0x00000005;

/**
* Current Volume for Flashlite viewer
**/
const TUint32 KFlashliteVwrVolume = 0x00000006;

/**
* Max dynamic heap size for stub
**/
const TUint32 KFlashliteStubMaxHeapSise = 0x00000007;


/**
* User agent string for Flash lite viewer to form HTTP GET/POSt packets
**/
const TUint32 KFlashliteVwrUserAgent = 0x00000008;

//-------------------------------------------------------
//   Browser plugin related cenrep keys
//-------------------------------------------------------

/** Flashlite Browser Plugin Settings */
const TUid KCRUIdFlashliteBrowserPlugin = {0x1027506F};

/**
* Initial Static Heap Size for Flashlite player
*/
const TUint32 KFlashliteBrplginStaticHeapSize = 0x00000001;

/**
* Max Dynamic Heap Size for Flashlite player
*/
const TUint32 KFlashliteBrplginDynamicHeapMaxSize = 0x00000002;

/**
* Engine mode is a bitmap of flags to enable / disable various
* features like sound, svg, video, etc
*/
const TUint32 KFlashliteBrplginEngineMode = 0x00000003;

/**
* Total Heap size for sum of all flash instances
*/

const TUint32 KFlashliteBrplginTotalHeapSize = 0x00000004;

/**
* Max heap size for a single flash instance
*/

const TUint32 KFlashliteBrplginPerInstanceMaxHeapSize = 0x00000005;

//-------------------------------------------------------------------
//       Screensaver configurations
//-------------------------------------------------------------------

/** Flashlite Screensaver Settings */
const TUid KCRUIdFlashliteScreensaver = {0x10281F17};

/**
* Initial Static Heap Size for Flashlite player
*/
const TUint32 KFlashliteScreensaverStaticHeapSize = 0x00000001;

/**
* Max Dynamic Heap Size for Flashlite player
*/
const TUint32 KFlashliteScreensaverDynamicHeapMaxSize = 0x00000002;

/**
* Engine mode is a bitmap of flags to enable / disable various
* features like sound, svg, video, etc
*/
const TUint32 KFlashliteScreensaverEngineMode = 0x00000003;

//------------------------------------------------------------------------------
//   Host engine related cenrep keys, common for both viewer and browser plugin
//------------------------------------------------------------------------------

/** Flashlite Platform Engine Settings */
const TUid KCRUIdFlashlitePlatformEngine = {0x2000B486};

//-------------------------------------------------------------------
// MMC Related cenrep keys, common for both viewer and plugin
//-------------------------------------------------------------------

/**
* Default value for Maximum allowed usage of memory card for 
* storing flv files
*/

const TUint32 KFlashLiteMMCResHighWatermark = 0x00000001;

/**
* Default value for minimum memory that will be used on 
* memory card for storing flv files
*/

const TUint32 KFlashLiteMMCResLowWatermark = 0x00000002;

/**
* The MMC memory management policy
* storing flv files
*/

const TUint32 KFlashliteMMCResUsagePolicy = 0x00000003;


/**
* The min Res size in RAM that will be used
* in presence of MMC
*/

const TUint32 KFlashliteRAMResLowWatermark = 0x00000004;

/**
* The max Res size in RAM that will be used
* in presence of MMC
*/

const TUint32 KFlashliteRAMResHighWatermark = 0x00000005;

/**
* Reserved unutilised space in MMC
*/

const TUint32 KFlashliteMMCResvUnUtilisedSpace = 0x00000006;

//------------------------------------------------------------------------------------
// Other common cenrep keys - for both viewer and browser plugin, part of hostengine
//------------------------------------------------------------------------------------

/**
*Minimum space that should be ensured to be left in RAM beyond Flash's usage. 
*For ex: If this value is 4MB and if the system has 7 MB left, then Flash will not allocate beyond 3MB.
*Units: MB. This is specific to FL 3.1.
*/

const TUint32 KFlashLiteRAMResUnUtilisedSpace = 0x00000007;

/**
* Maximum dynamic heap size for flv video.
* Units: MB. This is specific to FL 3.1.
*/

const TUint32 KFlashLiteVideoDynHeapSize = 0x00000008;

/**
* Maximum number of frames to be decoded in a platform timer tick for progressive flv. 
* This is specific to FL 3.1.
*/

const TUint32 KFLVMaxFramesToDecodePerInterval = 0x00000009;

/**
User configurable Res drive for FLV video where multidrive support is present. 
*Default is E:/ . This is specific to FL 3.1. 
*/

const TUint32 KFlashliteVideoResDrive = 0x0000000A;


/**
override bit which is used to select the drive with the maximum freespace if multiple drive support exists. 
*/

const TUint32 KFlashliteMMCDriveOverridePolicy =0x0000000B;

#endif  // FLASHLITE_2_0_PRIVATECRKEYS_H
