/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class used to store extensions to the main 
				 PlatformFlashEngine functionality.
*
*/


#ifndef __MMI_CUSTOM_H__
#define __MMI_CUSTOM_H__

#include <ecom/ecomresolverparams.h>
#include <ecom/ecom.h>

struct KeyEventInfo
{
	TBuf<40> keyBuffer;
	TBool downAndUpOnly;
};

class MMIUtils;

/**
 * The CFlashExtension class is used to store extensions to the main PlatformFlashEngine functionality.
 * It is used to provide MMI functionality.
 */
class CFlashMMICustomization : public CBase
{
public:
	static CFlashMMICustomization* NewL();
	virtual ~CFlashMMICustomization() { REComSession::DestroyedImplementation (iDtor_ID_Key);	}

	virtual TBool HandleKeyEvent(const TKeyEvent& aKeyEvent, KeyEventInfo* aKeyEventInfo) = 0;
	virtual TBool HandleLaunchL(RPointerArray<HBufC>& aArray) = 0;
	virtual TBool RegisterExtendedEventL(const TDesC& aEventName) = 0;
	virtual TBool UnregisterExtendedEvent(const TDesC& aEventName) = 0;
	virtual void UnregisterExtendedEvents() = 0;
	virtual void SetMMIUtils(MMIUtils* aUtils) = 0;
	virtual void NotifyIncomingCall(TBool aStatus) =0;

public:
	TUid ExtensionUid() { return iExtensionUid;}

private:
	TUid iDtor_ID_Key;
protected:
    TUid iExtensionUid;
};

#endif // __MMI_CUSTOM_H__
