/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class which is an 
				 extension to the main PlatformFlashEngine functionality.
*
*/


#include <e32keys.h>
#include <e32std.h>
#include <w32std.h>
#include <ecom/implementationproxy.h>
#include "mmicustom.h"
#include "mmicustomimpl.h"
#include "mmieventhndlrincomingcall.h"

const TImplementationProxy ImplementationTable[] =
{
	{{0x1027367e}, CFlashMMICustomizationImpl::NewL }
};

EXPORT_C const TImplementationProxy* ImplementationGroupProxy(TInt& aTableCount)
{
	aTableCount = sizeof(ImplementationTable) / sizeof(TImplementationProxy);
	return ImplementationTable;
}

TAny* CFlashMMICustomizationImpl::NewL()
{
	CFlashMMICustomizationImpl* object = new(ELeave)CFlashMMICustomizationImpl();
	return object;
}

CFlashMMICustomizationImpl::CFlashMMICustomizationImpl()
{
    iExtensionUid = TUid::Uid(0x1027367e);
}

CFlashMMICustomizationImpl::~CFlashMMICustomizationImpl()
{
    if(iExtEventListener)
    {
        iExtEventListener->Destroy();
        iExtEventListener = NULL;
    }
}

void CFlashMMICustomizationImpl::SetMMIUtils(MMIUtils* aUtils)
{
	iMMIUtils=aUtils;
}

_LIT(KLitExtendedKeySend,"send");
_LIT(KLitExtendedKeyEndCall,"endcall");
_LIT(KLitExtendedKeyEdit,"edit");
// Commenting it out as it is no longer used 
// anywhere in the code. See HanldKeyEvent function
// for more info
//_LIT(KLitExtendedKeyClear,"clear");
_LIT(KLitExtendedKeySoft1,"soft1");
_LIT(KLitExtendedKeySoft2,"soft2");


TBool CFlashMMICustomizationImpl::HandleKeyEvent(const TKeyEvent& aKeyEvent, KeyEventInfo* aKeyEventInfo)
{

	TBool consumed = EFalse;

	switch (aKeyEvent.iScanCode)
	{

	case EStdKeyYes: // call
		aKeyEventInfo->keyBuffer.Copy(KLitExtendedKeySend);
		consumed = ETrue;
		break;

	case EStdKeyNo: // hang up
		aKeyEventInfo->keyBuffer.Copy(KLitExtendedKeyEndCall);
		consumed = ETrue;
		break;

	case EStdKeyDevice0: // left soft
		aKeyEventInfo->keyBuffer.Copy(KLitExtendedKeySoft1);
        aKeyEventInfo->downAndUpOnly=ETrue;
		consumed = ETrue;
		break;

	case EStdKeyDevice1: // right soft
		aKeyEventInfo->keyBuffer.Copy(KLitExtendedKeySoft2);
        aKeyEventInfo->downAndUpOnly=ETrue;
		consumed = ETrue;
		break;


#ifdef __WINS__
	case EStdKeyRightShift: // edit key
		aKeyEventInfo->keyBuffer.Copy(KLitExtendedKeyEdit);
        aKeyEventInfo->downAndUpOnly=ETrue;
		consumed = ETrue;
		break;
#else
	case EStdKeyLeftShift: // edit key
		aKeyEventInfo->keyBuffer.Copy(KLitExtendedKeyEdit);
        aKeyEventInfo->downAndUpOnly=ETrue;
		consumed = ETrue;
		break;
#endif
// Fix for ASHA-7QR9SC. ATS certification issue
// As adobe is not handling BackSpace as extended key in MMI so
// commenting out this piece of code as it is causing regression for
// one of the ATS content 001_key2b.swf
	/*case EStdKeyBackspace: // clear key
		aKeyEventInfo->keyBuffer.Copy(KLitExtendedKeyClear);
		consumed = ETrue;
		break;*/

	}

	return consumed;
}

_LIT(KLitVideoCall,"videocall");
_LIT(KLitVideoCallArg1, "4");
_LIT(KLitCalculator,"calculator");
_LIT(KLitMusicPlayer,"music");

#ifdef __WINS__
_LIT(KLitVideoCallPath,"\\epoc32\\RELEASE\\winscw\\UDEB\\videotelui.exe");
_LIT(KLitCalculatorPath,"\\epoc32\\RELEASE\\winscw\\UDEB\\calcsoft.exe");
_LIT(KLitMusicPlayerPath,"\\epoc32\\RELEASE\\winscw\\UDEB\\musicplayer.exe");
#else
_LIT(KLitVideoCallPath,"z:\\sys\\bin\\videotelui.exe");
_LIT(KLitCalculatorPath,"z:\\sys\\bin\\calcsoft.exe");
_LIT(KLitMusicPlayerPath,"z:\\sys\\bin\\musicplayer.exe");
#endif


TBool CFlashMMICustomizationImpl::HandleLaunchL(RPointerArray<HBufC>& aArray)
{
	TBool status = EFalse;

	if (aArray.Count() > 0)
	{
		HBufC* appName = aArray[0];

		if (appName->Compare(KLitVideoCall) == 0)
		{
			aArray.Remove(0);
			aArray.Insert(KLitVideoCallPath().AllocL(), 0);
			delete appName;
			if (aArray.Count() > 1)
			{
				HBufC* newArg = KLitVideoCallArg1().AllocL();
				aArray.Insert(newArg, 1);
			}
			status = ETrue;
		}
		else if (appName->Compare(KLitCalculator) == 0)
		{
			aArray.Remove(0);
			aArray.Insert(KLitCalculatorPath().AllocL(), 0);
			delete appName;
			status = ETrue;
		}
		else if (appName->Compare(KLitMusicPlayer) == 0)
		{
			aArray.Remove(0);
			aArray.Insert(KLitMusicPlayerPath().AllocL(), 0);
			delete appName;
			status = ETrue;
		}
	}

	return status;
}

_LIT(KIncomingCallString, "IncomingCall");





TBool CFlashMMICustomizationImpl::RegisterExtendedEventL(const TDesC& aEventName)
{
	TBool registered = EFalse;

	if(aEventName.Compare(KIncomingCallString) == 0)
	{
		AddIncomingCallListenerL();
		registered = ETrue;
	}
	return registered;
}

TBool CFlashMMICustomizationImpl::UnregisterExtendedEvent(const TDesC& aEventName)
{
	TBool unregistered = EFalse;

	if(aEventName.Compare(KIncomingCallString) == 0)
	{
		RemoveIncomingCallListener();
		unregistered = ETrue;
	}
	return unregistered;
}

void CFlashMMICustomizationImpl::UnregisterExtendedEvents()
{
	//eventually should call remove functions for all event listeners

	RemoveIncomingCallListener();
}


void CFlashMMICustomizationImpl::AddIncomingCallListenerL()
{
    // Eventually should be an array to close and clean up. For now, the only
    // working event listener is the incoming call listener, so no array is
    // needed.
    if(!iExtEventListener)
    {
        iExtEventListener = CMMIIncomingCallNotification::NewL(iMMIUtils);
    }
}

void CFlashMMICustomizationImpl::RemoveIncomingCallListener()
{
	// eventually should be an array to close and clean up
	if(iExtEventListener)
    {
		iExtEventListener->Destroy();
		iExtEventListener = NULL;
    }
}

void CFlashMMICustomizationImpl::NotifyIncomingCall(TBool aStatus)
{
	
	if(iExtEventListener)
    {
		iExtEventListener->NotifyIncomingCall(aStatus);
    }
}
