/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of Extended Event Handler 
				 that allows Flash MMI Movies to be notified of the incoming 
				 call events.
*
*/


#include <eikenv.h>

#include "mmieventhndlrincomingcall.h"
#include <mmiutils.h>

#include <flashmmi_resources.rsg>

_LIT(KEventIncomingCall, "IncomingCall");


//static
CMMIIncomingCallNotification* CMMIIncomingCallNotification::NewL(MMIUtils* aMMIUtils)
{
    if(!aMMIUtils)
    {
        User::Leave(KErrGeneral);
    }

    CMMIIncomingCallNotification* self = new(ELeave) CMMIIncomingCallNotification();
	CleanupStack::PushL(self);
	self->ConstructL(aMMIUtils);
	CleanupStack::Pop(self);

	return self;
}

CMMIIncomingCallNotification::~CMMIIncomingCallNotification()
{

}


CMMIIncomingCallNotification::CMMIIncomingCallNotification()
{
}

void CMMIIncomingCallNotification::ConstructL(MMIUtils* aMMIUtils)
{
    iMMIUtils = aMMIUtils;
}

void CMMIIncomingCallNotification::NotifyIncomingCall(TBool aStatus)
	{
	TRAP_IGNORE(NotifyIncomingCallL(aStatus));
	}

void CMMIIncomingCallNotification::NotifyIncomingCallL(TBool /*aStatus*/)
{

	// ideally we would retrieve the incoming phone number here
	// but this does not seem to be possible with the 2.2 SDK, and seems to be different
	// for each release of the OS, so to be safe we
	// will not do this in the reference implementation.

	
	CEikonEnv* env = iMMIUtils->GetEikonEnv();

    // These resource strings are zero terminated in the resource file.
	HBufC16* name = env->AllocReadResourceAsDes16L(R_STR_INCOMING_CALL_CALLER_UNKNOWN);
	name = name->ReAllocL(name->Length() + 1);
	CleanupStack::PushL(name);

    HBufC16* number = env->AllocReadResourceAsDes16L(R_STR_INCOMING_CALL_NUMBER_WITHHELD);
	number = number->ReAllocL(number->Length() + 1);
	CleanupStack::PushL(number);

    HBufC16* group = env->AllocReadResourceAsDes16L(R_STR_INCOMING_CALL_GROUP_UNKNOWN);
	group = group->ReAllocL(group->Length() + 1);
	CleanupStack::PushL(group);


	name->Des().ZeroTerminate();
	number->Des().ZeroTerminate();
	group->Des().ZeroTerminate();

	/*
	FI_Text arg0, arg1, arg2, arg3;

    arg0.textEncoding = FI_UTF16EncodingForm;
    arg0.text.utf16String = (&KEventIncomingCall)->Ptr();
    arg0.textLen = (&KEventIncomingCall)->Length();
    
    arg1.textEncoding = FI_UTF16EncodingForm;
    arg1.text.utf16String = name->Ptr();
    arg1.textLen = name->Length();

    arg2.textEncoding = FI_UTF16EncodingForm;
    arg2.text.utf16String = number->Ptr();
    arg2.textLen = number->Length();
    
    arg3.textEncoding = FI_UTF16EncodingForm;
    arg3.text.utf16String = group->Ptr();
    arg3.textLen = group->Length();
	
	FI_Text* args[4];

	args[0] = &arg0;
	args[1] = &arg1;
	args[2] = &arg2;
	args[3] = &arg3;

    if(iMMObj && iMMObj->pFIFncs->pFIMMIFncs->pFncFI_SendEvent)
	{
		iMMObj->pFIFncs->pFIMMIFncs->pFncFI_SendEvent(iMMObj, 4, (const struct FI_Text**)args);
	}
	*/

	RPointerArray<TDesC> args;
	User::LeaveIfError(args.Append(&KEventIncomingCall));
	User::LeaveIfError(args.Append(name));
	User::LeaveIfError(args.Append(number));
	User::LeaveIfError(args.Append(group));

	iMMIUtils->SendEvent(args);

	args.Close();

    CleanupStack::PopAndDestroy(3, name);

}

void CMMIIncomingCallNotification::Destroy()
{
	delete this;
}

//End of file


