/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the Phonebook service extension class
*
*/

#ifndef __MMI_PHONEBOOK_H__
#define __MMI_PHONEBOOK_H__

#include <e32std.h>
#include <ecom/implementationproxy.h>
#include <cntdb.h>
#include <cntitem.h>
#include <cntfldst.h>

#include <mmidelegatebase.h>
#include <mmiutils.h>

class CMMIPhonebookArrayWrapper;

// encapsulates a String opCommand
// and its logical pairing in SymbianOS Contacts
class TMMIPbkUidStringPair
{
public:
	TMMIPbkUidStringPair() : iUid(KNullUid),iString(NULL) {}
public:
	TUid iUid;
	const TDesC* iString;
};

class CPbDataRequestBase;

/**
* The CMMIPhonebook class encapsulates all MMI phonebook functionality
*/
class CMMIPhonebook : public CMMI_Delegate , public MObjectOwner
{
public:
	static TAny* NewL();
	~CMMIPhonebook();

public: // from MMI_Delegate
	const TDesC8* Name() const;
	void Initialize(MMIUtils* aMMIUtils);
	short GetL(const RPointerArray<TDesC>& aArgs, struct FI_ReturnString* retVal);
	short GetVarsL(const RPointerArray<TDesC>& aArgs, struct FI_ReturnString* retVal);
	short SetL(const RPointerArray<TDesC>& aArgs, const TDesC& aValue);	
	void SetVarsL(const RPointerArray<TDesC>& aArgs, const TDesC& aValue, short& aRetVal);
	short SendL(const TDesC& aVerbArg, const RPointerArray<TDesC>& aArgs);
	short RequestDataL(const RPointerArray<TDesC>& aArgs, const TAny* aDataRequestHandle);
	short CancelRequestL(const TAny* aDataRequestHandle);
	void PlayerDestroyed();
	TBool RegisterExtendedEvent(const TDesC& aEventName);
	TBool UnregisterExtendedEvent(const TDesC& aEventName);
	void* ExecuteFunctionL(TInt aOpCode, TAny* aParam1, TAny* aParam2, TAny* aParam3, TAny* aParam4);

public: // from MObjectOwner
	void DeleteObject(CBase* aObject);

private:
	CMMIPhonebook();
	void ConstructL();

private:
	void RebuildItemArrayL();
	TInt GetIndexFromCntId(TContactItemId aId);
	TContactItemId GetCntIdFromIndex(TInt aIndex);
	short FindOrCreateFieldAndSetValueL(CContactItemFieldSet& aFields, const TDesC& aCommand, const TDesC& aNewValue, const TDesC& aDefaultLabel);
	TInt FieldIndexForString(CContactItemFieldSet& aFields, const TDesC& aString, TInt& aIndexFound) const;
	CContactItemField* GetFieldByLabel(CContactItemFieldSet& aFields, TFieldType aType, const TDesC& aLabel) const;
	TInt GetTotalFieldsAndPosOfNthFieldOfType(const CContactItemFieldSet& aFields, TFieldType aType, TInt aNth, TInt& aHowManyFound) const;
	short TryAddSpeedDial(CContactItem& aItem, TInt aIndex);
	short DoTryAddSpeedDialL(CContactItem& aItem, TInt aIndex);
	TInt GetSpeedDialL(TContactItemId aId);
	CContactIdArray* SearchAllFieldsL(const TDesC& aMatch);
	
	short SetContactRingToneL(const TDesC& aValue, TContactItemId cntId);
	short SetGroupRingToneL(const TDesC& aValue, TContactItemId cntId);
	HBufC* GetGroupRingToneL(TContactItemId cntId);
	HBufC* GetContactRingToneL( TContactItemId cntId);
private:
//#ifdef MMI_FL2.0
	enum {KNumGetValItems=9};
//#endif MMI_FL2.0
private:
	CContactDatabase* iContDB;

	CMMIPhonebookArrayWrapper* iSortedArrayWrapper;
	
	CContactItem* iPendingAdd;
	TInt iPendingAddSpdDial;
	
	CContactIdArray* iLastSearch;
	
	TFixedArray<TMMIPbkUidStringPair,KNumGetValItems> iLabelUIDPairs;

	CContactItemField* iPendingAddField;
	CArrayPtr<CPbDataRequestBase>* iObjects;

	MMIUtils* iMMIUtils;

};

const TImplementationProxy ImplementationTable[] =
{
	{{0x10273678}, CMMIPhonebook::NewL }
};


EXPORT_C const TImplementationProxy* ImplementationGroupProxy(TInt& aTableCount)
{
	aTableCount = sizeof(ImplementationTable) / sizeof(TImplementationProxy);
	return ImplementationTable;
}

#endif // __MMI_PHONEBOOK_H__
