/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of Phonebook service
				 extension class
*
*/


#include <cntdef.hrh>
#include <cntdb.h>
#include <PbkFields.hrh>
#include <CPbkContactEngine.h> 
#include <CPbkContactItem.h>
#include <TPbkContactItemField.h> 
#include <f32file.h> 
#include <DRMHelper.h>
#include <CPbkFieldsInfo.h>
#include <CPbkFieldInfo.h> 

#include "mmiphonebk.h"
// 
// general constants etc
//
const TInt KMinSpeedDial=1;
const TInt KMaxSpeedDial=9;

_LIT(KLitEntries,"entries");
_LIT(KLitNumEntries,"numEntries");
_LIT(KLitSearchResult,"searchResult");
_LIT(KLitGetEntry,"entry");
_LIT(KLitGetGroup, "group");
_LIT(KLitPhnNumByLabel,"numberByLabel");
_LIT(KLitGetNumNumbers,"numNumbers");
_LIT(KLitGetLabel,"label");
_LIT(KLitGetNumberInfo,"numbers");
_LIT(KLitGetNumber,"number");
_LIT(KLitContactID,"phonebookID");
_LIT(KLitSpeedDial,"speedDial");
_LIT(KLitFirstNameKey,"firstName");
_LIT(KLitLastNameKey,"lastName");
_LIT(KLitNameKey,"name");
_LIT(KLitImagePathKey,"imagePath");
_LIT(KLitRingerKey,"ringer");
_LIT(KLitAddEntry, "addEntry");
_LIT(KLitAddNumber,"addNumber");
_LIT(KLitRemoveAll,"removeAll");
_LIT(KLitDoSearch, "search");
_LIT(KLitRemove, "remove");
_LIT(KLitGroupID,"groupID");

//#ifdef MMI_FL2.0
_LIT(KLitPhoneNumber,"homeNumber");
_LIT(KLitRingTone, "ringingTone");
_LIT(KLitWorkNumber, "workNumber");
_LIT(KLitMobileNumber, "mobileNumber");
//#endif MMI_FL2.0
//
// CMMIPhonebookArrayWrapper
//

// ensures no unneccesary and slow rebuilding of the item array...
class CMMIPhonebookArrayWrapper : public CBase
{
public:
	static CMMIPhonebookArrayWrapper* NewL(CContactDatabase& aContDB);
	~CMMIPhonebookArrayWrapper();

private:
	CMMIPhonebookArrayWrapper(CContactDatabase& aContDB);

public: // API
	void SetArrayInvalid();
	TBool ArrayInvalid() const;
	const CContactIdArray* ArrayL();
	TInt GetIndexFromCntIdL(TContactItemId aContactId);
	TContactItemId GetCntIdFromIndexL(TInt aIndex);
private:
	void RebuildItemArrayL();
private:
	TBool iArrayValid;
	CContactIdArray* iSortedArray;
	CContactDatabase& iContDB;
};

CMMIPhonebookArrayWrapper* CMMIPhonebookArrayWrapper::NewL(CContactDatabase& aContDB)
{
	CMMIPhonebookArrayWrapper* self=new(ELeave) CMMIPhonebookArrayWrapper(aContDB);
	CleanupStack::PushL(self);
	self->RebuildItemArrayL();
	CleanupStack::Pop(self);
	return self;
}

CMMIPhonebookArrayWrapper::~CMMIPhonebookArrayWrapper()
{
	delete iSortedArray;
}


CMMIPhonebookArrayWrapper::CMMIPhonebookArrayWrapper(CContactDatabase& aContDB)
:iContDB(aContDB)
{
}

TBool CMMIPhonebookArrayWrapper::ArrayInvalid() const
{
	return !iArrayValid;
}

void CMMIPhonebookArrayWrapper::RebuildItemArrayL()
{
//	if(!iArrayValid)
	{
		CContactDatabase::TSortPref pref1;
		pref1.iFieldType=KUidContactFieldGivenName;
		pref1.iOrder=CContactDatabase::TSortPref::EAsc;
		CContactDatabase::TSortPref pref2;
		pref2.iFieldType=KUidContactFieldFamilyName;
		pref2.iOrder=CContactDatabase::TSortPref::EAsc;

		CArrayFixFlat<CContactDatabase::TSortPref>* array=new(ELeave) CArrayFixFlat<CContactDatabase::TSortPref>(2);
		CleanupStack::PushL(array);
		array->AppendL(pref1);
		array->AppendL(pref2);
		CleanupStack::Pop(array);
		
		iContDB.SortByTypeL(array);
		
		const CContactIdArray* temp=iContDB.SortedItemsL(); // not owned!
		CContactIdArray* temp2=CContactIdArray::NewL(temp);
		delete iSortedArray;
		iSortedArray=temp2;
		iArrayValid=ETrue;
	}
}

void CMMIPhonebookArrayWrapper::SetArrayInvalid()
{
	iArrayValid=EFalse;
}

const CContactIdArray* CMMIPhonebookArrayWrapper::ArrayL()
{
	RebuildItemArrayL();
	return iSortedArray;
}

TInt CMMIPhonebookArrayWrapper::GetIndexFromCntIdL(TContactItemId aId)
{
	RebuildItemArrayL();
	TInt index=KErrNotFound;
	const TInt count=iSortedArray->Count();

	TInt temp;
	for(TInt i=0;i<count;i++)
	{
		temp=(*iSortedArray)[i];
		if(temp==aId)
		{
			index=i;
			break;
		}
	}
	
	return index;
	
}

TContactItemId CMMIPhonebookArrayWrapper::GetCntIdFromIndexL(TInt aIndex)
{
	RebuildItemArrayL();
	TContactItemId ret=KNullContactId;
	
//#ifdef MMI_FL2.0	
	if(aIndex>=0 && aIndex<iSortedArray->Count())
	{
		ret=(*iSortedArray)[aIndex];
	}
	
	/*if(aIndex>=0 && aIndex<iSortedArray->Count())
	{
		ret=(*iSortedArray)[aIndex];
	}*/
// #endif MMI_FL2.0	
	return ret;
}

//
// class CPbDataReqBase
//

// base class for all Phonebook async data requests - a similar design can likely be used for all MMI content types
class CPbDataRequestBase : public CActive
{
protected:
	CPbDataRequestBase(TInt aPriority, MObjectOwner& aOwner, const TAny* aDataRequestHandle) 
		: CActive(aPriority), iOwner(aOwner), iDataRequestHandle(aDataRequestHandle) {}

public:
	~CPbDataRequestBase() {}

public:
	TBool IsSameRequest(const TAny* aCompareRequest) const {return (aCompareRequest == iDataRequestHandle);}

protected:
	MObjectOwner& iOwner;
	const TAny* iDataRequestHandle;
};

//
// class CPbReqData
//

// class for doing Async fetches of contact data.
class CPbReqData : public CPbDataRequestBase
{
public:
	static CPbReqData* NewLC(const TAny* aDataRequestHandle, MObjectOwner& aOwner,
		CContactDatabase& aDb, CMMIPhonebookArrayWrapper& aContacts, MMIUtils* aMMIUtils);
	~CPbReqData();
private:
	CPbReqData(const TAny* aDataRequestHandle, MObjectOwner& aOwner, CContactDatabase& aDb, CMMIPhonebookArrayWrapper& aContacts, MMIUtils* aMMIUtils);
private:
	// from CActive
	void RunL();
	TInt RunError(TInt aError);
	void DoCancel();
	// new funcs
	void NextEvent();
	void SetTextFieldL(TAny* aEntryhandle, CContactItemFieldSet& aFields,
			const TDesC& aKeyDes, TUid aFieldUid);
private:
	enum TState { EStateGetNumEntries=0, EStateGetData=1, EStateSendToPlayer=2 };
private:
	TAny* iEntriesHandle; // owned by the player
	TInt iState;
	TInt iIndex;
	CContactDatabase& iContDB;
	CMMIPhonebookArrayWrapper& iContacts;
	MMIUtils* iMMIUtils;
};


//static 
CPbReqData* CPbReqData::NewLC(const TAny* aDataRequestHandle, MObjectOwner& aOwner,
		CContactDatabase& aDb, CMMIPhonebookArrayWrapper& aContacts, MMIUtils* aMMIUtils)
{
	CPbReqData* self = new(ELeave) CPbReqData(aDataRequestHandle, aOwner, aDb, aContacts, aMMIUtils);
	CleanupStack::PushL(self);
	return self;
}

CPbReqData::~CPbReqData()
{
	Cancel();
}

CPbReqData::CPbReqData(const TAny* aDataRequestHandle, MObjectOwner& aOwner, CContactDatabase& aDb, CMMIPhonebookArrayWrapper& aContacts, MMIUtils* aMMIUtils)
:CPbDataRequestBase(CActive::EPriorityIdle, aOwner, aDataRequestHandle), iContDB(aDb), iContacts(aContacts), iMMIUtils(aMMIUtils)
{
	CActiveScheduler::Add(this);
	NextEvent();
}

void CPbReqData::SetTextFieldL(TAny* aEntryHandle, CContactItemFieldSet& aFields, 
							  const TDesC& aKeyDes, TUid aFieldUid)
{
	TInt fieldIndex=aFields.Find(aFieldUid);

	TPtrC des(KNullDesC);

	if(fieldIndex!=KErrNotFound)
	{
		const CContactItemField* field=&(aFields[fieldIndex]);
		if(KStorageTypeText==field->StorageType())
		{
			des.Set(field->TextStorage()->Text());
		}
	}
	iMMIUtils->DataRequestSetStringProperty(aEntryHandle, aKeyDes, des);
}

void CPbReqData::RunL()
{
	// bail out if the array has changed since we sent the count back to the player...
	// we could be smarter and actually start again, by not telling the player anything till we are complete,
	// but for reference this is fine.
	if(iContacts.ArrayInvalid() && iState!=EStateGetNumEntries)
	{
		User::Leave(KErrGeneral);
	}

	const TInt count=iContacts.ArrayL()->Count();
	const TDesC* des;

	if(iState==EStateGetNumEntries)
	{
		// get the number of entries

		// first set the number of entries
		iMMIUtils->DataRequestSetNumberProperty(iDataRequestHandle, KLitNumEntries, count);

		// create the entries item
		iEntriesHandle = iMMIUtils->DataRequestCreateObjectProperty(iDataRequestHandle, KLitEntries );
		User::LeaveIfNull(iEntriesHandle); // this will clean everything up
		
		iState++;
		NextEvent();
	}
	else if(iState==EStateGetData)
	{
		// get the next 5 entry details
		TBool done=EFalse;
		TInt stop=iIndex+5;
		
		if(count<stop)
		{
			stop=count;
			done=ETrue;
		}

		CContactItem* item;
		TContactItemId id;
		CContactItemFieldSet* fields;
		TBuf<16> numBuf;
		TAny* pEntry;

		for(;iIndex<stop;iIndex++)
		{
			// get the contact item
			id=iContacts.GetCntIdFromIndexL(iIndex);
			item=iContDB.ReadContactLC(id);
			fields=&(item->CardFields());
			
			// item number
			numBuf.SetLength(0);
			numBuf.AppendNum(iIndex);
			
			pEntry = iMMIUtils->DataRequestCreateObjectProperty(iEntriesHandle, numBuf);
			User::LeaveIfNull(pEntry);
			
			// get the first name
			
			des=&KLitFirstNameKey;
			SetTextFieldL(pEntry, *fields, *des, KUidContactFieldGivenName);
			
			// last name

			des=&KLitLastNameKey;
			SetTextFieldL(pEntry, *fields, *des, KUidContactFieldFamilyName);
			
//#ifdef MMI_FL2.0			
			des=&KLitPhoneNumber;
			SetTextFieldL(pEntry, *fields, *des, KUidContactFieldPhoneNumber);
			
			des=&KLitRingTone;
			SetTextFieldL(pEntry, *fields, *des, KUidContactFieldRingTone);
			
			des=&KLitWorkNumber;
			SetTextFieldL(pEntry, *fields, *des, KUidContactFieldVCardMapWORK);	
			
			des=&KLitMobileNumber;
			SetTextFieldL(pEntry, *fields, *des, KUidContactFieldVCardMapCELL);	
//#endif MMI_FL2.0
			
			CleanupStack::PopAndDestroy(item);
			iMMIUtils->DataRequestSetComplete(pEntry, KDataRequestSuccess);			
		}

		if(done)
			iState++;
		NextEvent();
	}
	else if(iState==EStateSendToPlayer)
	{
		// send it all back
		iMMIUtils->DataRequestSetComplete(iEntriesHandle, KDataRequestSuccess);			
		iMMIUtils->DataRequestSetComplete(iDataRequestHandle, KDataRequestSuccess);
		// delete ourselves
		iOwner.DeleteObject(this);
	}
}

void CPbReqData::NextEvent()
{
	iStatus=KRequestPending;
	SetActive();
	TRequestStatus* temp=&iStatus;
	User::RequestComplete(temp,KErrNone);
}

TInt CPbReqData::RunError(TInt /*aError*/)
{
	// notify the player of failure, then delete ourselves
	iMMIUtils->DataRequestSetComplete(iDataRequestHandle, KDataRequestFail);
	iOwner.DeleteObject(this);
	return KErrNone;
}

void CPbReqData::DoCancel()
{
 	// notify the player of failure
	iMMIUtils->DataRequestSetComplete(iDataRequestHandle, KDataRequestFail);
}

//
// class CMMIPhonebook
//

TAny* CMMIPhonebook::NewL()
{
	// static
	CMMIPhonebook* self=new(ELeave) CMMIPhonebook();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

_LIT8(KLit8Phonebook, "phonebook");

void CMMIPhonebook::ConstructL()
{
	// DK: This code assumes (legitimately) that we are working with the default
	// contacts database file see Symbian API docs for details.

	TRAPD(error, iContDB=CContactDatabase::OpenL());
	if(error)
	{
		if(error==KErrNotFound || error==KErrPathNotFound)
		{
			iContDB=CContactDatabase::CreateL();
		}
		User::Leave(error);
	}

	iContDB->SetDbViewContactType(KUidContactCard);

	iObjects=new(ELeave) CArrayPtrSeg<CPbDataRequestBase>(1);

	iSortedArrayWrapper=CMMIPhonebookArrayWrapper::NewL(*iContDB);
	
	// TODO: (DK) possibly filter out certain items by fields etc.
	// update list when add/delete happens, and when
	// a change notification is received.
}

CMMIPhonebook::CMMIPhonebook()
{
	iPendingAddSpdDial=KErrNotFound;
	iLabelUIDPairs[0].iString=&KLitFirstNameKey;
	iLabelUIDPairs[0].iUid=KUidContactFieldGivenName;
	iLabelUIDPairs[1].iString=&KLitLastNameKey;
	iLabelUIDPairs[1].iUid=KUidContactFieldFamilyName;
	iLabelUIDPairs[2].iString=&KLitNameKey;
	iLabelUIDPairs[2].iUid=KUidContactFieldCompanyName;
	iLabelUIDPairs[3].iString=&KLitImagePathKey;
	iLabelUIDPairs[3].iUid=KUidContactFieldPicture;
	iLabelUIDPairs[4].iString=&KLitRingerKey;
	iLabelUIDPairs[4].iUid=KUidContactFieldRingTone;
//#ifdef MMI_FL2.0	
	iLabelUIDPairs[5].iString=&KLitPhoneNumber;
	iLabelUIDPairs[5].iUid=KUidContactFieldVCardMapHOME;
	
	iLabelUIDPairs[6].iString=&KLitRingTone;
	iLabelUIDPairs[6].iUid=KUidContactFieldRingTone;
	
	iLabelUIDPairs[7].iString=&KLitWorkNumber;
	iLabelUIDPairs[7].iUid=KUidContactFieldVCardMapWORK;
	
	iLabelUIDPairs[8].iString=&KLitMobileNumber;	
	iLabelUIDPairs[8].iUid=KUidContactFieldVCardMapCELL;
//#endif MMI_FL2.0
}

CMMIPhonebook::~CMMIPhonebook()
{
	delete iPendingAdd;
	delete iLastSearch;
	delete iSortedArrayWrapper;
	delete iPendingAddField;
	delete iContDB;
	if(iObjects)
	{
		iObjects->ResetAndDestroy();
	}
	delete iObjects;
}

const TDesC8* CMMIPhonebook::Name() const
{
	return &KLit8Phonebook;
}

void CMMIPhonebook::Initialize(MMIUtils* aMMIUtils)
{
	iMMIUtils = aMMIUtils;
}

CContactIdArray* CMMIPhonebook::SearchAllFieldsL(const TDesC& aMatch)
{
	CContactItemFieldDef* fields=new(ELeave) CContactItemFieldDef();
	CleanupStack::PushL(fields);
	for(TInt i=0;i<KNumGetValItems;i++)
	{
		fields->AppendL(iLabelUIDPairs[i].iUid);
	}
	CContactIdArray* arr=iContDB->FindLC(aMatch,fields);
	
	// Todo: DK: Efficiency?
	
	TInt count=arr->Count();
	for(TInt j=0;j<count;j++)
	{
		if(KErrNotFound==iSortedArrayWrapper->GetIndexFromCntIdL((*arr)[j]))
		{
			arr->Remove(j);
			count--;
			j--;
		}
	}

	CleanupStack::Pop(arr);
	CleanupStack::PopAndDestroy(fields);
	
	return arr;
}

short CMMIPhonebook::CancelRequestL(const TAny *aDataRequestHandle)
{

	short cancelled = 0;
	const TInt count=iObjects->Count();

	CPbDataRequestBase* pCur = NULL;
	for(TInt i=0; cancelled == 0 && i<count; i++)
	{
		pCur = ((*iObjects)[i]);
		if(pCur && pCur->IsSameRequest(aDataRequestHandle))
		{
			delete pCur;
			iObjects->Delete(i);
			cancelled = 1;
		}
	}

	return cancelled;
	
}

short CMMIPhonebook::RequestDataL(const RPointerArray<TDesC>& aArgs, const TAny* aDataRequestHandle )
{
	short retVal=0;
	
	if(aArgs.Count()==1)
	{
		if(0 == aArgs[0]->Compare(KLitEntries))
		{
			CPbReqData* temp=CPbReqData::NewLC(aDataRequestHandle, *this, *iContDB, *iSortedArrayWrapper, iMMIUtils);
			iObjects->AppendL(temp);
			CleanupStack::Pop(temp);
			retVal=1;
		}
	}

	return retVal;
}


/**
* The GetVarsL() function gets a set of key value pairs associated with a contact entry ID.
*/
short CMMIPhonebook::GetVarsL(const RPointerArray<TDesC>& aArgs, FI_ReturnString* aRetVal)
{


	if(aArgs.Count() == 0)
	{
		return 0;
	}

	const TInt KGranMMIPbkGerValArr=6;

	short res = 0;


	if((aArgs.Count() > 1) && (0 == aArgs[0]->Compare(KLitGetEntry)))
	{
		CArrayFixSeg<TMMIKeyValuePair>* arr=new(ELeave) CArrayFixSeg<TMMIKeyValuePair>(KGranMMIPbkGerValArr);
		CleanupStack::PushL(arr);

		TBuf<30> speeddial;
		TContactItemId indexId = iMMIUtils->GetIntFromDescL(*aArgs[1]);

		TContactItemId id=iSortedArrayWrapper->GetCntIdFromIndexL(indexId);

		TMMIKeyValuePair pair;
		
		TInt temps=0;

		// see if it has a speedDial...
		TInt speed=GetSpeedDialL(id);
		if(speed!=KErrNotFound)
		{
			pair.iKey=&KLitSpeedDial;
			//add key/value pair etc
			speeddial.AppendNum(speed);
			HBufC* temp=speeddial.AllocLC();
			temps++;
			pair.iVal=temp;
			arr->AppendL(pair);
		}
		CContactItem* item=iContDB->ReadContactLC(id);

		// get all the fields
		CContactItemFieldSet& fields=item->CardFields();
		
		//if any are in our wanted list, add them to the arr
		TInt pos;
		const TDesC* temp;
		
	//	iEntriesHandle = iMMIUtils->DataRequestCreateObjectProperty(iDataRequestHandle, KLitEntries );
	//	User::LeaveIfNull(iEntriesHandle); // this will clean 

		for(TInt i=0;i<KNumGetValItems;i++)
		{
			pos=fields.Find(iLabelUIDPairs[i].iUid);
			
			if(pos!=KErrNotFound)
			{
				CContactItemField& field=fields[pos];

				temp=&KNullDesC;
				if(KStorageTypeText==field.StorageType())
				{
					temp=field.TextStorage()->Text().AllocLC();
					temps++;
				}
				pair.iVal=temp;
				pair.iKey=iLabelUIDPairs[i].iString;
				arr->AppendL(pair);
			}
		}

		// invoke the utility function to make the list into a formatted string for the player

		res = iMMIUtils->SetReturnStringFromKeyValuePairsL(arr, aRetVal);

		// all done
		CleanupStack::PopAndDestroy(2+temps,arr);
	}
	return res;
}

/** 
* The SendL function acts upon messages that have been Sent to the the CMMIPhonebook class.
* These are generally instructions to do something with phonebook data.
*/	
short CMMIPhonebook::SendL(const TDesC& aVerbArg, const RPointerArray<TDesC>& aArgs)
{
	short res=0;

	if (0 == aVerbArg.Compare(KLitAddEntry))
	{
		if(iPendingAdd)
		{
			TContactItemId id=iContDB->AddNewContactL(*iPendingAdd);
			
			// now add the speed dial if the contact has any valid fields...
			delete iPendingAdd;
			iPendingAdd=NULL;

			// for consistency with Harness
			delete iLastSearch;
			iLastSearch=NULL;

			if(iPendingAddSpdDial!=KErrNotFound)
			{
				// have to open it first...
				CContactItem* item=iContDB->OpenContactL(id);
				TryAddSpeedDial(*item,iPendingAddSpdDial);
				
				delete item;
				iContDB->CloseContactL(id); // does not leave - see SDK docs

				iPendingAddSpdDial=KErrNotFound;
			}
			
			iSortedArrayWrapper->SetArrayInvalid();
			res=1;
		}
	}
	else if (0 == aVerbArg.Compare(KLitRemoveAll))
	{
		// At present only removes items that are visible from MMI
		// not all contacts items, like groups, templates, etc...

		iContDB->DeleteContactsL(*(iSortedArrayWrapper->ArrayL()));
		
		delete iLastSearch;
		iLastSearch=NULL;

		iSortedArrayWrapper->SetArrayInvalid();
		res=1;
	}

	else if(aArgs.Count()>0) // one parameter
	{
		if (( 0 == aVerbArg.Compare(KLitDoSearch)))
		{
			// todo - note that it will likely be very slow to search all fields, maybe it is better
			// to let the caller specify a fieldset to search?
			// possibly make async?
			
			CContactIdArray* temp=SearchAllFieldsL(*aArgs[0]);

			delete iLastSearch;
			iLastSearch=temp;
			res=1;
		}
		
		else if(0==aVerbArg.Compare(KLitAddNumber) && iPendingAddField)
		{
			TInt index = iMMIUtils->GetIntFromDescL(*aArgs[1]);
			TContactItemId id=iSortedArrayWrapper->GetCntIdFromIndexL(index);
			
			if(id!=KNullContactId)
			{
				CContactItem* item = iContDB->OpenContactLX(id);
				CleanupStack::PushL(item);
				iPendingAddField=NULL;
				item->CardFields().AddL(*iPendingAddField);
				iContDB->CommitContactL(*item);

				CleanupStack::PopAndDestroy(2); //item and lock
				res=1;
			}
		}

		else if(aArgs.Count()>1) // two params
		{
			if ((0 == aVerbArg.Compare(KLitRemove)) 
				&& (0 == aArgs[0]->Compare(KLitGetEntry)))
			{
				TInt index = iMMIUtils->GetIntFromDescL(*aArgs[1]);
				TContactItemId id=iSortedArrayWrapper->GetCntIdFromIndexL(index);
				iContDB->DeleteContactL(id);
				iSortedArrayWrapper->SetArrayInvalid();
				res=1;
			}

		}
	}
	return res;
}

void CMMIPhonebook::DeleteObject(CBase* aObject)
{
	const TInt count=iObjects->Count();

	for(TInt i=0; i<count; i++)
	{
		if((*iObjects)[i]==aObject)
		{
			delete aObject;
			iObjects->Delete(i);
			break;
		}
	}
}

short CMMIPhonebook::DoTryAddSpeedDialL(CContactItem& aItem, TInt aIndex)
{
	TBuf<100> temp;
	// first find any existing speed dials associated with this contact
	TFixedArray<TBool,KMaxSpeedDial> spdDials;
	
	Mem::FillZ(&spdDials,KMaxSpeedDial*sizeof(TBool));

	for(TInt i=KMinSpeedDial;i<=KMaxSpeedDial;i++)
	{
		if(aItem.Id()==iContDB->GetSpeedDialFieldL(i,temp))
		{
			spdDials[i-1]=ETrue;
		}
	}

	// assign the new speed dial, Must assign even if already assigned as the act
	// of assigning commits the contact.
	// Until the contact is commited, no other speeddials can be removed!
	
	const TInt fieldIndex=aItem.CardFields().Find(KUidContactFieldPhoneNumber);
	if(fieldIndex!=KErrNotFound)
	{
		iContDB->SetFieldAsSpeedDialL(aItem,fieldIndex,aIndex);
	}
	

	// now remove the old numbers - if any
	// but don't remove the one we just added
	const TInt count=(KMaxSpeedDial-KMinSpeedDial);
	for(TInt j=0;j<=count;j++)
	{
		if(spdDials[j] && (!((j+KMinSpeedDial)==aIndex)))
			iContDB->RemoveSpeedDialFieldL(aItem.Id(),j+KMinSpeedDial);
	}

	short ret=1;
	return ret;
}

short CMMIPhonebook::TryAddSpeedDial(CContactItem& aItem, TInt aIndex)
{	
	short ret=0;
	TRAP_IGNORE(ret=DoTryAddSpeedDialL(aItem,aIndex));
	return ret;
}



/**
* The GetL() function gets a single value associated with a contact entry,
* or with the contacts store.
*
* 
* 
*/
short CMMIPhonebook::GetL(const RPointerArray<TDesC>& aArgs, FI_ReturnString* aRetVal)
{
	if(aArgs.Count() == 0)
	{
		return NULL;
	}

	if (0 == aArgs[0]->Compare(KLitNumEntries))
	{
		return iMMIUtils->SetReturnStringFromIntL(iSortedArrayWrapper->ArrayL()->Count(), aRetVal);
	}
	
	TInt argvMax=1; // the maximum argv that will be used without checking first

	if(aArgs.Count()<=argvMax)
	{
		return NULL;
	}
	

	TContactItemId id = KNullContactId;

	if ((0 == aArgs[0]->Compare(KLitSearchResult)))
	{
		if(0 == aArgs[1]->Compare(KLitNumEntries))
		{
			TInt num=0;
			if(iLastSearch)
			{
				num = iLastSearch->Count();
			}
			return iMMIUtils->SetReturnStringFromIntL(num, aRetVal);
		}
		else if(0 == aArgs[1]->Compare(KLitGetEntry))
		{
			argvMax++;
			if(aArgs.Count()<=argvMax)
			{
				return NULL;
			}
			
			TInt index = iMMIUtils->GetIntFromDescL(*aArgs[2]);

			if(iLastSearch && index<iLastSearch->Count())
			{	
				id=(*iLastSearch)[index];
			}
			if(id!=KNullContactId)
			{//get the index in the full array
				index=iSortedArrayWrapper->GetIndexFromCntIdL(id);

				if(index!=KErrNotFound)
				{
					return iMMIUtils->SetReturnStringFromIntL(index, aRetVal);
				}
			}
		}

	}
	else if(0 == aArgs[0]->Compare(KLitGetEntry))
	{
		TInt indexId = iMMIUtils->GetIntFromDescL(*aArgs[1]);
		id = iSortedArrayWrapper->GetCntIdFromIndexL(indexId);
		
		if (aArgs.Count() > argvMax)
			{
			
			if(0 == (*aArgs[2]).Compare(KLitRingerKey))
				{
				argvMax++;
				TInt res = 0;
				//TContactItemId cntId=iSortedArrayWrapper->GetCntIdFromIndexL(id);
				
				//Get the ringing tone of the contact item.
				HBufC* ringingTonePtr = GetContactRingToneL(id);
		       	if (ringingTonePtr == NULL)	
					{
					res = 0;
					}
				else
					{
					res = iMMIUtils->SetReturnStringFromDes16L(*ringingTonePtr, aRetVal);
					delete ringingTonePtr;
					}
					return res;
				}
			}
	}
	
	else if(0 == aArgs[0]->Compare(KLitGetGroup))
	    {
	    TInt res = 0;
	    if(iContDB->GetDbViewContactType() !=KUidContactGroup)
	        {
	        iContDB->SetDbViewContactType(KUidContactGroup);	        
	        iSortedArrayWrapper->SetArrayInvalid();
	        }                	        
		TInt indexId = iMMIUtils->GetIntFromDescL(*aArgs[1]);
		id = iSortedArrayWrapper->GetCntIdFromIndexL(indexId);
		
		if ( 0 == aArgs[2]->Compare(KLitRingTone) ) 
		{
			//Get the ringing tone of the group.
			HBufC* ringingToneGroupPtr = GetGroupRingToneL(id);
			if ( ringingToneGroupPtr == NULL )	
				{
				res = 0;
				}
			else
				{
				res = iMMIUtils->SetReturnStringFromDes16L(*ringingToneGroupPtr, aRetVal);
				delete ringingToneGroupPtr;
				}
			return res;
		}
	    }
	
	argvMax++;

	// from here on everything is to do with getting info from the contact item

	if(id==KNullContactId || aArgs.Count()<=argvMax)
	{
		return NULL;
	}
	
	// get the ID of the requested entry - don't need any details from the contact for this
	// so do first
	if(0== (aArgs[argvMax]->Compare(KLitContactID)))
	{
		TInt index =iSortedArrayWrapper->GetIndexFromCntIdL(id);
		
		if(index==KErrNotFound)
		{
			return NULL;
		}
		
		return iMMIUtils->SetReturnStringFromIntL(index, aRetVal);
	}

	if( 0 == (aArgs[argvMax]->Compare(KLitSpeedDial)))
	{
		TInt spdDial=GetSpeedDialL(id);
		if(spdDial==KErrNotFound)
		{
			return iMMIUtils->SetReturnStringFromDes16L(KNullDesC, aRetVal);
		}
		else
		{
			return iMMIUtils->SetReturnStringFromIntL(spdDial, aRetVal);
		}
	}


	// all other gets require field information from the contact
	CContactItem* item=iContDB->ReadContactLC(id);
	if(iContDB->GetDbViewContactType() != KUidContactCard)
        {
        iContDB->SetDbViewContactType(KUidContactCard);	        
        }

	// will leave if contact not found, so can safely assume item is set to non-NULL
	CContactItemFieldSet& fields=item->CardFields();

	//
	// pick off the less simple cases first
	//
	short res=0;
	// number of numbers this contact has
	if(0== (aArgs[argvMax]->Compare(KLitGetNumNumbers)))
	{
		TInt howMany;
		GetTotalFieldsAndPosOfNthFieldOfType(fields,KUidContactFieldPhoneNumber,1,howMany);
		res = iMMIUtils->SetReturnStringFromIntL(howMany, aRetVal);
	}

	// get telephone number by label - note this will always get the first if there
	// are duplicate labels
	else if((0 == aArgs[argvMax]->Compare(KLitPhnNumByLabel)) && aArgs.Count()>argvMax)
	{
		//const FI_Text* labelText=aArgv[argvMax+1];
		//TPtrC label(labelText->text.utf16String,labelText->textLen);

		CContactItemField* field=GetFieldByLabel(fields,KUidContactFieldPhoneNumber, *aArgs[argvMax+1]);

		if(field && KStorageTypeText==field->StorageType())
		{
			const TPtrC txt=field->TextStorage()->Text();
			res = iMMIUtils->SetReturnStringFromDes16L(txt, aRetVal);
		}
	}

	// get the number/label by index
	else if ((0 == aArgs[argvMax]->Compare(KLitGetNumberInfo)) && aArgs.Count()>(argvMax+2))
	{
		TInt telNumIndex = iMMIUtils->GetIntFromDescL(*aArgs[argvMax+1]);
		TInt howMany=0;
		TInt index=GetTotalFieldsAndPosOfNthFieldOfType(fields,KUidContactFieldPhoneNumber,telNumIndex,howMany);
		
		if(index!=KErrNotFound)
		{
			CContactItemField& field=fields[index];

			//const FI_Text* finalArg = aArgv[argvMax+2];
			//TPtrC finalArgPtr(finalArg->text.utf16String, finalArg->textLen);
			
			if(0 == aArgs[argvMax+2]->Compare(KLitGetLabel))
			{
				res = iMMIUtils->SetReturnStringFromDes16L(field.Label(), aRetVal);
			}
			
			else if (0 == aArgs[argvMax+2]->Compare(KLitGetNumber))
			{
				if(KStorageTypeText==field.StorageType())
				{
					const TPtrC ptrC(field.TextStorage()->Text());
					res = iMMIUtils->SetReturnStringFromDes16L(ptrC, aRetVal);
				}
			}
		}
	}		

	else
		{
		
		//
		// then the simpler 'get value' types - supports only TextStorage Items for now...
		//
		TInt indexIntoUidArray;
		TInt index=FieldIndexForString(fields, *aArgs[argvMax], indexIntoUidArray);
		if(index!=KErrNotFound)
		{
			CContactItemField& field=fields[index];
			if(KStorageTypeText==field.StorageType())
			{
				TPtrC ptrC(field.TextStorage()->Text());
				res = iMMIUtils->SetReturnStringFromDes16L(ptrC, aRetVal);
			}
		}

	}
	
	CleanupStack::PopAndDestroy(item);

	return res;
}

CContactItemField* CMMIPhonebook::GetFieldByLabel(CContactItemFieldSet& aFields, TFieldType aType, const TDesC& aLabel) const
{
	CContactItemField* ret=NULL;
	TInt pos=0;
	CContactItemField* field=NULL;

	const TInt bound=aFields.Count();

	while(pos<bound)
	{
		pos=aFields.FindNext(aType,pos);
		
		if(pos==KErrNotFound)
		{
			break;
		}
		else
		{
			field=&aFields[pos];
			
			if(0== aLabel.Compare(field->Label()))
			{
				ret=field;			
				break;
			}
			pos++;
		}
	}
	return ret;
}


TInt CMMIPhonebook::GetTotalFieldsAndPosOfNthFieldOfType(const CContactItemFieldSet& aFields, TFieldType aType, TInt aNth, TInt& aHowManyFound) const
{
	aHowManyFound=0;
	TInt index=KErrNotFound; // index of the item we are looking for
	TInt pos=0; // current position in field list
	const TInt bound=aFields.Count(); // absolute highest field.
	while(pos!=KErrNotFound && pos<bound)
		{
			pos=aFields.FindNext(aType,pos);
			if(pos!=KErrNotFound)
			{
				aHowManyFound++;
				
				if(aNth==(aHowManyFound-1)) // index vs running count
				{
					index=pos;
				}
			
				pos++;
			}
		}
	return index;
}

TInt CMMIPhonebook::GetSpeedDialL(TContactItemId aId)
{
	TBuf<50> phoneNum; // we will ignore this but the API requires it
	for(TInt i=KMinSpeedDial;i<=KMaxSpeedDial;i++)
	{
		if(aId==iContDB->GetSpeedDialFieldL(i,phoneNum))
		{
			return i;
		}
	}
	return KErrNotFound;
}

short CMMIPhonebook::SetL(const RPointerArray<TDesC>& aArgs, const TDesC& aValue)
{
	if(aArgs.Count() == 0)
	{
		return 0;
	}

	// first get hold of the entry we need to update

	CContactItem* item=NULL;

	short res=0;

	TInt minArgs=0;

	if(aArgs.Count() > minArgs)
	{
		minArgs++;
		if(0 == aArgs[0]->Compare(KLitGetEntry))
		{
			if(aArgs.Count() > minArgs)
			{
				TInt id = iMMIUtils->GetIntFromDescL(*aArgs[minArgs]);
				
				TContactItemId cntId=iSortedArrayWrapper->GetCntIdFromIndexL(id);
				minArgs++;

			// To set the ring tone for a contacts item	
			if(aArgs.Count() > minArgs)
				{	
				if(0 == (*aArgs[minArgs]).Compare(KLitRingTone))
					{
					res = SetContactRingToneL(aValue, cntId);
		        	return res;
					}
				}

				item=iContDB->OpenContactLX(cntId);
				CleanupStack::PushL(item); // X only pushes the 'lock'...
			}
			// do speed dial now, since it is not done through the contact field APIs
			if(aArgs.Count()>minArgs) // speedial command,  number to allocate to contact
			{
				//const FI_Text* command = aArgv[minArgs];
				//TPtrC commandPtr(command->text.utf16String, command->textLen);

				if( 0== aArgs[minArgs]->Compare(KLitSpeedDial))
				{
					TInt assign = iMMIUtils->GetIntFromDescL(aValue);
					// find a phone number in the contact
					res=TryAddSpeedDial(*item,assign);
					CleanupStack::PopAndDestroy(2); // item and 'X' thing to close contact...
					item=NULL;
				}
			}

			
		}
		else if(0 == aArgs[0]->Compare(KLitAddEntry))
		{
			if(!iPendingAdd)
			{
				CContactItem* templateItem=iContDB->ReadContactLC(iContDB->TemplateId());
				iPendingAdd=CContactCard::NewL(templateItem);
				CleanupStack::PopAndDestroy(templateItem);
			}
			item=iPendingAdd;

			// now do the speedDial stuff which is basically to remember until we commit the contact...
			if(aArgs.Count() > minArgs)
			{
				//const FI_Text* command = aArgv[minArgs];
				//TPtrC commandPtr(command->text.utf16String, command->textLen);

				if( 0== aArgs[minArgs]->Compare(KLitSpeedDial))
				{
					iPendingAddSpdDial = iMMIUtils->GetIntFromDescL(aValue);
					res=1;
					item=NULL;
				}
			}
			
		}
		// new Add number functions
		else if(0 == aArgs[0]->Compare(KLitAddNumber))
		{
			if(!iPendingAddField)
			{
				iPendingAddField=CContactItemField::NewL(KStorageTypeText,KUidContactFieldPhoneNumber);
			}
			
			if(aArgs.Count() > 2)
			{
				//const FI_Text* toSet = aArgv[2];
				//TPtrC toSetPtr(toSet->text.utf16String, toSet->textLen);

				//const FI_Text* valOrLabel = aArgv[1];
				//TPtrC valOrLabelPtr(valOrLabel->text.utf16String, valOrLabel->textLen);


				if(0==aArgs[1]->Compare(KLitGetLabel))
				{
					iPendingAddField->SetLabelL(*aArgs[2]);
				}
				else
				{
					iPendingAddField->TextStorage()->SetTextL(*aArgs[2]);
				}
				res=1;
			}
		}
		else if(0 == aArgs[0]->Compare(KLitGetGroup))
		    {	
		    // Creates a group item
    		if( 0 == aArgs[minArgs+1]->Compare(KLitGroupID) )
		    	{
				CContactItem* item = iContDB->CreateContactGroupL(aValue);
            	if(item)
                	{
                	res  = 1;
                	delete item;
                	return res;
			    	}
		    	}
		    // Sets the ringtone of a group	
		    else if ( 0 == aArgs[minArgs+1]->Compare(KLitRingTone) )
		    	{
				iContDB->SetDbViewContactType(KUidContactGroup);	        
			    TInt id = iMMIUtils->GetIntFromDescL(*aArgs[minArgs]);			
				TContactItemId cntId=iSortedArrayWrapper->GetCntIdFromIndexL(id);
				res = SetGroupRingToneL(aValue, cntId);
				return res;
		    	}
		    }
		
	}

	// if we don't have an item, we are either done or all bets are off..
	if(!item || !(aArgs.Count() > minArgs))
	{
		return res;
	}

	//const FI_Text* command = aArgv[minArgs];
	//TPtrC commandPtr(command->text.utf16String, command->textLen);

	// For future use
	//TPtrC valuePtr(aValue->text.utf16String, aValue->textLen);

	CContactItemFieldSet& fields=item->CardFields();

	if(aArgs.Count() > minArgs+2)
	{
		// special case of number property by index...		
		if( 0 == aArgs[minArgs]->Compare(KLitGetNumberInfo))
		{
			const TInt inxOfNumber = iMMIUtils->GetIntFromDescL(*aArgs[minArgs+1]);

			//const FI_Text* setWhat = aArgv[minArgs+2];
			//TPtrC setWhatPtr(setWhat->text.utf16String, setWhat->textLen);
			
			TBool isAValue=ETrue;
			if(0 == aArgs[minArgs+2]->Compare(KLitGetLabel))
			{
				isAValue=EFalse;
			}

			// lets get the item, if it exists...
			TInt howmany=0;
			TInt pos=GetTotalFieldsAndPosOfNthFieldOfType(fields,KUidContactFieldPhoneNumber,inxOfNumber, howmany);
			
			// found it
			if(pos!=KErrNotFound)
			{
				CContactItemField& field=fields[pos];
				if(!isAValue)
				{
					//field.SetLabelL(*toSet);
					field.SetLabelL(aValue);
					res=1;
				}
				else if(KStorageTypeText==field.StorageType())
				{
					//field.TextStorage()->SetTextL(*toSet);
					field.TextStorage()->SetTextL(aValue);
					res=1;
				}
				
			}
		}
	}
	else if(aArgs.Count() > minArgs+1)
	{
		// special case of set number by label...
		if( 0 == aArgs[minArgs]->Compare(KLitPhnNumByLabel))
		{
			//const FI_Text* label = aArgv[minArgs+1];
			//TPtrC labelPtr(label->text.utf16String, label->textLen);

			CContactItemField* field=GetFieldByLabel(fields,KUidContactFieldPhoneNumber, *aArgs[minArgs+1]);
			if(field && KStorageTypeText==field->StorageType())
			{
				field->TextStorage()->SetTextL(aValue);
				res=1;
			}
		}
	}
	else if(aArgs.Count() > minArgs)
	{	
		//general case of adding a plain old single one per item field.	
		// TODO: Implement a better default labeling system, using localizable strings
		// in resource file.

		//const FI_Text* defaultLabel = aArgv[minArgs];
		//TPtrC defaultLabelPtr(defaultLabel->text.utf16String, defaultLabel->textLen);

		res=FindOrCreateFieldAndSetValueL(fields, *aArgs[minArgs], aValue, *aArgs[minArgs]);
	}

	// clean up/ commit etc if the contact is in the DB...
	if(item!=iPendingAdd)
	{
		
		if(res)
		{
			iContDB->CommitContactL(*item);
		}
		else
		{
			iContDB->CloseContactL(item->Id());
		}

		CleanupStack::PopAndDestroy(item);
		CleanupStack::Pop(); //lock
	}
	return res;
}

short CMMIPhonebook::FindOrCreateFieldAndSetValueL(CContactItemFieldSet& aFields, const TDesC& aCommand, const TDesC& aNewValue, const TDesC& aDefaultLabel)
{
	short ret=0;

	TInt indexIntoUidArray;
		
	TInt indexOfField = FieldIndexForString(aFields,aCommand,indexIntoUidArray);

	if(indexOfField!=KErrNotFound && KStorageTypeText==aFields[indexOfField].StorageType())
	{
		// found it - assign value if its text storage
		aFields[indexOfField].TextStorage()->SetTextL(aNewValue);
		ret=1;
	}
	else if(indexIntoUidArray!=KErrNotFound)
	{
		// make a new one - need field type from somewhere...
		CContactItemField* field=CContactItemField::NewLC(KStorageTypeText,iLabelUIDPairs[indexIntoUidArray].iUid);
		// TODO: Get real localizable labels from a Resource file.
		field->SetLabelL(aDefaultLabel);
		field->TextStorage()->SetTextL(aNewValue);
		aFields.AddL(*field);
		CleanupStack::Pop(field);
		ret=1;
	}
	return ret;
}

void CMMIPhonebook::SetVarsL(const RPointerArray<TDesC>& aArgs, const TDesC& aValue, short& aRetVal)
{
	if(aArgs.Count() == 0)
	{
		return;
	}

	//const FI_Text* argv0 = aArgv[0];
	//TPtrC argv0Ptr(argv0->text.utf16String, argv0->textLen);

	CContactItem* item=NULL;
	TContactItemId id=KNullContactId;

	if(0 == aArgs[0]->Compare(KLitGetEntry))
	{
		TInt indexId = iMMIUtils->GetIntFromDescL(*aArgs[1]);
		id=iSortedArrayWrapper->GetCntIdFromIndexL(indexId);

		item=iContDB->OpenContactLX(id);
		CleanupStack::PushL(item);
	}
	
	TInt speedDl=0;// must set speed dial last - it commmits the contact
							  // preventing any further changes!

	if(item)
	{
		CContactItemFieldSet& fields=item->CardFields();
		TValueKeyPairParser parser(aValue);
		while(!(parser.Done()))
		{
			const TPtrC key(parser.NextKey());
			const TPtrC val(parser.NextValue());

			// special case - set in CContactDatabase AFTER we have set the other item details...
			if( 0 == key.Compare(KLitSpeedDial))
			{
				speedDl=val[0]-char('0');
			}
			
			// all other supported cases...
			else
			{
				// TODO: resource based default label scheme
				aRetVal=(short)(aRetVal + FindOrCreateFieldAndSetValueL(fields, key, val, key));
			}
		}
		iContDB->CommitContactL(*item);
		CleanupStack::PopAndDestroy(item);
		CleanupStack::Pop(); //  X-thing

		if(speedDl)
		{
			item=iContDB->OpenContactLX(id);
			CleanupStack::PushL(item); // X only pushes the 'lock'...
			aRetVal =(short)(aRetVal + TryAddSpeedDial(*item,speedDl));
			CleanupStack::PopAndDestroy(2); // X and item
		}
	}
}

TInt CMMIPhonebook::FieldIndexForString(CContactItemFieldSet& aFields, const TDesC& aString, TInt& aIndexFound) const
{
	TInt ret=KErrNotFound;
	aIndexFound=KErrNotFound;

	for(TInt i=0;i<KNumGetValItems;i++)
	{
		if(0 == aString.Compare(*(iLabelUIDPairs[i].iString)))
		{
			ret=aFields.Find(iLabelUIDPairs[i].iUid);
			aIndexFound=i;
			break;
		}
	}
	return ret;
}


void CMMIPhonebook::PlayerDestroyed()
{
	delete iPendingAdd;
	iPendingAdd=NULL;
	
	delete iLastSearch;
	iLastSearch=NULL;

	delete iPendingAddField;
	iPendingAddField=NULL;

	iObjects->ResetAndDestroy();
}

_LIT(KLitPhonebook, "phonebook");
TBool CMMIPhonebook::RegisterExtendedEvent(const TDesC& aEventName)
{
	if(aEventName.Compare(KLitPhonebook) == 0)
	{
		// phonebook doesn't register any events.
		return ETrue;
	}
	return EFalse;
}

TBool CMMIPhonebook::UnregisterExtendedEvent(const TDesC& aEventName)
{
	if(aEventName.Compare(KLitPhonebook) == 0)
	{
		// phonebook doesn't register any events.
		return ETrue;
	}
	return EFalse;
}

TAny* CMMIPhonebook::ExecuteFunctionL(TInt /*aOpCode*/, TAny* /*aParam1*/, TAny* /*aParam2*/, TAny* /*aParam3*/, TAny* /*aParam4*/)
{
	return NULL;
}

short CMMIPhonebook::SetContactRingToneL(const TDesC& aValue, TContactItemId cntId)
{
	short res=0;
	RFs fs;
	TInt err = fs.Connect();
	
	if (KErrNone == err)
		{
		CPbkContactEngine* pbkEngine = CPbkContactEngine::NewL(&fs);
		CleanupStack::PushL(pbkEngine);
		
		CPbkContactItem* pbkItem = NULL;
		pbkItem = pbkEngine->OpenContactLCX(cntId);
		
		if ( pbkItem != NULL )
			{
			TPbkContactItemField* itemField = pbkItem->FindField(EPbkFieldIdPersonalRingingToneIndication);
		
			if (itemField)
				{
	    		// there was an old ringing tone stored => handle removal
	    		const TDesC& oldRingingToneFile = itemField->Text();
	    	
	    		CDRMHelper* drmHelper = CDRMHelper::NewL();
				TInt err1 = drmHelper->RemoveAutomatedPassive(oldRingingToneFile);
				delete drmHelper;
				// Set the ringing tone.
	    		itemField->TextStorage()->SetTextL(aValue);
				pbkEngine->CommitContactL(*pbkItem);				
				res = 1;
				}
			else
				{
				// Create the ringing tone field if not present.
				CPbkFieldInfo* fieldInfo = pbkEngine->FieldsInfo().Find(EPbkFieldIdPersonalRingingToneIndication);
				TPbkContactItemField& newField = pbkItem->AddFieldL(*fieldInfo);
				newField.TextStorage()->SetTextL(aValue);
				pbkEngine->CommitContactL(*pbkItem);
				res = 1;
				}
	        
			CleanupStack::PopAndDestroy(pbkItem);
			CleanupStack::Pop();
			}

		CleanupStack::PopAndDestroy(pbkEngine);		
		}
		
	fs.Close();
	return res;	
}

short CMMIPhonebook::SetGroupRingToneL(const TDesC& aValue, TContactItemId cntId)
{	
	short res=0;
	RFs fs;
	TInt err = fs.Connect();
	
	if (KErrNone == err)
		{
		CPbkContactEngine* pbkEngine =  CPbkContactEngine::NewL(&fs);
		CleanupStack::PushL(pbkEngine);

		//Check if the group is present.
		CContactGroup* group = NULL;
		TRAPD(error, group = pbkEngine->ReadContactGroupL(cntId) );
		
		if ( KErrNone == error )
			{
    		CleanupStack::PushL(group);
    		const CContactIdArray* groupContainsIds = group->ItemsContained();    	

			// Set the ringing tone for all the contacts in the group.
    		for(TInt i=0;i<groupContainsIds->Count();i++)
    			{
    			const TContactItemId id = (*groupContainsIds)[i];
    			CPbkContactItem* pbkItem = pbkEngine->OpenContactLCX(id);
    			TPbkContactItemField* itemField = pbkItem->FindField(EPbkFieldIdPersonalRingingToneIndication);
    		
    			if (itemField)
            		{
					// Set the ringing tone.
            		itemField->TextStorage()->SetTextL(aValue);
            		pbkEngine->CommitContactL(*pbkItem);				
					res = 1;
            		}
				else
            		{
            		// Create the ringing tone field if not present.
            		CPbkFieldInfo* fieldInfo = pbkEngine->FieldsInfo().Find(EPbkFieldIdPersonalRingingToneIndication);
            		TPbkContactItemField& newField = pbkItem->AddFieldL(*fieldInfo);
            		newField.TextStorage()->SetTextL(aValue);
            		pbkEngine->CommitContactL(*pbkItem);	
            		res=1;
            		}
            	
				CleanupStack::PopAndDestroy(pbkItem);
				CleanupStack::Pop();	
    			} //for
    	
    		CleanupStack::PopAndDestroy(group);
			}
    	CleanupStack::PopAndDestroy(pbkEngine);
		}
	fs.Close();
	return res;	
}

HBufC* CMMIPhonebook::GetGroupRingToneL( TContactItemId cntId)
{
	RFs fs;
	TInt err = fs.Connect();
	HBufC* bufTemp = NULL;
	
	if (KErrNone == err)
		{
		CPbkContactEngine* pbkEngine =   CPbkContactEngine::NewL(&fs);
		CleanupStack::PushL(pbkEngine);
		
		//Check if the group is present.
		CContactGroup* group = NULL; 
		TRAPD(error, group = pbkEngine->ReadContactGroupL(cntId) );
		
		if ( KErrNone == error )
			{
			CleanupStack::PushL(group);
	    	const CContactIdArray* groupContainsIds = group->ItemsContained();
	    	
	    	const TContactItemId id = (*groupContainsIds)[0];
	    	CPbkContactItem* pbkItem = pbkEngine->ReadContactL(id);
	    	CleanupStack::PushL(pbkItem);
	    	
	    	TPbkContactItemField* itemField = pbkItem->FindField(EPbkFieldIdPersonalRingingToneIndication);
	    	if (itemField)
	    		{
	    		TPtrC ptr( itemField->TextStorage()->Text());
	    		TChar char1('\\');
	    		TInt pos = ptr.LocateReverse(char1);
	    		pos = ptr.Length() - pos -1;
	    		ptr.Set(ptr.Right(pos));
	    		bufTemp = HBufC::NewL(ptr.Length());
	    		*bufTemp = ptr;
	    		}
	    	CleanupStack::PopAndDestroy(pbkItem);
			CleanupStack::PopAndDestroy(group);
			}
			
		CleanupStack::PopAndDestroy(pbkEngine);
		}
		
	fs.Close();
	return bufTemp;
}

HBufC* CMMIPhonebook::GetContactRingToneL( TContactItemId cntId)
{
	
	RFs fs;
	TInt err = fs.Connect();
	
	HBufC* bufTemp = NULL;
		
	if (KErrNone == err)
		{
		CPbkContactEngine* pbkEngine = CPbkContactEngine::NewL(&fs);
		CleanupStack::PushL(pbkEngine);
		
		//Check if the contact item is present
		CPbkContactItem* pbkItem = NULL;
		TRAPD(error, pbkItem = pbkEngine->ReadContactL(cntId));
		
		if ( KErrNone == error)
			{
			CleanupStack::PushL(pbkItem);
			
			TPbkContactItemField* itemField = pbkItem->FindField(EPbkFieldIdPersonalRingingToneIndication);
		    if (itemField)
	    		{
	    		TPtrC ptr( itemField->TextStorage()->Text());
	    		TChar char1('\\');
	    		TInt pos = ptr.LocateReverse(char1);
	    		pos = ptr.Length() - pos -1;
	    		ptr.Set(ptr.Right(pos));
	    		bufTemp = HBufC::NewL(ptr.Length());
	    		*bufTemp = ptr;
	    		}
		    		
			CleanupStack::PopAndDestroy(pbkItem);
			}
		CleanupStack::PopAndDestroy(pbkEngine);
		}
		
		fs.Close();
		return bufTemp;
}
