/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the functions of CMMI_Implementation class which 
				 provides the MMI specific implementation.
				 Implements the functions of CFileManRequestInfo which 
				 encapsulates a File Manager request from the Flash Player. 
*
*/


#include <e32std.h>
#include <w32std.h>
#include <eikenv.h>
#include <eikappui.h>
#include <eikapp.h>
#include <bautils.h>

#include "platformbuild.h"
#include "platformtypes.h"

#include "flashliteinterfaces.h"
#include "httprequestbase.h"
#include "flashMMIinterfaces.h"
#include <mmidelegatebase.h>

#include "csi_mmiobject.h"
#include "fi_interface.h"
#include "symbian_engine.h"
#include "symbian_http.h"
#include "mmieventhndlrbase.h"
#include "mmieventhndlrincomingcall.h"
#include "saflash_consts.h"
#include "symbian_file.h"
#include "mmi_defs.h"
#include "mmicustom.h"
#include "mmiutilsimpl.h"

#include <flashmmi_resources.rsg>

//added

_LIT(KMMICustomResource, "\\resource\\apps\\flashmmi_resources.RSC");

_LIT8(KLIWBinding8, "LIWBinding");


TInt GetIntFromString(const char* aString);
HBufC8* AllocDes8FromDes16L(const TDesC16& aDes);
HBufC* AllocDes16FromStringLC(const char* aString);
HBufC* AllocDes16FromDes8LC(const TDesC8& aDes);
const RPointerArray<TDesC>* ConvertArgsToDesArrayLC(const FI_Text** aArray, TInt aCount);

class CMMIUtilsImpl;
class CPlatformFlashEngine;

/** 
 * The CMMI_Implementation class provides the MMI specific implementation of the SI functions
 * that receive calls from the Core Player. This implementation is private to the MMI code, and
 * is loaded into the player through the abstract CFlashExtension ECom plug-in interface.
 *
 * @see CFlashExtension
 */
class CMMI_Implementation : public CFlashExtension
{
public:
	static CMMI_Implementation* NewL();
	~CMMI_Implementation();
private:
	CMMI_Implementation();
	void ConstructL();

public: // from CFlashExtension
	//virtual TInt Flags() { return FI_FlashLiteMMI | FI_MMIPersistData | FI_SendStartupShutdown; }
	virtual TInt Flags() { return FI_FlashLiteMMI | FI_SendStartupShutdown; }
	virtual void InitExtensionSIFunctions(SI_Fncs*& aSI_Fns);
	virtual void InitExtensionL(MM_Object* aMMObj);
	virtual TBool HandleExtendedKeyEventL(MM_Object* aMMObj,
											const TKeyEvent& aKeyEvent, 
											TEventCode aType);
	virtual void PlayerDestroyed();
	virtual void SetAllowedSubExtensionsL(const TDesC8* aExtensions);
	virtual void NotifyPauseState(TInt aPauseFlags);
	virtual void NotifyIncomingCall(TBool aStatus);

	virtual TAny* ExecuteFunctionL(TInt aOpCode, TAny* aParam1, TAny* /*aParam2*/, 
		TAny* /*aParam3*/, TAny* /*aParam4*/);

public:

	// SI_MMIFncs
	static short SI_Get (struct MM_Object* pMMObj, unsigned short argc, const FI_Text** argv, FI_ReturnString* retVal);
	static short SI_GetVars (struct MM_Object* pMMObj, unsigned short argc, const FI_Text** argv, FI_ReturnString* retVal);
	static short SI_Set (struct MM_Object* pMMObj, unsigned short argc, const FI_Text** argv, struct FI_PlayerEventInfo* pEventInfo);
	static short SI_SetVars (struct MM_Object* pMMObj, unsigned short argc, const FI_Text** argv, struct FI_PlayerEventInfo* pEventInfo);
	static short SI_Send (struct MM_Object* pMMObj, unsigned short argc, const FI_Text** argv, struct FI_PlayerEventInfo* pEventInfo);
    static short SI_RegisterForEvent(struct MM_Object* pMMObj, const struct FI_Text* eventType);
    static short SI_UnregisterEvent(struct MM_Object* pMMObj, const struct FI_Text* eventType);
	static short SI_RequestData (struct MM_Object* pMMObj, unsigned short argc, FI_Text* argv, FI_DataRequest* pDataRequest);
	static short SI_CancelDataRequest(struct MM_Object* pMMObj, FI_DataRequest* pDataRequest);
#ifdef FEATURE_FILE_MANAGER
	static short SI_FileManagerDownload(struct MM_Object* pMMObj, FI_URLStreamInfo* pStreamInfo, unsigned long id, const char* category, FI_PlayerEventInfo* pEventInfo);
	static short Do_SI_FileManagerDownloadL(struct MM_Object* pMMObj, FI_URLStreamInfo* pStreamInfo, unsigned long id, const char* category, FI_PlayerEventInfo* pEventInfo);
	static short SI_FileManagerCancelDownload(struct MM_Object* pMMObj, unsigned long id, FI_PlayerEventInfo* pEventInfo);
#endif	// FEATURE_FILE_MANAGER

	// other SI functions

	static short SI_SetPersistentData(
												MM_Object* pMMObj, 
                                                const char* nameSpace, 
                                                const char* name,
                                                const unsigned char* data, 
                                                unsigned long dataSize, unsigned long aRequestId
									 );
	
   static short SI_GetMaxPersistentStorage(MM_Object* pMMObj, const char* nameSpace, unsigned long* pSize);

   static short SI_Launch (struct MM_Object* pMMObj, unsigned short argc, const FI_Text ** argv, struct FI_PlayerEventInfo* pEventInfo, short isEmbedded);
	


	static CMMI_Delegate* GetDelegateL(struct MM_Object* pMMObj, const struct FI_Text* aString);
	static short CancelDataRequestL(struct MM_Object* pMMObj, FI_DataRequest *request);

private:
	static short FI_OfferKeyPressEvent(struct MM_Object* pMMObj, const struct FI_KeyId* keyCode);

public:

#ifdef FEATURE_FILE_MANAGER
	static unsigned short FI_FileManagerStatus(struct MM_Object* pMMObj, short pStatusEvent, long pDataParamA, long pDataParamB, unsigned long pId);
#endif // FEATURE_FILE_MANAGER

private:

	static CMMI_Implementation* GetOutOfLineImp(struct MM_Object* pMMObj);
	short DoLaunchL (struct MM_Object* pMMObj, unsigned short argc, const FI_Text** argv, struct FI_PlayerEventInfo* pEventInfo, short isEmbedded);
    void AddRegisteredEventL(struct MM_Object* pMMObj, const struct FI_Text &eventType);
    void RemoveRegisteredEvent(struct MM_Object* pMMObj, const struct FI_Text &eventType);

private:
	enum {KDelegateArrayGranularity=1};
	CArrayPtrFlat<CMMI_Delegate>* iDelegates;
	struct SI_MMIFncs* iMMIFns;
	CEikonEnv* iEnv;
	PFNC_SI_LAUNCH iHostSI_Launch; 
	TInt iResourceOffset;
	CDesC8ArrayFlat* iAllowedExtensions;
	CFlashMMICustomization* iMMICustomization;
	CMMIUtilsImpl* iMMIUtils;
	MM_Object* iMMObj;
	RPointerArray<HBufC> iRegisteredEvents;

	TBool iIsSMCheckNeeded;

};


#ifdef FEATURE_FILE_MANAGER

/**
* The CFileManRequestInfo encapsulates a File Manager request from the Flash Player. 
* It lasts for the lifetime of the request, notifying the Player on error or successful completion, and
* writing data received from the network to the file specified by the Player.
*/
class CFileManRequestInfo : public CHttpRequestInfo
{
public:
	static CFileManRequestInfo* NewL(void* aOwner, 
										struct MM_Object* aMMObj, 
										struct FI_URLStreamInfo* aStreamInfo, 
										TUint32 aId, 
										const char* aCategory);
	~CFileManRequestInfo();

private:
	CFileManRequestInfo(void* aOwner, struct MM_Object* aMMObj, 
		TUint32 aId, struct FI_URLStreamInfo* aStreamInfo);
	void ConstructL(const char* aCategory);

public:
	virtual void PrepareToBeServicedL();
	
	virtual void HandleRequestFailed(TInt aError, TInt aHttpStatusCode);
	
	virtual void HandleHeaderResponseL(struct FI_URLStreamHeaderInfo& aHeaders, 
										const TDesC8& aFinalUrl);
	virtual TBool HandleResponseDataL(const TDesC8& aDataPart);
	virtual void HandleResponseCompleteL(TInt aHttpStatusCode); // request has completed normally
private:
	void NotifyCoreComplete(TInt aError);
	void NotifyCoreOfProgress();
	TInt64 FreeSpaceOnVolume(TChar aChar);
	void LoadResourceFileL(TInt aResourceId);
private:
	HBufC*  iCategory;
	RFile	iDownloadFile;
	TInt	iCurrentDownloadSize;
	TInt	iTotalDownloadSize;
	TInt iResourceOffset;
	RFs iFs;
	CPlatformFlashEngine* iEngine; 
	
};

/**
* The NewL function fully constructs a new CFileManRequestInfo object ready for use.
*
* Leaves with a standard system error if unable to create the CFileManRequestInfo object.
*
* @param aOwner The address of, or another quantity unique to the owning object. Must be valid when the object is cancelled, for example.
* @param aMMObj The MM_Object for the requesting movie.
* @param aStreamInfo The stream info from the core player
* @param aId  The identifier for this object, unique among all other objects owned by aOwner.
* @param aCategory The string for the filemanager download category that this object represents.
*
* @return a new CFileManRequestInfo transferring ownership to the caller.
* @static
*/
CFileManRequestInfo* CFileManRequestInfo::NewL(void* aOwner, 
											   struct MM_Object* aMMObj, 
											   struct FI_URLStreamInfo* aStreamInfo, 
											   TUint32 aId, 
											   const char* aCategory)
{
	CFileManRequestInfo* self = new(ELeave) CFileManRequestInfo(aOwner, aMMObj, aId, aStreamInfo);
	CleanupStack::PushL(self);
	self->ConstructL(aCategory);
	CleanupStack::Pop(self);
	return self;
}

/**
* The first phase constructor for the CFileManRequestInfo class.
*
* @param aOwner The address of, or another quantity unique to the owning object. Must be valid when the object is cancelled, for example.
* @param aMMObj The MM_Object for the requesting movie.
* @param aStreamInfo The stream info from the core player
* @param aId  The identifier for this object, unique among all other objects owned by aOwner.
*/
CFileManRequestInfo::CFileManRequestInfo(void* aOwner, 
										 struct MM_Object* aMMObj, 
										 TUint32 aId, 
										 struct FI_URLStreamInfo* aStreamInfo)
:CHttpRequestInfo(aOwner, aMMObj, aStreamInfo, aId)
{}



/**
* The second phase constructor for the CFileManRequestInfo class.
*
* Leaves with a standard system error if unable to create the CFileManRequestInfo object.
*
* @param aCategory The string for the filemanager download category that this object represents.
*/
void CFileManRequestInfo::ConstructL(const char* aCategory)
{
 	User::LeaveIfError( iFs.Connect() );

	// PLAYERASSERT(aCategory);
	TPtrC8 catPtr((TUint8*)aCategory);

 
	// check the category is valid
	
	TInt resourceId=-1;

	if(catPtr.Compare(KLit8Ringtone) == 0)
	{
		resourceId=R_STR_FILEMAN_RINGTONE_LOCATION;
	} 
	else if(catPtr.Compare(KLit8WallPaper) == 0)
	{
		resourceId=R_STR_FILEMAN_WALLPAPER_LOCATION;
	}
	else if(catPtr.Compare(KLit8Applications) == 0)
	{
		resourceId=R_STR_FILEMAN_APPLICATIONS_LOCATION;
	}
	else
	{
		User::Leave(KErrNotSupported);
	}
 
    // Open the engine resource file -- this contains the device and platform strings.
	LoadResourceFileL(resourceId);
}

void CFileManRequestInfo::LoadResourceFileL(TInt aResourceId)
{
	TFileName dllName;
	TFileName rscfn;
 	Dll::FileName(dllName);
  	TBuf<2> drive = dllName.Left(2);			
	rscfn.Copy(drive); // get the drive letter
	rscfn.Append(KMMICustomResource);
	CEikonEnv* env = CEikonEnv::Static();
	BaflUtils::NearestLanguageFile(iFs, rscfn);
		
	TBool fileExists( BaflUtils::FileExists(iFs, rscfn) );
	if ( fileExists )
	{
	    iResourceOffset = env->AddResourceFileL(rscfn);		
	}
	else
	{
		User::Leave(KErrNotFound);
	}
	
	if (iResourceOffset < 0)
	{
		User::Leave(iResourceOffset);
	}
	TRAPD(err,iCategory=env->AllocReadResourceL(aResourceId));
	
	if(err!=KErrNone)
	{
		User::Leave(KErrNotSupported);
	}

}

/**
* Destructor for the CFileManRequestInfo class. Frees all owned resources.
*/
CFileManRequestInfo::~CFileManRequestInfo()
{
	delete iCategory;

	if(iDownloadFile.SubSessionHandle())
	{
		iDownloadFile.Close();
	}

	iFs.Close();
}

/**
* The PrepareToBeServicedL function is called by the object that is responsible for managing the queue of network data
* requests. It is called after the queued object reaches the head of the queue, and is about to be serviced.
*/
void CFileManRequestInfo::PrepareToBeServicedL()
{
	const TChar KCharBackSlash(KCharBackslash);
	const TChar KFwdSlash(KCharForwardSlash);
	
	TFileName* fn=new(ELeave) TFileName; // create on heap - it is 512bytes!
	CleanupStack::PushL(fn);
	fn->Copy(KFileRoot);	 

	fn->Append(*iCategory);
	fn->Append(KCharBackSlash);

	//RFs& fs=pEngine->GetRFs();

	// Make sure the directory exists

	TUint lAttValue = 0;
	// Check to see if the directory already exists
	TInt err = 	iFs.Att(*fn, lAttValue);

	if (err != KErrNone || !(lAttValue & KEntryAttDir) )
	{
		// Create all sub-directory hierarchy
		err = iFs.MkDirAll(*fn);
	}

	User::LeaveIfError(err);

	const TDesC8& url=Url();
	TInt lastBS=url.LocateReverse(KFwdSlash);

	// Append the file name
	TPtrC8 fnPtr(url.Right(url.Length()-1-lastBS));
	// convert to 16 bit
	HBufC* temp=AllocDes16FromDes8LC(fnPtr);
	fn->Append(*temp);
	CleanupStack::PopAndDestroy(temp);
	// open the file
	
	User::LeaveIfError(iDownloadFile.Replace(iFs, *fn, EFileWrite | EFileShareExclusive));
	
	CleanupStack::PopAndDestroy(fn);	
}

/**
* The HandleRequestFailed function is called by the object that is responsible for managing the queue of network data
* requests. It is called if the request fails to complete successfully.
*
* @param aError the reason the request failed - one of the system wide error codes.
*/
void CFileManRequestInfo::HandleRequestFailed(TInt aError, TInt /*aHttpStatusCode*/)
{
	NotifyCoreComplete(aError);
}
	
/**
* The HandleHeaderResponseL function is called by the object that is responsible for managing the queue of network data
* requests. It is called when the request is being serviced, and the header response has been received from the remote
* server.
*
* @param aHeaders - the HTTP headers that have been received.
* @param aFinalUrl - the Url that the headers were received from.
*/
void CFileManRequestInfo::HandleHeaderResponseL(struct FI_URLStreamHeaderInfo& aHeaders, 
												const TDesC8& /*aFinalUrl*/)
{
	// Get the total size
	for (TUint i = 0 ; i < aHeaders.numHeaders; i ++)
		{
			if (!aHeaders.headerNames[i])
				continue;
			TPtrC8 ptr((TUint8*)aHeaders.headerNames[i]);
			if ( ptr.Compare(KLitContLen) == 0 )
			{
				iTotalDownloadSize = GetIntFromString(aHeaders.headerValues[i]);
				break;
			}
		}
		
	if (iTotalDownloadSize)
	{
		TPtrC ptr(KFileRoot);
		TInt64 freeSpace = FreeSpaceOnVolume(ptr[0]);
		if ((freeSpace - iTotalDownloadSize) < 0)
			{
			//HandleRequestFailed(KErrDiskFull);
			}	

		NotifyCoreOfProgress();
	}
}

/**
* The FreeSpaceOnVolume function returns the number of bytes free on the volume.
* 
* @param aChar The drive letter of the drive that is being checked for free space.
* @return The number of free bytes on the volume, or -1 if an error occurred.
*/
TInt64 CFileManRequestInfo::FreeSpaceOnVolume(TChar aChar)
	{
	TInt64 retVal = -1;

    RFs fs;
    TInt err = fs.Connect();

	if ( err == KErrNone )
		{
    
		TVolumeInfo volInfo;
		TInt drive;
		TInt err = fs.CharToDrive(aChar,drive);
		if (err!=KErrNone)
			return -1;
		err = fs.Volume(volInfo,drive);
		if (err!=KErrNone)
			return -1;
		retVal = volInfo.iFree;

		fs.Close();
		}

	return retVal;
	}

/**
* The HandleResponseDataL function is called by the object that is responsible for managing the queue of network data
* requests. It is called when the request is being serviced, and a piece of response data has been received from the remote
* server.
*
* @param aDataPart - a descriptor containing the received data
* @return ETrue if the aDataPart has been consumed and can be freed by its owner, EFalse if the data must not be released yet.
*/
TBool CFileManRequestInfo::HandleResponseDataL(const TDesC8& aDataPart)
{
	
	User::LeaveIfError(iDownloadFile.Write(aDataPart));
	
	iCurrentDownloadSize += aDataPart.Length();
	NotifyCoreOfProgress();
	
	return ETrue;
}

/**
* The NotifyCoreOfProgress function is called by this object when a function wishes to inform the Core Player object 
* how much of the associated request has completed so far.
*/
void CFileManRequestInfo::NotifyCoreOfProgress()
{
	CMMI_Implementation::FI_FileManagerStatus(iMMObj,
												FI_FileManagerProgress,
												iCurrentDownloadSize,
												iTotalDownloadSize,
												Id());
}

/**
* The HandleResponseCompleteL function is called by the object that is responsible for managing the queue of network data
* requests. It is called when the request has completed being serviced successfully.
*/
void CFileManRequestInfo::HandleResponseCompleteL(TInt /*aHttpStatusCode*/) // request has completed normally
{
	// Close the file
	TInt error = iDownloadFile.Flush();
	if (error!=KErrNone)
		{
		iDownloadFile.Close();
		NotifyCoreComplete(-1);
		}
	else
		{
		iDownloadFile.Close();
		NotifyCoreComplete(KErrNone);
		}
}


/**
* The NotifyCoreComplete function is called by this object when a function wishes to inform the Core Player object 
* that issued the request that the request has completed.
*
* @param aError the system wide error code that the request completed with.
*/
void CFileManRequestInfo::NotifyCoreComplete(TInt aError)
{
	CMMI_Implementation::FI_FileManagerStatus(iMMObj,
												FI_FileManagerComplete,
												aError ? 0 : 1,
												0, // ignored
												Id());
}

#endif // FEATURE_FILE_MANAGER



/**
* First phase constructor for the CMMI_Implementation class
*/
CMMI_Implementation::CMMI_Implementation()

:iIsSMCheckNeeded(EFalse)

{}

/**
* Factory method to create and return a fully constructed CMMI_Implementation
* object.
*
* @return a new and fully constructed CMMI_Implementation onject, owned by the caller
*/
CMMI_Implementation* CMMI_Implementation::NewL()
{ // static
	CMMI_Implementation* self= new(ELeave) CMMI_Implementation();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}


void CMMI_Implementation::AddRegisteredEventL(struct MM_Object* /*pMMObj*/, const struct FI_Text& eventType)
{
    TPtrC16 eventTypePtr(eventType.text.utf16String, eventType.textLen);
	HBufC16* newEvent = eventTypePtr.AllocLC();
	User::LeaveIfError( iRegisteredEvents.Append(newEvent) );
	CleanupStack::Pop(newEvent);
	iMMICustomization->RegisterExtendedEventL(*newEvent);
	const TInt count=iDelegates->Count();
	CMMI_Delegate* temp;
	for(TInt i=0; i<count; i++)
	{
		temp=(*iDelegates)[i];
		if(temp->RegisterExtendedEvent(*newEvent))
		{
			break;
		}
	}
}

void CMMI_Implementation::RemoveRegisteredEvent(struct MM_Object* /*pMMObj*/, const struct FI_Text& eventType)
{
    TPtrC16 eventTypePtr(eventType.text.utf16String, eventType.textLen);

	TInt count = iRegisteredEvents.Count();
	TInt i;
	for(i=0;i<count;i++)
	{
		HBufC16* eventName = iRegisteredEvents[i];
		if(eventTypePtr.Compare(*eventName) == 0)
		{
			iRegisteredEvents.Remove(i);
			delete eventName;

			break;
		}
	}

	iMMICustomization->UnregisterExtendedEvent(eventTypePtr);
	
	count=iDelegates->Count();
	CMMI_Delegate* temp;
	for(i=0; i<count; i++)
	{
		temp=(*iDelegates)[i];
		if(temp->UnregisterExtendedEvent(eventTypePtr))
		{
			break;
		}
	}
}


//static 
short CMMI_Implementation::SI_RegisterForEvent(struct MM_Object* pMMObj, const struct FI_Text* eventType)
{
    CMMI_Implementation* imp = GetOutOfLineImp(pMMObj);
    if(eventType)
    {
        TRAPD(error, imp->AddRegisteredEventL(pMMObj, *eventType));
        if(error != KErrNone)
        {
            return 0;
        }
    }
    else
    {
        return 0;
    }
    return 1;
}
short CMMI_Implementation::SI_UnregisterEvent(struct MM_Object* pMMObj, const struct FI_Text* eventType)
{
	short retVal = 0;
	
    CMMI_Implementation* imp = GetOutOfLineImp(pMMObj);
    if(eventType)
    {
        imp->RemoveRegisteredEvent(pMMObj, *eventType);
        retVal = 1;
    }
    return retVal;
}


/**
* The FI_OfferKeyPressEvent function is allows a call to be made to the player with a key event.
* The movie can then be notified of the key press, if required.
*
* @return 1 for success, or a flash error code on failure
* @static
*/
short CMMI_Implementation::FI_OfferKeyPressEvent(struct MM_Object* pMMObj, const FI_KeyId* keyId)
{
	// this can be null if the player has been terminated...
	short result=0;
	
	if(pMMObj)
	{
	    result = pMMObj->pFIFncs->pFIKeyInputFncs->pFncFI_OfferKeyPressEvent( pMMObj, keyId );
	}
	
	return result;
}


#ifdef FEATURE_FILE_MANAGER

unsigned short CMMI_Implementation::FI_FileManagerStatus(struct MM_Object* pMMObj,
															short pStatusEvent,
															long pDataParamA,
															long pDataParamB,
															unsigned long pId)
{
	unsigned short result = 0;
	if(pMMObj && pMMObj->pFIFncs->pFIMMIFncs->pFncFI_FileManagerStatus)
	{
    	result = pMMObj->pFIFncs->pFIMMIFncs->pFncFI_FileManagerStatus(pMMObj,
																		pStatusEvent,
																		pDataParamA,
																		pDataParamB,
																		pId);
	}
	return result;
}

#endif // FEATURE_FILE_MANAGER

/**
* Second phase constructor for the CMMI_Implementation class.
*/
void CMMI_Implementation::ConstructL()
{
	iEnv = CEikonEnv::Static();
    iExtensionUid = KMMIExtensionUID;
	iMMICustomization = CFlashMMICustomization::NewL();

	iDelegates=new(ELeave) CArrayPtrFlat<CMMI_Delegate>(KDelegateArrayGranularity);
	iDelegates->Reset();
	
	iMMIFns=new(ELeave) SI_MMIFncs;
	Mem::FillZ(iMMIFns,sizeof(SI_MMIFncs));
	iMMIFns->pFncSI_Get=SI_Get;
    iMMIFns->pFncSI_Set=SI_Set;
    iMMIFns->pFncSI_GetVars=SI_GetVars;
    iMMIFns->pFncSI_SetVars=SI_SetVars;
    iMMIFns->pFncSI_Send=SI_Send;
    iMMIFns->pFncSI_RegisterForEvent=SI_RegisterForEvent;
	iMMIFns->pFncSI_RequestData=SI_RequestData;
	iMMIFns->pFncSI_CancelDataRequest=SI_CancelDataRequest;
    iMMIFns->pFncSI_UnregisterEvent=SI_UnregisterEvent;
	TFileName rscfn;
	TInt numDrives = KFlashResourceDrivesInOrder().Length();
	TBool foundFile = EFalse;

	for (TInt i=0; i<numDrives; i++)
	{
		rscfn.Copy(KFlashResourceDrivesInOrder().Mid(i,1)); // get the drive letter
		rscfn.Append(TChar(KDriveDelimiter));
		rscfn.Append(KMMICustomResource);
		BaflUtils::NearestLanguageFile(CCoeEnv::Static()->FsSession(), rscfn);
		// check for rsc file on 1st choice drive
		
		RFs& iFs=CEikonEnv::Static()->FsSession();
		if (BaflUtils::FileExists(iFs, rscfn))
		{
			foundFile = ETrue;
			break;
		}
	}

	// If we didn't find the resource file, then we should leave
	if (!foundFile)
	{
		User::Leave(KErrNotFound);
	}

 
    iResourceOffset=0;
    
    CEikonEnv* env = CEikonEnv::Static();
    iResourceOffset = env->AddResourceFileL(rscfn);

	if (iResourceOffset < 0)
	{
		User::Leave(iResourceOffset);
	}
}

/**
* The GetOutOfLineImp function, gets a pointer to the CMMI_Implementation from the host engine for the player
* instance represented by pMMObj.
*
* @return The CMMI_Implemenation for the player instance, or NULL if there is no implementation object instantiated for the player instance.
* @static
*/
CMMI_Implementation* CMMI_Implementation::GetOutOfLineImp(struct MM_Object* pMMObj)
{
	CMMI_Implementation* retVal=NULL;

	CFlashExtension* ext=pMMObj->pHostObj->iHostFncs->GetExtension(pMMObj, KMMIExtensionUID);
	if(ext)
	{
		retVal= static_cast<CMMI_Implementation*>(ext);
	}

	return retVal;
}

/**
* Destroys the CMMI_Implementation freeing all owned data.
*/
CMMI_Implementation::~CMMI_Implementation()
{

	if (iMMICustomization)
	{
		iMMICustomization->UnregisterExtendedEvents();
		delete iMMICustomization;
	}

	if(iDelegates)
	{
		iDelegates->ResetAndDestroy();
		delete iDelegates;
	}
	
	delete iMMIFns;
	
	if(iResourceOffset)
    {
		iEnv->DeleteResourceFile(iResourceOffset);
    }

	delete iAllowedExtensions;

	delete iMMIUtils;
}

/**
* The CancelDataRequestL method allows an outstanding data request to be cancelled.
*/
short CMMI_Implementation::CancelDataRequestL(struct MM_Object* pMMObj, FI_DataRequest *request){

	//  Get the array of delegates
	CMMI_Implementation* imp=GetOutOfLineImp(pMMObj);
	CArrayPtr<CMMI_Delegate>* array=imp->iDelegates;
	
	//  Send the cancel request to all delegates.  Delegate will determine if a specific request
	//  is associated with that delegate and handle the request properly.  Once a delegate tells
	//  us that it owns the request and has cancelled it, then we stop iterating.
	const TInt count=array->Count();
	CMMI_Delegate* temp;
	short sCancelled = 0;
	for(TInt i=0; sCancelled == 0 &&  i<count; i++)
	{
		temp=(*array)[i];
		sCancelled = temp->CancelRequestL(request);
	}
	return sCancelled;
}

/**
* The GetDelegateL method returns a pointer to the CMMI_Delegate with name aString, or NULL if unable to load
* a matching delegate.
*
* @see CMMI_Delegate
* @static
*/
CMMI_Delegate* CMMI_Implementation::GetDelegateL(struct MM_Object* aMMObj, 
												 const struct FI_Text* aString)
{
	if(!aString)
	{
		return NULL;
	}

	CMMI_Implementation* imp=GetOutOfLineImp(aMMObj);

	ASSERT(aString->textEncoding==FI_UTF16EncodingForm);
	
	TPtrC ptr(aString->text.utf16String, aString->textLen);

	//Ecom uses 8 bit names so must convert...
	HBufC8* name8=AllocDes8FromDes16L(ptr);
	CleanupStack::PushL(name8);

	CArrayPtr<CMMI_Delegate>* array=imp->iDelegates;
	CMMI_Delegate* ret=NULL;
	
	const TInt count=array->Count();
	CMMI_Delegate* temp;
	for(TInt i=0; i<count; i++)
	{
		temp=(*array)[i];
		if(0 == temp->Name()->Compare(*name8))
		{
			ret=temp;
			break;
		}
	}
	
	if(!ret) // delegate not loaded yet...
	{
		// first check to see if the name is on our allowed extensions list
		const TInt ecount = imp->iAllowedExtensions ? imp->iAllowedExtensions->MdcaCount() : 0;
		for (TInt i=0; i<ecount; i++)
		{
			if ((*imp->iAllowedExtensions)[i].Compare(*name8) == 0)
			{
				ret=CMMI_Delegate::NewL(*name8, imp->iMMIUtils);
				if(ret)
				{
					CleanupStack::PushL(ret);
					imp->iDelegates->AppendL(ret);
					CleanupStack::Pop(ret);

                    if (!name8->Compare(KLIWBinding8))
                        {
#ifndef __S60_32__
                        ret->ExecuteFunctionL(CMMI_Delegate::EOpCodeSMCheckNeeded, (TAny*)(imp->iIsSMCheckNeeded), NULL,NULL,NULL);
#endif
                        }
                    else
                        {

						for (TInt e=0; e<imp->iRegisteredEvents.Count(); e++)
							{
							ret->RegisterExtendedEvent(*imp->iRegisteredEvents[e]);
							}

						}

				}
				break;
			}
		}
	}

	CleanupStack::PopAndDestroy(name8);
	return ret;
}

/**
 * The DoSI_RequestDataL provides the implementation for the SI_RequestData method, but is allowed to Leave.
 *
 * @see SI_RequestData
 */
short DoSI_RequestDataL(struct MM_Object* pMMObj, unsigned short argc, 
						FI_Text* argv, 
						FI_DataRequest* pDataRequest)
{
	short result = 0;

	if (pMMObj) 
	{
		CMMI_Delegate* del=CMMI_Implementation::GetDelegateL(pMMObj, argv);
		if ( del )
		{
			FI_Text** pArgv=new(ELeave) FI_Text*[argc-1];
			CleanupStack::PushL(pArgv);
			for(TInt i=1;i<argc;i++)
			{
				pArgv[i-1]=&argv[i];
			}
			const RPointerArray<TDesC>* args=ConvertArgsToDesArrayLC((const FI_Text**)pArgv,argc-1);
			result= del->RequestDataL(*args, (TAny*)pDataRequest);
			CleanupStack::PopAndDestroy(2,pArgv); //args
		}
	}

	return result;
}

/**
 * The SI_RequestData function is called by the Core Player to make a request for data from an MMI content type plugin.
 * The request is to be serviced asynchronously.
 *
 * @return  0 if the request failed, 1 if it was made successfully.
 *
 * @static
 */
short CMMI_Implementation::SI_RequestData(struct MM_Object* pMMObj, 
										  unsigned short argc, 
										  FI_Text* argv, 
										  FI_DataRequest* pDataRequest)
{
	short result = 0;
	
	TRAP_IGNORE(result= DoSI_RequestDataL(pMMObj, argc,argv,pDataRequest));
	
	return result;	
}

/**
 * The DoSI_CancelDataRequestL provides the implementation for the SI_CancelDataRequest method, but is allowed to Leave.
 *
 * @see SI_CancelDataRequest
 */
short DoSI_CancelDataRequestL(struct MM_Object* pMMObj, FI_DataRequest* pDataRequest)
{
	short result = 0;

	if (pMMObj) 
	{
		result = CMMI_Implementation::CancelDataRequestL(pMMObj,pDataRequest);
	}

	return result;

}

/**
 * The SI_CancelDataRequest function is called by the Core Player to cancel an outstanding request for data from an MMI content type plugin.
 *
 * @return  0 if the cancel failed, 1 if successful.
 *
 * @static
 */
short CMMI_Implementation::SI_CancelDataRequest(struct MM_Object* pMMObj, 
												FI_DataRequest* pDataRequest)
{
	short result = 0;
	
	TRAP_IGNORE(result= DoSI_CancelDataRequestL(pMMObj, pDataRequest));
	
	return result;	
}

/**
 * The DoSI_GetL provides the implementation for the SI_Get method, but is allowed to Leave.
 *
 * @see SI_Get
 */
short DoSI_GetL(struct MM_Object* pMMObj, 
				unsigned short argc, 
				const FI_Text** argv, 
				FI_ReturnString* retVal)
{
	short res = 0;

	if(pMMObj) 
	{
		CMMI_Delegate* del = CMMI_Implementation::GetDelegateL(pMMObj, argv[0]);

		if(del)
		{	
			const RPointerArray<TDesC>* args=ConvertArgsToDesArrayLC(argv+1,argc-1);
			res = del->GetL(*args, retVal);
			CleanupStack::PopAndDestroy(); //args
		}
	}
	return res;
}

/**
 * The SI_Get function is called by the Core Player to make a request for data from an MMI content type plugin.
 * The request is serviced synchronously.
 *
 * @return  0 if the request failed, 1 if it was made successfully.
 *
 * @static
 */
short CMMI_Implementation::SI_Get(struct MM_Object* pMMObj, 
								  unsigned short argc, 
								  const FI_Text** argv, 
								  FI_ReturnString* retVal)
{
	short res=0;
	TRAP_IGNORE( res=DoSI_GetL(pMMObj, argc, argv, retVal));
	return res;
}

/**
 * The DoSI_SetL provides the implementation for the SI_Set method, but is allowed to Leave.
 *
 * @see SI_Set
 */
short DoSI_SetL(struct MM_Object* pMMObj, unsigned short argc, const FI_Text** argv, 
				struct FI_PlayerEventInfo* /*pEventInfo*/)
{
	short res = 0;

	// Got to have at least application name and the value to set
	if(argc>1)
	{
		if (pMMObj) 
		{	
			CMMI_Delegate* del = CMMI_Implementation::GetDelegateL(pMMObj, argv[0]);

			if(del)
			{
				const RPointerArray<TDesC>* args=ConvertArgsToDesArrayLC(argv+1,argc-2);
				TPtrC value(argv[argc-1]->text.utf16String, argv[argc-1]->textLen);
				res = del->SetL(*args, value);
				CleanupStack::PopAndDestroy(); //args
			}
		}
	}
	return res;
}

/**
 * The SI_Set function is called by the Core Player to make a request to set data in an MMI content type plugin.
 * The request is serviced synchronously.
 *
 * @return  0 if the request failed, 1 if it was made sucessfully.
 *
 * @static
 */
short CMMI_Implementation::SI_Set(struct MM_Object* pMMObj, 
								  unsigned short argc, 
								  const FI_Text** argv, 
								  struct FI_PlayerEventInfo* pEventInfo)
{
	short res=0;
	TRAP_IGNORE( res=DoSI_SetL(pMMObj, argc, argv, pEventInfo));
	return res;
}

/**
 * The DoSI_GetVarsL provides the implementation for the SI_GetVars method, but is allowed to Leave.
 *
 * @see SI_GetVars
 */
short DoSI_GetVarsL (struct MM_Object* pMMObj, 
					 unsigned short argc, 
					 const FI_Text** argv, 
					 FI_ReturnString* retVal)
{
	short res = 0;

	if (pMMObj) 
	{
		CMMI_Delegate* del = CMMI_Implementation::GetDelegateL(pMMObj, argv[0]);

		if (del)
		{
			const RPointerArray<TDesC>* args=ConvertArgsToDesArrayLC(argv+1,argc-1);
			res = del->GetVarsL(*args, retVal);
			CleanupStack::PopAndDestroy(); //args
		}
	}
	return res;
}

/**
 * The SI_GetVars function is called by the Core Player to make a request for a set of value/pair data 
 * from an MMI content type plugin. The request is serviced synchronously.
 *
 * @return  0 if the request failed, 1 if it was made sucessfully.
 *
 * @static
 */
short CMMI_Implementation::SI_GetVars(struct MM_Object* pMMObj, 
									  unsigned short argc, 
									  const FI_Text** argv, 
									  FI_ReturnString* retVal)
{
	short res=0;
	TRAP_IGNORE( res=DoSI_GetVarsL(pMMObj, argc, argv, retVal));
	return res;
}

/**
 * The DoSI_GetSarsL provides the implementation for the SI_SetVars method, but is allowed to Leave.
 *
 * @see SI_GetSetVars
 */
short DoSI_SetVarsL (struct MM_Object* pMMObj, 
					 unsigned short argc, 
					 const FI_Text** argv, 
					 struct FI_PlayerEventInfo* /*pEventInfo*/)
{
	short res = 0;
	
    if (pMMObj) 
	{
		CMMI_Delegate* del=CMMI_Implementation::GetDelegateL(pMMObj, argv[0]);
		
		if (del)
		{
			const RPointerArray<TDesC>* args=ConvertArgsToDesArrayLC(argv+1,argc-2);
			TPtrC value(argv[argc-1]->text.utf16String, argv[argc-1]->textLen);
			del->SetVarsL(*args, value, res);
			CleanupStack::PopAndDestroy();
		}
	}
	return res;
}

/**
 * The SI_SetVars function is called by the Core Player to make a request to set a set of value/pair data 
 * into an MMI content type plugin. The request is serviced synchronously.
 *
 * @return  0 if the request failed, 1 if it was made sucessfully.
 *
 * @static
 */
short CMMI_Implementation::SI_SetVars(struct MM_Object* pMMObj, 
									  unsigned short argc, 
									  const FI_Text** argv, 
									  struct FI_PlayerEventInfo* pEventInfo)
{
	short res=0;
	TRAP_IGNORE( res=DoSI_SetVarsL(pMMObj, argc, argv, pEventInfo));
	return res;
}
/**
 * The DoSI_SendL provides the implementation for the SI_Send method, but is allowed to Leave.
 *
 * @see SI_Send
 */
short DoSI_SendL (struct MM_Object* pMMObj, unsigned short argc, const FI_Text** argv, 
				  struct FI_PlayerEventInfo* /*pEventInfo*/)
{
	short res = 0;
	
	if (pMMObj) 
	{
		// verb is the first parameter, then content type 

		CMMI_Delegate* del = CMMI_Implementation::GetDelegateL(pMMObj, argv[1]);

		if (del)
		{
			const RPointerArray<TDesC>* args=ConvertArgsToDesArrayLC(argv+2,argc-2);
			TPtrC argv0(argv[0]->text.utf16String, argv[0]->textLen);
			res= del->SendL(argv0, *args);
			CleanupStack::PopAndDestroy();
		}
	}
	return res;
}

/**
 * The SI_Send function is called by the Core Player to send a command to an MMI content type plugin.
 * The request is serviced synchronously.
 *
 * @return  0 if the request failed, 1 if it was made sucessfully.
 *
 * @static
 */
short CMMI_Implementation::SI_Send(struct MM_Object* pMMObj, 
								   unsigned short argc, 
								   const FI_Text** argv, 
								   struct FI_PlayerEventInfo* pEventInfo)
{
	short res=0;
	TRAP_IGNORE( res=DoSI_SendL(pMMObj, argc, argv, pEventInfo));
	return res;
}

#ifdef FEATURE_FILE_MANAGER

/**
 * The Do_SI_FileManagerDownload function provides the implementation for the SI_FileManagerDownload method, but is allowed to Leave.
 *
 * @see SI_FileManagerDownload
 */
short CMMI_Implementation::Do_SI_FileManagerDownloadL(struct MM_Object* pMMObj, 
													  FI_URLStreamInfo* pStreamInfo, 
													  unsigned long id, 
													  const char* category, 
													  FI_PlayerEventInfo* /*pEventInfo*/)
{
	short result = 0;
	
	CMMI_Implementation* aThis=GetOutOfLineImp(pMMObj);

	CFileManRequestInfo* info = NULL;
	info = CFileManRequestInfo::NewL(aThis,pMMObj,
									pStreamInfo,id,category);

	result = pMMObj->pHostObj->iHostFncs->StartHttpRequestL(pMMObj, info);

	return result;
}

/**
 * The SI_FileManagerDownload function is called by the Core Player to queue a request to begin
 * a FileManger download.
 *
 * The request is serviced asynchronously.
 *
 * @return  0 if the request failed, 1 if it was made sucessfully.
 *
 * @static
 */
short CMMI_Implementation::SI_FileManagerDownload(struct MM_Object* pMMObj, 
												  FI_URLStreamInfo* pStreamInfo, 
												  unsigned long id, const char* category, 
												  FI_PlayerEventInfo* pEventInfo)
{
	short result=0;
	TRAP_IGNORE( result=CMMI_Implementation::Do_SI_FileManagerDownloadL(pMMObj,pStreamInfo,id,category,pEventInfo));
	return result;
}

/**
 * The SI_FileManagerCancelDownload function is called by the Core Player to cancel a previously made 
 * FileManger download.
 *
 * @return  0 if the request failed, 1 if it was made sucessfully.
 *
 * @static
 */
short CMMI_Implementation::SI_FileManagerCancelDownload(struct MM_Object* pMMObj, 
														unsigned long id, 
														FI_PlayerEventInfo* /*pEventInfo*/)
{
	short result = -1;

	CMMI_Implementation* aThis=GetOutOfLineImp(pMMObj);

	result = pMMObj->pHostObj->iHostFncs->CancelHttpRequest(pMMObj, aThis, id);
	
	if(result)
	{
		FI_FileManagerStatus(pMMObj,
								FI_FileManagerCancelComplete,
								0, // ignored
								0, // ignored
								id);
	}

	return result;
}
#endif	// FEATURE_FILE_MANAGER

/**
 * The InitExtensionSIFunctions function sets up the appropriate function pointers 
 * in SI_Fncs struct passed in with pointers to the handling functions provided by the 
 * CMMI_Implementation class.
 *
 * @param aSI_Fns The SI_Fncs structure to be initialized.
 */
void CMMI_Implementation::InitExtensionSIFunctions(SI_Fncs*& aSI_Fns)
{
	aSI_Fns->pSIMMIFncs = iMMIFns;
	
	iHostSI_Launch=aSI_Fns->pSIDeviceControlFncs->pFncSI_Launch;
	aSI_Fns->pSIDeviceControlFncs->pFncSI_Launch = SI_Launch;

	aSI_Fns->pSIDataAccessFncs->pFncSI_SetPersistentData=SI_SetPersistentData;

    aSI_Fns->pSIDataAccessFncs->pFncSI_GetMaxPersistentStorage=SI_GetMaxPersistentStorage;

#ifdef FEATURE_FILE_MANAGER
	aSI_Fns->pSIMMIFncs->pFncSI_FileManagerDownload=SI_FileManagerDownload;
	aSI_Fns->pSIMMIFncs->pFncSI_FileManagerCancelDownload=SI_FileManagerCancelDownload;
#endif	// FEATURE_FILE_MANAGER
}

void CMMI_Implementation::InitExtensionL(MM_Object* aMMObj)
{
	iMMObj = aMMObj;
	if (iMMIUtils)
	{
		delete iMMIUtils;
		iMMIUtils = NULL;
	}
	iMMIUtils = CMMIUtilsImpl::NewL(iMMObj,iEnv);
	iMMICustomization->SetMMIUtils(iMMIUtils);
}

//static 
/**
* The SI_SetPersistenData function is called by the Core Player to write data to a shared object.
* The request is serviced asynchronously.
*
* @return 0 failed to queue, -1 not supported, 1 queued successfully 
* @static 
*/
short CMMI_Implementation::SI_SetPersistentData(
												MM_Object* pMMObj, 
                                                const char* nameSpace, 
                                                const char* name,
                                                const unsigned char* data, 
                                                unsigned long dataSize, unsigned long aRequestId
												)
{
    short retVal=-1;

    TRAP_IGNORE(
		retVal = pMMObj->pHostObj->iHostFncs->SavePersistentDataL(pMMObj, 
                                                                  nameSpace, 
                                                                  name,
																  aRequestId,
                                                                  data, 
                                                                  dataSize,
                                                                  EFalse));
    return retVal;
}

/**
* The SI_GetMaxPersistenStorage function returns the maximum amount of space that shared objects can occupy.
* If this is exceeded, the oldest shared objects will be removed to free up space.
*
* @return 0 failed to determine max size, -1 not supported, 1 returned successfully 
* @static 
*/
short CMMI_Implementation::SI_GetMaxPersistentStorage(MM_Object* aMMObj, 
                                                      const char* /*aNameSpace*/,
                                                      unsigned long* aSize)
{
    if (!aMMObj)
	{
		return 0;
 	}
    *aSize = KMaxPersistentStoragePerMovie;
    return 1;
}


/**
* The SI_Launch function is called by the player to launch an external application.
* The MMI overload may choose to allow additional applications to be launched, and supports all other applications
* that can be launched by the non-MMI SI_Launch functionality also. 
*
* @return 0 failed to launch, -1 not supported, 1 launched successfully 
* @static 
*/
short CMMI_Implementation::SI_Launch (struct MM_Object* pMMObj, 
									  unsigned short argc, 
									  const FI_Text** argv, 
									  struct FI_PlayerEventInfo* pEventInfo, 
									  short isEmbedded)
{
	// Check for supported apps only...
	short retVal=0;


	CMMI_Implementation* imp=GetOutOfLineImp(pMMObj);

	TRAP_IGNORE( retVal=imp->DoLaunchL(pMMObj,argc,argv,pEventInfo,isEmbedded));

	return  retVal;
}

/**
* TCleanupOperation function used to clean up pointer array in CMMI_Implementation::DoLaunchL()
*
* @see CMMI_Implementation::Do_LaunchL.
*/
void CleanupArgArray(TAny* aArray)
{
	RPointerArray<HBufC>* array = (RPointerArray<HBufC>*)aArray;
	TInt len = array->Count();
	for (TInt i=len;i>0;i--)
	{
		HBufC* arg = (*array)[0];
		array->Remove(0);
		delete arg;
	}
}

/**
 * The Do_LaunchL function provides the implementation for the SI_Launch method, but is allowed to Leave.
 *
 * @see SI_Launch
 */
short CMMI_Implementation::DoLaunchL(struct MM_Object* pMMObj, 
									 unsigned short argc,
									 const FI_Text** argv, 
									 struct FI_PlayerEventInfo* pEventInfo, short isEmbedded)
{
	if(!argc)
	{
		return -1;
	}

	short status = -1;
	
	// add MMI specific SI_Launch handling code here...

	RPointerArray<HBufC> inArray;
	CleanupStack::PushL(TCleanupItem(CleanupArgArray, &inArray));

	// Create a pointer array of HBufC strings for the MMI customization library to
	// manipulate anyway it wants. The call to HandleLaunch() may allocate and insert
	// new strings, delete some of these strings, etc. We create a TCleanupItem to
	// handle freeing these strings if something leaves.
	TInt i;
	for (i=0;i<argc;i++)
	{
	    TPtrC arg(argv[i]->text.utf16String, argv[i]->textLen);
		HBufC* newArg = arg.AllocLC();
		User::LeaveIfError(inArray.Append(newArg));
		CleanupStack::Pop(newArg);
	}

	TBool handled = iMMICustomization->HandleLaunchL(inArray);
	TInt len = inArray.Count();
	if (handled)
	{
		unsigned short newArgc = (unsigned short)inArray.Count();
		FI_Text** newArgv = new(ELeave) FI_Text*[newArgc];
		CleanupStack::PushL(newArgv);
		for (i=0;i<len;i++)
		{
			TDesC* arg = inArray[i];
			newArgv[i] = new (ELeave) FI_Text;
			CleanupStack::PushL(newArgv[i]);
			newArgv[i]->text.utf16String = arg->Ptr();
			newArgv[i]->textEncoding = FI_UTF16EncodingForm;
			newArgv[i]->textLen = arg->Length();
		}
		status = iHostSI_Launch(pMMObj, newArgc, (const FI_Text**)newArgv, pEventInfo, isEmbedded);
	//	CleanupStack::PopAndDestroy(len-1);
	    
	    for (i=len-1;i==0;i--)
	    {
	     CleanupStack::PopAndDestroy(newArgv[i]);
	    }
	    CleanupStack::PopAndDestroy(newArgv);
	}
	else
	{
		status = iHostSI_Launch(pMMObj,argc,argv,pEventInfo,isEmbedded);
	}

	CleanupStack::PopAndDestroy(); // TCleanupItem

	inArray.Close();
	
	return status;
}



/**
 * The HandleExtendedKeyEventL function is called on a CFlashExtension derived object to allow the extension
 * to process the key event and/or act upon it.
 * 
 * @return ETrue if the keypress was acted upon by the extension, EFalse otherwise.
 */
TBool CMMI_Implementation::HandleExtendedKeyEventL(MM_Object* aMMObj, const TKeyEvent& aKeyEvent, TEventCode aType)
{
	FI_EventCode eventCode;

	if ( aType == EEventKeyDown )
	{
		eventCode = FI_EventKeyDown;
	}
	else if ( aType == EEventKeyUp )
	{
		eventCode = FI_EventKeyUp;
	}
	else if ( aType == EEventKey )
	{
		eventCode = FI_EventKey;
	}
	else
	{
		return EFalse;
	}


	KeyEventInfo eventInfo;
	eventInfo.downAndUpOnly = EFalse;

	TBool consumed = iMMICustomization->HandleKeyEvent(aKeyEvent, &eventInfo);

	if(consumed)
	{
		FI_Text textToPlayer ={ { ((char*) eventInfo.keyBuffer.PtrZ()) }, 
								FI_UTF16EncodingForm, eventInfo.keyBuffer.Length() };
		
		FI_KeyId toPlayer;
		toPlayer.keyName=&textToPlayer;
		toPlayer.keyCode=0;
		
        if (eventInfo.downAndUpOnly && (eventCode == FI_EventKeyDown))
		{
			// these keys only sends down and up event, so when we see down, 
			// send down and press to the player.
			FI_OfferKeyPressEvent(aMMObj, &toPlayer);
			//Core player takes care of sending down, press and up event. So no need to 
			//do this. Commented to prevent adding of double entries in contacts.
			//FI_OfferKeyPressEvent(aMMObj, &toPlayer);
		}
		else if(aKeyEvent.iScanCode==EStdKeyNo)
		{
			//for End key its not up or down, but text is displayed only on up or down listener events
			// now call the FI function to send it off to the player
			FI_OfferKeyPressEvent(aMMObj, &toPlayer);
		}
		else 
		{
			// now call the FI function to send it off to the player
			FI_OfferKeyPressEvent(aMMObj, &toPlayer);
		}
		return ETrue;
	}
	else
	{
		return EFalse;
	}
}

/**
 * The PlayerDestroyed function is called on a CFlashExtension derived object when the player is destroyed.
 * This allows the CFlashExtension to delete items associated with the player instance.
 */
void CMMI_Implementation::PlayerDestroyed()
{
	if (iMMICustomization)
	{
		iMMICustomization->UnregisterExtendedEvents();
	}

	TInt count=iDelegates->Count();
	TInt i;

	CMMI_Delegate* temp;
	for(i=0; i<count; i++)
	{
		temp=(*iDelegates)[i];
		temp->PlayerDestroyed();
	}

	iDelegates->ResetAndDestroy();

	count = iRegisteredEvents.Count();
	for (i=0; i<count; i++)
	{
		delete iRegisteredEvents[i];
	}
	iRegisteredEvents.Reset();

	delete iMMIUtils;
	iMMIUtils = NULL;
}

/**
 * The SetAllowedSubExtensionsL function is called on a CFlashExtension derived object by its owner to 
 * inform the extension which named plugin's it may load to extend its functionality. In the case of MMI
 * these plugins are Content extensions.
 *
 * @param aExtensions an array of names of the allowed extensions.
 */
void CMMI_Implementation::SetAllowedSubExtensionsL(const TDesC8* aExtensions)
{
    const TChar KExtensionSeparator = ',';

	if (iAllowedExtensions)
	{
		iAllowedExtensions->Reset();
	}
	else
	{
		iAllowedExtensions = new(ELeave) CDesC8ArrayFlat(1);
	}

    TBool extPresent = ETrue;
    if ( !aExtensions || (aExtensions->Compare(KNullDesC8)==0) )
        {
            extPresent = EFalse;
        }
    TPtrC8 ext(KLIWBinding8);
    iAllowedExtensions->AppendL(ext);
    
    if ( extPresent && ( iIsSMCheckNeeded==EFalse ) )
        {

			TPtrC8 tempExts;
			tempExts.Set(aExtensions->Mid(0));
			TInt pos = 0;
			while (pos != KErrNotFound)
				{
				pos = tempExts.Locate(KExtensionSeparator);
				if (pos == KErrNotFound)
					{
					iAllowedExtensions->AppendL(tempExts);
					}
				else
					{
					TPtrC8 ext = tempExts.Left(pos);
					iAllowedExtensions->AppendL(ext);
					tempExts.Set(tempExts.Mid(pos+1));
					}

            }

	}
}


/**
* The NotifyPauseState function is called by the PlatformFlashEngine to 
* notify MMI that the pause flags have changed.
*/
void CMMI_Implementation::NotifyPauseState(TInt aPauseFlags)
	{
	TInt count=iDelegates->Count();
	TInt i;

	CMMI_Delegate* temp;
	for(i=0; i<count; i++)
		{
		temp=(*iDelegates)[i];
		TRAPD( err, temp->ExecuteFunctionL(CMMI_Delegate::EOpCodePauseState, (TAny*)aPauseFlags,NULL,NULL,NULL));
		if ( err == KErrNone )
			{
			}
		}
	}


/**
* The NotifyPauseState function is called by the PlatformFlashEngine to 
* notify MMI that the pause flags have changed.
*/
void CMMI_Implementation::NotifyIncomingCall(TBool aStatus)
	{
		if (iMMICustomization)
		{
			iMMICustomization->NotifyIncomingCall(aStatus);
		}
	}


// MMI delegate loading

/**
* The NewL function is a factory method to allow creation of CMMI_Delegate objects
* through ECom.
*
* Leaves on error, with a Symbian error code.
*
* @param aMatch the name of the plugin to be loaded.
* @return the fully constructed plugin.
*/
CMMI_Delegate* CMMI_Delegate::NewL(const TDesC8& aMatch, MMIUtils* aMMIUtils)
{
	const TUid KCMMIServiceExtInterfaceUid = {0x101FB453};

	TEComResolverParams resolverParams;
	resolverParams.SetDataType (aMatch);
	resolverParams.SetWildcardMatch (EFalse); //exact name or nothing

	TAny* interface =NULL;
	interface = REComSession::CreateImplementationL (KCMMIServiceExtInterfaceUid,
		_FOFF (CMMI_Delegate, iDtor_ID_Key),resolverParams);

	CMMI_Delegate* del = reinterpret_cast <CMMI_Delegate*> (interface);
	del->Initialize(aMMIUtils);
	return del;
}

// non member functions

/**
 * The AllocDes8FromDes16L function is a utility function to create a newly allocated HBufC8 from a 16-bit
 * descriptor 
 *
 * Leaves on error.
 * @return returns the newly constructed HBufC on the cleanupstack.
 */

HBufC8* AllocDes8FromDes16L(const TDesC16& aDes)
{
	const TInt len=aDes.Length();
	HBufC8* des8=HBufC8::NewL(len);
	TPtr8 ptr8=des8->Des();
	
	TUint8 ch;
	for(TInt i=0;i<len;i++)
	{
		ch=(TUint8)aDes[i];
		ptr8.Append(ch);
	}

	return des8;
}

/**
 * The GetIntFromString function is a utility function to return an integer
 * type whose value is set to the value of the integer in a string.
 *
 * @param aString the string containing the numeric infomation.
 * @return returns the integer value of numeric string.
 */
TInt GetIntFromString(const char* aString)
{
	// PLAYERASSERT(aString);
	const TPtrC8 ptr((TUint8*)aString);
	const TInt len=ptr.Length();

	TInt ret=0;
	TInt j;
	TInt digit;

	for(TInt i=0; i<len; i++)
	{
		digit=(ptr[i]);
		digit-=KAsciiNill; //ascii code for 0
		for(j=0;j<(len-i-1);j++)
		{
			digit*=10;
		}
		ret+=digit;
	}
	return ret;
}

/**
 * The AllocDes16FromDes8LC function is a utility function to create a newly allocated HBufC from an 8-bit
 * descriptor 
 *
 * Leaves on error.
 * @return returns the newly constructed HBufC on the cleanupstack.
 */
HBufC* AllocDes16FromDes8LC(const TDesC8& aDes)
{
	const TInt len=aDes.Length();

	HBufC* wide=HBufC::NewLC(len);
	TPtr widePtr=wide->Des();
	widePtr.SetLength(len);

	TUint16 wideChar;
	for(TInt i=0;i<len;i++)
	{
		wideChar=aDes[i];
		widePtr[i]=wideChar;
	}

	return wide;

}

/**
 * The AllocDes16FromStringLC function is a utility function to create a newly allocated HBufC from a zero terminated
 * string.
 *
 * Leaves on error.
 * @return returns the newly constructed HBufC on the cleanupstack.
 */
HBufC* AllocDes16FromStringLC(const char* aString)
{
	// PLAYERASSERT(aString);
	const TPtrC8 wrap((TUint8*)aString);
	return AllocDes16FromDes8LC(wrap);
}

/**
 * The NewFlashExtensionL function returns a newly constructed CFlashExtension of type CMMI_Implementation
 * to the caller.
 *
 * Leaves on error.
 * @return returns the newly constructed object.
 */
TAny* NewFlashExtensionL()
{
	return CMMI_Implementation::NewL();
}

_LIT8(KLitFlashMMICustomization, "mmicustom");

CFlashMMICustomization* CFlashMMICustomization::NewL()
{
	const TUid KCFlashMMICustomizationInterfaceUid = {0x1027367d};

	TEComResolverParams resolverParams;
	resolverParams.SetDataType(KLitFlashMMICustomization());
	resolverParams.SetWildcardMatch (EFalse); //exact name or nothing

	TAny* interface = REComSession::CreateImplementationL (KCFlashMMICustomizationInterfaceUid,
		_FOFF (CFlashMMICustomization, iDtor_ID_Key),resolverParams);

	return reinterpret_cast <CFlashMMICustomization*> (interface);
}

const RPointerArray<TDesC>* ConvertArgsToDesArrayLC(const FI_Text** aArray, TInt aCount)
{

	class CArgsWrapper : public CBase
	{
	public:
		static CArgsWrapper* NewLC(const FI_Text** aArray, TInt aCount)
		{
			CArgsWrapper* self=new(ELeave) CArgsWrapper;
			CleanupStack::PushL(self);
			self->ConstructL(aArray,aCount);
			return self;
		}
		~CArgsWrapper()
		{
			iArray.Close();
			delete[] iDesArray;
		}

		const RPointerArray<TDesC>* Array() const { return &iArray; }
	private:
		void ConstructL(const FI_Text** aArray, TInt aCount)
		{
			if(aCount>0)
			{
				iDesArray=new(ELeave) TPtrC[aCount];
			
				TInt i;
				for (i=0;i<aCount;i++)
				{
					iDesArray[i].Set(aArray[i]->text.utf16String, aArray[i]->textLen);
					User::LeaveIfError(iArray.Append(&(iDesArray[i])));
				}
			}
		}
	private:
		TPtrC* iDesArray;
		RPointerArray<TDesC> iArray;
	};

	CArgsWrapper* argWrap = CArgsWrapper::NewLC(aArray,aCount);
	return argWrap->Array();
}

#ifndef __S60_32__
TAny* CMMI_Implementation::ExecuteFunctionL(TInt aOpCode, TAny* aParam1, TAny* /*aParam2*/, 
    TAny* /*aParam3*/, TAny* /*aParam4*/)
#else
TAny* CMMI_Implementation::ExecuteFunctionL(TInt /*aOpCode*/, TAny* /*aParam1*/, TAny* /*aParam2*/, 
    TAny* /*aParam3*/, TAny* /*aParam4*/)
#endif
{
#ifndef __S60_32__
    if (CMMI_Delegate::EOpCodeSMCheckNeeded==aOpCode)
        {
        iIsSMCheckNeeded = (TBool)aParam1;
      }
#endif
  return NULL;
}


