/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the utility class
*
*/


#include "mmiutilsimpl.h"
#include "flashliteinterfaces.h"
#include "flashMMIinterfaces.h"

// static
CMMIUtilsImpl* CMMIUtilsImpl::NewL(MM_Object* aMMObj, CEikonEnv* aEnv)
{
	CMMIUtilsImpl* obj = new(ELeave)CMMIUtilsImpl(aMMObj, aEnv);
	return obj;
}

CMMIUtilsImpl::CMMIUtilsImpl(MM_Object* aMMObj, CEikonEnv* aEnv)
:iEnv(aEnv), iMMObj(aMMObj)
{
}

//static
short CMMIUtilsImpl::SetReturnStringFromKeyValuePairsL(const CArrayFix<TMMIKeyValuePair>* aArray, const TAny* aRetString)
{
	FI_ReturnString* retString = (FI_ReturnString*)aRetString;

	const TInt count=aArray->Count();

	const TMMIKeyValuePair* kvp=NULL;

	TInt lenNeeded=0;
	for(TInt i=0;i<count;i++)
	{
		kvp=&(*aArray)[i];
		lenNeeded+=kvp->iKey->Length()+2+kvp->iVal->Length(); // 1 for equals sign, 1 for & or null terminator.
	}

	HBufC* buffer = HBufC::NewLC(lenNeeded);
	TPtr ptr = buffer->Des();
	
	// copy stuff
	const TChar equals('=');
	const TChar separator('&');

	const TDesC* key;
	const TDesC* val;

	for(TInt k=0;k<count;k++)
	{
		kvp = &(*aArray)[k];
		key = kvp->iKey;

		ptr.Append(*key);
		ptr.Append(equals);

		val = kvp->iVal;

		ptr.Append(*val);

		if(k < (count-1))
		{
			ptr.Append(separator);
		}
	}
	
	ptr.ZeroTerminate();

	short res = retString->SetUTF16String(retString, ptr.Ptr());

	CleanupStack::PopAndDestroy(buffer);
	return res;
}

//static
short CMMIUtilsImpl::SetReturnStringFromIntL(TInt aInt, const TAny* aRetString)
{
	TBuf<30> buf; // long enough for any int
	buf.AppendNum(aInt);
	buf.ZeroTerminate();

	FI_ReturnString* retString = (FI_ReturnString*)aRetString;

	return retString->SetUTF16String(retString, buf.Ptr());
}

//static
short CMMIUtilsImpl::SetReturnStringFromDes16L(const TDesC& aDes, const TAny* aRetString)
{
	const TInt length=aDes.Length()+1; // for zero terminator
	HBufC* buf=HBufC::NewL(length);
	TPtr ptr=buf->Des();
	
	ptr.Copy(aDes);
	ptr.ZeroTerminate();

	FI_ReturnString* retString = (FI_ReturnString*)aRetString;

	short ret = retString->SetUTF16String(retString, ptr.Ptr());

	delete buf;	

	return ret;
}

//static 
TInt CMMIUtilsImpl::GetIntFromDescL(const TDesC& aDesc)
{
	const TInt len = aDesc.Length();
	if(!len)
	{
		User::Leave(KErrArgument);
	}

	TInt ret=0;
	TInt j;
	TInt digit;

	for(TInt i=0; i<len; i++)
	{
		digit=(aDesc[i]);
		digit-=48; //ascii code for 0
		for(j=0;j<(len-i-1);j++)
		{
			digit*=10;
		}
		ret+=digit;
	}
	return ret;
}

FI_Text* FI_TextFromDesc(const TDesC& aDesc)
{
	FI_Text* text = new FI_Text();
	if (text)
	{
		TInt len = aDesc.Length();
		text->textEncoding = FI_UTF16EncodingForm;
		text->textLen = len;
		text->text.utf16String = new FI_UTF16Char[len+1];
		if (text->text.utf16String)
		{
			Mem::FillZ((TAny*)text->text.utf16String, (len+1)*sizeof(FI_UTF16Char));
			Mem::Copy((TAny*)text->text.utf16String, aDesc.Ptr(), len*sizeof(FI_UTF16Char));
		}
	}
	return text;
}

//static
void CMMIUtilsImpl::DataRequestSetStringProperty(const TAny* aRequestHandle, const TDesC& aKey, const TDesC& aValue)
{
	FI_DataRequest* request = (FI_DataRequest*)aRequestHandle;
	FI_Text* keyText = FI_TextFromDesc(aKey);
	FI_Text* valueText = FI_TextFromDesc(aValue);
	FI_DataRequest_SetStringProperty(request, keyText, valueText);
	if (keyText)
	{
		delete (FI_UTF16Char*)keyText->text.utf16String;
		delete keyText;
	}
	if (valueText)
	{
		delete (FI_UTF16Char*)valueText->text.utf16String;
		delete valueText;
	}
}

//static
void CMMIUtilsImpl::DataRequestSetNumberProperty(const TAny* aRequestHandle, const TDesC& aKey, TInt aValue)
{
	FI_DataRequest* request = (FI_DataRequest*)aRequestHandle;
	FI_Text* keyText = FI_TextFromDesc(aKey);
	FI_DataRequest_SetNumberProperty(request, keyText, aValue);
	if (keyText)
	{
		delete (FI_UTF16Char*)keyText->text.utf16String;
		delete keyText;
	}
}

//static
TAny* CMMIUtilsImpl::DataRequestCreateObjectProperty(const TAny* aRequestHandle, const TDesC& aKey)
{
	FI_DataRequest* request = (FI_DataRequest*)aRequestHandle;
	FI_Text* keyText = FI_TextFromDesc(aKey);
	TAny* handle = (TAny*)FI_DataRequest_CreateObjectProperty(request, keyText);
	if (keyText)
	{
		delete (FI_UTF16Char*)keyText->text.utf16String;
		delete keyText;
	}
	return handle;
}

//static
void CMMIUtilsImpl::DataRequestSetComplete(const TAny* aRequestHandle, TUint8 aSuccess)
{
	FI_DataRequest* request = (FI_DataRequest*)aRequestHandle;
	FI_DataRequest_SetDataRequestComplete(request, aSuccess);
}

void CMMIUtilsImpl::SendEvent(RPointerArray<TDesC>& aArgs)
{
	if(iMMObj && iMMObj->pFIFncs->pFIMMIFncs->pFncFI_SendEvent)
	{
		TUint16 count = (TUint16)aArgs.Count();
		FI_Text** textArgs = new FI_Text*[count];
		if (textArgs)
		{
			TInt i;
			for (i=0;i<count;i++)
			{
				textArgs[i] = FI_TextFromDesc(*aArgs[i]);
			}
			iMMObj->pFIFncs->pFIMMIFncs->pFncFI_SendEvent(iMMObj, count, (const FI_Text**)textArgs);
			for (i=0;i<count;i++)
			{
				delete[] textArgs[i]->text.utf16String;
				delete textArgs[i];
			}
			delete[] textArgs;
		}
	}
}

