﻿/*
* ============================================================================
*  Name        : Service.as
*  Part of     : S60 Flash API for Platform Services
*  Description : Service class to access device services
*                
*  Copyright © 2008-2009 Nokia. All rights reserved.
*  This material, including documentation and any related 
*  computer programs, is protected by copyright controlled by 
*  Nokia. All rights are reserved. Copying, including 
*  reproducing, storing, adapting or translating, any 
*  or all of this material requires the prior written consent of 
*  Nokia. This material also contains confidential 
*  information which may not be disclosed to others without the 
*  prior written consent of Nokia
* ============================================================================
*/

/*
  Class: com.nokia.lib.Service

  This class offers methods to access device services. A content can load a service provider and
  request a service from the service provider.
  
*/
dynamic intrinsic class com.nokia.lib.Service {
	/*
	 Constructor.
	 
  	 Initializes the service parameters and loads the service provider.
	 
	
	 Parameters:
		 aServiceName - Name of the service provider.
		 aInterfaceName - Name of the interface. 
		 aInitialParam -  Initial parameter to be passed during the service instantiation.
		
	 Returns:
		 Throws exception if service or interface name is null.
	*/
	public function Service(aServiceName:String, aInterfaceName:String, aVersion:Object, aInitialParam:Object);
	
	/*
	 Returns the requested service name.
	
	 Parameters:
		 None.
	
	 Returns:
		 Returns the requested service name.
	*/
	public function Name():String;

	/*
	 Returns the requested interface name.
	
	 Parameters:
		 None.
	
	 Returns:
		 Returns the requested interface name.
	*/
	public function InterfaceName():String;

	/*
	 Cancels an ongoing asynchronous operation identified by the transactionID parameter.
	
	 Parameters:
		 aTransID - Transaction ID of an ongoing asynchronous operation that has to be cancelled.
	
	 Returns:
		 Returns true if successful.
	*/
	public function Cancel(aTransID:Number):Boolean;
}
