/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the Iterator class. The Iterator class provides iterator data type implementation.
 
 *
*/


#ifndef __ITERATOR_IMPL_H__
#define __ITERATOR_IMPL_H__

#include <e32base.h> // CBase
#include "flashliteExtensionInterfaces.h"

class TLiwVariant;

class Iterator
    {
public:
    /**
     Method to construct a new Iterator object.

     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player.
     */
    static void Constructor(FnInfo* fnInfo);

    /**
     static Method to call Destructor of Iterator object.
     
     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player and contains
     argument Info and ExtensionInterface Pointer.
     */
    static void Destructor(FnInfo* fnInfo);

    /**
     static Reset to Reset the LIW specific iterator whic is a memeber varibale. 
     
     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player and contains
     argument Info and ExtensionInterface Pointer.
     */
    static void Reset(FnInfo* fnInfo);

    /**
     static Method Next which is used to fetch next entry from LIW Iterator Variable 
     
     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player and contains
     argument Info and ExtensionInterface Pointer.
     */
    static void Next(FnInfo* fnInfo);
    /**
     static Method Close which is used to close the iterator as frees the memory. 
     
     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player and contains
     argument Info and ExtensionInterface Pointer.
     */    
    static void Close(FnInfo* aFnInfo);

    static ClassInfo* GetClassInfoL();

private:
    /**
     Constructor
     */
    Iterator()
        {
        iLiwVar = NULL;
        }
    ;
    /**
     Destructor
     */
    ~Iterator()
        {
        }
    ;

    /**
     Function to Handle Next() request of Iterator
     
     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player and contains
     argument Info and ExtensionInterface Pointer.
     */
    void HandleNextL(FnInfo* aFnInfo, TInt& aIsItemFound);

    TLiwVariant* iLiwVar;

    };

#endif //__ITERATOR_IMPL_H__
