/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the CServiceProxy class. The CServiceProxy class interacts with LIW framework to load 
 the specified service provider and request the services to the 
 provider. 
 
 *
*/


#ifndef __SERVICEPROXY_H__
#define __SERVICEPROXY_H__

#include <e32std.h>	
#include <stdio.h>
#include <liwcommon.h> 
#include "flashliteExtensionInterfaces.h"
#include "flashliteinterfaces.h"
#include "flashengineintftoasextension.h"
#include "saplayer_build.h"

class CFlashBinding;
class CLiwGenericParamList;
class CBindingHelper;
class TAsyncReqIDMap;
class CAsyncEvent;

class ServiceUtilsProxy
    {
public:

    static void GetServiceList(FnInfo* aFnInfo);

    static ClassInfo* GetClassInfoL();

private:
    static void HandleGetServiceListL(FnInfo* aFnInfo,
            FL_OBJECT_HANDLE* aOutputObj);

    };


class CMapProxy: public CBase
    {
public: 
    static ClassInfo* GetClassInfoL();
    static void Constructor(FnInfo* aFnInfo);   
    
    static void Destructor(FnInfo* aFnInfo);    
    
    static void Resolve(FnInfo* aFnInfo);   
    
    static CMapProxy* CreateInstanceL(FnInfo* aFnInfo);
    
private:
    ~CMapProxy()
        {
        iVariant.Reset();
        }
    
    void HandleResolveL(FnInfo* aFnInfo);
    
    TLiwVariant iVariant;
    };


class CServiceProxy : public CBase, public MLiwNotifyCallback
    {
public:

    /**
     Method to construct a new CServiceProxy object.

     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player.
     */
    static void Constructor(FnInfo* aFnInfo);
    /**
     static Method to call Destructor of CServiceProxy object.
     
     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player and contains
     argument Info and ExtensionInterface Pointer.
     */
    static void Destructor(FnInfo* aFnInfo);

    /**
     Loads the LIW Interface instance which mapped to service class.      
     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player and contains
     argument Info and ExtensionInterface Pointer.
     */
    static void Load(FnInfo* aFnInfo);

    /**
     Handles the ASync request.
     
     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player and contains
     argument Info and ExtensionInterface Pointer.    
     */
    static void ExecuteAsyncReq(FnInfo* aFnInfo);

    /**
     Handles the Sync request.
     
     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player and contains
     argument Info and ExtensionInterface Pointer.    
     */
    static void ExecuteSyncReq(FnInfo* aFnInfo);

    /**
     Cancels Ongoing ASync Request.
     
     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player and contains
     argument Info and ExtensionInterface Pointer.    
     */
    static void CancelAsyncReq(FnInfo* aFnInfo);

    /**
     Unloads a Service Provider.
     
     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player and contains
     argument Info and ExtensionInterface Pointer.    
     */
    static void UnloadService(FnInfo* aFnInfo);

    /*
     Hanldles corePlayer update events. 
     */
    static void Update(FnInfo* aFnInfo);

    static ClassInfo* GetClassInfoL();   

private:

    /**
     Constructor 
     */
    inline CServiceProxy()
        {
        iLiwInterface = NULL;
        iFlashBinding = NULL;
        iExtInterface = NULL;
        isCorePlayerUpdateRunning = EFalse;
        }
    ;
    /**
     Destructor
     */
    inline ~CServiceProxy()
        {
        Destroy();
        }
    ;

    /**
     Hanldles deleting up the member variables.	  
     */
    void Destroy();

    /**
     Handles Loading of Service Provider Interace. Function can leave so it has been called by Load() under TRAP.
     
     @param fnInfo pointer of FnInfo which is specific to Function called by Core Player and contains
     argument Info and ExtensionInterface Pointer.    
     */
    void HandleLoadL(FnInfo* aFnInfo);

    TInt HandleNotifyL(TInt aCmdId, TInt aEventId,
            CLiwGenericParamList& aEventParamList,
            const CLiwGenericParamList & aInParamList);

    TInt HandleAsyncReturnL(TInt aCmdId, TInt aEventId,
            CLiwGenericParamList& aOutParamList);

    void HandleAsyncInterfaceExistsL(FnInfo* aFnInfo);

    void HandleSyncInterfaceExistsL(FnInfo* aFnInfo,
            FL_OBJECT_HANDLE* aOutputObj);

    void HandleCancelAsyncReqL(FnInfo* aFnInfo);

    TInt GetAsyncID(TInt aTransID);

    void CleanAsyncResource(TInt aTransID);
    void ResumePlayerL(TBool aBringToForeground);
    
    static void CleanUpArray(TAny* aArray);
    
    void PrepareExtensionArgL(RASExtensionArgArray& aArray, TBool aBringToForeground);
    
    void GetNotificationSettings(TInt aTransID,TInt& aResumeOnEvent, TInt& aBringToForeground);

private:
    //members
    MLiwInterface* iLiwInterface;

    CFlashBindingASExtDelegate* iFlashBinding;

    RPointerArray<CAsyncEvent> iEventQueue;
    
    ExtensionInterface* iExtInterface;
    
    RCriteriaArray iCritArray;
    
    RPointerArray<TAsyncReqIDMap> iAsyncIDMap;
    TBool isCorePlayerUpdateRunning;
    };

#endif //__SERVICEPROXY_H__
