/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the member functions of BindingHelper class.

 *
*/


#include <stdlib.h>
#include <e32const.h>
#include <escapeutils.h>
#include <eikenv.h>
#include <e32math.h>

#include "iteratorimpl.h"
#include "bindinghelper.h"
#include "debuglog.h"
#include "commonconsts.h"
#include "asextconsts.h"

void BindingHelper::ConstructInputParamsL(FnInfo* aFnInfo, FL_OBJECT_HANDLE aObject, CLiwGenericParamList& aInpsList)
{
BINDINGTRACEF("BindingHelper.ConstructInputParamsL< ++");
    ExtensionInterface* lExtInterface = aFnInfo->pEI;
    //get no of property
    TInt lPropCnt = 0;
    lExtInterface->getPropertyCount(lExtInterface, aObject, &lPropCnt);

    FL_VAR_HANDLE lPropValue= NULL;

    for (TInt i = 0; i < lPropCnt; i++)
    {
        CFlvVarHandle* lVarHandle = CFlvVarHandle::NewLC(aFnInfo->pEI);

        lPropValue = lVarHandle->Handle();

        if (lPropValue)
        {
            char** lPropName= NULL;
            lPropName = new(ELeave) char*[1];
            CleanupStack::PushL((TAny*)lPropName);
            //get the key value pair of property
            lExtInterface->getPropertyByIndex(lExtInterface, aObject, i, lPropValue, (const char**)lPropName);
            if (lPropName)
            {
            BINDINGTRACEF("BindingHelper.ConstructInputParamsL< ++ %s", *lPropName);
            TPtrC8 lPtr(reinterpret_cast<const TUint8*>(*lPropName));

                //check first two properties as if we use var a:Object = new Object() will contain first 2 properties
                // as __resolve, __proto__  
                //but with var a:Object Object(); above properties will not be there.
                //so put hardcode check to ignore  __resolve, __proto__, __consturctor__.
                if (BindingHelper::IsInternalProperty(lPtr))
                {
                    CleanupStack::Pop(); //pop propName

                    delete lPropName;
                    lPropName = NULL;

                    CleanupStack::PopAndDestroy();//destroys pVarHandle
                    continue;
                }
                else
                {
                    TLiwVariant lLiwVar;
                    lLiwVar.PushL();

                    TBool lIsVartypeNull;
                    ConstructInputParamFromNodeL(aFnInfo, lLiwVar, lPropValue, lIsVartypeNull);
                    if (!lIsVartypeNull)
                    {
                        TLiwGenericParam pLiwGenericParam(lPtr, lLiwVar);
                        pLiwGenericParam.PushL();
                        aInpsList.AppendL(pLiwGenericParam);

                        CleanupStack::Pop(&pLiwGenericParam);
                        pLiwGenericParam.Reset();
                    }

                    CleanupStack::Pop(&lLiwVar);
                    lLiwVar.Reset();

                    CleanupStack::Pop(); //pop propName

                    delete lPropName;
                    lPropName = NULL;
                }
            }
        }
        CleanupStack::PopAndDestroy();//destroys pVarHandle
    }//for     


BINDINGTRACEF("BindingHelper.ConstructInputParamsL> --");
}

void BindingHelper::ConstructInputParamFromNodeL(FnInfo* aFnInfo, TLiwVariant& aInpVariant, FL_VAR_HANDLE aVar, TBool &aIsVartypeNull)
{
BINDINGTRACEF( "BindingHelper.ConstructInputParamFromNodeL< ++");
            FLExtensionVarType lTypeOfVariable;

    ExtensionInterface* lExtInterface = aFnInfo->pEI;

    lExtInterface->getType(lExtInterface, aVar, &lTypeOfVariable); //get type of variable

    aIsVartypeNull = EFalse;
    switch (lTypeOfVariable)
    {
        case kExtnVarTypeInteger:
        {
            HandleInt(aFnInfo, aInpVariant, aVar);
            break;
        }
        case kExtnVarTypeNumber:
        {
            HandleFloat(aFnInfo, aInpVariant, aVar);
            break;
        }
        case kExtnVarTypeString:
        {
            HandleStringL(aFnInfo, aInpVariant, aVar);
            break;
        }
        case kExtnVarTypeString2:
        {
            HandleStringL(aFnInfo, aInpVariant, aVar);
            break;
        }
        case kExtnVarTypeBoolean:
        {
            HandleBool(aFnInfo, aInpVariant, aVar);
            break;
        }
        case kExtnVarTypeObject:
        {
            HandleObjectL(aFnInfo, aInpVariant, aVar);
            break;
        }
        case kExtnVarTypeNull:
        default:
            aIsVartypeNull = ETrue;

    }
BINDINGTRACEF( "BindingHelper.ConstructInputParamFromNodeL> --");
}

/**
 Handle array object and converts into LIW specific List Object.
 **/
void BindingHelper::HandleListL(FnInfo* aFnInfo, TLiwVariant& aListVar, FL_OBJECT_HANDLE aObject)
{
BINDINGTRACEF    ("BindingHelper.HandleListL< ++");

            ExtensionInterface* lExtInterface = aFnInfo->pEI;

    if (aObject)
    {
        CLiwList* lList = CLiwDefaultList::NewL();
        lList->PushL();

        FL_VAR_HANDLE lVar= NULL;
        TInt lNoOfArrayElements = -1;
        lExtInterface->createVariable(lExtInterface, &lVar, NULL);
        //get lenght of array in var
        if (lVar)
        {
            lExtInterface->getPropertyByName(lExtInterface, aObject, (char*)KStrLength, lVar);
            lExtInterface->getInt(lExtInterface, lVar, &lNoOfArrayElements);//get length of Array in an Int variable.
            //must free var's memory
            lExtInterface->freeVariable(lExtInterface, lVar);
            lVar = NULL;
        }

        //create a structure to describe a function properties to be called by callmethod
        ArgInfo* lArgInfo = BindingHelper::GetArgInfoLC(KArg0, NULL, (char*)KStrReverse);

        lExtInterface->callMethod(lExtInterface, aObject, lArgInfo, NULL); //reverse array and then use pop to extract variables.

        //now set the function name to 'pop' which will pop values from Array.

        lArgInfo->name = (char*)KStrPop;

        for (TInt count = 0; count < lNoOfArrayElements; count++)
        {
            TLiwVariant lInpVariant;
            lInpVariant.PushL();

            FL_VAR_HANDLE lArrayMember= NULL;

            //create a var to hold value of array member	     	
            CFlvVarHandle* lVarHandle = CFlvVarHandle::NewLC(aFnInfo->pEI);
            lArrayMember = lVarHandle->Handle();

            lExtInterface->callMethod(lExtInterface, aObject, lArgInfo, lArrayMember);

            if (lArrayMember)
            {
                FLExtensionVarType lTypeOfVar;
                lExtInterface->getType(lExtInterface, lArrayMember, &lTypeOfVar);

                TBool lIsVartypeNull = EFalse;

                switch (lTypeOfVar)
                {
                    case kExtnVarTypeInteger:
                    {
                        HandleInt(aFnInfo, lInpVariant, lArrayMember);
                        break;
                    }
                    case kExtnVarTypeNumber:
                    {
                        HandleFloat(aFnInfo, lInpVariant, lArrayMember);
                        break;
                    }
                    case kExtnVarTypeString:
                    {
                        HandleStringL(aFnInfo, lInpVariant, lArrayMember);
                        break;
                    }
                    case kExtnVarTypeString2:
                    {
                        HandleStringL(aFnInfo, lInpVariant, lArrayMember);
                        break;
                    }
                    case kExtnVarTypeBoolean:
                    {
                        HandleBool(aFnInfo, lInpVariant, lArrayMember);
                        break;
                    }
                    case kExtnVarTypeObject:
                    {
                        HandleObjectL(aFnInfo, lInpVariant, lArrayMember);
                        break;
                    }
                    case kExtnVarTypeNull:
                    default:
                        lIsVartypeNull = ETrue;
                }//switch
                if (!lIsVartypeNull)
                {
                    lList->AppendL(lInpVariant);
                }
            }//if(arrayMember)	
            //free memory of var
            CleanupStack::PopAndDestroy(lVarHandle);//pVarHandle
            lArrayMember = NULL;

            CleanupStack::Pop(&lInpVariant);
            lInpVariant.Reset();
        }//if(object)	 

        CleanupStack::Pop(lArgInfo); //pops arginfo from cleanupstack        
        delete lArgInfo;
        lArgInfo = NULL;

        aListVar.SetL(lList);
        CleanupStack::PopAndDestroy(lList);

    }//if
BINDINGTRACEF("BindingHelper.HandleListL> --");
}

void BindingHelper::HandleObjectL(FnInfo* aFnInfo, TLiwVariant& aMapVar, FL_VAR_HANDLE aVar)
{
BINDINGTRACEF    ("BindingHelper.HandleObjectL< ++");
            ExtensionInterface* lExtInterface = aFnInfo->pEI;

    if (aVar)
    {
        //get Object from variable
        FL_OBJECT_HANDLE lObject= NULL;

        lExtInterface->getObject(lExtInterface, aVar, &lObject);

        TInt lIsInstance = -1;

        lExtInterface->isInstanceOf(lExtInterface, lObject, "Date", &lIsInstance);

        if (lIsInstance)
        {
            HandleDateL(aFnInfo, aMapVar, lObject);
BINDINGTRACEF            ( "BindingHelper.HandleObjectL> --");
            return;
        }

        lExtInterface->isInstanceOf(lExtInterface, lObject, "Array", &lIsInstance);
        if(lIsInstance)
        {
            HandleListL(aFnInfo,aMapVar,lObject);
            BINDINGTRACEF( "BindingHelper.HandleObjectL> --");
            return;
        }

        CLiwMap* lMap = CLiwDefaultMap::NewL();
        lMap->PushL();

        //get no of property
        TInt lPropCnt = -1;
        if(lObject)
        {
            lExtInterface->getPropertyCount(lExtInterface, lObject, &lPropCnt);
        }

        FL_VAR_HANDLE lPropValue = NULL;
        char** lPropName = NULL;

        for (TInt lIndex = 0; lIndex < lPropCnt; lIndex++)
        {
            CFlvVarHandle* lVarHandle = CFlvVarHandle::NewLC(aFnInfo->pEI);
            lPropValue = lVarHandle->Handle();

            lPropName = new(ELeave) char*[1];
            CleanupStack::PushL((TAny*)lPropName);

            if(lPropValue)
            {
                lExtInterface->getPropertyByIndex(lExtInterface, lObject, lIndex , lPropValue, (const char**)lPropName); //it will fecth the property by the index value it will give us key value pair say (key,value)

                if(lPropName)
                {
                    TPtrC8 lPtr(reinterpret_cast<const TUint8*>(*lPropName));

                    //check first two properties as if we use var a:Object = new Object() will contain first 2 properties
                    // as __resolve, __proto__  
                    //but with var a:Object = Object(); above properties will not be there.
                    //so put hardcode check to ignore  __resolve, __proto__, __consturctor__.
                    if (BindingHelper::IsInternalProperty(lPtr))
                    {
                        CleanupStack::Pop(); //pop propName

                        delete lPropName;
                        lPropName = NULL;

                        CleanupStack::PopAndDestroy();//destroys pVarHandle
                        continue;
                    }
                    else
                    {
                        TLiwVariant lInpVariant;

                        lInpVariant.PushL();
                        FLExtensionVarType lTypeOfVariable;

                        lExtInterface->getType(lExtInterface, lPropValue, &lTypeOfVariable); //get the type of property using getDataType() which i need to define at plugin side to fetch the Property of the class to identify.

                        TBool lIsVartypeNull = EFalse;
                        switch(lTypeOfVariable)
                        {
                            case kExtnVarTypeInteger:
                            {
                                HandleInt(aFnInfo,lInpVariant,lPropValue);
                                break;
                            }
                            case kExtnVarTypeNumber:
                            {
                                HandleFloat(aFnInfo,lInpVariant,lPropValue);
                                break;
                            }
                            case kExtnVarTypeString:
                            {
                                HandleStringL(aFnInfo,lInpVariant,lPropValue);
                                break;
                            }
                            case kExtnVarTypeString2:
                            {
                                HandleStringL(aFnInfo,lInpVariant,lPropValue);
                                break;
                            }
                            case kExtnVarTypeBoolean:
                            {
                                HandleBool(aFnInfo,lInpVariant,lPropValue);
                                break;
                            }
                            case kExtnVarTypeObject:
                            {
                                HandleObjectL(aFnInfo,lInpVariant,lPropValue);
                                break;
                            }
                            case kExtnVarTypeNull:
                            default:
                            lIsVartypeNull = ETrue;

                        } //switch

                        if(!lIsVartypeNull)
                        {
                            lMap->InsertL(lPtr, lInpVariant);
                        }
                        CleanupStack::Pop(&lInpVariant);
                        lInpVariant.Reset();

                        CleanupStack::Pop(); //pop propName
                        delete lPropName;
                        lPropName = NULL;

                        CleanupStack::PopAndDestroy();//propValue
                    }//else

                }//if(propname)

            }//if (propValue)
        } //for

        aMapVar.SetL(lMap);
        CleanupStack::PopAndDestroy(lMap);
    }//if

    BINDINGTRACEF("BindingHelper.HandleObjectL> --");
}

TBool BindingHelper::IsInternalProperty(TDesC8& aPropName)
{

        if (!aPropName.Compare(KStrObjectPropertyResolve()))
            return ETrue;
        else if (!aPropName.Compare(KStrObjectPropertyProto()))
            return ETrue;
        else if (!aPropName.Compare(KStrObjectPropertyConstructor()))
            return ETrue;
        else if (!aPropName.Compare(KStrObjectPropertyProtoType()))
            return ETrue;
        else if (!aPropName.Compare(KStrObjectPropertyConstructor2()))
            return ETrue;
        else
            return EFalse;
    
}

void BindingHelper::HandleStringL(FnInfo* aFnInfo, TLiwVariant& aStrVar, FL_VAR_HANDLE aVar)
{
BINDINGTRACEF    ("BindingHelper.HandleStringL< ++");

            ExtensionInterface* lExtInterface = aFnInfo->pEI;

    if (aVar)
    {
        char* lData= NULL;

        lExtInterface->getString(lExtInterface, aVar, &lData);

        if (lData)
        {
            TPtrC8 lPtr(reinterpret_cast<const TUint8*>(lData));
            HBufC* lBuffer = EscapeUtils::ConvertToUnicodeFromUtf8L(lPtr);
            aStrVar.Set(*lBuffer);
            //frees the string
            lExtInterface->freeString(lExtInterface, lData);
            lData = NULL;
        }
        else //if data node is null i.e. "" then pass KNullDesC.

        {
            aStrVar.Set(KNullDesC);
        }
    }
BINDINGTRACEF("BindingHelper.HandleStringL> --");
}

void BindingHelper::HandleFloat(FnInfo* aFnInfo, TLiwVariant& aFloatVar, FL_VAR_HANDLE aVar)
{
BINDINGTRACEF    ("BindingHelper.HandleFloat< ++");
    if(aVar)
    {
        TReal lData;
        aFnInfo->pEI->getNumber(aFnInfo->pEI,aVar,&lData);

        BINDINGTRACEF ("BindingHelper.HandleFloat< Value of real %f", lData);

        TReal lFracPart;
        TInt lResult = Math::Frac(lFracPart, lData);
        if(lResult == KErrNone)
        {
            if(lFracPart)
            {
                aFloatVar.Set(lData);
                BINDINGTRACEF ("BindingHelper.HandleFloat< fraction value value %f", lFracPart);
            }
            else
            {
                TInt32 lIntPart;
                Math::Int(lIntPart, lData);
                aFloatVar.Set(lIntPart);
                BINDINGTRACEF ("BindingHelper.HandleFloat< int value %d", lIntPart);
            }
        }
    }
    BINDINGTRACEF("BindingHelper.HandleFloat> --");
}

void BindingHelper::HandleBool(FnInfo* aFnInfo, TLiwVariant& aBoolVar, FL_VAR_HANDLE aVar)
{
BINDINGTRACEF    ( "BindingHelper.HandleBool< ++");
    if(aVar)
    {
        TBool lData;
        aFnInfo->pEI->getBool(aFnInfo->pEI,aVar,&lData);
        aBoolVar.Set(lData);
    }
    BINDINGTRACEF("BindingHelper.HandleBool> --");
}

void BindingHelper::HandleDateL(FnInfo* aFnInfo, TLiwVariant& aDateVar, FL_OBJECT_HANDLE aObject)
{
BINDINGTRACEF    ("BindingHelper.HandleDateL< ++");
            ExtensionInterface* lExtInterface = aFnInfo->pEI;
    if (aObject)
    {

        FL_VAR_HANDLE lRetVal= NULL;

        ArgInfo* lArgInfo = new(ELeave) ArgInfo;
        CleanupStack::PushL((TAny*)lArgInfo);
        lArgInfo->argCount = 0;
        lArgInfo->arguments = NULL;
BINDINGTRACEF( "BindingHelper.HandleDateL< + HandleDateL arginfo creation");
        //array variable to hold date components (year,month,date,hr,...)
                        TInt* lDateComponents = new(ELeave) TInt[ 7 ];
        //array having list of fns which will be fired to get dateComponents.
        char * lDatefns[] = { (char*)KStrGetFullYear, (char*)KStrGetMonth, (char*)KStrGetDate, (char*)KStrGetHours, (char*)KStrGetMinutes,
        (char*)KStrGetSeconds, (char*)KStrGetMilliSeconds };

        for (TInt i = 0; i < 7; i++)
        {
            lArgInfo->name = lDatefns[i];

            lExtInterface->createVariable(lExtInterface, &lRetVal, NULL);
            if (lRetVal)
            {
                //call the method specified by argInfo on Date object. retVal will hold the values returned by fns. 
                lExtInterface->callMethod(lExtInterface, aObject, lArgInfo, lRetVal);

                double value = -1;

                //get the Number value from retVal.
                lExtInterface->getNumber(lExtInterface, lRetVal, &value);

                lDateComponents[i] = value;

                //variable should be freed.
                lExtInterface->freeVariable(lExtInterface, lRetVal);
                lRetVal = NULL;
            }
        }//for 

        TDateTime
        time(lDateComponents[0], (TMonth)lDateComponents[1], lDateComponents[2] - 1, lDateComponents[3], lDateComponents[4], lDateComponents[5], lDateComponents[6]);

        TTime inputTime(time);
        aDateVar.Set(inputTime);
        //clean up
        delete[] lDateComponents;
        for (TInt i = 0; i < 7; i++)
            lDatefns[i] = NULL;

        CleanupStack::Pop(); //pops arginfo
        delete lArgInfo;
        lArgInfo = NULL;

    }//if	
BINDINGTRACEF("BindingHelper.HandleDateL> --");
}

void BindingHelper::HandleInt(FnInfo* aFnInfo, TLiwVariant& aIntVar, FL_VAR_HANDLE aVar)
{
BINDINGTRACEF    ("BindingHelper.HandleInt< ++");
    if(aVar)
    {
        TInt lData = 0;
        aFnInfo->pEI->getInt(aFnInfo->pEI,aVar,&lData);
        aIntVar.Set((TInt32)lData);
    }
    BINDINGTRACEF( "BindingHelper.HandleInt> --");
}

void BindingHelper::ConstructOutputParamL(ExtensionInterface* aExtInterface, TLiwVariant& aParamVar, FL_VAR_HANDLE* aVar)
{
BINDINGTRACEF    ( "BindingHelper.ConstructOutputParamL< ++");
    //check type of node.
    if((aParamVar.TypeId() == LIW::EVariantTypeTInt32) || (aParamVar.TypeId() == LIW::EVariantTypeTUint))//change this accoriding to fl3.0

    {
        //Form attribute "name" with value "int"
        TInt lIntValue = aParamVar.AsTInt32(); //check how to extract key value pair. 		
        aExtInterface->setInt(aExtInterface, *aVar, lIntValue);
    }
    else if(aParamVar.TypeId()== LIW::EVariantTypeTBool)
    {
        //Form attribute "name" with value "bool"
        TBool lBoolValue = aParamVar.AsTBool();

        aExtInterface->setBool(aExtInterface, *aVar, lBoolValue);
    }
    else if(aParamVar.TypeId() == LIW::EVariantTypeTTime)
    {
        BINDINGTRACEF("BindingHelper.ConstructOutputParamL<LIW::EVariantTypeTTime ++");
        // Convert the date as ActionScript requires that the date be passed as
        // milliseconds since midnight January 1, 1970.
        //TBuf<KNumDateLen> lDateBuf(KStrStdDate);
        //TTime lStdTime(lDateBuf);

        TTime lTimeVal = aParamVar.AsTTime();

        TDateTime lDatetimeVal = lTimeVal.DateTime();

        double* dateComponents = new(ELeave) double[7];
        CleanupStack::PushL((TAny*)dateComponents);
        dateComponents[0] = lDatetimeVal.Year();
        dateComponents[1] = lDatetimeVal.Month();
        dateComponents[2] = lDatetimeVal.Day() + 1; //add 1 to date as Actionscript date starts from 1.
        dateComponents[3] = lDatetimeVal.Hour();
        dateComponents[4] = lDatetimeVal.Minute();
        dateComponents[5] = lDatetimeVal.Second();
        dateComponents[6] = lDatetimeVal.MicroSecond()/1000; //it should be in milliseconds


        FL_OBJECT_HANDLE lDateOject = NULL;

        ArgInfo* argInfo = new(ELeave) ArgInfo;
        CleanupStack::PushL((TAny*)argInfo);
        argInfo->name = "Date";
        argInfo->argCount = 7;

        argInfo->arguments = new(ELeave) FL_VAR_HANDLE[7];

        for (TInt i = 0; i < 7; i++)
        {
            aExtInterface->createVariable(aExtInterface, &(argInfo->arguments[i]), NULL);

            if(argInfo->arguments[i])
            {
                aExtInterface->setNumber(aExtInterface, argInfo->arguments[i] , dateComponents[i]);
            }
        }

        aExtInterface->createObject(aExtInterface, argInfo, &lDateOject);

        if(lDateOject)
        {
            aExtInterface->setObject(aExtInterface, *aVar, lDateOject);
        }
        //clean up
        CleanupStack::Pop(2);// pops out argInfo and datecomponents
        delete[] dateComponents;

        for(TInt i = 0; i < 7; i++)
        {
            if(argInfo->arguments[i])
            {
                aExtInterface->freeVariable(aExtInterface, argInfo->arguments[i]);
            }
        }
        delete argInfo;
        argInfo = NULL;
        BINDINGTRACEF( "BindingHelper.ConstructOutputParamL<LIW::EVariantTypeTTime --");

    }
    else if(aParamVar.TypeId() == LIW::EVariantTypeTReal)
    {
        TReal lFloatValue = aParamVar.AsTReal();
        aExtInterface->setNumber(aExtInterface, *aVar, lFloatValue);
    }

    else if(aParamVar.TypeId() == LIW::EVariantTypeDesC)
    {
        //Form attribute "name" with value "string"
        BINDINGTRACEF("BindingHelper.ConstructOutputParamL<LIW::EVariantTypeDesC ++");

        TPtrC lDesCValue=aParamVar.AsDes();
        HBufC8* lBuffer = EscapeUtils::ConvertFromUnicodeToUtf8L(lDesCValue);
        CleanupStack::PushL(lBuffer);
        char* pStrValue = NULL;        
        if(lBuffer->Length())
            {
            pStrValue = Convert8BitDescritptorToStringL( *lBuffer );
            aExtInterface->setString(aExtInterface, *aVar, pStrValue);
            }         

        CleanupStack::PopAndDestroy(lBuffer);
        delete pStrValue;
        pStrValue = NULL;
        BINDINGTRACEF( "BindingHelper.ConstructOutputParamL<LIW::EVariantTypeDesC --");
    }
    else if( aParamVar.TypeId() == LIW::EVariantTypeList)
    {
        BINDINGTRACEF("BindingHelper.ConstructOutputParamL<LIW::EVariantTypeList ++");
        const CLiwList* lList=aParamVar.AsList();
        if ( lList )
        {
            const_cast<CLiwList*>(lList)->PushL();
            ArgInfo* lArgInfo = new(ELeave) ArgInfo;
            CleanupStack::PushL((TAny*)lArgInfo);
            lArgInfo->name = (char*)KStrArray;
            lArgInfo->argCount = 0;
            lArgInfo->arguments = NULL;

            FL_OBJECT_HANDLE pArrayObject = NULL;
            aExtInterface->createObject(aExtInterface, lArgInfo, &pArrayObject); //create a Array Object                 

            if(pArrayObject)
            {
                lArgInfo->name = (char*)KStrPush;
                lArgInfo->argCount = 1;
                for(TInt i= 0; i != lList->Count(); ++i)
                {
                    TLiwVariant pLiwlistEntry;
                    CFlvVarHandle* pFlVarHandle = CFlvVarHandle::NewLC(aExtInterface);
                    FL_VAR_HANDLE pASArrayElement = pFlVarHandle->Handle();

                    pLiwlistEntry.PushL();
                    lList->AtL(i,pLiwlistEntry);
                    if(pASArrayElement)
                    {
                        ConstructOutputParamL(aExtInterface,pLiwlistEntry,&pASArrayElement);

                        lArgInfo->arguments = &pASArrayElement;
                        FL_VAR_HANDLE pLengthVar = NULL;
                        aExtInterface->createVariable(aExtInterface, &pLengthVar, NULL);

                        if(pLengthVar)
                        {
                            aExtInterface->callMethod(aExtInterface, pArrayObject, lArgInfo, pLengthVar); //push element into the array object.
                        }

                        aExtInterface->freeVariable(aExtInterface, pLengthVar);
                        pLengthVar = NULL;
                    }
                    CleanupStack::Pop(&pLiwlistEntry);
                    pLiwlistEntry.Reset();
                    CleanupStack::PopAndDestroy(); //pFlVarHandle
                    pASArrayElement = NULL;
                }
                aExtInterface->setObject(aExtInterface, *aVar, pArrayObject);
                pArrayObject = NULL;
            }
            CleanupStack::Pop(lArgInfo);//argInfo
            delete lArgInfo;
            lArgInfo = NULL;
            CleanupStack::PopAndDestroy(const_cast<CLiwList*>(lList));
        }
        BINDINGTRACEF("BindingHelper.ConstructOutputParamL>LIW::EVariantTypeList --");
    }
    else if(aParamVar.TypeId() == LIW::EVariantTypeIterable)
    {
        BINDINGTRACEF("BindingHelper.ConstructOutputParamL<LIW::EVariantTypeIterable ++");
        TLiwVariant* lIterVar = new(ELeave) TLiwVariant();
        CleanupStack::PushL((TAny*)lIterVar);
        lIterVar->SetL(aParamVar);

        FL_VAR_HANDLE lVar = NULL;
        FL_OBJECT_HANDLE lIterOject = NULL;

        ArgInfo* lArgInfo = new(ELeave) ArgInfo;

        lArgInfo->name = (char*)KClassNameIterator;
        lArgInfo->argCount = 1;

        TInt* lIiterAddress = reinterpret_cast<TInt*>((void*)&lIterVar);

        aExtInterface->createVariable(aExtInterface, &lVar, NULL);

        CleanupStack::Pop(lIterVar); //pops iterVar
        if(lVar)
        {
            aExtInterface->setInt(aExtInterface, lVar , *lIiterAddress);
            lArgInfo->arguments = &lVar;
            aExtInterface->createObject(aExtInterface, lArgInfo , &lIterOject);
        }

        if(lIterOject)
        {
            aExtInterface->setObject(aExtInterface, *aVar, lIterOject);
            aExtInterface->addRef(aExtInterface, lIterOject);
        }
        else
        {
            aExtInterface->setSpecial(aExtInterface, *aVar, kExtnVarTypeUndefined);
            BINDINGTRACEF("BindingHelper.ConstructOutputParamL> LIW::EVariantTypeIterable undefined Iterator --");
        }

        //cleanup 
        //dont' delete iterVar, it has been stored for Iterator class ref.
        if(lVar)
        {
            aExtInterface->freeVariable(aExtInterface, lVar);
        }
        delete lArgInfo;
        lArgInfo = NULL;

        BINDINGTRACEF("BindingHelper.ConstructOutputParamL> LIW::EVariantTypeIterable --");
    }
    else if(aParamVar.TypeId() == LIW::EVariantTypeMap)
    {
        HandleOutPutObjectL( aExtInterface, aParamVar, aVar );
    }
    BINDINGTRACEF("BindingHelper.ConstructOutputParamL> --");
}

void BindingHelper::ConstructOutputParamL(ExtensionInterface* aExtInterface, CLiwGenericParamList& aOutParamList, FL_OBJECT_HANDLE* aOutObject)
{
BINDINGTRACEF    ( "BindingHelper.ConstructOutputParamL from list< ++");
    for(TInt lIndex = 0; lIndex < aOutParamList.Count(); ++lIndex)
    {
        FL_VAR_HANDLE lVar = NULL;
        //extract name of the node
        TLiwGenericParam lOutParam = aOutParamList[lIndex];

        CFlvVarHandle* lVarHandle = CFlvVarHandle::NewLC(aExtInterface);

        lVar = lVarHandle->Handle();
        if(lVar)
        {
            ConstructOutputParamL(aExtInterface,lOutParam.Value(),&lVar);

            //convert paramName to char* from symbianspecific buffer		        

            HBufC8* lBuffer = NULL;
            if(lOutParam.Name().Length()!= 0)
            {
                lBuffer = lOutParam.Name().AllocLC();
            }
            else
            {
                TInt semID = lOutParam.SemanticId();
                lBuffer = GetParamAsString(semID).AllocLC();
            }
            //Add property key value pair to Object
            lBuffer = lBuffer->ReAllocL(lBuffer->Length() + 1);
            aExtInterface->setProperty(aExtInterface, *aOutObject, (char*)(lBuffer->Des().PtrZ()) , lVar);

            CleanupStack::PopAndDestroy(lBuffer);
        }
        CleanupStack::PopAndDestroy(lVarHandle);//lVarHandle
        lVar = NULL;
    }
    BINDINGTRACEF("BindingHelper.ConstructOutputParamL from list> --");
}

void BindingHelper::HandleOutPutObjectL(ExtensionInterface* aExtInterface, TLiwVariant& aParamVar, FL_VAR_HANDLE* aVar)
{
BINDINGTRACEF("BindingHelper.HandleOutPutObjectL< ++");

    TLiwVariant* lMapVar = new(ELeave) TLiwVariant();
    CleanupStack::PushL((TAny*)lMapVar);
    lMapVar->SetL(aParamVar);
    
    FL_VAR_HANDLE lVar = NULL;
    FL_OBJECT_HANDLE lMapOject = NULL;
    
    ArgInfo* lArgInfo = new(ELeave) ArgInfo;
    
    lArgInfo->name = (char*)KClassNameMapProxy;
    lArgInfo->argCount = 1;

    TInt* lVarAddress = reinterpret_cast<TInt*>((void*)&lMapVar);
    
    aExtInterface->createVariable(aExtInterface, &lVar, NULL);
    
    CleanupStack::Pop(lMapVar); //pops lMapVar
    if(lVar)
    {
        aExtInterface->setInt(aExtInterface, lVar , *lVarAddress);
        lArgInfo->arguments = &lVar;
        aExtInterface->createObject(aExtInterface, lArgInfo , &lMapOject);
    }
    
    if(lMapOject)
    {
        aExtInterface->setObject(aExtInterface, *aVar, lMapOject);
        aExtInterface->addRef(aExtInterface, lMapOject);
    }
    else
    {
        aExtInterface->setSpecial(aExtInterface, *aVar, kExtnVarTypeUndefined);
        BINDINGTRACEF("BindingHelper.ConstructOutputParamL> undefined Map --");
    }
    
    if(lVar)
    {
        aExtInterface->freeVariable(aExtInterface, lVar);
    }
    delete lArgInfo;
    lArgInfo = NULL;



BINDINGTRACEF("BindingHelper.HandleOutPutObjectL> --");
}

// Sets the version meta data options for the criteria item.	
TBool BindingHelper::SetMetaDataForCriteriaItemL(ExtensionInterface* aExtInterface, CLiwCriteriaItem* aCrit, FL_OBJECT_HANDLE aVerParams)
{
BINDINGTRACEF( "BindingHelper.SetMetaDataForCriteriaItemL< ++");
            TReal lMinVersion;
    TReal lMaxVersion;
    TBool lIsVerFound = GetMinAndMaxVersionL(aExtInterface, lMinVersion, lMaxVersion, aVerParams);

    if (EFalse == lIsVerFound)
    {
BINDINGTRACEF        ("BindingHelper.SetMetaDataForCriteriaItemL> --");
        return EFalse;
    }

    TLiwVariant lMetadataOption;
    lMetadataOption.PushL();
    CLiwMap* lMetaDataMap = CLiwDefaultMap::NewL();
    CleanupStack::PushL(lMetaDataMap);

    CLiwList* lRangeList = CLiwDefaultList::NewL();
    CleanupStack::PushL(lRangeList);

    lRangeList->AppendL(TLiwVariant(KStrVersion));
    lRangeList->AppendL(TLiwVariant(lMinVersion));
    lRangeList->AppendL(TLiwVariant(lMaxVersion));

    lMetaDataMap->InsertL(KStrRange,lRangeList);
    lMetadataOption.Set(lMetaDataMap);
    aCrit->SetMetaDataOptions(lMetadataOption);

    CleanupStack::Pop(lRangeList);
    CleanupStack::Pop(lMetaDataMap);
    CleanupStack::Pop(&lMetadataOption);
    lRangeList->DecRef();
    lMetaDataMap->DecRef();
    lMetadataOption.Reset();

    BINDINGTRACEF("BindingHelper.SetMetaDataForCriteriaItemL> --");
    return ETrue;
}

// Gets the min and max version number for which the list of providers
// is queried.
TBool BindingHelper::GetMinAndMaxVersionL(ExtensionInterface* aExtInterface, TReal& aMinVersion, TReal& aMaxVersion, FL_OBJECT_HANDLE aVerParams)
{
BINDINGTRACEF("BindingHelper.GetMinAndMaxVersionL< ++");
            TBool lIsVerFound = EFalse;
    TInt lCount =0;
    if (!aVerParams)
    {
BINDINGTRACEF        ( "BindingHelper.GetMinAndMaxVersionL> --");
        return lIsVerFound;
    }

    aExtInterface->getPropertyCount(aExtInterface, aVerParams, &lCount);
    for( TInt lIndex = 0; lIndex < lCount; ++lIndex)
    {
        char** lKeyName = new(ELeave) char*[ 1 ];
        FL_VAR_HANDLE lKeyValue = NULL;
        TReal lVersion = 0.0;
        aExtInterface->createVariable(aExtInterface, &lKeyValue, NULL);
        aExtInterface->getPropertyByIndex(aExtInterface, aVerParams, lIndex, lKeyValue, (const char**)lKeyName);
        if(lKeyValue)
        {
            if(lKeyName)
            {
                TPtrC8 lPtr(reinterpret_cast<const TUint8*>(*lKeyName));

                if (!(lPtr.Compare(KStrMin)))
                {
                    aExtInterface->getNumber(aExtInterface, lKeyValue, &lVersion);
                    aMinVersion = lVersion;
                    lIsVerFound = ETrue;
                }
                else if (!(lPtr.Compare(KStrMax)))
                {
                    aExtInterface->getNumber(aExtInterface, lKeyValue, &lVersion);
                    aMaxVersion = lVersion;
                    lIsVerFound = ETrue;
                }
                delete lKeyName;
                lKeyName = NULL;
            }

            aExtInterface->freeVariable(aExtInterface, lKeyValue);
            lKeyValue = NULL;
        }
    }
    BINDINGTRACEF( "BindingHelper.GetMinAndMaxVersionL> --");
    return lIsVerFound;
}

HBufC* BindingHelper::StringToDescriptorL(const char* aString)
{
BINDINGTRACEF("BindingHelper.StringToDescriptorL< ++");
            TPtrC8 lPtr(reinterpret_cast<const TUint8*>(aString));
    HBufC* lBuffer = EscapeUtils::ConvertToUnicodeFromUtf8L(lPtr);

BINDINGTRACEF    ( "BindingHelper.StringToDescriptorL> --");
    return lBuffer;
}

// Converts the TGenericParamId passed by the service provider
// to string, as we can pass only string values to flash content.
const TDesC8& BindingHelper::GetParamAsString(TInt aParamID)
{
BINDINGTRACEF    ("BindingHelper.GetParamAsString< ++");
    switch(aParamID)
    {
        case LIW::EGenericParamError:
        {
            BINDINGTRACEF("BindingHelper.GetParamAsString> --");
            return KStrErrorId();
        }

        //Convert other generic param IDs if required
        default:
        break;
    }
    BINDINGTRACEF("BindingHelper.GetParamAsString> --");
    return KNullDesC8();
}

char* BindingHelper::Convert8BitDescritptorToStringL(const TDesC8& aDes)
{
BINDINGTRACEF("BindingHelper.Convert8BitDescritptorToStringL< ++");

            char* lBuffer = new(ELeave) char[ aDes.Length() + 1 ];
    Mem::Copy(lBuffer, aDes.Ptr(), aDes.Length());
    lBuffer[aDes.Length()] = '\0';

BINDINGTRACEF    ("BindingHelper.Convert8BitDescritptorToStringL> --");
    return lBuffer;
}

ArgInfo* BindingHelper::GetArgInfoLC(TInt aArgCnt, FL_VAR_HANDLE* aArgList, char* aName)
{
BINDINGTRACEF("BindingHelper.GetArgInfoLC< ++");
            ArgInfo* lArgInfo = new(ELeave) ArgInfo;
    CleanupStack::PushL(lArgInfo);
    lArgInfo->argCount = aArgCnt;
    lArgInfo->arguments = aArgList;
    lArgInfo->name = aName;
BINDINGTRACEF    ("BindingHelper.GetArgInfoLC> --");
    return lArgInfo;
}

void BindingHelper::PointerArrayCleanup(TAny* aArray)
{
    static_cast<RCriteriaArray*>(aArray)->ResetAndDestroy();
}
//class FLvarHandle

void CFlvVarHandle::ConstructL()
{
    if (iExtInterface)
    {
        iExtInterface->createVariable(iExtInterface, &iVarHandle, NULL);
    }
}

FL_VAR_HANDLE CFlvVarHandle::Handle()
{
    return iVarHandle;
}

CFlvVarHandle::~CFlvVarHandle()
{
    if (iExtInterface && iVarHandle)
    {
        iExtInterface->freeVariable(iExtInterface, iVarHandle);
    }
    iVarHandle = NULL;
    iExtInterface = NULL;
}

CFlvVarHandle* CFlvVarHandle::NewLC(ExtensionInterface* aEI)
{
    CFlvVarHandle* lSelf =new(ELeave) CFlvVarHandle(aEI);
    CleanupStack::PushL(lSelf);
    lSelf->ConstructL();
    return lSelf;
}

