/*
* Copyright (c) 2008-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the member functions of CFlashBindingASExtDelegate class.
 
 *
*/


#include <stdlib.h> 
#include <e32std.h>
#include <ecom/implementationproxy.h>
#include <escapeutils.h>
#include <liwservicehandler.h>

#include "flashbinding.h"
#include "serviceproxy.h"
#include "iteratorimpl.h"
#include "flashrtpolicyclient.h"
#include "flashrtpolicycontentsession.h"
#include "debuglog.h"

const char* const KFlashBindingPluginName = "FlashBinding";
const TUint KUidLiwBindingImplementationUid = 0x20010115;


CFlashBindingASExtDelegate* CFlashBindingASExtDelegate::NewL()
    {
BINDINGTRACEF("CFlashBindingASExtDelegate.NewL< ++");
        CFlashBindingASExtDelegate* lSelf = CFlashBindingASExtDelegate::NewLC();
    CleanupStack::Pop(lSelf);
BINDINGTRACEF    ( "CFlashBindingASExtDelegate.NewL> --");
    return lSelf;
    }

// Constructs a new CFlashBindingASExtDelegate object. 
CFlashBindingASExtDelegate* CFlashBindingASExtDelegate::NewLC()
    {
BINDINGTRACEF( "CFlashBindingASExtDelegate.NewLC< ++");
    CFlashBindingASExtDelegate* lSelf=new(ELeave) CFlashBindingASExtDelegate();
    CleanupStack::PushL(lSelf);
    lSelf->ConstructL();
BINDINGTRACEF    ("CFlashBindingASExtDelegate.NewLC> --");
    return lSelf;
    }

void CFlashBindingASExtDelegate::InitPlugin(TInt aOpCode, TAny* aParam)
    {
    BINDINGTRACEF( "CFlashBindingASExtDelegate.InitPlugin< ++");
    if (CASExtensionDelegate::EOpCodeSMCheckNeeded == aOpCode)
        {
        iIsSecMgrCheckNeeded = TBool(aParam);
        }       
BINDINGTRACEF    ( "CFlashBindingASExtDelegate.InitPlugin> --");
    }

PluginInfo* CFlashBindingASExtDelegate::GetPluginInfoL(MFlashEngineIntfToASExtension* aFalshEngine,
        ExtensionInterface* /*aEI*/)
    {
BINDINGTRACEF("CFlashBindingASExtDelegate.GetPluginInfoL< ++");
    iFlashEngine = aFalshEngine;
    PluginInfo* lPluginInfo = new(ELeave) PluginInfo;
    CleanupStack::PushL((TAny*)lPluginInfo);
    lPluginInfo->pluginName = (char*)KFlashBindingPluginName;
    lPluginInfo->userData = 0;
    lPluginInfo->destroyPlugin = CFlashBindingASExtDelegate::DestroyPlugin;

    //CServiceProxy    
    ClassInfo* lServiceProxyInfo = CServiceProxy::GetClassInfoL();
    CleanupStack::PushL((TAny*)lServiceProxyInfo);
    //Iterator Class
    ClassInfo* lIteratorInfo = Iterator::GetClassInfoL();
    CleanupStack::PushL((TAny*)lIteratorInfo);
    //class ServiceUtils
    ClassInfo* lServiceUtilsProxyInfo = ServiceUtilsProxy::GetClassInfoL();
    CleanupStack::PushL((TAny*)lServiceUtilsProxyInfo);
    
    ClassInfo* lMapProxyInfo = CMapProxy::GetClassInfoL();
    CleanupStack::PushL((TAny*)lMapProxyInfo);

    
    lServiceUtilsProxyInfo->next = lMapProxyInfo;
    
    lIteratorInfo->next = lServiceUtilsProxyInfo;

    lServiceProxyInfo->next = lIteratorInfo;

    lPluginInfo->classes = lServiceProxyInfo;
    lPluginInfo->reserved1 = NULL;
    
    lPluginInfo->methodHook = NULL;
    CleanupStack::Pop(5);
BINDINGTRACEF    ( "CFlashBindingASExtDelegate.GetPluginInfoL> --");
    return lPluginInfo;
    }


void CFlashBindingASExtDelegate::ConstructL()
    {
    BINDINGTRACEF("CFlashBindingASExtDelegate.ConstructL< ++");
    iServiceHandler = CLiwServiceHandler::NewL();
    iIsSecMgrCheckNeeded = ETrue;
    Dll::SetTls(this);
    BINDINGTRACEF("CFlashBindingASExtDelegate.ConstructL> --");
    }

MFlashRTPolicyContentSession* CFlashBindingASExtDelegate::GetSecurityManagerSessionL()
    {
    BINDINGTRACEF("CFlashBindingASExtDelegate.GetSecurityManagerSessionL<++");
    if (iIsSecMgrCheckNeeded)
        {
        if(!iScriptSession)
            {
            iFpsClient = CFlashRTPolicyClient::NewL();

            TPolicyID lPolicyId;

            lPolicyId = iFpsClient->GetPolicyIdL();

            iScriptSession = iFpsClient->GetScriptSessionL(lPolicyId);   
            }
        BINDINGTRACEF("CFlashBindingASExtDelegate.GetSecurityManagerSessionL>--");
        return iScriptSession;
        }     
    BINDINGTRACEF("CFlashBindingASExtDelegate.GetSecurityManagerSessionL>--");
    return NULL;
    }

CLiwServiceHandler* CFlashBindingASExtDelegate::GetLiwServiceHandler()
    {
    BINDINGTRACEF("CFlashBindingASExtDelegate.GetLiwServiceHandler< ++");
    BINDINGTRACEF("CFlashBindingASExtDelegate.GetLiwServiceHandler>--");
    return iServiceHandler;
    }

CFlashBindingASExtDelegate::~CFlashBindingASExtDelegate()
    {
    if (iScriptSession)
        {
        delete iScriptSession;
        iScriptSession = NULL;
        }

    if (iFpsClient)
        {
        delete iFpsClient;
        iFpsClient = NULL;
        }

    if (iServiceHandler)
        {
        iServiceHandler->Reset();
        delete iServiceHandler;
        iServiceHandler = NULL;
        }
    }

void CFlashBindingASExtDelegate::DestroyPlugin(PluginInfo* pluginInfo)
    {
BINDINGTRACEF("DestroyPlugin< ++");
    CFlashBindingASExtDelegate* lFlashBinding =
            static_cast<CFlashBindingASExtDelegate*>(Dll::Tls() );
    if (lFlashBinding)
        {
        delete lFlashBinding;
        lFlashBinding = NULL;
        }
    Dll::FreeTls();

    while (pluginInfo->classes)
        {
        delete pluginInfo->classes->methodNames;
        delete pluginInfo->classes->methods;
        delete pluginInfo->classes->methodTypes;
        ClassInfo* temp_classInfo = pluginInfo->classes;
        pluginInfo->classes = pluginInfo->classes->next;
        delete temp_classInfo;
        }
    delete pluginInfo;
    pluginInfo = NULL;    
BINDINGTRACEF    ( "DestroyPlugin> --");
    }

const TImplementationProxy
        ImplementationTable[] =
            {
                    IMPLEMENTATION_PROXY_ENTRY(KUidLiwBindingImplementationUid, CFlashBindingASExtDelegate::NewL)
            };

EXPORT_C const TImplementationProxy* ImplementationGroupProxy(TInt& aTableCount)
    {
    aTableCount = sizeof(ImplementationTable) / sizeof(TImplementationProxy);
    return ImplementationTable;
    }

