/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the member functions of Iterator class.
 
 *
*/


#include <liwservicehandler.h>
#include <liwvariant.h>
#include <liwvarianttype.hrh>

#include "iteratorimpl.h"
#include "bindinghelper.h"
#include "debuglog.h"
#include "commonconsts.h"

void Iterator::Constructor(FnInfo* aFnInfo)
    {
BINDINGTRACEF("Iterator.Constructor< ++");
    Iterator* lIter = new Iterator();
    int lIteraddress;

    aFnInfo->pEI->getInt(aFnInfo->pEI, aFnInfo->pArgs[0], &lIteraddress);

    lIter->iLiwVar = reinterpret_cast<TLiwVariant*>(lIteraddress);

    aFnInfo->instanceData = (FL_VAR_HANDLE)lIter;
BINDINGTRACEF    ("Iterator.Constructor> --");
    }

void Iterator::Destructor(FnInfo* aFnInfo)
    {
BINDINGTRACEF("Iterator.Destructor< ++");
    Iterator* lInstanceObject = (Iterator*)aFnInfo->instanceData;

    if (lInstanceObject->iLiwVar)
        {
        /*lInstanceObject->iLiwVar->Reset();        
        delete lInstanceObject->iLiwVar;
        lInstanceObject->iLiwVar = NULL;*/             
        }

    delete lInstanceObject;
    lInstanceObject = NULL;
BINDINGTRACEF    ("Iterator.Destructor> --");
    }

void Iterator::Reset(FnInfo* aFnInfo)
    {
    BINDINGTRACEF( "Iterator::Reset< ++");
    ASSERT(aFnInfo);
    Iterator* lIterObject = (Iterator*)aFnInfo->instanceData;

    ASSERT(lIterObject);

    if (lIterObject->iLiwVar)
        {
        lIterObject->iLiwVar->AsIterable()->Reset();
        }

BINDINGTRACEF    ( "Iterator::Reset> --");
    }

void Iterator::Close(FnInfo* aFnInfo)
    {
    BINDINGTRACEF( "Iterator::Close< ++");
    ASSERT(aFnInfo);
    Iterator* lIterObject = (Iterator*)aFnInfo->instanceData;

    ASSERT(lIterObject);

    if (lIterObject->iLiwVar)
        {
        lIterObject->iLiwVar->Reset();
        delete lIterObject->iLiwVar;
        lIterObject->iLiwVar = NULL;
        }

BINDINGTRACEF    ( "Iterator::Close> --");
    }

void Iterator::Next(FnInfo* aFnInfo)
    {
    BINDINGTRACEF( "Iterator.next< ++");
    ASSERT(aFnInfo);
    ASSERT(aFnInfo->pEI);
    Iterator* lIterObject = (Iterator*)aFnInfo->instanceData;
    ASSERT(lIterObject);
    TInt lIsItemFound;
    TRAPD(lErr,lIterObject->HandleNextL(aFnInfo, lIsItemFound));
    
    if (lErr != KErrNone )
        {
        aFnInfo->pEI->setSpecial(aFnInfo->pEI, aFnInfo->returnValue,kExtnVarTypeNull);
        }
    else if(lIsItemFound)
        {
        aFnInfo->pEI->setSpecial(aFnInfo->pEI, aFnInfo->returnValue,kExtnVarTypeNull);
        }

BINDINGTRACEF    ("Iterator.next> --");
    }

void Iterator::HandleNextL(FnInfo* aFnInfo, TInt& aIsItemFound)
    {
BINDINGTRACEF( "Iterator.HandlenextL< ++");
    if(!iLiwVar)
        {
        User::Leave(KFlashBindingErrGeneral);
        }
    

    TLiwVariant lEntry;
    lEntry.PushL();

    if (EFalse!=iLiwVar->AsIterable()->NextL(lEntry))
        {
        BindingHelper::ConstructOutputParamL(aFnInfo->pEI, lEntry,
                &aFnInfo->returnValue); //convert from LIW to AS specific                  
        aIsItemFound = KErrNone;
        }
    else
        {
        aIsItemFound = KErrNotFound;
        }

    CleanupStack::Pop(&lEntry);
    lEntry.Reset();

BINDINGTRACEF    ("Iterator.HandlenextL> --");
    }

ClassInfo* Iterator::GetClassInfoL()
    {
BINDINGTRACEF( "Iterator.GetClassInfo< ++");

    ClassInfo* lIteratorInfo = new(ELeave) ClassInfo;
    CleanupStack::PushL((TAny*)lIteratorInfo);

    lIteratorInfo->className = (char*)KClassNameIterator;
    lIteratorInfo->methodCount = 3;

    lIteratorInfo->methodNames = new(ELeave) char*[3];
    CleanupStack::PushL((TAny*)lIteratorInfo->methodNames);

    lIteratorInfo->methodNames[0] = (char*)KMethodNameNext;
    lIteratorInfo->methodNames[1] = (char*)KMethodNameReset;
    lIteratorInfo->methodNames[2] = (char*)KMethodNameClose;

    lIteratorInfo->methodTypes = new(ELeave) int[3];
    CleanupStack::PushL((TAny*)lIteratorInfo->methodTypes);

    lIteratorInfo->methodTypes[0] = METHODTYPE_MEMBER_FUNCTION;
    lIteratorInfo->methodTypes[1] = METHODTYPE_MEMBER_FUNCTION;
    lIteratorInfo->methodTypes[2] = METHODTYPE_MEMBER_FUNCTION;

    lIteratorInfo->methods = new(ELeave) METHOD[3];
    CleanupStack::PushL((TAny*)lIteratorInfo->methods);

    lIteratorInfo->methods[0] = Iterator::Next;
    lIteratorInfo->methods[1] = Iterator::Reset;
    lIteratorInfo->methods[2] = Iterator::Close;

    lIteratorInfo->constructor = Iterator::Constructor;
    lIteratorInfo->destructor = Iterator::Destructor;
    lIteratorInfo->update = NULL;
    lIteratorInfo->firesEvents = 0;
    lIteratorInfo->next = NULL;

    CleanupStack::Pop(4);

BINDINGTRACEF    ("Iterator.GetClassInfo> --");
    return lIteratorInfo;
    }

