/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the member functions of CServiceProxy class.
 
 *
*/

#include <liwservicehandler.h>
#include <liwvariant.h>
#include <liwvarianttype.hrh>
#include <utf.h>
#include <escapeutils.h>
#include <stdlib.h>
#include <eikenv.h>
#include <s32strm.h>

#include "flashbinding.h"
#include "bindinghelper.h"
#include "serviceproxy.h"
#include "asyncreqidmap.h"
#include "flashrtpolicyclient.h"
#include "flashrtpolicycontentsession.h"
#include "debuglog.h"
#include "commonconsts.h"
#include "asextconsts.h"

void CServiceProxy::Constructor(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CServiceProxy.Constructor< ++");
    ASSERT(aFnInfo);
    aFnInfo->instanceData = (FL_VAR_HANDLE) new CServiceProxy;
    BINDINGTRACEF("CServiceProxy.Constructor> --");
    }

void CServiceProxy::Destructor(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CServiceProxy.Destructor< ++");
    ASSERT(aFnInfo);
    ASSERT(aFnInfo->pEI);
    CServiceProxy* lInstanceObject = reinterpret_cast<CServiceProxy*>(aFnInfo->instanceData);
    ASSERT(lInstanceObject);
    if(lInstanceObject->isCorePlayerUpdateRunning)
        {
        BINDINGTRACEF("CServiceProxy.Destructor> isCorePlayerUpdateRunning ++");
        aFnInfo->pEI->stopUpdates(aFnInfo->pEI, aFnInfo->objectHandle);
        aFnInfo->pEI->release(aFnInfo->pEI,  aFnInfo->objectHandle);
        lInstanceObject->isCorePlayerUpdateRunning = EFalse;
        BINDINGTRACEF("CServiceProxy.Destructor> isCorePlayerUpdateRunning--");
        }
    delete lInstanceObject;
    lInstanceObject = NULL;
    BINDINGTRACEF("CServiceProxy.Destructor> --");
    }

void CServiceProxy::Destroy()
    {
    BINDINGTRACEF("CServiceProxy.Destroy< ++");
    iAsyncIDMap.ResetAndDestroy();
    iAsyncIDMap.Close();
    
    
    iEventQueue.ResetAndDestroy();
    iEventQueue.Close();
    
    iCritArray.ResetAndDestroy();
    iCritArray.Close();
    
    if(iLiwInterface)
        {
        iLiwInterface->Close();
        
        iLiwInterface = NULL;
        }
    
    
    if(iFlashBinding)
        {        
        TRAP_IGNORE(iFlashBinding->GetLiwServiceHandler()->DetachL(iCritArray));
        }
    
    //never delete iExtInterface and iFlashBinding because we don't own this
    iFlashBinding = NULL;
    iExtInterface = NULL;

    BINDINGTRACEF("CServiceProxy.Destroy> --");
    }

void CServiceProxy::UnloadService(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CServiceProxy.UnLoadService< ++");
    CServiceProxy* lInstanceObject = (CServiceProxy*)aFnInfo->instanceData;
    ASSERT(lInstanceObject);
    
    lInstanceObject->Destroy();
    
    BINDINGTRACEF("CServiceProxy.UnLoadService> --");
    }

void CServiceProxy::Load(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CServiceProxy.Load< ++");
    ASSERT(aFnInfo);
    ASSERT(aFnInfo->pEI);
    CServiceProxy* instanceObject = (CServiceProxy*)aFnInfo->instanceData;

    ASSERT(instanceObject);

    TRAPD(err, instanceObject->HandleLoadL(aFnInfo));
    aFnInfo->pEI->setInt(aFnInfo->pEI, aFnInfo->returnValue, err);
   
    BINDINGTRACEF("CServiceProxy.Load> --");
    }

void CServiceProxy::HandleLoadL(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CServiceProxy.HandleLoadL< ++");
    TInt lResult = KErrNone;

    iExtInterface = aFnInfo->pEI; //store extension manager pointer don't delete this it will be taken care by core player.
    
    iFlashBinding = static_cast<CFlashBindingASExtDelegate*>(Dll::Tls() );
    if(!iFlashBinding)
        {
        BINDINGTRACEF("CServiceProxy.HandleLoadL> -- flashbinding NULL");
        User::Leave(KFlashBindingErrProviderNotFound);
        }
    
    CLiwServiceHandler* lServiceHandler = iFlashBinding->GetLiwServiceHandler();
    MFlashRTPolicyContentSession* lSecManagerSession  = iFlashBinding->GetSecurityManagerSessionL();
    if(!lServiceHandler)
        {
        BINDINGTRACEF("CServiceProxy.HandleLoadL> -- service Handler NULL");
        User::Leave(KFlashBindingErrProviderNotFound);
        }
    //get the number of args
    TInt lArgCount = aFnInfo->numArgs;

    if (lArgCount >KArg0 && lArgCount < KArg5)
        {
        char* lServiceString;
        char* lInterfaceString;
        FL_OBJECT_HANDLE lVerObj= NULL;
        FL_OBJECT_HANDLE lInputParamObj= NULL;
        
        lResult = iExtInterface->getString(iExtInterface, aFnInfo->pArgs[KArg0],&lServiceString);
 
        if (lResult)
            {
            BINDINGTRACEF("CServiceProxy.HandleLoadL> Parsing Input failed--");
            User::Leave(KFlashBindingErrGeneral);
            }
        
        lResult = iExtInterface->getString(iExtInterface,aFnInfo->pArgs[KArg1], &lInterfaceString);        
        CleanupStack::PushL(lServiceString);
        
        if (lResult)
            {
            BINDINGTRACEF("CServiceProxy.HandleLoadL> Parsing Input failed--");
            User::Leave(KFlashBindingErrGeneral);
            }
        CleanupStack::PushL(lInterfaceString);

        HBufC* lArgService = BindingHelper::StringToDescriptorL(lServiceString);
        CleanupStack::PushL(lArgService);

        HBufC8* lServiceName = EscapeUtils::ConvertFromUnicodeToUtf8L(*lArgService);
        CleanupStack::PushL(lServiceName);

        HBufC* lArgInterface = BindingHelper::StringToDescriptorL(lInterfaceString);
        CleanupStack::PushL(lArgInterface);

        HBufC8* lServiceInterface = EscapeUtils::ConvertFromUnicodeToUtf8L(*lArgInterface);
        CleanupStack::PushL(lServiceInterface);

        
        CLiwCriteriaItem* lCrit = CLiwCriteriaItem::NewLC(1, *lServiceInterface, *lServiceName);
        lCrit->SetServiceClass(TUid::Uid(KLiwClassBase));

        // Version information is present.
        if (lArgCount> KArg2)
            {
            iExtInterface->getObject( iExtInterface, aFnInfo->pArgs[KArg2], &lVerObj );
            if(lVerObj)
                {
                BindingHelper::SetMetaDataForCriteriaItemL(iExtInterface, lCrit, lVerObj);
                }
            }

        iCritArray.AppendL(lCrit);

        if(lSecManagerSession)
            {
            lServiceHandler->AttachL(iCritArray, lSecManagerSession->Session());
            }
        else
            {
            CRTSecMgrScriptSession* lSession = NULL;
            lServiceHandler->AttachL(iCritArray, *lSession);
            }
        //AttachL is blocking call and in that duration core Player didn't get any updates so holding AttachL() for a longer period 
        //will result "Aciton Script Stuck Error -4" , so we put  resetScriptStartTime() call to Core Player to Avoid "Flash Content Error -4"
        iExtInterface->resetScriptStartTime(iExtInterface);

        if( 0 ==lServiceHandler->NbrOfProviders(lCrit))
            {
            //Form an error code and return back the outparam KFLASH_PROVIDER_NOT_FOUND
            CleanupStack::Pop(lCrit);
            CleanupStack::PopAndDestroy(lServiceInterface);
            CleanupStack::PopAndDestroy(lArgInterface);
            CleanupStack::PopAndDestroy(lServiceName);
            CleanupStack::PopAndDestroy(lArgService);
            CleanupStack::Pop(lInterfaceString);
            CleanupStack::Pop(lServiceString);            
            iExtInterface->freeString(iExtInterface,lServiceString);
            iExtInterface->freeString(iExtInterface,lInterfaceString);
            lServiceString = NULL;
            lInterfaceString = NULL;

            iExtInterface->setInt(iExtInterface, aFnInfo->returnValue, KFlashBindingErrProviderNotFound);
            BINDINGTRACEF("CServiceProxy.HandleLoadL> KFlashBindingErrProviderNotFound--");
            User::Leave(KFlashBindingErrProviderNotFound);
            }

        CLiwGenericParamList* lInputParam = &(lServiceHandler->InParamListL());
        CLiwGenericParamList* lOuputParam = &(lServiceHandler->OutParamListL());

        // Initial param needs to be passed while creating instance

        if ( lArgCount> KArg3 )
            {
            iExtInterface->getObject( iExtInterface, aFnInfo->pArgs[KArg3], &lInputParamObj );
            if(lInputParamObj)
                {
                BindingHelper::ConstructInputParamsL(aFnInfo, lInputParamObj, *lInputParam);
                }
            }

        // Request for the service.
        lServiceHandler->ExecuteServiceCmdL(*lCrit, *lInputParam, *lOuputParam);
        CleanupStack::Pop(lCrit);
        
        // Store the interface instance passed.
        TInt lPos = 0;
        lOuputParam->FindFirst(lPos,*lServiceInterface);
        if(KErrNotFound != lPos)
            {
            MLiwInterface* ifp = (*lOuputParam)[lPos].Value().AsInterface();
            if ( NULL != ifp )
                {
                iLiwInterface = ifp; //store the service provider interface
                }
            else
                {
                lResult = KFlashBindingErrInterfaceNotFound;
                }
            }
        else
            {
            lResult = KFlashBindingErrInterfaceNotFound;
            }

        lInputParam->Reset();
        lOuputParam->Reset();
        CleanupStack::PopAndDestroy(lServiceInterface);
        CleanupStack::PopAndDestroy(lArgInterface);
        CleanupStack::PopAndDestroy(lServiceName);
        CleanupStack::PopAndDestroy(lArgService);
        CleanupStack::Pop(lInterfaceString);
        CleanupStack::Pop(lServiceString);
        iExtInterface->freeString(iExtInterface,lServiceString);
        iExtInterface->freeString(iExtInterface,lInterfaceString);
        lServiceString = NULL;
        lInterfaceString = NULL;
        }
    BINDINGTRACEF("CServiceProxy.HandleLoadL> --");
    }

void CServiceProxy::Update(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CServiceProxy::Update < ++");
    ASSERT(aFnInfo);
    CServiceProxy* lInstanceObject = (CServiceProxy*)aFnInfo->instanceData;
    ASSERT(lInstanceObject);
    ExtensionInterface* lExtInterface = aFnInfo->pEI;
    
    FL_OBJECT_HANDLE lOutputObject= NULL;
    

   if(lInstanceObject->iEventQueue.Count()>0)
       {
       for(TInt lIndex =0; lIndex<lInstanceObject->iEventQueue.Count(); ++lIndex)
           {
               CLiwGenericParamList* lTempAsyncOp = lInstanceObject->iEventQueue[0]->iAsyncOutput;
               if(lTempAsyncOp)
                   {                  
                   ArgInfo* lArgInfo = NULL;
                   TInt lErr;
                   TRAP(lErr, 
                               lArgInfo = BindingHelper::GetArgInfoLC(KArg0, NULL, (char*)KStrObject );
                               CleanupStack::Pop(lArgInfo)
                   );
                   
                   if(lErr == KErrNone)
                   {
                       lExtInterface->createObject(aFnInfo->pEI, lArgInfo, &lOutputObject);
                   }
                   
                   delete lArgInfo;
                   lArgInfo = NULL;  

                   if(lOutputObject)
                       {
                       TRAP(lErr, BindingHelper::ConstructOutputParamL( lExtInterface, *lTempAsyncOp, &lOutputObject ));           
                       BINDINGTRACEF("CServiceProxy::Update output data processed");
                       if(lErr == KErrNone)
                           {
                        FL_VAR_HANDLE lVar = NULL;
         
                        lExtInterface->createVariable(lExtInterface, &lVar, NULL );
                        
                        if(lVar)
                            {
                               BINDINGTRACEF("CServiceProxy::Update before firing event");
                            lExtInterface->setObject(lExtInterface, lVar, lOutputObject);
                            lErr = lExtInterface->fireEvent(lExtInterface, aFnInfo->objectHandle,"onEvent", lVar);
                            lExtInterface->freeVariable(lExtInterface, lVar);  
                               }                          
                            }     
                       }
                   lInstanceObject->iEventQueue.Remove(0);
                   lTempAsyncOp->Reset();     
                   delete lTempAsyncOp;
                   lTempAsyncOp = NULL;
                   }
               break; //fire only one event at once.
               }
       }        
    BINDINGTRACEF("CServiceProxy::Update>--");
    }
void CServiceProxy::ResumePlayerL(TBool aBringToForeground)
    {    
    BINDINGTRACEF("CServiceProxy::ResumePlayerL< ++");
    
    MFlashEngineIntfToASExtension* lFlashEngine = iFlashBinding->GetFlashEngine();//get the Host engine reference
    TInt lResult;
    
    RASExtensionArgArray lArgArray;    
    TCleanupItem lArrayCleanItem( CServiceProxy::CleanUpArray , &lArgArray); 
    CleanupStack::PushL(lArrayCleanItem);
    
    PrepareExtensionArgL(lArgArray, aBringToForeground);
    
    lFlashEngine->ProcessHostCmdL(KStrCmdResumePlayer(), lArgArray, lResult);
    
    CleanupStack::PopAndDestroy();//lArrayCleanItem
    BINDINGTRACEF("CServiceProxy::ResumePlayerL> --");    
    }

void CServiceProxy::PrepareExtensionArgL(RASExtensionArgArray& aArray, TBool aBringToForeground)
    {
    //make a Extension specific variable to hold "BringtoForeground" value.
    ExtensionArg* lArgtoEngine  = new (ELeave) ExtensionArg;
    CleanupStack::PushL(lArgtoEngine);
    lArgtoEngine->pVariableValue = (void*)aBringToForeground;
    lArgtoEngine->variableType = kExtnVarTypeBoolean; 
    
    aArray.AppendL(lArgtoEngine);
    CleanupStack::Pop(lArgtoEngine);
    }

void CServiceProxy::CleanUpArray(TAny* aArray)
    {
    static_cast<RASExtensionArgArray*>(aArray)->ResetAndDestroy();
    }
TInt CServiceProxy::HandleNotifyL(TInt aCmdId, TInt aEventId,
        CLiwGenericParamList& aEventParamList, const CLiwGenericParamList & /*aInParamList*/)
    {
    BINDINGTRACEF("CServiceProxy.HandleNotifyL< ++");
	TInt lResult = HandleAsyncReturnL(aCmdId, aEventId, aEventParamList);
	BINDINGTRACEF("CServiceProxy.HandleNotifyL> --");
    return lResult;
    }

// Handles the return value of async operation.
TInt CServiceProxy::HandleAsyncReturnL(TInt aCmdId, TInt aEventId,
        CLiwGenericParamList& aOutParamList)
    {
    BINDINGTRACEF("CServiceProxy.HandleAsyncReturnL< ++");
    if(!iExtInterface)
        {
        BINDINGTRACEF("CServiceProxy.HandleAsyncReturnL> -- iExtInterface is Null");
        return 1; //return from if iExtinterface is null.
        }
    TInt lAsyncID = GetAsyncID(aCmdId);
    if(lAsyncID != KAsyncIDNone)
        {
        aOutParamList.AppendL(TLiwGenericParam(KStrTransactionId, TLiwVariant((TInt32)lAsyncID)));
        }
    else
        {
        if (-1 == aCmdId) // provider has not returned the transaction id.

            {
            aOutParamList.AppendL(TLiwGenericParam(KStrTransactionId, TLiwVariant(lAsyncID)));
            aOutParamList.AppendL(TLiwGenericParam(KStrErrorId, TLiwVariant(TInt32(KFlashBindingErrGeneral))));
            }
        }

    //Check if cancel request.
    if ( KLiwEventCanceled == aEventId )
        {
        CleanAsyncResource(aCmdId);
        aOutParamList.Reset();
        BINDINGTRACEF("CServiceProxy.HandleAsyncReturnL> KLiwEventCanceled --");
        return 1;
        }

    //resume player iResumePlayerInBackGround is set true.
    TBool lResumeOnEvent = EFalse;
    TBool lBringToForeground = EFalse;
    
    GetNotificationSettings(aCmdId, lResumeOnEvent, lBringToForeground);
 
    
    if(lResumeOnEvent)
        {
        ResumePlayerL(lBringToForeground);
        }
    
    aOutParamList.AppendL(TLiwGenericParam(KStrEventID, TLiwVariant(TInt32(aEventId))));
    
    for(TInt i = 0; i<iAsyncIDMap.Count(); i++)
         {
         if ( iAsyncIDMap[i]->iAsyncID == lAsyncID )
             {
             CAsyncEvent* lAsyncEvent = CAsyncEvent::NewL( aCmdId, aOutParamList);
             iEventQueue.Append(lAsyncEvent);
             }
         }
    
    if ( KLiwEventCompleted == aEventId) //cleean Async id if async operation is completed.
         {
         CleanAsyncResource(aCmdId);
         }
    aOutParamList.Reset();
    BINDINGTRACEF("CServiceProxy.HandleAsyncReturnL> --");
    return 1;
    }

//global utils function
void CServiceProxy::CancelAsyncReq(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CServiceProxy.CancelAsyncReq< ++");
    ASSERT(aFnInfo);
    ASSERT(aFnInfo->pEI);
    CServiceProxy* lInstanceObject = (CServiceProxy*)aFnInfo->instanceData;
    ASSERT(lInstanceObject);
    ExtensionInterface* lExtInterface = aFnInfo->pEI;
    TInt lArgCount = aFnInfo->numArgs;

    if (lArgCount < KArg3)
        {

        TRAPD(lErr,lInstanceObject->HandleCancelAsyncReqL(aFnInfo))
        ;

        if (lErr)
            {
            FL_OBJECT_HANDLE lOutputObj= NULL;
            FL_VAR_HANDLE lVar= NULL;
            TInt lResult = 0;
            lResult = (lExtInterface->createObject(aFnInfo->pEI, NULL, &lOutputObj)
                    || lExtInterface->createVariable(aFnInfo->pEI, &lVar, NULL));
            if (!lResult)
                {
                lExtInterface->setInt(lExtInterface, lVar, lErr);
                lExtInterface->setProperty(lExtInterface, lOutputObj, (char*)KStrErrorCode,
                        lVar);
                lExtInterface->setObject(lExtInterface, aFnInfo->returnValue,
                        lOutputObj);
                lExtInterface->freeVariable(lExtInterface, lVar);
                }
            }
        }
    BINDINGTRACEF("CServiceProxy.CancelAsyncReq> --");
    }

void CServiceProxy::HandleCancelAsyncReqL(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CServiceProxy.HandleCancelAsyncReqL< ++");
    TInt lAsyncID = 0;

    TInt lResult = 0;
    //get AsyncID to Be cancelled
    lResult = iExtInterface->getInt(iExtInterface, aFnInfo->pArgs[KArg0],
                    &lAsyncID);

    FL_OBJECT_HANDLE lOutputObj= NULL;
    
    if (!lResult)
        {
        for (TInt i = 0; i<iAsyncIDMap.Count(); ++i)
            {

            if (iAsyncIDMap[i]->iAsyncID == lAsyncID)
                {
                CLiwGenericParamList* lInputputParams = CLiwGenericParamList::NewLC();
                CLiwGenericParamList* lOutputParams = CLiwGenericParamList::NewLC();

                // Get the transaction ID.
                TInt32 lTransID = iAsyncIDMap[i]->iTransID;
                TLiwGenericParam lTransIDParam(KStrTransactionId, lTransID);
                lTransIDParam.PushL();
                lInputputParams->AppendL(lTransIDParam);

                iLiwInterface->ExecuteCmdL(KStrCancel, *lInputputParams, *lOutputParams,
                        KLiwOptCancel);
                //parse the output                
                ArgInfo* lArgInfo = BindingHelper::GetArgInfoLC(KArg0, NULL, (char*)KStrObject);
                
                iExtInterface->createObject(iExtInterface, lArgInfo,
                        &lOutputObj);

                CleanupStack::Pop(lArgInfo);    
                delete lArgInfo;
                lArgInfo = NULL;
                if (lOutputObj)
                    {
                    BindingHelper::ConstructOutputParamL(iExtInterface,
                            *lOutputParams, &lOutputObj);
                    iExtInterface->setObject(iExtInterface,
                            aFnInfo->returnValue, lOutputObj);
                    }

                CleanupStack::Pop(&lTransIDParam);
                lTransIDParam.Reset();
                CleanupStack::PopAndDestroy(2);//delete lInputputParams, lOutputParams
                }
            else
                {
                lResult = 1;//in case of second time cancel it should return error code
                }
            }
        BINDINGTRACEF("CServiceProxy.HandleCancelAsyncReqL< ++ for Loop --");
        }
    if(lResult)
        {
        TLiwGenericParam lErrorIDParam;
        lErrorIDParam.PushL();
        lErrorIDParam.SetNameL(KStrErrorId);
        lErrorIDParam.Value().Set(-1);

        CLiwGenericParamList* lErrOutput = CLiwGenericParamList::NewLC();
        lErrOutput->AppendL(lErrorIDParam);
        
        iExtInterface->createObject( iExtInterface, NULL, &lOutputObj );
        BindingHelper::ConstructOutputParamL( iExtInterface, *lErrOutput, &lOutputObj);
        iExtInterface->setObject( iExtInterface, aFnInfo->returnValue, lOutputObj );
        CleanupStack::PopAndDestroy(lErrOutput);
        CleanupStack::Pop(&lErrorIDParam);
        lErrorIDParam.Reset();
        }
    BINDINGTRACEF("CServiceProxy.HandleCancelAsyncReqL> --");
    }

void CServiceProxy::ExecuteAsyncReq(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CServiceProxy.ExecuteAsyncReq< ++");
    ASSERT(aFnInfo);
    ASSERT(aFnInfo->pEI);
    CServiceProxy* lInstanceObject = (CServiceProxy*)aFnInfo->instanceData;
    ASSERT(lInstanceObject);
    
    ExtensionInterface* lExtInterface = aFnInfo->pEI;
    if(!lInstanceObject->isCorePlayerUpdateRunning)
        {
    lExtInterface->startUpdates(lExtInterface, aFnInfo->objectHandle);
        lExtInterface->addRef(aFnInfo->pEI, aFnInfo->objectHandle);
        lInstanceObject->isCorePlayerUpdateRunning = ETrue;
        }
    //get the number of args
    TInt lArgCount = aFnInfo->numArgs;

    if (lArgCount <= KArg4)
        {
        TInt lErr;
        TRAP(lErr,lInstanceObject->HandleAsyncInterfaceExistsL(aFnInfo));

        if (lErr)
            {
            FL_OBJECT_HANDLE lErrOutput= NULL;
            ArgInfo* lArgInfo = NULL;
            
            TRAP(lErr,
                            lArgInfo = BindingHelper::GetArgInfoLC(KArg0, NULL, (char*)KStrObject); 
                            CleanupStack::Pop(lArgInfo) 
            );
            
            if(lErr == KErrNone)
            {
            lExtInterface->createObject(lExtInterface, lArgInfo, &lErrOutput);            
            }
           
            delete lArgInfo;
            lArgInfo = NULL;

            if (lErrOutput)
                {
                FL_VAR_HANDLE lVar= NULL;
                lExtInterface->createVariable(lExtInterface, &lVar, NULL);
                if (lVar)
                    {
                    lExtInterface->setInt(lExtInterface, lVar, lErr);
                    lExtInterface->setProperty(lExtInterface, lErrOutput,
                            (char*)KStrTransId, lVar);
                    lExtInterface->setObject(lExtInterface, aFnInfo->returnValue, lErrOutput);
                    lExtInterface->freeVariable(lExtInterface, lVar);
                    lVar = NULL;
                    }
                }
            }
        }
    BINDINGTRACEF("CServiceProxy.ExecuteAsyncReq> --");
    }

void CServiceProxy::HandleAsyncInterfaceExistsL(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CServiceProxy.HandleAsyncInterfaceExistsL< ++");
    
    if(!iLiwInterface)
        {
        BINDINGTRACEF("CServiceProxy.HandleAsyncInterfaceExistsL> Null LiwInterface--");
        User::Leave(KFlashBindingErrGeneral);// should return because interface is not initialised.
        }
    TInt lResult = 0;
    iExtInterface = aFnInfo->pEI;

    //get the Operation Name
    char* lOperationNameString = NULL;
    FL_OBJECT_HANDLE lInputObject = NULL;
    TInt lAsyncID = 0;

    //get inputs 
    lResult = iExtInterface->getString(iExtInterface, aFnInfo->pArgs[KArg0], &lOperationNameString);
    if(lResult)
        {
        BINDINGTRACEF("CServiceProxy.HandleAsyncInterfaceExistsL>  Error parsing input--");
        User::Leave(KFlashBindingErrGeneral);
        }
    
    lResult = iExtInterface->getInt(iExtInterface, aFnInfo->pArgs[KArg1], &lAsyncID);
    if(lResult)
        {
        BINDINGTRACEF("CServiceProxy.HandleAsyncInterfaceExistsL> Error parsing input--");
        User::Leave(KFlashBindingErrGeneral);
        }   
    
    lResult = iExtInterface->getObject(iExtInterface, aFnInfo->pArgs[KArg2], &lInputObject);
    if(lResult)
        {
        BINDINGTRACEF("CServiceProxy.HandleAsyncInterfaceExistsL> Error parsing input--");
        User::Leave(KFlashBindingErrGeneral);
        }       
   
    FL_OBJECT_HANDLE lNotificationSettingObj = NULL;
    FLExtensionVarType lTypeOfArg; 
    TInt lBringToForeground = 0;
    TInt lResumeOnEvent = 0;
    
    //ParseNotificationSettings(); can be implemented to put this code under that.
    iExtInterface->getType(iExtInterface, aFnInfo->pArgs[KArg3], &lTypeOfArg);
    if(lTypeOfArg == kExtnVarTypeObject)
            {           
            lResult = iExtInterface->getObject(iExtInterface, aFnInfo->pArgs[KArg3], &lNotificationSettingObj);        
            
            if(!lResult && lNotificationSettingObj) //in case of Error set this to default 0.
                {
                FL_VAR_HANDLE lVar = NULL;
                iExtInterface->createVariable(iExtInterface, &lVar, NULL);
                if(!lResult)
                    {
                    lResult = iExtInterface->getPropertyByName(iExtInterface, lNotificationSettingObj, (char*)KStrResumeOnEvent, lVar);
        
                    if(!lResult && lVar)
                        {
                        lResult = iExtInterface->getBool(iExtInterface, lVar, &lResumeOnEvent);
                        if(lResult)
                            {
                            lResumeOnEvent = 0; //set default on error in fetching  
                            }
                        }
                    iExtInterface->freeVariable(iExtInterface, lVar);
                    lVar = NULL;
                    }   
                
                iExtInterface->createVariable(iExtInterface, &lVar, NULL);
                if(!lResult)
                    {
                    iExtInterface->getPropertyByName(iExtInterface, lNotificationSettingObj, (char*)KStrBringToForeGround, lVar);
                    
                    if(!lResult && lVar)
                        {
                        lResult = iExtInterface->getBool(iExtInterface, lVar, &lBringToForeground);
                        if(lResult)
                            {
                            lBringToForeground = 0; //set default on error in fetching  
                            }
                        }
                    iExtInterface->freeVariable(iExtInterface, lVar);
                    lVar = NULL;   
                    }
                BINDINGTRACEF("CServiceProxy.HandleAsyncInterfaceExistsL> lResumeOnEvent %d  lBringToForeground %d ", lResumeOnEvent,lBringToForeground);
                }
            }
    //convert to LIW specific 
    HBufC* lArgOperation = BindingHelper::StringToDescriptorL(lOperationNameString);
    CleanupStack::PushL(lArgOperation);

    HBufC8* lOperationName = EscapeUtils::ConvertFromUnicodeToUtf8L(*lArgOperation);
    CleanupStack::PushL(lOperationName);

    CLiwGenericParamList* lInputParams = CLiwGenericParamList::NewLC();
    
    BindingHelper::ConstructInputParamsL(aFnInfo,lInputObject,*lInputParams);

    CLiwGenericParamList* lOutputParams = CLiwGenericParamList::NewLC();
    
    iLiwInterface->ExecuteCmdL(*lOperationName,*lInputParams,*lOutputParams,KLiwOptASyncronous,this);

    FL_OBJECT_HANDLE lErrOutput = NULL;
    ArgInfo* lArgInfo = BindingHelper::GetArgInfoLC( KArg0, NULL, (char*)KStrObject);

    iExtInterface->createObject( iExtInterface, lArgInfo, &lErrOutput );

    CleanupStack::Pop(lArgInfo);
    delete lArgInfo;
    lArgInfo = NULL;
    
    if(!lErrOutput)
        {
        CleanupStack::PopAndDestroy(lOutputParams);
        CleanupStack::PopAndDestroy(lInputParams);
        CleanupStack::PopAndDestroy(lOperationName);
        CleanupStack::PopAndDestroy(lArgOperation);
        BINDINGTRACEF("CServiceProxy.HandleAsyncInterfaceExistsL> Error creating Object--");
        User::Leave(KFlashBindingErrGeneral);
        }
    
    // Check for error. 
    TInt lErrPos = 0;
    lOutputParams->FindFirst(lErrPos, KStrErrorId);
    if (lErrPos != KErrNotFound)
        {
        TInt32 lErrorCode((*lOutputParams)[lErrPos].Value().AsTInt32());

        FL_VAR_HANDLE lVar = NULL;
        iExtInterface->createVariable(iExtInterface, &lVar, NULL);
        if(lErrOutput && lVar)
            {
            iExtInterface->setInt( iExtInterface, lVar, lErrorCode );

            iExtInterface->setProperty( iExtInterface, lErrOutput, (char*)KStrErrorCode, lVar );
            iExtInterface->freeVariable( iExtInterface, lVar );
            lVar = NULL;

            lErrPos = 0;
            lOutputParams->FindFirst(lErrPos, KStrErrorMessage);
            if(lErrPos != KErrNotFound)
                {
                TPtrC lErrorMessage = (*lOutputParams)[lErrPos].Value().AsDes();

                HBufC8* lBuffer = EscapeUtils::ConvertFromUnicodeToUtf8L(lErrorMessage);
                CleanupStack::PushL(lBuffer);
                char* lErroMessageAsString = BindingHelper::Convert8BitDescritptorToStringL(*lBuffer);

                CleanupStack::PopAndDestroy(lBuffer);

                iExtInterface->createVariable(iExtInterface, &lVar, NULL);
                if(lVar)
                    {
                    iExtInterface->setString( iExtInterface, lVar, lErroMessageAsString );
                    iExtInterface->setProperty( iExtInterface, lErrOutput, (char*)KStrErrMessage , lVar );
                    iExtInterface->freeVariable( iExtInterface, lVar );

                    lVar = NULL;
                    }
                delete lErroMessageAsString;
                lErroMessageAsString = NULL;
                }
            }
        }

    // Store the transaction id for the async operation. The map stores the transcation id returned
    //by the service provider and the corresponding Async Id passed by the ActionScript.
    TInt lPos = 0;
    TInt32 lTransactionId;
    lOutputParams->FindFirst(lPos, KStrTransactionId);
    if(lPos != KErrNotFound)
        {
        lTransactionId = (*lOutputParams)[lPos].Value().AsTInt32();
        TAsyncReqIDMap* asyncReq = new (ELeave)TAsyncReqIDMap(lTransactionId,lAsyncID, TBool(lResumeOnEvent), TBool(lBringToForeground));
        iAsyncIDMap.Append(asyncReq);
        lTransactionId = lAsyncID;
        }
    else
        {
        lTransactionId = (TInt32) KErrNotFound;
        }

    FL_VAR_HANDLE lVar = NULL;
    iExtInterface->createVariable(iExtInterface, &lVar, NULL);
    if( lVar )
        {
        iExtInterface->setInt(iExtInterface, lVar, lTransactionId);
        iExtInterface->setProperty( iExtInterface, lErrOutput, (char*)KStrTransId, lVar );        
        iExtInterface->freeVariable( iExtInterface, lVar);
        lVar = NULL;
        }

    CleanupStack::PopAndDestroy(lOutputParams);
    CleanupStack::PopAndDestroy(lInputParams);
    CleanupStack::PopAndDestroy(lOperationName);
    CleanupStack::PopAndDestroy(lArgOperation);


    iExtInterface->setObject( iExtInterface, aFnInfo->returnValue, lErrOutput );


    BINDINGTRACEF("CServiceProxy.HandleAsyncInterfaceExistsL> --");
    }

void CServiceProxy::ExecuteSyncReq(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CServiceProxy.ExecuteSyncReq< ++");
    ASSERT(aFnInfo);
    ASSERT(aFnInfo->pEI);
    CServiceProxy* lInstanceObject = (CServiceProxy*)aFnInfo->instanceData;
    ASSERT(lInstanceObject);
    
    ExtensionInterface* lExtInterface = aFnInfo->pEI;
    //get the number of args
    TInt lArgCount = aFnInfo->numArgs;
    FL_OBJECT_HANDLE lOutputObj= NULL;

    if (lArgCount < KArg3)
        {
        TInt lErr;
        TRAP(lErr,lInstanceObject->HandleSyncInterfaceExistsL(aFnInfo, &lOutputObj));
        if (lErr != KErrNone)
            {
            FL_VAR_HANDLE lErrcodeVar = NULL;
            ArgInfo* lArgInfo = NULL;
            TRAP(lErr, 
                        lArgInfo = BindingHelper::GetArgInfoLC(KArg0, NULL, (char*)KStrObject); 
                        CleanupStack::Pop(lArgInfo)
             );
            if(lErr == KErrNone)
            {
            lExtInterface->createObject(lExtInterface, NULL, &lOutputObj);
            }
            
            delete lArgInfo;
            lArgInfo = NULL;
            
            if(lOutputObj)
                {
                lExtInterface->createVariable(lExtInterface, &lErrcodeVar, NULL);
                if(lErrcodeVar)
                    {
                    lExtInterface->setInt(lExtInterface, lErrcodeVar, lErr);

                     lExtInterface->setProperty(lExtInterface, lOutputObj, (char*)KStrErrorCode,
                             lErrcodeVar);
                     lExtInterface->setObject(lExtInterface, aFnInfo->returnValue,
                             lOutputObj);                   
                    }
                lExtInterface->freeVariable(lExtInterface, lErrcodeVar);                
                }
            }
         else if (lOutputObj)
            {
            lExtInterface->setObject(lExtInterface, aFnInfo->returnValue,
                    lOutputObj);//return the object to AS.
            }
        }
    BINDINGTRACEF("CServiceProxy.ExecuteSyncReq> --");
    }

void CServiceProxy::HandleSyncInterfaceExistsL(FnInfo* aFnInfo,
        FL_OBJECT_HANDLE* aOutputObj)
    {
    BINDINGTRACEF("CServiceProxy.HandleSyncInterfaceExistsL< ++");
	TInt lResult = KErrNone;

    if (!iLiwInterface)
        {
        BINDINGTRACEF("CServiceProxy.HandleSyncInterfaceExistsL> --");
        User::Leave(KFlashBindingErrGeneral);// should return because interface is not initialised.
        }

    //get the Operation Name
    char* lOperationNameString = NULL;
    FL_OBJECT_HANDLE lInputObject =NULL;

    //get inputs as Object
    lResult = iExtInterface->getString(iExtInterface, aFnInfo->pArgs[KArg0], &lOperationNameString);
        
    if(lResult)
        {
        BINDINGTRACEF("CServiceProxy.HandleSyncInterfaceExistsL>  Error parsing input--");
        User::Leave(KFlashBindingErrGeneral);
        }  
    
    CleanupStack::PushL(lOperationNameString);
    lResult = iExtInterface->getObject(iExtInterface, aFnInfo->pArgs[KArg1], &lInputObject);

    if(!lInputObject)
        {
        ArgInfo* lArgInfo = BindingHelper::GetArgInfoLC(KArg0, NULL, (char*)KStrObject);  
        iExtInterface->createObject(iExtInterface, lArgInfo, &lInputObject);//create dummy object if Object Passed is NULL; this is valid for sync op only    
        CleanupStack::Pop(lArgInfo);
        delete lArgInfo;
        lArgInfo = NULL;        
        }
    
    CleanupStack::PushL(lInputObject);

    //convert it to LIW specific HBufC8*
    HBufC* lArgOperation = BindingHelper::StringToDescriptorL(lOperationNameString);
    CleanupStack::PushL(lArgOperation);

    HBufC8* lOperationName = EscapeUtils::ConvertFromUnicodeToUtf8L(*lArgOperation);
    CleanupStack::PushL(lOperationName);

    CLiwGenericParamList* lInputParams = CLiwGenericParamList::NewLC();
    CLiwGenericParamList* lOutputParams = CLiwGenericParamList::NewLC();

    //parse the args using helper
    BindingHelper::ConstructInputParamsL(aFnInfo,lInputObject,*lInputParams);

    iLiwInterface->ExecuteCmdL(*lOperationName, *lInputParams, *lOutputParams);

    

    ArgInfo* lArgInfo = BindingHelper::GetArgInfoLC(KArg0, NULL, (char*)KStrObject);  

    lResult = iExtInterface->createObject(iExtInterface, lArgInfo, aOutputObj);

    CleanupStack::Pop(lArgInfo);
    delete lArgInfo;
    lArgInfo = NULL;

    if( aOutputObj )
        {        
        BindingHelper::ConstructOutputParamL(iExtInterface,*lOutputParams, aOutputObj); //convert from LIW to AS specific
        }
    else
        {
        lResult = KFlashBindingErrGeneral;
        }
    //parse the input from AS specific to LIW specific

    CleanupStack::PopAndDestroy(lOutputParams);
    CleanupStack::PopAndDestroy(lInputParams);
    CleanupStack::PopAndDestroy(lOperationName);
    CleanupStack::PopAndDestroy(lArgOperation);
    CleanupStack::Pop(lInputObject);
    CleanupStack::Pop(lOperationNameString);
    //free string from core player.
    iExtInterface->freeString(iExtInterface,lOperationNameString);
    lOperationNameString = NULL;    
    
    BINDINGTRACEF("CServiceProxy.HandleSyncInterfaceExistsL> --");
    }

// Gets the Async Id corresponding to the transaction id. The ActionScript passes the Async Id
// to identify the different async operations. The binding maintains a map between this 
// async id and the transaction id passed by the service provider.
TInt CServiceProxy::GetAsyncID(TInt aTransID)
    {
    BINDINGTRACEF("CServiceProxy.GetAsyncID< ++");
    for(TInt i = 0; i!=iAsyncIDMap.Count(); ++i)
        {
        if(aTransID==iAsyncIDMap[i]->iTransID)
            {
            BINDINGTRACEF("CServiceProxy.GetAsyncID> --");
            return iAsyncIDMap[i]->iAsyncID;
            }
        }
    BINDINGTRACEF("CServiceProxy.GetAsyncID> --");
    return KAsyncIDNone;
    }

void CServiceProxy::GetNotificationSettings(TInt aTransID, TBool& aResumeOnEvent, TBool& aBringToForeground)
    {
    for(TInt i = 0; i!=iAsyncIDMap.Count(); ++i)
        {
        if(aTransID==iAsyncIDMap[i]->iTransID)
            {
            aResumeOnEvent = iAsyncIDMap[i]->iResumeOnEvent;
            aBringToForeground = iAsyncIDMap[i]->iBringToForeground;            
            }
        }
    }
// Cleans the resources when an async event is cancelled.
void CServiceProxy::CleanAsyncResource(TInt aTransID)
    {
    BINDINGTRACEF("CServiceProxy.CleanAsyncResource< ++");
    for(TInt i = 0; i!=iAsyncIDMap.Count(); ++i)
        {
        if(aTransID==iAsyncIDMap[i]->iTransID)
            {
            TAsyncReqIDMap* lTempAsyncIDMap = iAsyncIDMap[i];
            iAsyncIDMap.Remove(i);
            delete lTempAsyncIDMap;
            lTempAsyncIDMap = NULL;
            if ( 0 ==iAsyncIDMap.Count())
                {
                iAsyncIDMap.ResetAndDestroy();
                }
            BINDINGTRACEF("CServiceProxy.CleanAsyncResource> --");
            return;
            }
        }
    BINDINGTRACEF("CServiceProxy.CleanAsyncResource> --");
    }

ClassInfo* CServiceProxy::GetClassInfoL()
    {
    BINDINGTRACEF("CServiceProxy.GetClassInfo< ++");
        ClassInfo* lServiceProxyInfo = new(ELeave) ClassInfo;
    CleanupStack::PushL((TAny*)lServiceProxyInfo);
    lServiceProxyInfo->className = (char*)KClassNameServiceProxy;
    lServiceProxyInfo->methodCount = 5;

    lServiceProxyInfo->methodNames = new(ELeave) char*[5];
    CleanupStack::PushL((TAny*)lServiceProxyInfo->methodNames);
    lServiceProxyInfo->methodNames[0] = (char*)KMethodNameLoad;
    lServiceProxyInfo->methodNames[1] = (char*)KMethodNameExecuteAsyncReq;
    lServiceProxyInfo->methodNames[2] = (char*)KMethodNameExecuteSyncReq;
    lServiceProxyInfo->methodNames[3] = (char*)KMethodNameCancelAsyncReq;
    lServiceProxyInfo->methodNames[4] = (char*)KMethodNameUnloadService;

    lServiceProxyInfo->methodTypes = new(ELeave) int[5];
    CleanupStack::PushL((TAny*)lServiceProxyInfo->methodTypes);
    lServiceProxyInfo->methodTypes[0] = METHODTYPE_MEMBER_FUNCTION;
    lServiceProxyInfo->methodTypes[1] = METHODTYPE_MEMBER_FUNCTION;
    lServiceProxyInfo->methodTypes[2] = METHODTYPE_MEMBER_FUNCTION;
    lServiceProxyInfo->methodTypes[3] = METHODTYPE_MEMBER_FUNCTION;
    lServiceProxyInfo->methodTypes[4] = METHODTYPE_MEMBER_FUNCTION;

    lServiceProxyInfo->methods = new(ELeave) METHOD[5];
    CleanupStack::PushL((TAny*)lServiceProxyInfo->methods);
    lServiceProxyInfo->methods[0] = CServiceProxy::Load;
    lServiceProxyInfo->methods[1] = CServiceProxy::ExecuteAsyncReq;
    lServiceProxyInfo->methods[2] = CServiceProxy::ExecuteSyncReq;
    lServiceProxyInfo->methods[3] = CServiceProxy::CancelAsyncReq;
    lServiceProxyInfo->methods[4] = CServiceProxy::UnloadService;

    lServiceProxyInfo->constructor = CServiceProxy::Constructor;
    lServiceProxyInfo->destructor = CServiceProxy::Destructor;
    lServiceProxyInfo->update = CServiceProxy::Update;
    lServiceProxyInfo->firesEvents = 1;
    lServiceProxyInfo->next = NULL;

    CleanupStack::Pop(4);

    BINDINGTRACEF("CServiceProxy.GetClassInfo> --");
    return lServiceProxyInfo;

    }

// Gets the list of service providers with the specified criteria and
// version specification.
void ServiceUtilsProxy::GetServiceList(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("ServiceUtilsProxy.GetServiceList< ++");
    ASSERT(aFnInfo);
    ASSERT(aFnInfo->pEI);
    ExtensionInterface* lExtInterface = aFnInfo->pEI;

    FL_OBJECT_HANDLE lOutputObj= NULL;
    TInt lErr;
    TRAP(lErr,HandleGetServiceListL( aFnInfo, &lOutputObj ));
    if (lErr != KErrNone)
        {
        FL_VAR_HANDLE lErrcodeVar= NULL;
        ArgInfo* lArgInfo = NULL;
        TRAP(lErr, 
                    lArgInfo = BindingHelper::GetArgInfoLC(KArg0, NULL, (char*)KStrObject );
                    CleanupStack::Pop(lArgInfo)
        );
        if(lErr == KErrNone)
        {
        lExtInterface->createObject(aFnInfo->pEI, lArgInfo, &lOutputObj);
        }
        
        delete lArgInfo;
        lArgInfo = NULL;
        
        if(lOutputObj)
            {
            lExtInterface->createVariable(aFnInfo->pEI, &lErrcodeVar, NULL);
            if(lErrcodeVar)
                {
                lExtInterface->setInt(aFnInfo->pEI, lErrcodeVar, lErr);
                lExtInterface->setProperty(aFnInfo->pEI, lOutputObj, (char*)KStrErrorCode,
                        lErrcodeVar);
                lExtInterface->setObject(aFnInfo->pEI, aFnInfo->returnValue, lOutputObj);
                lExtInterface->freeVariable(aFnInfo->pEI, lErrcodeVar);               
                }
            }
        }
    else if (lOutputObj)
        {
        aFnInfo->pEI->setObject(aFnInfo->pEI, aFnInfo->returnValue,
                lOutputObj);
        }

    BINDINGTRACEF("ServiceUtilsProxy.GetServiceList> --");
    }

void ServiceUtilsProxy::HandleGetServiceListL(FnInfo* aFnInfo,
        FL_OBJECT_HANDLE* aOutputObj)
    {
    BINDINGTRACEF("ServiceUtilsProxy.HandleGetServiceListL< ++");
    TInt lResult = KErrNone;
    
    ExtensionInterface* lExtInterface = aFnInfo->pEI;

    CFlashBindingASExtDelegate* lFlashBinding =
            static_cast<CFlashBindingASExtDelegate*>(Dll::Tls() );
            
    if(!lFlashBinding)
        {
        User::Leave(KFlashBindingErrGeneral);
        }
    
    CLiwServiceHandler* lServiceHandler = lFlashBinding->GetLiwServiceHandler();
    
    if(!lServiceHandler)
        {
        User::Leave(KFlashBindingErrGeneral);
        }
    
    if (aFnInfo->numArgs == KArg3)
        {
        char* lServiceNameString= NULL;
        char* lInterfaceNameString= NULL;
        FL_OBJECT_HANDLE lVerObject= NULL;

        lResult = lExtInterface->getString(lExtInterface, aFnInfo->pArgs[KArg0], &lServiceNameString);
 
        if (lResult)
            {
            BINDINGTRACEF("ServiceUtilsProxy.HandleGetServiceListL> --");
            User::Leave(KFlashBindingErrGeneral);
            }
        
        CleanupStack::PushL(lServiceNameString);
        
        lResult = lExtInterface->getString(lExtInterface, aFnInfo->pArgs[KArg1], &lInterfaceNameString);
        
        if (lResult)
            {
            BINDINGTRACEF("ServiceUtilsProxy.HandleGetServiceListL> --");
            User::Leave(KFlashBindingErrGeneral);
            }
        
        CleanupStack::PushL(lInterfaceNameString);
        
        lResult = lExtInterface->getObject(lExtInterface, aFnInfo->pArgs[KArg2], &lVerObject);
        
        if (!lVerObject)
            {
            ArgInfo* lArgInfo = BindingHelper::GetArgInfoLC(KArg0, NULL, (char*)KStrObject);  
            lExtInterface->createObject(lExtInterface, lArgInfo, &lVerObject);//create dummy object if Object Passed is NULL; this is valid for sync op only    
            CleanupStack::Pop(lArgInfo);
            delete lArgInfo;
            lArgInfo = NULL;            
            }        

        CleanupStack::PushL(lVerObject);
        
        TPtrC8 lServiceName(reinterpret_cast<const TUint8*>(lServiceNameString));
        TPtrC8 lInterfaceName(reinterpret_cast<const TUint8*>(lInterfaceNameString));

        RCriteriaArray lCritArray;
        TCleanupItem lCritArrayCleanItem( BindingHelper::PointerArrayCleanup , &lCritArray); 
        CleanupStack::PushL(lCritArrayCleanItem);
                
        CLiwCriteriaItem* lCrit = CLiwCriteriaItem::NewLC(1, lServiceName, lInterfaceName);
        lCrit->SetServiceClass(TUid::Uid(KLiwClassBase));       
        
        // Set the version information.
        lResult = BindingHelper::SetMetaDataForCriteriaItemL(lExtInterface, lCrit, lVerObject);
        lCritArray.AppendL(lCrit);
        
        if (lResult)
            {
            RCriteriaArray lProviderList;
            TCleanupItem lProviderListCleanItem( BindingHelper::PointerArrayCleanup , &lProviderList); 
            CleanupStack::PushL(lProviderListCleanItem);

            //Query for the list of providers with the above criteria.
            lServiceHandler->QueryImplementationL(lCritArray, lProviderList);
            CLiwGenericParamList* lOutparams = &(lServiceHandler->OutParamListL());

            // for each provider
            for(TInt idx = 0; idx < lProviderList.Count(); ++idx)
                {
                CLiwCriteriaItem* lProvInfo = lProviderList[idx];

                TPtrC8 lServNamebuffer = lProvInfo->ServiceCmdStr();
                HBufC* lServiceName = EscapeUtils::ConvertToUnicodeFromUtf8L(lServNamebuffer);
                CleanupStack::PushL(lServiceName);

                TLiwVariant lServiceVar;
                lServiceVar.PushL();
                lServiceVar.Set(lServiceName);
                TLiwGenericParam lServiceList(KStrServiceName(), lServiceVar);
                lServiceList.PushL();
                lOutparams->AppendL(lServiceList);
                CleanupStack::Pop(&lServiceList);
                CleanupStack::Pop(&lServiceVar);
                lServiceVar.Reset();
                lServiceList.Reset();
                CleanupStack::PopAndDestroy(lServiceName);

                TPtrC8 lIntfNameBuffer = lProvInfo->ContentType();
                HBufC* lInterfaceName = EscapeUtils::ConvertToUnicodeFromUtf8L(lIntfNameBuffer);
                CleanupStack::PushL(lInterfaceName);

                TLiwVariant lInterfaceVar;
                lInterfaceVar.PushL();
                lInterfaceVar.Set(lInterfaceName);
                TLiwGenericParam lInterfaceList(KStrInterfaceName(), lInterfaceVar);
                lInterfaceList.PushL();
                lOutparams->AppendL(lInterfaceList);
                
                CleanupStack::Pop(&lInterfaceList);
                CleanupStack::Pop(&lInterfaceVar);
                lInterfaceVar.Reset();
                lInterfaceList.Reset();
                CleanupStack::PopAndDestroy(lInterfaceName);

                TLiwVariant lProvMetadata;
                lProvMetadata.PushL();
                //getting metadata from the provider (contains version information)
                lProvInfo->GetMetaDataOptions(lProvMetadata);
                TLiwGenericParam lOutputproviderList(KStrServiceList(), lProvMetadata);
                lOutputproviderList.PushL();
                lOutparams->AppendL(lOutputproviderList);
                CleanupStack::Pop(&lOutputproviderList);
                CleanupStack::Pop(&lProvMetadata);
                lProvMetadata.Reset();
                lOutputproviderList.Reset();
                }

            lExtInterface->createObject(lExtInterface, NULL, aOutputObj);
            if(*aOutputObj)
                {
                BindingHelper::ConstructOutputParamL(lExtInterface, *lOutparams, aOutputObj);
                }
            else
                {
                lResult = KFlashBindingErrGeneral;
                }

            lOutparams->Reset();
            CleanupStack::PopAndDestroy(&lProviderListCleanItem); 
            }
        else
            {
            lResult = KFlashBindingErrVersion;
            }

        
        CleanupStack::Pop(lCrit);
        CleanupStack::PopAndDestroy(&lCritArrayCleanItem);      
        CleanupStack::Pop(lVerObject);
        CleanupStack::Pop(lInterfaceNameString);
        CleanupStack::Pop(lServiceNameString);
        lExtInterface->freeString(lExtInterface, lServiceNameString);
        lExtInterface->freeString(lExtInterface, lInterfaceNameString);
        lServiceNameString = NULL;
        lInterfaceNameString = NULL;
        }
    lFlashBinding = NULL;
    lExtInterface = NULL;

    BINDINGTRACEF("ServiceUtilsProxy.HandleGetServiceListL> --");
    }

ClassInfo* ServiceUtilsProxy::GetClassInfoL()
    {
    BINDINGTRACEF("ServiceUtilsProxy.GetClassInfo< ++");
    ClassInfo* lServiceUtilsProxyInfo = new(ELeave) ClassInfo;
    CleanupStack::PushL((TAny*)lServiceUtilsProxyInfo);
    lServiceUtilsProxyInfo->className = (char*)KClassNameServiceUtilsProxy;
    lServiceUtilsProxyInfo->methodCount = 1;

    lServiceUtilsProxyInfo->methodNames = new(ELeave) char*[1];
    CleanupStack::PushL((TAny*)lServiceUtilsProxyInfo->methodNames);
    lServiceUtilsProxyInfo->methodNames[0] = (char*)KMethodNameGetServiceList;

    lServiceUtilsProxyInfo->methodTypes = new(ELeave) int[1];
    CleanupStack::PushL((TAny*)lServiceUtilsProxyInfo->methodTypes);
    lServiceUtilsProxyInfo->methodTypes[0] = METHODTYPE_STATIC_FUNCTION;

    lServiceUtilsProxyInfo->methods = new(ELeave) METHOD[1];
    CleanupStack::PushL((TAny*)lServiceUtilsProxyInfo->methods);
    lServiceUtilsProxyInfo->methods[0] = ServiceUtilsProxy::GetServiceList;

    lServiceUtilsProxyInfo->constructor = NULL;
    lServiceUtilsProxyInfo->destructor = NULL;
    lServiceUtilsProxyInfo->update = NULL;
    lServiceUtilsProxyInfo->firesEvents = 0;
    lServiceUtilsProxyInfo->next = NULL;

    CleanupStack::Pop(4);
    BINDINGTRACEF("ServiceUtilsProxy.GetClassInfo> --");
    return lServiceUtilsProxyInfo;
    }

CAsyncEvent* CAsyncEvent::NewL(TInt32 aTransID, CLiwGenericParamList& aLiwOutputParams)
    {
    BINDINGTRACEF("CAsyncEvent.NewL<++");
    CAsyncEvent* self =new (ELeave) CAsyncEvent;
    CleanupStack::PushL(self);
    self->ConstructL(aTransID,aLiwOutputParams);
    CleanupStack::Pop(self);
    BINDINGTRACEF("CAsyncEvent.NewL> --");
    return self;    
    }

void CAsyncEvent::ConstructL(TInt32 aTransID, CLiwGenericParamList& aLiwOutputParams)
    {
    BINDINGTRACEF("CAsyncEvent.ConstructL< ++");
    iTransID = aTransID;
    iAsyncOutput = CLiwGenericParamList::NewL();
    iAsyncOutput->AppendL(aLiwOutputParams);
    BINDINGTRACEF("CAsyncEvent.ConstructL> --");
    }

CAsyncEvent::~CAsyncEvent()
    {
    BINDINGTRACEF("CAsyncEvent.~CAsyncEvent< ++");
    if(iAsyncOutput)
        {
        iAsyncOutput->Reset();
        delete iAsyncOutput;
        iAsyncOutput = NULL;
        }         
    BINDINGTRACEF("CAsyncEvent.~CAsyncEvent> --");
    }


ClassInfo* CMapProxy::GetClassInfoL()
    {
    BINDINGTRACEF("CMapProxy.GetClassInfo< ++");
    ClassInfo* lMapProxyInfo = new(ELeave) ClassInfo;
    CleanupStack::PushL((TAny*)lMapProxyInfo);
    lMapProxyInfo->className = (char*)KClassNameMapProxy;
    lMapProxyInfo->methodCount = 1;

    lMapProxyInfo->methodNames = new(ELeave) char*[1];
    CleanupStack::PushL((TAny*)lMapProxyInfo->methodNames);
    lMapProxyInfo->methodNames[0] = (char*)KMethodNameResolve;

    lMapProxyInfo->methodTypes = new(ELeave) int[1];
    CleanupStack::PushL((TAny*)lMapProxyInfo->methodTypes);
    lMapProxyInfo->methodTypes[0] = METHODTYPE_MEMBER_FUNCTION;

    lMapProxyInfo->methods = new(ELeave) METHOD[1];
    CleanupStack::PushL((TAny*)lMapProxyInfo->methods);
    lMapProxyInfo->methods[0] = CMapProxy::Resolve;

    lMapProxyInfo->constructor = CMapProxy::Constructor;
    lMapProxyInfo->destructor = CMapProxy::Destructor;
    lMapProxyInfo->update = NULL;
    lMapProxyInfo->firesEvents = 0;
    lMapProxyInfo->next = NULL;

    CleanupStack::Pop(4);

    BINDINGTRACEF("CMapProxy.GetClassInfo> --");
    return lMapProxyInfo;
    }


void CMapProxy::Constructor(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CMapProxy.Constructor< ++");
    ASSERT(aFnInfo);   
    CMapProxy* lMap = NULL;
    TRAPD(lErr, lMap = CreateInstanceL(aFnInfo));  
    if(lErr)
        {
        aFnInfo->instanceData = NULL;     
        }
    else
        {        
        aFnInfo->instanceData = (FL_VAR_HANDLE)lMap;
        }      

    BINDINGTRACEF("CMapProxy.Constructor> --");
    }
    
CMapProxy* CMapProxy::CreateInstanceL(FnInfo* aFnInfo)
        {
            
    CMapProxy* lMap = new(ELeave) CMapProxy();
    CleanupStack::PushL(lMap);
    int lMapaddress;

    aFnInfo->pEI->getInt(aFnInfo->pEI, aFnInfo->pArgs[0], &lMapaddress);

    TLiwVariant* lVar = reinterpret_cast<TLiwVariant*>(lMapaddress);
    if(lVar)
        {
        lMap->iVariant.SetL(*lVar);        
        }
    
    CleanupStack::Pop(lMap);    
    return lMap;
        }

void CMapProxy::Destructor(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CMapProxy.Destructor< ++");
    ASSERT(aFnInfo);
    ASSERT(aFnInfo->pEI);
    CMapProxy* lInstanceObject = reinterpret_cast<CMapProxy*>(aFnInfo->instanceData);
    delete lInstanceObject;
    lInstanceObject = NULL;
    BINDINGTRACEF("CMapProxy.Destructor> --");
    }

void CMapProxy::Resolve(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CMapProxy.Resolve< ++");
    ASSERT(aFnInfo);
    ASSERT(aFnInfo->pEI);
    ExtensionInterface* lExtInterface = aFnInfo->pEI;
    CMapProxy* lInstanceObject = (CMapProxy*)aFnInfo->instanceData;
    
    TRAPD(lErr,lInstanceObject->HandleResolveL(aFnInfo));
    
    if(lErr)
        {
        lExtInterface->setSpecial(lExtInterface, aFnInfo->returnValue, kExtnVarTypeUndefined);
        }   
    
    BINDINGTRACEF("CMapProxy.Resolve> --");
    }

void CMapProxy::HandleResolveL(FnInfo* aFnInfo)
    {
    BINDINGTRACEF("CMapProxy.HandleResolveL< ++");
    ExtensionInterface* lExtInterface = aFnInfo->pEI;
    
    TInt lArgCnt = aFnInfo->numArgs;

    char* lFunctionNameAsString;
    TInt lResult = 0;
    if(lArgCnt == 1)
        {
        lResult = lExtInterface->getString(lExtInterface, aFnInfo->pArgs[0], &lFunctionNameAsString);
        CleanupStack::PushL(lFunctionNameAsString);
        
        if(!lResult && lFunctionNameAsString)
            {            
            TPtrC8 lFunctionNameAsDes(reinterpret_cast<const TUint8*>(lFunctionNameAsString));      
           
            if(!lFunctionNameAsDes.Compare(KStrMapClose)) //if Map.Close is called 
                {
                iVariant.Reset();
                }
            else //find the key
                {             
                const CLiwMap* lMap = iVariant.AsMap();     
                if(lMap)
                    {
                    TLiwVariant lEntry;
                    lEntry.PushL();
                    
                    if(EFalse!=lMap->FindL(lFunctionNameAsDes,lEntry))
                        {
                        BindingHelper::ConstructOutputParamL(lExtInterface, lEntry, &aFnInfo->returnValue);
                        }
                    else
                        {
                        lExtInterface->setSpecial(lExtInterface, aFnInfo->returnValue, kExtnVarTypeUndefined);
                        }
                    
                    CleanupStack::Pop(&lEntry);
                    lEntry.Reset();
                    }
                }
            }        
        
        CleanupStack::Pop(lFunctionNameAsString);
        lExtInterface->freeString(lExtInterface, lFunctionNameAsString);
        lFunctionNameAsString = NULL;
        } 
    else
        {
        lExtInterface->setSpecial(lExtInterface, aFnInfo->returnValue, kExtnVarTypeUndefined);
        }
    BINDINGTRACEF("CMapProxy.HandleResolveL> --");
    }
