/*
* Copyright (c) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the DeviceCmdsProxy class. 
 *
*/


#ifndef __DEVICECMDS_PROXY_H__
#define __DEVICECMDS_PROXY_H__

#include <e32std.h>	
#include <stdio.h> 
#include "flashliteExtensionInterfaces.h"
#include "flashliteinterfaces.h"

#include "flashengineintftoasextension.h"

#ifdef FLASH_EXTENSION_GENERIC  
class CFLVarHandle : public CBase
    {
public:
    /**
     Constructs a new CFlvVarHandle object.
     
     @param aEI The ExtensionInterface pointer which will be used for cleaning up of mamber variable iVarHandle.
     @return a new CFlvVarHandle object and puts it on to Cleanup Stack
     */
    static CFLVarHandle* NewLC(ExtensionInterface* aEI);

    /**
     Returns Handle to FL_VAR_HANDLE.
     
     @return a Hanlde to variable created on core player side of type FL_VAR_HANDLE.
     */
    FL_VAR_HANDLE Handle();

    /**
     Destructor
     */
    ~CFLVarHandle();

private:

    /**
     Constructor.
     
     @param pointer to ExtensionInterface which will be used to clenup of iVarHandle.
     */
    CFLVarHandle(ExtensionInterface* aEI) :
        iExtInterface(aEI)
        {
        }
    ;

    /**
     Second Phase constructor.      
     */
    void ConstructL();

private:
    //members

    FL_VAR_HANDLE iVarHandle;

    ExtensionInterface* iExtInterface;
    };

#endif

class DeviceCmdsProxy
    {
public:

    static ClassInfo* GetClassInfoL();
    
    static void ProcessDeviceCmd(FnInfo* aFnInfo);
    
private:

    void ProcessDeviceCmdL(FnInfo* aFnInfo);
    
    static void HandleBoolL(ExtensionInterface* aEI, FL_VAR_HANDLE aBool, RASExtensionArgArray& aArgList);
    
    static void PointerArrayCleanup(TAny* aArray);     

    //private implementation for handling Coreplayer specific string
    class CorePlayerStringWrapper: public CBase
        {
        public:
            ~CorePlayerStringWrapper()
                {
                iExtInterface->freeString(iExtInterface, iStr);
                iStr = NULL;
                iExtInterface = NULL;
                }
        public:
            char* iStr;
            ExtensionInterface* iExtInterface;        
        };
    
#ifdef FLASH_EXTENSION_GENERIC     
    
    struct ExtensionArgListBundle
        {
        RASExtensionArg iDeviceArgList;
        ExtensionInterface* iEI;
        };
    
    static void HandleIntL(ExtensionInterface* aEI, FL_VAR_HANDLE aInteger, RASExtensionArg& aArgList);    
    
    static void HandleNumberL(ExtensionInterface* aEI, FL_VAR_HANDLE aNumber, RASExtensionArg& aArgList);
    
    static void HandleStringL(ExtensionInterface* aEI, FL_VAR_HANDLE aString, RASExtensionArg& aArgList);
 
    static TBool IsInternalProperty(TDesC8& aPropName);
    
    void HandleArrayL(ExtensionInterface* aExtInterface, FL_VAR_HANDLE aObject, RASExtensionArg& aArgList);
    
    ArgInfo* GetArgInfoLC(TInt aArgCnt, FL_VAR_HANDLE* aArgList, char* aName);
#endif    
    };

#endif //__DEVICECMDS_PROXY_H__
