/*
* Copyright (c) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the member functions of DeviceCmdsProxy class.
 
 *
*/

#include <escapeutils.h>
#include "devicecmdsproxy.h"
#include "debuglog.h"
#include "devicecmdsdelegate.h"
#include "asextconsts.h"

const TInt KArg0 = 0;
const TInt KArg1 = 1;
const TInt KArg2 = 2;

const TInt KMethodCount = 1;

//class and method names
const char* const KClassNameDevice = "com.nokia.impl.DeviceProxy";
const char* const KMethodNameProcessDeviceCmds = "ProcessDeviceCmd";


ClassInfo* DeviceCmdsProxy::GetClassInfoL()
    {
    DEVICECMDSTRACEF("DeviceCmdsProxy.GetClassInfo< ++");
    ClassInfo* lDeviceCmdsProxyInfo = new(ELeave) ClassInfo;
    CleanupStack::PushL((TAny*)lDeviceCmdsProxyInfo);
    lDeviceCmdsProxyInfo->className = (char*)KClassNameDevice;
    lDeviceCmdsProxyInfo->methodCount = KMethodCount;

    lDeviceCmdsProxyInfo->methodNames = new(ELeave) char*[KMethodCount];
    CleanupStack::PushL((TAny*)lDeviceCmdsProxyInfo->methodNames);
    lDeviceCmdsProxyInfo->methodNames[0] = (char*)KMethodNameProcessDeviceCmds;
 
    lDeviceCmdsProxyInfo->methodTypes = new(ELeave) int[KMethodCount];
    CleanupStack::PushL((TAny*)lDeviceCmdsProxyInfo->methodTypes);
    lDeviceCmdsProxyInfo->methodTypes[0] = METHODTYPE_STATIC_FUNCTION;

    lDeviceCmdsProxyInfo->methods = new(ELeave) METHOD[KMethodCount];
    CleanupStack::PushL((TAny*)lDeviceCmdsProxyInfo->methods);
    lDeviceCmdsProxyInfo->methods[0] = DeviceCmdsProxy::ProcessDeviceCmd;

    lDeviceCmdsProxyInfo->constructor = NULL;
    lDeviceCmdsProxyInfo->destructor = NULL;
    
    lDeviceCmdsProxyInfo->update = NULL;
    lDeviceCmdsProxyInfo->firesEvents = 0;
    lDeviceCmdsProxyInfo->next = NULL;

    CleanupStack::Pop(4);

    DEVICECMDSTRACEF("DeviceCmdsProxy.GetClassInfo> --");
    return lDeviceCmdsProxyInfo;

    }

void DeviceCmdsProxy::ProcessDeviceCmd(FnInfo* aFnInfo)
    {   
    DEVICECMDSTRACEF("DeviceCmdsProxy::ProcessDeviceCmds< ++");
    DeviceCmdsProxy* lDeviceProxyInstance = (DeviceCmdsProxy*)aFnInfo->instanceData;
   
    TRAPD(err,lDeviceProxyInstance->ProcessDeviceCmdL(aFnInfo));
    
    if(err)
        {        
        aFnInfo->pEI->setInt(aFnInfo->pEI, aFnInfo->returnValue, KASErrGenral);       
        }
    
    DEVICECMDSTRACEF("DeviceCmdsProxy::ProcessDeviceCmds> --");
    }

void DeviceCmdsProxy::ProcessDeviceCmdL(FnInfo* aFnInfo)
    {
    DEVICECMDSTRACEF("DeviceCmdsProxy::HandleProcessDeviceCmdsL< ++");
    CDeviceCmdsASExtDelegate* lDeviceCmdsASExtDelegate = static_cast<CDeviceCmdsASExtDelegate*>(Dll::Tls() );
    MFlashEngineIntfToASExtension* lFlashEngine = lDeviceCmdsASExtDelegate->GetFlashEngine();
    
    ExtensionInterface* lExtInterface = aFnInfo->pEI;
    
    TInt lArgCount = aFnInfo->numArgs;
    TInt lResult;

    RASExtensionArgArray lDeviceArgList;
        
    TCleanupItem lDeviceCmdArgListCleanItem( DeviceCmdsProxy::PointerArrayCleanup , &lDeviceArgList); 
    CleanupStack::PushL(lDeviceCmdArgListCleanItem);
    
    if (lArgCount == KArg2)// Always there will be 2 paramters, first as Command Name and second "agruments" array or element of argument 
        {
        CorePlayerStringWrapper* lCPStringWrapper = new (ELeave) CorePlayerStringWrapper;
        CleanupStack::PushL(lCPStringWrapper);
        lCPStringWrapper->iExtInterface = lExtInterface;        
        
        lResult = lExtInterface->getString(lExtInterface, aFnInfo->pArgs[KArg0], &lCPStringWrapper->iStr);

        if (lResult && !lCPStringWrapper->iStr)
            {
            User::Leave(KASErrGenral);
            }        

        TPtrC8 lPtr(reinterpret_cast<const TUint8*>(lCPStringWrapper->iStr));
        HBufC* lCmdNameAsDesc = EscapeUtils::ConvertToUnicodeFromUtf8L(lPtr);
        CleanupStack::PushL(lCmdNameAsDesc);

        FLExtensionVarType lTypeOfVariable;

        lExtInterface->getType(lExtInterface, aFnInfo->pArgs[KArg1],&lTypeOfVariable);
          
        if(lTypeOfVariable == kExtnVarTypeBoolean) //as we are using second parameter "arguments[0]" from actionscript which is of boolean type meant for SensorAutorotation. 
            {
            DeviceCmdsProxy::HandleBoolL(lExtInterface, aFnInfo->pArgs[KArg1], lDeviceArgList);  
            lFlashEngine->ProcessHostCmdL(*lCmdNameAsDesc, lDeviceArgList, lResult);
            }
        else
            {
            lResult = KASErrInvalidInput;
            }       

        lExtInterface->setInt(lExtInterface, aFnInfo->returnValue, lResult);
          
        CleanupStack::PopAndDestroy(lCmdNameAsDesc);
        CleanupStack::PopAndDestroy(lCPStringWrapper);        
        }    
    else
        {
        lExtInterface->setInt(lExtInterface, aFnInfo->returnValue, KASErrInvalidInput);
        }
    CleanupStack::PopAndDestroy();
    DEVICECMDSTRACEF("DeviceCmdsProxy::HandleProcessDeviceCmdsL> --");
    }

void DeviceCmdsProxy::HandleBoolL(ExtensionInterface* aEI, FL_VAR_HANDLE aBool, RASExtensionArgArray& aArgList)
    {
    TInt lData;
    TInt lResult; 
    
    lResult = aEI->getBool(aEI, aBool, &lData);
    
    if(!lResult)
        {
        ExtensionArg* lEngineSpecificArg = new(ELeave) ExtensionArg;
        CleanupStack::PushL(lEngineSpecificArg);
        lEngineSpecificArg->pVariableValue = (void*) lData;
        lEngineSpecificArg->variableType = kExtnVarTypeBoolean;
        
        aArgList.AppendL(lEngineSpecificArg);        
        CleanupStack::Pop(lEngineSpecificArg);        
        }
    else
        {
        User::Leave(KASErrGenral);
        }
    }

void DeviceCmdsProxy::PointerArrayCleanup(TAny* aArray)
    {
    static_cast<RASExtensionArgArray*>(aArray)->ResetAndDestroy();    
    }

#undef FLASH_EXTENSION_GENERIC
#ifdef FLASH_EXTENSION_GENERIC
void DeviceCmdsProxy::ProcessDeviceCmdsL(FnInfo* aFnInfo)
    {
    DEVICECMDSTRACEF("DeviceCmdsProxy::HandleProcessDeviceCmdsL< ++");
    ExtensionInterface* lExtInterface = aFnInfo->pEI;
    
    TInt lArgCount = aFnInfo->numArgs;
    TInt lResult;

    ExtensionArgListBundle* lBundledDeviceArg = new ExtensionArgListBundle;
    
    lBundledDeviceArg->iEI = aFnInfo->pEI;    
    
    TCleanupItem lDeviceCmdArgListCleanItem( DeviceCmdsProxy::PointerArrayCleanup , lBundledDeviceArg); 
    CleanupStack::PushL(lDeviceCmdArgListCleanItem);
    
    if (lArgCount == KArg2)
        {
        char *lCmdNameAsString;
        lResult = lExtInterface->getString(lExtInterface, aFnInfo->pArgs[KArg0], &lCmdNameAsString);

        if (lResult && !lCmdNameAsString)
            {
            User::Leave(KErrArgument);
            }

        CleanupStack::PushL(lCmdNameAsString);

        TPtrC8 lPtr(reinterpret_cast<const TUint8*>(lCmdNameAsString));
        HBufC* lCmdNameAsDesc = EscapeUtils::ConvertToUnicodeFromUtf8L(lPtr);
        CleanupStack::PushL(lCmdNameAsDesc);

        FLExtensionVarType lTypeOfVariable;

        lExtInterface->getType(lExtInterface, aFnInfo->pArgs[KArg1],
                &lTypeOfVariable);
            
        if(lTypeOfVariable == kExtnVarTypeObject) //as we are using second parameter "arguments" from actionscript which is Object. 
            {
            DeviceCmdsProxy::HandleArrayL(lExtInterface, aFnInfo->pArgs[KArg1], lBundledDeviceArg->iDeviceArgList);
            }
        
        iFlashEngine->ProcessDeviceCmd(*lCmdNameAsDesc, lBundledDeviceArg->iDeviceArgList, lResult);

        aFnInfo->pEI->setInt(aFnInfo->pEI, aFnInfo->returnValue, lResult);
        
        CleanupStack::PopAndDestroy(lCmdNameAsDesc);
        CleanupStack::Pop(lCmdNameAsString);
        lExtInterface->freeString(lExtInterface, lCmdNameAsString);
        lCmdNameAsString = NULL;
        
        }
     else
        {
        aFnInfo->pEI->setInt(aFnInfo->pEI, aFnInfo->returnValue, KErrArgument);
        }
    
    CleanupStack::PopAndDestroy();
    DEVICECMDSTRACEF("DeviceCmdsProxy::HandleProcessDeviceCmdsL> --");
    }


void DeviceCmdsProxy::HandleBoolL(ExtensionInterface* aEI, FL_VAR_HANDLE aBool, RASExtensionArg& aArgList)
    {
    TInt lData;
    TInt lResult; 
    
    lResult = aEI->getBool(aEI, aBool, &lData);
    
    if(!lResult)
        {
        ExtensionArg* lEngineSpecificArg = new(ELeave) ExtensionArg;
        CleanupStack::PushL(lEngineSpecificArg);
        lEngineSpecificArg->pVariableValue = (void*) lData;
        lEngineSpecificArg->variableType = kExtnVarTypeBoolean;
        
        aArgList.AppendL(lEngineSpecificArg);        
        CleanupStack::Pop(lEngineSpecificArg);        
        }    
    }


void DeviceCmdsProxy::HandleArrayL(ExtensionInterface* aExtInterface, FL_VAR_HANDLE aVar, RASExtensionArg& aArgList)
    {
    TInt lIsArrayInstance = -1;        
    //get Object from variable
    FL_OBJECT_HANDLE lObject= NULL;

    aExtInterface->getObject(aExtInterface, aVar, &lObject);
    if(lObject)
        {
        aExtInterface->isInstanceOf(aExtInterface, lObject, "Array", &lIsArrayInstance);
    
        if (lIsArrayInstance)
            {
            FL_VAR_HANDLE lVar= NULL;
            TInt lNoOfArrayElements = -1;
            aExtInterface->createVariable(aExtInterface, &lVar, NULL);
            //get lenght of array in var
            if (lVar)
                {
                aExtInterface->getPropertyByName(aExtInterface, lObject, (char*)KStrLength, lVar);
                aExtInterface->getInt(aExtInterface, lVar, &lNoOfArrayElements);//get length of Array in an Int variable.
                    //must free var's memory
                aExtInterface->freeVariable(aExtInterface, lVar);
                lVar = NULL;
                }
            
            if(lNoOfArrayElements > 0) 
                {
                //create a structure to describe a function properties to be called by callmethod
                ArgInfo* lArgInfo = DeviceCmdsProxy::GetArgInfoLC(KArg0, NULL, (char*)KStrReverse);
    
                aExtInterface->callMethod(aExtInterface, lObject, lArgInfo, NULL); //reverse array and then use pop to extract variables.
    
                //now set the function name to 'pop' which will pop values from Array.
    
                lArgInfo->name = (char*)KStrPop;
    
                for (TInt lIndex(0); lIndex < lNoOfArrayElements; lIndex++)
                    {
                    FL_VAR_HANDLE lArrayMember= NULL;
            
                    //create a var to hold value of array member            
                    CFLVarHandle* lVarHandle = CFLVarHandle::NewLC(aExtInterface);
                    lArrayMember = lVarHandle->Handle();
            
                    aExtInterface->callMethod(aExtInterface, lObject, lArgInfo, lArrayMember);
            
                    if (lArrayMember)
                        {
                        FLExtensionVarType lTypeOfVariable;
                        aExtInterface->getType(aExtInterface, lArrayMember, &lTypeOfVariable);
                        switch (lTypeOfVariable)
                            {
                            case kExtnVarTypeBoolean:
                                {
                                DeviceCmdsProxy::HandleBoolL(aExtInterface, lArrayMember, aArgList);
                                break;
                                }
                            case kExtnVarTypeString2:
                            case kExtnVarTypeString:
                                {
                                DeviceCmdsProxy::HandleStringL(aExtInterface, lArrayMember, aArgList);
                                break;
                                }                
                            case kExtnVarTypeInteger:
                                {
                                DeviceCmdsProxy::HandleIntL(aExtInterface, lArrayMember, aArgList);
                                break;
                                }
                            case kExtnVarTypeNumber:
                                {
                                DeviceCmdsProxy::HandleNumberL(aExtInterface, lArrayMember, aArgList);
                                break;
                                }
                            case kExtnVarTypeObject:
                                {
                                DeviceCmdsProxy::HandleArrayL(aExtInterface, lArrayMember, aArgList);
                                }
                            case kExtnVarTypeNull:
                            default:
                               ;    
                            }
                        } 
                    
                    CleanupStack::PopAndDestroy(lVarHandle);//pVarHandle
                    lArrayMember = NULL;
                    } //for loop
                
                CleanupStack::Pop(lArgInfo); //pops arginfo from cleanupstack        
                delete lArgInfo;
                lArgInfo = NULL;
    
                } //if(lNoOfArrayElements > 0) 
            }
        else
            {
            User::Leave(KErrArgument);
            }
        
        }
    }


void DeviceCmdsProxy::HandleIntL(ExtensionInterface* aEI, FL_VAR_HANDLE aInteger, RASExtensionArg& aArgList)
    {
    TInt lData;
    TInt lResult; 
    
    lResult = aEI->getInt(aEI, aInteger, &lData);
    
    if(!lResult)
        {
        ExtensionArg* lEngineSpecificArg = new(ELeave) ExtensionArg;
        CleanupStack::PushL(lEngineSpecificArg);
        lEngineSpecificArg->pVariableValue = (void*)(lData);
        lEngineSpecificArg->variableType = kExtnVarTypeInteger;        
        aArgList.AppendL(lEngineSpecificArg);
        CleanupStack::Pop(lEngineSpecificArg);    
        }    
    }


void DeviceCmdsProxy::HandleNumberL(ExtensionInterface* aEI, FL_VAR_HANDLE aNumber, RASExtensionArg& aArgList)
    {
    TReal* lData = new TReal();
    TInt lResult; 
    
    lResult = aEI->getNumber(aEI, aNumber, lData);
    
    if(!lResult)
        {
        ExtensionArg* lEngineSpecificArg = new(ELeave) ExtensionArg;
        CleanupStack::PushL(lEngineSpecificArg);
        lEngineSpecificArg->pVariableValue = (void*)(lData);
        lEngineSpecificArg->variableType = kExtnVarTypeNumber;
     
        aArgList.AppendL(lEngineSpecificArg);
        CleanupStack::Pop(lEngineSpecificArg);    
        }    
    }

void DeviceCmdsProxy::HandleStringL(ExtensionInterface* aEI, FL_VAR_HANDLE aString, RASExtensionArg& aArgList)
    {
    char** lData = new char*[1];
    TInt lResult; 
    
    lResult = aEI->getString(aEI, aString, lData); // as this string is allocated by core player, 
                                                   // use freeString to clear this string data
    
    if(!lResult && *lData)
        {
        ExtensionArg* lEngineSpecificArg = new(ELeave) ExtensionArg;
        CleanupStack::PushL(lEngineSpecificArg);
        lEngineSpecificArg->pVariableValue = (void*)(lData);
        lEngineSpecificArg->variableType = kExtnVarTypeString;
        
        aArgList.AppendL(lEngineSpecificArg);
        
        CleanupStack::Pop(lEngineSpecificArg);
        }   
    }

void DeviceCmdsProxy::PointerArrayCleanup(TAny* aArray)
{
    ExtensionArgListBundle* lBundledArg =  static_cast<ExtensionArgListBundle*>(aArray);
       
    ExtensionInterface* lExtInterface = lBundledArg->iEI;
    
    RASExtensionArg lArgList = lBundledArg->iDeviceArgList;
    
    TInt lArgCount = lArgList.Count();
    
    for(TInt lIndex(0); lIndex < lArgCount; lIndex++)
        {
        if(lArgList[0]->variableType == kExtnVarTypeString)
            {
            char** lData = (char**)lArgList[0]->pVariableValue;
            lExtInterface->freeString(lExtInterface, lData[0]); //frees string allocated by core player by getString method
            delete lData;
            lData = NULL;
            }
        else if(lArgList[0]->variableType == kExtnVarTypeNumber )
            {
            TReal *lData = (TReal*)lArgList[0]->pVariableValue;            
            delete lData;
            }
        ExtensionArg* lArg = lArgList[0];
        lArgList.Remove(0);
        delete lArg;        
        }
    
    lArgList.ResetAndDestroy();
    
    lBundledArg->iEI = NULL;
    
    delete lBundledArg;        
    
}


TBool DeviceCmdsProxy::IsInternalProperty(TDesC8& aPropName)
{

    if (!aPropName.Compare(KStrObjectPropertyResolve()))
        return ETrue;
    else if (!aPropName.Compare(KStrObjectPropertyProto()))
        return ETrue;
    else if (!aPropName.Compare(KStrObjectPropertyConstructor()))
        return ETrue;
    else if (!aPropName.Compare(KStrObjectPropertyProtoType()))
        return ETrue;
    else if (!aPropName.Compare(KStrObjectPropertyConstructor2()))
        return ETrue;
    else
        return EFalse;
    
}

//class FLvarHandle

void CFLVarHandle::ConstructL()
{
    if (iExtInterface)
    {
        iExtInterface->createVariable(iExtInterface, &iVarHandle, NULL);
    }
}

FL_VAR_HANDLE CFLVarHandle::Handle()
{
    return iVarHandle;
}

CFLVarHandle::~CFLVarHandle()
{
    if (iExtInterface && iVarHandle)
    {
        iExtInterface->freeVariable(iExtInterface, iVarHandle);
    }
    iVarHandle = NULL;
    iExtInterface = NULL;
}

CFLVarHandle* CFLVarHandle::NewLC(ExtensionInterface* aEI)
{
    CFLVarHandle* lSelf =new(ELeave) CFLVarHandle(aEI);
    CleanupStack::PushL(lSelf);
    lSelf->ConstructL();
    return lSelf;
}


ArgInfo* DeviceCmdsProxy::GetArgInfoLC(TInt aArgCnt, FL_VAR_HANDLE* aArgList, char* aName)
{
    ArgInfo* lArgInfo = new(ELeave) ArgInfo;
    CleanupStack::PushL(lArgInfo);
    lArgInfo->argCount = aArgCnt;
    lArgInfo->arguments = aArgList;
    lArgInfo->name = aName;
    return lArgInfo;
}
#endif
