/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  browser dll initialisation code and exported functions defs
 *
*/


#ifndef __FLASH_PLG_MAIN_H__
#define __FLASH_PLG_MAIN_H__

#include <e32std.h>

#if defined(__SYMBIAN32__) && !defined(EPOC)
#define EPOC
#endif

#include "npheaders.h"
#include "saplayer_build.h"
#include "netscapeif.h"
#ifdef FEATURE_WEB_BROWSABILITY
#include "javascripthandler.h"
#endif //FEATURE_WEB_BROWSABILITY
#include "pluginargumentsimpl.h"
#include "flashpluginif.h"
#include "cecombrowserplugininterface.h"

class MPluginArguments;

// ============================= ECOM interface ===============================

class CGlueGlobals: public CBase
{
public:
	static CGlueGlobals* NewL(NPNetscapeFuncs* aFuncs);
	void DeletePluginGlobal();
	 CPluginArgProcessor* ArgProc() const
    {
        return iArgProc;
    }
	MFlashPluginIf* LoadFlashPluginL(MNetscapeIf* netscapeIf,MPluginArguments* aPluginArguments);
	void* NpFuncs()
    {
        return iNetscapeFunctions;
    }
#ifdef FEATURE_WEB_BROWSABILITY    
        void InitializeScriptableFuncPtrs();
        NPClass* GetFlashJavaScriptHandlerObject()
        {
            return (&iFlashJavaScriptHandlerObject);
        }
    #endif //FEATURE_WEB_BROWSABILITY


private:
	CGlueGlobals(NPNetscapeFuncs* aFuncs);
	void ConstructL();
	~CGlueGlobals();

private:
	//NPP instance; //set by pluginProxy...
	NPNetscapeFuncs* iNetscapeFunctions;
	TInt iRefCnt;

#ifdef FEATURE_WEB_BROWSABILITY    
	NPClass iFlashJavaScriptHandlerObject;
#endif //FEATURE_WEB_BROWSABILITY    
	CPluginArgProcessor* iArgProc;

	RLibrary iLibrary;

	friend class CFlashPluginProxy;
};

class CNpEcomMain: public CEcomBrowserPluginInterface
{
public:
	// Functions from base classes
	static CNpEcomMain* NewL(TAny* aInitParam);
	~CNpEcomMain();

private:
	CNpEcomMain(NPNetscapeFuncs* aNpf);
	void ConstructL(NPPluginFuncs* aPluginFuncs);

private:
	CGlueGlobals* iGlueGlobals;
	NPNetscapeFuncs* iNpf;
};

class CFlashPluginProxy: public CBase , public MNetscapeIf
{
public:
	static CFlashPluginProxy* NewL(NPP aInstance,
			CPluginArgProcessor* argProcessor);

	~CFlashPluginProxy();

	MFlashPluginIf* Plugin() const;
	CFlashJavaScriptHandler* JavaScriptHandler();

	int GetValue(NPP instance, NPPVariable aVariable, void* ret_value);
	   TBool QueryAllowNetworkInWidgetMode();
	    int ResolveExecutionMode();

	
	    void* NPPInstance()
	    {
	        return instance;
	    }
	    void* NpFuncs()
	    {
            CGlueGlobals* globals = static_cast<CGlueGlobals*> (Dll::Tls());
	        return globals->NpFuncs();
	    }


	    virtual int GetUrl(const TDesC& url, const TDesC* target);
	    virtual int PostUrl(const TDesC& url, const TDesC* target,
	            const TDesC& buf, bool file);

	    virtual int GetUrlNotify(const TDesC& url, const TDesC* target,
	            void* notifyData);

	    virtual int PostUrlNotify(const TDesC& url, const TDesC* target,
	            const TDesC& buf, bool file, void* notifyData);

	    virtual int GetValue(int variable, void* ret_alue);
	    virtual int SetValue(int variable, void* ret_alue);

	    virtual void InvalidateRect(const TRect& aRect);
	    virtual void SetScriptAccess();
	   //ou1cimx1#449490
	    virtual void DeleteFlashMethodObject();
	    virtual HBufC8* GetBaseUrlLC();
	    virtual HBufC8* GetHTMLBaseTagValueLC();
	    void SetFuncEnterVal(int aEnterexitval);
	    int GetFuncEnterVal();
	    void SetDestroyVal(bool aDestroyflag);
	    bool GetDestroyVal();

private:
    NPP instance;
	CFlashPluginProxy();

	void ConstructL(NPP aInstance, CPluginArgProcessor* argProcessor);

	MFlashPluginIf* plugin;
	
	TInt iFuncEnterExitVal;
    TBool iSetDestroyFlag;

#ifdef FEATURE_WEB_BROWSABILITY    
	CFlashJavaScriptHandler* iFlashJavaScriptHandler;
	TPageAccessType iPageAccessType;
	FlashJavaScriptHandlerObject* iJScriptNPObject;
#endif // FEATURE_WEB_BROWSABILITY
};

/*********************************************************************************
 Plugin API methods
 */

/**
 * Create a plugin.
 */
NPError my_newp(NPMIMEType pluginType, NPP instance, uint16 mode,
		CDesCArray* argn, CDesCArray* argv, NPSavedData* saved);

/**
 * Destroy a plugin.
 */
NPError my_destroy(const NPP instance, NPSavedData** save);

/**
 * This is the parent window of a plugin.
 */
NPError my_setwindow(const NPP instance, const NPWindow* window);

/**
 * A new data stream has been created for sending data to the plugin.
 */
NPError my_newstream(const NPP instance, NPMIMEType type, NPStream* stream,
		NPBool seekable, uint16* stype);

/**
 * A data stream has been terminated.
 */
NPError my_destroystream(const NPP instance, NPStream* stream, NPReason reason);

/**
 * A data stream has been fully saved to a file.
 */
void my_asfile(NPP instance, NPStream* stream, const TDesC& fname);

/**
 * Return the maximum number of bytes this plugin can accept from the stream.
 */
int32 my_writeready(const NPP instance, NPStream* stream);

/**
 * Receive more data - return number of bytes processed.
 */
int32 my_write(const NPP instance, NPStream* stream, int32 offset, int32 len,
		void* buffer);

/**
 * Print
 */
void my_print(NPP instance, NPPrint* platformPrint);

/**
 * Event
 */
int16 my_event(NPP instance, void* event);

/**
 * This is the URL being sensed.
 */
void my_urlnotify(const NPP instance, const TDesC8& url, NPReason reason,
		void* notifyData);

/**
 * Generic hook to retrieve values/attributes from the plugin.
 */
NPError my_getvalue(NPP instance, NPPVariable variable, void *ret_value);

/**
 * Generic hook to set values/attributes within the plugin.
 */
NPError my_setvalue(NPP instance, NPNVariable variable, void *ret_value);

/**
 * Function to load the plugin resource file 
 */
int load_pluginresourceL(TBufC<128>& aResourceFile);

void FunctionEnter(CFlashPluginProxy *proxy);
void FunctionExit(NPP aInstance, CFlashPluginProxy *proxy);
#endif
