/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Java Script Interaction Handler class declaration
*
*/



#ifndef __JAVA_SCRIPT_HANDLER_H__
#define __JAVA_SCRIPT_HANDLER_H__

#include "saplayer_build.h"
#include "pluginscripthandler.h"
#include "npheaders.h"
#include "flashmethods.h"

class MFlashPluginIf;
class CFlashMethods;
class MHostScriptHandler;
class MNetscapeIf;

/**
*  CFlashJavaScriptHandler class
*  Interfaces to Browser, used to pass information between browser and Flash plugin
*  Used for Javascript and Flash interaction
*  @since S60 v3.2
*/
class CFlashJavaScriptHandler : public CBase
								#ifdef FEATURE_WEB_BROWSABILITY    	
								, public MPluginScriptHandler
								#endif //FEATURE_WEB_BROWSABILITY    	
    {
    public: 
      	static CFlashJavaScriptHandler* NewL(NPP aInstance,MFlashPluginIf* pBrowserInstance
      								,MHostScriptHandler* aMHostScriptHandler
      								,TPageAccessType aAccessType = EPageUrlNormal
      								);

        virtual ~CFlashJavaScriptHandler();

	public: //MPluginScriptHandler
	
#ifdef FEATURE_WEB_BROWSABILITY    	
		virtual int CallMethodL(const char* aReq);
		
		virtual int EvaluateJavaScript(const char* aExpr) ;
		
		virtual void* GetDOMObjectID() ;
		
		virtual TInt DoFsCommandL(TDesC8& name
       							,const char* aCmd
       							,const char* aArgs );

#endif
			
    public: 
        void Deallocate ();
        
        bool HasProperty( NPIdentifier name );
        
        bool HasMethod( NPIdentifier name);
        
        void GetPropertyL( NPIdentifier name, NPVariant *variant );
        
        void SetPropertyL( NPIdentifier name, NPVariant *variant );
        
        bool InvokeMethodL(NPIdentifier name, 
							const NPVariant *args, 
							uint32_t argCount, 
							NPVariant *result);
							
		bool InvokeDefaultMethod(const NPVariant *args,
									uint32_t argCount,
									NPVariant *result);
									
		void SetScriptAccess()
			{
			if(iFlashMethods)
			TRAP_IGNORE(iFlashMethods->SetPropAllowScriptAccessL( iAccessType ));
			}
									
		void DeleteFlashMethods();

    public:
       
        bool InvokeCallback ( TDesC8& name, 
        			const NPVariant *args,
        			uint32_t argCount );

    private: //methods
        CFlashJavaScriptHandler(NPP aInstance,MFlashPluginIf* pBrowserInstance
        						,MHostScriptHandler* aMHostScriptHandler
        						,TPageAccessType aAccessType
      							);
        
        void ConstructL();

		TInt GetMethodIndex(   const NPIdentifier* aTable
							, TInt aTableSize
							, NPIdentifier name);

		void ReleaseFlashMethodIdentifiers();
		
		void ReleaseFlashProprtyIdentifiers();
		
		void CreateFlashMethodIdentifiersL();
		void CreateFlashProprtyIdentifiersL();
		
		
    private: //attributes
        MFlashPluginIf* iBrowserInstance;
        
        CFlashMethods* iFlashMethods;
        
        
      	MHostScriptHandler* iHostScriptHandler;
      											 
        
        NPIdentifier* iFlashJavaScriptMethodIdentifiers;
        
        NPIdentifier* iFlashJavaScriptPropertyIdentifiers;
        NPNetscapeFuncs* iNetscapeFuncs;
        NPP instance;
        
        //CWidgetSecurity* iSecurity; // need to clarify whether we need to use the same for
        // java script security verification
        TUid iUid;
        TPageAccessType iAccessType;
    };
    
/**
********************************************************************************
    Flash Java Script Interaction Handler API methods. 
*********************************************************************************
*/
typedef struct
{

    NPObject object;
    
    CFlashJavaScriptHandler *handler;
    
} FlashJavaScriptHandlerObject;

/**
 * FlashJavaScriptHandlerAllocate
 */
NPObject *FlashJavaScriptHandlerAllocate ();

/**
 * FlashJavaScriptHandlerInvalidate
 */
void FlashJavaScriptHandlerInvalidate ();

/**
 * FlashJavaScriptHandlerDeallocate
 */
void FlashJavaScriptHandlerDeallocate (FlashJavaScriptHandlerObject *obj);

/**
 * FlashJavaScriptHandlerHasProperty
 */
bool FlashJavaScriptHandlerHasProperty(FlashJavaScriptHandlerObject *obj, 
										NPIdentifier name);

/**
 * FlashJavaScriptHandlerHasMethod
 */
bool FlashJavaScriptHandlerHasMethod(FlashJavaScriptHandlerObject *obj, 
										NPIdentifier name
										);

/**
 * FlashJavaScriptHandlerInvokeFunctionPtr
 */
bool FlashJavaScriptHandlerInvokeFunctionPtr(FlashJavaScriptHandlerObject *obj, 
							NPIdentifier name, 
							const NPVariant *args, 
							uint32_t argCount, 
							NPVariant *result);

/**
 * FlashJavaScriptHandlerInvokeDefaultFunctionPtr
 */							
bool FlashJavaScriptHandlerInvokeDefaultFunctionPtr(FlashJavaScriptHandlerObject *npobj, 
									const NPVariant *args,
									uint32_t argCount,
									NPVariant *result);

/**
 * FlashJavaScriptHandlerGetProperty
 */
void FlashJavaScriptHandlerGetProperty (FlashJavaScriptHandlerObject *obj, 
										NPIdentifier name, 
										NPVariant *variant);

/**
 * FlashJavaScriptHandlerSetProperty
 */
void FlashJavaScriptHandlerSetProperty (FlashJavaScriptHandlerObject *obj, 
										NPIdentifier name, 
										NPVariant *variant);


/**
 * _FlashJavaScriptHandlerClass
 
 
static NPClass _FlashJavaScriptHandlerClass = {
    0,
    (NPAllocateFunctionPtr) FlashJavaScriptHandlerAllocate,
    (NPDeallocateFunctionPtr) FlashJavaScriptHandlerDeallocate,
    (NPInvalidateFunctionPtr) FlashJavaScriptHandlerInvalidate,
    (NPHasMethodFunctionPtr) FlashJavaScriptHandlerHasMethod,
    (NPInvokeFunctionPtr) FlashJavaScriptHandlerInvokeFunctionPtr,
    (NPInvokeDefaultFunctionPtr) FlashJavaScriptHandlerInvokeDefaultFunctionPtr,
    (NPHasPropertyFunctionPtr) FlashJavaScriptHandlerHasProperty, //TODO:check need
    (NPGetPropertyFunctionPtr) FlashJavaScriptHandlerGetProperty,//TODO:check need
    (NPSetPropertyFunctionPtr) FlashJavaScriptHandlerSetProperty,//TODO:check need
    (NPRemovePropertyFunctionPtr) 0
};

/**
 * static instance of the function pointer table
 
static NPClass *FlashJavaScriptHandlerClass = &_FlashJavaScriptHandlerClass;
 */
#endif //
