/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash Methods Handler class implementation
*
*/
 
#include <e32std.h>
#include <charconv.h>
#include <eikenv.h>
#include "saplayer_build.h"
#include "flashpluginif.h"
#include "flashmethods.h"
#include "NPNFlashDefs.h"
#include "commondef.h"
#include "externalinterface.h"
#include <string.h>
#include "flashpluginif.h"
#include "flashplgmain.h"

//TGetProperty,TSetProperty,
//const int NUM_PROPERTY_IDENTIFIERS = 19;
const short KLoopOn = 1;
const short KLoopOff = 0;
const short KQualityLow = 0;
const short KQualityMedium = 1;
const short KQualityHigh = 2;

/*const NPUTF8 *FlashJavaScriptHandlerPropertyIdentifierNames[NUM_PROPERTY_IDENTIFIERS]
= { "X POSITION",  		// 0 	Get,Set
    "Y POSITION",  		// 1 	Get,Set
    "X SCALE",    		// 2  	Get,Set
    "Y SCALE",  		// 3	Get,Set
    "CURRENTFRAME", 	//4 	Get
    "TOTALFRAMES", 		//5  	Get
    "ALPHA", 			//6		Get,Set
    "VISIBILITY", 		//7		Get,Set
    "WIDTH", 			//8		Get
    "HEIGHT",			//9		Get
    "ROTATION", 		//10	Get,Set
    "TARGET", 			//11	Get
    "FRAMESLOADED",		//12 	Get
    "NAME", 			//13	Get,Set
    "DROPTARGET", 		//14	Get
    "URL(_url)",		//15	Get
    
    //Global properties
    "HIGHQUALITY", 		//16 	Get,Set
    "FOCUSRECT", 		//17	Get,Set
    "SOUNDBUFTIME", 	//18	Get,Set
    };*/
    
//TODO:somehow above table needs to be in sortted order so that 
// bin search can be performed.. but remember to keep property number
// same but not based on index

//Property identifier by ID
enum EFlashProperty
{
	
	 E_PROP_X_POS 					= 0,
	 E_PROP_Y_POS 					= 1,
	 E_PROP_X_SCALE 				= 2,
	 E_PROP_Y_SCALE 				= 3,
	 E_PROP_CURRENT_FRAME 			= 4,
     E_PROP_TOTAL_FRAMES 			= 5,
	 E_PROP_ALPHA			 		= 6,
	 E_PROP_VISIBLE		 			= 7,
	 E_PROP_WIDTH			 		= 8,
	 E_PROP_HEIGHT					= 9,
	 E_PROP_ROTATE					= 10,
	 E_PROP_TARGET		 			= 11,
	 E_PROP_FRAMES_LOADED 			= 12,
	 E_PROP_NAME					= 13,
	 E_PROP_DROP_TARGET				= 14,

//Global Properties
	E_PROP_URL						= 15,
	E_PROP_HIGH_QUALITY				= 16,
	E_PROP_FOCUS_RECT				= 17,
	E_PROP_SOUND_BUF_TIME			= 18
};
    

/**
*  CFlashMethods class
*  Flash Methods handler
*  Used for Javascript and Flash interaction
*  @since S60 v3.2
*/
 
// -------------------------------------------------------------------------
// CFlashMethods* CFlashMethods::NewL(CFlashPluginInstance* aBrowserInstance
//									,MFlashMethods* aFlashMethods )
// FlashMethods object creator
// --------------------------------------------------------------------------
//
CFlashMethods* CFlashMethods::NewL(MFlashPluginIf* aBrowserInstance
									,MFlashMethods* aFlashMethods
									,MExternalInterface* aExternalInterface
									,TPageAccessType aAccessType)
{
	CFlashMethods* self = new (ELeave) CFlashMethods(aBrowserInstance
													,aFlashMethods
													,aExternalInterface
													,aAccessType);
    CleanupStack::PushL( self );
    self->ConstructL();
    CleanupStack::Pop( self );
    return self;
}

// -------------------------------------------------------------------------
// CFlashMethods::CFlashMethods(CFlashPluginInstance* aBrowserInstance
//							, MFlashMethods* aFlashMethods
//							 ):	
// First phase constructor
// --------------------------------------------------------------------------
//

CFlashMethods::CFlashMethods(MFlashPluginIf* aBrowserInstance
							, MFlashMethods* aFlashMethods
							, MExternalInterface* aExternalInterface
							, TPageAccessType aAccessType ):	
							 iBrowserInstance(aBrowserInstance)
							,iFlashMethods(aFlashMethods)
							,iExternalInterface(aExternalInterface)
							,iCharacterConverter(NULL)
							,iAccessType(aAccessType)
						
{
	
}

// -------------------------------------------------------------------------
// CFlashMethods::~CFlashMethods()
// destructor
// --------------------------------------------------------------------------
//
CFlashMethods::~CFlashMethods()
{
	if(iCharacterConverter)
	{
		delete iCharacterConverter;
		iCharacterConverter = NULL;
	}
}

// -------------------------------------------------------------------------
// void CFlashMethods::ConstructL()

// Second phase constructor
// --------------------------------------------------------------------------
//

char* CFlashMethods::CreateFlashStringL ( const TDesC8& des )
{
	char* result = NULL;
	if(des != KNullDesC8)
	{
		result = new char[des.Length()+1];
		User::LeaveIfNull(result);
			
		TPtr8 ptrstr ( (unsigned char*)result, des.Length ( ), des.Length ( ) );
		ptrstr.Copy ( des );
		result[des.Length ( )] = 0;
		}
	return result;
}
void CFlashMethods::ConstructL()
{
    iGlobals = static_cast<CGlueGlobals*> (Dll::Tls());
                              
    CreateCharacterConverterL();
    
}

// -------------------------------------------------------------------------
// HBufC* CFlashMethods::CreateEpocString( const char* aStr, int aLength )

// 
// --------------------------------------------------------------------------
//
HBufC* CFlashMethods::CreateEpocStringL( const char* aStr, int aLength )
{
	HBufC* pResult=NULL;

	if( iCharacterConverter )
		{

		TPtrC8 ptr( (TUint8*)aStr, aLength );

		pResult = HBufC::NewLC( aLength );	

		TPtr ptrResult( pResult->Des() );

		TInt state = CCnvCharacterSetConverter::KStateDefault;
		User::LeaveIfError(iCharacterConverter->ConvertToUnicode( ptrResult, ptr, state ));
		CleanupStack::Pop(pResult);
		}
	else
		User::LeaveIfNull(iCharacterConverter);

	return pResult;
}

// -------------------------------------------------------------------------
// char* CFlashMethods::CreateFlashString( const TDesC& aStr )

// 
// --------------------------------------------------------------------------
//
/*char* CFlashMethods::CreateFlashString( const TDesC& aStr )
{
	char* pResult = NULL;
	
	if( iCharacterConverter )
	{
		TInt maxlen = aStr.Length() << 1;
		char* pResult = new(ELeave) char[maxlen + 1];
		

		TPtr8 ptrResult( (TUint8*)pResult, maxlen );
	
		iCharacterConverter->ConvertFromUnicode ( ptrResult, aStr ) ;
		
		pResult[ptrResult.Length()] = 0;	
		
	}

	return pResult;
	
}*/
char* CFlashMethods::CreateFlashStringL ( const TDesC16& des )
{
    User::LeaveIfNull( iCharacterConverter );
	char* result = new(ELeave) char[des.Length()+1];
	
	User::LeaveIfNull(result);
		
	TPtr8 ptrstr ( (unsigned char*)result, des.Length ( ), des.Length ( ) );
	User::LeaveIfError( iCharacterConverter->ConvertFromUnicode ( ptrstr, des ) );
	result[des.Length ( )] = 0;
	return result;
}

// -------------------------------------------------------------------------
// void CFlashMethods::CreateCharacterConverterL()

// 
// --------------------------------------------------------------------------
//
void CFlashMethods::CreateCharacterConverterL()
{
	if ( iCharacterConverter )
	{
		return;
	}

	CCnvCharacterSetConverter* characterConverter = CCnvCharacterSetConverter::NewL();
	CleanupStack::PushL(characterConverter);

	if ( characterConverter->PrepareToConvertToOrFromL( KCharacterSetIdentifierCodePage1252
						, CEikonEnv::Static ()->FsSession() )
		!= CCnvCharacterSetConverter::EAvailable )
	{
		User::Leave ( KErrNotSupported );
	}
	
	User::LeaveIfNull ( characterConverter );
	
	CleanupStack::Pop(characterConverter);

	iCharacterConverter = characterConverter;

}

// -------------------------------------------------------------------------
// TInt CFlashMethods::GetIntFromDesc(const TDesC& aDesc)

// atoi 
// --------------------------------------------------------------------------
//
TInt CFlashMethods::GetIntFromDesc(const TDesC& aDesc)
{
	TLex conv(aDesc);
	TInt Number=0;
	conv.Val(Number);
	
	return Number;
	
}
////////////////////////////////////////////////////////////////////////////////////
// Flash Properties handlers

// -------------------------------------------------------------------------
// void CFlashMethods::GetPropAllowScriptAccess(NPVariant *result)
//
// example  if( flash.AllowScriptAccess == "Never" ) 
// Description:  Gets the AllowScriptAccess value for flash content
// --------------------------------------------------------------------------
//
void CFlashMethods::GetPropAllowScriptAccess(NPVariant *result)
{
	if(result)	
	{
		TInt scriptAccess=0;
		
		iFlashMethods->GetPropAllowScriptAccess(scriptAccess);
		
		TPtr ptrScriptAccess(NULL,0);
		
		switch(scriptAccess)
		{
			case 0:
			{
				ptrScriptAccess = KSCriptAccessAlways();
				
			}
			break;	
			case 1:
			{
				ptrScriptAccess = KSCriptAccessnever();
				
			}
			break;
			case 2:
			{
				ptrScriptAccess = KSCriptAccessSameDomain();
				
			}
			break;
		}
		
		const char* temp = 	CreateFlashStringL(ptrScriptAccess);
			
			//hope temp gets deleted on caller side
		STRINGZ_TO_NPVARIANT(temp,*result);
		
		
	}
}
  
// -------------------------------------------------------------------------
// void CFlashMethods::GetPropBGColorL(NPVariant *result)
//
// example  Alert(flash.BGColor) 
// Description:  Gets the Backgroundcolor for flash content
// --------------------------------------------------------------------------
//
  
void CFlashMethods::GetPropBGColorL(NPVariant *result)
{
	if(result)	
	{
		FI_Color structColor;
		
		iFlashMethods->GetPropBGColorL(structColor);
		
		HBufC* bgcolor = HBufC::NewLC(10);
		
		TPtr ptrBbgColor= bgcolor->Des();
			
		ptrBbgColor.Format(_L("#%02x%02x%02x"),structColor.red
								,structColor.green
								,structColor.blue);
		
		const char* temp = 	CreateFlashStringL(ptrBbgColor);
		
		//hope temp gets deleted on caller side
		STRINGN_TO_NPVARIANT(temp, 7,*result);
		
		CleanupStack::PopAndDestroy(bgcolor);						
	}
	
}
  
// -------------------------------------------------------------------------
// void CFlashMethods::GetPropQuality(NPVariant *result)
//
// example  if(flash.quality = "High")
// Description:  Gets the Quality of Flash display Low, Medium, High
// --------------------------------------------------------------------------
//
  
void CFlashMethods::GetPropQuality(NPVariant *result)
{
	if(result)	
	{
		TInt qual=0;
		
		iFlashMethods->GetPropQuality(qual);
		
		TPtr ptrQuality(NULL,0);
		
		switch(qual)
		{
			case KQualityLow:
			{
				ptrQuality = KLow();
			}
			break;
			
			case KQualityMedium:
			{
				ptrQuality = KMedium();	
			}
			break;
			
			case KQualityHigh:
			{
				ptrQuality = KHigh();
			}
			break;
		}
		
		const char* temp = 	CreateFlashStringL(ptrQuality);
			
		//hope temp gets deleted on caller side
		STRINGZ_TO_NPVARIANT(temp,*result);
	}
}

// -------------------------------------------------------------------------
// void CFlashMethods::SetPropBaseURL(NPVariant *result)
//
// <param name = "base" value = "http://www.adobe.com" >    
// Description:  Sets the base URL to resolve relative path statements in 
// HTML page
// --------------------------------------------------------------------------
//

void CFlashMethods::SetPropBaseURL(NPVariant *result)
{
	if(result)	
	{
		NPString str = NPVARIANT_TO_STRING(*result);
		
		HBufC* baseUrl = CreateEpocStringL(str.UTF8Characters
											,str.UTF8Length);		
		CleanupStack::PushL(baseUrl);
		
		TPtr ptrBaseUrl= baseUrl->Des();
		
		iFlashMethods->SetPropBaseURL(ptrBaseUrl);
		
		CleanupStack::PopAndDestroy(baseUrl);
	}
	
}   

// -------------------------------------------------------------------------
// bool CFlashMethods::SetPropAllowScriptAccessL(NPVariant *result)
//
// AllowScriptAccess( value )    
// Description:  Sets outbound script access level to perform outbound
// scripting from withing the SWF. Outbound scripting is acheived through fscommnd
// getURL or through External API interface
// values taken are "Always", "Never","sameDomain"
// --------------------------------------------------------------------------
//
 
void CFlashMethods::SetPropAllowScriptAccessL(NPVariant *result)
{
	if(result)	
	{
		NPString str = NPVARIANT_TO_STRING(*result);
		
		HBufC* scriptAccess = CreateEpocStringL(str.UTF8Characters
											,str.UTF8Length);		
		CleanupStack::PushL(scriptAccess);
		
		TPtr ptrScriptAccess= scriptAccess->Des();
		
		TInt scrAcc = -1;
		
		if(0 == ptrScriptAccess.Compare(KSCriptAccessAlways))
		{
			scrAcc = 0;
		}
		else
		{
			if(0 == ptrScriptAccess.Compare(KSCriptAccessSameDomain))
			{
				scrAcc = 2;
			}
			else
			{
				if(0 == ptrScriptAccess.Compare(KSCriptAccessnever))
				{
					scrAcc = 1;
				}
			}	
			
		}
		
		if(scrAcc != -1)
		{
			iFlashMethods->SetPropAllowScriptAccessL(scrAcc);
		}
		
		CleanupStack::PopAndDestroy(scriptAccess);
	}
}
 
static int Parse1Hex(TUint c)
{
    if (c >= '0' && c <= '9')
        return c - '0';
    if (c >= 'a' && c <= 'f')
        return c - 'a' + 10;
    if (c >= 'A' && c <= 'F')
        return c - 'A' + 10;
    return 0;
}

inline int Parse2Hex(const TUint16* c)
{
    return (Parse1Hex(c[0])<<4) + Parse1Hex(c[1]);
}
 
// -------------------------------------------------------------------------
// bool CFlashMethods::SetPropBGColorL(NPVariant *result)
//
// backgroundColor( value )    
// Description:  Sets backgroundcolor. values passed as Hex values
// Example #62F320 : 2 digits each corresponds to red ,green blue in order
// --------------------------------------------------------------------------
//  
void CFlashMethods::SetPropBGColorL(NPVariant *result)
{
	if(result)
	{
		FI_Color structColor;
		
		NPString str = NPVARIANT_TO_STRING(*result);
		
		HBufC* bgColor = CreateEpocStringL(str.UTF8Characters
											,str.UTF8Length);		
		CleanupStack::PushL(bgColor);
		
		TPtr ptrbgColor= bgColor->Des();
		
		const TUint16* ptr = ptrbgColor.Ptr ( );
		
        int len = ptrbgColor.Length ( );
        
        if ( len >= 6 )
         {
            if ( *ptr == '#' )
			{
				ptr++;
				len--;
			}

            if ( len >= 6 )
              {
                structColor.red = (unsigned char) Parse2Hex ( ptr );
                structColor.green = (unsigned char) Parse2Hex ( ptr+2 );
                structColor.blue = (unsigned char) Parse2Hex ( ptr+4 );
           		structColor.alpha = 0xFF;
               		     
               	iFlashMethods->SetPropBGColorL(structColor);
               	
              }
         }
         
        CleanupStack::PopAndDestroy(bgColor);
	}
}
    
void CFlashMethods::SetPropQualityL(NPVariant *result)
{
	if(result)
	{
		NPString str = NPVARIANT_TO_STRING(*result);
		
		HBufC* qualStr= CreateEpocStringL(str.UTF8Characters
											,str.UTF8Length);		
		CleanupStack::PushL(qualStr);
		
		TPtr ptrQuality= qualStr->Des();
		
		TInt SwfQuality = -1;
		
		if (0 == ptrQuality.CompareF(KLow))
		{
			SwfQuality = KQualityLow;
		}
		else if (0 == ptrQuality.CompareF(KMedium))
		{
			SwfQuality = KQualityMedium;
		}
		else if (0 == ptrQuality.CompareF(KHigh))
		{
			SwfQuality = KQualityHigh;
		}
		
		if(SwfQuality != -1)
		{
			iFlashMethods->SetPropQualityL(SwfQuality);
		}
		
		CleanupStack::PopAndDestroy(qualStr);
	}
	
	
}
    
void CFlashMethods::SetPropLoopL(NPVariant *result)
{
	if(result)
	{
		NPString str = NPVARIANT_TO_STRING(*result);
		
		HBufC* loopStr= CreateEpocStringL(str.UTF8Characters
											,str.UTF8Length);		
		CleanupStack::PushL(loopStr);
		
		TPtr ptrLoop= loopStr->Des();
		
		TInt loop = -1;
		
		if ((0 == ptrLoop.CompareF(KOn))||
			(0 == ptrLoop.CompareF(KTrue)) ||
			 (0 == ptrLoop.CompareF(KYes)) )
		{
			loop = KLoopOn;
		}
	    else if ((0 == ptrLoop.CompareF(KOff))||
	    		(0 == ptrLoop.CompareF(KFalse)) || 
	    		(0 == ptrLoop.CompareF(KNo)))
		{
			loop = KLoopOff;
		}
		
		if(loop != -1)
		{
			iFlashMethods->SetPropLoopL(loop);
		}
		
		CleanupStack::PopAndDestroy(loopStr);
		
	}
		
}
    
    
////////////////////////////////////////////////////////////////////////////////////
// Flash Methods handlers

// -------------------------------------------------------------------------
// bool CFlashMethods::GetVariable(	const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)
//
// GetVariable( varName )    
// Description:  Returns the value of the Flash variable specified by varName.
// Returns: null if the variable does not exist. The argument type is string.    
// Example:  var firstName = movie.GetVariable("FirstName");  
// --------------------------------------------------------------------------
//

bool CFlashMethods::GetVariableL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result)
{
	if(argCount == 1)
	{		
	
	 HBufC* urlIn=iBrowserInstance->GetFlashBaseURL();
	 NPString variableName = NPVARIANT_TO_STRING(args[0]);
	        
	    if(urlIn && (variableName.UTF8Characters!= NULL))
	      {       
	        HBufC* varName = CreateEpocStringL(variableName.UTF8Characters
	                ,variableName.UTF8Length);		
	        CleanupStack::PushL(varName);

	        TPtr ptrVarName = varName->Des();
	        HBufC* varOut=NULL;

	        iFlashMethods->GetVariableL(ptrVarName, 
	                (&varOut),
	                *urlIn,
	                iAccessType, 
	                ETrue);


	        CleanupStack::PopAndDestroy(varName);
	        char* val = NULL;

	        if( varOut )
	            {
	            val = CreateFlashStringL(*varOut);
	            STRINGZ_TO_NPVARIANT(val,*result);	
	            delete varOut;
	            varOut = NULL;
	            }
	        return true;
	}
	
}
	return false;  
}

// -------------------------------------------------------------------------
// bool CFlashMethods::GotoFrame(	const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant *result)
//
// Activates the frame number specified by frameNumber in the current movie.
// If the data for a requested frame is not yet available,
// the player goes to the last frame available and stops,
// causing unexpected results during playback.
// Use the PercentLoaded() method to determine if enough of the movie is 
// available to execute the GotoFrame() method.
// The argument frameNumber is zero-based;
// that is, frameNumber is 0 in the first frame of the movie,
// 1 for the second frame, and so on. This differs from the 
// Goto action within Flash, which begins at 1. The argument type is integer.
//
//Example:  movie.GotoFrame(24); 
// --------------------------------------------------------------------------
//
						
bool CFlashMethods::GotoFrame(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)//TODO:check do we need to pass result
{
	if(argCount >= 1)
	{
		TInt FrameNo = NPVARIANT_TO_INT32(args[0]);
		
		iFlashMethods->GotoFrame(FrameNo);
		
		return true;
	}

	return false;		
	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::IsPlaying(	const NPVariant */*args*/, 
//						uint32_t /*argCount*/, 
//						NPVariant *result)
//
// Returns true if the movie is currently playing.    
// Example  if (movie.IsPlaying()) { alert("movie is playing"); }    
//
// --------------------------------------------------------------------------
//
					
bool CFlashMethods::IsPlaying(	const NPVariant */*args*/, 
						uint32_t /*argCount*/, 
						NPVariant *result)
{
	TBool stat = iFlashMethods->IsPlaying();
	bool status =false;
	if(stat)
	{
		status = true;
	}
	
	BOOLEAN_TO_NPVARIANT( status, *result );
	
	return true;	
	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::LoadMovieL(	const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)
//
//  Loads the movie identified by url to the layer specified by layerNumber. 
//The argument type is integer for layerNumber and string for url. 
//     
// Example  movie.LoadMovieL(0, "mymovie.swf"); 
//
// --------------------------------------------------------------------------
//
					
bool CFlashMethods::LoadMovieL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
{
	TBool retVal = false;
	if(argCount >= 2 )
	{	
	    NPString url = NPVARIANT_TO_STRING(args[1]);
	    if(url.UTF8Characters != NULL)
	        {

	        NPVariantType argType = args[0].type;
	        TInt32 layerNo = 0;
	        if( argType == NPVariantType_Int32)
	            {
	            layerNo = NPVARIANT_TO_INT32(args[0]);	
	            }
	        else if(argType == NPVariantType_Double)
	            {
	            layerNo = NPVARIANT_TO_DOUBLE(args[0]);	
	            }

	        HBufC* urlBuf = CreateEpocStringL(url.UTF8Characters
	                ,url.UTF8Length);
	        CleanupStack::PushL(urlBuf);
	        TPtr urlStr = urlBuf->Des();

	        HBufC* urlIn=iBrowserInstance->GetFlashBaseURL();			
	        if(urlIn)
	            {
	            iFlashMethods->LoadMovieL(urlStr,layerNo,*urlIn, iAccessType, ETrue);
	            retVal = true;
	            }

	        CleanupStack::PopAndDestroy(urlBuf);
	        }
	}
	return retVal;	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::Pan(	const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)

// Pans a zoomed-in movie to the coordinates specified by x and y.
// Use mode to specify whether the values for x and y are pixels
// or a percent of the window. When mode is 0, the coordinates are pixels;
// when mode is 1, the coordinates are percent of the window.
// Pan does not pan beyond the boundaries of the zoomed-in movie. 
//The argument type for all arguments is integer.    
// Example  This example pans 50% right and 50% down: movie.Pan(50, 50, 1) 
//This example pans -25 pixels left and -25 pixels up:movie.Pan(-25, -25, 0) 
//
// --------------------------------------------------------------------------
//
					
bool CFlashMethods::Pan(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
{

	if(argCount >=3)
	{
		TInt32 x = NPVARIANT_TO_INT32(args[0]);
		TInt32 y = NPVARIANT_TO_INT32(args[1]);
		TInt32 mode = NPVARIANT_TO_INT32(args[2]);
		
		iFlashMethods->Pan(x,y,mode);
		
		return true;
	}
	return false;	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::PercentLoaded(const NPVariant */*args*/, 
//						uint32_t /*argCount*/, 
//						NPVariant *result)
//
// Returns the percent of the Flash Player movie that has streamed
// into the browser so far; possible values are from 0 to 100.    
// Example:  if (movie.PercentLoaded() == 100) { loaded = true; } 
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::PercentLoaded(const NPVariant */*args*/, 
						uint32_t /*argCount*/, 
						NPVariant *result)
{
	TReal percent = 0;
	bool success=false;
	success = iBrowserInstance->PercentLoaded(percent);
	
	TRealFormat realformat;
	realformat.iType = KRealFormatFixed;
	realformat.iPlaces = 2;
	realformat.iWidth = 10;
	if(percent>100)
	{
		return success;
	}
/*	HBufC8* percentString = HBufC8::NewL(10);
	TPtr8 ptrPercent = percentString->Des();
	ptrPercent.Num(percent,realformat);
	ptrPercent.ZeroTerminate();
	char* percentCString = CreateFlashString(ptrPercent);
	STRINGZ_TO_NPVARIANT (percentCString, *result );*/
	DOUBLE_TO_NPVARIANT(percent, *result );
	success=true;
	
	return success;	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::FlashVersion(const NPVariant */*args*/, 
// 						uint32_t /*argCount*/, 
//						NPVariant *result)
//
// Returns the version of the flash player
//
// --------------------------------------------------------------------------
//


bool CFlashMethods::FlashVersion(const NPVariant */*args*/, 
		uint32_t /*argCount*/, 
		NPVariant *result)
{
	HBufC* version = NULL;
	iFlashMethods->FlashVersion(&version);
	char* vesrionCString = CreateFlashStringL( *version );
	STRINGZ_TO_NPVARIANT(vesrionCString,*result);
	return true ; 
}

// -------------------------------------------------------------------------
// bool CFlashMethods::Play(	const NPVariant */*args*/, 
//						uint32_t /*argCount*/, 
//						NPVariant */*result*/)
//
// Starts playing the movie.    
// Example  movie.Play(); 
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::Play(	const NPVariant */*args*/, 
						uint32_t /*argCount*/, 
						NPVariant */*result*/)
{	
	return iFlashMethods->Play();					
}

// -------------------------------------------------------------------------
// bool CFlashMethods::Rewind(	const NPVariant */*args*/, 
//						uint32_t /*argCount*/, 
//						NPVariant */*result*/)
//
// Goes to the first frame.    
// Example  movie.Rewind(); 
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::Rewind(	const NPVariant */*args*/, 
						uint32_t /*argCount*/, 
						NPVariant */*result*/)
{

	TBool stat = iFlashMethods->Rewind();
	
	return true;
	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::SetVariableL(const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant *result)
//
// Sets the value of the Flash variable specified by variableName 
// to the value specified by value. The argument type for both arguments
// is string.    
// Example  movie.SetVariable("/Form:UserName", "John Smith"); 
// --------------------------------------------------------------------------
//

bool CFlashMethods::SetVariableL(const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
{
	TBool retVal = false;
	if(argCount == 2)
	{		
		NPString variableName = NPVARIANT_TO_STRING(args[0]);
		if(variableName.UTF8Characters != NULL)
		    {
		    NPVariantType argType = args[1].type;
		    NPString value;
		    HBufC* val =NULL;
		    if( argType == NPVariantType_String )
		        {
		        value = NPVARIANT_TO_STRING(args[1]);
		        if(value.UTF8Characters != NULL)
		            {
                    val = CreateEpocStringL(value.UTF8Characters,value.UTF8Length);
                    CleanupStack::PushL(val);
		            }
		        else
		            return retVal;

		        }
		    else if ( argType == NPVariantType_Int32 )
		        {
		        TInt value = NPVARIANT_TO_INT32(args[1]);
		        val  = HBufC::NewLC(20);
		        TPtr tmpPtr = val ->Des();
		        tmpPtr.AppendNum(value);
		        }
		    else if( argType == NPVariantType_Double )
		        {
		        TInt value = NPVARIANT_TO_DOUBLE(args[1]);
		        val  = HBufC::NewLC(20);
		        TPtr tmpPtr = val ->Des();
		        tmpPtr.AppendNum(value);
		        }
		    else if( argType == NPVariantType_Bool )
		        {
		        //iSrc: 16059, Core player accepts values in String format so Boolean has to be sent as "true"/"false"
		        _LIT(KStrTrue, "true");
		        _LIT(KStrFalse, "false");			
		        TInt value = NPVARIANT_TO_BOOLEAN(args[1]);
		        val  = HBufC::NewLC(20);
		        TPtr tmpPtr = val ->Des();
		        if(value)
		            tmpPtr.Append(KStrTrue);
		        else
		            tmpPtr.Append(KStrFalse);
		        }
		    else
		        {
		        return retVal;
		        }


		    TPtr ptrVal = val->Des();

		    HBufC* varName = CreateEpocStringL(variableName.UTF8Characters
		            ,variableName.UTF8Length);		
		    CleanupStack::PushL(varName);

		    TPtr ptrVarName = varName->Des();

		    HBufC* urlIn=iBrowserInstance->GetFlashBaseURL();

		    if(urlIn)
		        {
		        iFlashMethods->SetVariableL(ptrVarName,
		                ptrVal,
		                *urlIn,
		                iAccessType, 
		                ETrue);
		        retVal = true;
		        }

		    CleanupStack::PopAndDestroy(varName);
		    CleanupStack::PopAndDestroy(val);
		    }
	}
	return retVal;	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::SetZoomRect(const NPVariant */*args*/, 
//						uint32_t /*argCount*/, 
//						NPVariant */*result*/)

// Zooms in on a rectangular area of the movie. 
// The units of the coordinates are in twips (1440 units per inch). 
// To calculate a rectangle in Flash, set the ruler units to Points
// and multiply the coordinates by 20 to get twips. (There are 72 points per inch.)
// The argument type for all arguments is integer.    
// Example  This example zooms in on a 200 x 200 pixel rectangle in the 
// upper left corner of the movie: 
//
// var pointsToTwips = 20;
// movie.SetZoomRect(0, 0, 200 * pointsToTwips, 200 * pointsToTwips);
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::SetZoomRect(const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
{
	if(argCount >=4)
	{
		TInt32 left = NPVARIANT_TO_INT32(args[0]);
		TInt32 top = NPVARIANT_TO_INT32(args[1]);
		TInt32 right = NPVARIANT_TO_INT32(args[2]);
		TInt32 bottom = NPVARIANT_TO_INT32(args[3]);
		
		iFlashMethods->SetZoomRect(left,top,right,bottom);
		
		return true;
	}
	
	return false;	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::StopPlay(	const NPVariant */*args*/, 
//						uint32_t /*argCount*/, 
//						NPVariant */*result*/)
//
// Stops playing the movie.    
// Example movie.StopPlay() 
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::StopPlay(	const NPVariant */*args*/, 
						uint32_t /*argCount*/, 
						NPVariant */*result*/)
{
	TBool stat = iFlashMethods->StopPlay();
	
	return true;
	
	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::TotalFrames(const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant *result)
//
// Returns the total number of frames in the movie.    
// Example  var totalFrames = movie.TotalFrames(); 
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::TotalFrames(const NPVariant */*args*/, 
						uint32_t /*argCount*/, 
						NPVariant *result)
{
	TInt totFrame= 0;
		
	iFlashMethods->TotalFrames(totFrame);
		//put result back		
	INT32_TO_NPVARIANT( totFrame, *result );	
		
	return true;
	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::Zoom(const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)

// Zooms the view by a relative scale factor specified by percent.
// Zoom(50) doubles the size of the objects in the view. 
// Zoom(200) reduces the size of objects in the view by one half. 
// Zoom(0) resets the view to 100%. 
// You cannot specify a reduction in the size of objects in the view 
// when the current view is already 100%. The argument type is integer.
//    
// Example movie.Zoom(50);  
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::Zoom(const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
{
	if(argCount >= 1)
	{
		TInt32 zoomPercent = NPVARIANT_TO_INT32(args[0]);	
		
		iFlashMethods->Zoom(zoomPercent);
		
		return true;	
	}
	
	return false;	
}

//////////////////////////////////////////////////////////////////////////
//TellTarget Methods


// -------------------------------------------------------------------------
// bool CFlashMethods::TCallFrame(const NPVariant */*args*/, 
//						uint32_t /*argCount*/, 
//						NPVariant */*result*/)

//  In the timeline specified by target, executes the action in 
//  the frame specified by frameNumber. 
//     
// Example  This example runs the actions in the fifth frame of the main timeline: 
// 				movie.TCallFrame("/", 4);
// --------------------------------------------------------------------------
//

bool CFlashMethods::TCallFrameL(const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
{
	if(argCount >=2)
	{
		NPString target = NPVARIANT_TO_STRING(args[0]);
		if(target.UTF8Characters != NULL)
		    {
            TInt32 frameNumber  = NPVARIANT_TO_INT32(args[1]);
            
            HBufC* targetBuf = CreateEpocStringL(target.UTF8Characters
                                                ,target.UTF8Length);		
            CleanupStack::PushL(targetBuf);
            
            TPtr ptrTarget = targetBuf->Des();
            
            iFlashMethods->TCallFrameL(ptrTarget,frameNumber);
            
            CleanupStack::PopAndDestroy(targetBuf);
                
            return true;	
		    }
	}
	
	return false;	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::TCallLabel(	const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)

//  In the Timeline indicated by target, executes the action in the frame
// specified by the label frame label. The argument type for both arguments
// is string.
//     
// Example  This example runs the actions in the frame labeled 
// 			HandleScriptNotify in the main timeline: 
//			movie.TCallLabel("/", "HandleScriptNotify");
//
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::TCallLabelL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
{
	if(argCount >=2)
	{
		NPString target = NPVARIANT_TO_STRING(args[0]);
		NPString label = NPVARIANT_TO_STRING(args[1]);
		
		if((target.UTF8Characters != NULL) && (label.UTF8Characters != NULL))
		    {
		    HBufC* targetBuf = CreateEpocStringL(target.UTF8Characters
		            ,target.UTF8Length);		
		    CleanupStack::PushL(targetBuf);

		    HBufC* labelBuf = CreateEpocStringL(label.UTF8Characters
		            ,label.UTF8Length);		
		    CleanupStack::PushL(labelBuf);

		    TPtr ptrTarget = targetBuf->Des();
		    TPtr ptrLabel = labelBuf->Des();

		    iFlashMethods->TCallLabelL(ptrTarget,ptrLabel);

		    CleanupStack::PopAndDestroy(labelBuf);
		    CleanupStack::PopAndDestroy(targetBuf);

		    return true;
		    }
	}
	return false;	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::TCurrentFrame(	const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant *result)
//
//  TCurrentFrame( target ) 
//	Returns the number of the current frame for the timeline specified by target.
//  The frame number returned is zero-based, meaning frame 1 of the Flash movie
//  would be 0, frame 2 would be 1, and so on. The argument type is string.
//    
// Example  var currentFrame = movie.TCurrentFrame("/MovieClip"); 
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::TCurrentFrameL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result)
{
	TBool retVal = false;
	if(argCount >=1)
	{
		NPString target = NPVARIANT_TO_STRING(args[0]);
		
		if(target.UTF8Characters != NULL)
		    {
			
		    HBufC* targetBuf = CreateEpocStringL(target.UTF8Characters
		            ,target.UTF8Length);		
		    CleanupStack::PushL(targetBuf);

		    TPtr ptrTarget = targetBuf->Des();

		    HBufC* urlIn=iBrowserInstance->GetFlashBaseURL();
		    if(urlIn)
		        {

		        TInt frameNo = 0;

		        iFlashMethods->TCurrentFrameL(ptrTarget,frameNo,*urlIn
		                ,iAccessType,ETrue);

		        INT32_TO_NPVARIANT( frameNo, *result );	
		        retVal = true;
		        }

		    CleanupStack::PopAndDestroy(targetBuf);
		    }
		}

	return retVal;	
	}

// -------------------------------------------------------------------------
// bool CFlashMethods::CurrentFrame(   const NPVariant *args, 
//                      uint32_t argCount, 
//                      NPVariant *result)
//
//  CurrentFrame() 
//  Returns the frame number the root object. 
// Example  var currentFrame = movie.CurrentFrame(); 
//
// --------------------------------------------------------------------------
//
bool CFlashMethods::CurrentFrame( const NPVariant */*args*/, 
                        uint32_t /*argCount*/, 
                        NPVariant *result)
{
    bool retVal = false;
    TInt frameNo = 0;
    retVal = iFlashMethods->CurrentFrame(frameNo);
	if(result != NULL)
		INT32_TO_NPVARIANT( frameNo, *result ); 
    return retVal;  
 }


// -------------------------------------------------------------------------
// bool CFlashMethods::TCurrentLabel(	const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)
//
// Returns the label of the current frame of the timeline specified by target.
// If there is no current frame label, an empty string is returned. 
// The argument type is string.    
// Example  var currentLabel = movie.TCurrentLabel("/MovieClip");    
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::TCurrentLabelL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result)
{
	TBool retVal = false;
	if(argCount >=1)
	{
		NPString target = NPVARIANT_TO_STRING(args[0]);
		
		if(target.UTF8Characters != NULL)
		    {
			
		    HBufC* targetBuf = CreateEpocStringL(target.UTF8Characters
		            ,target.UTF8Length);		
		    CleanupStack::PushL(targetBuf);

		    TPtr ptrTarget = targetBuf->Des();

		    HBufC* urlIn=iBrowserInstance->GetFlashBaseURL();
		    HBufC* label = NULL;
		    if(urlIn)
		        {

		        iFlashMethods->TCurrentLabelL(ptrTarget
		                ,&label
		                ,*urlIn
		                ,iAccessType
		                ,ETrue);
		        retVal = true;
		        char* labelCString = CreateFlashStringL( *label );

		        STRINGZ_TO_NPVARIANT(labelCString,*result);
		        }

		    CleanupStack::PopAndDestroy(targetBuf);
		    }		
		}

	return retVal;	
	}

// -------------------------------------------------------------------------
//bool CFlashMethods::TGetProperty(	const NPVariant */*args*/, 
//						uint32_t /*argCount*/, 
//						NPVariant */*result*/)

//  For the timeline indicated by target, returns a string indicating 
//  the value of the property specified by property. For property, enter the
//  integer corresponding to the desired property. For a list of all properties
//  and their corresponding integers, see Getting and setting properties. 
//     
//  Example  var nameIndex = 13;  
//			var name = movie.TGetProperty("/", nameIndex);
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::TGetPropertyL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result)
    {
    TBool retVal = false;
    if(argCount >=2)
        {
        NPString target = NPVARIANT_TO_STRING(args[0]);

        if(target.UTF8Characters != NULL)
            {
            TUint32 propIdx=0;
            NPVariantType argType = args[1].type;
            if( argType == NPVariantType_String )
                {
                NPString property = NPVARIANT_TO_STRING(args[1]);
                if(property.UTF8Characters != NULL)
                    {
                    HBufC* propertyBuf = CreateEpocStringL(property.UTF8Characters
                            ,property.UTF8Length);		

                    propIdx = GetIntFromDesc(*propertyBuf);
                    delete propertyBuf;
                    }
                else 
                    return retVal;
                }
            else if ( argType == NPVariantType_Int32 )
                {
                propIdx = NPVARIANT_TO_INT32(args[1]);
                }
            else if( argType == NPVariantType_Double )
                {
                propIdx = NPVARIANT_TO_DOUBLE(args[1]);
                }

            HBufC* targetBuf = CreateEpocStringL(target.UTF8Characters
                    ,target.UTF8Length);		
            CleanupStack::PushL(targetBuf);

            TPtr ptrTarget = targetBuf->Des();

            HBufC* urlIn=iBrowserInstance->GetFlashBaseURL();
            HBufC* propValue = NULL;

            if(urlIn)
                {
                iFlashMethods->TGetPropertyL(ptrTarget
                        ,propIdx
                        ,&propValue
                        ,*urlIn
                        ,iAccessType
                        ,ETrue);
                retVal = true;			
                }

            CleanupStack::PopAndDestroy(targetBuf);
            char* propVal = NULL;
            if( propValue )
                {
                propVal = CreateFlashStringL( *propValue );

                STRINGZ_TO_NPVARIANT(propVal, *result);
                delete propValue;
                }
            else
                {
                return false;			
                }		
            }
        }

    return retVal;	
    }

// -------------------------------------------------------------------------
// bool CFlashMethods::TGetPropertyAsNumber(	const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)
//
// For the timeline indicated by target, returns a number indicating 
// the value of the property specified by property. For property, 
// enter the integer corresponding to the desired property. 
// For a list of all properties and their corresponding integers, 
// see Getting and setting properties. 
//
// var framesLoadedIndex = 12; 
//
// var framesLoaded = movie.TGetPropertyAsNumber("/", framesLoadedIndex);
//
//
// --------------------------------------------------------------------------
//
bool CFlashMethods::TGetPropertyAsNumberL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant * result)
{
	if(argCount >=2)
	{
		NPString target = NPVARIANT_TO_STRING(args[0]);
		if(target.UTF8Characters != NULL)
		    {
		    NPVariantType argType = args[1].type;
		    TUint32 propIdx=0;
		    if( argType == NPVariantType_String )
		        {
		        NPString property = NPVARIANT_TO_STRING(args[1]);
		        if(property.UTF8Characters != NULL)
		            {
		            HBufC* propertyBuf = CreateEpocStringL(property.UTF8Characters
		                    ,property.UTF8Length);		

		            propIdx = GetIntFromDesc(*propertyBuf);
		            delete propertyBuf;
		            }
		        else
		            return false;
		        }
		    else if ( argType == NPVariantType_Int32 )
		        {
		        propIdx = NPVARIANT_TO_INT32(args[1]);
		        }
		    else if( argType == NPVariantType_Double )
		        {
		        propIdx = NPVARIANT_TO_DOUBLE(args[1]);
		        }
		    HBufC* targetBuf = CreateEpocStringL(target.UTF8Characters
		            ,target.UTF8Length);		
		    CleanupStack::PushL(targetBuf);

		    TPtr ptrTarget = targetBuf->Des();

		    HBufC* urlIn=iBrowserInstance->GetFlashBaseURL();
		    HBufC* propValue = NULL;
		    if(urlIn)
		        {
		        iFlashMethods->TGetPropertyAsNumberL(ptrTarget
		                ,propIdx
		                ,&propValue
		                ,*urlIn
		                ,iAccessType
		                ,ETrue);

		        }
		    CleanupStack::PopAndDestroy(targetBuf);
		    if ( /* retVal && */ propValue )
		        {
		        char* propVal = CreateFlashStringL( *propValue );

		        STRINGZ_TO_NPVARIANT(propVal, *result);	
		        delete propValue;
		        return true;				
		        }
		    else
		        {
		        return false;
		        }	
		    }
	}
	return false;	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::TGotoFrame(	const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)
//
// For the timeline indicated by target, goes to the frame number
// specified by frameNumber. The argument type for target is string. 
// The argument type for frameNumber is integer. 
//     
//  Example : movie.TGotoFrame("/MovieClip", 2);  
//
// --------------------------------------------------------------------------
//
bool CFlashMethods::TGotoFrameL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
{
	if(argCount >=2)
	{
		NPString target = NPVARIANT_TO_STRING(args[0]);
		TInt32 FrameNo  = NPVARIANT_TO_INT32(args[1]);
		
		if(target.UTF8Characters != NULL)
		    {
		    HBufC* targetBuf = CreateEpocStringL(target.UTF8Characters
		            ,target.UTF8Length);		
		    CleanupStack::PushL(targetBuf);

		    TPtr ptrTarget = targetBuf->Des();

		    iFlashMethods->TGotoFrameL(ptrTarget,FrameNo);

		    CleanupStack::PopAndDestroy(targetBuf);

		    return true;	
		    }
	}
	
	return false;	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::TGotoLabel(	const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)
//
// For the timeline indicated by target, goes to the frame label 
// specified by label. The argument type for both arguments is string. 
//     
// Example  movie.TGotoLabel("/MovieClip", "MyLabel");  
//
// --------------------------------------------------------------------------
//
bool CFlashMethods::TGotoLabelL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
{
	TBool result = false;
	if(argCount >=2)
	{
		NPString target = NPVARIANT_TO_STRING(args[0]);
		NPString label = NPVARIANT_TO_STRING(args[1]);
		
		if((target.UTF8Characters != NULL) && (label.UTF8Characters != NULL))
		    {		
		    HBufC* targetBuf = CreateEpocStringL(target.UTF8Characters
		            ,target.UTF8Length);		
		    CleanupStack::PushL(targetBuf);

		    HBufC* labelBuf = CreateEpocStringL(label.UTF8Characters
		            ,label.UTF8Length);		
		    CleanupStack::PushL(labelBuf);

		    TPtr ptrTarget = targetBuf->Des();
		    TPtr ptrLabel = labelBuf->Des();

		    result = iFlashMethods->TGotoLabelL(ptrTarget,ptrLabel);

		    CleanupStack::PopAndDestroy(labelBuf);
		    CleanupStack::PopAndDestroy(targetBuf);
		    }
	}
	return result;	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::TPlay(	const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)
//
//  Plays the timeline specified by target. The argument type is string. 
//     
//  Example  movie.TPlay("/MovieClip");  
//
// --------------------------------------------------------------------------
//
bool CFlashMethods::TPlayL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
{
	TBool retVal = false;
	if(argCount >=1)
	{
		NPString target = NPVARIANT_TO_STRING(args[0]);
		if(target.UTF8Characters != NULL)
		    {			
            HBufC* targetBuf = CreateEpocStringL(target.UTF8Characters
                                                ,target.UTF8Length);		
            CleanupStack::PushL(targetBuf);
            
            TPtr ptrTarget = targetBuf->Des();
            
            retVal = iFlashMethods->TPlayL(ptrTarget);
            
            CleanupStack::PopAndDestroy(targetBuf);
		    }
	}
	
	return retVal;	
}

bool CFlashMethods::IsTSetPropertySupported(TInt32 aPropIdx)
{
	switch(aPropIdx)
	{
		case E_PROP_CURRENT_FRAME:
		case E_PROP_TOTAL_FRAMES :
		case E_PROP_WIDTH		:
		case E_PROP_HEIGHT		:
		case E_PROP_TARGET		 :
		case E_PROP_FRAMES_LOADED :
		case E_PROP_DROP_TARGET	:
		case E_PROP_URL		:
								{
									return false;
								}
		default:
				{
					return true;
				}
	}
}

// -------------------------------------------------------------------------
// bool CFlashMethods::TSetProperty(const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)
//
// For the timeline indicated by target, sets the value of the property 
// specified by property to the value specified by value, 
// which can be a string or a number. For property, enter the integer 
// corresponding to the desired property. For a list of all properties
// and their corresponding integers, see Getting and setting properties.
//    
// Example  var visibilityIndex = 7;var nameIndex = 13;
// 			movie.TSetProperty("/MovieClip", visibilityIndex, 1);
//			movie.TSetProperty("/MovieClip", nameIndex, "NewName");    
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::TSetPropertyL(const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
    {
    TBool retVal = false;
    if(argCount >=3)
        {
        NPString target = NPVARIANT_TO_STRING(args[0]);
        if(target.UTF8Characters != NULL)
            {
            NPVariantType argType = args[1].type;     
            NPVariant Arg2 = args[2];

            TInt32 propIdx=0;
            if( argType == NPVariantType_String )
                {
                NPString property = NPVARIANT_TO_STRING(args[1]);
                if(property.UTF8Characters != NULL)
                    {
                    TLex8 propLex((TUint8*)(property.UTF8Characters));
                    propLex.Val(propIdx);
                    }
                else 
                    return retVal;
                }
            else if ( argType == NPVariantType_Int32 )
                {
                propIdx = NPVARIANT_TO_INT32(args[1]);
                }
            else if( argType == NPVariantType_Double )
                {
                propIdx = NPVARIANT_TO_DOUBLE(args[1]);
                }


            if(IsTSetPropertySupported(propIdx)) //check if set is supported
                {
                HBufC* propValBuf=NULL;
                if(Arg2.type == NPVariantType_String)
                    {
                    NPString PropVal = NPVARIANT_TO_STRING(args[2]);	
                    propValBuf = CreateEpocStringL(PropVal.UTF8Characters
                            ,PropVal.UTF8Length);		
                    CleanupStack::PushL(propValBuf);
                    }
                else if ( Arg2.type == NPVariantType_Int32 )
                    {
                    TInt PropVal = NPVARIANT_TO_INT32(args[2]);
                    propValBuf = HBufC::NewLC(20);
                    TPtr tmpPtr = propValBuf->Des();
                    tmpPtr.AppendNum(PropVal);
                    }
                else if( Arg2.type == NPVariantType_Double )
                    {
                    TInt PropVal = NPVARIANT_TO_DOUBLE(args[2]);
                    propValBuf = HBufC::NewLC(20);
                    TPtr tmpPtr = propValBuf->Des();
                    tmpPtr.AppendNum(PropVal);
                    }

                HBufC* targetBuf = CreateEpocStringL(target.UTF8Characters
                        ,target.UTF8Length);		
                CleanupStack::PushL(targetBuf);


                TPtr ptrTarget = targetBuf->Des();
                TPtr ptrPropVal = propValBuf->Des();

                HBufC* urlIn=iBrowserInstance->GetFlashBaseURL();
                if(urlIn)
                    {
                    iFlashMethods->TSetPropertyL(ptrTarget,propIdx,ptrPropVal
                            ,*urlIn
                            ,iAccessType
                            ,ETrue);
                    retVal = true;
                    }

                CleanupStack::PopAndDestroy(targetBuf);
                CleanupStack::PopAndDestroy(propValBuf);
                }

            }
        }
    return retVal;		
}

// -------------------------------------------------------------------------
// bool CFlashMethods::TStopPlay(const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)
//
// Stops the timeline specified by target. The argument type is string. 
//     
// Example  movie.TStopPlay("/MovieClipToStop"); 
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::TStopPlayL(const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
{
	if(argCount >=1)
	{
		NPString target = NPVARIANT_TO_STRING(args[0]);
		if(target.UTF8Characters != NULL)
		    {
			
            HBufC* targetBuf = CreateEpocStringL(target.UTF8Characters
                                                ,target.UTF8Length);		
            CleanupStack::PushL(targetBuf);
            
            TPtr ptrTarget = targetBuf->Des();
            
            iFlashMethods->TStopPlayL(ptrTarget);
            
            CleanupStack::PopAndDestroy(targetBuf);
                
            return true;	
		    }
	}
	
	return false;	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::CallMethod( const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant *result)
//
// calls the flash method
//
// --------------------------------------------------------------------------
//
bool CFlashMethods::CallMethodL( const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result)
{
	if(argCount >=1)
	{
		NPString request = NPVARIANT_TO_STRING(args[0]);
		if(request.UTF8Characters != NULL)
		  {
			
		    HBufC* requestBuf = CreateEpocStringL(request.UTF8Characters
		            ,request.UTF8Length);		
		    CleanupStack::PushL(requestBuf);

		    TPtr ptrRequest = requestBuf->Des();

		    HBufC* retValue = NULL;

		    iExternalInterface->CallMethodL(ptrRequest,&retValue);

		    char* retVal = CreateFlashStringL( *retValue );

		    STRINGZ_TO_NPVARIANT(retVal, *result);	

		    delete retValue;

		    CleanupStack::PopAndDestroy(requestBuf);

		    return true;
		  }
	}
	
	return false;	
}

// -------------------------------------------------------------------------
// bool CFlashMethods::SetReturnValue(const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)
//
// Set return values after script execution 
//     
//
// --------------------------------------------------------------------------
//

bool CFlashMethods::SetReturnValueL(const NPVariant *args, 
						uint32_t argCount, 
						NPVariant */*result*/)
{
	if(argCount >=1)
	{
		NPString returnValue = NPVARIANT_TO_STRING(args[0]);
		if(returnValue.UTF8Characters != NULL)
		    {
			
            HBufC* valBuf = CreateEpocStringL(returnValue.UTF8Characters
                                                ,returnValue.UTF8Length);		
            CleanupStack::PushL(valBuf);
            
            iExternalInterface->SetReturnValueL(*valBuf);
            
            CleanupStack::PopAndDestroy();
            
            return true;
		    }
	}
	
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////
//

