/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  browser dll initialisation code and exported functions
*
*/


// INCLUDE FILES
#include "saplayer_build.h"
#include "commontypes.h"

#include <implementationproxy.h>
#include <eikenv.h>
#include <bautils.h> // NearestLanguage
#include <data_caging_path_literals.hrh>
#include <featmgr.h>
#include <string.h>

#include "flashplgmain.h"
_LIT(KAllowNetworkAccess,"AllowNetworkAccess");
// CONSTANTS
const TUint KPluginImplementation1UidValue=0x101f4e5d; //from 101F4E5B.rss (ECOM DLL UID)
const TUint KByteLength = 8;

extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );

const TImplementationProxy KImplementationTable[] =
    {
    IMPLEMENTATION_PROXY_ENTRY(KPluginImplementation1UidValue, CNpEcomMain::NewL)
    };

// -----------------------------------------------------------------------------
// ImplementationGroupProxy
// Returns the filters implemented in this DLL
// Returns: The filters implemented in this DLL
// -----------------------------------------------------------------------------
//
EXPORT_C const TImplementationProxy* ImplementationGroupProxy(TInt& aTableCount)
    {
    aTableCount = sizeof(KImplementationTable) /
    sizeof(TImplementationProxy);
    return KImplementationTable;    
    }

// -------------------------------------------------------------------
// NPError InitializeFuncs(NPPluginFuncs* aPluginFuncs)
//
// This function is called to initialize the struct NPPluginFuncs
// --------------------------------------------------------------------
// 
NPError InitializeFuncs(NPPluginFuncs* aPluginFuncs)
    {
    aPluginFuncs->size          = sizeof(NPPluginFuncs);
    aPluginFuncs->version       = (NP_VERSION_MAJOR << KByteLength) + NP_VERSION_MINOR;
    aPluginFuncs->newp          = NewNPP_NewProc(my_newp);
    aPluginFuncs->destroy       = NewNPP_DestroyProc(my_destroy);
    aPluginFuncs->setwindow     = NewNPP_SetWindowProc(my_setwindow);
    aPluginFuncs->newstream     = NewNPP_NewStreamProc(my_newstream);
    aPluginFuncs->destroystream = NewNPP_DestroyStreamProc(my_destroystream);
    aPluginFuncs->asfile        = NewNPP_StreamAsFileProc(my_asfile);
    aPluginFuncs->writeready    = NewNPP_WriteReadyProc(my_writeready);
    aPluginFuncs->write         = NewNPP_WriteProc(my_write);
    aPluginFuncs->print         = NewNPP_PrintProc(my_print);
    aPluginFuncs->event         = NewNPP_HandleEventProc(my_event);
    aPluginFuncs->urlnotify     = NewNPP_URLNotifyProc(my_urlnotify);
    aPluginFuncs->javaClass     = NULL;
    aPluginFuncs->getvalue      = NewNPP_GetValueProc(my_getvalue);
    aPluginFuncs->setvalue      = NewNPP_SetValueProc(my_setvalue);

    return NPERR_NO_ERROR;
    }

// -------------------------------------------------------------------
// CNpEcomMain* CNpEcomMain::NewL(TAny* aInitParam)
//
// This function is called to initialize the struct NPPluginFuncs
// --------------------------------------------------------------------
// 
CNpEcomMain* CNpEcomMain::NewL(TAny* aInitParam)
{
	NPPDebug(LOG_DEBUG,_L("CNpEcomMain::NewL ++"));
	TFuncs* funcs = REINTERPRET_CAST( TFuncs*, aInitParam);
	CNpEcomMain* self = new(ELeave)CNpEcomMain(funcs->iNetscapeFuncs);
	CleanupStack::PushL(self);
	self->ConstructL(funcs->iPluginFuncs);
	CleanupStack::Pop(self);
	NPPDebug(LOG_DEBUG,_L("CNpEcomMain::NewL --"));

	return self;
}

// -------------------------------------------------------------------
// void CNpEcomMain::ConstructL(NPPluginFuncs* aPluginFuncs)
//
// This function is called to initialize the struct NPPluginFuncs
// --------------------------------------------------------------------
// 
void CNpEcomMain::ConstructL(NPPluginFuncs* aPluginFuncs)
    {
    InitializeFuncs(aPluginFuncs);

    iGlueGlobals = CGlueGlobals::NewL(iNpf);
    }

// -------------------------------------------------------------------
// CNpEcomMain::CNpEcomMain(NPNetscapeFuncs* aNpf)
//
// First phase constructor
// --------------------------------------------------------------------
// 
CNpEcomMain::CNpEcomMain(NPNetscapeFuncs* aNpf) : iNpf(aNpf)
    {
    }

// -------------------------------------------------------------------
// CNpEcomMain::~CNpEcomMain()
//
// Destructor
// --------------------------------------------------------------------
// 
CNpEcomMain::~CNpEcomMain()
    {
    if (iGlueGlobals)
        {
        iGlueGlobals->DeletePluginGlobal();
        iGlueGlobals = NULL;
        }
    }

// ================= CGlueGlobals ==========================


// -------------------------------------------------------------------------
// CGlueGlobals::CGlueGlobals(NPNetscapeFuncs* aFuncs)
//
// First phase constructor
// --------------------------------------------------------------------------
// 

CGlueGlobals* CGlueGlobals::NewL(NPNetscapeFuncs* aFuncs)
    {
	    CGlueGlobals* pluginGlobals=NULL;
	    
	    // Check local thread storage
	    if( Dll::Tls() == NULL )
	        {
		        pluginGlobals = new (ELeave) CGlueGlobals(aFuncs);
		        
		        if(pluginGlobals)
		        {
		        	CleanupStack::PushL(pluginGlobals);
		        	pluginGlobals->ConstructL();
		        	CleanupStack::Pop(); 
		    		
		    		TInt error = Dll::SetTls(static_cast<TAny*>(pluginGlobals));    	
		    		
		    		if( error == KErrNone )
		            {
			            User::LockedInc(pluginGlobals->iRefCnt);//pluginGlobals->iRefCnt++;
			            
			            return pluginGlobals;
		            }
	        
	        		delete pluginGlobals;

		        }
	        
	        }
	    else
	        {        
		        pluginGlobals = STATIC_CAST(CGlueGlobals*, Dll::Tls() );
		        
		        if(pluginGlobals)
			        {
		        		User::LockedInc(pluginGlobals->iRefCnt);//pluginGlobals->iRefCnt++;
		
		        		return pluginGlobals;
			        }
	        }
	        
	    
	    return NULL;    
    }


// -------------------------------------------------------------------------
// CGlueGlobals::CGlueGlobals(NPNetscapeFuncs* aFuncs)
//
// Default constructor
// --------------------------------------------------------------------------
// 

CGlueGlobals::CGlueGlobals(NPNetscapeFuncs* aFuncs)
               : iNetscapeFunctions(aFuncs)
    {    
    }

// -------------------------------------------------------------------------
// void CGlueGlobals::ConstructL()
//
// Second phase constructor
// --------------------------------------------------------------------------
// 
void CGlueGlobals::ConstructL()
    {
 
 
   	// Load resources - as we are not an app we need to do this manually
/*  commented Reason : to avoid locking of the resource file right here

  	_LIT(KPluginRscFile, "flashliteplugin.rsc");
	NPPDebug(LOG_DEBUG,_L("CGlueGlobals::ConstructL++"));
	TFileName rscfn;
	TFileName dllName;
 	Dll::FileName(dllName);
  	TBuf<KDriveLetterLength> drive = dllName.Left(KDriveLetterLength);		

	rscfn.Copy(drive); // get the drive letter
	rscfn.Append(KDC_APP_RESOURCE_DIR);
	rscfn.Append(KPluginRscFile);

	BaflUtils::NearestLanguageFile(CCoeEnv::Static()->FsSession(), rscfn);
	
	TBool fileExists( BaflUtils::FileExists(CCoeEnv::Static()->FsSession(), rscfn) );
	if ( fileExists )
    {
    	iResOffset = CCoeEnv::Static()->AddResourceFileL(rscfn);
    }
*/
#ifdef FEATURE_WEB_BROWSABILITY    
		NPPDebug(LOG_DEBUG,_L("CGlueGlobals::add script plug-in funs++"));
    InitializeScriptableFuncPtrs();
#endif //FEATURE_WEB_BROWSABILITY    
    }

_LIT(KFlashPluginLibraryName, "NPFlashliteInstance.dll");
const TInt KFlashCreateDocumentOrdinal = 1;

typedef void*   (*PFNC_CreateFlashPlugin)(MNetscapeIf*, MPluginArguments*,TPluginUiFramework);

MFlashPluginIf* CGlueGlobals::LoadFlashPluginL(MNetscapeIf* netscapeIf,MPluginArguments* aPluginArguments)
    {    
    User::LeaveIfError(iLibrary.Load(KFlashPluginLibraryName));
    
    PFNC_CreateFlashPlugin pFncCreateFlashPlugin =
            (PFNC_CreateFlashPlugin) iLibrary.Lookup(
                    KFlashCreateDocumentOrdinal);
    if (pFncCreateFlashPlugin == NULL)
        {
        User::Leave(KErrGeneral);
        }
    return (MFlashPluginIf*) pFncCreateFlashPlugin(netscapeIf, aPluginArguments,EAvkonFrameWork);    
    }

#ifdef FEATURE_WEB_BROWSABILITY      
void CGlueGlobals::InitializeScriptableFuncPtrs()
	{
	iFlashJavaScriptHandlerObject.allocate = (NPAllocateFunctionPtr) FlashJavaScriptHandlerAllocate;
	iFlashJavaScriptHandlerObject.deallocate = (NPDeallocateFunctionPtr) FlashJavaScriptHandlerDeallocate;    
    iFlashJavaScriptHandlerObject.invalidate =    (NPInvalidateFunctionPtr) FlashJavaScriptHandlerInvalidate;
    iFlashJavaScriptHandlerObject.hasMethod =  (NPHasMethodFunctionPtr)FlashJavaScriptHandlerHasMethod;
	iFlashJavaScriptHandlerObject.invoke = (NPInvokeFunctionPtr) FlashJavaScriptHandlerInvokeFunctionPtr;
    iFlashJavaScriptHandlerObject.invokeDefault = (NPInvokeDefaultFunctionPtr) FlashJavaScriptHandlerInvokeDefaultFunctionPtr;
    iFlashJavaScriptHandlerObject.hasProperty = (NPHasPropertyFunctionPtr) FlashJavaScriptHandlerHasProperty; //TODO:check need
	iFlashJavaScriptHandlerObject.getProperty = (NPGetPropertyFunctionPtr) FlashJavaScriptHandlerGetProperty;//TODO:check need
    iFlashJavaScriptHandlerObject.setProperty = (NPSetPropertyFunctionPtr) FlashJavaScriptHandlerSetProperty;//TODO:check need
    iFlashJavaScriptHandlerObject.removeProperty = (NPRemovePropertyFunctionPtr) 0;
	}
#endif // FEATURE_WEB_BROWSABILITY  	

void CGlueGlobals::DeletePluginGlobal()
    {
	    // Still used by many; so dont delete or there are no users.
	    if( iRefCnt > 1 || iRefCnt == 0 )
	        {
		        if( iRefCnt ) 
		        {
		        	User::LockedDec(iRefCnt); //iRefCnt--;
		        }
	        	
	        	return;
	        }        

		//we have one object left, let us delete
		Dll::FreeTls() ;   //free tls first
		
	    delete this;
    }

// -------------------------------------------------------------------------
// CGlueGlobals::~CGlueGlobals()
//
// destructor
// --------------------------------------------------------------------------
// 
CGlueGlobals::~CGlueGlobals()
{    
    if(iLibrary.Handle())
        iLibrary.Close();
    
   // CEikonEnv::Static()->DeleteResourceFile(iResOffset);
}

//////////////////////////////////////////////////////////////////////////////////////////////////


/////// CFlashPluginProxy ///////

CFlashPluginProxy* CFlashPluginProxy::NewL(NPP aInstance, CPluginArgProcessor* argProcessor)
{
	CFlashPluginProxy* me = new (ELeave) CFlashPluginProxy();
	CleanupStack::PushL(me);
	me->ConstructL(aInstance, argProcessor);
	CleanupStack::Pop(me);
	return me;
}

CFlashPluginProxy::CFlashPluginProxy() :
    instance(NULL),
	plugin(NULL),
	iFlashJavaScriptHandler(NULL),
	iJScriptNPObject(NULL)
{
}

void CFlashPluginProxy::ConstructL(NPP aInstance,
		CPluginArgProcessor* argProcessor)
{
	CGlueGlobals* globals = static_cast<CGlueGlobals*> (Dll::Tls());
	instance = aInstance;
	globals->iArgProc = argProcessor;
	iPageAccessType = argProcessor->AccessType();
	plugin = globals->LoadFlashPluginL(this,argProcessor);
	if(!plugin)
	    {
        User::Leave(KErrNoMemory);
	    }
	//todo remove plugin->GetHostScriptHandler()
	iFlashJavaScriptHandler = CFlashJavaScriptHandler::NewL(aInstance,plugin,plugin->GetHostScriptHandler(),iPageAccessType);
}

int CFlashPluginProxy::GetUrl(const TDesC& url, const TDesC* target)
    {
    NPNetscapeFuncs* NetscapeFunctions = static_cast<NPNetscapeFuncs*>(NpFuncs());
        return NetscapeFunctions->geturl(instance, url, target);
    }

int CFlashPluginProxy::PostUrl(const TDesC& url, const TDesC* target,
        const TDesC& buf, bool file)
    {
    NPNetscapeFuncs* NetscapeFunctions = static_cast<NPNetscapeFuncs*>(NpFuncs());
    return NetscapeFunctions->posturl(instance, url,
                              target,
                              buf, file);
    }

int CFlashPluginProxy::GetUrlNotify(const TDesC& url, const TDesC* target,
        void* notifyData)
    {
    NPNetscapeFuncs* NetscapeFunctions = static_cast<NPNetscapeFuncs*>(NpFuncs());
    return NetscapeFunctions->geturlnotify(instance, url,
                                   target, notifyData);
    }

int CFlashPluginProxy::PostUrlNotify(const TDesC& url, const TDesC* target,
        const TDesC& buf, bool file, void* notifyData)
    {
    NPNetscapeFuncs* NetscapeFunctions = static_cast<NPNetscapeFuncs*>(NpFuncs());
    return NetscapeFunctions->posturlnotify(instance, url, target, buf,
            file, notifyData);
    }

int CFlashPluginProxy::GetValue(int variable, void* ret_value)
    {
    NPNetscapeFuncs* NetscapeFunctions = static_cast<NPNetscapeFuncs*>(NpFuncs());
    return NetscapeFunctions->getvalue(instance, static_cast<NPNVariable>(variable),ret_value);
    }

int CFlashPluginProxy::SetValue(int variable, void* ret_value)
    {
    NPNetscapeFuncs* NetscapeFunctions = static_cast<NPNetscapeFuncs*>(NpFuncs());
    return NetscapeFunctions->setvalue(instance, static_cast<NPPVariable>(variable),ret_value);
    }

void CFlashPluginProxy::InvalidateRect(const TRect& aRect)
    {
    NPRect rect;
    rect.top = aRect.iTl.iY;
    rect.left = aRect.iTl.iX;
    rect.right = aRect.iBr.iX;
    rect.bottom = aRect.iBr.iY;
    
    NPNetscapeFuncs* NetscapeFunctions = static_cast<NPNetscapeFuncs*>(NpFuncs());
    NetscapeFunctions->invalidaterect(instance, &rect);
    
    }

void CFlashPluginProxy::SetScriptAccess()
	{
	if (iFlashJavaScriptHandler)
		{
			//CGlueGlobals* globals = static_cast<CGlueGlobals*> (Dll::Tls());
			
				iFlashJavaScriptHandler->SetScriptAccess();
			
		}
	}
void CFlashPluginProxy::DeleteFlashMethodObject()
{
	if (iFlashJavaScriptHandler)
	{
		iFlashJavaScriptHandler->DeleteFlashMethods();				
	}
}
_LIT(KStrWidgetValue,"WidgetMode");
_LIT(KStrWidgetIdentifier,"ApplicationId");
const int DefaultMode = 0;
const int WidgetMode = 1;
    
TBool CFlashPluginProxy::QueryAllowNetworkInWidgetMode()
{

    RArray<NPN_GenericElement>* genericElements = NULL;
    int err;

    TBool allow = ETrue;
    
    err = GetValue((NPNVariable) NPNVGenericParameter, (void*) &genericElements);

    if (genericElements != NULL && err == KErrNone)
    {
        for (int i = 0; i < genericElements->Count(); i++)
        {
            if (!((*genericElements)[i].genericElementId.Compare(KAllowNetworkAccess)))
            {
                allow = (*genericElements)[i].genericElementValue.intValue;
                break;
            }
        }  
    }
    
    return allow;
}
int CFlashPluginProxy::ResolveExecutionMode()
    {
        RArray<NPN_GenericElement>* genericElements = NULL;
        NPError err;

        
        
        const int GenericParam(NPNVGenericParameter); //NPNVGenericParameter
        err = GetValue(
                GenericParam, (void*) &genericElements);

        if (genericElements != NULL && err == KErrNone/*NPERR_NO_ERROR*/)
        {
            for (int i = 0; i < genericElements->Count(); i++)
            {
                if (!((*genericElements)[i].genericElementId.Compare(
                        KStrWidgetIdentifier)))
                {
                    if (!((*genericElements)[i].genericElementValue.strValue.Compare(
                            KStrWidgetValue)))
                    {
                        //No reason how execution mode is associated with this flag PLUG_IN_MEM_ALLOC
                        //#ifdef PLUG_IN_MEM_ALLOC
                        return WidgetMode;
                        //#endif
                        //break;
                    }
                }
            }
        }
        
        return DefaultMode; 
        // If this code base is used on older platform, uncomment this else part
        /*
        else
        {
            RProcess currentprocess;
            TSecureId currentProcessUid = currentprocess.SecureId();
            //if SID == browser SID or Screen Saver SID  then it is not widget mode
            //Set Widegt Mode True If both are false
            if (!(currentProcessUid == KBrowserUid || currentProcessUid
                    == KScreenSaverUid))
            {
                //No reason how execution mode is associated with this flag PLUG_IN_MEM_ALLOC
                //#ifdef PLUG_IN_MEM_ALLOC
                iInteractionMode = EWidgetMode;
                //#endif
            }
        }
        */
    }

CFlashPluginProxy::~CFlashPluginProxy()
{
#ifdef FEATURE_WEB_BROWSABILITY        
		
	if(iFlashJavaScriptHandler)
		{
		delete iFlashJavaScriptHandler;
		iFlashJavaScriptHandler =NULL;
		}
	
	if (iJScriptNPObject)
	{
		//javascript has handle to this NPObject
		//we need to clean the handle so that Javascript core does not call 
		//when we are in OOM situation

		iJScriptNPObject->handler = NULL;
	}
#endif

	if (plugin)
	{
		plugin->PrepareToExit();

		plugin->Close();
	}

}

MFlashPluginIf* CFlashPluginProxy::Plugin() const
{
	return plugin;
}

CFlashJavaScriptHandler* CFlashPluginProxy::JavaScriptHandler()
{
    if (!iFlashJavaScriptHandler)
	{
		//CGlueGlobals* globals = static_cast<CGlueGlobals*> (Dll::Tls());
		TRAPD(error, iFlashJavaScriptHandler = CFlashJavaScriptHandler::NewL(instance,plugin,plugin->GetHostScriptHandler()));
		if(error == KErrNone)
		{
			iFlashJavaScriptHandler->SetScriptAccess();
		}
	}

	return iFlashJavaScriptHandler;
}

int CFlashPluginProxy::GetValue(NPP instance, NPPVariable aVariable,
		void* ret_value)
{

	int error = NPERR_Generic;
	switch (aVariable)
	{
#ifdef FEATURE_WEB_BROWSABILITY

	case NPPVpluginScriptableNPObject:
	{
		if (!iFlashJavaScriptHandler)
		{ //need to trap here
			iFlashJavaScriptHandler = this->JavaScriptHandler();
		}

		if (!iJScriptNPObject)
		{ //let us keep the FlashJavaScriptHandlerObject copy for this instance
			CGlueGlobals* globals = static_cast<CGlueGlobals*> (Dll::Tls());

			iJScriptNPObject
					= (FlashJavaScriptHandlerObject *) NPN_CreateObject(
							instance,
							globals->GetFlashJavaScriptHandlerObject());

			iJScriptNPObject->handler = iFlashJavaScriptHandler;

		}

		void** ret = (void**) ret_value;
		*ret = (void*) iJScriptNPObject; //return our copy for the instance. 
	}
		error = NPERR_NO_ERROR;
		break;

#endif
	default:
		if (plugin)
			error = plugin->GetValue(aVariable, ret_value);
	}

	return error;
}

// function added to obtain the value of the 'base' tag defined in the HTML's <head> section
HBufC8* CFlashPluginProxy::GetHTMLBaseTagValueLC()
{
	NPError error;
	NPObject *windowScriptObject = NULL;
	HBufC8* htmlBaseTagURL8 = NULL;
	NPNetscapeFuncs* netscapeFunctions = static_cast<NPNetscapeFuncs*>(NpFuncs());	

	error = netscapeFunctions->getvalue(instance,NPNVWindowNPObject,&windowScriptObject);

	if (error != NPERR_NO_ERROR)
	{
		return NULL;
	}

	NPString npStr;
	NPVariant result;

	npStr.UTF8Characters = "document.getElementsByTagName('base')[0].getAttribute('href');";
	npStr.UTF8Length = strlen(npStr.UTF8Characters);

	bool isSuccessful = NPN_Evaluate(instance,windowScriptObject,&npStr, &result);

	if (isSuccessful && NPVARIANT_IS_STRING (result))
	{			
		htmlBaseTagURL8 = HBufC8::NewLC(NPVARIANT_TO_STRING(result).UTF8Length + 1);
		TPtr8 htmlUrl8Ptr = htmlBaseTagURL8->Des();
		htmlUrl8Ptr.Copy((TUint8*)(NPVARIANT_TO_STRING(result).UTF8Characters),NPVARIANT_TO_STRING(result).UTF8Length);
		htmlUrl8Ptr.ZeroTerminate();
	}

	return htmlBaseTagURL8;

}



HBufC8* CFlashPluginProxy::GetBaseUrlLC()
{
    NPError err;
    NPObject* window = NULL;
    HBufC8* htmlUrl = NULL;
    NPNetscapeFuncs* NetscapeFunctions = static_cast<NPNetscapeFuncs*>(NpFuncs());
    err = NetscapeFunctions->getvalue(instance, NPNVWindowNPObject, &window);
    if (err == NPERR_NO_ERROR) 
    {
        NPIdentifier locationId = NPN_GetStringIdentifier("location");
        NPVariant locationVar;
        /* ou1cimx1#473009 :
         * NPN_GetProperty() returns false on failure, or the pointer returned may be NULL
         * We need to accomodate for both cases.
		**/
        bool isSuccessful = true;
        isSuccessful = NPN_GetProperty(instance, window, locationId, &locationVar);
        if (isSuccessful && locationVar.value.objectValue)
        {
        	NPObject * locationObj = locationVar.value.objectValue;
        	locationId = NPN_GetStringIdentifier("href");
        	NPVariant hrefVar;
        	isSuccessful = NPN_GetProperty(instance, locationObj, locationId, &hrefVar);
        	if(isSuccessful && hrefVar.value.stringValue.UTF8Characters)
        	{
        		htmlUrl = HBufC8::NewLC(hrefVar.value.stringValue.UTF8Length + 1);
        		TPtr8 htmlUrlPtr = htmlUrl->Des();
        		htmlUrlPtr.Copy((TUint8*)(hrefVar.value.stringValue.UTF8Characters),hrefVar.value.stringValue.UTF8Length);
        		htmlUrlPtr.ZeroTerminate();
        	}
        	//Since NetscapeFuncs->getvalue() is not used to obtain NPObject:locationObj, plugin has to release it
        	NPN_ReleaseObject(locationObj);
        }
        
    }
    return htmlUrl;
}

void CFlashPluginProxy::SetFuncEnterVal(int aEnterexitval)
    {
        iFuncEnterExitVal = aEnterexitval;
    }
int CFlashPluginProxy::GetFuncEnterVal()
    {
        return iFuncEnterExitVal;
    }
void CFlashPluginProxy::SetDestroyVal(bool aDestroflag)
    {
        iSetDestroyFlag = aDestroflag;
    }
bool CFlashPluginProxy::GetDestroyVal()
    {
        return iSetDestroyFlag;
    }

// End of File
