/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Java Script Interaction Handler class implementation
 *
*/


#include <e32std.h>
#include "saplayer_build.h"
#if defined(__SYMBIAN32__) && !defined(EPOC)
#define EPOC
#endif
#include "npheaders.h"
#include "hostscripthandler.h"
#include "flashmethods.h"
#include "javascripthandler.h"
#include <libc/string.h>
#include "NPNFlashExtension.h"
#include "flashplgmain.h"
#include "mflashmethods.h"
#include "flashmethods.h"
#include <eikenv.h>
#include "flashpluginif.h"
#include "netscapeif.h"

// CONSTANTS
const int TOTAL_FLASH_METHODS = 31;
const int TOTAL_FLASH_PROPERTIES = 6;

//--------------------------------------------------------------------
//* Flash Methods
//* A Flash method is JavaScript function that is specific to Flash movies.
//* Use Flash methods to send JavaScript calls to Flash movies from 
//* a scripting environment. Each method has a name and most methods take arguments.
//* An argument specifies a value that the method operates upon.
//* The calculation performed by some methods returns a value that
//* can be used by the scripting environment
//-------------------------------------------------------------------------------

//Refer:http://www.adobe.com/support/flash/publishexport/scriptingwithflash/scriptingwithflash_04.html

//TODO:Keep this  in sorted order


//This mapping is depending on index for FlashMethodIdentifierNames
//any change need to modified here also
enum EFlashMethods
{
	EGET_VARIABLE = 0,
	EGOTO_FRAME,
	EIS_PLAYING,
	ELOAD_MOVIE,
	EPAN,
	EPERCENT_LOADED,
	EPLAY,
	EREWIND,
	ESET_VARIABLE,
	ESET_ZOOM_RECT,
	ESTOP_PLAY,
	ETOTAL_FRAMES,
	EZOOM,
	EFLASHVERSION, //enum to get flash version
	ETCALL_FRAME,
	ETCALL_LABEL,
	ETCURRENT_FRAME,
	ECURRENT_FRAME, 
	ETCURRENT_LABEL,
	ETGET_PROPERTY,
	ETGET_PROPERTY_AS_NUMBER,
	ETGOTO_FRAME,
	ETGOTO_LABEL,
	ETPLAY,
	ETSET_PROPERTY,
	ETSTOP_PLAY,
	EON_PROGRESS,
	EON_READY_STATE_CHANGE,
	EFS_COMMAND,
	ECALL_FUNCTION,
	ESETRETURNVALUE
};

//This mapping is depending on index for FlashPropertyIdentifierNames
//any change need to modified here also
enum EFlashProperties
{
	EALLOW_SCRIPT_ACCESS = 0,
	EBACKGROUND_COLOR,
	EBG_COLOR,
	EBASE_URL,
	ELOOP,
	EQUALITY
};

//prompts


/**
 ********************************************************************************
 Flash Java Script Interaction Handler API methods. 
 *********************************************************************************
 */

/**
 * FlashJavaScriptHandlerAllocate
 */
NPObject *FlashJavaScriptHandlerAllocate()
{
	FlashJavaScriptHandlerObject *newInstance =
			(FlashJavaScriptHandlerObject *) User::Alloc(
					sizeof(FlashJavaScriptHandlerObject));
	return (NPObject *) newInstance;
}

/**
 * FlashJavaScriptHandlerInvalidate
 */
void FlashJavaScriptHandlerInvalidate()
{

}

/**
 * FlashJavaScriptHandlerDeallocate
 */
void FlashJavaScriptHandlerDeallocate(FlashJavaScriptHandlerObject *obj)
{
	if ((obj) && ((obj->handler)))
		obj->handler->Deallocate();

	User::Free((void *) obj);
}

/**
 * FlashJavaScriptHandlerHasProperty
 */
bool FlashJavaScriptHandlerHasProperty(FlashJavaScriptHandlerObject *obj,
		NPIdentifier name)
{
	if ((obj) && ((obj->handler)))
		return obj->handler->HasProperty(name);

	return false;
}

/**
 * FlashJavaScriptHandlerHasMethod
 */
bool FlashJavaScriptHandlerHasMethod(FlashJavaScriptHandlerObject *obj,
		NPIdentifier name)
{
	if ((obj) && ((obj->handler)))
		return obj->handler->HasMethod(name);

	return false;
}

/**
 * FlashJavaScriptHandlerGetProperty
 */
void FlashJavaScriptHandlerGetProperty(FlashJavaScriptHandlerObject *obj,
		NPIdentifier name, NPVariant *variant)
{
	if ((obj) && ((obj->handler)))
		{
		TRAP_IGNORE(obj->handler->GetPropertyL(name, variant));
		}		
}

/**
 * FlashJavaScriptHandlerSetProperty
 */
void FlashJavaScriptHandlerSetProperty(FlashJavaScriptHandlerObject *obj,
		NPIdentifier name, NPVariant *variant)
{
	if ((obj) && ((obj->handler)))
	{
		TRAP_IGNORE(obj->handler->SetPropertyL(name,variant));
	}
}

/**
 * FlashJavaScriptHandlerInvokeDefaultFunctionPtr
 */
bool FlashJavaScriptHandlerInvokeDefaultFunctionPtr(
		FlashJavaScriptHandlerObject *obj, 
		const NPVariant *args,
		uint32_t argCount, 
		NPVariant *result)
{
	if ((obj) && ((obj->handler)))
		return obj->handler->InvokeDefaultMethod(args, argCount, result);

	return false;

}

/**
 * FlashJavaScriptHandlerInvokeFunctionPtr
 */
bool FlashJavaScriptHandlerInvokeFunctionPtr(
		FlashJavaScriptHandlerObject *obj,
		NPIdentifier name,
		const NPVariant *args,
		uint32_t argCount,
		NPVariant *result)
{
	if ((obj) && ((obj->handler)))
		{
		TRAPD(retVal,obj->handler->InvokeMethodL(name, args, argCount, result));
		if(retVal == KErrNone)
			return true;
		}

	return false;
}

/////////////////////////////////////////////////////////////////////////////////
//

/**
 *  CFlashJavaScriptHandler class
 *  Interfaces to Browser, used to pass information between browser and Flash plugin
 *  Used for Javascript and Flash interaction
 *  @since S60 v3.2
 */

// -------------------------------------------------------------------------
// CFlashJavaScriptHandler* CFlashJavaScriptHandler::NewL(CFlashPluginInstance* aBrowserInstance
//												,MHostScriptHandler* aHostScriptHandler
//      											)
// 
// --------------------------------------------------------------------------
//
CFlashJavaScriptHandler* CFlashJavaScriptHandler::NewL( NPP aInstance,MFlashPluginIf* aBrowserInstance,
														MHostScriptHandler* aHostScriptHandler,
														TPageAccessType aAccessType)
{
	CFlashJavaScriptHandler* self = new (ELeave) CFlashJavaScriptHandler(aInstance,
			aBrowserInstance, aHostScriptHandler, aAccessType);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

// -------------------------------------------------------------------------
// CFlashJavaScriptHandler::CFlashJavaScriptHandler(CFlashPluginInstance* aBrowserInstance
//												,MHostScriptHandler* aHostScriptHandler
//      											)
// 
// --------------------------------------------------------------------------
//

CFlashJavaScriptHandler::CFlashJavaScriptHandler(NPP aInstance,
		MFlashPluginIf* aBrowserInstance,
		MHostScriptHandler* aHostScriptHandler, 
		TPageAccessType aAccessType) :
	iBrowserInstance(aBrowserInstance)
	,iFlashMethods(NULL)
	,iHostScriptHandler(aHostScriptHandler)
	,iFlashJavaScriptMethodIdentifiers(NULL)
	,iFlashJavaScriptPropertyIdentifiers(NULL)
	,iAccessType(aAccessType)

{
	CGlueGlobals* globals = static_cast<CGlueGlobals*> (Dll::Tls());
	iNetscapeFuncs = static_cast<NPNetscapeFuncs*> (globals->NpFuncs());
	instance = aInstance;
}

// -------------------------------------------------------------------------
// void CFlashJavaScriptHandler::ConstructL()
// 
// --------------------------------------------------------------------------
//

void CFlashJavaScriptHandler::ConstructL()
{
	iHostScriptHandler->SetPluginScriptHandler(STATIC_CAST(MPluginScriptHandler*,this));

	iFlashMethods = CFlashMethods::NewL(iBrowserInstance,
			iHostScriptHandler->GetFlashMethods(),
			iHostScriptHandler->GetExternalInterface());

}

// -------------------------------------------------------------------------
// CFlashJavaScriptHandler::~CFlashJavaScriptHandler()
// 
// --------------------------------------------------------------------------
//

CFlashJavaScriptHandler::~CFlashJavaScriptHandler()
{
	if (iFlashMethods)
	{
		delete iFlashMethods;
		iFlashMethods = NULL;
	}

	ReleaseFlashMethodIdentifiers();
	ReleaseFlashProprtyIdentifiers();
}

// -------------------------------------------------------------------------
//void CFlashJavaScriptHandler::CreateFlashMethodIdentifiers ()
// 
// --------------------------------------------------------------------------
//

void CFlashJavaScriptHandler::CreateFlashMethodIdentifiersL()
{

	if (iFlashJavaScriptMethodIdentifiers != NULL)
		return;

	const NPUTF8 *FlashMethodIdentifierNames[TOTAL_FLASH_METHODS]
		= 
		{
			"GetVariable",
			"GotoFrame",
			"IsPlaying",
			"LoadMovie",
			"Pan",
			"PercentLoaded",
			"Play",
			"Rewind",
			"SetVariable",
			"SetZoomRect",
			"StopPlay",
			"TotalFrames",
			"Zoom",
			"FlashVersion",
			//TellTarget Methods
			"TCallFrame",
			"TCallLabel",
			"TCurrentFrame",
			"CurrentFrame",  
			"TCurrentLabel",
			"TGetProperty",
			"TGetPropertyAsNumber",
			"TGotoFrame",
			"TGotoLabel",
			"TPlay",
			"TSetProperty",
			"TStopPlay",
			//standard events
			"OnProgress",
			"OnReadyStateChange",
			"FSCommand",
			//External Interface
			"CallFunction",
			"SetReturnValue"
		};

	iFlashJavaScriptMethodIdentifiers = new (ELeave) 
						NPIdentifier[TOTAL_FLASH_METHODS];
						
    NPN_GetStringIdentifiers (FlashMethodIdentifierNames
    						, TOTAL_FLASH_METHODS
                              ,iFlashJavaScriptMethodIdentifiers);

}

// -------------------------------------------------------------------------
//void CFlashJavaScriptHandler::CreateFlashProprtyIdentifiers ()
// 
// --------------------------------------------------------------------------
//


void CFlashJavaScriptHandler::CreateFlashProprtyIdentifiersL()
{
	if (iFlashJavaScriptPropertyIdentifiers != NULL)
		return;

	const NPUTF8* FlashPropertyIdentifierNames[TOTAL_FLASH_PROPERTIES]
	= 
	{
		"AllowScriptAccess",
		"BackgroundColor",
		"BGColor",
		"Base",
		"Loop",
		"Quality",
	};

	iFlashJavaScriptPropertyIdentifiers = new (ELeave) 
						NPIdentifier[TOTAL_FLASH_PROPERTIES];


    NPN_GetStringIdentifiers (FlashPropertyIdentifierNames
    							, TOTAL_FLASH_PROPERTIES
                              , iFlashJavaScriptPropertyIdentifiers);                           
	
}


// -------------------------------------------------------------------------
//void CFlashJavaScriptHandler::ReleaseFlashMethodIdentifiers ()
// 
// --------------------------------------------------------------------------
//


void CFlashJavaScriptHandler::ReleaseFlashMethodIdentifiers()
{
	delete[] iFlashJavaScriptMethodIdentifiers;
}

// -------------------------------------------------------------------------
//void CFlashJavaScriptHandler::ReleaseFlashProprtyIdentifiers ()
// 
// --------------------------------------------------------------------------
//


void CFlashJavaScriptHandler::ReleaseFlashProprtyIdentifiers()
{
	delete[] iFlashJavaScriptPropertyIdentifiers;
}

// -------------------------------------------------------------------------
//void CFlashJavaScriptHandler::Deallocate ()
// 
// --------------------------------------------------------------------------
//

void CFlashJavaScriptHandler::Deallocate()
{

}

// -------------------------------------------------------------------------
//bool CFlashJavaScriptHandler::HasProperty( NPIdentifier /*name*/ )
// 
// --------------------------------------------------------------------------
//

bool CFlashJavaScriptHandler::HasProperty(NPIdentifier name)
{
	TRAPD( err,CreateFlashProprtyIdentifiersL());

	if (err != KErrNone)
	{
		return false;
	}

	TInt PropIdx = GetMethodIndex(  iFlashJavaScriptPropertyIdentifiers
									,TOTAL_FLASH_PROPERTIES	
									,name);

	if (PropIdx == -1)
	{
		return false;
	}

	return true;
}

// -------------------------------------------------------------------------
// void CFlashJavaScriptHandler::GetProperty( NPIdentifier /*name*/
//								, NPVariant */*variant*/ )
// 
// --------------------------------------------------------------------------
//

void CFlashJavaScriptHandler::GetPropertyL( NPIdentifier name
								, NPVariant *variant )
{
	TInt PropIdx = GetMethodIndex(iFlashJavaScriptPropertyIdentifiers
									,TOTAL_FLASH_PROPERTIES	
									,name);

	if (PropIdx == -1 || (iFlashMethods == NULL))
	{
		return;
	}

	switch (PropIdx)
	{
	case EALLOW_SCRIPT_ACCESS:
	{
		return iFlashMethods->GetPropAllowScriptAccess(variant);
	}

	case EBACKGROUND_COLOR:
	case EBG_COLOR:
	{
		return iFlashMethods->GetPropBGColorL(variant);
	}

	case EBASE_URL:
	{

	}
		break;

	case ELOOP:
	{

	}
		break;
	case EQUALITY:
	{

	}
		break;
	}

}

// -------------------------------------------------------------------------
// void CFlashJavaScriptHandler::SetPropertyL( NPIdentifier /*name*/
//						, NPVariant */*variant*/ )
// 
// --------------------------------------------------------------------------
//
void CFlashJavaScriptHandler::SetPropertyL( NPIdentifier name
						, NPVariant *variant )
{
	TInt PropIdx = GetMethodIndex(iFlashJavaScriptPropertyIdentifiers
									,TOTAL_FLASH_PROPERTIES	
									,name);

	if (PropIdx == -1 || (iFlashMethods == NULL))
	{
		return;
	}

	switch (PropIdx)
	{

	case EALLOW_SCRIPT_ACCESS:
	{
		return iFlashMethods->SetPropAllowScriptAccessL(variant);
	}

	case EBACKGROUND_COLOR:
	case EBG_COLOR:
	{
		return iFlashMethods->SetPropBGColorL(variant);
	}

	case EBASE_URL:
	{
		return iFlashMethods->SetPropBaseURL(variant);
	}
	case ELOOP:
	{

	}
		break;
	case EQUALITY:
	{

	}
		break;

	}
}

// -------------------------------------------------------------------------
// bool CFlashJavaScriptHandler::HasMethod( NPIdentifier name )
// 
// --------------------------------------------------------------------------
//
bool CFlashJavaScriptHandler::HasMethod(NPIdentifier name)
{

	TRAPD(err,CreateFlashMethodIdentifiersL());
	if (err != KErrNone)
	{
		return false;
	}

	TInt MethodIdx = GetMethodIndex( iFlashJavaScriptMethodIdentifiers
									,TOTAL_FLASH_METHODS	
									,name);

	if (MethodIdx == -1)
	{
		return false;
	}

	return true;
}

// -------------------------------------------------------------------------
// TInt CFlashJavaScriptHandler::GetMethodIndex(NPIdentifier name)
// 
// --------------------------------------------------------------------------
//
TInt CFlashJavaScriptHandler::GetMethodIndex(   const NPIdentifier* aTable
												, TInt aTableSize
												, NPIdentifier name)
{
	//TODO: This needs to change to bin search
	for (TInt i = 0; i < aTableSize; i++)
	{
		if (name == aTable[i])
		{
			return i;
		}
	}

	return -1;
}
// -------------------------------------------------------------------------
// bool CFlashJavaScriptHandler::InvokeMethod(NPIdentifier name, 
//										const NPVariant *args, 
//										uint32_t argCount, 
//										NPVariant *result)
// 
// --------------------------------------------------------------------------
//

bool CFlashJavaScriptHandler::InvokeMethodL(NPIdentifier name,
										const NPVariant *args, 
										uint32_t argCount, 
										NPVariant *result)
{
	TInt MethodIdx = GetMethodIndex(iFlashJavaScriptMethodIdentifiers
									,TOTAL_FLASH_METHODS	
									,name);

	if (MethodIdx == -1 || (iFlashMethods == NULL))
	{
		return false;
	}
	TBool success = EFalse;
	if ((MethodIdx >= 0) && (MethodIdx < TOTAL_FLASH_METHODS))
	{

		//handle calls here
		switch (MethodIdx)
		{
		case EGET_VARIABLE:
		{
			success = iFlashMethods->GetVariableL(args, argCount, result);
		}
			break;

		case EGOTO_FRAME:
		{
			success = iFlashMethods->GotoFrame(args, argCount, result);
		}
			break;

		case EIS_PLAYING:
		{
			success = iFlashMethods->IsPlaying(args, argCount, result);
		}
			break;

		case ELOAD_MOVIE:
		{
			success = iFlashMethods->LoadMovieL(args, argCount, result);
		}
			break;

		case EPAN:
		{
			success = iFlashMethods->Pan(args, argCount, result);
		}
			break;

		case EPERCENT_LOADED:
		{
			success = iFlashMethods->PercentLoaded(args, argCount, result);
		}
			break;
		case EPLAY:
		{
			success = iFlashMethods->Play(args, argCount, result);
		}
			break;

		case EREWIND:
		{
			success = iFlashMethods->Rewind(args, argCount, result);
		}
			break;

		case ESET_VARIABLE:
		{
			success = iFlashMethods->SetVariableL(args, argCount, result);
		}
			break;

		case ESET_ZOOM_RECT:
		{
			success = iFlashMethods->SetZoomRect(args, argCount, result);
		}
			break;
		case ESTOP_PLAY:
		{
			success = iFlashMethods->StopPlay(args, argCount, result);
		}
			break;

		case ETOTAL_FRAMES:
		{
			success = iFlashMethods->TotalFrames(args, argCount, result);
		}
			break;

		case EZOOM:
		{
			success = iFlashMethods->Zoom(args, argCount, result);
		}
			break;

		case ETCALL_FRAME:
		{
			success = iFlashMethods->TCallFrameL(args, argCount, result);
		}
			break;

		case ETCALL_LABEL:
		{
			success = iFlashMethods->TCallLabelL(args, argCount, result);
		}
			break;

		case ETCURRENT_FRAME:
		{
			success = iFlashMethods->TCurrentFrameL(args, argCount, result);
		}
			break;
        //newly added method RC ou1cimx1#661308
		case ECURRENT_FRAME:
        {
			
			success = iFlashMethods->CurrentFrame(args, argCount, result);
        }
		   break;
		
		case ETCURRENT_LABEL:
		{
			success = iFlashMethods->TCurrentLabelL(args, argCount, result);
		}
			break;

		case ETGET_PROPERTY:
		{
			success = iFlashMethods->TGetPropertyL(args, argCount, result);
		}
			break;

		case ETGET_PROPERTY_AS_NUMBER:
		{
				success = iFlashMethods->TGetPropertyAsNumberL(args,argCount,result);	
		}
			break;

		case ETGOTO_FRAME:
		{
			success = iFlashMethods->TGotoFrameL(args, argCount, result);
		}
			break;

		case ETGOTO_LABEL:
		{
			success = iFlashMethods->TGotoLabelL(args, argCount, result);
		}
			break;
		case ETPLAY:
		{
			success = iFlashMethods->TPlayL(args, argCount, result);
		}
			break;

		case ETSET_PROPERTY:
		{
			success = iFlashMethods->TSetPropertyL(args, argCount, result);
		}
			break;

		case ETSTOP_PLAY:
		{
			success = iFlashMethods->TStopPlayL(args, argCount, result);
		}
			break;
		case ECALL_FUNCTION:
		{
			success = iFlashMethods->CallMethodL(args, argCount, result);
		}
			break;
		case ESETRETURNVALUE:
		{
			success = iFlashMethods->SetReturnValueL(args, argCount, result);
		}
			break;
		case EFLASHVERSION: // enum to get flash version
		{
			success = iFlashMethods->FlashVersion(args, argCount, result);
		}
			break;	
		default:
		{
			success = EFalse;
		}
			break;

		}

	}

	return success;
}

// -------------------------------------------------------------------------
// bool CFlashJavaScriptHandler::InvokeDefaultMethod(const NPVariant */*args*/,
//									uint32_t /*argCount*/,
//									NPVariant */*result*/)
// 
// --------------------------------------------------------------------------
//
bool CFlashJavaScriptHandler::InvokeDefaultMethod(const NPVariant */*args*/,
									uint32_t /*argCount*/,
									NPVariant */*result*/)
{
	return false;
}

// -------------------------------------------------------------------------
// bool CFlashJavaScriptHandler::DeleteFlashMethods()
// To delete flashmethods when engine is destroyed
// --------------------------------------------------------------------------
//
void CFlashJavaScriptHandler::DeleteFlashMethods()
	{
	if (iFlashMethods)
		{
		delete iFlashMethods;
		iFlashMethods = NULL;
		}
	}


// -------------------------------------------------------------------------
// void CFlashJavaScriptHandler::SetPropertyL( NPIdentifier /*name*/
//						, NPVariant */*variant*/ )
// 
// --------------------------------------------------------------------------
//


// -------------------------------------------------------------------------
// bool CFlashJavaScriptHandler::InvokeCallback ( TDesC8& /*name*/, 
//										const NPVariant */*args*/, uint32_t /*argCount*/ )

// Used to call Javascript to browser side
// --------------------------------------------------------------------------
////standard events to be called from here
bool CFlashJavaScriptHandler::InvokeCallback(TDesC8& name,
										const NPVariant *args
										, uint32_t argCount )
{

	if (iNetscapeFuncs)
	{
		void *value = 0;
		NPError npErr = NPERR_GENERIC_ERROR;
		TRAP_IGNORE( npErr = iNetscapeFuncs->getvalue((NPP)instance,
						STATIC_CAST(NPNVariable,NPNVWindowNPObject)
						, (void *)&value));

		if (npErr == NPERR_NO_ERROR && value)
		{
			NPVariant resultVariant;
			//VOID_TO_NPVARIANT(resultVariant);

	            NPIdentifier ident = NPN_GetStringIdentifier ((const NPUTF8 *)(name.Ptr()));
	            

			TRAP_IGNORE(NPN_Invoke((NPP)instance,(NPObject*)value
							, ident
							, args
							, argCount
							, &resultVariant));
			
			return true;
		}
	}

	return EFalse;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//MPluginScriptHandler
#ifdef FEATURE_JAVASCRIPT_SUPPORT

// -------------------------------------------------------------------------
// int CFlashJavaScriptHandler::CallMethod(const char* aReq)

// Used to call Javascript to browser side
// --------------------------------------------------------------------------

int CFlashJavaScriptHandler::CallMethodL(const char* aReq)
{

	if (iNetscapeFuncs)
	{
		void *value = 0;

		NPError npErr = NPERR_GENERIC_ERROR;
		TRAP_IGNORE(npErr = iNetscapeFuncs->getvalue((NPP)instance,
						STATIC_CAST(NPNVariable,NPNVWindowNPObject)
						, (void *)&value));
		if (npErr == NPERR_NO_ERROR)
		{
			NPVariant resultVariant;
			VOID_TO_NPVARIANT(resultVariant);//TODO: check

			NPIdentifier ident = NPN_GetStringIdentifier(aReq);

			TRAP_IGNORE(NPN_Invoke((NPP)instance,(NPObject*)value
							, ident
							, NULL
							, 0
							, &resultVariant));
			

			return NPVARIANT_TO_INT32(resultVariant);
		}
	}

	return 0;
}

// -------------------------------------------------------------------------
// int CFlashJavaScriptHandler::EvaluateJavaScript(const char* aExpr)
//
// Used to call Javascript to browser side
// --------------------------------------------------------------------------

int CFlashJavaScriptHandler::EvaluateJavaScript(const char* aExpr)
{


    if (iNetscapeFuncs)
    {
        void *value = 0;

        NPError npErr = NPERR_GENERIC_ERROR;
        TRAP_IGNORE(npErr = iNetscapeFuncs->getvalue((NPP)instance,
                        STATIC_CAST(NPNVariable,NPNVWindowNPObject)
                        , (void *)&value));

        if (npErr == NPERR_NO_ERROR)
        {
            NPVariant resultVariant;
            VOID_TO_NPVARIANT(resultVariant);//TODO: check

            //NPString ident = NPN_GetStringIdentifier (aExpr);
            
            //Core player is not sending proper expression for AS3 addcallback request.
            //So if its as3 request, convert it to proper request and process it.
            //For example : Convert "__flash__addCallback(document.getElementById(\"extint\"), \"asF\");" to  "__flash__addCallback(document.extint, \"asF\");"
            //Incase of AS2, core player sends the proper request.
            
            _LIT8(laddCallback, "__flash__addCallback(");
            _LIT8(KDocument, "document.");
            _LIT8(KStringGetElementID, "getElementById(\"");
            _LIT8(KColon,":");

            TInt len = strlen(aExpr);
            TPtr8 aExprPtr((TUint8*)aExpr, len, len);
            
            NPVariant expression;
            BOOLEAN_TO_NPVARIANT( NULL, expression );
            
            //As a fix For eroom 994 ,check for colon is added.
            //If colon is present in id, then send the expression to browser as received by the core player.
            if(aExprPtr.Find(laddCallback) == 0 && aExprPtr.Find(KColon)==KErrNotFound )
            {
                const TInt KStringGetElementIDLen = 16;
                const TInt KClosingBraceLen = 2;
                const TInt KDocumentDotLen = 9;
                TInt copiedchars = 21;
                TInt pos1 = aExprPtr.Find(laddCallback);
                TInt pos2 = aExprPtr.Find(_L8("getElementById(\""));
                TInt pos3 = aExprPtr.Find(_L8("\"),"));
               
                HBufC8* tempBuf ;
                if(aExprPtr.Find(KDocument) == KErrNotFound )
                {
                    tempBuf = HBufC8::NewL(len+KDocumentDotLen+1);
                }
                else 
                {
                    tempBuf = HBufC8::NewL(len+1);
                    copiedchars = copiedchars + KDocumentDotLen;
                }
                TPtr8 tempBufPtr(tempBuf->Des());
                tempBufPtr.Copy(laddCallback);
                tempBufPtr.Append(KDocument);
                if (aExprPtr.Find(KStringGetElementID)!= KErrNotFound)
                {
                    //tempBufPtr.Copy(aExprPtr.Left(pos1));
                    tempBufPtr.Append(aExprPtr.Mid( pos2+KStringGetElementIDLen, pos3-(pos2+KStringGetElementIDLen) ));
                    tempBufPtr.Append(aExprPtr.Mid( pos3+KClosingBraceLen, len-(pos3+KClosingBraceLen) ));
                }
                else 
                {
                    tempBufPtr.Append(aExprPtr.Mid( pos1+copiedchars, (len-pos1-copiedchars)));
                }
                tempBufPtr.ZeroTerminate();
                char* lExpStr = (char*)tempBuf->Ptr();
                if(tempBuf)
                {
                    STRINGZ_TO_NPVARIANT( lExpStr, expression );
                }
                //CleanupStack::PopAndDestroy();
            }
            else 
                STRINGZ_TO_NPVARIANT( aExpr, expression );
            NPString expString = NPVARIANT_TO_STRING (expression);

            //TODO: need to add this  
              NPN_Evaluate((NPP)instance,(NPObject*)value
                                        , &expString
                                        , &resultVariant);
            //  CleanupStack::PopAndDestroy();
            return 1;
        }
    }

    return 0;
}

// -------------------------------------------------------------------------
// void* CFlashJavaScriptHandler::GetDOMObjectID()
//
// Used to call Javascript to browser side
// --------------------------------------------------------------------------
//need to find out whether its property
void* CFlashJavaScriptHandler::GetDOMObjectID()
{
	return NULL;
}
_LIT(KFSCommandSuffix , "_DoFSCommand");
// -------------------------------------------------------------------------
// void* CFlashJavaScriptHandler::DoFsCommand (TDesC8& aName
//       							,const char* aCmd
//       							,const char* aArgs )
//
// Used to call Javascript to browser side
// --------------------------------------------------------------------------

TInt CFlashJavaScriptHandler::DoFsCommandL(TDesC8& aName
       							,const char* aCmd
       							,const char* aArgs )
{
	HBufC8* jsMethod = HBufC8::NewLC(aName.Length() + 12 + 1);

	TPtr8 jsMethodPtr = jsMethod->Des();
	jsMethodPtr.Copy(aName);
	jsMethodPtr.Append(KFSCommandSuffix);
	jsMethodPtr.ZeroTerminate();

	NPVariant args[2];
	STRINGZ_TO_NPVARIANT(aCmd, args[0]);
	STRINGZ_TO_NPVARIANT(aArgs, args[1]);

	TInt retVal = InvokeCallback(*jsMethod, args, 2);

	CleanupStack::PopAndDestroy(jsMethod);

	return retVal;
}
#endif //FEATURE_JAVASCRIPT_SUPPORT
