/*
* Copyright (c) 2002-2004 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  browser API control of plugin
 *
*/

// INCLUDE FILES
#include "saplayer_build.h"

#include <eikenv.h>
#include <bautils.h> // NearestLanguage
#include <data_caging_path_literals.hrh>
#include <featmgr.h>
#include <defaultcaps.hrh>
#ifdef FEATURE_MANAGER_CHECK
#include <featmgr.h>
#include <flogger.h>
#endif // FEATURE_MANAGER_CHECK
#include "NPNFlashExtension.h"
#include "NPNFlashDefs.h" //parameter definitions
#include "flashplgmain.h"
#include "pluginarguments.h"
#include "flashpluginif.h"
#include "symbian_ui_interface.h"
// MACROS
_LIT(KNPModeEmbed, "NP_EMBED");
_LIT(KNPModeFull, "NP_FULL");
_LIT(KNPModeUndefined, "Mode undefined %d");

const TUint KModeTypeLength = 32;

// ================= MEMBER FUNCTIONS =======================

#ifdef NP_DEBUG

const TUint KMaxLogLength           = 512;
#include <flogger.h>

void NPPDebug(TUint8 aLogLevel,TRefByValue<const TDesC> aText, ... )
{

	if(aLogLevel < CUR_LOG_LEVEL)
	return;

	VA_LIST args;
	VA_START( args, aText );

	TBuf<KMaxLogLength> buf;
	buf.FormatList( aText, args );

#ifdef __FILE_MODE_LOGGING

	RFileLogger logger;

	TInt ret=logger.Connect();

	if (ret==KErrNone)
	{
		logger.SetDateAndTime( EFalse,EFalse );
		logger.CreateLog( _L("FlashLogs"), _L("NPLogFile"), EFileLoggingModeAppend );
		logger.Write(buf);
	}

	logger.Close();

#else

	RDebug::Print(buf);

#endif    

	VA_END( args );
}

#else

void NPPDebug(TUint8 /*aLogLevel*/, TRefByValue<const TDesC> /*aText*/, ...)
{
}

#endif

/*
 // ================= CPluginArgProcessor ==========================
 
 class CPluginArgProcessor : public CBase
 {
 public:
 static CPluginArgProcessor* NewL(CDesCArray* aArgn, CDesCArray* aArgv);
 
 CPluginArgProcessor(CDesCArray* aArgn, CDesCArray* aArgv);
 ~CPluginArgProcessor();
 
 public:
 void ProcessArgs();
 TInt GetInteractionMode();
 TPtrC& GetSwfSourcePath();
 TPtrC& GetBaseUrl();
 TPageAccessType GetAccessType();
 void DebugPrint();
 TBool GetAllowMobiFullScreen();	
 #ifdef	FEATURE_WINDOWLESS	
 EWindowMode GetWindowMode() const;
 #endif //FEATURE_WINDOWLESS
 
 private:
 CDesCArray* iArgn;
 CDesCArray* iArgv;
 TInt iInteractionMode;	
 TPtrC   iSwfSrcPath;
 TPtrC   iBaseUrl;
 TPageAccessType iAllowScriptAccess;
 TBool	iAllowMobiFullScreen;
 
 #ifdef  FEATURE_WINDOWLESS	
 EWindowMode eWindowMode;
 #endif //FEATURE_WINDOWLESS	
 };
 
 
 CPluginArgProcessor* CPluginArgProcessor::NewL(CDesCArray* aArgn, CDesCArray* aArgv)
 {
 CPluginArgProcessor* self = new (ELeave) CPluginArgProcessor(aArgn, aArgv);
 
 // NOTE: No II phase constructor.
 
 return self;
 }
 
 
 CPluginArgProcessor::CPluginArgProcessor(CDesCArray* aArgn,
 CDesCArray* aArgv)
 :iArgn(aArgn)
 ,iArgv(aArgv)
 ,iInteractionMode(NPDefaultMode)
 ,iSwfSrcPath(NULL, 0)
 ,iBaseUrl(NULL, 0)
 ,iAllowMobiFullScreen(ETrue)
 #ifdef FEATURE_WINDOWLESS
 ,eWindowMode(eWindow)
 #endif //FEATURE_WINDOWLESS					
 {						
 }
 
 CPluginArgProcessor::~CPluginArgProcessor()
 {
 }
 
 TBool CPluginArgProcessor::GetAllowMobiFullScreen()
 {
 return iAllowMobiFullScreen;
 }
 
 TInt CPluginArgProcessor::GetInteractionMode()
 {
 return iInteractionMode;
 }
 
 TPtrC& CPluginArgProcessor::GetSwfSourcePath()
 {
 return iSwfSrcPath;
 }
 
 TPtrC& CPluginArgProcessor::GetBaseUrl()
 {
 return iBaseUrl;
 }
 
 TPageAccessType CPluginArgProcessor::GetAccessType()
 {
 return iAllowScriptAccess;
 }
 
 #ifdef  FEATURE_WINDOWLESS
 EWindowMode CPluginArgProcessor::GetWindowMode() const
 {
 return eWindowMode;
 }
 #endif //FEATURE_WINDOWLESS
 
 void CPluginArgProcessor::ProcessArgs()
 {
 iInteractionMode = NPDefaultMode;
 
 if( !iArgn || !iArgv )
 {
 return;
 }
 
 for ( int i = 0; i != iArgn->Count ( ); i++ )
 {
 if (0 == (*iArgn)[i].CompareF(KFlashInteractionMode) )
 {
 if (0 == (*iArgv)[i].CompareF(KValueScreensaverMode))
 {
 iInteractionMode = NPScreenSaverMode;
 }
 
 if (0 == (*iArgv)[i].CompareF(KValueFlashAnimation))
 {
 iInteractionMode = NPAnimPluginMode;
 }
 }
 else if( 0 == (*iArgn)[i].CompareF(KFlashSwfSrc) || 
 0 == (*iArgn)[i].CompareF(KFlashSwfMovie))
 {
 iSwfSrcPath.Set((*iArgv)[i]);     
 }
 else if( 0 == (*iArgn)[i].CompareF(KFlashBase) )
 {
 iBaseUrl.Set((*iArgv)[i]);     
 }
 else if( 0 == (*iArgn)[i].CompareF(KSCriptAccess) )
 {
 if (0 == (*iArgv)[i].CompareF(KSCriptAccessAlways))
 {
 iAllowScriptAccess = EPageUrlNormal; 
 }
 else
 if (0 == (*iArgv)[i].CompareF(KSCriptAccessnever))
 {
 iAllowScriptAccess = EPageUrlUnknown_AllowOutboundScriptAccess; 
 }
 else
 if (0 == (*iArgv)[i].CompareF(KSCriptAccessSameDomain))
 {
 iAllowScriptAccess = EPageUrlUnknown_ProhibitOutboundScriptAccess; 
 }
 
 }
 else if(0 == (*iArgn)[i].CompareF(KFlashAllowMobiFullScreen) )
 {
 if ((0 == (*iArgv)[i].CompareF(KFalse)) || 
 (0 == (*iArgv)[i].CompareF(KNo)) ||
 (0 == (*iArgv)[i].CompareF(KOff)) )
 {
 iAllowMobiFullScreen = EFalse; 
 }
 
 }
 #ifdef FEATURE_WINDOWLESS    	
 else if(0 == (*iArgn)[i].CompareF(KWMode))
 {
 if (0 == (*iArgv)[i].CompareF(KWModeTransparent))
 {
 eWindowMode = eWindowless;
 }
 else if (0 == (*iArgv)[i].CompareF(KWModeOpaque))
 {
 eWindowMode = eWindowlessOpaque;     
 }            
 }
 #endif //FEATURE_WINDOWLESS    	
 }
 }
 
 void CPluginArgProcessor::DebugPrint()
 {
 if( !iArgn || !iArgv )
 {
 return;
 }
 
 for ( int i = 0; i != iArgn->Count ( ); i++ )
 {
 TPtrC aname = (*iArgn)[i];
 TPtrC avalue = (*iArgv)[i];
 
 if (aname.Length() + avalue.Length() <= KMaxLogLength - KPrintChar1)  
 {
 NPPDebug(LOG_DEBUG,_L("%S = %S"), &aname, &avalue);
 }
 else
 {
 
 #ifdef NP_DEBUG
 
 _LIT(KArgLog, "NPP_NewP() : Arguments too long!");
 NPPDebug(LOG_DEBUG,KArgLog);
 
 #endif
 }
 }
 }
 */


void WrapStreamData(NPAPIStreamData& aTarget, NPStream* aSource)
{
	if (aSource)
	{
		aTarget.end = aSource->end;
		aTarget.headers = aSource->headers;
		aTarget.lastmodified = aSource->lastmodified;
		aTarget.notifyData = aSource->notifyData;
		aTarget.url = aSource->url;
	}
}

// -------------------------------------------------------------------
// NPError my_newp(NPMIMEType /*pluginType*/, NPP instance, uint16 mode, 
//    CDesCArray* argn, CDesCArray* argv, NPSavedData* /*saved*/)
//
// Create a new instance of a plugin.
// -------------------------------------------------------------------
//  
NPError my_newp(NPMIMEType /*pluginType*/, NPP instance, uint16 mode,
		CDesCArray* argn, CDesCArray* argv, NPSavedData* /*saved*/)
{
#ifdef NP_DEBUG

	_LIT(KLogNPPNewP1, "===========================================================");
	_LIT(KLogNPPNewP2, "NPP_NewP() ++");
	_LIT(KLogNPPNewP3, "NPP_NewP(): NPERR_INVALID_INSTANCE_ERROR");
	_LIT(KLogNPPNewP4, "NPP_NewP(): NPERR_MODULE_LOAD_FAILED_ERROR");
	_LIT(KLogNPPNewP6, "NPP_NewP(): NPERR_OUT_OF_MEMORY_ERROR");
	_LIT(KLogNPPNewP5, "NPP_NewP() --");

	NPPDebug(LOG_DEBUG,KLogNPPNewP1);
	NPPDebug(LOG_DEBUG,KLogNPPNewP2);

#endif    

	if (!instance)
	{
#ifdef NP_DEBUG        
		NPPDebug(LOG_ERROR,KLogNPPNewP3);
#endif

		return NPERR_Invalid_Instance; //NPERR_INVALID_INSTANCE_ERROR;   
	}

	TBuf<KModeTypeLength> modeType;
	if (mode == NP_EMBED)
		modeType = KNPModeEmbed;
	else if (mode == NP_FULL)
		modeType = KNPModeFull;
	else
		modeType.Format(KNPModeUndefined, mode);

	//get interaction mode
	CPluginArgProcessor* argProcessor = NULL;
	TRAPD(arg_err, argProcessor = CPluginArgProcessor::NewL(argn,argv));

	if (arg_err != KErrNone)
	{
        instance->pdata = NULL;
		return NPERR_Module_Load_Failed; //NPERR_MODULE_LOAD_FAILED_ERROR;    
	}

	CFlashPluginProxy* player = NULL;
	TRAPD( error, player = CFlashPluginProxy::NewL(instance,argProcessor));
	if (error != KErrNone)
	    {
        instance->pdata = NULL;
	    return NPERR_Module_Load_Failed; //NPERR_MODULE_LOAD_FAILED_ERROR;    
	    }
    if(!player)
        {
        instance->pdata = NULL;
        return NPERR_Module_Load_Failed; //NPERR_MODULE_LOAD_FAILED_ERROR; 
        }
	//Set the initial values of func_enter_exit and set_destroy val
	player->SetFuncEnterVal(0);
	player->SetDestroyVal(EFalse);

	instance->pdata = player;
	return KErrNone; //NPERR_NO_ERROR;
}

// -------------------------------------------------------------------
// NPError my_destroy(const NPP instance, NPSavedData** /*save*/)
//
// Destroy a plugin.
// -------------------------------------------------------------------
//   
NPError my_destroy(const NPP instance, NPSavedData** /*save*/)
{

#ifdef NP_DEBUG

	_LIT(KLogNPPDestroy1, "NPP_Destroy() ++");
	_LIT(KLogNPPDestroy2, "NPP_Destroy() : NPERR_INVALID_INSTANCE_ERROR");
	_LIT(KLogNPPDestroy3, "NPP_Destroy() : Player is NULL");
	_LIT(KLogNPPDestroy4, "NPP_Destroy() : err != KErrNone");
	_LIT(KLogNPPDestroy5, "NPP_Destroy() --");

	NPPDebug(LOG_DEBUG,KLogNPPDestroy1);

#endif

	if (!instance)
	{
#ifdef NP_DEBUG
		NPPDebug(LOG_ERROR,KLogNPPDestroy2);
#endif

		return NPERR_Invalid_Instance; //NPERR_INVALID_INSTANCE_ERROR;
	}

    CFlashPluginProxy* player =
            static_cast<CFlashPluginProxy*> (instance->pdata);
    
    if(!player)
	{
#ifdef NP_DEBUG
      NPPDebug(LOG_ERROR,KLogNPPDestroy3);
#endif
      return NPERR_Generic; 
    }
    if (!player->GetFuncEnterVal())
        {
#ifdef NP_DEBUG
	NPPDebug(LOG_INFO,_L("NPP_Destroy() : NPP->pdata = [%x]"), player);
#endif

        if (player->GetDestroyVal())
            player->SetDestroyVal(EFalse);
        delete player;
        player = NULL;

#ifdef NP_DEBUG
			NPPDebug(LOG_DEBUG,KLogNPPDestroy5);
#endif    

        }
    else
        {
        player->SetDestroyVal(ETrue);
        }
    return KErrNone; //NPERR_NO_ERROR;

    }

	// -------------------------------------------------------------------
	// NPError my_setwindow(const NPP instance, const NPWindow* window)
	//
	// This is the parent window of a plugin.
	// -------------------------------------------------------------------
	//   

NPError my_setwindow(const NPP instance, const NPWindow* window)
{

#ifdef NP_DEBUG

	_LIT(KLogNPPSetWindow2, "NPP_SetWindow() : NPERR_INVALID_INSTANCE_ERROR");
	_LIT(KLogNPPSetWindow3, "NPP_SetWindow() : NPERR_GENERIC_ERROR");
	_LIT(KLogNPPSetWindow4, "NPP_SetWindow() : NPERR_INVALID_PARAM");
	_LIT(KLogNPPSetWindow5, "NPP_SetWindow() : KErrArgument! height <= 0 || width <= 0");
	_LIT(KLogNPPSetWindow6, "NPP_SetWindow() : NPERR_OUT_OF_MEMORY_ERROR");

#endif
	if (!instance)
	{
#ifdef NP_DEBUG
		NPPDebug(LOG_ERROR,KLogNPPSetWindow2);
#endif    

		return NPERR_Invalid_Instance; //NPERR_INVALID_INSTANCE_ERROR;   
	}

	CFlashPluginProxy* proxy = static_cast<CFlashPluginProxy*> (instance->pdata);
	if(!proxy)
	    {
        return NPERR_Generic; //NPERR_GENERIC_ERROR;   
	    }
	
	FunctionEnter(proxy);
	MFlashPluginIf* player = proxy->Plugin();

	if (!player)
	{
#ifdef NP_DEBUG
		NPPDebug(LOG_ERROR,KLogNPPSetWindow3);
#endif 

		FunctionExit(instance, proxy);
		return NPERR_Generic; //NPERR_GENERIC_ERROR;
	}

	if (!window)
	{
#ifdef NP_DEBUG
		NPPDebug(LOG_ERROR,KLogNPPSetWindow4);
#endif 

		FunctionExit(instance, proxy);
		return NPERR_Invalid_Param; //NPERR_INVALID_PARAM;   
	}
	TInt err = player->SetWindow(const_cast<NPWindow*> (window));

	if (err == KErrNoMemory)
	{
#ifdef NP_DEBUG
		NPPDebug(LOG_ERROR,KLogNPPSetWindow6);
#endif 

		FunctionExit(instance, proxy);
		return NPERR_Out_Of_Memory; //NPERR_OUT_OF_MEMORY_ERROR;
	}

	if (err != KErrNone)
	{
#ifdef NP_DEBUG
		NPPDebug(LOG_ERROR,KLogNPPSetWindow3);
#endif 

		FunctionExit(instance, proxy);
		return NPERR_Generic; //NPERR_GENERIC_ERROR;
	}

	FunctionExit(instance, proxy);
  return KErrNone; //NPERR_NO_ERROR;
}

// -------------------------------------------------------------------
// NPError my_newstream(const NPP instance, NPMIMEType /* type */,
// NPStream* stream, NPBool /*seekable*/, uint16* stype)
//
// A new data stream has been created for sending data to the plugin.
// param stream - A pointer to plugin specific data can be placed in 
// stream->pdata
// -------------------------------------------------------------------
//    
NPError my_newstream(const NPP instance, NPMIMEType type, NPStream* stream,
		NPBool seekable, uint16* stype)
{
#ifdef NP_DEBUG

	_LIT(KLogNPPNewStream1, "NPP_NewStream() ++");
	_LIT(KLogNPPNewStream2, "NPP_NewStream() : NPERR_INVALID_INSTANCE_ERROR");
	_LIT(KLogNPPNewStream4, "NPP_NewStream() : Player is NULL");
	_LIT(KLogNPPNewStream5, "NPP_NewStream() --");

	NPPDebug(LOG_DEBUG,KLogNPPNewStream1);

#endif

	if (!instance)
	{
#ifdef NP_DEBUG
		NPPDebug(LOG_ERROR,KLogNPPNewStream2);
#endif 

		return NPERR_Invalid_Instance; //NPERR_INVALID_INSTANCE_ERROR;   
	}

    CFlashPluginProxy* proxy = static_cast<CFlashPluginProxy*> (instance->pdata);
    if(!proxy)
        {
        return NPERR_Generic; //NPERR_GENERIC_ERROR
        }
    FunctionEnter(proxy);
	MFlashPluginIf* player = proxy->Plugin();
	NPPDebug(LOG_DEBUG,_L("NPP_NewStream() : Player = [%x]"), player);

	if (!player)
	{
#ifdef NP_DEBUG
			NPPDebug(LOG_ERROR,KLogNPPNewStream4);
#endif 

			FunctionExit(instance, proxy);
			return NPERR_Generic; //NPERR_GENERIC_ERROR;
		}

		NPAPIStreamData streamDataWrapper;
		WrapStreamData(streamDataWrapper,stream);

		FunctionExit(instance, proxy);
		return player->NewStream(type
				,&streamDataWrapper
				,seekable
				,stype);
	}

	// -------------------------------------------------------------------
	// NPError my_destroystream(const NPP aInstance, NPStream* aStream,
	// NPReason aReason)
	//
	// A data stream has been terminated.
	// -------------------------------------------------------------------
	//    
NPError my_destroystream(const NPP aInstance, NPStream* aStream,
		NPReason aReason)
{
#ifdef NP_DEBUG

	_LIT(KLogNPPDestroyStream1, "NPP_DestroyStream() ++");
	_LIT(KLogNPPDestroyStream2, "NPP_DestroyStream() : NPERR_INVALID_INSTANCE_ERROR");
	_LIT(KLogNPPDestroyStream3, "NPP_DestroyStream() : NPERR_GENERIC_ERROR");
	_LIT(KLogNPPDestroyStream4, "NPP_DestroyStream() --");

	NPPDebug(LOG_DEBUG,KLogNPPDestroyStream1);

#endif

	if (!aInstance)
	{
#ifdef NP_DEBUG
		NPPDebug(LOG_ERROR,KLogNPPDestroyStream2);
#endif  

		return NPERR_Invalid_Instance; //NPERR_INVALID_INSTANCE_ERROR;
	}

	//CFlashPluginInstance *player = static_cast<CFlashPluginInstance*>(aInstance->pdata);
    CFlashPluginProxy* proxy = static_cast<CFlashPluginProxy*> (aInstance->pdata);
    if(!proxy)
        {
        return NPERR_Invalid_Instance; //NPERR_INVALID_INSTANCE_ERROR;
        }
    FunctionEnter(proxy);
	MFlashPluginIf* player = NULL;
	player = proxy->Plugin();

	NPPDebug(LOG_DEBUG,_L("NPP_DestroyStream() : Player = [%x]"), player);

	// Need to destroy any plugin specific data...
			// delete stream->pdata;

			if (!player || !aStream)
			{
#ifdef NP_DEBUG
			NPPDebug(LOG_ERROR,KLogNPPDestroyStream3);
#endif  

			FunctionExit(aInstance, proxy);
			return NPERR_Generic; //NPERR_GENERIC_ERROR;
		}

		NPAPIStreamData streamDataWrapper;
		WrapStreamData(streamDataWrapper,aStream);
		int error = player->DestroyStream(&streamDataWrapper, aReason);

		// need to convert from symbian error codes to NP error codes
		if(error != KErrNone)
		{
			return NPERR_Generic;
		}

		FunctionExit(aInstance, proxy);
		return KErrNone;

	}

	// -------------------------------------------------------------------
	// void my_asfile(NPP aInstance, NPStream* aStream, const TDesC& aFname)
	//
	// A data stream has been fully saved to a file.
	// -------------------------------------------------------------------
	//    
void my_asfile(NPP aInstance, NPStream* aStream, const TDesC& aFname)
{
#ifdef NP_DEBUG

	_LIT(KLogNPPAsFile1, "NPP_StreamAsFile() ++");
	_LIT(KLogNPPAsFile2, "NPP_StreamAsFile() : NPERR_INVALID_INSTANCE_ERROR");
	_LIT(KLogNPPAsFile3, "NPP_StreamAsfile() : Player is NULL");
	_LIT(KLogNPPAsFile4, "NPP_StreamAsFile() : Filename too long!");
	_LIT(KLogNPPAsFile5, "NPP_StreamAsfile() : Stream is NULL");
	_LIT(KLogNPPAsFile6, "NPP_StreamAsFile() --");

	NPPDebug(LOG_DEBUG,KLogNPPAsFile1);

#endif

	if (!aInstance)
	{
#ifdef NP_DEBUG
		NPPDebug(LOG_ERROR,KLogNPPAsFile2);
#endif 

		return;
	}

	CFlashPluginProxy* proxy = static_cast<CFlashPluginProxy*> (aInstance->pdata);
	FunctionEnter(proxy);
	if(!proxy){
	FunctionExit(aInstance, proxy);
        return;
	}
  
	MFlashPluginIf* player = proxy->Plugin();

	NPPDebug(LOG_DEBUG,_L("NPP_StreamAsFile() : Player = [%x]"), player);

	if (!player)
	{
#ifdef NP_DEBUG
			NPPDebug(LOG_ERROR,KLogNPPAsFile3);
#endif 

			FunctionExit(aInstance, proxy);
			return;
	}

		NPAPIStreamData streamDataWrapper;
		WrapStreamData(streamDataWrapper,aStream);
		proxy->SetScriptAccess();
		player->AsFile(&streamDataWrapper,aFname);
		
		FunctionExit(aInstance, proxy);
}

	// -------------------------------------------------------------------
	// int32 my_writeready(const NPP aInstance, NPStream* aStream)
	//
	// Return the maximum number of bytes this plugin can accept from the
	// stream.
	// -------------------------------------------------------------------
	//   
int32 my_writeready(const NPP aInstance, NPStream* aStream)
{
	if (!aInstance)
	{
		return 0;
	}

    CFlashPluginProxy* proxy = static_cast<CFlashPluginProxy*> (aInstance->pdata);
    FunctionEnter(proxy);

	MFlashPluginIf* player = proxy->Plugin();

	if (!player || !aStream)
	{
#ifdef NP_DEBUG    
		_LIT(KLogNPPWriteReady1, "NPP_WriteReady() : (!player || !aStream) returning 0");
		NPPDebug(LOG_ERROR,KLogNPPWriteReady1);
#endif

		FunctionExit(aInstance, proxy);
		return 0;
	}

	NPAPIStreamData streamDataWrapper;
	WrapStreamData(streamDataWrapper, aStream);
	FunctionExit(aInstance, proxy);
	return player->WriteReady(&streamDataWrapper);
}

// -------------------------------------------------------------------------
// int32 my_write(const NPP aInstance, NPStream* aStream, int32 /*aOffset*/,
// int32 aLen, void* aBuffer)
//
// Receive more data 
// param buffer - contains the data.
// param len    - the number of bytes in buffer.
// param offset - the number of bytes already sent/processed.
// return TInt number of bytes processed.
// --------------------------------------------------------------------------
//   

int32 my_write(const NPP aInstance, NPStream* aStream, int32 aOffset,
		int32 aLen, void* aBuffer)
{
	if (!aInstance)
	{
		return 0;
	}

    CFlashPluginProxy* proxy = static_cast<CFlashPluginProxy*> (aInstance->pdata);
    FunctionEnter(proxy);

	MFlashPluginIf* player = proxy->Plugin();

	if (!player || !aStream) // AFRT-6HSJKJ -> remove "player->IsControlValid()" test
	{
		// "A negative return value causes an error on the stream, 
		// which will subsequently be destroyed via a call to NPP_DestroyStream"

#ifdef NP_DEBUG    
		_LIT(KLogNPPWrite1, "NPP_Write() : (!player || !aStream) return -1");
		NPPDebug(LOG_ERROR,KLogNPPWrite1);
#endif

		FunctionExit(aInstance, proxy);
		return KErrNotFound;
	}

	// buffer contains the data.
	// len is the number of bytes in buffer.
	// offset is the number of bytes already sent/processed.
	NPAPIStreamData streamDataWrapper;
	WrapStreamData(streamDataWrapper, aStream);
	 // If -1 (or any negative number) is returned here, the browser calls my_destroystream.
	 // This is acceptable if either URLStreamWriteL leaves or returns -1.
	FunctionExit(aInstance, proxy);
	return player->WriteStream(&streamDataWrapper, aOffset, aLen, aBuffer);
}

// -------------------------------------------------------------------------
// void my_print(NPP /*instance*/, NPPrint* /*platformPrint*/)
//
// Print
// --------------------------------------------------------------------------
//   
void my_print(NPP /*instance*/, NPPrint* /*platformPrint*/)
{
}

// -------------------------------------------------------------------------
// int16 my_event(NPP /*instance*/, void* /*event*/)
//
// Event
// --------------------------------------------------------------------------
//    
int16 my_event(const NPP aInstance, void* aEvent)
{
    CFlashPluginProxy* proxy = static_cast<CFlashPluginProxy*> (aInstance->pdata);
    FunctionEnter(proxy);
	MFlashPluginIf* player = proxy->Plugin();
	FunctionExit(aInstance, proxy);
	return player->HandlePluginEvent(aEvent);

	/*            
	 // NPP_HandleEvent - Delivers a platform-specific window event to the instance
	 // If the plug-in ignores the event, the function returns false.
	 int16 ret=0;
	 #ifdef GESTURE_LIBRARY_SUPPORTED
	 CFlashPluginInstance *player = static_cast<CFlashPluginInstance*>(aInstance->pdata);
	 if (!aInstance ||!player )
	 {
	 return ret;   
	 }      
	 NPEvent *event1 = static_cast<NPEvent*>(aEvent);
	 if(event1->event == ENppEventPointer)
	 {
	 
	 NPEventPointer *ev =static_cast<NPEventPointer*> (event1->param);
	 TRAP_IGNORE(ret=player->HandleGestureFromBrowserL(ev->reserved));
	 
	 }
	 #endif// GESTURE_LIBRARY_SUPPORTED
	 */
}

// -------------------------------------------------------------------------
// void my_urlnotify(const NPP instance, const TDesC8& /*url*/, 
// NPReason reason, void* notifyData)
//
// Notifies the plugin of the completion of a URL request.
// --------------------------------------------------------------------------
//    
void my_urlnotify(const NPP instance, const TDesC8& /*url*/, NPReason aReason,
		void* notifyData)
{
	if (!instance)
	{
		return;
	}

    CFlashPluginProxy* proxy = static_cast<CFlashPluginProxy*> (instance->pdata);
    FunctionEnter(proxy);

	MFlashPluginIf* player = proxy->Plugin();

	player->UrlNotify(aReason, notifyData);
	FunctionExit(instance, proxy);
}

// -------------------------------------------------------------------------
// NPError my_getvalue(NPP /*instance*/, NPPVariable /*variable*/, void* 
//    /*ret_value*/)
//
// Generic hook to retrieve values/attributes from the plugin.
// --------------------------------------------------------------------------
//    

NPError my_getvalue(NPP instance, NPPVariable variable, void* ret_value)

{
#ifdef NP_DEBUG

	_LIT(KLogNPPGetValue2, "NPP_GetValue() : NPERR_INVALID_INSTANCE_ERROR");
	_LIT(KLogNPPGetValue3, "NPP_GetValue() : NPERR_GENERIC_ERROR");

#endif
	if (!instance)
	{
#ifdef NP_DEBUG    
		NPPDebug(LOG_ERROR,KLogNPPGetValue2);
#endif 

		return NPERR_Invalid_Instance; //NPERR_INVALID_INSTANCE_ERROR;   
	}
   CFlashPluginProxy* proxy = static_cast<CFlashPluginProxy*> (instance->pdata);
   FunctionEnter(proxy);
	if (!proxy)
	{
	FunctionExit(instance, proxy);
		return NPERR_Invalid_Instance; //NPERR_INVALID_INSTANCE_ERROR;          
	}

	FunctionExit(instance, proxy);
	return proxy->GetValue(instance, variable, ret_value);
}

// -------------------------------------------------------------------------
// NPError my_setvalue(NPP /*instance*/, NPNVariable /*variable*/, 
//    void* /*ret_value*/)
//
// Generic hook to set values/attributes within the plugin.
// --------------------------------------------------------------------------
// 
NPError my_setvalue(NPP instance, NPNVariable variable, void* ret_value)
{
#ifdef NP_DEBUG

	_LIT(KLogNPPSetValue1, "NPP_SetValue() ++");
	_LIT(KLogNPPSetValue2, "NPP_SetValue() : NPERR_INVALID_INSTANCE_ERROR");
	_LIT(KLogNPPSetValue3, "NPP_SetValue() : NPERR_GENERIC_ERROR");
	_LIT(KLogNPPSetValue4, "NPP_SetValue() --");

	NPPDebug(LOG_DEBUG,KLogNPPSetValue1);

#endif

	if (!instance)
	{
#ifdef NP_DEBUG    
		NPPDebug(LOG_ERROR,KLogNPPSetValue2);
#endif

		return NPERR_Invalid_Instance; //NPERR_INVALID_INSTANCE_ERROR;
	}

    CFlashPluginProxy* proxy = static_cast<CFlashPluginProxy*> (instance->pdata);
    FunctionEnter(proxy);

	MFlashPluginIf* player = proxy->Plugin();
	if (!player)
	{
#ifdef NP_DEBUG    
		NPPDebug(LOG_ERROR,KLogNPPSetValue2);
#endif

		FunctionExit(instance, proxy);
		return NPERR_Invalid_Instance; //NPERR_INVALID_INSTANCE_ERROR;          
	}

	

	FunctionExit(instance, proxy);
	return player->SetValue(variable, ret_value);
}

void FunctionEnter(CFlashPluginProxy *proxy)
    {
    if(!proxy)
        return;
    int enter_exit_val = proxy->GetFuncEnterVal();
    enter_exit_val++;
    proxy->SetFuncEnterVal(enter_exit_val);
    }
void FunctionExit(NPP aInstance, CFlashPluginProxy *proxy)
    {
        if(!proxy)
            return;
        int enter_exit_val = proxy->GetFuncEnterVal();
        enter_exit_val--;
        proxy->SetFuncEnterVal(enter_exit_val);
        if(proxy->GetDestroyVal())
            my_destroy(aInstance, NULL);

    }

// End of File
