/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
*
*/


#include "saplayer_build.h"
#include "pluginargumentsimpl.h"
#include <NPNFlashExtension.h>
#include <NPNFlashDefs.h>

// MACROS

// CONSTANTS

CPluginArgProcessor* CPluginArgProcessor::NewL(CDesCArray* aArgn, CDesCArray* aArgv)
{
    CPluginArgProcessor* argProc = new (ELeave) CPluginArgProcessor(aArgn, aArgv);
	argProc->ProcessArgs(aArgn, aArgv);
	return argProc;
}

CPluginArgProcessor::CPluginArgProcessor(CDesCArray* aArgN, CDesCArray* aArgV)
                    :iArgN(aArgN)
                    ,iArgV(aArgV)
                    ,iInteractionMode(NPDefaultMode)
                    ,iSwfSrcPath(NULL, 0)
                    ,iBaseUrl(NULL, 0)
                    /*Setting Default value for "not specified" cases*/
                    ,iAllowScriptAccess(EPageUrlUnknown_ProhibitOutboundScriptAccess)
                    ,iAllowMobiFullScreen(ETrue)

#ifdef FEATURE_WINDOWLESS
			, iWindowMode(EWindow)
#endif                  
{
}

CPluginArgProcessor::~CPluginArgProcessor()
{
}

TBool CPluginArgProcessor::AllowMobiFullScreen()
{
	return iAllowMobiFullScreen;
}

InteractionMode CPluginArgProcessor::InteractionMode()
{
	return iInteractionMode;
}

const TDesC& CPluginArgProcessor::SwfSourcePath()
{
	return iSwfSrcPath;
}

const TDesC& CPluginArgProcessor::BaseUrl()
{
	return iBaseUrl;
}

TPageAccessType CPluginArgProcessor::AccessType()
{
	return iAllowScriptAccess;
}

TWindowMode CPluginArgProcessor::WindowMode()
{
	return iWindowMode;
}

void CPluginArgProcessor::ProcessArgs(CDesCArray* aArgn, CDesCArray* aArgv)
{
	iInteractionMode = NPDefaultMode;

	if (!aArgn || !aArgv)
	{
		return;
	}

	for (int i = 0; i != aArgn->Count(); i++)
	{
		if (0 == (*aArgn)[i].CompareF(KFlashInteractionMode))
		{
			if (0 == (*aArgv)[i].CompareF(KValueScreensaverMode))
			{
				iInteractionMode = NPScreenSaverMode;
			}

			if (0 == (*aArgv)[i].CompareF(KValueFlashAnimation))
			{
				iInteractionMode = NPAnimPluginMode;
			}
		}
        else if( 0 == (*aArgn)[i].CompareF(KFlashSwfSrc) || 
                        0 == (*aArgn)[i].CompareF(KFlashSwfMovie))
		{
			iSwfSrcPath.Set((*aArgv)[i]);
		}
		else if (0 == (*aArgn)[i].CompareF(KFlashBase))
		{
			iBaseUrl.Set((*aArgv)[i]);
		}
		else if (0 == (*aArgn)[i].CompareF(KSCriptAccess))
		{
			if (0 == (*aArgv)[i].CompareF(KSCriptAccessAlways))
			{
				iAllowScriptAccess = EPageUrlNormal;
			}
			else if (0 == (*aArgv)[i].CompareF(KSCriptAccessnever))
			{
				iAllowScriptAccess = EPageUrlUnknown_AllowOutboundScriptAccess;
			}
			else if (0 == (*aArgv)[i].CompareF(KSCriptAccessSameDomain))
			{
               iAllowScriptAccess = EPageUrlUnknown_ProhibitOutboundScriptAccess; 
			}

		}
		else if (0 == (*aArgn)[i].CompareF(KFlashAllowMobiFullScreen))
		{
            if ((0 == (*aArgv)[i].CompareF(KFalse)) || 
                    (0 == (*aArgv)[i].CompareF(KNo)) ||
                    (0 == (*aArgv)[i].CompareF(KOff)) )
			{
				iAllowMobiFullScreen = EFalse;
			}

		}
#ifdef  FEATURE_WINDOWLESS      
		else if (0 == (*aArgn)[i].CompareF(KWMode))
		{
			if (0 == (*aArgv)[i].CompareF(KWModeTransparent))
			{
				iWindowMode = EWindowless;
			}
			else if (0 == (*aArgv)[i].CompareF(KWModeOpaque))
			{
				iWindowMode = EWindowlessOpaque;
			}
		}
#endif      
	}
}

