/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  plugin control header file
*
*/


#ifndef __FLASHPLUGINCONTROL_H
#define __FLASHPLUGINCONTROL_H
#include "pointergesturedetector.h"
#include <sendui.h> 
//#include "flashpluginimecontrol.h"
#include "commondef.h"
#include "symbian_plugin_interface.h"
#include "flashplugincontrol.h"
#include "NPNFlashDefs.h"   // owned by flash
#include "saplayer_build.h"

#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR 
#include <aknpointereventsuppressor.h>
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR

#include <rt_gesturehelper.h>

#include <touchfeedback.h>

#ifdef __FEATURE_PLUGIN_UI_OPTIONS
#include <aknappui.h>
#endif

#ifdef SELF_IAD_UPDATE
#include "flash_iad_update_callback.h"
#endif

#include "pluginviewbase.h"

class CAknWsEventMonitor;

class MFlashEngine;
class CFlashPluginIMEControl;
class CFlashPluginControl;

#ifdef __FEATURE_PLUGIN_UI_OPTIONS
class CBrowserPluginFullScreenControl;
class CAsyncFullScreenHandler;
class CPluginMenuManager;
class CFlashVolumeControl;
#endif



#undef __DUMP_TO_FILE

class CSurfaceContainerControl;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// a control to contain the flash lite object
// cf. PlatformPlayer in flash 6 plugin / SAFlashPlayerControl in saplayer
//


class CBrowserPluginControl :  public CFlashPluginControl
							 , public MAknWsEventObserver							  
#ifndef GESTURE_LIBRARY_SUPPORTED							  
							 , MGestureEventObserver
#endif    
{
public:
	static CBrowserPluginControl* NewL(MFlashPluginInstance* aInstance);
    CBrowserPluginControl(MFlashPluginInstance* aInstance);
    virtual ~CBrowserPluginControl();
    void ConstructL();
public:	
    int SetWindow(NetscapeWindow* aWindow);
#ifdef GESTURE_LIBRARY_SUPPORTED
    virtual TBool HandleGestureFromBrowserL( void* aEvent);
#endif //GESTURE_LIBRARY_SUPPORTED

    virtual void PrepareToExitL();
	
public:
    TInt NotifyL(TNotificationType aCallType, void* aParam);
    void NotifyClientError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent);

    void RecreateBitmapL(const TRect& aRect);
	void ChangeFLVPlayStateL(EFLVStreamState aFLVStreamState, EMediaStreamType aMediaStreamType);
private:
    int SetWindow(MPluginAdapter* aPlugin, const TRect& aRect, const TRect& aClipRect);
    void DoSetWindowL(MPluginAdapter* aPlugin, const TRect& aRect, const TRect& aClipRect);
    void HandleFocusLostL() {}
#ifndef __FEATURE_PLUGIN_UI_OPTIONS 
private:
    void Draw(const TRect& /*aRect*/)  const;
#else
public:
	void Draw(const TRect& /*aRect*/)  const;
#endif

private:
	void SetRect(const TRect& aRect);

private:
    TBool QueryAllowDenyWithFormatResourceL(const TDesC& aQueryText,TInt aResId);
    HBufC * Format1L(TInt aFormatResource, const TDesC & aArg1);
    TBool QueryAllowNetAccessL();
    
    void HandlePluginActivated(void* aParam);
    void HandlePluginDeactivatedL(void* aParam);
    
    TBool HasCriticalErrorOccured();
#ifdef ENABLE_CURSOR	
	TBool IsNavigationKey(const TKeyEvent& aKeyEvent) const;
	TBool IsSelectKey(const TKeyEvent& aKeyEvent) const;
#endif

#ifdef __DUMP_TO_FILE
    void DumpToFile(TInt& aStreamId, const char* aData, unsigned long aDataLength);
#endif
	
	virtual void NotifyMp3FocusChangeL(TBool aFocus);
	void NotifyFocusChangeL(TBool aFocus);
#ifdef FEATURE_COLLECT_BITMAP
	void NotifyScrollL(TBool aFocus);
#endif
	void ResumeL();
    void PauseL();
	
public:
    
    void HandleWsEventL(const TWsEvent& aEvent,CCoeControl* aDestination);    
    void AddEventObserverToControl();
    void RemoveEventObserverFromControl();

public: // from MPluginView
    virtual TBool IsFLVVideo();
    virtual TRect GetSWFRect() { return Rect();}
    
public: //from MDisplay
    virtual TRect ApplicationRect();
    virtual TRect ClientRect();
    virtual TRect GetClipRect();
    virtual RWindow* GetWindow();
	virtual RWindow* GetNewVideoWindow(void* aClient);
	virtual void CloseVideoWindow(void* aClient);
	
    //virtual BitBuffer* BitBuffer();
#ifdef _BITBUFFER_ENABLED_     
    virtual CFbsBitGc* GetBitmapContext();
    virtual TInt Resize(TSize aSize);
    virtual CFbsBitmapDevice* GetBitmapDevice();  	
	virtual void CreateBitBuffer();
#endif
	
private:
    //TextInput*                  iTextInput;
    TBool 						iErrorIsCritical;  
    TInt 						iContentLength;
    EFLVStreamState				iFLVPlayState;
    TInt 						iJustActivatedIME;
    TBool						iTogglingScreen;
    TBool						iPreparingToExit;
    
	TBool                       iFlashScroll;
    TBool                       iBrowserScroll;
    TBool                       iCurrentPositionNotSet;
    TPoint                      iCurrentPositionRelativeToScreen;
    TBool                       iControlPresent;
    MPluginAdapter* 			iPluginAdapter;
#ifdef GESTURE_LIBRARY_SUPPORTED
    TPoint *                        iPtrBuffer;
    TBool                           iLongTap;
#else
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR  

    enum TTouchType
    { 
    ETouchStarted, 
    ETouchTap, 
    ETouchLongTap, 
    ETouchDrag 
    } iTouchType;
    
    TPoint iCorePosition;
    
    
    CAknPointerEventSuppressor* iSuppressor;
    
    TPoint ipreviousPoint;
    
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
    CGestureDetector            *iGestureDetector;
    TPoint                      iBrowserScrollOffset;
    TBool                       iFirstDrag;
    TBool                       iPreviousPointInvalid;
    TTime                       iPreviousPointInvalidTime;
    TTime                       iPreviousDownPointTime;
#endif // GESTURE_LIBRARY_SUPPORTED  

public:
    TRect GetRect();
    TRect GetClipRectFromScreenOrigin();
    void  Stop();

    void NotifyError(TInt aErrorCode);
    
    TBool QueryAllowDeviceIDL();
    TBool QueryAllowPhoneCallL(const TDesC& aNumberString);
	
    // from textInput
//    TBool IsTextInputBusy();
//    void GetInputTextL(TGetInputTextData& aData);

	void ShowVolumePopup(TInt currentVolume);
	
	void  SetVideoDisplayRectL(const TRect& aRect, const TRect& aClipRect);
	void UpdateVideoScreen();
	TBool IsFullScreenMode();

	#ifdef ENABLE_CURSOR
	// this function is used to update the cursor position in Full Screen
	void UpdateCursorPosition(TPoint aCursorpos);
	#endif // ENABLE_CURSOR

	void DrawDirtyRect(const TRect& aDirtyRect);
    int HandlePluginEvent(void* event);
	
	void SetScreenL(EScreenMode aMode);
	void SetFullScreenL(TBool aSetFullScreen);

	virtual void FLVideoStarted();
	//virtual TBool IsFLVVideo() const;
	void  HandleVideoError(const TInt& aError);
	
#ifdef __FEATURE_PLUGIN_UI_OPTIONS
public:
		CPluginMenuManager* GetMenuManager();
		
    	virtual MOptionMenuHandler* GetOptionHandler();
    	
    	CFlashVolumeControl* GetVolumeControl();
    	/*{
    		return iVolPopup;
    	}*/

private:
		
		CPluginMenuManager*		iMenuMgr;
		CFlashVolumeControl* 	iVolPopup;
private:
		void HandleFullScreenDelete();		
		
		TBool HandleURLReqForFullScreen(const TDesC& aUrl);
				
#endif //__FEATURE_PLUGIN_UI_OPTIONS
		
#ifdef FEATURE_COLLECT_BITMAP        
        CFbsBitmap* iLastRenderedBitmap;     
        TBool iSwfFirstPlay;
        
        //Share bitmap handle of last rendered frame with browser
        void SendLastRenderedBitmapToBrowser(const CFbsBitmap*  myBitmap);
#endif
        
#ifdef GESTURE_LIBRARY_SUPPORTED
        void SendPointerEventToBrowserL(const TPointerEvent &aPointerEvent);
        void ResetPointerBuffer();
#else
        void SendPointerEventToGestureDetectorL(const TPointerEvent &aPointerEvent);
        virtual void HandleGestureL(CGestureDetector::TGesture,const TPointerEvent &aPointerEvent);
#endif // GESTURE_LIBRARY_SUPPORTED

#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN

private:
		TBool iFullScreen;
		CAsyncFullScreenHandler* iFullScreenHandler;

		CBrowserPluginFullScreenControl* iFullScreenControl;
public:
	TBool GoFullScreenL();
	   	
	void ToggleScreenL();
		
	TBool GoNormalScreenL();
	
	TBool IsFullScreen();
        
    TBool IsPlayingVideo();
	
    CSurfaceContainerControl* GetVideoContainer();
    void SetVideoContainer(
#ifdef USE_VPU2    		
    		CSurfaceContainerControl*
#endif    		
    		);
#endif
    
    void LaunchVKbL();    
public: // from CCoeControl
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);			
	void  HandlePointerEventL(const TPointerEvent &aPointerEvent);	
	void HandleResourceChange(TInt aType);
	
	void HandPointerEventToImeL(const TPointerEvent& aPointerEvent);

#ifdef ENABLE_CURSOR

private:
    TPoint iSavedCursorPoint;

#endif //ENABLE_CURSOR


private:
    TInt 							iNumURLRequestCount;
    
    // Swf rectangles with absolute co-ordinate values (not relative to window origin) 
    TRect							iSwfRect;
    TRect							iSwfClipRect;
    // point stored for scrolling in browser page  
    TPoint ipreviousPosition;

public:


    TBool IsVideoRectEmpty();

public:
    void Redraw();
private:
#ifdef SELF_IAD_UPDATE
    //virtual void IADUpdateCallback( TInt aErrorCode );
    void ShowIADUpdateNotification();
    void DoShowIADUpdateNotificationL();
#endif //SELF_IAD_UPDATE

#ifdef USE_VPU2
private:
    //A coecontrol to create a dummy RWindow establish the z-order 
    //of egl and mmf surfaces.
    CSurfaceContainerControl*    iVideoSurfaceContainer;
#endif //SELF_IAD_UPDATE
};

class CAsyncFullScreenHandler : public CActive
{
public:
    CAsyncFullScreenHandler(CBrowserPluginControl* aControl);
                  
    ~CAsyncFullScreenHandler();
    
public:
    void RunL();
    void DoCancel();
    void IssueRequest(TBool aFullScreenRequest=ETrue);
    
private:
    CBrowserPluginControl*  iControl;
    TBool iFullScreenRequest;
};
#endif // __FLASHPLUGINCONTROL_H

// End of File
