/*
* Copyright (c) 2005-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  plugin control header file for full screen impl
*
*/


#ifndef __FLASHPLUGIN_FULLSCREEN_CONTROL_H
#define __FLASHPLUGIN_FULLSCREEN_CONTROL_H

#include <coecntrl.h>
#include <eikbtgpc.h>
#include <e32base.h>
#include <aknappui.h>
#include "saplayer_build.h"


#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR 
#include <aknpointereventsuppressor.h>
#include "flashlongtapdetector.h"	
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR

#include <touchfeedback.h>

class CBrowserPluginControl;
class MFlashEngine;
class CCbaHandler;
class CFullScreenMenuManager;

class CBrowserPluginFullScreenControl : public CCoeControl 
#ifndef GESTURE_LIBRARY_SUPPORTED
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR 
, public MFlashLongTapDetectorCallBack
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
#endif //ifndef GESTURE_LIBRARY_SUPPORTED
{
	public:  // Constructors and destructor

        /**
        * Two-phased constructor.
        */
        static CBrowserPluginFullScreenControl* NewL(CBrowserPluginControl* aControl
        											,MFlashEngine*           aEngine);

        /**
        * Destructor.
        */
        virtual ~CBrowserPluginFullScreenControl();
	
	public: //CCoecontrol
		 RWindow& Window() const;
		
		 TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
		 
		 void  HandlePointerEventL(const TPointerEvent &aPointerEvent);
		 
		 void HandleFullScreenCbaL();
		 void SizeChanged();
		 void CancelTimer();
	
		/**
    	* Callback for timer
    	*/
    	static TInt TimerCallbackL(TAny* aPtr);
    	
    	void DrawDirtyRect(const TRect& aRect); 
#ifdef GESTURE_LIBRARY_SUPPORTED
        TBool HandleGestureFromBrowserL( void* aEvent);
#else
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
    //Inherited from MFlashLongTapDetectorCallBack       
        void HandleFlashLongTapEventL( const TPoint& aPenEventLocation, 
                                      const TPoint& aPenEventScreenLocation );
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
#endif // GESTURE_LIBRARY_SUPPORTED   
	protected:
		void Draw(const TRect& /*aRect*/)  const;
			
	private:
#ifdef GESTURE_LIBRARY_SUPPORTED
	    void SendPointerEventToBrowserL(const TPointerEvent &aPointerEvent);
#else
	    void HandleFullScreenPointerEventL(const TPointerEvent &aPointerEvent);
#endif //GESTURE_LIBRARY_SUPPORTED
        /**
        * C++ default constructor.
        */
        CBrowserPluginFullScreenControl(CBrowserPluginControl* aControl
        								,MFlashEngine*           aEngine);

        /**
        * By default Symbian 2nd phase constructor is private.
        */
        void ConstructL();
        
        // changed for defect :ESJY-7PB9NN :Start
        // commented, not required
        //void  GoLandscapeL();   
        //void RestoreOrientationL();     
        // changed for defect :ESJY-7PB9NN :End
        
        void CreateCbaL();
        
        
        void DestroyCba();
        
        void InitFlashEngineL();
    private:
    
		// Rectangle that defines the control's extent
        TRect 						iRect;
        TRect 						iScreenRect;
        CBrowserPluginControl* 		iControl;
        MFlashEngine*           	iEngine;   	
        CCbaHandler*				iCbaHandler;
        CEikButtonGroupContainer* 	iFullScreenCba;
        CPeriodic*                  iTimer; // owned
        
        CAknAppUiBase::TAppUiOrientation iAppOrtn;
        
        //TBool						iStatusPaneHidden;//status pane flag
        //TBool   					iCbaPaneHidden;//cba pane status
        CCoeAppUi* 					iAppUi;
        CWindowGc*					iWindowGc;
        
        /*
	    * Flag which determines whether touch feature is supported or not.
	    */
	    TBool iTouchFeatureSupported;

	    /*
		 * Flag which determines whether tactile feedback is supported or not.
		 */
		TBool iControlPresent;
#ifndef GESTURE_LIBRARY_SUPPORTED        
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR  

    CFlashLongTapDetector* iFlashLongTapDetector;
    
    enum 
    { 
    ETouchStarted, 
    ETouchTap, 
    ETouchLongTap, 
    ETouchDrag 
    } iTouchType;
    
    TPoint iCorePosition;
    
    
    CAknPointerEventSuppressor* iSuppressor;
    
    TPoint ipreviousPoint;
    
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
#endif //GESTURE_LIBRARY_SUPPORTED    
};

class CCbaHandler : public CBase ,public MEikCommandObserver
	{
public:
	/**
        * Two-phased constructor.
        */
    static CCbaHandler* NewL(CBrowserPluginControl* aSwfControl
				,CBrowserPluginFullScreenControl* aAppContainer);
				
	virtual ~CCbaHandler();
        
	void ProcessCommandL(TInt aCommandId);
	
	TBool IsFullScreenMenuShowing();
	
	/**
        * By default Symbian 2nd phase constructor 
        */
    void ConstructL();
	
private:
	
	
    
    CCbaHandler(CBrowserPluginControl* aSwfControl
				,CBrowserPluginFullScreenControl* aAppContainer);
	
	
public:
	CBrowserPluginControl* iSwfControl;
	CBrowserPluginFullScreenControl* iAppContainer;
	CFullScreenMenuManager* iMenuHandler;
		
	};


#endif
//End of File
