/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CFlashPluginControl header file
*
*/

#ifndef __MFLASH_PLUGIN_CONTROL_H
#define __MFLASH_PLUGIN_CONTROL_H

#include <pluginadapterinterface.h>
#include "saplayer_build.h"
#include "flashimeclientif.h"
#include "symbian_plugin_interface.h"
#include "pluginbase.h"
//#include "flashpluginimecontrol.h"
//#include "FlashPlayerControl.h"
#include "mp3streamstateobserver.h"
#include <AknInfoPopupNoteController.h> 
#include <AvkonInternalCRKeys.h> // for KAknQwertyInputModeActive
#include <e32property.h> // for RProperty
#include "keypadhandler.h" // for CKeypadHandler

#ifdef ENABLE_CURSOR

#include "flashplugincursor.h"

#endif

#include "commontypes.h"
#include "flashpluginif.h"


// FORWARD DECLARATIONS
//class TextInput;
//class CFlashPluginInstance;
class MFlashEngine;
class RWindow;
class MFlashPlayerControl;
//class CFlashPluginIMEBaseControl;
class MMp3StateObserver;
class MInputTxtIf;
class CAknInfoPopupNoteController;

#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
#include <eikmobs.h>
class CAknStylusPopUpMenu;
#endif

#ifdef __FEATURE_PLUGIN_UI_OPTIONS
class MOptionMenuHandler;
#endif
/*enum TFlashPluginVisibilityState
{
	EFlashPluginVisible,
	EFlashPluginInvisible
};*/
// this enum has moved from FlashPlgControl to this class so that 
// other classes too can set plugin focus state. Fix for ASHA-7RKHZT.
/*enum TFlashPluginFocusState
{
    EFlashPluginBackground,
    EFlashPluginForeground  
};*/
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// a control to contain the flash lite object
// cf. PlatformPlayer in flash 6 plugin / SAFlashPlayerControl in saplayer
//
class CFlashPluginControl : public CCoeControl, 
                            public  PluginBase,
                            public MPluginNotifier
#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
                            ,public MEikMenuObserver
#endif
                           //public MFlashPlugin,
                            //public MFlashPlayerControl //Moved to MFlashPluginInstance
                            //public MMp3StateObserver,
                            //public MQwertyKeypadObserver //Moved to PluginBase
                            
                            
{
public:
    CFlashPluginControl(MFlashPluginInstance* aInstance);
    
    virtual ~CFlashPluginControl();
 
    virtual int SetWindow(NetscapeWindow* /*window*/) { return KErrNone;}
    /* Overridden in BrowserPlgControl */
    virtual TRect GetSWFRect() { return TRect(0,0,0,0);}
    void SetWindowL(MPluginAdapter* aPlugin
                                          , const TRect& aRect
                                          , const TRect& /*aClipRect*/);
protected: //call CreateWindowL from SetWindow    
    virtual void CreateWindowL(const CCoeControl* aParent, const TRect& aRect);
    
public:    
    virtual void OpenFileL(const TDesC& aFilename, const TDesC* aUrl, 
                           const TBool& aFirstMovie, CContent* aContent = NULL);
    
    virtual void RecreateBitmapL(const TRect& aRect);
    
#ifdef GESTURE_LIBRARY_SUPPORTED
    virtual void HandlePointerBufferReadyL();
    virtual TBool HandleGestureFromBrowserL(void* aEvent);
#endif //GESTURE_LIBRARY_SUPPORTED
    
    void ShowTipsL(TInt aResID);
	void HideTips();
	
	virtual void AddEventObserverToControl();
	
	virtual void RemoveEventObserverFromControl();
	
	virtual TBool HandleURLReqForFullScreen(const TDesC& /*aUrl*/) 
	    {
	    return EFalse; 
	    }
	
public: //MPluginNotifier
    virtual TInt NotifyL(TNotificationType aCallType, void* aParam)=0;

public: //CCoeControl

	virtual void Draw(const TRect& /*aRect*/)  const;
    
    virtual TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
    
    virtual void  HandlePointerEventL(const TPointerEvent &aPointerEvent);
    
    virtual void SizeChanged();	
     
public: //MFlashImeClientIf
	void* GetParentControl( ){return this ;}
    
public:	
    TBool IsVisible() const { return CCoeControl::IsVisible(); }
    
    virtual TBool IsFLVVideo();
	
    void SetPluginFocus(TFlashPluginFocusState aFocus);
    void SetControlVisible(TFlashPluginVisibilityState aVisible);
    virtual void NotifyFocusChangeL(TBool aFocus);
    TBool IsControlPresent(const TPoint& aPoint);
    
    virtual void ToggleScreenL() {}

#ifdef ENABLE_CURSOR
    
    CFbsBitGc* GetBitmapContext();

    void DrawCursor();
    
    void ClearCursor(const TRect& aRect);
    
    
#endif

	// This function is to get the present status(ON/OFF) of sliding keypad.
	//TInt GetQwertyKeyPadStatus() const { return PluginBase::GetQwertyKeyPadStatus(); }
	
#ifdef FEATURE_EXTENSION	
	void ResumeOnEventL(TBool aForeGround); //resume player if SAPI event recieved. 
	void PauseOnRequest(TBool aForeGround);
#endif	
	// This function will create an instance of RPoperty that will be attached to Qwerty Keypad
	void CreateQwertyKeypadObserverL(); //Moved to PluginBase

protected:
    virtual void ChangeFLVPlayStateL(EFLVStreamState /*aFLVStreamState*/, EMediaStreamType /*aMediaStreamType*/) {}
    
public:
#ifdef FEATURE_TESTAUTOMATION
    virtual TRect PositionRelativeToDeviceScreen() { return this->PositionRelativeToDeviceScreen(); }
#endif
    
protected:

    FlashEngineConfig       iConfig;    
    TFileName               iFilename;     
    //Not used
    //TBool                   iVersionNotChecked;    
    //TEventCode              iLastKeyEventCode;  
    TBool                   iPluginActivated;
   	//TFlashPluginVisibilityState		iVisibilityState;
   	// This var contains the plugin state
   	// this var has moved from FlashPlgControl to this class so that 
   	// other classes too can set plugin focus state. Fix for ASHA-7RKHZT.
   	//TFlashPluginFocusState  iFocusState;
   	// a flag which specify the status of IME: Enabled or Disabled
   	//TBool                   iIsIMEEnabled;
    CAknInfoPopupNoteController* 	iNote;
    TRect							iExtent;
    MPluginAdapter* iPlugin;
    
protected:
    
//  	#ifdef FEATURE_INLINE_TEXT_ENTRY
//  	  
//   	void SimulateDisableIMEEventL();
//	void SimulateEnableIMEEventL();
//
//    #endif //FEATURE_INLINE_TEXT_ENTRY

	// Not Used
	//virtual TBool QuerySharedObject() { return PluginBase::QuerySharedObject(); }
	
	//TBool QueryAllowDenyL(const TDesC& aQueryText);

public : 
    
    /**  Gets the player control's window, if the player view has been created.
      @return handle to the player control's window.
     */
    virtual RWindow* GetWindow() ;
   
	virtual RWindow* GetNewVideoWindow(void* /*aClient*/) {return NULL;}
	virtual void CloseVideoWindow(void* /*aClient*/) { }
	
    // from textInput

    /**
     Specifies if the text is being input.
     @return TBool ETrue if the user has not yet dismissed the text input
     dialog.
    */
    //virtual TBool IsTextInputBusy() ;
    //TKeyResponse HandleKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
    /**
     This function initiates a modal input text dialog, pausing the 
     movie until the user dismisses the dialog
     @param aData structure for the input text data.
     */
    //virtual void GetInputTextL(TGetInputTextData& /*aData */) ;
    
    
    virtual void  NotifyError(TInt /*aErrorCode*/, TBool aIsCritical);

    /**
        Gets the plugin control's extent 
        @return The rect of the plugin control
    */
    virtual TRect GetRect() ;

  /**
	 This function is called by the host engine when there is an error in
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/
	virtual void NotifyFlashError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue);
	/**
	 This function is called by the host engine when there is an error in
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/
	virtual void NotifyClientError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue);
	
	/**
	 This function is called by the host engine flash status
	 processing the flash content.
	 @param aFlashStatus The status code.
	 @param aParam specific to status code.
	 
	*/
	
	virtual void NotifyFlashStatus(TInt aFlashStatus, TAny* aParam) ;
	
#ifdef ENABLE_CURSOR
	
	virtual MFlashCursor* Cursor();
	
#endif

	virtual void DrawDirtyRect(const TRect& aDirtyRect);

    virtual int HandleEvent(NPEvent* /*event*/) { return KErrNone; }
	
	void SetScreenL(EScreenMode aMode);

	virtual void SetFullScreenL(TBool aSetFullScreen) ;

	
	virtual TBool IsFullScreenMode();
	
	virtual void SetRectRelativeToParentControl(const TRect& aRect);
	
	virtual void  HandleVideoError(const TInt& aError);
    
public: 
//#ifdef FEATURE_INLINE_TEXT_ENTRY
		
//	virtual void EnableIMEL();
//
//	virtual void DisableIME();
//
//	virtual TInt GetCurrentInputMode();
	
//	virtual void ResetIMEL();
	
//	virtual	void GetInputTextL(const TUint16* aInitialString, TUint aStrLen, 
//	                                TUint aMaxNumChars, TUint aTextFlags);

//	void SetCurrentInputLanguage(TLanguage aCurrentInputLanguage) 
//		
//	{	iCurrentInputLanguage = aCurrentInputLanguage; }	
//	
	//Moved to PLuginBase
	//TLanguage CurrentInputLanguage() 	{	return iCurrentInputLanguage ; }
	 	
//protected:
//	CFlashPluginIMEBaseControl*     iIMEControl;
//	//TLanguage                       iCurrentInputLanguage;
//	TBool                           iIsNoninline;
//#endif // FEATURE_INLINE_TEXT_ENTRY    	
	
	TRect							iRectRelativeToParentControl;
				
    // Flag, which specify whether touch support is there or not. 
    TBool iIsTouchFeatureSupported;
#ifdef __FEATURE_PLUGIN_UI_OPTIONS
public:
    	virtual MOptionMenuHandler* GetOptionHandler();
#endif


public:
	virtual void FLVideoStarted();
	virtual TBool IsPluginActivated();

public:	
	virtual void SetVideoDisplayRectL(const TRect& aRect, const TRect& aClipRect);
	
private:// from MQwertyKeypadObserver
	//This function is notified whenever sliding keypad status is changed to (ON/OFF)
    void HandleQwertyModeChangeNotification(TInt aQwertyMode); //Moved to PluginBase
    
#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
public:
    void DynInitContextSensitiveMenu(MEikMenuObserver* aObserver, const TPoint& aPoint);
    TBool IsContextMenuVisible() {return iContextMenuVisible;}
private:
    // from MEikMenuObserver
    virtual void SetEmphasis(CCoeControl* /*aMenuControl*/,TBool /*aEmphasis*/) {}
    virtual void ProcessCommandL(TInt aCommandId);
    
    CAknStylusPopUpMenu* iStylusPopupMenu;
    TBool iContextMenuVisible;
#endif

};

#endif // __FLASHPLUGINCONTROL_H

// End of File
