/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  plugin IME control header file
*
*/


#ifndef __FLASHPLUGINIMECONTROL_H
#define __FLASHPLUGINIMECONTROL_H

// INCLUDES
#include <coecntrl.h>
#include <fepbase.h>
#include <aknedsts.h> 

#include "flashimeclientif.h"

#include "browserplugintimer_object.h"
// CONSTANTS
const TUint KT9SpellSoftkeyCommandId = 16385;

// FORWARD DECLARATIONS
class MFlashEngine;

#ifdef FEATURE_INLINE_TEXT_ENTRY

#include "amp_mtextnavigation.h"


//This class decides which input mode the text field should be in. 
//The decision is based on initial string and current input language if avaiable. 

class InputModeSelector : public CBase 
{
public:

	static InputModeSelector* NewL(AMP_MTextNavigation* pNav, 
	        MFlashImeClientIf* aPluginControl);
	InputModeSelector(AMP_MTextNavigation* pNav, MFlashImeClientIf* aPluginControl);
																 
	
	
	void ConstructL(AMP_MTextNavigation* pNav);
	TBool IsNoninline();
	~InputModeSelector(); 
	
private:

	//Returns true if the initial string is in complex language.
	TBool ContainsComplexLangCharacters(FI_UTF16Char * string, long length);

	//Returns true if the current input language is complex. 	
	TBool IsInComplexLanguageMode();
	
private:
	FI_UTF16Char *mpInitialString;
	int          mLen;

	MFlashImeClientIf* iPluginControl;	

};

// Based on decision taken from InputModeSelector,
// this class is responsible for creating IME or FEP.

class CFlashPluginIMEBaseControl : public CCoeControl 
{
public:
	virtual ~CFlashPluginIMEBaseControl() 
	{	
	}
	
	virtual void  ConstructL()  =0;
	virtual TBool IsInLine()    =0;	
	virtual void  CommitConversion(TBool aConversionCanceled)=0;
	virtual void ReportStateEventL() = 0;
	virtual void LaunchVKbL()=0;

};

class CFlashPluginIMENonInlineControl : public CFlashPluginIMEBaseControl
{
public:

    static CFlashPluginIMENonInlineControl* NewL(MFlashEngine* aEngine,
            MFlashImeClientIf* aPluginControl
								      /*,TPoint aPoint*/);
    
	CFlashPluginIMENonInlineControl(MFlashEngine* aEngine,
	        MFlashImeClientIf* aPluginControl
						            /*,TPoint aPoint*/);
	void ConstructL();	
	
	virtual ~CFlashPluginIMENonInlineControl()
	{
	    // Do-nothing
	}
	
	void CommitConversion(TBool /*aConversionCanceled*/) 
	{
	    // Do-nothing
	}

	TBool IsInLine();

	void LaunchVKbL()
    {
        //Do -nothing
    }

	void ReportStateEventL()
	{
		
	}
	
	MFlashEngine*           iEngine;

	MFlashImeClientIf*    iPluginControl;

};

class CFlashPluginIMEControl : public  CFlashPluginIMEBaseControl, 
                               public  MCoeFepAwareTextEditor, 
                               private MCoeFepAwareTextEditor_Extension1,
                               public MPlgAsyncHandler
{
public:

    static CFlashPluginIMEControl* NewL(MFlashEngine* aEngine,
            MFlashImeClientIf* aPluginControl
									   /*,TPoint aPoint*/);
    
    CFlashPluginIMEControl(MFlashEngine* aEngine, 
            MFlashImeClientIf* aPluginControl
                           /*,TPoint aPoint*/);
 


	virtual ~CFlashPluginIMEControl();

	MCoeFepAwareTextEditor_Extension1* Extension1();
	
	CAknEdwinState* FepState() { return iFepState; }
	
	void EnableIMEL();
	
	void DisableIME();
	
  	// Set and Get methods for current input language.
	TLanguage CurrentInputLanguage() 	
	{	return iCurrentInputLanguage ; }
	
	void SetCurrentInputLanguage(TLanguage aCurrentInputLanguage) 	
	{	iCurrentInputLanguage = aCurrentInputLanguage; }	

public: // From CFlashPluginIMEBaseControl

    void  ConstructL();
	TBool IsInLine();	
	void  CommitConversion(TBool aConversionCanceled);
	void  ReportStateEventL();
	void  LaunchVKbL();
	
	void  RemoveCba(TUint aButtonPos);
public://from CCoeControl
	void HandlePointerEventL(const TPointerEvent &aPointerEvent);
public: // from MPlgAsyncHandler
    virtual void DoRunL(const CActive* /*aActive*/);
    virtual TInt DoRunError(const CActive* /*aActive*/, TInt aError);
protected:

     void SetFepStateInputModesToTextFieldFlags(CAknEdwinState *state, 
                                                 unsigned long flags);
	
private:

	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
	

	virtual MCoeFepAwareTextEditor_Extension1* Extension1(TBool& aSetToTrue);

private:  // From CCoeControl
	
	// Implement the InputCapabilities interface of CCoeControl
	// so that we can override the FEP interface
	
	virtual TCoeInputCapabilities InputCapabilities() const;
	void    FocusChanged(TDrawNow aDrawNow);

private:  // From MCoeFepAwareTextEditor

	void StartFepInlineEditL(const TDesC& aInitialInlineText, 
	                         TInt aPositionOfInsertionPointInInlineText, 
	                         TBool aCursorVisibility, 
	                         const MFormCustomDraw* aCustomDraw, 
	                         MFepInlineTextFormatRetriever& aInlineTextFormatRetriever, 
	                         MFepPointerEventHandlerDuringInlineEdit& 
	                         aPointerEventHandlerDuringInlineEdit);
	                         
	void StartFepInlineEditL(TBool& aSetToTrue, 
	                         const TCursorSelection& aCursorSelection,
	                         const TDesC& aInitialInlineText, 
	                         TInt aPositionOfInsertionPointInInlineText, 
	                         TBool aCursorVisibility, 
	                         const MFormCustomDraw* aCustomDraw, 
	                         MFepInlineTextFormatRetriever& aInlineTextFormatRetriever, 
	                         MFepPointerEventHandlerDuringInlineEdit& 
	                         aPointerEventHandlerDuringInlineEdit);
	                         
	void UpdateFepInlineTextL(const TDesC& aNewInlineText, 
	                          TInt aPositionOfInsertionPointInInlineText);
	                          
	void SetInlineEditingCursorVisibilityL(TBool aCursorVisibility);
	void CancelFepInlineEdit();
	TInt DocumentLengthForFep() const;
	TInt DocumentMaximumLengthForFep() const;
	void SetCursorSelectionForFepL(const TCursorSelection& aCursorSelection);
	void GetCursorSelectionForFep(TCursorSelection& aCursorSelection) const;
	
	void GetEditorContentForFep(TDes& aEditorContent, 
	                            TInt aDocumentPosition, 
	                            TInt aLengthToRetrieve) const;
	                            
	void GetFormatForFep(TCharFormat& aFormat, TInt aDocumentPosition) const;
	
	void GetScreenCoordinatesForFepL(TPoint& aLeftSideOfBaseLine, 
	                                 TInt& aHeight, 
	                                 TInt& aAscent, 
	                                 TInt aDocumentPosition) const;
	                                 
	void DoCommitFepInlineEditL();
	void MCoeFepAwareTextEditor_Reserved_2();

private: // From MCoeFepAwareTextEditor_Extension1

	void SetStateTransferingOwnershipL(CState* aState, TUid aTypeSafetyUid);
	CState* State(TUid aTypeSafetyUid);
	void SetCursorType(TBool& aSetToTrue, const TTextCursor& aTextCursor);
	void MCoeFepAwareTextEditor_Extension1_Reserved_3();
	void MCoeFepAwareTextEditor_Extension1_Reserved_4();

private:
    void CreateStateL();
    TBool IsLangChanged();

protected:	// specific

	TUid            iStateFepUid;
	CAknEdwinState* iFepState;

	TBool           iIsCommitting;

	// State for an inline editing session
	HBufC*		    iCompositionStringBuf;		// A buffer to hold the current composition string
	TInt		    iCompositionStringBufSize;	// The size of the composition string buffer, in bytes
	TInt		    iCompositionStringLen;		// The number of characters in the composition string

	TInt		    iCompositionStringCursorPos;	// Character offset in the composition string of the cursor
	TBool		    iCursorVisibility;				// ETrue for visible text cursor, EFalse for invisible
												    // text cursor in the text editor.
private:

	MFlashEngine*       iEngine;
	TPoint              iPoint;
	TBool               iPresentOnStack;

	// When calling SetConversionString with embedded font
	// the call sometimes fail to insert all the chars because of missing glyphs.
	// T9 crashes on this. The following variable is made to make T9 to believe
	// we inserted the chars we didn't. This is unfortunately hacky but seems to work.
   	TInt iCursorPosAdd;
   	
   	// Store the retriever referece
	MFepInlineTextFormatRetriever* iTextFormatRetriever;
	
	MFepPointerEventHandlerDuringInlineEdit* iFepPointerEventHandler;

	MFlashImeClientIf* iPluginControl;

    TLanguage   iCurrentInputLanguage;
    TBool       iIsPencilKeyPressed;
    //This  is required for VKB  In multitap mode(Secret text editor mode)  Fep manager sends EKeyF20 to delete previous key we need to know if we have consumed previous character
    TInt iCharsConsumed;

    // Flag, which specify whether touch support is there or not. 
    TBool iIsTouchFeatureSupported;

    // a timer object to launch VKB asynchronously, after notification from IME control.
    CPlgTimerObject* iVKBActivateTimer;

#ifdef PREDECTIVE_TEXT_SUPPORT

    TBool iIsSpelling;

	// This is related to the same issue as above.
	TBool iMonitorConversionStringLengthForZi;
	
	// Some FEPs may set too large a selection when embedded font is used and glyphs are missing.
	// Please see above.
	TInt iLastConversionStringCharactersInserted;
	
	// Fix for Chinese phone's multitap mode. They seem to use
	// StartFepInlineEditL function's version that has the initial selection.
	// Other T9 devices use that only after spell. To fix some weird issues
	// with Chinese phones' multitap mode disable that function if no spelling occurs.
	TBool iLetCallSecondTime;

#endif // PREDECTIVE_TEXT_SUPPORT

	// Store the retriever referece
};

#endif // FEATURE_INLINE_TEXT_ENTRY

#endif // __FLASHPLUGINIMECONTROL_H
