/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  plugin control header file
*
*/

#ifndef __FLASH_SCREENSAVER_CONTROL_H
#define __FLASH_SCREENSAVER_CONTROL_H

#include "flashplugincontrol.h"

// FORWARD DECLARATIONS
class CFlashPluginInstance;
class CFlashPluginControl;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// a control to contain the flash lite object
// cf. PlatformPlayer in flash 6 plugin / SAFlashPlayerControl in saplayer
//
class CScreensaverPluginControl : public CFlashPluginControl
{
public:

	static CScreensaverPluginControl* NewL(CFlashPluginInstance* aInstance);	
    virtual ~CScreensaverPluginControl();
    TBool IsSoundPlaying() const;
    
private:
    CScreensaverPluginControl(CFlashPluginInstance* aInstance);
    void  Construct();    

public: //MPluginNotifier
    TInt NotifyL(TNotificationType aCallType, void* aParam);

private:
    TKeyResponse OfferKeyEventL(const TKeyEvent& /*aKeyEvent*/, TEventCode /*aType*/);
    void HandlePointerEventL(const TPointerEvent &aPointerEvent);
    
public: 
	/*virtual void Start() ;
	virtual void End()	;*/
    	  
public : //MFlashPlugin
    
	/**
      Displays a confirmation query that the flash content is
    attempting to access the phone's serial number.
      @return ETrue if the user allows the access.
    */
    //virtual TBool QueryAllowDeviceIDL() ;      

      /**
       Specifies if the plugin is running in screensaver mode or not
       @return ETrue if the plugin is the screensaver mode else EFalse
     */
    //virtual TBool IsScreenSaverActive() ;

	/**
     Notify the plugin of any run time error which occurs 
     @param aErrorCode The error code corresponding to the error which has occured
    */
    virtual void  NotifyError(TInt aErrorCode);

    /**
	 This function is called by the host engine when there is an error in
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/
	virtual void NotifyFlashError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue);

	/**
	 This function is called by the host engine when there is an error in
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/
	virtual void NotifyClientError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue);
	
	//Moved to CScreenSaverPluginInstance
	//virtual TUint GetFeatureCapability();
};

#endif // __FLASHPLUGINCONTROL_H

// End of File
