/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash Lite full screen menu handler
*
*/


#ifndef FULL_SCREEN_MENU_MANAGER_H
#define FULL_SCREEN_MENU_MANAGER_H

//Include
#include <eikmobs.h> //MEikMenuObserver
#include <eikmenub.h>

//Foward declaration
class CBrowserPluginControl;

/**
 *  Class of management of menu launched by plugin
 *
 *  @since S60 v3.1
 */
class CFullScreenMenuManager: public CBase, public MEikMenuObserver
    {
public:
    /**
     * Symbian constructor.
     *
     * @since S60 v3.1
     * @return Pointer to created CFullScreenMenuManager object
     */
    static CFullScreenMenuManager* NewL(CBrowserPluginControl* aCtrl);
   
    /**
     * Destructor.
     */
    virtual ~CFullScreenMenuManager();
    
    

public:
    /**
     * Show speicfied menu pane
     * 
     * @since S60 v3.1
     * @param aResourceId Menu resource id.
     * @return
     */
    void ShowL(TInt aResourceId);

    /**
     * Get menu show status
     * 
     * @since S60 v3.1
     * @return ETrue if show, otherwise EFalse.
     */
    TBool IsShowing();
    
    /**
     * Hide current showed menu.
     * 
     * @since S60 v3.1
     * @return None.
     */
    void Hide();
    
    /**
     * Get current menu resource id.
     * 
     * @since S60 v3.1
     * @return Resource id.
     */
    TInt MenuPaneId();

public: 
    // MEikMenuObserver
    /**
     * Called when menu get or lose focus.
     * @since S60 v3.1
     * @param aMenuControl The menu control
     * @param aEmphasis ETrue to emphasize the menu, EFalse otherwise
     * @return None.
     */
    virtual void SetEmphasis(CCoeControl* aMenuControl, TBool aEmphasis);
    
    /**
     * Process command user selects.
     * @since S60 v3.1
     * @param 
     * @return None.
     */
    virtual void ProcessCommandL(TInt aCommandId);
    
    /**
     * Init Menu items.
     * @since S60 v3.1
     * @param aResourceId   Id of the resource
     * @param aMenuPane   	MenuPane Object
     * @return None.
     */
    virtual void DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane);
    
    /**
     * Init Menu bar.
     * @since S60 v3.1
     * @param aResourceId   Id of the resource
     * @param aMenuBar   	MenuBar Object
     * @return None.
     */
    virtual void DynInitMenuBarL(TInt aResourceId,CEikMenuBar* aMenuBar);
    

private:
    /**
     * C++ default constructor.
     *
     * @param aCtrl CBrowserPluginControl object.
     */
    CFullScreenMenuManager(CBrowserPluginControl* aCtrl);
    
    /**
        * By default Symbian 2nd phase constructor is private.
        */
    void ConstructL();
    
    void AddMuteOptionMenu(CEikMenuPane* aMenuPane);
    
    void ProcessVolumeCommandL(TInt aCommandId);

private:
    /**
     * Current menu bar. Own.
     */
    CEikMenuBar*    iOptionMenuBar;
    /**
     * Browser Plugin Control Not own.
     */
    CBrowserPluginControl* iCtrl;
    
    TBool		iOnStack;
    
    };

#endif //FULL_SCREEN_MENU_MANAGER_H

// End of file
