/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CFlashPluginInstance header file
*
*/


#ifndef PLUGINBASE_H_
#define PLUGINBASE_H_

#include <e32def.h>
#include <e32std.h>
#include <e32lang.h>
#include <e32cmn.h>
#include <badesca.h>
#include <AknWsEventObserver.h>

#include "saplayer_build.h"
#include "symbian_textinput.h"

#include "mp3streamstateobserver.h"
#include "uinotifierif.h"
#include "screenfurnitureif.h"
#include "flashvideoif.h"
#include "flash_iad_update_callback.h"
#include "flashpluginimecontrol.h"

#include "keypadhandler.h"
#include "commontypes.h"

#ifdef _BITBUFFER_ENABLED_
#include "Flashbitbuffer.h"
#endif

#include "pluginview.h"
class MTouchFeedback;
class MFlashPluginInstance;

class CFlashIconHandler;
class CFlashCursor;
class CEikonEnv;
class MFlashEngine;

enum TFlashPluginPlayState
{
    EFlashPluginInitial,
    EFlashPluginPlaying,
    EFlashPluginPaused,
    EFlashPluginPausedWhileScrolling
};
#ifdef FEATURE_COLLECT_BITMAP
enum TFlashPluginScrollState
{
	EScrollingNone,
	EScrollingStarted,
	EScrollingStopped
};
#endif
/*
// this enum has moved from browserplugincontrol to this class so that 
// other classes too can set plugin focus state. Fix for ASHA-7RKHZT.
enum TFlashPluginFocusState
{
    EFlashPluginBackground,
    EFlashPluginForeground  
};

enum EFLVStreamState
{
    EFlashPluginNoFLVStream
    ,EFlashPluginFLVStreamInProgress
    ,EFlashPluginFLVStreamClosed
};



enum TFlashPluginVisibilityState
{
    EFlashPluginVisible,
    EFlashPluginInvisible
};
*/

class PluginBase : public MPluginView                    
                   ,public MMp3StateObserver
                   ,public MQwertyKeypadObserver
                   ,public MFlashVideoClientIf
#ifdef FEATURE_INLINE_TEXT_ENTRY                   
                   ,public MFlashImeClientIf
#endif 
    {  
public:
    virtual ~PluginBase();    

public: //MPluginView    
    virtual int SetWindow(void* /*window*/) { return KErrNone; }    
    virtual void PrepareToExit() { TRAP_IGNORE( this->PrepareToExitL() ); }    
    virtual HBufC* GetFlashBaseURL(){return NULL;};
    virtual void Close() { delete this; }
    
    virtual int OpenFile(const TDesC& aFilename, const TDesC& aUrl, 
                                           TBool aFirstMovie, CContent* aContent=NULL) 
        { 
        TRAPD(err, this->OpenFileL(aFilename, &aUrl, aFirstMovie, aContent )); 
        return err;
        }
    
    //void SetRect(const TRect& aRect) =0;
    virtual TRect GetClipRect() { return this->iClipRect; }
    
    //TODO : Should be part of MDisplay
    //virtual void DrawDirtyRect(const TRect& aRect) =0;
    
    virtual void MakeVisible(TBool /*aVisible*/) {} 
    
    //virtual TBool IsFileOpened() =0;    
    //virtual TBool IsValid() =0;
    //virtual TBool IsVisible() =0;
    //virtual TBool IsConstructed() =0;
    
    //virtual void NotifyClientError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent) =0;
    
    virtual void SetPluginFocus(TFlashPluginFocusState /*aFocus*/) {}
    
    virtual int HandlePluginEvent(void* /*event*/) { return KErrNone;/*return this->HandleEvent(event);*/ }
    
    virtual TBool HandleURLReqForFullScreen(const TDesC& /*aUrl*/) { return EFalse; }
    
    //virtual void ChangeFLVPlayStateL(EFLVStreamState aFLVStreamState, EMediaStreamType aMediaStreamType) =0;
    
    //virtual int HandleEvent(void*);
    
    //virtual int GetLastError() =0;
    //virtual void SetErrorOccured() =0;
    
    virtual MMp3StateObserver* GetMP3StateObserver()
    {
        return this;
    }
    
    virtual MInputTxtIf* GetInputTextIf()
    {
        return this;
    }
    
    virtual MFlashVideoClientIf* GetFlvClientIf()
    {
        return this;
    }
  //  virtual TBool QueryAllowNetworkInWidgetMode();
    
public:    
    MFlashPluginInstance* FlashPluginInstance() const;
    MFlashEngine* GetPlayerEngine() const;
    
    virtual void OpenFileL(const TDesC& /*aFilename*/, const TDesC* /*aUrl*/, 
                                       const TBool& /*aFirstMovie*/, CContent* /*aContent*/ = NULL) =0;
    
    virtual void DrawDirtyRect(const TRect& /*aRect*/) =0;
    
    //Override to handle key/pointer/ notification events
    //from the container [browser]
    //virtual int HandleEvent(void* /*event*/) { return KErrNone; }
    
    virtual TBool IsConstructed();
    virtual TBool IsInitialized();
    virtual TBool IsValid();    
    virtual TBool IsFileOpened();
    
    //Intended for derived classes to override
    virtual TBool IsVisible() { return (iVisibilityState==EFlashPluginVisible); }
    virtual TBool IsControlVisible() const;
    
    virtual void SetErrorOccured(TInt aError=KErrNone);
    TInt GetLastError();
    virtual void PrepareToExitL() {}
	virtual void ShowVolumePopup(TInt /*aCurrentVolume*/) { };
    
public: //From MUserNotiferIf
    //Not Used
    //virtual TBool QuerySharedObject() { return EFalse; }    
    virtual TBool QueryAllowDeviceIDL() { return EFalse; }
    virtual TBool QueryAllowNetAccessL() { return EFalse; }
    virtual TBool QueryAllowPhoneCallL(const TDesC& /*aNumberString*/) { return EFalse; }
    virtual void  NotifyError(TInt /*aErrorCode*/, TBool /*aIsCritical*/) { return; }
    virtual void NotifyFlashError(TInt /*aErrorCode*/, TBool /*aIsCritical*/,
            TBool /*aPersistent*/ = ETrue, ErrorInfo* /*aErroInfo*/ = NULL) { return; }
    virtual void NotifyClientError(TInt /*aErrorCode*/, TBool /*aIsCritical*/,
            TBool /*aPersistent*/ = ETrue) { return ; }
    virtual void NotifyFlashStatus(TInt /*aFlashStatus*/, TAny* /*aParam*/) { return; }

    virtual void NotifyDataAccessError(const char* /*aUrl*/, TInt /*aErrorCode*/){ return; }

    virtual TBool QueryAllowDenyL(const TDesC& aQueryText);
    virtual TBool QueryTrustDomainL(const TDesC& aDomain);
    
    TBool QueryAllowDenyWithFormatResourceL(const TDesC& aQueryText,
            TInt aResId);
    static HBufC* FormatL(TInt aFormatResource, const TDesC& aArg);

public: //from MInputTextIf 
    virtual TBool IsTextInputBusy();
    virtual void GetInputTextL(TGetInputTextData& aData);

//#ifdef FEATURE_INLINE_TEXT_ENTRY
//    virtual void EnableIMEL() { return; }
//    virtual void DisableIME() { return; }
//    virtual TInt GetCurrentInputMode() { return 0; }
//#endif  // FEATURE_INLINE_TEXT_ENTRY

public: //from MDisplay

#ifdef FEATURE_EXTENSION
    virtual void SetAutoRotationModeL(TBool /*aMode*/) {} //enable or disable Sensor autoroatation mode
#endif
    virtual void SetFullScreenL(TBool /*aMode*/=EFalse) {}
    virtual void SetScreenL(EScreenMode /*aMode*/) { }
    virtual TRect ApplicationRect() { TRect r; return r;}
    virtual TRect ClientRect() { TRect r; return r;}
    virtual void SetAppScreenL() {}
    virtual void ResetScreenInCurrentModeL() {}
    virtual void SetSoftkeysL(HBufC** /* aSoftkeyArray */, const TInt /* aSoftkeyArraySize*/) {}
    virtual void ResetSoftkeysL() {}
    //virtual void SetFullScreenL(TBool aMode=EFalse) = 0;
    virtual TBool IsFullScreenMode() { return EFalse; }
    //virtual TRect GetClipRect();
    
    //from MPluginView
    virtual void SetClipRect(const TRect& aClipRect);    
    virtual RWindow* GetWindow() {  return NULL; }
    
    virtual RWindow* GetNewVideoWindow(void* aClient);
    virtual void CloseVideoWindow(void* aClient);    
    
#ifdef __FEATURE_OSK__
    virtual void SetOSKIsActive(TBool /*aShow*/) {} //Do nothing
#endif
    
    virtual void ZoomShowAll() {} //Do nothing
    
    //virtual TInt GetQwertyKeyPadStatus() const;
    void CreateQwertyKeypadObserverL();
    
public: //MQwertyKeypadObserver    
    void HandleQwertyModeChangeNotification(TInt aQwertyMode);
    
public: //MFlvideoClientIf    
    virtual void HandleVideoError(const TInt& /*aError*/) {}
    virtual void FLVideoStarted(){}
    virtual void FLVideoFinished() {}
    virtual void VideoIsEmbedded(TBool /*aEmbeddedVideo*/) { }
    
    virtual void ChangeFLVPlayStateL(EFLVStreamState /*aFLVStreamState*/, EMediaStreamType /*aMediaStreamType*/) {}

public: //MMp3StateObserver
    //mp3 streaming
    virtual void NotifyMp3FocusChangeL(TBool aFocus);
       
    //TODO : Move to FlashPluginInstance as a static helper function..
    static int LoadPluginResourceL(const TDesC& aFileName);
    
#ifdef FEATURE_INLINE_TEXT_ENTRY    
public: //MFlashImeClientIf
    virtual TLanguage CurrentInputLanguage() 	{	return iCurrentInputLanguage ; }
    virtual void GetInputTextL(const TUint16* aInitialString, TUint aStrLen, 
                                        TUint aMaxNumChars, TUint aTextFlags);
    
    virtual TKeyResponse HandleKeyEventL(const TKeyEvent& aKeyEvent, 
                                                     TEventCode aType);
    virtual void ResetIMEL();
    virtual void* GetParentControl() { return NULL; }
    
public:
    virtual void ProvideFeedbackL(const TPointerEvent& aPointerEvent);

	virtual void EnableIMEL();

	virtual void DisableIME();

	virtual TInt GetCurrentInputMode() { return 0; }
	
	virtual void SetCurrentInputLanguage(TLanguage aCurrentInputLanguage) 
	{	
		iCurrentInputLanguage = aCurrentInputLanguage; 
	}	
	
    virtual void AddEventObserverToControl()
    {
    	
    }
 
    virtual void RemoveEventObserverFromControl()
    {
    	
    }    
    
public:
#ifdef FEATURE_TESTAUTOMATION
    virtual TRect PositionRelativeToDeviceScreen() { return TRect();}
#endif
    
protected:

	virtual void SimulateDisableIMEEventL();
	virtual void SimulateEnableIMEEventL();
#endif //FEATURE_INLINE_TEXT_ENTRY
	
    virtual TInt GetQwertyKeyPadStatus();
    
#ifdef _BITBUFFER_ENABLED_    
    virtual void CreateBitBuffer() { } //Do Nothing
    virtual BitmapBuffer* BitBuffer() =0;
    virtual CFbsBitGc*  GetBitmapContext() =0;
    virtual TInt Resize(TSize aSize) 
        {
        return KErrNone;
        }
    
    virtual CFbsBitmapDevice* GetBitmapDevice() =0;
#endif
    
protected:
    
#ifdef _BITBUFFER_ENABLED_     
    CFbsBitmap* iBitmap;
    CFbsBitGc* iBitmapGc;
    CFbsBitmapDevice* iBitmapDevice;
    CFlashBitmap* iBitBuffer;
#endif


protected:
    PluginBase(MFlashPluginInstance* aPluginInstance=NULL);
    void ConstructL();    
    virtual void Redraw(){}
    virtual void MP3PlayStateChangedL(EMP3StreamState aMP3StreamState);

protected:    
    TRect 						iClipRect;
    
    MTouchFeedback* 			iFeedback;
    TBool 						iTactileFeedbackSupported;
    
    MFlashPluginInstance* 		iPlayerInstance;    
    CFlashIconHandler* 			iIconHandler;
        
    TBool 						iCanGoFullScrn;
    TBool 						iConstructed;
    TInt  						iLastError;
    TBool 						iValid;
    TBool 						iFileOpened;
    
    EMP3StreamState 			iMP3StreamState;     
    TFlashPluginVisibilityState iVisibilityState;
    TFlashPluginFocusState 		iFocusState;
    TFlashPluginPlayState 		iPlayState;    
    #ifdef FEATURE_COLLECT_BITMAP
    TFlashPluginScrollState     iPluginScrollState;
    #endif
    TBool 						iErrorOccured;
    CAknWsEventMonitor*         iEventMonitor;
    
#ifdef ENABLE_CURSOR
    CFlashCursor* 				iCursor;
#endif

    CKeypadHandler* 			iQwertyModeStatusSubscriber;
    RProperty 					iQwertyModeStatusProperty;
    TInt 						iQwertyMode;
    TEventCode              	iLastKeyEventCode;    

#ifdef FEATURE_INLINE_TEXT_ENTRY    
protected:
	CFlashPluginIMEBaseControl* iIMEControl;
	TLanguage                   iCurrentInputLanguage;
	TBool                       iIsNoninline;
#endif    
   	TBool                   	iIsIMEEnabled;
    TextInput*                  iTextInput;  

    MFlashEngine* iEngine;
 
private:
	void CreateTextInputL();
};
#endif /* PLUGINBASE_H_ */
