/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description: 
*       Defines  CPluginMenuManager methods.
*
*
*/



#ifndef __PLUG_IN_MENU_MGR_H__
#define __PLUG_IN_MENU_MGR_H__

class MOptionMenuHandler;
class CBrowserPluginControl;
class CBase;

class CPluginMenuManager : 	  public CBase
					   		, public MOptionMenuHandler
{
	public:
    /**
     * Symbian constructor.
     *
     * @since S60 v3.1
     * @return Pointer to created CPluginMenuManager object
     */
    static CPluginMenuManager* NewL(CBrowserPluginControl* aCtrl);
   
    /**
     * Destructor.
     */
    virtual ~CPluginMenuManager();
    
    private:
    /**
     * C++ default constructor.
     *
     * @param aCtrl CBrowserPluginControl object.
     */
    CPluginMenuManager(CBrowserPluginControl* aCtrl);
    
    /**
        * By default Symbian 2nd phase constructor is private.
        */
    void ConstructL();
    
    
    public: //	MOptionMenuHandler

		/**
        * AddPluginOptionMenuItemsL
        * Add the Option Menu items to the menuPane, these will be handled by plugin.
		* @since S60 v3.1
        * @param aMenuPane handle of the menu that will contain the options 
        * @param aCommandBase base id that needs to be added to the command ID 
        * @param aAfter Plugin options should be added after this index
        * @return void
        */
		void AddPluginOptionMenuItemsL( CEikMenuPane& aMenuPane, TInt aCommandBase,
		                                TInt aAfter );

		 /**
        * HandlePluginCommandL
        * Handle the user command from the option menu to the plugin.
		* @since S60 v3.1
        * @param aCommand the command ID of the option
        * @return void
        */
		void HandlePluginCommandL( TInt aCommand );
	
	public:
	
		TBool IsMuted();
		
		void MuteVolume();
		
		void UnmuteVolume();
		
		
	private:
		
		void InsertOneMenuItemL( CEikMenuPane& aMenuPane, TInt aCommand,
                                   TInt aResourceId, TInt index );
                                   
   	private:
	
		CBrowserPluginControl* iCtrl;
		
	    TInt  		iCommandBase;

};


#endif //__PLUG_IN_MENU_MGR_H__