/*
* Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  FlashPluginControl class implementation
*
*/


#ifndef _POINTER_GESTURE_DETECTOR_INCL_
#define _POINTER_GESTURE_DETECTOR_INCL_

#include "flashlongtapdetector.h"
#include "browserplugintimer_object.h"

//class MPlgAsyncHandler;
class MGestureEventObserver;

class CGestureDetector: public CBase, public MFlashLongTapDetectorCallBack , public MPlgAsyncHandler
{
	public:	
	    enum TGesture
	    {
	        EGestureLongTap,
	        EGestureDoubleTap,
	        EGestureDrag,
	        EGestureTap,
	        EGestureDragDown,
	    };	    
	    
		void HandlePointerEventL(const TPointerEvent &aPointerEvent);
		void ConstructL();
		CGestureDetector(MGestureEventObserver* aObserver);
		static CGestureDetector* NewL(MGestureEventObserver* aObserver);
		virtual void DoRunL(const CActive* aActive);
		virtual TInt DoRunError(const CActive* aActive, TInt aError);    
		void Reset();  
	//Inherited from MFlashLongTapDetectorCallBack   
    void HandleFlashLongTapEventL( const TPoint& aPenEventLocation, 
                                      const TPoint& aPenEventScreenLocation );
    ~CGestureDetector();
 private:  

    CFlashLongTapDetector* iFlashLongTapDetector;
    
    enum 
    {
    EStateWaiting,
    
    //Intermediate states
    EStateIntermediate1, 
    EStateIntermediate2,
    EStateIntermediate3,
    EStateIntermediate4,
    EStateDrag,
    
    //Gesture states
    EStatELongTap
    } iState;

    RArray <TPointerEvent> iPointerEventArray;
    TPoint ipreviousPoint;
    


    CPlgTimerObject* iDoubleTapTimer;
    CPlgTimerObject* iAccidentalUpTimer;
    TPoint iPoint;
    
    TTime iPrevUpTime;
    TTime iDragStateUpTime;
    TPoint iPrevUpPoint;
   
 
    TBool iDoubleTap;
    MGestureEventObserver *iGestureObserver;
    
    TBool  iInsidePointerEventHandler;

};


class MGestureEventObserver
{
public:
    virtual void HandleGestureL(CGestureDetector::TGesture,const TPointerEvent &aPointerEvent) =0 ;
    
};

#endif // _POINTER_GESTURE_DETECTOR_INCL_
