/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the text input class.
*
*/


#ifndef __SYMBIAN_TEXTINPUT_H__
#define __SYMBIAN_TEXTINPUT_H__

/************************************************************************/
/* This file should be shared identically between SAPlayer and FlashCast*/
/* Please be sure to keep the two files in sync until they can be shared*/
/* properly.																														*/
/************************************************************************/
#include <mdaaudiosampleplayer.h>
#include <bitstd.h>
#include <bitdev.h>
#include <etel.h> // for RPhone, RLine, etc.


class CGetInputTextDialog;
class CGetInputNumberDialog;
class TGetInputTextData;
class MFlashEngine;

/*
*	TextInput handles the platform specific modal text-input dialog for Symbian.
*  The TextInput object is an aggregate private member of PlatformFlashEngine.
*/
class TextInput
{
public:
	TextInput(MFlashEngine* apEngine) 
	  : iTextEntryActive(EFalse),
		iTextEntryPtr ( NULL, 0, 0 ),
		iEngine(apEngine) { }
	
	/*
	 *	GetInputText is called from the FlashLite DLL via the BI_GetInputText hook.
	 *  Initiates a modal input text dialog, pausing the movie until the user dismisses the dialog
	 */
	void GetInputTextL(TGetInputTextData& aData);
	void CancelInputText();
	TBool IsBusy() { return iTextEntryActive; }
	
private:
	/*
	 *	SetInputText is called by the symbian dialog upon close.
	 *  This method updates the state of the FlashLite instance and resumes the movie.
	 */
	void SetInputTextL(const TDesC& aNewStr);
	void ResetBusyFlag() { iTextEntryActive = EFalse; }
	
	TBool iTextEntryActive;
	HBufC* iTextData;
	TPtr iTextEntryPtr;
	MFlashEngine* iEngine;		
	
	friend class CGetInputTextDialog;
	friend class CGetInputNumberDialog;
};

#endif	// __SYMBIAN_TEXTINPUT_H__
