/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CAnimationPluginControl class implementation
*
*/


// INCLUDE FILES
#include "saplayer_build.h"

#include <eikenv.h>
#include <bldvariant.hrh>
#include <featmgr.h>
#include <flashliteplugin.rsg>

#include "flashanimcontrol.h"
#include "flash_engine.h"
#include "flashliteplugin.hrh"
#include "symbian_textinput.h"
//#include "FlashPlgMain.h"
#include "hwkeywatch.h"
#include "saflash_consts.h" //capability feature info 
#include "flashpluginif.h"
//#include "FlashplayerControl.h"

// CONSTANTS
/**
* Current default Volume for animation control
**/
const TUint32 KFlashlitePluginVolume = 0x00000006;
//pluginglobals defs
const TInt KMinVolume = 0;  // Minimum volume level(Mute)
const TInt KMaxVolume = 10; // Maximum volume level

extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );

extern int load_pluginresourceL(TBufC<128>& aFileName);

// ==========================================================
// FlashPluginControl

// -------------------------------------------------------------------------
// CAnimationPluginControl* CAnimationPluginControl::NewL(CFlashPluginInstance* aInstance)
//
// Static constructor
// --------------------------------------------------------------------------
//
CAnimationPluginControl* CAnimationPluginControl::NewL(MFlashPluginInstance* aInstance)
{
	NPPDebug(LOG_DEBUG,_L("CAnimationPluginControl::NewL() ++"));
    

    CAnimationPluginControl* self = new(ELeave) CAnimationPluginControl(aInstance);

    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop(self);
    
    NPPDebug(LOG_DEBUG,_L("CAnimationPluginControl::NewL() --"));
    return self;	
	
}

// -------------------------------------------------------------------------
// CAnimationPluginControl::CAnimationPluginControl(CFlashPluginInstance* aInstance)
//
// First phase constructor
// --------------------------------------------------------------------------
// 
CAnimationPluginControl::CAnimationPluginControl(MFlashPluginInstance* aInstance)
                    : CFlashPluginControl(aInstance)
					//, iSender(NULL)
					, iTextInput(NULL)
					,iVolume(KFlashlitePluginVolume)
    {
    }


// ----------------------------------------------------------------------
// CAnimationPluginControl::~CAnimationPluginControl()
//
// Destructor
// ----------------------------------------------------------------------
// 
CAnimationPluginControl::~CAnimationPluginControl()
    {
    
    	//RemoveVolumeObserver();
    	
    	/*if(iSender)
    	{
    		delete iSender;	
    	}*/
  			
		if(iTextInput)
	    {
	    	delete iTextInput;
	    }
	    
    }

/**void CAnimationPluginControl::RemoveVolumeObserver()
{
	CPluginGlobals* globals = static_cast<CPluginGlobals*>(Dll::Tls());

    CHwKeyPropertyWatch* hwKeyPropertyWatch = NULL;
    
    if(globals)
    {
    	hwKeyPropertyWatch = globals->GetHwKeyPropertyWatch();
    }
                    
    if (hwKeyPropertyWatch)
	    {
	    	hwKeyPropertyWatch->RemoveObserver(this);
	    }
}
*/

// ----------------------------------------------------------------------
// void CAnimationPluginControl::ConstructL()
//
// This function is used to construct an instance of host engine
// ----------------------------------------------------------------------
// 
void CAnimationPluginControl::ConstructL()
    {
    	
    /*CFlashConfigGen::FillFlashConfig( iPlayerInstance->GetInteractionMode(),
    																	iConfig);
        
    FillConfig(); //fill additional config values
    */
    NPPDebug(LOG_DEBUG,_L("CAnimationPluginControl::ConstructL() --"));
    
    //Moved to CAnimationPluginInstance::ConstructL
    //CreateEngineL();
    
    MFlashEngine* engine = FlashPluginInstance()->GetEngine();
    
    iTextInput = new(ELeave) TextInput(engine);
    
    //Moved to CFlashPluginInstance
    //Observer is added in AnimationpluginInstance
    /*if (FeatureManager::FeatureSupported(KFeatureIdSideVolumeKeys))  
      {
            CPluginGlobals* globals = static_cast<CPluginGlobals*>(Dll::Tls());
        
       	    CHwKeyPropertyWatch* hwKeyPropertyWatch = NULL;
	    
		    if(globals)
		    {
		    	hwKeyPropertyWatch = globals->GetHwKeyPropertyWatch();
		    }

			if (hwKeyPropertyWatch)
    		    {
	    		    hwKeyPropertyWatch->AddObserverL(this); 
	    		    engine->SetVolume(iVolume);    
    		    }       
      }*/
      
    }

// ----------------------------------------------------------------------
// void CBrowserPluginControl::MHandleVolumeCmd(TInt aNewValue)
//
// This function is callback from volume watchers
// h/w volume key monitor
// ----------------------------------------------------------------------
//    
void CAnimationPluginControl::MHandleVolumeCmdL(TInt aNewValue)
{
    TInt oldVal=iVolume;  
                        
	switch(aNewValue)
    {
        case EVolumeControlCmdVolumeUpByOne:
        {
        	if(iVolume< (KMaxVolume))
        	    {
                ++iVolume;
        	    }
        	else
        	    iVolume = KMaxVolume;
        	
        }
            break;
        
        case EVolumeControlCmdVolumeDownByOne:
        {
        	if(iVolume > KMinVolume)
        	    {
                --iVolume;
        	    }
        	else
        	    iVolume = KMinVolume;
        		 
        }
        break;
        
        default:
        		break;
    }
    
    if(iVolume != oldVal)        
    { //set it only when value is changed
    this->FlashPluginInstance()->SetVolumeToEngine(iVolume);
    }
}

// ----------------------------------------------------------------------
// void CBrowserPluginControl::MVolumeChanged(TInt aNewValue)
//
// This function is callback from volume watchers
// for vollume change with touch 
// ----------------------------------------------------------------------
//    

 void CAnimationPluginControl::MVolumeChanged(TInt aNewValue)
{
	if(iVolume != aNewValue)        
    { //set it only when value is changed
    this->FlashPluginInstance()->SetVolumeToEngine(aNewValue);
    }
	
}

/*void CAnimationPluginControl::MHandleVolumeRectRedraw()
{
	return;
}*/

// ---------------------------------------------------------------------------
// TInt CAnimationPluginControl::NotifyL(TNotificationType aCallType, void* aParam)
//
// This function is used to send notifications from the browser to the host 
// engine
// ---------------------------------------------------------------------------
//
TInt CAnimationPluginControl::NotifyL(TNotificationType aCallType, void* aParam)
{
	MFlashEngine* engine = NULL;
	switch (aCallType)
	{
	case EApplicationFocusChanged:
		engine = FlashPluginInstance()->GetEngine();

		if (engine)
		{
			engine->SetForeground(ETrue);
			engine->NotifyFocusChangeL(TBool(!aParam));
		}
		break;
	default:
		break;
	}
	return KErrNone;
}


//-----------------------------------------------------------------------
// void CAnimationPluginControl::NotifyError(TInt aErrorCode)    
//
// This function is called when there is an error while opening 
// a flash file in the screensaver mode
//-----------------------------------------------------------------------
 
void CAnimationPluginControl::NotifyError(TInt aErrorCode) 
    {
    iLastError = aErrorCode;
    iPlayerInstance->PrepareToExit();
    }
    
/**
	mutes the volume.
	@param none.
*/
		
void CAnimationPluginControl::MuteVolume() 
{
	this->FlashPluginInstance()->SetVolumeToEngine(0);
}

/**
	unmutes the volume.
	@param none.
*/		
void CAnimationPluginControl::UnMuteVolume()
{
	this->FlashPluginInstance()->SetVolumeToEngine(iVolume); 
}
  

void CAnimationPluginControl::MPausePlayKeyPressed()
{
    MFlashEngine* engine = FlashPluginInstance()->GetEngine();
	if(engine)	
	{
		if(engine->IsRunning())
		{
			FlashPluginInstance()->Pause(EReasonUser);
		}
		else
		{
			if(engine->IsPaused())
			{
				TRAP_IGNORE(engine->ResumeL(EReasonUser,EFalse));	
			}
			
		}
	}
}
	  
void CAnimationPluginControl::MStopKeyPressed()
{
    this->FlashPluginInstance()->Stop();
}
	  
void CAnimationPluginControl::MRewindKeyPressed()
{
    this->FlashPluginInstance()->Rewind();
}
	
void CAnimationPluginControl::MForwardKeyPressed()
{
	
}
			
void CAnimationPluginControl::MBackwardKeyPressed()
{
	
}
			
void CAnimationPluginControl::MFastForwardKeyPressed()
{
	
}
// ---------------------------------------------------------------------------
// void CAnimationPluginControl::SendMessage(const TUid aMtmUid, char* aAddress, 
// char* aBodyText)
//
// This function is used to send a message
// ---------------------------------------------------------------------------
// 
//Moved to CDeviceExtn
/*void CAnimationPluginControl::SendMessage(const TUid aMtmUid, 
                                           char* aAddress, 
                                           char* aBodyText)
    {
    if(!iSender)
    {
    	
    	TRAP_IGNORE(iSender = CSender::NewL());
    }
    
    TRAP_IGNORE(iSender->CreateAndEditMessageL(aMtmUid, 
                                               aAddress, 
                                               aBodyText));
    }*/
 
 /**
 Specifies if the text is being input.
 @return TBool ETrue if the user has not yet dismissed the text input
 dialog.
*/   
    
TBool CAnimationPluginControl::IsTextInputBusy()
{
	return iTextInput->IsBusy();
}          

/**
 This function initiates a modal input text dialog, pausing the 
 movie until the user dismisses the dialog
 @param aData structure for the input text data.
 */

void CAnimationPluginControl::GetInputTextL(TGetInputTextData& aData)
{
	iTextInput->GetInputTextL(aData);
}

/**
	 This function is called by the host engine when there is an error in
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/
	
void CAnimationPluginControl::NotifyFlashError(TInt aErrorCode, 
                                               TBool /*aIsCritical*/, 
                                               TBool /*aPersistent*/ )
{
	NPFlashConfig* pFlashconfig = iPlayerInstance->GetFlashconfig();
	
	if((NULL != pFlashconfig) && (NULL != pFlashconfig->iNotifyCaller)) 
	{
		if((pFlashconfig->iPrefFlag) & NPPrefErrorNotification)
		{
		    TRAP_IGNORE (pFlashconfig->iNotifyCaller->NotifyL(NPNotifyContentError,
		                                                      (TAny*)&aErrorCode));
		}
	}
}

/**
	 This function is called by the host engine when there is an error in
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/		
void CAnimationPluginControl::NotifyClientError(TInt aErrorCode, 
                                                TBool /*aIsCritical*/, 
                                                TBool /*aPersistent*/)
{

	NPFlashConfig* pFlashconfig = iPlayerInstance->GetFlashconfig();
	
	if((NULL != pFlashconfig) && (NULL != pFlashconfig->iNotifyCaller)) 
	{
		if((pFlashconfig->iPrefFlag) & NPPrefErrorNotification)
		{
			TRAP_IGNORE (pFlashconfig->iNotifyCaller->NotifyL(NPNotifyInstanceError,
			                                                  (TAny*)&aErrorCode));
		}
	}

}

/**
	 This function is called by the host engine flash status
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/		

void CAnimationPluginControl::NotifyFlashStatus(TInt aFlashStatus, TAny* /*aParam*/) 
{

	NPFlashConfig* pFlashconfig = iPlayerInstance->GetFlashconfig();
	
	if((NULL != pFlashconfig) && (NULL != pFlashconfig->iNotifyCaller)) 
	{
		if(((pFlashconfig->iPrefFlag) & NPPrefContentStatusNotification) ||
			((pFlashconfig->iPrefFlag) & NPPrefFrameReadyNotification))	
		{
			TRAP_IGNORE (pFlashconfig->iNotifyCaller->NotifyL(NPNotifyInstanceStatus,(TAny*)&aFlashStatus));
		}
	}
		
}

//-----------------------------------------------------------------------
// short CAnimationPluginControl::GetFeatureCapability()    
//
// This function returns supported feature for fscommnd
//-----------------------------------------------------------------------
//TUint CAnimationPluginControl::GetFeatureCapabilityFromMode(const TUint16 aModeType)
//{
//	switch(aModeType)	
//	{
//	
//  	/** 
//		*  flash running in idle 
//    */
//    
//  		case NPIdle : 
//					{ 
//					return KCapabilitiesIdle ; 
//					}
//
//  	
//  	/** 
//		*  flash running as video ring tone 
//    */
//    
//  		case NPRingTone: { 
//					return KCapabilitiesRingTone ; 
//					}
//
//  	
//  	/** 
//		*  flash running as call image 
//    */
//    
//  		case NPCallImage: { 
//					return KCapabilitiesCallImage ; 
//					}
//
//  	
//  	/** 
//		*  flash running as animated background in idle 
//    */
//    
//  		case NPBackground: { 
//					return KCapabilitiesBackground ; 
//					}
//
//  	
//  	/** 
//		*  flash running as wall paper 
//    */
//    
//  		case NPWallpaper: { 
//					return KCapabilitiesWallpaper ; 
//					}
//
//  	
//  	/** 
//		*  flash running as notification  for calendar apointments
//    */
//    
//  		case NPNotification: { 
//					return KCapabilitiesNotification ; 
//					}
//
//  	
//  	/** 
//		*  flash running as startup screen
//    */  	
//  		case NPStartupScreen: { 
//					return KCapabilitiesStartupScreen ; 
//					}
//
//  	
//  	/** 
//		*  flash running as shutdown screen
//    */
//  		case NPShutdownScreen: { 
//					return KCapabilitiesShutdownScreen ; 
//					}
//
//  		
//  				/** 
//		*  simple animation, this is default 
//    */
//    
//		case NPAnimation :
//		default :
//				{ 
//					return KCapabilitiesAnimation ; 
//					}
//
//  		
//	}
//
//}

//-----------------------------------------------------------------------
// short CAnimationPluginControl::GetFeatureCapability()    
//
// This function returns supported feature for fscommnd
//-----------------------------------------------------------------------
//Moved to CAnimationPluginInstance
/*TUint CAnimationPluginControl::GetFeatureCapability()
{
	NPFlashConfig* pFlashconfig = iPlayerInstance->GetFlashconfig();
	
	if(NULL != pFlashconfig)
	{
		return GetFeatureCapabilityFromMode(pFlashconfig->iModeType);	
	}
	
	return KCapabilitiesAnimation;
}*/

// ----------------------------------------------------------------------
// TBool CAnimationPluginControl::QueryAllowDeviceIDL()
//
// This function is used to display a query that the swf is trying
// to access the phone's device id 
// ----------------------------------------------------------------------
// 
TBool CAnimationPluginControl::QueryAllowDeviceIDL()
    {
    	TBufC<128> KResFile(_L("FlashlitePlugin.rsc") );
    TInt resOffset = load_pluginresourceL( KResFile );
	HBufC* pMessage = iEikonEnv->AllocReadResourceLC(R_STR_ALLOW_DEVICE_ID);
	TBool rval=PluginBase::QueryAllowDenyL(*pMessage);
	CleanupStack::PopAndDestroy(pMessage);
	
	CEikonEnv::Static()->DeleteResourceFile(resOffset);

	return rval; //by default allow device id access
    }


// End of File
