/*
* Copyright (c) 2005-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  FlashPluginControl class implementation
*
*/


// INCLUDE FILES
#include "flashbrowserplugincontrol.h"
#ifdef __FEATURE_PLUGIN_UI_OPTIONS
#include "flashbrowserpluginfullscreenctrl.h"
#endif
// -- #include "flashsendmessage.h"
// -- #include "flashplugin.h"
#include "flash_engine.h"
// -- #include "FlashConfigGen.h"
#include "flashliteplugin.hrh"
#include "symbian_textinput.h"
#include "saflash_consts.h"
// -- #include "StreamFactory.h"
// -- #include "StreamHandler.h"
// includes for message sending
#include <stdlib.h>
//#include <mmsconst.h>
#include <mtuireg.h>
#include <mtmuibas.h>
#include <eikfutil.h>
#include <txtrich.h>
#include <httperr.h>
#include <coemain.h>
#include <charconv.h>
#include <eikenv.h>
#include <CMessageData.h>
#include <aknnotewrappers.h>
#include <bldvariant.hrh>
#include <featmgr.h>
#include <AknSettingCache.h>
#include <sysutil.h>
//To get keyboard layout
#include <AvkonInternalCRKeys.h>
#include <PtiDefs.h>


// for popups
#include <StringLoader.h>
#include <AknGlobalNote.h>
#ifdef __FEATURE_PLUGIN_UI_OPTIONS
#include "pluginmenumgr.h"
#include "flashvolumecontrol.h"
#include <aknclearer.h> //CAknLocalScreenClearer
#endif

//#define _GestureDebug_

#ifdef FEATURE_INLINE_TEXT_ENTRY
#include "amp_mtextnavigation.h"
#endif // FEATURE_INLINE_TEXT_ENTRY

#include <flashliteplugin.rsg>
#include <flashlitepluginiad.rsg>
#include <gdi.h>
#include <AknIconUtils.h>
#ifdef __SCALABLE_ICONS
#include <FLASHPLUGIN.MBG>
#else
#include <flashplugin_bitmap.mbg>
#endif

#include "flashiconhandler.h"
#include <w32std.h>

using namespace RT_GestureHelper;

#ifdef USE_VPU2
#include "surfacecontainercontrol.h"
#endif

#ifndef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR

const TInt  KStylusDragThreshold	= 5;
#endif

#ifdef NP_DEBUG
const TUint KMaxLogLength           = 512;
const TInt  KPrintChars1            = 45;
const TInt  KPrintChars2            = 47;
#endif


#ifdef ENABLE_CURSOR
const TInt  KCursorPointX        = 40;
const TInt  KCursorPointY        = 40;
#endif

#ifndef GESTURE_LIBRARY_SUPPORTED
const TInt KSetWindowTimeOut      =300000;
#endif //GESTURE_LIBRARY_SUPPORTED
extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );

enum TMenuOptionType {
			EOptionOpenInViewer=10 //this can not be 0 or 1 bcs return value checked
		};

// ==========================================================
// FlashPluginControl

// -------------------------------------------------------------------------
// CBrowserPluginControl* CBrowserPluginControl::NewL(CFlashPluginInstance* aInstance)
//
// Static constructor
// --------------------------------------------------------------------------
//
CBrowserPluginControl* CBrowserPluginControl::NewL(MFlashPluginInstance* aInstance)
{
    CBrowserPluginControl* self = new(ELeave) CBrowserPluginControl(aInstance);
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop(self);
    return self;
}

// -------------------------------------------------------------------------
// CBrowserPluginControl::CBrowserPluginControl(CFlashPluginInstance* aInstance)
//
// First phase constructor
// --------------------------------------------------------------------------
//

CBrowserPluginControl::CBrowserPluginControl(MFlashPluginInstance* aInstance)
                    :   CFlashPluginControl(aInstance)
//                      , iTextInput(NULL)
                      ,iFLVPlayState(EFlashPluginNoFLVStream)
                      #ifdef __FEATURE_PLUGIN_UI_OPTIONS
    					,iMenuMgr(NULL)
    					,iVolPopup(NULL)
    				  #endif
                      
                      #ifdef FEATURE_COLLECT_BITMAP
                      ,iLastRenderedBitmap(NULL)
                      ,iSwfFirstPlay(EFalse)
                      #endif
                      #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
					  , iFullScreen(EFalse)
    				  // ,iCanGoFullScrn(EFalse)
    				   ,iFullScreenHandler(NULL)
    				   ,iFullScreenControl(NULL)
					  #endif
					  #ifdef ENABLE_CURSOR
					  , iSavedCursorPoint(TPoint(0, 0))
					  #endif
					  //, iPlayState(EFlashPluginInitial)
					  , iNumURLRequestCount(0)
					  
                     

 {
#ifdef GESTURE_LIBRARY_SUPPORTED
    iLongTap= EFalse;
#else
    iSuppressor         = NULL;
    iGestureDetector    = NULL;
#endif
 	iTogglingScreen = EFalse;
 	iPreparingToExit = EFalse;
  	iBrowserScroll = EFalse;
 	iFlashScroll  =EFalse;
 	iCurrentPositionNotSet =ETrue;
    

 }

// ----------------------------------------------------------------------
// CBrowserPluginControl::~CBrowserPluginControl()
//
// Destructor
// ----------------------------------------------------------------------
//
CBrowserPluginControl::~CBrowserPluginControl()
{
    #ifdef __FEATURE_PLUGIN_UI_OPTIONS
	HandleFullScreenDelete();
	#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
    if(iFullScreenHandler)
    {
        delete iFullScreenHandler;
        iFullScreenHandler = NULL;
    }
	#endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN
	#endif

#ifdef GESTURE_LIBRARY_SUPPORTED
	if(iPtrBuffer)
	{
        delete [] iPtrBuffer;
	}
#else
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR

    if(iSuppressor)
    {
		delete iSuppressor;
		iSuppressor = NULL;
    }
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR

    if(iGestureDetector)
    {
		delete iGestureDetector;
		iGestureDetector =NULL;
    }
#endif //GESTURE_LIBRARY_SUPPORTED

//    if(iTextInput)
//	    {
//	    delete iTextInput;
//	    }

	if(iIconHandler)
	{
		delete iIconHandler;
		iIconHandler = NULL;
	}
}

#ifdef __FEATURE_PLUGIN_UI_OPTIONS
void CBrowserPluginControl::HandleFullScreenDelete()
{
	if(iFullScreenControl && iFullScreen )
	{
		TRAP_IGNORE(GoNormalScreenL());
	}

    if(iVolPopup)
	{
		delete iVolPopup;
		iVolPopup = NULL;
	}


	if(iMenuMgr)
	{
		delete iMenuMgr;
		iMenuMgr = NULL;
	}
}

#endif

void CBrowserPluginControl::PrepareToExitL()
{
	if(iPluginAdapter)
	{
        iPluginAdapter->SetPluginNotifier(NULL);		//browser request
        iPluginAdapter->SetOptionMenuHandler(NULL);
	}
	iPreparingToExit = ETrue;

	#ifdef __FEATURE_PLUGIN_UI_OPTIONS

    HandleFullScreenDelete();

    #endif
}

// ----------------------------------------------------------------------
// void CBrowserPluginControl::ConstructL(TUint32 aMaxMem, TUint32 aDebugMode,
//    TUint32 aMinMem)
//
// This function is used to construct an instance of host engine
// ----------------------------------------------------------------------
//

void CBrowserPluginControl::ConstructL()
{
    PluginBase::ConstructL();

#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
    iFullScreenHandler = new (ELeave)CAsyncFullScreenHandler(this);
#endif

    // Gets the current input launguage id.
	CAknSettingCache& cache = CAknEnv::Static()->SettingCache();
	#ifdef FEATURE_INLINE_TEXT_ENTRY
	iCurrentInputLanguage   = cache.InputLanguage();
	#endif

//	MFlashEngine* engine = this->iPlayerInstance->GetEngine();

//	if(engine)
//	    iTextInput = new(ELeave) TextInput(engine);

   	iIconHandler = CFlashIconHandler::CreateIconHandlerL();

   	#ifdef ENABLE_CURSOR
    //iCursor = CFlashCursor::NewL(this);
	iCursor = 0;
	#endif

    #ifdef __FEATURE_PLUGIN_UI_OPTIONS
   	iMenuMgr 	= CPluginMenuManager::NewL(this);
    #endif

   	#ifdef GESTURE_LIBRARY_SUPPORTED
    iPtrBuffer = new TPoint[256];
    #else
    if(iIsTouchFeatureSupported)
    {
    #ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
    iSuppressor = CAknPointerEventSuppressor::NewL();
    iSuppressor->SetMaxTapDuration(KMaxTInt64);
    #endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
    iGestureDetector = CGestureDetector::NewL(this);
    }
    #endif // GESTURE_LIBRARY_SUPPORTED
    iPluginAdapter = NULL;
}

int CBrowserPluginControl::SetWindow(NetscapeWindow* aWindow)
{
    NPWindow* window = static_cast<NPWindow*> (aWindow);

    TInt width = STATIC_CAST(TInt, window->width);
    TInt height = STATIC_CAST(TInt, window->height);

    TRect clipRect;
#ifndef __TIGER_IAD_ADAPTATION
    clipRect.iTl.iY = window->clipRect.top;
    clipRect.iTl.iX = window->clipRect.left;
    clipRect.iBr.iY = window->clipRect.bottom;
    clipRect.iBr.iX = window->clipRect.right;
#else
    clipRect.iTl.iY = window->clipRect.left;
    clipRect.iTl.iX = window->clipRect.top;
    clipRect.iBr.iY = window->clipRect.right;
    clipRect.iBr.iX = window->clipRect.bottom;
#endif

    TRect swfRect;
    swfRect.iTl.iY = window->y;
    swfRect.iTl.iX = window->x;
    swfRect.iBr.iY = swfRect.iTl.iY + height;
    swfRect.iBr.iX = swfRect.iTl.iX + width;

/*
#ifdef FEATURE_WINDOWLESS
    if (0 != iWindowMode) in windowless mode, call bitmap resize explicitly
        {
        void* parent = NULL;
        iGlobals->NetscapeFuncs()->getvalue(this->iInstance,NPNVnetscapeWindow,(void *)&parent);
        CCoeControl* pCtrl = static_cast<CCoeControl*> (parent);
        }
#endif
*/

   // if (height > 0 && width > 0)
    {
        MPluginAdapter* pluginAdapter = STATIC_CAST(MPluginAdapter*, window->window);
        if (!pluginAdapter)
        {
            #ifdef NP_DEBUG
            NPPDebug(LOG_ERROR,KLogNPPSetWindow2);
            #endif
            //User::Leave(NPERR_INVALID_INSTANCE_ERROR);
            return NPERR_INVALID_INSTANCE_ERROR;
        }
        else
        {
            this->SetWindow(pluginAdapter, swfRect, clipRect);
        }
    }

    return KErrNone;
}

int CBrowserPluginControl::SetWindow(MPluginAdapter* aPlugin, const TRect& aRect, const TRect& aClipRect)
{
    TRAPD( error, DoSetWindowL(aPlugin,aRect,aClipRect) );
    return error;
}

void CBrowserPluginControl::DoSetWindowL(MPluginAdapter* aPlugin, const TRect& aRect, const TRect& aClipRect)
{
//     This check is added to make sure that broken flash
//     icon should get displayed even when user double tap the
//     webpage to zoom it. Fix for bug ESJY-7QEHPF
    iPluginAdapter = aPlugin;
    if(iErrorOccured)
    {
        this->SetRect(aRect);
        User::Leave(KErrGeneral);
    }

    CFlashPluginControl::SetWindowL(aPlugin, aRect, aClipRect);

    SetRect(aRect);

    SetRectRelativeToParentControl(aRect);

    #ifdef __FEATURE_PLUGIN_UI_OPTIONS
  if(aPlugin)
      aPlugin->SetOptionMenuHandler(GetOptionHandler());
      //aPlugin->SetOptionMenuHandler(GetOptionHandler());
  #endif


   //Set the new rectangle so that helix video player can set the rect properly.
  SetVideoDisplayRectL(aRect, aClipRect);

#ifdef HANDLE_PLUGIN_INSTANCE

     //Now coming back to focus; So if the player engine is deleted,
     //then recreate it so that we can start playing.

//    if( !IsValid() )
//        {
//        CreateControlEngineL();
//        }

#else

#ifndef __S60_32__
    if( IsFocused() )
    {
        NotifyL(MPluginNotifier::EPluginInvisible, (TAny*) 1);
    }
    else
    {
        HandleFocusLostL();
    }
#endif
#endif
}

// ----------------------------------------------------------------------
// TRect CBrowserPluginControl::GetRect()
//
// To get the rect for video object in the browser web-page.
// ----------------------------------------------------------------------
//

TRect CBrowserPluginControl::GetRect()
{
    TRect rect;
    TSize size;
	TPoint topLeftCorner;

    #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
    if(IsFullScreen())
	{
		if(iFullScreenControl)
		{
			topLeftCorner = iFullScreenControl->PositionRelativeToScreen();
			size = iFullScreenControl->Size();
		}
	}
    else
    #endif
	{
		size = Size();
		topLeftCorner = PositionRelativeToScreen();
	}
    rect = TRect(topLeftCorner.iX, topLeftCorner.iY,
                 topLeftCorner.iX + size.iWidth,
                 topLeftCorner.iY + size.iHeight);

    return rect;
}

// ----------------------------------------------------------------------
// TRect CBrowserPluginControl::GetClipRectFromScreenOrigin()
//
// To get the clipping rect for video object in the browser web-page.
//
// NOTE:
// Formula to determine clip rect in browser (relative to screen origin) :
// ClipRect.x = (Browser's clipRect).x + (TopLeftCornerRelativeToSO).x - (Browser's rect).x
// ClipRect.y = (Browser's clipRect).y + (TopLeftCornerRelativeToSO).y - (Browser's rect).y
// ----------------------------------------------------------------------
//

TRect CBrowserPluginControl::GetClipRectFromScreenOrigin()
{
    TRect clipRect = iClipRect;
    TPoint topLeftCorner = PositionRelativeToScreen();
    clipRect.Move(topLeftCorner);
    clipRect.iTl -= iRectRelativeToParentControl.iTl;
    clipRect.iBr -= iRectRelativeToParentControl.iTl;
    return clipRect;
}

// ----------------------------------------------------------------------
// TRect CBrowserPluginControl::GetClipRect()
//
// To get the clipping rect for video object in the browser web-page.
// This function is called from the engine of this control to query the clip rect.
// ----------------------------------------------------------------------
//

TRect CBrowserPluginControl::GetClipRect()
{
	return GetClipRectFromScreenOrigin();
}

// ---------------------------------------------------------------------------
// void CBrowserPluginControl::RecreateBitmapL(const TRect& aRect)
//
// This function is used to pass the rectangle parameters to the host engine
// ---------------------------------------------------------------------------
//
void CBrowserPluginControl::RecreateBitmapL(const TRect& aRect)
{
#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
    if(iFullScreenControl)
    {
    	iFullScreenControl->SetRect(aRect);
    }
#ifdef USE_VPU2
    if(iVideoSurfaceContainer && !IsFullScreen())
        iVideoSurfaceContainer->SetRect(aRect);
#endif
#endif
	CFlashPluginControl::RecreateBitmapL(aRect);
}

void CBrowserPluginControl::DrawDirtyRect(const TRect& aDirtyRect)
{
	if( !aDirtyRect.IsEmpty() )
	{
		#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
		if(iFullScreenControl)
		{
		  	iFullScreenControl->DrawDirtyRect(aDirtyRect);
		}
		else
		#endif
		{
			DrawNow(aDirtyRect);
		}
	}
}

TInt CBrowserPluginControl::HandlePluginEvent(void* aEvent)
{
    NPEvent* pEvent = (NPEvent*)aEvent;
    NPEventPointer *param = NULL;
	TInt retVal = 0;

	switch (pEvent->event)
	{
		case ENppEventKey:
			break;

		case ENppEventPointer:
		    param = static_cast<NPEventPointer*>(pEvent->param);
			TRAP_IGNORE(retVal = HandleGestureFromBrowserL(param->reserved));
			break;

		case ENppEventDraw:
			break;

		case ENppEventVisibility:
			break;

		default:
			break;
	}
	return retVal;
}

void CBrowserPluginControl::SetScreenL(EScreenMode aMode)
{
	if(aMode == eFullScreen)
		SetFullScreenL(TRUE);
	if(aMode == eAppScreen)
		SetFullScreenL(FALSE);
}

void CBrowserPluginControl::SetFullScreenL(TBool aSetFullScreen)
{
	if(aSetFullScreen)
	{
		if(!iFullScreen)
		{
#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
			GoFullScreenL();
#endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN
		}
	}
	else
	{
		if(iFullScreen)
		{
#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
			GoNormalScreenL();
#endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN
		}
	}
}

void CBrowserPluginControl::Redraw()
{
    DrawNow();
}

// ----------------------------------------------------------------------
// void CBrowserPluginControl::Draw(const TRect& aRect) const
//
// This function is used to draw the view
// ----------------------------------------------------------------------
//
void CBrowserPluginControl::Draw(const TRect& aRect) const
{
    CWindowGc& gc = SystemGc();

    if( iTogglingScreen )
	{
		return;
	}

    if( iErrorOccured )
    {
        iIconHandler->Show(&gc, FlashPluginInstance()->GetWMode(), eErrorIcon,Rect());
        return;
    }

    MFlashEngine* engine = FlashPluginInstance()->GetEngine();

    if (engine)
    {
        if (iFileOpened)
        {
            engine->Draw(aRect, gc);
        }
        else //If the swf not openened we draw the background colour
        {
            TRgb bgColour = engine->GetBgcolour();
            gc.SetBrushColor(bgColour);
            gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
            gc.Clear(Rect());
        }
    }

#ifdef SYMBIAN_BUILD_GCE
    ControlEnv()->WsSession().Finish();
#endif
}

void CBrowserPluginControl::SetRect(const TRect& aRect)
{
    CCoeControl::SetRect(aRect);
#ifdef USE_VPU2
    if(iVideoSurfaceContainer && !IsFullScreen())
        iVideoSurfaceContainer->SetRect(aRect);
#endif
}

// ---------------------------------------------------------------------------
// TInt CBrowserPluginControl::HandlePluginActivated(void* aParam)
//
// This function is used to handle plugin activated
// ---------------------------------------------------------------------------
//
#ifdef ENABLE_CURSOR
void CBrowserPluginControl::HandlePluginActivated(void* aParam)
#else
void CBrowserPluginControl::HandlePluginActivated(void* /*aParam*/)
#endif
{
	NPPDebug(LOG_DEBUG,_L("CBrowserPluginControl::HandlePluginActivated() ++"));

  	iPluginActivated = ETrue;

#ifdef ENABLE_CURSOR

  	if( !aParam )
    {
        iSavedCursorPoint.SetXY(KCursorPointX, KCursorPointY);
    }
    else
    {
        TPoint* pt = (TPoint*) aParam;
        iSavedCursorPoint.SetXY(pt->iX, pt->iY);

		NPPDebug(LOG_DEBUG,_L("Plugin activated pos: %d %d"), iSavedCursorPoint.iX,iSavedCursorPoint.iY);
#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
		if(!IsFullScreen())
		{
			TPoint cursorAbsPos;
			cursorAbsPos.SetXY(pt->iX,pt->iY);

            //Cursor's absolute position w.r.t screeen
			cursorAbsPos += PositionRelativeToScreen();

            //clip rect - absolute co-ordinates.
            //Note: It is important to get rect from GetClipRectFromScreenOrigin() api as it takes care
            //of interpreting the co-ordinates in the right way from netscape apis
			TRect clipRect = GetClipRectFromScreenOrigin();

			if(!clipRect.Contains(cursorAbsPos))
			{
				//cursor position falls outside visible area. Possibly a browser bug.
                //Place the cursor in the middle of the visible area.

				TInt widthDelta = cursorAbsPos.iX - (clipRect.iTl.iX + clipRect.Width()/2);
				TInt heightDelta = cursorAbsPos.iY - (clipRect.iTl.iY + clipRect.Height()/2);

                //delta indicates how much the cursor pos has to be moved (relative) so that it falls in the
                //middle of the clip rect.
				widthDelta = -widthDelta;
				heightDelta = -heightDelta;
				iSavedCursorPoint.SetXY(pt->iX + widthDelta, pt->iY + heightDelta);
			}
		}
#endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN
    }

	NPPDebug(LOG_DEBUG,_L("BPC::HandlePluginActivated"));
	if(!iCursor)
	{
		TRAP_IGNORE(iCursor = CFlashCursor::NewL(this,GetPlayerEngine()));
		if (iCursor)
		    iCursor->SetWindowSize(Rect().Size());
	}

#endif

	NPPDebug(LOG_DEBUG,_L("CBrowserPluginControl::HandlePluginActivated() --"));
}

// ---------------------------------------------------------------------------
// TInt CBrowserPluginControl::HandlePluginDeactivated(void* aParam)
//
// This function is used to handle plugin deactivation
// engine
// ---------------------------------------------------------------------------
//

void CBrowserPluginControl::HandlePluginDeactivatedL(void* /*aParam*/)
{
	NPPDebug(LOG_DEBUG,_L("CBrowserPluginControl::HandlePluginDeactivatedL() ++"));
  	if( HasCriticalErrorOccured() )
  	{
  		return;
  	}

	#ifdef ENABLE_CURSOR
  	MFlashEngine* engine = this->FlashPluginInstance()->GetEngine();
  	// Hide flash cursor if plugin get deactivated and
  	// flash cursor is still visible.
    if(engine && iCursor && iCursor->IsVisible())
    {
       iCursor->HideCursor();
    }
	#endif

	#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN

	if(iCanGoFullScrn && iPluginActivated)
  	{
  		if(iFullScreen)
  		{
			if(!iFullScreenHandler)
				iFullScreenHandler = new(ELeave) CAsyncFullScreenHandler(this);

			iFullScreenHandler->IssueRequest(EFalse);
			iFullScreen = EFalse;
  		}
  	}

    #endif

	iPluginActivated = EFalse;

	NPPDebug(LOG_DEBUG,_L("CBrowserPluginControl::HandlePluginDeactivatedL() --"));
}

// ---------------------------------------------------------------------------
// TBool CBrowserPluginControl::HasCriticalErrorOccured()
//
// returns true if a critical error has occured
// ---------------------------------------------------------------------------
//

TBool CBrowserPluginControl::HasCriticalErrorOccured()
{
	if ( iLastError && iErrorIsCritical)
	{
		return ETrue;
	}
	return EFalse;
}

// ---------------------------------------------------------------------------
// TInt CBrowserPluginControl::NotifyL(TNotificationType aCallType, void* aParam)
//
// This function is used to send notifications from the browser to the host
// engine
// ---------------------------------------------------------------------------
//
TInt CBrowserPluginControl::NotifyL(TNotificationType aCallType, void* aParam)
{
    TInt retVal = 0;
    switch(aCallType)
    {
       //Commented temporarily needs to be added and tested might give better performance during panning
        /*case EPluginPause:
        {
            if (aParam)
            {
                iEngine->Pause(EReasonScrolling);
            }
            else
            {
                iEngine->ResumeL(EReasonScrolling,ETrue);
            }
        }
        break;*/

#ifdef FEATURE_COLLECT_BITMAP

        case ECollectBitmapSupported:
            retVal = 1;
            break;

        case ECollectBitmap:
        {

            #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
            if(IsFullScreen())
            {
               //Ignore BitmapCollect calls  if plugin is in fullscreen (youtube)
                break;
            }
            #endif

            /*  ECollectBitmap_1 is recieved once panning/pinch zoom/flick is started
             *  Pause swf playback and send last rendered frame to browser,
             *  Browser will draw plugin area till scrolling is stopped
             */
            if(aParam)
            {
                iPluginScrollState = EScrollingStarted;
                if(iPlayerInstance->IsPlayerInitialzed() && iVisibilityState == EFlashPluginVisible)
                {
                    MFlashEngine* engine = FlashPluginInstance()->GetEngine();
                    TInt err = engine->ToggleBrowserRendering(EBrowserRenderingMode);
                    if(err == KErrNone)
                    {
                        engine->Pause(EReasonBackground);
                        iLastRenderedBitmap = engine->GetLastRenderedBitmap();
                        if(iLastRenderedBitmap)
                        {
                            SendLastRenderedBitmapToBrowser(iLastRenderedBitmap);
                            iPlayState = EFlashPluginPausedWhileScrolling;
                        }
                    }
                }
                else if(iPlayerInstance->IsPlayerInitialzed() && iVisibilityState == EFlashPluginInvisible)
                {
                    if(iLastRenderedBitmap)
                    {
                        SendLastRenderedBitmapToBrowser(iLastRenderedBitmap);
                    }
                }
            }
            else
            {
                iPluginScrollState = EScrollingStopped;
            }
        }
        break;
#endif

        case EApplicationFocusChanged:
        {
        	if( iErrorOccured )
            {
                DrawNow();
                break;
            }

        	// changed to support 0 size swf play thru plugin
        	// changed as part of iSource 11267: Start
			// iSource 11267 -> Check for swf rect rather than clip rect
			// if plugin is not visible, then swf rect will be 0,0,0,0
        	if (iSwfRect.Width() == 0 && iSwfRect.Height() == 0)
        	{
        	    if(!iPlayerInstance->IsPlayerInitialzed() && iPlayerInstance->DeferredLaunch())
        	    {
        	        TInt error = iPlayerInstance->OpenMovie();
        	        if(error != KErrNone)
        	        {
        	            return error;
        	        }
        	    }

        	    break;
        	}
        	// changed as part of iSource 11267: End
        	MFlashEngine* engine = FlashPluginInstance()->GetEngine();

            if( !aParam )
      		{
                iFocusState = EFlashPluginBackground;
                if( engine )
                  engine->SetForeground(EFalse);
      		}

      		else if(aParam)
      		{
      			iFocusState = EFlashPluginForeground;
      			if( engine )
      			  engine->SetForeground(ETrue);
      			// if swf file embedded with in the webpage is not yet
      			// opened then open it. Such a situation can arise
      			// if browser moves to background before we get a newp call
      			// from browser. fix for ASHA-7RKHZT.
      			//if(!iPlayerInstance->IsPlayerInitialzed() && iBaseURL && iVisibilityState == EFlashPluginVisible)
				//if(!iPlayerInstance->IsPlayerInitialzed() && iPlayerInstance->GetBaseURL() && iVisibilityState == EFlashPluginVisible)
				if(!iPlayerInstance->IsPlayerInitialzed() && iPlayerInstance->DeferredLaunch() && iVisibilityState == EFlashPluginVisible)
      			{
      			    TInt error = iPlayerInstance->OpenMovie();
					if(error != KErrNone)
						return error;
      			}
      		}
      		NotifyFocusChangeL((TBool)aParam);

        }
        break;
        case EPluginActivated:
        {
           	HandlePluginActivated(aParam);
        }
        break;

        case EPluginDeactivated:
    	{
    		HandlePluginDeactivatedL(aParam);
    	}
        break;

        case EPluginInvisible:
        {
          //if(iFocusState!=EFlashPluginBackground) //we should not ignore plugin visibility status while in background; hence removing this clause
            {
                // changed to support 0 size swf play thru plugin
                // changed as part of iSource 11267: Start
				// iSource 11267 -> Check for swf rect rather than clip rect
				// if plugin is not visible, then swf rect will be 0,0,0,0
                if (iSwfRect.Width() == 0 && iSwfRect.Height() == 0)
                {
                    if(!iPlayerInstance->IsPlayerInitialzed() && iPlayerInstance->DeferredLaunch())
                    {
                        TInt error = iPlayerInstance->OpenMovie();
                        if(error != KErrNone)
                        {
                            #ifdef FEATURE_COLLECT_BITMAP
	                        iSwfFirstPlay = EFalse;
                            #endif
                            return error;
                        }
                        #ifdef FEATURE_COLLECT_BITMAP
	                    iSwfFirstPlay = ETrue;
                        #endif
                    }

                    break;
                }
                // changed as part of iSource 11267: End

      		if( !aParam )
      		{
      			iVisibilityState = EFlashPluginInvisible;
      		}
      		else
      		{
      			iVisibilityState = EFlashPluginVisible;
      			// if swf file embedded with in the webpage is not yet
      			// opened then open it. fix for ASHA-7RKHZT.
      			//if(iFocusState == EFlashPluginForeground && !iPlayerInstance->IsPlayerInitialzed() && iPlayerInstance->GetBaseURL())
				if(iFocusState == EFlashPluginForeground && !iPlayerInstance->IsPlayerInitialzed() && iPlayerInstance->DeferredLaunch())
				{
      			    TInt error = iPlayerInstance->OpenMovie();
					if(error != KErrNone)
					{
                        #ifdef FEATURE_COLLECT_BITMAP
	                    iSwfFirstPlay = EFalse;
                        #endif
						return error;
					}
                    #ifdef FEATURE_COLLECT_BITMAP
	                iSwfFirstPlay = ETrue;
                    #endif
				}
      		}
			//In fullscreen and while switching to fullscreen,
      		//we ignore plugininvisible events.
      		#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
      		if(!IsFullScreen() || !iTogglingScreen)
      		#else
      		if(!iTogglingScreen)
      		#endif
      		{
#ifdef FEATURE_COLLECT_BITMAP
      		//If CollectBitmap events are not sent from browser: handle focus change events in NotifyFocusChangeL()
                //If CollectBitmap events are sent from browser: handle scroll events in NotifyScrollL()
                if(iPluginScrollState == EScrollingNone)
                {
                    TRAP_IGNORE(NotifyFocusChangeL((TBool)aParam));
                }
                else
                {
                    TRAP_IGNORE(NotifyScrollL((TBool)aParam));
                }
#else
                TRAP_IGNORE(NotifyFocusChangeL((TBool)aParam));
#endif
      		}
      		NPPDebug(LOG_DEBUG,_L("CBrowserPluginControl::NotifyL() : visibility = [%d]"), iVisibilityState);
            }
         }
        break;

		case EMenuOptionEnableDecider:
		{ //browser menu option enabler /disabler
			if(aParam)
			{
				TInt* menuFlag = static_cast<TInt*>(aParam);
				switch(*menuFlag)
				{
					case EOptionOpenInViewer:
					{
						if(iFLVPlayState == EFlashPluginFLVStreamInProgress)
						{
							*menuFlag = 0; //disable option
						}
					}
					break;
				}
			}
		}
		break;

		default:
		break;
    }

    return retVal;
}

void CBrowserPluginControl::NotifyMp3FocusChangeL(TBool aFocus)
{
    PluginBase::NotifyMp3FocusChangeL(aFocus);
}

void CBrowserPluginControl::NotifyFocusChangeL(TBool aFocus)
{
    if(iMP3StreamState != EFlashNoMP3Stream)
        PluginBase::NotifyMp3FocusChangeL(aFocus);
    else
    {
#ifdef SELF_IAD_UPDATE
		/*if( iIadNotificationState == EUpdateComplete )
		{
			ShowIADUpdateNotification();
			iIadNotificationState = EUserNotified;
		}*/
    	if(FlashPluginInstance()->GetIADProgressState() == EIadUpdateComplete)
    	{
			ShowIADUpdateNotification();
			FlashPluginInstance()->SetIADProgressState(EIadUserNotifiedOnCompletion);
    	}
#endif

#ifdef FEATURE_COLLECT_BITMAP
    	/* If SWF goes to BG while scrolling, browser will not send ECollectBitmap_0 calls
    	 * iPluginScrollState should not reset as this is the only flag to know scrollstate
    	 * Once SWF comes to FG create EGL surface and resume playback reset ScrollState
    	 */
    	if(aFocus && iPluginScrollState == EScrollingStarted)
        {
            ResumeL();
            iPluginScrollState = EScrollingStopped;
            return;
        }
#endif
    	//If plugin is already playing and is in Focus no need to create EGL surface
        if( aFocus && (iPlayState == EFlashPluginPlaying || iVisibilityState == EFlashPluginInvisible) )
		{
			return;
		}
		//Plugin is not in focus and is already paused, no need to clear EGL surface
		if( !aFocus && iPlayState == EFlashPluginPaused )
		{
			return;
		}
		//If swf is in focus and is visible create EGL surface and resume playback
		if( aFocus && ((iFocusState == EFlashPluginForeground) && (iVisibilityState == EFlashPluginVisible)))
		{
            ResumeL();
        }
        //Pause only if not paused.playstate is EFlashPluginInitial in the begining.
        // Pause only when IME control is disabled. This check has been introduced to resolve the
        // VKB popup issue (PRRR-7QG8V8).
        else if((iPlayState == EFlashPluginPlaying || iPlayState ==EFlashPluginInitial) && !iIsIMEEnabled)
        {
            PauseL();
        }

        UpdateVideoScreen();
    }
}

#ifdef FEATURE_COLLECT_BITMAP

void CBrowserPluginControl::NotifyScrollL(TBool aFocus)
{
    if(iMP3StreamState != EFlashNoMP3Stream)
        PluginBase::NotifyMp3FocusChangeL(aFocus);

    #ifdef SELF_IAD_UPDATE
        /*if( iIadNotificationState == EUpdateComplete )
        {
            ShowIADUpdateNotification();
            iIadNotificationState = EUserNotified;
        }*/
        if(FlashPluginInstance()->GetIADProgressState() == EIadUpdateComplete)
        {
            ShowIADUpdateNotification();
            FlashPluginInstance()->SetIADProgressState(EIadUserNotifiedOnCompletion);
        }
    #endif

    //SWF enterts from outside viewport, first bitmap should be captured and handle needs to be sent to browser
    if(iPluginScrollState == EScrollingStarted && iSwfFirstPlay)
    {
        MFlashEngine* engine = FlashPluginInstance()->GetEngine();
        TInt err = engine->ToggleBrowserRendering(EBrowserRenderingMode);
        if(err == KErrNone)
        {
            engine->Pause(EReasonBackground);
            iLastRenderedBitmap = engine->GetLastRenderedBitmap();
            if(iLastRenderedBitmap)
            {
                SendLastRenderedBitmapToBrowser(iLastRenderedBitmap);
                iPlayState = EFlashPluginPausedWhileScrolling;
            }
        }
        iSwfFirstPlay = EFalse;
        return;
    }

    //If swf is in focus and scrolling has started , ignore NotifyL(6,1) calls
    if(aFocus && iPluginScrollState == EScrollingStarted )
    {
        return;
    }

    //If SWF is in focus and playing already , ignore NotifyL(6,1) calls
    if( aFocus && (iPlayState == EFlashPluginPlaying || iVisibilityState == EFlashPluginInvisible) )
    {
        return;
    }

    //Scrollig stopped, now resume playback
    if(iPluginScrollState == EScrollingStopped && iPlayerInstance->IsPlayerInitialzed())
    {
        //Resume playback if swf is visible inside viewport, no need to create EGL surface
        if(iVisibilityState == EFlashPluginVisible && iPlayState == EFlashPluginPausedWhileScrolling)
        {
            MFlashEngine* engine = FlashPluginInstance()->GetEngine();
            engine->ToggleBrowserRendering(EFlashRenderingMode);
            engine->ResumeL(EReasonBackground, ETrue);
            iPlayState = EFlashPluginPlaying;
        }
        //Create EGL surface and resume playback if EGL surface is removed in previous scroll
        else if(iFocusState == EFlashPluginForeground && iVisibilityState == EFlashPluginVisible && iPlayState == EFlashPluginPaused)
        {
            ResumeL();
        }
        //If swf is not visible (out of viewport), remove EGL surface and pause playback
        else if(iVisibilityState == EFlashPluginInvisible)
        {
            PauseL();
        }
    }

    UpdateVideoScreen();
}
#endif
// ----------------------------------------------------------------------
// TBool CBrowserPluginControl::Stop()
//
// This function is used to stop the flash player
// ----------------------------------------------------------------------
//
void CBrowserPluginControl::Stop()
{
    MFlashEngine* engine = FlashPluginInstance()->GetEngine();

	if(engine)
	{
        engine->Stop();
	}
}

TKeyResponse CBrowserPluginControl::OfferKeyEventL(const TKeyEvent& aKeyEvent,
                                                   TEventCode aType)
{
    MFlashEngine* engine = FlashPluginInstance()->GetEngine();
    if (!engine)
    {
        return EKeyWasNotConsumed;
    }


#ifdef ENABLE_CURSOR

    // Consume the event silently if file is not yet opened.

    if( !iFileOpened )
    {
        return EKeyWasConsumed;
    }


    if(iIMEControl && iJustActivatedIME)
    {
    	//when flash is activated, browser cursor may be top of
		//flash edit box. this event will make core to call
		//enableime. Now IME is enabled still our browser
		//control is yet to receive 2 more key events
		//if we send these key events core player will
		//disable ime session. this we want to avoid
		//so consume the event till iJustActivatedIME non zero
    	iJustActivatedIME--;
    	return EKeyWasConsumed;
    }

	// If the plugin is activated, cursor should be displayed.
    // But the cursor is not displayed when inside the IME control.
    // Furthermore, page should not scroll in fullscreen mode.


    if( iPluginActivated && iCursor && ( !iIMEControl))
    {
        iJustActivatedIME = 0;
        if( IsNavigationKey(aKeyEvent) )
        {
            // Display cursor when user presses the navigation key
            if(engine && (!iCursor->IsVisible()))
            {
                // Before creating the cursor, current curson position is set.
                // The cursor will be created at this position.
                iCursor->SetCursorPosition(iSavedCursorPoint);
                iCursor->ShowCursor(iSavedCursorPoint);
            }
            if( aType == EEventKey )
            {
                TBool aBoundaryHit;
                TPoint offset = iCursor->HandleNavigation(aKeyEvent, aType,aBoundaryHit);

                if(aBoundaryHit)
                {
                    TPoint clickPoint = iCursor->Position();
                    TPoint movePoint = offset + clickPoint;
                    iSavedCursorPoint = clickPoint;
                    // Calling HideCursor to hide the cursor
                    iCursor->HideCursor();
                    MNetscapeIf* nsfuncs = iPlayerInstance->NetscapeFns();
                    if( nsfuncs && !iFullScreen )
                    {
                        // Calling HideCursor to hide the cursor
                        // Cursor should not be hidden in case of full screen.
                        iCursor->HideCursor();
                        const int pluginDeActivate(101);
                        nsfuncs->SetValue(pluginDeActivate,
                                                //(void*) &dummy );
                                                (void*) &movePoint );
                        /* This call is to ensure that the cursor is moved out of the flash area and
                                              if any existing hover events are there will be cancelled */
                         engine->OfferPointerEventL(movePoint,TPointerEvent::EMove);
                    }

                }
                else
                {
                    MNetscapeIf* nsfuncs = iPlayerInstance->NetscapeFns();
                    TPoint clickPoint = iCursor->Position();
                    iSavedCursorPoint = clickPoint;
                    if( nsfuncs && !iFullScreen )
                    {
                        const int pluginFocusPosition(100);
                        nsfuncs->SetValue(pluginFocusPosition,(void*) &offset );
                    }
                    engine->OfferPointerEventL( clickPoint , TPointerEvent::EMove );
                }
            }
        }
        else if( IsSelectKey(aKeyEvent) )
        {
            // Display flash cursor when plugin get activated on pressing OK button
            if(engine && PluginBase::GetQwertyKeyPadStatus() && (!iCursor->IsVisible()))
            {
                iCursor->ShowCursor(iSavedCursorPoint);
            }
            TPoint clickPoint = iCursor->Position();
            clickPoint.iX += (iCursor->Size().iWidth/4);
            clickPoint.iY += (iCursor->Size().iHeight/4);

            #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
            TRect  videoRect=engine->GetVideoRectL();
            if(iPlayerInstance->GetAllowMobiFullScreen() &&
                    !IsControlPresent(clickPoint) && aType==EEventKey &&  videoRect.Contains(clickPoint))
            {
                ToggleScreenL();//do full or normal
            }

            #endif

            //This move event is redundant. Hence removed.
            //iEngine->OfferPointerEventL( clickPoint, TPointerEvent::EMove );

            if( aType == EEventKeyDown )
            {
                engine->OfferPointerEventL( clickPoint, TPointerEvent::EButton1Down );
                if(iIMEControl)  // check ime session created
                {
                    //when flash is activated, browser cursor may be top of
                    //flash edit box. this event will make core to call
                    //enableime. Now IME is enabled still our browser
                    //control is yet to receive 2 more key events
                    //if we send these key events core player will
                    //disable ime session. this we want to avoid
                    iJustActivatedIME = 2;
                }

            }
            else if(aType == EEventKeyUp )
            {
                engine->OfferPointerEventL( clickPoint, TPointerEvent::EButton1Up);
            }

            if(!iIMEControl)
                iCursor->UpdateCursor();
        }
        if(!IsNavigationKey(aKeyEvent) && !IsSelectKey(aKeyEvent))
            return CFlashPluginControl::OfferKeyEventL(aKeyEvent, aType);
        else
            return EKeyWasConsumed;
    }
#endif
    return CFlashPluginControl::OfferKeyEventL(aKeyEvent, aType);
}

void CBrowserPluginControl::HandleResourceChange(TInt aType)
{
	MFlashEngine* engine = this->FlashPluginInstance()->GetEngine();
	//need to redraw if windows are faded
	if( (engine) && (aType == KEikMessageWindowsFadeChange))
	{
		engine->RefreshScreen();
	}
}

#ifdef GESTURE_LIBRARY_SUPPORTED

TBool CBrowserPluginControl::HandleGestureFromBrowserL( void* aEvent)
{
      TBool ret = EFalse;

      MFlashEngine* engine = this->iPlayerInstance->GetEngine();

      //If engine is destoryed no need to process gesture we return false and browser will process them
      if(engine && aEvent)
      {
          TPoint pRTS = PositionRelativeToScreen();
		  #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
          if(iFullScreenControl)
          {   //iFullScreenControl seems to be the only reliable flag that indicates if we are in
              //fullscreen or not.

              //Dont handle gestures in fullscreen mode.
              ret = iFullScreenControl->HandleGestureFromBrowserL(aEvent);
              return ret;
          }
		  #endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN

          TGestureEvent *gesture = static_cast<TGestureEvent*>(aEvent);
          //Browser will send position relative to screen we need convert them to flash co-ordinates.
          TPoint  point = gesture->CurrentPos() - pRTS;
          TPoint startpoint = gesture->StartPos()- pRTS;

          TPointerEvent pointerEvent;

          pointerEvent.iPosition = point ;
          TGestureCode gtype =  gesture->Code(EAxisBoth);
          // application need to say which axis to consider?
          #ifdef _GestureDebug_
          NPPDebug(LOG_DEBUG,_L("CBrowserPluginControl::Consumegesture: code %d, pos(%d, %d)"), gtype, point.iX, point.iY);
          #endif //_GestureDebug_
          switch (gtype)
          {
              case EGestureStart:
              {
                  iControlPresent = engine->IsControlPresent(point);
				   #if defined(PLUG_IN_MEM_ALLOC) && defined (__FEATURE_UI_LANDSCAPE_FULLSCREEN)

                  //Commented because we want to pan the page even when video is playing
                    if( /*(!IsPlayingVideo() || //if there is no video in swf, send scroll notif to browser
                         !iControlPresent)&& */
                         (iPlayerInstance->GetBrPluginExecutionMode() == EDefaultMode)
                        )   //if there is video in swf, send scroll notif to browser only if there is no control on the drag point

                   #elif defined(__FEATURE_UI_LANDSCAPE_FULLSCREEN)
				   if( (!IsPlayingVideo() || //if there is no video in swf, send scroll notif to browser
                       !iControlPresent))
                   #elif defined(PLUG_IN_MEM_ALLOC)
				       //if((!iControlPresent)&& iInteractionMode == EDefaultMode )
				       if( (!iControlPresent) &&
				           ( (iPlayerInstance->GetExecutionMode() == eWidget) ||
				             (iPlayerInstance->GetExecutionMode() == eBrowserPlugin)
				           )
				         )
                  #else
				  if(!iControlPresent)
				  #endif
				  {
                      iFlashScroll = EFalse;
                      iBrowserScroll = ETrue;
                  }
                  else
                  {
                     pointerEvent.iType=TPointerEvent::EButton1Down;
                     if(iTactileFeedbackSupported)
                    	 ProvideFeedbackL(pointerEvent);
                     engine->OfferPointerEventL(point, TPointerEvent::EButton1Down);

                     iBrowserScroll = EFalse;
                     iFlashScroll = ETrue;
                  }
                  ret = EFalse;
                  break;
              }
              case EGestureDrag:
              {
                  if(iBrowserScroll)
                  {
                      #ifdef _GestureDebug_
                      NPPDebug(LOG_DEBUG,_L("Browser panning start point = (%d,%d) current point = (%d,%d)"), startpoint.iX,startpoint.iY,point.iX,point.iY);
                      #endif // _GestureDebug_
                      ret = EFalse;
                  }
                  else if(iFlashScroll)//Send the events to coreplayer only if Browser is not scrolled
                  {
                      #ifdef _GestureDebug_
                      NPPDebug(LOG_DEBUG,_L("Drag: In flash"));
                      #endif // _GestureDebug_
                      pointerEvent.iType=TPointerEvent::EDrag;
                      if(iTactileFeedbackSupported)
                    	  ProvideFeedbackL(pointerEvent);
                      engine->OfferPointerEventL(point, TPointerEvent::EDrag);
                      ret = ETrue;
                  }
                  break;
              }
              //This are events generated after up event is received.
              case EGestureSwipeLeft:
              case EGestureSwipeRight:
              case EGestureSwipeUp:
              case EGestureSwipeDown:
              case EGestureFlick:
              case EGestureDrop:
              case EGestureReleased:
              {
                  if (iFlashScroll)
                  {
#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
				  	if(!IsContextMenuVisible()) // Showing the popup menu => gesture consumed. Hence don't pass it to the core
#endif
                    {
                         pointerEvent.iType=TPointerEvent::EButton1Up;
                         if(iTactileFeedbackSupported)
                             ProvideFeedbackL(pointerEvent);
                         engine->OfferPointerEventL(point, TPointerEvent::EButton1Up);
                    }
                  }
                  //Up has been received we reset the flags
                  iFlashScroll = EFalse;
                  iBrowserScroll = EFalse;
                  ret = EFalse;
                  break;
              }
                  // gestures
              case EGestureUnknown:
              {
                  ret = EFalse;
                  break;
              }

              case EGestureDoubleTap:
              {
                #ifdef __FEATURE_TOGGLE_ON_DOUBLE_CLICK__
                  if( IsPlayingVideo() &&
					  // do it only in browser mode, not in cwrt mode
				  	  (iPlayerInstance->GetBrPluginExecutionMode()  == EDefaultMode)
				  	)
                      {
                   	  NPPDebug(LOG_DEBUG,_L("toggle screen"));
                	  // toggle between fullscreen and normal screen
                	  ToggleScreenL();
                	  ret = ETrue;

					  break;
                      }
				#endif
                ret = EFalse;
				break;
              }
              case EGestureTap:
              {
                  //if flashscroll is not set send Down followed by Up.otherwise dont send anything
                  //Down is already sent up will be sent when released
                  if(!iFlashScroll)
                  {
                      pointerEvent.iType=TPointerEvent::EButton1Down;
                      if(iTactileFeedbackSupported)
                    	  ProvideFeedbackL(pointerEvent);
                      engine->OfferPointerEventL(point, TPointerEvent::EButton1Down);
                      pointerEvent.iType=TPointerEvent::EButton1Up;
                      engine->OfferPointerEventL(point, TPointerEvent::EButton1Up);

                  }

                  if(iIMEControl)
                  {
                      iIMEControl->LaunchVKbL();
                  }

                  #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
#ifndef __FEATURE_CONTEXT_SENSITIVE_MENU__
                  iControlPresent = engine->IsControlPresent(point);
                  if((!iControlPresent)
				  	&&  (iPlayerInstance->GetBrPluginExecutionMode()  != EWidgetMode))
                  {
                     TRect  videoRect=engine->GetVideoRectL();

                     // If Ph - 1, just provide the feedback and toggle
                      if(!engine->IsHelixFlvMode())
                         ToggleScreenL();
                     // If ph - 2, if no videorect toggle, otherwise check the point and then toggle.
                      else if( (/*videoRect.IsEmpty() ||*/ videoRect.Contains(point)))   //removed condition as even some swfs with out video were toggling
                         ToggleScreenL();
                  }
#endif // __FEATURE_CONTEXT_SENSITIVE_MENU__
                  #endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN
                  ret = ETrue;
                  break;
              }
              case EGestureLongTap:
              {
                  iLongTap = ETrue;
            	  NPPDebug(LOG_DEBUG,_L("Gesture longtap"));
            	  TBool sendToPlayer = ETrue;
#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
                  if( IsPlayingVideo() &&
				  	  (iPlayerInstance->GetBrPluginExecutionMode()  == EDefaultMode)
				  	)
                  {
                      NPPDebug(LOG_DEBUG,_L("Showing context sensitive menu"));
                      // show context menu only incase of video
                      DynInitContextSensitiveMenu(this,point + pRTS);
                      sendToPlayer = EFalse;
                  }
#endif
                  //if flash scroll is not set send down
                  if(sendToPlayer && !iFlashScroll)
                  {
                      pointerEvent.iType=TPointerEvent::EButton1Down;
                      if(iTactileFeedbackSupported)
                    	  ProvideFeedbackL(pointerEvent);
                      engine->OfferPointerEventL(point, TPointerEvent::EButton1Down);
                      iFlashScroll = ETrue;
                      iBrowserScroll=EFalse;
                  }

                  ret = ETrue;
                  break;
              }

              default:
              {
                  ret = EFalse;
                  break;
              }
          }
      }
      return ret;
}

// ---------------------------------------------------------------------------
// void CBrowserPluginControl::ResetPointerBuffer()
//
// clear the pointer move buffer if window has changed
// ---------------------------------------------------------------------------
//

void CBrowserPluginControl::ResetPointerBuffer()
{
    TPoint swfWindowDisplaement(0,0);
    if(iCurrentPositionNotSet)
     {
         iCurrentPositionRelativeToScreen = PositionRelativeToScreen();
         iCurrentPositionNotSet =EFalse;
     }

    swfWindowDisplaement = PositionRelativeToScreen() - iCurrentPositionRelativeToScreen;

    if(abs(swfWindowDisplaement.iX) > 0 || abs(swfWindowDisplaement.iY) >0)
    {
        NPPDebug(LOG_DEBUG,_L("PointerBuffer Reset"));
        Window().DiscardPointerMoveBuffer();
        iCurrentPositionRelativeToScreen = PositionRelativeToScreen();
    }
}

#ifdef FEATURE_COLLECT_BITMAP
void CBrowserPluginControl::SendLastRenderedBitmapToBrowser(const CFbsBitmap* aBitmap)
{
    //If broken icon is displayed dont send any bitmap to browser
    if(iErrorOccured && iErrorIsCritical)
    {
        return;
    }

    if(iPlayerInstance->GetEngine()->GetSwfHasVideo())
    {
        return;
    }

    MNetscapeIf* nsfuncss = iPlayerInstance->NetscapeFns();
    if(nsfuncss)
    {
        const int bitmapEvent(NPPVPluginBitmap);

	    int handle(KErrNone);
	    if(aBitmap)
	    {
	        handle = aBitmap->Handle();
	        nsfuncss->SetValue(bitmapEvent,(void*) &(handle));
	    }
    }
}
#endif

void CBrowserPluginControl::SendPointerEventToBrowserL(const TPointerEvent &aPointerEvent)
{
     #ifdef _GestureDebug_
     NPPDebug(LOG_DEBUG,_L("CBrowserPluginControl::SendPointerEventToBrowserL ++"));
     #endif

     //We need to send absolute co-ordinates to Browser
     TPoint point(aPointerEvent.iPosition  + PositionRelativeToScreen());
     MNetscapeIf* nsfuncss = iPlayerInstance->NetscapeFns();
     //NPNetscapeFuncs* nsfuncss = iPlayerInstance->GetGlobals()->NetscapeFuncs();
     //We need to send absolute co-ordinates to Browser
     //This has to be under compi;e time flag need to check
#ifdef ADVANCEDPOINTEREVENT_H
     if(aPointerEvent.AdvancedPointerEvent())
     {
         TAdvancedPointerEvent tmpEvent = *(static_cast<const TAdvancedPointerEvent *>(&aPointerEvent));
         tmpEvent.iPosition = point;
         if(nsfuncss)
          {

          const int pluginPointerEvent(103);
		         nsfuncss->SetValue(pluginPointerEvent,
		                             (void*) &(tmpEvent));
          }
     }
     else
         {
#endif
		     TPointerEvent tmpEvent(aPointerEvent);
		     tmpEvent.iPosition = point;

		     #ifdef _GestureDebug_
		     NPPDebug(LOG_DEBUG,_L(" CPlgControl::SendPointerEventToBrowserL PointerType = %d, point(%d %d)"), aPointerEvent.iType,point.iX,point.iY);
		     #endif  //_GestureDebug_



		     if(nsfuncss)
		     {
		         const int pluginPointerEvent(103);
		         nsfuncss->SetValue(pluginPointerEvent,
		                             (void*) &(tmpEvent));
		     }
#ifdef ADVANCEDPOINTEREVENT_H
         }
#endif
     #ifdef _GestureDebug_
     NPPDebug(LOG_DEBUG,_L(" CBrowserPluginControl::SendPointerEventToBrowserL --"));
     #endif

}

#else
void CBrowserPluginControl::SendPointerEventToGestureDetectorL(const TPointerEvent &aPointerEvent)
{
        #ifdef _GestureDebug_
        //NPPDebug(LOG_DEBUG,_L(" CBrowserPluginControl::SendPointerEventToGestureDetectorL ++"));
        #endif

        TPoint point(aPointerEvent.iPosition);
        #ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
        if(iSuppressor && iSuppressor->SuppressPointerEvent(aPointerEvent))
        {
            #ifdef _GestureDebug_
            NPPDebug(LOG_DEBUG,_L(" CBrowserPluginControl::SendPointerEventToGestureDetectorL (Event suppressed) --"));
            #endif
            return;
        }
            #endif

        if(iGestureDetector)
        {
        iGestureDetector->HandlePointerEventL(aPointerEvent);
        }
        #ifdef _GestureDebug_
        //NPPDebug(LOG_DEBUG,_L(CBrowserPluginControl::SendPointerEventToGestureDetectorL --"));
        #endif

}
void CBrowserPluginControl::HandleGestureL(CGestureDetector::TGesture aGesture,const TPointerEvent &aPointerEvent)
{
    MFlashEngine* engine = FlashPluginInstance()->GetEngine();

    if(iFullScreenControl)
    {   //iFullScreenControl seems to be the only reliable flag that indicates if we are in
        //fullscreen or not.

        //Dont handle gestures in fullscreen mode.
        return;
    }

    TPoint point = aPointerEvent.iPosition;
    TPoint pointRelativeToScreen = point + PositionRelativeToScreen();

    #ifdef _GestureDebug_
    NPPDebug(LOG_DEBUG,_L(" CBrowserPluginControl::Handlegesture ++"));
    NPPDebug(LOG_DEBUG,_L("gesture received = %d"), aGesture);
    #endif//  _GestureDebug_
    TPoint offset;
    switch(aGesture)
    {
       case CGestureDetector::EGestureLongTap://Enable Flash scrolling if it is a long tap
    	   if(iTactileFeedbackSupported)
    		   ProvideFeedbackL(aPointerEvent);
           switch(aPointerEvent.iType)
           {

               case TPointerEvent::EButton1Down:
                   iFlashScroll = ETrue;
                   ipreviousPoint = pointRelativeToScreen;
                   //send down event tp core player only for long tap
                   engine->OfferPointerEventL(point, TPointerEvent::EButton1Down);
                   break;
               case TPointerEvent::EButton1Up:
                   iFlashScroll = EFalse;
                   ipreviousPoint = pointRelativeToScreen;
                   //send Up event tp core player only for long tap
                   engine->OfferPointerEventL(point, TPointerEvent::EButton1Up);
                   break;
               case  TPointerEvent::EDrag:
                   #ifdef _GestureDebug_
                   NPPDebug(LOG_DEBUG,_L("Drag: In flash"));
                   #endif// _GestureDebug_
                   engine->OfferPointerEventL(point, TPointerEvent::EDrag);
                   ipreviousPoint  = pointRelativeToScreen;
                   break;
           }
		    // For the first time, VKB will get launched on a call to function IMEFepStateReady()
                    // but if user tap on inline editor again when it is already in focus IMEFepStateReady
                    // won't get called.
                    HandPointerEventToImeL(aPointerEvent);
           break;

       case CGestureDetector::EGestureDrag://Here we scroll Browser page .Exception is flv video case.
           switch(aPointerEvent.iType)
           {
                case TPointerEvent::EButton1Down:
                {
                       ipreviousPoint = pointRelativeToScreen ;
                       iControlPresent = engine->IsControlPresent(point);
					   #ifdef PLUG_IN_MEM_ALLOC
                       if( (!IsPlayingVideo() || //if there is no video in swf, send scroll notif to browser
                           !iControlPresent)&&
                           ( (iPlayerInstance->GetExecutionMode() == eWidget) ||
                             (iPlayerInstance->GetExecutionMode() == eBrowserPlugin)
                           )
                         )   //if there is video in swf, send scroll notif to browser only if there is no control on the drag point
					   #else
					   if( (!IsPlayingVideo() || //if there is no video in swf, send scroll notif to browser
                           !iControlPresent))
					   #endif
                       {
                           iFlashScroll = EFalse;
                           iBrowserScroll = ETrue;
                       }
                       else
                       {
                    	   if(iTactileFeedbackSupported)
                    		   ProvideFeedbackL(aPointerEvent);
                          engine->OfferPointerEventL(point, TPointerEvent::EButton1Down);

                          iBrowserScroll = EFalse;
                          iFlashScroll = ETrue;
                       }

                       /*iPreviousPointInvalid = EFalse;
                       iFirstDrag = ETrue;*/
                       break;
                }

                case TPointerEvent::EDrag:
                    if(iBrowserScroll)
                    {
                        offset = ipreviousPoint - pointRelativeToScreen;
                        NPNetscapeFuncs* nsfuncs = iPlayerInstance->GetGlobals()->NetscapeFuncs();
                        if(nsfuncs &&
                           !(offset.iX == 0 && offset.iY == 0))
                        {

                            (nsfuncs->SetValue)(iPlayerInstance->GetInstance(),
                                 (NPPVariable) NPPVPluginFocusPosition,
                                 (void*) &offset);
                        }
                        ipreviousPoint = pointRelativeToScreen;

                    }
                             }
                    else if(iFlashScroll)//Send the events to coreplayer only if Browser is not scrolled
                    {
                       #ifdef _GestureDebug_
                       NPPDebug(LOG_DEBUG,_L("Drag: In flash"));
                       #endif//  _GestureDebug_

                       if(iTactileFeedbackSupported)
                    	   ProvideFeedbackL(aPointerEvent);

                       engine->OfferPointerEventL(point, TPointerEvent::EDrag);
                       ipreviousPoint  = pointRelativeToScreen;
                    }
                    break;


                case  TPointerEvent::EButton1Up:
                    if (iFlashScroll)
                    {
                    	if(iTactileFeedbackSupported)
                    		ProvideFeedbackL(aPointerEvent);

						engine->OfferPointerEventL(point, TPointerEvent::EButton1Up);
                        iFlashScroll = EFalse;
                    }
                    else if(iBrowserScroll)
                        iBrowserScroll = EFalse;
                    break;
           }
           break;

       case CGestureDetector::EGestureTap:
           //If it is a tap send Down followed by up to core player
    	   if(iTactileFeedbackSupported)
    		   ProvideFeedbackL(aPointerEvent);

           engine->OfferPointerEventL(point, TPointerEvent::EButton1Down);
           engine->OfferPointerEventL(point, TPointerEvent::EButton1Up);
			if(iIMEControl)
            {
               iIMEControl->LaunchVKbL();
            }
            iControlPresent = engine->IsControlPresent(aPointerEvent.iPosition);
			#ifdef PLUG_IN_MEM_ALLOC
            if(!iControlPresent &&
               //iInteractionMode != EWidgetMode)//Dont toggle in widget mode
               iPlayerInstance->GetExecutionMode() != eWidget )
			#else
			if(!iControlPresent)
			#endif
            {
                   #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
                           TRect  videoRect=engine->GetVideoRectL();

                           // If Ph - 1, just provide the feedback and toggle
                           if(!engine->IsHelixFlvMode())
                               ToggleScreenL();
                           // If ph - 2, if no videorect toggle, otherwise check the point and then toggle.
                           else if( (/*videoRect.IsEmpty() ||*/ videoRect.Contains(point)))  //do not toggle screen if video rect is empty
                               ToggleScreenL();
                   #endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN
                }
                iBrowserScroll = EFalse;
                iFlashScroll  =EFalse;
		// For the first time, VKB will get launched on a call to function IMEFepStateReady()
                // but if user tap on inline editor again when it is already in focus IMEFepStateReady
                // won't get called.
                HandPointerEventToImeL(aPointerEvent);
                break;
            case CGestureDetector::EGestureDoubleTap:
                #ifdef _GestureDebug_
                NPPDebug(LOG_DEBUG,_L("EGestureDoubleTap"));
                #endif // _GestureDebug_
                //In Double we zoom the page no other action is taken
                NPNetscapeFuncs* nsfuncss = iPlayerInstance->GetGlobals()->NetscapeFuncs();

                // Zooming on double tap may cause unexpected crashs in Tube Pr2.0
                // hence disabling double tap
                // TSW ID: ESJY-7RC7M8
                //if(nsfuncss && (!iIsTubePR20) && iInteractionMode != EWidgetMode)//Dont zoom in widget mode
                if(nsfuncss && (!iIsTubePR20) && iPlayerInstance->GetExecutionMode() != eWidget)
                    {
                    TPoint absCoord = point + PositionRelativeToScreen();
                    (nsfuncss->SetValue)(iPlayerInstance->GetInstance(),
                                         (NPPVariable) NPPVPluginZoom,
                                         (void*) &(absCoord));
                    }
                break;
            }
        #ifdef _GestureDebug_
        NPPDebug(LOG_DEBUG,_L(" CBrowserPluginControl::Handlegesture --"));
        #endif // _GestureDebug_
    }
#endif // GESTURE_LIBRARY_SUPPORTED

/*
NPP CBrowserPluginControl::GetPlayerInstance()
{
    return iPlayerInstance->GetInstance();
}
*/
void CBrowserPluginControl::LaunchVKbL()
{
    if(iIMEControl)
    {
        iIMEControl->LaunchVKbL();
    }
}
void CBrowserPluginControl::HandlePointerEventL(const TPointerEvent &aPointerEvent)
{
	if(FeatureManager::FeatureSupported(KFeatureIdPenSupport)) //adding feature manager to check the feature at run time.
	{
        MFlashEngine* engine = this->iPlayerInstance->GetEngine();

        /*
         * Events need to be sent to Browser even on error (Broken flash icon) to support
         * scrolling and panning.
         * If not when we show Broken flash icon in a swf covering the entire screen
         * we cannot scroll/pan the page and user will be forced to close the browser.
         */
		if(!engine && iErrorOccured && iPreparingToExit)
		{
			SendPointerEventToBrowserL(aPointerEvent);
			return;
		}

		// if the engine is already destroyed then we do not handle the pointer event
	    // if pen is not enabled, we do not handle the pointer event
        //if we are in fullscreen control we should not reach here.
#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
	if (!AknLayoutUtils::PenEnabled() || !engine || iFullScreenControl )
#else
	if (!AknLayoutUtils::PenEnabled() || !engine)
#endif
	   return;


        #ifdef GESTURE_LIBRARY_SUPPORTED
	    /*The combination event of 'long tap & drag' is given to the core player.*/
        if(iLongTap && aPointerEvent.iType != TPointerEvent::EButton1Up)
        {
            TPoint point(aPointerEvent.iPosition);
            if(iTactileFeedbackSupported)
                ProvideFeedbackL(aPointerEvent);
            if (engine)
                engine->OfferPointerEventL(point, aPointerEvent.iType);
        }
        else
        {
            iLongTap = EFalse;
            SendPointerEventToBrowserL(aPointerEvent);
        }
        #else

        SendPointerEventToGestureDetectorL(aPointerEvent);

        #endif //GESTURE_LIBRARY_SUPPORTED

        //Store the position of point for cursor.
        #ifdef ENABLE_CURSOR
	    iSavedCursorPoint = aPointerEvent.iPosition;


	    switch (aPointerEvent.iType)
	    {
			    case TPointerEvent::EButton1Down:
			    {

					// hide cursor when user taps on Flash content even if Qwerty keypad is ON
					if(engine && iCursor && iPluginActivated && iCursor->IsVisible() && !iIMEControl)
					{
					    // Set the current position. Whenever the cursor is shown up,
					    // it will appear at this position.
                        iCursor->SetCursorPosition(iSavedCursorPoint);
						iCursor->HideCursor();
					}
					break;
		     	}
			    case TPointerEvent::EButton1Up:
			    case TPointerEvent::EDrag:
			    {
			    	break;
				}
		}
        #endif //ENABLE_CURSOR

	}
}


void CBrowserPluginControl::HandPointerEventToImeL(const TPointerEvent& aPointerEvent)
{
#ifdef FEATURE_INLINE_TEXT_ENTRY
    if(iIMEControl)  // check ime session created
    {
        iIMEControl->HandlePointerEventL(aPointerEvent);
    }
#endif
}


void CBrowserPluginControl::HandleWsEventL(const TWsEvent& aEvent,CCoeControl* aDestination)
{
    if(aEvent.Type() == EEventPointer)
    {
        if(aEvent.Pointer()->iType == TPointerEvent::EButton1Down && aDestination != this && !(Window().IsFaded()))
        {
            #ifdef FEATURE_INLINE_TEXT_ENTRY
            SimulateDisableIMEEventL();
            #endif
        }
    }
}


void CBrowserPluginControl::AddEventObserverToControl()
{
    iEventMonitor = ((CAknAppUi*)ControlEnv()->AppUi())->EventMonitor();

    if(iEventMonitor)
    {
        TRAP_IGNORE(iEventMonitor->AddObserverL(this));
        if (iEventMonitor)
            iEventMonitor->Enable(ETrue);
    }
}


void CBrowserPluginControl::RemoveEventObserverFromControl()
{
    if(iEventMonitor)
        iEventMonitor->RemoveObserver(this);
}

#ifdef ENABLE_CURSOR

TBool CBrowserPluginControl::IsNavigationKey(const TKeyEvent& aKeyEvent) const
{
    switch(aKeyEvent.iScanCode)
    {
        case EStdKeyLeftArrow:
        case EStdKeyRightArrow:
        case EStdKeyUpArrow:
        case EStdKeyDownArrow:
#ifdef  ENABLE_OFN_SUPPORT
        case EStdKeyLeftUpArrow:
        case EStdKeyRightUpArrow:
        case EStdKeyLeftDownArrow:
        case EStdKeyRightDownArrow:
#endif
            return ETrue;
        default:
            return EFalse;
    }
}

TBool CBrowserPluginControl::IsSelectKey(const TKeyEvent& aKeyEvent) const
{
    return aKeyEvent.iScanCode == EStdKeyDevice3 ? ETrue : EFalse;
}

#endif

// ---------------------------------------------------------------------------
// void CBrowserPluginControl::SetVideoDisplayRectL(const TRect& /*aRect*/,
//												    const TRect& aClipRect)
//
// Sets the swf rect and clip rect to engine so that it can set the video display rect.
// ---------------------------------------------------------------------------
//

void CBrowserPluginControl::SetVideoDisplayRectL(const TRect& /*aRect*/,
												 const TRect& aClipRect)
{
    MFlashEngine* engine = this->iPlayerInstance->GetEngine();

	if( !engine || iTogglingScreen || iFullScreen )
	{
		return;
	}

	// Set the cliprect in case not set earlier.
	iClipRect = aClipRect;

	// Get the rectangle in absolute co-ordinates. (They will not be relative to plg window origin)
	iSwfRect  		= GetRect();
	iSwfClipRect 	= GetClipRectFromScreenOrigin();



    //Reset pointer buffer if control has moved
#ifdef GESTURE_LIBRARY_SUPPORTED
	ResetPointerBuffer();
#endif //#ifdef GESTURE_LIBRARY_SUPPORTED

	engine->SetVideoDisplayRectL(iSwfRect, iSwfClipRect);
}


// ---------------------------------------------------------------------------
// void CBrowserPluginControl::UpdateVideoScreen()
//
// Updates the helix video player screen (DSA screen)
// ---------------------------------------------------------------------------
//

void CBrowserPluginControl::UpdateVideoScreen()
{
    MFlashEngine* engine = this->iPlayerInstance->GetEngine();

	if( engine )
	{
        engine->UpdateVideoScreen();
	}
}


TBool CBrowserPluginControl::IsFullScreenMode()
{
	return iFullScreen;
}

#ifdef __FEATURE_PLUGIN_UI_OPTIONS

//TBool CBrowserPluginControl::CheckURLReqForFullScreen(const TDesC& aUrl)
TBool CBrowserPluginControl::HandleURLReqForFullScreen(const TDesC& /*aUrl*/)
{
    #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
        if(iFullScreenHandler && !IsFullScreen())
        {
            iFullScreenHandler->IssueRequest();
        }
    #endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN

    return EFalse; //TODO: Does return value matter here?
}
#endif

// ----------------------------------------------------------------------
// TBool CBrowserPluginControl::QueryAllowDeviceIDL()
//
// This function is used to display a query that the swf is trying
// to access the phone's device id
// ----------------------------------------------------------------------
//
TBool CBrowserPluginControl::QueryAllowDeviceIDL()
{
/*	HBufC* pMessage = iEikonEnv->AllocReadResourceLC(R_STR_ALLOW_DEVICE_ID);
	TBool rval=QueryAllowDenyL(*pMessage);
	CleanupStack::PopAndDestroy(pMessage);
*/
	return ETrue; //by default allow device id access
}

// ----------------------------------------------------------------------
// TBool CBrowserPluginControl::QueryAllowNetAccessL()
//
// This function is used to display a query that the swf is trying
// to access the network
// ----------------------------------------------------------------------
//
TBool CBrowserPluginControl::QueryAllowNetAccessL()
{
// commented need to check VodaKK REQ
//{	HBufC* pMessage = iEikonEnv->AllocReadResourceLC(R_STR_ALLOW_NWORK_ACCESS);
//	TBool rval=QueryAllowDenyL(*pMessage);
//	CleanupStack::PopAndDestroy(pMessage);
//	return rval;
//}
	TBool allowNetworkAccess = iPlayerInstance->QueryAllowNetworkAccess();
	return allowNetworkAccess;
}
// ----------------------------------------------------------------------
// TBool CBrowserPluginControl::QueryAllowPhoneCallL(const TDesC&
//        aNumberString)
//
// This function is used to display a dialog to the user that the swf
// is trying to make a phone call
// ----------------------------------------------------------------------
//
TBool CBrowserPluginControl::QueryAllowPhoneCallL(const TDesC& aNumberString)
{
    TBufC<128> KResFile(_L("FlashlitePlugin.rsc") );
    //TInt resOffset = load_pluginresourceL( KResFile );
    TInt resOffset = PluginBase::LoadPluginResourceL( KResFile );
	TBool query = PluginBase::QueryAllowDenyWithFormatResourceL(aNumberString,
	        R_STR_ALLOW_PHONE_CALL);

	CEikonEnv::Static()->DeleteResourceFile(resOffset);
	return(query);
}



//// ----------------------------------------------------------------------
//// TBool CBrowserPluginControl::IsTextInputBusy()
////
//// This function is used to check text input is running.
//// ----------------------------------------------------------------------
////
//TBool CBrowserPluginControl::IsTextInputBusy()
//{
//	return iTextInput->IsBusy();
//}
//
//// ----------------------------------------------------------------------
//// void CBrowserPluginControl::GetInputTextL(TGetInputTextData& aData)
////
//// This function is used to get text input from user
//// ----------------------------------------------------------------------
////
//
//void CBrowserPluginControl::GetInputTextL(TGetInputTextData& aData)
//{
//	iTextInput->GetInputTextL(aData);
//}

//-----------------------------------------------------------------------
// void CBrowserPluginControl::NotifyError(TInt aErrorCode)
//
// This function is called when there is an error while opening
// a flash file in the screensaver mode
//-----------------------------------------------------------------------

void CBrowserPluginControl::NotifyError(TInt aErrorCode)
{
	NPPDebug(LOG_DEBUG,_L("CBrowserPluginControl::NotifyError() : ERROR = [%d]"), aErrorCode);

	PluginBase::SetErrorOccured(aErrorCode);
}

void CBrowserPluginControl::NotifyClientError(TInt aErrorCode,
                                              TBool aIsCritical,
                                              TBool /*aPersistent*/)
{
   	NPPDebug(LOG_ERROR,_L("CBrowserPluginControl::NotifyClientError() : ERROR = [%d]"), aErrorCode);
	iLastError = aErrorCode;
	iErrorIsCritical = aIsCritical;

	if( !aIsCritical )
    {
	    return;
    }
	PluginBase::SetErrorOccured(aErrorCode);
    iPlayerInstance->PrepareToExit();
}

//-----------------------------------------------------------------------
// void CBrowserPluginControl::HandleVideoError(const TInt& aError)
//
// Video Error Handler.
//-----------------------------------------------------------------------

void CBrowserPluginControl::HandleVideoError(const TInt& aError)
{
	NPPDebug(LOG_ERROR,_L("CBrowserPluginControl::HandleVideoError() : ERROR = [%d]"), aError);

	if(iPluginActivated)
	{
		TRAP_IGNORE(HandlePluginDeactivatedL(NULL));

		MNetscapeIf* nsfuncs = iPlayerInstance->NetscapeFns();

		if( nsfuncs )
	    {

	    	TPoint CurPt(0,0);
	    	#ifdef ENABLE_CURSOR
	    	if(iCursor)
	    	{
	    		  CurPt = iCursor->Position();
	    	}
	    	#endif
	    	const int pluginDeActivate(101);
		        nsfuncs->SetValue(pluginDeActivate,
		        		(void*) &CurPt );
		}
	}


	iLastError = aError;
	iErrorIsCritical = ETrue;

	PluginBase::SetErrorOccured(aError);

	iPlayerInstance->PrepareToExit();
}

void CBrowserPluginControl::ChangeFLVPlayStateL(EFLVStreamState aFLVStreamState, EMediaStreamType aMediaStreamType)
{
	iFLVPlayState = aFLVStreamState;

	if(aMediaStreamType == EVideoStream)
	{
		TBufC<128> KResFile(_L("FlashlitePlugin.rsc") );
    //TInt resOffset = load_pluginresourceL( KResFile );
		TInt resOffset = PluginBase::LoadPluginResourceL( KResFile );
        switch(aFLVStreamState)
        {
            case EFlashPluginFLVStreamInProgress:
            {
                #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
                    FLVideoStarted();

                    TPoint curPoint;
                    TRect aRect= GetRect();
                    curPoint.iX = (aRect.iTl.iX+aRect.iBr.iX)/2;
                    curPoint.iY = (aRect.iTl.iY+aRect.iBr.iY)/2;

//#ifdef ENABLE_CURSOR
//				if(iPlayerInstance->GetAllowMobiFullScreen() &&
//						!IsFullScreen() && !(iCursor && iCursor->CheckControl(curPoint)) && (iPlayerInstance->GetBrPluginExecutionMode() != EWidgetMode))
//#else
//								if (!IsFullScreen() && (iPlayerInstance->GetBrPluginExecutionMode() != EWidgetMode))
//#endif
//				{//Tooltip disabled as web services UX has Pause/resume function attached to video area.
//					ShowTipsL(R_TEXT_FLITE_PLUGIN_TIP_FULLSCREEN);
//				}

			#endif

            }
            break;

            case EFlashPluginFLVStreamClosed:
            {
                #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
                if(IsFullScreen())
                {
                    TPoint curPoint;
                    TRect aRect= GetRect();
                    curPoint.iX = (aRect.iTl.iX+aRect.iBr.iX)/2;
                    curPoint.iY = (aRect.iTl.iY+aRect.iBr.iY)/2;

    //#ifdef ENABLE_CURSOR
    //										if(!(iCursor && iCursor->CheckControl(curPoint)))
    //#endif
    //				{//Tooltip disabled as web services have overloaded Pause/Resume function on
    //					ShowTipsL(R_TEXT_FLITE_PLUGIN_TIP_NORMALSCREEN_RSK);
    //				}
                }
                #endif
            }
            break;
        }

        CEikonEnv::Static()->DeleteResourceFile(resOffset);
	}
}

RWindow* CBrowserPluginControl::GetWindow()
{

#ifdef __FEATURE_PLUGIN_UI_OPTIONS
	if(IsFullScreen() && iFullScreenControl)
	{
		return &(iFullScreenControl->Window());
	}
	else
#endif
	{
		return &Window();
	}

}

RWindow* CBrowserPluginControl::GetNewVideoWindow(void* /*aClient*/)
{
    RWindow* videoWindow = NULL;
    MFlashEngine* engine = iPlayerInstance->GetEngine();

#ifdef USE_VPU2
    if(!iVideoSurfaceContainer && engine && !engine->IsSwitchingToSWRenderingInPrgress() && engine->IsOpenVgRenderingMode())
    {
        TRAP_IGNORE(
            if(IsFullScreen() && iFullScreenControl)
            {
                iVideoSurfaceContainer = CSurfaceContainerControl::NewL((RWindowTreeNode*)&(iCoeEnv->RootWin()),
                                                                        TRect(iFullScreenControl->Window().Position(),
                                                                        iFullScreenControl->Window().Size()));
            }
            else
            {
                CCoeControl* ctrl = this->Parent();
                iVideoSurfaceContainer = CSurfaceContainerControl::NewL(ctrl->DrawableWindow(),TRect(Window().Position(),Window().Size()));
            }

            Window().SetOrdinalPosition(0);
            iVideoSurfaceContainer->GetWindow().SetOrdinalPosition(Window().OrdinalPosition() + 1);
            videoWindow = &(iVideoSurfaceContainer->GetWindow());
        );
    }
#endif
    if(videoWindow == NULL)
        videoWindow = GetWindow();

    return videoWindow;
}

void CBrowserPluginControl::CloseVideoWindow(void* /*aClient*/)
{
#ifdef USE_VPU2
    delete iVideoSurfaceContainer;
    iVideoSurfaceContainer = NULL;
#endif
}

TBool CBrowserPluginControl::IsFLVVideo()
{
	#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN

	 return ((iFLVPlayState == EFlashPluginFLVStreamInProgress) || (iCanGoFullScrn));

	#else
		return EFalse;
	#endif
}

#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN

TBool CBrowserPluginControl::IsFullScreen()
{
	return iFullScreen;
}

TBool CBrowserPluginControl::IsPlayingVideo()
{
	return iCanGoFullScrn;
}

#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
CSurfaceContainerControl* CBrowserPluginControl::GetVideoContainer()
{
    CSurfaceContainerControl* ctrl = NULL;
#ifdef USE_VPU2
    ctrl = iVideoSurfaceContainer;
#endif
    return ctrl;
}

void CBrowserPluginControl::SetVideoContainer(
#ifdef USE_VPU2
		CSurfaceContainerControl* aVideoContainer
#endif
)
{
#ifdef USE_VPU2
    iVideoSurfaceContainer = aVideoContainer;
#endif
}

TBool CBrowserPluginControl::GoFullScreenL()
{
    #ifdef ENABLE_CURSOR

    // Save the original point before you switch the mode.
    if( iCursor )
    {
    	iSavedCursorPoint = iCursor->Position();
    }

    #endif

	iFullScreen = ETrue;

	if(iTactileFeedbackSupported) //adding feature manager to check the feature at run time.
	{
		if( iFeedback  && iFeedback->TouchFeedbackSupported() )
		{
			iFeedback->InstantFeedback(ETouchFeedbackSensitive);
		}
	}

	MFlashEngine* engine = iPlayerInstance->GetEngine();
	if(!engine)
	    return EFalse;
	engine->PauseVideo();

	iTogglingScreen = ETrue;

	// changed for defect :ESJY-7PB9NN :Start
    // added to send notification to the Browser that the orientation should be changed to
    // full screen. Flashlite will not change the orientation
    TBool temp =   ETrue;
    MNetscapeIf* nsfuncs = iPlayerInstance->NetscapeFns();
    if (nsfuncs)
    {
        const int NPPVpluginFullScreenBool(17);
        nsfuncs->SetValue(NPPVpluginFullScreenBool,(void*) &temp );
    }

    // changed for defect :ESJY-7PB9NN :End

    if ( !iFullScreenControl )
	{
		// Handles all key presses
		iFullScreenControl = CBrowserPluginFullScreenControl::NewL(this, engine);
#ifdef ENABLE_CURSOR
		// Perform this operation if Cursor is not hidden
        if(iCursor && (iCursor->State()!= EFlashCursorHidden))
        {
            iCursor->FullScreenMode(iFullScreenControl);
            iCursor->HideCursor();
        }
#endif
	}
	iTogglingScreen = EFalse;

    engine->NotifyFullScreen(TRUE);
	engine->ResumeVideo();
	return ETrue;
}


void CBrowserPluginControl::ToggleScreenL()
{
	HideTips();

	if(IsFullScreen())
	{
		iFullScreen = EFalse;
		//GoNormalScreenL();
		if(!iFullScreenHandler)
		{
            iFullScreenHandler = new(ELeave) CAsyncFullScreenHandler(this);
		}

        iFullScreenHandler->IssueRequest(EFalse);
	}
	else if(iCanGoFullScrn)
	{
		//GoFullScreenL();
		if(!iFullScreenHandler)
		{
        	iFullScreenHandler = new(ELeave) CAsyncFullScreenHandler(this);
		}

        iFullScreenHandler->IssueRequest(ETrue);
	}
}


TBool CBrowserPluginControl::GoNormalScreenL()
{
    TPoint curPoint(0,0);

    MFlashEngine* engine = iPlayerInstance->GetEngine();
    engine->PauseVideo();

    if(!engine)
        return EFalse;

	iTogglingScreen = ETrue;

#ifdef FEATURE_OPENVG
	//Causes EGL surface to be recreated.
	//Necessary to trigger this before fullscreen
	//window is destroyed. otherwise, egl surface will be orphaned
	//without a window and may crash.
    engine->SetEglContainerWindow(&Window());
#endif

#ifdef USE_VPU2
    engine->SetVideoWindow(NULL);

    if(iVideoSurfaceContainer)
    {
        delete iVideoSurfaceContainer;
        iVideoSurfaceContainer = NULL;
    }
#endif

	if(iFullScreenControl)
	{
		delete iFullScreenControl;
		iFullScreenControl = NULL;
	}

    iFullScreen = EFalse;

#ifdef USE_VPU2
    if(engine->IsOpenVgRenderingMode() && !engine->IsSwitchingToSWRenderingInPrgress())
    {
        TRAP_IGNORE(
            CCoeControl* ctrl = this->Parent();
            iVideoSurfaceContainer = CSurfaceContainerControl::NewL(ctrl->DrawableWindow(),TRect(Window().Position(),Window().Size()));
        );

        engine->SetVideoWindow(&(iVideoSurfaceContainer->GetWindow()));

        Window().SetOrdinalPosition(0);
        iVideoSurfaceContainer->GetWindow().SetOrdinalPosition(Window().OrdinalPosition() + 1);
    }
    else
#endif
	{
    	engine->SetVideoWindow(&Window());
	}

#ifdef ENABLE_CURSOR
	if(iCursor)
	{
		curPoint = iCursor->Position();
	}
#endif

	MNetscapeIf* nsfuncs = iPlayerInstance->NetscapeFns();

	//notify browser to deactivate the flash content, so that
	//user can scroll the web page.
	//browser do not check the value.so value is dummy (void* )1
	if( nsfuncs )
	{
	  const int pluginDeActivate(101);
	  nsfuncs->SetValue( pluginDeActivate,
						  (void*) &curPoint );

	  // changed for defect :ESJY-7PB9NN :Start
	  // added to send notification to the Browser that the orientation should be changed to
	  // normal screen. Flashlite will not change the orientation
	  TBool temp = EFalse;
	  const int pluginFullScreenBool(17);
	  nsfuncs->SetValue(pluginFullScreenBool,(void*) &temp );

	  // changed for defect :ESJY-7PB9NN : End
	}


	#ifndef GESTURE_LIBRARY_SUPPORTED
	if(iGestureDetector)
	{
	iGestureDetector->Reset();     //Make sure this is called after fullscreen control is deleted.
	}
	#endif //#ifndef GESTURE_LIBRARY_SUPPORTED
	iTogglingScreen = EFalse;

	if(iTactileFeedbackSupported)  //adding feature manager to check the feature at run time.
	{
		if( iFeedback  && iFeedback->TouchFeedbackSupported() )
		{
			iFeedback->InstantFeedback(ETouchFeedbackSensitive);
		}
	}

	SetRect(iRectRelativeToParentControl);

	if( !iErrorOccured && !iPreparingToExit )
	{
		TBool isInit = engine->IsInitialized();

		engine->InitializeL(iRectRelativeToParentControl, isInit);
		//iEngine->InitializePluginL(this, iRectRelativeToParentControl, isInit);
	}

	//Recalculate swf rect/clip rect since they might have changed while we are in fullscreen
	iSwfRect        = GetRect();
	iSwfClipRect    = GetClipRectFromScreenOrigin();

	// Set the normal screen saved video display rect for helix player.
	engine->SetVideoDisplayRectL(iSwfRect, iSwfClipRect);

	TRect screenRect;

	AknLayoutUtils::LayoutMetricsRect( AknLayoutUtils::EScreen, screenRect );

	DrawNow(screenRect);


	engine->NotifyFullScreen(FALSE);

	#ifdef ENABLE_CURSOR
	if(iCursor)
	{
		iCursor->NormalScreenMode(this);

		// NOTE: The page will not be scrolled in fullscreen.
		// Hence while returning to normal screen, we should first move the page
		// to the current location.
		// Distance = diff betn the current point and the saved point
        TPoint offset(0,0);

        // iSource: 10331:In case of exiting, from Fullscreen, updating the browser with a offset value
        // Anything other than (0,0) makes the browser to crash. Hence this check
        if(!iPreparingToExit)
        {
			offset.SetXY(curPoint.iX - iSavedCursorPoint.iX,
					 	curPoint.iY - iSavedCursorPoint.iY);
        }
		if( nsfuncs )
		{
			const int pluginFocusPosition(100);
			nsfuncs->SetValue(
					pluginFocusPosition,
								(void*) &offset );
		}
		// Hide the cursor when we are Coming to normal screen
		// and the plugin is deactivated.
		if(iCursor->IsVisible() && !iPluginActivated)
		{
			iCursor->HideCursor();
		}
	}
	#endif
	engine->ResumeVideo();
	return ETrue;
}
#endif // __FEATURE_UI_LANDSCAPE_FULLSCREEN

CPluginMenuManager* CBrowserPluginControl::GetMenuManager()
{
	return iMenuMgr;
}

MOptionMenuHandler* CBrowserPluginControl::GetOptionHandler()
{
	return iMenuMgr;
}

#endif

void CBrowserPluginControl::FLVideoStarted()
{
	NPPDebug(LOG_DEBUG,_L("CBrowserPluginControl::Video FLVideoStarted +") );

	if(!iPlayerInstance)
	    return;
	MFlashEngine* engine = iPlayerInstance->GetEngine();

	//readjust dyn mem need to flv max
#ifdef PLUG_IN_MEM_ALLOC
	iPlayerInstance->ReAdjustMaxHeapforVideoStream( engine->GetFLVMaxDynHeapSize() );
#endif	//PLUG_IN_MEM_ALLOC

#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
	if(!iCanGoFullScrn)
	{
		iCanGoFullScrn = ETrue;

		//commented as some users may want to mute the video volume ( dont want to watch video)
		//they may not want to go to full screen and then mute.. its annoying the user.
		//so to go to fullscreen another click required
		/*if(iPluginActivated)
		{
			if(!IsFullScreen() && !(engine->ControlPresent()))
				iUILauncher->IssueRequest();
			else
				iUILauncher->IssueRequest(EFalse);
		}*/
	}
#endif
	NPPDebug(LOG_DEBUG,_L("CBrowserPluginControl::FLVideoStarted -") );
}

#ifdef ENABLE_CURSOR
void CBrowserPluginControl::UpdateCursorPosition(TPoint aCursorpos)
{
	iSavedCursorPoint = aCursorpos;
}
#endif //ENABLE_CURSOR

#ifdef SELF_IAD_UPDATE
void CBrowserPluginControl::ShowIADUpdateNotification()
{
    NPPDebug( LOG_DEBUG, _L("CBrowserPluginControl::ShowIADUpdateNotification() ++ ") );

    // Caution! iInteractionMode is currently Default and Widget mode.
    // Default mode => Browser plugin mode.
    // May lead to bugs in the future!
    if((iPlayerInstance->GetBrPluginExecutionMode()) != EWidgetMode)
    {
		TRAP_IGNORE(DoShowIADUpdateNotificationL());
    }
    NPPDebug( LOG_DEBUG, _L("CBrowserPluginControl::ShowIADUpdateNotification() -- ") );
}

void CBrowserPluginControl::DoShowIADUpdateNotificationL()
{
	TBufC<128> KResFile(_L("FlashlitePluginIAD.rsc") );
    //TInt resOffset = load_pluginresourceL( KResFile );
	TInt resOffset = PluginBase::LoadPluginResourceL( KResFile );

    // Load a string from the resource file and display it
    HBufC* textResource = StringLoader::LoadLC(R_TEXT_FLITE_PLUGIN_NOTE_IAD_UPDATE_COMPLETE);

    CAknGlobalNote* pNote = CAknGlobalNote::NewLC();
    pNote->ShowNoteL(EAknGlobalInformationNote, *textResource);
    CleanupStack::PopAndDestroy(pNote);
    CEikonEnv::Static()->DeleteResourceFile(resOffset);

    // Pop HBuf from CleanUpStack and Destroy it.
    CleanupStack::PopAndDestroy(textResource);
}
#endif

TBool CBrowserPluginControl::IsVideoRectEmpty()
{
	TRect videoRect(0,0,0,0);

	MFlashEngine* engine = iPlayerInstance->GetEngine();

	if(engine)
	    TRAP_IGNORE(videoRect = engine->GetVideoRectL()); // ignoring as GetVideoRectL does not leave!

	return (videoRect.IsEmpty());  // by default TRect is (0,0,0,0)
}

#ifdef _BITBUFFER_ENABLED_
/*
BitBuffer* CBrowserPluginControl::BitBuffer()
    {
    if(!iBitBuffer)
        {
        iBitBuffer = CFlashBitmap::NewL(iWlessBitmap);
        }
    return iBitBuffer;
    }
*/

CFbsBitGc* CBrowserPluginControl::GetBitmapContext()
    {
    if(!iBitmapGc && iBitmapDevice)
        {
        iBitmapDevice->CreateContext(iBitmapGc);
        }
    return iBitmapGc;
    }

TInt CBrowserPluginControl::Resize(TSize aSize)
    {
    if(iBitBuffer)
        {
        iBitBuffer->SetSize(aSize,ETrue);
        }
    }

CFbsBitmapDevice* CBrowserPluginControl::GetBitmapDevice()
    {
    if(iBitmap && !iBitmapDevice)
        iBitmapDevice = CFbsBitmapDevice::NewL(iBitmap);

    return iBitmapDevice;
    }
#endif

TRect CBrowserPluginControl::ApplicationRect()
    {
    //TODO : Revisit this
    return ClientRect();
    }

TRect CBrowserPluginControl::ClientRect()
    {
    return GetRect();
    }

#ifdef _BITBUFFER_ENABLED_
void CBrowserPluginControl::CreateBitBuffer(TSize aBitmapSize, TDisplayMode aDisplayMode)
    {
    if ( iBitBuffer->IsSurfaceValid() )
        {
            return ETrue;
        }

        iBitBuffer->Free();

        //iIsBitmapLocked = EFalse;

        if ((iBitmap = new CWsBitmap(CCoeEnv::Static()->WsSession())) == NULL ||
            (iBitmap->Create(TSize(aBitmapSize), aDisplayMode) != KErrNone) ||
            (iBitmapUtil = new TBitmapUtil(iBitmap)) == NULL ||
            (iBitmapDevice = CFbsBitmapDevice::NewL(iBitmap)) == NULL)
        {
            // note: the bitmap will be deallocated automatically by the destructor
            FreeBits();
            return EFalse;
        }

        return ETrue;
    }
#endif

CFlashVolumeControl* CBrowserPluginControl::GetVolumeControl()
{
	/* ou1cimx1#491635: If engine is unloaded at this point(error condition), calling
	 * CFlashVolumeControl functions (NewL(), ShowPopupL())causes reference to NULL Engine
	 * Pointer, and causes crash. Return NULL pointer for Volume Control in this case:
	 */
    MFlashEngine* engine = FlashPluginInstance()->GetEngine();
	if (!engine)
	{
		return NULL;
	}

	if(!iVolPopup)
	{
		CHwKeyPropertyWatch* hwKeyPropertyWatch = iPlayerInstance->GetHwKeyPropertyWatch();

         MVolumePropertyWatchObserver* volumePropertyWatched = iPlayerInstance->GetMHwKeyWatchObserver();

		TRAP_IGNORE(
					if(hwKeyPropertyWatch)
						iVolPopup =  CFlashVolumeControl::NewL(this, volumePropertyWatched, EFalse, engine);
					else
						iVolPopup =  CFlashVolumeControl::NewL(this, volumePropertyWatched, ETrue, engine));
	}

	return iVolPopup;
}

void CBrowserPluginControl::ShowVolumePopup(TInt aCurrentVolume)
{
    // ou1cimx1#491635: If engine is unloaded at this point(error condition), it does not make sense to show volume popup.
    MFlashEngine* engine = FlashPluginInstance()->GetEngine();
    if (!engine)
    {
        return;
    }

    //Show Volume slider in case of Widgets and Browser fullscreen
#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
    if(IsFullScreen() || (iPlayerInstance->GetBrPluginExecutionMode() == EWidgetMode))
    {
        if (!iVolPopup)
            GetVolumeControl();

#ifdef __FEATURE_PLUGIN_UI_OPTIONS
        if(iVolPopup)
        {
            TRAP_IGNORE(iVolPopup->ShowPopupL(aCurrentVolume));
        }
#endif
    }
#endif
}

void CBrowserPluginControl::ResumeL()
{
    iPlayState = EFlashPluginPlaying;
    iVisibilityState = EFlashPluginVisible;

    MFlashEngine* engine = FlashPluginInstance()->GetEngine();

    if( engine )
    {
      engine->NotifyFocusChangeL(EFalse); // Resuming
    }
    #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
    if(IsFullScreen())
    {
        //GoFullScreenL();

        if(!iFullScreenHandler)
        iFullScreenHandler = new(ELeave) CAsyncFullScreenHandler(this);

        iFullScreenHandler->IssueRequest(ETrue);
    }
    #endif

    FlashPluginInstance()->AddHwSideKeyObserverL();
}
void CBrowserPluginControl::PauseL()
{
    iPlayState = EFlashPluginPaused;

    MFlashEngine* engine = FlashPluginInstance()->GetEngine();

    if( engine )
    {
        engine->NotifyFocusChangeL(ETrue);  // Pausing
    }

    #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
    if(IsFullScreen() && !IsVisible())
    {
        //GoNormalScreenL();

         if(!iFullScreenHandler)
                iFullScreenHandler = new(ELeave) CAsyncFullScreenHandler(this);

        iFullScreenHandler->IssueRequest(EFalse);

        iFullScreen = ETrue;
    }
    #endif

    FlashPluginInstance()->RemoveHwSideKeyObserver();
}

#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN

////////////////////////////////////////////////////////////////
// CAsyncFullScreenHandler
///////////////////////////////////////////////////////////////

// -------------------------------------------------------------------------
//CAsyncFullScreenHandler::CAsyncFullScreenHandler(CBrowserPluginInstance* aPlayer)
// constructor
// --------------------------------------------------------------------------
//

CAsyncFullScreenHandler::CAsyncFullScreenHandler(CBrowserPluginControl*  aControl)
              : CActive(EPriorityStandard)
              ,iControl(aControl)
              ,iFullScreenRequest(ETrue)
{
    CActiveScheduler::Add(this);

}

// -------------------------------------------------------------------------
// CAsyncFullScreenHandler::~CAsyncFullScreenHandler()
// destructor
// --------------------------------------------------------------------------
//


CAsyncFullScreenHandler::~CAsyncFullScreenHandler()
{
    Cancel();
}

// -------------------------------------------------------------------------
// void CAsyncFullScreenHandler::RunL()
// Active object execution method
// --------------------------------------------------------------------------
//


void CAsyncFullScreenHandler::RunL()
{
    NPPDebug(LOG_DEBUG,_L("CAsyncFullScreenLauncher::RunL() ++"));

    if( iStatus.Int() != KErrNone )
    {
        return;
    }

	if(iFullScreenRequest)
	{
		iControl->GoFullScreenL();
	}
    else
    {
    	iControl->GoNormalScreenL();
    }

    NPPDebug(LOG_DEBUG,_L("CAsyncFullScreenLauncher::RunL() --"));
}

// -------------------------------------------------------------------------
// void CAsyncFullScreenHandler::IssueRequest()
// Used to Handle asynchronous request for deletion
// --------------------------------------------------------------------------
//

void CAsyncFullScreenHandler::IssueRequest(TBool aFullScreenRequest)
{
    NPPDebug(LOG_DEBUG,_L("CAsyncFullScreenLauncher::IssueRequest() ++"));

    if(IsActive())
    {
        return;
    }

    iFullScreenRequest =  aFullScreenRequest;
    TRequestStatus* status = &iStatus;
    SetActive();
    User::RequestComplete(status, KErrNone);

    NPPDebug(LOG_DEBUG,_L("CAsyncFullScreenLauncher::IssueRequest() --"));
}

// -------------------------------------------------------------------------
// void CAsyncFullScreenHandler::DoCancel()
// cancel active object
// --------------------------------------------------------------------------
//

void CAsyncFullScreenHandler::DoCancel()
{
}

#endif

// End of File
