/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  FlashPluginfullScreenControl class implementation
*
*/


// INCLUDE FILES
#include "flashbrowserplugincontrol.h"
//#include "flashplgmain.h"
#include "flashbrowserpluginfullscreenctrl.h"
#include "flashliteplugin.hrh"
#include <flashliteplugin.rsg>
#include <eikenv.h>
#include <coemain.h>
#include <coecntrl.h>
#include "aknappui.h"
#include <AknUtils.h>
#include <aknconsts.h>
#include <avkon.hrh>
#include <featmgr.h>
#include "aknappui.h"
#include "fullscreenmenumgr.h" 
#include "flash_engine.h"
#include "flashplugincursor.h"
#ifdef USE_VPU2
#include "surfacecontainercontrol.h"
#endif
#include "flashpluginif.h"
#include "saflash_consts.h"

#include <rt_gesturehelper.h> 

using namespace RT_GestureHelper;

// CONSTANTS
const TInt KFirstTimerExpiryInterval = 5000000; //5 * 1000 * 1000; // 5 seconds
const TInt KTimerExpiryInterval = KAknStandardKeyboardRepeatRate;
//Number of pixels subtracted by browser from the actual pointer co-ordinate values
const TUint KBrowserYCoordDiff= 8;

extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );
extern int load_pluginresourceL(TBufC<128>& aFileName);
// ================= MEMBER FUNCTIONS =======================
// -----------------------------------------------------------------------------
// CBrowserPluginFullScreenControl::NewL()
// Two-phased constructor.
// -----------------------------------------------------------------------------
CBrowserPluginFullScreenControl* CBrowserPluginFullScreenControl::NewL(CBrowserPluginControl* aControl,
																	   MFlashEngine*          aEngine)
    {
    CBrowserPluginFullScreenControl* self = new (ELeave) CBrowserPluginFullScreenControl(aControl,aEngine);

    CleanupStack::PushL( self );
    self->ConstructL();
    CleanupStack::Pop();

    return self;
	}

// -----------------------------------------------------------------------------
// CBrowserPluginFullScreenControl::CBrowserPluginFullScreenControl
// C++ default constructor can NOT contain any code, that
// might leave.
// -----------------------------------------------------------------------------
CBrowserPluginFullScreenControl::CBrowserPluginFullScreenControl(CBrowserPluginControl* aControl, 
																 MFlashEngine*          aEngine)
								:iControl(aControl)
								,iEngine(aEngine)
								,iCbaHandler(NULL)
								,iFullScreenCba(NULL)
								,iTimer(NULL)
								//,iStatusPaneHidden(EFalse)
								//,iCbaPaneHidden(EFalse)
								,iAppUi(NULL)
								,iWindowGc(NULL)

{    
}

// changed for defect :ESJY-7PB9NN :Start
//Browser changes the orientation, flashlite must not take care of it, so commentaing this
// function
/*void  CBrowserPluginFullScreenControl::GoLandscapeL()
{
	//Go landscape if not already
    iAppOrtn = iAvkonAppUi->Orientation();
    if(iAppOrtn != CAknAppUiBase::EAppUiOrientationLandscape)
	{
		iAvkonAppUi->SetOrientationL(CAknAppUiBase::EAppUiOrientationLandscape);
	}
}*/    
// changed for defect :ESJY-7PB9NN :End

void CBrowserPluginFullScreenControl::CreateCbaL()
{
	// Timer for implementing repeat
    iTimer = CPeriodic::NewL(CActive::EPriorityStandard);
    
	// Create the CBA control for the popout
    // NB MUST BE DONE BEFORE CREATING CBA!!
    iCbaHandler = CCbaHandler::NewL(iControl,this);
    
    	TBufC<128> KResFile(_L("FlashlitePlugin.rsc") );
    //TInt resOffset = load_pluginresourceL( KResFile );
    	TInt resOffset = PluginBase::LoadPluginResourceL( KResFile );
     // NB MUST create the cba prior to creating scrollbarframe on listbox
     //need the orientation to be handled
     
    CAknAppUiBase::TAppUiOrientation apportn = iAvkonAppUi->Orientation();
    CEikButtonGroupContainer::TOrientation cbaortn = CEikButtonGroupContainer::EVertical;
    
    if(apportn != CAknAppUiBase::EAppUiOrientationLandscape)
    	cbaortn = CEikButtonGroupContainer::EHorizontal;
    iFullScreenCba = CEikButtonGroupContainer::NewL(CEikButtonGroupContainer::ECba,
        cbaortn, iCbaHandler, R_CBA_FLITE_FULL_SCREEN /*, *popout*/);
      
    iFullScreenCba->MakeVisible(ETrue);	

    CEikonEnv::Static()->DeleteResourceFile(resOffset);    
    
    iTimer->Start(KFirstTimerExpiryInterval,
					KTimerExpiryInterval, 
					TCallBack(TimerCallbackL, this));
}


void CBrowserPluginFullScreenControl::InitFlashEngineL()
{
	TBool isInit = iEngine->IsInitialized();
	
	iEngine->InitializeL(Rect(), isInit);
	
	//iEngine->InitializePluginL(iControl,Rect(), isInit);
}

// -----------------------------------------------------------------------------
// CBrowserPluginFullScreenControl::ConstructL
// Symbian 2nd phase constructor can leave.
// -----------------------------------------------------------------------------
void CBrowserPluginFullScreenControl::ConstructL()
{    
	FeatureManager::InitializeLibL(); // Initialise Feature manager
	iTouchFeatureSupported = FeatureManager::FeatureSupported(KFeatureIdPenSupport);
	
	FeatureManager::UnInitializeLib(); // Uninitialise Feature manager
	
	// changed for defect :ESJY-7PB9NN :Start
    //Browser changes the orientation, flashlite must not take care of it, so commentaing this
    // call
    //GoLandscapeL();
    // changed for defect :ESJY-7PB9NN :End
	
    CreateWindowL();
    DrawableWindow()->SetPointerGrab(ETrue);
    EnableDragEvents();
    
    SetMopParent(iControl); //set ctrl heirarchy
#ifndef GESTURE_LIBRARY_SUPPORTED   
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR  
    iFlashLongTapDetector = CFlashLongTapDetector::NewL(this);
    
    iSuppressor = CAknPointerEventSuppressor::NewL();  
    iSuppressor->SetMaxTapDuration(KMaxTInt64);
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
#endif //#ifndef GESTURE_LIBRARY_SUPPORTED

    
    
  
    // changed for defect :ESJY-7PB9NN :Start
    // change in orientation is done by browser, so no need of this function
    // removing this function
  	/*CEikStatusPane* statuspane = ((CAknAppUi*)iCoeEnv->AppUi())->StatusPane();
  	CEikButtonGroupContainer* cbapane = ((CAknAppUi*)iCoeEnv->AppUi())->Cba();
  	
  	if(statuspane && statuspane->IsVisible())
  	{
  		statuspane->MakeVisible(EFalse);
  		iStatusPaneHidden = ETrue; //manually hidden, need to rollback once we go to normal screen
  	}
    if(cbapane && cbapane->IsVisible())
    {
    	cbapane->MakeVisible(EFalse);
    	iCbaPaneHidden = ETrue; //manually hidden, need to rollback once we go to normal screen 	
    }*/
	// changed for defect :ESJY-7PB9NN :End
	
    //go to Top of stack
    iAppUi = iCoeEnv->AppUi();
    iAppUi->AddToStackL(this,ECoeStackPriorityDefault+1, ECoeStackFlagStandard);
    
    AknLayoutUtils::LayoutMetricsRect( AknLayoutUtils::EScreen, iScreenRect );
    SetRect(iScreenRect);

	InitFlashEngineL();
	
	ActivateL();
	
	if(!iTouchFeatureSupported) //adding feature manager to check the feature at run time.
    	CreateCbaL();
    	
	
#ifdef FEATURE_OPENVG
    //Causes EGL surface to be recreated.
    //Necessary to trigger this before fullscreen 
    //window is destroyed. otherwise, egl surface will be orphaned
    //without a window and may crash.
    iEngine->SetEglContainerWindow(&Window());
#endif	
	
#ifdef USE_VPU2    
    iEngine->SetVideoWindow(NULL);

    CSurfaceContainerControl* videoControl = iControl->GetVideoContainer();
    if(videoControl)
    {    
        iControl->SetVideoContainer(NULL);
        delete videoControl;
    }
    
    if(iEngine->IsOpenVgRenderingMode() && !iEngine->IsSwitchingToSWRenderingInPrgress())
    {
        videoControl = CSurfaceContainerControl::NewL((RWindowTreeNode*)&(iCoeEnv->RootWin()),TRect(Window().Position(),Window().Size()));
        iControl->SetVideoContainer(videoControl);
        iEngine->SetVideoWindow(&(videoControl->GetWindow()));
        
        Window().SetOrdinalPosition(0);
        videoControl->GetWindow().SetOrdinalPosition(Window().OrdinalPosition()+1);
    }
    else
#endif
    {
        iEngine->SetVideoWindow(&Window());
    }
}

// changed for defect :ESJY-7PB9NN :Start
// change in orientation is done by browser, so no need of this function
// removing this function
/*void CBrowserPluginFullScreenControl::RestoreOrientationL()
{
	// Trying to make status pane visible, when plugin is in background, panics with AVKON 76
	if(iStatusPaneHidden && (iControl->iFocusState == EFlashPluginForeground)) 
	{
		((CAknAppUi*)iAppUi)->StatusPane()->MakeVisible(ETrue);
		iStatusPaneHidden = EFalse;	
	}	
	if(iCbaPaneHidden && (iControl->iFocusState == EFlashPluginForeground))
	{
		((CAknAppUi*)iAppUi)->Cba()->MakeVisible(ETrue);
		iCbaPaneHidden = EFalse;	
	}*/	

    // changed for defect :ESJY-7PB9NN :Start
    //Browser changes the orientation, flashlite must not take care of it, so this part
    // of the code is commented
    /*if((iAppOrtn != CAknAppUiBase::EAppUiOrientationLandscape ) && (iControl->iFocusState == EFlashPluginForeground)) //if App was not in landscape mode
    {
        iAvkonAppUi->SetOrientationL(iAppOrtn);
    }*/
    // changed for defect :ESJY-7PB9NN :End
//}
// changed for defect :ESJY-7PB9NN :End


void CBrowserPluginFullScreenControl::SizeChanged()
{
	CWsScreenDevice* wsScreen   = iCoeEnv->ScreenDevice();
        
    if( !wsScreen )
        {
        return;
      	}
      
    TSize screenSize = wsScreen->SizeInPixels();
	if(iScreenRect.Size() != screenSize)
		{
		iScreenRect.SetRect(TPoint(0,0), screenSize);
		
		#ifdef ENABLE_CURSOR
		
		// Cursor co-ordinates and also the extent has to be mapped when:
		// 1. Switching to fullscreen and 
		// 2. Size of the fullscreen control changes. (like cover screen => PDA screen)
		
		if( iControl && iControl->Cursor() )
			{			
			CFlashCursor* cursor = (CFlashCursor*) iControl->Cursor();
			
			cursor->FullScreenMode(this);  
			}

		#endif
		}
}

void CBrowserPluginFullScreenControl::DestroyCba()
{
	if(iTimer)
    {
        iTimer->Cancel();
        delete iTimer;
        iTimer= NULL;
    }
        
	delete iFullScreenCba;
	iFullScreenCba = NULL;
	
	delete iCbaHandler;
	iCbaHandler = NULL;
}

// -----------------------------------------------------------------------------
// CBrowserPluginFullScreenControl::~CBrowserPluginFullScreenControl
// Destructor
// -----------------------------------------------------------------------------
CBrowserPluginFullScreenControl::~CBrowserPluginFullScreenControl()
{
	if(!iTouchFeatureSupported) //adding feature manager to check the feature at run time.
    	DestroyCba();
  
// changed for defect :ESJY-7PB9NN :Start
// change in orientation is done by browser, so no need of this function
// removing this function  
//    TRAP_IGNORE ( RestoreOrientationL() );
#ifndef GESTURE_LIBRARY_SUPPORTED  
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR  
    if(iFlashLongTapDetector)
        {
        delete iFlashLongTapDetector;
        iFlashLongTapDetector = NULL;
        }
        
    if(iSuppressor)
        {
        delete iSuppressor;
        iSuppressor = NULL;
        }   
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
#endif //#ifndef GESTURE_LIBRARY_SUPPORTED    
	
	CCoeEnv* coeenv = iCoeEnv;
    coeenv->AppUi()->RemoveFromStack(this);
    
    if( iWindowGc )
    {
    	delete iWindowGc;
    	iWindowGc = NULL;
    }
}

// ----------------------------------------------------------------------
// void CBrowserPluginControl::Draw(const TRect& aRect) const
//
// This function is used to draw the view
// ----------------------------------------------------------------------
// 
void CBrowserPluginFullScreenControl::Draw(const TRect& aRect) const
{
	
	CWsScreenDevice* wsScreen   = iCoeEnv->ScreenDevice();
     
    if( !wsScreen )
    {
        return;
    }   
      
    TSize screenSize = wsScreen->SizeInPixels();
    TRect lScreenRect;

    if(iScreenRect.Size() != screenSize)
	{
		lScreenRect.SetRect(TPoint(0,0), screenSize);
		if(iControl)
       	{
       		TRAP_IGNORE ( iControl->RecreateBitmapL(lScreenRect) );
       	}
    }
    
    if( iEngine )
	{
		CWindowGc& gc = SystemGc();
		iEngine->Draw(aRect, gc);
	}
	
#ifdef SYMBIAN_BUILD_GCE    
    ControlEnv()->WsSession().Finish();
#endif
}

// ----------------------------------------------------------------------
// void CBrowserPluginControl::DrawDirtyRect(const TRect& aRect) const
//
// This function is used to draw only the dirty rect in the control extent.
// ----------------------------------------------------------------------
// 
void CBrowserPluginFullScreenControl::DrawDirtyRect(const TRect& aRect) 
{

	CWsScreenDevice* screenDevice = ControlEnv()->ScreenDevice();	
	if( screenDevice && !iWindowGc ) 
    {
    	if( screenDevice->CreateContext(iWindowGc) != KErrNone) 
    		iWindowGc = NULL;
    }
    if( iEngine && iWindowGc )
	{
		Window().Invalidate(aRect);
		Window().BeginRedraw(aRect);
		iWindowGc->Activate(Window());
		iEngine->Draw(aRect, *iWindowGc);
		iWindowGc->Deactivate();
		Window().EndRedraw();		
	}
}


 
//Host engine is getting Rwindow    
RWindow& CBrowserPluginFullScreenControl::Window() const
{
	return CCoeControl::Window();
}

TKeyResponse CBrowserPluginFullScreenControl::OfferKeyEventL(const TKeyEvent& aKeyEvent, 
                                                             TEventCode aType)
{
	switch(aKeyEvent.iScanCode)
	{
#ifdef ENABLE_CURSOR
        case EStdKeyLeftArrow:
        case EStdKeyRightArrow:
        case EStdKeyUpArrow:
        case EStdKeyDownArrow:
#ifdef ENABLE_OFN_SUPPORT
        case EStdKeyLeftUpArrow:
        case EStdKeyLeftDownArrow:
        case EStdKeyRightDownArrow:
        case EStdKeyRightUpArrow:
#endif
            {
            if( iFullScreenCba && iFullScreenCba->IsVisible() )
               {
                   iFullScreenCba->MakeVisible(EFalse);
                   iControl->Cursor()->ShowCursor();

                   if( iEngine )
                   {
                       iEngine->UpdateVideoScreen();   
                   }   
               }
               if( iTimer )
                   iTimer->Cancel();
               
               break;
            }
#endif
		case EStdKeyDevice0:
		case EStdKeyDevice1:
		case EStdKeyDevice2:
		{
#ifdef ENABLE_CURSOR		
            if (iControl && iControl->Cursor()) {
                iControl->Cursor()->HideCursor();
            }
#endif
			if(iFullScreenCba && !iFullScreenCba->IsVisible())
			{
				iFullScreenCba->MakeVisible(ETrue);
				
				if (iTimer && !iTimer->IsActive())
	            {
		            iTimer->Start(KFirstTimerExpiryInterval,
		                          KTimerExpiryInterval, 
		                          TCallBack(TimerCallbackL, this));
	            }
			}			
		}
		break;
		
		default:
		{
			if(iControl)
			{
				return iControl->OfferKeyEventL(aKeyEvent,aType);
			}	
		}		
		break;
	}
	
	return EKeyWasNotConsumed;  
}

#ifndef GESTURE_LIBRARY_SUPPORTED
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
void CBrowserPluginFullScreenControl::HandleFlashLongTapEventL( const TPoint& /*aPoint*/, const TPoint& /*aParentPoint*/)
    {
    if (iTouchType == ETouchStarted) 
        { 
        iTouchType = ETouchLongTap; 
        } 
    }
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
#endif //#ifndef GESTURE_LIBRARY_SUPPORTED

#ifdef GESTURE_LIBRARY_SUPPORTED
void CBrowserPluginFullScreenControl::SendPointerEventToBrowserL(const TPointerEvent &aPointerEvent)
{
    MNetscapeIf* nsfuncss = iControl->FlashPluginInstance()->NetscapeFns();
    if(nsfuncss)
    {
        (nsfuncss->SetValue)((NPPVariable) NPPVPluginPointerEvent,
                            (void*) &(aPointerEvent));
    }
}

TBool CBrowserPluginFullScreenControl::HandleGestureFromBrowserL( void* aEvent)
{
    TBool ret = ETrue; //We always consume events in full screen
    //If engine is destoryed no need to process gesture we return false and browser will process them'
    if(!iControl)
        return EFalse;
    if(iEngine)
    {    
          
      TGestureEvent *gesture = static_cast<TGestureEvent*>(aEvent);
      TPoint  point = gesture->CurrentPos() ;
      TPoint startPoint = gesture->StartPos();
      TPointerEvent pointerEvent;

      //Negate the modifications done by browser, to get the actual pointer co-ordinates
	  #ifdef ADVANCEDPOINTEREVENT_H
      point.iY += KBrowserYCoordDiff; 
      startPoint.iY += KBrowserYCoordDiff;
	  #endif
      pointerEvent.iPosition = point;
      TGestureCode gtype =  gesture->Code(EAxisBoth);
      
      #ifdef ENABLE_CURSOR
      iControl->UpdateCursorPosition(point);
      #endif //ENABLE_CURSOR
    
       //application need to say which axis to consider?
      #ifdef _GestureDebug_
      NPPDebug(LOG_DEBUG,_L("CBrowserPluginFullScreenControl::Consumegesture: code %d, pos(%d, %d)"), gtype, point.iX, point.iY);
      #endif //_GestureDebug_
      
      switch (gtype)
      {
          case EGestureStart:
          {     
              pointerEvent.iType=TPointerEvent::EButton1Down;
              iControl->ProvideFeedbackL(pointerEvent);
              iEngine->OfferPointerEventL(point, TPointerEvent::EButton1Down);
              #ifdef ENABLE_CURSOR
              // hide cursor when user taps on Flash content even if Qwerty keypad is ON
              if(iEngine && iControl->Cursor() && iControl->Cursor()->IsVisible())
              {
                  ((CFlashCursor*)iControl->Cursor())->HideCursor();
              }
        
              #endif // ENABLE_CURSOR
              break;
          }
          case EGestureDrag:
          {
              pointerEvent.iType=TPointerEvent::EDrag;
              iControl->ProvideFeedbackL(pointerEvent);
              iEngine->OfferPointerEventL(point, TPointerEvent::EDrag);
              break;
          }
          //This are events generated after up event is received.
          case EGestureSwipeLeft:
          case EGestureSwipeRight:
          case EGestureSwipeUp:  
          case EGestureSwipeDown:
          case EGestureFlick:  
          case EGestureDrop:
          case EGestureReleased:
          {
#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
              if(!iControl->IsContextMenuVisible())	// Showing the popup menu => gesture consumed. Hence don't pass it to the core
#endif
              {
              iEngine->OfferPointerEventL(point, TPointerEvent::EButton1Up);
              }
              break;
          }
              // gestures
          case EGestureUnknown:
          {
              break;
          }
              // these codes are sent when user lifts stylus (if holding not started)
          case EGestureDoubleTap:
          {
          
#ifdef __FEATURE_TOGGLE_ON_DOUBLE_CLICK__
              if( iControl->IsPlayingVideo() && !iEngine->GetFullScreenViaFSCommand()) 
              {
                  // toggle screen between full screen mode and normal mode
                  iControl->ToggleScreenL();
              }
#endif              
              iEngine->OfferPointerEventL(startPoint, TPointerEvent::EButton1Up);
              
              pointerEvent.iType=TPointerEvent::EButton1Down;
              iControl->ProvideFeedbackL(pointerEvent);                
              iEngine->OfferPointerEventL(point, TPointerEvent::EButton1Down);

              break;
          }
          case EGestureTap:
          {
              iControl->LaunchVKbL();
    
              #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
#ifndef __FEATURE_CONTEXT_SENSITIVE_MENU__
                  iControlPresent = iEngine->IsControlPresent(point);
                  if(!iControlPresent)
                  {
                      TRect  videoRect=iEngine->GetVideoRectL();      
                                                  
                      // NOTE: Toggling to NS will delete *this* instance.
                      
                      // If Ph - 1, just provide the feedback and toggle
                      if(!iEngine->IsHelixFlvMode())  
                          iControl->ToggleScreenL();
                      // If ph - 2, if no videorect toggle, otherwise check the point and then toggle. 
                      else if( (videoRect.IsEmpty() || videoRect.Contains(point)))  
                          iControl->ToggleScreenL();
                      
                      return ETrue;
                  }  
#endif // __FEATURE_CONTEXT_SENSITIVE_MENU__
              #endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN
              break;
          }
          case EGestureLongTap:
          {
#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
              if( iControl->IsPlayingVideo() && !iEngine->GetFullScreenViaFSCommand()) 
              {
                  // show context menu only incase of video
                  iControl->DynInitContextSensitiveMenu(iControl,point);
              }
#endif
              break;
          }
          default:
          {
          
              break;
          }
      }
    }
    return ret;
}
#else

void CBrowserPluginFullScreenControl::HandleFullScreenPointerEventL(const TPointerEvent &aPointerEvent)
{
    TPoint point = aPointerEvent.iPosition;
    iControl->ProvideFeedbackL(aPointerEvent);

    // update the cursor position to get in sync with pointer events
    #ifdef ENABLE_CURSOR
          iControl->UpdateCursorPosition(aPointerEvent.iPosition);
    #endif //ENABLE_CURSOR

    #ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
        iFlashLongTapDetector->PointerEventL(aPointerEvent);
        
        if(iSuppressor->SuppressPointerEvent(aPointerEvent))
        {    
           return;
        }
        
        switch (aPointerEvent.iType) 
        { 
            case TPointerEvent::EButton1Down:
            {
                iTouchType = ETouchStarted; 
                //ClaimPointerGrab(ETrue);
                NPPDebug(LOG_DEBUG,_L("CBrowserPluginControl::HandlePointerEventL, TPointerEvent::EButton1Down,(%d, %d)"), aPointerEvent.iPosition.iX, aPointerEvent.iPosition.iY);                 
                iCorePosition = aPointerEvent.iPosition;
                iControlPresent = iEngine->IsControlPresent(aPointerEvent.iPosition);
                
                ipreviousPoint = point;
                
                #ifdef ENABLE_CURSOR
                // hide cursor when user taps on Flash content even if Qwerty keypad is ON
                if(iEngine && iControl->Cursor() && iControl->Cursor()->IsVisible())
                {
                    ((CFlashCursor*)iControl->Cursor())->HideCursor();
                }
                
                #endif // ENABLE_CURSOR
                
                iEngine->OfferPointerEventL(point, TPointerEvent::EButton1Down);
                
                if(iControl)
                {
                   iControl->HandPointerEventToImeL(aPointerEvent);
                }

                return;
            }    
            
            case TPointerEvent::EButton1Up: 
            {
                if (iTouchType == ETouchStarted) 
                { 
                    iTouchType = ETouchTap; 
                }
                    
                if((iTouchType == ETouchTap) || (iTouchType == ETouchLongTap))
				{
                    iEngine->OfferPointerEventL(iCorePosition, TPointerEvent::EButton1Up);
                }
				else
				{
                    iEngine->OfferPointerEventL(point, TPointerEvent::EButton1Up);
                }
                if(iControl)
                {
                   iControl->HandPointerEventToImeL(aPointerEvent);
                }
                
                #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
                if(iTouchType == ETouchTap)
                {
                    if(!iControlPresent)
                    {
                
    
                        TRect  videoRect=iEngine->GetVideoRectL();      
                        
                        // NOTE: Toggling to NS will delete *this* instance.
                        
                        // If Ph - 1, just provide the feedback and toggle
                        if(!iEngine->IsHelixFlvMode())  
                            iControl->ToggleScreenL();
                        // If ph - 2, if no videorect toggle, otherwise check the point and then toggle. 
                        else if( (videoRect.IsEmpty() || videoRect.Contains(point)))  
                            iControl->ToggleScreenL();
                    }
                }
                #endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN
                return; 
             }    
            case TPointerEvent::EDrag:
            {    
                iTouchType = ETouchDrag;                                 
                           
                iEngine->OfferPointerEventL(point, TPointerEvent::EDrag);
                
                if(iControl)
                {
                   iControl->HandPointerEventToImeL(aPointerEvent);
                }
                return; 
            }        
            default: 
                    return; 
        }
        #else
        
            //This will not work becasue the plg control handlepointer has been customized.
            //Full screen control needs to have a separate function.
            //should be implented if suppressor is disabled
                /*if(iControl)
                {
                    iControl->HandlePointerEventL(aPointerEvent);
                }*/
        
    #endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
}
#endif // GESTURE_LIBRARY_SUPPORTED

void  CBrowserPluginFullScreenControl::HandlePointerEventL(const TPointerEvent &aPointerEvent)
{
        if(iTouchFeatureSupported) //adding feature manager to check the feature at run time.
        {       
            // if the engine is already destroyed then we do not handle the pointer event
            // if pen is not enabled, we do not handle the pointer event
    
    if (!AknLayoutUtils::PenEnabled() || !iEngine)
       return;
    

            #ifdef GESTURE_LIBRARY_SUPPORTED
                SendPointerEventToBrowserL(aPointerEvent);
            #else
            #endif//GESTURE_LIBRARY_SUPPORTED
        }    
}

void CBrowserPluginFullScreenControl::HandleFullScreenCbaL()
{
	if( iFullScreenCba && iFullScreenCba->IsVisible() )
	{
		iFullScreenCba->MakeVisible(EFalse);
		
#ifdef ENABLE_CURSOR
          if (!iCbaHandler->IsFullScreenMenuShowing())
              iControl->Cursor()->ShowCursor();
#endif
		if( iEngine )
		{
			iEngine->UpdateVideoScreen();	
		}	
	}
	if( iTimer )
		iTimer->Cancel();
}

TInt CBrowserPluginFullScreenControl::TimerCallbackL(TAny* aPtr)
{
    static_cast<CBrowserPluginFullScreenControl*>(aPtr)->HandleFullScreenCbaL();
    return KErrNone;
}


void CBrowserPluginFullScreenControl::CancelTimer()
{
	if (iTimer->IsActive())
	{
		iTimer->Cancel();
	}
}
    
////////////////////////////////////////////////////////////////////////////////////
//

CCbaHandler* CCbaHandler::NewL(CBrowserPluginControl* aSwfControl
				,CBrowserPluginFullScreenControl* aAppContainer)
{
	CCbaHandler* self = new(ELeave) CCbaHandler(aSwfControl,aAppContainer);

    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop(self);
    
    return self;	
}
	
				
CCbaHandler::CCbaHandler(CBrowserPluginControl* aSwfControl,CBrowserPluginFullScreenControl* aAppContainer):
	iSwfControl(aSwfControl)
	,iAppContainer(aAppContainer)
{
}
       
CCbaHandler::~CCbaHandler()
{
	if(iMenuHandler)	
	{
		delete iMenuHandler;
		iMenuHandler = NULL;
	}
}

void CCbaHandler::ConstructL()
{
	iMenuHandler = CFullScreenMenuManager::NewL(iSwfControl);	
}

TBool CCbaHandler::IsFullScreenMenuShowing() {

    if (iMenuHandler)
        return iMenuHandler->IsShowing();
    else
        return EFalse;
    
}
void CCbaHandler::ProcessCommandL(TInt aCommandId)
{
	switch (aCommandId)
    {
    	case EAknSoftkeyClose:
        {
			if(iAppContainer)	        		
			{
				iAppContainer->CancelTimer();
			}
			
    		if(iSwfControl)
			{
				iSwfControl->GoNormalScreenL();
			}
        }
        break;
    	case EAknSoftkeyOptions:
    	{
    		if(iMenuHandler)
    		{
    			if(!iMenuHandler->IsShowing())
    			{
    	TBufC<128> KResFile(_L("FlashlitePlugin.rsc") );
    TInt resOffset = load_pluginresourceL( KResFile );
					
    				iMenuHandler->ShowL(R_FLASH_FULLSCREEN_MENUBAR);	
    				
				    CEikonEnv::Static()->DeleteResourceFile(resOffset);    
   				}
    		}
#ifdef ENABLE_CURSOR
    		iSwfControl->Cursor()->HideCursor();
#endif
    	}
    	break;        	
    }
}

//////////////////////////////////////////////////////////////////////////////////////
//
// End of File

