/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  FlashCursor class implementation
*
*/

#include "saplayer_build.h"
#include "flashplugincursor.h"
#include "flashplugincontrol.h"

#include <eikfutil.h>

#include <AknIconUtils.h>
#include <AknsUtils.h> 
#include <e32math.h>

#ifdef __SCALABLE_ICONS
#include <FLASHPLUGIN.MBG>
#else
#include <flashplugin_bitmap.mbg>
#endif

#ifdef ENABLE_CURSOR

_LIT(KFlashPluginMifFilePath,"\\resource\\apps\\Flashplugin.mif");

CFlashCursor::CFlashCursor(CCoeControl* aParentControl,
              MFlashEngine* aEngine)
             : iSize(0, 0)
             , iPosition(0, 0)
             , iBitmap(NULL)
             , iBitmapMask(NULL)
             , iEngine(aEngine)
    {    
    iPreviousPoint.SetXY(0, 0);
    iWindowSize.SetSize(0, 0);
    iActualWindowSize.SetSize(0, 0);
    iHiddenRect.SetSize(TSize(0, 0));
    iHiddenMovement = EFalse;
    //iFlashControl  = aFlashControl;
    iParentControl = aParentControl;
    iCursorCleared  = EFalse;
    iFullScreenMode = EFalse;
	iSpriteCreated = EFalse;
    iState = EFlashCursorHidden;
    }


CFlashCursor* CFlashCursor::NewL(CCoeControl* aParentControl, MFlashEngine* aEngine)
    {
    CFlashCursor* self = new (ELeave) CFlashCursor(aParentControl,aEngine);
    
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop();
    
    return self;
    }


CFlashCursor::~CFlashCursor()
    {
	CloseSprite();

    if( iBitmap )    
        {
        delete iBitmap;
        iBitmap = NULL;
        }
    if( iBitmapMask )    
        {
        delete iBitmapMask;
        iBitmapMask = NULL;
        }
    }


void CFlashCursor::ConstructL()
{ 
	 TFileName rscfn; //cautious symbian does not recommend auto var for TfileName
	 TFileName dllName;
	 Dll::FileName(dllName);
	 const TUint KDriveLetterLength = 2;
	 TBuf<KDriveLetterLength> drive = dllName.Left(KDriveLetterLength);
	 
	 rscfn.Copy(drive); // get the drive letter
	 rscfn.Append(KFlashPluginMifFilePath);
	 
	 TBool fileExists( BaflUtils::FileExists(CCoeEnv::Static()->FsSession(), rscfn) );
	 
	 if ( !fileExists )
	    {
	    // If we didn't find the resource file, then we should leave 
	    User::Leave(KErrNotFound);
	    }
    
	TInt iconId = 0;
	TInt iconMaskId = 0;
    
#ifdef __SCALABLE_ICONS

    iconId      = EMbmFlashpluginQgn_indi_viewer_pointer_hand;
    iconMaskId  = EMbmFlashpluginQgn_indi_viewer_pointer_hand_mask;
        
#else

    iconId      = EMbmFlashplugin_bitmapQgn_indi_viewer_pointer_hand;
    iconMaskId  = EMbmFlashplugin_bitmapQgn_indi_viewer_pointer_hand_mask;
	
#endif

    AknIconUtils::CreateIconL(
        iBitmap,
        iBitmapMask,
        rscfn,	
        iconId, 
        iconMaskId );

        
    if ( AknIconUtils::IsMifIcon( iBitmap ) )
    {
        TSize bitmapSize(KCursorWidth, KCursorHeight);
        
        // Sets the sizes of both bitmap and mask
        AknIconUtils::SetSize( iBitmap, bitmapSize );
        AknIconUtils::SetSize( iBitmapMask, bitmapSize );
    }

	//CreateSprite();
}    
    
void CFlashCursor::ShowCursor(const TPoint& aPoint)
    {
	if(iState == EFlashCursorHidden)
	{
		CreateSprite();
	}

    iPosition   = aPoint;
    iState      = EFlashCursorDisplaying;
    
    iSize.SetSize(KCursorWidth, KCursorHeight);
    iPreviousPoint = iPosition;
    
    UpdateCursor();
    }
    
void CFlashCursor::DisplayCursor()
{
    if (iState == EFlashCursorHidden )
    {
        return;
    }
    
    /* - Disabed because cursor is now sprite.
    TSize  bitmapSize = iBitmap->SizeInPixels();
    TRect  bitmapRect(0, 0, bitmapSize.iWidth, bitmapSize.iHeight);    
    
    aBitmapContext->AlphaBlendBitmaps(iPosition, iBitmap, bitmapRect, iBitmapMask, TPoint(0, 0));            
    */
    
    if (iSpriteCreated)
    	iSprite.SetPosition(iPosition);

	iState = EFlashCursorDisplaying;
}

void CFlashCursor::HideCursor()
{
    if(iSpriteCreated)
    {
	CloseSprite();
    iState = EFlashCursorHidden;
    UpdateCursor();
    }
}

/**********************************************************************
	Function: SetCursorPosition
	Description: This function will update the current cursor position
	Input:   aCurrentPosition  -current cursor position
	Output: None
	Return: None
	Remarks: Added as a fix for iSource #8885
**********************************************************************/
void CFlashCursor::SetCursorPosition(const TPoint &aCurrentPosition)
{
    iPreviousPoint = iPosition;
    iPosition = aCurrentPosition;
}


/* TBool CFlashCursor::MoveCursor(TPoint& aDistance)
 *  Return Value of this function
 *   = True if there is any movement in the cursor in the flash area
 *   = False if the cursor hit the boundary and there is no mevement
 *  
 *  Parameter aDistance
 *   = Is updated with the distance by which the cursor is moved
 *   = If the cursor hits the boundary, the aDistance is set to the size of the cursor 
 */
TBool CFlashCursor::MoveCursor(TPoint& aDistance)
    {
    iPreviousPoint = iPosition;
    TBool retVal = ETrue;
    // Out of bounds cases - so dont move the cursor
    
    if( ( aDistance.iX < 0 && iPosition.iX == 0 ) ||
        ( aDistance.iY < 0 && iPosition.iY == 0 ) || 
        ( aDistance.iX > 0 && iPosition.iX == iWindowSize.iWidth - iSize.iWidth ) ||
        ( aDistance.iY > 0 && iPosition.iY == iWindowSize.iHeight - iSize.iHeight ) )
        {

		iState = EFlashCursorDisplaying;
        retVal = EFalse; // There is no moment, set it to false;
        
        TInt xValue = aDistance.iX == 0 ? 0 : aDistance.iX > 0 ? iSize.iWidth : -1*iSize.iWidth;
        TInt yValue = aDistance.iY == 0 ? 0 : aDistance.iY > 0 ? iSize.iHeight : -1*iSize.iHeight;
        aDistance.SetXY(xValue,yValue);
        }
    else
        {
        iPosition.iX += aDistance.iX;
        iPosition.iY += aDistance.iY; 
        
        iState = EFlashCursorMoving;

        if(aDistance.iX < 0 && iPosition.iX < 0)
            {
            iPosition.iX  = 0;
            }
        if(aDistance.iY < 0 && iPosition.iY < 0)
            {        
            iPosition.iY = 0;
            }
        
        if(aDistance.iX > 0 && iPosition.iX > iWindowSize.iWidth - iSize.iWidth)
            {
            iPosition.iX  = iWindowSize.iWidth  - iSize.iWidth;
            }
        if(aDistance.iY > 0 && iPosition.iY > iWindowSize.iHeight - iSize.iHeight)
            {        
            iPosition.iY = iWindowSize.iHeight - iSize.iHeight;
            }
            
        aDistance.SetXY(iPosition.iX - iPreviousPoint.iX, 
                        iPosition.iY - iPreviousPoint.iY);
        }        
    
        
    // If the navigation is inside the hidden window (e.g., video rect)
    // then need to update the position but dont show the cursor.
    if( iHiddenRect.Intersects(GetRect()) )
        {
        iState = EFlashCursorHiddenMoving;
        iHiddenMovement = ETrue;    
        }
    else
        {
        iHiddenMovement = EFalse;
        iCursorCleared  = EFalse;
        }    
    
    UpdateCursor();
    return retVal;
    }

TPoint CFlashCursor::Position()
    {
    return iPosition;    
    }

TBool CFlashCursor::IsVisible() const
    {
    return (iState != EFlashCursorHidden);
    }

TPoint CFlashCursor::PreviousPosition()
    {
    return iPreviousPoint;    
    }

TSize CFlashCursor::Size()
    {
    return iSize;    
    }
    
void CFlashCursor::SetWindowSize(const TSize& aSize)
    {
    iWindowSize = aSize;
    iActualWindowSize = aSize;
    }
    
void CFlashCursor::SetHiddenWindow(const TRect& aRect)
    {
    iHiddenRect = aRect;
    }
    

TPoint CFlashCursor::HandleNavigation(const TKeyEvent& aKeyEvent,
                                            TEventCode aType, TBool& aBoundaryHit)
{
	TPoint newPos(0, 0);
	aBoundaryHit = EFalse; 
#ifdef  ENABLE_OFN_SUPPORT
	TInt temp=0;
#endif

	if( aType != EEventKey )
	{
		return newPos;
	}

    switch(aKeyEvent.iScanCode)
    {
        case EStdKeyLeftArrow:
        {
            for( TInt step = 1; step <= KStepCount;  step++, newPos.iY = 0 )
            {
                newPos.iX = -1 * step * KCursorStepWidth;
                if( CheckControl(newPos) )
                    break;

                for( TInt i = 1; i <= 2; i++ )
                {
                	newPos.iY = KCursorStepHeight * i;
                	if( CheckControl(newPos) )
                    	break;

                	newPos.iY *= -1;
                	if( CheckControl(newPos) )
                    	break;
                }
            }

            break;
        }
        case EStdKeyRightArrow:
        {
            for( TInt step = 1; step <= KStepCount;  step++, newPos.iY = 0 )
            {
                newPos.iX = step * KCursorStepWidth;
                if( CheckControl(newPos) )
                    break;

                for( TInt i = 1; i <= 2; i++ )
                {
                	newPos.iY = KCursorStepHeight * i;
                	if( CheckControl(newPos) )
                    	break;

                	newPos.iY *= -1;
                	if( CheckControl(newPos) )
                    	break;
                }
            }

            break;
        }
        case EStdKeyDownArrow:
        {
            for( TInt step = 1; step <= KStepCount;  step++, newPos.iX = 0 )
            {
                newPos.iY = step * KCursorStepHeight;
                if( CheckControl(newPos) )
                    break;

                for( TInt i = 1; i <= 2; i++ )
                {
                	newPos.iX = KCursorStepWidth * i;
                	if( CheckControl(newPos) )
                    	break;

                	newPos.iX *= -1;
                	if( CheckControl(newPos) )
                    	break;
                }
            }

            break;
        }
        case EStdKeyUpArrow:
        {
            for( TInt step = 1; step <= KStepCount;  step++, newPos.iX = 0 )
            {
                newPos.iY = -1 * step * KCursorStepHeight;
                if( CheckControl(newPos) )
                    break;

                for( TInt i = 1; i <= 2; i++ )
                {
                	newPos.iX = KCursorStepWidth * i;
                	if( CheckControl(newPos) )
                    	break;

                	newPos.iX *= -1;
                	if( CheckControl(newPos) )
                    	break;
                }
            }

            break;
        }
#ifdef  ENABLE_OFN_SUPPORT
        case EStdKeyLeftUpArrow:
            for( TInt step = 1; step <= KStepCount;  step++, newPos.iY = 0 )
            {
                newPos.iX = -1 * step * KCursorStepWidth;
                if( CheckControl(newPos) )
                    break;

                for( TInt i = 1; i <= 2; i++ )
                {
                    newPos.iY = KCursorStepHeight * i;
                    if( CheckControl(newPos) )
                        break;

                    newPos.iY *= -1;
                    if( CheckControl(newPos) )
                        break;
                }
            }

            temp = newPos.iX;

            for( TInt step = 1; step <= KStepCount;  step++, newPos.iX = 0 )
            {
                newPos.iY = -1 * step * KCursorStepHeight;
                if( CheckControl(newPos) )
                    break;

                for( TInt i = 1; i <= 2; i++ )
                {
                    newPos.iX = KCursorStepWidth * i;
                    if( CheckControl(newPos) )
                        break;

                    newPos.iX *= -1;
                    if( CheckControl(newPos) )
                        break;
                }
            }
            newPos.iX = temp;

            break;
        case EStdKeyRightUpArrow:

            for( TInt step = 1; step <= KStepCount;  step++, newPos.iY = 0 )
            {
                newPos.iX = step * KCursorStepWidth;
                if( CheckControl(newPos) )
                    break;

                for( TInt i = 1; i <= 2; i++ )
                {
                    newPos.iY = KCursorStepHeight * i;
                    if( CheckControl(newPos) )
                        break;

                    newPos.iY *= -1;
                    if( CheckControl(newPos) )
                        break;
                }
            }

            temp = newPos.iX;

            for( TInt step = 1; step <= KStepCount;  step++, newPos.iX = 0 )
            {
                newPos.iY = -1 * step * KCursorStepHeight;
                if( CheckControl(newPos) )
                    break;

                for( TInt i = 1; i <= 2; i++ )
                {
                    newPos.iX = KCursorStepWidth * i;
                    if( CheckControl(newPos) )
                        break;

                    newPos.iX *= -1;
                    if( CheckControl(newPos) )
                        break;
                }
            }
            newPos.iX = temp;


            break;
        case EStdKeyLeftDownArrow:

            for( TInt step = 1; step <= KStepCount;  step++, newPos.iY = 0 )
            {
                newPos.iX = -1 * step * KCursorStepWidth;
                if( CheckControl(newPos) )
                    break;

                for( TInt i = 1; i <= 2; i++ )
                {
                    newPos.iY = KCursorStepHeight * i;
                    if( CheckControl(newPos) )
                        break;

                    newPos.iY *= -1;
                    if( CheckControl(newPos) )
                        break;
                }
            }

            temp = newPos.iX;

            for( TInt step = 1; step <= KStepCount;  step++, newPos.iX = 0 )
            {
                newPos.iY = step * KCursorStepHeight;
                if( CheckControl(newPos) )
                    break;

                for( TInt i = 1; i <= 2; i++ )
                {
                    newPos.iX = KCursorStepWidth * i;
                    if( CheckControl(newPos) )
                        break;

                    newPos.iX *= -1;
                    if( CheckControl(newPos) )
                        break;
                }
            }
            newPos.iX = temp;

            break;
        case EStdKeyRightDownArrow:

            for( TInt step = 1; step <= KStepCount;  step++, newPos.iY = 0 )
            {
                newPos.iX = step * KCursorStepWidth;
                if( CheckControl(newPos) )
                    break;

                for( TInt i = 1; i <= 2; i++ )
                {
                    newPos.iY = KCursorStepHeight * i;
                    if( CheckControl(newPos) )
                        break;

                    newPos.iY *= -1;
                    if( CheckControl(newPos) )
                        break;
                }
            }
            temp = newPos.iX;

            for( TInt step = 1; step <= KStepCount;  step++, newPos.iX = 0 )
            {
                newPos.iY = step * KCursorStepHeight;
                if( CheckControl(newPos) )
                    break;

                for( TInt i = 1; i <= 2; i++ )
                {
                    newPos.iX = KCursorStepWidth * i;
                    if( CheckControl(newPos) )
                        break;

                    newPos.iX *= -1;
                    if( CheckControl(newPos) )
                        break;
                }
            }

            newPos.iX = temp;

            break;
#endif
        default:
        {
            return newPos;        
        }
    }
	
   if(!MoveCursor(newPos))
        aBoundaryHit = ETrue; //There is no moment in the cursor. It has touched the Bounds. 
   return newPos;
}

TBool CFlashCursor::CheckControl(TPoint newPos)
{
    //if ( !iFlashControl )
    if( !iEngine )
	{
		return EFalse;
	}
	
	TPoint currentPosition = iPosition; 
	currentPosition.iX += newPos.iX;
	currentPosition.iX += iSize.iWidth / 4;
	currentPosition.iY += newPos.iY; 
	currentPosition.iY += iSize.iHeight / 4;
	
    
	//return iFlashControl->IsControlPresent(currentPosition);
	return iEngine->IsControlPresent(currentPosition);
}


TFlashCursorState CFlashCursor::State()
    {
    return iState;
    }
    
TRect CFlashCursor::GetRect()
    {
    TRect rect;
    rect.iTl = iPosition;
    rect.SetSize(iSize);
    return rect;
    }
    
void CFlashCursor::UpdateCursor()
{    
    //if ( !iFlashControl )
    if ( !iEngine )
	{
		return;
	}
	
    switch(iState)
    {
        case EFlashCursorHidden:
        {
            TRect rect;
            rect.iTl.iX = rect.iTl.iY = 0;
            rect.SetSize(iWindowSize);

            //iFlashControl->ClearCursor(rect);    
            iEngine->ClearCursor(rect);
            break;
        }
        case EFlashCursorDisplaying:
        {
            DisplayCursor();
            //iFlashControl->DrawCursor();
            iEngine->DrawCursor();
            break;
        }
        case EFlashCursorMoving:
        {
            TRect rect; 
            rect.iTl = iPreviousPoint;
            rect.SetSize(iSize);             
            
            //iFlashControl->ClearCursor(rect);
            iEngine->ClearCursor(rect);
            
            DisplayCursor();
            
            //iFlashControl->DrawCursor();
            iEngine->DrawCursor();
            
            iPreviousPoint = iPosition;                           

            break;
        }
        case EFlashCursorHiddenMoving:
        { 
            if( !iCursorCleared )           
            {
                TRect rect; 
                rect.iTl = iPreviousPoint;
                rect.SetSize(iSize);             
                
                //iFlashControl->ClearCursor(rect);
                iEngine->ClearCursor(rect);
                iCursorCleared = ETrue;
            }
            
            break;
        }
        default:
        {
            return;        
        }
    }
}

void CFlashCursor::SetState(TFlashCursorState aState)
{
    iState = aState;
}
    
void CFlashCursor::NormalScreenMode(CCoeControl* aParentControl)
{
    if(!iFullScreenMode)
    {
        return;
    }
    
    CWsScreenDevice* wsScreen   = CCoeEnv::Static()->ScreenDevice();
        
    if( !wsScreen )
    {
        return;
    }
    
    iFullScreenMode = EFalse;
    
    // ***********************************************************
    // In normal mode, the relative co-ordinates change to :
    // 
    // X = (WindowWidth/ScreenHeight) (x) 
    //
    // Y = (WindowHeight/ScreenWidth) (y)
    //
    // IMPORTANT : This is being calculated when the view has already 
    //             switched to normal screen. If that is not the case,
    //             swap the ScreenHeight and ScreenWidth in the above 
    //             formula.    
    // 
    // ***********************************************************
    
        
    TSize screenSize = wsScreen->SizeInPixels();
    
    TInt newX = ((TReal) iActualWindowSize.iWidth  * iPosition.iX / screenSize.iHeight);
    TInt newY = ((TReal) iActualWindowSize.iHeight * iPosition.iY / screenSize.iWidth);
    
    if(iWindowSize.iWidth == screenSize.iWidth)
	{
		newX = ((TReal) iActualWindowSize.iWidth  * iPosition.iX / screenSize.iWidth);
    	newY = ((TReal) iActualWindowSize.iHeight * iPosition.iY / screenSize.iHeight);	
	}
    
    iPosition.SetXY(newX, newY);
    iPreviousPoint  = iPosition;
    iState          = EFlashCursorDisplaying;
    
    iWindowSize = iActualWindowSize;
    
	//Now the windowing parent of the cursor is normal screen plugin control
    iParentControl = aParentControl;

	CloseSprite();
	CreateSprite();	

    UpdateCursor();
}
    
void CFlashCursor::FullScreenMode(CCoeControl* aParentControl)
{
    CWsScreenDevice* wsScreen   = CCoeEnv::Static()->ScreenDevice();
        
    if( !wsScreen )
    {
        return;
    }
    
    iFullScreenMode = ETrue;
    
    // ***********************************************************
    // In fullscreen mode, the relative co-ordinates change to :
    // 
    // X = (ScreenWidth/WindowWidth) (x) 
    //
    // Y = (ScreenHeight/WindowHeight) (y)
    // 
    // IMPORTANT : This is being calculated when the view has already 
    //             switched to full screen. If that is not the case,
    //             swap the ScreenHeight and ScreenWidth in the above 
    //             formula.   
    //
    // ***********************************************************
        
    TSize screenSize = wsScreen->SizeInPixels();
    
    TInt newX = ((TReal) screenSize.iWidth  * iPosition.iX / iWindowSize.iWidth);
    TInt newY = ((TReal) screenSize.iHeight * iPosition.iY / iWindowSize.iHeight);

    iPosition.SetXY(newX, newY);
    iPreviousPoint  = iPosition;
    iState          = EFlashCursorDisplaying;
    
    iWindowSize = screenSize;

	//Now the windowing parent of the cursor is full screen plugin control
    iParentControl = aParentControl;

	CloseSprite();
	CreateSprite();

    UpdateCursor();
}

void CFlashCursor::CreateSprite()
{
	if(iSpriteCreated)
	{
		CloseSprite();
	}

	//Now recreate sprite with the normal screen plugin control
	iSprite = RWsSprite(CCoeEnv::Static()->WsSession());
	RWindowTreeNode *window =  iParentControl->DrawableWindow();
	
    // Removed as part of fix for iSource #8885
    // Cursor was getting created at position (5,5). This was quite noticible.
    //iSprite.Construct(*window,TPoint(5,5),ESpriteNoChildClip);

	// Create sprite at the current sursor position
      iSprite.Construct(*window,TPoint(iPosition.iX,iPosition.iY),ESpriteNoChildClip);

	TSpriteMember spriteMem;
	spriteMem.iBitmap = iBitmap;
	spriteMem.iMaskBitmap = iBitmapMask;
	spriteMem.iInvertMask = ETrue;

	iSprite.AppendMember(spriteMem);
	iSprite.Activate();

	iSpriteCreated = ETrue;
}

void CFlashCursor::CloseSprite()
{
	if(iSpriteCreated)
	{
		//A cursor sprite is already open -
		//the one created during construction or 
		//the one created while switching to normal screen.
		iSprite.Close();
	}

	iSpriteCreated = EFalse;
}

void CFlashCursor::RecreateCursor()
{
    CloseSprite();
    CreateSprite();

}
void CFlashCursor::ShowCursor() 
{
    ShowCursor(iPreviousPoint);
}
#endif // ENABLE_CURSOR
