/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
*
*/


#include "saplayer_build.h"

#include <e32std.h>
#include <w32std.h>
#include <bautils.h>
#include <coemain.h>
#include <AknIconUtils.h>
#include <gdi.h>

#ifdef __SCALABLE_ICONS
#include <FLASHPLUGIN.MBG>
#else
#include <flashplugin_bitmap.mbg>
#endif

#include <bitdev.h>
#include <bitstd.h>
#include "flashiconhandler.h"

const TInt KErrorImageWidth = 20;
const TInt KErrorImageHeight = 20;

_LIT(KFlashPluginMifFilePath,"\\resource\\apps\\Flashplugin.mif");

CFlashIconHandler* CFlashIconHandler::CreateIconHandlerL()
{
    CFlashIconHandler* self = new (ELeave) CFlashIconHandler();
    return self;
}

CFlashIconHandler::CFlashIconHandler() :
	iIconLoaded(eIconNotLoaded)
{
}

CFlashIconHandler::~CFlashIconHandler()
{
    if (iErrorImageBitmap)
    {
        delete iErrorImageBitmap;
    }

    if (iErrorImageBitmapMask)
    {
        delete iErrorImageBitmapMask;
    }
}

void CFlashIconHandler::Show(CBitmapContext *aGraphicsContext, TWindowMode aWMode, EIcons eIconId, const TRect& aIconRegion)
{
    if (eIconNotLoaded == iIconLoaded)
    {
        TRAPD(err,LoadIconsL());
        
        if(err!=KErrNone)
            return; //Do nothing..Anything better to do when there is an error
                    //and if we cant show error icon?
    }

    if (eErrorIcon == eIconId)
    {       
        if(aWMode == EWindowless)
        {
            aGraphicsContext->SetBrushColor(TRgb(0,0,0,0));
            aGraphicsContext->SetBrushStyle(CGraphicsContext::ESolidBrush);
            aGraphicsContext->SetDrawMode(CGraphicsContext::EDrawModeWriteAlpha);
            aGraphicsContext->SetPenColor(TRgb(0,0,0,0));
            aGraphicsContext->DrawRect(aIconRegion);        
        }
        else
        {
            aGraphicsContext->SetBrushColor(KRgbWhite);
            aGraphicsContext->SetBrushStyle(CGraphicsContext::ESolidBrush);
            aGraphicsContext->Clear();        
        }
        if(iErrorImageBitmap)
        {
	        TSize bitmapSize = iErrorImageBitmap->SizeInPixels();

	        /*TInt xPos = Rect().iTl.iX + (Size().iWidth / 2) - (bitmapSize.iWidth /2);
	         TInt yPos = Rect().iTl.iY + (Size().iHeight / 2) - (bitmapSize.iHeight /2);
         
	         TPoint centerPoint(xPos, yPos);*/
	        TInt yPos = aIconRegion.iTl.iY;
	        TPoint topLeft(0, yPos);
        
	        TRect bitmapRect(0, 0, bitmapSize.iWidth, bitmapSize.iHeight);
	        aGraphicsContext->BitBltMasked(topLeft, iErrorImageBitmap, bitmapRect,
	                iErrorImageBitmapMask, ETrue);
	        
#ifdef SYMBIAN_BUILD_GCE
	        CCoeEnv::Static()->WsSession().Finish();
#endif        
        }
        return;
    } 
}

void CFlashIconHandler::LoadIconsL()
{
    TFileName rscfn; //cautious symbian does not recommend auto var for TfileName
    TFileName dllName;
    Dll::FileName(dllName);
    const TUint KDriveLetterLength = 2;
    TBuf<KDriveLetterLength> drive = dllName.Left(KDriveLetterLength);

    rscfn.Copy(drive); // get the drive letter
    rscfn.Append(KFlashPluginMifFilePath);

    TBool fileExists(BaflUtils::FileExists(CCoeEnv::Static()->FsSession(),
            rscfn));

    if (!fileExists)
    {
        // If we didn't find the resource file, then we should leave 
        User::Leave(KErrNotFound);
    }

    TInt iconId = 0;
    TInt iconMaskId = 0;

#ifdef __SCALABLE_ICONS

    //const TInt KBrokenIconId = EMbmFlashpluginQgn_indi_flash_error;
    //const TInt KBrokenIconMaskId = EMbmFlashpluginQgn_indi_flash_error;
    iconId = EMbmFlashpluginQgn_indi_flash_error;
    iconMaskId = EMbmFlashpluginQgn_indi_flash_error;

#else

    //const TInt KBrokenBitmapIconId = EMbmFlashplugin_bitmapQgn_indi_flash_error;
    //const TInt KBrokenBitmapIconMaskId = EMbmFlashplugin_bitmapQgn_indi_flash_error;

    iconId = EMbmFlashplugin_bitmapQgn_indi_flash_error;
    iconMaskId = EMbmFlashplugin_bitmapQgn_indi_flash_error;

#endif

    AknIconUtils::CreateIconL(iErrorImageBitmap, iErrorImageBitmapMask,
            rscfn, iconId, iconMaskId);

    if (AknIconUtils::IsMifIcon(iErrorImageBitmap))
    {
        //TODO : How to notify the parent control that the size is changed ??
        //CCoeControl::SizeChanged(); // Init layout

        TSize bitmapSize(KErrorImageWidth, KErrorImageHeight);

        // Sets the sizes of both bitmap and mask
        AknIconUtils::SetSize(iErrorImageBitmap, bitmapSize);
    }

    iIconLoaded = eIconLoaded;
}
