/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CFlashPluginControl implementation file
*
*/

#include "saplayer_build.h"
#include "flashplugincontrol.h"
#include "NPNFlashExtension.h"
#include "flash_engine.h"
#include "saflash_consts.h"
#include "symbian_textinput.h"
#include "amp_mtextnavigation.h"
#include "flashpluginif.h"
#include <eikenv.h>
#include <AknQueryDialog.h>
#include <flashliteplugin.rsg>
#include <featmgr.h>
#include "mp3streamstateobserver.h"

#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
#include <bautils.h>
#include <aknstyluspopupmenu.h>
#include "flashliteplugin.hrh"
#include <flashliteplugin.rsg>
#endif


//#ifdef PLUG_IN_MEM_ALLOC
//#include "SwfMemMgr.h"
//#include "PluginInstanceMemInfo.h"
//#endif

//#include "FlashConfigGen.h"

//const TUint KVolumeValue = 6;
//const TUint32 KBrowserUid = 0x10008D39;
//const TUint32 KScreenSaverUid = 0x100056CF;

extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );
extern HBufC8* GetBaseURL8(const TDesC& aBaseUrl);


// -------------------------------------------------------------------------
// CFlashPluginControl::CFlashPluginControl(CFlashPluginInstance* aInstance)
//
// First phase constructor
// --------------------------------------------------------------------------
// 
CFlashPluginControl::CFlashPluginControl(MFlashPluginInstance* aInstance):PluginBase(aInstance)
								, iPluginActivated(EFalse)
                            	, iNote(NULL)
#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
                            	, iStylusPopupMenu(NULL)
                            	, iContextMenuVisible(EFalse)

#endif
{	
}

// ----------------------------------------------------------------------
// CFlashPluginControl::~CFlashPluginControl()
//
// Destructor
// ----------------------------------------------------------------------
// 
CFlashPluginControl::~CFlashPluginControl()
{
	NPPDebug(LOG_DEBUG,_L("CFlashPluginControl::~CFlashPluginControl() ++"));
#ifdef GESTURE_LIBRARY_SUPPORTED
	if(IsConstructed())
		Window().FreePointerMoveBuffer();
#endif // GESTURE_LIBRARY_SUPPORTED	
	
/*//Moved to ~PluginBase
#ifdef ENABLE_CURSOR	

    if( iCursor )    	
        {
        delete iCursor;
        iCursor = NULL;
        }
        
#endif
*/

	if(iNote)
	{
		delete iNote;
		iNote = NULL;	
	}

	NPPDebug(LOG_DEBUG,_L("CFlashPluginControl::~CFlashPluginControl() --"));
}
// ---------------------------------------------------------------------------
// void CFlashScreensaverControl::RecreateBitmapL(const TRect& aRect)
//
// This function is used to pass the rectangle parameters to the host engine
// ---------------------------------------------------------------------------
//
void CFlashPluginControl::RecreateBitmapL(const TRect& aRect)
{
    MFlashEngine* engine = iPlayerInstance->GetEngine();
    if(!engine)
    {
        return;    
    }    
    
    iValid = EFalse;
    
    TBool isInit = IsInitialized();
    
    engine->InitializeL(aRect, isInit);

    iValid = ETrue;
}
    
void CFlashPluginControl::SizeChanged()
{
	TRect newRect = Rect();
		
	if( iExtent.Size() != newRect.Size() )
	{    
		iExtent = newRect;
		
		TRAPD(err, RecreateBitmapL(iExtent)); 
		// Display the broken flash icon whenever 
		// RecreateBitmapL function leaves especially because 
		// of out of memory condition. Fix for bug ESJY-7QEHPF.
		if(err != KErrNone)
		    {
		    NotifyClientError(err, ETrue, ETrue);
		    }
		    
		#ifdef ENABLE_CURSOR    
    
	    if( iCursor )
	    {
	        iCursor->SetWindowSize(newRect.Size());  
	    }
	
		#endif    
	}
}

void CFlashPluginControl::SetRectRelativeToParentControl(const TRect& aRect)
{
	iRectRelativeToParentControl = aRect;
}


// ----------------------------------------------------------------------
// void CFlashPluginControl::CreateWindowL(const CCoeControl* aParent, 
//    const TRect& aRect)
//
// This function is used to create a new window owning control and 
// display it in the specified rectangle
// ----------------------------------------------------------------------
// 
void CFlashPluginControl::CreateWindowL(const CCoeControl* aParent, 
                                        const TRect& aRect)
    {
    // set protection at start in case NPP_SetWindow is called again before
    // this function completes (eg. lots of plugins loaded simultaneously)
    
    iConstructed = ETrue;

    CCoeControl::CreateWindowL(aParent);
    
    //Window().PointerFilter(EPointerFilterDrag, 0);
    //As such a platform specific flag is not found for multitouch events
    //when found replace this flag with that
    //Advancepointerevent.h is a new addition to the HW79 SDK
    #ifdef ADVANCEDPOINTEREVENT_H
    Window().EnableAdvancedPointers();
    #endif
    //This line here solves two bugs
    //1.Panning with single finger used to zoom in/out the page
    //2.Double tap didn't work after the pinch zoom
    //This is used to get the EButtonUp event to the owning
    //window and not the window its extended to.
    DrawableWindow()->SetPointerGrab(ETrue);
	CCoeControl::EnableDragEvents(); 
    RecreateBitmapL(aRect);
    
    iExtent = aRect;

#ifdef ENABLE_CURSOR    
    
    if( iCursor )
        {
        iCursor->SetWindowSize(aRect.Size());  
        }
#endif    

    ActivateL();
    }

void CFlashPluginControl::SetWindowL(MPluginAdapter* aPlugin
                                      , const TRect& aRect
                                      , const TRect& aClipRect)
{

  iPlugin = aPlugin;
  
    
  if (!IsVisible())
        {
//        NPPDebug(LOG_INFO,_L("CFlashPluginInstance::SetWindowL() : Control [%x] no longer visible"), iPluginView);
        }

  SetClipRect(aClipRect);
     
  if (!IsConstructed())
        {
        // complete control construction now we know its size

        CreateWindowL((CCoeControl*)aPlugin->GetParentControl(),aRect);
        SetObserver(aPlugin->GetParentControlObserver());
        aPlugin->SetPluginNotifier(this);
        aPlugin->PluginConstructedL(this);
        iValid=ETrue;
        }
}
//TKeyResponse CFlashPluginControl::HandleKeyEventL(const TKeyEvent& aKeyEvent, 
//                                                 TEventCode aType)
//{
//    return EKeyWasNotConsumed;
//}

// ----------------------------------------------------------------------
// TKeyResponse CFlashPluginControl::OfferKeyEventL(
//    const TKeyEvent& aKeyEvent, TEventCode aType)
//
// This function is used to pass a key event to the host engine
// ----------------------------------------------------------------------
// 
TKeyResponse CFlashPluginControl::OfferKeyEventL(const TKeyEvent& aKeyEvent, 
                                                 TEventCode aType)
    {
    TKeyEvent keyEvent = aKeyEvent;   
    TEventCode eventType = aType;
    
    MFlashEngine* engine = iPlayerInstance->GetEngine();
    
    if( !engine )
        {
        return EKeyWasNotConsumed;
        }
         
    // This change is from S60 Rel 3.2 WK 44, 2006
    // In case of EStdKeyDevice3 (Select key) the order is found to be 
    // EEventKeyDown => EKeyEventUp => EKeyEventUp
    // So there is no EEventKey event generated.
    // So we send this in between down and up events for the EStdKeyDevice3 key
    
    if( aKeyEvent.iScanCode == EStdKeyDevice3 )
        {
        if( iLastKeyEventCode == EEventKeyDown && 
            aType == EEventKeyUp )
            {
            eventType = EEventKey;                      
            }
        }
 
    TKeyResponse keyResponse = EKeyWasNotConsumed;
 
//    MFlashEngine* engine = FlashPluginInstance()->GetEngine();
    keyResponse = engine->OfferKeyEventL(keyEvent, eventType);
        
    if( aKeyEvent.iScanCode == EStdKeyDevice3 )
        {
        if( iLastKeyEventCode == EEventKeyDown && 
            aType == EEventKeyUp )
            {
            iLastKeyEventCode = EEventNull;
            }
        else
            {
            iLastKeyEventCode = eventType;
            }
        }
#ifdef FEATURE_INLINE_TEXT_ENTRY
    if( iIMEControl && keyResponse )   
    	{
    	iIMEControl->ReportStateEventL();
    	}
#endif
    return keyResponse;
    }

#ifdef GESTURE_LIBRARY_SUPPORTED
// ----------------------------------------------------------------------
// void CFlashPluginControl::HandleGestureFromBrowserL(const TPointerEvent &aPointerEvent)
//Handle the gesture events from browser
// ----------------------------------------------------------------------
//

TBool CFlashPluginControl::HandleGestureFromBrowserL(void* /*aEvent*/)
    {
        return EFalse;
    }



// ----------------------------------------------------------------------
// void CFlashPluginControl::HandlePointerBufferReadyL(const TPointerEvent &aPointerEvent)
//Handle the move events 
// ----------------------------------------------------------------------
//
void CFlashPluginControl::HandlePointerBufferReadyL()
{
    
}
#endif//GESTURE_LIBRARY_SUPPORTED
// ----------------------------------------------------------------------
// void CFlashPluginControl::HandlePointerEventL(const TPointerEvent &aPointerEvent)
//
// This function is used to pass a joystick/pointer event to the host engine
// ----------------------------------------------------------------------
//     
void CFlashPluginControl::HandlePointerEventL(const TPointerEvent &aPointerEvent)
    {
    CCoeControl::HandlePointerEventL(aPointerEvent);
    FeatureManager::InitializeLibL(); // initialise feature manager
	if(FeatureManager::FeatureSupported(KFeatureIdPenSupport)) //adding feature manager to check the feature at run time.
		{
	    if( !IsInitialized() ) 
	        {
	        return;  
	        } 
	    MFlashEngine* engine = iPlayerInstance->GetEngine();
	    if(engine)
	        {
	        TPoint point(aPointerEvent.iPosition);
	        engine->OfferPointerEventL(point, (TInt) aPointerEvent.iType);
	        }
		}
	FeatureManager::UnInitializeLib(); // Uninitialise Feature manager
    }
    


// ----------------------------------------------------------------------
// void CFlashPluginControl::OpenFileL(const TDesC& aFilename, 
//  const TDesC* aUrl)
// This function is used to play a swf file
// ----------------------------------------------------------------------
// 
void CFlashPluginControl::OpenFileL(const TDesC& aFilename, 
                                    const TDesC* aUrl,
                                    const TBool& aFirstMovie,
                                    CContent* aContent)
    {
    NPPDebug(LOG_DEBUG,_L("CFlashPluginControl::OpenFileL() ++"));
    
    if (IsConstructed())
        {
        MFlashEngine* engine = iPlayerInstance->GetEngine();
        
        if (engine)
            {
            TInt result;
            if(!engine->IsInitialized())
                {                
                TRAP(result, engine->InitializeL(Rect(), EFalse));
                //TRAP(result, iEngine->InitializePluginL(this, Rect(), EFalse));
                }
            else
                {
                TRAP(result, engine->InitializeL(Rect(), ETrue));
                //TRAP(result, iEngine->InitializePluginL(this, Rect(), ETrue));
                }    
            if (result == KErrNoMemory)
                {
                // Display the broken flash icon whenever 
                // InitializePluginL function leaves especially
                // because of out of memory condition. Fix for bug ESJY-7QEHPF
                NotifyClientError(result, ETrue, ETrue);
                return;
                }
                
            NPPDebug(LOG_INFO,_L("CFlashPluginControl::OpenFileL() : Sending file to Host engine"));
           
            //Moved engine->SetBaseUrlL() before  engine->OpenBrowserFileAndStartMovieL()
            //as base url is required in OpenBrowserFileAndStartMovieL()
            
            HBufC8* baseUrl8 = GetBaseURL8(iPlayerInstance->GetBaseURL());
            
            HBufC8* securityContextUrl = GetBaseURL8(iPlayerInstance->GetSecurityContextURL());
            if (securityContextUrl != NULL)
                {
                engine->SetSecurityContextUrl(securityContextUrl);
                }
            engine->SetBaseUrlL(baseUrl8);
            if (baseUrl8)
            	delete baseUrl8;
            
            if (securityContextUrl)
                delete securityContextUrl;
                       
            engine->OpenBrowserFileAndStartMovieL
            (
            aFilename, 
            ETrue, 
            aUrl, 
            aFirstMovie,
            aContent
            );                
            
           
            
            iFileOpened = ETrue;
                          
            }             

            iFilename = aFilename;
            
            DrawDeferred();
        }
    
    NPPDebug(LOG_DEBUG,_L("CFlashPluginControl::OpenFileL() --"));
    }
    
/*TBool CFlashPluginControl::IsConstructed() const
{ 
	return PluginBase::IsConstructed(); 
}
    
    
TBool CFlashPluginControl::IsInitialized() const
{ 
    return PluginBase::IsInitialized();
}

TBool CFlashPluginControl::IsValid() const
{
	return PluginBase::IsValid();
}

TBool CFlashPluginControl::IsFileOpened() const
{ 
	return PluginBase::IsFileOpened(); 
}*/
    
void CFlashPluginControl::ShowTipsL(TInt aResID)
{
	CCoeEnv* coeenv = iCoeEnv;
	if(!iNote)
	{
		iNote = CAknInfoPopupNoteController::NewL();	
	}
	
	HBufC* popTxt = HBufC16::NewLC(256);//popTxt pushed 
	TPtr popPtr(popTxt->Des());
	
	coeenv->ReadResourceL(popPtr,aResID);
	iNote->SetTextL( popPtr );
    //iNote->SetTimeDelayBeforeShow( aDelay );
    //iNote->SetTimePopupInView( aDuration );
    iNote->ShowInfoPopupNote();
    CleanupStack::PopAndDestroy(popTxt);//pop and destroy
}

void CFlashPluginControl::HideTips()
{
	if( iNote )
	{
		iNote->HideInfoPopupNote();
	}
}

/**  Gets the player control's window, if the player view has been created.
  @return handle to the player control's window.
 */
RWindow* CFlashPluginControl::GetWindow() 
{
	 return &Window(); 
};

//// from textInput
///**
// Specifies if the text is being input.
// @return TBool ETrue if the user has not yet dismissed the text input
// dialog.
//*/
//TBool CFlashPluginControl::IsTextInputBusy() 
//{
//	return EFalse;
//}
///**
// This function initiates a modal input text dialog, pausing the 
// movie until the user dismisses the dialog
// @param aData structure for the input text data.
// */
//void CFlashPluginControl::GetInputTextL(TGetInputTextData& /*aData */) 
//{
//}
    
/**
 Notify the plugin of any run time error which occurs 
 @param aErrorCode The error code corresponding to the error which has occured
*/
void  CFlashPluginControl::NotifyError(TInt /*aErrorCode*/, TBool /*aIsCritical*/ )
{
}

/**
    Gets the plugin control's extent 
    @return The rect of the plugin control
*/

TRect CFlashPluginControl::GetRect()
{
    return Rect();
}
    
void CFlashPluginControl::DrawDirtyRect(const TRect& aDirtyRect)
{
	if( !aDirtyRect.IsEmpty() )
		DrawNow(aDirtyRect);
}

void CFlashPluginControl::SetFullScreenL(TBool /*aSetFullScreen*/)
{
	//Base class implementation. 
	User::Leave(KErrNotSupported);
}

void CFlashPluginControl::SetScreenL(EScreenMode /*aSetFullScreen*/)
{
	//Base class implementation. 
	User::Leave(KErrNotSupported);
}

// ----------------------------------------------------------------------
// void CFlashPluginControl::Draw(const TRect& aRect) const
//
// This function is used to draw the view
// ----------------------------------------------------------------------
// 
void CFlashPluginControl::Draw(const TRect& aRect) const
{
    MFlashEngine* engine = iPlayerInstance->GetEngine();
    
	if (engine)
	    {
	    CWindowGc& gc = SystemGc();
	    engine->SetForeground(ETrue);
	    engine->Draw(aRect, gc);
	    }
}

/*void CFlashPluginControl::PrepareToExitL()
{	
}*/

/**
	 This function is called by the host engine when there is an error in
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/
	
void CFlashPluginControl::NotifyFlashError(TInt aErrorCode, TBool aIsCritical, TBool /*aPersistent*/ )
{
	NPPDebug(LOG_ERROR,_L("CFlashPluginControl::NotifyFlashError() : ERROR = [%d]"), aErrorCode);
	
	iLastError = aErrorCode;
	
	if( !aIsCritical )
	    {
	    return;
	    }
	iPlayerInstance->PrepareToExit();
}

/**
	 This function is called by the host engine when there is an error in
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/
	
void CFlashPluginControl::NotifyClientError(TInt aErrorCode, TBool aIsCritical, TBool /*aPersistent*/)
{
	NPPDebug(LOG_ERROR,_L("CFlashPluginControl::NotifyClientError() : ERROR = [%d]"), aErrorCode);
	
	iLastError = aErrorCode;
	
	if( !aIsCritical )
	    {
	    return;
	    }
	
	iPlayerInstance->PrepareToExit();
}

/**
	 This function is called by the host engine flash status
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/

void CFlashPluginControl::NotifyFlashStatus(TInt aFlashStatus, TAny* /*aParam*/) 
{
	NPPDebug(LOG_INFO,_L("CFlashPluginControl::NotifyFlashStatus() : ERROR = [%d]"), aFlashStatus);
}

#ifdef ENABLE_CURSOR

// ------- Cursor Related -------------------

MFlashCursor* CFlashPluginControl::Cursor()
{
    return iCursor;
}
	
CFbsBitGc* CFlashPluginControl::GetBitmapContext()	
    {
    MFlashEngine* engine = iPlayerInstance->GetEngine();
    if( !engine ) 
        return NULL;
    
    return engine->GetBitmapContext();
    }

void CFlashPluginControl::DrawCursor()	
    {
    MFlashEngine* engine = iPlayerInstance->GetEngine();
    
    if( !engine ) 
        return;
    
    engine->DrawCursor();
    }

void CFlashPluginControl::ClearCursor(const TRect& aRect)	
    {
    MFlashEngine* engine = iPlayerInstance->GetEngine();
    
    if( !engine ) 
        return;
    
    engine->ClearCursor(aRect);
    }
    
#endif
TBool CFlashPluginControl::IsControlPresent(const TPoint& aPoint)
    {
    MFlashEngine* engine = iPlayerInstance->GetEngine();
    
    if(!engine)
        return EFalse;
    
    return engine->IsControlPresent(aPoint);
    }
    
//-----------------------------------------------


//#ifdef FEATURE_INLINE_TEXT_ENTRY
//
//void CFlashPluginControl::EnableIMEL()
//    {
//    iIsIMEEnabled = ETrue;
//	CAknSettingCache& cache = CAknEnv::Static()->SettingCache();
//    if (CurrentInputLanguage() != cache.InputLanguage())
//	    {
//	    SetCurrentInputLanguage(cache.InputLanguage());	
//	    }
//	
//    MFlashEngine* engine = iPlayerInstance->GetEngine();
//    
//    InputModeSelector* inputModeSelector 
//    = InputModeSelector::NewL(engine->GetTextNavigation(), this);
//    CleanupStack::PushL(inputModeSelector);
//
//		if( !inputModeSelector )
//			User::Leave(KErrNoMemory);
//                                        
//    iIsNoninline  = inputModeSelector->IsNoninline();
//	
//	if ((iIMEControl) && !iIMEControl->IsInLine()) 
//	    {	
//		delete iIMEControl;
//		iIMEControl = NULL;		
//	    }
//
//	if(!iIMEControl)
//    	{
//		if (iIsNoninline) 
//			{	
//			iIMEControl = CFlashPluginIMENonInlineControl::NewL(engine,
//																this,
//																PositionRelativeToScreen());
//	
//			}
//		else 
//			{
//			iIMEControl = CFlashPluginIMEControl::NewL(engine,
//													  this,
//													  PositionRelativeToScreen());
//            
//            (STATIC_CAST(CFlashPluginIMEControl*, iIMEControl))
//            ->SetCurrentInputLanguage(iCurrentInputLanguage);
//			}
//
//#ifdef ENABLE_CURSOR
//			
//        // Hide the flash cursor (if exists) during inline editing.
//        if( iCursor )
//            {
//            iCursor->HideCursor();
//            }
//#endif
//        }
//        
//    CleanupStack::PopAndDestroy(); // inputModeSelector
//    }
//
//void CFlashPluginControl::DisableIME()
//{   
//    if(iIMEControl)
//	    {
//	    
//	    if( iIMEControl->IsInLine() )
//	        {
//	        CFlashPluginIMEControl* ime = (CFlashPluginIMEControl*) iIMEControl;
//	        ime->DisableIME();
//	        }
//	    
//		delete iIMEControl;
//		iIMEControl = NULL;
//		iIsIMEEnabled = EFalse;
//		iIsNoninline = EFalse;
//
//		MFlashEngine* engine = iPlayerInstance->GetEngine();
//		
//		engine->GetTextNavigation()->ShowCursor(ETrue);
//		engine->RestoreSoftKeys();
//
//// as we are displaying the cursor when user presses the navigation
//// keys so no need to display it here
///*#ifdef ENABLE_CURSOR
//		
//		// Re-show the flash cursor (if exists) 
//        if( iCursor )
//            {
//            iCursor->ShowCursor(iCursor->Position());            
//            }
//#endif*/
//    	}
//}
//    
//TInt CFlashPluginControl::GetCurrentInputMode()
//    {
//    return 0;
//    }
//    
//void CFlashPluginControl::GetInputTextL(const TUint16* aInitialString, 
//                                        TUint aStrLen, 
//                                        TUint aMaxNumChars, 
//                                        TUint aTextFlags)
//	{
//	TGetInputTextData *getInputTextRequest = new (ELeave) TGetInputTextData;
//	TPtrC16 ptr(aInitialString, aStrLen);
//	
//	getInputTextRequest->iInitialString = ptr.AllocL();  
//	
//	if (aMaxNumChars > KMaxNumChar)
//		{
//		// Max number characters which can be entered to text field.
//		// Host will never get Zero for it.
//		aMaxNumChars =  KMaxNumChar ;
//		}	
//		
//	getInputTextRequest->iMaxNumChars = aMaxNumChars;
//	getInputTextRequest->iTextFlags   = aTextFlags;
//	
//	GetInputTextL(*getInputTextRequest);
//
//	delete getInputTextRequest->iInitialString;
//	delete getInputTextRequest;
//	}
	
	
//void CFlashPluginControl::ResetIMEL()
//{
//    if( !iIMEControl )
//        return;
//    
//    SimulateDisableIMEEventL();
//    SimulateEnableIMEEventL();
//}

//void CFlashPluginControl::SimulateDisableIMEEventL()
//{
//    TKeyEvent keyEvent;
//        
//    keyEvent.iCode      = 0;
//    keyEvent.iScanCode  = EStdKeyDevice3;
//    keyEvent.iModifiers = 0;
//    keyEvent.iRepeats   = 0;   
//    
//    MFlashEngine* engine = iPlayerInstance->GetEngine();
//    
//    if( !engine )
//        return;
//    
//    // NOTE : This will disable IME and delete this.
//    // So should be the last statement here. 
//    
//    engine->OfferKeyEventL(keyEvent, EEventKey);
//        
//    return;    
//}
//
//void CFlashPluginControl::SimulateEnableIMEEventL()
//{
//    TKeyEvent keyEvent;
//        
//    keyEvent.iCode      = 0;
//    keyEvent.iScanCode  = EStdKeyDevice3;
//    keyEvent.iModifiers = 0;
//    keyEvent.iRepeats   = 0;   
//    
//    MFlashEngine* engine = iPlayerInstance->GetEngine();
//    
//    if( !engine )
//        return;
//    
//    engine->OfferKeyEventL(keyEvent, EEventKey);
//        
//    return;    
//}
//#endif

#ifdef __FEATURE_PLUGIN_UI_OPTIONS

MOptionMenuHandler* CFlashPluginControl::GetOptionHandler()
{
	return NULL;
}

#endif


TBool CFlashPluginControl::IsFLVVideo()
{
	return EFalse;
}

void CFlashPluginControl::FLVideoStarted()
{
	//do nothing function
}

TBool CFlashPluginControl::IsPluginActivated()
{
	return iPluginActivated;
}


TBool CFlashPluginControl::IsFullScreenMode()
{
	return EFalse;
}


/*TBool CFlashPluginControl::IsControlVisible() const
{
	return PluginBase::IsControlVisible();
}*/

void CFlashPluginControl::SetControlVisible(TFlashPluginVisibilityState aVisible)
{
	iVisibilityState = aVisible;
	NPPDebug(LOG_DEBUG,_L("CFlashPluginControl::SetControlVisible() : Visibility = [%d]"), iVisibilityState);
}

void CFlashPluginControl::SetPluginFocus(TFlashPluginFocusState aFocus)
{
    iFocusState = aFocus;
}

void CFlashPluginControl::NotifyFocusChangeL(TBool aFocus)
{
    MFlashEngine* engine = iPlayerInstance->GetEngine();

	if( !engine )
	{
		return;
	}
	
	engine->NotifyFocusChangeL(aFocus);
}


void CFlashPluginControl::SetVideoDisplayRectL(const TRect& /*aRect*/, const TRect& /*aClipRect*/)
{
	// Do nothing. 
}

void CFlashPluginControl::HandleVideoError(const TInt& aError)
{
	NPPDebug(LOG_ERROR,_L("CFlashPluginControl::HandleVideoError() : ERROR = [%d]"), aError);
	
	iLastError = aError;
	
	iPlayerInstance->PrepareToExit();
}

void CFlashPluginControl::AddEventObserverToControl()
    {
    
    }


void CFlashPluginControl::RemoveEventObserverFromControl()
    {
    
    }


void CFlashPluginControl::CreateQwertyKeypadObserverL()
{
    PluginBase::CreateQwertyKeypadObserverL();
	/*// Start listening qwerty mode status.
    User::LeaveIfError(iQwertyModeStatusProperty.Attach(KCRUidAvkon,KAknQwertyInputModeActive));
    iQwertyModeStatusSubscriber = new (ELeave) CKeypadHandler(*this, iQwertyModeStatusProperty);
    iQwertyMode = iQwertyModeStatusSubscriber->GetQwertMode();
    iQwertyModeStatusSubscriber->SubscribeL();*/
}

void CFlashPluginControl::HandleQwertyModeChangeNotification(TInt aQwertyMode)
{
    PluginBase::HandleQwertyModeChangeNotification(aQwertyMode);
	//iQwertyMode = aQwertyMode;

	#ifdef ENABLE_CURSOR
	// Hide the flash cursor (if exists) when qwerty keypaded slided to OFF state
	if( iCursor && !iQwertyMode && iCursor->IsVisible())
	{
	    iCursor->HideCursor();
	}
	#endif
}

/*//Moved to PluginBase
TInt CFlashPluginControl::GetQwertyKeyPadStatus()
{
 	return iQwertyMode;
}
*/

#ifdef FEATURE_EXTENSION
void CFlashPluginControl::ResumeOnEventL(TBool aForeGround) 
    {
    if(aForeGround)// If at the time of registering for the event, bringToForeground option was enabled, then bring to foreground
        {        
        TApaTask task(iEikonEnv->WsSession( ));
        task.SetWgId(CEikonEnv::Static()->RootWin().Identifier());
        task.BringToForeground();// BringToForeground takes care of resuming        
        }
    else
        {
        MFlashEngine* engine = iPlayerInstance->GetEngine();
        engine->ResumeL(EReasonBackground, ETrue); //If bringToForeground oprions was disabled then resume player only no change needed in UI.
        }
    }
void CFlashPluginControl::PauseOnRequest(TBool aForeGround)
    {
    if(aForeGround) 
        {
        TApaTask task(iEikonEnv->WsSession( ));        
        task.SetWgId(CEikonEnv::Static()->RootWin().Identifier());        
        task.SendToBackground();
        }
    else
        {
        MFlashEngine* engine = iPlayerInstance->GetEngine();
        engine->Pause(EReasonBackground);
        }
    }
#endif

#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
void CFlashPluginControl::DynInitContextSensitiveMenu(MEikMenuObserver* aObserver, const TPoint& aPoint)
{
    if(!iStylusPopupMenu)
    {
        // try to create the pop up menu
        TRAP_IGNORE(
        TBufC<128> KResFile(_L("FlashlitePlugin.rsc") );
        TInt resOffset = PluginBase::LoadPluginResourceL( KResFile );
        
        iStylusPopupMenu = CAknStylusPopUpMenu::NewL(aObserver,aPoint);
        TResourceReader reader;
        CEikonEnv::Static()->CreateResourceReaderLC(reader,R_STYLUS_POPUP_MENU);
        iStylusPopupMenu->ConstructFromResourceL(reader);
        CleanupStack::PopAndDestroy();

        CEikonEnv::Static()->DeleteResourceFile(resOffset);
        );
    }
    
    if(iStylusPopupMenu == NULL)
        return;
    
    // hide all items by default
    iStylusPopupMenu->SetItemDimmed(EEikFlashCmdUnmute,ETrue);
    iStylusPopupMenu->SetItemDimmed(EEikFlashCmdMute,ETrue);
    iStylusPopupMenu->SetItemDimmed(EEikFlashCmdFullScreen,ETrue);
    iStylusPopupMenu->SetItemDimmed(EEikFlashCmdPartScreen,ETrue);

    
    MFlashEngine* engine = iPlayerInstance->GetEngine();
    // decide what to show and what to hide

    if(IsFullScreenMode())
    {
        iStylusPopupMenu->SetItemDimmed(EEikFlashCmdFullScreen,ETrue);
        iStylusPopupMenu->SetItemDimmed(EEikFlashCmdPartScreen,EFalse);
    }
    else
    {
        iStylusPopupMenu->SetItemDimmed(EEikFlashCmdFullScreen,EFalse);
        iStylusPopupMenu->SetItemDimmed(EEikFlashCmdPartScreen,ETrue);
    }

    if(engine->GetSwfHasAudio())
    {
        if(engine->GetVolume() == 0)
        {
            // we have muted
            iStylusPopupMenu->SetItemDimmed(EEikFlashCmdUnmute,EFalse);
            iStylusPopupMenu->SetItemDimmed(EEikFlashCmdMute,ETrue);
        }
        else
        {
            iStylusPopupMenu->SetItemDimmed(EEikFlashCmdMute,EFalse);
            iStylusPopupMenu->SetItemDimmed(EEikFlashCmdUnmute,ETrue);
        }
    }

    iStylusPopupMenu->ShowMenu();
    iContextMenuVisible = ETrue;

    iStylusPopupMenu->SetPosition(aPoint, CAknStylusPopUpMenu::EPositionTypeRightBottom);

}

void CFlashPluginControl::ProcessCommandL(TInt aCommandId)
{
    switch(aCommandId)
    {
    case EEikFlashCmdPartScreen:
    case EEikFlashCmdFullScreen:
        ToggleScreenL();
        break;
    case EEikFlashCmdMute:
        iPlayerInstance->MuteVolume();
        break;
    case EEikFlashCmdUnmute:
        iPlayerInstance->UnMuteVolume();
        break;
    }

    iContextMenuVisible = EFalse;
}

#endif

//End of File
