/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CScreensaverPluginControl class implementation
*
*/


// INCLUDE FILES
#include "saplayer_build.h"
#include "flashscreensavercontrol.h"
//#include "flashScreensaverplugin.h"
#include "flash_engine.h"
#include "saflash_consts.h"
#include "flashliteplugin.hrh"
//#include "FlashConfigGen.h"
#include <coemain.h>
#include <charconv.h>
#include <eikenv.h>
#include <bldvariant.hrh>
#include <featmgr.h>
#include "commondef.h"
#include "flashpluginif.h"

// CONSTANTS
#include "saflash_consts.h"

extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );

// ==========================================================
// FlashPluginControl
// -------------------------------------------------------------------------
// CScreensaverPluginControl* CScreensaverPluginControl::NewL(CFlashPluginInstance* aInstance)
//
// Static constructor
// --------------------------------------------------------------------------
// 
CScreensaverPluginControl* CScreensaverPluginControl::NewL(CFlashPluginInstance* aInstance)
{
	NPPDebug(LOG_DEBUG,_L("CScreensaverPluginControl::NewL() ++"));

    CScreensaverPluginControl* self = new (ELeave) CScreensaverPluginControl(aInstance);

    self->Construct(); // NOTE: Non-leaving.

    NPPDebug(LOG_DEBUG,_L("CScreensaverPluginControl::NewL() --"));
    
    return self;		
}

// -------------------------------------------------------------------------
// CScreensaverPluginControl::CScreensaverPluginControl(CFlashPluginInstance* aInstance)
//
// First phase constructor
// --------------------------------------------------------------------------
// 
CScreensaverPluginControl::CScreensaverPluginControl(CFlashPluginInstance* aInstance)
                    : CFlashPluginControl((MFlashPluginInstance*)aInstance)
                    	
    {
    }

// ----------------------------------------------------------------------
// CScreensaverPluginControl::~CScreensaverPluginControl()
//
// Destructor
// ----------------------------------------------------------------------
// 
CScreensaverPluginControl::~CScreensaverPluginControl()
    {
    }

void CScreensaverPluginControl::Construct()
    {  
    //Moved to CScreensaverPluginInstance::ConstructL
   /* CFlashConfigGen::FillFlashConfig(iPlayerInstance->GetInteractionMode(),
    								 iConfig);    */								 
    }

   
// ----------------------------------------------------------------------
// TKeyResponse CScreensaverPluginControl::OfferKeyEventL(
//    const TKeyEvent& aKeyEvent, TEventCode aType)
//
// This function is used to pass a key event to the host engine
// ----------------------------------------------------------------------
// 
TKeyResponse CScreensaverPluginControl::OfferKeyEventL(
                                        const TKeyEvent& /*aKeyEvent*/, 
                                        TEventCode /*aType*/)
    {
       return EKeyWasNotConsumed;
    }

// ----------------------------------------------------------------------
// void CScreensaverPluginControl::HandlePointerEventL(const TPointerEvent &aPointerEvent)
//
// This function is used to pass a joystick/pointer event to the host engine
// ----------------------------------------------------------------------
//
void CScreensaverPluginControl::HandlePointerEventL(const TPointerEvent& /*aPointerEvent*/)
{
}

// ---------------------------------------------------------------------------
// TInt CScreensaverPluginControl::NotifyL(TNotificationType aCallType, void* aParam)
//
// This function is used to send notifications from the browser to the host 
// engine
// ---------------------------------------------------------------------------
//
TInt CScreensaverPluginControl::NotifyL(TNotificationType aCallType, void* aParam)
    {
    MFlashEngine* engine = NULL;
    switch(aCallType)
        {
        case EApplicationFocusChanged:
            engine = iPlayerInstance->GetEngine();
	        if(engine)
			    {
			    engine->SetForeground(ETrue);
			    engine->NotifyFocusChangeL(TBool(!aParam));
			    }
            break;
        default:
            break;
        }

    return KErrNone;
    }

//-----------------------------------------------------------------------
// void CScreensaverPluginControl::NotifyError(TInt aErrorCode)    
//
// This function is called when there is an error while opening 
// a flash file in the screensaver mode
//-----------------------------------------------------------------------
 
void CScreensaverPluginControl::NotifyError(TInt aErrorCode) 
    {
    iLastError = aErrorCode;
    iPlayerInstance->PrepareToExit();
    }
    
//-----------------------------------------------------------------------
// short CScreensaverPluginControl::IsSoundPlaying()    
//
// This function returns whether sound is playing
//-----------------------------------------------------------------------
   
TBool CScreensaverPluginControl::IsSoundPlaying() const
{
	return EFalse;
}

//-----------------------------------------------------------------------
// short CScreensaverPluginControl::Start()    
//
// This function starts playing flash content
//-----------------------------------------------------------------------
  
/*void CScreensaverPluginControl::Start() 
{
	TRAPD(err,CreateEngineAndPlayL(iFilename, ETrue, NULL, ETrue));
	
	if(err)
	{
		NotifyError(err);
	}
	
}*/

//-----------------------------------------------------------------------
// short CScreensaverPluginControl::End()    
//
// This function ends playing flash content and destroys engine
//-----------------------------------------------------------------------
     	
//No need as destroyengine is called directly from SSPluginInstance
/*void CScreensaverPluginControl::End()
{
	DestroyEngine();
}*/

/**
	 This function is called by the host engine when there is an error in
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/

void CScreensaverPluginControl::NotifyFlashError(TInt aErrorCode, 
                                                 TBool /*aIsCritical*/, 
                                                 TBool /*aPersistent*/ )
{
	if(aErrorCode !=FI_BadSoundDataError)	
	{
		NotifyError(aErrorCode);
	}
}

/**
	 This function is called by the host engine when there is an error in
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
*/		

void CScreensaverPluginControl::NotifyClientError(TInt aErrorCode, 
                                                 TBool /*aIsCritical*/, 
                                                 TBool /*aPersistent*/ )
{
   	NPPDebug(LOG_ERROR,_L("CScreensaverPluginControl::NotifyClientError() : ERROR = [%d]"), aErrorCode);
	
	iLastError = aErrorCode;
	
    iPlayerInstance->PrepareToExit();
}

/**
	 This function is called by the host engine flash status
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/

/*TUint CScreensaverPluginControl::GetFeatureCapability()
{
	return KCapabilitiesScreensaver;
}*/
// End of File
