/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
                                 
*
*/

#include "saplayer_build.h"

#include <aknappui.h>
#include <coecntrl.h>
#include <caf/content.h>
#include <eikenv.h>
#include <flashliteplugin.rsg>

//#include "flashbitbuffer.h"
#include "flashwlessplugincontrol.h"
//#include "FlashPluginInstance.h"
//#include "flashconfiggen.h"
//#include "flashplgmain.h"
//#include "modalhandler.h"
#include "flashiconhandler.h"
#include "flashplugincursor.h"
#include "flashpluginif.h"


using namespace ContentAccess;
using namespace RT_GestureHelper;

extern HBufC8* GetBaseURL8(const TDesC& aBaseUrl);

#ifdef FEATURE_WINDOWLESS

CFlashWlessPluginControl* CFlashWlessPluginControl::NewL(MFlashPluginInstance* aInstance, CCoeControl* aParentControl)
{
    CFlashWlessPluginControl* self = new (ELeave) CFlashWlessPluginControl(
            aInstance, aParentControl);
    self->ConstructL();
    return self;
}

CFlashWlessPluginControl::CFlashWlessPluginControl(MFlashPluginInstance* aInstance, CCoeControl* aParentControl)
    :PluginBase(aInstance)
    ,iParentControl(aParentControl)
{
    iFocusState = EFlashPluginForeground;
    iVisibilityState = EFlashPluginVisible;
    iNpWindow.window = NULL;
}

CFlashWlessPluginControl::~CFlashWlessPluginControl()
{
	CCoeEnv::Static()->RemoveForegroundObserver(*this);
	
     if(iCursor)
     {
         delete iCursor;
     }

     if(iIconHandler)
     {
    	 delete iIconHandler;
    	 iIconHandler = NULL;
     }
}

void CFlashWlessPluginControl::ConstructL()
{
    PluginBase::ConstructL();

    iConstructed = ETrue;    
    iValid = ETrue;
    
    CCoeEnv::Static()->AddForegroundObserverL(*this);
    
    
    //iModalHandler = CModalHandler::NewL(iPluginInstance,iEngine);

    //CreateQwertyKeypadObserverL();
    
    //iStreamFactory = CPluginStreamFactory::NewL(this);

    //LoadIconsL();

#ifdef ENABLE_CURSOR
    //iCursor = 0;
#endif

#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR  

    //iSuppressor = CAknPointerEventSuppressor::NewL(); 
    //iSuppressor->SetMaxTapDuration(KMaxTInt64);
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR

#ifdef __FEATURE_PLUGIN_UI_OPTIONS
    //iMenuMgr    = CPluginMenuManager::NewL(this);

#ifdef __PEN_SUPPORT    
    //iGestureDetector = CGestureDetector::NewL(this);
#endif   
#endif
}

void CopyNpWindow(NPWindow* aDst, NPWindow* aSrc)
{
    aDst->window = aSrc->window;
    aDst->x = aSrc->x;
    aDst->y = aSrc->y;
    aDst->width = aSrc->width;
    aDst->height = aSrc->height;
    aDst->clipRect = aSrc->clipRect;
    aDst->type = aSrc->type;
}

bool WindowSame(NPWindow* aDst, NPWindow* aSrc)
{
    if( (aDst->window == aSrc->window)
       && (aDst->x == aSrc->x)
       && (aDst->y == aSrc->y)
       && (aDst->width == aSrc->width)
       && (aDst->height == aSrc->height)
      // && (aDst->clipRect == aSrc->clipRect)
       && (aDst->type == aSrc->type) )

        return true;
    else
        return false;
}

int CFlashWlessPluginControl::SetWindow(void* aWindow)
{
    TRAPD(error, DoSetWindowL(aWindow));
    return error;
}
void CFlashWlessPluginControl::DoSetWindowL(void* aWindow)
{    
    NPWindow* window = STATIC_CAST(NPWindow*, aWindow);
    TInt width = STATIC_CAST(TInt, window->width);
    TInt height = STATIC_CAST(TInt, window->height);
    
    // On Error just return
    if(iErrorOccured)
    {
    	CopyNpWindow(&iNpWindow, window);
    	DrawDirtyRect(TRect(TSize(TInt(iNpWindow.width),TInt(iNpWindow.height))));
    	User::Leave(KErrGeneral);
    }

    TRect swfRect;
    swfRect.iTl.iY = window->y;
    swfRect.iTl.iX = window->x;
    swfRect.iBr.iY = swfRect.iTl.iY + height;
    swfRect.iBr.iX = swfRect.iTl.iX + width;

    MFlashEngine* engine = iPlayerInstance->GetEngine();
    
    RDebug::Print(_L("CFlashWlessPluginControl::SetWindow %d %d %d %d"),swfRect.iTl.iX, swfRect.iTl.iY, swfRect.iBr.iX, swfRect.iBr.iY);
    
    if (NULL == iNpWindow.window)
    {
        iParentBitmap = static_cast<CFbsBitmap*> (window->window);
        
       	iIconHandler = CFlashIconHandler::CreateIconHandlerL();
#ifdef _BITBUFFER_ENABLED_ 
        iBitmap = static_cast<CFbsBitmap*> (window->window);
        iBitBuffer->SetSize(TSize(1, 1),ETrue); //hack
#endif

        if (iParentBitmap->SizeInPixels().iHeight == 0
              || iParentBitmap->SizeInPixels().iWidth)
        iParentBitmap->Resize(TSize(1, 1));
        
        //}

        CopyNpWindow(&iNpWindow, window);

        if (!engine->IsInitialized())
        {
            TRect r;
            r.SetWidth(static_cast<TInt> (iNpWindow.width));
            r.SetHeight(static_cast<TInt> (iNpWindow.height));
            
#ifdef _BITBUFFER_ENABLED_             
            iBitBuffer->SetSize(TSize(r.Width(), r.Height()),ETrue); //hack
#endif
            
            iParentBitmap->Resize(TSize(r.Width(), r.Height()));
            
            engine->SetWlessSurface(iParentBitmap);
            
            engine->InitializeL(r,EFalse);
            /*
            iEngine->InitializeWlessPluginL(this, iParentBitmap, r,
                    NULL, this);
            */
        }
        else
        {
            engine->SetWlessSurface(iParentBitmap);
#ifdef _BITBUFFER_ENABLED_             
            iBitBuffer->UpdateBitmap(iBitmap);
#endif            
        }
        //plugin initialisation takes place later..
        return;
    }
    else
    {
        iParentBitmap = static_cast<CFbsBitmap*> (window->window);

#ifdef _BITBUFFER_ENABLED_ 
        iBitmap = static_cast<CFbsBitmap*> (window->window);
#endif
        
        if (iParentBitmap->SizeInPixels().iHeight == 0
                && iParentBitmap->SizeInPixels().iWidth)
            iParentBitmap->Resize(TSize(1, 1));
        
        
#ifdef _BITBUFFER_ENABLED_     
        if (iBitmap->SizeInPixels().iHeight == 0
                        && iBitmap->SizeInPixels().iWidth)
        {
            iBitmap->Resize(TSize(1, 1));
        }
#endif        

        //if (!engine->IsInitialized())
        {
            CopyNpWindow(&iNpWindow, window);

            TRect r;
            r.SetWidth(static_cast<TInt> (iNpWindow.width));
            r.SetHeight(static_cast<TInt> (iNpWindow.height));
            engine->SetWlessSurface(iParentBitmap);

#ifdef _BITBUFFER_ENABLED_ 
            iBitBuffer->UpdateBitmap(iBitmap);
#endif
            engine->InitializeL(r,EFalse);
            
            //iEngine->InitializeWlessPluginL(this, iParentBitmap, r,
            //        NULL, this);
        }
        /*else
        {
            engine->SetWlessSurface(iParentBitmap);
#ifdef _BITBUFFER_ENABLED_             
            iBitBuffer->UpdateBitmap(iBitmap);
#endif            
            TRect r;
            r.SetWidth(static_cast<TInt> (iNpWindow.width));
            r.SetHeight(static_cast<TInt> (iNpWindow.height));
            engine->InitializeL(e, EFalse);
        }*/
        //}
    }
}

TBool CFlashWlessPluginControl::IsValid() const
{
    return iValid;
}

TBool CFlashWlessPluginControl::IsVisible() const
{
    return ETrue; //TODO: Revisit this part.
}

//Moved to CFlashPluginInstance :: ProcessParameters
/*void CFlashWlessPluginControl::ProcessParameters(CDesCArray* argnames,
        CDesCArray* argvalues)
    {
    PluginBase::ProcessParameters(argnames,argvalues);
    }*/

/*NPFlashPlayerState CFlashWlessPluginControl::State() const
    {
    return NPEngineNotInitialized;
    }*/

/*void CFlashWlessPluginControl::SetBaseUrlL(const TDesC* aBaseUrl)
    {
    
     if( !aBaseUrl || !iEngine ) 
     return;
     
     if( aBaseUrl->Length() <= 0 )
     return;
     
     iBaseURL = HBufC8::NewL(aBaseUrl->Length());
     
     if( iBaseURL )
     {        
     TPtr8 ptr8  = iBaseURL->Des();          
     ptr8.Copy(((HBufC*)aBaseUrl)->Des());
     }
     
    }*/

TInt CFlashWlessPluginControl::GetCurrentFrameNo() const
{
    /*
     if (iEngine)
     {
     return iEngine->GetCurrentFrameNumber();
     }
     */
    return -1;
}

TInt CFlashWlessPluginControl::GetTotalFrames() const
{
    /*
     if (iEngine)
     {
     return iEngine->GetTotalFrameNumber();
     }
     */
    return 0;
}

void CFlashWlessPluginControl::SetVolume(TInt /*aVolume*/)
{

}

void CFlashWlessPluginControl::OpenFileL(const TDesC& aFilename,
        const TDesC* aUrl, const TBool& aFirstMovie, CContent* aContent)
{
    
	MFlashEngine* engine = FlashPluginInstance()->GetEngine();
	if (engine)
	{
		TInt result(KErrNone);
		TRect r;
		r.SetWidth(this->iNpWindow.width);
		r.SetHeight(this->iNpWindow.height);
		if (!engine->IsInitialized())
		{
			engine->SetWlessSurface(iParentBitmap);
#ifdef _BITBUFFER_ENABLED_                 
			if(iBitmap && iBitBuffer)
			{
				//iEngine->SetWlessSurface(iParentBitmap);
				iBitBuffer->UpdateBitmap(iBitmap);
				TRAP(result, engine->InitializeL(r, EFalse));
			}
#endif                
			TRAP(result, engine->InitializeL(r, EFalse));
				//TRAP(result, iEngine->InitializePluginL(this, r, EFalse));
		}
		else
		{
			engine->SetWlessSurface(iParentBitmap);
#ifdef _BITBUFFER_ENABLED_                
			if(iBitmap && iBitBuffer)
			{
				//iEngine->SetWlessSurface(iParentBitmap);
				iBitBuffer->UpdateBitmap(iBitmap);
				TRAP(result, engine->InitializeL(r, ETrue));
			}
#endif
			TRAP(result, engine->InitializeL(r, EFalse));
				//TRAP(result, iEngine->InitializePluginL(this, r, ETrue));
		}
		if (result == KErrNoMemory)
		{
			// Display the broken flash icon whenever 
			// InitializePluginL function leaves especially
			// because of out of memory condition. Fix for bug ESJY-7QEHPF
			NotifyClientError(result, ETrue, ETrue);
			return;
		}

		engine->OpenBrowserFileAndStartMovieL(aFilename, ETrue, aUrl,
											  aFirstMovie,
											  const_cast<CContent*>(aContent));

		HBufC8* baseURL8 = GetBaseURL8(iPlayerInstance->GetBaseURL());
		engine->SetBaseUrlL(baseURL8);
		
		if(baseURL8)
			delete baseURL8;

		iFileOpened = ETrue;

	}

	iFilename = aFilename;
}

//Moved to CFlashPluginInstance
/*
MPluginStreamHandler* CFlashWlessPluginControl::GetStreamHandler()
    {
    return PluginBase::GetStreamHandler();
    }
*/

//TODO : Implementation to be completed for windowless
void CFlashWlessPluginControl::PrepareToExitL()
{
}

TBool CFlashWlessPluginControl::IsFileOpened() const
{
    return iFileOpened;
}

TBool CFlashWlessPluginControl::QueryAllowDeviceIDL()
{
    return ETrue;
}

TBool CFlashWlessPluginControl::QueryAllowNetAccessL()
{
	TBool allowNetworkAccess = FlashPluginInstance()->QueryAllowNetworkAccess();
	return allowNetworkAccess;
}

_LIT(KResFile,"flashliteplugin.rsc");
TBool CFlashWlessPluginControl::QueryAllowPhoneCallL(
        const TDesC& aNumberString)
{
    //TBufC<128> KResFile(_L("flashliteplugin.rsc") );
    //TInt resOffset = load_pluginresourceL( KResFile );
    TInt resOffset = PluginBase::LoadPluginResourceL(KResFile());

    TBool query = PluginBase::QueryAllowDenyWithFormatResourceL(
            aNumberString, R_STR_ALLOW_PHONE_CALL);

    CEikonEnv::Static()->DeleteResourceFile(resOffset);
    return (query);
}

void CFlashWlessPluginControl::NotifyError(TInt aErrorCode,
        TBool /* aIsCritical */)
{
    PluginBase::SetErrorOccured(aErrorCode);
    return;
}

void CFlashWlessPluginControl::NotifyFlashError(TInt aErrorCode,
        TBool aIsCritical, TBool /*aPersistent*/)
{
    iLastError = aErrorCode;

    if (!aIsCritical)
    {
        return;
    }
    iPlayerInstance->PrepareToExit();
}

void CFlashWlessPluginControl::NotifyClientError(TInt aErrorCode,
        TBool aIsCritical, TBool /*aPersistent*/)
{
    //PluginBase::SetErrorOccured(aErrorCode);
        
        iLastError = aErrorCode;

    if (!aIsCritical)
    {
        return;
    }

    iPlayerInstance->PrepareToExit();
}

void CFlashWlessPluginControl::NotifyFlashStatus(TInt /*aFlashStatus*/,
        TAny* /*aParam*/)
{
    return;
}

void CFlashWlessPluginControl::DrawDirtyRect(const TRect& aDirtyRect)
{
           
    if (iErrorOccured)
    {
    	CFbsBitmapDevice* bitmapDevice = NULL;
    	TRAPD(error, bitmapDevice = CFbsBitmapDevice::NewL(iParentBitmap));
    	if( error != KErrNone )
    	    {
    	    return;// error;
    	    }
        
    	CFbsBitGc* bitmapGc;
    	bitmapDevice->CreateContext(bitmapGc);

    	
    	iIconHandler->Show(bitmapGc, FlashPluginInstance()->GetWMode(),eErrorIcon, 
    			TRect(TSize(TInt(iNpWindow.width),TInt(iNpWindow.height))));

    	//error icon is blitted onto the bitmap provided by browser..now request
        //browser to blend..
        this->iPlayerInstance->NetscapeFns()->InvalidateRect(aDirtyRect);
        return;
    }

    if (!aDirtyRect.IsEmpty())
    {
        this->iPlayerInstance->NetscapeFns()->InvalidateRect(aDirtyRect);
    }
}

void CFlashWlessPluginControl::FLVideoStarted()
{
    iCanGoFullScrn = EFalse; //TODO : Revisit this part if video has to go full screen in windowless mode.
}

TBool CFlashWlessPluginControl::IsPlayingVideo()
{
    return iCanGoFullScrn;
}

TInt CFlashWlessPluginControl::HandleGestureFromBrowserL(void* aEvent)
{
    TBool ret = EFalse;
    
    MFlashEngine* engine =  this->FlashPluginInstance()->GetEngine();
    
    //If engine is destoryed no need to process gesture we return false and browser will process them
    if(engine && aEvent)
    {    
        TPoint pRTS = iParentControl->PositionRelativeToScreen(); //WLESS : Cant get in windowless, no ccoecontrol exists..
        /*        
         #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
         if(iFullScreenControl)
         {   //iFullScreenControl seems to be the only reliable flag that indicates if we are in 
         //fullscreen or not.
         
         //Dont handle gestures in fullscreen mode.
         ret = iFullScreenControl->HandleGestureFromBrowserL(aEvent);
         return ret;
         }
         #endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN
         */    
        
        TGestureEvent *gesture = static_cast<TGestureEvent*>(aEvent);
        
        //Browser will send position relative to screen we need convert them to flash co-ordinates.
        TPoint  point = gesture->CurrentPos() - pRTS;
        TPoint startpoint = gesture->StartPos()- pRTS;
        
        TPointerEvent pointerEvent;
        
        pointerEvent.iPosition = point ;
        TGestureCode gtype =  gesture->Code(EAxisBoth);  
        TBool iControlPresent(EFalse);
        
        // application need to say which axis to consider?
        #ifdef _GestureDebug_
        RDebug::Printf("CFlashWlessPluginControl::Consumegesture: code %d, pos(%d, %d)", gtype, point.iX, point.iY);
        #endif //_GestureDebug_
        switch (gtype)
        {
            case EGestureStart:
            {
                iControlPresent = engine->IsControlPresent(point);
                
                 if( (!IsPlayingVideo() || //if there is no video in swf, send scroll notif to browser 
                     !iControlPresent)&& 
                     ( (iPlayerInstance->GetBrPluginExecutionMode() == EWidgetMode) || (iPlayerInstance->GetBrPluginExecutionMode() == EDefaultMode)))
                     
                     
                 {
                    iFlashScroll = EFalse;
                    iBrowserScroll = ETrue;
                }
                else 
                {
                   pointerEvent.iType=TPointerEvent::EButton1Down;
                   
                   if(iTactileFeedbackSupported)
                	   ProvideFeedbackL(pointerEvent);
                   
                   engine->OfferPointerEventL(point, TPointerEvent::EButton1Down);
                   
                   iBrowserScroll = EFalse;
                   iFlashScroll = ETrue;
                }
                ret = EFalse; 
                break;
            }
            case EGestureDrag:
            {
                if(iBrowserScroll)
                {
                    #ifdef _GestureDebug_
                    RDebug::Printf("Browser panning start point = (%d,%d) current point = (%d,%d)",startpoint.iX,startpoint.iY,point.iX,point.iY);
                    #endif // _GestureDebug_
                    ret = EFalse;
                }
                else if(iFlashScroll)//Send the events to coreplayer only if Browser is not scrolled
                {
                    #ifdef _GestureDebug_
                    RDebug::Printf("Drag: In flash");
                    #endif // _GestureDebug_
                    pointerEvent.iType=TPointerEvent::EDrag;
                    if(iTactileFeedbackSupported)
                    	ProvideFeedbackL(pointerEvent); 
                    engine->OfferPointerEventL(point, TPointerEvent::EDrag);
                    ret = ETrue;
                }
                break;
            }
            //This are events generated after up event is received.
            case EGestureSwipeLeft:
            case EGestureSwipeRight:
            case EGestureSwipeUp:  
            case EGestureSwipeDown:
            case EGestureFlick:  
            case EGestureDrop:
            case EGestureReleased:
            {
                if (iFlashScroll)
                {
                   pointerEvent.iType=TPointerEvent::EButton1Up;
                   if(iTactileFeedbackSupported)
                	   ProvideFeedbackL(pointerEvent); 
                   engine->OfferPointerEventL(point, TPointerEvent::EButton1Up);
                }
                //Up has been received we reset the flags
                iFlashScroll = EFalse;
                iBrowserScroll = EFalse;
                ret = EFalse;
                break;
            }
                // gestures
            case EGestureUnknown:
            {
                ret = EFalse;
                break;
            }
                
            case EGestureDoubleTap:
            {
                ret = EFalse;
                break;
            }
            case EGestureTap:
            {
                //if flashscroll is not set send Down followed by Up.otherwise dont send anything
                //Down is already sent up will be sent when released
                if(!iFlashScroll)
                {    
                    pointerEvent.iType=TPointerEvent::EButton1Down;
                    if(iTactileFeedbackSupported)
                    	ProvideFeedbackL(pointerEvent);
                    engine->OfferPointerEventL(point, TPointerEvent::EButton1Down);
                    pointerEvent.iType=TPointerEvent::EButton1Up;
                    engine->OfferPointerEventL(point, TPointerEvent::EButton1Up);

                }    
                
                
                if(iIMEControl)
                {
                    iIMEControl->LaunchVKbL();
                }
                
                
                #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
                iControlPresent = engine->IsControlPresent(point);
                if(!iControlPresent)
                {
                    //TODO : Revisit this code when you want to support video in windowless
                   /*
                   TRect  videoRect=iEngine->GetVideoRectL();      

                   // If Ph - 1, just provide the feedback and toggle
                    if(!iEngine->IsHelixFlvMode())  
                       ToggleScreenL();
                   // If ph - 2, if no videorect toggle, otherwise check the point and then toggle. 
                    else if( ( videoRect.Contains(point)))   //removed condition as even some swfs with out video were toggling  
                       ToggleScreenL();
                   */
                }  
                #endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN
                ret = ETrue;
                break;
            }
            case EGestureLongTap:
            {
                
                //if flash scroll is not set send down
                if(!iFlashScroll)
                {
                    pointerEvent.iType=TPointerEvent::EButton1Down;
                    if(iTactileFeedbackSupported)
                    	ProvideFeedbackL(pointerEvent);
                    engine->OfferPointerEventL(point, TPointerEvent::EButton1Down);
                    iFlashScroll = ETrue;
                    iBrowserScroll=EFalse;
                }
                ret = ETrue;
                break;
            }
            default:
            {
                ret = EFalse;
                break;
            }
        }
    }
    return ret;
}

TBool IsNavigationKey(const TKeyEvent& aKeyEvent)
{
    switch(aKeyEvent.iScanCode)
    {
        case EStdKeyLeftArrow:
        case EStdKeyRightArrow:
        case EStdKeyUpArrow:
        case EStdKeyDownArrow:
            return ETrue;
        default:
            return EFalse;            
    }
}

TBool IsSelectKey(const TKeyEvent& aKeyEvent)
{
    return aKeyEvent.iScanCode == EStdKeyDevice3 ? ETrue : EFalse;
}

int CFlashWlessPluginControl::HandleKeyEvent(NPEvent* event)
{
	TInt error = EKeyWasNotConsumed;
	TRAP_IGNORE(error = DoHandleKeyEventL(event));
	return error;
}

int CFlashWlessPluginControl::DoHandleKeyEventL(NPEvent* event)
{
    MFlashEngine* engine = this->FlashPluginInstance()->GetEngine();
    
    if (!engine || !iFileOpened )
    {
        return EKeyWasNotConsumed;
    }
		
    NPEventKey* eventKey = static_cast<NPEventKey*> (event->param);
	const TKeyEvent* keyEvent = eventKey->keyEvent;
	unsigned int eventType = eventKey->type;
	TPoint* point = static_cast<TPoint*> (eventKey->reserved);
	TPoint cursorPosition(point->iX, point->iY);
	
//#ifdef ENABLE_CURSOR
    /*
    if(iIMEControl && iJustActivatedIME)
            {
            //when flash is activated, browser cursor may be top of
            //flash edit box. this event will make core to call
            //enableime. Now IME is enabled still our browser
            //control is yet to receive 2 more key events
            //if we send these key events core player will
            //disable ime session. this we want to avoid 
            //so consume the event till iJustActivatedIME non zero
            iJustActivatedIME--;
            return EKeyWasConsumed;
            }
     */
    
   // if( /*iPluginActivated && */ iCursor /* && ( !iIMEControl) */)
    //{
	   //iJustActivatedIME = 0;
	   if( IsNavigationKey(*keyEvent) )
	   {            
		   // Display cursor when user presses the navigation key
//		   if(engine && (!iCursor->IsVisible()))
//		   {
//			   //iCursor->ShowCursor(iSavedCursorPoint);
//		   }
//		   if( eventType == EEventKey )
//		   {
//			   TBool aBoundaryHit; 
//			   TPoint offset = iCursor->HandleNavigation(ke, (TEventCode)type,aBoundaryHit);
//			   
//			   if(aBoundaryHit)
//			   {
//				   TPoint clickPoint = iCursor->Position();
//				   TPoint movePoint = offset + clickPoint;
//				   //iSavedCursorPoint = clickPoint;
//				   // Calling HideCursor to hide the cursor
//				   iCursor->HideCursor();
//				   NPNetscapeFuncs* nsfuncs = iPlayerInstance->GetGlobals()->NetscapeFuncs();
//				   if( nsfuncs /* && !iFullScreen */ )
//				   {
//					   // Calling HideCursor to hide the cursor
//					   // Cursor should not be hidden in case of full screen. 
//					   iCursor->HideCursor();
//					   //TBool dummy = ETrue;
//					   (nsfuncs->setvalue)(iPlayerInstance->GetInstance(), 
//											   (NPPVariable) NPPVPluginDeactivate,
//											   //(void*) &dummy );
//											   (void*) &movePoint );                                                        
//					   /* This call is to ensure that the cursor is moved out of the flash area and
//											 if any existing hover events are there will be cancelled */ 
//					   engine->OfferPointerEventL(movePoint,TPointerEvent::EMove);
//				   }
				   engine->OfferPointerEventL(cursorPosition,TPointerEvent::EMove);
//			   }
//			   else
//			   {
//				   NPNetscapeFuncs* nsfuncs = iPlayerInstance->GetGlobals()->NetscapeFuncs();
//				   TPoint clickPoint = iCursor->Position();
//				   //iSavedCursorPoint = clickPoint;
//				   if( nsfuncs /*&& !iFullScreen */ )
//				   {
//						(nsfuncs->setvalue)(iPlayerInstance->GetInstance(), 
//												   (NPPVariable) NPPVPluginFocusPosition,
//												   (void*) &offset );
//				   }
//				   engine->OfferPointerEventL( clickPoint , TPointerEvent::EMove );       
//			   }
//		   }
	   }
	   else if( IsSelectKey(*keyEvent) )
	   {
		   // Display flash cursor when plugin get activated on pressing OK button
//		   if(engine && GetQwertyKeyPadStatus() && (!iCursor->IsVisible()))
//		   {
//			   //iCursor->ShowCursor(iSavedCursorPoint);
//		   }
//		   TPoint clickPoint = iCursor->Position();
//		   clickPoint.iX += (iCursor->Size().iWidth/4);
//		   clickPoint.iY += (iCursor->Size().iHeight/4);
		   
	   /*
			   #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
			   TRect  videoRect=iEngine->GetVideoRectL();
			   if(iPlayerInstance->GetAllowMobiFullScreen() && 
					   !IsControlPresent(clickPoint) && aType==EEventKey &&  videoRect.Contains(clickPoint))
			   {
				   ToggleScreenL();//do full or normal 
			   }
			   
			   #endif
	   */
		   //This move event is redundant. Hence removed.
		   //iEngine->OfferPointerEventL( clickPoint, TPointerEvent::EMove );
		   
		   if( eventType == EEventKeyDown )
		   {
			   engine->OfferPointerEventL( cursorPosition, TPointerEvent::EButton1Down );     
			   /*if(iIMEControl)  // check ime session created
			   {
				   //when flash is activated, browser cursor may be top of
				   //flash edit box. this event will make core to call
				   //enableime. Now IME is enabled still our browser
				   //control is yet to receive 2 more key events
				   //if we send these key events core player will
				   //disable ime session. this we want to avoid 
				   iJustActivatedIME = 2;
			   }*/
		   }
		   else if(eventType == EEventKeyUp )
		   {
			   engine->OfferPointerEventL( cursorPosition, TPointerEvent::EButton1Up);    
		   }
	   
		   //if(!iIMEControl)            
			  // iCursor->UpdateCursor();
	   }
	 if(!IsNavigationKey(*keyEvent) && !IsSelectKey(*keyEvent))  
		 return SendKeyEventToEngineL(*keyEvent, (TEventCode)eventType);
	 else
		 return EKeyWasConsumed;
   // }        
//    #endif



    
       //return CFlashPluginControl::OfferKeyEventL(aKeyEvent, aType);

       /*
    NPEventKey* eventKey = static_cast<NPEventKey*> (event->param);
    const TKeyEvent* keyEvent = eventKey->keyEvent;
    unsigned int type = eventKey->type;
    const TKeyEvent ke(*keyEvent);
    int result = EKeyWasNotConsumed;
    //TRAP_IGNORE(result = OfferKeyEventL(ke, (TEventCode)type));

    return iModalHandler->OfferKeyEventL(ke, (TEventCode) type);
    */
     /*
     if (iIMEControl && keyResponse)
        {
        iIMEControl->ReportStateEventL();
        }
        
        */
    
    //return keyResponse;
}


int CFlashWlessPluginControl::SendKeyEventToEngineL(const TKeyEvent& aKeyEvent, 
                                                        TEventCode aType)
{
    TKeyEvent keyEvent = aKeyEvent;   
    TEventCode eventType = aType;

    MFlashEngine* engine = FlashPluginInstance()->GetEngine();
    
    // This change is from S60 Rel 3.2 WK 44, 2006
    // In case of EStdKeyDevice3 (Select key) the order is found to be 
    // EEventKeyDown => EKeyEventUp => EKeyEventUp
    // So there is no EEventKey event generated.
    // So we send this in between down and up events for the EStdKeyDevice3 key

    if( aKeyEvent.iScanCode == EStdKeyDevice3 )
    {
        if( iLastKeyEventCode == EEventKeyDown && 
                aType == EEventKeyUp )
        {
        aType = EEventKey;                      
        }
    }

    TKeyResponse keyResponse = EKeyWasNotConsumed;

    keyResponse = engine->OfferKeyEventL(keyEvent, (TEventCode)aType);
    
    if( aKeyEvent.iScanCode == EStdKeyDevice3 )
    {
        if( iLastKeyEventCode == EEventKeyDown && 
                aType == EEventKeyUp )
        {
            iLastKeyEventCode = EEventNull;
        }
        else
        {
            iLastKeyEventCode = (TEventCode)aType;
        }
    }
#ifdef FEATURE_INLINE_TEXT_ENTRY
    if( iIMEControl && keyResponse )   
    {
        iIMEControl->ReportStateEventL();
    }
#endif
    return keyResponse;
}
	
void CFlashWlessPluginControl::Redraw()
{
    //a dummy rect is passed here..
    DrawDirtyRect(TRect(TSize(TInt(iNpWindow.width),TInt(iNpWindow.height))));
}


#ifdef _BITBUFFER_ENABLED_
BitmapBuffer* CFlashWlessPluginControl::BitBuffer()
    {
    if(!iBitBuffer)
        {
        TRAPD(error, iBitBuffer = CFlashBitmap::NewL(iBitmap));
        if (error != KErrNone)
        	return NULL;
        }
    return iBitBuffer;
    }


CFbsBitGc* CFlashWlessPluginControl::GetBitmapContext()
    {
    if(!iBitmapGc && iBitmapDevice)
        {        
        iBitmapDevice->CreateContext(iBitmapGc);
        }
    return iBitmapGc;
    }

TInt CFlashWlessPluginControl::Resize(TSize aSize)
    {
    /*if(iBitBuffer)
        {
        iBitBuffer->SetSize(aSize,ETrue);
        }*/
    }
    
CFbsBitmapDevice* CFlashWlessPluginControl::GetBitmapDevice()
    {
    if(iBitmap && !iBitmapDevice)
    {
        TRAPD(error, iBitmapDevice = CFbsBitmapDevice::NewL(iBitmap));
        if (error != KErrNone)
        	return NULL;
    }
    
    return iBitmapDevice;
    }
#endif

TRect CFlashWlessPluginControl::ApplicationRect()
{
    return TRect(TSize(TInt(iNpWindow.width),TInt(iNpWindow.height)));
}
    
TRect CFlashWlessPluginControl::ClientRect()
{
    return TRect(TSize(TInt(iNpWindow.width),TInt(iNpWindow.height)));
}

//Overridden SetClipRect function, no reason why someone
//needs to call this function...
//If there is a valid reason, remove this comment
void CFlashWlessPluginControl::SetClipRect(const TRect& aClipRect)
{
    iNpWindow.clipRect.top = aClipRect.iTl.iX;
    iNpWindow.clipRect.left = aClipRect.iTl.iY;
    iNpWindow.clipRect.right = aClipRect.iBr.iX;
    iNpWindow.clipRect.bottom = aClipRect.iBr.iY;
}

TRect CFlashWlessPluginControl::GetClipRect()
{
    return TRect(iNpWindow.clipRect.top,
                iNpWindow.clipRect.left,
                iNpWindow.clipRect.right,
                iNpWindow.clipRect.bottom);
}

RWindow* CFlashWlessPluginControl::GetWindow()
{
    return static_cast<RWindow*>(this->iParentControl->DrawableWindow());
}

int CFlashWlessPluginControl::HandlePluginEvent(void* aEvent)
{
    RDebug::Printf("CFlashWlessPluginControl::HandleEvent");
    NPEvent* pEvent = (NPEvent*)aEvent;
	TInt retVal = 0;
	NPEventPointer *param = NULL;
	NpEventVisibility* visibleEvent = NULL;
	
    switch (pEvent->event)
    {
        case ENppEventKey:
            HandleKeyEvent(pEvent);
            break;
        
        case ENppEventPointer:
			param = static_cast<NPEventPointer*>(pEvent->param);
			TRAP_IGNORE(retVal = HandleGestureFromBrowserL(param->reserved));        	
            break;
    
        case ENppEventDraw:
            break;

        case ENppEventVisibility:
            visibleEvent = static_cast<NpEventVisibility*> (pEvent->param);
            if (visibleEvent)
            {
            	TRAP_IGNORE( if(visibleEvent->visible)
								retVal = NotifyL(MPluginNotifier::EPluginInvisible, (void *)1);
							 else
								retVal = NotifyL(MPluginNotifier::EPluginInvisible, (void *)0);
						   );            	
            }
            break;
    }
    
    return retVal;
}

void CFlashWlessPluginControl::HandleGainingForeground()
{
	void* param = (void*) 1;
	TRAP_IGNORE(NotifyL(MPluginNotifier::EApplicationFocusChanged, param));
	
	return;
}

void CFlashWlessPluginControl::HandleLosingForeground()
{
	void* param = (void*) 0;
	TRAP_IGNORE(NotifyL(MPluginNotifier::EApplicationFocusChanged, param));
	
	return;
}
TInt CFlashWlessPluginControl::NotifyL(MPluginNotifier::TNotificationType aCallType, void* aParam)
{     
    MFlashEngine* engine = this->FlashPluginInstance()->GetEngine();

    switch(aCallType)
    {
		case MPluginNotifier::EApplicationFocusChanged:
		{	
			if( !aParam )
			{
				iFocusState = EFlashPluginBackground;		      		    
			}
					
			else if(aParam)
			{
				iFocusState = EFlashPluginForeground;	
				// if swf file embedded with in the webpage is not yet 
				// opened then open it. Such a situation can arise
				// if browser moves to background before we get a newp call
				// from browser. fix for ASHA-7RKHZT.
				if(!iPlayerInstance->IsPlayerInitialzed() && iPlayerInstance->DeferredLaunch() && iVisibilityState == EFlashPluginVisible)
				{      			  
					User::LeaveIfError(iPlayerInstance->OpenMovie());
				}
			}
			HandleFocusVisibilityChangeL((TBool)aParam);		
		}
		break;
    
        case MPluginNotifier::EPluginInvisible:
        {
			if( !aParam )
			{
				iVisibilityState = EFlashPluginInvisible;
			}
			else
			{
				iVisibilityState = EFlashPluginVisible;
				// if swf file embedded with in the webpage is not yet 
				// opened then open it. fix for ASHA-7RKHZT.
				if(iFocusState == EFlashPluginForeground && !iPlayerInstance->IsPlayerInitialzed() && iPlayerInstance->DeferredLaunch())
				    User::LeaveIfError(iPlayerInstance->OpenMovie());
			}
			HandleFocusVisibilityChangeL((TBool)aParam);
         }
        break;

		default:
			break;
    }
    
    return KErrNone;  
}

void CFlashWlessPluginControl::HandleFocusVisibilityChangeL(TBool aFocus)
{
    if(iMP3StreamState != EFlashNoMP3Stream)
    {
        PluginBase::NotifyMp3FocusChangeL(aFocus);
    }
    else
    {
		if( aFocus && (iPlayState == EFlashPluginPlaying || iVisibilityState == EFlashPluginInvisible) )
		{
			return;
		}
		
		if( !aFocus && iPlayState == EFlashPluginPaused )
		{
			return;
		}
	
		if( aFocus && ((iFocusState == EFlashPluginForeground) && (iVisibilityState == EFlashPluginVisible)))
		{
			iPlayState = EFlashPluginPlaying;
			iVisibilityState = EFlashPluginVisible;
	
			MFlashEngine* engine = FlashPluginInstance()->GetEngine();
			
			if( engine )
			{
				engine->SetForeground(ETrue);
				engine->NotifyFocusChangeL(EFalse);	// Resuming
			}
				
			FlashPluginInstance()->AddHwSideKeyObserverL();
		}
		//TODO : Handle IME
		// Pause only when IME control is disabled. This check has been introduced to resolve the 
		// VKB popup issue (PRRR-7QG8V8).
		else if(iPlayState != EFlashPluginPaused)// && !iIsIMEEnabled)
		{
			iPlayState = EFlashPluginPaused;
			
			MFlashEngine* engine = FlashPluginInstance()->GetEngine();
			
			if( engine )
			{
				engine->SetForeground(EFalse);
				engine->NotifyFocusChangeL(ETrue);	// Pausing
			}
			FlashPluginInstance()->RemoveHwSideKeyObserver();
		}
    }
}


void CFlashWlessPluginControl::HandleWsEventL(const TWsEvent& aEvent,CCoeControl* aDestination)
    {
    if(aEvent.Type() == EEventPointer)
        {
        if(aEvent.Pointer()->iType == TPointerEvent::EButton1Down && aDestination != iParentControl && !(GetWindow()->IsFaded())) 
            {
            #ifdef FEATURE_INLINE_TEXT_ENTRY
            SimulateDisableIMEEventL();
            #endif
            }
        }
    }


void CFlashWlessPluginControl::AddEventObserverToControl()
{
    iEventMonitor = ((CAknAppUi*)iParentControl->ControlEnv()->AppUi())->EventMonitor();
    
    if(iEventMonitor)
    {
        TRAP_IGNORE(iEventMonitor->AddObserverL(this));
        if (iEventMonitor)
            iEventMonitor->Enable(ETrue);
    }
}


void CFlashWlessPluginControl::RemoveEventObserverFromControl()
{
    if(iEventMonitor)
        iEventMonitor->RemoveObserver(this);
}

TBool CFlashWlessPluginControl::IsFLVVideo()
{
    // No video in windowless mode as of now...
    // Modify this later if required
    return EFalse;
}
#endif //FEATURE_WINDOWLESS

//End of File
