/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description: 
*       Provides the CFullScreenMenuManager methods.
*
*
*/

#include <coemain.h>
#include <coeaui.h>
#include "saplayer_build.h"
#include "fullscreenmenumgr.h" 
#include "flashbrowserplugincontrol.h" 
#include <flashliteplugin.rsg>
#include "flashliteplugin.hrh"
#include "pluginmenumgr.h"
#include "flashvolumecontrol.h"
#include "volumecenrepkeywatcher.h"

extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );

// -----------------------------------------------------------------------------
// CFullScreenMenuManager::NewL
// factory function
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//
CFullScreenMenuManager* CFullScreenMenuManager::NewL(CBrowserPluginControl* aCtrl)
    {
    	NPPDebug(LOG_DEBUG,_L("CFullScreenMenuManager::NewL() ++"));
    
    	CFullScreenMenuManager* self = new(ELeave) CFullScreenMenuManager(aCtrl);

    	CleanupStack::PushL(self);
    	self->ConstructL();
    	CleanupStack::Pop(self);
    
    	NPPDebug(LOG_DEBUG,_L("CFullScreenMenuManager::NewL() --"));
    
    	return self;
    }

// -----------------------------------------------------------------------------
// CFullScreenMenuManager::~CFullScreenMenuManager
// Destructor
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//
CFullScreenMenuManager::~CFullScreenMenuManager()
    {
    if( iOptionMenuBar != NULL )
        {
        if(iOnStack)
        {
        	CCoeEnv* coeenv = CCoeEnv::Static();
        	coeenv->AppUi()->RemoveFromStack(iOptionMenuBar);	
        }
        
        delete iOptionMenuBar;
        }
    }

// -----------------------------------------------------------------------------
// CFullScreenMenuManager::ConstructL
// Show speicfied menu pane
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//
void CFullScreenMenuManager::ConstructL()
{
	
}
// -----------------------------------------------------------------------------
// CFullScreenMenuManager::aResourceId
// Show speicfied menu pane
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//
void CFullScreenMenuManager::ShowL(TInt aResourceId)
    {
    if( iOptionMenuBar == NULL )
        {
        CEikMenuBar* menuBar = new (ELeave) CEikMenuBar() ;
        CleanupStack::PushL(menuBar);
        menuBar->ConstructL( this, 0, aResourceId) ;
        iOptionMenuBar = menuBar;
        CleanupStack::Pop(menuBar); // now owned by this CFullScreenMenuManager.
        }
    else
        {
        iOptionMenuBar->SetMenuTitleResourceId(aResourceId);
        }

    if( !iOptionMenuBar->IsDisplayed() )
        {
        CCoeEnv::Static()->AppUi()->AddToStackL(iOptionMenuBar,
                                                ECoeStackPriorityDialog + 1,
                                                ECoeStackFlagStandard);
        iOnStack = ETrue;
        iOptionMenuBar->TryDisplayMenuBarWithoutFepMenusL();
        }
    }

// -----------------------------------------------------------------------------
// CFullScreenMenuManager::IsShowing
// Get menu show status
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//
TBool CFullScreenMenuManager::IsShowing()
    {
    if( iOptionMenuBar != NULL )
        {
        return iOptionMenuBar->IsDisplayed();
        }
    return EFalse;
    }

// -----------------------------------------------------------------------------
// CFullScreenMenuManager::Hide
// Hide current showed menu
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//
void CFullScreenMenuManager::Hide()
    {
    if( IsShowing() )
        {
        iOptionMenuBar->StopDisplayingMenuBar();
        }
    }

// -----------------------------------------------------------------------------
// CFullScreenMenuManager::MenuPaneId
// Get current menu resource id
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//
TInt CFullScreenMenuManager::MenuPaneId()
    {
    if( iOptionMenuBar )
        {
        CEikMenuBar::CTitleArray* titles = iOptionMenuBar->TitleArray();
        if( titles->Count()	> 0 )
            {
            return (*titles)[0]->iData.iMenuPaneResourceId;
            }
        }
    return 0;
    }

// -----------------------------------------------------------------------------
// CFullScreenMenuManager::SetEmphasis
// called when menu get or lose focus
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//
void CFullScreenMenuManager::SetEmphasis(CCoeControl* aMenuControl, TBool aEmphasis)
    {
    if( !aEmphasis && aMenuControl == iOptionMenuBar)
        {
        iOptionMenuBar->SetFocus(EFalse);
        CCoeEnv* coeenv = CCoeEnv::Static();
        coeenv->AppUi()->RemoveFromStack(iOptionMenuBar);
        iOnStack = EFalse;
        }
    }

// -----------------------------------------------------------------------------
// CFullScreenMenuManager::ProcessVolumeCommandL
// called when menu is clicked by ProcessCommandL
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//
    
void CFullScreenMenuManager::ProcessVolumeCommandL(TInt aCommandId)
{
	CPluginMenuManager* menuHandler = iCtrl->GetMenuManager();
		
	switch(aCommandId)
	{
		case EEikFlashCmdMute:
				{
					menuHandler->MuteVolume();
				}
				break;
		case EEikFlashCmdUnmute:
				{
					menuHandler->UnmuteVolume();
				}
				break;
		case EEikFlashCmdVolume:
				{
					//need to display volume control
					CFlashVolumeControl* volumPopup = iCtrl->GetVolumeControl();
					
					if(volumPopup)
					{
						TInt currentVolume = 0;
				
						//if(iCtrl->GetVolumeProp())
    						//currentVolume = iCtrl->GetVolumeProp()->GetCurrentVolume();
						CVolumeCenRepKeyWatcher* volWatcher = iCtrl->FlashPluginInstance()->GetVolumeCenRepKeyWatcher();
						if(volWatcher)
							currentVolume = volWatcher->GetCurrentVolume();

						volumPopup->ShowPopupL(currentVolume);	
					}
										
				}
				break;
	}
	
}

// -----------------------------------------------------------------------------
// CFullScreenMenuManager::ProcessCommandL
// Process command user selects
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//
void CFullScreenMenuManager::ProcessCommandL(TInt aCommandId)
    {

         Hide();
        
     if( aCommandId == EEikFlashCmdPartScreen)
     {	
     	iCtrl->GoNormalScreenL(); //TDO: crashing , fix later ,
     	return;
     }
     
     ProcessVolumeCommandL(aCommandId);
     
    }

// -----------------------------------------------------------------------------
// CFullScreenMenuManager::MenuPaneId
// C++ default constructor
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//
CFullScreenMenuManager::CFullScreenMenuManager(CBrowserPluginControl* aCtrl)
												:iCtrl(aCtrl)
												,iOnStack(EFalse)
												
    {
    }

// -----------------------------------------------------------------------------
// CFullScreenMenuManager::DynInitMenuBarL
// Init menu bar
// 
// -----------------------------------------------------------------------------
//
void CFullScreenMenuManager::DynInitMenuBarL(TInt /*aResourceId*/,CEikMenuBar* /*aMenuBar*/)
{
	
}

// -----------------------------------------------------------------------------
// CFullScreenMenuManager::AddMuteOptionMenu
// Process volume menu option init
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//

void CFullScreenMenuManager::AddMuteOptionMenu(CEikMenuPane* aMenuPane)
{
	CPluginMenuManager* menuHandler = iCtrl->GetMenuManager();
	
	if(menuHandler->IsMuted())
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdMute,ETrue);
	}
	else
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdUnmute,ETrue);
	}	
	
}


// -----------------------------------------------------------------------------
// CFullScreenMenuManager::DynInitMenuPaneL
// Init menu items
// 
// -----------------------------------------------------------------------------
//
void CFullScreenMenuManager::DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane)
{
	if (aResourceId == R_FLASH_FULLSCREEN_CONTROL_MENU)
	{
		if(iCtrl->FlashPluginInstance()->IsVolumeKeySupported())	
		{
			aMenuPane->SetItemDimmed(EEikFlashCmdVolume,ETrue);
		}
		
		AddMuteOptionMenu(aMenuPane);
		
	}
	
}
//End Of File
