/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CFlashPluginInstance header file
*
*/


#include "pluginbase.h"

#include <AknSettingCache.h>
#include <featmgr.h>
#include <touchfeedback.h>
#include <AknInfoPopupNoteController.h>
#include <coemain.h>
#include <e32property.h>
#include <PtiDefs.h>
#include <bautils.h>
#include <AknQueryDialog.h>
#include <eikenv.h>
#include <flashliteplugin.rsg>
#include <flashlitepluginiad.rsg>
#include <AvkonInternalCRKeys.h>
#include <StringLoader.h>
#include <npapi.h>

#include "saflash_consts.h"
#include "flashpluginif.h"

#include "flashiconhandler.h"
#include "flashplugincursor.h"




PluginBase::PluginBase(MFlashPluginInstance* aPluginInstance):
                         iFeedback(NULL),
                        iPlayerInstance(aPluginInstance)
                        ,iIconHandler(NULL)
                        ,iConstructed(EFalse)
                        ,iLastError(KErrNone)
                        ,iValid(EFalse)
                        ,iFileOpened(EFalse)
                        ,iMP3StreamState(EFlashNoMP3Stream)                       
                        ,iVisibilityState(EFlashPluginInvisible)                       
                        ,iFocusState(EFlashPluginForeground)                       
                        ,iPlayState(EFlashPluginInitial)
                #ifdef FEATURE_COLLECT_BITMAP
                        ,iPluginScrollState(EScrollingNone)
                #endif
                        ,iErrorOccured(EFalse)
                #ifdef ENABLE_CURSOR
                        ,iCursor(NULL)
                #endif
#ifdef FEATURE_INLINE_TEXT_ENTRY
                        ,iIMEControl(NULL)
                        ,iCurrentInputLanguage(ELangEnglish)
                        ,iIsNoninline(EFalse)
#endif //FEATURE_INLINE_TEXT_ENTRY      
                        , iTextInput(NULL)                        
                       
    {
    }

PluginBase::~PluginBase()
{
    FeatureManager::UnInitializeLib(); // Uninitialise Feature manager    
    
#ifdef _BITBUFFER_ENABLED_ 
    /*
    if(iBitBuffer)
     {
     delete iBitBuffer;        
     iBitBuffer = NULL;
     }
     
     if (iBitmapGc)
     {
     delete iBitmapGc;
     iBitmapGc = NULL;
     }
     if (iBitmapDevice)
     {
     delete iBitmapDevice;
     iBitmapDevice = NULL;
     }
     */    
#endif
    
#ifdef ENABLE_CURSOR    
    if( iCursor )       
        {
        delete iCursor;
        iCursor = NULL;
        }        
#endif

    if (iIconHandler)
	{
        delete iIconHandler;
	}
    
    if(iTextInput)
	{
	    delete iTextInput;
	}    

     // stop listening qwerty mode status
	if (iQwertyModeStatusSubscriber)
	{
		iQwertyModeStatusSubscriber->UnSubscribe();
		delete iQwertyModeStatusSubscriber;
		iQwertyModeStatusSubscriber = NULL;
	}
	iQwertyModeStatusProperty.Close();
}

void PluginBase::ConstructL()
    {
    FeatureManager::InitializeLibL(); // Initialising feature manager
    iTactileFeedbackSupported = FeatureManager::FeatureSupported(
            KFeatureIdTactileFeedback);
   // iSideVolumeKeysSupported = FeatureManager::FeatureSupported(
     //       KFeatureIdSideVolumeKeys);

    //TODO : Should qwerty keypad registration happen here?
    CreateQwertyKeypadObserverL(); 
    
    if(iTactileFeedbackSupported)  //adding feature manager to check the feature at run time.
        iFeedback = MTouchFeedback::Instance();
    
    }

MFlashPluginInstance* PluginBase::FlashPluginInstance() const
    {
    return iPlayerInstance;
    }

void PluginBase::SetErrorOccured(TInt aError)
    {
    iErrorOccured = ETrue;
    if (aError != KErrNone)
        iLastError = aError;

    Redraw();
    }

void PluginBase::MP3PlayStateChangedL(EMP3StreamState aMP3StreamState)
    {
    if (aMP3StreamState == EFlashNoMP3Stream && iMP3StreamState
            != EFlashNoMP3Stream)
        {
        //pause the plugin if we are in background or invisible
        //If already paused don't do anything 
        if (iPlayState == EFlashPluginPaused)
            {
            }
        else if (((iFocusState == EFlashPluginBackground)
                || (iVisibilityState == EFlashPluginInvisible)))
            {

            iPlayState = EFlashPluginPaused;

            MFlashEngine* engine = FlashPluginInstance()->GetEngine();
            if (engine)
                {
                engine->SetForeground(EFalse);
                engine->NotifyFocusChangeL(ETrue); // Pausing
                }
            //TODO : Hw volume key handling..
            /*if(!iRemovedHwVolumeKeyObserver)
             {
             CancelHwVolumeKeyObserver();
             }*/
            }
        }
    else if ((iPlayState == EFlashPluginPaused) && (aMP3StreamState == EFlashMP3StreamInProgress && iMP3StreamState
                    == EFlashNoMP3Stream))
       {
           iPlayState = EFlashPluginPlaying;
           MFlashEngine* engine = FlashPluginInstance()->GetEngine();               
           if( engine )
           {
               engine->SetForeground(ETrue);
               engine->NotifyFocusChangeL(EFalse); // Resuming
           }
       }
        
    iMP3StreamState = aMP3StreamState;
    }

void PluginBase::SetClipRect(const TRect& aClipRect)
{
    iClipRect = aClipRect;
}

RWindow* PluginBase::GetNewVideoWindow(void* /*aClient*/) 
{
    return NULL; 
}

void PluginBase::CloseVideoWindow(void* /*aClient*/)
{
    return;
}



TBool PluginBase::IsConstructed() 
{ 
    return iConstructed; 
}
   
TBool PluginBase::IsInitialized() 
    {
    if (iPlayerInstance)
        {
        MFlashEngine* engine = iPlayerInstance->GetEngine();

        if (engine && engine->IsInitialized())
            {
            return ETrue;
            }
        }

    return EFalse;

    }

TInt PluginBase::GetLastError()
{
    return iLastError;
}

MFlashEngine* PluginBase::GetPlayerEngine() const 
    { 
    return iPlayerInstance->GetEngine(); 
    }

TBool PluginBase::IsValid() 
{
    return iValid;
}

TBool PluginBase::IsFileOpened()
{ 
    return iFileOpened; 
}

TBool PluginBase::IsControlVisible() const
{
    return iVisibilityState;
}

void PluginBase::ProvideFeedbackL(const TPointerEvent& aPointerEvent)
{
    if(iTactileFeedbackSupported) //adding feature manager to check the feature at run time.
    {
        // Provide feedback only if platform supports it. 
        if( !iFeedback  || (iFeedback && !iFeedback->TouchFeedbackSupported()) )
        {
            return; 
        }
        
        MFlashEngine* engine = this->iPlayerInstance->GetEngine();
        
        TBool controlPresent = engine->IsControlPresent(aPointerEvent.iPosition);
            
        if( controlPresent )
        {
            switch( aPointerEvent.iType )
            {
                case TPointerEvent::EButton1Down:
                {
                    iFeedback->InstantFeedback( ETouchFeedbackBasic );
                    break;
                }
                case TPointerEvent::EDrag:
                case TPointerEvent::EButtonRepeat:
                {
                    iFeedback->InstantFeedback( ETouchFeedbackSensitive );
                    break;
                }
            }
        }
    }
}

void PluginBase::CreateQwertyKeypadObserverL()
{
    // Start listening qwerty mode status.
    User::LeaveIfError(iQwertyModeStatusProperty.Attach(KCRUidAvkon,KAknQwertyInputModeActive));
    iQwertyModeStatusSubscriber = new (ELeave) CKeypadHandler(*this, iQwertyModeStatusProperty);
    iQwertyMode = iQwertyModeStatusSubscriber->GetQwertMode();
    iQwertyModeStatusSubscriber->SubscribeL();
}

void PluginBase::HandleQwertyModeChangeNotification(TInt aQwertyMode)
{
    iQwertyMode = aQwertyMode;

    /*
    #ifdef ENABLE_CURSOR
    // Hide the flash cursor (if exists) when qwerty keypaded slided to OFF state
    if( iCursor && !iQwertyMode && iCursor->IsVisible())
    {
        iCursor->HideCursor();
    }
    #endif
    */
}

TInt PluginBase::GetQwertyKeyPadStatus()
{
    return iQwertyMode;
}

void PluginBase::NotifyMp3FocusChangeL(TBool aFocus)
    {
    //Do nothing when mp3 is playing we don't want to pause the swf
    if (aFocus && ((iFocusState == EFlashPluginForeground)
            || (iVisibilityState == EFlashPluginVisible)))
        {
        //if all mp3 streams are closed and we get foreground event we swith to nomp3stream state    
        if (iMP3StreamState == EFlashMP3StreamClosed)
            iMP3StreamState = EFlashNoMP3Stream;

        }
    }

int PluginBase::LoadPluginResourceL(const TDesC& aFileName)
    {
    TInt resOffset = -1;

    TBuf16<128> aResourcePath(_L("\\resource\\apps"));

    // strip the extension
    TInt fileNameLength = aFileName.Length();
    TFileName resourceFileName;
    
    CEikonEnv* env = CEikonEnv::Static();

    RFs fileServerSession = env->FsSession();

    if (fileNameLength > 4)
        {
        // we want just the name 
        // ex, if aFileName is flash2ui.rsc,
        // fName will be flash2ui
        TBuf16<128> fName = aFileName.Mid(0, fileNameLength - 4);

        TBuf16<512> variantFile;
        _LIT16( KFormat, ":%S\\%S.r%S%d");

        // search for the resource file based on the current language
        TInt languageID = User::Language();
        TBuf16<8> leadingZero;
        leadingZero.Copy(_L(""));
        if (languageID < 10)
            leadingZero.Copy(_L("0"));

        // start from Y: go to A: and then finally search Z:
        for (TInt i = EDriveY; i >= EDriveA; i--)
            {
            //TChar driveLetter = 'a'; // +i;
            TChar driveLetter;
            fileServerSession.DriveToChar(i, driveLetter);
            variantFile.Zero();
            variantFile.Append(driveLetter);
            variantFile.AppendFormat(KFormat,  &aResourcePath, &fName,
                    &leadingZero, languageID);

            resourceFileName.Copy(variantFile);
            ////NPPDebug(LOG_DEBUG, _L("Trying: %S\n"), &resourceFileName);
            TBool fileExists = BaflUtils::FileExists(
                    env->FsSession(), resourceFileName);
            if (fileExists)
                {
                resOffset = env->AddResourceFileL(
                        resourceFileName);
                //NPPDebug(LOG_DEBUG, _L("resource : %S"), &resourceFileName);
                return resOffset;
                }

            }
        // see if it is present on z drive
        TChar driveLetterZ;
        fileServerSession.DriveToChar(EDriveZ, driveLetterZ);
        variantFile.Zero();
        variantFile.Append(driveLetterZ);
        variantFile.AppendFormat(KFormat,  &aResourcePath, &fName,
                &leadingZero, languageID);

        resourceFileName.Copy(variantFile);
        //NPPDebug(LOG_DEBUG, _L("Trying: %S\n"), &resourceFileName);
        TBool fileExists = BaflUtils::FileExists(
                env->FsSession(), resourceFileName);
        if (fileExists)
            {
            resOffset = env->AddResourceFileL(resourceFileName);
            //NPPDebug(LOG_DEBUG, _L("resource : %S"), &resourceFileName);
            return resOffset;
            }
        // if exact language is not found, search for the nearest language in the respective drive
        // start from Y: go to A: and then finally search Z:
        _LIT16( KNearestLangFormat, ":%S\\%S");
        for (TInt i = EDriveY; i >= EDriveA; i--)
            {
            TChar driveLetter;
            fileServerSession.DriveToChar(i, driveLetter);
            //TBuf16<128> KFormat( _L("%c:%S\\%S"));
            variantFile.Zero();
            variantFile.Append(driveLetter);
            variantFile.AppendFormat(KNearestLangFormat, 
                    &aResourcePath, &aFileName); // use the filename that was passed in

            resourceFileName.Copy(variantFile);
            BaflUtils::NearestLanguageFile(env->FsSession(),
                    resourceFileName);
            //NPPDebug(LOG_DEBUG, _L("Trying: %S\n"), &resourceFileName);
            TBool fileExists = BaflUtils::FileExists(
                    env->FsSession(), resourceFileName);
            if (fileExists)
                {
                resOffset = env->AddResourceFileL(
                        resourceFileName);
                //NPPDebug(LOG_DEBUG, _L("resource : %S"), &resourceFileName);
                return resOffset;
                }

            }

        // see if it is present on z drive
        // TBuf16<128> KZFormat( _L("z:%S\\%S"));
        variantFile.Zero();
        variantFile.Append(driveLetterZ);
        variantFile.AppendFormat(KNearestLangFormat,  &aResourcePath,
                &aFileName);
        resourceFileName.Copy(variantFile);
        BaflUtils::NearestLanguageFile(env->FsSession(),
                resourceFileName);
        //NPPDebug(LOG_DEBUG, _L("Trying: %S\n"), &resourceFileName);
        fileExists = BaflUtils::FileExists(env->FsSession(),
                resourceFileName);
        if (fileExists)
            {
            resOffset = env->AddResourceFileL(resourceFileName);
            //NPPDebug(LOG_DEBUG, _L("resource : %S\n"), &resourceFileName);
            return resOffset;
            }

        }

    return resOffset; // -1

    }

TBool PluginBase::QueryAllowDenyL(const TDesC& aQueryText)
    {
    TBool rval = EFalse;
    TBufC<128> KResFile(_L("FlashlitePlugin.rsc") );
    //TInt resOffset = load_pluginresourceL( KResFile );
    TInt resOffset = PluginBase::LoadPluginResourceL(KResFile);

    CAknQueryDialog* dlg = CAknQueryDialog::NewL();
    TInt result = (dlg->ExecuteLD(R_SAFLASH_DIALOG_ALLOW_DENY, aQueryText));

    if (result)
        {
        rval = ETrue;
        }

    CEikonEnv::Static()->DeleteResourceFile(resOffset);

    return rval;
    }

TBool PluginBase::QueryTrustDomainL(const TDesC& aDomain)
    {
    
    TBufC<128> KResFile(_L("FlashlitePluginIAD.rsc") );
    TInt resOffset = PluginBase::LoadPluginResourceL(KResFile);

    HBufC * errorMsg = StringLoader::LoadLC(R_STR_SECURITY_TRUST_PLUGIN, aDomain, CCoeEnv::Static() );
    
    TBool res = PluginBase::QueryAllowDenyL(*errorMsg);
    
    CleanupStack::PopAndDestroy(errorMsg);
    CEikonEnv::Static()->DeleteResourceFile(resOffset);

    return res;
    }

TBool PluginBase::QueryAllowDenyWithFormatResourceL(const TDesC&
     aQueryText, TInt aResId)
    {
    HBufC* pMsg = FormatL(aResId, aQueryText);
    CleanupStack::PushL(pMsg);
    //TBool allow=QueryAllowDenyL(*pMsg);
    TBool allow = PluginBase::QueryAllowDenyL(*pMsg);
    CleanupStack::PopAndDestroy(pMsg);
    return allow;
    }
/*
TBool PluginBase::QueryAllowDenyWithFormatResourceL(const TDesC&
     aQueryText, TInt aResId)
    {
    HBufC* pMsg = FormatL(aResId, aQueryText);
    CleanupStack::PushL(pMsg);
    //TBool allow=QueryAllowDenyL(*pMsg);
    TBool allow = PluginBase::QueryAllowDenyL(*pMsg);
    //TBool allow = ETrue;
    
    CleanupStack::PopAndDestroy(pMsg);
    return allow;
    }
*/

HBufC * PluginBase::FormatL(TInt aFormatResource,
 const TDesC& aArg )
    {
    CEikonEnv* env = CEikonEnv::Static();
    HBufC* pMsg;
    HBufC* pFormat =
    env->AllocReadResourceLC(aFormatResource);
    pMsg = HBufC::NewLC(pFormat->Length() + aArg.Length());
    TPtr ptr = pMsg->Des();
    ptr.Format(*pFormat, &aArg);
    CleanupStack::Pop(pMsg);
    CleanupStack::PopAndDestroy(pFormat);
    return pMsg;
    }

#ifdef _BITBUFFER_ENABLED_ 
BitmapBuffer* PluginBase::BitBuffer()
    {
    return static_cast<BitmapBuffer*>(iBitBuffer); 
    }
#endif

#ifdef FEATURE_INLINE_TEXT_ENTRY

void PluginBase::GetInputTextL(const TUint16* aInitialString, 
                                        TUint aStrLen, 
                                        TUint aMaxNumChars, 
                                        TUint aTextFlags)
	{
	TGetInputTextData *getInputTextRequest = new (ELeave) TGetInputTextData;
	TPtrC16 ptr(aInitialString, aStrLen);
	
	getInputTextRequest->iInitialString = ptr.AllocL();  
	
	if (aMaxNumChars > KMaxNumChar)
		{
		// Max number characters which can be entered to text field.
		// Host will never get Zero for it.
		aMaxNumChars =  KMaxNumChar ;
		}	
		
	getInputTextRequest->iMaxNumChars = aMaxNumChars;
	getInputTextRequest->iTextFlags   = aTextFlags;
	
	GetInputTextL(*getInputTextRequest);

	delete getInputTextRequest->iInitialString;
	delete getInputTextRequest;
	}

TKeyResponse PluginBase::HandleKeyEventL(const TKeyEvent& aKeyEvent, 
                                                 TEventCode aType)
{
	TKeyEvent keyEvent = aKeyEvent;   
	TEventCode eventType = aType;
	    
	MFlashEngine* engine = iPlayerInstance->GetEngine();
	    
	if( !engine )
	{
		return EKeyWasNotConsumed;
	}
	         
	// This change is from S60 Rel 3.2 WK 44, 2006
	// In case of EStdKeyDevice3 (Select key) the order is found to be 
	// EEventKeyDown => EKeyEventUp => EKeyEventUp
	// So there is no EEventKey event generated.
	// So we send this in between down and up events for the EStdKeyDevice3 key
	    
	if( aKeyEvent.iScanCode == EStdKeyDevice3 )
	{
		if( iLastKeyEventCode == EEventKeyDown && 
					    aType == EEventKeyUp )
		{
			eventType = EEventKey;                      
		}
	}
	 
	TKeyResponse keyResponse = EKeyWasNotConsumed;
	 
	//    MFlashEngine* engine = FlashPluginInstance()->GetEngine();
	keyResponse = engine->OfferKeyEventL(keyEvent, eventType);
	        
	if( aKeyEvent.iScanCode == EStdKeyDevice3 )
	{
		if( iLastKeyEventCode == EEventKeyDown && 
						aType == EEventKeyUp )
		{
			iLastKeyEventCode = EEventNull;
		}
		else
		{
			iLastKeyEventCode = eventType;
		}
	}
#ifdef FEATURE_INLINE_TEXT_ENTRY
	if( iIMEControl && keyResponse )   
	{
		iIMEControl->ReportStateEventL();
	}
#endif
	return keyResponse;
}

void PluginBase::ResetIMEL()
{
    if( !iIMEControl )
        return;
    
    SimulateDisableIMEEventL();
    SimulateEnableIMEEventL();
}

void PluginBase::SimulateDisableIMEEventL()
{
    TKeyEvent keyEvent;
        
    keyEvent.iCode      = 0;
    keyEvent.iScanCode  = EStdKeyDevice3;
    keyEvent.iModifiers = 0;
    keyEvent.iRepeats   = 0;   
    
    MFlashEngine* engine = iPlayerInstance->GetEngine();
    
    if( !engine )
        return;
    
    // NOTE : This will disable IME and delete this.
    // So should be the last statement here. 
    
    engine->OfferKeyEventL(keyEvent, EEventKey);
        
    return;    
}

void PluginBase::SimulateEnableIMEEventL()
{
    TKeyEvent keyEvent;
        
    keyEvent.iCode      = 0;
    keyEvent.iScanCode  = EStdKeyDevice3;
    keyEvent.iModifiers = 0;
    keyEvent.iRepeats   = 0;   
    
    MFlashEngine* engine = iPlayerInstance->GetEngine();
    
    if( !engine )
        return;
    
    engine->OfferKeyEventL(keyEvent, EEventKey);
        
    return;    
}

void PluginBase::EnableIMEL()
    {
    if(this->iVisibilityState == EFlashPluginInvisible )
        return;
    iIsIMEEnabled = ETrue;
	CAknSettingCache& cache = CAknEnv::Static()->SettingCache();
    if (CurrentInputLanguage() != cache.InputLanguage())
	    {
	    SetCurrentInputLanguage(cache.InputLanguage());	
	    }
	
    MFlashEngine* engine = iPlayerInstance->GetEngine();
    
    InputModeSelector* inputModeSelector 
    = InputModeSelector::NewL(engine->GetTextNavigation(), this);
    CleanupStack::PushL(inputModeSelector);

		if( !inputModeSelector )
			User::Leave(KErrNoMemory);
                                        
    iIsNoninline  = inputModeSelector->IsNoninline();
	
	if ((iIMEControl) && !iIMEControl->IsInLine()) 
	    {	
		delete iIMEControl;
		iIMEControl = NULL;		
	    }

	if(!iIMEControl)
    	{
		if (iIsNoninline) 
			{	
			iIMEControl = CFlashPluginIMENonInlineControl::NewL(engine,
																this
																/*,PositionRelativeToScreen()*/);
	
			}
		else 
			{
			iIMEControl = CFlashPluginIMEControl::NewL(engine,
													  this
													  /*,PositionRelativeToScreen()*/);
            
            (STATIC_CAST(CFlashPluginIMEControl*, iIMEControl))
            ->SetCurrentInputLanguage(iCurrentInputLanguage);
			}

#ifdef ENABLE_CURSOR
			
        // Hide the flash cursor (if exists) during inline editing.
        if( iCursor )
            {
            iCursor->HideCursor();
            }
#endif
        }
       
	AddEventObserverToControl();
    CleanupStack::PopAndDestroy(); // inputModeSelector
    }

void PluginBase::DisableIME()
{   
    if(iIMEControl)
	{
	    
	    if( iIMEControl->IsInLine() )
	    {
	        CFlashPluginIMEControl* ime = (CFlashPluginIMEControl*) iIMEControl;
	        ime->DisableIME();
	    }
	    
		delete iIMEControl;
		iIMEControl = NULL;
		iIsIMEEnabled = EFalse;
		iIsNoninline = EFalse;

		MFlashEngine* engine = iPlayerInstance->GetEngine();
		
		engine->GetTextNavigation()->ShowCursor(ETrue);
		engine->RestoreSoftKeys();

		RemoveEventObserverFromControl();
	}
}   
#endif

// ----------------------------------------------------------------------
// TBool CBrowserPluginControl::IsTextInputBusy()
//  
// This function is used to check text input is running.
// ----------------------------------------------------------------------
//
TBool PluginBase::IsTextInputBusy()
{
    // coverity[var_compare_op : FALSE]
	if( iTextInput == NULL )
	{
		TRAPD(error,CreateTextInputL());
		if(error != KErrNone)
		{
			return EFalse;
		}
	}
	// coverity[var_deref_model : FALSE]
	return iTextInput->IsBusy();
}

// ----------------------------------------------------------------------
// void CBrowserPluginControl::GetInputTextL(TGetInputTextData& aData)
//  
// This function is used to get text input from user
// ----------------------------------------------------------------------
//

void PluginBase::GetInputTextL(TGetInputTextData& aData)
{
	if(!iTextInput)
		CreateTextInputL();
	else
		iTextInput->GetInputTextL(aData);
}

void PluginBase::CreateTextInputL()
{
	if(iTextInput)
		return;
	
    MFlashEngine* engine = FlashPluginInstance()->GetEngine();
    
	if(engine)
		iTextInput = new(ELeave) TextInput(engine);
}

/*TBool PluginBase::QueryAllowNetworkInWidgetMode()
{
    RArray<NPN_GenericElement>* genericElements = NULL;
    int err;

    TBool allow = ETrue;
    
    err = iPlayerInstance->NetscapeFns()->GetValue((NPNVariable) NPNVGenericParameter, (void*) &genericElements);

    if (genericElements != NULL && err == KErrNone)
    {
        for (int i = 0; i < genericElements->Count(); i++)
		{
			if (!((*genericElements)[i].genericElementId.Compare(KAllowNetworkAccess)))
			{
				allow = (*genericElements)[i].genericElementValue.intValue;
				break;
			}
		}  
    }
    
    return allow;
}*/
