/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description: 
*       Provides the CPluginMenuManager methods.
*
*
*/


#include "saplayer_build.h"
#include <coemain.h>
#include <coeaui.h>
#include "flashbrowserplugincontrol.h"
#include "pluginmenumgr.h"
#include "flashliteplugin.hrh"
#include <flashliteplugin.rsg>
#include <eikmenup.h>
#include <StringLoader.h> //stringLoader
#include "volumecenrepkeywatcher.h"
#include "flashvolumecontrol.h"

extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );

enum TFlashPlayerCommandIds
{
	EFlashPlgnCmdUnknown = 0
    ,EFlashPlgnCmdPlayFullScreen
    ,EFlashPlgnCmdMuteVolume
    ,EFlashPlgnCmdVolumeLevel
   /* EBvpCmdStop,
    EBvpCmdClose,
    EBvpCmdPause,
    EBvpCmdChangeVolume,
    EBvpCmdRewind,
    EBvpCmdFastForward
*/
};



/**
 * Symbian constructor.
 *
 * @since S60 v3.1
 * @return Pointer to created CPluginMenuManager object
 */
CPluginMenuManager* CPluginMenuManager::NewL(CBrowserPluginControl* aCtrl)
{
   	NPPDebug(LOG_DEBUG,_L("CPluginMenuManager::NewL() ++"));

	CPluginMenuManager* self = new(ELeave) CPluginMenuManager(aCtrl);

	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);

	NPPDebug(LOG_DEBUG,_L("CPluginMenuManager::NewL() --"));

	return self;	
}
   
/**
 * Destructor.
 */
CPluginMenuManager::~CPluginMenuManager()
{
	
}

/**
 * C++ default constructor.
 *
 * @param aCtrl CBrowserPluginControl object.
 */
CPluginMenuManager::CPluginMenuManager(CBrowserPluginControl* aCtrl):
												iCtrl(aCtrl)
												, iCommandBase(0)
{
	
}

/**
    * By default Symbian 2nd phase constructor is private.
    */
void CPluginMenuManager::ConstructL()
{
}
 
TBool CPluginMenuManager::IsMuted()
{
	// get the current volume from the control. This value is considered as volume may be
	// muted using hw keys.
	TInt volVal = iCtrl->FlashPluginInstance()->GetCurrentEngineVolume();

	// changed for TSW ID: EPMR-7UPFEJ: Start
	//its possible that h/w key used to unmute
	// Actual volume level is checked to find if it is muted. Member variable iMuted is not
	// changed in this, use iMuted to identify the mute from menu.
	if(volVal > 0)
	{
		// Not muted
		return EFalse;
	}
	else
	{
		//Volume level is equal or less than 0, hence return Muted
		return ETrue;
	}
	
	// changed for TSW ID: EPMR-7UPFEJ: End
}
		
void CPluginMenuManager::MuteVolume()
{
	iCtrl->FlashPluginInstance()->MuteVolume();
}

void CPluginMenuManager::UnmuteVolume()
{
	// changed for TSW ID: EPMR-7UPFEJ: Start
	iCtrl->FlashPluginInstance()->UnMuteVolume();
	// changed for TSW ID: EPMR-7UPFEJ: End
 
}

/**
* AddPluginOptionMenuItemsL
* Add the Option Menu items to the menuPane, these will be handled by plugin.
* @since S60 v3.1
* @param aMenuPane handle of the menu that will contain the options 
* @param aCommandBase base id that needs to be added to the command ID 
* @param aAfter Plugin options should be added after this index
* @return void
*/
void CPluginMenuManager::AddPluginOptionMenuItemsL( CEikMenuPane& aMenuPane, TInt aCommandBase,
                                TInt /*aAfter*/ )
{
	if(iCtrl->IsPlayingVideo() && iCtrl->IsPluginActivated())
	{
		iCommandBase = aCommandBase;
		
    	TBufC<128> KResFile(_L("FlashlitePlugin.rsc") );
    //TInt resOffset = load_pluginresourceL( KResFile );
    	TInt resOffset = PluginBase::LoadPluginResourceL( KResFile );
		TInt index( 0 );
		
		if(iCtrl->FlashPluginInstance()->GetAllowMobiFullScreen() && !iCtrl->IsVideoRectEmpty() 
#ifdef FEATURE_WINDOWLESS		
		&& (iCtrl->FlashPluginInstance()->GetWMode() == EWindow) //to stop FullScreen in options when no video is present
#endif
	    )
		{	
			if(!iCtrl->IsFullScreen())
			{
				InsertOneMenuItemL( aMenuPane, EFlashPlgnCmdPlayFullScreen,
	    				               R_TEXT_FLITE_PLUGIN_FULLSCREEN, 
	    				               index );
	    			
	    	
			}
			else
			{
				InsertOneMenuItemL( aMenuPane, EFlashPlgnCmdPlayFullScreen,
	    				               R_TEXT_FLITE_PLUGIN_NORMALSCREEN, 
	    				               index );
	    	}
			
			index++;
			
		}
		
		if(IsMuted())
		{
			//its possible that h/w key used to unmute
			
			InsertOneMenuItemL( aMenuPane, EFlashPlgnCmdMuteVolume,
    				               R_TEXT_FLITE_PLUGIN_UNMUTEVOLUME, 
    				               index );
    
		}
		else
		{
			InsertOneMenuItemL( aMenuPane, EFlashPlgnCmdMuteVolume,
    				               R_TEXT_FLITE_PLUGIN_MUTEVOLUME, 
    				               index );
    		
		}
		
		index++;
		
		if(!(iCtrl->FlashPluginInstance()->IsVolumeKeySupported()))	
		{
			InsertOneMenuItemL( aMenuPane, EFlashPlgnCmdVolumeLevel,
    				               R_TEXT_FLITE_PLUGIN_VOLUMELEVEL, 
    				               index );
    		index++;		               
		}
			
    	CEikonEnv::Static()->DeleteResourceFile(resOffset);
	}
	
}

 /**
* HandlePluginCommandL
* Handle the user command from the option menu to the plugin.
* @since S60 v3.1
* @param aCommand the command ID of the option
* @return void
*/
void CPluginMenuManager::HandlePluginCommandL( TInt aCommand )
{
	NPPDebug(LOG_DEBUG,_L("CPluginMenuManager::Go HandlePluginCommandL") );	
	
	TFlashPlayerCommandIds  cmd = (TFlashPlayerCommandIds)(aCommand - iCommandBase) ;
	
	switch(cmd)
	{
		case EFlashPlgnCmdPlayFullScreen: //Toggle screen
			{
				NPPDebug(LOG_DEBUG,_L("CPluginMenuManager::Toggle Screen") );	
				
				iCtrl->ToggleScreenL();
				
			}
			break;
			
		case EFlashPlgnCmdMuteVolume: //toggle mute/unmute
			{
				NPPDebug(LOG_DEBUG,_L("CPluginMenuManager::Toggle Mute/Unmute") );	
				
				if(IsMuted())
				{
					UnmuteVolume();
				}
				else
				{
					MuteVolume();
				}
			}
			break;
		
		case EFlashPlgnCmdVolumeLevel:
			{
				NPPDebug(LOG_DEBUG,_L("CPluginMenuManager::Volume Change") );	
				
				//need to display volume control
				CFlashVolumeControl* volumPopup = iCtrl->GetVolumeControl();
				
				if(volumPopup)
				{
					TInt currentVolume = 0;
				
					//if(iCtrl->GetVolumeProp())
    					//currentVolume = iCtrl->GetVolumeProp()->GetCurrentVolume();
					CVolumeCenRepKeyWatcher* volWatcher = iCtrl->FlashPluginInstance()->GetVolumeCenRepKeyWatcher();
					if(volWatcher)
						currentVolume = volWatcher->GetCurrentVolume();
    			
					volumPopup->ShowPopupL(currentVolume);	
				}
			}
			break;
	}
}
	
		
void CPluginMenuManager::InsertOneMenuItemL( CEikMenuPane& aMenuPane, TInt aCommand,
                                   TInt aResourceId, TInt index )
{
	NPPDebug(LOG_DEBUG,_L("CPluginMenuManager::InsertOneMenuItemL") );

    CEikMenuPaneItem::SData item;
    HBufC* buf = StringLoader::LoadLC( aResourceId );
    item.iText.Copy( *buf );
    CleanupStack::PopAndDestroy();  // buf
    buf = NULL;

    item.iCommandId = aCommand + iCommandBase;
    item.iFlags = 0;
    item.iCascadeId = 0;
    aMenuPane.InsertMenuItemL( item, index );	
    
}

//End of file

