/*
* Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  FlashPluginControl class implementation
*
*/


#include "pointergesturedetector.h"
//#define _GestureDebug_


const TInt KDoubleTapInterval = 500000; //500 milli secs
const TInt KAccidentalUpTimer = 250000;  
const TInt KMinDoubleTapInterval = 100000;
const TInt KDoubleTapArea = 3600; // 60 pixel radius


CGestureDetector::CGestureDetector(MGestureEventObserver* aObserver)
{
 	 iGestureObserver = aObserver;
 	 iInsidePointerEventHandler = EFalse; 	 
}

CGestureDetector* CGestureDetector::NewL(MGestureEventObserver* aObserver)
{
	CGestureDetector * self = new(ELeave) CGestureDetector(aObserver);
	self->ConstructL();
	return self;
}

void CGestureDetector::ConstructL()
{
    iFlashLongTapDetector = CFlashLongTapDetector::NewL(this);
    iDoubleTapTimer = CPlgTimerObject::NewL(CActive::EPriorityStandard, *this);
    iAccidentalUpTimer = CPlgTimerObject::NewL(CActive::EPriorityStandard, *this);
    iState = EStateWaiting;
} 
   
void CGestureDetector::HandleFlashLongTapEventL( const TPoint& /*aPoint*/, const TPoint& /*aParentPoint*/)
{
    #ifdef _GestureDebug_
    RDebug::Printf("CGestureDetector::HandleFlashLongTapEventL ++ iState = %d",iState);
    #endif//_GestureDebug_
    
    if (iState == EStateIntermediate1) 
    { 
        iState = EStatELongTap;
        iDoubleTapTimer->Cancel(); //Just in case.
        
        TPointerEvent tempPtrEvt = iPointerEventArray[0];
        iPointerEventArray.Reset();		
		
        iGestureObserver->HandleGestureL(EGestureLongTap,tempPtrEvt);
    }
    
    #ifdef _GestureDebug_
    else if(iState != EStateIntermediate4)
    {
        //User::Panic(_L("gesture detection alert - incorrect state"),iState);
    }
    RDebug::Printf("CGestureDetector::HandleFlashLongTapEventL -- iState = %d",iState);    
    #endif
}

void CGestureDetector::Reset()
{
    iState = EStateWaiting;
	
    iDoubleTapTimer->Cancel();
    iAccidentalUpTimer->Cancel();  
    iPointerEventArray.Reset();
}

void CGestureDetector::HandlePointerEventL(const TPointerEvent &aPointerEvent)
{
#ifdef _GestureDebug_
    if(aPointerEvent.iType == TPointerEvent::EButton1Down) {
        RDebug::Printf(" CGestureDetector::HandlePointerEventL, DOWN %d %d",aPointerEvent.iPosition.iX,aPointerEvent.iPosition.iY);    
    }
    else if (aPointerEvent.iType == TPointerEvent::EButton1Up) {
        RDebug::Printf(" CGestureDetector::HandlePointerEventL, UP %d %d",aPointerEvent.iPosition.iX,aPointerEvent.iPosition.iY);    
    }
    else if (aPointerEvent.iType == TPointerEvent::EDrag) {
        RDebug::Printf(" CGestureDetector::HandlePointerEventL, MOVE %d %d",aPointerEvent.iPosition.iX,aPointerEvent.iPosition.iY);    
    } 
#endif

    if(iInsidePointerEventHandler) {
        RDebug::Printf(" CGestureDetector::HandlePointerEventL, Already inside HandlePointerEventL; Do nothing");
    }    
    iInsidePointerEventHandler = ETrue;

    
    TPoint iPoint = aPointerEvent.iPosition;
	
    if(iFlashLongTapDetector)
    {
    iFlashLongTapDetector->PointerEventL(aPointerEvent);
    }
    
    switch(iState)
    {
        case EStateWaiting:
        {
            #ifdef _GestureDebug_
            RDebug::Printf(" CGestureDetector::HandlePointerEventL, state = WAITING");  
            #endif
        
            if(aPointerEvent.iType == TPointerEvent::EButton1Down)
            {
               iState = EStateIntermediate1;
               iPointerEventArray.Append(aPointerEvent);
            }
            else if (aPointerEvent.iType == TPointerEvent::EButton1Up)
            {//Not possible.
            }
            else if (aPointerEvent.iType == TPointerEvent::EDrag)
            {
                iState = EStateDrag;
				
				TPointerEvent temp(aPointerEvent);
                temp.iType = TPointerEvent::EButton1Down;
                iGestureObserver->HandleGestureL(EGestureDrag,temp);
				
                iGestureObserver->HandleGestureL(EGestureDrag,aPointerEvent);             
            }
            break;
        }
            
        case EStateIntermediate1: //Came to this state from Down
        {
            #ifdef _GestureDebug_
            RDebug::Printf(" CGestureDetector::HandlePointerEventL, state = EStateIntermediate1");   
            #endif
        
            if(aPointerEvent.iType == TPointerEvent::EButton1Down)
            {//Not possible
            }
            else if (aPointerEvent.iType == TPointerEvent::EButton1Up)
            {//Start timer
                iPrevUpPoint = aPointerEvent.iPosition;   
                iPrevUpTime.HomeTime();
				
                iState = EStateIntermediate2;
                iDoubleTapTimer->Cancel();
                iDoubleTapTimer->After(KDoubleTapInterval);
				
                iPointerEventArray.Append(aPointerEvent);
            }
            else if (aPointerEvent.iType == TPointerEvent::EDrag)
            {
                iState = EStateDrag;				
                iDoubleTapTimer->Cancel();

				TPointerEvent tempPtrEvt = iPointerEventArray[0];
                iPointerEventArray.Reset();				
                iGestureObserver->HandleGestureL(EGestureDrag,tempPtrEvt);
				
                iGestureObserver->HandleGestureL(EGestureDrag,aPointerEvent);             
            }
            break;
        }
        
        case EStateIntermediate2: //Came to this state from UP
        {
            #ifdef _GestureDebug_
            RDebug::Printf(" CGestureDetector::HandlePointerEventL, state = EStateIntermediate2");        
            #endif
            if(aPointerEvent.iType == TPointerEvent::EButton1Down)
            {
                TPoint displacement = iPoint - iPrevUpPoint;
                if( (displacement.iX * displacement.iX + displacement.iY *  displacement.iY) < KDoubleTapArea )
                {//Down at 'double tappable' distance.
                    iState = EStateIntermediate3;                    
                    iPointerEventArray.Append(aPointerEvent);
                }
                else
                {//Second down at a distance.
                    iState = EStateIntermediate1;
                    iDoubleTapTimer->Cancel();

					TPointerEvent tempPtrEvt = iPointerEventArray[0];
                    iPointerEventArray.Reset();
                    iPointerEventArray.Append(aPointerEvent);
                   
                    iGestureObserver->HandleGestureL(EGestureTap,tempPtrEvt);
                }
            }
            else if (aPointerEvent.iType == TPointerEvent::EButton1Up)
            {//Not possible.
            }
            else if (aPointerEvent.iType == TPointerEvent::EDrag)
            {//Not possible.
            }
            break;
        }
    
        case EStateIntermediate3: //Came here on DOWN
        {
            #ifdef _GestureDebug_
            RDebug::Printf(" CGestureDetector::HandlePointerEventL, state = EStateIntermediate3");
            #endif
            if(aPointerEvent.iType == TPointerEvent::EButton1Down)
            {//Not possible.
            }
            else if (aPointerEvent.iType == TPointerEvent::EButton1Up)
            {
                TTime now;
                now.HomeTime();
                if(now.MicroSecondsFrom(iPrevUpTime).Int64() < KDoubleTapInterval)
                {
                    if(now.MicroSecondsFrom(iPrevUpTime).Int64() < KMinDoubleTapInterval)
                    {
                        iState = EStateWaiting;                    
                        iDoubleTapTimer->Cancel();
                        iPointerEventArray.Reset(); //Clear out events.
                    }
                    else
                    {//Double tap.
                        iState = EStateWaiting;                        
                        iDoubleTapTimer->Cancel();
                        
                        iPointerEventArray.Reset(); //Clear out events.
                        iGestureObserver->HandleGestureL(EGestureDoubleTap,aPointerEvent);
                    }
                }
                else
                {//For some reason, double tap timer didnt fire. Anyway we know.
                    iState = EStateWaiting;
                    iDoubleTapTimer->Cancel();

					TPointerEvent tempPtrEvt0 = iPointerEventArray[0];
					TPointerEvent tempPtrEvt2 = iPointerEventArray[2];
                    iPointerEventArray.Reset(); //Clear out events.  
                    iGestureObserver->HandleGestureL(EGestureTap,tempPtrEvt0);
                    iGestureObserver->HandleGestureL(EGestureTap,tempPtrEvt2);
                }
            }
            else if (aPointerEvent.iType == TPointerEvent::EDrag)
            {
                iState = EStateDrag;
                iDoubleTapTimer->Cancel();

				TPointerEvent tempPtrEvt0 = iPointerEventArray[0];
				TPointerEvent tempPtrEvt2 = iPointerEventArray[2];
                iPointerEventArray.Reset(); //Clear out events.  
                iGestureObserver->HandleGestureL(EGestureTap, tempPtrEvt0);
                iGestureObserver->HandleGestureL(EGestureDrag, tempPtrEvt2);
                iGestureObserver->HandleGestureL(EGestureDrag, aPointerEvent);                
            }
            break;
        }
        
        case EStateDrag:
        {
            #ifdef _GestureDebug_
            RDebug::Printf(" CGestureDetector::HandlePointerEventL, state = EStateDrag");
            #endif
            
            if(aPointerEvent.iType == TPointerEvent::EButton1Down)
            {//Special case user drags outside flash window.Up event is not received   
                iState = EStateIntermediate1;
                iPointerEventArray.Reset();
                iPointerEventArray.Append(aPointerEvent);
            }
            else if (aPointerEvent.iType == TPointerEvent::EButton1Up)
            {
                iDragStateUpTime.HomeTime();
				
                //Move to EStateIntermediate4 this is to avoid accidental up-down-up-down-drag ...events
                iState = EStateIntermediate4;
                iAccidentalUpTimer->Cancel();
                iAccidentalUpTimer->After(KAccidentalUpTimer);

                iPointerEventArray.Reset();
                iPointerEventArray.Append(aPointerEvent);
            }
            else if (aPointerEvent.iType == TPointerEvent::EDrag)
            {
                //No state change;
                iPointerEventArray.Reset(); //Stack should be empty.
                iGestureObserver->HandleGestureL(EGestureDrag, aPointerEvent);
            }            
            break;
        }
        
        case EStatELongTap:
        {
            #ifdef _GestureDebug_
            RDebug::Printf(" CGestureDetector::HandlePointerEventL, state = EStatELongTap");
            #endif
            if(aPointerEvent.iType == TPointerEvent::EButton1Down)
            {//possible when we are in long tap mode and drag outside flash area            
                iState = EStateIntermediate1;  
                iPointerEventArray.Reset();
                iPointerEventArray.Append(aPointerEvent);
            }
            else if (aPointerEvent.iType == TPointerEvent::EButton1Up)
            {
                iState = EStateWaiting;
                iPointerEventArray.Reset(); //Stack should be empty.            
                iGestureObserver->HandleGestureL(EGestureLongTap, aPointerEvent);
            }
            else if (aPointerEvent.iType == TPointerEvent::EDrag)
            {
                //No state change;
                iPointerEventArray.Reset(); //Stack should be empty.
                iGestureObserver->HandleGestureL(EGestureLongTap, aPointerEvent);
            }            
            break;
        } 
        case EStateIntermediate4:
        {
            #ifdef _GestureDebug_
            RDebug::Printf(" CGestureDetector::HandlePointerEventL, state = EStateIntermediate4");
            #endif
            if(aPointerEvent.iType == TPointerEvent::EButton1Down)
            {            
                TTime now;
                now.HomeTime();
                iAccidentalUpTimer->Cancel();
                
                //Accidental up+down move to Estate Drag
                if(now.MicroSecondsFrom(iDragStateUpTime)<KAccidentalUpTimer)
                {
                    iPointerEventArray.Reset();
                    iState=EStateDrag;
                }
                else
                {
                    iState = EStateIntermediate1;
					
                	TPointerEvent tempPtrEvt = iPointerEventArray[0];
                    iPointerEventArray.Reset();
                    iPointerEventArray.Append(aPointerEvent);
					
                    iGestureObserver->HandleGestureL(EGestureDrag, tempPtrEvt);
                }
            }
            else if (aPointerEvent.iType == TPointerEvent::EButton1Up)
            {
                //Not Possible
            }
            else if (aPointerEvent.iType == TPointerEvent::EDrag)
            {
                //Not Possible
            }            
            break;
        }
}
    #ifdef _GestureDebug_
    RDebug::Printf("CGestureDetector::HandlePointerEventL--");
    #endif
    
    iInsidePointerEventHandler = EFalse;    
}


//RunL for Double tap timer
 void CGestureDetector::DoRunL(const CActive* aActive)
{
    #ifdef _GestureDebug_
    RDebug::Printf("CGestureDetector::DoRunL ++ iState = %d",iState);
    #endif

    if(aActive == iDoubleTapTimer)
    {    
        if(iState == EStateIntermediate3)
        {
            iState = EStateIntermediate1;

        	TPointerEvent tempPtrEvt = iPointerEventArray[0];
            iPointerEventArray.Remove(0);
            iPointerEventArray.Remove(0);
            iGestureObserver->HandleGestureL(EGestureTap,tempPtrEvt);
        }
        else if(iState == EStateIntermediate2)
        {
            iState = EStateWaiting;
			
        	TPointerEvent tempPtrEvt = iPointerEventArray[0];
            iPointerEventArray.Reset();
            iGestureObserver->HandleGestureL(EGestureTap,tempPtrEvt);
        }
        else
        {
            #ifdef _GestureDebug_
            RDebug::Printf("CGestureDetector::DoRunL State unexpected %d",iState);    
            User::Panic(_L("CGestureDetector::DoRunL State unexpected"),KErrGeneral);
            #endif//  _GestureDebug_
        }
    }
    else if(iAccidentalUpTimer == aActive)
    {
        if(iState == EStateIntermediate4)
        {    
            iState = EStateWaiting;

        	TPointerEvent tempPtrEvt = iPointerEventArray[0];
            iPointerEventArray.Reset(); 
            iGestureObserver->HandleGestureL(EGestureDrag, tempPtrEvt);
		}
        else
        {
            #ifdef _GestureDebug_
            RDebug::Printf("CGestureDetector::DoRunL State unexpected %d",iState);
            #endif//  _GestureDebug_
        }
    }
    
    #ifdef _GestureDebug_    
    RDebug::Printf("CGestureDetector::DoRunL --");  
    #endif//  _GestureDebug_
}
 
TInt CGestureDetector::DoRunError(const CActive* /*aActive*/, TInt aError)
{
     return aError;
}

CGestureDetector::~CGestureDetector()
{
    if(iFlashLongTapDetector)
    {
        delete iFlashLongTapDetector;
        iFlashLongTapDetector = NULL;
    }
    
    if(iDoubleTapTimer)
    {
        iDoubleTapTimer->Cancel();
        delete iDoubleTapTimer;
        iDoubleTapTimer = NULL;
    }
    if(iAccidentalUpTimer)
    {
        iAccidentalUpTimer->Cancel();
        delete iAccidentalUpTimer;
        iAccidentalUpTimer =NULL;
    }
    
    iPointerEventArray.Reset();
    iPointerEventArray.Close();
}


