/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
*
*/
 

#include <eikenv.h>
#include <bautils.h> // NearestLanguage
#include <data_caging_path_literals.hrh>
#include <featmgr.h>

#include "saplayer_build.h"


#include "uimain.h"
#include "flashplugincontrolfactory.h"

void NPPDebug(unsigned char, class TRefByValue<class TDesC16 const >, ...)
{

}

EXPORT_C MPluginView* CreatePluginUI(MFlashPluginInstance* aFlashPluginInstance, TWindowMode aWindowMode)
{
    MPluginView* ui = NULL;
    TRAP_IGNORE(ui=CFlashPluginControlFactory::GetInstanceL(aFlashPluginInstance,aWindowMode));
    return ui;
    
}

HBufC8* GetBaseURL8(const TDesC& aBaseUrl)
    {

    if (aBaseUrl.Length() <= 0)
        return NULL;

    HBufC8* baseURL8 = NULL;
    TRAP_IGNORE(baseURL8 = HBufC8::NewL(aBaseUrl.Length() + 1));

    if (baseURL8)
        {
        TPtr8 ptr8 = baseURL8->Des();
        ptr8.Copy(aBaseUrl);
        ptr8.ZeroTerminate();
        }
    
    return baseURL8;
    }

int load_pluginresourceL(TBufC<128>& aFileName)
{
    TInt resOffset = -1;
    
    TBuf16<128> aResourcePath( _L("\\resource\\apps"));
    // strip the extension
    TInt fileNameLength = aFileName.Length();
    TFileName resourceFileName;
    
    RFs fileServerSession = CCoeEnv::Static()->FsSession();
    
    if( fileNameLength > 4 )
    {
        // we want just the name 
        // ex, if aFileName is flash2ui.rsc,
        // fName will be flash2ui
        TBuf16<128> fName = aFileName.Mid( 0, fileNameLength - 4 );
        
        TBuf16<512> variantFile;
        _LIT16( KFormat, ":%S\\%S.r%S%d");
    
        // search for the resource file based on the current language
        TInt languageID = User::Language();
        TBuf16<8> leadingZero;
        leadingZero.Copy( _L("") );
        if( languageID < 10 )
            leadingZero.Copy( _L("0") );
            
    
        // start from Y: go to A: and then finally search Z:
        for( TInt i=EDriveY; i>=EDriveA; i-- )
        {
            //TChar driveLetter = 'a'; // +i;
            TChar driveLetter;
            fileServerSession.DriveToChar( i, driveLetter );
            variantFile.Append(driveLetter);
            variantFile.AppendFormat(KFormat,  &aResourcePath, &fName, &leadingZero, languageID );
            
            resourceFileName.Copy( variantFile );
            NPPDebug( LOG_DEBUG, _L("Trying: %S\n"), &resourceFileName );
            TBool fileExists = BaflUtils::FileExists( CCoeEnv::Static()->FsSession(), resourceFileName );
            if( fileExists )
            {
                resOffset = CCoeEnv::Static()->AddResourceFileL(resourceFileName);
                NPPDebug( LOG_DEBUG, _L("resource : %S"), &resourceFileName );
                return resOffset;
            }
            
        }
        // see if it is present on z drive
        TChar driveLetterZ;
        fileServerSession.DriveToChar( EDriveZ, driveLetterZ );
        variantFile.Append(driveLetterZ);
        variantFile.AppendFormat(KFormat,  &aResourcePath, &fName, &leadingZero, languageID );
        
        resourceFileName.Copy( variantFile );
        NPPDebug( LOG_DEBUG, _L("Trying: %S\n"), &resourceFileName );
        TBool fileExists = BaflUtils::FileExists( CCoeEnv::Static()->FsSession(), resourceFileName );
        if( fileExists )
        {
            resOffset = CCoeEnv::Static()->AddResourceFileL(resourceFileName);
            NPPDebug( LOG_DEBUG, _L("resource : %S"), &resourceFileName );
            return resOffset;
        }
        

        // if exact language is not found, search for the nearest language in the respective drive
        // start from Y: go to A: and then finally search Z:
        _LIT16( KNearestLangFormat, ":%S\\%S");
        for( TInt i=EDriveY; i>=EDriveA; i-- )
        {
            TChar driveLetter;
            fileServerSession.DriveToChar( i, driveLetter );
            //TBuf16<128> KFormat( _L("%c:%S\\%S"));
            variantFile.Append(driveLetter);
            variantFile.AppendFormat(KNearestLangFormat,  &aResourcePath, &aFileName );    // use the filename that was passed in
        
            resourceFileName.Copy( variantFile );
            BaflUtils::NearestLanguageFile( CCoeEnv::Static()->FsSession(), resourceFileName );
            NPPDebug( LOG_DEBUG, _L("Trying: %S\n"), &resourceFileName );
            TBool fileExists = BaflUtils::FileExists( CCoeEnv::Static()->FsSession(), resourceFileName );
            if( fileExists )
            {
                resOffset = CCoeEnv::Static()->AddResourceFileL(resourceFileName);
                NPPDebug( LOG_DEBUG, _L("resource : %S"), &resourceFileName );
                return resOffset;
            }
            
        }
        
        // see if it is present on z drive
        // TBuf16<128> KZFormat( _L("z:%S\\%S"));
        variantFile.Append(driveLetterZ);
        variantFile.AppendFormat(KNearestLangFormat,  &aResourcePath, &aFileName );
        resourceFileName.Copy( variantFile );
        BaflUtils::NearestLanguageFile( CCoeEnv::Static()->FsSession(), resourceFileName );
        NPPDebug( LOG_DEBUG, _L("Trying: %S\n"), &resourceFileName );
        fileExists = BaflUtils::FileExists( CCoeEnv::Static()->FsSession(), resourceFileName );
        if( fileExists )
        {
            resOffset = CCoeEnv::Static()->AddResourceFileL(resourceFileName);
            NPPDebug( LOG_DEBUG, _L("resource : %S\n"), &resourceFileName );
            return resOffset;
        }
        
    }
    
    return resOffset;   // -1
}
