/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash Methods Handler class 
*
*/
 


#ifndef __FLASH_METHODS_H__

#define __FLASH_METHODS_H__

#include "saplayer_build.h"
#include <charconv.h>

#include "commontypes.h"
#include "npheaders.h"
#include "MFlashMethods.h"

#if defined(__SYMBIAN32__) && !defined(EPOC)
#define EPOC
#endif



class MExternalInterface;
class MFlashPluginIf;
class CCnvCharacterSetConverter;
class CGlueGlobals;

class CFlashMethods : public CBase
{
	public: 
      	static CFlashMethods* NewL(MFlashPluginIf* aBrowserInstance 
      							,MFlashMethods* aFlashMethods
      							,MExternalInterface* aExternalInterface
      							,TPageAccessType aAccessType = EPageUrlNormal);

        virtual ~CFlashMethods();	
    
    public: //Flash Properties
    
    void	GetPropBaseURL(NPVariant *result);
    
    void	GetPropAllowScriptAccess(NPVariant *result);
    
    void 	GetPropBGColorL(NPVariant *result);
    
    void	GetPropQuality(NPVariant *result);
    
    void	GetPropLoop(NPVariant *result);
    
    void	SetPropBaseURL(NPVariant *result);
    
    void	SetPropAllowScriptAccessL(NPVariant *result);
    
    void	SetPropAllowScriptAccessL(TPageAccessType aPageAccessType)
    	{
    	if(iFlashMethods)
    		{
    		iFlashMethods->SetPropAllowScriptAccessL( aPageAccessType );	
    		}
    	
    	}
    
    void 	SetPropBGColorL(NPVariant *result);
    
    void	SetPropQualityL(NPVariant *result);
    
    void	SetPropLoopL(NPVariant *result);
    
    
    public: //Flash methods
    bool GetVariableL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
						
	bool GotoFrame(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
						
	bool IsPlaying(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
						
	bool LoadMovieL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
						
	bool Pan(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool PercentLoaded(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool Play(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool Rewind(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool SetVariableL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool SetZoomRect(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool StopPlay(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool TotalFrames(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool Zoom(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	//TellTarget Methods
	bool TCallFrameL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool TCallLabelL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool TCurrentFrameL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool TCurrentLabelL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool TGetPropertyL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool TGetPropertyAsNumberL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool TGotoFrameL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool TGotoLabelL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool TPlayL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool TSetPropertyL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
	
	bool TStopPlayL(	const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
    
    bool CallMethodL( const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
						
	bool SetReturnValueL(const NPVariant *args, 
						uint32_t argCount, 
						NPVariant *result);
    
    private: //methods
    
        CFlashMethods(MFlashPluginIf* aBrowserInstance 
        			,MFlashMethods* aFlashMethods
        			,MExternalInterface* aExternalInterface
        			,TPageAccessType aAccessType);
        
        void ConstructL();
        
        HBufC* CreateEpocString( const char* aStr
        						, int aLength );
        						
        void CreateCharacterConverterL();
        
        TInt GetIntFromDesc(const TDesC& aDesc);
        
        char* CreateFlashString( const TDesC& aStr );
        
        char* CreateFlashString ( const TDesC8& des );
        
        bool IsTSetPropertySupported(TInt32 aPropIdx);
    
    private: //attributes
    
    	MFlashPluginIf* iBrowserInstance;
    	
    	MFlashMethods* iFlashMethods;
    	
    	MExternalInterface* iExternalInterface;
    	
    	//NPIdentifier* iFlashJavaScriptPropertyIdentifiers;
    	
    	CCnvCharacterSetConverter*		iCharacterConverter;
    	
    	TPageAccessType iAccessType;
    	CGlueGlobals*	iGlobals;
};



#endif //__FLASH_METHODS_H__