/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  MFlashPlayerControl header file
*
*/



#ifndef __MFLASH_PLAYER_CONTROL_H
#define __MFLASH_PLAYER_CONTROL_H

//INCLUDE headers
#include "NPNFlashExtension.h"

//FORWARD DECLARATIONS


/**
*   Flash player control
*
*  @lib 
*  @since Series 60 3.X
*/

class MFlashPlayerControl
{
	public:
	
	//destructor
	virtual ~MFlashPlayerControl()
	{			
	}

public:
	
	/**
	* Checks whether the given file is corrupt or not 
	*/ 
	virtual TInt CheckFileValidityL(const TDesC& aFilename) = 0;
	
	virtual void Start() =0;
	
	virtual void End()	 =0;
	
	/**
    * Stops video playback.
    * @since Series 60 3.1
    */
    virtual void Stop() =0;
     /**
    * Pauses video playback.
    * @since Series 60 3.1
    * @return KErrNone is successful.
    */
    virtual void Pause(TInt aReason)=0;
    
    /**
    * Resumes video playback.
    * @since Series 60 3.1
    */
    virtual void Resume(TInt aReason, TBool aIgnoreActiveView)=0;
    
    /**
    * Rewind video playback.
    * @since Series 60 3.1
    */
    virtual void Rewind() = 0;
    
    /**
 		Sets the volume.
 		@param The value of the volume.
	*/
	virtual void SetVolumeToEngine(TInt aVolume) = 0;

	virtual void MuteVolume() =0;
	
	virtual void UnMuteVolume() = 0;
    /**
    * Get the state of the flash player.
    * @since Series 60 3.1
    * @return The status of video player
    */
    virtual NPFlashPlayerState State() const=0 ;

	/**
	Gets the version information for the FlashLite player library.
	@return The version information.
	*/
	
	virtual HBufC* GetFlashVersion() = 0;

	/**
	This function is called to Zoom in or Zoom out the swf file.
	@param aMode Specifies Zoom in, Zoom out or Show all mode.
	*/
	virtual TBool Zoom(TInt aMode) = 0;

	virtual TBool IsSoundPlaying() const = 0;
	
	virtual TInt GetLastError()const =0;
	
	virtual TInt GetCurrentFrameNo() const =0;
	
	virtual TInt GetTotalFrames() const = 0;
				
};

#endif //End of file