/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  flash plugin header file
*
*/


#ifndef __FLASHPLUGIN_H
#define __FLASHPLUGIN_H

#include <e32std.h>
#include <DRMHelper.h>
#include <caf/content.h>

using namespace ContentAccess;

_LIT(KLitFile,"file://");
_LIT(KLitHttp,"http://");
_LIT(KLitHttps,"https://");

//FORWARD DECLARATIONS
class CVolumeKeyPropertyWatch;

/**
 * Note: Need to ensure that EPOC is defined before including these headers...
 */
#if defined(__SYMBIAN32__) && !defined(EPOC)
#define EPOC
#endif


#include "flashplugininstance.h"
#include "flashplgmain.h"

//Forward declaration

class FlashString;
class CAsyncDeleter;

class CPluginInstanceInfo : public CBase
{
public:
    static CPluginInstanceInfo* NewL();
    ~CPluginInstanceInfo();

    void AddArgsL(CDesCArray* aArgN, CDesCArray* aArgV);
    void AddInitStreamL();
    void AddFileNameL(const TDesC& aFileName);
    void AddUrlL(const TDesC& aUrl);

    CDesCArray* ArgumentNames();
    CDesCArray* ArgumentValues();
 
    TDesC* FileName();
    TDesC* Url();

private:
    CPluginInstanceInfo();
    void ConstructL();
    
private:
    CDesCArrayFlat*     iArgNameArray;
    CDesCArrayFlat*     iArgVectArray;
    HBufC*              iFileName;
    HBufC*              iUrl;
};

class CBrowserPluginInstance : public CFlashPluginInstance,
							   public MPlgAsyncHandler
{
public:
    
    static CBrowserPluginInstance* NewL(MNetscapeIf* aNetscapeIf,CPluginGlobals* aGlobals 
                                         ,uint16 aMode
                                         ,TBool aInteractionMode
#ifdef FEATURE_WINDOWLESS                                         
                                         ,TWindowMode aWindowMode /*= EWindow*/
#endif //FEATURE_WINDOWLESS
                                         ,TPluginUiFramework aPluginUiFramework

							);

                                 
    CBrowserPluginInstance(MNetscapeIf* aNetscapeIf,CPluginGlobals* aGlobals 
                            ,uint16 aMode
                            ,TBool aInteractionMode
#ifdef FEATURE_WINDOWLESS                            
                            ,TWindowMode aWindowMode
#endif //FEATURE_WINDOWLESS                            
							);
                    
    virtual ~CBrowserPluginInstance();
    
    void ConstructL(TPluginUiFramework aPluginUiFramework);

public: //CFlashPluginInstance
    TBool QueryAllowNetworkAccess();
    virtual void PrepareToExitL();
    virtual HBufC* GetFlashBaseURL();
    virtual void OpenFileL(const TDesC& aFilename, 
						   const TDesC& aUrl, 
                           const TBool& aFirstMovie);
    
    TBool PerformDrmChecksL(const TDesC& aFilename, const CContent* aContent,
                            TBool& aSwfWithAdequateRights);
    
   	virtual NPError HandleGetCommandL(int aVariable, void* ret_value);
  	virtual NPError HandleSetCommandL(int aVariable, void* ret_value);
  	
  	//Moved to CFlashPluginInstance::ProcessParameters
  	//virtual void    ProcessParameters (CDesCArray* argnames, CDesCArray* argvalues);
  	virtual CDRMHelper* GetDRMHelperL();
  	
  	CDesCArray* ArgumentNames();
    CDesCArray* ArgumentValues();
    
    #ifdef FEATURE_WEB_BROWSABILITY
	virtual MHostScriptHandler* GetHostScriptHandler();
	TBool PercentLoaded(TReal& aPercentLoaded);
	/*HBufC*  GetBaseURL()
	{
		return 	iBaseURL;
	}*/

	virtual TPageAccessType GetScriptAccess()
		{
		return iPageAccessType;
		}
	
	virtual void SetScriptAccess( TPageAccessType aPageAccessType)
		{
		iPageAccessType = aPageAccessType;
		}

	#endif
public: //MFlashClientIf
    virtual TUint GetFeatureCapability();
    
    /* For restarting the swf after swf is trusted in widget mode*/
    virtual void KickoffRestartTimerL();    
    
public: //MPluginStreamIf
    virtual TBool OpenUrlL(const TDesC& aUrl, const TDesC& aPostData);
    
    virtual TBool OpenUrlL(const TDesC& aUrl, const TDesC& aTarget, const TDesC& aPostData);

    virtual TBool OpenStreamL(unsigned short aStreamId, const TDesC& aUrl,
            const TDesC& /*aPostDataContentType*/,
            TInt /*aPostDataContentLength*/, const TDesC& aPostData);

    virtual void SetStreamBufferSizeL(unsigned short streamId,
            unsigned long aBufferSize);

    virtual void CancelStream(unsigned short streamId);
    
    NPError FillPostDataL(const TDesC& aPostDataContentType,
                                                 TInt         aPostDataContentLength,
                                                 const TDesC& aPostData, 
                                                 HBufC*&      aHttpPostdata);
    
    NPError FillPostDataL(const TDesC& aPostData,HBufC*& aHttpPostdata);
    TBool CanContentBeTrustedL(const TDesC& pAccessedDomain);

private:
	// RunL of timer object
	virtual void DoRunL(const CActive* /*aActive*/);
	virtual TInt DoRunError(const CActive* /*aActive*/, TInt aError);
	
protected:
    TBool IsFocused();
    void  HandleFocusLostL();
    //void  CreateControlEngineL();    
    TBool IsRelativeBaseUrl();
    TInt ReduceURL(HBufC8*& aCompleteURL8);
    void TruncateURL(TDes8& aURLPtr);
    void  CreateBaseURLL(const TDesC& aSwfSrcURL);

private:
	void FindExecutionMode();
	void QueryAllowNetworkInWidgetMode();
	//TBool     iAllowNetworkAccess;    

private:

    // This array contains all the file names passed to NPP_StreamAsFile()
    CDesCArray*         iSavedFileNames;
    CAsyncDeleter*      iAsyncDeleter;
	CDRMHelper*         iDrmHelper;    

	#ifdef FEATURE_WEB_BROWSABILITY    
	TPageAccessType iPageAccessType;
	#endif // FEATURE_WEB_BROWSABILITY

	CPluginInstanceInfo*    iPluginInfo; 
	TBool                   iErrorOccured;
	TBool 					iCallOpenFile;
	TBool					iPausedDueToFocusLost;
	TRect					iClipRect;	 
	TBool 					iIsRelativeBaseUrl;
	CPlgTimerObject*        iRestartTimer;	
	TBool                   iAllowNetworkAccess;    
};

class MPluginView;
class CAsyncDeleter : public CActive
{
public:
    CAsyncDeleter(CBrowserPluginInstance* aPlayer, 
                  //CBrowserPluginControl* aControl,
                  MPluginView* aPluginView);
                  
    ~CAsyncDeleter();
    
public:
    void RunL();
    void DoCancel();
    void IssueRequest();
    
private:
    CBrowserPluginInstance* iPlayer;
    MPluginView* iPluginView;
};

#endif // __FLASHPLGPLUGIN_H

// End of File
