/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CFlashPluginInstance header file
*
*/


#ifndef __MFLASH_PLUGIN_INSTANCE_H
#define __MFLASH_PLUGIN_INSTANCE_H
#include <content.h>
#include <e32std.h>
#include <DRMHelper.h>
#include <centralrepository.h>
#include <featmgr.h>
#include <browseruisdkcrkeys.h>
#include <e32base.h>  

/**
 * Note: Need to ensure that EPOC is defined before including these headers...
 */
 
#if defined(__SYMBIAN32__) && !defined(EPOC)
#define EPOC
#endif

#include "NPNFlashExtension.h"
#include "hwkeywatch.h"
#include "mflashmethods.h"

#include "flashengineclientif.h"
#include "pluginstreamif.h"
#include "appinfoif.h"
#include "flashvideoif.h"
#include "flashplayercontrol.h"
#include "symbian_ui_interface.h"
#include "streamfactory.h"
#include "browserplugintimer_object.h"
#include "hwkeywatchobserver.h"
#include "volumecenrepkeywatcher.h"
#include "flashpluginif.h"
#include "commontypes.h"
#include "pluginmemhandlerimpl.h"
using namespace ContentAccess;
class CHwKeyPropertyWatch;
class CPluginGlobals;
class CDeviceExtn;
//class MFlashClientAppInfo;
#include "pluginview.h"

//class NPNetscapeFuncs;
class MPluginStreamHandler;
class CPluginStreamFactory;
class MFlashEngine;
#include "flash_iad_update_callback.h"

class MFlashPluginInstance;


class CFlashPluginInstance : public CBase 
                            ,public MFlashPluginInstance
                            ,public MFlashPluginIf
                            ,public MFlashClientIf
                            ,public MPluginStreamIf
                            ,public MFlashClientAppInfo
                            ,public MFlashPlayerControl
                            ,public MHwKeyWatchObserver
#ifdef SELF_IAD_UPDATE
                            ,public MIadUpdateCallback
#endif   
{
 public:
    
    CFlashPluginInstance(MNetscapeIf* iNetscapeIf,CPluginGlobals* aGlobals 
    					 ,uint16 aMode
    					 ,TInt aInteractionMode
#ifdef FEATURE_WINDOWLESS    					 
    					 ,TWindowMode aWindowMode=EWindow
#endif //FEATURE_WINDOWLESS    					 
						 );
    										
    virtual ~CFlashPluginInstance();

    virtual void Close() { delete this; } //part of MFlashPluginIf; called by plugin wrapper

    //to be overridden by concrete plugin instances like browser plugin instance.
	//in case if the concrete plugin instances dont support scriptability, do not 
	//override; leave the base implementation as it is to indicate that scriptability
	//is not supported
    virtual MHostScriptHandler* GetHostScriptHandler() { return NULL; }
protected:
    MPluginView* CreatePluginView(TPluginUiFramework aPluginUiFramework);
public:
    virtual CPluginStreamFactory* GetStreamHandler();    
    virtual void DestroyStreamHandler();
    
    virtual void PrepareToExit() { TRAP_IGNORE(this->PrepareToExitL()); }
    virtual HBufC* GetFlashBaseURL(){return NULL;};
    virtual int SetWindow(NetscapeWindow* window);    
    virtual int NewStream(const TDesC8& aType, NetscapeStream* aStream, bool seekable,
                unsigned short* stype);
    virtual int DestroyStream(NetscapeStream* aStream, NetscapeReason aReason);
    virtual int AsFile(NetscapeStream* aStream, const TDesC& aFname);
    virtual int WriteReady(NetscapeStream* aStream);
    virtual int WriteStream(NetscapeStream* aStream, int aOffset, int aLen,
                void* aBuffer);
    virtual int UrlNotify(NetscapeReason aReason, void* notifyData);
    virtual int GetValue(NetscapeVariable variable, void* ret_value);
    virtual int SetValue(NetscapeVariable variable, void* ret_value);
    virtual MNetscapeIf* NetscapeFns();
    virtual MVolumePropertyWatchObserver* GetMHwKeyWatchObserver();
    virtual int AddHwKeyObserver(MHwKeyWatchObserver*) { return KErrNone;}
    virtual void RemoveHwKeyObserver(MHwKeyWatchObserver*) {}
    virtual MFlashPlayerControl* GetFlashPlayerControl()  { return (MFlashPlayerControl*)this;}
    virtual CHwKeyPropertyWatch* GetHwKeyPropertyWatch() { return NULL; }
    virtual MFlashEngine* GetEngine();
    virtual TBool QueryAllowNetworkAccess(){return EFalse;}
    TBool IsWidgetMode(){ return (iBrPluginExecutionMode == EWidgetMode)? ETrue: EFalse;}
    
public:
    virtual void StartShutDownProcess();
    
protected:
	CPluginStreamFactory* iStreamFactory;
    
protected:
 
    //following constrution and destruction phase functions are intentionally
    //virtual to help derived ones override when the need arises
    virtual void DestroyEngine();
    virtual void CreateEngineL();
    
public: //from MFlashClientIf
	
    virtual TWindowMode GetWMode() { return iWindowMode; }
    
    virtual TUint GetFeatureCapability();
      
    virtual TInt GetPersistentDirectory(const TDesC& aSwfPath, TFileName& aPersistentDir);    
    
    virtual void KickoffRestartTimerL() {}
    
    TBool AllowAMCExtension() {return (iBrPluginExecutionMode == EWidgetMode)? ETrue: EFalse;}
    
public: //from MFlashClientAppInfo
    virtual const TFileName GetAppPath() { TFileName temp; return temp; }
    
public: //from MFlashVideoClientIf
    //virtual void  HandleVideoError(const TInt& aError);
    //virtual void FLVideoStarted();

public: //from MFlashClientIf
    
    virtual MFlashClientAppInfo* GetClientAppInfoIf() { return this; }
    virtual MDisplay* GetDisplay() 
    {       
    	if (iPluginView)
    		return iPluginView;
    	return NULL; 
    }
    
    virtual MPluginMemHandlerIf* GetPluginMemHdlrIf()
    {
        return iPluginMemHandler;
    }
    virtual MUserNotiferIf* GetUserNotifierIf()
    {
    	if (iPluginView)
                return iPluginView;
    	return NULL;  
	}
    virtual MPluginStreamIf* GetStreamIf() {return this;}
     
    virtual MDeviceExtnIf* GetDeviceIf();  //{return iDeviceExtn;}
    virtual MMp3StateObserver* GetMp3StateObserver() 
    {
    	if (iPluginView)
    		return iPluginView->GetMP3StateObserver();
    	return NULL;  
    }
     
    virtual MFlashCursor* GetCursor() { return NULL; }
    virtual MInputTxtIf* GetInputTextIf()
    {
        return iPluginView;
	}
     
    virtual MFlashVideoClientIf* GetFlvClientIf()
    {
    	if (iPluginView)
    		return iPluginView->GetFlvClientIf();
        
        return NULL;
    }

#ifdef SELF_IAD_UPDATE     
virtual MIadUpdateCallback* GetIADUpdateCallback()
{
#ifdef FEATURE_WINDOWLESS    	
	 if(iWindowMode == 0)
	 {
#endif //FEATURE_WINDOWLESS
         return this;
#ifdef FEATURE_WINDOWLESS
	 }
	 else //1,2 indicates windowless mode
		 return (NULL);
#endif //FEATURE_WINDOWLESS    	 
/*    		 
    	
#ifdef FEATURE_WINDOWLESS    	
    	 if(iWindowMode == 0)
    	 {
#endif //FEATURE_WINDOWLESS
    		 if(iPluginView)
    			 return (MIadUpdateCallback*)iPluginView;
    		 else
    			 return (NULL);
#ifdef FEATURE_WINDOWLESS
    	 }
    	 else //1,2 indicates windowless mode
    		 return (NULL);
#endif //FEATURE_WINDOWLESS
*/    	 
}
#endif //SELF_IAD_UPDATE
    
    virtual MAccessPointIf* GetIAPIf() { return NULL; } 
         
    //virtual EEngineExecutionMode GetExecutionMode() {return ePlugin;}

    virtual void DrawDirtyRect(const TRect& aRect);
    virtual void AddEventObserverToControl();
//	{
//	}

    virtual void RemoveEventObserverFromControl();
//	{
//	}

    virtual void NotifyEngineState(TEngineState /*aState*/)
	{
	}

    virtual void NotifyShutdownCountdown(TInt /*aItemsRemaining*/)
	{
	}

    virtual RFile* GetRFileHandleForPlay() { return NULL; }

    virtual void NotifyPhoneActiveL(TBool /*aIsActive*/) {}
    virtual void ClearErrors() {}
    virtual TBool  IsUrlDescSet() { return EFalse; }
    
    virtual const TPtr GetUrlDesc() { return  TPtr(0,0,0);}
   
    virtual TBool CanContentBeTrustedL(const TDesC& /*pAccessedDomain*/) { return EFalse; }

    virtual void Quit() {}
    
public:
    MFlashEngine* GetEngine() const { return iEngine; }
        
public: //from MFlashPlayerControl
    virtual TInt CheckFileValidityL(const TDesC& aFilename);
    virtual void Start();
    virtual void End();
    virtual void Stop();
    virtual void Pause(TInt aReason);
    virtual void Resume(TInt aReason, TBool aIgnoreActiveView);
    virtual void Rewind();
    virtual void SetVolumeToEngine(TInt aVolume);
    virtual TInt GetCurrentEngineVolume();
    virtual void MuteVolume();
    virtual void UnMuteVolume();
    virtual NPFlashPlayerState State() const;
    virtual HBufC* GetFlashVersion();
    virtual TBool Zoom(TInt aMode);
    virtual TBool IsSoundPlaying() const;    
    virtual TInt GetLastError() const;    
    virtual TInt GetCurrentFrameNo() const;    
    virtual TInt GetTotalFrames() const;
    
public:    
    virtual void MHandleVolumeCmdL(TInt aNewValue);
    virtual void MVolumeChanged(TInt aNewValue); 
    
    virtual void MPausePlayKeyPressed();
	virtual void MStopKeyPressed() ;
	virtual void MRewindKeyPressed();
	virtual void MForwardKeyPressed();
	virtual void MBackwardKeyPressed();
	virtual void MFastForwardKeyPressed();
	TBool IsVolumeKeySupported();

	
public: //CFlashPluginInstance methods
   
   	//virtual MPluginStreamHandler* GetStreamHandler() ; 
   	
    virtual void PrepareToExitL();

	virtual int HandlePluginEvent(void* event);

	virtual void ProcessParameters(CDesCArray* aArgnames, CDesCArray* aArgvalues);

    virtual void OpenFileL(const TDesC& aFilename, 
                           const TDesC& aUrl, 
                           const TBool& aFirstMovie);

    virtual TBool IsControlValid() ;
       
	virtual TBool PerformDrmChecksL(const TDesC& aFilename,	const CContent* aContent,  
                                    TBool& aSwfWithAdequateRights) ;

	static TBool isFlashContentL(const CContent* aContent) ;
	
	void SetSwfSourcePath(const TDesC& aSwfSrcPath);
	
	virtual const TDesC& GetBaseURL();
	virtual const TDesC& GetSecurityContextURL();
#ifdef __OLD_CODE__	
	virtual EPageAccessType GetScriptAccess()
	{
		return EPageUrlNormal;
	}

	virtual void SetScriptAccess( EPageAccessType /*aPageAccessType*/) { }
#endif // __OLD_CODE__
	//TODO : need to handle this
	virtual TBool PercentLoaded(TReal& /*aPercentLoaded*/);
	
	virtual int InteractionMode() { return GetInteractionMode(); }
	
	TInt GetInteractionMode();
	
	virtual EBrPluginExecutionMode GetBrPluginExecutionMode();
	
	virtual CDRMHelper* GetDRMHelperL();
	
	CPluginGlobals* GetGlobals();
		
	virtual NPFlashConfig* GetFlashconfig();
		
	virtual NPError HandleSetCommandL(int variable, 
                                      void* ret_value);
                
    virtual NPError HandleGetCommandL(int variable, 
                                      void* ret_value);
#ifdef __OLD_CODE__                                      
    virtual CDesCArray* ArgumentNames();
    virtual CDesCArray* ArgumentValues();
#endif // __OLD_CODE__    
    virtual void SetBaseUrlL(const TDesC& aBaseUrl);
    
    static HBufC8* GetBaseURL8(const TDesC& aBaseUrl);
    
    void SetAllowMobiFullScreen(TBool aAllowMobiFullScreen);
    
    TBool GetAllowMobiFullScreen();
    
/*    #ifdef GESTURE_LIBRARY_SUPPORTED
    virtual TBool HandleGestureFromBrowserL(void* aEvent);
    #endif //GESTURE_LIBRARY_SUPPORTED
    */// This function is used to open a swf file. Fix for ASHA-7RKHZT.
    
    int OpenMovie();
    
    virtual void ReAdjustMaxHeapforVideoStream(TUint aMaxHeapSize);
    
#ifdef SELF_IAD_UPDATE
public://MIadUpdateCallback
	virtual void IADUpdateCallback( TInt /*aErrorCode*/ );
	virtual void SetIADProgressState(TIadProgressState aState);
	TIadProgressState GetIADProgressState();
#endif // SELF_IAD_UPDATE

public: //MPluginStreamIf
    virtual TBool OpenUrlL(const TDesC& aUrl, const TDesC& aPostData);
    
    virtual TBool OpenUrlL(const TDesC& aUrl, const TDesC& aTarget, const TDesC& aPostData);

    virtual TBool OpenStreamL(unsigned short aStreamId, const TDesC& aUrl,
							  const TDesC& /*aPostDataContentType*/,
							  TInt /*aPostDataContentLength*/, 
							  const TDesC& aPostData);

    virtual void SetStreamBufferSizeL(unsigned short streamId,
            unsigned long aBufferSize);

    virtual void CancelStream(unsigned short streamId);

public: 
    TBool IsPlayerInitialzed();
    TBool DeferredLaunch();
    
public:
	void InitializeVolumeObserverL();
    void AddHwSideKeyObserverL();
    void RemoveHwSideKeyObserver();
	
	void SetInitialVolumeToEngine();
	CVolumeCenRepKeyWatcher* GetVolumeCenRepKeyWatcher();

protected:
	TBool CheckPlayRightsL(const TDesC& aFilename);
	TBool CheckPlayRightsL(const CContent* aContent);
    void  CacheFileNameL(const TDesC& aFilename, 
                         const TDesC* aUrl, 
                         const TBool& aFirstMovie);
    
private:    
    void GetDefaultPersistentPath(TDes& aPersistentDir);    
    void ReversePathDelimiter(TDes& symbianStylePathPtr);    
    HBufC* GetWidgetNameSpace(const TDesC& aSwfPath);
    //TInt GetDefaultVolume();
    void ExtendConfigforWidgetMode();
    void DoOpenMovieL();
    MPluginView* DoCreatePluginViewL(TPluginUiFramework aPluginUiFramework);
    MNetscapeIf* iNetScapeIf;

protected:
    FlashEngineConfig    		iConfig;
    RLibrary                	iFELib;
    
    CDeviceExtn* 				iDeviceExtn;
	CPluginGlobals*         	iGlobals;
	TInt                    	iInteractionMode;
	EBrPluginExecutionMode 		iBrPluginExecutionMode;
	uint16                  	iMode;
    MPluginView*        iPluginView;
	HBufC*                  	iCachedFileName;
	HBufC*                  	iCachedUrl;
	TBool                   	iFirstMovie;
	TReal32 					iContentLength;	 
	TReal32						iLengthWritten;
	TPtrC                   	iSwfSrcPath;
	HBufC*                  	iBaseURL;
  HBufC*                    iSecurityContextUrl;
	TBool						iAllowMobiFullScreen;
	
	CPluginMemHandlerImpl*  	iPluginMemHandler;
	TBool 						iSideVolumeKeysSupported;
    CRepository* 				iCenRepos;
    TBool						iHwKeyObserverAdded;
    
private:
    // this boolean variable determine whether player get initialised or not.
    // As we initialise the player in StartMovieL()
    // function of symbian_engine.cpp class 
    TBool                   	iIsPlayerIntialised;
	//Indicates whether a deferred player launch is pending
    TBool                   	iDeferredLaunch;
    CVolumeCenRepKeyWatcher*    iVolumeCenRepKeyWatcher;
#ifdef SELF_IAD_UPDATE    
    TIadProgressState 			iIadProgressState;
#endif //SELF_IAD_UPDATE    
    
protected:	
    TWindowMode                 iWindowMode;
	MFlashEngine*           	iEngine;
	RFs							iFs;
    HBufC*                  	iFilename; 
    // stores the swf url
    HBufC*                  	iUrl;
    TBool                   	iFirstFlashMovie;	
	
	
    RLibrary iLibrary;
};

#endif // __MFLASH_PLUGIN_INSTANCE_H

// End of File
