/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  AO that completes on a change in the volume property.  Used for 
*                the new Central Repository Server.  Used in EKA2 builds only
*
*/


#ifndef CVOLUME_PROPERTY_WATCH
#define CVOLUME_PROPERTY_WATCH

#include <e32std.h>
#include <e32base.h>
#include <e32property.h>

//
// for emulator max volume = 65535
#define WINS_DEFAULT_VOLUME 43690

//
// For HW max volume = 9
#define HW_DEFAULT_VOLUME 6

class MVolumePropertyWatchObserver;

#include<RemConCoreApiTargetObserver.h>    //for MRemConCoreApiTargetObserver
#include<RemConInterfaceSelector.h>     //for CRemConInterfaceSelector
#include<remconcoreapitarget.h>         //for CRemConCoreApiTarget

// DATA TYPES
// Used for volume up/down events
enum TVolumeControlCommands
    {
    EVolumeControlCmdVolumeUpByOne =-100,
    EVolumeControlCmdVolumeDownByOne
    };

class CVolumeKeyPropertyWatch : public CBase, MRemConCoreApiTargetObserver
    {
public:
    
    static CVolumeKeyPropertyWatch* NewL(const TInt& aMinVolume, 
                                         const TInt& aMaxVolume);
                                         
    virtual ~CVolumeKeyPropertyWatch();
   // TInt GetCurrentVolume();
    /**
     * Function to add an observer
     */
    void AddObserverL(MVolumePropertyWatchObserver* aObserver);  
    
    TInt GetMaxVolume()  
    {
    	return iMaxVolume;	
    }
    
    TInt GetMinVolume()
    {
    	return iMinVolume;
    }

private:
    CVolumeKeyPropertyWatch(const TInt& aMinVolume, const TInt& aMaxVolume);
    void ConstructL();

    //From MRemConCoreApiTargetObserver (remconcoreapitargetobserver.h)
    void MrccatoCommand( TRemConCoreApiOperationId aOperationId, 
                         TRemConCoreApiButtonAction aButtonAct );
    
    /**
      * Handle repeat event
      */
    void HandleRepeatEvent();

    /**
    * Send play command to app UI or view.
    * @since 3.1
    */
    void FilterAndSendCommand(TInt aCommandId);
   /**
    * Callback for timer
    */
    static TInt TimerCallback(TAny* aPtr);

private:
    const TInt                  iMinVolume; 
    const TInt                  iMaxVolume;
    CRemConCoreApiTarget*       iInterfaceSelector;
    CRemConInterfaceSelector*   iSelector;
    CPeriodic*                  iTimer; // owned
    TInt                        iCommandId;

    // Stores the observers.
    CArrayFix<MVolumePropertyWatchObserver*>* iObservers;
    };

#endif

// End of File
