/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CPluginInstanceMemInfo header file
*
*/


#ifndef __PLUG_IN_INSTANCE_MEM_INFO__H__
#define __PLUG_IN_INSTANCE_MEM_INFO__H__

class CSwfMemMgr;
class CFlashPluginInstance;

/**
 * The DynamicMemBlock struct is used to keep track of blocks of memory requested by the Flash library
 * through calls to SI_Malloc() and freed by calls to SI_Free().
 * 
 */
struct DynamicMemBlock
{
	TAny* mem;
	unsigned long numBlocks;
	DynamicMemBlock* pNext;
};


class CPluginInstanceMemInfo : public CBase
{
	public:
			static CPluginInstanceMemInfo* NewL(CSwfMemMgr* aMemMgr,
												CFlashPluginInstance* aSwfInstance
												, TUint aMaxHeapSize);
    		virtual ~CPluginInstanceMemInfo();
    private:
    		CPluginInstanceMemInfo(	CSwfMemMgr* aMemMgr,
									CFlashPluginInstance* aSwfInstance
									, TUint aMaxHeapSize);
							
    		void ConstructL();   
   
    public:
    		void AddDynHeapAlloced(TUint aSize);
    		
    		void DynHeapFreed(TUint aSize);
    		
    		TAny* AllocMem(TUint16 aNumBlocks);
    		
    		void FreeMem(TAny* aMemory);
    		
    		void DestroyAllMem();
    		
    		void ReAdjustMaxHeapforVideoStream(TUint aMaxHeapSize);
    		
    		TUint GetCurrAllocSize();
    		
    		
   	private: //attribs
   			 TUint     iDynMaxHeapSize;
   			 TUint     iDynHeapSize;
   			 
   			 CSwfMemMgr* iMemMgr;
   			 
   			 CFlashPluginInstance* iSwfInstance;
   			 
   			 DynamicMemBlock* iDynamicMemBlocks;
    
};


#endif //__PLUG_IN_INSTANCE_MEM_INFO__H__

