/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Stream Handler for Flash
*
*/


#ifndef __FLASH_STREAM_HANDLER_H__
#define __FLASH_STREAM_HANDLER_H__

#include <content.h>

#if defined(__SYMBIAN32__) && !defined(EPOC)
#define EPOC
#endif

#include <caf/content.h> //CContent defs
#include "commontypes.h"
#include "flash_engine.h"

class CFlashPluginInstance;

using namespace ContentAccess;

//forward declaration

//class CBrowserPluginControl;
class MPluginView;

class CFlashPluginStreamHandler : public CBase
{
	
	public:
	/**
     * Symbian constructor.
     * const char* aContentEncoding is commented out
     * reason  given in task description : synergy task id. bh1s60#71512.
     *
     * @since S60 v3.1
     * @return Pointer to created CFlashPluginStreamHandler object
     */
    static CFlashPluginStreamHandler* NewL(CFlashPluginInstance* aPluginInstance
                                            ,MPluginView* aPluginView
    										,unsigned short aStreamId
                                            ,unsigned long aBufferSize
                                            ,NPAPIMIMEType aType
                                            ,const TDesC& aURL
                                            ,TUint32 aContentLength
                                            /*,const char* aContentEncoding*/);
   
    /**
     * Destructor.
     */
    virtual ~CFlashPluginStreamHandler();
	
	
	private:
	
	/**
     * C++ default constructor.
     *
     * @param aCtrl CBrowserPluginControl object.
     */
    CFlashPluginStreamHandler(CFlashPluginInstance* aPluginInstance
                              ,MPluginView* aPluginView
    						  ,unsigned short aStreamId
                              ,unsigned long aBufferSize
                              ,TUint32 aContentLength
                              );
    
    /**
    * By default Symbian 2nd phase constructor is private.
    * const char* aContentEncoding is commented out
    * reason  given in task description : synergy task id. bh1s60#71512.
    */
    void ConstructL(NPAPIMIMEType type,const TDesC& aURL/*,const char* aContentEncoding*/);
	
	
	public:
		TInt32 WriteStreamL(NPAPIStreamData* aStream, TInt32 aOffset,
               TInt32 aLen, void* aBuffer);
    
    	TBool operator==(unsigned short aStreamId);
    	
    	TInt32 GetStreamBufferSize();
    	
    	void SetStreamBufferSize(TInt32 aSize);
    	
    	void CloseStreamL(NPAPIReason aReason);
    	
    	void CancelStream();
    	
    private:
    
    TBool CheckSwfVersionForStream(const char* aSwfData);
    
    TBool CheckforDRMData(TInt32 aLen, void* aBuffer);
    
    TBool CheckHeaderAndWriteForDRML(TBool& isDRM);
    
    TBool isFlashContentL(const CContent* aContent) ;
    
    TBool CheckPlayRightsL(const CContent* aContent);
    
    TBool PerformDrmChecksL(const TDesC& aFilename, 
                            const CContent* aContent, 
                            TBool& aCorrectMimeAndRights);

    TBool ExecuteIntentAndWriteL(const TInt& aStreamId, 
                                 const TDesC& aFilename);
   
    
    /**
     * Sets the content length to -1 if the data is compressed, that is
     * if the data encoding is any thing other than identity, utf-8 or iso-8859-1
     * This is introduced in fix for defect ids: isource: 14192, eroom issue 1039
     * 
     * reason for code revert given in task description : synergy task id. bh1s60#71512.
     *  
     * @param the header info obtained which will contain the data encoding type
     */
    /*     
    void ModifyContentLengthIfCompressed( const char* aContentEncoding );
    */
        
#if 0    
    TBool IsFLVStreamL(const TDes8& aBuf);
#endif
    
    /**
     * Sets the FLV stream play status to in-progress
     * This is introduced in fix for defect ids: isource: 14192, eroom issue 1039
     * 
     * This API is introduced as replacement for IsMediaStreamL. 
     * reason for code revert is given in task description : synergy task id. bh1s60#71512.
     */
   /* void SetFLVPlayStateInProgressL();*/
    TBool IsMediaStreamL();
                                 
    #ifdef __DUMP_TO_FILE
    	void DumpToFile(TInt& aStreamId,
                        const char* aData, 
                        unsigned long aDataLength);
	#endif
	
	
    private: //attributes
	
        CFlashPluginInstance* iPluginInstance;
        
        MPluginView* iPluginView;
		
		unsigned short iStreamId;

	    unsigned long  iBufferSize;
	    
	    HBufC8*  		iURL;
	    
	    //Buffer used to store the stream content until the 
	    //DRM headers are received fully. The DRM headers are 
	    //contained in the first 1024 bytes of the DRM content 
	    //so the size of this buffer should be a minimum of 1KB.
	    HBufC8*  iDRMHeaderBuffer; 
	    
	    //Number of bytes of DRM header received so far
	    TInt iNumHdrBytesReceivedSoFar; 
	    
	    //Whether DRM header has arrived
	    TBool iDRMHeaderHasArrived;
	    
	    TBool iIsProtected;
	    
	    HBufC8*  iMimeType;
	    
	    TBool    iVersionNotChecked; 
	    
	    TInt32 iContentLength;
	    
	    TBool iCancelState;
	    
	    EMediaStreamType iMediaStreamType;
	           	
};



#endif //__FLASH_STREAM_HANDLER_H__
