/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CAnimationPluginInstance class implementation
*
*/
 
// INCLUDE FILES
#include "saplayer_build.h"
#include "flashanimplugin.h"
#include <eikenv.h>
#include <bautils.h> // NearestLanguage
#include <content.h>
#include <data_caging_path_literals.hrh>
#include <featmgr.h>

#include "flashconfiggen.h"
#include "saflash_consts.h"
#include "pluginview.h"

// CONSTANTS

extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );

// ================= MEMBER FUNCTIONS =======================

// -------------------------------------------------------------------------
// CAnimationPluginInstance::CAnimationPluginInstance(PluginGlobals* aGlobals, NPP aInstance, 
// uint16 /*aMode*/)
//
// First phase constructor
// --------------------------------------------------------------------------
//    

using namespace ContentAccess;

CAnimationPluginInstance::CAnimationPluginInstance(MNetscapeIf* aNetscapeIf,NPFlashConfig* aConfig,
				CPluginGlobals* aGlobals,
				uint16 aMode,TInt aInteractionMode):
				CFlashPluginInstance(aNetscapeIf,aGlobals,aMode,aInteractionMode),
				iFlashConfig(aConfig)
   {
   }

// -------------------------------------------------------------------------
// CAnimationPluginInstance* CAnimationPluginInstance::NewL(PluginGlobals* aGlobals, 
// NPP aInstance, uint16 aMode)
//
// Static constructor
// --------------------------------------------------------------------------
// 


CAnimationPluginInstance* CAnimationPluginInstance::NewL(MNetscapeIf* aNetscapeIf,NPFlashConfig* aConfig,
														 CPluginGlobals* aGlobals,
                                                         uint16 aMode,
                                                         TInt aInteractionMode,TPluginUiFramework aPluginUiFramework)

    {
    NPPDebug(LOG_DEBUG,_L("CAnimationPluginInstance::NewL() ++"));
    

    CAnimationPluginInstance* self = new(ELeave) CAnimationPluginInstance(aNetscapeIf,aConfig,
    									aGlobals, 
    									aMode,
    									aInteractionMode);

    CleanupStack::PushL(self);
    self->ConstructL(aPluginUiFramework);
    CleanupStack::Pop(self);
    
    NPPDebug(LOG_DEBUG,_L("CAnimationPluginInstance::NewL() --"));
    return self;
    }

// -------------------------------------------------------------------------
// CAnimationPluginInstance::~CAnimationPluginInstance()
//
// Destructor
// --------------------------------------------------------------------------
// 
CAnimationPluginInstance::~CAnimationPluginInstance()
    {
    NPPDebug(LOG_DEBUG,_L("CAnimationPluginInstance::~CAnimationPluginInstance() ++"));
    
    if(iFlashConfig)
    {
    	delete iFlashConfig;
    }
    
    NPPDebug(LOG_DEBUG,_L("CAnimationPluginInstance::~CAnimationPluginInstance() --"));
    }


// -------------------------------------------------------------------------
// void CAnimationPluginInstance::ConstructL()
//
// Second phase constructor
// --------------------------------------------------------------------------
// 
void CAnimationPluginInstance::ConstructL(TPluginUiFramework aPluginUiFramework)
    {
    CFlashConfigGen::FillFlashConfig(GetInteractionMode(),
            iConfig);

    FillConfig(); //fill additional config values


    CreateEngineL();

    AddHwSideKeyObserverL();
    
    SetInitialVolumeToEngine();
    
    iPluginView = CreatePluginView(aPluginUiFramework);
    }

// -------------------------------------------------------------------------
// CAnimationPluginControl::FillConfig()
//
// First flash engine config based on Flashconfig
// --------------------------------------------------------------------------
//
void CAnimationPluginInstance::FillConfig()
{
    
    NPFlashConfig* pFlashconfig = GetFlashconfig();
    
    if(NULL != pFlashconfig) 
    {
        
        if((pFlashconfig->iIsMMI) && (NULL != pFlashconfig->iExtensions))
        {
             //Fill MMI info
                
            iConfig.iMode |= KSupportMMI;
            iConfig.iExtensions = pFlashconfig->iExtensions;
            iConfig.iSMCheckNeeded=EFalse;

        }
    
        if(NULL != pFlashconfig->iExtensions)
        {
            iConfig.iExtensions = pFlashconfig->iExtensions;
        }
        
        switch(pFlashconfig->iModeType)
        {
            case NPIdle :
            {
                iConfig.iMode |=  KSupportNoVibra
                                | KSupportNoGetURL
                                | KSupportNoBackLightProcessing 
                                | KSupportNoCallMonitor;
            }
            break;
        
            /** 
            *  flash running as video ring tone 
            */
        
            case NPRingTone:
            {
                iConfig.iMode |=  KSupportNoGetURL 
                                | KSupportNoBackLightProcessing 
                                | KSupportNoCallMonitor;
            }
            break;

            /** 
            *  flash running as call image 
            */
        
            case NPCallImage:
        
            /** 
            *  flash running as animated background in idle 
            */
            
            case NPBackground:
        
            /** 
            *  flash running as wall paper 
            */
        
            case NPWallpaper:
            {
                iConfig.iMode |=  KSupportNoVibra
                                | KSupportNoGetURL
                                | KSupportNoBackLightProcessing 
                                | KSupportNoCallMonitor;
            }
            break;
            
            /** 
            *  flash running as notification  for calendar apointments
            */
        
            case NPNotification:
            break;

            /** 
            *  flash running as startup screen
            */      
            case NPStartupScreen:
                        
            /** 
            *  flash running as shutdown screen
            */
            case NPShutdownScreen:
            {
                iConfig.iMode |=  KSupportNoVibra
                                | KSupportNoGetURL
                                | KSupportNoBackLightProcessing 
                                | KSupportNoCallMonitor;
            }
            break;

            default :
            break;
        }
    }
}

// -------------------------------------------------------------------------
// void CAnimationPluginInstance::SetWindowL(MPluginAdapter* aPlugin, const TRect& 
// aRect)
//
// Creates the window owing control and sets the window parameters
// --------------------------------------------------------------------------
// 
//void CAnimationPluginInstance::SetWindowL(MPluginAdapter* aPlugin, const TRect& aRect, const TRect& aClipRect)
//    {
//    //CFlashPluginInstance::SetWindowL(aPlugin, aRect, aClipRect);
//    //(static_cast<CFlashPluginControl*>(iPluginView))->SetRect(aRect);        
//    }

// -------------------------------------------------------------------------
// void CFlashPluginInstance::OpenFileL(const TDesC& aFilename, const TDesC* aUrl)
// Used to open a SWF file
// --------------------------------------------------------------------------
// 
void CAnimationPluginInstance::OpenFileL(const TDesC& aFilename, 
                                         const TDesC* aUrl, 
                                         const TBool& aFirstMovie)
{

    if(iFlashConfig && (!(iFlashConfig->iPrefFlag & NPPrefControlAnim)))
    {
        CFlashPluginInstance::OpenFileL(aFilename,*aUrl,aFirstMovie);
    }
    else
    {
        if(iCachedFileName)
        {
            delete iCachedFileName; 
            iCachedFileName = NULL;
        }
        if(iCachedUrl)   
        {
            delete iCachedUrl; 
            iCachedUrl = NULL;    
        }
        
        CacheFileNameL(aFilename,aUrl,aFirstMovie);
    }
}
    
    
 // -------------------------------------------------------------------------
// void CFlashPluginInstance :: HandleSetCommandL(NPNVariable aVariable, 
//                    void* ret_value)
//
//
// Processes the set_value commands
// --------------------------------------------------------------------------
//                     
NPError CAnimationPluginInstance::HandleSetCommandL(int aVariable, 
                                                    void* ret_value) 
{
	if (!iPluginView)
	{
		return -1;
	}
    	
	switch(aVariable)
    {
     	
		case NPNAnimationEvent :
		{
		    NPAnimationEvent animEvent = ( NPAnimationEvent )(*(NPAnimationEvent*)ret_value);
            
			switch (animEvent)
			{					
				case  NPStartAnimation:
			    {
			        if(iFlashConfig && (iFlashConfig->iPrefFlag & NPPrefControlAnim))
                    {
                       	if(iCachedFileName)
    					{
    						//use the base class OpenFile as this class OpenFile 
    						//does not handle NPPrefControlAnim state
    						TRAPD (err, CFlashPluginInstance::OpenFileL(
    						                                  iCachedFileName->Des(),
    						                                  KNullDesC(),
    						                                  ETrue));
    						if (err != KErrNone) 
    						{	
    							return -1;
    						}
    					}
    					
    				    return KErrNone; //NPERR_NO_ERROR;;
                    }
			    }        			
        			
                default:
		            return CFlashPluginInstance :: HandleSetCommandL(aVariable,ret_value);
			}
		}
		default:
		{
		    return CFlashPluginInstance :: HandleSetCommandL(aVariable,ret_value);
		}	
    }
}

TUint CAnimationPluginInstance::GetFeatureCapability()
    {
    NPFlashConfig* pFlashconfig = GetFlashconfig();

    if (NULL != pFlashconfig)
        {
        return GetFeatureCapabilityFromMode(pFlashconfig->iModeType);
        }

    return KCapabilitiesAnimation;
    }

TUint CAnimationPluginInstance::GetFeatureCapabilityFromMode(const TUint16 aModeType)
{
    switch(aModeType)   
    {
    
    /** 
        *  flash running in idle 
    */
    
        case NPIdle : 
                    { 
                    return KCapabilitiesIdle ; 
                    }

    
    /** 
        *  flash running as video ring tone 
    */
    
        case NPRingTone: { 
                    return KCapabilitiesRingTone ; 
                    }

    
    /** 
        *  flash running as call image 
    */
    
        case NPCallImage: { 
                    return KCapabilitiesCallImage ; 
                    }

    
    /** 
        *  flash running as animated background in idle 
    */
    
        case NPBackground: { 
                    return KCapabilitiesBackground ; 
                    }

    
    /** 
        *  flash running as wall paper 
    */
    
        case NPWallpaper: { 
                    return KCapabilitiesWallpaper ; 
                    }

    
    /** 
        *  flash running as notification  for calendar apointments
    */
    
        case NPNotification: { 
                    return KCapabilitiesNotification ; 
                    }

    
    /** 
        *  flash running as startup screen
    */      
        case NPStartupScreen: { 
                    return KCapabilitiesStartupScreen ; 
                    }

    
    /** 
        *  flash running as shutdown screen
    */
        case NPShutdownScreen: { 
                    return KCapabilitiesShutdownScreen ; 
                    }

        
                /** 
        *  simple animation, this is default 
    */
    
        case NPAnimation :
        default :
                { 
                    return KCapabilitiesAnimation ; 
                    }

        
    }

}
// End of File
