/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  FlashConfigGen class implementation
*
*/


// INCLUDE FILES
#include <eikenv.h>
#include "saplayer_build.h"
#include "flashconfiggen.h"
#include "NPNFlashExtension.h"
#include "flash_engine.h"

#include <centralrepository.h> //CRepository
#ifndef ENABLE_SF_CHANGES
#include <Flashlite_2_0PrivateCRKeys.h> //senRep keys
#else
#include "Flashlite_2_0PrivateCRKeys.h" //senRep keys
#endif

//CONST definitions
const TInt KFlashStaticHeapSize = 800 << 10; // 800KB for the Flash static heap
const TInt KFlashDynHeapSize    = 48 << 20; // 48MB for the Flash dynamic heap
const TInt KFlashDynMaxHeapSize = 40 << 20; // 40MB for the Flash dynamic heap for flv case
const TInt KAlphaMaxValue       = 255;

#ifdef FEATURE_SCREENSAVER
const TInt KFlashScreensaverDynHeapSize = 4 << 20; // 2MB for the Flash dynamic heap for screensaver
#endif

//=====================CFlashConfigGen======================

// ----------------------------------------------------------------------
// void CFlashConfigGen::FillFlashConfig(TInt aInteractionMode,
//								FlashEngineConfig& aConfig)
//
// This function is used to fill FlashEngineConfig
// ----------------------------------------------------------------------
// 

void CFlashConfigGen::FillFlashConfig(TInt aInteractionMode,
								FlashEngineConfig& aConfig)
{
    FillConfigDefault(aInteractionMode,aConfig);
    FillConfigFromRepo(aInteractionMode,aConfig);
    FillConfigInternal(aInteractionMode,aConfig);
}

// ----------------------------------------------------------------------
// CFlashConfigGen::CFlashConfigGen(TInt aInteractionMode)
//
// This function is destructor for CFlashConfigGen
// ----------------------------------------------------------------------
//
CFlashConfigGen::CFlashConfigGen()
{
}


// ----------------------------------------------------------------------
// CFlashConfigGen::CFlashConfigGen(TInt aInteractionMode)
//
// This function is destructor for CFlashConfigGen
// ----------------------------------------------------------------------
//
CFlashConfigGen::~CFlashConfigGen()
{
}

// ----------------------------------------------------------------------
// void CFlashConfigGen::FillConfigDefault()
//
// This function is used to fill default values for Flashconfig structure
// ----------------------------------------------------------------------
// 
void CFlashConfigGen::FillConfigDefault( TInt aInteractionMode,
										FlashEngineConfig& aConfig)
{
	#ifdef FEATURE_BG_TRANSPARENCY    
    aConfig.iPlayerBlendsBackground  = EFalse;
	aConfig.iBackgroundAlpha         = KAlphaMaxValue;    
	#endif
  

    aConfig.iSMCheckNeeded=ETrue;

  //default settings
  
  // 250KB for the Flash static heap
 aConfig.iSoftkeyCount = 	aConfig.iMode = 0;
 aConfig.iStaticHeapSize =  KFlashStaticHeapSize; 
 
 switch(aInteractionMode)
 {
 	
  	#ifdef FEATURE_SCREENSAVER	
  		case NPScreenSaverMode:
  			{
  				aConfig.iDynamicHeapMaxSize = KFlashScreensaverDynHeapSize;
            	aConfig.iMode               = KSupportVideo 
                        	                | KSupportNoSound
                        	                | KSupportNoVibra 
                        	                | KSupportNoGetURL
                        	                | KSupportNoBackLightProcessing 
                        	                | KSupportNoCallMonitor
                        	                | KSupportNoETelInfo;  		
  									
                #ifdef SELF_IAD_UPDATE
                aConfig.iCheckIADUpdate = EFalse;
                #endif
  			}
  						break;
  	#endif
  						
  	#ifdef FEATURE_ANIMATION_PLUGIN
  		case NPAnimPluginMode:
			{
				aConfig.iDynamicHeapMaxSize = KFlashDynMaxHeapSize; //max is set
				aConfig.iMode  = KSupportSound 
                          | KSupportVideo 
                          | KSupportNetwork 
                          | KSupportLocalFile 
                          | KSupportStereo
                          | KSupportSharedObject
                          | KSupportMMI;
                #ifdef SELF_IAD_UPDATE
                aConfig.iCheckIADUpdate = EFalse;
                #endif
			}	
			break;
  	#endif
  						
  	default :
  			{
  			 aConfig.iDynamicHeapMaxSize = KFlashDynMaxHeapSize; 
       		 aConfig.iMode               =  KSupportSound 
       		                              | KSupportVideo 
       		                              | KSupportNetwork 
       		                              | KSupportStereo
				                          | KSupportSharedObject;
            #ifdef SELF_IAD_UPDATE
            aConfig.iCheckIADUpdate = ETrue;
            #endif
  			}
  			break;
  						
  }
  		
}

// ----------------------------------------------------------------------
// void CFlashConfigGen::FillConfigInternal()
//
// This function is used to fill flashconfig app specific settings
// ----------------------------------------------------------------------
//
#ifdef FEATURE_SCREENSAVER
void CFlashConfigGen::FillConfigInternal(TInt aInteractionMode,
								  FlashEngineConfig&  aConfig
								  )
#else
void CFlashConfigGen::FillConfigInternal(TInt aInteractionMode,
                                  FlashEngineConfig&  /*aConfig*/
                                  )
#endif
{
   //always do ORing of Mode value earlier filled from Repo/default
	switch(aInteractionMode)
	 {
	 	
	  	#ifdef FEATURE_SCREENSAVER	
	  		case NPScreenSaverMode:
	  			{
	  			   	aConfig.iMode               |=  KSupportNoSound
	                        	                | KSupportNoVibra 
	                        	                | KSupportNoGetURL
	                        	                | KSupportNoBackLightProcessing 
	                        	                | KSupportNoCallMonitor;  		
	  									
	  			}
	  						break;
	  	#endif
	 }
	 
}
// ----------------------------------------------------------------------
// void CFlashConfigGen::FillConfigFromRepo()
//
// This function is used to fill flashconfig from CenRep
// ----------------------------------------------------------------------
// 
void CFlashConfigGen::FillConfigFromRepo(TInt aInteractionMode,
																FlashEngineConfig& aConfig)
{
  TInt staticHeapsSizeIntID =0, dynamicHeapMaxSizeIntID =0, modeID =0;
  
	// Connecting to repository
  CRepository* repository=NULL;
  TInt err = KErrNotFound;
  
  switch(aInteractionMode)
  {
  	#ifdef FEATURE_SCREENSAVER
  	case NPScreenSaverMode:
  						{
  							TRAP( err, repository = CRepository::NewL(KCRUIdFlashliteScreensaver));
        					staticHeapsSizeIntID = KFlashliteScreensaverStaticHeapSize;
        					dynamicHeapMaxSizeIntID = KFlashliteScreensaverDynamicHeapMaxSize;
        					modeID = KFlashliteScreensaverEngineMode;		
  							
  						}
  						break;
  	#endif

  	#ifdef FEATURE_ANIMATION_PLUGIN
	  	case NPAnimPluginMode:
			// do nothing for animation plugin.
			break;
  	#endif					
  						
  	default :
		{
			 TRAP( err, repository = CRepository::NewL(KCRUIdFlashliteBrowserPlugin));        
			 staticHeapsSizeIntID = KFlashliteBrplginStaticHeapSize;
			 dynamicHeapMaxSizeIntID = KFlashliteBrplginDynamicHeapMaxSize;
			 modeID = KFlashliteBrplginEngineMode;	
		}
		break;
  						
  }
  
  TInt retVal = KErrNone;

  if(err == KErrNone) 
   {
   		TInt staticHeapsSizeInt=0;
    	TInt dynamicHeapMaxSizeInt=0 ;
    	TInt mode=0;
        // Getting the values
        retVal = repository->Get(staticHeapsSizeIntID,
                                 staticHeapsSizeInt);
        //TODO: what will happen if get fails
        if ( retVal == KErrNone ) 
            {
            aConfig.iStaticHeapSize = (TUint32)staticHeapsSizeInt;    
            }
            
        
        retVal = repository->Get(dynamicHeapMaxSizeIntID,
                                             dynamicHeapMaxSizeInt);
        if ( retVal == KErrNone ) 
            {
            aConfig.iDynamicHeapMaxSize = (TUint32)dynamicHeapMaxSizeInt;    
            }
        
        retVal = repository->Get(modeID , mode);
        if ( retVal == KErrNone ) 
        {
        	aConfig.iMode = (TUint32)mode;
        }
   		
        // Close connection
        delete repository;        
   }
}

TInt CFlashConfigGen::GetFlashMaxDynHeapSize(TInt aInteractionMode)
{
    switch(aInteractionMode)
    {
     	
      	#ifdef FEATURE_SCREENSAVER	
      	case NPScreenSaverMode:
      	{
      	    return KFlashScreensaverDynHeapSize;
      	}
      	#endif
      						
      	default :
		{
		    return KFlashDynHeapSize; 
		}					
    }
}

TInt CFlashConfigGen::GetFLVMaxDynHeapSize()
{
	return KFlashDynMaxHeapSize;
}
//==========================================================
