/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  browser dll initialisation code and exported functions
 *
*/


// INCLUDE FILES
#include "saplayer_build.h"

#include <implementationproxy.h>
//pluginglobals include
#include <eikenv.h>
#include <bautils.h> // NearestLanguage
#include <data_caging_path_literals.hrh>
#include <featmgr.h>

#include "flashconfiggen.h"
#include "flashplgmain.h"
#include "swfmemmgr.h"
#include "netscapeif.h"
#include "pluginarguments.h"
#include "flashplugininstancefactory.h"

// CONSTANTS
const TUint KTotalHeapSize = (32 * 1024 * 1024); //allow 20 MB

void NPPDebug(TUint8 /*aLogLevel*/, TRefByValue<const TDesC> /*aText*/, ...)
{
}

EXPORT_C void* LoadFlashPlugin(MNetscapeIf* aNetscapeIf,
        MPluginArguments* aPluginArguments,TPluginUiFramework aPluginUiFramework)
{
	//Create plugin globals here..Pass netscape if to plugin globals
	CPluginGlobals* pluginGlobals = NULL;
	TRAPD(error,  pluginGlobals = CPluginGlobals::NewL());
	if (error != KErrNone)
	{
		return NULL;
	}

#ifdef  FEATURE_WINDOWLESS  
	//TODO: Move to Glue.DLL
	if (EWindowless == aPluginArguments->WindowMode())
	{
		NPBool windowed = 0;
		NPBool transparent = 1;

		aNetscapeIf->SetValue(NPPVpluginWindowBool,
				static_cast<void*> (&windowed));
		aNetscapeIf->SetValue(NPPVpluginTransparentBool,
				static_cast<void*> (&transparent));
	}
	else if (EWindowlessOpaque == aPluginArguments->WindowMode())
	{
		NPBool windowed = 0;
		NPBool transparent = 0;

		aNetscapeIf->SetValue(NPPVpluginWindowBool,
				static_cast<void*> (&windowed));
		aNetscapeIf->SetValue(NPPVpluginTransparentBool,
				static_cast<void*> (&transparent));
	}
#endif

	const int KEmbed = 1;
	//Call pluginInstanceFactory to create plugin; 
	//NPP instance = (NPP) aNetscapeIf->NPPInstance();
	CFlashPluginInstance* plugin = NULL;

	TRAPD(err, plugin = CFlashPluginInstanceFactory::GetInstanceL(aNetscapeIf,pluginGlobals,KEmbed,aPluginArguments->InteractionMode(),aPluginArguments->WindowMode(), aPluginUiFramework));
	
	if (err != KErrNone)
	{
		return NULL;
	}

	plugin->SetSwfSourcePath(aPluginArguments->SwfSourcePath());

	TRAP_IGNORE (plugin->SetBaseUrlL(aPluginArguments->BaseUrl()));

	plugin->SetAllowMobiFullScreen(aPluginArguments->AllowMobiFullScreen());

	// pass plugin arguments to plugin instance as well as plugin globals
	// (got to remove the const-ness and pass a pointer
	CDesCArray* argNames =
			&(const_cast<CDesCArray&> (aPluginArguments->ArgNames()));
	CDesCArray* argValues =
			&(const_cast<CDesCArray&> (aPluginArguments->ArgValues()));
	plugin->ProcessParameters(argNames, argValues);

	//Plugininstance will take the ownership of plugin globals
	return (MFlashPluginIf*) plugin;
}

// ================= CPluginGlobals ==========================


// -------------------------------------------------------------------------
// CPluginGlobals::CPluginGlobals(NPNetscapeFuncs* aFuncs)
//
// First phase constructor
// --------------------------------------------------------------------------
// 

CPluginGlobals* CPluginGlobals::NewL()
{
	CPluginGlobals* pluginGlobals = NULL;

	// Check local thread storage
	if (Dll::Tls() == NULL)
	{
		pluginGlobals = new (ELeave) CPluginGlobals();

		if (pluginGlobals)
		{
			CleanupStack::PushL(pluginGlobals);
			pluginGlobals->ConstructL();
			CleanupStack::Pop();

			TInt error = Dll::SetTls(static_cast<TAny*> (pluginGlobals));

			if (error == KErrNone)
			{
				User::LockedInc(pluginGlobals->iRefCnt);//pluginGlobals->iRefCnt++;

				return pluginGlobals;
			}

			delete pluginGlobals;

		}

	}
	else
	{
		pluginGlobals = STATIC_CAST(CPluginGlobals*, Dll::Tls() );

		if (pluginGlobals)
		{
			User::LockedInc(pluginGlobals->iRefCnt);//pluginGlobals->iRefCnt++;

			return pluginGlobals;
		}
	}

	return NULL;
}

// -------------------------------------------------------------------------
// CPluginGlobals::CPluginGlobals(NPNetscapeFuncs* aFuncs)
//
// Default constructor
// --------------------------------------------------------------------------
// 

CPluginGlobals::CPluginGlobals()
               : iResOffset(0)
                 ,iHwKeyProperty(NULL)
                 ,iRefCnt(0)
                 ,iDynHeapSize(0)
#ifdef PLUG_IN_MEM_ALLOC
			, iSwfMemMgr(NULL)
#endif
{
}

// -------------------------------------------------------------------------
// void CPluginGlobals::ConstructL()
//
// Second phase constructor
// --------------------------------------------------------------------------
// 
void CPluginGlobals::ConstructL()
{
#ifdef PLUG_IN_MEM_ALLOC
	iSwfMemMgr = CSwfMemMgr::NewL(this, KTotalHeapSize);
#endif

	// Load resources - as we are not an app we need to do this manually
	/*  commented Reason : to avoid locking of the resource file right here

	 _LIT(KPluginRscFile, "flashliteplugin.rsc");
	 NPPDebug(LOG_DEBUG,_L("CPluginGlobals::ConstructL++"));
	 TFileName rscfn;
	 TFileName dllName;
	 Dll::FileName(dllName);
	 TBuf<KDriveLetterLength> drive = dllName.Left(KDriveLetterLength);		

	 rscfn.Copy(drive); // get the drive letter
	 rscfn.Append(KDC_APP_RESOURCE_DIR);
	 rscfn.Append(KPluginRscFile);

	 BaflUtils::NearestLanguageFile(CCoeEnv::Static()->FsSession(), rscfn);
	 
	 TBool fileExists( BaflUtils::FileExists(CCoeEnv::Static()->FsSession(), rscfn) );
	 if ( fileExists )
	 {
	 iResOffset = CCoeEnv::Static()->AddResourceFileL(rscfn);
	 }
	 */
	if (FeatureManager::FeatureSupported(KFeatureIdSideVolumeKeys))
	{
		iHwKeyProperty = CHwKeyPropertyWatch::NewL();
	}
	NPPDebug(LOG_DEBUG,_L("CPluginGlobals::ConstructL--"));
}

TUint CPluginGlobals::GetFreeHeapSize(TInt aInteractionMode)
{
    return CFlashConfigGen::GetFlashMaxDynHeapSize(aInteractionMode) - iDynHeapSize;
	//return KFlashDynHeapSize - iDynHeapSize;
}

void CPluginGlobals::AddDynHeapAlloced(TUint aSize)
{
	iDynHeapSize += aSize;
}

void CPluginGlobals::DynHeapFreed(TUint aSize)
{
	iDynHeapSize -= aSize;
}
#ifdef PLUG_IN_MEM_ALLOC    
CSwfMemMgr* CPluginGlobals::GetMemMgr()
{
	return iSwfMemMgr;
}
#endif

TUint CPluginGlobals::GetMaxHeapSize(TInt aInteractionMode)
{
	// return KFlashDynHeapSize;
	return CFlashConfigGen::GetFlashMaxDynHeapSize(aInteractionMode);
}

void CPluginGlobals::DeletePluginGlobal()
{
	// Still used by many; so dont delete or there are no users.
	if (iRefCnt > 1 || iRefCnt == 0)
	{
		if (iRefCnt)
		{
			User::LockedDec(iRefCnt); //iRefCnt--;
		}

		return;
	}

	//we have one object left, let us delete
	Dll::FreeTls(); //free tls first

	delete this;
}

// -------------------------------------------------------------------------
// CPluginGlobals::~CPluginGlobals()
//
// destructor
// --------------------------------------------------------------------------
// 
CPluginGlobals::~CPluginGlobals()
{
	// CEikonEnv::Static()->DeleteResourceFile(iResOffset);

	if (iHwKeyProperty)
	{
		delete iHwKeyProperty;
		iHwKeyProperty = NULL;
	}
#ifdef PLUG_IN_MEM_ALLOC    
	if (iSwfMemMgr)
	{
		delete iSwfMemMgr;
		iSwfMemMgr = NULL;
	}
#endif
}

//////////////////////////////////////////////////////////////////////////////////////////////////

// -------------------------------------------------------------------------
// int load_pluginresource()
// Function which loads the plugin resource and returns the resource offset
// -------------------------------------------------------------------------
int load_pluginresourceL(TBufC<128>& aFileName)
{
	TInt resOffset = -1;

	TBuf16<128> aResourcePath(_L("\\resource\\apps"));
	// strip the extension
	TInt fileNameLength = aFileName.Length();
	TFileName resourceFileName;

	RFs fileServerSession = CCoeEnv::Static()->FsSession();

	if (fileNameLength > 4)
	{
		// we want just the name 
		// ex, if aFileName is flash2ui.rsc,
		// fName will be flash2ui
		TBuf16<128> fName = aFileName.Mid(0, fileNameLength - 4);

		TBuf16<512> variantFile;
		_LIT16( KFormat, ":%S\\%S.r%S%d");

		// search for the resource file based on the current language
		TInt languageID = User::Language();
		TBuf16<8> leadingZero;
		leadingZero.Copy(_L(""));
		if (languageID < 10)
			leadingZero.Copy(_L("0"));

		// start from Y: go to A: and then finally search Z:
		for (TInt i = EDriveY; i >= EDriveA; i--)
		{
			//TChar driveLetter = 'a'; // +i;
			TChar driveLetter;
			fileServerSession.DriveToChar(i, driveLetter);
			variantFile.Append(driveLetter);
            variantFile.AppendFormat(KFormat,  &aResourcePath, &fName, &leadingZero, languageID );

			resourceFileName.Copy(variantFile);
			NPPDebug(LOG_DEBUG, _L("Trying: %S\n"), &resourceFileName);
            TBool fileExists = BaflUtils::FileExists( CCoeEnv::Static()->FsSession(), resourceFileName );
			if (fileExists)
			{
                resOffset = CCoeEnv::Static()->AddResourceFileL(resourceFileName);
				NPPDebug(LOG_DEBUG, _L("resource : %S"), &resourceFileName);
				return resOffset;
			}

		}
		// see if it is present on z drive
					TChar driveLetterZ;
					fileServerSession.DriveToChar(EDriveZ, driveLetterZ);
					variantFile.Append(driveLetterZ);
					variantFile.AppendFormat(KFormat,  &aResourcePath, &fName, &leadingZero, languageID );

					resourceFileName.Copy(variantFile);
					NPPDebug(LOG_DEBUG, _L("Trying: %S\n"), &resourceFileName);
        TBool fileExists = BaflUtils::FileExists( CCoeEnv::Static()->FsSession(), resourceFileName );
					if (fileExists)
					{
						resOffset = CCoeEnv::Static()->AddResourceFileL(resourceFileName);
						NPPDebug(LOG_DEBUG, _L("resource : %S"), &resourceFileName);
						return resOffset;
					}
					// if exact language is not found, search for the nearest language in the respective drive
					// start from Y: go to A: and then finally search Z:
					_LIT16( KNearestLangFormat, ":%S\\%S");
					for (TInt i = EDriveY; i >= EDriveA; i--)
					{
						TChar driveLetter;
						fileServerSession.DriveToChar(i, driveLetter);
						//TBuf16<128> KFormat( _L("%c:%S\\%S"));
						variantFile.Append(driveLetter);
            			variantFile.AppendFormat(KNearestLangFormat,  &aResourcePath, &aFileName );    // use the filename that was passed in

						resourceFileName.Copy(variantFile);
            			BaflUtils::NearestLanguageFile( CCoeEnv::Static()->FsSession(), resourceFileName );
						NPPDebug(LOG_DEBUG, _L("Trying: %S\n"), &resourceFileName);
            			TBool fileExists = BaflUtils::FileExists( CCoeEnv::Static()->FsSession(), resourceFileName );
						if (fileExists)
						{
                			resOffset = CCoeEnv::Static()->AddResourceFileL(resourceFileName);
							NPPDebug(LOG_DEBUG, _L("resource : %S"), &resourceFileName);
							return resOffset;
						}

					}

					// see if it is present on z drive
					// TBuf16<128> KZFormat( _L("z:%S\\%S"));
					variantFile.Append(driveLetterZ);
        			variantFile.AppendFormat(KNearestLangFormat,  &aResourcePath, &aFileName );
					resourceFileName.Copy(variantFile);
        			BaflUtils::NearestLanguageFile( CCoeEnv::Static()->FsSession(), resourceFileName );
					NPPDebug(LOG_DEBUG, _L("Trying: %S\n"), &resourceFileName);
        			fileExists = BaflUtils::FileExists( CCoeEnv::Static()->FsSession(), resourceFileName );
					if (fileExists)
					{
						resOffset = CCoeEnv::Static()->AddResourceFileL(resourceFileName);
						NPPDebug(LOG_DEBUG, _L("resource : %S\n"), &resourceFileName);
						return resOffset;
					}

				}

				return resOffset; // -1
			}
			// End of File
