/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CBrowserPluginInstance class implementation
*
*/
 
// INCLUDE FILES
#include "saplayer_build.h"
#include "flashplugin.h"
#include <eikenv.h>
#include <bautils.h> // NearestLanguage
#include <caf/content.h>
#include <data_caging_path_literals.hrh>
#include <featmgr.h>
#include <httperr.h>
#include <StringLoader.h>
#include <AvkonInternalCRKeys.h>

//#include "FlashPluginControlFactory.h"
#include "hwkeywatch.h"

#include "netscapeif.h"


#ifdef FEATURE_WEB_BROWSABILITY
#include "pluginscripthandler.h"
#endif

#include "pluginmemhandlerimpl.h"

#include "deviceextnimpl.h"
#include "pluginview.h"
#include "streamfactory.h"

#include "saflash_consts.h"

#include <flashlitepluginiad.rsg>
#include <PtiDefs.h>
#include "commondef.h"

// CONSTANTS
const TUint KBrowserPluginArrayGranularity = 4;
const TUint KNPPVPluginOpenInViewer  = 101;
extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );
using namespace ContentAccess;


_LIT(KDefaultTargetWindow, "_top");
_LIT(KDefaultHeader,"Content-Type: application/x-www-form-urlencoded\r\n");
_LIT(KHttpContentTypeTag,"Content-Type:");
_LIT(KHttpTagTerminator,"\r\n");
_LIT(KContentLengthHeader,"Content-length: ");
_LIT(KHeaderTerminator,"\r\n\r\n");
//_LIT(KFlashPluginMifFilePath,"\\resource\\apps\\Flashplugin.mif");

_LIT(KFullScreenURLReq,"javascript:openFull()");
//_LIT(KAllowNetworkAccess,"AllowNetworkAccess");

const TUint KMaxLenSizeString       = 11;

// ================= MEMBER FUNCTIONS =======================

// -------------------------------------------------------------------------
// CBrowserPluginInstance::CBrowserPluginInstance(PluginGlobals* aGlobals, NPP aInstance, 
// uint16 /*aMode*/)
//
// First phase constructor
// --------------------------------------------------------------------------
//    

CBrowserPluginInstance::CBrowserPluginInstance(MNetscapeIf* aNetscapeIf,CPluginGlobals* aGlobals,  
				                               uint16 aMode,TInt aInteractionMode
#ifdef FEATURE_WINDOWLESS				                               
				                               , TWindowMode aWindowMode
#endif //FEATURE_WINDOWLESS				                               
				                               )
        				: CFlashPluginInstance(aNetscapeIf,aGlobals,aMode,aInteractionMode
#ifdef FEATURE_WINDOWLESS        						
        						, aWindowMode
#endif //FEATURE_WINDOWLESS        						
        				)
        				,iSavedFileNames(NULL)
        				,iAsyncDeleter(NULL)
        				,iDrmHelper(NULL)
        				,iPluginInfo(NULL)
        				,iErrorOccured(EFalse)
        				,iCallOpenFile(ETrue)
        				,iPausedDueToFocusLost(EFalse)
        				,iIsRelativeBaseUrl(EFalse)
{
    iAllowNetworkAccess = ETrue;
}

// -------------------------------------------------------------------------
// CBrowserPluginInstance* CBrowserPluginInstance::NewL(PluginGlobals* aGlobals, 
// NPP aInstance, uint16 aMode)
//
// Static constructor
// --------------------------------------------------------------------------
// 

CBrowserPluginInstance* CBrowserPluginInstance::NewL(MNetscapeIf* aNetscapeIf,CPluginGlobals* aGlobals 
                                                     ,uint16 aMode
                                                     ,TInt aInteractionMode
#ifdef FEATURE_WINDOWLESS                                                     
                                                     ,TWindowMode aWindowMode
#endif //FEATURE_WINDOWLESS
                                                     ,TPluginUiFramework aPluginUiFramework
													)

    {
    NPPDebug(LOG_DEBUG,_L("CBrowserPluginInstance::NewL() ++"));

    CBrowserPluginInstance* self = new(ELeave) CBrowserPluginInstance(aNetscapeIf,aGlobals 
    									,aMode
    									,aInteractionMode
#ifdef FEATURE_WINDOWLESS    									
    									,aWindowMode
#endif //FEATURE_WINDOWLESS    									
										);

    CleanupStack::PushL(self);
    self->ConstructL(aPluginUiFramework);
    CleanupStack::Pop(self);

    NPPDebug(LOG_DEBUG,_L("CBrowserPluginInstance::NewL() --"));
    return self;
    }

// -------------------------------------------------------------------------
// CBrowserPluginInstance::~CBrowserPluginInstance()
//
// Destructor
// --------------------------------------------------------------------------
// 
CBrowserPluginInstance::~CBrowserPluginInstance()
{
    NPPDebug(LOG_DEBUG,_L("CBrowserPluginInstance::~CBrowserPluginInstance() ++"));
    
    if (iDrmHelper)
    {
        delete iDrmHelper;
        iDrmHelper = NULL;
    }
    
    if (iSavedFileNames)
    {
        iSavedFileNames->Reset();    
        delete iSavedFileNames;       
        iSavedFileNames = NULL;
    }
        
    if( iPluginInfo )
    {
        delete iPluginInfo;
        iPluginInfo = NULL;
    }
        
    if( iAsyncDeleter )
    {
        delete iAsyncDeleter;
        iAsyncDeleter = NULL;
    }
    

    if(iRestartTimer)
    {
        iRestartTimer->Cancel();
        delete iRestartTimer;
        iRestartTimer = NULL;
    }
    
    NPPDebug(LOG_DEBUG,_L("CBrowserPluginInstance::~CBrowserPluginInstance() --"));
}

HBufC* CBrowserPluginInstance::GetFlashBaseURL()
	{
	if(iBaseURL)
		return iBaseURL;
	else
		return NULL;
	}


// -------------------------------------------------------------------------
// void CBrowserPluginInstance::PrepareToExitL()
//
// Called in the destructor to notify the Browser
// --------------------------------------------------------------------------
// 
void CBrowserPluginInstance::PrepareToExitL()
{
	if(iPluginView)
	{
        iPluginView->PrepareToExit();
		if (iPluginView->GetLastError() != KErrNone)
		{
			NPPDebug(LOG_INFO,_L("CBrowserPluginInstance::PrepareToExitL() : Plugin is exiting"));

			iAsyncDeleter->IssueRequest();

			iErrorOccured = ETrue;
		}
	}
		
	//iInstance->pdata = NULL;  
}

// -------------------------------------------------------------------------
// void CBrowserPluginInstance::ConstructL()
//
// Second phase constructor
// --------------------------------------------------------------------------
// 
void CBrowserPluginInstance::ConstructL(TPluginUiFramework aPluginUiFramework)
{
    
    
	FindExecutionMode();
	if(iBrPluginExecutionMode == EWidgetMode)
    {
	    QueryAllowNetworkInWidgetMode();
    }
	
    iPluginMemHandler = CPluginMemHandlerImpl::NewL(this);
        
    iDeviceExtn = CDeviceExtn::NewL();
        
    iPluginInfo = CPluginInstanceInfo::NewL();
    
    //Can the engine be created after the control?
    //Does the order matter?    
	iPluginView = CreatePluginView(aPluginUiFramework);
    
    
	
    //Moved down to experiment
    CreateEngineL();
    
    InitializeVolumeObserverL();
    
    SetInitialVolumeToEngine();
    
    iStreamFactory = CPluginStreamFactory::NewL(this,iPluginView);
    
    iSavedFileNames =  new (ELeave) CDesCArrayFlat
                                    (KBrowserPluginArrayGranularity); 
    
 //   if(EWindow==iWindowMode)
 //   {
    iAsyncDeleter = new (ELeave) CAsyncDeleter(this,iPluginView);
 //   }
    
}

    
TBool CBrowserPluginInstance::IsFocused()
{
    
    if (!iPluginView || !iPluginView->IsConstructed() || iErrorOccured)
    {
        return EFalse;        
    }

    TRect clipRect = iPluginView->GetClipRect();

    if( clipRect.Width() == 0 || clipRect.Height() == 0 )
    	{
    	return EFalse;
    	}

    return ETrue;
}
    
// -------------------------------------------------------------------------
// void CBrowserPluginInstance :: ProcessParameters ( CDesCArray* argnames, 
// CDesCArray* argvalues )
//
//
// Processes the parameters passed to the HTML tag
// --------------------------------------------------------------------------
//
//Moved to CFlashPluginInstance::ProcessParameters
/*
void CBrowserPluginInstance::ProcessParameters( CDesCArray* argnames, 
                                                CDesCArray* argvalues )
    {
    if (0 != iWindowLessMode)
        {
            {
            if (iWlessCtrl)
                {
                iWlessCtrl->ProcessParameters(argnames, argvalues);
                return;
                }
            }
        }

    if (!iControl || !iPluginInfo || iErrorOccured)
        {
        return;
        }

        TRAP_IGNORE(iPluginInfo->AddArgsL(argnames, argvalues));
    iControl->ProcessParameters(argnames, argvalues);
    }
*/

CDRMHelper* CBrowserPluginInstance::GetDRMHelperL()
{
	if( !iDrmHelper )
    {
		iDrmHelper = CDRMHelper::NewL( *CCoeEnv::Static() );   	
    }
    
	return iDrmHelper;
}
 
// -------------------------------------------------------------------------
// TBool CBrowserPluginInstance::PerformDrmChecksL(const TDesC& aFilename, 
//                                            TBool& aCorrectMimeAndRights)
// Used to perform checks related to DRM
// aFileName - DRM protected file
// aCorrectMimeAndRights - ETrue if the file has correct MIME type and it has
//                          adequate playrights, EFalse otherwise
// Returns ETrue if file is DRM protected SWF file and EFalse otherwise 
// --------------------------------------------------------------------------
// 
TBool CBrowserPluginInstance::PerformDrmChecksL(const TDesC& aFilename, 
                                                const CContent* aContent, 
                                            TBool& aCorrectMimeAndRights)
{
	TBool isProtected     = EFalse;
	aCorrectMimeAndRights = EFalse;
	
    if( !aContent )
    {
        User::Leave(KErrArgument);
    }

	aContent->GetAttribute( EIsProtected, isProtected );

	if ( !isProtected )
	{	    
	    return EFalse; 
	}
	    
    //Check if the MIME type is correct

    if ( CFlashPluginInstance::isFlashContentL(aContent) )
    {
        CDRMHelper* drmHelp = GetDRMHelperL();
    	
		if(drmHelp)
        {
        		drmHelp->CheckRightsAmountL( aFilename );    		
        }
        
        if ( CheckPlayRightsL(aContent) )
        {
            aCorrectMimeAndRights = ETrue;
        }        
	}

	return ETrue;
}

/* 
 * Function: TInt CBrowserPluginInstance::ReduceURL(HBufC8*&)
 * (1) if only domain exists in the URL, ensures it is terminated with a '/'
 * (2) otherwise, truncates all characters AFTER the last '/'
 * returns: the position of the third '/' which separates the domain and the rest of the URL
 */
TInt CBrowserPluginInstance::ReduceURL(HBufC8*& aCompleteURL8)
{
	if (aCompleteURL8 == NULL)
		return 0;
	
	TPtr8 completeURL8Ptr = aCompleteURL8->Des();
	TInt i, count = 0,thirdSlashPosition = 0, completeURL8Length = completeURL8Ptr.Length();
	for (i = 0; i < completeURL8Length; i++)
	{
		if (completeURL8Ptr[i] == '/')
		{
			count++;
			if (count == 3)
			{
				thirdSlashPosition = i;
				break;
			}
		}
	}    	
	// if only 2 slashes are present then trailing slash missing
	if(count == 2)
	{
		// this means we have only domain in the URL without trailing slash
		// url = "http://www.abc.com" -> append a trailing slash, accomodate for this:
		aCompleteURL8 = aCompleteURL8->ReAlloc(aCompleteURL8->Length()+1);
		completeURL8Ptr.Append('/');
	}
	else // atleast 3 '/'s . truncate after Last '/'.
	{
		TruncateURL(completeURL8Ptr);
	}
	return thirdSlashPosition;
}

/* 
 * Function: CBrowserPluginInstance::TruncateURL(TDes8&)
 * removes all data starting AFTER the last '/', so last character is guaranteed to be '/'
 * now the url will look like : "http://www.abc.com/token1/token2/"
 */
void CBrowserPluginInstance::TruncateURL(TDes8& aURLPtr)
{
	TInt lastSlashposition = aURLPtr.LocateReverse(TChar('/'));  
	if(lastSlashposition != aURLPtr.Length()-1)
	{        
		aURLPtr.Delete(lastSlashposition+1, aURLPtr.Length()-lastSlashposition);
	} 
}

void CBrowserPluginInstance::CreateBaseURLL(const TDesC& aSwfSrcURL)
{
	_LIT8(KSingleDot, ".");
	_LIT8(KDoubleDot, "..");
	_LIT8(KSingleDotSlash, "./");

	 
	/* 
	 * the Parsing done here for creating an absolute Base URL using the
	 * current page URL and relative base embed param
	 * follows recommendations of RFC 1808 - Relative URLs. 
	 * Document Path : http://www.faqs.org/rfcs/rfc1808.html
	 */

	MNetscapeIf* nsfuncs = NetscapeFns();
	
	/*
	 * if 'base' tag is present in the <head></head> section of the webpage
	 * then the value of this should be used as the Base URL (eRoom 1176).
	 * this URL is ALWAYS absolute.
	 */
	HBufC8 *htmlBaseTag8 = nsfuncs->GetHTMLBaseTagValueLC();

	if (htmlBaseTag8)
	{		
		if (iBaseURL)
		{
			delete iBaseURL;
			iBaseURL = NULL;
		}
		
		ReduceURL(htmlBaseTag8);
		TPtr8 htmlBaseTag8Ptr = htmlBaseTag8->Des();
		
		iBaseURL = HBufC::NewL(htmlBaseTag8Ptr.Length() + 1);
		TPtr baseUrlPtr = iBaseURL->Des();
		
		baseUrlPtr.Zero();
		baseUrlPtr.Copy(htmlBaseTag8Ptr);
		baseUrlPtr.ZeroTerminate();
		
		NPPDebug(LOG_INFO,_L("CreateBaseURLL(): Absolute Base URL formed: %S\n"), iBaseURL);
		CleanupStack::PopAndDestroy(htmlBaseTag8); // cleanup HTML Base Tag Descriptor
		
		return;
	}	
	
	HBufC8 *currentPageURL8 = nsfuncs->GetBaseUrlLC();	
	
	// if JavaScript is disabled or any error in NPAPIs in GetBaseUrlLC()
	// currentPageURL8 will be NULL, and we cannot form the Base URL.
	if (currentPageURL8 == NULL)
	{
		NPPDebug(LOG_INFO,_L("CreateBaseURLL(): JavaScript disabled or NPAPI error"));
		return;
	}

	// Security Context URL is the complete URL of the current HTML page		
	if (iSecurityContextUrl )
	{
		delete iSecurityContextUrl;
		iSecurityContextUrl = 0;
	}

	iSecurityContextUrl = HBufC::NewL(currentPageURL8->Length() + 1);
	TPtr securityUrlPtr = iSecurityContextUrl->Des();
	securityUrlPtr.Copy(*currentPageURL8);    
	securityUrlPtr.ZeroTerminate();
	
	NPPDebug(LOG_INFO,_L("Security Context URL formed: %S\n"),iSecurityContextUrl);

	TPtr8 currentPageURL8Ptr = currentPageURL8->Des();
	TInt hashOccurrenceFound, questionmarkOccurrenceFound; 
	_LIT8(KHashCharacter,"#");
	_LIT8(KQuestionmarkCharacter,"?");
	if ((hashOccurrenceFound = currentPageURL8Ptr.FindF(KHashCharacter)) > 0)
		currentPageURL8Ptr = currentPageURL8Ptr.Left(hashOccurrenceFound);
	if ((questionmarkOccurrenceFound = currentPageURL8Ptr.FindF(KQuestionmarkCharacter)) > 0)
		currentPageURL8Ptr = currentPageURL8Ptr.Left(questionmarkOccurrenceFound);
	/* 
	 * extract the current page's domain : The domain is from the start
	 * of the current url till the third slash '/' : e.g.: "http://www.abc.com/"
	 * we will keep current domain in the form : "http://www.abc.com", no trailing slash 
	 */
	TInt thirdSlashPosition = ReduceURL(currentPageURL8);
	currentPageURL8Ptr = currentPageURL8->Des();
	
	HBufC8* currentPageDomain8 =  HBufC8::NewLC(thirdSlashPosition);
	TPtr8 currentPageDomain8Ptr(currentPageDomain8->Des());   
	
	currentPageDomain8Ptr.Copy(currentPageURL8Ptr.Left(thirdSlashPosition));

	// is the current page at the root of the domain hierarchy ?
	TBool isDomainOnly = (thirdSlashPosition == currentPageURL8Ptr.Length() - 1);  

	// once we have the current page URL, check if base embed param exists, and if it is relative:
	if(iBaseURL && iIsRelativeBaseUrl)
	{ 
		// store the previously obtained base embed params
		// accomodate for appending a '/' in case last char is NOT '/' for parsing
		
		HBufC8* baseEmbedParam8 = HBufC8::NewLC(iBaseURL->Length()+1);
		TPtr8 baseEmbedParam8Ptr(baseEmbedParam8->Des());
		baseEmbedParam8Ptr.Copy(*iBaseURL);		

		// ensure that last character of base embed param is always '/'
		// form: "token1/token2/"
		if (baseEmbedParam8Ptr[baseEmbedParam8Ptr.Length()-1] != '/' )
		{
			baseEmbedParam8Ptr.Append('/');
		}

		delete iBaseURL;
		iBaseURL = NULL;
		
		// accomodate for max possible length: (base embed param + current page url)
		HBufC8* finalBaseURL8 = HBufC8::NewLC(baseEmbedParam8Ptr.Length() + currentPageURL8Ptr.Length());
		TPtr8 finalBaseURL8Ptr(finalBaseURL8->Des());
		
		// Special Case: base embed param = '.' (eRoom 1176)
		// pass the SWF URL as the Base URL:
		if (0 == baseEmbedParam8Ptr.CompareF(KSingleDotSlash()) && aSwfSrcURL.Length()>0)
		{
				CleanupStack::Pop(finalBaseURL8);
				// re-alloc of finalBaseURL8 is required as we will use the SWF URL now.
				finalBaseURL8 = finalBaseURL8->ReAlloc(aSwfSrcURL.Length() + 1);
				CleanupStack::PushL(finalBaseURL8);
				finalBaseURL8Ptr.Set(finalBaseURL8->Des());				
				finalBaseURL8Ptr.Copy(aSwfSrcURL);

				TruncateURL(finalBaseURL8Ptr);
		}   	   	   	
		/* 
		 * Case 2:
		 * if base param starts with '/' then base embed param must be appended to domain:
		 * e.g. base = "/token1/token2/something/" and 
		 * current page = "http://www.abc.com/test/test2/"
		 * Base URL should be "http://www.abc.com/token1/token2/something/" 	
		 * or, Case 1: only domain exists : current page domain = "http://www.abc.com"
		 */
		else if (isDomainOnly || baseEmbedParam8Ptr[0] == '/')
		{
			finalBaseURL8Ptr.Copy(currentPageDomain8Ptr); // domain "http://www.abc.com"
			if ( baseEmbedParam8Ptr[0] != '/')
				finalBaseURL8Ptr.Append('/');

			finalBaseURL8Ptr.Append(baseEmbedParam8Ptr); 	
		}    		
		else
		{   	
			/*
			 * copy the rest of the url after the domain to finalBaseURL8Ptr. While parsing
			 * the base embedded params, if valid token is encountered, append to finalBaseURL8Ptr
			 * if "./" do nothing and if "../" delete the last token from finalBaseURL8Ptr.
			 * guaranteed first char is always '/' and there is at least one char after '/'.
			 */
			finalBaseURL8Ptr.Copy(currentPageURL8Ptr.Right(currentPageURL8Ptr.Length() - thirdSlashPosition));

			// For our parsing purposes,  last character should be guaranteed
			// to NOT be '/', so remove the last character (always '/'), form: "/token1/token2"
			finalBaseURL8Ptr.Delete(finalBaseURL8Ptr.Length()-1,1);
			
			/* 
			 * The domain is fixed, and shouldn't be deleted in case of error in base embed param structure
			 * for e.g. if current page URL = "www.abc.com/token1" and base embed param is "../../something"
			 * then Base URL should be "www.abc.com/../something/"-> need to handle this correctly.
			 * this is a content issue (RFC 1808)
			 */

			TBool isParseComplete = EFalse;  

			// accomodate for max possible length of a token
			HBufC8* parsedDataBuffer8 = HBufC8::NewLC(baseEmbedParam8Ptr.Length());
			TPtr8 parsedDataBuffer8Ptr(parsedDataBuffer8->Des());

			TInt tokenSeparatorPosition = KErrNotFound;			

			while(!isParseComplete)
			{  
				tokenSeparatorPosition = baseEmbedParam8Ptr.LocateF('/');

				// parsed data buffer has at least one "token/" -> copy only the "token" part
				parsedDataBuffer8Ptr.Copy(baseEmbedParam8Ptr.Left(tokenSeparatorPosition));

				if (0 == parsedDataBuffer8Ptr.CompareF(KSingleDot))// "token" is "."
				{
					//just remove the "./" and continue
					baseEmbedParam8Ptr.Delete(0,tokenSeparatorPosition+1);    					
				}
				else if (0 == parsedDataBuffer8Ptr.CompareF(KDoubleDot))// "token" is ".."
				{
					/* 
					 * (1)Remove the "../" from embed parameters
					 * (2)remove the corresponding token from finalBaseURL8
					 * e.g. from finalBaseURL8 = "/token1/token2" remove "/token2"
					 */

					baseEmbedParam8Ptr.Delete(0,tokenSeparatorPosition+1);

					TInt tokenStart = finalBaseURL8Ptr.LocateReverseF('/');
					finalBaseURL8Ptr.Delete(tokenStart,(finalBaseURL8Ptr.Length() - tokenStart));//(2)
				}    				
				else // neither of the above means some valid url token follows : e.g : "token"
				{
					/* 
					 * remove the "token/" from base embed param (baseEmbedParam8Ptr)
					 * append '/' and then "token" (from parsed data buffer) to finalBaseURL8
					 * if finalBaseURL8 was "/token1" -> becomes: "token1/token"
					 */
					baseEmbedParam8Ptr.Delete(0, tokenSeparatorPosition + 1);

					finalBaseURL8Ptr.Append('/');						
					finalBaseURL8Ptr.Append(parsedDataBuffer8Ptr.Left(tokenSeparatorPosition));
				}

				// Check if another round of parsing is required:
				if (finalBaseURL8Ptr.Length() ==  0 || baseEmbedParam8Ptr.Length() == 0)
				{
					isParseComplete = ETrue;
				}

			}

			/* 
			 * Now parsing is complete, we have reqd base url = domain + finalBaseURL8
			 * the whole Absolute Base URL is to be held in finalBaseURL8.
			 * domain will be of the form : "http://www.abc.com"
			 * finalBaseURL8 will contain rest of the URL + base embed param and will
			 * have the form: "/token1/token2"
			 */

			// insert domain at the front:
			finalBaseURL8Ptr.Insert(0, currentPageDomain8Ptr);

			// last character will NOT be slash, so append one
			finalBaseURL8Ptr.Append('/');

			// cleanup (5)parsedDataBuffer8
			CleanupStack::PopAndDestroy(parsedDataBuffer8);
		}

		// finally, copy this to iBaseURL and ZeroTerminate:
		iBaseURL = HBufC::NewL(finalBaseURL8Ptr.Length() + 1);
		TPtr baseUrlPtr = iBaseURL->Des();
		baseUrlPtr.Zero();
		baseUrlPtr.Copy(finalBaseURL8Ptr);
		baseUrlPtr.ZeroTerminate();  

		// cleanup (4)baseEmbedParam8 and (3)finalBaseURL8
		CleanupStack::PopAndDestroy(finalBaseURL8);
		CleanupStack::PopAndDestroy(baseEmbedParam8);
	}
	else // base embed params is empty: Use this current page URL as BaseURL
	{
		iBaseURL = HBufC::NewL(currentPageURL8->Length() + 1);
		TPtr baseUrlPtr = iBaseURL->Des();
		baseUrlPtr.Zero();
		baseUrlPtr.Copy(currentPageURL8Ptr);
		baseUrlPtr.ZeroTerminate();
	}   
	// cleanup (2)currentPageDomain8 and (1)currentPageURL8
	CleanupStack::PopAndDestroy(currentPageDomain8);
	CleanupStack::PopAndDestroy(currentPageURL8);
	NPPDebug(LOG_INFO,_L("CreateBaseURLL(): Absolute Base URL formed: %S\n"), iBaseURL);
}


// -------------------------------------------------------------------------
// void CBrowserPluginInstance::IsRelativeBaseUrl
// Used to check if the base url obtained from "base" param is relative or not
// --------------------------------------------------------------------------
//
TBool CBrowserPluginInstance::IsRelativeBaseUrl()
{
	if(iBaseURL)
	{
		if((iBaseURL->Find(KLitFile) == KErrNotFound) 
	    	&& (iBaseURL->Find(KLitHttp) == KErrNotFound)
	    		 && (iBaseURL->Find(KLitHttps) == KErrNotFound))
		{
			iIsRelativeBaseUrl = ETrue;
			return ETrue;
		}
		else
		{
			iIsRelativeBaseUrl = EFalse;
			return EFalse;
		}
	}
	return EFalse;
}

// -------------------------------------------------------------------------
// void CBrowserPluginInstance::OpenFileL(const TDesC& aFilename, const TDesC* aUrl)
// Used to open a SWF file
// --------------------------------------------------------------------------
// 
void CBrowserPluginInstance::OpenFileL(const TDesC& aFilename, 
                                       const TDesC& aUrl, 
                                       const TBool& aFirstMovie)
{
    if(iErrorOccured)
    {
        return;
    }
        	
    if (IsControlValid())
    {
        iSavedFileNames->AppendL(aFilename);
    }
        
    if(!iBaseURL || IsRelativeBaseUrl())
    {
        CreateBaseURLL(aUrl);    
    }
    
    CFlashPluginInstance::OpenFileL(aFilename, aUrl, aFirstMovie); 


    	
    /*
    // if not in focus, pause the player. 
    if( (!iControl->IsVisible()) !IsFocused() )
    {
    	HandleFocusLostL();    	
    }*/
}

    
// -------------------------------------------------------------------------
// void CBrowserPluginInstance::HandleGetCommandL(NPPVariable aVariable, 
//                    void* ret_value)
// Used to Handle NP get_Val commands from client app
// --------------------------------------------------------------------------
//
NPError CBrowserPluginInstance::HandleGetCommandL(int aVariable, 
                    void* ret_value)
{
	switch (aVariable)
	{
	// Commenting it out as at present __FEATURE_BROWSER_INTERACTIVE_BOOL flag is undefined across all the platforms
	/*#ifdef FEATURE_BROWSER_INTERACTIVE_BOOL
		case NPPVpluginInteractiveBool://flash plug-in is interactive
        {
            TInt* error = (TInt*) ret_value;
            if(!error)
            {
            	*error = 0;	
            }
            
            break;
        }
    #endif*/
   
   
		case KNPPVPluginOpenInViewer:
		{
			if(ret_value)
    		{
    			*((TBool*) ret_value) = EFalse;	
    		}
		}
		break;
    	default :
    	{
    		if(ret_value)
    		{
    			*((TBool*) ret_value) = EFalse;	
    		}
    		
    	}
    	break;
	}

	return KErrNone; //NPERR_NO_ERROR;;
}

// -------------------------------------------------------------------------
// void CBrowserPluginInstance::HandleSetCommandL(NPNVariable /* aVariable*/, 
//                    void* /*ret_value*/) 
// Used to Handle NP set_Val commands from client app
// --------------------------------------------------------------------------
//

NPError CBrowserPluginInstance::HandleSetCommandL(int /* aVariable*/, 
                                                  void* ret_value) 

{	
	if(ret_value)
	{
		*((TBool*) ret_value) = EFalse;	
	}
	return KErrNone; //NPERR_NO_ERROR;;
}

CDesCArray* CBrowserPluginInstance::ArgumentNames()
    {
    if( !iPluginInfo )
        return NULL;
    
    return iPluginInfo->ArgumentNames();
    }
    
CDesCArray* CBrowserPluginInstance::ArgumentValues()
    {
    if( !iPluginInfo )
        return NULL;
    
    return iPluginInfo->ArgumentValues();
    }

void CBrowserPluginInstance::HandleFocusLostL()
{
    if( !IsControlValid() || iErrorOccured )
    {
        return;
    }

    // Engine init happened; but file hasnt been opened.
    // So we have to wait a little longer to delete the engine.
    
    if (!iPluginView)
        {
        return;
        }
    if(!iPluginView->IsFileOpened())
        {
        return;
        }
    
    MPluginView* plgControl = iPluginView;
        
#ifdef HANDLE_PLUGIN_INSTANCE        

    //TODO: Revisit..seems to be not used..Modify for windowless
    //iControl->DestroyEngine();
    plgControl->DestroyEngine();
    
#else
    
    
    
    // Pause only if audio is not playing in the flash content.        
    // for FLV videos anycase do pause
    //if( (!iControl) && (!iControl->IsFLVVideo() ) && iControl->IsSoundPlaying() )
    //if( (!iControl) && (!iControl->IsFLVVideo() ) && IsSoundPlaying() )
    
    if(plgControl)  
        if(plgControl->IsFLVVideo()) 
            if(IsSoundPlaying() )
                {
                NPPDebug(LOG_INFO,_L("CPlayerInstance::HandleFocusLostL() : Sound is present so the playback is not paused!"));
                }
    
    NPPDebug(LOG_INFO,_L("CPlayerInstance::HandleFocusLostL() call NotifyL:EApplicationFocusChanged"));
    
//    if(iControl)
//        iControl->NotifyL(MPluginNotifier::EPluginInvisible, param); 
    
    
#endif
}
    
/*
void CBrowserPluginInstance::CreateControlEngineL()
    {
    if( iErrorOccured ) return; 
        
    if( !iPluginInfo ) User::Leave(KErrGeneral);
    
    if( IsControlValid() ) return;
    
    HBufC* fileName = REINTERPRET_CAST(HBufC*, iPluginInfo->FileName());
    HBufC* url      = REINTERPRET_CAST(HBufC*, iPluginInfo->Url());
    
    if(iControl)
        iControl->CreateEngineAndPlayL(*fileName, ETrue, url, ETrue);
    }
*/

//-----------------------------------------------------------------------
// MPluginScriptHandler* CBrowserPluginInstance::GetHostScriptHandler()
//
// This function returns script handler object
//-----------------------------------------------------------------------

 #ifdef FEATURE_WEB_BROWSABILITY

MHostScriptHandler* CBrowserPluginInstance::GetHostScriptHandler()
{
	if(iEngine)
	    return iEngine->GetScriptHandler();
    /*if (0 == iWindowLessMode)
        {
        if (iControl)
            return iControl->GetPlayerEngine()->GetScriptHandler();
        }
    else
        {
        return this->iWlessCtrl->GetPlayerEngine()->GetScriptHandler();
        }*/
    return NULL;
}

 
TBool CBrowserPluginInstance::PercentLoaded(TReal& aPercentLoaded)
{
	if(!IsControlValid())
	{
		aPercentLoaded = 0;	
		return ETrue;
	}
	if( iContentLength == iLengthWritten )
	{
		aPercentLoaded = 100;	
		return ETrue;
	}
	else if( iContentLength )
	{
		aPercentLoaded = ((TReal)iLengthWritten)/(TReal)iContentLength;		
		return ETrue;
	}
	else
	{
		return EFalse;
	}		
}

#endif //FEATURE_WEB_BROWSABILITY

TBool CBrowserPluginInstance::OpenUrlL(const TDesC& aUrl, const TDesC& aTarget, const TDesC& aPostData)
    {
#ifdef __FEATURE_PLUGIN_UI_OPTIONS
    if(aUrl.CompareF(KFullScreenURLReq) == 0)    //youtube specific?
    { 
        // full screen button is clicked
        // In FP8 , when fullsceen button is clicked it opens another instance
        // In FP9, adobe has introduced fullscreen state, here 
        // it toggles b/w state. TODO: developer needs to remember when writng for FP9
#ifdef FEATURE_WINDOWLESS       
        if(EWindow==iWindowMode)
#endif //FEATURE_WINDOWLESS         
            if (iPluginView)
                iPluginView->HandleURLReqForFullScreen(aUrl);
        return KErrNone; //NPERR_NO_ERROR;;;
    }       
#endif  
        // Request the browser to navigate to a url
        // (thus closing the page containing the plug-in)
        // function is moved from plugininstance to here to reduce one function stack call
    MNetscapeIf* nsfuncs = NetscapeFns();

    NPError error = KErrNone; //NPERR_NO_ERROR;;

    if (aPostData.Length() == 0)
    {
		if(aTarget.Compare(_L("")) == 0)
			error = nsfuncs->GetUrl(aUrl, &KDefaultTargetWindow);
		else
			error = nsfuncs->GetUrl(aUrl, &aTarget);
    }
    else
    {
        //Set up a buffer for the header and the data
        HBufC* postdata =  NULL;//who will delete this memory??
        error = FillPostDataL(aPostData,postdata);//postdata allocated memory here
        
        CleanupStack::PushL(postdata);
        
        if(error == KErrNone/*NPERR_NO_ERROR*/)
        {
            const TDesC& buf = postdata->Des();
            if(aTarget.Compare(_L("")) == 0)
                error = nsfuncs->PostUrl(aUrl, &KDefaultTargetWindow,buf, false);
            else
                error = nsfuncs->PostUrl(aUrl, &aTarget,buf, false);
        }
        CleanupStack::PopAndDestroy(postdata);
    }

    return (error == KErrNone /*NPERR_NO_ERROR*/);

    }

TBool CBrowserPluginInstance::OpenUrlL(const TDesC& aUrl, const TDesC& aPostData)
{
#ifdef __FEATURE_PLUGIN_UI_OPTIONS
        
	if(aUrl.CompareF(KFullScreenURLReq) == 0)    //youtube specific?
	{ 
		// full screen button is clicked
		// In FP8 , when fullsceen button is clicked it opens another instance
		// In FP9, adobe has introduced fullscreen state, here 
		// it toggles b/w state. TODO: developer needs to remember when writng for FP9
#ifdef FEATURE_WINDOWLESS		
		if(EWindow==iWindowMode)
#endif //FEATURE_WINDOWLESS			
            if (iPluginView)
                iPluginView->HandleURLReqForFullScreen(aUrl);
		return KErrNone; //NPERR_NO_ERROR;;;
	}
        
        //TODO : Revisit this part once refactoring is over
        /*
        if(iFLVPlayState == EFlashPluginFLVStreamClosed)    
        {
            #ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
            if(iFullScreenHandler && IsFullScreen())    
            {
                iFullScreenHandler->IssueRequest(EFalse);
            }
            #endif//#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
        }
       */
       
#endif  
        //Dont uncomment the below ones...
        /*
        if(IsFullScreen())
            { 
            if(GoNormalScreenL())
                        {
                            iFullScreen = EFalse;   
                        }
            }*/
            
        // Request the browser to navigate to a url
        // (thus closing the page containing the plug-in)
        // function is moved from plugininstance to here to reduce one function stack call
	MNetscapeIf* nsfuncs = NetscapeFns();

	NPError error = KErrNone; //NPERR_NO_ERROR;;

	if (aPostData.Length() == 0)
	{
        error = nsfuncs->GetUrl(aUrl, &KDefaultTargetWindow);
	}
	else
	{
		//Set up a buffer for the header and the data
		HBufC* postdata =  NULL;//who will delete this memory??
		error = FillPostDataL(aPostData,postdata);//postdata allocated memory here
		
		CleanupStack::PushL(postdata);
		
		if(error == KErrNone/*NPERR_NO_ERROR*/)
		{
			const TDesC& buf = postdata->Des();
			error = nsfuncs->PostUrl(aUrl, &KDefaultTargetWindow,
							 buf, false);
		}
		CleanupStack::PopAndDestroy(postdata);
	}

	return (error == KErrNone /*NPERR_NO_ERROR*/);
}

TBool CBrowserPluginInstance::OpenStreamL(unsigned short aStreamId, const TDesC& aUrl,
        const TDesC& aPostDataContentType,
        TInt aPostDataContentLength, const TDesC& aPostData)
{
    // Request the browser to stream data to flash
    //function is moved from plugininstance to here to reduce one function stack call
    MNetscapeIf* nsfuncs = NetscapeFns();
    NPError error = KErrNone; //NPERR_NO_ERROR;;

    if (aPostData.Length() == 0)
    {
        error = nsfuncs->GetUrlNotify(aUrl, NULL, (void*)aStreamId);
    }
    else
    {
        //Set up a buffer for the header and the data
        HBufC* postdata =  NULL; //Deletion should be checked.
        
        error = FillPostDataL(aPostDataContentType, aPostDataContentLength,aPostData,postdata);//postdata allocated memory here
        CleanupStack::PushL(postdata);
        
        if(error == KErrNone/*NPERR_NO_ERROR*/)
        {
				//TODO: We are passing NULL as target parameter, may cause problems, may have to change to default
            error = nsfuncs->PostUrlNotify(aUrl, NULL,postdata->Des(), false, (void*)aStreamId);
        }
        
        CleanupStack::PopAndDestroy(postdata);
    }
    return (error == KErrNone/*NPERR_NO_ERROR*/);
}

NPError CBrowserPluginInstance::FillPostDataL(const TDesC& aPostDataContentType,
                                             TInt         aPostDataContentLength,
                                             const TDesC& aPostData, 
                                             HBufC*&      aHttpPostdata)
{
    //Set up a buffer for the header and the data
    aHttpPostdata = HBufC::NewL( KHttpContentTypeTag().Size() +
                                 KHttpTagTerminator().Size() +
                                 aPostDataContentType.Size() +
                                 //KDefaultHeader().Size()+
                                  KContentLengthHeader().Size()+
                                  KMaxLenSizeString +
                                  aPostData.Size());

    if (!aHttpPostdata)
    {
        return NPERR_Out_Of_Memory; //NPERR_OUT_OF_MEMORY_ERROR;
    }
    else
    {
        //Set the header on top of the data to send.
        TPtr postDataWithHeader(aHttpPostdata->Des());
        //postDataWithHeader.Append(KDefaultHeader);
        postDataWithHeader.Append(KHttpContentTypeTag);     
        postDataWithHeader.Append(aPostDataContentType);        
        postDataWithHeader.Append(KHttpTagTerminator);      
        
        postDataWithHeader.Append(KContentLengthHeader);
        postDataWithHeader.AppendNum(aPostDataContentLength);
        postDataWithHeader.Append(KHeaderTerminator);
        postDataWithHeader.Append(aPostData);
    }
        
    return KErrNone; //NPERR_NO_ERROR;;  
}

NPError CBrowserPluginInstance::FillPostDataL(const TDesC& aPostData, 
                                                HBufC*& aHttpPostdata)
{
    //Set up a buffer for the header and the data
    aHttpPostdata = HBufC::NewL(  KDefaultHeader().Size()+
                                  KContentLengthHeader().Size()+
                                  KMaxLenSizeString +
                                  aPostData.Size());

    if (!aHttpPostdata)
    {
        return NPERR_Out_Of_Memory; //NPERR_OUT_OF_MEMORY_ERROR;
    }
    else
    {
        //Set the header on top of the data to send.
        TPtr postDataWithHeader(aHttpPostdata->Des());
        postDataWithHeader.Append(KDefaultHeader);
        postDataWithHeader.Append(KContentLengthHeader);
        postDataWithHeader.AppendNum(aPostData.Length());
        postDataWithHeader.Append(KHeaderTerminator);
        postDataWithHeader.Append(aPostData);
    }
        
    return KErrNone; //NPERR_NO_ERROR;;  
}


void CBrowserPluginInstance::SetStreamBufferSizeL(unsigned short aStreamId,
        unsigned long aBufferSize)
{
    CPluginStreamFactory* streamFactory =
            static_cast<CPluginStreamFactory*> (GetStreamHandler());
    if (streamFactory)
    {
        streamFactory->SetStreamBufferSizeL(aStreamId, aBufferSize);
    }
}

void CBrowserPluginInstance::CancelStream(unsigned short aStreamId)
{
    CPluginStreamFactory* streamFactory =
            static_cast<CPluginStreamFactory*> (GetStreamHandler());
    if (streamFactory)
    {
        streamFactory->CancelStream(aStreamId);
    }
}

TUint CBrowserPluginInstance::GetFeatureCapability()
{
    TUint deviceCapability = KCapabilitiesBrowser | FI_DeviceCap_XMLSocket
            | FI_DeviceCap_LoadData | FI_DeviceCap_SMS | FI_DeviceCap_MMS;
            /*| FI_DeviceCap_MappableSoftKeys*/ //Mappable softkeys are disabled in case of browser plug in hence commenting

    if (FeatureManager::FeatureSupported(KFeatureIdPenSupport)) //adding feature manager to check the feature at run time.
    {
        deviceCapability |= FI_DeviceCap_Stylus;
    }

#ifdef ENABLE_CURSOR
    TInt keyboardLayout = 0;
    RProperty::Get(KCRUidAvkon, KAknKeyBoardLayout, keyboardLayout);
    TPtiKeyboardType layout = (TPtiKeyboardType) keyboardLayout;
    //Cursor is shown only is keypad is available
    if (layout != EPtiKeyboardNone)
    {
        deviceCapability |= FI_DeviceCap_Mouse;
    }

#endif
    return deviceCapability;
}

void CBrowserPluginInstance::FindExecutionMode()
{
    MNetscapeIf* nsfuncs = NetscapeFns();
    iBrPluginExecutionMode = (EBrPluginExecutionMode)nsfuncs->ResolveExecutionMode();
  
}
void CBrowserPluginInstance::QueryAllowNetworkInWidgetMode()
{
        MNetscapeIf* nsfuncs = NetscapeFns();
        iAllowNetworkAccess = nsfuncs->QueryAllowNetworkInWidgetMode();
    
 
}
TBool CBrowserPluginInstance::QueryAllowNetworkAccess()
{
    return iAllowNetworkAccess;
}
TBool CBrowserPluginInstance::CanContentBeTrustedL(const TDesC& pAccessedDomain)
{
	TBool ret = EFalse;
    if(( iBrPluginExecutionMode == EWidgetMode) && iAllowNetworkAccess)
    {
        //In automation mode, security sandbox prompts should be disabled               
        #if defined(FEATURE_PERFORMANCE_AUTOMATION) || defined(FEATURE_ATS_AUTOMATION) || defined(FEATURE_TESTAUTOMATION)
        if((iConfig.iMode & KPlayerInATSMode) || (iConfig.iMode & KPerformanceTestMode))    
        {
            return EFalse;
        }
        #endif
  
    	ret = iPluginView->QueryTrustDomainL(pAccessedDomain);
    }
    
    return ret;
}

void CBrowserPluginInstance::KickoffRestartTimerL()
{
    if(!iRestartTimer)
    {
        iRestartTimer = CPlgTimerObject::NewL(CActive::EPriorityStandard, *this);
    }
    if(iRestartTimer)
    {
        iRestartTimer->Cancel();
        iRestartTimer->After(1);
    }	
}

void CBrowserPluginInstance::DoRunL(const CActive* aActive)
{
    if(aActive == iRestartTimer && iEngine)
    {
        iEngine->DestroyPlayer();
        //TODO
        //Need to check if DRM check needs to done before re-load
        iPluginView->OpenFile(*iFilename, iUrl->Des(), iFirstFlashMovie);
    }
}

TInt CBrowserPluginInstance::DoRunError(const CActive* /*aActive*/, TInt /*aError*/)
{
    return KErrNone;
}

////////////////////////////////////////////////////////////////
// CAsyncDeleter
///////////////////////////////////////////////////////////////    

// -------------------------------------------------------------------------
//CAsyncDeleter::CAsyncDeleter(CBrowserPluginInstance* aPlayer) 
// constructor
// --------------------------------------------------------------------------
//
CAsyncDeleter::CAsyncDeleter(CBrowserPluginInstance* aPlayer, MPluginView* aPluginView)
              : CActive(EPriorityStandard)
              
{
    CActiveScheduler::Add(this);
    iPlayer = aPlayer;
    iPluginView = aPluginView;
}

// -------------------------------------------------------------------------
// CAsyncDeleter::~CAsyncDeleter()
// destructor
// --------------------------------------------------------------------------
//    
CAsyncDeleter::~CAsyncDeleter()
{
    Cancel();
}

// -------------------------------------------------------------------------
// void CAsyncDeleter::RunL()
// Active object execution method
// --------------------------------------------------------------------------
//
void CAsyncDeleter::RunL()
{
    NPPDebug(LOG_DEBUG,_L("CAsyncDeleter::RunL() ++"));
    
    if( iStatus.Int() != KErrNone )
    {
        return;        
    }

    iPlayer->StartShutDownProcess();
            
    NPPDebug(LOG_DEBUG,_L("CAsyncDeleter::RunL() --"));
}

// -------------------------------------------------------------------------
// void CAsyncDeleter::IssueRequest()
// Used to Handle asynchronous request for deletion
// --------------------------------------------------------------------------
//
    
void CAsyncDeleter::IssueRequest()
{
    NPPDebug(LOG_DEBUG,_L("CAsyncDeleter::IssueRequest() ++"));
    
    if(IsActive())
    {
        return;
    }
        
    TRequestStatus* status = &iStatus;
    SetActive();
    User::RequestComplete(status, KErrNone);
    
    NPPDebug(LOG_DEBUG,_L("CAsyncDeleter::IssueRequest() --"));
}

// -------------------------------------------------------------------------
// void CAsyncDeleter::DoCancel()
// cancel active object
// --------------------------------------------------------------------------
//
void CAsyncDeleter::DoCancel()
{
}

// End of File
