/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CFlashPluginInstance implementation file
*
*/

#include "saplayer_build.h"
#include <content.h>
#include <eikenv.h>
#include <apgtask.h> // for TApaTask
#include "flashplugininstance.h"
#include "flashplgmain.h"
#include "commondef.h"
#include "commontypes.h"
#include <data_caging_path_literals.hrh>

#include <apgwgnam.h> // Added
#include "pluginmemhandlerimpl.h"
#include "pluginview.h"
#include "flashconfiggen.h"
#include "saflash_consts.h"
#include "streamfactory.h"
#include "deviceextnimpl.h"

using namespace ContentAccess;
extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );

_LIT16(KMimeType,"application/x-shockwave-flash");

const TInt KMinVolume = 0;  // Minimum volume level(Mute)
const TInt KMaxVolume = 10; // Maximum volume level

_LIT(KFlashDirectory, "101FD693");
_LIT(KProtocolPrefixFileUnixStyle, "file:///");

// -------------------------------------------------------------------------
// CFlashPluginInstance::CFlashPluginInstance(PluginGlobals* aGlobals, NPP aInstance, 
// uint16 /*aMode*/)
//
// First phase constructor
// --------------------------------------------------------------------------
// 
CFlashPluginInstance::CFlashPluginInstance(	MNetscapeIf* aNetscapeIf,CPluginGlobals* aGlobals
											,uint16 aMode
											,TInt aInteractionMode
#ifdef FEATURE_WINDOWLESS											
											,TWindowMode aWindowMode
#endif //FEATURE_WINDOWLESS											
											):
											iGlobals(aGlobals)
											,iInteractionMode(aInteractionMode)
											,iBrPluginExecutionMode(EDefaultMode)
											,iMode(aMode)
                                            ,iPluginView(NULL)
											,iCachedFileName (NULL)
    										,iCachedUrl (NULL)
											,iFirstMovie (ETrue)
											,iSwfSrcPath(NULL, 0)
											,iBaseURL(NULL)
											,iSecurityContextUrl(NULL)
											,iAllowMobiFullScreen(ETrue)	
											,iPluginMemHandler(NULL)
											,iSideVolumeKeysSupported(EFalse)
											,iCenRepos(NULL)
											,iHwKeyObserverAdded(EFalse)
											,iVolumeCenRepKeyWatcher(NULL)
#ifdef SELF_IAD_UPDATE
											,iIadProgressState(EIadUpdateNotComplete)
#endif										
#ifdef FEATURE_WINDOWLESS											
											,iWindowMode(aWindowMode)
#endif //FEATURE_WINDOWLESS											
											
{
    iNetScapeIf=aNetscapeIf;
	iFs = CCoeEnv::Static()->FsSession();
}

// -------------------------------------------------------------------------
// CFlashPluginInstance::CFlashPluginInstance()
//
// Destructor
// --------------------------------------------------------------------------
//
CFlashPluginInstance::~CFlashPluginInstance()
{
    DestroyEngine();
    
    RemoveHwSideKeyObserver();
        
    DestroyStreamHandler();
    
	if(iVolumeCenRepKeyWatcher)
	{
		delete iVolumeCenRepKeyWatcher;
		iVolumeCenRepKeyWatcher = NULL;
	}

    if (iPluginView)
    {
		iPluginView->Close();
    }
	
    if (iCachedFileName)
    {
        delete iCachedFileName;
        iCachedFileName = NULL;
    }

    if (iCachedUrl)
    {
        delete iCachedUrl;
        iCachedUrl = NULL;
    }
    
    if(iBaseURL)
    {
        delete iBaseURL;
        iBaseURL = NULL;
    }       
    
    if(iFilename)
    {
        delete iFilename;
        iFilename = NULL;
    }
    
    if(iUrl)
    {
        delete iUrl;
        iUrl = NULL;
    }
    
    if(iCenRepos)
    {
    	delete iCenRepos;
    	iCenRepos = NULL;
    }
    
    if(iDeviceExtn)
    {
		delete iDeviceExtn;
		iDeviceExtn = NULL;
    }
    
    if(iPluginMemHandler)
    {
    	delete iPluginMemHandler;
    	iPluginMemHandler = NULL;
    }
    
	if (iGlobals)
	{
		iGlobals->DeletePluginGlobal();
		iGlobals = NULL;
	}
	if(iLibrary.Handle())
	{
        iLibrary.Close();
	}
}

typedef MPluginView* (*PFNC_CreatePluginView)(MFlashPluginInstance*,TWindowMode);

_LIT(KFlashAvkonUILibraryName, "aknflashpluginui.dll");
_LIT(KFlashQtUILibraryName,"qtflashpluginui.dll");

const TInt KFlashCreatePluginViewOrdinal = 1;

MPluginView* CFlashPluginInstance::DoCreatePluginViewL(TPluginUiFramework aPluginUiFramework)
{
    if(EQtFrameWork == aPluginUiFramework )
    {
        User::LeaveIfError(iLibrary.Load(KFlashQtUILibraryName));
    }
    else
    {
        User::LeaveIfError(iLibrary.Load(KFlashAvkonUILibraryName));
    }
    
    PFNC_CreatePluginView pFncCreateFlashPluginView =
            (PFNC_CreatePluginView) iLibrary.Lookup(
                    KFlashCreatePluginViewOrdinal);
    if (pFncCreateFlashPluginView == NULL)
        {
        User::Leave(KErrGeneral);
        }
    return (MPluginView*) pFncCreateFlashPluginView(this, iWindowMode);
}

MPluginView* CFlashPluginInstance::CreatePluginView(TPluginUiFramework aPluginUiFramework)
{
	MPluginView* pluginView = NULL;
	TRAP_IGNORE(pluginView = DoCreatePluginViewL(aPluginUiFramework));
	return pluginView;
}

void CFlashPluginInstance::CreateEngineL()
{
    if (!iEngine)
    {
        CFlashConfigGen::FillFlashConfig( iInteractionMode,iConfig);
        if(iBrPluginExecutionMode == EWidgetMode)
        	ExtendConfigforWidgetMode();
        
        iEngine = CreateFlashEngineL(iFELib);
        iEngine->ConstructL(iConfig,this,ePlugin,iBrPluginExecutionMode);
    }
}

void CFlashPluginInstance::ExtendConfigforWidgetMode()
{
    iConfig.iMode |= KSupportLocalFile;
    //Flash contents in widget are trusted by default
    iConfig.iMode |= KSupportTrustedSandBox;
}

void CFlashPluginInstance::DestroyEngine()    
{
    if (iEngine)
    {
        iEngine->DestroyPlayer();
        iEngine->Destroy();
        iEngine = NULL;
    }    
        
    if(iFELib.Handle())
    {
        iFELib.Close();
    }
	if(iNetScapeIf)    
		iNetScapeIf->DeleteFlashMethodObject();
}

CPluginStreamFactory* CFlashPluginInstance::GetStreamHandler() 
{
    return iStreamFactory;
}

void CFlashPluginInstance::DestroyStreamHandler()
{
    if (iStreamFactory)
    {
        delete iStreamFactory;
        iStreamFactory = NULL;
    }
}

// -------------------------------------------------------------------------
// CFlashPluginInstance::HandleGestureL(void* aEvent) 
//
// handle the gesture event sent by browser
// --------------------------------------------------------------------------
//
/*
#ifdef GESTURE_LIBRARY_SUPPORTED
TBool CFlashPluginInstance::HandleGestureFromBrowserL(void* aEvent)
{
    if(EWindow==iWindowMode)
        return ((CFlashPluginControl*)iPluginView)->HandleGestureFromBrowserL(aEvent);
   // else
     //   return iWlessCtrl->HandleGestureFromBrowserL(aEvent);
    return NULL;
}
#endif //GESTURE_LIBRARY_SUPPORTED
*/

// -------------------------------------------------------------------------
// CFlashPluginInstance::isFlashContentL(CContent* aContent) 
//
// check whether content is flash swf
// --------------------------------------------------------------------------
//
TBool CFlashPluginInstance::isFlashContentL(const CContent* aContent) 
{
	TBool isFlash = EFalse;
		
	if(aContent)
	{
    	HBufC* tmpMIME = HBufC::NewLC( KMaxDataTypeLength );

    	TPtr mimePtr( tmpMIME->Des() );            

    	aContent->GetStringAttribute(EMimeType, mimePtr );

      	isFlash =  (0 != mimePtr.Compare(KMimeType))? EFalse : ETrue ;

      	CleanupStack::PopAndDestroy(tmpMIME);
    }

    return isFlash;
}

// -------------------------------------------------------------------------
// CFlashPluginInstance::SetSwfSourcePath() 
//
// Sets the source path of swf file which can be used to determine the BaseURL
// --------------------------------------------------------------------------
//

void CFlashPluginInstance::SetSwfSourcePath(const TDesC& aSwfSrcPath)
{
    iSwfSrcPath.Set(aSwfSrcPath);
}

void CFlashPluginInstance::SetBaseUrlL(const TDesC& aBaseUrl)
{
    //if( aBaseUrl.Length() <= 0 || !iControl  || !iWlessCtrl )
    if( aBaseUrl.Length() <= 0 )
    {
        return;
    }
    
    if( iBaseURL )
    {
        delete iBaseURL;
        iBaseURL = 0;
    }
    
    iBaseURL = HBufC::NewL(aBaseUrl.Length() + 1);
    TPtr baseUrlPtr = iBaseURL->Des();
    baseUrlPtr.Copy(aBaseUrl);
    baseUrlPtr.ZeroTerminate();
}

CDRMHelper* CFlashPluginInstance::GetDRMHelperL()
{
	return NULL;
}

CPluginGlobals* CFlashPluginInstance::GetGlobals()
{
	return iGlobals; 
}
	
// -------------------------------------------------------------------------
// TBool CFlashPluginInstance::IsControlValid()
//
// Check if this is a valid control
// --------------------------------------------------------------------------
// 
TBool CFlashPluginInstance::IsControlValid()
{
    return iPluginView->IsValid();
    
//    return EFalse;        
}

// -------------------------------------------------------------------------
// void CFlashPluginInstance::PrepareToExitL()
//
// Called in the destructor to notify the Browser
// --------------------------------------------------------------------------
// 
void CFlashPluginInstance::PrepareToExitL()
{
	iPluginView->PrepareToExit(); //below code needs to be moved to pluginUI
	/*
	 if (iPlugin)
	 {
	 iPlugin->PluginFinishedL();
	 iPlugin = NULL;                       
	 }
	 */
}

// -------------------------------------------------------------------------
// void CFlashPluginInstance::OpenFileL(const TDesC& aFilename, const TDesC* aUrl)
// Used to open a SWF file if plugin is in foreground and in visible state.
// --------------------------------------------------------------------------
// 
void CFlashPluginInstance::OpenFileL(const TDesC& aFilename, 
                                     const TDesC& aUrl, 
	                                 const TBool& aFirstMovie)
{
	iIsPlayerIntialised = EFalse;
	if(iFilename)
	{
		delete iFilename;
		iFilename = NULL;
	}
	if(iUrl)
	{
		delete iUrl;
		iUrl = NULL;
	}
	iFilename = aFilename.AllocL();
	iUrl = aUrl.AllocL();
	iFirstFlashMovie = aFirstMovie;
	
	// Do not open the swf file if plugin is in background or 
	// invisible state. Swf file will be opened whenever plugin
	//comes to ForeGround and in visible state through NotifyL function
	// of CBrowserPluginControl class. fix for ASHA-7RKHZT
	TApaTask task(CEikonEnv::Static()->WsSession( ));
	task.SetWgId(CEikonEnv::Static()->RootWin().Identifier());
	TApaTaskList tasklist(CEikonEnv::Static()->WsSession());
	TApaTask task1 = tasklist.FindByPos(0); 

#ifdef FEATURE_WINDOWLESS	
	if(EWindow==iWindowMode)
#endif //FEATURE_WINDOWLESS		
	{
		CApaWindowGroupName* task1App = CApaWindowGroupName::NewLC(CEikonEnv::Static()->WsSession(),task1.WgId()); // 
		if(task.WgId() != task1.WgId() && task1App->AppUid().iUid != 0)
		{                
			iPluginView->SetPluginFocus(EFlashPluginBackground);
			iDeferredLaunch = ETrue;
			CleanupStack::PopAndDestroy(); // task1App			
			return;
		}
		//else if (iControl->IsControlVisible()) // Not using ! operator as with this check as EFlashPluginInvisible is kept at 1 position
        /* ReTe failure: In case of SWF size as (0,0) we do not get NotifyL() call from Browser,  
         * to call Engine->OpenFileL. Hence, we cannot defer the SWF opening with size as (0,0) */
		else if (!iPluginView->IsVisible() && !(iPluginView->GetSWFRect().IsEmpty()))
        {
            iDeferredLaunch = ETrue;
			CleanupStack::PopAndDestroy(); // task1App			
			return;
        }
		CleanupStack::PopAndDestroy(); // task1App			
	}
	
	// open the swf file
	User::LeaveIfError(OpenMovie());
}
    
// -------------------------------------------------------------------------
// void CBrowserPluginInstance::CacheFileNameL(const TDesC& aFilename, 
// const TDesC& aUrl)

// Used to cache a filename so that it can be played later after the display
// rectangle has been specified.
// --------------------------------------------------------------------------
// 
void CFlashPluginInstance::CacheFileNameL(const TDesC& aFilename, 
                                          const TDesC* aUrl,
                                          const TBool& aFirstMovie)
{
    iFirstMovie = aFirstMovie;
    iCachedFileName = aFilename.AllocL();
    if (aUrl)
    {
        iCachedUrl = aUrl->AllocL();            
    }
}
    
int CFlashPluginInstance::NewStream(const TDesC8& aType, NetscapeStream* aStream, bool seekable,
            unsigned short* stype)
{
	CPluginStreamFactory* streamHandler = this->GetStreamHandler();

	NPAPIStreamData* streamData = static_cast<NPAPIStreamData*> (aStream);

	TInt err(KErrNone);

	if (streamHandler)
	{
		TRAP(err,streamHandler->NewStreamL(aType
						,streamData
						,seekable
						,stype));
	}
	return err;
}

int CFlashPluginInstance::DestroyStream(NetscapeStream* aStream,
										NetscapeReason aReason)
{
	NPAPIStreamData* streamData = static_cast<NPAPIStreamData*> (aStream);

	TInt streamId = REINTERPRET_CAST(TInt, streamData->notifyData);

	NPError err = KErrNone; //NPERR_NO_ERROR;

	CPluginStreamFactory* streamHandler = this->GetStreamHandler();

	if (streamHandler)
	{
		TRAP_IGNORE(err = streamHandler->DestroyStreamL(streamId, aReason));
	}

	return err;
}

int CFlashPluginInstance::AsFile(NetscapeStream* aStream, const TDesC& aFname)
{
	int err = KErrNone;
	NPAPIStreamData* streamData = static_cast<NPAPIStreamData*> (aStream);

	if (!streamData)
	{
		// NOTE: This is specific to screensaver.
		TRAP(err,OpenFileL(aFname, KNullDesC(), ETrue));
	}
	else
	{
		TInt streamId = REINTERPRET_CAST(TInt, streamData->notifyData);
		TBool firstMovie = (streamId == 0);

		// Pass file name to plugin, also url so flash can set its base url
		if (firstMovie)
		{

			TRAP(err,OpenFileL(aFname, streamData->url->Des(), firstMovie));
		}
	}

	return err;
}

int CFlashPluginInstance::WriteReady(NetscapeStream* aStream)
{
	TInt result = 0;

	NPAPIStreamData* streamData = static_cast<NPAPIStreamData*> (aStream);

	CPluginStreamFactory* streamHandler = this->GetStreamHandler();

	if (streamHandler)
	{
		TRAP_IGNORE(result = streamHandler->WriteReadyL(streamData));
	}
	else
	{
		result = 1;
	}

	return result;
}

int CFlashPluginInstance::WriteStream(NetscapeStream* aStream, int aOffset,
		int aLen, void* aBuffer)
{
	int result = -1;

	NPAPIStreamData* streamData = static_cast<NPAPIStreamData*> (aStream);

	CPluginStreamFactory* streamHandler = GetStreamHandler();

	if (streamHandler)
	{
		TRAP_IGNORE(result = streamHandler->WriteStreamL(streamData,aOffset,aLen,aBuffer));
	}

	// If -1 (or any negative number) is returned here, the browser calls my_destroystream.
	// This is acceptable if either URLStreamWriteL leaves or returns -1.

	return result;
}

int CFlashPluginInstance::UrlNotify(NetscapeReason aReason, void* notifyData)
{
	//const int NPRES_NETWORK_ERR = 1;
	if (aReason == 1) //NPRES_NETWORK_ERR)
	{
		// Only call this for NPRES_NETWORK_ERR - NPRES_DONE and 
		// NPRES_USER_BREAK should be handled by my_destroystream

		TInt streamId = REINTERPRET_CAST(TInt, notifyData);

		CPluginStreamFactory* streamHandler = this->GetStreamHandler();

		if (streamHandler)
		{
			TRAP_IGNORE(streamHandler->DestroyStreamL(streamId,aReason));
		}
	}
	return KErrNone; //NPERR_NO_ERROR;
}

int CFlashPluginInstance::GetValue(NetscapeVariable variable, void* ret_value)
{
	TInt ret_val = KErrNone; //NPERR_NO_ERROR;
	TRAPD(err, ret_val = this->HandleGetCommandL(variable, ret_value));
	if (err != KErrNone)
	{
		return NPERR_GENERIC_ERROR;
	}

	return ret_val;
}
int CFlashPluginInstance::SetValue(NetscapeVariable variable, void* ret_value)
{
	TInt ret_val = KErrNone; //NPERR_NO_ERROR;
	TRAPD(err, ret_val = HandleSetCommandL(variable,ret_value));

	if (err != KErrNone)
	{
		return NPERR_GENERIC_ERROR;
	}

	return ret_val;
}

int CFlashPluginInstance::SetWindow(NetscapeWindow* aWindow)
{
	//    NPWindow* window = (NPWindow*)aWindow;
	//TRAPD(err,this->SetWindowL(window));
	//return err;
	if (iPluginView)
		return iPluginView->SetWindow(aWindow);
	else
		return -1; //return an identifier that the view is not yet created
}

int CFlashPluginInstance::HandlePluginEvent(void* event)
{
	if (iPluginView)
		return iPluginView->HandlePluginEvent(event);

	return KErrNone; //NPERR_NO_ERROR;
}


// -------------------------------------------------------------------------
// void CFlashPluginInstance :: ProcessParameters ( CDesCArray* argnames, 
// CDesCArray* argvalues )
//
//
// Processes the parameters passed to the HTML tag
// --------------------------------------------------------------------------
// 
void CFlashPluginInstance :: ProcessParameters ( CDesCArray* argnames, CDesCArray* argvalues )
{
    if(iEngine)
        iEngine->ProcessParameters(argnames, argvalues);
}
    
// -------------------------------------------------------------------------
// void CFlashPluginInstance :: HandleGetCommandL(NPPVariable aVariable, 
//                    void* ret_value)
//
//
// Processes the get_Value commands
// --------------------------------------------------------------------------
// 
NPError CFlashPluginInstance::HandleGetCommandL(int aVariable,
                    void* ret_value)
{
	switch (aVariable)
	{
		case  NPPCheckFileIsValid:
		{
			if(ret_value)
        	{
        		NPCheckFile* checkFile = (NPCheckFile*)ret_value;
        		checkFile->fileValidityCheckError = KErrNone;
        		TFileName fileName = checkFile->fileName->Des();

        		TInt err=0;
        		TInt ret_val=0;            	
        		
        		TRAP (err, ret_val = CheckFileValidityL(fileName));
        		
        		checkFile->fileValidityCheckError = ret_val;

        		if(err != KErrNone)
            	{
				return NPERR_Invalid_Url; //NPERR_INVALID_URL;
            	}
        	}	
		}
		break;
			
		case  NPPGetLastError :
    	{
    		TInt* error = (TInt*)ret_value;
    		*error = GetLastError();
    	}
        break;
        
		case NPPGetPluginStatus :
			{
                TInt* status = (TInt*)ret_value;
                *status = State();
    		}
    		break;
    		
    	case NPPGetCurFrame:
    		{
                TInt* curFrame = (TInt*)ret_value;
                *curFrame =  GetCurrentFrameNo();
    		}
    		break;
	    
	    case NPPGetTotalFrames:
	    	{
    			TInt* totalFrame = (TInt*)ret_value;
    			*totalFrame = GetTotalFrames();
	    	}
    		break;
	}

	return KErrNone; //NPERR_NO_ERROR;
}
  
// -------------------------------------------------------------------------
// void CFlashPluginInstance :: HandleSetCommandL(NPNVariable aVariable, 
//                    void* ret_value)
//
//
// Processes the set_value commands
// --------------------------------------------------------------------------
//                     
NPError CFlashPluginInstance::HandleSetCommandL(int aVariable,
                    void* ret_value) 
{
    switch(aVariable)
    {
    	case  NPNInteractionMode :
			//NPNInteractionMode specific function handles needs to be added.
        	{
          	}
            break;
    	
    	case NPNVolumeEvent:
    	{//set volume event
    		if(ret_value)
    		{
    			TInt volume = *((TInt*)ret_value);
    			SetVolumeToEngine(volume);
    		}
    		
    	}
    	break;
    	
		case NPNAnimationEvent :
            NPAnimationEvent animEvent = ( NPAnimationEvent )(*(NPAnimationEvent*)ret_value);
            
			switch (animEvent)
			{					
				case  NPStartAnimation:
				    Start();
        			break;
                			
        		case  NPEndAnimation :                		
        		    End();
        			break;
          	  		
        		case NPPPauseAnimation:            			
        		    Pause(EReasonUser);
        			break;
            			
				case NPPResumeAnimation:
				    Resume(EReasonUser, ETrue);
        			break;
            			
    			case NPPZoomInAnimation:    	        		
    			    Zoom(EZoomIn);
    				break;
    	        		
    			case NPPZoomOutAnimation:    	        	
    			    Zoom(EZoomOut);
    				break;
            			
    			case NPPStopAnimation:
    			    Stop();
    				break;
    	        		
    			case NPPRewindAnimation:
    			    Rewind();
    				break;
    	        		
				case NPPMuteAnimation:
				    MuteVolume();
					break;
    	        		
				case NPPUnmuteAnimation:
				    UnMuteVolume();
					break;
			
				default:
					return -1;
			}//switch (animEvent)
			break;
	}//switch (aVariable)
	
	return KErrNone; //NPERR_NO_ERROR;
}


// -------------------------------------------------------------------------
// TBool CFlashPluginInstance::PerformDrmChecksL(const TDesC& aFilename, 
//                                            TBool& aCorrectMimeAndRights)
// Used to perform checks related to DRM
// aFileName - DRM protected file
// aCorrectMimeAndRights - ETrue if the file has correct MIME type and it has
//                          adequate playrights, EFalse otherwise
// Returns ETrue if file is DRM protected SWF file and EFalse otherwise 
// --------------------------------------------------------------------------
//
TBool CFlashPluginInstance::PerformDrmChecksL(const TDesC& /*aFilename*/, 
                                              const CContent* aContent,
                                              TBool& aCorrectMimeAndRights)
{
    TBool isProtected     = EFalse;
    aCorrectMimeAndRights = EFalse;
    
    aContent->GetAttribute( EIsProtected, isProtected );
    
    if (isProtected && (CFlashPluginInstance::isFlashContentL(aContent)))
    {
        aCorrectMimeAndRights = CheckPlayRightsL(aContent);
	    
	    return ETrue;
    }

    return EFalse;
}

// -------------------------------------------------------------------------
// void CFlashPluginInstance::CheckPlayRightsL(const TDesC& aFilename)
// Used to check if the DRM file has the required play rights
// --------------------------------------------------------------------------
// 
TBool CFlashPluginInstance::CheckPlayRightsL(const TDesC& aFilename)
{
    TBool ret = EFalse;
    ContentAccess::RAttributeSet attributeSet;
    CleanupClosePushL(attributeSet);
    attributeSet.AddL(ECanExecute);

    CManager *manager = CManager::NewLC();

    // Retrieve the attribute values from the agent
    User::LeaveIfError(manager->GetAttributeSet(attributeSet, aFilename));

    // Check if the content object is protected
    TInt attributeValue;
    // Check if the content object can be display on screen
    TInt err = attributeSet.GetValue(ECanExecute, attributeValue);
    if(err == KErrNone && attributeValue)
        {
        // content object has rights that allow it to be displayed on screen
        ret = ETrue;
        }

    CleanupStack::PopAndDestroy(manager);
    CleanupStack::PopAndDestroy(); //attributeset

    return ret;
} 
    
// -------------------------------------------------------------------------
// void CFlashPluginInstance::CheckPlayRightsL(CContent* aContent)
// Used to check if the DRM file has the required play rights
// --------------------------------------------------------------------------
// 
TBool CFlashPluginInstance::CheckPlayRightsL(const CContent* aContent)
{
    if( !aContent )
    {
        User::Leave(KErrArgument);
    }
        
    // Check if the content object is protected
    TInt attributeValue = 0;
    aContent->GetAttribute(ECanExecute, attributeValue);    
    
    // Check if the content object can be display on screen
    if(attributeValue)
    {
        // content object has rights that allow it to be displayed on screen
        return ETrue;
    }

    return EFalse;
} 
    
TInt CFlashPluginInstance::GetInteractionMode()
{
	return iInteractionMode;
}

EBrPluginExecutionMode CFlashPluginInstance::GetBrPluginExecutionMode()
{
    return iBrPluginExecutionMode;
}

const TDesC& CFlashPluginInstance::GetBaseURL()
{
	if (iBaseURL)
		return *iBaseURL;
	else
		return KNullDesC();
}

const TDesC& CFlashPluginInstance::GetSecurityContextURL()
{
    if (iSecurityContextUrl)
        return *iSecurityContextUrl;
    else
        return KNullDesC();
}

	
TBool CFlashPluginInstance::PercentLoaded(TReal& /*aPercentLoaded*/)
{
	return EFalse;
	
}

NPFlashConfig* CFlashPluginInstance::GetFlashconfig()
{
    return NULL;
}

void CFlashPluginInstance::SetAllowMobiFullScreen(TBool aAllowMobiFullScreen)
{
	iAllowMobiFullScreen = aAllowMobiFullScreen;
}

TBool CFlashPluginInstance::GetAllowMobiFullScreen()
{
	return iAllowMobiFullScreen;
}

int CFlashPluginInstance::OpenMovie()
{
	TRAPD(error,DoOpenMovieL());
	return error;
}

void CFlashPluginInstance::DoOpenMovieL()
{
	if (iFilename == NULL)
	{
		// file name is not set
		// => OpenFileL() was not called from my_asfile
		// so don't do anything
		User::Leave(KErrGeneral);
	}
	if (IsControlValid())
	{
		TBool correctMimeAndRights = EFalse;
	
		// Check for DRM rights
		CContent* content = CContent::NewLC(*iFilename);
		
		TBool drm = PerformDrmChecksL(*iFilename, content, correctMimeAndRights);
	
		// Call OpenFile if "the file is DRM protected with correct MIME type
		// and rights" or "it is not DRM protected" 
		if(drm && !correctMimeAndRights)
		{
			//invalid DRM rights
			CleanupStack::PopAndDestroy(content);
			
			if(iPluginView)
				iPluginView->NotifyClientError(SI_UnsupportedError,ETrue,ETrue);

			User::Leave(KErrGeneral);
		}          
	
		if(iPluginView)
		{
			iPluginView->OpenFile(iFilename->Des(), iUrl->Des(),iFirstFlashMovie, content);
		}
		
		iIsPlayerIntialised = ETrue;
		CleanupStack::PopAndDestroy(content);          
	}
	else
	{
		// control window has not been created yet - save filename in case
		// the browser calls setwindow again, with correct values        
		CacheFileNameL(*iFilename, iUrl,iFirstFlashMovie);
	}
	
	if(iDeferredLaunch)
    {
		iDeferredLaunch = EFalse;
	}

	return;
}

TBool CFlashPluginInstance::IsPlayerInitialzed()
{
    return iIsPlayerIntialised;
}

void CFlashPluginInstance::ReAdjustMaxHeapforVideoStream(TUint aMaxHeapSize)
{
    if (iPluginMemHandler)
        iPluginMemHandler->ReAdjustMaxHeapforVideoStream(aMaxHeapSize);
}

TBool CFlashPluginInstance::OpenUrlL(const TDesC& /*aUrl*/, const TDesC& /*aPostData*/)
{
    return EFalse;
}

TBool CFlashPluginInstance::OpenUrlL(const TDesC& /*aUrl*/, const TDesC& /*aTarget*/, const TDesC& /*aPostData*/)
{
    return EFalse;
}

TBool CFlashPluginInstance::OpenStreamL(unsigned short /*aStreamId*/,
		const TDesC& /*aUrl*/, const TDesC& /*aPostDataContentType*/,
        TInt /*aPostDataContentLength*/, const TDesC& /*aPostData*/)
{
    return EFalse; //default handling, override if required
}

void CFlashPluginInstance::SetStreamBufferSizeL(unsigned short /*streamId*/,
        unsigned long /*aBufferSize*/)
{
    return; //default handling, override if required
}

void CFlashPluginInstance::CancelStream(unsigned short /*streamId*/)
{
    return; //default handling, override if required
}
    
void CFlashPluginInstance::DrawDirtyRect(const TRect& aRect)
    {
    if(iPluginView)
        iPluginView->DrawDirtyRect(aRect);
}

TInt CFlashPluginInstance::CheckFileValidityL(const TDesC& 
#ifdef FF_FLASHLITE_SCREENSAVER
            aFilename
#endif
        )
{
    if (iEngine)
    {
#ifdef FF_FLASHLITE_SCREENSAVER  
        return iEngine->CheckFileValidityL(aFilename);
#else
        return 0;
#endif
    }
    return 0;
}

void CFlashPluginInstance::Start()
{
}

void CFlashPluginInstance::End()
{
    Stop();
}

void CFlashPluginInstance::Stop() 
{
    if (iEngine)
    {
        return iEngine->Stop();
    }
}

void CFlashPluginInstance::Pause(TInt aReason)
{
	if (iEngine)
	{
		iEngine->Pause(aReason);
	}   
}

void CFlashPluginInstance::Resume(TInt aReason, TBool aIgnoreActiveView)
{
   if (iEngine)
   {
	   TRAP_IGNORE (iEngine->ResumeL(aReason,aIgnoreActiveView));
   }       
}
        
/**
* Rewind video playback.
* @since Series 60 3.1
*/
void CFlashPluginInstance::Rewind() 
{
    if (iEngine)
    {
        TRAP_IGNORE (iEngine->RewindL());
    }       
}
       
/**
    Sets the volume.
    @param The value of the volume.
*/
/*

void CFlashPluginInstance::SetVolume(TInt aVolume)
{
	SetVolumeToEngine(aVolume);
}
*/

void CFlashPluginInstance::SetVolumeToEngine(TInt aVolume) 
{
    if (iEngine)
    {
        return iEngine->SetVolume(aVolume);
    }       
}

TInt CFlashPluginInstance::GetCurrentEngineVolume()
{  
	if(iEngine)
	{
		return iEngine->GetVolume();
	}
	
	return 0;
}

/**********************************************************************
	Function : MuteVolume
	Description : Sets the volume to Zero
	Input: None
	Output: None
	Return: None
	Comment :  changed for TSW ID: EPMR-7UPFEJ
**********************************************************************/
void CFlashPluginInstance::MuteVolume() 
{
	SetVolumeToEngine(0);
}

/**********************************************************************
	Function : UnMuteVolume
	Description : Sets the volume to the current volume level
	Input: None
	Output: None
	Return: None
	Comment :  changed for TSW ID: EPMR-7UPFEJ
**********************************************************************/
void CFlashPluginInstance::UnMuteVolume()
{
	TInt volume = iVolumeCenRepKeyWatcher->GetCurrentVolume();
	if (!volume)
	{
		volume = KMinVolume + 1;
	}

	iVolumeCenRepKeyWatcher->SetCurrentVolume(volume);	
	SetVolumeToEngine(volume);
}

/**
    * Get the state of the flash player.
    * @since Series 60 3.1
    * @return The status of video player
*/
NPFlashPlayerState CFlashPluginInstance::State() const
{
    if(iEngine)
    {
        if(iEngine->IsRunning())
        {
            if(iEngine->IsPaused())
            {
                return NPPlayerPaused;
            }
            else
            {
                return NPContentPlaying;    
            }
            
        }   
        else
        {
            if(iEngine->PlayerStoppedWithError())
            {
                return NPInstanceError;
            }
        
            if(iEngine->IsInitialized())
            {
                return NPEngineInitialized;
            }
            
            return NPPlayerStopped; 
        }
        
    }
    
    return NPEngineNotInitialized;
}


/**
Gets the version information for the FlashLite player library.
@return The version information.
*/
        
HBufC* CFlashPluginInstance::GetFlashVersion() 
{
    if (iEngine)
    {
        return iEngine->GetFlashVersion();
    } 
    
    return NULL;
}
    
/**
    This function is called to Zoom in or Zoom out the swf file.
    @param aMode Specifies Zoom in, Zoom out or Show all mode.
*/
TBool CFlashPluginInstance::Zoom(TInt aMode) 
{
    if (iEngine)
    {
        TInt limitReached(0);
        return iEngine->Zoom(aMode,limitReached);
    }   
    return EFalse;
}

//-----------------------------------------------------------------------
// short CFlashPluginControl::IsSoundPlaying()    
//
// This function returns whether sound is playing
//-----------------------------------------------------------------------

TBool CFlashPluginInstance::IsSoundPlaying() const
{
    if( !iEngine || iEngine->IsInitialized() )
    {
        return EFalse;
    }
 
    return iEngine->IsSoundPlaying();    
}
        
//-----------------------------------------------------------------------
// short CFlashPluginControl::GetCurrentFrameNo()    
//
// This function returns current frame number
//-----------------------------------------------------------------------
        
TInt CFlashPluginInstance::GetCurrentFrameNo() const 
{
    if (iEngine)
    {
        return iEngine->GetCurrentFrameNumber();
    }
    
    return -1;
    
}

//-----------------------------------------------------------------------
// short CFlashPluginControl::GetTotalFrames()    
//
// This function returns total frames
//-----------------------------------------------------------------------
        
TInt CFlashPluginInstance::GetTotalFrames() const 
{
    if (iEngine)
    {
        return iEngine->GetTotalFrameNumber();
    }
    
    return 0;
}

TInt CFlashPluginInstance::GetLastError() const
{
    if(iPluginView)
        return iPluginView->GetLastError();
    
    return KErrNone;
}

HBufC8* CFlashPluginInstance::GetBaseURL8(const TDesC& aBaseUrl)
{

	if (aBaseUrl.Length() <= 0)
        return NULL;

	HBufC8* baseURL8 = NULL;
	TRAP_IGNORE( baseURL8 = HBufC8::NewL(aBaseUrl.Length()));

    if (baseURL8)
        {
        TPtr8 ptr8 = baseURL8->Des();
		ptr8.Copy(aBaseUrl);
        }
    
	return baseURL8;
}

void CFlashPluginInstance::StartShutDownProcess()
{ 
   if(iEngine)
   {
	   iEngine->Stop();  
   }
   
   DestroyEngine();
	
   DestroyStreamHandler();    
   
   if(iPluginView)
	   iPluginView->SetErrorOccured();
	
   //TODO : Should I call NotifyError on control, NotifyError
   //does nothing
   //iControl->NotifyError(iControl->GetLastError());
}

TUint CFlashPluginInstance::GetFeatureCapability()
{
    return KCapabilitiesAnimation; // Send animation by default
}

TBool CFlashPluginInstance::DeferredLaunch()
{
    return iDeferredLaunch;
}

TInt CFlashPluginInstance::GetPersistentDirectory(const TDesC& aSwfPath, TFileName& aPersistentDir)
{
    HBufC* symbianStylePath = NULL;
    TPtr symbianStylePathPtr(0,0,0);
    HBufC* nameSpace = NULL;
    
    //construct default persistent path    
    GetDefaultPersistentPath(aPersistentDir);    
    
    symbianStylePath = aSwfPath.Alloc();
    if(symbianStylePath == NULL) 
        return KErrNone; 
        
    symbianStylePathPtr.Set(symbianStylePath->Des());
    
    if(iBrPluginExecutionMode != EWidgetMode) 
        return KErrNone; // In non-widget mode, use default persistant path        
    
    //Remove file:///
    TInt pos = symbianStylePathPtr.Find(KProtocolPrefixFileUnixStyle);
	if (pos >= 0)
	{
		symbianStylePathPtr = symbianStylePathPtr.Mid(pos
				+ KProtocolPrefixFileUnixStyle().Length());
    }
	else
	{
        return KErrNone; // For remote swfs use default persistant path 
    }
    
    //replace '/' with '\'
    ReversePathDelimiter(symbianStylePathPtr);
    
    //Get drive on which widget is installed. Z drive being an exception.
    TParse pathParser;
    TInt err = pathParser.Set(symbianStylePathPtr,NULL,NULL);
    if(err != KErrNone)
     return KErrNone; // path is not well formed. fallback to default persistant path
    
    TChar drive = pathParser.Drive()[0];
    
    TChar zDrive;
    iFs.DriveToChar(EDriveZ,zDrive); 
    if(zDrive == drive)
        return KErrNone; // Z is not writeable. fallback to default persistant path
    
     //Get widget namespace
    nameSpace = GetWidgetNameSpace(symbianStylePathPtr);
    if(nameSpace)
    {
        aPersistentDir[0] = drive; 
        
        aPersistentDir.SetLength(aPersistentDir.Length() - KFlashDirectory().Length()); //To remove KFlashDirectory
        aPersistentDir.Append(*nameSpace);
        aPersistentDir.Append(KPathDelimiter);
        aPersistentDir.Append(KFlashDirectory);        
    
        delete nameSpace;
        nameSpace = NULL;
    }
    
    return KErrNone;
}

void CFlashPluginInstance::GetDefaultPersistentPath(TDes& aPersistentDir)
{
    aPersistentDir.SetLength(0);
    iFs.PrivatePath(aPersistentDir);    
    
    TBuf<2> driveLetterAndDelimiter;
    TChar drive = iFs.GetSystemDriveChar();   
    driveLetterAndDelimiter.Append(drive);
    driveLetterAndDelimiter.Append(KDriveDelimiter); 
    aPersistentDir.Insert(0,driveLetterAndDelimiter);
    
	if (iBrPluginExecutionMode == EWidgetMode)
	{
        aPersistentDir.Append(KFlashDirectory);
    }
	else
	{ //To make sure there is no trailing '\'
        aPersistentDir.SetLength(aPersistentDir.Length()-1);
    }
}

void CFlashPluginInstance::ReversePathDelimiter(TDes& symbianStylePathPtr)
{
    //replace '/' with '\'
    const TChar KForwardSlash16 = '/';

    TInt delimiterPos = symbianStylePathPtr.Locate(KForwardSlash16);
    while(delimiterPos >= 0)
    {
        symbianStylePathPtr[delimiterPos] = KPathDelimiter;
        delimiterPos = symbianStylePathPtr.Locate(KForwardSlash16);
    }
}

HBufC* CFlashPluginInstance::GetWidgetNameSpace(const TDesC& aSwfPath)
{
    // private path will be of the form : \private\WRT_UID\
    // widget namespace : <drive>:\private\WRT_UID\<widget-namespace>\<custom_path>\swfname.swf
    TFileName privatePath;
    iFs.PrivatePath(privatePath);     
    
    TPtrC widgetName;
    //TBuf<KMaxFileName> widgetName;
    TBool widgetNameFound = EFalse;
    TInt pos = aSwfPath.FindF(privatePath);
    
    if(pos >= 0)
    {
        pos += privatePath.Length();
        //widgetName = aSwfPath.Mid(pos);
        widgetName.Set(aSwfPath.Mid(pos));
        
        TInt delimiterPos = widgetName.Locate(KPathDelimiter);
        if(delimiterPos >= 0)
        {
            widgetName.Set(aSwfPath.Mid(pos,delimiterPos));
            widgetNameFound = ETrue;
        }
    }  
    
    HBufC* ret = NULL;
	if (widgetNameFound)
	{
        ret = widgetName.Alloc();
    }
    
    return ret;
}

MNetscapeIf* CFlashPluginInstance::NetscapeFns()
{
	return iNetScapeIf;
}

void CFlashPluginInstance::InitializeVolumeObserverL()
{	
    FeatureManager::InitializeLibL(); // Initialising feature manager
    iSideVolumeKeysSupported = FeatureManager::FeatureSupported(
											KFeatureIdSideVolumeKeys);
    
    CHwKeyPropertyWatch* hwKeyPropertyWatch = NULL;
    if (iSideVolumeKeysSupported)  
    {
    	if(iGlobals)
		{
			hwKeyPropertyWatch = iGlobals->GetHwKeyPropertyWatch();
			if(iInteractionMode!= NPAnimPluginMode 
#ifdef FEATURE_SCREENSAVER
				&& iInteractionMode != NPScreenSaverMode
#endif
			)
				iVolumeCenRepKeyWatcher = CVolumeCenRepKeyWatcher::NewL(this,EFalse);
		}
						
		if (hwKeyPropertyWatch && !iHwKeyObserverAdded)
		{
			hwKeyPropertyWatch->AddObserverL(this);
			if(iInteractionMode!= NPAnimPluginMode 
#ifdef FEATURE_SCREENSAVER
				&& iInteractionMode != NPScreenSaverMode
#endif
			  )
				iVolumeCenRepKeyWatcher = CVolumeCenRepKeyWatcher::NewL(this);
		}
	}
    else
    {
		if(iInteractionMode!= NPAnimPluginMode 
#ifdef FEATURE_SCREENSAVER
			&& iInteractionMode != NPScreenSaverMode
#endif
			)
			iVolumeCenRepKeyWatcher = CVolumeCenRepKeyWatcher::NewL(this);

    }
	

    FeatureManager::UnInitializeLib();
    iHwKeyObserverAdded = ETrue;
}

void CFlashPluginInstance::AddHwSideKeyObserverL()
{
	CHwKeyPropertyWatch* hwKeyPropertyWatch = NULL;
	if (iSideVolumeKeysSupported)  
	{
		if(iGlobals)
		{
			hwKeyPropertyWatch = iGlobals->GetHwKeyPropertyWatch();
		}
						
		if (hwKeyPropertyWatch && !iHwKeyObserverAdded)
		{
			hwKeyPropertyWatch->AddObserverL(this);
			iHwKeyObserverAdded = ETrue;
		}   
	}  	
}

void CFlashPluginInstance::RemoveHwSideKeyObserver()
{
	CHwKeyPropertyWatch* hwKeyPropertyWatch = NULL;
	if (iSideVolumeKeysSupported)  
	{
		if(iGlobals)
		{
			hwKeyPropertyWatch = iGlobals->GetHwKeyPropertyWatch();
		}
						
		if (hwKeyPropertyWatch && iHwKeyObserverAdded)
		{
			hwKeyPropertyWatch->RemoveObserver(this);
			iHwKeyObserverAdded = EFalse;
		}   
	}  	
}

void CFlashPluginInstance::MHandleVolumeCmdL(TInt aNewValue)
{
	TInt currentVolume = 0;

    currentVolume = iVolumeCenRepKeyWatcher->GetCurrentVolume();
    
    switch(aNewValue)
    {	
    	case EVolumeControlCmdVolumeUpByOne:
        {
            if(currentVolume < KMaxVolume)
            {
                ++currentVolume;
            }
            else
                currentVolume = (KMaxVolume);
        	
            iVolumeCenRepKeyWatcher->SetCurrentVolume(currentVolume);	
		    iPluginView->ShowVolumePopup(currentVolume);
        }
        break;
        
        case EVolumeControlCmdVolumeDownByOne:
        {
            if(currentVolume > (KMinVolume))
            {
                --currentVolume;
            }
            else
                currentVolume = (KMinVolume);
        	
        	iVolumeCenRepKeyWatcher->SetCurrentVolume(currentVolume);	
        	iPluginView->ShowVolumePopup(currentVolume);
        }
    	break;
    	   	
       	default:
		    if ((aNewValue >= KMinVolume) && (aNewValue <= KMaxVolume))
            {
		        currentVolume = aNewValue;
		        iVolumeCenRepKeyWatcher->SetCurrentVolume(currentVolume);		        
            }
        break;          
    }
    
	#if defined(__WINS__) //offset required on the emulator
    currentVolume += WINS_DEFAULT_VOLUME;
	#endif  
	
    SetVolumeToEngine(currentVolume);	 
}

///////////////////////////////////////////////////////////////////////////
//MVolumePropertyWatchObserver

// ----------------------------------------------------------------------
// void CBrowserPluginControl::MVolumeChanged(TInt aNewValue)
//
// This function is callback from volume watchers
// for volume change with finger touch 
// ----------------------------------------------------------------------
// 
MVolumePropertyWatchObserver* CFlashPluginInstance::GetMHwKeyWatchObserver()
{
    return this;
}
void CFlashPluginInstance::MVolumeChanged(TInt aNewValue)
{
    TInt currentVolume = 0;

    currentVolume = iVolumeCenRepKeyWatcher->GetCurrentVolume();
    
	
	if ((aNewValue >= KMinVolume) && (aNewValue <= KMaxVolume))
    {
        currentVolume = aNewValue;
        iVolumeCenRepKeyWatcher->SetCurrentVolume(currentVolume);
    }
    #if defined(__WINS__) //offset required on the emulator
    currentVolume += WINS_DEFAULT_VOLUME;
	#endif  
	
    SetVolumeToEngine(currentVolume);	 
}

void CFlashPluginInstance::MPausePlayKeyPressed()
	{
	/*
	if(iEngine)	
		{
		if(iEngine->IsRunning())
			{	
			if(iEngine->IsPaused())
				{
				
				TRAP_IGNORE( iEngine->ResumeL(EReasonUser,EFalse) );	
				}
			else
				{
				
				Pause(EReasonUser);
				}
			}
		}*/
	}
	  
void CFlashPluginInstance::MStopKeyPressed()
{
	/*Stop();*/
}
	  
void CFlashPluginInstance::MRewindKeyPressed()
{
   /* Rewind();*/
}
	
void CFlashPluginInstance::MForwardKeyPressed()
{
	
}
			
void CFlashPluginInstance::MBackwardKeyPressed()
{
	
}
			
void CFlashPluginInstance::MFastForwardKeyPressed()
{
	
}

void CFlashPluginInstance::SetInitialVolumeToEngine()
{
	TInt initialVolume = 0;
	
	initialVolume = iVolumeCenRepKeyWatcher->GetCurrentVolume();
	SetVolumeToEngine(initialVolume);
}

CVolumeCenRepKeyWatcher*  CFlashPluginInstance::GetVolumeCenRepKeyWatcher()
{
	return iVolumeCenRepKeyWatcher;
}


TBool CFlashPluginInstance::IsVolumeKeySupported()
{
	return iSideVolumeKeysSupported;
}

#ifdef SELF_IAD_UPDATE
void CFlashPluginInstance::IADUpdateCallback( TInt /*aErrorCode*/ )
{
    NPPDebug( LOG_DEBUG, _L("CFlashPluginInstance::IADUpdateCallback() ++ ") );
    // FlashLite 3.1
    // Notify the user that the update is complete.
    
    // defer the notification untill the flash is active again
    iIadProgressState = EIadUpdateComplete;

    NPPDebug( LOG_DEBUG, _L("CFlashPluginInstance::IADUpdateCallback() -- ") );
}

void CFlashPluginInstance::SetIADProgressState(TIadProgressState aState)
{
	iIadProgressState = aState;
}

TIadProgressState CFlashPluginInstance::GetIADProgressState()
{
	return iIadProgressState;
}
#endif //SELF_IAD_UPDATE

void CFlashPluginInstance::AddEventObserverToControl()
{
	iPluginView->AddEventObserverToControl();
}

void CFlashPluginInstance::RemoveEventObserverFromControl()
{
	iPluginView->RemoveEventObserverFromControl();
}

MFlashEngine* CFlashPluginInstance::GetEngine()
{
	return iEngine;
}

MDeviceExtnIf* CFlashPluginInstance::GetDeviceIf()
{
	return iDeviceExtn;
}



//End of file
