/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CFlashPluginInstanceFactory implementation file
*
*/

#include "saplayer_build.h"

#include "flashplugininstancefactory.h"

#ifdef FEATURE_SCREENSAVER	
#include "FlashScreensaverPlugin.h"
#endif

#ifdef FEATURE_BROWSER
#include "flashplugin.h"
#endif

#ifdef FEATURE_ANIMATION_PLUGIN
#include "flashanimplugin.h"
#endif //FEATURE_ANIMATION_PLUGIN

#include "flashplgmain.h"
#include <e32cmn.h>
#ifdef CHECK_SECURE_ID
_LIT_SECURE_ID(uidLowRange,0x20000000); // allowed UID1
_LIT_SECURE_ID(uidHighRange,0x9FFFFFFF); // allowed UID2
#endif

// -------------------------------------------------------------------------
// CFlashPluginInstanceFactory::CFlashPluginInstanceFactory()
//
// First phase private constructor
// --------------------------------------------------------------------------
//
CFlashPluginInstanceFactory::CFlashPluginInstanceFactory()
{
	
}

// ----------------------------------------------------------------------
// CFlashPluginInstanceFactory::~CFlashPluginInstanceFactory()
//
// Private Destructor
// ----------------------------------------------------------------------
// 
CFlashPluginInstanceFactory::~CFlashPluginInstanceFactory()
{
	
}

// ----------------------------------------------------------------------
// CFlashPluginInstanceFactory::ValidAppL()
//
// check secure ID of the app
// ----------------------------------------------------------------------
//
	
TBool CFlashPluginInstanceFactory::ValidAppL()
{
	TBool ret= EFalse;
	RProcess currentProc;
  User::LeaveIfError(currentProc.Open(currentProc.Id()));
  const TSecureId secureId(currentProc.SecureId());
  currentProc.Close();
  
	//uidLowRange;uidHighRange; //warning purpose
	//check for secure id range for process

#ifdef CHECK_SECURE_ID
	if((secureId >=uidLowRange) && (secureId <= uidHighRange))
	{
		ret= ETrue;
	}
#else
		ret= ETrue;
#endif
	
	return ret;
}

// ----------------------------------------------------------------------
// CFlashPluginInstance* CFlashPluginInstanceFactory::GetInstanceL(CPluginGlobals* aGlobals, 
//        										NPP aInstance,
//        										uint16 aMode,
//        										TInt aInteractionMode)
//
// returns the plug-in instnace handler
// ----------------------------------------------------------------------
//
CFlashPluginInstance* CFlashPluginInstanceFactory::GetInstanceL(MNetscapeIf* aNetscapeIf,CPluginGlobals* aGlobals, 
        										uint16 aMode,
        										TInt aInteractionMode
        										,TWindowMode /*aWindowMode*/
        										,TPluginUiFramework aPluginUiFramework
                                                )
{

   //need to check secure id here
    
   TInt err;
   TBool valid=EFalse;
   TRAP (err, valid = ValidAppL());
   if(!valid)
   {
   	return NULL;	
   }
  
   switch(aInteractionMode)
   {
     #ifdef FEATURE_SCREENSAVER	
  		case NPScreenSaverMode:
  			{
  						return STATIC_CAST(CFlashPluginInstance*,
  										CScreensaverPluginInstance::NewL(aNetscapeIf,aGlobals, 
  													aMode,aInteractionMode,aPluginUiFramework));
  							
  			}
  					//	break;
  	#endif
  	
  	#ifdef FEATURE_ANIMATION_PLUGIN
  	case NPAnimPluginMode:
  	{
  		//get the config structure
  	//MNetscapeIf* pNPNetscapeFuncs = NetscapeFns();
        //aGlobals->NetscapeIf();
  		NPFlashConfig* param=NULL;
  		
  		param = new (ELeave) NPFlashConfig;
  		
  		CleanupStack::PushL(param);
  		
  		NPError npError=KErrNone; //NPERR_NO_ERROR;
  		
  		if (param)
  		{
  			Mem::FillZ(STATIC_CAST(TAny*,param),sizeof(NPFlashConfig));
  			
  			npError = aNetscapeIf->GetValue(NPNGetconfigParam,
  													param);
  		}
 																			
  		if((KErrNone == npError) && 
  		   (NULL!= param) &&
  		   (sizeof(NPFlashConfig) == param->iSize))
  		{
  		
  	    CFlashPluginInstance* pInst =  
  	                            STATIC_CAST(CFlashPluginInstance*,
							    CAnimationPluginInstance::NewL
							    (aNetscapeIf,
							    param,
    							aGlobals, 
    							aMode,
    							aInteractionMode,aPluginUiFramework)
    							);
        
        CleanupStack::Pop(); //param            
            							
        return pInst;    							  						
  		}
  		
  		//fall back to default mode
  		aInteractionMode = NPDefaultMode;
  		
  		CleanupStack::Pop(); //param
  	}
  	
  	#endif //FEATURE_ANIMATION_PLUGIN
  						
  	default :
  	{
  		#ifdef FEATURE_BROWSER
  		
  		CBrowserPluginInstance* plgInstance = CBrowserPluginInstance::NewL(aNetscapeIf,aGlobals,
  												        aMode,aInteractionMode
#ifdef FEATURE_WINDOWLESS
  												        ,aWindowMode
#endif //FEATURE_WINDOWLESS	
  												        ,aPluginUiFramework
													);
  		return STATIC_CAST(CFlashPluginInstance*, plgInstance);
  		  				
  		#endif
  	}
   }     											
}

