/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CPluginInstanceInfo class implementation
*
*/
 
#include "saplayer_build.h"
#include "flashplugin.h"

CPluginInstanceInfo* CPluginInstanceInfo::NewL()
{
    CPluginInstanceInfo* self = new (ELeave) CPluginInstanceInfo();
    
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop();
    
    return self;
}

CPluginInstanceInfo::CPluginInstanceInfo()
                    : iArgNameArray(NULL),
                      iArgVectArray(NULL),
                      iFileName(NULL),
                      iUrl(NULL)
{
}

CPluginInstanceInfo::~CPluginInstanceInfo()
{
    if(iArgNameArray)
    {
		iArgNameArray->Reset();
		 delete iArgNameArray;
    }
    if(iArgVectArray)
    {
    	iArgVectArray->Reset();
    	delete iArgVectArray;
    }
    
    //delete iArgNameArray;
    //delete iArgVectArray;
    if(iFileName)
    {
    	delete iFileName;
    	iFileName=NULL;
    }
    if(iUrl)
    {
    	delete iUrl;
    	iUrl=NULL;
    }
}

void CPluginInstanceInfo::ConstructL()
{
    iArgNameArray = new (ELeave) CDesCArrayFlat( 1 );
    iArgVectArray = new (ELeave) CDesCArrayFlat( 1 );
}

void CPluginInstanceInfo::AddArgsL(CDesCArray* aArgN, CDesCArray* aArgV)
{
    if( aArgN == NULL || aArgV == NULL )
    {
        User::Leave(KErrArgument);
    }
    
    for ( TInt i=0; i < aArgN->Count(); i++ )
    {
        TPtrC name( (*aArgN)[i] );
        TPtrC value( (*aArgV)[i] );
        iArgNameArray->AppendL( name );
        iArgVectArray->AppendL( value );
    }
}

void CPluginInstanceInfo::AddInitStreamL()
{
}


void CPluginInstanceInfo::AddFileNameL(const TDesC& aFileName)
{
    if(!aFileName.Length()) 
    {
        return;
    }
        
    if( iFileName )
    {
        delete iFileName;
        iFileName=NULL;
    }
        
    iFileName = HBufC::NewL(aFileName.Length());
    TPtr ptr  = iFileName->Des();
    ptr.Copy(aFileName);
}
    
void CPluginInstanceInfo::AddUrlL(const TDesC& aUrl)
{
    if(!aUrl.Length()) 
    {
        return;
    }

    if( iUrl )
    {
        delete iUrl;
        iUrl=NULL;
    }        
        
    iUrl = HBufC::NewL(aUrl.Length());
    
    TPtr ptr = iUrl->Des();
    ptr.Copy(aUrl);
}

CDesCArray* CPluginInstanceInfo::ArgumentNames()
{
    if( iArgNameArray && iArgNameArray->Count() == 0 ) return NULL;
	return iArgNameArray;
}

CDesCArray* CPluginInstanceInfo::ArgumentValues()
{
    if( iArgVectArray && iArgVectArray->Count() == 0 ) return NULL;
    return iArgVectArray;
}

TDesC* CPluginInstanceInfo::FileName()
{
    return iFileName;
}

TDesC* CPluginInstanceInfo::Url()
{
    return iUrl;
}
