/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CScreensaverPluginInstance class implementation
*
*/
 
// INCLUDE FILES
#include "saplayer_build.h"
#include "FlashScreensaverPlugin.h"
#include "flashPlugincontrolFactory.h"
#include "NPNFlashExtension.h"
#include <eikenv.h>
#include <bautils.h> // NearestLanguage
#include <content.h>
#include <data_caging_path_literals.hrh>
#include <featmgr.h>

#include "FlashConfigGen.h"
#include "saflash_consts.h"
#include "flash_engine.h"
// CONSTANTS

// MACROS

extern void NPPDebug( TUint8 aLogLevel,TRefByValue<const TDesC> aText, ... );

using namespace ContentAccess;
// ================= MEMBER FUNCTIONS =======================

// -------------------------------------------------------------------------
// CScreensaverPluginInstance::CScreensaverPluginInstance(PluginGlobals* aGlobals, NPP aInstance, 
// uint16 /*aMode*/)
//
// First phase constructor
// --------------------------------------------------------------------------
//    


CScreensaverPluginInstance::CScreensaverPluginInstance(MNetscapeIf* aNetscapeIf,CPluginGlobals* aGlobals, NPP aInstance, 
				uint16 aMode,TInt aInteractionMode)
				: CFlashPluginInstance(aNetscapeIf,aGlobals,aInstance,aMode,aInteractionMode
#ifdef FEATURE_WINDOWLESS
				,EWindow
#endif //FEATURE_WINDOWLESS
				)

    {
    
    }

// -------------------------------------------------------------------------
// CScreensaverPluginInstance* CScreensaverPluginInstance::NewL(PluginGlobals* aGlobals, 
// NPP aInstance, uint16 aMode)
//
// Static constructor
// --------------------------------------------------------------------------
// 


CScreensaverPluginInstance* CScreensaverPluginInstance::NewL(MNetscapeIf* aNetscapeIf,
                                        CPluginGlobals* aGlobals, 
                                        NPP aInstance, 
                                        uint16 aMode,
                                        TInt aInteractionMode,TPluginUiFramework aPluginUiFramework)

    {
    NPPDebug(LOG_DEBUG,_L("CScreensaverPluginInstance::NewL() ++"));
    

    CScreensaverPluginInstance* self = new (ELeave) CScreensaverPluginInstance
                                        (
                                         aNetscapeIf,
                                        aGlobals, 
    									aInstance,
    									aMode,
    									aInteractionMode
    									);

    CleanupStack::PushL(self);
    self->ConstructL(TPluginUiFramework aPluginUiFramework);
    CleanupStack::Pop(self);
    
    NPPDebug(LOG_DEBUG,_L("CScreensaverPluginInstance::NewL() --"));
    return self;
    }

// -------------------------------------------------------------------------
// CScreensaverPluginInstance::~CScreensaverPluginInstance()
//
// Destructor
// --------------------------------------------------------------------------
// 
CScreensaverPluginInstance::~CScreensaverPluginInstance()
{
    NPPDebug(LOG_DEBUG,_L("CScreensaverPluginInstance::~CScreensaverPluginInstance() ++"));
    
}

// -------------------------------------------------------------------------
// void CScreensaverPluginInstance::ConstructL()
//
// Second phase constructor
// --------------------------------------------------------------------------
// 
void CScreensaverPluginInstance::ConstructL()
    {
    CFlashConfigGen::FillFlashConfig(GetInteractionMode(),
                                         iConfig);    
    
    iPluginView = CreatePluginView( aPluginUiFramework);
 
    }

// -------------------------------------------------------------------------
// void CScreensaverPluginInstance::SetWindowL(MPluginAdapter* aPlugin, const TRect& 
// aRect)
//
// Creates the window owing control and sets the window parameters
// --------------------------------------------------------------------------
// 
void CScreensaverPluginInstance::SetWindowL(MPluginAdapter* aPlugin, const TRect& aRect, const TRect& aClipRect)
{

	 CFlashPluginInstance::SetWindowL(aPlugin,aRect, aClipRect);
	 
	    
	 (static_cast<CFlashPluginControl*>(iPluginView))->SetRect(aRect);        

	if (iCachedFileName && iCachedUrl && IsControlValid())
	    {
	    TRAPD(err, OpenFileL(*iCachedFileName, iCachedUrl, iFirstMovie));
	    delete iCachedFileName; iCachedFileName = NULL;
	    delete iCachedUrl; iCachedUrl = NULL;
	    if (err != KErrNone)
	        {
	        User::Leave(err);
	        }
	    }
}


// -------------------------------------------------------------------------
// void CScreensaverPluginInstance :: HandleGetCommandL(NPPVariable aVariable, 
//                    void* ret_value)
//
//
// Processes the get_Value commands
// --------------------------------------------------------------------------
// 
NPError CScreensaverPluginInstance::HandleGetCommandL(NPPVariable aVariable, 
                    void* ret_value)
{

	if (!iPluginView)
	{
		return -1;
	}

	switch (aVariable)
	{
	case  NPPCheckFileIsValid:
			{
				if(ret_value)
            	{
 
            		//TRAPD(err,iControl->CreateEngineL());  
                    TRAPD(err,CreateEngineL());
    				if(err == KErrNone)
    				{
    					err = CFlashPluginInstance::HandleGetCommandL(aVariable, ret_value);
   						//iControl->DestroyEngine();
    					DestroyEngine();
    				}
    				return err;
            	}	
			}
			break;
	case NPPScreenSaverGetLastError:
		{
		TInt* error = (TInt*)ret_value;
        *error = iPluginView->GetLastError();
		}
		break;		
		
		case NPPGetPluginStatus :
			
    	case NPPGetCurFrame:
        
	    case NPPGetTotalFrames: 
	    {
	    	// do nothing
	    }
	    		break;
			
		default:
			{
				return CFlashPluginInstance::HandleGetCommandL(aVariable, ret_value);
			}
		    //break;
	}// switch (aVariable)

	return NPERR_NO_ERROR;
}

// -------------------------------------------------------------------------
// void CScreensaverPluginInstance :: HandleSetCommandL(NPNVariable aVariable, 
//                    void* ret_value)
//
//
// Processes the set_value commands
// --------------------------------------------------------------------------
// 

NPError CScreensaverPluginInstance::HandleSetCommandL(NPNVariable aVariable, 
                    void* ret_value) 

{

	if (!iPluginView)
	{
		return -1;
	}
	
	switch(aVariable)
    	{
    		case  NPNInteractionMode :
			//NPNInteractionMode specific-- do nothing
        		{
	          	}
        	    break;
          
			case NPNAnimationEvent :
			{
				
        		NPAnimationEvent animEvent = ( NPAnimationEvent )(*(NPAnimationEvent*)ret_value);
            
				switch (animEvent)
				{					
					case  NPStartAnimation:
					    (static_cast<CFlashPluginControl*>(iPluginView))->MakeVisible( ETrue );
					//TRAPD (err, iControl->CreateEngineL());
	 				TRAPD (err, CreateEngineL());
					if (err != KErrNone) 
					{
						return -1;
					}
					if(iCachedFileName)
					{
						TRAPD (err, iPluginView->OpenFileL(iCachedFileName->Des(),NULL,ETrue));
						if (err != KErrNone) 
						{	
							return -1;
						}
					}
					break;
	                			
	        		case  NPEndAnimation :                		
	        		    (static_cast<CFlashPluginControl*>(iPluginView))->MakeVisible( EFalse );		
					//iControl->End();
	 				DestroyEngine();
					break;
	          	  		
				default:
					{
					// do nothing
					}

				}//switch (animEvent)
			}
			break;
	}//switch (aVariable)
	
	return NPERR_NO_ERROR;

}

TUint CScreensaverPluginInstance::GetFeatureCapability()
{
    return KCapabilitiesScreensaver;
}

TBool CScreensaverPluginInstance::CanContentBeTrustedL(const TPtrC /*pAccessedDomain*/)
{
  	return EFalse;
}

typedef MPluginView* (*PFNC_CreatePluginView)(MFlashPluginInstance*,TWindowMode);

_LIT(KFlashAvkonUILibraryName, "aknflashpluginui.dll");

const TInt KFlashCreatePluginViewOrdinal = 1;

MPluginView* CScreensaverPluginInstance::DoCreatePluginViewL(TPluginUiFramework aPluginUiFramework)
{
	User::LeaveIfError(iLibrary.Load(KFlashAvkonUILibraryName));

	PFNC_CreatePluginView pFncCreateFlashPluginView =
			(PFNC_CreatePluginView) iLibrary.Lookup(KFlashCreatePluginViewOrdinal);
	if (pFncCreateFlashPluginView == NULL)
	{
		User::Leave(KErrGeneral);
	}
	return (MPluginView*) pFncCreateFlashPluginView(this, iWindowMode);
}

MPluginView* CScreensaverPluginInstance::CreatePluginView(TPluginUiFramework aPluginUiFramework)
{
	MPluginView* pluginView = NULL;
	TRAP_IGNORE(pluginView = DoCreatePluginViewL(aPluginUiFramework));
	return pluginView;
}
// End of File
