/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  FlashPluginControl class implementation
*
*/


// INCLUDE FILES
#include <e32std.h>
#include "saplayer_build.h"
#include "flashsendmessage.h"

// includes for message sending
//#include <mmsconst.h>
#include <mtuireg.h>
#include <mtmuibas.h>
#include <eikfutil.h>
#include <txtrich.h>

#include <coemain.h>
#include <charconv.h>
#include <eikenv.h>

#include <CMessageData.h>

// CONSTANTS

// MACROS
_LIT(KTempTextFileName, "c:\\system\\temp\\msg.txt" );

// Dummy observer class for CMsvSession - used when opening messaging ui

class CDummyObserver : public CBase, public MMsvSessionObserver
    {
    public:
    void HandleSessionEventL(TMsvSessionEvent, TAny*, TAny*, TAny*) {};
    };

// ----------------------------------------------------------------------
// CSender* CSender::NewL()
//
// Static constructor
// ----------------------------------------------------------------------
// 
CSender* CSender::NewL()
    {
    CSender* self = new(ELeave) CSender();
    CleanupStack::PushL(self);
    self -> ConstructL();
    CleanupStack::Pop(self);
    return self;
    }

// ----------------------------------------------------------------------
// CSender::CSender()
//
// First phase constructor
// ----------------------------------------------------------------------
// 
CSender::CSender()
    {
    }

// ----------------------------------------------------------------------
// CSender::~CSender()
//
// Destructor
// ----------------------------------------------------------------------
// 
CSender::~CSender()
    {
    delete iSendAppUi;
    iSendAppUi = NULL;
    }

// ----------------------------------------------------------------------
// void CSender::ConstructL()
//
// Second phase constructor
// ----------------------------------------------------------------------
//
void CSender::ConstructL()
    {
    iSendAppUi = CSendUi::NewL();
    }

// ----------------------------------------------------------------------
// HBufC* CSender::CreateEpocStringL ( const char* str, int strlen )
//
// Converts a character string to Unicode string 
// ----------------------------------------------------------------------
//
HBufC* CSender::CreateEpocStringL ( const char* str, int strlen )
    {
    CCnvCharacterSetConverter* pCharacterConverter = 
                CCnvCharacterSetConverter::NewL ( );
    CleanupStack::PushL ( pCharacterConverter );

    if ( pCharacterConverter->PrepareToConvertToOrFromL (
     KCharacterSetIdentifierCodePage1252, CEikonEnv::Static ()->FsSession())
        != CCnvCharacterSetConverter::EAvailable )
        {
        User::Leave ( KErrNotSupported );
        }

    User::LeaveIfNull ( pCharacterConverter );

    TPtrC8 ptr ( (TUint8*)str, strlen );

    HBufC* pResult = HBufC::NewL ( strlen );
    CleanupStack::PushL ( pResult );

    TPtr ptrResult ( pResult->Des ( ) );

    TInt state = CCnvCharacterSetConverter::KStateDefault;
    User::LeaveIfError ( pCharacterConverter->ConvertToUnicode ( ptrResult,
                                     ptr, state ) );
    
    CleanupStack::Pop (pResult);
    CleanupStack::PopAndDestroy(pCharacterConverter);
    return pResult;
    }

// ----------------------------------------------------------------------
// HBufC* CSender::CreateEpocStringL ( const char* text )
//
// Converts a character string to Unicode string 
// ----------------------------------------------------------------------
//
HBufC* CSender::CreateEpocStringL ( const char* text )
    {
    // TPtrC8 does not like NULL pointers
    if (!text)
        text = "";

    TPtrC8 ptr ( (TUint8*) text );
    return CreateEpocStringL ( text, ptr.Length ( ) );
    }

// ----------------------------------------------------------------------
// TUint32 CSender::FlashStrLength ( const char* str )
//
// Returns length of the string 
// ----------------------------------------------------------------------
//
TUint32 CSender::FlashStrLength ( const char* str )
    {
    return ( str ) ? TPtrC8((TUint8*)str).Length ( ) : 0;
    }

// ----------------------------------------------------------------------
// void CSender::CreateAndEditMessageL(const TUid aMtmUid, 
// char* aAddress, char* aBodyText)
//
// Creates and send a message
// ----------------------------------------------------------------------
//
void CSender::CreateAndEditMessageL(const TUid aMtmUid, 
                char* aAddress, char* aBodyText)
    {
    CMessageData* messageBody;
    
    messageBody = CMessageData::NewL();
    CleanupStack::PushL(messageBody);
            
    CParaFormatLayer* paraFormatLayer = CParaFormatLayer::NewL();
    CleanupStack::PushL(paraFormatLayer);

    CCharFormatLayer* charFormatLayer = CCharFormatLayer::NewL();
    CleanupStack::PushL(charFormatLayer);

    CRichText* messageBodyContent = CRichText::NewL(paraFormatLayer,
                                     charFormatLayer); 
    CleanupStack::PushL(messageBodyContent);
  
    if (aAddress)
        {
        TDesC* addr = CreateEpocStringL(aAddress);
        messageBody->AppendToAddressL(*addr);    
        delete addr;
        }
                  
    if (aBodyText && FlashStrLength(aBodyText))
        {
        HBufC* hBody = CreateEpocStringL(aBodyText);
        CleanupStack::PushL(hBody);
        messageBodyContent->InsertL(0, *hBody);
        messageBody->SetBodyTextL(messageBodyContent);
        CleanupStack::PopAndDestroy(hBody);
        }

    iSendAppUi->CreateAndSendMessageL(aMtmUid, messageBody);
    
    // make sure text file attachment is cleaned up.
    CEikonEnv::Static()->FsSession().Delete(KTempTextFileName);

    CleanupStack::PopAndDestroy(messageBodyContent);
    CleanupStack::PopAndDestroy(charFormatLayer);
    CleanupStack::PopAndDestroy(paraFormatLayer);
    CleanupStack::PopAndDestroy(messageBody);
    }
void CSender::CreateAndEditMessageL(const TUid aMtmUid,const TDesC& aAddress,const TDesC& aBodyText)
	{
	CMessageData* messageBody;	
	messageBody = CMessageData::NewLC();

	CParaFormatLayer* paraFormatLayer = CParaFormatLayer::NewL();
	CleanupStack::PushL(paraFormatLayer);

	CCharFormatLayer* charFormatLayer = CCharFormatLayer::NewL();
	CleanupStack::PushL(charFormatLayer);

	CRichText* messageBodyContent = CRichText::NewL(paraFormatLayer, charFormatLayer); 
	CleanupStack::PushL(messageBodyContent);

	if (aAddress.Length() != 0)
		{
		messageBody->AppendToAddressL(aAddress);
		}

	if (aBodyText.Length() != 0)
		{
		messageBodyContent->InsertL(0, aBodyText);
		messageBody->SetBodyTextL(messageBodyContent);
		}
	
	iSendAppUi->CreateAndSendMessageL(aMtmUid, messageBody);

	CleanupStack::PopAndDestroy(messageBodyContent);
	CleanupStack::PopAndDestroy(charFormatLayer);
	CleanupStack::PopAndDestroy(paraFormatLayer);
	CleanupStack::PopAndDestroy(messageBody);
	}
	

//void CSender::CreateAndEditMessageL(const TUid aMtmUid, const TDesC& aAddress,const TDesC& aCc,const TDesC& aBcc,const TDesC& aSubject,const TDesC& aBodyText )
void CSender::CreateAndEditMessageL(const TUid aMtmUid, const TMailInfo& aMailInfo )								  							 
								 
{
	
	CMessageData* messageBody;
	
	messageBody = CMessageData::NewLC();
    		
    CParaFormatLayer* paraFormatLayer = CParaFormatLayer::NewL();
    CleanupStack::PushL(paraFormatLayer);

    CCharFormatLayer* charFormatLayer = CCharFormatLayer::NewL();
	CleanupStack::PushL(charFormatLayer);

    CRichText* messageBodyContent = CRichText::NewL(paraFormatLayer, charFormatLayer); 
    CleanupStack::PushL(messageBodyContent);
  
	if (aMailInfo.iAddress.Length() != 0)
	{
	 	messageBody->AppendToAddressL(aMailInfo.iAddress);			
	}
    if (aMailInfo.iSubject.Length() != 0)
    {
        messageBody->SetSubjectL(&aMailInfo.iSubject);
    }
  	
    if (aMailInfo.iCc.Length() != 0)
    {
        messageBody->AppendCcAddressL(aMailInfo.iCc);
    }
    
    if (aMailInfo.iBcc.Length() != 0)
    {
        messageBody->AppendBccAddressL(aMailInfo.iBcc);
    }

  	            
    if (aMailInfo.iBodyText.Length() != 0)
    {
    	messageBodyContent->InsertL(0, aMailInfo.iBodyText);
    	messageBody->SetBodyTextL(messageBodyContent);
    }

	iSendAppUi->CreateAndSendMessageL(aMtmUid, messageBody);

	CleanupStack::PopAndDestroy(4,messageBody); // messageBody, messageBodyContent, charFormatLayer, paraFormatLayer

}
// End of File
