/*
* Copyright (c) 2007-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash Browser Plugin Instance Memry handler implementation
*
*/

#include "saplayer_build.h"
#include <eikenv.h>
#include "plugininstancememinfo.h"
#include "swfmemmgr.h"
#include <e32hal.h>
#include <hal.h>

extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );

CPluginInstanceMemInfo* CPluginInstanceMemInfo::NewL(CSwfMemMgr* aMemMgr
													,CFlashPluginInstance* aSwfInstance
													, TUint aMaxHeapSize)
{
	NPPDebug(LOG_DEBUG,_L("CPluginInstanceMemInfo::NewL() +"));
	CPluginInstanceMemInfo* self = new (ELeave) CPluginInstanceMemInfo(aMemMgr,aSwfInstance,aMaxHeapSize);
    
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop();
    NPPDebug(LOG_DEBUG,_L("CPluginInstanceMemInfo::NewL() -"));
    return self;	

}

CPluginInstanceMemInfo::~CPluginInstanceMemInfo()
{
	DestroyAllMem();//TODO : check who has to free it
}

void CPluginInstanceMemInfo::DestroyAllMem()
{
	NPPDebug(LOG_DEBUG,_L("CPluginInstanceMemInfo::DestroyAllMem() +"));
	
	DynamicMemBlock* pBlock = iDynamicMemBlocks;
	DynamicMemBlock* pTemp=NULL;
	while(pBlock != NULL)
	{
		//call free to mem mgr
		iMemMgr->FreeMem(pBlock->mem,pBlock->numBlocks);	
		
		pTemp = pBlock;
		
		pBlock = pBlock->pNext;
		
		delete pTemp;
	}
	
	DynHeapFreed(iDynHeapSize);
	
	NPPDebug(LOG_DEBUG,_L("CPluginInstanceMemInfo::DestroyAllMem() -"));
} 


CPluginInstanceMemInfo::CPluginInstanceMemInfo(CSwfMemMgr* aMemMgr,
							CFlashPluginInstance* aSwfInstance
							, TUint aMaxHeapSize):
							iDynMaxHeapSize(aMaxHeapSize)
							,iDynHeapSize(0)
							,iMemMgr(aMemMgr)
							,iSwfInstance(aSwfInstance)
							,iDynamicMemBlocks(NULL)
							
{
	NPPDebug(LOG_DEBUG,_L("CPluginInstanceMemInfo::CPluginInstanceMemInfo : Heap Limit%d "),iDynMaxHeapSize);	
}

void CPluginInstanceMemInfo::ConstructL()
{

}

void CPluginInstanceMemInfo::ReAdjustMaxHeapforVideoStream(TUint aMaxHeapSize)
{
	NPPDebug(LOG_INFO,_L("CPluginInstanceMemInfo::ReAdjustMaxHeapforVideoStream : %d "),aMaxHeapSize);
	iDynMaxHeapSize = aMaxHeapSize;
}

TUint CPluginInstanceMemInfo::GetCurrAllocSize()
{
	return iDynHeapSize;
}
    		
    		
   
void CPluginInstanceMemInfo::AddDynHeapAlloced(TUint aSize)
{
	iDynHeapSize += aSize; 
}
 
 
void CPluginInstanceMemInfo::DynHeapFreed(TUint aSize)
{
	iDynHeapSize -= aSize;
}

 
TAny* CPluginInstanceMemInfo::AllocMem(TUint16 aNumBlocks)
{
	TInt size = aNumBlocks * KDLC_BLOCK_SIZE;

	// check to see if this allocation will exceed our limit.
	if ((iDynHeapSize + size) > iDynMaxHeapSize)
	{	 
		
 			return NULL;
 	
	}
	
	TAny*  mem = iMemMgr->AllocMem(aNumBlocks);
	
	if(mem !=NULL)
	{
		DynamicMemBlock* pBlock = new DynamicMemBlock();
		
		if (pBlock)
		{
			pBlock->numBlocks = aNumBlocks;
			pBlock->mem = mem;
			pBlock->pNext = iDynamicMemBlocks;
			iDynamicMemBlocks = pBlock;
			AddDynHeapAlloced(size);
		}
	}
	
	return mem;
}
 
void CPluginInstanceMemInfo::FreeMem(TAny* aMemory)
{
	TInt size = 0;
	DynamicMemBlock* pBlock = iDynamicMemBlocks;
	DynamicMemBlock* pPrevBlock = NULL;

	while (pBlock)
	{
		if (pBlock->mem == aMemory)
		{
			//call free to mem mgr
			iMemMgr->FreeMem(aMemory,pBlock->numBlocks);
			
			if (pPrevBlock)
			{
				pPrevBlock->pNext = pBlock->pNext;
			}
			else
			{
				iDynamicMemBlocks = pBlock->pNext;
			}

			size = pBlock->numBlocks * KDLC_BLOCK_SIZE;
			delete pBlock;
			break;
		}
		pPrevBlock = pBlock;
		pBlock = pBlock->pNext;
	}
	
	DynHeapFreed(size);
}
    		
//End of file

    		
