/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  FlashPluginControl class implementation
*
*/


#include "saplayer_build.h"
#include "pluginmemhandlerimpl.h"
#include "flashplugininstance.h"
#include "swfmemmgr.h"
#include "plugininstancememinfo.h"
#include "flashplgmain.h"
#include "flashconfiggen.h"

CPluginMemHandlerImpl::CPluginMemHandlerImpl(CFlashPluginInstance* aPlayerInstance)
    :iPlayerInstance(aPlayerInstance)
    {
    }

CPluginMemHandlerImpl* CPluginMemHandlerImpl::NewL(CFlashPluginInstance* aPlayerInstance)
    {
    CPluginMemHandlerImpl* self = new (ELeave) CPluginMemHandlerImpl(aPlayerInstance);
    CleanupStack::PushL(self);
#ifdef PLUG_IN_MEM_ALLOC
    self->AddMemHandlerL();
#endif
    CleanupStack::Pop(self);
    return self;
    }

CPluginMemHandlerImpl::~CPluginMemHandlerImpl()
	{
#ifdef PLUG_IN_MEM_ALLOC
	RemoveMemHandler();
#endif
    }

#ifdef PLUG_IN_MEM_ALLOC
void CPluginMemHandlerImpl::RemoveMemHandler()
{
    CSwfMemMgr* memMgr = iPlayerInstance->GetGlobals()->GetMemMgr();

    if(iInstanceMemInfo)
    {
        memMgr->RemoveMemInfo(iInstanceMemInfo);
    
        delete iInstanceMemInfo;
    
        iInstanceMemInfo= NULL; 
    }
    
    iAllocSupported = EFalse;   
}

void CPluginMemHandlerImpl::AddMemHandlerL()
{
    if(!iInstanceMemInfo)
    {
        CSwfMemMgr* memMgr = iPlayerInstance->GetGlobals()->GetMemMgr();
        
        TInt maxHeap = CFlashConfigGen::GetFlashMaxDynHeapSize(iPlayerInstance->GetInteractionMode());
        
        iInstanceMemInfo = CPluginInstanceMemInfo::NewL(memMgr,iPlayerInstance,maxHeap);
        
        memMgr->AddPluginMemInfoL(iInstanceMemInfo);
        
        iAllocSupported = ETrue;
    }    
}

#endif //PLUG_IN_MEM_ALLOC

TUint CPluginMemHandlerImpl::GetMaxHeapSize()
    {
    CPluginGlobals* pluginGlobals = iPlayerInstance->GetGlobals();

    if (!pluginGlobals)
        return 0;

    return pluginGlobals->GetMaxHeapSize(iPlayerInstance->GetInteractionMode());
    }

void CPluginMemHandlerImpl::AddDynHeapAlloced(TUint size)
    {
    CPluginGlobals* pluginGlobals = iPlayerInstance->GetGlobals();

    if (!pluginGlobals)
        return;

    return pluginGlobals->AddDynHeapAlloced(size);
    }

void CPluginMemHandlerImpl::DynHeapFreed(TUint size)
    {
    CPluginGlobals* pluginGlobals = iPlayerInstance->GetGlobals();

    if (!pluginGlobals)
        return;

    return pluginGlobals->DynHeapFreed(size);
    }

#ifdef PLUG_IN_MEM_ALLOC    
TBool CPluginMemHandlerImpl::IsMemAllocSupported()
    {
    return iAllocSupported;
    }

TAny* CPluginMemHandlerImpl::AllocMem(TUint16 aNumBlocks)
    {
    if (iInstanceMemInfo)
        {
        return iInstanceMemInfo->AllocMem(aNumBlocks);
        }

    //NPPDebug(LOG_ERROR,_L("Error : not supported , asking CFlashPluginControl:AllocMem"));

    return NULL;
    }

void CPluginMemHandlerImpl::FreeMem(TAny* aMemory)
    {
    if (iInstanceMemInfo)
        {
        return iInstanceMemInfo->FreeMem(aMemory);
        }

    //NPPDebug(LOG_ERROR,_L("Error : not supported , asking CFlashPluginControl:FreeMem"));
    }

void CPluginMemHandlerImpl::ReAdjustMaxHeapforVideoStream(TUint aMaxHeapSize)
    {
    if (iInstanceMemInfo)
        {
#ifdef PLUG_IN_MEM_ALLOC
        iInstanceMemInfo->ReAdjustMaxHeapforVideoStream(aMaxHeapSize);
#endif        
        }
    }
#endif

