/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Stream Factory Handler for Flash
*
*/
 

#include "saplayer_build.h"
#include <coemain.h>
#include "flashplugininstance.h"
#include "streamhandler.h"
#include "streamfactory.h"

#include "pluginview.h"

const TUint KBrowserPluginArrayGranularity = 4;
const TInt  KDefaultStreamSize      = 1024 * 8;
const TUint KZerothStream = 0;
extern void NPPDebug( TUint8 aLogLevel,TRefByValue<const TDesC> aText, ... );

_LIT8(KDefaultMimeType,"text/html");

/**
 * Symbian constructor.
 *
 * @since S60 v3.1
 * @return Pointer to created CPluginStreamFactory object
 */
CPluginStreamFactory* CPluginStreamFactory::NewL(CFlashPluginInstance* aPluginInstance,
        MPluginView* aPluginView)
{
	NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::NewL() ++"));

	CPluginStreamFactory* self = new(ELeave) CPluginStreamFactory(aPluginInstance,aPluginView);

	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);

	NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::NewL() --"));

	return self;	
}

/**
 * Destructor.
 */
CPluginStreamFactory::~CPluginStreamFactory()
{
	CFlashPluginStreamHandler* StreamInfo=NULL;
    while(iStreamSizeArray->Count() > 0)
    {
        StreamInfo = iStreamSizeArray->At(0);
        
        iStreamSizeArray->Delete(0);           
        
        if(StreamInfo)
        {
        	delete StreamInfo;
        	StreamInfo = NULL;	
        }
        
    }
    
    delete iStreamSizeArray;
}


/**
 * C++ default constructor.
 *
 * @param aCtrl CBrowserPluginControl object.
 */
CPluginStreamFactory::CPluginStreamFactory(CFlashPluginInstance* aPluginInstance,
        MPluginView* aPluginView):
        iPluginInstance(aPluginInstance),
        iPluginView(aPluginView)
{
	
}

/**
    * By default Symbian 2nd phase constructor is private.
    */
void CPluginStreamFactory::ConstructL()
{
	iStreamSizeArray = new (ELeave) CArrayPtrFlat<CFlashPluginStreamHandler>(
                                KBrowserPluginArrayGranularity); 
    
    
}

TInt CPluginStreamFactory::FindStream(unsigned short aStreamId)
{
	TInt index = -1;
    TInt maxIndex = iStreamSizeArray->Count();
    
    //Search if this StreamId is already there in the array
    for ( index=0; index < maxIndex; index++)
        {
        	CFlashPluginStreamHandler* stream = iStreamSizeArray->At(index); 
        	
        	if(*stream == aStreamId)
        		return index;
        }
       
    return -1;
        
}

///////////////////////////////////////////////////////////////////////////
//MPluginStreamHandler methods
 								
NPError	CPluginStreamFactory::NewStreamL(NPAPIMIMEType  aType ,
        					NPAPIStreamData* aStream,
							NPAPIBool /*aSeekable*/,
							uint16* aStype)
{
	
	TInt streamID = REINTERPRET_CAST(TInt, aStream->notifyData);
	
	NPPDebug(LOG_INFO,_L("CPluginStreamFactory::NewStreamL:%d ++"),streamID);
	
	CFlashPluginStreamHandler* objStream=NULL;
	
	if(streamID == KZerothStream)
   		{
   		NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::NewStreamL:Stream As File"));
   		const int np_ASFILE  = 3;   		
   		*aStype = np_ASFILE; //first stream assume dto swf content, we want this as file
   		}
    else
        {
        NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::NewStreamL:normal Stream "));
        const int np_NORMAL  = 1;
     	*aStype = np_NORMAL;
     	}
    
    TInt idx = FindStream(streamID);
     	
	if( idx == -1)
	{
		//new stream, add entry
		NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::NewStreamL, Adding stream"));
		
         // Reason for revert is given in task description : synergy task id. bh1s60#71512.
		objStream = CFlashPluginStreamHandler::NewL(iPluginInstance
                                            ,iPluginView
											,streamID
											,KDefaultStreamSize
											,aType
											,aStream->url->Des()
											,aStream->end
											/*,aStream->headers*/);
											
		iStreamSizeArray->AppendL(objStream);
	}
	
	NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::NewStreamL: --"));
	
	return KErrNone; //NPERR_NO_ERROR;	
}

NPError CPluginStreamFactory::DestroyStreamL(TInt32 aStreamID
						,NPAPIReason aReason)
{
	NPPDebug(LOG_INFO,_L("CPluginStreamFactory::DestroyStreamL:%d ++"),aStreamID);
	
	TInt idx = FindStream(aStreamID);
	
	CFlashPluginStreamHandler* objStream=NULL;
	if(idx != -1)
	{
		// Pass http status code to flash engine
		NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::DestroyStreamL, Found Stream:%d"),idx);
		objStream = iStreamSizeArray->At(idx);
		if(objStream)
		{
			objStream->CloseStreamL(aReason);
		}
       
		iStreamSizeArray->Delete(idx);	
		
		if(objStream)
		{
			delete objStream;//now delete the object
			objStream= NULL;	
		}
		
	}
	else
	{          
        if(iPluginInstance->GetEngine() && aReason == 1)
	        iPluginInstance->GetEngine()->StreamClosed(aStreamID, aReason);
	}
	
	NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::DestroyStreamL:--"));
	
	return KErrNone; //NPERR_NO_ERROR;	
}
 									
	
TInt32 CPluginStreamFactory::WriteReadyL(NPAPIStreamData* aStream)
{
	TInt streamID = REINTERPRET_CAST(TInt, aStream->notifyData);
	NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::WriteReadyL:%d ++"),streamID);
	TInt idx = FindStream(streamID);
	CFlashPluginStreamHandler* objStream=NULL;
	TInt32 writeSize=KDefaultStreamSize;
	if(idx != -1)
	{
		objStream = iStreamSizeArray->At(idx);
		
		if(objStream)
		{
			writeSize = objStream->GetStreamBufferSize();	
		}
		
	}
	
	NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::WriteReadyL, return=%d --"),writeSize);
	
	return (writeSize > 0) ? writeSize : -1;	
}

        	
	
TInt32 CPluginStreamFactory::WriteStreamL(NPAPIStreamData* aStream, TInt32 aOffset,
           TInt32 aLen, void* aBuffer)
{
	TInt streamID = REINTERPRET_CAST(TInt, aStream->notifyData);
	
	NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::WriteStreamL:%d ++"),streamID);
	TInt idx = FindStream(streamID);
	
	CFlashPluginStreamHandler* objStream=NULL;
	
	if(idx== -1)
	{ //somehow we did not get newstream call
		//create new one
		NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::WriteStreamL: Stream Not found"));
		
		
		 // Reason for revert is given in task description : synergy task id. bh1s60#71512.
		objStream = CFlashPluginStreamHandler::NewL(iPluginInstance
                                            ,iPluginView
											,streamID
											,KDefaultStreamSize
											,KDefaultMimeType()
											,aStream->url->Des()
											,aStream->end
											/*,aStream->headers*/);
											
		iStreamSizeArray->AppendL(objStream);	
	}
	else
	{
		NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::WriteStreamL: Stream found:%d"),idx);
		
		objStream = iStreamSizeArray->At(idx);
	}
	
	
	if(objStream)
	{
		return objStream->WriteStreamL(aStream,aOffset,aLen,aBuffer);
	}
	
	NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::WriteStreamL: Returning 0--"));
		
	return 0;	
}

//
///////////////////////////////////////////////////////////////////////

void CPluginStreamFactory::SetStreamBufferSizeL(unsigned short aStreamId
        			,unsigned long aBufferSize)
{
	NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::SetStreamBufferSizeL:%d ,len=%d++"),aStreamId,aBufferSize);
	
	TInt idx = FindStream(aStreamId);
	CFlashPluginStreamHandler* objStream=NULL;
	if(idx != -1)
	{
		NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::SetStreamBufferSizeL,found stream:%d"),idx);
		objStream = iStreamSizeArray->At(idx);
		
		if(objStream)
		{
			objStream->SetStreamBufferSize(aBufferSize);
		}
	}
	
	NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::SetStreamBufferSizeL --"));
	
}

 void CPluginStreamFactory::CancelStream(TInt32 aStreamID)
 {
 	NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::CancelStreamL:%d ++"),aStreamID);
	
	TInt idx = FindStream(aStreamID);
	CFlashPluginStreamHandler* objStream=NULL;
	if(idx != -1)
	{
		NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::CancelStreamL,found stream:%d"),idx);
		objStream = iStreamSizeArray->At(idx);
		
		if(objStream)
		{
			objStream->CancelStream();
		}
	}
	
	NPPDebug(LOG_DEBUG,_L("CPluginStreamFactory::CancelStreamL --"));
 }
//End of file

