/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Stream Handler for Flash
*
*/
 


#include "saplayer_build.h"
#include <coemain.h>
#include "streamhandler.h"
#include <httperr.h>
#include <eikenv.h>
#include <DRMHelper.h>
#include <s32file.h>
#include "flash_engine.h"
#include "flashplugininstance.h"
#include "pluginview.h"

extern void NPPDebug( TUint8 aLogLevel,TRefByValue<const TDesC> aText, ... );


const TInt  KDRMHeaderSize          = 1024;

const TInt  KSwfSupportedVersion    = 0xA0;

const TUint KZerothStream = 0;
const TInt  KSwfHeaderSize = 4;

_LIT16(KSwfMimeType,"application/x-shockwave-flash");
_LIT8(KFlashMimeType,"application/x-shockwave-flash");
_LIT8(KFlashRTMPTType,"application/x-fcs");
_LIT8(KFlashOctetType,"application/octet-stream");
_LIT(KTempFileDir, "c:\\system\\temp\\");
//_LIT8( KFLVMimeType,"flv" );
//_LIT8( KFLVMimeTypeOctet,"octet" );
_LIT8( KVideoMediaMimeType,"video" );
_LIT8( KAudioMediaMimeType,"audio" );

/**
 * Symbian constructor.
 *
 * @since S60 v3.1
 * @return Pointer to created CFlashPluginStreamHandler object
 */
CFlashPluginStreamHandler* CFlashPluginStreamHandler::NewL(CFlashPluginInstance* aPluginInstance
                                            ,MPluginView* aPluginView
											,unsigned short aStreamId
                                            ,unsigned long aBufferSize
                                            ,NPAPIMIMEType aType
                                            ,const TDesC& aURL
                                            ,TUint32 aContentLength
                                           /*,const char* aContentEncoding*/
                                            )
{
	NPPDebug(LOG_DEBUG,_L("CFlashPluginStreamHandler::NewL() ++"));

	CFlashPluginStreamHandler* self = new(ELeave) CFlashPluginStreamHandler(aPluginInstance
                                                    ,aPluginView
													,aStreamId
													,aBufferSize
													,aContentLength);

	CleanupStack::PushL(self);
	/**
    * Reason for commenting out aContentEncoding 
    * is given in task description : synergy task id. bh1s60#71512.
    */
	self->ConstructL(aType,aURL/*,aContentEncoding*/);
	CleanupStack::Pop(self);

	NPPDebug(LOG_DEBUG,_L("CFlashPluginStreamHandler::NewL() --"));

	return self;
}

/**
 * Destructor.
 */
CFlashPluginStreamHandler::~CFlashPluginStreamHandler()
{
	NPPDebug(LOG_DEBUG,_L("CFlashPluginStreamHandler::~CFlashPluginStreamHandler() ++"));
	
	if(iDRMHeaderBuffer)
	{
		delete iDRMHeaderBuffer;
    	iDRMHeaderBuffer = NULL;	
	}
	
	if(iMimeType)
	{
		delete iMimeType;
    	iMimeType = NULL;
	}
	
	if(iURL)	
	{
		delete iURL;
		iURL = NULL;
	}
	
	NPPDebug(LOG_DEBUG,_L("CFlashPluginStreamHandler::~CFlashPluginStreamHandler() --"));
}

/**
 * C++ default constructor.
 *
 * @param aCtrl CBrowserPluginControl object.
 */
CFlashPluginStreamHandler::CFlashPluginStreamHandler(CFlashPluginInstance* aPluginInstance
                                    ,MPluginView* aPluginView
									,unsigned short aStreamId
                                    ,unsigned long aBufferSize
                                    ,TUint32 aContentLength
									): iPluginInstance(aPluginInstance)
									,iPluginView(aPluginView)
									,iStreamId(aStreamId)
									,iBufferSize(aBufferSize)
									,iNumHdrBytesReceivedSoFar(0)
									,iDRMHeaderHasArrived(EFalse)
									,iIsProtected(EFalse)
									,iVersionNotChecked(ETrue)
									,iContentLength(aContentLength)
									,iCancelState(EFalse)
									,iMediaStreamType(ENotMediaStream)
{
	
}

/**
    * By default Symbian 2nd phase constructor is private.
    */
void CFlashPluginStreamHandler::ConstructL(NPAPIMIMEType aType,const TDesC& aURL /*,const char* aContentEncoding*/)
{
	//if(aURL)
	{
		iURL = HBufC8::NewL(aURL.Length() + 1);
    	TPtr8 finalURL = iURL->Des();
    	finalURL.Copy(aURL);	
    	finalURL.ZeroTerminate();
    	NPPDebug(LOG_DEBUG,_L("CFlashPluginStreamHandler::New URL Stream; [%s]")
    	,iURL);	
	}
	/*
    * Reason for revert is given in task description : synergy task id. bh1s60#71512.

	ModifyContentLengthIfCompressed(aContentEncoding);
	*/	
	iDRMHeaderBuffer = HBufC8::NewL(iBufferSize);
	
	iMimeType = HBufC8::NewL(aType.Length()+1);
	TPtr8 mimetype (iMimeType->Des());
	mimetype.Copy(aType);
	mimetype.ZeroTerminate();	

    if(mimetype.Find(KVideoMediaMimeType) >=0)
        iMediaStreamType = EVideoStream;
    else
    if(mimetype.Find(KAudioMediaMimeType) >=0)
        iMediaStreamType = EAudioStream;
    else
    if(mimetype.Find(KFlashRTMPTType) >=0)
    	iMediaStreamType = ERTMPTStream;
    	    
    else
    if(mimetype.Find(KFlashOctetType) >=0)
        iMediaStreamType = EOctetStream;
        	
    else
    	iMediaStreamType = ENotMediaStream;

	
/*	TInt flvPos = aType.Find( KFLVMimeType );
	TInt octetPos = aType.Find( KFLVMimeTypeOctet );
	
	if(flvPos>=0 || octetPos>=0)
	{
		NPPDebug(LOG_DEBUG,_L("CFlashPluginStreamHandler::New FLV Stream"));
	}
*/
}

TBool CFlashPluginStreamHandler::operator==(unsigned short aStreamId)
{
	if(aStreamId  == iStreamId)	
	{
		return ETrue;
	}
	else
	{
		return EFalse;
	}
}

TBool CFlashPluginStreamHandler::CheckSwfVersionForStream(const char* aSwfData)
{
    // Version is already checked.
    if ( !iVersionNotChecked )
    {
        return ETrue;
    }
       
    // we have enough to go on: check the version
    if (( NULL != aSwfData) && ( aSwfData[KSwfHeaderSize - 1] <= KSwfSupportedVersion ))
    {
        // correct version
        iVersionNotChecked = EFalse;
        NPPDebug(LOG_ERROR,_L("CFlashPluginStreamHandler::CheckVersion() : Flash Version mismatch %x"), aSwfData[KSwfHeaderSize - 1]);
        return ETrue;
    }


    NPPDebug(LOG_ERROR,_L("CFlashPluginStreamHandler::CheckVersion() : Flash Version mismatch"));

    return EFalse;
}

// -------------------------------------------------------------------------
// CFlashPluginStreamHandler::isFlashContentL(CContent* aContent) 
//
// check whether content is flash swf
// --------------------------------------------------------------------------
//
TBool CFlashPluginStreamHandler::isFlashContentL(const CContent* aContent) 
{
	TBool isFlash = EFalse;
		
	if(aContent)
	{
    	HBufC* tmpMIME = HBufC::NewLC( KMaxDataTypeLength );

    	TPtr mimePtr( tmpMIME->Des() );            

    	aContent->GetStringAttribute(EMimeType, mimePtr );

      isFlash =  (0 != mimePtr.Compare(KSwfMimeType))? EFalse : ETrue ;

      CleanupStack::PopAndDestroy(tmpMIME);
    }

    return isFlash;
}

// -------------------------------------------------------------------------
// void CFlashPluginStreamHandler::CheckPlayRightsL(CContent* aContent)
// Used to check if the DRM file has the required play rights
// --------------------------------------------------------------------------
// 
TBool CFlashPluginStreamHandler::CheckPlayRightsL(const CContent* aContent)
    {
    if( !aContent )
        {
        User::Leave(KErrArgument);
        }
        
    // Check if the content object is protected
    TInt attributeValue = 0;
    aContent->GetAttribute(ECanExecute, attributeValue);    
    
    // Check if the content object can be display on screen
    if(attributeValue)
        {
        // content object has rights that allow it to be displayed on screen
        return ETrue;
        }

    return EFalse;
    } 
    
    
// -------------------------------------------------------------------------
// TBool CFlashPluginStreamHandler::PerformDrmChecksL(const TDesC& aFilename, 
//                                            TBool& aCorrectMimeAndRights)
// Used to perform checks related to DRM
// aFileName - DRM protected file
// aCorrectMimeAndRights - ETrue if the file has correct MIME type and it has
//                          adequate playrights, EFalse otherwise
// Returns ETrue if file is DRM protected SWF file and EFalse otherwise 
// --------------------------------------------------------------------------
// 
TBool CFlashPluginStreamHandler::PerformDrmChecksL(const TDesC& aFilename, 
                                                const CContent* aContent, 
                                            TBool& aCorrectMimeAndRights)
{
	TBool isProtected     = EFalse;
	aCorrectMimeAndRights = EFalse;
	
    if( !aContent )
        {
        User::Leave(KErrArgument);
        }

	aContent->GetAttribute( EIsProtected, isProtected );

	if ( !isProtected )
	    {	    
	    return EFalse; 
	    }
	    
    //Check if the MIME type is correct

    if ( isFlashContentL(aContent) )
        {
    		CDRMHelper* drmHelp = iPluginInstance->GetDRMHelperL();
    		
			if(drmHelp)
	        	{
	        		drmHelp->CheckRightsAmountL( aFilename );    	
	        	}
	     	
        
	        if ( CheckPlayRightsL(aContent) )
	            {
	            aCorrectMimeAndRights = ETrue;
	            }        
        }

	return ETrue;
}

// ----------------------------------------------------------------------
// TBool CFlashPluginStreamHandler::ExecuteIntentAndWriteL(TInt& aStreamId, 
//                                                  const TDesC& aFilename)
//
// This function is used to check the DRM stream and execute intent
// ----------------------------------------------------------------------
// 

TBool CFlashPluginStreamHandler::ExecuteIntentAndWriteL(const TInt& aStreamId, 
                                                  const TDesC& aFilename)
    {
    TInt length = 0;
	HBufC8* swfbuf = 0;
	CData* data = NULL;
	TBool result = EFalse;
	TPtr8 bufptr(NULL,0);
	
	//Create a CContent Object.
	CContent* content = CContent::NewLC(aFilename);

		//Opening the content with Intention of playing.

	TRAPD(err, data = content->OpenContentL( ContentAccess::EExecute ));

		if ( err == KErrNone )
		{
			CleanupStack::PushL( data );
			//Get the size of content.
			TRAPD( sizeError, data->DataSizeL( length ) );

			if ( sizeError != KErrNone )
				{
				CleanupStack::PopAndDestroy( data );
			CleanupStack::PopAndDestroy( content ); // content
			NPPDebug(LOG_ERROR,_L("CFlashPluginStreamHandler::ExecuteIntentAndWriteL() : sizeError != KErrNone"));
			return EFalse;
			}

		if( aStreamId != KZerothStream )
		    {
		  	data->ExecuteIntent(ContentAccess::EExecute);      
		    }		

		// Length because the buffer needs to be null terminated
		swfbuf = HBufC8::NewLC(length);

		bufptr.Set(swfbuf->Des());
		data->Read(bufptr);
	}

	// check for "SWF" header
    if ( aStreamId == KZerothStream )
        {
        
        // are we at the start
        result = CheckSwfVersionForStream((const char*)swfbuf->Des().Ptr());
            
    	CleanupStack::PopAndDestroy(swfbuf);
    	
    	if ( err == KErrNone )
    	    {
    		CleanupStack::PopAndDestroy(data);
    	    }
    	
    	CleanupStack::PopAndDestroy(content);
    	
    	return result;
        }
        		
    NPPDebug(LOG_ERROR,_L("CFlashPluginStreamHandler::ExecuteIntentAndWriteL() : OOPS! This should never happen!"));	
    
	return result;
    }



// ----------------------------------------------------------------------
// TBool CFlashPluginStreamHandler::CheckHeaderAndWriteForDRML(TBool& isDRM)
//
// This function is used to check the DRM header write into file
// ----------------------------------------------------------------------
// 

TBool CFlashPluginStreamHandler::CheckHeaderAndWriteForDRML(TBool& isDRM)
{
    RFs fs;
    TBool success = ETrue;
    
    User::LeaveIfError( fs.Connect() );
    CleanupClosePushL( fs );
    fs.SetSessionPath( KTempFileDir );    
    
    RFileWriteStream writeStream;
    TFileName fileName;
    
    if( writeStream.Temp( fs, KTempFileDir, fileName, EFileWrite ) == KErrNone )
    {
        CleanupClosePushL( writeStream );
        writeStream.WriteL(*iDRMHeaderBuffer,
                            iNumHdrBytesReceivedSoFar);
        writeStream.CommitL();
        CleanupStack::PopAndDestroy(); // writeStream
    }       
    
    TBool correctMimeAndRights = EFalse;
    CContent* content = CContent::NewLC(fileName); 
       
    isDRM = PerformDrmChecksL(fileName,content,
                                               correctMimeAndRights);
    if (isDRM && correctMimeAndRights)
    {
               
        if( iStreamId == KZerothStream )
        {            
            success = ExecuteIntentAndWriteL(iStreamId, fileName);
        }
        else
        {
        	iIsProtected = ETrue;
        }
    }
    
    CleanupStack::PopAndDestroy(content); // content    
    
    fs.Delete(fileName);    
    CleanupStack::PopAndDestroy(); // fs  
    
    return success;
}

#if 0
TBool CFlashPluginStreamHandler::IsFLVStreamL(const TDes8& aBuf)
{
	_LIT8(KFLV, "FLV");
	
	
	TPtrC8 typePtr = aBuf.Left(3);
	if( !typePtr.Compare(KFLV) )
	{
		iFLVStream = ETrue;
		iPluginInstance->ChangeFLVPlayStateL(EFlashPluginFLVStreamInProgress);
		return ETrue;
	}
		
	
	return EFalse;
			
}
#endif

/*
 Reason for revert is given in task description : synergy task id. bh1s60#71512.

//-----------------------------------------------------------------------------
// Sets the FLV play state to EFlashPluginFLVStreamInProgress
//-----------------------------------------------------------------------------
//
void CFlashPluginStreamHandler::SetFLVPlayStateInProgressL()
    {
    if(iMediaStreamType == EVideoStream || iMediaStreamType== EAudioStream || 
        iMediaStreamType== ERTMPTStream || iMediaStreamType== EOctetStream)
        {
        iPluginView->ChangeFLVPlayStateL(EFlashPluginFLVStreamInProgress, iMediaStreamType);
        }    
    }
*/

TBool CFlashPluginStreamHandler::IsMediaStreamL()
{
    if(iMediaStreamType == EVideoStream || iMediaStreamType== EAudioStream || iMediaStreamType== ERTMPTStream || iMediaStreamType== EOctetStream)
    {
        RDebug::Print(_L("Media Stream"));
        iPluginView->ChangeFLVPlayStateL(EFlashPluginFLVStreamInProgress, iMediaStreamType);
        return ETrue;
    }
    
    return EFalse;      
}

TInt32 CFlashPluginStreamHandler::WriteStreamL(NPAPIStreamData* /*aStream*/, TInt32 aOffset,
           TInt32 aLen, void* aBuffer)
{
	
	if(iCancelState)
		{
		return KErrOverflow;
		}
	
	if ((iStreamId == KZerothStream) || 
    		 (iIsProtected ) || iCancelState)
	{
	//This is commented out as DRM protected files will not give version info in the header and need to be decrypted first
	//this same check is done in host engine
	/*
		TUint8* buff = (TUint8*) aBuffer + aOffset;	
	    TBool	success = CheckSwfVersionForStream((const char*)buff) ;
        if(!success)
		{
			iPluginInstance->SetErrorOccured(EInvalidFileVersionError);
			iPluginInstance->NotifyClientError(iPluginInstance->GetLastError(),ETrue,ETrue); 
	    }
	*/
		return aLen;
	}
	
	NPPDebug(LOG_DEBUG,_L("CFlashPluginStreamHandler:WriteStreamL:len=%d offset=%d++")
	        					,aLen,aOffset);	
	        						
	TBool stat = EFalse;
	TUint8* buff = (TUint8*) aBuffer + aOffset;	
	
	MFlashEngine* engine = iPluginInstance->GetEngine();	
	
	if(!engine)
	{
		return KErrOverflow;
	}
	
	if (!iDRMHeaderHasArrived)
    {   
        // If data received is less than KDRMHeaderSize then it needs 
        // to be stored until the entire DRM header has arrived
        
        TPtr8 buf = iDRMHeaderBuffer->Des();
        buf.Append((const TUint8*)buff, aLen);
        
        iNumHdrBytesReceivedSoFar += aLen;   
        
        if (iNumHdrBytesReceivedSoFar >= KDRMHeaderSize)
        {
	        iDRMHeaderHasArrived = ETrue;
	            
	        // Check whether the header data is actually DRM
	        // and whether the write is successful.
	        
	        TBool isDRM = EFalse;
	        TBool success = CheckHeaderAndWriteForDRML(isDRM);
	        //TODO: this can be buggy,where is DRM stream handled?
	        if(isDRM)
	        	{
	        	return aLen; //return len we will take it from asfile?
	        	}
	           	
	           	
	        if(iMimeType->CompareF(KFlashMimeType) == KErrNone)
				{
				if( iVersionNotChecked && iNumHdrBytesReceivedSoFar > KSwfHeaderSize)
					{
					if( !CheckSwfVersionForStream((const char*)buf.Ptr()) )
						{
						return KErrOverflow;
						}
					}
				}	        
	        
	        /*
	         * Reason for revert is given in task description : synergy task id. bh1s60#71512.
             * The fix is expected from browser, with the fix, SetFLVPlayStateInProgressL(); call
	         * should replace:  if(!IsMediaStreamL()) { iContentLength = -1; }	         
	        
	        SetFLVPlayStateInProgressL();
	        */
	        
	        // Non - drm content.
            // Checking if buf has FLV instead of buff since buf has the entire data.
            // This is the fix for the bug : SRAA-78WD6L.
            if(!IsMediaStreamL())
                {
                iContentLength = -1; //for Non FLv make it -1
                }
	        
	        NPPDebug(LOG_DEBUG,_L("CFlashPluginStreamHandler:NewURLStreamL to engine:%d,len=%d")
	        					,iStreamId,iNumHdrBytesReceivedSoFar);	
	        
	        
			const char* url =       (const char*)(iURL->Ptr());
			const char* data =   (const char*)(iDRMHeaderBuffer->Des().Ptr());
			
			TPtr8 lPtr(iMimeType->Des());
	        engine->NewURLStreamL(url, iStreamId ,iContentLength, lPtr );
	        
	        NPPDebug(LOG_DEBUG,_L("CFlashPluginStreamHandler:WriteStreamDataL to engine:%d,len=%d")
	        					,iStreamId,iNumHdrBytesReceivedSoFar);
	        					
	        TBool stat = engine->WriteStreamDataL(
	                                		iStreamId,
	                                		data, 
	                                		iNumHdrBytesReceivedSoFar
	                                        );
			/*
			 *Changed from : return (stat)? iNumHdrBytesReceivedSoFar:-1;
			 *This was causing us to return wrong value to Browser if the
			 *first data block from HTTP is <1024 Bytes, as we waited for
			 *atleast 1024 bytes to arrive(CompleteHeader) and returned the
			 *total of the HTTP data arrived at this point(iNumHdrBytesReceivedSoFar)
			 *instead of just the length of data sent by Browser(aLen).This caused
			 *skipping of data as length of data received and returned didnt match
			 *causing a Memory Leak
			 */
			return (stat)? aLen:-1;
        }
        else
        { //DRM header ha not arrived so wait for next stream
        	return aLen; 	
        }
        
        
    }
    else
    {
    	NPPDebug(LOG_DEBUG,_L("CFlashPluginStreamHandler:WriteStreamDataL to engine:%d,len=%d ")
	        					,iStreamId,aLen);
	        
    	stat= engine->WriteStreamDataL(iStreamId, (const char*)(buff), aLen);  	
    }
       
    

	return (stat)? aLen:-1;
	
}


TInt32 CFlashPluginStreamHandler::GetStreamBufferSize()
{
//HACK
	if(iCancelState)
		{
		return KErrOverflow;
		}
	
	if (iDRMHeaderHasArrived == (TInt)ETrue)
    {
     return iBufferSize;    
    }
	else
    {
	    //The entire DRM header has not arrived.
	    //So we do not ask for a new buffer from 
	    //the core player
	    return (iBufferSize - iNumHdrBytesReceivedSoFar);    

    }
	
}
    	
void CFlashPluginStreamHandler::SetStreamBufferSize(TInt32 aSize)
{
	iBufferSize = aSize;
}
    	
void CFlashPluginStreamHandler::CloseStreamL(NPAPIReason aReason)
{
	if( iStreamId == KZerothStream || iCancelState )
		return;
	
	unsigned short status;

	enum 
	{
		np_RES_BASE, 
		np_RES_DONE 		= (np_RES_BASE + 0),
		np_RES_NETWORK_ERR 	= (np_RES_BASE + 1),
		np_RES_USER_BREAK 	= (np_RES_BASE + 2)
	};

	switch (aReason)
    {
        case np_RES_DONE:
            status = HTTPStatus::EOk;
            break;
        case np_RES_NETWORK_ERR:
        case np_RES_USER_BREAK:
        default:
        // use same error code for all other cases
        // flash destroys stream if not 200
        status = HTTPStatus::EGatewayTimeout;
        break;
    }
    
    MFlashEngine* engine = iPluginInstance->GetEngine();	
    
    if(!engine)
    {
    	return;
    }
    
    //need to send notification to player engine
    if (iStreamId && !iDRMHeaderHasArrived && (status == HTTPStatus::EOk))
	    { 
	    TPtr8 lPtr(iMimeType->Des());
	    //we have not send the stream request , so send it now
	        TRAP_IGNORE(engine->NewURLStreamL(((const char*)iURL->Ptr())
	        						, iStreamId, iNumHdrBytesReceivedSoFar, lPtr));
	        
	        //No DRM Header. Pass it as such to the core player
	        TRAP_IGNORE(engine->WriteStreamDataL(iStreamId, 
	            (const char*)iDRMHeaderBuffer->Des().Ptr(), 
	                                iNumHdrBytesReceivedSoFar));    
	    }
	    
	// The browser has completed the stream
	if(!iCancelState)
		engine->CloseStream(iStreamId, status,iMediaStreamType);
	
	if(iMediaStreamType != ENotMediaStream)
	{
	iPluginView->ChangeFLVPlayStateL(EFlashPluginFLVStreamClosed, iMediaStreamType);
	}
	
}


void CFlashPluginStreamHandler::CancelStream()
{ //set the state of the stream to cancelled
	// reason : to browser, if  we send error for write stream
	
	iCancelState = ETrue;
}

#ifdef __DUMP_TO_FILE

void CFlashPluginStreamHandler::DumpToFile(TInt& aStreamId,
                                       const char* aData, 
                                       unsigned long aDataLength)
{
    if( !aData || !aDataLength )
        return;
    
    TFileName fileName;
    fileName.Copy(_L("C:\\Logs\\NPLogs\\"));
    fileName.AppendFormat(_L("%d[%d]"), this, aStreamId);
    
    TPtr8 buf((TUint8*)aData, aDataLength, aDataLength);
    
    RFs fs;
    TRAPD(result,fs.Connect());
    if(result==KErrNone)
    {
    RFile dumpFile;
    TInt error  = dumpFile.Open(fs, fileName, EFileWrite);
    if( error == KErrNotFound )
        {
        error = dumpFile.Create(fs, fileName, EFileWrite);
        }
    if( error == KErrNone )
        {
        TInt pos = 0;
        dumpFile.Seek(ESeekEnd, pos);
        dumpFile.Write(buf);
        }
    dumpFile.Close();
    }
    fs.Close();
}

#endif

/*
Reason for revert is given in task description : synergy task id. bh1s60#71512.
//-----------------------------------------------------------------------------
// The function resets the content length according to the type of content
// encoding. For any data encoding other than UTF-8, iso 8859 1, 
// Identity [no encoding] the content length will be reset.
//-----------------------------------------------------------------------------
//
void CFlashPluginStreamHandler::ModifyContentLengthIfCompressed( 
    const char* aContentEncoding )
    {
    if( aContentEncoding )
        {
        TPtrC8 headerToParse;
        headerToParse.Set((TUint8*)aContentEncoding);
        _LIT8(KUtf8,"utf-8");
        _LIT8(KIso8859,"iso-8859-1");
        // reset the content length if the encoding type is not 
        // utf-8,identity,iso8859-1  
        if( ( 0 != headerToParse.CompareF( KNullDesC8 ) ) && 
            ( 0 != headerToParse.CompareF( KUtf8) ) &&
            ( 0 != headerToParse.CompareF( KIso8859 ) ) )
            {
            iContentLength = -1;
            }
        }
    else
        {
        // if the header is not present we will reset the content length to =-1 
        // so core can use the dynamic buffer
        iContentLength = -1;
        }
    }
 */       
	
//End of file

