/*
* Copyright (c) 2007-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash Browser Plugin Memry handler implementation
*
*/


#include "saplayer_build.h"
#include <eikenv.h>
#include <e32std.h>
#include "swfmemmgr.h"
#include "flashplgmain.h"
#include "plugininstancememinfo.h"
#include <e32debug.h>
#include <e32hal.h>
#include <hal.h>


extern void NPPDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );

const TUint 	KDLC_MAX_SIZE 		= 0x4000000; //(1024*1024*64); //64MB
const TUint		KRAMFreeCheckMark	= 0x500000; // (5*1024*1024); //RAM free check mark for 5 MB

/* This is a very simple allocator 
 * DLC_BLOCK_SIZE is defined in SwfMemMgr.h. */
#define DLC_BLOCK_OFFSET(i) ((i)*(KDLC_BLOCK_SIZE))


CSwfMemMgr* CSwfMemMgr::NewL(CPluginGlobals* aGlobal,TUint aMaxAllowdHeap)
{
	CSwfMemMgr* self = new (ELeave) CSwfMemMgr(aGlobal,aMaxAllowdHeap);
    
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop();
    
    return self;	
}
    
CSwfMemMgr::CSwfMemMgr(CPluginGlobals* aGlobal,TUint aMaxAllowdHeap):
						iGlobal(aGlobal)
						,iDynHeapSize(0)
						,iFirst(NULL)
						,iLast(NULL)
						,iMaxAllowdHeap(aMaxAllowdHeap)
						,iRAMFreeCheckMark(KRAMFreeCheckMark)
{
	
}

void CSwfMemMgr::ConstructL()
{
	NPPDebug(LOG_DEBUG,_L("CSwfMemMgr::ConstructL() +"));
	TInt error=KErrNone;
    int i;
    iDlcChunk=new RChunk;
    
    if (!iDlcChunk) {
        iDlcChunk=NULL;
    }
    
    /* Create a disconnected local chunk of maximum size DLC_MAX_SIZE, with no physical memory committed at first. */
    NPPDebug(LOG_DEBUG,_L("FL:Create Disconnected local"));
    error=iDlcChunk->CreateDisconnectedLocal(0, 0, KDLC_MAX_SIZE);
    if (error != KErrNone) {
        delete iDlcChunk;
        iDlcChunk=NULL;
        NPPDebug(LOG_ERROR,_L("Create Disconnected local failed;%d"),error);
        
    }
    
    for (i=0; i<KDLC_MAX_BLOCKS; i++)
        iDlcBlockFreeStats[i]=1;
    
    NPPDebug(LOG_DEBUG,_L("CSwfMemMgr::ConstructL() -"));
    
}

CSwfMemMgr::~CSwfMemMgr()
{
	NPPDebug(LOG_DEBUG,_L("CSwfMemMgr::~CSwfMemMgr() +"));
	FreeMemInfoNode();
	
	if (iDlcChunk != NULL) {    
        iDlcChunk->Close();
        delete iDlcChunk; 
        iDlcChunk=NULL;
    }
    NPPDebug(LOG_DEBUG,_L("CSwfMemMgr::~CSwfMemMgr() -"));
}

void CSwfMemMgr::FreeMemInfoNode()
{
	NPPDebug(LOG_DEBUG,_L("CSwfMemMgr::FreeMemInfoNode() +"));
	CPluginInfoList* pNode = iFirst;
	CPluginInfoList* pPrev = NULL;
		
	while(pNode != NULL)
	{
		pPrev = pNode;
		
		pNode = pNode->iNext;
		
		delete 	pPrev;
	}
	
 	NPPDebug(LOG_DEBUG,_L("CSwfMemMgr::FreeMemInfoNode() -"));
}

void CSwfMemMgr::AddDynHeapAlloced(TUint aSize)
    {
    iDynHeapSize += aSize; 
    }
    
void CSwfMemMgr::DynHeapFreed(TUint aSize)
    {
    iDynHeapSize -= aSize;
    }

/////////////////////////////////////////////////    

TInt CSwfMemMgr::FindFreeContigBlock(TUint16 aNumBlocks)
{
	//find first free block
	TInt i =0, j=0;
	
	do
	{
		//search for available free block
		for ( ; i<KDLC_MAX_BLOCKS; i++)		
		{
			if(iDlcBlockFreeStats[i])	
				break; //found free block exit loop
		}
		
		if( i < KDLC_MAX_BLOCKS) //check we reached the limit
		{
			//now enumerate for contig free numblcoks
			for (j=0 ; j < aNumBlocks ; j++ )
			{
				if(!iDlcBlockFreeStats[i])
					break;
				i++;
				
				if(i > KDLC_MAX_BLOCKS)
					break;
			}
			
			if(j == aNumBlocks) //if found one
			{
				//found matching block
				return (i - aNumBlocks);
			}
		}
		
	} while ( i < KDLC_MAX_BLOCKS);
	
	return -1;
}

//check for RAM critical level

TBool CSwfMemMgr::IsMemUnderLimit()
{
//	TMemoryInfoV1Buf meminfo;
 //   TInt r=UserHal::MemoryInfo(meminfo); 
 	TInt      freeRAM=0;
 	TInt r = HAL::Get(HAL::EMemoryRAMFree,freeRAM);
 	
    if((r!=KErrNone)  || ( freeRAM < iRAMFreeCheckMark))
     {
     	 NPPDebug(LOG_INFO,_L("Free RAM below Limit:%d or Error =%d"),freeRAM,r);
     	 return EFalse;
     }	
     
    return ETrue; 
}

TAny* CSwfMemMgr::AllocMem(TUint16 aNumBlocks)
{
	if(!IsMemUnderLimit())
	{
		NPPDebug(LOG_INFO,_L("Free RAM limit is below than expected:"));
		return NULL;
	}
		
	
	TInt size = aNumBlocks * KDLC_BLOCK_SIZE;

	// check to see if this allocation will exceed our limit.
	// if ((iPlayerDynamicHeapAlloced + size) > iPlayerDynamicHeapMaxSize)
	if ((iDynHeapSize + size) > iMaxAllowdHeap)
	{
		
			return NULL;	
	}

	//check if we have these many contigous blocks
	TInt blkIdx = FindFreeContigBlock(aNumBlocks);
	
	if( blkIdx != -1)
	{
		//commit mem
		//TODO : need to check poss of high mem alloc
	//	NPPDebug(_L("Re Mem allocation idx =%d,Adr =%d,noblk=%d")
	//				,blkIdx,DLC_BLOCK_OFFSET(blkIdx),aNumBlocks);
					
		TInt error=iDlcChunk->Commit(DLC_BLOCK_OFFSET(blkIdx), size); 
        
        if (error != KErrNone)
        {
        	NPPDebug(LOG_ERROR,_L("Mem allocation fail:%d"),error);
        	return NULL;            	
        }
        
        
		//now set the block as occupied
		NPPDebug(LOG_DEBUG,_L("DLC allocated block Idx#%d nos=%d"),blkIdx,aNumBlocks);
		TInt i =0,j=0;
		for (i=blkIdx, j=0 ; j <aNumBlocks ; i++,j++ )
		{
			iDlcBlockFreeStats[i] = 0;
	//		NPPDebug(_L("DLC block Mark bsy#%d"),i);	
		}
				
		AddDynHeapAlloced(size);

		
		
		return (iDlcChunk->Base()+DLC_BLOCK_OFFSET(blkIdx));
	}
	
		
	return NULL;
	
}

void CSwfMemMgr::FreeMem(TAny* aMemory, TUint16 aNumBlocks)
{
	//get the offset
	TInt offset=(unsigned char *)aMemory - iDlcChunk->Base();
	
	//get the block index
    TInt block_index=offset/KDLC_BLOCK_SIZE;
    
    //if block is free then double deletion
    if (iDlcBlockFreeStats[block_index]) {
    	NPPDebug(LOG_ERROR,_L("FlashLite:MemDoubleFree #%d"),block_index);
        User::Panic(_L("FlashLite:MemDoubleFree"),1);
    }
    
    //now free the blocks
    TInt err =iDlcChunk->Decommit(offset, KDLC_BLOCK_SIZE*aNumBlocks);
    
    if(err != KErrNone)
    {
    	NPPDebug(LOG_ERROR,_L("Mem decommitt failed: Idx=%d, nos=%d"),block_index,aNumBlocks);
    }
    else
    {
    	NPPDebug(LOG_DEBUG,_L("Mem decommitt success: Idx=%d, Nos=%d"),block_index,aNumBlocks);
    }
    
    //now set the block stat to free
    NPPDebug(LOG_DEBUG,_L("DLC freed block #%d:%d"),block_index,aNumBlocks);
    TInt i =0, j=0;
    for (i=block_index, j=0 ; j <aNumBlocks ; i++ ,j++)
	{
		iDlcBlockFreeStats[i] = 1;
	//	NPPDebug(_L("DLC block Mark free#%d"),i);		
	}
	
	DynHeapFreed(aNumBlocks*KDLC_BLOCK_SIZE);
	
}


void CSwfMemMgr::AddPluginMemInfoL(CPluginInstanceMemInfo* aInstanceMemInfo)
{
	NPPDebug(LOG_DEBUG,_L("CSwfMemMgr::AddPluginMemInfoL Plugin Instance addition+ "));
	CPluginInfoList* pNode = new CPluginInfoList;
	
	pNode->iInstanceMemInfo = aInstanceMemInfo;
	pNode->iNext = NULL;
	
	if(iFirst == NULL)
	{
		iFirst = pNode;
		iLast = iFirst;
	}
	else
	{
		iLast->iNext = pNode;
		iLast = pNode;
	}
	
	NPPDebug(LOG_DEBUG,_L("CSwfMemMgr::AddPluginMemInfoL - "));
	
}

void CSwfMemMgr::RemoveMemInfo(CPluginInstanceMemInfo* aNode)
{
	NPPDebug(LOG_DEBUG,_L("CSwfMemMgr::RemoveMemInfo : Plugin Instance deletion+ "));
	
	CPluginInfoList* pNode = iFirst;
	CPluginInfoList* pPrev = NULL;
		
	while(pNode != NULL)
	{
		if(pNode->iInstanceMemInfo == aNode )
		{
			if(pNode == iFirst)			
			{
				iFirst = pNode->iNext;
               /* ESKJ-7ZMD2S: Web crashes with 
                * npflashlite.dll when using browser ng
                */
				if (pNode == iLast)
				{
				    iLast = NULL;
				}
			}
			else
			{
				if(pPrev)
				{
					pPrev->iNext = pNode->iNext;
                    /* ESKJ-7ZMD2S: Web crashes with 
                     * npflashlite.dll when using browser ng
                     */
					if (pNode == iLast)
					{
					    iLast = pPrev;
					}
				}
				
			
			}
			
			delete pNode;
			pNode = NULL;
			break; //break while 
		}
		else
		{
			pPrev = pNode;
			
			pNode = pNode->iNext;
			
		}
		
	}
	
	NPPDebug(LOG_DEBUG,_L("CSwfMemMgr::RemoveMemInfo :Plugin Instance deletion- "));
}


//End of file


