/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class that queues an active object and allows a 
				 MFlhAsyncHandler to execute code when it completes.
*
*/


#ifndef __ASYNC_OBJECT_H__
#define __ASYNC_OBJECT_H__

#include <e32std.h>
#include <e32base.h>

class MFlashClientIf;

/**
 * The MFlhAsyncHandler mixin allows a deriving class to be called back upon completion of an active object
 */
class MFlhAsyncHandler
{
public:
	virtual void DoRunL(const CActive* aActive)=0;
	virtual TInt DoRunError(const CActive* aActive, TInt aError)=0;
};


/**
* The CAsyncObject class queues an active object an allows an MFlhAsyncHandler to
* execute code when it completes. This class is like CIdle - it encapsulates an active object,
* but unlike CIdle it allows the client to Leave and handle Leaves through DoRunError().
* This class can also be used without casts and function pointers.
*/
class CFlhAsyncObject : public CActive
{
public: // c'tor / d'tor
	static CFlhAsyncObject* NewL(TInt aPriority, MFlhAsyncHandler& aHandler);
	~CFlhAsyncObject();
private:

	CFlhAsyncObject(TInt aPriority, MFlhAsyncHandler& aHandler);

public: // API
	// queues the object
	void Start(); 

private: // from CActive
	void RunL();
	TInt RunError(TInt aError);
	void DoCancel();
private:
	MFlhAsyncHandler& iHandler;
};

class CAsyncDeleter : public CActive
{
public:
    CAsyncDeleter(MFlashClientIf* aPlayer); 
    ~CAsyncDeleter();
    
public:
    void RunL();
    void DoCancel();
    void IssueRequest();
    
private:
    MFlashClientIf* iPlayer;
};


#endif	// __ASYNC_OBJECT_H__

