/*
* Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class that provides a mechanism for priming a timer
*				 using CTimer functionality, and receiving a callback in the owning class.
*
*/


#ifndef __BROWSER_TIMER_OBJECT_H__
#define __BROWSER_TIMER_OBJECT_H__

#include <e32std.h>
#include <e32base.h>

class MPlgAsyncHandler
{
public:
    virtual void DoRunL(const CActive* aActive)=0;
    virtual TInt DoRunError(const CActive* aActive, TInt aError)=0;
};

/**
 * The CFlhTimerObject class provides a mechanism for priming a timer
 * using CTimer functionality, and receiving a callback in the owning class.
 */
class CPlgTimerObject : public CTimer
{
private:
    CPlgTimerObject(TInt aPriority, MPlgAsyncHandler& aHandler);

public: // c'tor / d'tor
	static CPlgTimerObject* NewL(TInt aPriority, MPlgAsyncHandler& aHandler);
	~CPlgTimerObject();


public:
	// no API - for API see CTimer class!

private: // from CTimer
	void RunL();
	TInt RunError(TInt aError);
	
	
private:
    MPlgAsyncHandler& iHandler;
};

#endif	//__BROWSER_TIMER_OBJECT_H__

