/*
* Copyright (c) 2005-2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CFlashVolumeControl class definition
*
*/


#ifndef __FLASH_VOLUME_CONTROL_H__
#define __FLASH_VOLUME_CONTROL_H__

#include <AknVolumePopup.h>
#include "volumepropertywatchobserver.h"

//forward declaration
class MFlashEngine;



class CFlashVolumeControl : public CCoeControl, public MCoeControlObserver
#ifndef __S60_32__
                            , public MCoeControlStateObserver
#endif
{
	public:  // Constructors and destructor

    /**
    * Two-phased constructor.
    */
    static CFlashVolumeControl* NewL(CCoeControl* aParent
    								,MVolumePropertyWatchObserver*  aVolObsrvr
    								,TBool aHandleNavKey, MFlashEngine* aEngine);

    /**
    * Destructor.
    */
    virtual ~CFlashVolumeControl();	
    
    
	public: //CCoecontrol
		
		 TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
		 
		 void  HandlePointerEventL(const TPointerEvent &aPointerEvent);
		 
	     void HandleResourceChange(TInt aType);
	        
	     TInt CountComponentControls() const;
	     
	     CCoeControl* ComponentControl(TInt aIndex) const;
	        
	        

		 void RemoveObserver();
		/**
    	* Callback for timer
    	*/
    	static TInt TimerCallback(TAny* aPtr);
    	
    	void GetViewRectangle(TRect& aRect, TPoint& aPos);
    	
    	void EndPopup();
    	void CloseVolPopUp();
		

		void ShowPopupL(TInt aVolume);
		
		void ResizeToToggle();
		
		
		//Derived from MCoeControlObserver
		void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);
		
#ifndef __S60_32__
		//Derived from MCoeControlStateObserver
		TInt HandleControlStateChange(CCoeControl* aControl,TCoeState aState);
#endif		

		
	private: //methods
	
			/**
        * C++ default constructor.
        */
        CFlashVolumeControl(CCoeControl* aParent, MVolumePropertyWatchObserver*  aVolObsrvr
    						,TBool aHandleNavKey, MFlashEngine* aEngine);

        /**
        * By default Symbian 2nd phase constructor is private.
        */
        void ConstructL(CCoeControl* aParent);
        
        void FilterAndSendCommandL(TInt aCommandId);
        
	private: //attributes
	        CCoeControl* iParentControl;
        	MVolumePropertyWatchObserver*  iVolObsrv;  //not owned
	    	
	    	//flag to indicate whether this needs to handle offerkeyeveny
	    	//for devices having h/w key this control can be passive
	    	//control displaying volume level
	    	TBool 							iHandleNavKey;
	    	MFlashEngine*             iEngine; 
	    	CAknVolumePopup*		iVolPopup;
	    	
	    	CPeriodic*              iTimer; // owned
	    	
	    	TBool					iOnStack;
	    	
	         
	    	
	    	
	    	
	    	TRect iRect;
	    	TRect iChangeRect;
	    	TPoint iPos;
};

#endif //__FLASH_VOLUME_CONTROL_H__


