/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  H/w key watch observer  Used in EKA2 builds only
*
*/


#ifndef HW_KEY_PROPERTY_WATCH_H
#define HW_KEY_PROPERTY_WATCH_H

#include <e32std.h>
#include <e32base.h>
#include <e32property.h>

//
// for emulator max volume = 65535
#define WINS_DEFAULT_VOLUME 43690

//
// For HW max volume = 9
#define HW_DEFAULT_VOLUME 6

class MVolumePropertyWatchObserver;
class MHwKeyWatchObserver;

#include<remconcoreapitargetobserver.h>    //for MRemConCoreApiTargetObserver
#include<remconinterfaceselector.h>     //for CRemConInterfaceSelector
#include<remconcoreapitarget.h>         //for CRemConCoreApiTarget

// DATA TYPES
// Used for volume up/down events
enum TVolumeControlCommands
    {
    EVolumeControlCmdVolumeUpByOne =-100,//keep it negative value
    EVolumeControlCmdVolumeDownByOne,
    EHwControlCmdPausePlay,
    EHwControlCmdStop,
    EHwControlCmdRewind,
    EHwControlCmdForward,
    EHwControlCmdFastForward,
    EHwControlCmdBackward
    };

class CHwKeyPropertyWatch : public CBase, MRemConCoreApiTargetObserver
    {
public:
    
    static CHwKeyPropertyWatch* NewL();
                                         
    virtual ~CHwKeyPropertyWatch();
   // TInt GetCurrentVolume();
    /**
     * Function to add an observer
     */
    void AddObserverL(MHwKeyWatchObserver* aObserver);  
    
    /**
     * Function to remove an observer
     */
     
    void RemoveObserver(MHwKeyWatchObserver* aObserver);
    
private:
    CHwKeyPropertyWatch();
    void ConstructL();

    //From MRemConCoreApiTargetObserver (remconcoreapitargetobserver.h)
    void MrccatoCommand( TRemConCoreApiOperationId aOperationId, 
                         TRemConCoreApiButtonAction aButtonAct );
    
    /**
      * Handle repeat event
      */
    void HandleRepeatEvent();

    /**
    * Send play command to app UI or view.
    * @since 3.1
    */
    void FilterAndSendCommand(TInt aCommandId);
   /**
    * Callback for timer
    */
    static TInt TimerCallback(TAny* aPtr);
    
    void HandleVolumeUp(TRemConCoreApiButtonAction aButtonAct );

	void HandleVolumeDown(TRemConCoreApiButtonAction aButtonAct );
	
	void HandlePlayPause(TRemConCoreApiButtonAction aButtonAct );
		
	void HandleStop(TRemConCoreApiButtonAction aButtonAct );
	
	void HandleRewind(TRemConCoreApiButtonAction aButtonAct );
	
	void HandleFastForward(TRemConCoreApiButtonAction aButtonAct );
	
	void HandleForward(TRemConCoreApiButtonAction aButtonAct );
	
	void HandleBackward(TRemConCoreApiButtonAction aButtonAct );
	
	TInt FindObserver(MHwKeyWatchObserver* aObserver);
private:
    //const TInt                  iMinVolume; 
    //const TInt                  iMaxVolume;
    CRemConCoreApiTarget*       iInterfaceSelector;
    CRemConInterfaceSelector*   iSelector;
    CPeriodic*                  iTimer; // owned
    TInt                        iCommandId;

    // Stores the observers.
    CArrayFix<MHwKeyWatchObserver*>* iObservers;
    };

#endif

// End of File
