/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that queues an 
*				 active object and allows a MFlhAsyncHandler to execute code 
*				 when it completes.
*
*/



#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "saplayer_build.h"
#include "async_object.h"
#include "flashengineclientif.h"

#include <e32std.h>

/**
 * The NewL function is a factory method.
 *
 * @param aPriority CActive priority which this object should use
 * @param aHandler reference to interface which will be called when this object is triggered
 *
 * @return a new CFlhAsyncObject instance transferring ownership to the caller.
 */
CFlhAsyncObject* CFlhAsyncObject::NewL(TInt aPriority, MFlhAsyncHandler& aHandler)
{
	return new(ELeave) CFlhAsyncObject(aPriority,aHandler);
}

/**
 * Destructor
 */
CFlhAsyncObject::~CFlhAsyncObject()
{
	Cancel(); // by convention 
}

/**
 * Start -- triggers the object to be run later.
 */
void CFlhAsyncObject::Start()
{	
	iStatus=KRequestPending;
	if( !IsActive() )
		SetActive();
	TRequestStatus* temp=&iStatus;
	User::RequestComplete(temp,KErrNone);
}

/**
 * Constructor
 *
 * @param aPriority CActive priority which this object should use
 * @param aHandler reference to interface which will be called when this object is triggered
 */
CFlhAsyncObject::CFlhAsyncObject(TInt aPriority, MFlhAsyncHandler& aHandler)
:CActive(aPriority), iHandler(aHandler)
{
	CActiveScheduler::Add(this);
}

/**
 * RunL -- invoked when active object is run. Invokes DoRunL() on MFlhAsyncHandler interface.
 */
void CFlhAsyncObject::RunL()
{

	iHandler.DoRunL(this);
}

/**
 * RunError -- invoked when DoRunL() leaves. Invokes DoRunError() on MFlhAsyncHandler interface.
 *
 * @param aError DoRunL exception code.
 */
TInt CFlhAsyncObject::RunError(TInt aError)
{
	return iHandler.DoRunError(this,aError);
}


/**
 * DoCancel -- called when active object is cancelled.
 */
void CFlhAsyncObject::DoCancel()
{
}



////////////////////////////////////////////////////////////////
// CAsyncDeleter
///////////////////////////////////////////////////////////////    

// -------------------------------------------------------------------------
//CAsyncDeleter::CAsyncDeleter(CBrowserPluginInstance* aPlayer) 
// constructor
// --------------------------------------------------------------------------
//
CAsyncDeleter::CAsyncDeleter(MFlashClientIf* aPlayer/*, PluginBase* aPluginView*/): CActive(EPriorityStandard)
              
{
    CActiveScheduler::Add(this);
    iPlayer = aPlayer;
//    iPluginView = aPluginView;
}

// -------------------------------------------------------------------------
// CAsyncDeleter::~CAsyncDeleter()
// destructor
// --------------------------------------------------------------------------
//    
CAsyncDeleter::~CAsyncDeleter()
{
    Cancel();
}

// -------------------------------------------------------------------------
// void CAsyncDeleter::RunL()
// Active object execution method
// --------------------------------------------------------------------------
//
void CAsyncDeleter::RunL()
{
    if( iStatus.Int() != KErrNone )
    {
        return;        
    }

    iPlayer->StartShutDownProcess();
}

// -------------------------------------------------------------------------
// void CAsyncDeleter::IssueRequest()
// Used to Handle asynchronous request for deletion
// --------------------------------------------------------------------------
//
    
void CAsyncDeleter::IssueRequest()
{
    if(IsActive())
    {
        return;
    }
        
    TRequestStatus* status = &iStatus;
    SetActive();
    User::RequestComplete(status, KErrNone);
}

// -------------------------------------------------------------------------
// void CAsyncDeleter::DoCancel()
// cancel active object
// --------------------------------------------------------------------------
//
void CAsyncDeleter::DoCancel()
{
}

