/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
/*
 * flashbitbuffer.cpp
 *
 *  Created on: Jul 28, 2009
 *      Author: vsriram
 */

#include "saplayer_build.h"
#include "flashbitbuffer.h"

CFlashBitmap* CFlashBitmap::NewL(CFbsBitmap* aBitmap)
    {
    CFlashBitmap* self = new (ELeave) CFlashBitmap(aBitmap);
    return self;
    }

CFlashBitmap::CFlashBitmap(CFbsBitmap* aBitmap)
    :iBitmap(aBitmap)
    ,iBitmapGc(NULL)
    ,iBitmapDevice(NULL)
    ,iIsBitmapLocked(EFalse)
    ,iBitmapUtil(NULL)
    ,iDoIOwnBitmap(EFalse)
    {    
    }

CFlashBitmap::~CFlashBitmap()
    {
    if (iBitmapGc)
        {
        delete iBitmapGc;
        iBitmapGc = NULL;
        }

    if (iBitmapDevice)
        {
        //TODO ; Font in windowless...
        /*
         if (iDefaultFont)
         {
         iBitmapDevice->ReleaseFont(iDefaultFont);
         iDefaultFont = NULL;
         }
         */
        delete iBitmapDevice;
        iBitmapDevice = NULL;
        }

    if (iBitmapUtil)
        {
        delete iBitmapUtil;
        iBitmapUtil = NULL;
        }
    
    if(iBitmap && iDoIOwnBitmap)
        {
        delete iBitmap; 
        }
    
    }

void* CFlashBitmap::Lock()
    {
    if (IsSurfaceValid() && iBitmap)
        {
        if (!iIsBitmapLocked)
            {
            iIsBitmapLocked = ETrue;
            
            return iBitmap->DataAddress();
            }
        }       
    
    return NULL;
    }
void CFlashBitmap::Unlock( TAny* /*aBitmap*/ )
    {
    if (iIsBitmapLocked)
        {
        if(iBitmapUtil)
            iBitmapUtil->End();
        
        iIsBitmapLocked = EFalse;
        }
    }

void CFlashBitmap::Free()
    {
    if (iBitmapGc)
        {
        delete iBitmapGc;
        iBitmapGc = NULL;
        }

    if (iBitmapDevice)
        {
        //TODO ; Font in windowless...
        /*
        if (iDefaultFont)
            {
            iBitmapDevice->ReleaseFont(iDefaultFont);
            iDefaultFont = NULL;
            }
        */
        delete iBitmapDevice;
        iBitmapDevice = NULL;
        }

    if (iBitmapUtil)
        {
        delete iBitmapUtil;
        iBitmapUtil = NULL;
        }

    if (iBitmap)
        {
        iBitmap->Reset();
        iBitmap = NULL;
        }
    }

TBool CFlashBitmap::IsSurfaceValid()
    {
    if (iBitmap)
        return ETrue;

    return EFalse;
    }

TBool CFlashBitmap::IsLocked()
    {
    return iIsBitmapLocked;
    }

void CFlashBitmap::Clear()
    {
    if(iBitmap && iBitmapGc)
        {
        TSize bitmapSize = iBitmap->SizeInPixels();
        TRect rect( 0, 0, bitmapSize.iWidth, bitmapSize.iHeight);
        TRgb white( 0xFF, 0xFF, 0xFF );
        iBitmapGc->SetPenStyle( CFbsBitGc::ESolidPen );
        iBitmapGc->SetPenColor( white );
        iBitmapGc->SetBrushStyle( CFbsBitGc::ESolidBrush );
        iBitmapGc->SetBrushColor( white );
        iBitmapGc->DrawRect( rect );
        }
    }

void CFlashBitmap::SetSize(TSize aSize, TBool aResize)
    {
    iBitmapSize = aSize;
    
    if(aResize)
        iBitmap->Resize(TSize(iBitmapSize.iHeight, iBitmapSize.iWidth));
    }
