/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
#include <e32base.h>
#include <e32cmn.h>
#include <w32std.h>
#include <AknUtils.h>
#include <featmgr.h> // Feature Manager
#include <aknlayoutscalable_avkon.cdl.h>

#include "AknPanic.h"

#include "flashlongtapdetector.h"

// CONSTANTS
const TInt KFlashLongTapDelay = 800000;              // 0,8 seconds


// ======== MEMBER FUNCTIONS ========


// ---------------------------------------------------------------------------
// CFlashLongTapDetector
// ---------------------------------------------------------------------------
//
CFlashLongTapDetector::CFlashLongTapDetector(MFlashLongTapDetectorCallBack* aOwner)
    :CTimer( 0 ),
    iState(CFlashLongTapDetector::EWaiting),
    iOwner(aOwner),
    iFlashLongTapDelay(KFlashLongTapDelay),
    iPointerEvent()
    {
    }


// ---------------------------------------------------------------------------
// ConstructL
// ---------------------------------------------------------------------------
//
void CFlashLongTapDetector::ConstructL()
    {
    // Initialize feature manager
    FeatureManager::InitializeLibL();
    iTouchFeatureSupported = FeatureManager::FeatureSupported(KFeatureIdPenSupport);
    FeatureManager::UnInitializeLib();
    
	if(iTouchFeatureSupported) //adding feature manager to check the feature at run time.
		{
		CTimer::ConstructL();
    	CActiveScheduler::Add( this );	
		}
    }


// ---------------------------------------------------------------------------
// NewL
// ---------------------------------------------------------------------------
//
CFlashLongTapDetector* CFlashLongTapDetector::NewL( MFlashLongTapDetectorCallBack* aOwner )
    {
    	CFlashLongTapDetector* self = CFlashLongTapDetector::NewLC( aOwner );
    	CleanupStack::Pop( self );
    	return self;
    }


// ---------------------------------------------------------------------------
// NewLC
// ---------------------------------------------------------------------------
//
CFlashLongTapDetector* CFlashLongTapDetector::NewLC( MFlashLongTapDetectorCallBack* aOwner )
    {
    CFlashLongTapDetector* self = new ( ELeave ) CFlashLongTapDetector( aOwner );
    CleanupStack::PushL( self );
    self->ConstructL();
    return self;
    }


// ---------------------------------------------------------------------------
// ~CFlashLongTapDetector
// ---------------------------------------------------------------------------
//
CFlashLongTapDetector::~CFlashLongTapDetector()
    {
	if(iTouchFeatureSupported) //adding feature manager to check the feature at run time.
    	Cancel(); // CTimer
    }


// ---------------------------------------------------------------------------
// PointerEventL
// ---------------------------------------------------------------------------
//
void CFlashLongTapDetector::PointerEventL( const TPointerEvent& aEvent )
    {
    if(iTouchFeatureSupported) //adding feature manager to check the feature at run time.
    	{
    if ( AknLayoutUtils::PenEnabled() )
        {
        // offset is now 6 layout units, if the pointer is dragged just a few 
        // pixels from the original point, long tap is not cancelled. 
        TInt offset = AknLayoutScalable_Avkon::aid_value_unit2().LayoutLine().iW * 6/10;
        if ((iState == CFlashLongTapDetector::ELongTapEvent ||
            iState == CFlashLongTapDetector::EWaiting) && 
            aEvent.iType == TPointerEvent::EDrag )
            {
            TPoint aCenter = iPointerEvent.iPosition; 
            TRect rect( aCenter.iX - offset, aCenter.iY - offset, 
                aCenter.iX + offset, aCenter.iY + offset ); 
            if ( rect.Contains( aEvent.iPosition ) )
                {
                //return; 
                }
            }
        // whatever happends, cancel the timer and stop animation
        Cancel();


	        iPointerEvent = aEvent;

	        if (iState == CFlashLongTapDetector::EWaiting &&
	            aEvent.iType == TPointerEvent::EButton1Down )
	            {
	            iState = CFlashLongTapDetector::ELongTapEvent;
	            After(TTimeIntervalMicroSeconds32(iFlashLongTapDelay));
	            }
	        else
	            {
	            iState = CFlashLongTapDetector::EWaiting;
	            }
	        }
    	}
    }




// ---------------------------------------------------------------------------
// SetLongTapDelay
//
// ---------------------------------------------------------------------------
//
void CFlashLongTapDetector::SetLongTapDelay( const TInt aMicroSeconds )
    {
    if(iTouchFeatureSupported) //adding feature manager to check the feature at run time.
    	{    	
    	// update longtap delay time only if it is reasonable
    	if(aMicroSeconds >= 0)
        	{
        	iFlashLongTapDelay = aMicroSeconds;
        	}
    	}
    }


// ---------------------------------------------------------------------------
// RunL
// ---------------------------------------------------------------------------
//
void CFlashLongTapDetector::RunL()
    {
	if(iTouchFeatureSupported) //adding feature manager to check the feature at run time.
		{
    	if ( AknLayoutUtils::PenEnabled() )
        	{
        	switch ( iState )
            	{
            	case CFlashLongTapDetector::ELongTapEvent:
                {
                iState = CFlashLongTapDetector::EWaiting;


                // Report long tap to owner with locations related to both owner rect and screen rect.
                iOwner->HandleFlashLongTapEventL( iPointerEvent.iPosition, iPointerEvent.iParentPosition );
                break;
                }
            	default:
                {
                // we should never end up here.
                User::Panic( _L( "FlashLongTapDetector" ), EAknPanicSelfCheckFailure );
                break;
                }
            	}
        	}
		}
    }


//  End of File
