/*
* Copyright (c) 2005-2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CFlashVolumeControl class implementation
*
*/


// INCLUDE FILES
#include "saplayer_build.h"
#include <coemain.h>
#include <eikenv.h>
#include <coeaui.h>
#include <AknDef.h>
#include "flash_engine.h"
#include "flashvolumecontrol.h"
#include "hwkeywatch.h"

#include "aknconsts.h" //KAknStandardKeyboardRepeatRate
#include "saflash_consts.h" //KMuteMask


/////////////////////////////////////////////////////////////////////////////////////
//


/**
* Two-phased constructor.
*/
CFlashVolumeControl* CFlashVolumeControl::NewL(CCoeControl* aParent
												,MVolumePropertyWatchObserver*  aVolObsrvr
    											,TBool aHandleNavKey
    											,MFlashEngine* aEngine)
{
	CFlashVolumeControl* self = new (ELeave) CFlashVolumeControl(aParent, aVolObsrvr
																,aHandleNavKey, aEngine);

    CleanupStack::PushL( self );
    self->ConstructL(aParent);
    CleanupStack::Pop();

    return self;	
}

/**
* Destructor.
*/
 CFlashVolumeControl::~CFlashVolumeControl()
 {
 	if(iTimer)
        {
	        iTimer->Cancel();
	        delete iTimer;
	        iTimer= NULL;
        }
    
    
 	if(iVolPopup)
 	{
 		delete iVolPopup;
 		iVolPopup = NULL;
 	}
 	
 	
 }


	
		 

/**
* C++ default constructor.
*/
CFlashVolumeControl::CFlashVolumeControl(CCoeControl* aParent, MVolumePropertyWatchObserver*  aVolObsrvr
    						,TBool aHandleNavKey, MFlashEngine* aEngine):iParentControl(aParent)
					,	iVolObsrv(aVolObsrvr)
					,   iHandleNavKey(aHandleNavKey)
					,   iEngine(aEngine)
					,	iVolPopup(NULL)
					,	iTimer(NULL)
					,	iOnStack(EFalse)
{
	
}

/**
* By default Symbian 2nd phase constructor is private.
*/
void CFlashVolumeControl::ConstructL(CCoeControl* aParent)
{

	
	SetContainerWindowL(*aParent);
	
	SetExtent( TPoint(), TSize() );
	

	
#ifdef __S60_32__
    iVolPopup = CAknVolumePopup::NewL(0); 
#else 
    if(iEngine->GetExecutionMode() == ePlugin)
    {
        iVolPopup = CAknVolumePopup::NewL(NULL, ETrue); // Vertical slider
        iVolPopup->SetSize(TSize(70, 275));
    }
    else
        iVolPopup = CAknVolumePopup::NewL(NULL, EFalse); // Horizontal slider
#endif //__S60_32__

    if( !iVolPopup )
        {
            // should this panic?
            // which panic?  User::Panic();
        }
	
	iVolPopup->SetTitleTextL(_L(" "));
	iVolPopup->SetObserver(this);
	//iVolPopup->SetMopParent(this);
	iVolPopup->SetRange(KMinUserVolume, KMaxUserVolume);
#ifndef __S60_32__ 
	iVolPopup->SetTimeOutAsSecond(5);
#endif
	
	
					
}



TInt CFlashVolumeControl::CountComponentControls() const
{
    if(iVolPopup)
        return 1;
    else
        return 0;
}



CCoeControl* CFlashVolumeControl::ComponentControl(TInt aIndex) const
{
    switch(aIndex)
        {
        case 0:
            if(iVolPopup)
               return iVolPopup;
                          
        default:
             return NULL;
        }
}


        
TKeyResponse CFlashVolumeControl::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
 {
 	//if dont handle key is set then dont consume
 	//case : when device have h/w key we use this control for display only
 	//this should not handle nav key
 	if(!iHandleNavKey)
 		return EKeyWasNotConsumed;
 	
 	if (aType == EEventKey)
	{
		if(iTimer)
		{
			iTimer->Cancel();		
		}

		switch (aKeyEvent.iCode) // The code of key event is...
		{
			case EKeyLeftArrow: // Left key.
			{
			
				FilterAndSendCommandL(EVolumeControlCmdVolumeDownByOne);
				
			
					
					
				break;	
			}
			
			
		case EKeyRightArrow: // Right Key.
			{
				FilterAndSendCommandL(EVolumeControlCmdVolumeUpByOne);
		
            	
            	break;
			}
			
		default:
			{
				break;
			}		
		
		} //switch
		
		DrawNow();
		


		return EKeyWasConsumed;

	}
	
	return EKeyWasNotConsumed;
 }
 
 
void  CFlashVolumeControl::HandlePointerEventL(const TPointerEvent & /*aPointerEvent*/)
{
    return;
}


void  CFlashVolumeControl::HandleResourceChange(TInt aType)
    {
    if(iVolPopup)
        {
        if(iVolPopup->IsVisible())
            // Changed for defect ESLI-7KF7V7:
            // iVolPopup->DrawableWindow()->SetVisible(EFalse);
            iVolPopup->CloseVolumePopup();
        }
    
    
    //CCoeControl::HandleResourceChange(aType);
    
    if(aType == KEikDynamicLayoutVariantSwitch)
        {
        SetExtent( TPoint(), TSize() );
        }
        
    iChangeRect = TRect();
   
    }


void  CFlashVolumeControl::ResizeToToggle()
{
    SetExtent( TPoint(), TSize() );
}


 //Function to support  volume change  with finger touch 
void CFlashVolumeControl::HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType)
{
	if (iVolPopup && iVolPopup->IsVisible() && (iVolPopup == aControl) && (aEventType == MCoeControlObserver::EEventStateChanged))
	{
		TInt volume=iVolPopup->Value();
	
		if(iVolObsrv)
		{
			iVolObsrv->MVolumeChanged(volume);
		}

	}
	
	// volume pop is disappearing...
	if(iVolPopup && !iVolPopup->IsVisible() && (iVolPopup == aControl) && (aEventType == MCoeControlObserver::EEventStateChanged))
    {
	    /**
	     * From MCoeControlStateObserver
	     
	    If an object implementing this interface has been attached to the MObjectProvider
	    hierarchy in which a CCoeControl is a member, any call to the CCoeControl's
	    SetDimmed() or MakeVisible() (that results in an actual state change) will trigger
	    a call to this interface.

	    @param aControl A pointer to the control that has changed.
	    @param aState   An enum from TCoeState to say how the control has changed.
	    @return         A system wide error code, currently ignored but may not be in future.
	    
	    */
	    /*
	     * This was called as the volume pane disappeared. Hence we refresh the screen again
	     */
	    
	    iEngine->RefreshScreen();
    }
	
}
#ifndef __S60_32__
TInt CFlashVolumeControl::HandleControlStateChange(CCoeControl* aControl,TCoeState aState)
    {
    if(aState == EStateVisibility && aControl == iVolPopup)
        if(iVolPopup->IsVisible())
            EndPopup();
    
    return KErrNone;
    }
#endif

// -----------------------------------------------------------------------
// CFlashVolumeControl::FilterAndSendCommandL
// -----------------------------------------------------------------------
//
void CFlashVolumeControl::FilterAndSendCommandL(TInt aCommandId)
{
   if(iVolObsrv)
   		iVolObsrv->MHandleVolumeCmdL(aCommandId);     
    
}
 
TInt CFlashVolumeControl::TimerCallback(TAny* aPtr)
{
    CFlashVolumeControl* volCtrl = static_cast<CFlashVolumeControl*>(aPtr);
    
    volCtrl->EndPopup();
    
    return KErrNone;
}

void CFlashVolumeControl::CloseVolPopUp()	 
{
	if(iVolPopup)
	{
		iVolPopup->CloseVolumePopup();
	}
}

void CFlashVolumeControl::EndPopup()	 
{
	
}


void CFlashVolumeControl::RemoveObserver()
{
	iVolObsrv = NULL;
}


void CFlashVolumeControl::ShowPopupL(TInt aVolume)
{
	if(aVolume < 0)
	{
		aVolume = 0;
	}
	else
	if ( aVolume > KMaxUserVolume )
	{
    	aVolume = KMaxUserVolume;
	}
	
	if(iEngine->GetExecutionMode() == ePlugin)
	{
        /* In case of browser plugin and widgets, display volume slider on the left side of the display
         * In Case of widgets we should not depend on viewRect and relative position calculations as scrolling is possible in widgets
         */
	
        CWsScreenDevice* screenDevice = CEikonEnv::Static()->ScreenDevice();
        if(!screenDevice)
            return;
        TSize screenSize = screenDevice->SizeInPixels();
        iVolPopup->SetPosition(TPoint(0, 0));
	}
	else
	{   	
        iRect = iParentControl->Rect();
        iPos = iParentControl->PositionRelativeToScreen();
        
        iRect.Move(iPos);
        if(iRect != iChangeRect)
        {
            TRect rect = iVolPopup->Rect();
            TPoint posreltoscreen = iVolPopup->PositionRelativeToScreen();
            
            TInt x = (iRect.iBr.iX - rect.Width())- posreltoscreen.iX;
            TInt y = iRect.iTl.iY - posreltoscreen.iY;
        
            SetPosition(TPoint(x, y));
            iChangeRect = iRect;
        }
	}
	 
	if(iVolPopup )
	{
		iVolPopup->SetValue( aVolume );
		
		iVolPopup->ShowVolumePopupL();	
	}
	
    iEngine->RefreshScreen();
}

//End of file
