/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that provides a 
*				 mechanism for priming a timer using CTimer functionality, and 
*				 receiving a callback in the owning class.
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "saplayer_build.h"
#include "timer_object.h"
#include "async_object.h"

#include <e32std.h>


CFlhTimerObject* CFlhTimerObject::NewL(TInt aPriority, MFlhAsyncHandler& aHandler)
{
	CFlhTimerObject* self = new(ELeave) CFlhTimerObject(aPriority, aHandler);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

CFlhTimerObject::~CFlhTimerObject()
{
	Cancel();
}


CFlhTimerObject::CFlhTimerObject(TInt aPriority, MFlhAsyncHandler& aHandler)
:CTimer(aPriority), iHandler(aHandler)
{
	CActiveScheduler::Add(this);
}

void CFlhTimerObject::RunL()
{
	iHandler.DoRunL(this);
}

TInt CFlhTimerObject::RunError(TInt aError)
{
	return iHandler.DoRunError(this,aError);
}


//////////// CFlhTimerObject2 //////////// 


CFlhTimerObject2* CFlhTimerObject2::NewL(TInt aPriority, TCallBack aTimerCallback)
{
	CFlhTimerObject2* self = new (ELeave) CFlhTimerObject2(aPriority, aTimerCallback);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

CFlhTimerObject2::CFlhTimerObject2(TInt aPriority, TCallBack aTimerCallback)
: CFlhTimerObject(aPriority,iAsyncHandler)
, iTimerCallback(aTimerCallback)
{
}

CFlhTimerObject2::~CFlhTimerObject2()
{
	
}

void CFlhTimerObject2::RunL()
{
	iTimerCallback.CallBack();
}

TInt CFlhTimerObject2::RunError(TInt aError)
{
	return aError;
}

