/*
* Copyright (c) 2002-2004 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  AO that completes on a change in the volume property.  Used for 
*                the new Central Repository Server.  Used in EKA2 builds only
*
*/


#include "saplayer_build.h"
#include "volumecenrepkeywatcher.h"

#include <e32debug.h>

//include this file when KSDUidMediaVolumeControl and KMediaVolume will be needed
//#include <SharedDataKeys.h> 

#include <browseruisdkcrkeys.h>
#include "volumepropertywatchobserver.h"

// ----------------------------------------------------
// CVolumePropertyWatch::NewL
// Description: static constructor.
// Input:      aObserver : observer
//             aConnect : ETrue-watch ,EFalse - passive
// Output: none
// Return: CVolumePropertyWatch object
// ----------------------------------------------------
CVolumeCenRepKeyWatcher* CVolumeCenRepKeyWatcher::NewL(MVolumePropertyWatchObserver* aObserver
												,TBool aConnect)
{
	CVolumeCenRepKeyWatcher* me=new(ELeave) CVolumeCenRepKeyWatcher(aConnect);
	CleanupStack::PushL(me);
	me->ConstructL(aObserver,aConnect);
	CleanupStack::Pop(me);
	return me;
}

// ----------------------------------------------------
// CVolumePropertyWatch::CVolumePropertyWatch
// Description: constructor.
// Input:      aConnect : ETrue-watch ,EFalse - passive
// Output: none
// Return: none
// ----------------------------------------------------

CVolumeCenRepKeyWatcher::CVolumeCenRepKeyWatcher(	TBool aConnect) : CActive(EPriorityStandard)
{
	if(aConnect)
	{
		
		iState = EUnableToConnect;
	
		CActiveScheduler::Add(this) ; // add to active scheduler
	}
	else
	{
		iState = ENoConnect; //no watch but perform get,set
	}
}


// ----------------------------------------------------
// CVolumePropertyWatch::ConstructL
// Description: second phase constructor.
// Input:  aObserver: link to the calling object
//			 aConnect : ETrue-watch ,EFalse - passive
// Output: none
// Return: none
// ----------------------------------------------------
void CVolumeCenRepKeyWatcher::ConstructL(MVolumePropertyWatchObserver* aObserver
										,	TBool aConnect)
{
	//
	// Open a connection to the Central Repository...
	iCenRepos = CRepository::NewL(KCRUidBrowser);

	if(aConnect)		
	{
			
		if ( iCenRepos->NotifyRequest(KBrowserMediaVolumeControl, iStatus) == KErrNone )
		{
			iState = EConnected;	
			SetActive();
		}
		else
		{
			iState = EUnableToConnect;
		}
	}
	
    iObserver = aObserver;
}

// ----------------------------------------------------
// CVolumePropertyWatch::CVolumePropertyWatch
// Description: destructor.
// Input:  none
// Output: none
// ----------------------------------------------------
CVolumeCenRepKeyWatcher::~CVolumeCenRepKeyWatcher()
{
	Cancel();
	delete iCenRepos;  // close the client session
}

//
// Get the current volume
// JRT 27_01_05 - Needs updating to return the current volume.
// ----------------------------------------------------
// CVolumePropertyWatch::GetCurrentVolume
// Description: Get the current volume from cen rep.
// Input:  none
// Output: volume value
// ----------------------------------------------------
TInt CVolumeCenRepKeyWatcher::GetCurrentVolume()
{
	TInt currentVolume = 0;
    
   	// Get the current volume
   	if ( iCenRepos->Get(KBrowserMediaVolumeControl, currentVolume) != KErrNone )
   	{
		// Error retrieving value... return default value
		currentVolume = GetDefaultVolume();
	}
    return currentVolume;
}

// ----------------------------------------------------
// CVolumePropertyWatch::SetCurrentVolume
// Description: Set the current volume into cen rep.
// Input:  none
// Output: 0 - fail ,1 - success
// ----------------------------------------------------
TInt CVolumeCenRepKeyWatcher::SetCurrentVolume(TInt aVolume)
{
 	// Get the current volume
   	if ( iCenRepos->Set(KBrowserMediaVolumeControl, aVolume) != KErrNone )
    	{
    		return 0;
    	}

    return 1;
}

// ----------------------------------------------------
// CVolumePropertyWatch::GetDefaultVolume
// Description: default volume in this case 2/3 of the max support by the system.
// Input:  none
// Output: volume value
// ----------------------------------------------------
// Return the default volume in this case 2/3 of the max support by the system
TInt CVolumeCenRepKeyWatcher::GetDefaultVolume()
{
	TInt defaultVol = 0;
	
	#if defined(__WINS__)
    	defaultVol = WINS_DEFAULT_VOLUME;
    #else
    	defaultVol = HW_DEFAULT_VOLUME;
    #endif
    
    return defaultVol;
}

// ----------------------------------------------------
// CVolumePropertyWatch::DoCancel
// Description: cancel active object
// Input:  none
// Output: none
// ----------------------------------------------------

void CVolumeCenRepKeyWatcher::DoCancel()
{
	iCenRepos->NotifyCancel(KBrowserMediaVolumeControl);
}

// ----------------------------------------------------
// CVolumePropertyWatch::RunL
// Description: Activated when the watched value updates.
// Input:  none
// Output: none
// ----------------------------------------------------

// 
void CVolumeCenRepKeyWatcher::RunL()
{
	TInt currentVolume = 0;
	
	if (EUnableToConnect == iState)
	{
		return;
	}
	
	//
	// Get the current volume
	if ( iCenRepos->Get(KBrowserMediaVolumeControl, currentVolume) == KErrNone )
	{
		//
		// Inform the observer
		iObserver->MHandleVolumeCmdL(currentVolume);
		if ( iCenRepos->NotifyRequest(KBrowserMediaVolumeControl, iStatus) == KErrNone )
		{
			SetActive();
		}
		else
		{
			iState = EUnableToConnect;
		}
	}
	else
	{
		//
		// error
		iState = EUnableToConnect;		
	}
}
