/*
* Copyright (c) 2007-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  A client to fetch policy id for flash runtime. The flash
 *                runtime can also fetch a content session, which is essential
 *                for loading a platform service providers
 *
*/


#ifndef __FLASHRTPOLICYCLIENT_H__
#define __FLASHRTPOLICYCLIENT_H__

#include <e32base.h>
#include <rtsecmgrcommondef.h>
#include "flashrtpolicycontentsession.h"

class CFlashRTPolicyUtil;
class CFlashRTPolicyContentSession;
typedef TInt TPolicyID;
class MFlashRTPolicyContentSession;
class CFlashRTPolicyClientProxy;

class CFlashRTPolicyClient : public CBase
    {
public:
	/**
	 * Two-phased constructor
	 *
	 * Constructs a CFlashRTPolicyClient instance
	 *
	 *
	 * @return pointer to an instance of CFlashRTPolicyClient
	 */
    IMPORT_C static CFlashRTPolicyClient* NewL();

	/**
	 * Two-phased constructor
	 *
	 * Constructs a CFlashRTPolicyClient instance and leaves the created instance
	 * onto the cleanupstack.
	 *
	 *
	 * @return CFlashRTPolicyClient* Pointer to an instance of CFlashRTPolicyClient
	 */
    IMPORT_C static CFlashRTPolicyClient* NewLC();

	/**
	 * Destructor
	 *
	 * Closes client side security manager session if script session had been fetched
	 */
    IMPORT_C ~CFlashRTPolicyClient();

	/**
	 * Flash runtime must obtain policy id using this API. The API fetches policy id
	 * from policy server. The policy server generates a policy id (from security
	 * manager) for flash runtime.
	 *
	 *
	 * @return TPolicyID Generated policy identifier if successul;
	 *
     * @leave  One of the system errors on failure.
	 */
    IMPORT_C TPolicyID GetPolicyIdL() const;


	/**
	 * Flash runtimes must use this api to get content session for a given policy id.
	 * The script session is valid as long as the client instance is alive.
	 *
	 * @param aPolicyID TPolicyID Valid registered policy identifier
	 *
	 * @return CFlashRTPolicyContentSession*    A valid flash runtime content session
	 *
     * @leave  One of the system errors on failure.
     *
	 */
    IMPORT_C MFlashRTPolicyContentSession* GetScriptSessionL(TPolicyID aPolicyID);

    IMPORT_C TInt CreateSharedMemoryL(const char* aShMemName, const char* aLockName, unsigned long aSize,
            RChunk& aChunkHdl, unsigned long& aLockID);

    IMPORT_C TInt LockSharedMemoryL(unsigned long aLockId);

    IMPORT_C TInt UnlockSharedMemoryL(unsigned long aLockId);

    IMPORT_C TInt DeleteSharedMemoryL(unsigned long aLockId);

    /**
     * Reads the entire OTF font file into the provided buffer 
     * 
     * @param aFont: buffer which into which the file is read
     * @param aBold: true if bold font is required, false otherwise
     * 
     * @return TInt One of the system wide error code
     */
    IMPORT_C TInt ReadFont(unsigned char** aFont, TBool aBold);

private:
	//Private default constructor
    inline CFlashRTPolicyClient();

	//Private default constructor
    void ConstructL();

private:
	//Proxy to client side session object
    CFlashRTPolicyUtil* iPolicyUtil;
    CFlashRTPolicyClientProxy* iClientProxy;
    };

#include "flashrtpolicyclient.inl"

#endif // __FLASHRTPOLICYCLIENT_H__
