/*
* Copyright (c) 2007-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Proxy delegate class for policy server's client-side
 *                session handle.
 *
*/

 
#ifndef __FLASHRTPOLICYCLIENTPROXY_H__
#define __FLASHRTPOLICYCLIENTPROXY_H__

#include <e32base.h>
#include <f32file.h>
#include <rtsecmanager.h>

#include "flashrtpolicyclientsession.h"

/**
 * Proxy delegate class for policy server's client-side session handle.
 */
NONSHARABLE_CLASS(CFlashRTPolicyClientProxy) : public CBase
    {
public:
	/**
	 * Two phased constructor
	 * 
	 * Constructs a CFlashRTPolicyClientProxy instance
	 */
    static CFlashRTPolicyClientProxy* NewL();

	/**
	 * Destructor.
	 *
	 * Closes the client side session handle. 
	 */
    ~CFlashRTPolicyClientProxy();

	/**
	 * Fetches policy id from policy server's client side
	 * session.
	 *
	 * @return TPolicyID A valid policy id from flash policy server's client 
	 *                   side session handle if successul; Otherwise one of 
	 *                   system wide error codes
	 */
    TPolicyID GetPolicyId() const;
    
    TInt CreateSharedMemory(const char* aShMemName, const char* aLockName, unsigned long aSize, 
                            RChunk& aChunkHdl, unsigned long& aLockID);
        
    TInt LockSharedMemory(unsigned long aLockId);
    
    TInt UnlockSharedMemory(unsigned long aLockId);
    
    TInt DeleteSharedMemory(unsigned long aLockId);
    
    /**
     * Reads the entire OTF font file into the provided buffer 
     * 
     * @param aFont: buffer which into which the file is read
     * @param aBold: true if bold font is required, false otherwise
     * 
     * @return TInt One of the system wide error code
     */
    TInt ReadFont(unsigned char** aFont, TBool aBold);
    
private:
	/**
	 * default constructor
	 */
	 CFlashRTPolicyClientProxy() {}
	 
	/**
	 * Second phase construction.
	 * 
	 */
    void ConstructL();

	/**
	 * Flash policy server's client side session handle.
	 * 
	 */
    RFlashRTClientSession iFPSSession;
    };  
    
#endif // __FLASHRTPOLICYCLIENTPROXY_H__    
