/*
* Copyright (c) 2007-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash policy server's client-side session.
 *
*/


#ifndef __FLASHRTPOLICYCLIENTSESSION_H__
#define __FLASHRTPOLICYCLIENTSESSION_H__

#include <e32base.h>
#include <f32file.h>

#include <rtsecmanager.h>

NONSHARABLE_CLASS(RFlashRTClientSession) : public RSessionBase
    {
public:
	/**
	 * Establishes an IPC connection with flash runtime policy server.
	 * Starts the server if the server is not running yet.
	 *
	 * @return TInt One of system wide error codes
	 *
	 */
    TInt Connect();
    
	/**
	 * Closes IPC connection with flash runtime policy server.
	 *
	 */  
    void Close();

	/**
	 * Return version of the policy server
	 *
	 * @return TVersion Version of the flash policy server 
	 */  
    TVersion Version() const;
    
    /**
     * Reads the entire OTF font file into the provided buffer 
     * 
     * @param aFont: buffer which into which the file is read
     * @param aBold: true if bold font is required, false otherwise
     * 
     * @return TInt One of the system wide error code
     */
    TInt ReadFont(unsigned char** aFont, TBool aBold);

public:
	/**
	 * Fetches policy id from flash policy server.
	 *
	 * @return TPolicyID   generated policy identifier from policy server if 
	 *                     successul; Otherwise one of system wide error codes
	 *
	 */
    TPolicyID GetPolicyId() const;
    TInt CreateSharedMemory(const char* aShMemName, const char* aLockName, unsigned long aSize, 
                RChunk& aChunkHdl, unsigned long& aLockID);
        
    TInt LockSharedMemory(unsigned long aLockId);
    
    TInt UnlockSharedMemory(unsigned long aLockId);
    
    TInt DeleteSharedMemory(unsigned long aLockId);

private:
	/**
	 * Starts policy server if not started yet
	 *
	 * @return TInt One of the system wide error codes on failure. KErrNone on
	 *              on success.
	 *
	 */
    TInt StartPolicyServer() const;
    
	/**
	 * Gives the location of the policy server with full path and name.
	 *
	 * @return TFullName Location of the policy server with full path and name.
	 *
	 */    
    TFullName ServerLocation() const;
    };

#endif // __FLASHRTPOLICYCLIENTSESSION_H__
