/*
* Copyright (c) 2007-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  A client to fetch policy id for flash runtime. The flash 
 *                runtime can also fetch a content session, which is essential 
 *                for loading a platform service providers
 *
*/



#include <f32file.h>
#include <s32strm.h>

#include <rtsecmanager.h>
#include <rtsecmgrscriptsession.h>

#include "flashrtpolicyutil.h"
#include "flashrtpolicyclient.h"
#include "flashrtpolicyclientproxy.h"

// ---------------------------------------------------------------------------
// Two-phased constructor
//
// Constructs a CFlashRTPolicyClient instance
// ---------------------------------------------------------------------------
EXPORT_C CFlashRTPolicyClient* CFlashRTPolicyClient::NewL()
    {
    CFlashRTPolicyClient* self = CFlashRTPolicyClient::NewLC();
    CleanupStack::Pop(self);
    return self;
    }

// ---------------------------------------------------------------------------
// Two-phased constructor
//
// Constructs a CFlashRTPolicyClient instance and leaves the created instance
// onto the cleanupstack.
// ---------------------------------------------------------------------------
EXPORT_C CFlashRTPolicyClient* CFlashRTPolicyClient::NewLC()
    {
    CFlashRTPolicyClient* self = new (ELeave) CFlashRTPolicyClient();
    CleanupStack::PushL(self);
    self->ConstructL();
    return self;
    }

// ---------------------------------------------------------------------------
// Destructor
//
// Deletes policy utility object. Policy utility destruction closes the 
// session with security manager session, rendering the script session invalid
// for any further use
// ---------------------------------------------------------------------------
EXPORT_C CFlashRTPolicyClient::~CFlashRTPolicyClient()
    {
    if(iPolicyUtil)
        {
        delete iPolicyUtil;
        iPolicyUtil = NULL;
        }

    if(iClientProxy)
    	{
    	delete iClientProxy;
    	iClientProxy = NULL;
    	}
    }

// ---------------------------------------------------------------------------
// Fetches policy id from policy server
//
// To get the policy id from server, client uses a proxy, which inturn uses a
// policy server's client side session.
// Proxy deletion closes the session with policy server. If there are no more
// session at the server, it closes itself after a timeout
// ---------------------------------------------------------------------------
EXPORT_C TPolicyID CFlashRTPolicyClient::GetPolicyIdL() const
    {
    CFlashRTPolicyClientProxy* lClientProxy = CFlashRTPolicyClientProxy::NewL();
    TPolicyID lPolicyId = lClientProxy->GetPolicyId();
    User::LeaveIfError(lPolicyId);
    delete lClientProxy;
    return lPolicyId;
    }

// ---------------------------------------------------------------------------
// Fetches content session from policy utility
//
// ---------------------------------------------------------------------------
EXPORT_C MFlashRTPolicyContentSession* 
                  CFlashRTPolicyClient::GetScriptSessionL(TPolicyID aPolicyId)
    {
    iPolicyUtil = CFlashRTPolicyUtil::GetInstanceL();
    return iPolicyUtil->GetScriptSessionL(aPolicyId);
    }

// ---------------------------------------------------------------------------
// Fetches a shared handle to font file
//
// ---------------------------------------------------------------------------
EXPORT_C TInt CFlashRTPolicyClient::ReadFont(unsigned char** aFont, TBool aBold)
    {
    CFlashRTPolicyClientProxy* lClientProxy = NULL;
    TRAPD(error, lClientProxy = CFlashRTPolicyClientProxy::NewL(); );
    
    if( KErrNone != error )
        {
        return error;   
        }
    error = lClientProxy->ReadFont(aFont, aBold);
    delete lClientProxy;
    return error;
    }

// ---------------------------------------------------------------------------
// Second phase construction
//
// ---------------------------------------------------------------------------
void CFlashRTPolicyClient::ConstructL()
    {
    }

EXPORT_C TInt CFlashRTPolicyClient::CreateSharedMemoryL(const char* aShMemName, const char* aLockName, unsigned long aSize,
        RChunk& aChunkHdl, unsigned long& aLockId)
    {
    if(!iClientProxy)
        iClientProxy = CFlashRTPolicyClientProxy::NewL();

    return iClientProxy->CreateSharedMemory(aShMemName, aLockName, aSize, aChunkHdl, aLockId);
    }

EXPORT_C TInt CFlashRTPolicyClient::LockSharedMemoryL(unsigned long aLockId)
    {
    if(!iClientProxy)
        iClientProxy = CFlashRTPolicyClientProxy::NewL();

    return iClientProxy->LockSharedMemory(aLockId);

    }

EXPORT_C TInt CFlashRTPolicyClient::UnlockSharedMemoryL(unsigned long aLockId)
    {
    if(!iClientProxy)
        iClientProxy = CFlashRTPolicyClientProxy::NewL();

    return iClientProxy->UnlockSharedMemory(aLockId);
    }

EXPORT_C TInt CFlashRTPolicyClient::DeleteSharedMemoryL(unsigned long aLockId)
    {
    if(!iClientProxy)
       iClientProxy = CFlashRTPolicyClientProxy::NewL();

    return iClientProxy->DeleteSharedMemory(aLockId);
    }

//EOF
