/*
* Copyright (c) 2007-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Proxy delegate class for policy server's client-side
 *                session handle.
 *
*/

 
#include "flashrtpolicyclientproxy.h"

// ---------------------------------------------------------------------------
// Two-phased constructor
//
// Constructs a CFlashRTPolicyClient instance
// ---------------------------------------------------------------------------
CFlashRTPolicyClientProxy* CFlashRTPolicyClientProxy::NewL()
    {
    CFlashRTPolicyClientProxy* self = new (ELeave) CFlashRTPolicyClientProxy();
    CleanupStack::PushL (self);
    self->ConstructL ();
    CleanupStack::Pop (self);
    return self;
    }

// ---------------------------------------------------------------------------
// Second phase construction
//
// Opens the client side session handle.
// ---------------------------------------------------------------------------
void CFlashRTPolicyClientProxy::ConstructL()
    {
    User::LeaveIfError (iFPSSession.Connect());
    }

// ---------------------------------------------------------------------------
// Destructor
//
// Closes the client side session handle
// ---------------------------------------------------------------------------
CFlashRTPolicyClientProxy::~CFlashRTPolicyClientProxy()
    {
    iFPSSession.Close();
    }
	
// ---------------------------------------------------------------------------
// Fetches policy id from policy server using server's client side session
// handle.
//
// ---------------------------------------------------------------------------
TPolicyID CFlashRTPolicyClientProxy::GetPolicyId() const
    {    
    return iFPSSession.GetPolicyId();
    }

// ---------------------------------------------------------------------------
// Reads the font file into the provided buffer
//
// ---------------------------------------------------------------------------
TInt CFlashRTPolicyClientProxy::ReadFont(unsigned char** aFont, TBool aBold)
    {
    return iFPSSession.ReadFont(aFont, aBold);
    }

TInt CFlashRTPolicyClientProxy::CreateSharedMemory(const char* aShMemName, const char* aLockName, unsigned long aSize,
                            RChunk& aChunkHdl, unsigned long& aLockID)
    {
    return iFPSSession.CreateSharedMemory(aShMemName, aLockName, aSize, aChunkHdl, aLockID);
    }

TInt CFlashRTPolicyClientProxy::LockSharedMemory(unsigned long aLockId)
    {
    return iFPSSession.LockSharedMemory(aLockId);
    }

TInt CFlashRTPolicyClientProxy::UnlockSharedMemory(unsigned long aLockId)
    {
    return iFPSSession.UnlockSharedMemory(aLockId);
    }

TInt CFlashRTPolicyClientProxy::DeleteSharedMemory(unsigned long aLockId)
    {
    return iFPSSession.DeleteSharedMemory(aLockId);
    }

//EOF
