/*
* Copyright (c) 2007-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Utility class abstracting common security manager operations. 
 *                Used both by policy server and its client.
 *
*/


#ifndef __FLASHRTPOLICYUTIL_H__
#define __FLASHRTPOLICYUTIL_H__

#include <e32base.h>

class MFlashRTPolicyContentSession;
class CRTSecManager;

class CFlashRTPolicyUtil : public CBase
    {
public:
	/**
	 * Gets policy utility instance
	 * 
	 * @return Pointer to an instance of CFlashRTPolicyUtil
	 */ 
    IMPORT_C static CFlashRTPolicyUtil* GetInstanceL();
    
	/**
	 * Destructor
	 * 
	 */     
    IMPORT_C ~CFlashRTPolicyUtil();

	/**
	 * Registers a runtime security policy. 
	 *
	 * @param aPolicyFile const RFile&  RFile Handle to security policy file
	 *
	 * @return TPolicyID Generated policy identifier if successul; Otherwise one of
	 *					 system wide error codes
	 */
    IMPORT_C TPolicyID SetPolicy(const RFile& aPolicyFile);
    
	/**
	 * Updates an already registered security policy. 
     *
	 * @param aPolicyFile   const RFile&  File Handle to security policy file. 
	 *                      RFile must be open with a file session which is 
	 *                      shareprotected.
	 *
	 * @param aPolicyID TPolicyID Policy identifier previously generated with SetPolicy
	 *
	 * @return TPolicyID Updated policy identifier if successul; Otherwise one of
	 *					 system wide error codes
	 */    
    IMPORT_C TPolicyID UpdatePolicy(const RFile& aPolicyFile, TPolicyID aPolicyId);
    
	/**
	 * UnRegisters a registered security policy. 
     *
	 * @param aPolicyID TPolicyID Policy identifier previously generated with SetPolicy
	 *
	 * @return TInt One of system wide error codes
	 */     
    IMPORT_C TInt UnsetPolicy(TPolicyID aPolicyId);

	/**
	 * Fetches script session for the given policy id from security manager.
     *
	 * @param aPolicyID TPolicyID Policy identifier previously generated with SetPolicy
	 *
	 * @return MFlashRTPolicyContentSession* Content session.
     *
	 * \note
	 * The content session is only valid until this utility is alive     *
	 */  
    IMPORT_C MFlashRTPolicyContentSession* GetScriptSessionL(TPolicyID aPolicyId);

private:
    //Second phase constructor
    void ConstructL();

private:
    //Security manager instance.
    CRTSecManager* iSecMgr;
    };

#endif //__FLASHRTPOLICYUTIL_H__

