/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
// flashpolicyutil.cpp

#include <e32base.h>
#include <f32file.h>
#include <s32strm.h>

#include <rtsecmanager.h>
#include <rtsecmgrscriptsession.h>

#include "flashrtpolicyutil.h"
#include "flashrtpolicycontentsessionimpl.h"

// ---------------------------------------------------------------------------
// Constructs and returns a CFlashRTPolicyUtil instance
//
// ---------------------------------------------------------------------------
EXPORT_C CFlashRTPolicyUtil* CFlashRTPolicyUtil::GetInstanceL()
    {
    CFlashRTPolicyUtil* self = new (ELeave) CFlashRTPolicyUtil();    
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop(self);
    return self;
    }

// ---------------------------------------------------------------------------
// Destructor
//
// ---------------------------------------------------------------------------
EXPORT_C CFlashRTPolicyUtil::~CFlashRTPolicyUtil()
    {
    if(iSecMgr)
        {
        delete iSecMgr;
        iSecMgr = NULL;
        }
    }

// ---------------------------------------------------------------------------
// Gets policy id from security manager providing an access policy file.
//
// Typically called from policy server which maintains and stores policy ids
// ---------------------------------------------------------------------------
EXPORT_C TPolicyID CFlashRTPolicyUtil::SetPolicy(const RFile& aPolicyFile)
    {
    return iSecMgr->SetPolicy(aPolicyFile);
    }

// ---------------------------------------------------------------------------
// Updates policy id for a given access policy file.
//
// Typically called from policy server which maintains and stores policy ids
// ---------------------------------------------------------------------------
EXPORT_C TPolicyID CFlashRTPolicyUtil::UpdatePolicy(const RFile& aPolicyFile, TPolicyID aPolicyID)
    {
    return iSecMgr->UpdatePolicy(aPolicyID,aPolicyFile);
    }

// ---------------------------------------------------------------------------
// Unsets policy id
//
// Typically called from policy server which maintains and stores policy ids
// ---------------------------------------------------------------------------
EXPORT_C TInt CFlashRTPolicyUtil::UnsetPolicy(TPolicyID aPolicyID)
    {
    return iSecMgr->UnSetPolicy(aPolicyID);
    }

// ---------------------------------------------------------------------------
// Fetches a script session for a given policy id.
//
// Typically called from policy server client 
// Note that the security manager instance needs to exist for the script 
// session to be valid. Hence it is necessary for this util object to be alive
// until script session is intended for use.
// ---------------------------------------------------------------------------
EXPORT_C MFlashRTPolicyContentSession* CFlashRTPolicyUtil::GetScriptSessionL(TPolicyID aPolicyId)
    {    
    CTrustInfo* trust = CTrustInfo::NewL();    
    CleanupStack::PushL(trust);    
    
    CRTSecMgrScriptSession* lSMScriptSess = iSecMgr->GetScriptSessionL(aPolicyId, *trust);
    User::LeaveIfNull(lSMScriptSess);
    CleanupStack::PushL(lSMScriptSess);
    
    MFlashRTPolicyContentSession* lScriptSess = CFlashRTPolicyContentSessionImpl::NewL(lSMScriptSess);
    
    CleanupStack::Pop(); //lSMScriptSess
    CleanupStack::PopAndDestroy(); //trust
    return lScriptSess;            
    }

// ---------------------------------------------------------------------------
// Second phase construction
//
// Creates security manager.
// ---------------------------------------------------------------------------
void CFlashRTPolicyUtil::ConstructL()
    {
    iSecMgr = CRTSecManager::NewL();    
    }

//EOF
