/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
/*
 * sharedmemory.h
 *
 *  Created on: Oct 25, 2009
 *      Author: vsriram
 */

#ifndef SHAREDMEMORY_H_
#define SHAREDMEMORY_H_

#include <e32std.h>
#include <e32base.h>
#include <string.h>

typedef unsigned long LockID;

#ifdef SHARED_MEMORY_INTER_PROCESS

class ShMemLock;

class CSharedMemory : public CBase
    {
public:
    static CSharedMemory* NewL(const TDesC& name, const TDesC& lockName,unsigned long size);
    LockID LockId();
    void* BaseAddress();
    RChunk& Chunk() { return iSegment; }
    void Lock();
    void Unlock();
    const TDesC& Name()
    {
		return iName;
	}
    ~CSharedMemory();

private:
    CSharedMemory(const TDesC& name, const TDesC& lockName, unsigned long size);
    void ConstructL(const TDesC& aLockName);

    TPtrC iName;
    ShMemLock* iLock;
    RChunk iSegment;
    unsigned long size;
    };

struct ShMemLock
    {
public:
    ShMemLock(const TDesC& aName)
        {
        iName.Set(aName);
        TInt err = iLock.CreateGlobal(iName);
        err = iLock.OpenGlobal(iName); //open the mutex..
        }

    ~ShMemLock()
        {
        iLock.Close();
        }

    void Lock()
        {
        iLock.Wait();
        return; //means that now we have grabbed the lock..
        }

    void UnLock()
        {
        //Should we check iLock.IsHeld before calling signal..??
        iLock.Signal();
        return; //means that we have released the lock..
        }

private:
    TPtrC iName;
    RMutex iLock;
//    CSharedMemory* iParent;

//    friend class CSharedMemory;
    };

class CServSharedMemManager : public CBase
    {
public:
    ~CServSharedMemManager();

    static CServSharedMemManager* NewL();

    CSharedMemory* GetSharedMemoryL(const TDesC& aShMemName, const TDesC& aLockName, TInt aSize);
    CSharedMemory* GetSharedMemory(unsigned long aLockId);
    TInt RemoveSharedMemory(unsigned long aLockId);

private:
    CServSharedMemManager();

    CSharedMemory* Find(const TDesC& aShMemName);
    CSharedMemory* Find(unsigned long aLockId);
    TInt IndexOf(unsigned long aLockId);

    RPointerArray<CSharedMemory> iShMems;
    };

#else
/*
 * Handles shared memory shared across object in the same DLL
 * Since it is quite closely tied to HostEngine globals, keeping it in the same file
 * (Can be moved if required)
 */
class CSharedMemory : public CBase
{
public:

    static CSharedMemory* NewL();

    virtual ~CSharedMemory();

    /* Allocate requests for a pointer to shared memory.
     * If memory has not been allocated yet, it will be.
     * Otherwise, the pointer to allocated memory is returned
     * and the reference count is incremented.
     */
    TAny* GetSharedMemoryL(TUint32 aSharedMemorySize);

    /*
     * Decreases the reference count by 1.
     * Incase the reference count is zero, then the memory is freed up.
     */
    void FreeSharedMemory();

private:
    /* Constructors */

    /* Making the constructor private so that the object cannot be constructed on the stack */
    CSharedMemory();

    /* First phase constructor */
    static CSharedMemory* NewLC();

    /* Second phase constructor */
    void ConstructL();

private:
    /* Variables */
    TUint32 iSharedMemorySize;  // size of the shared memory
    TAny* iPtrSharedMemory;     // a pointer to the shared memory
                                // memory is allocated only when the reference count is zero
    TInt iReferenceCount;       // keep track of how many objects are referencing the shared memory
};
#endif
#endif /* SHAREDMEMORY_H_ */
