/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
/*
 * sharedmemory.cpp
 *
 *  Created on: Oct 25, 2009
 *      Author: vsriram
 */

#include "sharedmemory.h"

//const int KMaxChunkSize = 0x4000;

#ifdef SHARED_MEMORY_INTER_PROCESS

CServSharedMemManager::~CServSharedMemManager()
    {
    iShMems.ResetAndDestroy();
    iShMems.Close();
    }

CServSharedMemManager::CServSharedMemManager()
    {
    }

CServSharedMemManager* CServSharedMemManager::NewL()
    {
    CServSharedMemManager* self = new (ELeave) CServSharedMemManager;
		return self;
    }

CSharedMemory* CServSharedMemManager::GetSharedMemoryL(const TDesC& aShMemName, const TDesC& aLockName, TInt aSize)
    {
    //STEP 1 : Find if shared memory with the name exists already..
        //STEP 1.1 : If no, Create a new shared memory instance with the lock and go to STEP 2
        //STEP 1.2 : If yes, return the pointer to the shared memory instance and the lockId
    CSharedMemory* shMem = Find(aShMemName);

    if(!shMem)
        {
        //create a new shared memory instance with the specified name
        //and create a lock for the same and return the lock handle
        shMem = CSharedMemory::NewL(aShMemName,aLockName,aSize);
        iShMems.AppendL(shMem);
        }
    return shMem;
    }

CSharedMemory* CServSharedMemManager::GetSharedMemory(unsigned long aLockId)
    {
    return Find(aLockId);
    }

TInt CServSharedMemManager::RemoveSharedMemory(unsigned long aLockId)
    {
		TInt err(KErrGeneral);
		TInt i = IndexOf(aLockId);
		if(i != -1)
		{
    	CSharedMemory* shMem = iShMems[i];
		if(shMem)
			{
			 iShMems.Remove(i);
			 delete shMem;
			 shMem = NULL;
			 err = KErrNone;
			}
		}

    return err;
    }


CSharedMemory* CServSharedMemManager::Find(const TDesC& aShMemName)
    {
    for (TInt i = 0; i < iShMems.Count(); ++i)
        {
        CSharedMemory* shMem = iShMems[i];
        if (0==shMem->Name().Compare(aShMemName))
            {
            return shMem;
            }
        }

    return NULL;
    }

CSharedMemory* CServSharedMemManager::Find(unsigned long aLockId)
    {
    for (TInt i = 0; i < iShMems.Count(); ++i)
        {
        CSharedMemory* shMem = iShMems[i];
        if (aLockId==shMem->LockId())
            {
            return shMem;
            }
        }
    return NULL;
    }


TInt CServSharedMemManager::IndexOf(unsigned long aLockId)
    {
    for (TInt i = 0; i < iShMems.Count(); ++i)
        {
        CSharedMemory* shMem = iShMems[i];
        if (aLockId==shMem->LockId())
            {
            return i;
            }
        }
    return -1;
    }

CSharedMemory* CSharedMemory::NewL(const TDesC& aName, const TDesC& aLockName, unsigned long aSize)
    {
    CSharedMemory* self = new (ELeave) CSharedMemory(aName,aLockName,aSize);
    CleanupStack::PushL(self);
    self->ConstructL(aLockName);
    CleanupStack::Pop(self);
    return self;
    }

CSharedMemory::CSharedMemory(const TDesC& aName, const TDesC& /*aLockName*/, unsigned long aSize)
    :size(aSize)
    {
		iName.Set(aName);
    }

CSharedMemory::~CSharedMemory()
    {
    if(iLock)
    	{
        delete iLock;
        iLock = NULL;
    	}

    iSegment.Close();
    }

void CSharedMemory::ConstructL(const TDesC& aLockName)
    {
    //create an anonymous shared memory segment
    TInt err = iSegment.CreateGlobal(KNullDesC,size,size);

    // Fills a specified block of data with binary zeroes.
      Mem::FillZ(iSegment.Base(), size);

    //open a lock for this segment..note that we have just created the lock; not
    //really locked the shmem segment..
    iLock = new ShMemLock(aLockName);
    }

LockID CSharedMemory::LockId()
    {
    return reinterpret_cast<LockID> (iLock);
    }

void* CSharedMemory::BaseAddress()
    {
    return iSegment.Base();
    }


void CSharedMemory::Lock()
    {
    iLock->Lock();
    }

void CSharedMemory::Unlock()
    {
    iLock->UnLock();
    }

#endif //SHARED_MEMORY_INTER_PROCESS
