/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  GOOM plugin for Flash.
*
*/


#include <ecom/ecom.h>
#include <ecom/implementationproxy.h>
#include <w32std.h>
#include <e32property.h>

#include "flashgoommonitorplugin.h"

static _LIT_SECURITY_POLICY_PASS(KAllowAllPolicy);
const TUid KFlashRuntimeCat = {0x10207218};
const TUint KFlashGPUMemWatchKey = 0x10275064;

#ifdef _DEBUG
#define DEBUGPRINT(args...) RDebug::Printf(##args)
#else
#define DEBUGPRINT(args...) 
#endif


// -----------------------------------------------------------------------------
// C++ constructor.
// -----------------------------------------------------------------------------
//
CFlashGoomMonitorPlugin::CFlashGoomMonitorPlugin()
{
}

// -----------------------------------------------------------------------------
// Destructor.
// -----------------------------------------------------------------------------
//
CFlashGoomMonitorPlugin::~CFlashGoomMonitorPlugin()
{
    iGPUMemStatus.Close();
    RProperty::Delete(KFlashRuntimeCat,KFlashGPUMemWatchKey);
}

CFlashGoomMonitorPlugin* CFlashGoomMonitorPlugin::NewL()
{
    CFlashGoomMonitorPlugin* plugin = new(ELeave) CFlashGoomMonitorPlugin;
    CleanupStack::PushL(plugin);
    plugin->ConstructL();
    CleanupStack::Pop(plugin);
    return plugin;
}


// -----------------------------------------------------------------------------
// Symbian two-phased constructor (2nd phase).
// -----------------------------------------------------------------------------
//
void CFlashGoomMonitorPlugin::ConstructL()
{
    CGOomMonitorPluginBase::ConstructL();
    
    RProcess process;
    TSecureId id = process.SecureId();
    DEBUGPRINT("secure id = 0x%x",id.iId); 
    
    TInt defineErr = RProperty::Define(KFlashRuntimeCat, KFlashGPUMemWatchKey,RProperty::EByteArray,KAllowAllPolicy,KAllowAllPolicy);
    DEBUGPRINT("[flashgoommonitor] define error %d",defineErr);
    
    if(defineErr == KErrNone)
    {
        TInt attachErr = iGPUMemStatus.Attach(TUid(id),KFlashGPUMemWatchKey,EOwnerThread);
        DEBUGPRINT("[flashgoommonitor] attach error %d",attachErr);
    }
}


// -----------------------------------------------------------------------------
// Tries to free RAM.
// -----------------------------------------------------------------------------
//
void CFlashGoomMonitorPlugin::FreeRam(TInt aBytesToFree, TInt /*aFlags*/)
{
    DEBUGPRINT("[flashgoommonitor] CFlashGoomMonitorPlugin::FreeRam %d",aBytesToFree);
    
    TBuf16<2> buf(2);
    buf[0] = EGPUMemLow;
    buf[1] = (aBytesToFree/(1024 * 1024));

#ifdef DEBUGPRINT
    TInt err =
#endif            
    iGPUMemStatus.Set(buf);
    
    DEBUGPRINT("[flashgoommonitor] value set error %d",err);
}

// -----------------------------------------------------------------------------
// CFlashGoomMonitorPlugin::MemoryGood
// -----------------------------------------------------------------------------
//
void CFlashGoomMonitorPlugin::MemoryGood(TInt /*aFlags*/)
{
    DEBUGPRINT("[flashgoommonitor] CFlashGoomMonitorPlugin::MemoryGood");
    
    TBuf16<2> buf(2);
    buf[0] = EGPUMemGood;
    buf[1] = 0;
  
#ifdef DEBUGPRINT
    TInt err =
#endif
    iGPUMemStatus.Set(buf);        
    
    DEBUGPRINT("[flashgoommonitor] value set error %d",err);    
}

// -----------------------------------------------------------------------------
// Creates plugin.
// -----------------------------------------------------------------------------
//
TAny* CreatePlugin()
{
    DEBUGPRINT("[flashgoommonitor] creating plugin, NewL");
    CFlashGoomMonitorPlugin* plugin = NULL;
    TRAP_IGNORE(plugin = CFlashGoomMonitorPlugin::NewL());
    return plugin;
}

// -----------------------------------------------------------------------------
// Implementation table for plugin.
// -----------------------------------------------------------------------------
//
const TImplementationProxy ImplementationTable[] =
{
    {{0x2002C30A}, ::CreatePlugin}
};

// -----------------------------------------------------------------------------
// ImplementationGroupProxy
// -----------------------------------------------------------------------------
//
EXPORT_C const TImplementationProxy* ImplementationGroupProxy(TInt& aTableCount)
{
    aTableCount = sizeof(ImplementationTable) / sizeof(TImplementationProxy) ;
    return ImplementationTable;
}

// End of file

