/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  defines the member functions of the class that is used to 
*				 to async events.
*
*/

#ifndef __ASYNC_EVENT_H__
#define __ASYNC_EVENT_H__

#include "async_object.h"

class CPlatformFlashEngine;

/**
* The CFlhAsyncEvent class is used by CPlatformFlashEngine to trigger asynchronous processing of
* certain SI functions: SI_GetInputText, SI_SetFullScreen, SI_SetSoftkeys. It also triggers the
* playback of sound and loading of new SWF files. It calls CPlatformFlashEngine::HandleAsyncEventsL().
*/

class CFlhAsyncEvent : public CBase, public MFlhAsyncHandler
{
private:
	CFlhAsyncEvent();

public:
	~CFlhAsyncEvent();
	static CFlhAsyncEvent* NewL(CPlatformFlashEngine* aEngine);

public:
	void TriggerEvent();
	void SetLeaveIsCritical(TBool aLeaveIsCritical);
private:
	void ConstructL(CPlatformFlashEngine* aEngine);

private: // from MFlhAsyncHandler
	void DoRunL(const CActive* aActive);
	TInt DoRunError(const CActive* aActive, TInt aError);

private:
	CFlhAsyncObject* iEventHandler;
	CPlatformFlashEngine* iEngine;
	TBool iLeaveIsCritical;

};

#endif	// __ASYNC_EVENT_H__
