/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the CBrowserURLRequestInfo class, which maintains
*				 info about a url request made to the browser & interfaces with 
*				 the core
*
*/



#ifndef __BROWSER_URL_REQUEST_INFO

#define __BROWSER_URL_REQUEST_INFO

#include <e32base.h>
#include "commondef.h"
#include "async_object.h"
#include "timer_object.h"
#include "symbian_engine.h"
#include "mmcactivityobserver.h"

class CMMCReservoir;
class MMMCActivityObserver;

const TInt KMaxChunkSize = 256*1024*2.5;
class CBrowserURLRequestInfo: public CBase, public MFlhAsyncHandler
											, public MMMCActivityObserver
{
	public:
	
		static CBrowserURLRequestInfo* NewL( CPlatformFlashEngine* aEngine, TInt aStreamID,
													const char* aFinalURL, TInt aContentLength, TDesC8& aContentType);
		static CBrowserURLRequestInfo* NewLC( CPlatformFlashEngine* aEngine, TInt aStreamID,
													const char* aFinalURL, TInt aContentLength, TDesC8& aContentType);
		~CBrowserURLRequestInfo();
	private:
		CBrowserURLRequestInfo(CPlatformFlashEngine* aEngine);
		void ConstructL(const char* aFinalURL, TInt aContentLength, TDesC8& aContentType, TInt aStreamID);
	public:
		void WriteStreamDataL( const TDesC8& aData, TInt aDataLength);
//		TInt ReadStreamDataL( TInt aOffset, TDes8& aData, TInt aDataLength);
		TBool Seek(TInt aReadOffset, FI_NetworkBufferInfo* aNetworkBufferInfo);
		unsigned int GetBytes();//Sathya
		unsigned int GetTotalBytes();
		void WriteComplete(FI_Result aStatus);
		TInt HandleError(TInt aError);
		void TriggerAsyncReadWrite();
		TInt CancelDownload();
		void SetNetworkBuffer(struct FI_NetworkBufferInfo* aNetworkBufferInfo);
		TInt GetMaxSize();
		TInt GetStreamID() 		{	return iStreamID; }
		void WriteStreamDataToCoreL( const TDesC8& aData );
		virtual void DoRunL( const CActive* aActive );
		virtual TInt DoRunError( const CActive* aActive, TInt aError );
		void DoCancelStream();
		
	// Storage reservoir related functions
		void ReadCompleteL( TInt aStatus, const TDesC8& aData );
		void NotifyErrorL(TInt aStatus);
		TInt NotifyError(TInt aStatus);
	private:
		void WriteToRAMReservoirL(const TDesC8& aData);
		void ClearRAMReservoir();
		TBool IsDataInRAMRes(TInt aOffset);
		void ShowErrorNoteL(const TDesC& aErrMsg);
		TInt ReadFromRAMResL(TDes8& aData,TInt aOffset, TInt aLength);
		void DestroyHeaderInfo() ;
	private:
		FI_NetworkBufferInfo* iNetworkBufferInfo;
		FI_URLStreamHeaderInfo iHeaderInfo;
		TInt iReadOffset;
		TInt iWriteOffset;
		CMMCReservoir* iMMCReservoir;
		CPlatformFlashEngine* iEngine;
		TInt iStreamID;
		CFlhTimerObject* iAsyncObject;
		FI_Result iWriteStatus;
		HBufC8* iRAMReservoir;
		TInt iRAMCycles;		
		TBool iUseRAMReservoir;
		TInt iRAMResSize;
		TBool iStreamCancelled;
		TBool iCancelState;
		TInt iRamOffset;
		TInt iLastErrorNote;
};


#endif //__BROWSER_URL_REQUEST_INFO
