/*
* Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class that handles IP connection and disconnection.
*
*/


#ifndef __IP_CONN_HANDLER_H__
#define __IP_CONN_HANDLER_H__

#include <e32base.h>
#include <es_sock.h>
#include <commdbconnpref.h>

//const TInt KMaxConnectionRetries=1;
class RSocketServ;
class RConnection;
class CConnHandler;
class CPlatformFlashEngine;

enum TFlhIPEvent
	{
	EEventIsConnectedOK,
	EEventIsDisconnected,
	EEventIsError
	};


/**
* The MIPConnMgrObs mixin receives notifications of network state changes
*/
class MConnHandlerObs
{
public:
	virtual void NotifyConnHandlerStatusL(TInt aEvent, CConnHandler* aConnHandler) = 0;
};


class MFlhIPConnectObserver
{
public:
	virtual void ConnectionEventL(TFlhIPEvent aEvent)=0;
	
	virtual void HandleNetworkAccessDenied() = 0;
};

/**
 * The CConnHandler class handles IP connection and disconnection.
 */
class CConnHandler: public CActive
{
public:

	static CConnHandler* NewL(CPlatformFlashEngine* aEngine);	
	
	void StartConnectionL(MConnHandlerObs* aConnHndlrObs, MFlhIPConnectObserver* aIPConnObs);
	
	TInt GetIAP(TUint32& aIAP);
	
	TInt GetBearerType();
	
	void CloseConnection();
	
	TInt SocketSessionHandle() const { return iSockServ.Handle();	}

	RSocketServ& SocketSession(){ return iSockServ; }

	RConnection& ConnectionInstance(){return iConnection;}
	
	TInt ConnectionHandle() const { return reinterpret_cast<TInt>(&iConnection);}
	
	void HandleNetworkQueryDeniedL(MConnHandlerObs* aConnHndlrObs, MFlhIPConnectObserver* aIPConnObserver);
	
	~CConnHandler();
	
private:
	CConnHandler(CPlatformFlashEngine* aEngine);
	
	void DoCancel();	
	void RunL();
	TInt RunError(TInt aError);

private://members variables	
	TInt iIAP;
	RSocketServ iSockServ;	
	RConnection iConnection;
	MConnHandlerObs* iConnhndlrObs;
	CPlatformFlashEngine* iEngine;
	MFlhIPConnectObserver* iIPConnObs;
	TBool iConnected; 
	TNifProgressBuf iBuf; 

};

#endif //__IP_CONN_HANDLER_H__
