/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  defines the member functions of the class that is used to 
*				 acces flash playr API.
*
*/
#ifndef __FI_INTERFACE_H__
#define __FI_INTERFACE_H__

#include "platformbuild.h"

#include "si_interface.h"
#include "flashextension.h"

// -----------------------------------------------------------------------------

struct SI_HostObject;
class CPlatformFlashEngine;

/**
 * The FI_Interface class is a wrapper for FI functions provided by the Flash library.
 * It also encapsulates the loading of the Flash library DLL.
 */
class FI_Interface
{
public:
	FI_Interface();
	~FI_Interface();

	// Initialize

  void FI_Interface::InitializeL(TInt aFlags, const TDesC8* aExtensions, CEikonEnv& aEnv, TBool aIsSMCheckNeeded);


public:
	void FI_SetFlashVars(const char* values, unsigned short useUTF8);
	// Non player instance specific functions
	const char *			FI_GetVersion();
	short					FI_CheckSWFDataBufferValidity( const unsigned char *aSwfBuffer, unsigned long aBufferLength, void* memoryPool, unsigned short poolSize);
	unsigned short  		FI_GetDisplayFormat(struct MM_Object* pMMObj);

	void					SetEngine(CPlatformFlashEngine* aEngine) { iEngine = aEngine; }
	CPlatformFlashEngine*	GetEngine() { return iEngine; }
	struct MM_Object*		GetMMObject() { return iMMObject; }
	bool 					IsValid() const { return iMMObject != NULL; }

	// accessor for display Rect
	const FI_Rect &	GetDisplayRect() const { return iDisplayRect; }


	// Wrappers for player specific FI_ functions
	bool 			FI_CreatePlayer (void* aMem, unsigned long aBytes, SI_Fncs& aSiFunctions, unsigned long aConfigFlags);

	void			FI_DestroyPlayer ();
	void			FI_SetBackgroundColor (short aRed, short aGreen, short aBlue);
	void			FI_SetNativeCodePage(FI_CodePage aNative_codepage);
	void			FI_SetTextFieldBoundaryBehavior (unsigned long aBehavior);
	short			FI_SetDisplayRect (const FI_Rect * aRect);
	short			FI_SetFrameBuffer (unsigned char* aFrameBuf, unsigned short aWidth, unsigned short aHeight, unsigned short aPitch, unsigned short aDisplayFormat);
#ifdef FEATURE_OPENVG	
	void			FI_SetRenderHint (FI_RenderHint renderHint);
	FI_RenderMode	FI_GetRenderMode ();
	short           FI_SetRenderMode(FI_RenderMode aMode);
	short           FI_SetHardwareFrameBuffer (FI_Display  display, struct FI_Surface* surface, FI_Context context, unsigned short width, unsigned short height);
	void			FI_SetHardwareDeviceLimit (enum FI_HardwareDeviceLimitIndex index, long value);
	long			FI_GetHardwareDeviceLimit (enum FI_HardwareDeviceLimitIndex index );
	short           FI_ConfigureHardwareRendering(struct FI_HardwareConfig* config);
#endif 
	void			FI_SetInputText (const struct FI_Text* aInputText);
	void			FI_CancelInputText ();
	void			FI_SetLooping (short aLoop);
	void			FI_SetQuality (short aQuality);
	unsigned short	FI_SetSVGDataBuffer (const char* aSvgURL, const unsigned char *aSvgBuffer, unsigned long aBufferLength);
	void			FI_GetSVGDimensions (unsigned short* aWidth, unsigned short* aHeight);
	void			FI_SetSVGViewportSize(unsigned short width, unsigned short height);
	unsigned short	FI_SetSWFDataBuffer (const char* aSwfURL, const unsigned char *aSwfBuffer, unsigned long aBufferLength, unsigned short aInlineContent, unsigned short makeCopy,enum FI_SecuritySandbox swfSandbox);
#ifdef __FEATURE_PRE_COMPILED_SWF
	unsigned short  FI_SetPreInstalledASClass(const unsigned char* aSwfBuffer,unsigned long aBufferLength,unsigned short aBIsNativeLibrary);
#endif /*__FEATURE_PRE_COMPILED_SWF*/


	unsigned short FI_GetSubDirtyRect(unsigned int index, struct FI_Rect* dirtyRect);
	unsigned short FI_GetNumberOfSubDirtyRects ();


#if defined(FEATURE_VIDEO_OVERLAY_RECTS)

	unsigned short FI_NumberOfVideoOverlayRects();
	unsigned short FI_GetVideoOverlayRects( struct FI_Rect* rects,
									   		unsigned short rectsLen );	
#endif

	unsigned short	FI_DoPlay (unsigned short aForceRender);
	void			FI_Pause ();
	void			FI_Resume ();
	void			FI_Stop ();
	void			FI_NotifyVideoStatus (unsigned long aVideoID, int aSeverity, FI_Text* aRetCode);

	unsigned short	FI_FrameNumber ();
	unsigned short	FI_GetDirtyRect (FI_Rect * rect);
	short			FI_SetDirtyRect(const FI_Rect * rect, unsigned short aOnCurrentFrame);
	short			FI_SetClippingRect(const FI_Rect * rect);

	short *			FI_GetInterruptVariable ();
	void			FI_RegisterClientObject (void* aClientPtr);
	void*			FI_GetRegisteredClientObject ();

	short			FI_GetQuality ();
	void			FI_GetSWFDimensions (unsigned short * aWidth, unsigned short * aHeight);
	unsigned short	FI_IsPlaying ();
	unsigned short	FI_NumberOfFrames ();

	unsigned short	FI_OfferKeyEvent (const FI_KeyId* aKeyId, unsigned short aEventCode);
	unsigned short	FI_OfferKeyPressEvent (const FI_KeyId* aKeyId);

	unsigned short	FI_DoHitTest (unsigned short aXPos, unsigned short aYPos);
	unsigned short	FI_OfferMouseEvent (unsigned short aXPos, unsigned short aYPos, unsigned short aEventCode);

	struct FI_NetworkBufferInfo* FI_URLStreamOpened	(unsigned short aStreamId,
										 struct FI_URLStreamHeaderInfo* aHeaders,
										 short* aErrorCode);

	void							FI_URLStreamSetFinalURL(unsigned short aStreamId,
															const char* aFinalURL,enum FI_SecuritySandbox swfSandbox);


	struct FI_NetworkBufferInfo* FI_URLStreamDataLoaded (unsigned short aStreamId, 
														 struct FI_NetworkBufferInfo* aNetBuf);

	void FI_URLStreamClosed (unsigned short aStreamId,
							 unsigned short aClosedStatusCode,
							 unsigned int aHttpStatusCode);

	void			FI_NotifySharedObjectStatus(unsigned long aRequestId, unsigned long aDataSize, short aStatus);

    unsigned short  FI_SetupDeviceImage(unsigned short aNumFormats, const char* aFormatInfo[]);
    unsigned short  FI_SetupDeviceVideo(unsigned short aNumFormats, const char* aFormatInfo[]);

	void			FI_SetupDeviceSound (unsigned short aNumFormats, const char* aFormatInfo[], bool aMustbeCached);
	void			FI_NotifyDeviceSoundStatus(unsigned short aStatus);

	unsigned short	FI_SetupStreamSound (FI_StreamSoundInfo* aSoundInfo, FI_StreamSoundSetupInfo* aSetupInfo);
	void			FI_StreamSoundBufferComplete ();
	void			FI_StreamSoundReset ();

	unsigned short	FI_Translate(long aDx, long aDy);
	unsigned short	FI_Scale(long aScale, long aXorigin, long aYorigin);
	unsigned short	FI_Rotate(long aDegrees, long aXorigin, long aYorigin);
	void			FI_ResetTransform();
	unsigned short	FI_GetZoomAndPanEnabled();
	unsigned short	FI_ConvertToUserSpace(long screenx, long screeny, long *userx, long *usery);
	unsigned short	FI_ConvertToUserSpaceF(float screenx, float screeny, float *userx, float *usery);
	unsigned short	FI_ConvertToViewportSpace(long userx, long usery, long *screenx, long *screeny);
	unsigned short	FI_ConvertToViewportSpaceF(float userx, float usery, float *screenx, float *screeny);
#ifdef FEATURE_XML_SOCKET
	short			FI_SocketRecv(unsigned long aSocketId, FI_NetworkBufferInfo* aDataBuffer, FI_NetworkBufferInfo** aNextRecvBuffer);	
	short			FI_NotifySocketStatus(unsigned long aSocketId, unsigned short aStatus, short aErrorCode);
#endif //FEATURE_XML_SOCKET
	FI_DecodeImageInfo*		FI_GetDecodeImageBuffer(unsigned long imageId, const FI_ImageInfo* pImageInfo);
	void					FI_NotifyImageDecodeStatus(unsigned long imageId, short error);

#ifdef FEATURE_BG_TRANSPARENCY
	void			FI_SetBackgroundAlpha (unsigned short aBackgroundAlpha, unsigned short aBlendWithBackground);
#endif //FEATURE_BG_TRANSPARENCY

    void 			FI_SetBaseURL( const char* baseURL);

#ifdef FEATURE_WEB_BROWSABILITY

	
	void			FI_SetScriptAccess(int pAccessLevel);
	
	//eroom 761 fix:
	void 			FI_SetContainerSecurityContext(const struct FI_BrowserURLInfo* pURLInfoIn);

	
	int				FI_GetScriptAccess();

    int				FI_TCurrentFrame( const char* target,const struct FI_BrowserURLInfo* pURLInfoIn);

    void*			FI_TCurrentLabel( const char* pTargetIn,
    								  const struct FI_BrowserURLInfo* pBrowserInfoIn,
									  struct FI_AllocatorWrapper* currentLabel);

    void*			FI_TGetProperty( const struct FI_TargetPropertyInfo* ptargetInfoIn,
									 const struct FI_BrowserURLInfo* pBrowserInfoIn,
									 struct FI_AllocatorWrapper* propertyValue);

    int				FI_TSetProperty( const struct FI_TargetPropertyInfo* ptargetInfoIn,
								 	 const struct FI_BrowserURLInfo* pBrowserInfoIn,
									 const char* value);
    int             FI_GetBackgroundColor(FI_Color *returnBgColor);

    int			    FI_CanContainerAccessMovie( const struct FI_BrowserURLInfo* pURLInfoIn,
											    int bAllowLocalUntrusted);

    int				FI_SetVariable2 (  const char* pVariableNameIn, const char* pVariableValueIn,
												const struct FI_BrowserURLInfo* pBrowserInfoIn);
    
    void*			FI_GetVariable2 (  const char* pVariableNameIn,  const struct FI_BrowserURLInfo* pBrowserInfoIn, struct FI_AllocatorWrapper* hostAllocWrapper);

    void			FI_LoadLayer (  const char*swfURL, int layer, struct FI_BrowserURLInfo* pURLInfo);
																		 
    void 			FI_SetScaleMode(int aScaleModeValue);
    
    void 			FI_SetAlignMode(int aAlignModeValue);
    
    /* to set and get the value of Embed parameter "allownetworking" 
       the value can be ""internal", "all", "none" */
    void            FI_SetAllowNetworking(int allowNetworking);
    int             FI_GetAllowNetworking();
    
#endif //FEATURE_WEB_BROWSABILITY

#ifdef FEATURE_EXTERNAL_API
	void* FI_CallFunction( const char* request, struct FI_AllocatorWrapper* allocWrapper);
	void FI_SetReturnValueFromContainer(const char* returnValue);
#endif	//FEATURE_EXTERNAL_API

	short			FI_GetMetaData( const unsigned char* swfBuffer,
										    unsigned long bufferLength,
                                            struct FI_MetaDataInfo* pMetaInfoStruct
										);
	short FI_SetMaxVideoFramesToProcess (unsigned int numFrames);										
	unsigned int 	GetStaticMemoryUsed ();
	unsigned int 	GetDynamicMemoryUsed ();

	bool			FileIsSVG() const { return iBFileIsSVG; }

    CFlashExtension* GetExtension(TUid aUid);
    CArrayPtr<CFlashExtension>* Extensions();

	void FI_NotifyFullScreenStatus (int flag);
	
    
#ifdef FEATURE_INLINE_TEXT_ENTRY
	void			FI_TextFieldRegisterNotification( const struct FI_KeyId* keyId, enum FI_EventCode eventCode, FI_TextFieldCondition* precondition );

	/*
	void			FI_SetConversionString(const struct FI_Text* pText, unsigned long cursorPos, unsigned long deltaStart, unsigned short final);
	void			FI_EditorSetSel(long selStart, long selEnd);
	void			FI_EditorGetSel(long &selStart, long &selEnd);
	long			FI_EditorGetTextLength();
	long			FI_EditorGetMaxChars();
	unsigned short	FI_EditorGetText(long startIndex, long endIndex, struct FI_Text* pText);
	*/
#endif // FEATURE_INLINE_TEXT_ENTRY

	TInt FI_GetThisSWFGUID(FI_ReturnString* aSwfGUID);
protected:

	FI_Fncs fiFunctions;

private:
	FI_Rect		iDisplayRect;
	bool 		iForceRender;

	bool 		iBFileIsSVG;
	
	struct MM_Object *						iMMObject;
	CPlatformFlashEngine*					iEngine;

private:
	void LoadLibraryL();
	void FindLibraryL();
	
	PFNC_FI_FLASHLITE_GETINTERFACE			iFncGetInterface;

	PFNC_FI_CREATEPLAYER					iFncCreatePlayer;
	PFNC_FI_CHECKSWFDATABUFFERVALIDITY		iFncCheckSwfDataBuffer;
	PFNC_FI_GETVERSION						iFncGetVersion;
	PFNC_FI_GETDISPLAYFORMAT				iFncGetDisplayFormat;

	RLibrary								iFlashDll;

    CArrayPtrFlat<CFlashExtension>*		iExtensions;

    struct SI_HostObject*				iHostObject;
};

class CHostAllocatorWrapper: public FI_AllocatorWrapper,public CBase
{
	public :
		CHostAllocatorWrapper();
		~CHostAllocatorWrapper();
		static void* MallocWrapper(AMP_AllocatorContext* aContext,size_t aSize) ;	
		static void FreeWrapper(AMP_AllocatorContext* aContext, void* aFreeString);	
};
#endif /* __FI_INTERFACE_H__ */
