/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
#ifndef HOSTENGINEGLOBALS_H_
#define HOSTENGINEGLOBALS_H_

// for SELF_IAD_UPDATE
#include "saplayer_build.h"

// forward declarations
class CSharedMemory;

#ifdef SELF_IAD_UPDATE
class CIADUpdateChecker;
class MIadUpdateCallback;
#endif
class CPlatformFlashEngine;

enum GeolocationAccess {ENotChecked, EAllow, EDeny};

/*
 * Class : CHostEngineGlobals
 * Description : Contains any globals required by Flash2Engine.dll
 *               This class is loaded into the TLS of Flash2Engine.dll
 */

class CHostEngineGlobals : public CBase
{
public:
    static CHostEngineGlobals* CreateHostEngineGlobalsL();
    void DestroyHostEngineGlobals();
    
public:
    /* Methods */
    CSharedMemory* SharedMemoryObject() const;
    
    #ifdef SELF_IAD_UPDATE
    /*
     * If the object is not created already, it creates it and returns it.
     */
    CIADUpdateChecker* IADUpdateChecker( const TUid& aApplicationUID
                                         ,const TDesC& aAppSearchCriteria
                                         ,const TDesC& aCommandLineExecutable 
                                         ,const TDesC8& aCommandLineArguments
                                         ,TBool aShowProgress
                                         ,MIadUpdateCallback* aIADUpdateCallbackInterface
                                        );
		#endif
  
    void PlatformFlashEngine(CPlatformFlashEngine *aPlatformFlashEngine);
    CPlatformFlashEngine* GetPlatformFlashEngine();		
	#ifdef FEATURE_AMCEXTENSIONS
    void GeolocationAccessStatus(GeolocationAccess);
    GeolocationAccess GeolocationAccessStatus();
    TInt GetNoOfPlugins();
	#endif
#ifdef FEATURE_OPENVG    
    void AddEGLClient(TAny* aClient);
    void RemoveEGLClient(TAny* aClient);
    TInt NumEGLClients();
#endif
    
    /*
	TBool UpdatesChecked();	// return the flag
	void  UpdatesChecked( TBool aUpdateCheckedStatus );	// set the flag
	*/

private:
    /* Constructors */
    
    /* Making the constructor private so that the object cannot be constructed on the stack */
    CHostEngineGlobals();
    
    /* First phase constructor */
    static CHostEngineGlobals* NewLC();
    
    /* Second phase constructor */
    void ConstructL();

public:
    /* Destructor */
    /* The object *must* be destroyed by calling DestroyHostEngineGlobals()
     * DestroyHostEngineGlobals() keeps count of the references
     * If the count reaches zero, then and only then is the object destroyed
     */
    ~CHostEngineGlobals();
    
private:
    /* Methods */
    TUint32 IncrementReference();
    TUint32 DecrementReference();
    
    
private:
    /* Variables */
    
    // Local Connection
    CSharedMemory* iSharedMemory;
	
	#ifdef SELF_IAD_UPDATE
	// IAD
	// TBool iUpdateCheckedStatus;
    CIADUpdateChecker* iIADUpdateChecker;
  #endif
    #ifdef FEATURE_AMCEXTENSIONS
    GeolocationAccess iGeolocationAccess;
 #endif
    CPlatformFlashEngine *iPlatformFlashEngine;
    
private:
    // Variables required to maintain the state of this object
    TUint32 iReferenceCount;
#ifdef FEATURE_OPENVG    
    RPointerArray<TAny> iEGLClients;
#endif    
};


/*
 * Handles shared memory shared across object in the same DLL
 * Since it is quite closely tied to HostEngine globals, keeping it in the same file
 * (Can be moved if required)
 */

class CSharedMemory : public CBase
{
public:
   
    static CSharedMemory* NewL();
    
    virtual ~CSharedMemory();
    
    /* Allocate requests for a pointer to shared memory.
     * If memory has not been allocated yet, it will be.
     * Otherwise, the pointer to allocated memory is returned
     * and the reference count is incremented.
     */
    TAny* GetSharedMemoryL(TUint32 aSharedMemorySize);
    
    /*
     * Decreases the reference count by 1.
     * Incase the reference count is zero, then the memory is freed up.
     */ 
    void FreeSharedMemory(); 
    
private:
    /* Constructors */
    
    /* Making the constructor private so that the object cannot be constructed on the stack */
    CSharedMemory();
    
    /* First phase constructor */
    static CSharedMemory* NewLC();
    
    /* Second phase constructor */
    void ConstructL();

private:
    /* Variables */
    TUint32 iSharedMemorySize;  // size of the shared memory
    TAny* iPtrSharedMemory;     // a pointer to the shared memory
                                // memory is allocated only when the reference count is zero
    TInt iReferenceCount;       // keep track of how many objects are referencing the shared memory
};

#endif /*HOSTENGINEGLOBALS_H_*/
