/*
* Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class that handles IP connection and disconnection.
*
*/


#ifndef __IP_CONNECTION_MGR_H__
#define __IP_CONNECTION_MGR_H__

#include <connpref.h>
#include <cmapplicationsettingsui.h>
using namespace CMManager;
#include <e32base.h>
#include "symbian_engine.h"
#include "conn_handler.h"


//const TInt KMaxConnectionRetries=1;
class CConnHanlder;

enum TFLIAPStatus
	{
	EFLIAPAlreadySet,
	EFLIAPNotSet
	};

enum TConnectionMgrStatus 
	{
	EFLIPConnMgrWaiting, 
	EFLIPConnMgrIdle
	};

enum {
	ENetworkQuery_NotSet,
	ENetworkQuery_Allow,
	ENetworkQuery_Deny,
	ENetworkQuery_Pending
	};

class CObsEntry: public CBase
	{	
public:
	CObsEntry(MFlhIPConnectObserver* aObserver,CConnHandler* aConnHandler):
	iIPConnObs(aObserver),iConnHandler(aConnHandler)
		{
		
		}
	
	~CObsEntry()
		{
		iIPConnObs = NULL;
		iConnHandler = NULL;
		}
	
	MFlhIPConnectObserver* iIPConnObs;
	CConnHandler* iConnHandler;
	};


/**
 * The CFlhIPConnection class handles IP connection and disconnection.
 */
class CIPConnMgr: 	public CBase,
					public MConnHandlerObs
{
public:
	
	static CIPConnMgr* NewL(CPlatformFlashEngine* aEngine );
	
	TFLIAPStatus GetIAP(TUint32& aIAP); //returns iIAPStatus and if IAPStatus is EFLIAPAlreadySet then aIAP will point to currently set IAP.
	CConnHandler* GetConnectionL(MFlhIPConnectObserver *aObserver);

	~CIPConnMgr();
	
private:
	//from MConnHndlrObs
	void NotifyConnHandlerStatusL(TInt aEvent, CConnHandler* aConnHandler);

	CIPConnMgr(CPlatformFlashEngine* aEngine);
	
	void IssueConnectionRequestL(MFlhIPConnectObserver* aObserver, CConnHandler* aHandler);
	
	void ConstructL();
	
	void QueryAllowNetwork();
	
private:
	CPlatformFlashEngine* iEngine; //not owned
	TUint32 iIAP;
	TInt iBearerType; 
	TInt iIAPStatus;
	TInt iConnectionDlgStatus;
	RPointerArray<CObsEntry> iObsQueue;
	
	TInt iHasUserAllowedNetAccess;

	TBool iNetWorkPromptNeeded;
	
	TConnectionMgrStatus iConnMgrStatus;
	
#if defined(FEATURE_ATS_AUTOMATION) || defined(FEATURE_PERFORMANCE_AUTOMATION) || defined(FEATURE_TESTAUTOMATION)
	TBool iAutomationMode;
#endif
};

#endif //__IP_CONNECTION_MGR_H__
