/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class that provides shared object utilities
*
*/


#ifndef USE_SYNC_MSO

#ifndef __SHOBJUTILS_H__
#define __SHOBJUTILS_H__

#include <e32std.h>

#include "symbian_sa_build.h"
#include "symbian_sharedobjects.h"
#include "async_object.h"
#include "fi_interface.h"

class MFlhShObjNodeListObserver;

/**
 * The CFlhShObjEngine class provides functions that implement processing for shared object transactions
 */
class CFlhShObjUtils : public CBase
{
public:
	static CFlhShObjUtils* NewL(RFs& aFs, CFlhFileUtils& aFileUtils, const TDesC& aPersistentDirectory, CPlatformFlashEngine& aEngine);
	~CFlhShObjUtils();
private:
	CFlhShObjUtils::CFlhShObjUtils(RFs& aFs, CFlhFileUtils& aFileUtils, CPlatformFlashEngine& aEngine);
	void SetPersistentDirectoryL(const TDesC& aPersistentDirectory);

public:
	TFileName* GetSharedObjectFileName(const char* aNameSpace, const char* aName);
    CBase* ListAllNodesL(const char* aNameSpace, TSglQue<CPersistentStorageNode>*& aNodeList, MFlhShObjNodeListObserver& aObs);
    TBool IsOlder(CPersistentStorageNode* aNode1, CPersistentStorageNode* aNode2);
    CBase* GetNameSpaceSizeL(const char *aNameSpace, TInt& aSize, MFlhShObjNodeListObserver& aObs);
	TInt GetObjectSizeL(const TDesC& aFilename);
    CBase* RemoveDirectoryL(const char* aNameSpace, MFlhShObjNodeListObserver& aObs);
	RFs& FsSession();
	CFlhFileUtils& Utils();
	TFileName* GetPersistenceDir(const char* aNameSpace);
	TFileName* GetSharedObjectFileName(const char* aNameSpace, const TDesC& aName);
	TBool EnsureDirectory(const TDesC& aDirectory);	
	void CleanupSharedObjectsL(struct MM_Object* aMMObj, 
                                                  TUint32 aNumBytesToClean,
                                                  TInt aId,
												  MFlhShObjCleanupObserver& aObs);

	void CancelCleanupSharedObjects();
    const TDesC& PersistentDirectory() const {return *iPersistentDirectory;}
    void 	MigrateMSODirectoryL(const TDesC& aMSOFileName, const TDesC& aNewMSODirectory);

private:
    TInt GetAccessTime(CPersistentStorageNode* aNode, TTime& aTimeStamp);

    HBufC* 	GetFlash8PersistentDirectoryLC();
    HBufC*	GetFlash8MSOFileLC(const TDesC& aMSOFileName);
    void   	MoveMSOFileToFlash9LocationL(const TDesC& aOldFile, const TDesC& aNewMSODirectory);

private:
	RFs&					iFs;
	CFlhFileUtils&			iFileUtils;
	CPersistentStorageCleanupRequest* iCleanupRequest;
	HBufC*                  iPersistentDirectory;
	CPlatformFlashEngine&	iEngine;
};

#endif // __SHOBJUTILS_H__

#endif //USE_SYNC_MSO
