/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  defines the member functions of the class that is used to 
*		implement adaptation API for flash player
*
*/
#ifndef _SI_INTERFACE_INCLUDE
#define _SI_INTERFACE_INCLUDE

#include "platformbuild.h"

#include "flashliteinterfaces.h"

#ifdef FEATURE_EXTENSION
#include "flashliteExtensionInterfaces.h"
#endif
#define KCleanupSharedObjectsCallbackSuccess  	1
#define KCleanupSharedObjectsCallbackFailure 	0
//RK: CodecTypes

#define KJPEGCodec 			1
#define KH263Codec 			2
#define KScreenShareCodec 	3
#define	KVP6Codec 			4
#define KVP6AlphaCodec 		5


#ifdef FEATURE_VIDEO_DLL
struct IVideoDecompressorInstance;
#endif

#ifdef FEATURE_OPENVG
#include <VG/openvg.h>
#include <EGL/egltypes.h>
#include <EGL/egl.h>
#endif

// class containing implementation of SI_ functions

/**
 * The SI_Interface class is a container class for all of the SI functions. Pointers to these functions
 * will be passed to the Flash library.
 *
 */
class SI_Interface
{
private:
	// Make constructor private to prevent anyone from ever trying to instantiate it.
	SI_Interface() {}
	
public:

	static void *			SI_Malloc (struct MM_Object* aMMObj, unsigned short aNumBlocks);
	
	static void				SI_Free (struct MM_Object* aMMObj, void * aMemory);
	
	static unsigned char *	SI_GetLockedFrameBuffer (struct MM_Object* aMMObj);
	
	static void				SI_UnlockFrameBuffer (struct MM_Object* aMMObj, unsigned char * aBits);
	
#ifdef FEATURE_OPENVG

	static short SI_GetHardwareInterfaces(struct MM_Object* aMMObj, void** ppEgl, void** ppApi);

	static FI_Display SI_eglGetDisplay(struct MM_Object* aMMObj);

	static short SI_eglInitialize(struct MM_Object* aMMObj, FI_Display display, short* pMajor, short* pMinor);

	static short SI_eglTerminate(struct MM_Object* aMMObj, FI_Display display);

	static FI_Context SI_eglCreateContext(struct MM_Object* aMMObj, FI_Display display, FI_Config config, FI_Context shareContext, const long* pAttribList);

	static short SI_eglChooseConfig(struct MM_Object* aMMObj, FI_Display display, const long* pAttribList, FI_Config* pConfigs, unsigned short configSize, unsigned short* pNumConfig);

	static struct FI_Surface* SI_eglCreatePbufferSurface(struct MM_Object* aMMObj, FI_Display display, FI_Config config, const long* pAttribList);

	static struct FI_Surface* SI_eglCreateNativePixmapSurface(struct MM_Object* aMMObj, FI_Display display, FI_Config config, unsigned short width, unsigned short height, const long* pAttribList, struct FI_PixmapInfo* pPixmapInfo);

	static struct FI_Surface* SI_eglCreatePbufferFromClientBuffer(struct MM_Object* aMMObj, FI_Display display, unsigned long buftype, FI_ClientBuffer buffer, FI_Config config, const long* pAttribList);

	static short SI_eglMakeCurrent(struct MM_Object* aMMObj, FI_Display display, const struct FI_Surface* pDraw, const struct FI_Surface* pRead, FI_Context context);

	static short SI_eglDestroyContext(struct MM_Object* aMMObj, FI_Display display, FI_Context context);

	static short SI_eglDestroySurface(struct MM_Object* aMMObj, FI_Display display, struct FI_Surface* pSurface);

	static short SI_eglGetConfigAttrib(struct MM_Object* aMMObj, FI_Display display, FI_Config config, long attrib, long* pValue);


	static short SI_eglCopySurfaceToFrameBuffer(struct MM_Object* aMMObj, FI_Display display, const struct FI_Surface* pSurface, unsigned char* pDst, unsigned short width, unsigned short height, unsigned short pitch, struct FI_Rect* dirtyRect);

	static void SI_ovgAppendPath(void* clientData, void* dstPath, void* srcPath);
	
	static void SI_ovgAppendPathData(void* clientData, void* dstPath, int numSegments, const unsigned char * pathSegments, const void * pathData);
	
	static void SI_ovgClear(void* clientData, int x, int y, int width, int height);
	
	static void SI_ovgClearImage(void* clientData, void* image, int x, int y, int width, int height);
	
	static void SI_ovgClearPath(void* clientData, void* path, unsigned int capabilities);
	
	static void* SI_ovgCreateImage(void* clientData, int format, int width, int height, unsigned int allowedQuality);
	
	static void* SI_ovgCreateMaskLayer(void* clientData, int width, int height);
	
	static void* SI_ovgCreatePaint(void* clientData);
	
	static void* SI_ovgCreatePath(void* clientData, int pathFormat, unsigned int datatype, float scale, float bias, int segmentCapacityHint, int coordCapacityHint, unsigned int capabilities);
	
	static void SI_ovgCopyMask(void* clientData, void* maskLayer, int sx, int sy, int dx, int dy, int width, int height);
	
	static void SI_ovgDestroyImage(void* clientData, void* image);
	
	static void SI_ovgDestroyMaskLayer(void* clientData, void* maskLayer);
	
	static void SI_ovgDestroyPaint(void* clientData, void* paint);
	
	static void SI_ovgDestroyPath(void* clientData, void* path);
	
	static void SI_ovgDrawImage(void* clientData, void* image);
	
	static void SI_ovgDrawPath(void* clientData, void* path, unsigned int paintModes);
	
	static void SI_ovgFinish(void* clientData);
	
	static void SI_ovgFlush(void* clientData);
	
	static unsigned int SI_ovgGetError(void* clientData);
	
	static void SI_ovgGetMatrix(void* clientData, float* m);
	
	static int SI_ovgGetParameteri(void* clientData, void* object, int paramType);
	
	static int SI_ovgGeti(void* clientData, int type);
	
	static void SI_ovgImageSubData(void* clientData, void* image, const void * data, int dataStride, unsigned int dataFormat, int x, int y, int width, int height);
	
	static int SI_ovgInterpolatePath(void* clientData, void* dstPath, void* startPath, void* endPath, float amount);
	
	static void SI_ovgLoadIdentity(void* clientData);
	
	static void SI_ovgLoadMatrix(void* clientData, const float * m);
	
	static void SI_ovgMask(void* clientData, void* mask,  unsigned int operation, int x, int y, int width, int height);
	
	static void SI_ovgModifyPathCoords(void* clientData, void* dstPath, int startIndex, int numSegments, const void * pathData);
	
	static void SI_ovgMultMatrix(void* clientData, const float * m);
	
	static void SI_ovgPaintPattern(void* clientData, void* paint, void* pattern);
	
	static void SI_ovgPathBounds(void* clientData, void* path, float * minX, float * minY, float * width, float * height);
	
	static void SI_ovgRenderToMask(void* clientData, void* path, unsigned int paintModes, unsigned int operation);
	
	static void SI_ovgScale(void* clientData,  float sx, float sy);
	
	static void SI_ovgSetColor(void* clientData, void* paint, unsigned int rgba);
	
	static void SI_ovgSetPaint(void* clientData, void* paint, unsigned int paintModes);
	
	static void SI_ovgSetParameterfv(void* clientData, void* object, int paramType, int count, const float * values);
	
	static void SI_ovgSetParameteri(void* clientData, void* object, int paramType, int value);
	
	static void SI_ovgSetf(void* clientData, int type, float value);
	
	static void SI_ovgSetfv(void* clientData, int type, int count, const float * values);
	
	static void SI_ovgSeti(void* clientData, int type, int value);
	
	static void SI_ovgSetiv(void* clientData, int type, int count, const int * values);
	
	static void SI_ovgTransformPath(void* clientData, void* dstPath, void* srcPath);
	
	static void SI_ovgTranslate(void* clientData, float tx, float ty);
	
	static void* SI_ovgChildImage(void* clientData, void* parent, int x, int y, int width, int height);
	
#endif
	
	static void             DOSI_ShowLocalSecurityDialogL(struct MM_Object* aMMObj,const char* pPathToDisplay, const char* pAccessedDomain);
	static void             SI_ShowLocalSecurityDialog(struct MM_Object* aMMObj,const char* pPathToDisplay, const char* pAccessedDomain);
	static void				SI_BeginTextDraw (struct MM_Object* aMMObj);
	
	static unsigned short	SI_DeviceTextOut (struct MM_Object* aMMObj, unsigned short aFontSize, const struct FI_Text * text, struct FI_TextInfo* aTextInfo, void *bits, int width, int height);

	static unsigned short	SI_ComplexLineBreak (struct MM_Object* aMMObj, unsigned short aFontSize, const struct FI_Text *aText, struct FI_DeviceTextInfo *aDeviceTextInfo);
		
	static void				SI_EndTextDraw (struct MM_Object* aMMObj);
	
	static unsigned short	SI_GetBitmapInfo (struct MM_Object* aMMObj, unsigned short aFontSize, const struct FI_Text * aText, struct FI_BitmapInfo* aInfo);
	
	static unsigned short	SI_GetCharacterInfo (struct MM_Object* aMMObj, unsigned short aFontSize, const struct FI_Text * aText, struct FI_CharacterInfo* aInfo);
	
	static void				SI_GetFontMetrics (struct MM_Object* aMMObj, unsigned short aFontSize, short* aAscent, short* aDescent, unsigned short* aSingleByteSpaceWidth);
	
	static unsigned short	SI_GetTextWidth (struct MM_Object* aMMObj, unsigned short aFontSize, const struct FI_Text * aText, unsigned long* aWidth);

	static unsigned short	SI_CreateFont (struct MM_Object* aMMObj, const struct FI_FontInfo* aFontInfo, void** aFontId);
	
	static unsigned short	SI_DestroyFont (struct MM_Object* aMMObj, void* aFontId);
	
	static unsigned short	SI_SelectFont (struct MM_Object* aMMObj, void* aFontId);
	
	static void             SI_NewFontSource( struct MM_Object* aMMObj, struct FI_FontInfo* aFontInfo, struct FI_FontSource** aFontSource );
	
	static void             SI_FreeFontSource(struct MM_Object* pMMObj, struct FI_FontSource* aSource);

#ifdef	FEATURE_OPENVG
	static unsigned char*   SI_GetLockedTextBuffer (struct MM_Object* pMMObj, unsigned int* width, unsigned int* height, unsigned int* pitch, unsigned int* depth);
	
	static void             SI_UnlockTextBuffer (struct MM_Object* pMMObj, unsigned char* pBits);	
#endif	
	
	static short			SI_CacheSound (struct MM_Object* aMMObj, void* aSoundBuffer, unsigned long soundBufferSize, unsigned long * aSoundID);
	
	static unsigned short	SI_PlayCachedSound (struct MM_Object* aMMObj, unsigned long aSoundID, unsigned long count);
	
	static void				SI_FreeCachedSound (struct MM_Object* aMMObj, unsigned long aSoundID);
	
	static short			SI_PlaySound (struct MM_Object* aMMObj, void* aSoundBuffer, unsigned long aSoundBufferSize, unsigned long count);
	
	static void				SI_StopSound (struct MM_Object* aMMObj);

	static unsigned short	SI_PauseSound (struct MM_Object* aMMObj);

	static unsigned short	SI_ResumeSound (struct MM_Object* aMMObj);
	
	static short			SI_CreateVideo(struct MM_Object* aMMObj, const FI_Rect* aLocation, unsigned long* aRetVideoID);

	static short			SI_PlayVideo(struct MM_Object* aMMObj, const FI_Video* aVideoInfo);

	static void				SI_CloseVideo(struct MM_Object* aMMObj, unsigned long aVideoID);

	static short			SI_StopVideo(struct MM_Object* aMMObj, unsigned long aVideoID);

	static short			SI_PauseVideo(struct MM_Object* aMMObj, unsigned long aVideoID);

	static short			SI_ResumeVideo(struct MM_Object* aMMObj, unsigned long aVideoID);

	static short			SI_GetVideoMethods(struct MM_Object* aMMObj, FI_ReturnString* aMethods);

	static short			SI_ProcessVideoMethod(struct MM_Object* aMMObj, unsigned long videoID, const FI_Method* aMethod, FI_ReturnString* aRetString);

	static void				SI_DestroyVideo(struct MM_Object* aMMObj, unsigned long aVideoID);

	static void				SI_SetVideoRect(struct MM_Object* aMMObj, unsigned long aVideoID, const FI_Rect* aLocation);
	
	static void             SI_GetVideoSize(struct MM_Object* aMMObj, unsigned long videoID, unsigned long* retWidth, unsigned long* retHeight);
	
	static short            SI_GetVideoBits(struct MM_Object* aMMObj, unsigned long videoID, unsigned char** bits);

	static unsigned short	SI_CloseStreamSoundDevice (struct MM_Object* aMMObj);
	
	static void *			SI_CreateSoundDecompressor (struct MM_Object* aMMObj, struct FI_StreamSoundInfo* aSoundInfo, unsigned long aFormatInfo);
	
	static long				SI_DecompressSound (struct MM_Object* aMMObj, void* aDecompressor, void* aDestBuffer, unsigned long aNumSamples);
	
	static void				SI_DestroySoundDecompressor (struct MM_Object* aMMObj, void* aDecompressor);
	
	static unsigned short	SI_OpenStreamSoundDevice (struct MM_Object* aMMObj);
	
	static unsigned short	SI_PlayStreamSoundBuffer (struct MM_Object* aMMObj, void* aData);
	
	static void				SI_SetupSoundDecompressor (struct MM_Object* aMMObj, void* aDecompressor, struct FI_StreamSoundDecompressorInfo* aDecompressorInfo);

	static void				SI_GetInputText (struct MM_Object* aMMObj,const struct FI_Text* aInitialText,
												const struct FI_Text* aDialogTitleText, unsigned short aMaxNumChars, unsigned short aTextFlags);
	
#ifdef FEATURE_INLINE_TEXT_ENTRY

	static void				SI_EnableIME (struct MM_Object* pMMObj, struct FI_TextField* aTextField);

	static void				SI_DisableIME (struct MM_Object* pMMObj);

	static void				SI_DrawFullScreenInputModeIndicator(struct MM_Object* aMMObj, struct FI_Rect* editingTextFieldBounds, struct FI_Rect* indicatorBounds);

#ifdef FEATURE_TEXTFIELDNOTIFICATION
	static SI_Error			SI_TextFieldDidSelect( struct MM_Object* aMMObj, struct FI_TextField* textField );

	static SI_Error			SI_TextFieldWillDeselect( struct MM_Object* aMMObj, struct FI_TextField* textField );

	static unsigned int		SI_TextFieldNotify(
								struct MM_Object* aMMObj,
								struct FI_TextField* textField,
								const struct FI_KeyId* keyId,
								enum FI_EventCode eventCode,
								FI_TextFieldCondition* precondition );
#endif
	
	/*
	static unsigned short	SI_IsIMEEnabled (struct MM_Object* pMMObj);

	static unsigned short	SI_IsConversionInProgress (struct MM_Object* pMMObj);

	static void				SI_CancelConversion (struct MM_Object* pMMObj);
	*/

#endif // FEATURE_INLINE_TEXT_ENTRY

	static unsigned short	SI_GetURL (struct MM_Object* aMMObj, struct FI_GetURLCallInfo* aGetURLInfo);

	static short			SI_OpenURLStream (struct MM_Object* aMMObj, struct FI_URLStreamInfo* aStreamInfo, unsigned short aStreamId);

	static void				SI_LoadURLStreamBuffer (struct MM_Object* aMMObj, unsigned short aStreamId, struct FI_NetworkBufferInfo* aNetBuf);

	static void				SI_CancelURLStream (struct MM_Object* aMMObj, unsigned short aStreamId);
#ifdef FEATURE_FLV_MMC
	
	static int 				SI_SeekURLStreamFLV(struct MM_Object* aMMObj,
							    unsigned short streamId,
								unsigned int seekTime,
								struct FI_NetworkBufferInfo* pNetBuf);
	static void				SI_FLVStatus(struct MM_Object* aMMObj, signed short streamId, const char* code); 
	static unsigned int 			SI_FLVGetBytesLoaded(struct MM_Object* aMMObj,unsigned short streamId);//Sathya
	static unsigned int     SI_FLVGetTotalBytes(struct MM_Object* aMMObj, unsigned short streamId);
#endif// FEATURE_FLV_MMC
	
#ifdef FEATURE_EXTENSION
	static PluginInfo* 			SI_GetPluginInfo(struct MM_Object* aMMObj,struct ExtensionInterface* pEI);
#endif
	static void				SI_NotifyError (struct MM_Object* aMMObj, short aErrorCode);
	
	static void				SI_UpdateFrameInterval (struct MM_Object* aMMObj, unsigned long aNewFrameInterval);

#ifdef FEATURE_COMPUTE_STACK
	static void*            SI_ComputeStackBase(struct MM_Object* aMMObj);
#endif
	
	static unsigned short	SI_DoFsCommand (struct MM_Object* aMMObj, const struct FI_Text* aCmd, const struct FI_Text* aArgs, struct FI_PlayerEventInfo* aEventInfo);
	
	static short			SI_FormatDate (struct MM_Object* aMMObj, const struct FI_LargeInteger* aTimeStamp, unsigned long aModifiers, FI_ReturnString* aString);

	static short			SI_FormatTimeOfDay (struct MM_Object* aMMObj, const struct FI_LargeInteger* aTimeStamp, unsigned long aModifiers, FI_ReturnString* aString);

	static short			SI_GetBatteryLevel (struct MM_Object* aMMObj);
	
	static short			SI_GetDevice (struct MM_Object* aMMObj, unsigned char * aBuffer);
	
	static unsigned short	SI_GetDeviceCapabilities (struct MM_Object* aMMObj);
	
	static short			SI_GetDeviceID (struct MM_Object* aMMObj, unsigned char * aBuffer);
	
	static short			SI_GetLanguage (struct MM_Object* aMMObj, unsigned char * aBuffer);
	
	static long				SI_GetMaxAvailablePlayerMemory (struct MM_Object* aMMObj);
	
	static short			SI_GetMaxBatteryLevel (struct MM_Object* aMMObj);
	
	static short			SI_GetMaxVolumeLevel (struct MM_Object* aMMObj);
	
	static short			SI_GetPlatform (struct MM_Object* aMMObj, unsigned char * aBuffer);
	
	static short			SI_GetPowerSource (struct MM_Object* aMMObj);
	
	static short		 	SI_GetUTCTimeStamp (struct MM_Object* aMMObj, struct FI_LargeInteger* aResult);

	static short			SI_ConvertUTCToLocalTime(struct MM_Object* aMMObj, 
													 const struct FI_LargeInteger* aUTCTimeStamp, 
													 struct FI_LargeInteger* aLocalTimeStamp);
	static short			SI_ConvertLocalToUTCTime(struct MM_Object* aMMObj, 
													 const struct FI_LargeInteger* aLocalTimeStamp,
													 struct FI_LargeInteger* aUTCTimeStamp);

	static short 			SI_GetVolumeLevel (struct MM_Object* aMMObj);
	
	static short			SI_Launch (struct MM_Object* aMMObj, unsigned short aArgc, const struct FI_Text** argv, struct FI_PlayerEventInfo* aEventInfo, short aIsEmbedded);
	
	static short			SI_QuitFlashLite (struct MM_Object* aMMObj, struct FI_PlayerEventInfo* aEventInfo);
	
	static short			SI_ResetSoftkeys (struct MM_Object* aMMObj);
	
	static short			SI_GetSoftKeyLocation (struct MM_Object* aMMObj);

	static short			SI_SetFullScreen (struct MM_Object* aMMObj, unsigned short aOnOrOff);
	
#ifdef __FEATURE_OSK__
	static short            SI_DisableKeypadCompatibilityMode (struct MM_Object* aMMObj);
#endif // __FEATURE_OSK__
	static short			SI_SetSoftkeys (struct MM_Object* aMMObj, unsigned short aArgc, const FI_Text** aArgv);

	static short			SI_GetNumberOfSoftkeys(struct MM_Object* aMMObj);
	
	static short			SI_StartVibrate (struct MM_Object* aMMObj, unsigned short aTimeOn, unsigned short aTimeOff, unsigned short aRepeatCount);
	static short 			SI_StopVibrate (struct MM_Object* aMMObj);
	
	static void				SI_Debug (struct MM_Object* aMMObj, const char* aMessage, long aNumber);
		
	static short			SI_ExtendBacklightDuration(struct MM_Object* aMMObj, unsigned short aDuration);

	static short			SI_GetScreenInfo(struct MM_Object* aMMObj, unsigned short* aX, unsigned short* aY, unsigned short* aRotation);

	static short			SI_GetNetworkInfo(struct MM_Object* aMMObj, unsigned short aInfoType, long* aRetVal);
	
	static short			SI_GetNetworkInfoString(struct MM_Object* aMMObj, unsigned short, FI_ReturnString* aRetVal);
	
	static short			SI_GetCallerInfo(struct MM_Object* aMMObj, struct FI_ReturnString* aName, struct FI_ReturnString* aNumber);

	static short			SI_GetImageInfo (struct MM_Object* aMMObj, unsigned long pImageId, const struct FI_ImageData* aImageData);

	static short			SI_CancelImageRequest (struct MM_Object* aMMObj, unsigned long pImageId);

#ifndef USE_SYNC_MSO	
	static short			SI_SetPersistentData(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, const unsigned char* aData, unsigned long aDataSize, unsigned long aRequestId);

	static short			SI_GetPersistentDataSize(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, unsigned long aRequestId);

	static short			SI_GetPersistentData(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, unsigned char* aData, unsigned long aDataSize, unsigned long aRequestId);

	static short			SI_GetMaxPersistentStorage(struct MM_Object* aMMObj, const char* aRootMovieGUID, unsigned long* aSize);

	static void				SI_CancelSharedObjectRequest(struct MM_Object* aMMObj, unsigned long aRequestId);
#else

	static short            SI_SetPersistentDataSync(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, const unsigned char* aData, unsigned long aDataSize);

    static short            SI_GetPersistentDataSizeSync(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, unsigned long* dataSize);

    static short            SI_GetPersistentDataSync(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, unsigned char* aData, unsigned long* aDataSize);

    static short            SI_GetMaxPersistentStorage(struct MM_Object* aMMObj, const char* aRootMovieGUID, unsigned long* aSize);
#endif // USE_SYNC_MSO
    
#ifdef FEATURE_LOCAL_CONNECTION
	static void*            SI_CreateSharedMemory(struct MM_Object* aMMObj, const char* aName,
											const char* aLockname, unsigned long aSharedMemSize, unsigned long* aHandle );

	static void             SI_DestroySharedMemory(struct MM_Object* aMMObj, unsigned long aHandle );
	
	static short            SI_LockSharedMemory (struct MM_Object* pMMObj,
	                                                         unsigned long handle);

	static short            SI_UnLockSharedMemory (struct MM_Object* pMMObj,
	                                                           unsigned long handle);
#endif // FEATURE_LOCAL_CONNECTION
	
#ifdef FEATURE_XML_SOCKET
	static short			SI_OpenSocket(struct MM_Object* aMMObj, FI_SocketConnectionInfo* aConnInfo, const struct FI_PlayerEventInfo* aEventInfo);

	static short			SI_CloseSocket(struct MM_Object* aMMObj, unsigned long aSocketId);

	static short			SI_SocketSend(struct MM_Object* aMMObj, unsigned long aSocketId, FI_NetworkBufferInfo* aDataBuffer);
#endif //FEATURE_XML_SOCKET

	static SI_Error			SI_ConvertCStringToUTF16String(struct MM_Object* aMMObj, const char* aSrc, unsigned int codePage, FI_ReturnString* aDst);
	static SI_Error			SI_ConvertUTF16StringToCString(struct MM_Object* aMMObj, const FI_UTF16Char* aSrc, unsigned int codePage, FI_ReturnString* aDst);

#ifdef FEATURE_BG_TRANSPARENCY
	static short			SI_UpdateBackgroundRegion(struct MM_Object* aMMObj, struct FI_Rect *rects, unsigned short numRects);
#endif //FEATURE_BG_TRANSPARENCY
	///file access SI functions///////////////////
	static unsigned short	SI_OpenFile(MM_Object* pMMObj, const char * fileName, FI_FileMode mode, void ** fileHandle);
	static void				SI_CloseFile(MM_Object* pMMObj, void * fileHandle);
	static unsigned long	SI_ReadFile(MM_Object* pMMObj, void * fileHandle, unsigned char * buffer, unsigned long bufferSize);
	static unsigned long	SI_GetFileSize(MM_Object* pMMObj, void * fileHandle);
	static unsigned short	SI_SeekFile(MM_Object* pMMObj, void * fileHandle, long offset, unsigned short origin);

#ifdef FEATURE_VIDEO_DLL
	static IVideoDecompressorInstance * SI_NativeVideoCreateDecompressor(MM_Object* pMMObj, void * iAllocator, U32 codecType );
	static void				SI_NativeVideoDestroyDecompressor(MM_Object* pMMObj, IVideoDecompressorInstance * decompressor);
#endif // FEATURE_VIDEO_DLL
#ifdef FEATURE_EXTERNAL_API
/*
 * External API functions
 */
	static int			SI_EvaluateJScript (struct MM_Object* pMMObj, 
										   const char* expression);

	static int			SI_DoFlashCall (struct MM_Object *pMMObj, 
												const char* request);

	static void *		SI_GetDOMObjectID (struct MM_Object *pMMObj, 
											struct FI_AllocatorWrapper * allocWrapper);

#endif //FEATURE_EXTERNAL_API
#if defined(FEATURE_ACHW) && (ACHW_DRAWING_LIB == ACHW_DRAWING_LIB_IS_OPENGL_ES)
	
	static unsigned char 	SI_glIsTexture(MM_Object* pMMObj, unsigned int texture);
	static unsigned int  	SI_glGetError(MM_Object* pMMObj);
	static void          	SI_glActiveTexture(MM_Object* pMMObj, unsigned int texture);
	static void          	SI_glAlphaFuncx(MM_Object* pMMObj, unsigned int func, int ref);
	static void          	SI_glBindBuffer(MM_Object* pMMObj, unsigned int target, unsigned int buffer);
	static void          	SI_glBindTexture(MM_Object* pMMObj, unsigned int target, unsigned int texture);
	static void          	SI_glBlendFunc(MM_Object* pMMObj, unsigned int sfactor, unsigned int dfactor);
	static void          	SI_glBufferData(MM_Object* pMMObj, unsigned int target, int size, const void* data, unsigned int usage);
	static void          	SI_glClear(MM_Object* pMMObj, unsigned int mask);
	static void          	SI_glClearColorx(MM_Object* pMMObj, int red, int green, int blue, int alpha);
	static void          	SI_glClearDepthx(MM_Object* pMMObj, int depth);
	static void          	SI_glClientActiveTexture(MM_Object* pMMObj, unsigned int texture);
	static void          	SI_glColor4x(MM_Object* pMMObj, int red, int green, int blue, int alpha);
	static void          	SI_glColorMask(MM_Object* pMMObj, unsigned char red, unsigned char green, unsigned char blue, unsigned char alpha);
	static void          	SI_glDeleteTextures(MM_Object* pMMObj, int n, const unsigned int* textures);
	static void          	SI_glDepthFunc(MM_Object* pMMObj, unsigned int func);
	static void          	SI_glDepthMask(MM_Object* pMMObj, unsigned char flag);
	static void          	SI_glDisable(MM_Object* pMMObj, unsigned int cap);
	static void          	SI_glDisableClientState(MM_Object* pMMObj, unsigned int array);
	static void          	SI_glDrawArrays(MM_Object* pMMObj, unsigned int mode, int first, int count);
	static void          	SI_glEnable(MM_Object* pMMObj, unsigned int cap);
	static void          	SI_glEnableClientState(MM_Object* pMMObj, unsigned int array);
	static void          	SI_glFinish(MM_Object* pMMObj);
	static void          	SI_glGenTextures(MM_Object* pMMObj, int n, unsigned int* textures);
	static void          	SI_glGetIntegerv(MM_Object* pMMObj, unsigned int pname, int* params);
	static void          	SI_glHint(MM_Object* pMMObj, unsigned int target, unsigned int mode);
	static void          	SI_glLineWidthx(MM_Object* pMMObj, int width);
	static void          	SI_glLoadIdentity(MM_Object* pMMObj);
	static void          	SI_glLoadMatrixx(MM_Object* pMMObj, const int* m);
	static void          	SI_glMatrixMode(MM_Object* pMMObj, unsigned int mode);
	static void          	SI_glMultMatrixx(MM_Object* pMMObj, const int* m);
	static void          	SI_glOrthox(MM_Object* pMMObj, int left, int right, int bottom, int top, int zNear, int zFar);
	static void          	SI_glPixelStorei(MM_Object* pMMObj, unsigned int pname, int param);
	static void          	SI_glPolygonOffsetx(MM_Object* pMMObj, int factor, int units);
	static void          	SI_glPopMatrix(MM_Object* pMMObj);
	static void          	SI_glPushMatrix(MM_Object* pMMObj);
	static void          	SI_glScalex(MM_Object* pMMObj, int x, int y, int z);
	static void          	SI_glScissor(MM_Object* pMMObj, int x, int y, int width, int height);
	static void          	SI_glShadeModel(MM_Object* pMMObj, unsigned int mode);
	static void          	SI_glTexCoordPointer(MM_Object* pMMObj, int size, unsigned int type, int stride, const void* pointer);
	static void          	SI_glTexEnvx(MM_Object* pMMObj, unsigned int target, unsigned int pname, int param);
	static void          	SI_glTexEnvxv(MM_Object* pMMObj, unsigned int target, unsigned int pname, const int* params);
	static void          	SI_glTexImage2D(MM_Object* pMMObj, unsigned int target, int level, int internalformat, int width, int height, int border, unsigned int format, unsigned int type, const void* pixels);
	static void          	SI_glTexParameterx(MM_Object* pMMObj, unsigned int target, unsigned int pname, int param);
	static void          	SI_glTexSubImage2D(MM_Object* pMMObj, unsigned int target, int level, int xoffset, int yoffset, int width, int height, unsigned int format, unsigned int type, const void* pixels);
	static void          	SI_glTranslatex(MM_Object* pMMObj, int x, int y, int z);
	static void          	SI_glVertexPointer(MM_Object* pMMObj, int size, unsigned int type, int stride, const void* pointer);
	static void          	SI_glViewport(MM_Object* pMMObj, int x, int y, int width, int height);
	
#endif /* defined(FEATURE_ACHW) && (ACHW_DRAWING_LIB == ACHW_DRAWING_LIB_IS_OPENGL_ES) */

#ifdef FEATURE_TESTAUTOMATION
    static signed long SI_WriteFile (MM_Object* pMMObj, const char* filePath, char* platformFileURL, int platformFileURLSize, const char* pBuf, FI_FileMode mode, int noOfChars);
    static signed long SI_GenerateMouseEvent(MM_Object* pMMObj, FI_TestAutomationMouseEventType action, int screenX, int screenY, int delta);
    static signed long SI_GenerateKeyEvent(MM_Object* pMMObj, FI_TestAutomationKeyEventType action, unsigned int keyCode);
    static signed long SI_GetAutomationConfigurationContent( MM_Object* pMMObj, char** buffer, int* bufferSize, FI_ConfigurationFileType type);
#endif//FEATURE_TESTAUTOMATION

    static void SI_GetDisplayRectCoord(struct MM_Object *pMMObj, struct FI_Rect *rect);
    
//#ifdef FEATURE_MULTITOUCH
    static int SI_GetMultiTouchCapabilities(struct MM_Object* pMMObj, int *maxTouchPoints);

    static void SI_GetSupportedGestures(struct MM_Object* pMMObj, FI_GestureID glist[] , int *len);
    
    static short SI_EnableMultiTouch(struct MM_Object* pMMObj, short enable);

//#endif
#ifdef FEATURE_AMCEXTENSIONS
    static int SI_GetExtensionPlatformInfo(MM_Object *, ExtensionPlatformInfo*);
    static int SI_GetSecurityContextPermissions(struct MM_Object *pMMObj, ExtensionSecurityPermissions *permissions);
#endif

};

#endif _SI_INTERFACE_INCLUDE */
