/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class which implements the
*                main hostengine functionality.
*
*/


#ifndef SYMBIAN_ACCELEROMETER_H_
#define SYMBIAN_ACCELEROMETER_H_

#include <sensrvtypes.h>
#include <sensrvchannel.h>
#include <sensrvdatalistener.h>

#include "AMBasicTypes.h"
#include "IAMAccelerometer.h"
#include "symbian_engine.h"
#include "mamcextensioncallback.h"

class AccelerometerCancel;

class AccelerometerImpl : public IAMAccelerometer, MSensrvDataListener, MAMCEXtensionCallback
    {
public:
    static AMErr Start(IAMAccelerometer* pAccel, AMAccelerometerStartCallback *pCallback, void*
            pUserData, IAMAccelerometerStart** ppStart);
    
    static void SetUpdateInterval(IAMAccelerometer* pAccel, S32 timer);    
    static AMErr Create(IAMAllocator* pAllocator, IAMAccelerometer** pAccel);
    static U32 AddRef(IAMAccelerometer* pAccel);
    static U32 Release(IAMAccelerometer* pAccel); 
    
    ~AccelerometerImpl();
    
    void StartImplL(AMAccelerometerStartCallback *pCallback, void*
                pUserData, IAMAccelerometerStart** ppStart);
        
    
    void DataReceived(CSensrvChannel& aChannel, TInt aCount, TInt aDataLost);
    void DataError(CSensrvChannel& aChannel, TSensrvErrorSeverity aError );
    void GetDataListenerInterfaceL(TUid aInterfaceUid, TAny*& aInterface);
    
    // From MAMCExtensionCallback
    void PlayerStateChanged(TPlayerState2 aPlayerState);

private:
    IAMAllocator* m_pAllocator;

    AccelerometerImpl(); //: m_refCount(0) /*, m_pAccelerometerEngine(0)*/ {}
    
    inline S32 GetElapsedTime(S32);
    void GetChannelDataRateInfoL();
    TInt GetDataRateValueIndex(S32 aTimer);
    void SetUpdateIntervalImpl(IAMAccelerometer* pAccel, S32 timer); 
    TReal GetScaleFactor();
    
    AMErr Init();

public:
    S32     m_refCount;
private:   
    CSensrvChannel* iChannel;    
    AMAccelerometerStartCallback* iCallback;
    void* ipUserData;
    S32   iStartTime;
    S32   iLastUpdateTime;
    S32   iUpdateInterval;
    TReal iScaleFactor;
    TInt iChannelDataRateWriteIndex;
    RArray<TInt> iChannelDataRatesIndexes;
    RArray<TInt> iChannelDataRatesValues;
    
private:
        friend class AccelerometerCancel;
    
private:
    AccelerometerCancel* iCancel;
    };

class AccelerometerCancel : public IAMAccelerometerStart
        {
        public:
            AccelerometerCancel(AccelerometerImpl& ,IAMAllocator*);
            ~AccelerometerCancel();

            static U32 AddRef(IAMAccelerometerStart* pAccelStart);
            
            static U32 Release(IAMAccelerometerStart* pAccelStart);
            
        public:
            AccelerometerCancel(AccelerometerImpl& aImpl);
            
            void Init();
                
            
            void AddRef();
            void Release();
            TBool IsPlayerListening();
            
            AccelerometerImpl& iImpl;
            IAMAllocator* m_pAllocator;
        private:
            TBool iPlayerListening;
        };


#endif /* SYMBIAN_ACCELEROMETER_H_ */
