/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the CPlatformFlashEngine class, which implements the 
*				 main hostengine functionality.
*
*/


#ifndef __SYMBIAN_ENGINE_H__
#define __SYMBIAN_ENGINE_H__

#include <e32property.h> // for RProperty
#include <coedef.h>
#include <w32std.h>
#include <eikenv.h>
#include <caf/content.h>
#include "saplayer_build.h"
#include "commondef.h"
#include <badesca.h>
#include "flashextension.h"
#include "fi_interface.h"
#include "si_interface.h"
#include "flash_engine.h"
#include "timer_object.h"
#include "engine_defs.h"
#include "symbian_truststore.h"
#include "browserurlrequestinfo.h"
#include "symbian_mso.h"
#include <aknnotewrappers.h>
#include <avkon.hrh>
#include <StringLoader.h>
#ifdef ENABLE_CURSOR
#include "flashcursor.h"
#endif

#include "IMemMgr.h"
#include "saflash_coredefs.h"
#include "platformtypes.h"
#include "async_object.h"
#include "symbian_sharedobjects.h"
#include "symbian_bklighthandler.h"

#ifdef FEATURE_PACK_3
#include "symbian_networkinfo.h"
#include "symbian_subscriberinfo.h"
#include "symbian_registrationstatus.h"
#include "symbian_phoneinfo.h"
#include "symbian_batteryinfo.h"
#include "symbian_signalinfo.h"
#include "symbian_charginginfo.h"
#endif // FEATURE_PACK_3

using namespace ContentAccess;

#ifdef FEATURE_WEB_BROWSABILITY
	#include "hostscripthandler.h"
	#include "symbian_hostscript_handler.h"
#endif	

#ifdef FEATURE_ATS_AUTOMATION
#include "atshandler.h"
#endif

#ifdef FEATURE_EXTENSION
#include "flashengineintftoasextension.h"
#endif
#ifdef FEATURE_VIDEO_DLL
#include "FL_ReferenceCountedObject.h"
#endif

#include "helixdef.h"

// HostEngineGlobals
#include "hostengineglobals.h"

#ifdef FEATURE_OPENVG
#include <VG/openvg.h>
#include <EGL/egltypes.h>
#include <EGL/egl.h>
#endif

#include "appinfoif.h"
#include "deviceextnif.h"
#include "flashengineclientif.h"
#include "flashvideoif.h"
#include "inputtexthandlerif.h"
#include "pluginmemhandlerif.h"
#include "pluginstreamif.h"
#include "uinotifierif.h"
#include "screenfurnitureif.h"

#ifdef FEATURE_WATCH_GPU_MEM
#include "gpumemwatch.h"
#endif

const TUint KNetworkPromptTimeOut = 10*1000*1000;   // 10 seconds

// FORCE_REFRESH_WORKAROUND -> required for certain static contents
#define FORCE_REFRESH_WORKAROUND

// Forward Declarations
class CFLPlatformDecoderInterface;
class CVibrationHandler;
class CCnvCharacterSetConverter;
class CQueuedSILaunch;
class CPlatformConfig;
class CWsBitmap;
//class MFlashUI;
//class MFlashPlugin;
class CSoundHandler;
class CFlashExtensionWrapper;
class CPhoneMonitor;
class CImageConverter;
class CFileEngine;
class CHttpEngine;
class CLocalHttpEngine;
class CVideoEngine;
class CFlhShObjEngine;
class CFlashSocketEngine;
class CFlhAsyncEvent;
struct TGetInputTextData;
class CFlhFileUtils;
class CFlhItemValuePairList;
class CFlhAsyncIPSelectDlg;
class CImageConverter;
class CPlatformInfo;
class CBrowserURLRequestInfo;

#ifndef USE_SYNC_MSO
class CFlhShObjEngine;
#else
class CSharedObjectHandler;
#endif //USE_SYNC_MSO

class CIPConnMgr;
#ifdef SELF_IAD_UPDATE
class CIADUpdateChecker;
#endif
#ifdef FEATURE_PACK_3
class CNetworkInfo;
#endif // FEATURE_PACK_3

#ifdef FEATURE_WEB_BROWSABILITY
class CHostScriptHandler;
#endif

#ifdef FEATURE_INLINE_TEXT_ENTRY
class AMP_VirtualTextNavigation;
class AMP_MTextNavigation;
#endif

class MPluginMemHandlerIf;
class MUserNotiferIf;
class MPluginStreamIf;
class MDeviceExtnIf;
class MFlashClientAppInfo;
class MDisplay;
class MFlashVideoClientIf;

class CPhoneInfo;
class CNetworkInfo;
class CSubscriberInfo;
class CRegistrationStatus;

class CPhoneBatteryInfo;
class CPhoneSignalInfo;
class CPhoneChargingInfo;
class CPhoneMonitor;
class MAMCEXtensionCallback;
    
#ifdef SELF_IAD_UPDATE
class  MIadUpdateCallback;
#endif
    
#if defined(FEATURE_OPENVG) && defined(USE_VPU2)
class CAlfCompositionSource;
#endif
class CSharedMemoryManager;

//These constants are defined in corebuild.h
#define MAX_MEM_ALLOCATOR_OVERHEAD      48
const unsigned int TELESTREAM_CYCLING_BUFFER_SIZE = ((128 * 1024) - MAX_MEM_ALLOCATOR_OVERHEAD);
const unsigned int TELESTREAM_CYCLING_BUFFER_COUNT = 8;
const unsigned int SOUNDSTREAM_CYCLING_BUFFER_SIZE = ((16 * 1024)- MAX_MEM_ALLOCATOR_OVERHEAD);
const unsigned int SOUNDSTREAM_CYCLING_BUFFER_COUNT = 8;

#ifdef __DEBUG_LOGS__
#define DEBUGPRINT(args...) RDebug::Printf(##args)
#define DEBUGPRINT_FBEGIN RDebug::Printf("CGPUMemWatch::%s ++",__FUNCTION__);
#define DEBUGPRINT_FEND   RDebug::Printf("CGPUMemWatch::%s --",__FUNCTION__);
#else
#define DEBUGPRINT(args...) 
#define DEBUGPRINT_FBEGIN 
#define DEBUGPRINT_FEND
#endif

//enum declaration for NetStream Status 
enum EFLVStatus
{
	ENetStreamNull = 0,
	ENetStreamBufferEmpty,
	ENetStreamBufferFull,
	ENetStreamBufferFlush,
	ENetStreamPlayStart,
	ENetStreamPlayStop,
	ENetStreamPlayPause,
	ENetStreamPlayResume,
};

enum TScaleValues
{
	EScaleShowAll = 0,
	EScaleNoBorder,
	EScaleExactFit,
	ENoScale
};

enum TAlignValues
{
	EAlignLeft 	 = 0x1,
	EAlignRight  = 0x2,
	EAlignTop	 = 0x4,
	EAlignBottom = 0x8
};

enum TBacklightStatus
{
    EOff,
    EOn
};
enum TPlayerState2
{
    ePlayerUnknown = -1,
    ePlayerPause,
    ePlayerResume,
    ePlayerStop
};

/*
 * States
 * EIMENotInitialized -> EIMEEnable -> EIMEEnabled -> EIMEInitialized -> EIMEDisable -> EIMEDisabled -|
 * ^                                                                                                  |
 * |                                                                                                  |
 * |--------------------------------------------------------------------------------------------------|
 *  
 */
enum TIMEState
{
    EIMEEnable,
    EIMEEnabled,
    EIMEDisable,
    //EIMEDisabled, // might be required if we decide to disable ime asynchronously
    EIMENotInitialized,
    EIMEInitialized
};

_LIT(KLitUrlProtocolFile,"file://");
_LIT(KLitUrlProtocolOpenFile,"openFile://");

// common protocols
_LIT(KProtocolPrefixVideoCall, "tel-av:");
_LIT(KProtocolPrefixTelCall, "tel:");
_LIT(KProtocolPrefixEmail, "mailto:");
_LIT(KProtocolPrefixSMS, "sms:");
_LIT(KProtocolPrefixMMS, "mms:");
_LIT(KProtocolPrefixFile, "file://");
_LIT(KProtocolPrefixFileUnixStyle, "file:///");
_LIT8(KProtocolPrefixHttp, "http://");
_LIT8(KProtocolPrefixHttps, "https://");

// Extended rendering protocols
_LIT(KProtocolPrefixExtendedRenderingMyData, "mydata://");
_LIT(KProtocolPrefixExtendedRenderingSystem, "system://");

const U8 KDefaultBackgroundRed 					= 0xFF;
const U8 KDefaultBackgroundGreen 				= 0xFF;
const U8 KDefaultBackgroundBlue 				= 0xFF;
const U8 KDefaultBackgroundTransparency 		= 0xFF;
const TInt KCenRepVibraFeatureSupport 			= 0x00000020;
const unsigned short KUrlProtocolFileLength 	= 8;
const unsigned short KUrlProtocolOpenFileLength = 11 ;
const short KLoopOn 							= 1;
const short KLoopOff 							= 0;
const short KQualityLow 						= 0;
const short KQualityMedium 						= 1;
const short KQualityHigh 						= 2;
const short	KDefaultQuality 					= KQualityMedium;
const short	KDefaultLoop 						= KLoopOn;


// TODO: These were copied from geom.h... they should be replaced with TRect, TPoint, and TSize
typedef S32 SCOORD, *P_SCOORD;

typedef struct SPOINT {
	SCOORD x;
	SCOORD y;
} SPOINT, *P_SPOINT;

typedef const SPOINT* CONSTP_SPOINT;

// Note that a rectangle is considered empty if xmin == rectEmptyFlag
// Flash defines the max coordinates as exclusive. So width = xmax - xmin.
//
struct SRECT {
	//mwang: since SRECT is used in a union, constructor is not allowed, so a init() function will be used to replace the constructor
	void Init(){xmin=xmax=ymin=ymax=0;}
	SCOORD xmin;
	SCOORD xmax;
	SCOORD ymin;
	SCOORD ymax;
}; 

typedef struct SRECT SRECT;
typedef struct SRECT *P_SRECT;
typedef const  SRECT *CONSTP_SRECT;

extern void ConvertEpocStringToCStr(TDesC& aEstr, char* aStr);

/**
 * The DynamicMemBlock struct is used to keep track of blocks of memory requested by the Flash library
 * through calls to SI_Malloc() and freed by calls to SI_Free().
 * 
 */
struct DynamicMemBlock
{
	TAny* mem;
	unsigned long numBlocks;
	DynamicMemBlock* pNext;
};


/**
 * The DynamicMemBlock struct is used to keep track of blocks of memory requested by the Flash library
 * through calls to SI_Malloc() and freed by calls to SI_Free().
 *
 */
/*struct DynamicMemBlock
{
	TAny* mem;
	unsigned long numBlocks;
	DynamicMemBlock* pNext;
};*/
struct THttpRequestInfo
	{
	HBufC* iUrl;
	TBool iIsFired;	
	unsigned short iStreamId;
	HBufC* iPost;
	};

enum TVideoType
{
	EVideoNone 	 = 0,
	EDeviceVideo = 1,
	ENativeVideo = 2
};

class CFontMapTable: public CBase
{
public:
	CFontMapTable(CPlatformFlashEngine* aEngine);
	~CFontMapTable();
	
	TInt AddNewFontEntryL(CFont* aFont, const FI_FontInfo* aFontInfo);
	void RemoveFontEntry(TInt aIndex);
	
	CFont* GetFont(TInt aFontId);
	CFont* GetFontEntryFromFontId(TInt aFontId);
	
	void ReleaseAllFonts(CFbsBitmapDevice* aBitmapDevice);
	
private: 
	class TFontIdFontInfo
    {
    public:
    	TFontIdFontInfo(const TInt aFontId, CFont* aFont, FI_FontInfo* aFontInfo ) 
    	{
            iFontId		= aFontId;
            iFont		= aFont;
            iFontInfo   = aFontInfo;
        }

        TInt 		 iFontId;
        CFont* 		 iFont;        
        FI_FontInfo* iFontInfo;
    };
    

	CPlatformFlashEngine* 	iEngine;
	RArray<TFontIdFontInfo> iFontIdFontInfoMapTable;
    TInt 					iFontId;
};

class TVideoRect
{
public: 
	TRect 		iRect;
	TVideoType 	iVideoType;
	
public:
	TVideoRect() { iRect.SetSize(TSize(0, 0)); iVideoType = EDeviceVideo; }
};

struct TInitInfo
	{
public:
	TRect iRect;
	TBool iResize;
	TBool iLandscape;
	};


/*
 * ==============================================================================
 * The CPlatformFlashEngine class controls an instance of the Flash Player. It handles most of the
 * platform specific functions required by the Flash library (called by SI functions) and is
 * responsible for creating and controlling the instance of the Flash Player through FI functions.
 * ==============================================================================
 */

class CPlatformFlashEngine : public CBase
							,public MFlashEngine
							,public MFlhAsyncHandler
#ifndef USE_SYNC_MSO
							,public MFlhShObjEngQueueNofifiee
#endif
							,public IMemMgrFunctions
#ifdef FEATURE_EXTENSION
							,public MFlashEngineIntfToASExtension
#endif
#ifdef FEATURE_WATCH_GPU_MEM
                            ,public MGPUMemStatusNotifier
#endif						
#ifdef FEATURE_OPENVG                            
                            ,public MCoeForegroundObserver
#endif                            
{
private:
    class CQueuedSILaunch : public CBase
    {
    	public:
    	static CQueuedSILaunch* NewL(struct MM_Object* aMMObj, 
    								 TUint16 aArgc, const TDesC** aArgv, 
    								 TInt16 aIsEmbedded);
    	~CQueuedSILaunch();
    	void LaunchL();
    	    	
    	private:
    	void ConstructL(const TDesC** aArgv);
    	CQueuedSILaunch(struct MM_Object* aMMObj, TUint16 aArgc, TInt16 aIsEmbedded);

    	public:
    	TSglQueLink iLink;
    	
    	private:
    	struct MM_Object* 	ipMMObj;
    	HBufC** 			iArgv;
    	TUint16 			iArgc;
    	TInt16 				iIsEmbedded;
    	FI_PlayerEventInfo* iEventInfo;
    }; 
       
public:
	FlashEngineConfig GetConfig();
	CPlatformFlashEngine();
	virtual ~CPlatformFlashEngine();
	
/* ================== From MFlashEngine =========================== */
public:
	/*void 		ConstructL(MFlashUI* aFlashUI, FlashEngineConfig& aConfig
#ifdef SELF_IAD_UPDATE
	                        , MIadUpdateCallback* aIADUpdateCompleteCallback
#endif
	                      );*/
	//Initilize the flash engine with exection mode ie either standalone or plugin
	//If the execution mode is "plugin", the engine should be initialized with variants of plugin execution mode.
	//Currently default parameter is added to keep the interface consistent.
	//TODO: Need to integrate these two types of enums into a single enum during code refactoring.
	virtual void ConstructL(FlashEngineConfig& aConfig,
	            MFlashClientIf* aClientInterfaces,
	            EEngineExecutionMode eExecutionMode,
	            EBrPluginExecutionMode eBrPluginMode = EDefaultMode 
	            );
	
#ifdef FEATURE_WINDOWLESS	
	virtual void SetWlessSurface(CFbsBitmap* aBitmap) 
	    {
	    iWlessSurface = aBitmap;
	    }
#endif //FEATURE_WINDOWLESS
	
	/*
    MFlashPlugin* GetFlashPlugin()
        {
        return this->
        }
*/
    /*MFlashUI* GetFlashUI()
        {
        return iFlashUI;
        }*/
    
	MFlashClientIf* GetClientIf()
        {
        return iFlashClientIf;
        }

    MDisplay* GetDisplay()
        {
        if (iFlashClientIf)
            return iFlashClientIf->GetDisplay();

        return NULL;
        }

    MUserNotiferIf* GetUserNotifierIf()
        {
        if(iFlashClientIf)
            return iFlashClientIf->GetUserNotifierIf();
        
        return NULL;
        }
    
    MDeviceExtnIf* GetDeviceIf()
        {
        if(iFlashClientIf)
            return iFlashClientIf->GetDeviceIf();
        return NULL;
        }  
    
    MPluginMemHandlerIf* GetPluginMemHandlerIf()
    {
    	if(iFlashClientIf)
    		return iFlashClientIf->GetPluginMemHdlrIf();
    	return NULL;
    }
    
	//Returns the background colour of the swf
	TRgb        GetBgcolour();
	void 		Destroy();
	void        InitializeL(const TRect& aRect, TBool aResize, TBool aLandscape);
	//void 		InitializeStandaloneL(const TRect& aRect, TBool aResize, TBool aLandscape = EFalse);
	//void 		InitializePluginL(MFlashPlugin* aControl, const TRect& aRect, TBool aResize, TBool aLandscape = EFalse);
	
	//Not used
	//void        AddMp3StateObserver(MMp3StateObserver* aObserver);
	
	void 		ProcessParameters(CDesCArray* argnames, CDesCArray* argvalues);
	void 		DestroyPlayer();
	void		DestroyStreamBuffers();//Added as part of HIIN-857FWY
	void 		PlayStep(TBool aCanInterrupt);
	
	void 		OpenFileAndStartMovieL	(const TDesC& aFilename, 
										 TBool aResetBaseURL = EFalse, 
										 const TBool& aFirstMovie = ETrue, 
										 CContent* aContent = NULL );
	void 		OpenFileHandleAndStartMovieL(RFile& aFile, 
										 TBool aResetBaseURL = EFalse, 
										 TBool aResetIAPAccess = ETrue, 
										 const TBool& aFirstMovie = ETrue, 
										 CContent* aContent = NULL );
	void 		OpenBrowserFileAndStartMovieL(const TDesC& aFilename,
										 TBool aResetBaseURL,
										 const TDesC* aBaseUrl, 
										 const TBool& aFirstMovie = ETrue,
										 CContent* aContent = NULL );
	void SetAllowScriptAccess(TInt aScriptValue)
		{
		iAllowScriptAccess = aScriptValue;
		}

	void 		NewURLStreamL(const char* aFinalURL, TUint aStreamId, TInt aContentLength, TDesC8& aContentType);
	void 		SetNetworkBufferL(unsigned short aStreamId, struct FI_NetworkBufferInfo* aNetBuf);															
   	TInt 		WriteStreamDataL(unsigned long streamId, const char* data, unsigned long dataLength);
   	void 		CloseStream(TUint16 aStreamId, TInt16 aClosedStatusCode, EMediaStreamType aMediaStreamType=ENotMediaStream);
   	void        StreamClosed(TUint16 aStreamId, TInt16 aClosedStatusCode);
   	void 		CancelURLStream(unsigned short aStreamId);

   	//Not required
   	//void        SetExecutionMode(EInteractionMode aExecutionMode);
    void 		RecreateBitmapL(const TRect& aRect);
    void        PauseForNetworkPrompt();   // when the engine is forced to pause by itself and must notify the controls
	void 		Pause(TInt aReason);
    void        ResumeOnNetworkPrompt();   // when the engine is forced to resume by itself and must notify the controls
	void 		ResumeL(TInt aReason, TBool aIgnoreActiveView);
	void 		RewindL();
	TBool 		Zoom(TInt aMode, TInt& aCurrentZoomLevel);
	void 		Stop();
	void 		AbortPlayer();
	void 		PauseVideo();
	void        ResumeVideo();
	void 		PlayVideo();
	
	void 		SetInputTextL(const TDesC& aNewStr);
	void 		CancelInputText();
		
	void 		Draw(const TRect& aDirtyRect, CWindowGc& aWGc);
	void 		RefreshScreen();
   	void 		NotifyFocusChangeL(TBool isActive);
	void 		UpdateVideoScreen();
	
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
	void 		 OfferPointerEventL(TPoint& aPoint, TInt aType);
	void 		 HandleSoftKeyLocationChangedL();
	void 		 SendPointertoSoftKey(TInt);
    
	void 		SetQuality(TInt aQuality);
	TInt 		GetQuality();
	void 		SetVolume(TInt aVolume);
	TInt 		GetVolume();
	HBufC* 		GetFlashVersion();
	void 		SetPanMode(TBool aPanMode) { iPanMode = aPanMode; }
	TInt 		GetCurrentFrameNumber() ;
	TInt 		GetTotalFrameNumber();	
    CFbsBitGc* 	GetBitmapContext();
    CFbsBitGc*  GetTextBitmapContext();    
    
    void 		SetBaseUrlL(const TDesC8* aBaseUrl);
    void        SetSecurityContextUrl(const TDesC8* aBaseUrl);
	void 		SetVideoDisplayRectL(const TRect& aVideoRect, const TRect& aClipRect);
	void        SetEglContainerWindow(RWindow* aWindow);
	void 		SetVideoWindow(RWindow* aWindow);
	TRect 		GetVideoRectL();
	TInt32 		GetFLVMaxDynHeapSize();	
	void 		SetForeground(TBool aForeground);
	
	HBufC* 		CreateEpocStringL(const char* aStr, int aLength);
	TUint32 	ConvertFromFloatToFixed16(TReal aFloatRep);
	//TInt  		StartShutdownSequenceReturnStepsToGo();
	void 		OffsetPointToPan(TPoint& aPoint, TInt aType);
	void		HandleHelixError(const TInt& aError);
	
   	TBool 		IsVibrationSupported() 	{ return iVibraFeatureSupported; }
    //TBool 		IsBrowserPluginActive()   	{ return (iControl != NULL); }
   	
   	EEngineExecutionMode GetExecutionMode() const 
   	{ return iExecutionMode; }

    /* The below method indicates whether the plugin is playing flv video or swf
   	   return True -> playing video , False -> Playing swf */ 
   	TBool GetFlvPlayState() const
   	{ return iFLVPlayStat; }
   	
	EBrPluginExecutionMode GetBrPluginMode() const
   	    {
   	    return iBrPluginMode;
   	    }
   	    
   	TBool       IsBrowserPluginActive()     
   	    {
   	    if(ePlugin==iExecutionMode)
   	        return ETrue;
   	    else
   	     return EFalse;
   	    }
   	
    TBool 		PlayerStoppedWithError()	{ return iPlayerState == EPlayerStoppedError; }
	TBool 		IsInitialized() 			{ return iIsInitialized; }
	TBool 		IsRunning() 				{ return iIsRunning; }
	TBool 		IsPaused() 					{ return iPauseFlags!=ENotPaused; }
	TBool 		IsZoomed() 					{ return (iZoomFactor > 0); }
	TBool 		IsPanning() 				{ return iPanMode; }
	TBool 		ControlPresent()			{ return iControlPresent; }
    
	TBool 		IsSoundPlaying() const;
	TBool 		IsZoomable();
	TBool 		IsHelixFlvMode();
    TBool 		IsControlPresent(const TPoint& aPoint);
    
    TBool       IsTouchSupported();
	
	#ifdef FF_FLASHLITE_SCREENSAVER 
    TInt CheckFileValidityL(const TDesC& aFilename);
	#endif //FF_FLASHLITE_SCREENSAVER 
	
	#ifdef FEATURE_INLINE_TEXT_ENTRY
	AMP_MTextNavigation* GetTextNavigation( );
	virtual void RestoreSoftKeys();
	#endif // FEATURE_INLINE_TEXT_ENTRY
	
	#ifdef FEATURE_WEB_BROWSABILITY
	virtual MHostScriptHandler* GetScriptHandler();
	#endif // FEATURE_WEB_BROWSABILITY
	
	#ifdef ENABLE_CURSOR
    void DrawCursor();    
    void ClearCursor(const TRect& aRect);
	#endif  // ENABLE_CURSOR

#ifdef FEATURE_AMCEXTENSIONS
private:
    void NotifyExt(TPlayerState2 aPlayerState);
public:
    void RegisterExtensionCallback(MAMCEXtensionCallback* aAMCClient);
    void DeregisterExtensionCallback(MAMCEXtensionCallback* aAMCClient);
    TBool AllowAMCExtension();
#endif

/* ================== From MFlhAsyncHandler =========================== */
private:
	void DoRunL(const CActive* aActive);
	TInt DoRunError(const CActive* aActive, TInt aError);

/* ================== From MFlhShObjEngQueueNofifiee =========================== */
#ifndef USE_SYNC_MSO	
private: 
	void NotifyNumberOfItemsInQueue(TInt aNumItemsRemaining);
#endif //USE_SYNC_MSO
	
/* ================== From IMemMgrFunctions =========================== */
public:
	virtual void  AddDynHeapAlloced(TUint size);
	virtual void  DynHeapFreed(TUint size);
public:
 #ifdef FEATURE_AMCEXTENSIONS
    static int      LoadPluginResourceL(const TDesC&);
    static TBool    QueryAllowDenyL(const TDesC&);
	#endif

// Other public methods
public:
	CIPConnMgr* GetIPConnectionMgr();
    TBool  IsContentTrustedL(const TPtrC &aSwfFileName);
    void NewMP3StreamStarted();
    void MP3StreamClosedL();
    TBool IsMp3Stream();
    void             TrustContentL(const TPtrC &aSwfFileName);
 	CEikonEnv* 			GetEikonEnv()			{ return iEnv;}
    RFs& 				GetRFs() 				{ return iFs; }	
   	CFlhFileUtils* 		GetFileUtils()  		{ return iFileUtils;}
	CFileEngine*   		GetFileEngine(); // 		{ return iFileEngine;}
	CHttpEngine*   		GetHttpEngine();
	CLocalHttpEngine*	GetLocalHttpEngine();
	CVideoEngine*		GetVideoEngine(); 		//{ return iVideoEngine; }
#ifndef USE_SYNC_MSO
	CFlhShObjEngine*	GetShObjEngine();
#else
	CSharedObjectHandler* GetShObjHandler();
#endif //USE_SYNC_MSO
	CImageConverter*	GetImageConverter(); 	//{ return iImageConverter;}
	struct MM_Object* 	GetMMObject() 			{ return iMMObject; }
	const SPOINT& 		GetBitmapSize() 		{ return iBitmapSize; }	
	CWsBitmap* 			GetBitmap() 			{ return iBitmap;}
	TUint16 			GetSoftkeyCount() 		{ return iConfig.iSoftkeyCount; }
	//Not used
	//TInt16 				GetRomVersion() 		{ return iRomVersion; }
	//Not used
	//HBufC*  			GetBaseURL() 			{ return iBaseURL; };
	TTime&				GetUTCReferenceTime() 	{ return iUtcReferenceTime; }
	CPlatformConfig* 	GetPlatformConfig()		{ return iPlatformConfig; }
	
	//MFlashUI* 			GetUI();
	//MFlashPlugin* 		GetPlugin();
	TInt32  			GetAvailableDynamicMemory();
	FI_Interface& 		GetFIObject(); 
	TUint   			GetMaxHeapSize();
	TUint32 			GetConnectedIAPIdL();
	TInt16 				GetDeviceName(TUint8* aCharBuffer, const TInt aMaxLen);
	TInt16 				GetDeviceID(TUint8* aCharBuffer, const TInt aMaxLen);
	TUint 				GetBatteryLevelL();
	TUint 				GetCallerInfoL(FI_ReturnString* aName,FI_ReturnString* aNumber);
	TBool 				GetBatterychargeStatusL();
	TInt 				GetSignalStrengthL();
	TInt16 				GetPlatformName(TUint8* aCharBuffer, const TInt aMaxLen);
	TRect   			GetSubDirtyRect(unsigned int index);
	unsigned short 		GetNumberOfSubDirtyRects();
	TBool 				GetCurrentFont(CFont*& aFont);
	TBool 				GetNetworkInfoL(unsigned short aInfoType, TDes8& aRetString);
	TBool 				GetNetworkInfoL(unsigned short aInfoType, long& aRetVal);
	TBool 				GetSubscriberInfoL(unsigned short aInfoType, TDes8& aRetString);
	TBool 				GetRegistrationStatusL(unsigned short aInfoType, long& aRetValue);
	short 				GetSoftKeyLocation() const;
	void 				GetTime(TInt aItem, char* aValue);
	void 				GetPlatformNameL(HBufC*& aPlatformName);
	TInt 				GetIntFromDesc(const TDesC& aDesc);
	void 				GetInputTextL(const TUint16* aInitialString, TUint aStrLen, TUint aMaxNumChars, TUint aTextFlags);
	RWindow* 			GetWindow();
	TRect               GetViewRectangle();

	void 				SetSoftKeyLocation();
	void 				SetVideoRectL(const FI_Rect* aRect, const TVideoType& aVideoType);
	unsigned short		SetCurrentFont(void* aFontId);
	void 				SetSoftKeys(HBufC** aKeyStrings, TInt aKeyCount);
    void 				SetFLVStatus(const char* code);
    
   	TBool 				IsPhoneCallActive();
   	TBool 				IsStopped() { return iIsStopped; };
	TBool 				IsBitmapLocked() { return iIsBitmapLocked; }
#ifdef FEATURE_OPENVG	
    TBool               IsAuxTextBitmapLocked() { return iIsAuxTextBitmapLocked; }
    CFbsBitGc*          GetAuxTextBitmapContext();    
#endif    
    TBool 				IsStereoSupported();
	TBool 				IsForegroundTask();	
	TBool 				CanRewind();	
	TBool 				NoGetURLSupport();
	TBool 				NoVibraSupport();
	TBool 				NoSoundSupport();
	TBool 				NoBackLightProcessing();
	TBool 				NoIncomingCallMonitor();
	TBool 				NoETelSuppport();
	TBool 				ConnectionPromptNeeded();
	TBool               HasQuertyKBSuppport();
	TBool 				InstanceInTrustedSandBox();
	TBool 				IsLandScapeMode(); 
   	    
   	void 				DrawVideoRect();   	
	void 				ChangeScreen(TInt aFormat); 
#ifdef __FEATURE_OSK__
	void                SetCompatibilityMode(TBool aIsOnOrOff); 
#endif  // __FEATURE_OSK__
	void 				ResetSoftKeys();	
	void 				ForceRefresh();
   	TBool 				Zoom(TReal aScaleFactor);
	TInt 				ExtendBacklight(TUint16 aDuration);
	void 				ExtendBacklightForVideoPlayback(TBacklightStatus aOnOrOff);
	void				ExtendBackLightWhileSeekingLocalFlv();
	
    TBool 				QueryAllowNetAccessL();
	void 				QueueSILaunchL(struct MM_Object* aMMObj, TUint16 aArgc, const TDesC** aArgv, struct FI_PlayerEventInfo* aEventInfo, TInt16 aIsEmbedded);
	TBool 				QueryAllowPhoneCallL(const TDesC& aNumberString);

	void 				CreateCharacterConverterL ( );
	void 				ChangeCodePageL(TUint aCodePage);
	inline TUint 		CurrentCodePage() const { return iCurrentCodePage; }
	TBool 				ImageDownScalingRequiredL(const TDesC& aFileName);
	HBufC* 				CreateEpocString(const char* aStr, int aLength);
	HBufC* 				CreateEpocStringLC(const char* aStr, int aLength);
	char* 				CreateFlashString(const TDesC& aStr);
	char* 				CreateFlashStringL(const TDesC& aStr);
	char* 				CreateFlashStringLC(const TDesC& aStr);
	void  				CreateFlashStringL( char* buf,const TDesC& aStr );
	void  				CreateFlashString(char* buf, const TDesC& aStr );
	TPtrC 				MatchedStringFromArrayL(TInt aArrayResId, const TDesC& aMatch);
	unsigned short		GetDeviceFont(const struct FI_FontInfo * aFontInfo, TInt* aFontId);
	TBool 				CreateFont(const struct FI_FontInfo* aFontInfo, CFont*& aFont);
	unsigned short		DestroyFont(void* aFontId);
	void 				BeginTextDraw();
	void			 	EndTextDraw();
	//void 				DrawTextL(const struct FI_Text* aText, struct FI_TextInfo* aTextInfo);
	TBool               DrawTextL(const struct FI_Text* aText, struct FI_TextInfo* aTextInfo, void* aBits = NULL, TInt aWidth = 0, TInt aHeight = 0);//Return FALSE if text has not been rendered on screen
	BOOL 				LinebreakL(const struct FI_Text *aText, struct FI_DeviceTextInfo* aDeviceTextInfo);
	HBufC* 				ConvertToUnicodeLC(const TDesC8& aStr, TBool aIsUTF8);
	HBufC* 				ConvertEncodingLC(const TDesC8& aStr, TBool aIsEncoded, TBool aIsUTF8);
	void 				ConvertCStringToUTF16L(const char* aSrcCString, FI_ReturnString* aDstString);
	void 				ConvertUTF16ToCStringL(const FI_UTF16Char* aSrcUnicodeString, FI_ReturnString* aDstString);
	char* 				ConvertUTF16ToCStringL(const FI_UTF16Char* aSrcUnicodeString);

	TAny* 				LockFrameBufferBits();
	void 				UnlockFrameBufferBits(TAny* aBitmap = NULL);	
#ifdef FEATURE_OPENVG	
    void*               LockAuxTextBits(unsigned int* width,unsigned int* height,unsigned int* pitch,unsigned int* depth);	
    void                UnlockAuxTextBits(TAny* aBitmap = NULL);
#endif    
	void 				HandleAsyncEventsL();
	void 				TriggerAsyncEvent();
	TUint32 			MemBlockSize();
	TAny*   			AllocateDynamicMemory(TUint16 aNumBlocks);
	void    			FreeDynamicMemory(TAny* aMemory);
	TUint32 			DefaultIAPId() const;
	TBool 				PromptForIAP() const;
	void 				LaunchAppL(TFixedArray<TUid,5>* aUidsOfApp, TInt aNumOfUidsToTry, TInt aArgc, HBufC** aArgv);
    void 				LaunchAppL(const TDesC &aPath, const TDesC &aArgs);
	void 				StopPlaying();
	
	//Not used
	//void 				Exit();
	TInt16 				StartVibrate(TUint16 aTimeOn, TUint16 aTimeOff, TUint16 aRepeat);
	TInt16 				StopVibrate();	
	void 				WriteLogMsgL(const char* aMsg, const TBool aBShowElapsedTime=EFalse);
	void 				WriteLogMsgWithIntL(const char* aMsg, TInt aNum);
	void 				SeekFlv();
	TUserMode 			UserMode();

	unsigned short 		HandleURLL(struct MM_Object* aMMObj, struct FI_GetURLCallInfo* aUrlInfo);
	TUint16 			HandleTelL(const TDesC& aTelString);
	unsigned short 		HandleMailToL(TInt aMsgType, const TDesC& aMsgString, TBool aIsEmbedded);
	void 				HandleBrowserURLL(const TDesC& aUrl);
	void 				HandleFileURLL(const TDesC& aUrl, struct FI_GetURLCallInfo* aOriginalUrlInfo);
	unsigned short 		HandleRelativeUrlL(const TDesC& aUrl, struct FI_GetURLCallInfo* aOriginalUrlInfo);

	void 				NotifyFlashError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue);	
	void 				NotifyClientError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue);	
	void 				NotifyFlashStatus(TInt aFlashStatus, TAny* aParam);	
	void 				NotifyDataAccessError(const char* aUrl, TInt aErrorCode);

	unsigned short 		CreateBrowserStreamL( unsigned short streamId, struct FI_URLStreamInfo* pStreamInfo);
    short 				OpenURLStreamL(unsigned short streamId,
   							  struct FI_URLStreamInfo* pStreamInfo,
   							  struct MM_Object* aMMObj);
   	void 				LoadURLStreamBufferL(unsigned short streamId, struct FI_NetworkBufferInfo* pNetBuf);
	void 				AddToHeaderInfoL(FI_URLStreamHeaderInfo& aHeaderInfo, TInt aContentLength, TDesC8& aContentType);								
	void 				DestroyStream(unsigned short aStreamId);		

	static void 		CreateFilenameEnsurePath(RFs& aFs, TDes& aFileLocation, TBool aTemp);
	CSoundHandler* 		SoundHandler() { return iSoundHandler; };
	//Not Used
	//TBool 				ShowSharedObjectConfirmationQueryL();
	void 				UpdateFrameInterval(TUint32 aNewFrameInterval);
	char** 				AllocReadResourceArrayIntoFlashStringArrayL(TInt& aCount, TInt aResourceId);
	RProperty& 			PSProperty() { return iProperty; }
	void 				NotifyPhoneActiveL(TBool aIsActive);
    unsigned short 		SendtoBrowserPluginL( unsigned short streamId,
											const char* url,
											unsigned int aUrlLen,
                                            const char* aPostDataContentType,
                                            unsigned int aPostDataContentLen,                                                       
											const unsigned char* postData,
											unsigned int aPostDataLen);
	CFLPlatformDecoderInterface* FlvDecoder();	
	unsigned short 		QueueHttpRequestL(unsigned short streamId, HBufC* hUrl, HBufC* hPost=NULL);
	unsigned short 		FireNextRequestL();
	void 				RemoveCompletedRequest(TUint16 aStreamId);
	TInt 				NumOfRequestsFired();
	void 				RemoveAllRequest();
	
	#ifdef __FEATURE_FLV_RECOG_PLAY
	RFile* 				GetRFileHandleForPlay();
	#endif 
	
#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
	void 				FLVideoStarted(TInt32 aCodecType);
    void                FLVideoFinished();
#endif
	
    void				VideoIsEmbedded(TBool aEmbeddedVideo);
	#ifdef FEATURE_WEB_BROWSABILITY
	TDesC8& 			GetMovieId();
	#endif 
	
	#ifdef FEATURE_INLINE_TEXT_ENTRY
	virtual void 		DisableIME();
	virtual void 		EnableIMEL();
	void 				DrawFullScreenInputModeIndicator(const TRect& editingTextFieldBounds, TRect& indicatorBounds);
	#endif 
	
	#ifdef FEATURE_XML_SOCKET	
	CFlashSocketEngine* GetSocketEngine();
	#endif 		
	
	#ifdef FEATURE_FLV_MMC
	TBool 				SeekURLStreamFLV( unsigned short streamId,
					  			unsigned int seekTime,
					   			struct FI_NetworkBufferInfo* pNetBuf);
	TUint                GetBytesLoaded(TUint16 streamId);
	TUint                GetTotalBytes(TUint16 streamId);
	#endif 	  
	
	#ifdef FEATURE_VIDEO_DLL
	FL_ReferenceCountedObjectShare*	GetShareForCodec( U32 codecType );
	IVideoDecompressorInstance* 	GetNativeVideoDecompressor(MM_Object* aMMObj, void * aAllocator, U32 codecType);
	void 							DestroyVideoDecompressor(MM_Object * aMMObj, IVideoDecompressorInstance * decompressorInstance);
	void 							EngineDispatchHXCommand(const THXVideoDecCommand& aCommand,THXVDCmdArg& aArgInfo);
	#endif 

    void   SetFullScreenViaFSCommand(TBool aFullScreen);
    TBool  GetFullScreenViaFSCommand() { return iFullScreenViaFSCommand; }
    
    TBool IsOpenVgRenderingMode();
    TBool IsRenderingToBitmap();    
    
    void   SetSwfHasVideo(TBool aVideo);
    void   SetSwfHasAudio(TBool aAudio);
    TUint16 FrameBufferDisplayFormat();    
    void   DumpDirtyRects(TRect aDirtyRect); 
    void   ShowRedrawRegions(TRect aDirtyRect, CWindowGc& aWGc);    
    TInt   GetSwfHasVideo() { return iSwfHasVideo; }
    TInt   GetSwfHasAudio() { return iSwfHasAudio; }
    HBufC* GetSwfGuidLC();

#ifdef  FEATURE_BG_TRANSPARENCY
    short  RedrawBackground(struct FI_Rect* aRects, unsigned short aNumRects); 
#endif      
    
	#ifdef FEATURE_EXTENSION
	PluginInfo* GetPluginInfoL(struct MM_Object* aMMObj,struct ExtensionInterface* pEI);
	
	void ProcessHostCmdL(const TDesC& aCommandName, RASExtensionArgArray& argument, TInt& aReturnValue);
	#endif 

	
	
	//MUserNotiferIf* GetUserNotifier() const;
	MFlashClientAppInfo* GetAppInfoIf() const { return iFlashClientIf->GetClientAppInfoIf(); }
	
	CPlatformInfo* GetPlatformInfo();
	CPhoneInfo* GetPhoneInfo();
	CNetworkInfo* GetNetworkInfoL();
	CSubscriberInfo* GetSubscriberInfoL();
	CRegistrationStatus* GetRegistrationStatusL();
	
	CPhoneBatteryInfo* GetBatteryInfoL();
	CPhoneSignalInfo* GetPhoneSignalInfoL();
	CPhoneChargingInfo* GetPhoneChargingInfoL();
	CPhoneMonitor* GetPhoneMonitorL();
	
	TInt GetOTFont(unsigned char** aFont, TBool aBold);

	#ifdef  FEATURE_COLLECT_BITMAP  	
	CFbsBitmap* GetLastRenderedBitmap();
	TInt ToggleBrowserRendering(EBrPluginRenderingMode aMode);
    #endif
	
private:
	void 			DrawBitBuffer();
	void 			DrawClippingRect(CWindowGc& aWGc);
	void 			HandleRedraw();
	TRect 			GetDirtyRect();
	void 			DrawDirtyRegion(const TRect& aDirtyRect);
	void    		BitBltExcludingVideoRect(CWindowGc& aWGc, const TRect& aDirtyRect);	
	void  			ClearBitmap();
    TBool           CreateBitsL(TSize aSize);	
	TBool 			CreateFrameBufferBitsL(TSize aSize);
	TBool 			FrameBufferBitsValid();
	static TInt 	BitsFormat(int aEpocDeviceMode);
	void   			FreeBits();
	void   			FreeFrameBufferBits();
    void            ResizeBitmapsL(TSize aSize);	
    void            RecreateBitmapsL(TSize aSize);    
    void            ResizeFrameBufferBitmapL(TSize aSize); 	
    void            RecreateFrameBufferBitmapL(TSize aSize);
#ifdef FEATURE_OPENVG
    TBool           CreateAuxTextBitsL(TSize aSize);	
    TBool           AuxTextBitsValid();
    void            FreeAuxTextBits();    
    void            ResizeAuxTextBitmapL(TSize aSize);    
    void            RecreateAuxTextBitmapL(TSize aSize);
    void            FreeInputIndicatorBits();
#endif
	
	TUint16 		FrameBufferPitch();
	void 			SetScreenMode();

	
	void 			InitializePlatformInfoL();
	void 			LoadResourceFileL();
	TBool 			CreatePlayerL();
	void 			OpenFileAndStartMovieL(const TDesC& aFilename, TBool aResetBaseURL, TBool aResetIAPAccess, const TBool& aFirstMovie, CContent* aContent = NULL);
	void 			StartMovieL(char* aSwfURL, const U8* aSwfData, U32 aSwfLength, TBool aResetIAPAccess, TBool aFirstMovie);

	void  			GetLanguageInfo(TInt& aSymbianCodepage, FI_CodePage& aFlashCodepage);
	CFbsDevice* 	GetFontDevice() const 
	{
#ifdef FEATURE_WINDOWLESS		
	if(iWindowMode!=EWindow)
	{
	    return this->iWlessDevice;
	}
	else
#endif //FEATURE_WINDOWLESS		
	    return iBitmapDevice; 
	}
#ifdef FEATURE_OPENVG	
	CFbsDevice*     GetAuxTextFontDevice() const;
#endif	

	CWindowGc* 		GetWindowContext();

	TBool 			CreateLatinFont(const struct FI_FontInfo* aFontInfo, CFont*& aFont);
	TBool 			CreateLocaleFont(const struct FI_FontInfo* aFontInfo, CFont*& aFont);
	TBool 			CreateChineseFont(const struct FI_FontInfo* aFontInfo, CFont*& aFont);	

	//void 			SetTempPathAndName();
	TBool 			CanContentBeTrusted(const TDesC8& aSwfURL);
	void 			CalculateInterval();
	TInt 			ValidSwfContentVersion(const TUint8 *aSwfBuffer, TInt aBufferLength);
	void 			CreateBaseURLL(const TDesC* aUrl);

	void 			NotifyExtsOfPauseStateChange(TInt aReason);
	void 			NotifyExtsOfIncomingCallStateChange(TBool aStatus);
	void 			NotifyCorePlayerOfSoftkeyLocationChangeL();
	void 			ProcessParamBGcolor(const TDesC& aBGColor);
	void 			ProcessParamQuality(const TDesC& aQuality);
	void 			ProcessParamLoop(const TDesC& aLoop);
	void 			ProcessParamFlashVars(const TDesC& aFlashVars);
	void 			ProcessParamBaseURL(const TDesC& aBaseURL);

	void 			PlatformTimerTick();
	void 			StartTimer();
	void 			StopTimer();	
	
	#ifdef  FEATURE_BG_TRANSPARENCY
	void 			ProcessParamWMode(const TDesC& awMode);
	#endif	
    

	#ifdef  FEATURE_INLINE_TEXT_ENTRY
	void 			HandleInlineTextBitBlt(CWindowGc& aWGc);
	#endif	
	
	#ifdef FEATURE_MULTIPLE_DRIVE
	//Removing implementation of this function coz handlefileurl
	// is able to handle geturl calls from all the available drives
	//TInt 			SearchInDrivesL(TDesC& aFile);
	TInt 			SearchASLibL(TDes& aFile);
	#endif	

	#ifdef __FEATURE_PRE_COMPILED_SWF
	TBool 			HasSwfExtension( const TDesC& aStr );
	TBool 			GetClassFileBufL(const char* aFilename, int, unsigned char*& aBuf, int& aLen);
	void 			FreeClassFileBuf(unsigned char*& aBuf);
	void 			SetPreInstalledASClassL ( );
	void 			CallFI_SetPreInstASClassL  (TDesC8& aASClass, unsigned short aBIsNativeLibrary);
	#ifdef __FEATURE_PRE_COMPILED_SWF_RSC
	void 			SetPreInsASClassFromRscL ();
	#endif // __FEATURE_PRE_COMPILED_SWF_RSC
	#endif 


	#ifdef ENABLE_CURSOR
	MFlashCursor* 	Cursor();
	void 			HandleCursorDraw();
	#endif

	#ifdef FEATURE_VIDEO_DLL
    IVideoDecompressorInstance* CreateNativeVideoDecompressorL(MM_Object* aMMObj, void *aAllocator, U32 codecType);
    IVideoDecompressorInstance* CreateFlvDecompressorL(MM_Object* aMMObj, void *aAllocator, U32 codecType);
    #endif

	void NotifyFullScreen(TBool aFullScreen);
    
	#ifdef  FEATURE_COLLECT_BITMAP 
	//Copy last rendered frame to CFbsBitmap
	void CopyLastRenderedFrameToBitmapL();
    #endif
	
//private:
public:

    TWindowMode GetWMode() const
    { 
    	return iWindowMode; 
    }

    CSharedMemoryManager* GetSharedMemoryManagerL();

#ifdef DUMP_DIRTY_RECTS
	RFile iDirtyRectDumpFile,iDirtyRectLengthsFile;
	TInt iDirtyRectNum;
	CFbsBitmap* iBitmapToBeDumped;
#ifdef FEATURE_OPENVG   
    CFbsBitmap* iSurfaceData;
#endif
#endif

    unsigned long                   iFIConfigFlags;	
	CCnvCharacterSetConverter* 		iCharacterConverter;
	TUid 							iFlashUid;
	TUint 							iCurrentCodePage;
	TInt 							iQuality;
	DynamicMemBlock* 				iDynamicMemBlocks;
    //Not used
	//TInt 							iMachineUid;
	//HBufC* 							iDeviceName;
	//HBufC* 							iPlatformName;
	TInt 							iResourceOffset;
	//TUint16 						iRomVersion; // ROM version stored as a pair of hex digits, e.g. 0x26 for 2.6
	struct MM_Object* 				iMMObject;
	SRECT 							iRect;
	
	// Timers used in the engine
	CFlhTimerObject* 				iTimer;
	//This timer is used to pause the plugin(if invisible or in background) after all mp3 streams are closed
    CFlhTimerObject*                iMp3Timer;
	// Inter Access Point timer.    Resumes the engine when wait for the network prompt times out 
    CFlhTimerObject*                iIAPTimer;  
	//This timer is used to extendbacklight when video is playing
    CFlhTimerObject*                iBacklightExtensionTimer;
	
	CFlhAsyncObject* 				iLowestPriorityActiveObj;
	TBool 							iIsRunning;
	TUint32 						iFrameInterval;
	TUint 							iDelay;
	CWsBitmap* 						iBitmap;

	CFbsBitGc* 						iBitmapGc;
	CFbsBitmapDevice* 				iBitmapDevice;
    CFbsBitmapDevice*               iDefaultFontBitmapDevice;
    
    //Indicates whether we core is rendering to a bitmap or not.
    //core is renders to bitmap in two scenarios - openvg is disabled, or pixmap surface is used.
    //If pixmap surface has to be used for windowless plugin, dynamic variation is required; hence this flag.
    TBool                           iRenderToBitmap;
    
    TBool                           iOpenVgMode;    
    
	TBool 							iLandscapeScreen;
	CWindowGc* 						iWindowGc;
	CFont * 						iDefaultFont;
	TInt							iCurrentFontId;
	SPOINT 							iBitmapSize;
	TSize                           iFrameBufferSize;
	TBuf<KMaxTypefaceNameLength> 	iLastTypeface;
	TBool 							iIsBitmapLocked;
	TBool 							iIsLastTypefaceValid;
	TBool 							iRedraw;
    TBool 							iVibraFeatureSupportedByDevice;
    TBool 							iVibraFeatureSupportedByCenRep;
    TBool 							iVibraFeatureSupported;
	TBool							iFrameBufferResized;
	TBool							iInDoPlay;
    TUint 							iRed;
    TUint 							iGreen;
    TUint 							iBlue;
    TUint 							iTransparency;
	TBool 							iBackgroundColorPresent;
    short 							iLoop;
    short 							iSwfQuality;
    TBool 							iIsStopped;
    //TBool 							iBrowserPluginActive;
	TRect 							iClippingRect;
	
	//not used
	//TBool 							iMustExit;
	TInt  							iPauseFlags;
	TBool 							iForeground;
	HBufC* 							iBaseURL;
	HBufC8* 						iHTMLBaseURL;
	HBufC8*                         iEngineSecurityContextUrl;
	TInt 							iAllowScriptAccess;
	HBufC* 							iNewFile;
	TBool 							iDownloading;
	FI_DisplayFormat                iFrameBufferDisplayFormat;	
	TDisplayMode 					iDisplayFormat;
	TInt 							iZoomFactor;
	TBool 							iPanMode;
    TPath 							iTempOpenNetFileName;
	CVibrationHandler* 				iVibrationHandler;
	FI_Interface					iFiObject;
	TUint32 						iPlayerStaticHeapSize;
	TAny* 							iPlayerStaticHeap;
	TUint32 						iPlayerDynamicHeapMaxSize;
	TUint32 						iPlayerDynamicHeapAlloced;
	TInt 							iPlayerState;
	RProperty 						iProperty;
    CPhoneMonitor* 					iPhoneMonitor;
	CSoundHandler* 					iSoundHandler;
	TBool							iSvgMode;
	TBool							iIsInitialized;
	CFlhAsyncEvent* 				iAsyncEventHandler;
	TGetInputTextData* 				iGetInputTextRequest;
	TIMEState                       iIMEControlState;
	TInt							iChangeScreenSizeRequest;
	TInt							iChangeKeysRequest;
	HBufC**							iSoftkeyArray;
	TInt							iSoftkeyArraySize;
	//MFlashUI* 						iFlashUI;
    //MFlashPlugin* 					iControl;
	FlashEngineConfig 				iConfig;
	TInt 							iAllowPhoneCall;
	TInt 							iAllowNetAccessId;
	TUint8 							iSWFContentVersion;
	TUint 							iCurrentVolume;
	CFileEngine*					iFileEngine;
	CIPConnMgr*						iIPConnMgr;
	CHttpEngine*					iHttpEngine;
	CLocalHttpEngine*				iLocalHttpEngine;
	CVideoEngine*					iVideoEngine;
	CFlhFileUtils*					iFileUtils;
#ifndef USE_SYNC_MSO	
	CFlhShObjEngine*				iShObjEngine;
#else	
	CSharedObjectHandler*           iSharedObjHandler;
#endif //USE_SYNC_MSO	
	CImageConverter*				iImageConverter;
	TTime							iLastTimeStamp;
	RFile*							iLogFile;
	RFile 							iHandle;
	CEikonEnv* 						iEnv;
	RFs& 							iFs;
	TSglQue<CQueuedSILaunch>		iLaunchQueue;
	CFlhAsyncObject* 				iLaunchQueueHandler;
	TBool 							iPromptForIAP;
	TUid 							iBrowserUid;
	short 							iSoftkeyLocation;
	TBool							iReplayFlag;
 	TVideoRect						iVideoRect;
	TBool 							iFlashVarsPresent;
	HBufC8* 						iFlashVarVal;
	TBool 							iNotifyEndOfMovie;
	TPtrC 							iSwfSrcPath;
	CFLPlatformDecoderInterface*    iFlvDecoder;
	TBool 							iFLVPlayStat; 
	CPlatformConfig* 				iPlatformConfig;
	TBool 							iControlPresent;
	CArrayPtrSeg<THttpRequestInfo>*	iRequestArray;
	EFLVStatus 						iFLVStatus;
	//This is used to keep count of mp3 streams
	TInt8                           iMP3StreamCount;
	CBackLightHandler* 				iBkLightHandler;
	CFlashTrustStore*                iTrustStore;

    //True if non-video content draw over video area after playback is complete
    //Used specifically to show share/replay screen.
	TBool 							iContentOverVideo;
	
	TBool                           iRequestedScreenChange; //TSWID:VKHA-7PD8CG - needed to delay the call to fi_setframebuffer

	RPointerArray<CFLPlatformDecoderInterface>	 iFlvDecoderArray;	
	CArrayPtrSeg<CFlhItemValuePairList>* iItemValuePairArrays;
	CArrayPtrSeg<CFlhAsyncIPSelectDlg>*	 iAsyncIPDlgArray;
#ifdef FEATURE_AMCEXTENSIONS
	RPointerArray<MAMCEXtensionCallback> iExtensionCallback;
#endif

	#ifdef FEATURE_PACK_3
    CNetworkInfo* 					iNetworkInfo;
    CSubscriberInfo* 				iSubscriberInfo;
    CRegistrationStatus* 			iRegistrationStatus;
    CPhoneInfo* 					iPhoneInfo;
    CPhoneBatteryInfo* 				iBatteryInfo;
    CPhoneSignalInfo* 				iSignalInfo;
    CPhoneChargingInfo* 			iChargingInfo;
    CPlatformInfo* 					iPlatformInfo;
	#endif // FEATURE_PACK_3

	#ifdef FEATURE_XML_SOCKET	
	CFlashSocketEngine* 			iSocketEngine;
	#endif //FEATURE_XML_SOCKET		
	
	#ifdef FEATURE_INLINE_TEXT_ENTRY
	AMP_VirtualTextNavigation* 		iTextNavigation;
	TRect 							iTIIndicatorDirtyRect;
//	TBool 							iImeActive;

		#ifdef FEATURE_OPENVG
	    CFbsBitmap*                     iIMIndicatorBmp; 
	    CFbsBitmapDevice*               iIMIndicatorBmpDevice;
	    CFbsBitGc*                      iIMIndicatorBmpGc;
    
	    TInt                            iCurrentTextInputMode;    
	    TRect                           iCurrentIMIndicatorBounds;
	    VGImage                         iInputModeIndicatorImage;    
		#endif
	#endif

	/*#ifdef RD_MULTIPLE_DRIVE
	TPtr 							iPtrRef;
	#endif*/	

	#ifdef FEATURE_FLV_MMC
	RArray<CBrowserURLRequestInfo*> 	iNetBufArray;	
	#else	
	RPointerArray<FI_NetworkBufferInfo> iNetBufArray;
	#endif	

	// This UTC reference time is used to convert localtime to UTC
	// UTC reference time is 1/1/1970 Midnight (00:00:00:0000)
	TTime		iUtcReferenceTime;
	
	TInitInfo iInitInfo;
	MMp3StateObserver* iMp3PlayStateObserver;
	TUint16   iPstatus;
	TBacklightStatus iExtendBacklight;
	
	MFlashCursor* iCursor;
    TBool iPassOnException;
public:
	TUint 								iHackCounter;
	TUint 								iHackCounter2;

	#ifdef FEATURE_WEB_BROWSABILITY		
	CHostScriptHandler* 				iHostScriptHandler;
	HBufC8* 							iName;
	#endif //FEATURE_WEB_BROWSABILITY

	#ifdef FEATURE_VIDEO_DLL
	FL_ReferenceCountedObjectShare		m_decompressorOn2;
	FL_ReferenceCountedObjectShare		m_decompressorSorenson;
	#endif
	
#ifdef FEATURE_OPENVG
private:
    void  InitializeEglL();
    void  DestroyEgl();
    void  CreateEglSurfaceL();
    void  ChooseEglConfigL();
	void  RecreateSurfaceL(RWindow* aWindow);
	void  DestroyEglSurface();
	
	
	//Needs to be called to set the current egl context, since another instance
	//might be running in the same thread (browser plugin / widget use case)
	void  SetCurrentEglContext();
	
    void  SetVgImagePosition(TInt x, TInt y);

    enum TRenderingSwitchType {
        ESwitchRenderingHWToSW,
        ESwitchRenderingSWToHW,
    }iRenderingSwitchType;
    
    CFlhTimerObject2* iRenderToggleTimer;
    
    TInt SwitchToSoftwareRendering(TBool aSync = EFalse); //asynchronous by default
    TInt SwitchToHwxRendering(TBool aSync = EFalse);  
	  
    static TInt ToggleRenderingMode(TAny* aPtr);
    TInt DoToggleRenderingMode();        
    void InitEGLMaxDimensionL();
public: //from MCoeForegroundObserver
    void HandleGainingForeground();
    void HandleLosingForeground();      
    
    TBool IsSwitchingToSWRenderingInPrgress(){ return iSwitchingToSWRenderingInPrgress;};
    
private:
    FI_HardwareConfig iHwConfig;
    
    EGLDisplay  iEGLDisplay;
    FI_Surface  iEGLSurface;  
    EGLContext  iEGLContext;
    EGLint      iEGLConfig;
    
    TBool       iEGLInitialized;
    EGLint      iEGLMaxSupportedWidth, iEGLMaxSupportedHeight;
    
    SPOINT      iAuxTextBitmapSize;     
    
    CFbsBitmap*                     iAuxTextBitmap;
    CFbsBitGc*                      iAuxTextBitmapGc;
    CFbsBitmapDevice*               iAuxTextBitmapDevice;
    TBool                           iIsAuxTextBitmapLocked;
    
	TInt 							iTextDumpNum;	
	TSize 							iCurrentWindowSize;
	
    #ifdef  FEATURE_COLLECT_BITMAP		    
    CFbsBitmap*                     iLastRenderedBitmap;
    TBool                           iIgnoreDrawBitBuffer;
    #endif
    
    #ifdef USE_VPU2
    private:
        void  EnableAlphaOnEgl(RWindow* aWindow); 
        CAlfCompositionSource* iCompositionControl;       
    #endif	
        

    #ifdef FEATURE_WATCH_GPU_MEM
    TBool                           iGPUMemLow;
    CGPUMemWatch*                   iGPUMemWatch;
        
    public:	
        void GPUMemoryLow(TBool aLow);
    #endif	
	
#endif //FEATURE_OPENVG

#ifdef FORCE_REFRESH_WORKAROUND
private:
    void ForceRedraw();
    static TInt ForceRedrawCallback(TAny* aPtr);
    void DoForceRedraw();
    
    TBool iForceRedraw;
    CFlhTimerObject2* iForceRedrawTimer;
#endif

public:    
    void HandleWidgetError(TBool);
	TDisplayMode GetDisplayMode(){ return iDisplayFormat;}

	#if defined(FEATURE_PERFORMANCE_AUTOMATION) || defined(FEATURE_ATS_AUTOMATION) || defined(FEATURE_TESTAUTOMATION)
	TBool iPerfTestMode;
	#endif
#ifdef FEATURE_ATS_AUTOMATION		
private:
	CATSHandler* iATSHandler;
	#endif

#ifdef SELF_IAD_UPDATE
    void HandleIADCheck( MIadUpdateCallback* aIADUpdateCompleteCallback, FlashEngineConfig& aConfig  );
    void DestroyIADComponents();
#endif
	TInt	iScaleValue;
	TInt	iAlignValue;    

// HostEngineGlobals
	CHostEngineGlobals* iHostEngineGlobals;

	TInt  iNetworkPromptPausedCount;
	//EInteractionMode iInteractionMode;
	TBool iFitVideoToScreen;   // Added to clear transformations in case Video contents are present

	TBool  iFullScreenViaFSCommand;
	TInt   iSwfHasVideo;
	TInt   iSwfHasAudio;
	TBool  iIsVideoContent;
	HBufC* iSwfPath;
	TBool iXMLFlag;
	
	MFlashClientIf* iFlashClientIf;
	
	EEngineExecutionMode iExecutionMode;
	EBrPluginExecutionMode iBrPluginMode;

    TWindowMode iWindowMode;	
	
#ifdef FEATURE_WINDOWLESS	
	CFbsBitmap* iWlessSurface;
	CFbsBitGc*  iWlessGc;
	CFbsBitmapDevice* iWlessDevice;
#endif //FEATURE_WINDOWLESS	

	CFontMapTable* iFontMapTable;

private:
	CSharedMemoryManager* iSharedMemMgr;
    RWindow* iRenderingWindow;
    TBool    iAllowFullscreen;
    TInt     iAllowNetworking;
    TBool iPauseforActiveCall;
    
    TBool   iSwfStarted;
    TBool   iSwfInDestruction;  //Flag to keep track of SWF destruction
    TBool iSwitchingToSWRenderingInPrgress;

};

#endif	// __SYMBIAN_ENGINE_H__
