/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash Methods Handler class 
*
*/
 

#ifndef __SYMBIAN_HOSTSCRIPT_HANDLER_H__
#define __SYMBIAN_HOSTSCRIPT_HANDLER_H__

#include "externalinterface.h"
#include "mflashmethods.h"
#include "hostscripthandler.h"
#include "pluginscripthandler.h"
#include "async_object.h"


class CPlatformFlashEngine;


enum TJSRequestType
	{
	EFsCommandRequest,
	EEvalJSRequest,
	ECallJSRequest
	};
	
class CJSRequest: public CBase
	{
	public:
		static CJSRequest* NewL(const char* aCommand, const char* aArgs, 
												TJSRequestType aJSRequestType);
		virtual ~CJSRequest();
		
		TJSRequestType GetRequestType();
		
		const char* GetCommand();
		
		const char* GetArgs();
	private:
		CJSRequest();
		void ConstructL(const char* aCommand, const char* aArgs, 
												TJSRequestType aJSRequestType);
	
	private:
		char* iCommand;
		char* iArgs;
		TJSRequestType iJSRequestType;
	};
	
class CHostScriptHandler :public CBase 
						  ,public MHostScriptHandler
						 , public MFlashMethods 
						 #ifdef FEATURE_EXTERNAL_API
						 , public MExternalInterface
						 #endif
						 , public MFlhAsyncHandler
{
    
public:

	
	static CHostScriptHandler* NewL(CPlatformFlashEngine* aEngine);
		   
    virtual ~CHostScriptHandler();
      
private:
	CHostScriptHandler(CPlatformFlashEngine* aEngine);        
	
	void ConstructL();
    

public:  //from MHostScriptHandler

	virtual MFlashMethods* GetFlashMethods() ;

#ifdef FEATURE_EXTERNAL_API

	virtual MExternalInterface* GetExternalInterface() ;

	TBool   SetReturnValueL(const TDesC& aRetVal) ;
	
	TBool CallJSMethodL( const char* aRequest);
	
	TBool EvaluateJavaScriptL( const char* aExpression);
	
	TBool DoFsCommandL(const char* aCommand, const char* aArgs);
	
	void ServiceRequestL( CJSRequest* aRequest );
	
	
	
#endif
					
	//Used by Host engine and core player to talk to Javascript
	virtual MPluginScriptHandler* GetPluginScriptHandler() ;
					
	//This function should be called by plugin before calling
	//GetFlashMethods,GetExternalInterface
	virtual void SetPluginScriptHandler(MPluginScriptHandler* aPluginScriptHandler) ;
							

public: //from  MFlashMethods

	virtual TBool GetVariableL(const TDesC& aVariableNameIn, 
    					HBufC** aVariableValueOut,
    					const TDesC& aURLIn, TPageAccessType aPageAccessType, 
    					TBool aIsTrusted) ;
						
	virtual TBool GotoFrame(TInt aFrameNumber) ;
						
	virtual TBool IsPlaying() const ;
	
	
	virtual TBool LoadMovieL( const TDesC& aSwfURL, TInt aLayerNo
							,const TDesC& aURLIn
							,TPageAccessType aPageAccessType
    						,TBool aIsTrusted);
													
	virtual TBool Pan(TInt32 aXCoordinate
					,TInt32 aYCoordinate
					, TInt32 aMode) ;
	
	

	virtual TBool PercentLoaded(TInt& aLoadPercent) ;	
	
	virtual TBool FlashVersion(HBufC** aVersion) ;	//API added to get the flash version
	
	virtual TBool Play() ;
	
	virtual TBool Rewind() ;
	
	virtual TBool SetVariableL(	const TDesC& aVariableNameIn,
	    				const TDesC& aVariableValueIn,const TDesC& aURLIn, 
	    				TPageAccessType aPageAccessType, 
    					TBool aIsTrusted) ;
	

	virtual TBool SetZoomRect(TInt aLeft, TInt aRight, TInt aBottom, TInt aTop) ;
	
	virtual TBool StopPlay() ;
	
	virtual TBool TotalFrames(TInt& aNumberOfFrames) ;
	
	virtual TBool Zoom(TInt aZoomPercent) ;
	
	//TellTarget Methods
	virtual TBool TCallFrameL( const TDesC& aTarget, 
						TInt aFrameNumber) ;
	
	virtual TBool TCallLabelL( const TDesC& aTarget,const TDesC& aLabel) ;
	
	virtual TBool TCurrentFrameL(  const TDesC& aTarget,
								 TInt& aCurrentFrame, const TDesC& aURLIn
								 , TPageAccessType aPageAccessType, TBool aIsTrusted) ;
	
	virtual TBool CurrentFrame(TInt& frameNumber) ;
	
	virtual TBool TCurrentLabelL(  const TDesC& aTarget,
									HBufC** aLabel, const TDesC16& aURLIn, 
									TPageAccessType aPageAccessType, TBool aIsTrusted) ;
	
	virtual TBool TGetPropertyL( const TDesC& aTarget,TInt aPropertyID,
								HBufC** aPropertyValue, const TDesC& aURLIn, 
	    				 TPageAccessType aPageAccessType,	TBool aIsTrusted);
	    				 
	virtual TBool TGetPropertyAsNumberL( const TDesC& aTarget,TInt aPropertyID,
						 HBufC** aPropertyValue, const TDesC& aURLIn, 
	    				 TPageAccessType aPageAccessType,	TBool aIsTrusted) ;
	
	virtual TBool TGotoFrameL(const TDesC& aTarget,
								 TInt aTargetFrame) ;
	
	virtual TBool TGotoLabelL( const TDesC& aTarget,const TDesC& aLabel) ;
	
	virtual TBool TPlayL( const TDesC& aTarget) ;
	    				 
	virtual TBool TSetPropertyL(	const TDesC& aTarget,TInt aPropertyID
						 , const TDesC& aPropertyValue, const TDesC& aURLIn, 
    					TPageAccessType aPageAccessType, TBool aIsTrusted) ;
	
	virtual TBool TStopPlayL( const TDesC& aTarget ) ;
	
public://from MExternalInterface

#ifdef FEATURE_EXTERNAL_API
	virtual TBool CallMethodL( const TDesC& aMethod,
							HBufC** aRetVal)  ;
#endif

public: //Flash Properties

	virtual void DoRunL(const CActive* aActive);
	
	virtual TInt DoRunError(const CActive* aActive, TInt aError);

public: //Flash Properties

	virtual  TBool	GetPropAllowScriptAccess(TInt& aAccessLevel);
    
    virtual TBool 	GetPropBGColorL(FI_Color& aBGColor);
    
    virtual TBool	GetPropQuality(TInt& aQuality);
    
    virtual TBool	SetPropBaseURL(const TDesC& aBaseURL);
    
    virtual TBool	SetPropAllowScriptAccessL(const TInt& aAccessLevel);
    
    virtual TBool 	SetPropBGColorL(const FI_Color& aBGColor);
    
    virtual TBool	SetPropQualityL(const TInt& aQuality);
    
    virtual TBool	SetPropLoopL(const TInt& aLoop);
    
private:
		
    MPluginScriptHandler*       iPluginScriptHandler;
    
   	CPlatformFlashEngine* iEngine;
   	
   	RPointerArray<CJSRequest> iReqArray;
   	
   	CFlhAsyncObject* iAsyncObject;
   	
   
};



#endif //__SYMBIAN_HOSTSCRIPT_HANDLER_H__
