/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class that provides a simple interface to the HTTP 
*				 Client API
*
*/


#ifndef __SYMBIAN_HTTP_H__
#define __SYMBIAN_HTTP_H__

#include <coecntrl.h>

#include <http/rhttpsession.h>
#include <http/mhttpdatasupplier.h>
#include <http/mhttptransactioncallback.h>

#include <http/mhttpauthenticationcallback.h>
#include <brctldialogsprovider.h>
#include <browserdialogsproviderobserver.h>
#include "async_object.h"
#include "timer_object.h"
#include "httprequestbase.h"
#include "symbian_useragent.h"
#include "ipconn_mgr.h"
#include "conn_handler.h"

enum TPreparingToExit
{
	EInitial,
	EPreparingToExit,
	EPreparingToExitDone
};

const TInt KFlashLiteHttpClientSideCancelErr=0;
/**
 * The CHttpRequestInfo class is used to wrap up an HTTP request call from the core player.
 * The request can be queued until needed, or used right away depending on the policy
 * of the CHttpEngine and what other requests are currently being serviced.
 * 
 */
 
/*class CHttpRequestInfo : public CBase
{
public:
	~CHttpRequestInfo() 
	{
		delete iUrl;
		delete iPostData;
		delete iAdditionalHdrs;
	}

	// CompleteConstructL is ONLY to be called by the HttpEngine in all cases
	// even derived classes should not call this function...
	void CompleteConstructL();
protected:
	CHttpRequestInfo(void* aOwner, struct MM_Object* aMMObj, struct FI_URLStreamInfo* aStreamInfo,TUint32 aId)
	:iMMObj(aMMObj), iOwner(aOwner), iStreamInfo(aStreamInfo), iId(aId)
	{}

public: // accessor functions

	// getters
	inline const TDesC8& Url() const				{ return *iUrl; }
	inline const TDesC8* PostData() const			{ return iPostData; }
	inline const TDesC8* AdditionalHeaders() const	{ return iAdditionalHdrs; }
	// non const getters
	inline struct MM_Object* MM_Object()						{ return iMMObj; }


	TBool IsMe(void* aOwner, TInt aId) { return (aOwner==iOwner && aId==Id());}

protected:
	// get the unique Id
	TInt Id() { return iId; }

public:
	// prepare any files/buffers etc that will need to be used for recording downloaded data
	// prior to the Http request being made...
	virtual void PrepareToBeServicedL()=0;

	// The request has failed - tell the client and prepare to be deleted.
	virtual void HandleRequestFailed(TInt aError)=0;
	
	// once being serviced handle the fact that there is a header response...
	virtual void HandleHeaderResponseL(struct FI_URLStreamHeaderInfo& aHeaders, const TDesC8& aFinalUrl)=0;
	virtual TBool HandleResponseDataL(const TDesC8& aDataPart)=0;
	virtual void HandleResponseCompleteL()=0; // request has completed normally
public: // link data
	TSglQueLink iLinkItem;
	
protected: // other data...
	struct MM_Object* iMMObj; // not owned
	HBufC8* iPostData;
private:
	HBufC8* iUrl;
//	HBufC8* iPostData;
	HBufC8* iAdditionalHdrs;
	
	void* iOwner;
	struct FI_URLStreamInfo* iStreamInfo;
	TUint32 iId;
};
*/
/*
* Forward declarations
*/
class RHTTPSession;
class RHTTPTransaction;

struct SI_StreamHandle;
struct FI_NetworkBufferInfo;
struct FI_URLStreamHeaderInfo;

class CPlatformFlashEngine;

class CHttpRequestInfo;
class CBrowserDialogsProvider;

/**
 * The CHttpEngine class provides a simple interface to the HTTP Client API.
 * 
 */
class CHttpEngine :	public CBase, 
					public MHTTPTransactionCallback,
					public MHTTPDataSupplier,
					public MFlhIPConnectObserver,
					public MFlhAsyncHandler,
					public MHTTPAuthenticationCallback,
					public MBrowserDialogsProviderObserver

{
public:
	
	// c'tor / d'tor
	static CHttpEngine* NewL(CPlatformFlashEngine* aEngine);
	
	~CHttpEngine();
	
	void ResetEngine(); // resets to its original state - should be called when movie ends for example
	
	void PrepareToExit();

	// command issuing functions...
	unsigned short StartHttpRequestL(CHttpRequestInfo* aInfo);
	unsigned short CancelHttpRequest(void* aOwner, unsigned long aId);

	CHttpRequestInfo* GetRequestObject(void* aOwner,TUint32 aId);

	//inline TBool IsRunning() { return m_CurrentRequest!=0; };
	TBool OKToPause();
	
	TInt GetStatusForPlayer() const;
	
	void CloseAllTransactions();

public: // From MHTTPAuthenticationCallback

	/*	Gets some credentials. Note that the first 3 parameters are
		informational and many clients may not have a use for
		them. Clients may want to display some or all of these
		parameters to the user, so that the user knows who they are
		giving their credentials to, and how securely they will be
		transmitted.
		*/
	
	TBool GetCredentialsL(const TUriC8& aURI, RString aRealm, 
								  RStringF aAuthenticationType,
								  RString& aUsername, 
								  RString& aPassword);
								  
public: // From MBrowserDialogsProviderObserver

	// Receives events from applications which want to report that a 
    // dialog event has taken place.
	void ReportDialogEventL( TInt aType, TInt aFlags );
	

private:

	void IssueHTTPGetL(const CHttpRequestInfo& aInfo);
	void IssueHTTPPostL(const CHttpRequestInfo& aInfo);

private:
	void ConstructL();
	void ConstructL(const TDesC8& aUserAgent);

	// CHttpEngine()
	CHttpEngine(CPlatformFlashEngine* aEngine);

	void MakeUserAgentStrL(const TDesC8& aUserAgent);
	
private:
	
	void CloseConnectionNow();

	void SetHeaderL(RHTTPHeaders aHeaders, TInt aHdrField, 
					const TDesC8& aHdrValue);

	void AddCustomHeaderL(RHTTPHeaders& aHeaders,
						  const TDesC8& aHeader);

	void CancelActiveTransaction();
	void TryQueueNextRequest();
	void DoIssueNextRequestL(CHttpRequestInfo& aInfo);


	CHttpRequestInfo* GetItemFromQueue(void* aOwner, TUint32 aId);
	
	CHttpRequestInfo* GetObjectFromCompletedRequestsQueue(void* aOwner, TInt aStreamId);

	void TransactionOverQueueNextL(RHTTPTransaction& aTransaction);
	
	void AddToCompletedRequestsIfFlvL(CHttpRequestInfo* aInfo);
	
	void DeleteObjectFromCompletedRequestsQueue(void* aOwner,TInt aStreamId);
	
	void RemoveAllCompletedRequests();

	void DoSynchronousRTMPTCloseL();
	void CancelSynchronousRTMPTClose();
	void RTMPTCloseCompleted();
	static TInt RTMPTTimerCallback(void* aArg);
	
	void ConnectToNetworkL(CHttpRequestInfo* aInfo);

/*
* From MHTTPSessionEventCallback
*/
private:
	// MHFRunL()
	void MHFRunL(RHTTPTransaction aTransaction, const THTTPEvent& aEvent);

	// MHFRunError()
	TInt MHFRunError(	TInt aError, 
						RHTTPTransaction aTransaction, 
						const THTTPEvent& aEvent);

/*
* From MHTTPDataSupplier (needed for HTTP POST)
*/
private:
	// ReleaseData()
	void ReleaseData();

	// GetNextDataPart()
	TBool GetNextDataPart(TPtrC8& aDataPart);

	// Reset()
	TInt Reset();

private: // from CFlhIPConnectObserver
	void ConnectionEventL(TFlhIPEvent aEvent);

private: // from MFlpUser
	void HandleNetworkAccessDenied();

private: // from MFlhAsyncHandler
	virtual void DoRunL(const CActive* aActive);
	virtual TInt DoRunError(const CActive* aActive, TInt aError);

private: // new func's
	// OverallDataSize()
	TInt OverallDataSize();

	void HandleHeaderResponseL(RHTTPTransaction& aTransaction);

	TInt ParseHttpHeadersL(RHTTPTransaction& aTrans, FI_URLStreamHeaderInfo& aHeaderInfo);

	void HandleResponseDataL(RHTTPTransaction& aTransaction);

	void HandleCompleteResponseL(RHTTPTransaction& aTransaction);

private:
	CPlatformFlashEngine*       iEngine;	
	
	RHTTPSession				iSession;
	RHTTPTransaction			iTransaction;

	TInt						iTransStat;
	TInt						iConnStat;
	
	TSglQue<CHttpRequestInfo>	iRequestQueue; 

	CHttpRequestInfo*			iCurrentRequest;

	CFlhTimerObject*			iTimer;
    CFlhTimerObject2*			iRTMPTCloseWaitTimer;   // use to make sure that we do not wait indefinitely

	TBool						iHandlingEventCallback;
	TBool						iCurrentRequestDeleted;
	
	TBool						iRetryAuthentication;
	HBufC8* 					iUserName;
	CBrowserDialogsProvider* 	iDlgProvider;
	CUserAgentInfo*				iUserAgentInfo;	
	CArrayPtrSeg<CHttpRequestInfo>* iCompletedRequestsQueue;
	CIPConnMgr* iIPConnectionMgr;
	CConnHandler* iConnHandler;
	#if	defined(FEATURE_ATS_AUTOMATION) || defined( FEATURE_PERFORMANCE_AUTOMATION)  || defined(FEATURE_TESTAUTOMATION)
	TBool iPerfTestMode;
	#endif

	//This CASW is to make RTMPT close request sync while player is being reset. 
	CActiveSchedulerWait*		iRTMPTCloseWait;
	TPreparingToExit			iPreparingToExit;

};

#endif	// __SYMBIAN_HTTP_H__
