/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class that is used to support the SI_GetImageInfo() 
*				 and SI_DecodeImage() Flash Lite API functions.
*
*/



#ifndef __SYMBIAN_IMAGE_H__
#define __SYMBIAN_IMAGE_H__

#include <imageconversion.h>


const TInt KTimeOutMicroSeconds = 10000000;

class CImageRequest : public CActive
{
public:
	static CImageRequest* NewL(CImageConverter*	aConverter, unsigned long aImageId, const FI_ImageData* aData, EEngineExecutionMode aExecutionMode,EBrPluginExecutionMode aBrPluginMode);

	~CImageRequest();

	unsigned long GetImageId();
	
	FI_ImageData& GetImageData();

	FI_ImageInfo& GetImageInfo();

	FI_DecodeImageInfo& GetDecodeImageInfo();

	void SetDecodeImageInfo(FI_DecodeImageInfo* aDecodeImageInfo);
	
	void ReadImageHeaderL(RFs& aFs);
	
	TBool IsReadyForDecode();
	HBufC* 				iImgDsFileName;
	TBool IsImageDownScaling();
private: //from CActive
	void RunL();

	TInt RunError(TInt aError);

	void DoCancel();
	
private:
	CImageRequest(CImageConverter* aConverter,unsigned long aImageId, const FI_ImageData* aData, EEngineExecutionMode aExecutionMode,EBrPluginExecutionMode iBrPluginMode );
	
	void ConstructL();

	TBool ImageDownScalingSupported(RFs& aFs,const FI_ImageData* aData);
	HBufC* UpdateFileNameFromImageDataL(FI_ImageData* aData,RFs& aFs);
private:

	CImageConverter*		iConverter;

	unsigned long			iImageId;

	FI_ImageInfo			iImageInfo;

	FI_ImageData			iImageData;

	FI_DecodeImageInfo*		iDecodeImageInfo;
	TBool 					iImageDownScaleRequest;
	EEngineExecutionMode 	iExecutionMode;
	EBrPluginExecutionMode iBrPluginMode;

};

class CImageConverter : public CActive
{
public:

	static CImageConverter* NewL(CPlatformFlashEngine& aEngine);

	~CImageConverter();

	void Reset();

	void HandleImageInfoRequestL(unsigned long aImageId, const FI_ImageData* aData);

	CImageRequest* RemoveFromList(unsigned long aImageId);

	TInt16 CancelImageRequest(unsigned long aImageId);

	TBool NotifyImageInfo(CImageRequest& aImageRequest, TInt aError);
	
private: //from CActive
	void RunL();

	TInt RunError(TInt aError);

	void DoCancel();
	
private:

	static void CopyImageDataL(CFbsBitmap* aBitmap, CFbsBitmap* aMaskBitmap, FI_DecodeImageInfo& aDecodeInfo, FI_ImageInfo& aImageInfo);

	CImageConverter(CPlatformFlashEngine& aEngine); 

	void ConstructL();

	void ProcessNextRequest();

	CImageRequest* GetNextAvailableRequest();

	void DecodeImageL(CImageRequest* aImageRequest);
	
	void DecodeImageCompleted();
	
private:

	CPlatformFlashEngine&			iEngine;

	CArrayPtrSeg<CImageRequest>*	iImageRequestList;

	CImageRequest*					iCurrentImageRequest;

	CImageDecoder*					iDecoder;

	CFbsBitmap*						iBitmap;

	CFbsBitmap*						iMaskBitmap;
	
	TPath                           iTempImgFile;

};




#endif /* __SYMBIAN_IMAGE_H__ */
