/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  defines the member functions of the class that is used to 
*				 HTTP information from local HTTP server.
*
*/

#ifndef __SYMBIAN_LOCALHTTP_H__
#define __SYMBIAN_LOCALHTTP_H__

#include <coecntrl.h>

#include <in_sock.h>

#include "symbian_urlrequestinfo.h"
#include "symbian_socket.h"

class CLocalUrlRequestInfo : public CUrlRequestInfo
{
public:
	static CLocalUrlRequestInfo* NewL(void* aOwner, struct MM_Object* aMMObj, struct FI_URLStreamInfo* aStreamInfo, TUint aId);
	~CLocalUrlRequestInfo();

	const TDesC8& Path() { return *iPath; }
	const TDesC8& HostName() { return *iHostName; }
	const TUint16 Port() { return iPort; }
	const TDesC8& ContentType() { return *iContentType; }
	const TUint32 ContentLength() { return iContentLength; }

private:
	CLocalUrlRequestInfo(void* aOwner, struct MM_Object* aMMObj, TUint32 aId, struct FI_URLStreamInfo* aStreamInfo);
	void ConstructL(struct FI_URLStreamInfo* aStreamInfo);

private:
	HBufC8*	iHostName;
	TUint16	iPort;
	HBufC8*	iPath;
	TUint32	iContentLength;
	HBufC8*	iContentType;
};

enum HttpQueryStageType
{
	STAGE_NONE,
	STAGE_QUERYHEADER,
	STAGE_HEADEREND,
	STAGE_QUERYDATA
};


enum
{
	ENotConnected,
	EConnecting,
	EConnected
};

class CLocalUrlRequestInfo;

/*
* Provides simple interface to HTTP Client API.
*/
class CLocalHttpEngine :	public CActive
							, public MSocketNotifier
							//,public MFlhAsyncHandler

{
public:
	
	// c'tor / d'tor

	static CLocalHttpEngine* NewL();
	~CLocalHttpEngine();
	
	void ResetEngine(); // resets to its original state - should be called when movie ends for example
	
	// command issuing functions...
	unsigned short StartLocalRequestL(CHttpRequestInfo* aInfo);
	unsigned short CancelHttpRequest(void* aOwner, unsigned long aId);

	CHttpRequestInfo* GetRequestObject(void* aOwner,TUint32 aId);

	TInt GetStatusForPlayer() const;

	void PrepareToExit();
	void CloseAllTransactions();

private:

	void ConnectL(CLocalUrlRequestInfo& aInfo);
	void IssueRequestL(CLocalUrlRequestInfo& aInfo);
	void RequestCompletedL(TBool aSuccessStatus);
	void DoSynchronousRTMPTCloseL();
	void CancelSynchronousRTMPTClose();
	static TInt RTMPTTimerCallback(TAny* aArg);	// RTMTP close timeout callback
	void RTMPTCloseCompleted();

private:
	
	void ConstructL();

	CLocalHttpEngine();

private:
	
	void CancelActiveTransaction();
	void TryQueueNextRequest();
	void DoIssueNextRequestL(CLocalUrlRequestInfo& aInfo);


	CHttpRequestInfo* GetItemFromQueue(void* aOwner, TUint32 aId);

private: // from CActive
	void RunL();
	void DoCancel();

private: // from MSocketNotifier
	virtual TBool ResponseReceivedL(const TDesC8& aBuffer);
	virtual void WriteCompleted();
	virtual void ReportErrorL(TErrorType aErrorType, TInt aErrorCode);
	TBool IsOpen(){return iSocketOpen;};

private: // new func's
	TInt ScanHeader(const TDesC8& aBuffer);
	TBool ProcessDataL(const TDesC8& aDataBuffer);
	void HandleReadEOFL();

private:
	
	//TInt						iTransStat;
	TInt						iConnStat;
	HttpQueryStageType			iHttpStage;

	TInt						iSizeExpected;
	TInt						iSizeRead;
	
	TSglQue<CHttpRequestInfo>	iRequestQueue; 

	CLocalUrlRequestInfo*		iCurrentRequest;

	CFlhTimerObject*			iTimer;

	RSocket                     iSocket;
	TBool						iSocketOpen;

	CFlashSocketReader*			iSocketReader;
	CFlashSocketWriter*			iSocketWriter;

	RSocketServ					iSocketServ;

	TInetAddr					iAddress;

	TBool						iChunkedTransfer;

	//This CASW is to make RTMPT close request sync while player is being reset. 
	CActiveSchedulerWait*		iRTMPTCloseWait;
	TBool						iSynchronizeRTMPTClose;
    CFlhTimerObject2*			iRTMPTCloseWaitTimer;   // use to make sure that we do not wait indefinitely
    
    TPreparingToExit			iPreparingToExit;
};

#endif	// __SYMBIAN_LOCALHTTP_H__
