/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
#ifndef SYMBIAN_LOCATION_H_
#define SYMBIAN_LOCATION_H_

#include "AMBasicTypes.h"
#include "IAMGPS.h"
#include "IAMAllocator.h"

#include <lbsfields.h>
#include <lbscommon.h>
#include <lbspositioninfo.h>

#include "locationservice.h"

const int KMaxSupportedFields = 25;

class LocationImpl : public IAMGPS
    {
public:
    static AMErr Create(IAMAllocator* pAllocator, IAMGPS** ppGPS);
    static U32 AddRef(IAMGPS* ppGPS);
    static U32 Release(IAMGPS* ppGPS);
    static AMErr StaticStartTracking(IAMGPS* pGPS, AMStartTrackingCallback* pCallback, void *pUserData, S32 time, IAMGPSTracker** pInstance);
   ~LocationImpl();

private:
    IAMAllocator* m_pAllocator;

    LocationImpl(); //: m_refCount(0)
    
    AMErr Init();
    
public:
    S32     m_refCount;
    
    TPositionModuleInfo ModuleInfo() { return iModuleInfo; }
private:   

    /**
     * Internal utility function which fills iGenericPosinfo  accoriding to capability of 
     * Positioning module used for location estimation
     */

    TInt SetSupportedFields();  
        
    //AMErr CreateGPSEngine();
    
    /**
     * Member variable which holds all the Generic position  information
     */

   HPositionGenericInfo *iGenericPosInfo;

    /**
     * Class member which contains module information which is used for location esitmation
     */
    TPositionModuleInfo iModuleInfo;

    /**
     * Class member which contains all the field ids of all the supported Hposition info fields
     */

    TPositionFieldId iFieldList[KMaxSupportedFields];
    
    friend class LocationTracker;
    };


class LocationTracker : public IAMGPSTracker, public MLocationCallBack
{
public:
    LocationTracker(IAMAllocator* pAllocator);
    ~LocationTracker();
    
    static AMErr CreateInstance(IAMAllocator* pAllocator, IAMGPSTracker** ppGPSTracker);
    static U32 AddRef(IAMGPSTracker* ppGPSTracker);
    static U32 Release(IAMGPSTracker* ppGPSTracker);    
    static AMErr SetUpdateInterval(IAMGPSTracker* pGPSTracker, S32 timer);

    AMErr StartTracking(AMStartTrackingCallback* pCallback, void *pUserData, LocationImpl* pGPSEngine, S32 timer);
    AMErr StopTracking();
    
    TInt HandleNotifyL(TPositionCourseInfo aOutPos , TInt aError);
    
private:
    AMErr Init();
    void CreateLocationServiceL();
    void SetUpdateInterval(S32 timer);
    
    AMStartTrackingCallback*    ipStartTrackCallBack;
    void*                   ipUserData;
    U32                     m_refCount;
    LocationImpl*              m_pLocationImpl;
    IAMAllocator*           m_pAllocator;
    TTime                   m_firstTime;
    TBool                   m_timeFlag;
    CLocationService* iLocationService;
};

#endif /* SYMBIAN_LOCATION_H_ */
