/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class which implements the 
*				 main hostengine functionality.
*
*/


#ifndef __SYMBIAN_MMC_MANAGER__

#define __SYMBIAN_MMC_MANAGER__

#include <e32base.h>
#include <f32file.h>

class CPlatformConfig;
class CPlatformFlashEngine;
class MMMCActivityObserver;

const TInt64 KMinThreshold = 1024 * 1024 * 20;	
const TInt64 KMaxThreshold = 1024 * 1024 * 100;
const TInt KMMCReservoirUsagePolicyConstant=0;
const TInt KMMCReservoirUsagePolicyVariable=1;
#ifdef FEATURE_MULTIPLE_DRIVE
const TInt KMMCReservoirOverridePolicyVariable=1;
#endif
_LIT(KVideoDrive,"E://");

struct TReadRequest
{
	TInt iOffset;
	TInt iDataLength;
};

class CMMCReservoir:public CActive
{
	public:
		CMMCReservoir(CPlatformFlashEngine* aEngine
								, MMMCActivityObserver* aObserver);	
		virtual ~CMMCReservoir();
		TInt InitializeL( TInt aStreamId, TInt aContentLength);
				
	public:	
		void WriteToMMCL( const TDesC8& aData, TInt aLength);
		TInt ReadFromMMCL( TInt aOffset, TInt aLength);
		TInt64 GetMaxSize();
		TInt CheckMMCStatusL(TInt aReadOffset, TInt aWriteLength)
			{
			if((iCycles) && ((iCycles * KMinThreshold + iWriteOffset +aWriteLength) > aReadOffset))
				{
				return EFalse;
				}
			return ETrue;
			}
		void CancelAllPendingRequests();			
		TInt IsDataInMMC(TInt aReadOffset)
			{
			if(GetSeekPoint(aReadOffset) >= 0)
				{
				return ETrue;
				}
			return EFalse;	
			}
		void RunL()	;
		TInt RunError(TInt aError);
		void DoCancel();
		TInt64 BytesWritten();
	private: 
	//utility functions	
		void GetFileName(TDes& aFilename, TInt aStreamID);	
		TInt MemoryCardDriveL();
		void ClearResidualTempFilesL();
		void DeleteTempFilesL(TPtr& aCacheDirectory);
		TInt CheckMemoryCardL(TInt aContentLength);
		TInt64 FreeSpaceL();
		void AddToQueueL(TInt aOffset, TInt aDataLength);
		#ifdef FEATURE_MULTIPLE_DRIVE
		TInt64 FreeSpaceL(TInt aDrive);
		#endif
		void RecycleL(const TDesC8& aData);
		TInt64 GetSeekPoint(TInt aOffset);
	private:
		RFile iFileHandle;
		CPlatformFlashEngine* iEngine;
		CPlatformConfig* iPlatformConfig;
		TInt iWriteOffset;		
		TInt iReadOffset;
		TInt iCycles;
		TBool iFileCreated ;TBool iFileToDestroy;
		TInt iRecycleThreshold;
		TInt iDrvNum;
		MMMCActivityObserver* iObserver;
		HBufC8* iTempDataBuffer;
		TPtr8 iTempDataBufferPtr;
		RPointerArray<TReadRequest> iRequestArray;
		
};
 
#endif //__SYMBIAN_MMC_MANAGER__