/*
* Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the classes used for 
*                shared object operations.
*
*/


#ifndef SYMBIAN_MSO_H_
#define SYMBIAN_MSO_H_

#include <e32base.h>
#include <f32file.h>

#include "engine_defs.h"

class CPlatformFlashEngine;
class CSharedObjectUtils;

class CSharedObjectStorage : public CBase
{
public:
    static CSharedObjectStorage* NewL(CPlatformFlashEngine& aEngine, const TDesC& aPersistentDir);    
    ~CSharedObjectStorage();
    
public:
    TInt StoreL(const char* rootMovieGUID, const char* name, const unsigned char* data, unsigned long dataSize);        
    TInt Read(const char* rootMovieGUID, const char* name, unsigned char* data, unsigned long dataSize);    
    TInt GetStoreSpace(const char* rootMovieGUID, const char* name,TInt& aSize);
    TInt RemoveDataL(const char* aNameSpace, const char* aName);
    
private:
    CSharedObjectStorage(CPlatformFlashEngine& aEngine);
    CSharedObjectStorage(); //private default constructor..no impl reqd
    void ConstructL(const TDesC& aPersistentDir);
    TBool IsEmptyDir(HBufC* msoNameSpace);
    
private:        
    CPlatformFlashEngine& iEngine;
    CSharedObjectUtils *iShObjUtils;
};


/*
 * Implements shared object handling logic.
 */

class CSharedObjectHandler : public CBase
{
public:
    ~CSharedObjectHandler(); //not virtual, this class not intended to overrided
    
    static CSharedObjectHandler* NewL(CPlatformFlashEngine& aEngine, const TDesC& aPersistentDir);
    
    TInt StorageDataSize(const char* rootMovieGUID, const char* name, unsigned long& aSize);
    TInt StoreData(const char* rootMovieGUID, const char* name,
            const unsigned char* data, unsigned long dataSize);
    
    TInt RetrieveData(const char* rootMovieGUID, const char* name,
            unsigned char* data, unsigned long dataSize);
    
    TInt RemoveData(const char* rootMovieGUID, const char* name);
    
    inline void GetMaxStorageSize(unsigned long& aSize) const
    {
        aSize = KMaxPersistentStoragePerMovie;        
    }
private:
    CSharedObjectHandler(CPlatformFlashEngine& aEngine);
    void ConstructL(const TDesC& aPersistentDir);
    
    HBufC* iPersistentDir;    
    CPlatformFlashEngine& iEngine;
    CSharedObjectStorage* iShObjStorage;
};

#endif /* SYMBIAN_MSO_H_ */
