/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class that provides shared object utilities
*
*/

#ifndef SYMBIAN_MSOUTILS_H_
#define SYMBIAN_MSOUTILS_H_

#include <e32base.h>
#include <f32file.h>
#include "symbian_engine.h"
#include "engine_defs.h"

class CFileNode : public CBase
{
public:    
    static CFileNode* NewL(TEntry &aFileEntry, HBufC *&aFilePath);
    TEntry GetFileEntry() { return iFileEntry; };
    HBufC* GetFilePath() { return iFilePath; };
    ~CFileNode();
    
public: // link data
    TSglQueLink iLinkItem;

private:
    CFileNode(TEntry &aFileEntry);
    void ConstructL(HBufC *&aFilePath);
    TEntry iFileEntry;  // Storing TEntry to get instance of iModified time
    HBufC* iFilePath; // Storing TFileName to get the Root Movie directory
};

class CSharedObjectUtils : public CBase, public MFlhAsyncHandler
{
public:
	static CSharedObjectUtils* NewL(CPlatformFlashEngine& aEngine, const TDesC& aPersistentDir);
	~CSharedObjectUtils();
private:
	CSharedObjectUtils(CPlatformFlashEngine& aEngine, const TDesC& aPersistentDir);
	void ConstructL();

public:
    HBufC* GetSharedObjectFileNameL(const char* aNameSpace, const char* aName);
    HBufC* GetPersistenceDirL(const char* aNameSpace) const;
    void MigrateMSODirectoryL(const TDesC& aMSOFileName, const TDesC& aNewMSODirectory);
    void StartMSOCleanup(unsigned long dataSize);
    	
	TBool IsDataSizeExceeded(const char* aNameSpace,  TDesC& msoFileName, unsigned long dataSize);
	TBool HasEnoughSpace(unsigned long dataSize) const;

	void DoRunL(const CActive* aActive);
	TInt DoRunError(const CActive* aActive, TInt aError);
	    
private:
    HBufC* GetFlash8PersistentDirectoryLC();
    HBufC* GetFlash8MSOFileLC(const TDesC& aMSOFileName);
    void MoveMSOFileToFlash9LocationL(const TDesC& aOldFile, const TDesC& aNewMSODirectory);
    TInt GetCurrentMovieSize (TDesC& msoFileName, TInt& aSize) const;
    TInt GetCurrentMSOSizeL(const char* aNameSpace, TInt& aSize, TSglQue<CFileNode>*& aFileList) const;        
    void DoMSOCleanupL();
    void DestoryFileList(TSglQue<CFileNode>* aFileList) const;
    
private:
	RFs                                iFs;
	CPlatformFlashEngine&              iEngine;
	const TDesC&                       iPersistentDir;
    TInt                               iCurrentMSOSize;
    TUint32                            iCurrentModeMaxStorageSize;
    TInt                               iSizeToCleanup;
    TInt                               iDeletedSize;
	CFlhAsyncObject*                   iAsyncObject;
	CFileMan*                          iFileManager;
};

#endif //SYMBIAN_MSOUTILS_H_
