/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class used by CPlatformFlashEngine to handle 
*				 tel: URLs and make phone calls. It is used with CPhoneMonitor class.
*
*/


#ifndef __SYMBIAN_PHONECALL_H__
#define __SYMBIAN_PHONECALL_H__

#include <e32base.h>
#include <etel3rdparty.h>

class CPhoneMonitor;

/**
 * The CPhoneCall class is used by CPlatformFlashEngine to handle tel: URLs and make
 * phone calls. It is used with CPhoneMonitor.
 * 
 */
class CPhoneCall : public CActive
    { 
		enum TCallState {
				kCallStateNone = 0,
				kCallStateDialling,
				kCallStateTalking,
				kCallStateTerminated
			};
	
private:
    CTelephony* iTelephony;
    CTelephony::TCallId iCallId;
    /*
       When the voice line status changes (and hence the asynchronous
       operation completes) the new voice line status is written into
       iLineStatus. Until this point, iLineStatus is not valid.

       If you change this class to get notification of other events,
       then change this class. CTelephony::TNotificationEvent lists
       the classes associated with each event.
       */
    CTelephony::TCallStatusV1 iLineStatus;

    CTelephony::TCallStatusV1Pckg iLineStatusPckg;

private:
	CPhoneMonitor* iPhoneMonitor;
	TCallState iCallState;
	
public:
    CPhoneCall(CPhoneMonitor* aPhoneMonitor);
	void InitialiseL();

	void StartCallL(const TDesC& aPhonenumber);

	TBool IsCallActive() ;

	void ResetCallState() ;
	
	virtual ~CPhoneCall();
	
private:
	void CallTerminated();
	
	CTelephony::TCallStatus GetCallStatus();

private:
    /*
       These are the pure virtual methods from CActive that 
       MUST be implemented by all active objects
       */
    void RunL();
    void DoCancel();
   };


#endif // __SYMBIAN_PHONECALL_H__
