/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class used by CPlatformFlashEngine to keep track of 
*				 the phone's call status -- the Flash Player will pause when 
*				 there is an incoming call. It is also used with CPhoneCall to 
*				 handle tel: URLs and initiate phone calls from Flash.
*
*/


#ifndef __SYMBIAN_PHONEMONITOR_H__
#define __SYMBIAN_PHONEMONITOR_H__

#include <e32base.h>
#include <etel3rdparty.h>
#include <e32property.h>
#include <commondef.h>

class CPlatformFlashEngine;
class CPhoneCall;

/**
 * The CPhoneMonitor class is used by CPlatformFlashEngine to keep track of the phone's
 * call status -- the Flash Player will pause when there is an incoming call. It is
 * also used with CPhoneCall to handle tel: URLs and initiate phone calls from Flash.
 * 
 */
class CPhoneMonitor : public CActive
{
public:
	CPhoneMonitor();
	~CPhoneMonitor();
	void ConstructL(CPlatformFlashEngine* aEngine);
	
	void MakeCallL(const TDesC& aPhoneNumber);
	void CallTerminated(CPhoneCall* aPhoneCall);
	TBool IsCallActive();
	TUint GetCallerInfoL(FI_ReturnString* aName,FI_ReturnString* aNumber);
	
	// from CActive
protected:
	void DoCancel();
	void RunL();
	TInt RunError(TInt aError);

private:
	TBool iNoMonitor;
    TBool iIsCallActive;
	CPlatformFlashEngine* iEngine;
	TRequestStatus iCallStatus;
	CPhoneCall* iPhoneCall;
	CTelephony* iTelephony;

	RProperty iProperty;
};


#endif /* __SYMBIAN_PHONEMONITOR_H__ */
