/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  defines the member functions of the class that is used to 
*				 hook to Adobe automated test execution.
*
*/

//#ifdef USE_ATS_REFERENCE

#ifndef __SYMBIAN_TIHOST_H__
#define __SYMBIAN_TIHOST_H__

// Header files related to Bluetooth
#include <es_sock.h>
#include <btdevice.h>
#include <bt_sock.h>
#include <btsdp.h>
#include <btmanclient.h>

#include "genericcommon.h"
#include "commondef.h"
#include "flashliteinterfaces.h"

enum FI_LSType
{
	LocalStoreOne,
	LocalStoreTwo,
	LocalStoreThree
};

enum FI_DisplayType
{
    FI_DisplayType_Total,
    FI_DisplayType_Player
};

enum ATS_Return
{
	ATS_ERROR = 0,
	ATS_OK,
	ATS_IN_AUTOSWF
};

enum FI_Operation
{
FI_Create,
FI_Append,
FI_Delete
};

// Functions to pass to ATS Test Library
typedef short			(*PFNC_TI_ALLOCATEBUFFER) (char** pBuf, unsigned long BufSize );
typedef short			(*PFNC_TI_RELEASEBUFFER) ( unsigned char* pBuf );
typedef short			(*PFNC_TI_GET_SPEC_CONTENTS) ( char *SpecName, char **pBuf, unsigned long *bufSize);
typedef short			(*PFNC_TI_GET_CURRENT_DISPLAY) (unsigned char *pMem, unsigned long MemSize, int XDim, int YDim, FI_DisplayFormat format, FI_DisplayType type);
typedef short			(*PFNC_TI_READ_FROM_LOCAL_STORE) (FI_LSType type, char **pBuf,	unsigned long *Size);
typedef short			(*PFNC_TI_WRITE_TO_LOCAL_STORE)(FI_LSType type, FI_Operation op, char *pBuf, unsigned long Size);
typedef short			(*PFNC_TI_SEND_TO_REMOTE_STORE) (unsigned char *pBuf, unsigned long Size);
typedef short			(*PFNC_TI_DESTROYPLAYER)(struct MM_Object *mm_obj,unsigned char* SwfToExecute);


typedef struct TI_Fncs
{
	PFNC_TI_ALLOCATEBUFFER						AllocateBuffer;
	PFNC_TI_RELEASEBUFFER						ReleaseBuffer;
	PFNC_TI_GET_SPEC_CONTENTS					GetSpecContents;
	PFNC_TI_GET_CURRENT_DISPLAY					GetCurrentDisplay;
	PFNC_TI_READ_FROM_LOCAL_STORE				ReadFromLocalStore;
	PFNC_TI_WRITE_TO_LOCAL_STORE				WriteToLocalStore;
	PFNC_TI_SEND_TO_REMOTE_STORE				SendToRemoteStore;
	PFNC_TI_DESTROYPLAYER						DestroyPlayer;
} TI_Fncs;

// Functions exported from ATS Test Library
typedef short			(*PFNC_ATS_SETUPTESTLIBRARY) (struct MM_Object *mm_obj,  struct TI_Fncs *hptrs, short UseUTF16,bool callURL);
typedef short			(*PFNC_ATS_SETUP_SI_FUNCS) (SI_Fncs *atsFunctions);

void SetupHostPtrs(TI_Fncs *hptr);


// structure that will hold the info about the remote device
struct TDeviceData
	{
	THostName iDeviceName;
	TBTDevAddr iDeviceAddr;
	TUint iDeviceServicePort;
	};

// for ease of use and clarity, the type definition for device data list
typedef RPointerArray<TDeviceData> TDeviceDataList;

class CServiceDiscoverer1 : public CBase,
						   public MSdpAgentNotifier,
						   public MSdpAttributeValueVisitor
{
public:

/*!
 * NewL()
 *
 * discussion Create new CServiceAdvertiser object
 * return a pointer to the created instance of CServiceAdvertiser
 */
	static CServiceDiscoverer1* NewL(RSocketServ& aServ, RSocket& aSock);

/*!
 * NewLC()
 *
 */
	static CServiceDiscoverer1* NewLC(RSocketServ& aServ, RSocket& aSock);

/*!
 * ~CServiceDiscoverer1()
 *
 * discussion Destroy the object and release all memory objects
 */
	~CServiceDiscoverer1();

/*!
 * DiscoverServiceOnDeviceL()
 *
 * discussion Discovers services on a given device.  service discovery
 * agent will be started to do the service discovery.  search pattern
 * will be used to limit the discovery for our services (matching service
 * id).
 *
 * param aDevData device data record
 */
	void DiscoverServicesOnDeviceL(TDeviceData* aDevData);

/*!
 * DiscoverServiceOnDeviceL()
 *
 * discussion Starts service discovery for all given devices.
 *
 * param aDevDataList list of devices to do the service discovery for.
 */
	void DiscoverServicesL(TDeviceDataList* aDevDataList, TInt* aStatus);

/*!
 * FinishDiscovery()
 *
 * discussion Stops the service discovery, stops the discovery agent.
 *
 * param aDevData device data record
 */
	void FinishDiscovery();

/*!
 * HasServices()
 *
 * discussion check if any services were discovered.
 *
 * return true if any usable services were discovered.
 */
	TBool HasServices();

void ServiceDiscoveryComplete();

private:

/*!
 * CServiceDiscoverer()
 *
 * discussion Perform the first phase of two phase construction
 */
	CServiceDiscoverer1(RSocketServ& aServ, RSocket& aSock);

/*!
 * ConstructL()
 *
 */
	void ConstructL();

/*!
 * HandleServiceDiscoveryCompleteL()
 *
 * discussion Handles the service discovery completed event.  the handling
 * request will be passed onto handling observer.
 */
	void HandleServiceDiscoveryCompleteL();

/*
 * NextRecordRequestComplete()
 *
 * discussion called when an service record
 *            (CSdpAgent::NextRecordRequestL()) operation completes.
 * param aError KErrNone, or an SDP error
 * param aHandle service record for which the query was made
 * param aTotalRecordsCount total number of matching records
 */
	void NextRecordRequestComplete(
		TInt aError,
		TSdpServRecordHandle aHandle,
		TInt aTotalRecordsCount);

/*
 * AttributeRequestResult()
 *
 * discussion Called by the attribute request
 *            (CSdpAgent::AttributeRequestL()) to pass the results of
 *            a successful attribute request.
 *
 * param aHandle service record for which the query was made
 * param aAttrID ID of the attribute obtained
 * param aAttrValue attribute value obtained
 */
	void AttributeRequestResult(
		TSdpServRecordHandle aHandle,
		TSdpAttributeID aAttrID,
		CSdpAttrValue* aAttrValue);

/*!
 * AttributeRequestComplete()
 *
 * discussion Called when the attribute request
 *            (CSdpAgent::AttributeRequestL()) completes.
 *
 * param aHandle service record for which the query was made
 * param aError KErrNone, or an SDP error
 */
    void AttributeRequestComplete(
		TSdpServRecordHandle aHandle,
		TInt aError);

/*!
 * VisitAttributeValueL()
 *
 * discussion Called to pass the attribute request values
 *
 * param aValue attribute value
 * param aType attribute type
 */
    void VisitAttributeValueL(CSdpAttrValue& aValue, TSdpElementType aType);
	// not needed -> not implemented
    void StartListL(CSdpAttrValueList &aList);
	// not needed -> not implemented
    void EndListL();
    
private:	// data

	// service discovery agent
	CSdpAgent* iAgent;
	// service discovery search pattern
	CSdpSearchPattern* iSpat;
	// last discovered uuid in the service attributes
	TUUID iLastUUID;
	// port (comm channel) found in the service attributes
	TUint iPort;
	// device data record reference
	TDeviceData* iDevData;
	// device data list reference
	TDeviceDataList* iDevDataList;
	// this indicates that a port number was found in the service attributes
	// and device data record for this device needs to be updated with the
	// port number.
	TBool iDevDataChanged;
	// index of device in device data list used for service discovery
	TInt iDeviceIdx;
	// number of services discovered
	TInt iDiscoveredServiceCount;
	// to update once we discover Service
	TInt* Status;
	//Socket Server Handle
	RSocketServ& iSocketServ;
	//Socket Handle
	RSocket&  iSock;
	
	TBTSockAddr iAddr;
	
};

#endif //__SYMBIAN_TIHOST_H__
//#endif /* USE_ATS_REFERENCE */