/*
* Copyright (c) 2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the the Trust store for list of trusted files.
*
*/


#ifndef SYMBIAN_TRUSTFILE_H_
#define SYMBIAN_TRUSTFILE_H_
#include <e32base.h>
#include <s32std.h>
#include <s32file.h>
#include <f32file.h>
#include <e32cmn.h>

//SecuritySandbox

class CFlashTrustStore : public CBase
    {

public:
    /*
     * Two-phased constructor
     * 
     * 
     * @return CFlashTrustStore* Pointer to an instance of CFlashTrustStore
     */
    static CFlashTrustStore* NewL();
    
    /*
     * Destructor
     * 
     */   

    ~CFlashTrustStore();
    /*
    *  create a corresponding file in the private directory
    * 
    * @param TPtrC& aSourceFileName is the file name whose hash value based file name will  be added
    */ 

    void TrustContentL(const TPtrC& aSourceFileName);
    

    /*
     * Check if the current flash content entry is present in the private Directory 
     * 
     * @param TPtrC& asourceFileName file name for which entry is to be checked
     *  
     * @return true if content is trusted else false
     */
    TBool IsContentTrustedL(const TPtrC& asourceFileName);
    /*
     * To get the num of enteries 
     * 
     * @return number of entries
     */

    TInt NumOfEntriesL();


    
protected:
    /*
     * Protected constructor.    
     */ 
    inline CFlashTrustStore();
    
     /*
      * Two-phased constructor
      * 
      */
    void ConstructL();
    /*
     * Get the Hash value for the given path
     * 
     * @param TPtrC aFileName Full path of the file name whose hash value is to be returned
     * 
     * @return HBufC8* Pointer to hash value of give path
     */
private:

    HBufC8* GetHashLC(const TDesC& aFileName) const;
    /*
     * Touch the file
     * 
     * @param TFileName  aFileName name of the file whose last modified time is changed to current value
     * 
     * @return void
     */
    
    void TouchFile(const TFileName  aFileName);
    /*
     * Function to convert binary to base 64
     * 
     * @param HBufC8* lHash Hash value to be converted to Base64
     * 
     * @return TPtr8 pointing the base64 value
     */
    
    TPtr8 ConvertHashToBase64LC(const HBufC8* lHash)const;
    /*
     * Function to get the hash value of give file name
     * 
     * @param TFileName& aFileName full path of the file whose hash value is returned
     * 
     * @return TPtr hashed value of filepath
      */

    HBufC* GetHashedFilePathL(const TDesC & aFileName)const;
    /*
     * Function to Cleanup the trust entries 
     * 
     * 
     */
    
    void CleanupTrustEntriesL();
    /*
     * Function to read from a File 
     * 
     * @param TFileName aFileName File whose value will be read
     * 
     * @return Read integer value 
     */
    
    TInt ReadFileStream(const TFileName  aFileName);
    /*
     * Function to write integer value to file
     * 
     * @param TFileName aFileName File name in to which value is written 
     * @param TInt numOfEntries Integer value to be written in to the file
     * 
     */
    void WriteFileStream(const TFileName  aFileName,TInt numOfEntries);
    
    static  int BinTo64(const TUint8* pInBuf, int len, char* pOutBuf);
    static  void Output64Chunk(int c1, int c2, int c3, int pads, char* pBuf,
                                        int bufOffset);
    private:  
    /*
     * Full path of the private directory path
     * 
     */
    TFileName iPrivateDirFullPath;
    /*
         * CEikon::Static()->fsSession() value stored for various file session activites
         * 
         */
    RFs ifsSession;
    };

// ---------------------------------------------------------------------------
//  construction
//
// ---------------------------------------------------------------------------


inline CFlashTrustStore::CFlashTrustStore()
    {
    
    }
#endif /*SYMBIAN_TRUSTFILE_H_*/
