/*
* Copyright (c) 2006-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description: 
*       Provides the CIADUpdateChecker definition.
*
*
*/



#ifndef __SYMBIAN_UPDATE_CHECKER_H__
#define __SYMBIAN_UPDATE_CHECKER_H__


#include "saplayer_build.h"
//#include "common_defs.h"

#include <iaupdateobserver.h>
#include <iaupdate.h>
#include <iaupdateparameters.h> 
#include <iaupdateresult.h>

//forward declaration
class MIAUpdateObserver;
class CIAUpdate;
class CIAUpdateParameters;
class MIADUpdateTimerCheck;
class MIadUpdateCallback;

const TUid KFlashLitePackageID = {0x101FD693};

class CIADUpdateChecker:	public CBase,
                            public MIAUpdateObserver
{
	public:
	    enum TIadUpdateStatus
	    {
	        KUpdateNotStarted,
	        KUpdateUserCancelled,  // not really required as the IAD client takes care of not showing 
	                               // the query if the user selects "later".
	                               // Keeping it so that we can save a few checks and function calls 
	                               // which would be needed for every swf in a page.
	        KUpdateInProgress,
	        KUpdateCompleted,
	        KUpdateFailed,
	        KUpdatesUnavailable
	    };

	 /**
     * Symbian constructor.
     *
     * @param aApplicationUID Uid describes either package UID or SID.
     *
     * @param aAppSearchCriteria When correct updates are searched,
     * this text is used as a criteria when checking the information 
     * that is attached to the updates.
     *
	 * @param aCommandLineExecutable When the update operation finishes,
     * the content of this desciptor will be used as a command line executable.    								
     *
     * @param aCommandLineArguments When the update operation finishes,
     * the content of this desciptor will be used as command line arguments
     * with the command line executable.
     *
     * @param aShowProgress If ETrue, a wait dialog is shown when
     * CIAUpdate::CheckUpdates operation is started. If EFalse,
     * the dialog is not shown.
     * @return Pointer to created CIADUpdateChecker object
	 *
     * @since S60 v3.2
     */
    static CIADUpdateChecker* NewL( const TUid& aApplicationUID
    								, const TDesC& aAppSearchCriteria
    								,const TDesC& aCommandLineExecutable 
    								,const TDesC8& aCommandLineArguments
    								,TBool aShowProgress
    								,MIadUpdateCallback* aIADUpdateCallbackInterface
    								);
   
    /**
     * Destructor.
     */
    virtual ~CIADUpdateChecker();
    
	private:
	
    /**
     * C++ default constructor.
     *
     * @param aApplicationUID Uid describes either package UID or SID.
     *
     * @param aAppSearchCriteria When correct updates are searched,
     * this text is used as a criteria when checking the information 
     * that is attached to the updates.
     *
	 * @param aCommandLineExecutable When the update operation finishes,
     * the content of this desciptor will be used as a command line executable.    								
     *
     * @param aCommandLineArguments When the update operation finishes,
     * the content of this desciptor will be used as command line arguments
     * with the command line executable.
	 *
	 * @param aShowProgress If ETrue, a wait dialog is shown when
     * CIAUpdate::CheckUpdates operation is started. If EFalse,
     * the dialog is not shown.
     */
    CIADUpdateChecker(const TUid& aApplicationUID
    								, const TDesC& aAppSearchCriteria
    								,const TDesC& aCommandLineExecutable 
    								,const TDesC8& aCommandLineArguments
    								,TBool aShowProgress
    								,MIadUpdateCallback* aIADUpdateCompleteCallback
    								);
    
    /**
         * By default Symbian 2nd phase constructor is private.
	     
    */
    void ConstructL();
   
	          // from MIAUpdateObserver
	          public:
	              /** 
	               * This callback function is called when the update checking operation has completed.
	               *
	               * @param aErrorCode The error code of the observed update operation.
	               *                   KErrNone for successful completion, 
	               *                   otherwise a system wide error code.
	               * @param aAvailableUpdates Number of the updates that were found available.
	               *
	               * @since S60 v3.2
	               */
	              virtual void CheckUpdatesComplete( TInt aErrorCode, TInt aAvailableUpdates );


	              /** 
	               * This callback function is called when an update operation has completed.
	               * Even if multiple functions are provided to start different update operations,
	               * this callback function is always called after an update operation has completed.
	               *
	               * @param aErrorCode The error code of the completed update operation.
	               *                   KErrNone for successful completion, 
	               *                   otherwise a system wide error code.
	               * @param aResult Details about the completed update operation.
	               *                Ownership is transferred.
	               *
	               * @since S60 v3.2
	               */
	              virtual void UpdateComplete( TInt aErrorCode, CIAUpdateResult* aResult );


	              /** 
	               * This callback function is called when an update query operation has completed.
	               *
	               * @param aErrorCode The error code of the observed query operation.
	               *                   KErrNone for successful completion, 
	               *                   otherwise a system wide error code.
	               * @param aUpdateNow ETrue informs that an update operation should be started.
	               *                   EFalse informs that there is no need to start an update
	               *                   operation.
	               * @since S60 v3.2
	               */
	              virtual void UpdateQueryComplete( TInt aErrorCode, TBool aUpdateNow );
	              
    
 
public: //CIADUpdateChecker
    
    	/** 
	     * This function is called to check for IAD feature support in platform.
	     *
         * @return TBool ,ETrue if feature supported.
	     *
	     * @since S60 v3.2
	     */
    
    	static TBool IsUpdateSupported();
    	
    	/** 
	     * This function is called to trigger check update.
	     * 
	     * @since S60 v3.2
	     */
    	
    	void CheckUpdateL();
    	
    	/*
    	 * This method is used to get the status of the upgrade process
    	 */
    	
    	TIadUpdateStatus UpdateStatus();
    	

private:
    static TBool IsIADFeatureSupportedL();

private: //attrib
 		
 		TUid iApplicationUID; //not owned
 		
 		const TDesC& iAppSearchCriteria ; //not owned
    	
 		const TDesC& iCommandLineExecutable ; //not owned
    								
 		const TDesC8& iCommandLineArguments; //not owned
    								
 		TBool iShowProgress ; //not owned
		
		CIAUpdate* iIADUpdateCheck; //owned
		    
		CIAUpdateParameters* iIADParam; //owned
		
		MIadUpdateCallback* iIADUpdateCompleteCallback;   // not owned
		
		TIadUpdateStatus iIADUpdateStatus;
};


#endif //__SYMBIAN_UPDATE_CHECKER_H__
