/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class used to wrap up a URL request call from the 
*				 core player. It is used to handle requests coming from SI_OpenURLStream().
*
*/


#ifndef __SYMBIAN_REQUEST_INFO_H__
#define __SYMBIAN_REQUEST_INFO_H__

#include "symbian_http.h"
#include "httprequestbase.h"
#include "mmcactivityobserver.h"

class CMMCReservoir;
class CPlatformConfig;
#undef __BUFFER_BEFORE_1ST_WRITE
#undef _TIMER_MSG
/**
 * The CUrlRequestInfo class is used to wrap up a URL request call from the core player.
 * It is used to handle requests coming from SI_OpenURLStream().
 * 
 */
class CUrlRequestInfo : public CHttpRequestInfo, public MFlhAsyncHandler
												, public MMMCActivityObserver
{
public:
	virtual ~CUrlRequestInfo();
	static CUrlRequestInfo* NewL(void* aOwner, struct MM_Object* aMMObj, 
		struct FI_URLStreamInfo* aStreamInfo, TUint aId);
protected:
	void ConstructL();
	CUrlRequestInfo(void* aOwner, struct MM_Object* aMMObj, TUint32 aId,
		struct FI_URLStreamInfo* aStreamInfo);

public: 
	virtual void DoRunL(const CActive* aActive);
	virtual TInt DoRunError(const CActive*, TInt aError) { return aError; }

	virtual void PrepareToBeServicedL();
#ifdef __BUFFER_BEFORE_1ST_WRITE	
	void WritePrivateDataL(const TDesC8& aDataPart);
#endif //__BUFFER_BEFORE_1ST_WRITE	
	virtual void HandleRequestFailed(TInt aError, TInt aHttpStatus);
	
	virtual void HandleHeaderResponseL(struct FI_URLStreamHeaderInfo& aHeaders, const TDesC8& aFinalUrl);
	virtual TBool HandleResponseDataL(const TDesC8& aDataPart);
	virtual void HandleResponseCompleteL(TInt aHttpStatusCode); // request has completed normally

	// for URL specific handler class
	void SetNetworkBuffer(struct FI_NetworkBufferInfo* aNetBuf);
	void SetPaused(MHTTPDataSupplier* /*pData*/) { /*pPausedData = pData;*/ }
	void SetContentLength(TInt aContentLength) 
	{
		iContentLength = aContentLength;
	}
	virtual FI_StreamMediaType GetStreamMediaType();
	void TriggerAsyncReadWrite();
#ifdef FEATURE_FLV_MMC	
	TInt GetRAMResSize();
	TBool SeekURLStreamFLV( unsigned int seekTime,
						   struct FI_NetworkBufferInfo* pNetBuf);
	unsigned int GetBytes();//Sathya
	unsigned int GetTotalBytes();
#endif //FEATURE_FLV_MMC	
	void ReadCompleteL(TInt aStatus, const TDesC8& aData);
	TInt NotifyError(TInt aStatus);
	void NotifyErrorL(TInt aStatus);

private:
// data reservoir related functions
	void WriteToRAMResL(const TDesC8& aData);
	void ClearRAMReservoir();
	TBool IsRAMResFull();
	void ClearReservoir();
	TBool IsDataInRAMRes(TInt aOffset);
	TInt ReadFromRAMResL(TDes8& aData,TInt aOffset, TInt aLength);
	void ShowErrorNoteL(const TDesC& aErrMsg);
	void WriteStreamDataToCoreL(const TDesC8& aData);
private:
	//void SetPausedBytes(unsigned long bytesToWrite) { }//numBytesPaused = bytesToWrite; }

	struct FI_NetworkBufferInfo* iNetBuf; // not owned

	//MHTTPDataSupplier* pPausedData;
	//unsigned int numBytesPaused;
	CFlhAsyncObject* pAsyncObject;
	TInt iNumBytesDwlded;
	TInt iNumBytesWritten;		
	TInt iContentLength;

#ifdef __BUFFER_BEFORE_1ST_WRITE
	HBufC8* iPrivateData;
#endif //__BUFFER_BEFORE_1ST_WRITE

#ifdef _TIMER_MSG
	CFlhTimerObject* iTimer;
	TTime iLastTimeStamp;
#endif //_TIMER_MSG
	TBool iDownloadComplete;
	FI_StreamMediaType iStreamMediaType;
#ifdef FEATURE_FLV_MMC
	CMMCReservoir* iMMCManager;
	TInt iRAMResSize;
	TBool iStreamCancelled;
	HBufC8* iRAMReservoir;
	TInt iRAMCycles;
	TBool iUseRAMReservoir;
	CFlhTimerObject* iMMCAsyncObject;
	TInt iReadOffset;
	CPlatformConfig* iPlatformConfig;
	TBool iUseDataReservoirs;
	TInt iRamOffset;
	TInt iLastErrorNote;
#endif //FEATURE_FLV_MMC	
};

#endif // #define __SYMBIAN_REQUEST_INFO_H__
