/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class used to enhance the interface of the Symbian 
*				 CVibraControl class.
*
*/


#ifndef __SYMBIAN_VIBRA_H__
#define __SYMBIAN_VIBRA_H__

#ifdef FEATURE_VIBRATION

//deprecated
//#include <vibractrl.h>

#include <hwrmvibra.h>
#include <e32base.h>

/**
 * The CVibrationHandler class is used to enhance the interface of the Symbian CVibraControl class.
 * Flash will make requests through SI_StartVibrate() and SI_StopVibrate(). Flash can request
 * periodic vibration (i.e. pulsing) which CVibraControl does not support. CVibrationHandler
 * adds support for periodic vibration and also converts the Flash request to something compatible
 * with CVibraControl.
 * 
 */
class CVibrationHandler : public CBase
{
public:
	CVibrationHandler();
	~CVibrationHandler();
	TInt16 StartVibrate(TUint16 aTimeOn, TUint16 aTimeOff, TUint16 aRepeat);
	TInt16 StopVibrate();
	void Tick();
	static TBool StaticTimerTick(TAny* aVibrationHandler);
private:
	TUint16 iOnTime;
	TUint16 iRepeats;
	CPeriodic* iTimer;
	//deprecated CVibraControl* iVibraCtrl;
	CHWRMVibra* iVibraCtrl;
};

#endif // FEATURE_VIBRATION

#endif // __SYMBIAN_VIBRA_H__
