/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class that manages instances of CVideoObject and 
*				 handles the Flash device video API.
 * functions
*
*/


#ifndef __SYMBIAN_VIDEOENGINE_H__
#define __SYMBIAN_VIDEOENGINE_H__

#include <e32base.h>
#include <videoplayer.h>
//#include <mmferrors.h>


#ifdef FEATURE_DEVICE_VIDEO

/**
 * The CVideoRequest class stores all of the item from FI_Video in a format
 * suitable for storing in a linked list.
 * 
 */
class CVideoRequest : public CBase
{
public:
	enum {
		ENoRequest = 0,
		EPlayRequest,
		EStopRequest,
		ECancelRequest,
		ECloseRequest,
		EPauseRequest,
		EResumeRequest,
		EMuteRequest,
		ELoopRequest,
		ESeekRequest,
		EDestroyRequest,
	};
public:
	static CVideoRequest* NewL(const FI_Video* aVideoRequest, const TInt aRequestType);
	static CVideoRequest* NewL(const TUint32 aVideoI, const TInt aRequestType);
	~CVideoRequest();
private:
	CVideoRequest(const TInt aRequestType);
	void ConstructL(const FI_Video* aVideoRequest);
public:
   TUint32 iVideoID;
   HBufC* iPlaySrc;
   TPtrC8 iVideoData;
   TInt iIntegerArg;
public:
	TInt iRequestType;
public: // link data
	TSglQueLink iLinkItem;
};

class CVideoObject;

/**
 * The CVideoEngine class manages instances of CVideoObject and handles the Flash device video API
 * functions: SI_CreateVideo(), SI_PlayVideo(), SI_StopVideo(), SI_PauseVideo(), SI_ResumeVideo(),
 * SI_DestroyVideo(), SI_CloseVideo, SI_GetVideoMethods(), SI_ProcessVideoMethod() and SI_SetVideoRect().
 */
class CVideoEngine : public CBase, public MFlhAsyncHandler
{
public: // Constructors and destructor
	enum TPlayerErrorCode
	{
		EErrReOpen = 101,
		EErrAutoDisconnect,
		EErrAudioLost
	};

	enum VideoMethod
	{
		EVideoMethod_Mute = 0,
		EVideoMethod_Seek,
		EVideoMethod_Loop
	};
	
	static CVideoEngine* NewL(CPlatformFlashEngine& aEngine, RFs& aFs);	
	static CVideoEngine* NewLC(CPlatformFlashEngine& aEngine, RFs& aFs);
	~CVideoEngine();

	// from MFlhAsyncHandler	
	void DoRunL(const CActive* aActive);
	TInt DoRunError(const CActive* aActive, TInt aError);

private:
	void ConstructL();
	CVideoEngine(CPlatformFlashEngine& aEngine, RFs& aFs);
	
public:
	CPlatformFlashEngine&	GetFlashEngine() { return iEngine; }
	RFs&					GetRFs() { return iFs; }

public:
	/************************************************************************/
	/* Interface for SI functions                                           */
	/************************************************************************/
	TInt16	CreateVideoL(const FI_Rect* aLocation, TUint32* aRetVideoID);
	TInt16	PlayVideoL(const FI_Video* aVideoInfo);
	void	CloseVideo(TUint32 aVideoID);
	TInt16	CancelVideoL(TUint32 aVideoID);
	TInt16	StopVideoL(TUint32 aVideoID);
	TInt16	PauseVideoL(TUint32 aVideoID);
	TInt16	ResumeVideoL(TUint32 aVideoID);
	TInt16	ProcessVideoMethodL(TUint32 aVideoID, const FI_Method* aMethod, FI_ReturnString* aRetString);
	void	DestroyVideoL(TUint32 aVideoID);
	TInt16	SupportedVideoMethods(FI_ReturnString* aMethods);
	void	SetVideoRectL(TUint32 aVideoID, const FI_Rect* aLocation);
	TSize   GetVideoSize(TUint32 aVideoID);	
	void	DeleteAllVideoObjects();
	void    NotifyNewVolumeLevelL(TInt aNewVolLevel);
	void	SetActiveVideoID(TUint32 id) { iActiveVideoId = id; }
    TBool   HasActivePlayback();
public:
	/************************************************************************/
	/* Interface for Platform Engine                                        */
	/************************************************************************/
	
	void    NotifySizeChangedL();
	void    NotifyScrollL();
	TUint32	VideoNowPlaying() { return iActiveVideoId; };
	void    SetVideoWindow(RWindow* aWindow);


protected:
	// Look up the VideoObject from list
	CVideoObject*	FindVideoObject(TUint32 aVideoID);
	void			CloseOtherVideos(TUint32 aVideoID);

private:
	CPlatformFlashEngine&			iEngine;
	RFs&							iFs;
				
	// Maintain a list of video objects
	TInt							iVideoObjectIdIndex;
	TSglQue<CVideoObject>			iVideoList;
	CFlhAsyncObject*				iAsyncObject;
	TSglQue<CVideoRequest>			iRequestList;
	TUint32							iActiveVideoId;
	FI_Rect                         iVideoRect;
};

#endif	// FEATURE_DEVICE_VIDEO

#endif	// __SYMBIAN_VIDEOENGINE_H__
