/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
                                 
*
*/


#ifndef __SYMBIAN_VIDEOOBJECT_H__
#define __SYMBIAN_VIDEOOBJECT_H__

#include <e32base.h>
#include <videoplayer.h>
//#include <mmferrors.h>
#include <coemain.h>

#ifdef FEATURE_DEVICE_VIDEO

#include <DocumentHandler.h>

#include "ipconn_mgr.h"

#include "conn_handler.h"
class CPlatformFlashEngine;
class CVideoPlayerUtility;
class CVideoEngine;
class CVideoRequest;

// This is a temporary work-around until Nokia can provide support for OpenDesL()
#define USING_TEMP_FILE_FOR_OPENDESL

#ifdef USING_TEMP_FILE_FOR_OPENDESL
	// Use this flag to support asynchronous file write of temp file
	#define ASYNC_FILE_WRITE
#endif

/**
 * The CVideoObject class encapsulates a single instance of a Flash video object. It works with
 * CVideoEngine to support the Flash device video API.
 */
#ifdef ASYNC_FILE_WRITE
class CVideoObject : public CActive
					, public MVideoPlayerUtilityObserver
					, public MFlhIPConnectObserver
					, public MFlhAsyncHandler
#else
class CVideoObject : public CBase
					, public MVideoPlayerUtilityObserver
					, public MFlhIpUser
					, public MFlhIPConnectObserver
#endif
{
public:
	enum TPlayerState
	{
		ENotInitialized,
		EInitializing,
        ELoading,
		EStopped,
		EStoppedBecauseOfError,
        EDone,
		EPlaying,
		EPaused,
        EClosed,
        EError
	};
	
	CVideoObject(CVideoEngine& aVideoEngine);
	virtual ~CVideoObject();

	// Create the player with the display rect
	TBool InitPlayerL(CPlatformFlashEngine* aEngine, const FI_Rect& aLocation, TInt aInitialVolume);

    // Set video to given rectangle. Overrides previously set rect.
    void SetRectL(const FI_Rect& aLocation, TBool aForceUpdate = EFalse);
    
    TSize GetVideoSize();
	
	/**
	* This method plays a video clip.
	* @return void
	*/
	void PlayL(const CVideoRequest& aVideoRequest);

	/**
	* This method pauses a playing video clip.
	* @return TRUE if video was playing and successfully paused. If video was
    *         not playing, returns FALSE.
	*/
	BOOL PauseL();

    void Resume();
    
    
	void CancelL();
		
	/**
	* This method stops a playing video clip.
 	* @return void
	*/
	void StopL();

	/**
	* This method stops and close a playing video clip.
 	* @return void
	*/
	void Close();

	/**
	* This method gets the state of video player engine.
	* @return player state
	*/
	TPlayerState State() const { return iState; }

	// Get/Set the id of this object
	void SetId(TUint32 aId) { iId = aId; }
	TUint32 Id() const { return iId; }

	// Get/Set current position... these functions are used to support Seek
	TInt GetPositionL();

	//Uses SetPositionL to seek, this starts playing from the frame as well 
	void SeekL(TInt aPosition);

	// Support looping for video
	TBool IsLooping() { return iLoop; }
	void SetLooping(TBool aLoop) { iLoop = aLoop; }

	// Change the volume of video
	TBool IsMuted();
	void MuteL(TBool aMute);

	// called when the user selects a new volume level
	void NotifyNewVolumeLevelL(TInt aNewVolLevel);
	
	void HandleSizeChangedL();
	void HandleScrollL();
	
	void SetVideoWindow(RWindow* aWindow);
	
	TBool EnsureDirectory(const TDesC& aDirectory); 


public: // from MFlhIpUser
	void HandleNetworkAccessDenied();
	
	void DoRunL(const CActive* aActive);
	TInt DoRunError(const CActive* aActive, TInt aError);

public: // Functions from MVideoPlayerUtilityObserver
    /**
	* See MVideoPlayerUtilityObserver::MvpuoOpenComplete(TInt aError).
	*/
	void MvpuoOpenComplete(TInt aError);
	
	/**
	* See MVideoPlayerUtilityObserver::MvpuoPrepareComplete(TInt aError).
	*/
	void MvpuoPrepareComplete(TInt aError);
	
	void MvpuoPrepareCompleteL(TInt aError);
	
	/**
	* See MVideoPlayerUtilityObserver::MvpuoFrameReady(CFbsBitmap& aFrame,TInt aError).
	*/
	void MvpuoFrameReady(CFbsBitmap& aFrame,TInt aError);
	
	/**
	* See MVideoPlayerUtilityObserver::MvpuoPlayComplete(TInt aError).
	*/
	void MvpuoPlayComplete(TInt aError);
	
	/**
	* See MVideoPlayerUtilityObserver::MvpuoEvent(const TMMFEvent& aEvent).
	*/
	void MvpuoEvent(const TMMFEvent& aEvent);

public: //MFlhIPConnectObserver
	virtual void ConnectionEventL(TFlhIPEvent aEvent);
	
#ifdef ASYNC_FILE_WRITE
public: // Functions from CActive
	void RunL();
	void DoCancel();
	TInt RunError(TInt aError);

#endif // ASYNC_FILE_WRITE
	
protected:
	// This function extract filepath from file:// and perform any necessary conversion
	HBufC* GetFilePathLC(const TDesC& aUrl);
	HBufC* GetReferenceFilePathLC(const TDesC& aUrl); // Handles reference to a local video file.
	void PlayBufferFromFileL(const CVideoRequest& aVideoRequest);
    void Reset();

private:
	void CalculateAndSetVolumeL(); // only call when the track is loaded!
	void UpdateRectL(const FI_Rect& aLocation);
	
	TBool IsStreamSoundPlaying();
	void StartNetworkConnectionL();
    void HandleConnectionL();

private:
	CVideoEngine&					iVideoEngine;
	
	// The state of video player engine
	TUint32							iId;
	TPlayerState					iState;
	
#ifndef USE_VPU2	
	CVideoPlayerUtility*			iPlayer;	
#else
    CVideoPlayerUtility2*           iPlayer;    
#endif

    TRect                           iRect; // Video rectangle.
    
    FI_Rect                         iLocation; //Location with respect the frame buffer
    
    TBool							iNeedsToStop;

	
	// volume related stuff

	TBool							iIsMutedByAS;
	TInt							iUserVolume;
	TBool							iLoop;

	//The new window where the video is to be played, 
    //created with the View window as its parrent.
    RWindow*            			iParentWindow;
#ifdef USE_VPU2        
    RWindow*                        iCurrentDisplayWindow;
#endif

    CCoeEnv*                        iCoeEnv;
	
	HBufC*							iUrlToOpen; // for streaming
	TBool                           iIsStreaming;

	TBool							iNeedsToPause;

public: // link data
	TSglQueLink						iLinkItem;

#ifdef USING_TEMP_FILE_FOR_OPENDESL
	TFileName						iTempFileName;
	
#ifdef ASYNC_FILE_WRITE
private:
	RFile*							iTempFile;
#endif	// ASYNC_FILE_WRITE

#endif	// USING_TEMP_FILE_FOR_OPENDESL
private:
	CConnHandler* iConnHandler;
	CIPConnMgr* iIPConnectionMgr;
	CFlhTimerObject* iTimer;

	FI_Rect                         iCurrentLocation;
};


	
#endif	// FEATURE_DEVICE_VIDEO

#endif	// __SYMBIAN_VIDEOOBJECT_H__
