/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the class that provides a mechanism for priming a timer
*				 using CTimer functionality, and receiving a callback in the owning class.
*
*/


#ifndef __TIMER_OBJECT_H__
#define __TIMER_OBJECT_H__

#include <e32std.h>
#include <e32base.h>

#include "async_object.h"

/**
 * The CFlhTimerObject class provides a mechanism for priming a timer
 * using CTimer functionality, and receiving a callback in the owning class.
 */
class CFlhTimerObject : public CTimer
{
protected:
	CFlhTimerObject(TInt aPriority, MFlhAsyncHandler& aHandler);

public: // c'tor / d'tor
	static CFlhTimerObject* NewL(TInt aPriority, MFlhAsyncHandler& aHandler);
	~CFlhTimerObject();


public:
	// no API - for API see CTimer class!

private: // from CTimer
	void RunL();
	TInt RunError(TInt aError);
private:
	MFlhAsyncHandler& iHandler;
};

class CFlhTimerObject2 : public CFlhTimerObject
{
private:
	// required as the base class constructor CFlhTimerObject expects a reference to a MFlhAsyncHandler object
	class TAsyncHandler : public MFlhAsyncHandler
	{
	public:
		void DoRunL(const CActive* /*aActive*/) {}
		TInt DoRunError(const CActive* /*aActive*/, TInt aError) {return aError;}
	};

	
public:
	static CFlhTimerObject2* NewL(TInt aPriority, TCallBack aTimerCallback);
	virtual ~CFlhTimerObject2();

protected:
	CFlhTimerObject2(TInt aPriority, TCallBack aFPTimerCallback);
	
private:
	void RunL();
	TInt RunError(TInt aError);

	TCallBack		iTimerCallback;
	
	TAsyncHandler	iAsyncHandler;
	
};
#endif	// __TIMER_OBJECT_H__

