/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the Async Event class.
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "saplayer_build.h"

#include "async_event.h"
#include "symbian_engine.h"
#include "symbian_ui_interface.h"

/**
 *  The CFlhAsyncEvent class is for handling asynchronous events that are triggered by SI_ function calls:
 *  SI_SetFullScreen
 *  SI_SetSoftkeys
 *  SI_GetInputText
 */

/**
 * The NewL function is a factory method.
 *
 * @param aEngine pointer to the CPlatformFlashEngine instance.
 *
 * @return a new CFlhAsyncEvent instance transferring ownership to the caller.
 */
CFlhAsyncEvent* CFlhAsyncEvent::NewL(CPlatformFlashEngine* aEngine)
{
	CFlhAsyncEvent* o = new(ELeave) CFlhAsyncEvent();
	CleanupStack::PushL(o);
	o->ConstructL(aEngine);
	CleanupStack::Pop(o);
	return o;
}

/**
 * Constructor.
 */
CFlhAsyncEvent::CFlhAsyncEvent()
{
}

/**
 * Destructor.
 */
CFlhAsyncEvent::~CFlhAsyncEvent()
{
	delete iEventHandler;
}

/**
 * Two stage construction method.
 *
 * @param aEngine pointer to the CPlatformFlashEngine instance.
 */
void CFlhAsyncEvent::ConstructL(CPlatformFlashEngine* aEngine)
{
	iEngine = aEngine;
	iEventHandler = CFlhAsyncObject::NewL(CActive::EPriorityHigh,*this); // run before absolutely anything else...
}

/**
 * DoRunL -- handles ActiveObject::RunL()
 *
 * @param aActive ignored
 * 
 */
void CFlhAsyncEvent::DoRunL(const CActive* /*aActive*/)
{
	iEngine->HandleAsyncEventsL();
}

/**
 * DoRunError -- handles ActiveObject::RunError()
 *
 * @param aActive ignored
 * @param aError error code from RunError().
 *
 * @return the error code that will be passed on to the OS.
 */
TInt CFlhAsyncEvent::DoRunError(const CActive* /*aActive*/, TInt aError)
{
	if(iLeaveIsCritical)
	{
		TInt flashErrorCode=EGeneralError;
		if(aError==KErrNoMemory)
		{
			flashErrorCode=EInsufficientMemoryError;
		}
		
		iEngine->GetUserNotifierIf()->NotifyClientError(flashErrorCode, ETrue);
		
		/*
		if(!(iEngine->IsBrowserPluginActive()))
		    {
		    iEngine->GetUI()->NotifyClientError(flashErrorCode, ETrue);    
		    }		
		*/
		aError=KErrNone; // don't show the system's error dialog...		
	}
	
	return aError;
}

/**
 * TriggerEvent -- called by CPlatformFlashEngine to trigger a delayed event.
 */
void CFlhAsyncEvent::TriggerEvent()
{
	if(!iEventHandler->IsActive())
	{
		iEventHandler->Start();
	}
}

/**
 * SetLeaveIsCritical -- informs object that if triggered event causes an exception that
 *  user should be informed
 *
 * @param aLeaveIsCritical boolean to indicate the severity of an exception.
 */
void CFlhAsyncEvent::SetLeaveIsCritical(TBool aLeaveIsCritical)
{
	iLeaveIsCritical=aLeaveIsCritical;
}
