/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Impelements wrapper over ATS code.
*				 
*
*/


#include <Aknutils.h>
#include <EIKENV.H> 
#include "symbian_engine.h"


_LIT(KATSDllName, "atstest.dll");

short TI_WriteToLocalStore(FI_LSType type,
						   FI_Operation op,
						   char *pBuf,
						   unsigned long Size);
short TI_ReadFromLocalStore(FI_LSType type,
							char **pBuf,
							unsigned long *Size);


/*
----------------------------------------------------------------------------
 CATSHandler::NewL()

 Creates instance of CATSHandler.
----------------------------------------------------------------------------
*/
CATSHandler* CATSHandler::NewL(CPlatformFlashEngine* aEngine)
	{
	CATSHandler* self = new(ELeave)CATSHandler(aEngine);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
	}
/*
----------------------------------------------------------------------------
 CATSHandler::CATSHandler()

 Constructor
----------------------------------------------------------------------------
*/
CATSHandler::CATSHandler(CPlatformFlashEngine* aEngine)
	{
	iFncGetInterface = NULL;
	iATS_SetupTestLibrary = NULL;
	iPeriodicTimer = NULL;
	iFlashEngine = aEngine;
	}

/*
----------------------------------------------------------------------------
 CATSHandler::ConstructL()

 Second Phase construction
----------------------------------------------------------------------------
*/
void CATSHandler::ConstructL()
	{	
	User::LeaveIfError(iATSTestLibrary.Load(KATSDllName));  
	iFncGetInterface = (PFNC_ATSTEST_GETINTERFACE)iATSTestLibrary.Lookup(1);
	//cleanup previous results
	char* p = NULL;
	unsigned long size = 0;
	short ret = TI_ReadFromLocalStore(LocalStoreOne,&p,&size);
	if(size > 0)
		{
		TI_WriteToLocalStore(LocalStoreOne,FI_Delete,(char *)0,0);	
		}
	
	ret = TI_ReadFromLocalStore(LocalStoreTwo,&p,&size);
	if(size > 0)
		{
		TI_WriteToLocalStore(LocalStoreTwo,FI_Delete,(char *)0,0);	
		}	
	}

/*
----------------------------------------------------------------------------
 CATSHandler::~CATSHandler()
 
 Destructor
---------------------------------------------------------------------------
*/
CATSHandler::~CATSHandler()
	{
  UnLoadATSLibrary();    
  //added for loading/unloading lib only once
  if(iATSTestLibrary.Handle())
      {
      iATSTestLibrary.Close();    
      }
	iFlashEngine = NULL;
	}
/*
----------------------------------------------------------------------------
 CATSHandler::LoadATSLibraryL()

 Loads ATS Library.
----------------------------------------------------------------------------
*/
void CATSHandler::LoadATSLibraryL(SI_Fncs* aATSFunction,SI_Fncs* aSIFunctions)
	{
	
	PFNC_ATS_SETUP_SI_FUNCS pATS_SetupSIFuncs = NULL;
	
	Mem::Copy ( (void *)aATSFunction, (void *)aSIFunctions, (*aSIFunctions).sizeof_SI_Fncs );

	if (iFncGetInterface == NULL)
		{	
		return;
		}
	else
		{
		iPeriodicTimer = CPeriodic::NewL ( CActive::EPriorityLow );
		if(iPeriodicTimer)
			{
			iPeriodicTimer->Start ( 500000, 500000, TCallBack(ATSRefreshScreen,(TAny *) iFlashEngine) );
			}
					
		iATS_SetupTestLibrary = (PFNC_ATS_SETUPTESTLIBRARY)iFncGetInterface(EIndex_ATSSetupTestLibrary);
		pATS_SetupSIFuncs = 	(PFNC_ATS_SETUP_SI_FUNCS)iFncGetInterface(EIndex_ATSSetupSIFunctions);
	
		if (iATS_SetupTestLibrary && pATS_SetupSIFuncs)
			{
			short status = pATS_SetupSIFuncs(aATSFunction);
			if (status == 0)	
				return;
			}	 
		else 
			{
			iATS_SetupTestLibrary = NULL;
			}	
		}	
	}

/*
----------------------------------------------------------------------------
 CATSHandler::SetUpATSLibrary()

 Sets up TI_Fns 
----------------------------------------------------------------------------
*/
void CATSHandler::SetUpATSLibrary(struct MM_Object* aMMObject,unsigned long afiConfigFlags)
	{

	TI_Fncs hostptrs;
	SetupHostPtrs(&hostptrs);

	if (iATS_SetupTestLibrary != NULL) 
			{
			short status = 0;
			if (afiConfigFlags & FI_UseUTF16EncodingForm)
				status = iATS_SetupTestLibrary(aMMObject, &hostptrs, 1, true);
			else
				status = iATS_SetupTestLibrary(aMMObject, &hostptrs, 0,true);
			if (status == 0)
				return;
			}

	}

/*----------------------------------------------------------------------------
 CATSHandler::UnLoadATSLibrary()
	
 Unloads ATS Library.
----------------------------------------------------------------------------
*/
void CATSHandler::UnLoadATSLibrary()
	{
	iATS_SetupTestLibrary = NULL;
	if(iPeriodicTimer)
			{
	    iPeriodicTimer->Cancel();
	    delete iPeriodicTimer;
	    iPeriodicTimer = NULL;
			}	

	}

TInt CATSHandler::ATSRefreshScreen(TAny *data)
	{
  CPlatformFlashEngine *lEngine = (CPlatformFlashEngine *)data;
  if(lEngine)
      {
      lEngine->RefreshScreen();
      }
    return 1;
	}


