/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the CBrowserURLRequestInfo class, which maintains
*				 info about a url request made to the browser & interfaces with 
*				 the core
*
*/

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "saplayer_build.h"
#include <AknGlobalNote.h>
#include <StringLoader.h>
#include "browserurlrequestinfo.h"
#include "symbian_mmc_reservoir.h"
#include "platformconfig.h"
#include <flash2engine.rsg>
#include <hal.h>
#include "flashengineresourcehandler.h"

//#define _LOG_ERROR_
//#define __DUMP_FLV_TO_FILE


const TInt KWriteInterval = 2; // two microseconds

extern void NPPDebug(TUint8 aLogLevel,TBool aIsPlugin,TRefByValue<const TDesC> /*aText*/, ... );

// -----------------------------------------------------------------------------------------
// CBrowserURLRequestInfo* CBrowserURLRequestInfo::NewL(TInt aStreamID
//														, CPlatformFlashEngine* aEngine)	
// CBrowserURLRequestInfo object creator
// -----------------------------------------------------------------------------------------
//
CBrowserURLRequestInfo* CBrowserURLRequestInfo::NewL( CPlatformFlashEngine* aEngine, TInt aStreamID,
													const char* aFinalURL, TInt aContentLength, TDesC8& aContentType)
{
	CBrowserURLRequestInfo* self = CBrowserURLRequestInfo::NewLC(aEngine, aStreamID, aFinalURL,aContentLength, aContentType );
	CleanupStack::Pop(self);
	return self;
	
}
// -----------------------------------------------------------------------------------------
// CBrowserURLRequestInfo* CBrowserURLRequestInfo::NewL(TInt aStreamID
//														, CPlatformFlashEngine* aEngine)
//
// CBrowserURLRequestInfo object creator
// -----------------------------------------------------------------------------------------
//

CBrowserURLRequestInfo* CBrowserURLRequestInfo::NewLC( CPlatformFlashEngine* aEngine, TInt aStreamID,
													const char* aFinalURL, TInt aContentLength, TDesC8& aContentType)
{
	CBrowserURLRequestInfo* self = new (ELeave) CBrowserURLRequestInfo( aEngine);	
	CleanupStack::PushL(self);
	self->ConstructL( aFinalURL, aContentLength, aContentType, aStreamID);
	return self;
	
}

// -----------------------------------------------------------------------------------------
// CBrowserURLRequestInfo::~CBrowserURLRequestInfo()
// destructor
// -----------------------------------------------------------------------------------------
//		   
CBrowserURLRequestInfo::~CBrowserURLRequestInfo()
	{
	delete iMMCReservoir;	
	iMMCReservoir = NULL;
	
	delete iAsyncObject;
	iAsyncObject = NULL;
	
	delete iRAMReservoir;
	iRAMReservoir = NULL;
	
	DestroyHeaderInfo()	;
	}
// -----------------------------------------------------------------------------------------
// void CBrowserURLRequestInfo::DestroyHeaderInfo() 
//
// Deletes the header info object
// -----------------------------------------------------------------------------------------
//		   
    
 void CBrowserURLRequestInfo::DestroyHeaderInfo() 
	{
 	if(iHeaderInfo.headerNames)
		{
		delete [] iHeaderInfo.headerNames[iHeaderInfo.numHeaders -1];
		delete [] iHeaderInfo.headerNames;
		}
	if(iHeaderInfo.headerValues)
		{
		delete [] iHeaderInfo.headerValues[iHeaderInfo.numHeaders -1];				
		delete [] iHeaderInfo.headerValues ;		
		}
	
	}
// -----------------------------------------------------------------------------------------
// CBrowserURLRequestInfo::CBrowserURLRequestInfo(CPlatformFlashEngine* aEngine)
//													  	
//
// First phase constructor
// -----------------------------------------------------------------------------------------
//      
CBrowserURLRequestInfo::CBrowserURLRequestInfo(CPlatformFlashEngine* aEngine):
													  	 iReadOffset(0)
													  	,iWriteOffset(0)
													  	,iEngine(aEngine)
													  	,iRAMReservoir(NULL)
													  	,iRAMCycles(0)													  	
													  	,iUseRAMReservoir(EFalse)
													  	,iStreamCancelled(EFalse)
													  	,iCancelState(EFalse)
													  	,iRamOffset(0)
							
	{
	 	iWriteStatus = FI_Result(-1);
        iLastErrorNote = 0;
	}
	
// translates the error code to the resource id corresponding to the string to be displayed.	
TInt GetErrorNote(TInt aErrorCode)
	{
	if( aErrorCode == KErrNotFound ||aErrorCode == KErrNotReady)
		{
		return R_FLASH_INSUFFICIENT_MEMORY_MMC_NEEDED;
		}
	else if( aErrorCode == KErrNoMemory )
		{
		return R_FLASH_MMC_MEMORY_LOW;
		}
	else 
		{
		return R_FLASH_MMC_ACCESS_ERROR;		
		}	
	}
// -----------------------------------------------------------------------------------------
// void CBrowserURLRequestInfo::ConstructL()

// Second phase constructor
// -----------------------------------------------------------------------------------------
//	
void CBrowserURLRequestInfo::ConstructL(const char* aFinalURL, TInt aContentLength, TDesC8& aContentType, TInt aStreamID)
	{	
	//Add content length to header
	//FI_URLStreamHeaderInfo headerInfo;

	iEngine->AddToHeaderInfoL(iHeaderInfo, aContentLength, aContentType);
	
	// send headers to core player
	short errorCode = KErrNone;
	iNetworkBufferInfo =  iEngine->GetFIObject().FI_URLStreamOpened( aStreamID, &iHeaderInfo, &errorCode);
//Camp : Removed
/*	
	if( iNetworkBufferInfo )
		{
		iNetworkBufferInfo->streamId = aStreamID; 
		}
*/		
	iStreamID = aStreamID;
 
    if( errorCode <= 0 )
		{
		User::Leave(KErrGeneral); // could map to flash error codes, HTTP queue does not care at present
		}
	
	iEngine->GetFIObject().FI_URLStreamSetFinalURL( aStreamID, aFinalURL, FI_UseDefault);

	if(!iNetworkBufferInfo)
		return;

	//Mp3Streaming
	if(iNetworkBufferInfo->streamMediaType == FI_StreamMediaType_Sound)
	    {
	        iEngine->NewMP3StreamStarted();
	    }
	//for sound streaming  numbers of buffers insterted in coreplayer =  ((SOUNDSTREAM_CYCLING_BUFFER_COUNT/2)-1) 
	if((iNetworkBufferInfo->streamMediaType == FI_StreamMediaType_FLV && aContentLength > TELESTREAM_CYCLING_BUFFER_SIZE * TELESTREAM_CYCLING_BUFFER_COUNT)||
	    (iNetworkBufferInfo->streamMediaType == FI_StreamMediaType_Sound&& aContentLength > SOUNDSTREAM_CYCLING_BUFFER_SIZE * ((SOUNDSTREAM_CYCLING_BUFFER_COUNT/2)-1))) 
									
		{
			iUseRAMReservoir = ETrue;
			
			iMMCReservoir = new CMMCReservoir(iEngine, this);
			
			if(iMMCReservoir)
				{
				TInt error = iMMCReservoir->InitializeL(aStreamID,aContentLength);
				if( error != KErrNone )
					{
					//problem initializing MMC reservoir, delete it.
					HBufC* errorNote = NULL;
					TInt resoffset = FlashEngineRscHandler::LoadEngineResourceFileL();
					TInt errorResID = GetErrorNote(error);
					errorNote = StringLoader::LoadLC( errorResID );
					ShowErrorNoteL( *errorNote );
					CleanupStack::PopAndDestroy( errorNote );
					delete iMMCReservoir; 
					iMMCReservoir = NULL;
					FlashEngineRscHandler::DeleteEngineResourceFile(iEngine->GetEikonEnv(), resoffset);
					}
				}
								
				TInt freeRAM;
				HAL::Get(HAL::EMemoryRAMFree, freeRAM);

				TInt32 ramHighWatermark = iEngine->GetPlatformConfig()->GetRAMResHighWatermark();
				TInt32 ramLowWatermark = iEngine->GetPlatformConfig()->GetRAMResLowWatermark();
				TInt32 ramUnUtilSpace = iEngine->GetPlatformConfig()->GetFlashRAMResvUnUtilisedSpace();



				if( iMMCReservoir && 
								( freeRAM > ( ramLowWatermark + ramUnUtilSpace ) ) )
					{
					iRAMResSize = ramLowWatermark;
					}
				else if( freeRAM > ( ramHighWatermark + ramUnUtilSpace  ) )
					{
					iRAMResSize = iEngine->GetPlatformConfig()->GetRAMResHighWatermark();
					}
				else if( freeRAM > ramUnUtilSpace )
					{
					iRAMResSize = freeRAM - ramUnUtilSpace;
					}
				else
					{
					User::Leave(KErrNoMemory);
					}
				// This is the fix for the bug : SRAA-78WD6L -  User 30 crash is 
				// seen during forward seek of flv video.
				if( (iRAMResSize>aContentLength) && (aContentLength != -1) )
					{
					iRAMResSize = aContentLength;
					}

			iRAMReservoir = HBufC8::NewL(iRAMResSize);
			
			//create the async timer
			iAsyncObject = CFlhTimerObject::NewL(CActive::EPriorityIdle, *this);						
			
		}
	
	}
//RAM storage related functions

void CBrowserURLRequestInfo::WriteToRAMReservoirL(const TDesC8& aData)
	{

	if( ( !iRAMReservoir ) || ( iStreamCancelled ) )
		{
		return;
		}
		
	TPtr8 dataPtr = iRAMReservoir->Des();
	
	// when there is no MMC we should make sure that the difference 
	// between the read offset and the new write offset does not exceed 
	// the size of the RAM storage
	
	if( ( !iMMCReservoir ) &&  (  ( iWriteOffset - iReadOffset ) > iRAMResSize )  )
		{
		iStreamCancelled = ETrue;
		//We should not have keyword video in case of mp3Streaming
		//Mp3Streaming

		if(iNetworkBufferInfo && iNetworkBufferInfo->streamMediaType == FI_StreamMediaType_Sound)
		    iLastErrorNote = R_FLASH_INSUFFICIENT_MEMORY_MMC_NEEDED;
		else
		
		iLastErrorNote = R_FLASH_ERROR_INSERT_MEMORY_CARD;
		return;
		}
		
	if( iRAMCycles == 0)
		{
		// this is the first time data is written to the RAM storage
		// so we can simply append the data
		if( iRamOffset + aData.Length() <= iRAMResSize)
			{						
			dataPtr.Append(aData);			
			iRamOffset += aData.Length();
			}
		// the data length is such that it will overshoot the reservoir capacity
		// so a turn around is required
		else
			{
			// append the data till you reach reservoir size			
			dataPtr.Append(aData.Left(iRAMResSize-iRamOffset));
			//TInt dataToBeWritten = iRamOffset+aData.Length()-iRAMResSize;
			TInt dataToBeWritten = aData.Length() - (iRAMResSize -iRamOffset);
			//the RAM reservoir is full. commit to MMC storage if there is an MMC		
			//check if this will overwrite unread data 
			if(iMMCReservoir && iMMCReservoir->CheckMMCStatusL(iReadOffset, iRAMResSize))
				{
				TRAPD( err,iMMCReservoir->WriteToMMCL( *iRAMReservoir, iRAMResSize ) );	
				if( err )	// mmc access error, abort 
					{
					iStreamCancelled = ETrue; 
					iLastErrorNote = R_FLASH_MMC_ACCESS_ERROR ;
					return;
					}
				}				
			else if( iMMCReservoir )
				{
				iStreamCancelled = ETrue;//further writes should be disabled
				//We should not have keyword video in case of mp3Streaming
				//Mp3Streaming
				if(iNetworkBufferInfo && iNetworkBufferInfo->streamMediaType == FI_StreamMediaType_Sound)
				    iLastErrorNote = R_FLASH_MMC_MEMORY_LOW;
				else
				iLastErrorNote = R_FLASH_ERROR_NOT_ENOUGH_MEMORY_VIDEO_TERMINATES ;
				return; //break off 
				}

			//do a turn around and start writing from the beginning
			ClearRAMReservoir();
			dataPtr.Replace(iRamOffset,dataToBeWritten,aData.Right(dataToBeWritten));
			iRamOffset = dataToBeWritten;
			}		
		}
	else
		{
		// the reservoir already contains data. 
		// we try to replace data that is already read to the core player
		
		// make sure that we do not go beyond the reservoir capacity
		if( iRamOffset + aData.Length() <= iRAMResSize)
			{
			
			// a turn around in write is not required as we are not going  
			// beyond the reservoir size. so simply replace with the new data
			
			dataPtr.Replace(iRamOffset, aData.Length(), aData);
			iRamOffset += aData.Length();
			}   
		else
			{
			// you can replace the data with out resetting the RAM offset till 
			// we reach the reservoir size			
			dataPtr.Replace( iRamOffset, iRAMResSize - iRamOffset, aData.Left( iRAMResSize - iRamOffset ) );
			TInt dataToBeWritten = iRamOffset + aData.Length() - iRAMResSize;
			
			//check if this will overwrite unread data
			if(iMMCReservoir && iMMCReservoir->CheckMMCStatusL(iReadOffset, iRAMResSize))
				{
				TRAPD( err,iMMCReservoir->WriteToMMCL( *iRAMReservoir, iRAMResSize ));	
				if( err )	// mmc access error, abort 
					{
					iStreamCancelled = ETrue; 
					iLastErrorNote = R_FLASH_MMC_ACCESS_ERROR ;
					return;
					}

				}
			else if( iMMCReservoir )
				{
				iStreamCancelled = ETrue;//further writes should be disabled				
				//We should not have keyword video in case of mp3Streaming
				//Mp3Streaming
                if(iNetworkBufferInfo && iNetworkBufferInfo->streamMediaType == FI_StreamMediaType_Sound)
                    iLastErrorNote = R_FLASH_MMC_MEMORY_LOW;
                else
				iLastErrorNote = R_FLASH_ERROR_NOT_ENOUGH_MEMORY_VIDEO_TERMINATES;
				return; //break off 
				}
			// since the reservoir is full we need to clear it	
			ClearRAMReservoir();
			//do a turn around and start writing from the beginning
			dataPtr.Replace( iRamOffset, dataToBeWritten, aData.Right( dataToBeWritten ) );
			iRamOffset = dataToBeWritten;
			}		
		
		}
	}

void CBrowserURLRequestInfo::ClearRAMReservoir()
	{
	// the reservoir is full
	// another write cycle is complete
	iRAMCycles++;
	// offset is reset as we want to write from the beginning
	iRamOffset = 0;
	}
	
TBool CBrowserURLRequestInfo::IsDataInRAMRes(TInt aOffset)
	{
		
	if( ( aOffset > ( ( iRAMCycles - 1) * iRAMResSize + iRamOffset ) )&& 
														aOffset < iWriteOffset )
		{
		return ETrue;
		}
	else
		{
		return EFalse;
		}	
	}
// -------------------------------------------------------------------------
// TInt CBrowserURLRequestInfo::ReadFromRAMResL(TDes8& aData,TInt aOffset,
//																TInt aLength)
//
// Reads the stream data from the RAM storage
//
// --------------------------------------------------------------------------
//	
TInt CBrowserURLRequestInfo::ReadFromRAMResL(TDes8& aData,TInt aOffset, 
															TInt aLength)
	{
	NPPDebug(LOG_DEBUG,ETrue,_L("CBrowserURLRequestInfo::ReadFromRAMResL:++ iReadOffset = %d "), iReadOffset);
	
	if(!iRAMReservoir)
		{		
		return 0;
		}
	if( aOffset > iRamOffset && iRAMCycles )
		{
		//check if length of the data to be read is such that a turn around is not required
		//iRAMResSize - iRamOffset is the part of the data that is not yet over-written
		if( aLength <= iRAMResSize - aOffset )
			{
			NPPDebug(LOG_DEBUG,ETrue,_L("CBrowserURLRequestInfo::ReadFromRAMResL: aLength <= iRAMResSize - aOffset --"));
			aData.Copy( iRAMReservoir->Mid(aOffset, aLength));
			NPPDebug(LOG_DEBUG,ETrue,_L("CBrowserURLRequestInfo::ReadFromRAMResL:%d --"));	
			return aLength;
			}
		else
			{
			NPPDebug(LOG_DEBUG,ETrue,_L("CBrowserURLRequestInfo::ReadFromRAMResL: aLength > iRAMResSize - aOffset --"));
			// read the data that can be read from aOffset directly
			aData.Copy( iRAMReservoir->Right( iRAMResSize - aOffset ));
			
			//turn around and read from the beginning and append
			
			aData.Append( iRAMReservoir->Left(Min((aLength -(iRAMResSize - aOffset)), iRamOffset )) );
			NPPDebug(LOG_DEBUG,ETrue,_L("CBrowserURLRequestInfo::ReadFromRAMResL:%d --"));
			return aData.Length();
			}	
		}
	else
		{
		// read and write are in the same cycle
			if(iRamOffset > aOffset)
				{
				NPPDebug(LOG_DEBUG,ETrue,_L("CBrowserURLRequestInfo::ReadFromRAMResL: iRamOffset > aOffset"));				
				aData.Copy( iRAMReservoir->Mid( aOffset, Min( aLength, iRamOffset - aOffset ) ) );
				NPPDebug(LOG_DEBUG,ETrue,_L("CBrowserURLRequestInfo::ReadFromRAMResL:%d --"));
				return aData.Length();				
				}
			else
				{
				return 0;
				}	

		}	
	
	
	}

void CBrowserURLRequestInfo::WriteStreamDataL(const TDesC8& aData, TInt aLength)
	{
	if( iReadOffset == iWriteOffset )
		{		
		WriteStreamDataToCoreL(aData);			
		}
	//if the stream has not been cancelled, push the write offset forward
	if( !iStreamCancelled )
		{
		iWriteOffset += aLength;	
		}
	
	// if the RAM storage is not being used, return without writing to it
	// RAM storage is currently used for flv content only

	if(!iUseRAMReservoir)	
		{
		return;
		}
#ifdef __DUMP_FLV_TO_FILE
	_LIT(KDebugFileName1, "C:\\Data\\orig.flv");	
	RFile fileHandle;
	TInt error = fileHandle.Open( CEikonEnv::Static()->FsSession(), KDebugFileName1, EFileWrite );
    if( error == KErrNotFound )
        {
        error = fileHandle.Create( CEikonEnv::Static()->FsSession(), KDebugFileName1, EFileWrite );
        }
    
	TInt pos = 0;
	fileHandle.Seek(ESeekEnd, pos);
    fileHandle.Write( aData );
    fileHandle.Close();
#endif
		
	// write to the RAM storage	
	WriteToRAMReservoirL(aData);
		
	}
// this function writes the provided data to the core player's buffers
void CBrowserURLRequestInfo::WriteStreamDataToCoreL(const TDesC8& aData)
	{
	// if the core has not given a buffer, return without writing to it
	if (iNetworkBufferInfo == NULL || iNetworkBufferInfo->pBuffer == NULL)
		{
		return;
		}			
	// the number of bytes we are attemping to write to the core player	
	unsigned long bytesToWrite = aData.Length();
	
	
	const unsigned char* dataToWrite;

	dataToWrite = aData.Ptr();
	

// Continue writing until there's no more data
	while (bytesToWrite > 0)
		{
		if ( ( iNetworkBufferInfo == NULL ) || ( iNetworkBufferInfo->pBuffer == NULL ) )
			{
			//break off now as the has not provided a core buffer
			return;
			}			
		// write only as much data as the core can take at once
		iNetworkBufferInfo->dataSize = 
					(bytesToWrite > iNetworkBufferInfo->maxBufSize) ? 
													iNetworkBufferInfo->maxBufSize : bytesToWrite;
		// copy data to the core's buffer
		FlashMemCpy(iNetworkBufferInfo->pBuffer, dataToWrite, iNetworkBufferInfo->dataSize);
		
		//advance the read offset 
		iReadOffset +=iNetworkBufferInfo->dataSize;
		// need to advance the dataToWrite pointer
		if (bytesToWrite > iNetworkBufferInfo->maxBufSize)
			{
			dataToWrite = dataToWrite + iNetworkBufferInfo->maxBufSize;
			bytesToWrite -= iNetworkBufferInfo->maxBufSize;
			}
		else
			{
			dataToWrite = NULL;
			bytesToWrite = 0;
			}		
		//Camp : Changed
		//iNetworkBufferInfo = iEngine->GetFIObject().FI_URLStreamDataLoaded(iNetworkBufferInfo->streamId, iNetworkBufferInfo );
		iNetworkBufferInfo = iEngine->GetFIObject().FI_URLStreamDataLoaded(iStreamID, iNetworkBufferInfo );
		// if read offset is equal to or exceeds the write offset, and if all the data has been downloaded,
		// indicate to the core that the stream can be closed
		if( ( iWriteStatus!= -1 ) && iReadOffset >= iWriteOffset)
			{
			if(!iCancelState)
			    {
				//as broswer donesn't support http status ocde to be propogated to flash plugin, hence we are 
				// sending "200" for success transaction and '0' for all other cases.
				iEngine->GetFIObject().FI_URLStreamClosed(iStreamID, iWriteStatus,200);
				//Mp3Streaming
				if(iNetworkBufferInfo && iNetworkBufferInfo->streamMediaType == FI_StreamMediaType_Sound)
				    TRAP_IGNORE(iEngine->MP3StreamClosedL());
			    }
			}
		}	
	}
// this function returns is the max data chunk size that the browser is permitted to 
// provide us
TInt CBrowserURLRequestInfo::GetMaxSize()
	{	
	if(iStreamCancelled)
		{
		return 0;
		}
	if(iNetworkBufferInfo && iNetworkBufferInfo->maxBufSize)
		{
		return iNetworkBufferInfo->maxBufSize;	
		}
	else if(iRAMReservoir)
		{
		return iRAMResSize;	
		}
	else
		{
		return 0;
		}			
	}

void CBrowserURLRequestInfo::SetNetworkBuffer( struct FI_NetworkBufferInfo* aNetworkBufferInfo )
	{	
	// the core has given us a new buffer to write to
	// trigger the read-write mechanism	
	iNetworkBufferInfo = aNetworkBufferInfo;
	TriggerAsyncReadWrite();
	}


void CBrowserURLRequestInfo::DoRunL(const CActive* /*aActive*/)
	{
    TBool dataReadComplete = EFalse;
    if(iStreamCancelled)
        {
        if(iMMCReservoir)
            {
            dataReadComplete = (iReadOffset == iMMCReservoir->BytesWritten()) ? ETrue : EFalse;        
            }
        else
            {
            dataReadComplete = (iReadOffset == iRAMCycles * iRAMResSize + iRamOffset) ? ETrue : EFalse;
            }
        }
	// if the stream has been cancelled, and if all the data written 
	// has been read to the core player we know that the RAM storage is full 
	// and further play back is not possible.
	if( iStreamCancelled && dataReadComplete && iLastErrorNote )
		{
		TInt resoffset = FlashEngineRscHandler::LoadEngineResourceFileL();
		HBufC* errorNote = NULL;
		errorNote = StringLoader::LoadLC( iLastErrorNote  );
		ShowErrorNoteL( *errorNote );
		CleanupStack::PopAndDestroy( errorNote );
		iEngine->NotifyClientError( KErrNoMemory, ETrue, ETrue );
		FlashEngineRscHandler::DeleteEngineResourceFile(iEngine->GetEikonEnv(), resoffset);
        iLastErrorNote = 0;
		return;
		}
	if( iReadOffset < iWriteOffset )
		{
		TInt bytesRead = 0;
		// check read and write are in the same RAM  cycle
		if( iReadOffset >= iRAMCycles*iRAMResSize )
			{
				if(iNetworkBufferInfo)
				{
					HBufC8* dataBuffer = HBufC8::NewLC( iNetworkBufferInfo->maxBufSize );
					TPtr8 dataPtr = dataBuffer->Des();
					//data is definitely in the RAM storage
					bytesRead = ReadFromRAMResL( dataPtr,
												iReadOffset - ( iRAMCycles*iRAMResSize ),
												iNetworkBufferInfo->maxBufSize );
					WriteStreamDataToCoreL( dataPtr );
					CleanupStack::PopAndDestroy( dataBuffer );
					if(( iNetworkBufferInfo && iNetworkBufferInfo->pBuffer )
												&& ( iReadOffset < iWriteOffset ) )
						{
						TriggerAsyncReadWrite();
						}
				}
			}
		else
			{
			if( iReadOffset < ( iRAMCycles*iRAMResSize ) )
				{
				// we know that the data is definitely not in the 
				// current RAM cycle
				// try to read from the MMC	
				if( iMMCReservoir )	
					{
					// since the MMC read now is asynchronous
					if(iNetworkBufferInfo)
						iMMCReservoir->ReadFromMMCL( iReadOffset, iNetworkBufferInfo->maxBufSize);
					}
				// no MMC.. should be in previous ram cycle
				else if(iNetworkBufferInfo)
					{
				
					HBufC8* tempData = HBufC8::NewLC( iNetworkBufferInfo->maxBufSize - bytesRead);
					TPtr8 tempDataPtr = tempData->Des();
					
				// offset in RAM 
					TInt ramReadOffset = iRAMResSize - ( (iRAMCycles*iRAMResSize) - ( iReadOffset + bytesRead ) );
					ReadFromRAMResL( tempDataPtr, ramReadOffset,( iNetworkBufferInfo->maxBufSize) 
																						- bytesRead );
					WriteStreamDataToCoreL( tempDataPtr );					
					CleanupStack::PopAndDestroy(tempData);
					if( ( iNetworkBufferInfo && iNetworkBufferInfo->pBuffer ) 
												&& ( iReadOffset < iWriteOffset ) )
						{
						TriggerAsyncReadWrite();
						}

					}
				 
				}
			}
			
		
		}		

	}
TInt CBrowserURLRequestInfo::DoRunError(const CActive* /*aActive*/, TInt aError)
	{
	if( iEngine )
		{
		iEngine->NotifyClientError( aError, ETrue ); 
		}
	return aError;
	}
void CBrowserURLRequestInfo::TriggerAsyncReadWrite()
	{
	
	if( iAsyncObject && !( iAsyncObject->IsActive() ) )
		{
		iAsyncObject->After( KWriteInterval );
		}
		
	}

void CBrowserURLRequestInfo::WriteComplete(FI_Result aStatus)
	{
	
	iWriteStatus = aStatus;
	if( iReadOffset == iWriteOffset )
		{
		if(!iCancelState)
			{
			//as broswer donesn't support http status ocde to be propogated to flash plugin, hence we are 
			// sending "200" for success transaction and '0' for all other cases.
			iEngine->GetFIObject().FI_URLStreamClosed( iStreamID, iWriteStatus, 200  );
			//Mp3Streaming
			if(iNetworkBufferInfo && iNetworkBufferInfo->streamMediaType == FI_StreamMediaType_Sound)
			    TRAP_IGNORE(iEngine->MP3StreamClosedL());
			}
			
		}
	if (iLastErrorNote)
    {
    TInt resoffset = FlashEngineRscHandler::LoadEngineResourceFileL();
    HBufC* errorNote = NULL;
    errorNote = StringLoader::LoadLC( iLastErrorNote  );
    ShowErrorNoteL( *errorNote );
    CleanupStack::PopAndDestroy( errorNote );
    iEngine->NotifyClientError( KErrNoMemory, ETrue, ETrue );
    FlashEngineRscHandler::DeleteEngineResourceFile(iEngine->GetEikonEnv(), resoffset);
    iLastErrorNote = 0;
    }
	}

TBool CBrowserURLRequestInfo::Seek(TInt aReadOffset, FI_NetworkBufferInfo* aNetworkBufferInfo)
	{
	// reset the network buffer.
	iNetworkBufferInfo = aNetworkBufferInfo;
	
	if(aReadOffset > iWriteOffset)
		{
		return EFalse;
		}
	//check if data is available on MMC or in RAM storage
	//check if data is available on MMC first
	if( ( iMMCReservoir && ( iMMCReservoir->IsDataInMMC(aReadOffset) == EFalse ) ) || (!iMMCReservoir) )
		{
		// if there is no MMC or the data is not in the MMC,
		// see if the data is present in the RAM
		if(!IsDataInRAMRes(aReadOffset))
			{
			
			// we have any way received a buffer ..
			// lets continue writing to it, without resetting our read offset
				TriggerAsyncReadWrite();
			// if data is absent in either, return an error
			// this causes the seek to be ignored 
			// and playback to be continued as before.
				return EFalse;
			}
		}
	// we now know that the data indeed is in one of the reservoirs.
	// so reset the read offset.	
	iReadOffset = aReadOffset;
	
	// trigger the read-write mechanism
	TriggerAsyncReadWrite();
	
	if(iMMCReservoir)
		{
		iMMCReservoir->CancelAllPendingRequests();
		}	
	return ETrue;
	}

//Sathya
unsigned int CBrowserURLRequestInfo::GetBytes()
{
	return iWriteOffset;
		
}//Sathya

unsigned int CBrowserURLRequestInfo::GetTotalBytes()
{
    if (iWriteStatus == FI_ResultOK)
        return iWriteOffset;
    else
        return 0;
}

void CBrowserURLRequestInfo::DoCancelStream()
	{
	iCancelState = ETrue;
	}
	
void CBrowserURLRequestInfo::ShowErrorNoteL(const TDesC& aErrMsg)
	{
   	CAknGlobalNote* globalNote = CAknGlobalNote::NewLC();        
    globalNote->ShowNoteL( EAknGlobalErrorNote, aErrMsg );
    CleanupStack::PopAndDestroy(globalNote);  
	}
void CBrowserURLRequestInfo::ReadCompleteL( TInt aStatus, const TDesC8& aData )
	{
	if( aStatus == KErrNone )
		{
		WriteStreamDataToCoreL( aData );
		TriggerAsyncReadWrite();
		}
	
	}
TInt CBrowserURLRequestInfo::NotifyError(TInt aStatus)	
	{
	TRAPD(err, NotifyErrorL(aStatus));
	return err;
	}
// the MMC read has failed and we have been notified of the error	
void CBrowserURLRequestInfo::NotifyErrorL(TInt aStatus)	
	{
	if( aStatus!=KErrNone )
		{
		iStreamCancelled = ETrue; 
		TInt resoffset = FlashEngineRscHandler::LoadEngineResourceFileL();
		iLastErrorNote = R_FLASH_MMC_ACCESS_ERROR;		
		HBufC* errorNote = StringLoader::LoadLC( iLastErrorNote );		
		ShowErrorNoteL( *errorNote );
		CleanupStack::PopAndDestroy( errorNote );	
		iEngine->NotifyFlashError(aStatus, ETrue, ETrue);
		FlashEngineRscHandler::DeleteEngineResourceFile(iEngine->GetEikonEnv(),resoffset);
		}			
	}
//end of file
