/*
* Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class which provides RConnection session to diff
*  engines
*
*/

#include <connpref.h>
#include "conn_handler.h"
#include "symbian_engine.h"
#include "ipconn_mgr.h"
#include <cmapplicationsettingsui.h>
using namespace CMManager;

CConnHandler* CConnHandler::NewL(CPlatformFlashEngine* aEngine)
	{
	DEBUGPRINT("CConnHandler::NewL << ++");
	DEBUGPRINT("CConnHandler::NewL >> --");
	return (new (ELeave) CConnHandler(aEngine));
	}

CConnHandler::CConnHandler(CPlatformFlashEngine* aEngine):
	CActive(CActive::EPriorityIdle),
	iEngine(aEngine)
	{	
	DEBUGPRINT("CConnHandler::CConnHandler << ++");
	CActiveScheduler::Add(this);
	DEBUGPRINT("CConnHandler::CConnHandler >> --");
	}

CConnHandler::~CConnHandler()
	{
	iEngine = NULL;
	
	CloseConnection();
	}

void CConnHandler::StartConnectionL(MConnHandlerObs* aConnHndlrObs, MFlhIPConnectObserver* aIPConnObs)
	{	
	DEBUGPRINT("CConnHandler::StartConnectionL << ++");
	iIPConnObs = aIPConnObs;
	iConnhndlrObs = aConnHndlrObs;
	
	User::LeaveIfError(iSockServ.Connect());
	CleanupClosePushL(iSockServ);
	User::LeaveIfError(iConnection.Open(iSockServ));
	CleanupStack::Pop(&iSockServ);
	
	iStatus = KRequestPending;
	
	//Use default connection which is already set in CommDb with priority one. if not set then we will get prompt
	//for selecting the IAP.
	iConnection.Start( iStatus );
	if(!IsActive())
		SetActive();

	DEBUGPRINT("CConnHandler::StartConnectionL >> --");
	}

void CConnHandler::CloseConnection()
	{
	DEBUGPRINT("CConnHandler::CloseConnection << ++");
	iIPConnObs = NULL;
	iConnhndlrObs = NULL;
	
	if(iConnection.SubSessionHandle())
		{
		iConnection.Close();
		}

	if(IsActive())
		{
		Cancel();
		}

	if(iSockServ.Handle())
		{
		iSockServ.Close();
		}
	iConnected = EFalse;	
	
	DEBUGPRINT("CConnHandler::CloseConnection >> --");
	}

TInt CConnHandler::GetIAP(TUint32& aIAP)
	{
	DEBUGPRINT("CConnHandler::GetIAP << ++");
	
	TInt lError = iConnection.GetIntSetting( _L("IAP\\Id"), aIAP );
	
	DEBUGPRINT("CConnHandler::GetIAP >> Error %d--", lError);
	if(lError != KErrNone)
		return KErrNotFound;
	
	return KErrNone;	
	}

TInt CConnHandler::GetBearerType()
	{
	DEBUGPRINT("CConnHandler::GetBearerType << ++");
	TUint32 lBearerType( 0 );	
	TInt lError = iConnection.GetIntSetting( _L("ModemBearer\\Type"), lBearerType );
	DEBUGPRINT("CConnHandler::GetBearerType >> Error %d --", lError);
	if(lError != KErrNone)
		return KErrNotFound;
	return lBearerType;		
	}


void CConnHandler::HandleNetworkQueryDeniedL(MConnHandlerObs* aConnHndlrObs, MFlhIPConnectObserver* aIPConnObserver)
	{
	aIPConnObserver->HandleNetworkAccessDenied();
	aConnHndlrObs->NotifyConnHandlerStatusL(EEventIsError,this);
	}

void CConnHandler::DoCancel()
	{
	//todo
	}

void CConnHandler::RunL()
	{
	DEBUGPRINT("CConnHandler::RunL status %d , iConntected = %d << ++", iStatus.Int(),iConnected);
	if(iConnected)
		{
		
		const TInt stage=iBuf().iStage;
		// These are connection failure events that are guaranteed for any bearer
		if(stage==KConnectionFailure || stage==KLinkLayerClosed)
			{
            DEBUGPRINT("KConnectionFailure or KLinkLayerClosed");
			iIPConnObs->ConnectionEventL(EEventIsDisconnected);
			CloseConnection();
			}
		else //keep listening
			{
            DEBUGPRINT("keep listening");
			iStatus=KRequestPending;
			SetActive();
			iConnection.ProgressNotification(iBuf,iStatus);	
			}
		}
	else 
		{
		if(iStatus==KErrNone || iStatus==KErrAlreadyExists)
			{
            DEBUGPRINT("InProgress");
			// KErrAlreadyExists means another program is connected to that IAP, and the server
			// side of the Socket Server will reference count the users of the IAP.
			iConnhndlrObs->NotifyConnHandlerStatusL(EEventIsConnectedOK,this);//send notification to IPconnection manager about connection which will store IAPId.

			iIPConnObs->ConnectionEventL(EEventIsConnectedOK);
		
			if(iEngine)
			    iEngine->ResumeOnNetworkPrompt();

			// now watch for disconnects etc!
			iConnected=ETrue;

			iStatus=KRequestPending;
			SetActive();
			iConnection.ProgressNotification(iBuf,iStatus);
			}
		else //failed to connect at all
			{
            DEBUGPRINT("Failed connection");
			if (iIPConnObs)
				{
					iConnhndlrObs->NotifyConnHandlerStatusL(EEventIsError,this); //connection not haapened due some error then notify ipconnection manager to delete the connection handler from queue.
					iIPConnObs->ConnectionEventL(EEventIsDisconnected);
					if(iEngine)
						iEngine->ResumeOnNetworkPrompt();
				}
			CloseConnection();
			}
		}
	
	DEBUGPRINT("CConnHandler::RunL >> --");

	}
TInt CConnHandler::RunError(TInt aError)
	{
	return aError;
	}
//end file
