/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that is a wrapper 
*				 for FI functions provided by the Flash library.
*
*/



#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "saplayer_build.h"
//#include "flashlite.h"

#include "fi_interface.h"
#include "engine_data_caging_literals.hrh"
#include "symbian_engine.h"
#include "charconv.h"
#include "symbian_http.h"
#include "symbian_file.h"
#include "symbian_sharedobjects.h"

#include "amp_mtextnavigation.h"
#include <mmidelegatebase.h>

#ifdef __LOG_FI_CALLS__

#ifdef __WINS__
#ifndef __FUNCTION__
#define __FUNCTION__ "undefined by compiler"
#endif // __FUNCTION__
#endif // __WINS__

#define FI_ENTER if(iMMObject) { SI_Interface::SI_Debug(iMMObject,"+ Entering FI_Interface.cpp, line ",__LINE__); SI_Interface::SI_Debug(iMMObject,__FUNCTION__,User::TickCount()); }
#define FI_EXIT if(iMMObject) { SI_Interface::SI_Debug(iMMObject,"- Exiting FI_Interface.cpp, line ",__LINE__); SI_Interface::SI_Debug(iMMObject,__FUNCTION__,User::TickCount()); }

#else // __LOG_FI_CALLS__
#define FI_ENTER_SP if(iMMObject) { SI_Interface::SI_Debug(iMMObject,"+ Entering FI_Interface.cpp, line ",__LINE__); SI_Interface::SI_Debug(iMMObject,__FUNCTION__,User::TickCount()); }
#define FI_EXIT_SP if(iMMObject) { SI_Interface::SI_Debug(iMMObject,"- Exiting FI_Interface.cpp, line ",__LINE__); SI_Interface::SI_Debug(iMMObject,__FUNCTION__,User::TickCount()); }

#define FI_ENTER ;
#define FI_EXIT ;

#endif // __LOG_FI_CALLS__


// Host functions

class HostFns : public MHostFncs
{
public:
	HostFns(CEikonEnv& aEnv);
public: //from MHostFns
	short StartHttpRequestL(struct MM_Object* aPMMObj, CHttpRequestInfo* aInfo);
	short CancelHttpRequest(struct MM_Object* aPMMObj, TAny* aOwner, TUint aId);
	CFlashExtension* GetExtension(struct MM_Object* aPMMObj, TUid aUidOfExt);
	short SavePersistentDataL(struct MM_Object* aPMMObj,
                             const char* nameSpace, 
                             const char* name,
							 unsigned long aRequestId,
                             const unsigned char* data, 
                             const unsigned long dataSize,
                             TBool enforceMaxSize);
    CEikonEnv* GetEikonEnv();
	void Release();
private:
	CEikonEnv& iEnv;
};

HostFns::HostFns(CEikonEnv& aEnv)
:iEnv(aEnv)
{
	
}

CEikonEnv* HostFns::GetEikonEnv()
{
	return &iEnv;
}

void HostFns::Release()
{
	delete this;
}

/**
 * StartHttpRequestL -- wrapper around CPlatformFlashEngine function used by Flash extensions.
 */
short HostFns::StartHttpRequestL(struct MM_Object* aPMMObj, CHttpRequestInfo* aInfo)
{
    CPlatformFlashEngine* pEngine = aPMMObj->pHostObj->iFIInterface->GetEngine();
    if (pEngine && pEngine->GetHttpEngine())
    {
        return pEngine->GetHttpEngine()->StartHttpRequestL(aInfo);
    }
    else
    {
        return -1;
    }
}

/**
 * CancelHttpRequest -- wrapper around CPlatformFlashEngine function used by Flash extensions.
 */
short HostFns::CancelHttpRequest(struct MM_Object* aPMMObj, TAny* aOwner, TUint aId)
{
    CPlatformFlashEngine* pEngine = aPMMObj->pHostObj->iFIInterface->GetEngine();
    if (pEngine && pEngine->GetHttpEngine())
    {
        return pEngine->GetHttpEngine()->CancelHttpRequest(aOwner,aId);
    }
    else
    {
        return -1;
    }
}

/**
 * GetExtension -- wrapper around function used by Flash extensions.
 */
CFlashExtension* HostFns::GetExtension(struct MM_Object* aPMMObj, TUid aUidOfExt)
{
    return aPMMObj->pHostObj->iFIInterface->GetExtension(aUidOfExt);
}


/**
 * SavePersistentDataL -- wrapper around function used by Flash extensions.
 */
short HostFns::SavePersistentDataL(struct MM_Object* aPMMObj,
                             const char* nameSpace, 
                             const char* name,
							 unsigned long /*aRequestId*/,
                             const unsigned char* data, 
                             const unsigned long dataSize,
                             TBool /*enforceMaxSize*/)

{
    CPlatformFlashEngine* pEngine = aPMMObj->pHostObj->iFIInterface->GetEngine();
#ifndef USE_SYNC_MSO
    CFlhShObjEngine* shObj = pEngine->GetShObjEngine();
    if( shObj != NULL )
    {
        return shObj->AsyncSetPersistentDataL(aPMMObj, nameSpace, name, aRequestId, data, dataSize, enforceMaxSize);
    }
#else
    CSharedObjectHandler* shObj = pEngine->GetShObjHandler();
    if( shObj != NULL )
    {
        return shObj->StoreData(nameSpace, name, data, dataSize);
    }    
#endif	//USE_SYNC_MSO
    return KErrNotFound;
}

// -----------------------------------------------------------------------------

/**
 * Constructor
 */
FI_Interface::FI_Interface()
  : iMMObject(0), iEngine(NULL), iExtensions(NULL), iHostObject(NULL)
{
	iForceRender = false;
	iDisplayRect.xmin = 0;
	iDisplayRect.ymin = 0;
	iDisplayRect.xmax = 0;
	iDisplayRect.ymax = 0;
	Mem::FillZ(&fiFunctions,sizeof(FI_Fncs));
	iBFileIsSVG = false;
}

/**
 * FindLibraryL -- responsible for loading the correct Flash library DLL
 */
void FI_Interface::FindLibraryL ( )
{
    TInt err = KErrNone;
	err = iFlashDll.Load(KFlashDllName);
	User::LeaveIfError(err);
}

/**
 * LoadLibraryL -- calls FindLibraryL to load the library, then retrieves the core FI function pointers from the DLL.
 */
void FI_Interface::LoadLibraryL()
{
	FindLibraryL();

	iFncGetInterface = (PFNC_FI_FLASHLITE_GETINTERFACE)iFlashDll.Lookup(1);

	if (iFncGetInterface == NULL)
	{
		User::Leave(KErrGeneral);
	}
	else
	{
		iFncCreatePlayer			= (PFNC_FI_CREATEPLAYER)iFncGetInterface(EIndex_FICreatePlayer);
		iFncCheckSwfDataBuffer		= (PFNC_FI_CHECKSWFDATABUFFERVALIDITY)iFncGetInterface(EIndex_FICheckSwfDataBufferValidity);
		iFncGetVersion				= (PFNC_FI_GETVERSION)iFncGetInterface(EIndex_FIGetVersion);
		iFncGetDisplayFormat		= (PFNC_FI_GETDISPLAYFORMAT)iFncGetInterface(EIndex_FIGetDisplayFormat);
	}
}


/**
 * InitializeL -- Initializes the FI_Interface with the core FI function pointers and loads MMI extensions.
 */

void FI_Interface::InitializeL(TInt aFlags, const TDesC8* aExtensions, CEikonEnv& aEnv, TBool 
#ifndef __S60_32__
        aIsSMCheckNeeded
#endif //__S60_32__
        )
{
	__ASSERT_DEBUG(!iHostObject,User::Invariant());
	iHostObject=new(ELeave) SI_HostObject;
	iHostObject->iFIInterface=this;
	iHostObject->iHostFncs=NULL;
	// Load flashlite.dll
	LoadLibraryL();
	
    if (aFlags & KSupportMMI)
    {

		iHostObject->iHostFncs=new(ELeave) HostFns(aEnv);
		__ASSERT_DEBUG(!iExtensions,User::Invariant());

        iExtensions=new(ELeave) CArrayPtrFlat<CFlashExtension>(1);
        
        _LIT8(KLitMMIExt,"FlashMMI");
        CFlashExtension* mmi=CFlashExtension::NewL(KLitMMIExt);
        CleanupStack::PushL(mmi);

#ifndef __S60_32__
            mmi->ExecuteFunctionL(CMMI_Delegate::EOpCodeSMCheckNeeded , (TAny*)aIsSMCheckNeeded,NULL,NULL,NULL );
#endif
		mmi->SetAllowedSubExtensionsL(aExtensions);

        iExtensions->AppendL(mmi);
        CleanupStack::Pop(mmi);

	}
}

/**
 * Destructor
 */
FI_Interface::~FI_Interface()
{
	if(iHostObject && iHostObject->iHostFncs)
	{
		iHostObject->iHostFncs->Release();
	}
	delete iHostObject;
	iForceRender = false;
	if(iFlashDll.Handle())
		iFlashDll.Close();

    if(iExtensions)
    {
        iExtensions->ResetAndDestroy();
    }
	delete iExtensions;
}


/**
 * FI_CreatePlayer -- Wrapper around FI function pointer.
 * Also handles setting Flash Player flags from extensions.
 */
bool FI_Interface::FI_CreatePlayer(void* aMem, unsigned long aBytes, SI_Fncs& aSiFunctions, unsigned long aConfigFlags )
{
    if (iExtensions)
    {
        SI_Fncs* ptr=&aSiFunctions;
        const CArrayPtrFlat<CFlashExtension>* exts = iExtensions;
        const TInt extCnt = exts->Count();
        
        CFlashExtension* extension;
        for (TInt ii=0; ii<extCnt; ii++)
        {
            extension=(*exts)[ii];       
            extension->InitExtensionSIFunctions(ptr);
            aConfigFlags |= extension->Flags();
        }
	}

	if (iFncCreatePlayer)
	{
		iMMObject = iFncCreatePlayer(aMem, aBytes, iHostObject, &aSiFunctions, aConfigFlags);
	}

    if (iExtensions)
    {
        const CArrayPtrFlat<CFlashExtension>* exts = iExtensions;
        const TInt extCnt = exts->Count();
        
        CFlashExtension* extension;
        for (TInt ii=0; ii<extCnt; ii++)
        {
            extension=(*exts)[ii];       
            TRAP_IGNORE(extension->InitExtensionL(iMMObject));
        }
	}

	return iMMObject != NULL;
}


/**
 * FI_GetVersion -- Wrapper around FI function pointer.
 */
const char * FI_Interface::FI_GetVersion ()
{
	FI_ENTER
	const char * version = NULL;
	version = iFncGetVersion();
	FI_EXIT
	return version;
}


/**
 * FI_GetDisplayFormat -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_GetDisplayFormat (struct MM_Object* pMMObj)
{
	FI_ENTER
	unsigned short format = FI_DisplayFormat_None;
	format = iFncGetDisplayFormat(pMMObj);
	FI_EXIT
	return format;
}


/**
 * FI_CheckSWFDataBufferValidity -- Wrapper around FI function pointer.
 */
short FI_Interface::FI_CheckSWFDataBufferValidity ( const unsigned char* aSwfBuffer, unsigned long aBufferLength, void* memoryPool, unsigned short poolSize)
{
	FI_ENTER
	short result = 0;
	result = iFncCheckSwfDataBuffer ( aSwfBuffer, aBufferLength, memoryPool, poolSize );
	FI_EXIT
	return result;
}



/**
 * FI_GetInterruptVariable -- Wrapper around FI function pointer.
 */
short* FI_Interface::FI_GetInterruptVariable ()
{
	FI_ENTER
	short* result = NULL;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetInterruptVariable )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetInterruptVariable( iMMObject );
	}
	FI_EXIT
	return result;
}


/**
 * FI_DestroyPlayer -- Wrapper around FI function pointer.
 * Also informs extensions that Flash Player instance has been destroyed.
 */
void FI_Interface::FI_DestroyPlayer ()
{
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_DestroyPlayer )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_DestroyPlayer ( iMMObject );

			iMMObject = NULL;
	}

    if (iExtensions)
    {
        const TInt numExts=iExtensions->Count();
        CFlashExtension* ext;
        for(TInt i=0;i<numExts;i++)
        {
            ext=(*iExtensions)[i];
            ext->PlayerDestroyed();
        }
	}
}


/**
 * FI_SetDisplayRect -- Wrapper around FI function pointer.
 */
short FI_Interface::FI_SetDisplayRect ( const struct FI_Rect * aRect )
{
	FI_ENTER
	short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetDisplayRect )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetDisplayRect ( iMMObject, aRect );

		if ( result )
		{
			iDisplayRect = *aRect;
			iForceRender = true;
		}
	}
	FI_EXIT
	return result;
}


/**
 * FI_SetFrameBuffer -- Wrapper around FI function pointer.
 */
short FI_Interface::FI_SetFrameBuffer ( unsigned char* pFrameBuf, unsigned short width, unsigned short height, unsigned short pitch,unsigned short aDisplayFormat )
{
	//TODO
	FI_ENTER
	short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetFrameBuffer )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetFrameBuffer ( iMMObject, pFrameBuf, width, height, pitch, aDisplayFormat  );

		if ( iDisplayRect.xmin == iDisplayRect.xmax &&
			 iDisplayRect.ymin == iDisplayRect.ymax )
		{
			iDisplayRect.xmin = 0;
			iDisplayRect.ymin = 0;
			iDisplayRect.xmax = width;
			iDisplayRect.ymax = height;
		}
	}
	FI_EXIT
	return result;
}

/**
 * FI_SetInputText -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_SetInputText ( const struct FI_Text* inputText)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIKeyInputFncs &&
			 iMMObject->pFIFncs->pFIKeyInputFncs->pFncFI_SetInputText )
			iMMObject->pFIFncs->pFIKeyInputFncs->pFncFI_SetInputText( iMMObject, inputText );
		
	}
	FI_EXIT
}

void FI_Interface::FI_CancelInputText ()
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIKeyInputFncs &&
			 iMMObject->pFIFncs->pFIKeyInputFncs->pFncFI_CancelInputText )
			iMMObject->pFIFncs->pFIKeyInputFncs->pFncFI_CancelInputText( iMMObject );
		
	}
	FI_EXIT
}


unsigned short FI_Interface::FI_SetSVGDataBuffer ( const char* svgURL, const unsigned char *svgBuffer, unsigned long aBufferLength )
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		bool saved = iBFileIsSVG;
		iBFileIsSVG = true;

		if ( iMMObject->pFIFncs->pFISVGFncs &&
			 iMMObject->pFIFncs->pFISVGFncs->pFncFI_SetSVGDataBuffer )
			result = iMMObject->pFIFncs->pFISVGFncs->pFncFI_SetSVGDataBuffer ( iMMObject, svgURL, svgBuffer, aBufferLength );

		if( !result )
			iBFileIsSVG = saved;

	}
	FI_EXIT
	return result;
}

/**
 * FI_GetSVGDimensions -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_GetSVGDimensions(unsigned short* width, unsigned short* height)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
//		bool saved = m_bFileIsSVG;
		iBFileIsSVG = true;

		if ( iMMObject->pFIFncs->pFISVGFncs &&
			 iMMObject->pFIFncs->pFISVGFncs->pFncFI_GetSVGDimensions )
			iMMObject->pFIFncs->pFISVGFncs->pFncFI_GetSVGDimensions ( iMMObject, width, height );
		
	}
	FI_EXIT
}


void FI_Interface::FI_SetSVGViewportSize(unsigned short width, unsigned short height)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs &&
		 iMMObject->pFIFncs->pFISVGFncs &&
		 iMMObject->pFIFncs->pFISVGFncs->pFncFI_SetSVGViewportSize )
    {
		iMMObject->pFIFncs->pFISVGFncs->pFncFI_SetSVGViewportSize(iMMObject, width, height);
    }
	FI_EXIT
}



/**
 * FI_SetSWFDataBuffer -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_SetSWFDataBuffer ( const char* swfURL, const unsigned char *aSwfBuffer, unsigned long aBufferLength, unsigned short inlineContent , unsigned short makeCopy ,enum FI_SecuritySandbox swfSandbox )
{
	FI_ENTER
	unsigned short result = 0;

	if ( iMMObject && iMMObject->pFIFncs )
	{
		bool saved = iBFileIsSVG;
		iBFileIsSVG = false;

		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetSWFDataBuffer )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetSWFDataBuffer ( iMMObject, swfURL, aSwfBuffer, aBufferLength, inlineContent, makeCopy, swfSandbox );

		if( !result )
			iBFileIsSVG = saved;
	}
	FI_EXIT
	return result;
}


/**
 * FI_DoPlay -- Wrapper around FI function pointer.
 */
unsigned short	FI_Interface::FI_DoPlay ( unsigned short forceRender )
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_DoPlay )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_DoPlay ( iMMObject, forceRender );

		iForceRender = false;
	}
	FI_EXIT
	return result;
}


unsigned short FI_Interface::FI_GetSubDirtyRect(unsigned int index, struct FI_Rect* dirtyRect)
{
		FI_ENTER
	unsigned short numRects = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetSubDirtyRect )
			numRects = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetSubDirtyRect ( iMMObject, index, dirtyRect );
	}
	FI_EXIT
	return numRects;
}

unsigned short FI_Interface::FI_GetNumberOfSubDirtyRects ()
{
		FI_ENTER
	unsigned short rects = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetNumberOfSubDirtyRects )
			rects = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetNumberOfSubDirtyRects ( iMMObject );
	}
	FI_EXIT
	return rects;	
}

/**
 * FI_SetLooping -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_SetLooping ( short loop )
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetLooping )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetLooping ( iMMObject, loop );
	}
	FI_EXIT
}


/**
 * FI_SetBackgroundColor -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_SetBackgroundColor ( short red, short green, short blue )
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetBackgroundColor )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetBackgroundColor( iMMObject, red, green, blue );
	}
	FI_EXIT
}

/**
 * FI_SetNativeCodePage -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_SetNativeCodePage (FI_CodePage fi_codePage)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetNativeCodePage )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetNativeCodePage( iMMObject, fi_codePage );
	}
	FI_EXIT
}

/**
 * FI_SetTextFieldBoundaryBehavior -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_SetTextFieldBoundaryBehavior (unsigned long aBehavior)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetTextFieldBoundaryBehavior )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetTextFieldBoundaryBehavior( iMMObject, aBehavior );
	}
	FI_EXIT
}

/**
 * FI_SetQuality -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_SetQuality ( short quality )
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetQuality )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetQuality( iMMObject, quality );
	}
	FI_EXIT
}


/**
 * FI_GetQuality -- Wrapper around FI function pointer.
 */
short FI_Interface::FI_GetQuality ()
{
	FI_ENTER
	short result = -1;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetQuality )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetQuality( iMMObject );
	}
	FI_EXIT
	return result;
}

/**
 * FI_GetDirtyRect -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_GetDirtyRect ( struct FI_Rect * rect )
{
	FI_ENTER
	unsigned short dirty = 0;

	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetDirtyRect )
		{
			dirty = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetDirtyRect( iMMObject, rect );
		}
	}
	FI_EXIT
	return dirty;
}


/**
 * FI_SetDirtyRect -- Wrapper around FI function pointer.
 */
short FI_Interface::FI_SetDirtyRect ( const FI_Rect * rect, unsigned short aOnCurrentFrame )
{
	FI_ENTER
	short result = 0;

	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetDirtyRect )
		{
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetDirtyRect( iMMObject, rect, aOnCurrentFrame );
		}
	}
	FI_EXIT
	return result;
}

/**
 * FI_SetClippingRect -- Wrapper around FI function pointer.
 */
short FI_Interface::FI_SetClippingRect ( const FI_Rect * rect )
{
	FI_ENTER
	short result = 0;

	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetClippingRect )
		{
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetClippingRect( iMMObject, rect );
		}
	}
	FI_EXIT
	return result;
}

/**
 * FI_GetSWFDimensions -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_GetSWFDimensions ( unsigned short * width, unsigned short * height )
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetSWFDimensions )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetSWFDimensions( iMMObject, width, height );
	}
	FI_EXIT
}


/**
 * FI_Stop -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_Stop ()
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_Stop )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_Stop( iMMObject );
	}
	FI_EXIT
}

/**
 * FI_NotifyDeviceSoundStatus -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_NotifyDeviceSoundStatus(unsigned short status)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFISoundControlFncs &&
			 iMMObject->pFIFncs->pFISoundControlFncs->pFncFI_NotifyDeviceSoundStatus )
			iMMObject->pFIFncs->pFISoundControlFncs->pFncFI_NotifyDeviceSoundStatus( iMMObject, status );
	}
	FI_EXIT
}

/**
 * FI_NotifyVideoStatus -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_NotifyVideoStatus ( unsigned long videoID, int severity, FI_Text* pRetCode )
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_NotifyVideoStatus )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_NotifyVideoStatus( iMMObject, videoID, severity, pRetCode );
	}
	FI_EXIT
}


/**
 * FI_Pause -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_Pause ()
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_Pause )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_Pause( iMMObject );
	}
	FI_EXIT
}


/**
 * FI_Resume -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_Resume ()
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_Resume )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_Resume( iMMObject );
	}
	FI_EXIT
}


/**
 * FI_IsPlaying -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_IsPlaying ()
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_IsPlaying )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_IsPlaying( iMMObject );
	}
	FI_EXIT
	return result;
}

/**
 * FI_FrameNumber -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_FrameNumber ()
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_FrameNumber )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_FrameNumber( iMMObject );
	}
	FI_EXIT
	return result;
}


/**
 * FI_NumberOfFrames -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_NumberOfFrames ()
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_NumberOfFrames )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_NumberOfFrames( iMMObject );
	}
	FI_EXIT
	return result;
}


/**
 * FI_OfferKeyEvent -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_OfferKeyEvent ( const FI_KeyId* aKeyId, unsigned short eventCode )
{
	FI_ENTER
	unsigned short result = 0;

	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIKeyInputFncs &&
			 iMMObject->pFIFncs->pFIKeyInputFncs->pFncFI_OfferKeyEvent )
			result = iMMObject->pFIFncs->pFIKeyInputFncs->pFncFI_OfferKeyEvent( iMMObject, aKeyId, eventCode );
	}

	if ( result == 1 )
		iForceRender = true;

	FI_EXIT
	return result;
}


/**
 * FI_OfferKeyPressEvent -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_OfferKeyPressEvent ( const FI_KeyId* aKeyId )
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIKeyInputFncs &&
			 iMMObject->pFIFncs->pFIKeyInputFncs->pFncFI_OfferKeyPressEvent )
			result = iMMObject->pFIFncs->pFIKeyInputFncs->pFncFI_OfferKeyPressEvent( iMMObject, aKeyId );

		if ( result == 1 )
			iForceRender = true;
	}
	FI_EXIT
	return result;
}


/**
 * FI_OfferMouseEvent -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_OfferMouseEvent ( unsigned short aXPos, unsigned short aYPos, unsigned short eventCode )
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIMouseInputFncs &&
			 iMMObject->pFIFncs->pFIMouseInputFncs->pFncFI_OfferMouseEvent )
			result = iMMObject->pFIFncs->pFIMouseInputFncs->pFncFI_OfferMouseEvent( iMMObject, aXPos, aYPos, eventCode );

		if ( result == 1 )
			iForceRender = true;
	}
	FI_EXIT
	return result;
}


/**
 * FI_DoHitTest -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_DoHitTest ( unsigned short aXPos, unsigned short aYPos )
{
	FI_ENTER
	unsigned short result = FI_NullHitPoint;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIMouseInputFncs &&
			 iMMObject->pFIFncs->pFIMouseInputFncs->pFncFI_DoHitTest )
			result = iMMObject->pFIFncs->pFIMouseInputFncs->pFncFI_DoHitTest( iMMObject, aXPos, aYPos );
	}
	FI_EXIT
	return result;
}

/**
 * FI_URLStreamOpened -- Wrapper around FI function pointer.
 */
struct FI_NetworkBufferInfo* FI_Interface::FI_URLStreamOpened	(unsigned short streamId,
													 struct FI_URLStreamHeaderInfo* pHeaders,
													 short* pErrorCode)
{
	FI_ENTER
	struct FI_NetworkBufferInfo* result = NULL;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIURLAccessFncs &&
			 iMMObject->pFIFncs->pFIURLAccessFncs->pFncFI_URLStreamOpened )
			result = iMMObject->pFIFncs->pFIURLAccessFncs->pFncFI_URLStreamOpened( iMMObject, streamId, pHeaders, pErrorCode );
	}
	FI_EXIT
	return result;
}

/**
 * FI_URLStreamSetFinalURL -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_URLStreamSetFinalURL(	unsigned short streamId,
											const char* pFinalURL,enum FI_SecuritySandbox swfSandbox)
{
	FI_ENTER
	if (iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIURLAccessFncs &&
				iMMObject->pFIFncs->pFIURLAccessFncs->pFncFI_URLStreamSetFinalURL)
			iMMObject->pFIFncs->pFIURLAccessFncs->pFncFI_URLStreamSetFinalURL(iMMObject, streamId, pFinalURL,swfSandbox );
	}
	FI_EXIT
}


/**
 * FI_URLStreamDataLoaded -- Wrapper around FI function pointer.
 */
struct FI_NetworkBufferInfo* FI_Interface::FI_URLStreamDataLoaded (unsigned short streamId, 
																	  struct FI_NetworkBufferInfo* pNetBuf)
{
	FI_ENTER
	struct FI_NetworkBufferInfo* result = 0;

	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIURLAccessFncs &&
			 iMMObject->pFIFncs->pFIURLAccessFncs->pFncFI_URLStreamDataLoaded )
			result = iMMObject->pFIFncs->pFIURLAccessFncs->pFncFI_URLStreamDataLoaded( iMMObject, streamId, pNetBuf );
	}
	FI_EXIT

	return result;
}


/**
 * FI_URLStreamClosed -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_URLStreamClosed (unsigned short streamId,
										  unsigned short closedStatusCode,
										  unsigned int httpStatusCode)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIURLAccessFncs &&
			 iMMObject->pFIFncs->pFIURLAccessFncs->pFncFI_URLStreamClosed )
			iMMObject->pFIFncs->pFIURLAccessFncs->pFncFI_URLStreamClosed( iMMObject, streamId, closedStatusCode, httpStatusCode );
	}

	FI_EXIT
	
}


/**
 * FI_SetupDeviceImage -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_SetupDeviceImage(unsigned short numFormats, const char* formatInfo[])
{
	FI_ENTER
    unsigned short result = 0;
    if ( iMMObject && iMMObject->pFIFncs )
    {
        if ( iMMObject->pFIFncs->pFIMIMETypeFncs &&
            iMMObject->pFIFncs->pFIMIMETypeFncs->pFncFI_SetupDeviceImage )
            result = iMMObject->pFIFncs->pFIMIMETypeFncs->pFncFI_SetupDeviceImage( iMMObject, numFormats, formatInfo );
    }
	FI_EXIT
    return result;
}


/**
 * FI_SetupDeviceVideo -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_SetupDeviceVideo(unsigned short numFormats, const char* formatInfo[])
{
	FI_ENTER
    unsigned short result = 0;
    if ( iMMObject && iMMObject->pFIFncs )
    {
        if ( iMMObject->pFIFncs->pFIMIMETypeFncs &&
            iMMObject->pFIFncs->pFIMIMETypeFncs->pFncFI_SetupDeviceVideo )
            result = iMMObject->pFIFncs->pFIMIMETypeFncs->pFncFI_SetupDeviceVideo( iMMObject, numFormats, formatInfo );
    }
	FI_EXIT
    return result;
}


/**
 * FI_SetupDeviceSound -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_SetupDeviceSound ( unsigned short numFormats, const char* formatInfo[], bool mustbeCached)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFISoundControlFncs &&
			 iMMObject->pFIFncs->pFISoundControlFncs->pFncFI_SetupDeviceSound )
			iMMObject->pFIFncs->pFISoundControlFncs->pFncFI_SetupDeviceSound( iMMObject, numFormats, formatInfo, mustbeCached );
	}
	FI_EXIT
}

/**
 * FI_SetupStreamSound -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_SetupStreamSound ( FI_StreamSoundInfo* pSoundInfo, FI_StreamSoundSetupInfo* pSetupInfo )
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFISoundControlFncs &&
			 iMMObject->pFIFncs->pFISoundControlFncs->pFncFI_SetupStreamSound )
			result = iMMObject->pFIFncs->pFISoundControlFncs->pFncFI_SetupStreamSound ( iMMObject, pSoundInfo, pSetupInfo);
	}
	FI_EXIT
	return result;
}


/**
 * FI_StreamSoundBufferComplete -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_StreamSoundBufferComplete ()
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFISoundControlFncs &&
			 iMMObject->pFIFncs->pFISoundControlFncs->pFncFI_StreamSoundBufferComplete )
			iMMObject->pFIFncs->pFISoundControlFncs->pFncFI_StreamSoundBufferComplete ( iMMObject );
	}
	FI_EXIT
}


/**
 * FI_StreamSoundReset -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_StreamSoundReset ()
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFISoundControlFncs &&
			 iMMObject->pFIFncs->pFISoundControlFncs->pFncFI_StreamSoundReset )
			iMMObject->pFIFncs->pFISoundControlFncs->pFncFI_StreamSoundReset(iMMObject);
	}
	FI_EXIT
}

/**
 * FI_Translate -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_Translate(long dx, long dy)
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_Translate )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_Translate( iMMObject, dx, dy );
	}
	FI_EXIT
	return result;
}

/**
 * FI_Scale -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_Scale(long scale, long xorigin, long yorigin)
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_Scale )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_Scale( iMMObject, scale, xorigin, yorigin );
	}
	FI_EXIT
	return result;
}

/**
 * FI_Rotate -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_Rotate(long degrees, long xorigin, long yorigin)
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_Rotate )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_Rotate( iMMObject, degrees, xorigin, yorigin );
	}
	FI_EXIT
	return result;
}

/**
 * FI_ResetTransform -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_ResetTransform()
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_ResetTransform )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_ResetTransform( iMMObject );
	}
	FI_EXIT
}

/**
 * FI_GetZoomAndPanEnabled -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_GetZoomAndPanEnabled()
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetZoomAndPanEnabled )
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetZoomAndPanEnabled( iMMObject );
	}
	FI_EXIT
	return result;
}


unsigned short FI_Interface::FI_ConvertToUserSpace(long screenx, long screeny, long *userx, long *usery)
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs &&
		 iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
		 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_ConvertToUserSpace )
	{
		result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_ConvertToUserSpace(iMMObject, screenx, screeny, userx, usery);
	}
	FI_EXIT
	return result;
}

unsigned short FI_Interface::FI_ConvertToUserSpaceF(float screenx, float screeny, float *userx, float *usery)
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs &&
		 iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
		 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_ConvertToUserSpaceF )
	{
		result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_ConvertToUserSpaceF(iMMObject, screenx, screeny, userx, usery);
	}
	FI_EXIT
	return result;
}

unsigned short FI_Interface::FI_ConvertToViewportSpace(long userx, long usery, long *screenx, long *screeny)
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs &&
		 iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
		 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_ConvertToViewportSpace )
	{
		result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_ConvertToViewportSpace(iMMObject, userx, usery, screenx, screeny);
	}
	FI_EXIT
	return result;
}

unsigned short FI_Interface::FI_ConvertToViewportSpaceF(float userx, float usery, float *screenx, float *screeny)
{
	FI_ENTER
	unsigned short result = 0;
	if ( iMMObject && iMMObject->pFIFncs &&
		 iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
		 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_ConvertToViewportSpaceF )
	{
		result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_ConvertToViewportSpaceF(iMMObject, userx, usery, screenx, screeny);
	}
	FI_EXIT
	return result;
}


typedef void * (*MM_GETDATA)(int);
typedef unsigned int (*GETUNSIGNEDINT)(void *);


/**
 * GetStaticMemoryUsed -- Wrapper around FI function pointer.
 */
unsigned int FI_Interface::GetStaticMemoryUsed ()
{
	FI_ENTER
	unsigned int result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		MM_GETDATA f = (MM_GETDATA)iMMObject->pFIFncs->pMM_Reserved;

		if ( f )
		{
			GETUNSIGNEDINT g = (GETUNSIGNEDINT)f(0);
			if ( g )
				result = g(iMMObject);
		}
	}
	FI_EXIT
	return result;
}

/**
 * GetDynamicMemoryUsed -- Wrapper around FI function pointer.
 */
unsigned int FI_Interface::GetDynamicMemoryUsed ()
{
	FI_ENTER
	unsigned int result = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		MM_GETDATA f = (MM_GETDATA)iMMObject->pFIFncs->pMM_Reserved;

		if ( f )
		{
			GETUNSIGNEDINT g = (GETUNSIGNEDINT)f(1);
			if ( g )
				result = g(iMMObject);
		}
	}
	FI_EXIT
	return result;
}

/**
 * GetExtension -- used to retrieve a pointer to a loaded extension from its UID
 */
CFlashExtension* FI_Interface::GetExtension(TUid aUid)
{ 
    if (iExtensions)
    {
        const TInt count=iExtensions->Count();
        CFlashExtension* ext;
        for (int i=0; i<count ; i++) 
        {
            ext = (*iExtensions)[i];
            if (ext->ExtensionUid() == aUid)
            {
                return ext;
            }
        }
    }
    return NULL;
}


/**
 * FI_NotifySharedObjectStatus -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_NotifySharedObjectStatus(unsigned long aRequestId,
                                 unsigned long aDataSize,
                                 short aStatus)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIDataAccessFncs &&
			 iMMObject->pFIFncs->pFIDataAccessFncs->pFncFI_NotifySharedObjectStatus)
			iMMObject->pFIFncs->pFIDataAccessFncs->pFncFI_NotifySharedObjectStatus( iMMObject,aRequestId,aDataSize,aStatus );
	}
	FI_EXIT
}

/**
 * Extensions -- used to retrieve the entire array of loaded extensions.
 */
CArrayPtr<CFlashExtension>* FI_Interface::Extensions()
{
    return iExtensions;
}

#ifdef FEATURE_XML_SOCKET
/**
 * FI_NotifySocketStatus -- Wrapper around FI function pointer.
 */
short FI_Interface::FI_NotifySocketStatus(unsigned long aSocketId,
										  unsigned short aStatus,
										  short aErrorCode)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFISocketFncs &&
			 iMMObject->pFIFncs->pFISocketFncs->pFncFI_NotifySocketStatus)
			iMMObject->pFIFncs->pFISocketFncs->pFncFI_NotifySocketStatus( iMMObject,aSocketId,aStatus, aErrorCode );
	}
	FI_EXIT

	return 1;
}

/**
 * FI_SocketRecv -- Wrapper around FI function pointer.
 */
short FI_Interface::FI_SocketRecv(unsigned long aSocketId,
								  FI_NetworkBufferInfo* aDataBuffer,
								  FI_NetworkBufferInfo** aNextRecvBuffer)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFISocketFncs &&
			 iMMObject->pFIFncs->pFISocketFncs->pFncFI_SocketRecv)
			iMMObject->pFIFncs->pFISocketFncs->pFncFI_SocketRecv( iMMObject, aSocketId, aDataBuffer, aNextRecvBuffer);
	}
	FI_EXIT

	return 1;
}
#endif //FEATURE_XML_SOCKET
/**
 * FI_ImageInfoCompleted -- Wrapper around FI function pointer.
 */
FI_DecodeImageInfo* FI_Interface::FI_GetDecodeImageBuffer(unsigned long pImageId, const FI_ImageInfo* pImageInfo)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIImageDecodeFncs &&
			 iMMObject->pFIFncs->pFIImageDecodeFncs->pFncFI_GetDecodeImageBuffer)
			return iMMObject->pFIFncs->pFIImageDecodeFncs->pFncFI_GetDecodeImageBuffer( iMMObject, pImageId, pImageInfo);
	}
	FI_EXIT

	return NULL;
}

/**
 * FI_ImageDecoded -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_NotifyImageDecodeStatus(unsigned long pImageId, short error)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIImageDecodeFncs &&
			 iMMObject->pFIFncs->pFIImageDecodeFncs->pFncFI_NotifyImageDecodeStatus)
			iMMObject->pFIFncs->pFIImageDecodeFncs->pFncFI_NotifyImageDecodeStatus( iMMObject, pImageId, error);
	}
	FI_EXIT
}

void FI_Interface::FI_SetFlashVars(const char* values, unsigned short useUTF8)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIAsApiFncs &&
			 iMMObject->pFIFncs->pFIAsApiFncs->pFncFI_AsApi )
			iMMObject->pFIFncs->pFIAsApiFncs->pFncFI_AsApi( iMMObject, values, useUTF8 );
	}
	FI_EXIT

	return;
}

#ifdef FEATURE_BG_TRANSPARENCY

/**
 * FI_SetBackgroundAlpha -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_SetBackgroundAlpha (unsigned short aBackgroundAlpha, unsigned short aBlendWithBackground)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetBackgroundAlpha )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetBackgroundAlpha( iMMObject, aBackgroundAlpha, aBlendWithBackground );
	}
	FI_EXIT

	return;
}

#endif //FEATURE_BG_TRANSPARENCY

#ifdef __FEATURE_PRE_COMPILED_SWF
/**
 * FI_SetPreInstalledASClass -- Wrapper around FI function pointer.
 */
unsigned short FI_Interface::FI_SetPreInstalledASClass(const unsigned char* aSwfBuffer,unsigned long aBufferLength,unsigned short aBIsNativeLibrary)
{
	FI_ENTER
	unsigned short result = 0;

	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetPreInstalledASClass )
		{
			result = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetPreInstalledASClass( iMMObject, aSwfBuffer, aBufferLength, aBIsNativeLibrary );
		}
	}
	FI_EXIT
	return result;
}

#endif //__FEATURE_PRE_COMPILED_SWF



#ifdef FEATURE_OPENVG
/**
 * FI_SetRenderHint -- Wrapper around FI function pointer.
 */
void FI_Interface::FI_SetRenderHint ( FI_RenderHint renderHint )
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetRenderHint )
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetRenderHint ( iMMObject, renderHint );
	}
	FI_EXIT
	return;
}

/**
 * FI_GetRenderMode -- Wrapper around FI function pointer.
 */
FI_RenderMode FI_Interface::FI_GetRenderMode ( )
{
	FI_ENTER
	FI_RenderMode renderMode = FI_RenderMode_Error;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetRenderMode )
			renderMode = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetRenderMode ( iMMObject );
	}
	FI_EXIT
	return renderMode;
}

short FI_Interface::FI_SetRenderMode(FI_RenderMode aRenderMode)
{
    FI_ENTER
    short ret = 0;
    if ( iMMObject && iMMObject->pFIFncs )
    {
        if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
             iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetRenderMode )
            ret = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetRenderMode ( iMMObject, aRenderMode );
    }
    FI_EXIT
    return ret;
}

void FI_Interface::FI_SetHardwareDeviceLimit (enum FI_HardwareDeviceLimitIndex index, long value)
{
	FI_ENTER
	if(iMMObject && iMMObject->pFIFncs)
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs && iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetHardwareDeviceLimit)
		{
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_SetHardwareDeviceLimit(iMMObject, index, value);
		}
	}
	
	FI_EXIT	
};

long FI_Interface::FI_GetHardwareDeviceLimit (enum FI_HardwareDeviceLimitIndex index )
{
    FI_ENTER
    if(iMMObject && iMMObject->pFIFncs)
    {
        if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs && iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetHardwareDeviceLimit)
        {
            return iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetHardwareDeviceLimit(iMMObject, index);
        }
    }
    
    FI_EXIT
    
    return 0;
};

short FI_Interface::FI_ConfigureHardwareRendering(struct FI_HardwareConfig* config)
{
    FI_ENTER
    if(iMMObject && iMMObject->pFIFncs)
    {
        if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs && iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_ConfigureHardwareRendering)
        {
            return iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_ConfigureHardwareRendering(iMMObject, config);
        }
    }
        
    FI_EXIT
        
    return 0;
}
#endif // FEATURE_OPENVG

void FI_Interface::FI_SetBaseURL( const char* baseURL)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetBaseURL )
			iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetBaseURL( iMMObject, baseURL);
		
	}
	FI_EXIT
	return;
}

#ifdef FEATURE_WEB_BROWSABILITY

void FI_Interface::FI_SetScriptAccess(int pAccessLevel)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetScriptAccess )
			iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetScriptAccess( iMMObject, pAccessLevel);
		
	}
	FI_EXIT
	return;

}
//eroom 761 fix:
void FI_Interface::FI_SetContainerSecurityContext(const struct FI_BrowserURLInfo* pURLInfoIn)
{
FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetContainerSecurityContext)
			iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetContainerSecurityContext(iMMObject,pURLInfoIn);
					

		
	}
	FI_EXIT
	return;
}

int	FI_Interface::FI_GetScriptAccess()
{
	FI_ENTER
	int retval = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetScriptAccess )
			retval= iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_GetScriptAccess( iMMObject);
		
	}
	FI_EXIT
	return retval;

}

int	FI_Interface::FI_TCurrentFrame(const char* target,const struct FI_BrowserURLInfo* pURLInfoIn)
{
	FI_ENTER
	TInt retval = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_TCurrentFrame )
			retval = iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_TCurrentFrame( iMMObject, target, pURLInfoIn);
		
	}
	FI_EXIT
	return retval;
}

void* FI_Interface::FI_TCurrentLabel(const char* pTargetIn, const struct FI_BrowserURLInfo* pBrowserInfoIn,
															   struct FI_AllocatorWrapper* currentLabel)
{
	FI_ENTER
	void* label = NULL;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_TCurrentLabel )
		label =	iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_TCurrentLabel( iMMObject, pTargetIn, pBrowserInfoIn, currentLabel);
		
	}
	FI_EXIT
	return label;
}

void* FI_Interface::FI_TGetProperty(const struct FI_TargetPropertyInfo* ptargetInfoIn,
															 const struct FI_BrowserURLInfo* pBrowserInfoIn,
															 struct FI_AllocatorWrapper* propertyValue)
{
	FI_ENTER
	void* property = NULL;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_TGetProperty )
			property = iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_TGetProperty( iMMObject, ptargetInfoIn, pBrowserInfoIn, propertyValue);
		
	}
	FI_EXIT
	return property;
}

int FI_Interface::FI_TSetProperty(const struct FI_TargetPropertyInfo* ptargetInfoIn,
								    const struct FI_BrowserURLInfo* pBrowserInfoIn,const char* value)
{
	FI_ENTER
	int retval = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_TSetProperty )
			retval = iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_TSetProperty( iMMObject, ptargetInfoIn, pBrowserInfoIn, value);
		
	}
	FI_EXIT
	return retval;
}
																		 
int FI_Interface::FI_GetBackgroundColor(FI_Color *returnBgColor)
{
	FI_ENTER
	
	int retval = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_GetBackgroundColor )
			retval = iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_GetBackgroundColor( iMMObject, returnBgColor);
		
	}
	FI_EXIT
	return retval;
}

int FI_Interface::FI_CanContainerAccessMovie(  const struct FI_BrowserURLInfo* pURLInfoIn,
																		 int bAllowLocalUntrusted)
{
	FI_ENTER
	int retval =0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_CanContainerAccessMovie )
			retval = iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_CanContainerAccessMovie( iMMObject, pURLInfoIn,bAllowLocalUntrusted);
		
	}
	FI_EXIT
	return retval;

}

int FI_Interface::FI_SetVariable2( const char* pVariableNameIn,
								   const char* pVariableValueIn, const struct FI_BrowserURLInfo* pBrowserInfoIn)
{
	FI_ENTER
	
	int retval = 0;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetVariable2)
			retval = iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetVariable2( iMMObject, pVariableNameIn,pVariableValueIn,pBrowserInfoIn);
		
	}
	FI_EXIT
	return retval;
}
void* FI_Interface::FI_GetVariable2( const char* pVariableNameIn,
									const struct FI_BrowserURLInfo* pBrowserInfoIn,
									struct FI_AllocatorWrapper* hostAllocWrapper)
{
	FI_ENTER
	void* variablePtr = NULL;
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_GetVariable2)
			variablePtr =iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_GetVariable2( iMMObject, pVariableNameIn,pBrowserInfoIn, hostAllocWrapper);
		
	}
	FI_EXIT
	return variablePtr;


}
void FI_Interface::FI_LoadLayer(const char*swfURL,
								int layer,struct FI_BrowserURLInfo* pURLInfo)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_LoadLayer )
			iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_LoadLayer( iMMObject, swfURL,layer,pURLInfo);
		
	}
	FI_EXIT
	return;


}

void FI_Interface::FI_SetScaleMode(int aScaleModeValue)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetScaleMode)
			iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetScaleMode( iMMObject, aScaleModeValue );
		
	}
	FI_EXIT	
	return;
}

void FI_Interface::FI_SetAlignMode(int aAlignModeValue)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
			 iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetAlignMode)
			iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetAlignMode( iMMObject, aAlignModeValue );
		
	}
	FI_EXIT
	return;	
}

void FI_Interface::FI_SetAllowNetworking(int aAllowNetworking)
{
    FI_ENTER
    if ( iMMObject && iMMObject->pFIFncs )
    {
        if ( iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
             iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetAlignMode)
            iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetAllowNetworking( iMMObject, aAllowNetworking);
        
    }
    
    FI_EXIT
    return; 
}

int FI_Interface::FI_GetAllowNetworking()
{
    FI_ENTER
    int retVal = 0;
    if ( iMMObject && iMMObject->pFIFncs )
    {
        if (iMMObject->pFIFncs->pFIWebBrowsabilityFncs &&
               iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_SetAlignMode)
            retVal = iMMObject->pFIFncs->pFIWebBrowsabilityFncs->pFncFI_GetAllowNetworking(iMMObject);
            
    }
    
    FI_EXIT
    return retVal;
}


#endif //FEATURE_WEB_BROWSABILITY

#ifdef FEATURE_EXTERNAL_API
void * FI_Interface::FI_CallFunction( const char* request, struct FI_AllocatorWrapper* allocWrapper)
{
	void* retPtr = NULL;
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIExternalAPIFncs &&
			 iMMObject->pFIFncs->pFIExternalAPIFncs->pFncFI_CallFunction )
			retPtr = iMMObject->pFIFncs->pFIExternalAPIFncs->pFncFI_CallFunction( iMMObject, request,allocWrapper);
		
	}
	FI_EXIT
	return retPtr;
}

void FI_Interface::FI_SetReturnValueFromContainer(const char* returnValue)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIExternalAPIFncs &&
			 iMMObject->pFIFncs->pFIExternalAPIFncs->pFncFI_SetReturnValueFromContainer )
			iMMObject->pFIFncs->pFIExternalAPIFncs->pFncFI_SetReturnValueFromContainer( iMMObject, returnValue);
		
	}
	FI_EXIT
	return;

}

#endif //FEATURE_EXTERNAL_API

short	FI_Interface::FI_GetMetaData( const unsigned char* swfBuffer,
										    unsigned long bufferLength,
                                            struct FI_MetaDataInfo* pMetaInfoStruct
										)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetMetaData)
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetMetaData( swfBuffer
													,bufferLength, pMetaInfoStruct);
		
	}
	FI_EXIT
	return -1;
}

short FI_Interface::FI_SetMaxVideoFramesToProcess (unsigned int numFrames)
	{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFINativeVideoFncs &&
			 iMMObject->pFIFncs->pFINativeVideoFncs->pFncFI_SetMaxVideoFramesToProcess)
			iMMObject->pFIFncs->pFINativeVideoFncs->pFncFI_SetMaxVideoFramesToProcess( iMMObject, numFrames );
		
	}
	FI_EXIT
	return -1;

	}

void FI_Interface::FI_NotifyFullScreenStatus (int flag)
{
	FI_ENTER
	if ( iMMObject && iMMObject->pFIFncs )
	{
		if ( iMMObject->pFIFncs->pFIFlashPlayerControlFncs &&
			 iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_NotifyFullScreenStatus )
		{
			iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_NotifyFullScreenStatus( iMMObject, flag );
		}
	}
	FI_EXIT
	return;
}

TInt FI_Interface::FI_GetThisSWFGUID(FI_ReturnString* aSwfGUID)
{
    FI_ENTER

    TInt error = KErrGeneral;
    if(iMMObject && iMMObject->pFIFncs && 
       iMMObject->pFIFncs->pFIFlashPlayerControlFncs && 
       iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetSWFGUID)
    {
        error = iMMObject->pFIFncs->pFIFlashPlayerControlFncs->pFncFI_GetSWFGUID(iMMObject, 0, NULL, 0, 0, aSwfGUID);

        if(error == 0)
        {
            error = KErrGeneral;
        }
        else
        {
            error = KErrNone;
        }
    }
    
    FI_EXIT
    return error;
}
//----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------
CHostAllocatorWrapper::CHostAllocatorWrapper()
{
	pMemAllocator = MallocWrapper;
	pMemDeallocator = FreeWrapper;
}

CHostAllocatorWrapper::~CHostAllocatorWrapper()
{
}

void* CHostAllocatorWrapper::MallocWrapper(AMP_AllocatorContext* /*aContext*/ ,size_t aSize)
{
	return User::Alloc(aSize);
}

void CHostAllocatorWrapper::FreeWrapper(AMP_AllocatorContext* /*aContext*/, void* aFreeString)
{
	User::Free(aFreeString);
}
