/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Watches GPU memory status and handles low GPU situation
*
*/


#include <e32base.h>
#include <e32cons.h>
#include <e32property.h>
#include <e32debug.h>

#include "gpumemwatch.h"
#include "gpumemwatchcommon.h"
    
const TUid KFlashRuntimeCat = {0x10207218};
const TUint KFlashGPUMemWatchKey = 0x10275064;

#ifdef _DEBUG
#define DEBUGPRINT(args...) RDebug::Printf(##args)
#define DEBUGPRINT_FBEGIN RDebug::Printf("CGPUMemWatch::%s ++",__FUNCTION__);
#define DEBUGPRINT_FEND   RDebug::Printf("CGPUMemWatch::%s --",__FUNCTION__);
#else
#define DEBUGPRINT(args...) 
#define DEBUGPRINT_FBEGIN 
#define DEBUGPRINT_FEND
#endif


CGPUMemWatch::CGPUMemWatch(MGPUMemStatusNotifier* aObserver)
    : CActive(CActive::EPriorityHigh),
      iObserver(aObserver)
    {
    }


void CGPUMemWatch::ConstructL()
    {
    CActiveScheduler::Add(this);    
    }


CGPUMemWatch* CGPUMemWatch::NewL(MGPUMemStatusNotifier* aObserver)
    {
#ifndef SYMBIAN_BUILD_GCE
    User::Leave(KErrNotSupported);
#endif   
    CGPUMemWatch* self = new (ELeave) CGPUMemWatch(aObserver);
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop(self);
    return self;
    }


CGPUMemWatch::~CGPUMemWatch()
    {
    DEBUGPRINT_FBEGIN;        
    Cancel();
    iProperty.Close();
    DEBUGPRINT_FEND;    
    }


void CGPUMemWatch::DoCancel()
    {
    DEBUGPRINT_FBEGIN;
    iProperty.Cancel();
    DEBUGPRINT_FEND;
    }


void CGPUMemWatch::RunL()
    {
    TBuf16<2> buf;
    TInt err = iProperty.Get(buf);
    if (err == KErrNotFound)
        {
        DEBUGPRINT("CGPUMemWatch::RunL value not found");
        }
    else if (err == KErrNone)
        {
        TInt16 aStatus = buf[0];
        TInt16 aFreeSpaceInMB =  buf[1];
        DEBUGPRINT("CGPUMemWatch::RunL status = %d freespace = %d",aStatus,aFreeSpaceInMB);
        
        TTime time;
        time.HomeTime();
        DEBUGPRINT("CGPUMemWatch::RunL %d %d",I64HIGH(time.Int64()), I64LOW(time.Int64()));
                    
        // Re-subscribe to the property
        IssueRequest();
        
        //Making sure callback is the last action in RunL
        if(aStatus == EGPUMemGood)
            iObserver->GPUMemoryLow(EFalse);
        else if(aStatus == EGPUMemLow)
            iObserver->GPUMemoryLow(ETrue);            
        }
    else
        {
        DEBUGPRINT("CGPUMemWatch::RunL err = %d",err);        
        User::Leave(err);
        }        
    }


void CGPUMemWatch::WatchL()
    {
    DEBUGPRINT_FBEGIN;
    User::LeaveIfError(iProperty.Attach(KFlashRuntimeCat,KFlashGPUMemWatchKey));
    
    TBuf16<2> buf;
    TInt res = iProperty.Get(buf);
    if (res == KErrNotFound)
        {
        DEBUGPRINT("CGPUMemWatch::WatchL property not defined");
        User::Leave(res);
        }
    
    IssueRequest();
    DEBUGPRINT_FEND;
    }


void CGPUMemWatch::IssueRequest()
    {
    iProperty.Subscribe(iStatus);
    SetActive();
    }

//end of file
