/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/


/*
 * Class : CHostEngineGlobals
 * Description : Contains any globals required by Flash2Engine.dll
 *               This class is loaded into the TLS of Flash2Engine.dll
 */

#include "platformbuild.h"      // Warning: this *must* the first include of every source file.
                                //          It contains all the #define build parameters.


// for SELF_IAD_UPDATE
#include "saplayer_build.h"

#include "hostengineglobals.h"
#ifdef SELF_IAD_UPDATE
#include "symbian_updatecheck.h"
#endif

CHostEngineGlobals::CHostEngineGlobals():
    iPlatformFlashEngine(NULL)
{
    // constructor
    iReferenceCount=0;

    iSharedMemory = NULL;
    
    #ifdef SELF_IAD_UPDATE
    iIADUpdateChecker = NULL;
    #endif

}

CHostEngineGlobals::~CHostEngineGlobals()
{
    // destructor
	if( iSharedMemory )
	{
		delete iSharedMemory;
		iSharedMemory = NULL;
	}
	
	#ifdef SELF_IAD_UPDATE
	if( iIADUpdateChecker )
	{
	    delete iIADUpdateChecker;
	    iIADUpdateChecker = NULL;
	}
	#endif
	
	// reclaim the memory allocated for EGLClients 
    #ifdef FEATURE_OPENVG    
    iEGLClients.Reset();
    iEGLClients.Close();
    #endif    
	
}

CHostEngineGlobals* CHostEngineGlobals::CreateHostEngineGlobalsL()
{
    CHostEngineGlobals* self=NULL;
    
    // Check local thread storage
    if( Dll::Tls() == NULL )
    {
        self = CHostEngineGlobals::NewLC();
        CleanupStack::Pop(); 
        return self;
    }
    else
    {        
        self = static_cast<CHostEngineGlobals*>(Dll::Tls()); 
        
        if(self)
        {
            self->IncrementReference();
            return self;
        }
    }

    return NULL;    
}

void CHostEngineGlobals::DestroyHostEngineGlobals()
{
    CHostEngineGlobals* self = static_cast<CHostEngineGlobals*>(Dll::Tls()); 
    if(self)
    {
        /* If this was the last reference */
        if( self->DecrementReference() == 0 )
        {
            /* free TLS first */
            Dll::FreeTls();
            
            /* self destruct! */
            delete self;
        }
    }
}

CHostEngineGlobals* CHostEngineGlobals::NewLC()
{
    // the first phase constructor
    CHostEngineGlobals* self = new (ELeave) CHostEngineGlobals();
    
    CleanupStack::PushL(self);
    self->ConstructL();
    return self;
}

void CHostEngineGlobals::ConstructL()
{
    // the second phase constructor
    iSharedMemory = CSharedMemory::NewL();

    TInt error = Dll::SetTls(static_cast<TAny*>(this));        
    
    if( error == KErrNone )
    {
        IncrementReference();
    }
    else
    {
       // User::Leave( reason );
    }
}

inline TUint32 CHostEngineGlobals::IncrementReference()
{
    iReferenceCount++;
    return iReferenceCount;
}

inline TUint32 CHostEngineGlobals::DecrementReference()
{
    iReferenceCount--;
    return iReferenceCount;
}


CSharedMemory* CHostEngineGlobals::SharedMemoryObject() const
{
    return iSharedMemory;
}

#ifdef SELF_IAD_UPDATE
CIADUpdateChecker* CHostEngineGlobals::IADUpdateChecker(const TUid& aApplicationUID
                                                        ,const TDesC& aAppSearchCriteria
                                                        ,const TDesC& aCommandLineExecutable 
                                                        ,const TDesC8& aCommandLineArguments
                                                        ,TBool aShowProgress
                                                        ,MIadUpdateCallback* aIADUpdateCompleteCallback
                                                        )
{
    if( iIADUpdateChecker )
    {
        return iIADUpdateChecker;
    }
    else
    {
           TRAPD( error, iIADUpdateChecker = CIADUpdateChecker::NewL( 
                                                                        aApplicationUID,
                                                                        aAppSearchCriteria,
                                                                        aCommandLineExecutable,
                                                                        aCommandLineArguments,
                                                                        aShowProgress,
                                                                        aIADUpdateCompleteCallback
                                                                     ) // NewL
                 ); // TRAPD
            
            if( error != KErrNone )
            {
                // could not create the object :(
                // There is no way to proceed further.
                // return to the caller.
                return NULL;
            }
            else
                return iIADUpdateChecker;
    }
    
}

#endif
#ifdef FEATURE_AMCEXTENSIONS
GeolocationAccess CHostEngineGlobals::GeolocationAccessStatus()
{
    return iGeolocationAccess;
}

void CHostEngineGlobals::GeolocationAccessStatus(GeolocationAccess status)
{
    iGeolocationAccess = status;
}
#endif
void CHostEngineGlobals::PlatformFlashEngine(CPlatformFlashEngine *aPlatformFlashEngine)
{
    iPlatformFlashEngine = aPlatformFlashEngine;
}
CPlatformFlashEngine* CHostEngineGlobals::GetPlatformFlashEngine()
{
    return iPlatformFlashEngine;
}

TInt CHostEngineGlobals::GetNoOfPlugins()
{
    return iReferenceCount;
}
#ifdef FEATURE_OPENVG
void CHostEngineGlobals::AddEGLClient(TAny* aClient)
{
    iEGLClients.Append(aClient);
}

void CHostEngineGlobals::RemoveEGLClient(TAny* aClient)
{
    TInt count = iEGLClients.Count();
    for(TInt index=0;index<count;index++)
    {
        if(iEGLClients[index] == aClient)
        {
            iEGLClients.Remove(index);
            break;
        }
    }
}

TInt CHostEngineGlobals::NumEGLClients()
{
    return iEGLClients.Count();
}
#endif

/*
void CHostEngineGlobals::UpdatesChecked( TBool aUpdateStatus )
{
    iUpdateCheckedStatus = aUpdateStatus;
}

TBool CHostEngineGlobals::UpdatesChecked()
{
    return iUpdateCheckedStatus;
}
*/

/*  CSharedMemory */
CSharedMemory::CSharedMemory()
{
    iSharedMemorySize = 0;      // size of the shared memory
    iPtrSharedMemory = NULL;    // a pointer to the shared memory
                                // memory is allocated only when the reference count is zero
    iReferenceCount = 0;        // keep track of how many objects are referencing the shared memory
}

CSharedMemory::~CSharedMemory()
{
    delete[] iPtrSharedMemory;
    iPtrSharedMemory = NULL;
}

CSharedMemory* CSharedMemory::NewL()
{
    CSharedMemory* self = CSharedMemory::NewLC();
    CleanupStack::Pop(self);
    return self;
}

CSharedMemory* CSharedMemory::NewLC()
{
    // the first phase constructor
    CSharedMemory* self = new (ELeave) CSharedMemory();
    
    CleanupStack::PushL(self);
    self->ConstructL();
    return self;
}

inline void CSharedMemory::ConstructL()
{
    // the second phase constructor
}

TAny* CSharedMemory::GetSharedMemoryL(TUint32 aSharedMemorySize)
{

    // if memory has not been allocated yet, allocate it.
    if( iReferenceCount == 0 )  // more clear than if( !iReferenceCount )
    {
        iPtrSharedMemory = new (ELeave) TUint8[aSharedMemorySize];
        if( iPtrSharedMemory )
        {
            iReferenceCount=0;  // we just created some shared memory
                                // reference count is incremented just before returning
                                // Its actually redundant as iReferenceCount *will* be 0 at this point
            
            iSharedMemorySize = aSharedMemorySize;
            Mem::FillZ( iPtrSharedMemory, aSharedMemorySize );
        }
    }
    
    // if memory has already been allocated, return the pointer
    iReferenceCount++;
    if( aSharedMemorySize != iSharedMemorySize )
        {
            // interesting situation!
            // the shared memory size has changed! 
            // what do we do?
        
            // ideally this should not happen
        
            User::Panic( _L("Different memory size"), KErrNotSupported );
        }
    
    // return it
    return iPtrSharedMemory;
}

void CSharedMemory::FreeSharedMemory()
{
    if( iReferenceCount > 0 )
    {
        iReferenceCount--;
    }
    else
    {
        // count was 0
        // there have been more Free()'s than Create()!
    }
    
    if( iReferenceCount == 0 )
    {
        // no more references to the shared memory
        // lets delete it!
        delete[] iPtrSharedMemory;
        iPtrSharedMemory = NULL;
    }
}
