/*
* Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class which implements the
*				 IP connection manager functionality
*
*/

#include "ipconn_mgr.h"
#include "symbian_engine.h"
#include "conn_handler.h"
#include "symbian_ui_interface.h"
#include "symbian_plugin_interface.h"

//this code is not needed as default is used every time IAP 
/*#if defined(FEATURE_ATS_AUTOMATION) || defined(FEATURE_PERFORMANCE_AUTOMATION)
#include "perfauto_util.h"
#endif*/


CIPConnMgr* CIPConnMgr::NewL(CPlatformFlashEngine* aEngine )
	{
    DEBUGPRINT("CIPConnMgr::NewL << ++");
	CIPConnMgr* self = new(ELeave) CIPConnMgr(aEngine);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	DEBUGPRINT("CIPConnMgr::NewL >> -- ");
	return self;	
	}

void CIPConnMgr::ConstructL()
	{
#if defined(FEATURE_ATS_AUTOMATION) || defined(FEATURE_PERFORMANCE_AUTOMATION) || defined(FEATURE_TESTAUTOMATION)
	iAutomationMode = iEngine->iPerfTestMode;
#endif
	}

						
CIPConnMgr::CIPConnMgr(CPlatformFlashEngine* aEngine):
						iEngine(aEngine)
						,iIAP(0)
						,iIAPStatus(EFLIAPNotSet)
						,iHasUserAllowedNetAccess(ENetworkQuery_NotSet)
						,iConnMgrStatus(EFLIPConnMgrIdle)
						
	{	
	}					
						
CIPConnMgr::~CIPConnMgr()
	{
	iEngine = NULL;
	if(iObsQueue.Count())
		{
		iObsQueue.ResetAndDestroy();
		}
	}

TFLIAPStatus CIPConnMgr::GetIAP(TUint32& aIAP) //returns iIAPStatus and if IAPStatus is EFLIAPAlreadySet then aIAP will point to currently set IAP.
	{
    DEBUGPRINT("CIPConnMgr::GetIAP << ++");
	if(iIAPStatus == EFLIAPAlreadySet)
		{		
		aIAP = iIAP;
		DEBUGPRINT("CIPConnMgr::GetIAP >> EFLIAPAlreadySet --");
		return EFLIAPAlreadySet;
		}
	DEBUGPRINT("CIPConnMgr::GetIAP >> EFLIAPNotSet -- ");
	return EFLIAPNotSet;		
	}

CConnHandler* CIPConnMgr::GetConnectionL(MFlhIPConnectObserver* aObserver)
	{
    DEBUGPRINT("CIPConnMgr::GetConnectionL << ++");
	
	
	CConnHandler* lConnectionHandler = CConnHandler::NewL(iEngine);	
	CleanupStack::PushL(lConnectionHandler);
	
	if(iConnMgrStatus == EFLIPConnMgrWaiting) 
		{
		//if already one connection is happening 
		//then we will return EFLIPConnectionWaiting and will try once first
		//connection finishes.
        DEBUGPRINT("CIPConnMgr::GetConnectionL << EFLIPConnectionWaiting Append to Queue");
		CObsEntry* lEntry = new (ELeave) CObsEntry(aObserver,lConnectionHandler);
		iObsQueue.AppendL(lEntry);		
		}
	else
		{
		IssueConnectionRequestL(aObserver, lConnectionHandler);	
		}	
	
	CleanupStack::Pop(lConnectionHandler);
	DEBUGPRINT("CIPConnMgr::GetConnectionL >> --");
	return lConnectionHandler;
	}

void CIPConnMgr::NotifyConnHandlerStatusL(TInt aEvent, CConnHandler* aConnHandler)
	{		
    DEBUGPRINT("CIPConnMgr::NotifyConnHandlerStatusL << ++");
	if(aEvent == EEventIsConnectedOK)
		{
		if(iIAPStatus == EFLIAPNotSet)
			{
			TInt lError = aConnHandler->GetIAP(iIAP);
			if(lError != KErrNotFound)
				{
				iIAPStatus = EFLIAPAlreadySet;
				}
			//iConnMgrStatus = EFLIPConnMgrIdle;
			//iBearerType = aConnHandler->GetBearerType();			
			}	
		}		

    iConnMgrStatus = EFLIPConnMgrIdle;
    
	//process next request
	if(iObsQueue.Count() > 0)
		{
		CObsEntry* lEntry = iObsQueue[0];
		IssueConnectionRequestL(lEntry->iIPConnObs,lEntry->iConnHandler);
		iObsQueue.Remove(0);
		delete lEntry;
		}
        DEBUGPRINT("CIPConnMgr::NotifyConnHandlerStatusL >> --");
	}


void CIPConnMgr::IssueConnectionRequestL(MFlhIPConnectObserver* aObserver, CConnHandler* aHandler)
	{
    DEBUGPRINT("CIPConnMgr::IssueConnectionRequestL << ++");
	#if	defined(FEATURE_ATS_AUTOMATION) || defined( FEATURE_PERFORMANCE_AUTOMATION) || defined(FEATURE_TESTAUTOMATION)
	if(iAutomationMode)
		{		
		aHandler->StartConnectionL(this , aObserver);
		return;
		}	
	#endif
	
	if(iHasUserAllowedNetAccess == ENetworkQuery_NotSet )
		{
		QueryAllowNetwork();
		}
	
	if(iHasUserAllowedNetAccess == ENetworkQuery_Deny )
		{		
		aHandler->HandleNetworkQueryDeniedL(this, aObserver);
		iConnMgrStatus = EFLIPConnMgrIdle;
		}		
	else if(iHasUserAllowedNetAccess == ENetworkQuery_Allow)
		{		
		aHandler->StartConnectionL(this , aObserver);
		iConnMgrStatus = EFLIPConnMgrWaiting;
		}
        DEBUGPRINT("CIPConnMgr::IssueConnectionRequestL >>-- ");	
	}


void CIPConnMgr::QueryAllowNetwork()
	{
	TBool lUserAllowed = ETrue;	// Allow by default
	
	TRAP_IGNORE(lUserAllowed = iEngine->QueryAllowNetAccessL());	
	
	if(lUserAllowed)
		{
		iHasUserAllowedNetAccess = ENetworkQuery_Allow;
		}
	else
		{
		iHasUserAllowedNetAccess = ENetworkQuery_Deny;
		}
	}

//end file
