/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the CFlhItemValuePairList class.
*
*/

#include "platformbuild.h"
#include "saplayer_build.h"
#include <barsread.h>

#include "item_val_pair_list.h"

CFlhItemValuePairList* CFlhItemValuePairList::NewLC(CEikonEnv& aEnv, TInt aArrayResId)
{
	CFlhItemValuePairList* self=new(ELeave) CFlhItemValuePairList(aArrayResId);
	CleanupStack::PushL(self);
	self->ConstructL(aEnv);
	return self;
}

CFlhItemValuePairList::~CFlhItemValuePairList()
{
	if(iKeyAndValue)
	{
		TFlhKeyAndValue* thisVal = NULL;
		for(TInt i=iKeyAndValue->Count()-1;i>=0;i--)
		{
			thisVal=&(*iKeyAndValue)[i];
			delete thisVal->iKey;
			delete thisVal->iValue;
		}
		delete iKeyAndValue;
	}
}

CFlhItemValuePairList::CFlhItemValuePairList(TInt aResourceId)
:iId(aResourceId)
{}

void CFlhItemValuePairList::ConstructL(CEikonEnv& aEnv)
{
	TResourceReader reader;
	aEnv.CreateResourceReaderLC(reader,iId);
	const TInt count=reader.ReadInt16();
	iKeyAndValue=new(ELeave) CArrayFixFlat<TFlhKeyAndValue>(count);

	TFlhKeyAndValue kAndV;

	for(TInt i=0;i<count;i++)
	{
		kAndV.iKey=reader.ReadHBufC16L();
		CleanupStack::PushL(kAndV.iKey);
		kAndV.iValue=reader.ReadHBufC16L();
		CleanupStack::PushL(kAndV.iValue);
		iKeyAndValue->AppendL(kAndV);
		CleanupStack::Pop(2,kAndV.iKey);
	}

	CleanupStack::PopAndDestroy(); // reader
}


TInt CFlhItemValuePairList::Id() const
{
	return iId;
}

const TPtrC CFlhItemValuePairList::ValueForKeyL(const TDesC& aKey) const
{
	const TFlhKeyAndValue* kAndV=NULL;

	for(TInt i=iKeyAndValue->Count()-1;i>=0;i--)
	{
		kAndV=&(*iKeyAndValue)[i];
		if(0==kAndV->iKey->Compare(aKey))
		{
			return TPtrC(*kAndV->iValue);
		}
	}

	User::Leave(KErrNotFound);
	return TPtrC(KNullDesC); // never reached, but shuts up compiler warning
}


