/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements location SAPI core class.
*
*/




#include "locationcoreimp.h"
#include "locationservice.h"
#include <e32debug.h>





//Constructor

CGetLoc :: CGetLoc():CActive( CActive::EPriorityStandard )
    {
    
    }
        
/**
 * Funtion name: ConstructL 
 * Constructs an active object and initalises this object 
 * with subsessions for communicating with location server
 */
        
void CGetLoc::ConstructL()
    {
    // Initialise the position request sequence
  
    DoInitialiseL();
    
    }


/**
 * Function name: NewL , 
 * Constructs a new core implementation object which will later 
 * Be added to active schedular 
 * on success returns pointer to constructed object, else leaves 
 * with various error codes 
*/
 
 
 
 
/*EXPORT_C*/ CGetLoc* CGetLoc::NewL( RPositionServer &aPositionServer , 
								 TPositionFieldIdList /*aList*/ ,
								 TInt aRequestType,
								 TInt /*aLocationInfoCategory*/  )
	{
	//Check for the presence of ActiveScheduler
    //Create the object
    CActiveScheduler *current = CActiveScheduler::Current() ;

    if(!current)
        {
        User::Leave(KErrNotFound); //Leave if consumer is not event driven
        }
    	

    CGetLoc* self = new( ELeave ) CGetLoc();
    CleanupStack::PushL(self);

    
    TInt error = (self->iPositioner).Open(aPositionServer) ; //Open a new subsession with position server
    
    
    if(error)
        {
        User :: Leave( error) ;
        }
    
    //Set the identity of this particular requestor
    User::LeaveIfError( (self->iPositioner).SetRequestor( CRequestor::ERequestorService,
													      CRequestor::EFormatApplication,
													      KIdentity ) );
    


    self->iPosInfoBase = &(self->iPositionInfo) ; 
    self->iRequestType = aRequestType ;
    User :: LeaveIfError(error) ;

    self->ConstructL();
    CleanupStack::Pop( self );

    //Return pointer to the created object
    return self;
	}

CGetLoc ::~CGetLoc()

    {
    Cancel();
    iPositioner.Close() ;

    }

TInt CGetLoc::DoInitialiseL()
    {
    iCallBack = NULL ;
    iRequestStatus =0;      
    return KErrNone  ;
    }


/**
 * Function Name :GetLocationUpdates
 * Notifies the user whenever his position changes
 * Returns status of the submitted job
 */

 TInt CGetLoc:: GetLocationUpdates( MLocationCallBack* aCallBackObj ,
 										 const TPositionUpdateOptions* aUpdateOptions )
    {
     

    //Register the call back address
    iCallBack = aCallBackObj ;

    if(aUpdateOptions)
    	{
    	 TInt error = iPositioner.SetUpdateOptions(*aUpdateOptions) ;
    	 
    	 if(error)     //If update options failed then no return error to caller
     		{
    	 	return error ; 
     		}
    	}
    else //set the default values
        {
        TPositionUpdateOptions updateopts ;
        
        // Set update interval to one second to receive one position data per second
	    updateopts.SetUpdateInterval(TTimeIntervalMicroSeconds(KSUpdateInterval));

	    // If position server could not get position
	    // In two minutes it will terminate the position request
	    updateopts.SetUpdateTimeOut(TTimeIntervalMicroSeconds(KSUpdateTimeOut));

	    // Positions which have time stamp below KMaxAge can be reused
	    updateopts.SetMaxUpdateAge(TTimeIntervalMicroSeconds(KSMaxAge));

	    // Enables location framework to send partial position data
	    updateopts.SetAcceptPartialUpdates(FALSE);

        
        iPositioner.SetUpdateOptions(updateopts) ;
       
        }


    if(!(this->IsAdded()))  //Add to the active list if not added 
    {
    	CActiveScheduler :: Add(this) 	;
    }
    
    if (this->IsActive())
    {               
        Cancel();                
    }

    iPositioner.NotifyPositionUpdate( iCourseInfo, iStatus );
    SetActive();
   
   
    return KErrNone ;
    }


/**
 * RunL Method called by Active Scheduler after reciving an event 
 * This method actually calls the registered callbacks 
 */

void CGetLoc :: RunL()
	{
    
   
	switch ( iStatus.Int() )
	{
	    

		case KErrNone:                    //For both KErrNone nad partial update call the same call back
		case KPositionPartialUpdate :
		{
		
			if(iCallBack)
			{
					iCallBack->HandleNotifyL(iCourseInfo , KErrNone ) ;	        		
			}

			if(iRequestType == KTraceRequest)	
			{
				if(this->IsAdded())
				
				{
					if(!this->IsStatusComplete())
					{
						iPositioner.NotifyPositionUpdate( iCourseInfo, iStatus );
						SetActive();
					}
					 	
				}
			}
			 

			break ;
			}

		case KErrArgument:
		   
		case KPositionQualityLoss:         //Need to check this returnig s60 error code from here
		    
		case KErrTimedOut :
		    
		case KErrNotFound :       //Service provider not available, or gone down in between
		{
           
			iCallBack->HandleNotifyL(iCourseInfo , iStatus.Int()) ;
			break ;
		}

		default :
		{
           
			iCallBack->HandleNotifyL(iCourseInfo , iStatus.Int()) ;
			break ;

		}
	} //End of Switch

	
	        	
	}

/**
 * Handle the leaves occuring in RunL()
*/

TInt CGetLoc::RunError(TInt aError)
	{
	return aError;
	}


/**
 *DoCancel Method called by Cancel Method 
 */
void CGetLoc :: DoCancel()
    {
   
    //Check weather any one has registered for callbacks then
    if(iCallBack)
    	{
    	iPositioner.CancelRequest(EPositionerNotifyPositionUpdate);
    	}
    

    }
