/*
* Copyright (c) 2006-2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements location SAPI core ;works as wrapper
*   around core implementaion.
*
*/



#include "locationcoreimp.h"
#include "locationservice.h"
#include "si_interface.h"
#include <e32debug.h>





/**
 * Default constructor
 */
 
 
CLocationService :: CLocationService()
    {
	//No Implementation Required Here
    }


/**
 * Destructor
 */

CLocationService :: ~CLocationService()
    {
    iPositioner.CancelRequest(EPositionerNotifyPositionUpdate);
    iPositioner.Close();	
    
    if(iRegTable)
        {
        if(iRegTable->IsActive())
            {
            iRegTable->Deque();
            }
        delete iRegTable;
        iRegTable = NULL;
        }
    
    iPosServer.CancelRequest(EPositionServerNotifyModuleStatusEvent);
    iPosServer.Close();
    //iRegTable.Close();

    }
 
/**
 * CLocationService::NewL
 * Two-phased constructor.
 */


CLocationService *CLocationService :: NewL()
    {
   
    CLocationService *self = new(ELeave)CLocationService() ;
    CleanupStack::PushL(self);
    self->ConstructL() ;
    CleanupStack::Pop(self) ;
    return self ;
   
    }
 
/**
 * CLocationService::ConstructL
 * Symbian 2nd phase constructor can leave.
 */

void CLocationService::ConstructL()
    {

    DoInitialiseL();
    }



/**
 * CLocationService::DoInitialiseL
 * Initialises position server and positioner and
 * begins the position request sequence.
 */

void CLocationService :: DoInitialiseL()
    {
    
    TInt error = iPosServer.Connect( );
    // The connection failed
  
    User :: LeaveIfError(error) ;
    
    // Open subsession to the position server
    error = iPositioner.Open(iPosServer);

    // The opening of a subsession failed
    if ( KErrNone != error )
        {
        iPosServer.Close();
        User :: Leave( error );
        }


    //setting identity for this requestor
    User::LeaveIfError( iPositioner.SetRequestor( CRequestor::ERequestorService,
												  CRequestor::EFormatApplication,
												  KIdentity ) );
    
    //Initialise index which means there is no 
    //active object created yet
    iIndex = 0 ;
    
	//Getthe module id used by location server for sapi location calls.
    //error = iPosServer.GetDefaultModuleId(iModuleId);
    
    //User :: LeaveIfError(error) ;
   
    }


void CLocationService::SetUpdateInterval(MLocationCallBack* aCallBackObj , const TPositionUpdateOptions* aUpateOptions)
    {
  
        TInt ret  =  iRegTable->GetLocationUpdates(aCallBackObj , aUpateOptions) ;

        
           
    }

TInt CLocationService :: TraceL( MLocationCallBack* aCallBackObj ,
												TInt aLocationInfoCategory, 
												TPositionFieldIdList aFiledList ,
												const TPositionUpdateOptions* aUpateOptions )
    {
    
    CGetLoc *activeTrace;

 
   
   
    
    activeTrace = CGetLoc::NewL(iPosServer ,aFiledList ,KTraceRequest,aLocationInfoCategory) ;
   

    TInt ret  = activeTrace->GetLocationUpdates(aCallBackObj , aUpateOptions) ;

    if (ret == KErrNone)  
        {
        //iIndex = KARRAY_INDEX_TRACE;	
        //iRegTable[KARRAY_INDEX_TRACE] = activeTrace ;
        iRegTable = activeTrace ;
        }
    else
        {         //Cleanup the allocated object
        delete activeTrace ;
        }	
    
    return ret;
    }






/**
 * CancelOnGoingService : Cancells Requested asynchronous requests,
 * Input : Type of request to be cancelled(Trace , Getlocation),
 * Returns success(KErrNone) if service cancelled or else error if 
 * input parameter is invalid
 */
TInt CLocationService::CancelOnGoingService()
    {
    

    if (iRegTable->IsAdded())
        {
        iRegTable->Deque();
        }
    iRegTable->SetStatusComplete();
    delete iRegTable;
    iRegTable = NULL;
    
    return KErrNone;
    }

