/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  FlashConfigGen class implementation
*
*/


// INCLUDE FILES
#include "platformconfig.h"
#include <centralrepository.h> //CRepository
#ifndef ENABLE_SF_CHANGES
#include <Flashlite_2_0PrivateCRKeys.h> //senRep keys
#else
#include "Flashlite_2_0PrivateCRKeys.h" //senRep keys
#endif

#ifdef FEATURE_FLV_MMC

const TUint32 KMMCResHighWatermark = 400 << 20; //400 MB  MMC max
const TUint32 KMMCResLowWatermark = 20 << 20; //20 MB min
const TUint32 KMMCResUsagePolicy = 1;
const TUint32 KRAMResLowWatermark = 6 << 20 ; //2 MB min RAM space if MMC not present
const TUint32 KRAMResHighWatermark = 8 << 20; // 8MB max RAM space if MMC not present
const TUint32 KMMCResvUnUtilisedSpace = 20 << 20; // 20MB min reserve space in MMC
const TUint32 KFlashRAMResvUnUtilisedSpace = 5 << 20; // 5MB min free RAM 
const TUint32 KVideoDynHeapSize = 40 << 20; // 40 MB max space for FLV
const TUint32 KFLVMaxFramesToDecode = 3;
const TUint32 KMBMultiplier = 1024*1024;


_LIT(KVideoResDrive,"E://");

#ifdef FEATURE_MULTIPLE_DRIVE
const TUint32 KMMCDriveOverridePolicy=1;
#endif


// ----------------------------------------------------------------------
// CPlatformConfig::CPlatformConfig()
//
// Constructor for CPlatformConfig
// ----------------------------------------------------------------------

CPlatformConfig::CPlatformConfig()
{
	iMMCResHighWatermark = KMMCResHighWatermark;
	iMMCResLowWatermark = KMMCResLowWatermark;
	iMMCResUsagePolicy = KMMCResUsagePolicy;
	iRAMResLowWatermark = KRAMResLowWatermark;
	iRAMResHighWatermark = KRAMResHighWatermark;
	iMMCResvUnUtilisedSpace = KMMCResvUnUtilisedSpace;
	iFlashRAMResvUnUtilisedSpace = KFlashRAMResvUnUtilisedSpace;
	iVideoDynHeapSize = KVideoDynHeapSize;
	iFLVMaxFramesToDecodePerInterval=KFLVMaxFramesToDecode;
	#ifdef FEATURE_MULTIPLE_DRIVE
	iMMCDriveOverridePolicy=KMMCDriveOverridePolicy;
	#endif
	iVideoResDrive=KVideoResDrive;
	
	FillConfigFromRepoEngineData();
}


// ----------------------------------------------------------------------
// CPlatformConfig::~CPlatformConfig()
//
// Destructor for CPlatformConfig
// ----------------------------------------------------------------------

CPlatformConfig::~CPlatformConfig()
{
}

// ----------------------------------------------------------------------
// CPlatformConfig::FillConfigFromRepoEngineData()
//
// To fill config from central repository
// ----------------------------------------------------------------------
	
void CPlatformConfig::FillConfigFromRepoEngineData()
	{
		CRepository* repository=NULL;
	  	TInt err = KErrNotFound;
	  	TRAP( err, repository = CRepository::NewL(KCRUIdFlashlitePlatformEngine));
	  	
	   	if(err == KErrNone)
	  	{
		  	TInt mmcHighWaterMark = KMMCResHighWatermark;
			TInt mmcLowWaterMark = KMMCResLowWatermark;
			TInt mmcUsagePolicy = KMMCResUsagePolicy;
			TInt ramLowWatermark = KRAMResLowWatermark;
			TInt ramHighWatermark = KRAMResHighWatermark;
			TInt mmcResvUnUtilisedSpace =KMMCResvUnUtilisedSpace;
			TInt flashRAMResvUnUtilisedSpace =KFlashRAMResvUnUtilisedSpace;
			TInt flvMaxDynHeapSize = KVideoDynHeapSize;
			TInt flvMaxFramesToDecodePerInterval = KFLVMaxFramesToDecode;
			#ifdef FEATURE_MULTIPLE_DRIVE
			TInt mmcDriveOverridePolicy=KMMCDriveOverridePolicy;
			#endif
			TBuf<KMaxDrvLength> ptrVideoResDrive(KVideoResDrive);
			
#ifdef FEATURE_FLV_MMC			
			TInt error = repository->Get(KFlashLiteMMCResHighWatermark, mmcHighWaterMark);
			if(error == KErrNone)
				iMMCResHighWatermark = mmcHighWaterMark * KMBMultiplier;
			if((repository->Get(KFlashLiteMMCResLowWatermark, mmcLowWaterMark)) == KErrNone)
				iMMCResLowWatermark = mmcLowWaterMark * KMBMultiplier;
			if((repository->Get(KFlashliteMMCResUsagePolicy, mmcUsagePolicy)) == KErrNone)
				iMMCResUsagePolicy = mmcUsagePolicy;
			if((repository->Get(KFlashliteRAMResLowWatermark, ramLowWatermark)) == KErrNone)
				iRAMResLowWatermark = ramLowWatermark * KMBMultiplier;
		  	if((repository->Get(KFlashliteRAMResHighWatermark, ramHighWatermark)) == KErrNone)
				iRAMResHighWatermark = ramHighWatermark* KMBMultiplier;
		  	if((repository->Get(KFlashliteMMCResvUnUtilisedSpace, mmcResvUnUtilisedSpace)) == KErrNone)
				iMMCResvUnUtilisedSpace = mmcResvUnUtilisedSpace * KMBMultiplier;
		  	if((repository->Get(KFlashliteVideoResDrive, ptrVideoResDrive)) == KErrNone)
				iVideoResDrive = ptrVideoResDrive;
		  			  	
		  	#ifdef FEATURE_MULTIPLE_DRIVE
		  	if((repository->Get(KFlashliteMMCDriveOverridePolicy, mmcDriveOverridePolicy)) == KErrNone)
		  		iMMCDriveOverridePolicy = mmcDriveOverridePolicy;
		  	#endif
#endif //FEATURE_FLV_MMC
		  	if((repository->Get(KFlashLiteRAMResUnUtilisedSpace, flashRAMResvUnUtilisedSpace)) == KErrNone)
				iFlashRAMResvUnUtilisedSpace = flashRAMResvUnUtilisedSpace * KMBMultiplier;
		  	if((repository->Get(KFlashLiteVideoDynHeapSize, flvMaxDynHeapSize)) == KErrNone)
				iVideoDynHeapSize = flvMaxDynHeapSize * KMBMultiplier;
		  	if((repository->Get(KFLVMaxFramesToDecodePerInterval, flvMaxFramesToDecodePerInterval)) == KErrNone)
				iFLVMaxFramesToDecodePerInterval = flvMaxFramesToDecodePerInterval;
		 }
	  	// Close connection
	    delete repository; 
	}
// ----------------------------------------------------------------------
// 	TUint32 CPlatformConfig::GetMMCHighWatermark()
//
// This function is used to get the value from cenrep
// ----------------------------------------------------------------------
#ifdef FEATURE_FLV_MMC

TReal CPlatformConfig::GetMMCResHighWatermark()
	{
		return iMMCResHighWatermark;
	}
	
// ----------------------------------------------------------------------
// 	TUint32 CPlatformConfig::GetMMCLowWatermark()
//
// This function is used to get the value from cenrep
// ----------------------------------------------------------------------
TReal CPlatformConfig::GetMMCResLowWatermark()
	{
		return iMMCResLowWatermark;
	}
// ----------------------------------------------------------------------
// 	TUint32 CPlatformConfig::GetMMCUsagePolicy()
//
// This function is used to get the value from cenrep
// ----------------------------------------------------------------------
TInt CPlatformConfig::GetMMCResUsagePolicy()
	{
	return iMMCResUsagePolicy;
	}
// ----------------------------------------------------------------------
// 	TUint32 CPlatformConfig::GetRAMLowWatermark()
//
// This function is used to get the value from cenrep
// ----------------------------------------------------------------------
TReal CPlatformConfig::GetRAMResLowWatermark()
	{
		return iRAMResLowWatermark;
	}
// ----------------------------------------------------------------------
// 	TUint32 CPlatformConfig::GetRAMHighWatermark()
//
// This function is used to get the value from cenrep
// ----------------------------------------------------------------------	
TReal CPlatformConfig::GetRAMResHighWatermark()
	{
		return iRAMResHighWatermark;
	}
// ----------------------------------------------------------------------
// 	TUint32 CPlatformConfig::GetMMCResvUnUtilisedSpeace()
//
// This function is used to get the value from cenrep
// ----------------------------------------------------------------------	
TInt CPlatformConfig::GetMMCResvUnUtilisedSpace()
	{
		return iMMCResvUnUtilisedSpace;
	}
	
#endif //FEATURE_FLV_MMC
// ----------------------------------------------------------------------
// 	TUint32 CPlatformConfig::GetFlashRAMResvUnUtilisedSpeace()
//
// This function is used to get the value from cenrep
// ----------------------------------------------------------------------	
TInt CPlatformConfig::GetFlashRAMResvUnUtilisedSpace()
	{
		return iFlashRAMResvUnUtilisedSpace;
	}
// ----------------------------------------------------------------------
// 	TUint32 CPlatformConfig::GetFLVMaxDynHeapSize()
//
// This function is used to get the value from cenrep
// ----------------------------------------------------------------------
TInt CPlatformConfig::GetVideoDynHeapSize()
	{
		return iVideoDynHeapSize;
	}
// ----------------------------------------------------------------------
// 	TUint32 CPlatformConfig::GetFLVMaxFramesToDecodePerInterval()
//
// This function is used to get the value from cenrep
// ----------------------------------------------------------------------	
TInt CPlatformConfig::GetFLVMaxFramesToDecodePerInterval()
	{
		return iFLVMaxFramesToDecodePerInterval;
	}
	
// ----------------------------------------------------------------------
// 	TUint32 CPlatformConfig::GetMMCDriveOverridePolicy()
//
// This function is used to get the value from cenrep
// ----------------------------------------------------------------------	

#ifdef FEATURE_MULTIPLE_DRIVE
TInt CPlatformConfig::GetMMCDriveOverridePolicy()
	{
		return iMMCDriveOverridePolicy;
	}
#endif ///FEATURE_MULTIPLE_DRIVE

// ----------------------------------------------------------------------
// 	TUint32 CPlatformConfig::GetFLVMaxFramesToDecodePerInterval()
//
// This function is used to get the value from cenrep
// ----------------------------------------------------------------------	
TDesC& CPlatformConfig::GetVideoResDrive()
	{
		return iVideoResDrive;
	}
	
#endif //MMC

