/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
/*
 * sharedmemory.cpp
 *
 *  Created on: Oct 25, 2009
 *      Author: vsriram
 */

#include "sharedmemoryclient.h"
#include "flashrtpolicycontentsession.h"
#include <rtsecmgrcommondef.h>
#include "flashrtpolicyclient.h"

CSharedMemoryClient* CSharedMemoryClient::NewL(const char* aName, const char* aLockName, unsigned long aSize)
    {
    CSharedMemoryClient* self = new (ELeave) CSharedMemoryClient();
    CleanupStack::PushL(self);
    self->ConstructL(aName,aLockName,aSize);
    CleanupStack::Pop(self);
    return self;
    }

CSharedMemoryClient::CSharedMemoryClient()
    {
    }

CSharedMemoryClient::~CSharedMemoryClient()
    {
    if(iPolicyClient)
    	{
        delete iPolicyClient;
        iPolicyClient = NULL;
		}
    }

void CSharedMemoryClient::ConstructL(const char* aName, const char* aLockName, unsigned long aSize)
    {
    //create a policy client instance if not created already
    if(!iPolicyClient)
        iPolicyClient = CFlashRTPolicyClient::NewL();

    //Thro' policy client, send a message to the server to create a global
    //chunk
    User::LeaveIfError(iPolicyClient->CreateSharedMemoryL(aName, aLockName, aSize, iChunk, iLockId));
    }

LockId CSharedMemoryClient::LockHandle()
    {
    return (iLockId);
    }

void* CSharedMemoryClient::BaseAddress()
    {
    return iChunk.Base();
    }

TInt CSharedMemoryClient::Lock()
    {
    if(!iPolicyClient)
        return KErrGeneral;

    TRAPD(err, iPolicyClient->LockSharedMemoryL(iLockId));
    return err;
    }

TInt CSharedMemoryClient::Unlock()
    {
    if(!iPolicyClient)
        return KErrGeneral;

    TRAPD(err, iPolicyClient->UnlockSharedMemoryL(iLockId));
    return err;
    }
TInt CSharedMemoryClient::DeleteSharedMemory()
	{
	if(!iPolicyClient)
		return KErrGeneral;

    TRAPD(err, iPolicyClient->DeleteSharedMemoryL(iLockId));
    return err;
	}

