/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
/*
 * sharedmemorymanager.cpp
 *
 *  Created on: Oct 25, 2009
 *      Author: vsriram
 */

#include "sharedmemorymanager.h"
#include "commondef.h"

CSharedMemoryManager* CSharedMemoryManager::CreateInstanceL()
    {
    return new (ELeave) CSharedMemoryManager();
    }

CSharedMemoryClient* CSharedMemoryManager::CreateSharedMemoryL(const char* name, const char* lockName,
        unsigned long size)
    {
    CSharedMemoryClient* shMemory = CSharedMemoryClient::NewL(name,lockName,size);
    CleanupStack::PushL(shMemory);
    iSharedMemorySegments.AppendL(shMemory);
    CleanupStack::Pop(shMemory);
    return shMemory;
    }

TInt CSharedMemoryManager::DeleteSharedMemory(LockId lockId)
    {
	TInt err = KErrNone;
    //get the shared memory instance associated with the lock...
    //and delete the shared memory..
    for (TInt i=0; i<iSharedMemorySegments.Count(); i++)
       	{
       	if(iSharedMemorySegments[i]->LockHandle() == lockId)
       		{
       		CSharedMemoryClient* shMemory = iSharedMemorySegments[i];
       		err = shMemory->DeleteSharedMemory();
       		iSharedMemorySegments.Remove(i);
       		iSharedMemorySegments.Compress();
       		if(shMemory)
       			{
       			delete shMemory;
       			shMemory = NULL;
       			}
       		break;
       		}
       	}
    if(err == KErrNone)
	   return SI_NoError;
	else
       return err;
    }
CSharedMemoryManager::~CSharedMemoryManager()
    {
    iSharedMemorySegments.ResetAndDestroy();
    iSharedMemorySegments.Close();
    }

TInt CSharedMemoryManager::Lock(LockId aLockId)
    {
	TInt err = KErrNone;
    for (TInt i=0; i<iSharedMemorySegments.Count(); i++)
    	{
    	//get the shared memory instance associated with the lock...
    	if(iSharedMemorySegments[i]->LockHandle() == aLockId)
    		{
    		//unlock the shared memory..
    		err = iSharedMemorySegments[i]->Lock();
    		break;
    		}
    	}
    if(err == KErrNone)
    	return SI_NoError;
    else
    	return err;
    }

TInt CSharedMemoryManager::Unlock(LockId aLockId)
    {
	TInt err = KErrNone;
    for (TInt i=0; i<iSharedMemorySegments.Count(); i++)
		{
		//get the shared memory instance associated with the lock...
		if(iSharedMemorySegments[i]->LockHandle() == aLockId)
			{
			//unlock the shared memory..
			err = iSharedMemorySegments[i]->Unlock();
			break;
			}
		}
	if(err == KErrNone)
	    return SI_NoError;
	 else
    	return err;
    }
