/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Contains shared object utility functions
*
*/


#ifndef USE_SYNC_MSO

#include "platformbuild.h"
#include "saplayer_build.h"

#include "ShObjUtils.h"
#include "symbian_file.h"
#include "symbian_engine.h"

#include <utf.h>
#include <bautils.h>
// local utility function
static void DestroyList(TSglQue<CPersistentStorageNode>* aNodeList)
{
	if(aNodeList)
	{
		CPersistentStorageNode* item;
		while(!aNodeList->IsEmpty())
		{
			item=aNodeList->Last();
			aNodeList->Remove(*item);
			delete item;
		}
		delete aNodeList;
	}	
}

// class CFlhShObjNodeLister
class CFlhShObjNodeLister : public CBase, public MFlhAsyncHandler
{
public:
	static CFlhShObjNodeLister* NewL(const char* aNameSpace, TSglQue<CPersistentStorageNode>*& aNodeList,MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils);
	~CFlhShObjNodeLister();
private:
	CFlhShObjNodeLister(TSglQue<CPersistentStorageNode>*& aNodeList,MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils);
	void ConstructL(const char* aNameSpace);
public:
	//from MFlhAsyncHandler
	void DoRunL(const CActive* aActive);
	TInt DoRunError(const CActive* aActive, TInt aError);

private:
	char* iNameSpace;
	TSglQue<CPersistentStorageNode>* iDirNodeList;
	TSglQue<CPersistentStorageNode>* iFileNodeList;
	TSglQue<CPersistentStorageNode>*& iClientNodeList;
	MFlhShObjNodeListObserver& iObs;
	CFlhAsyncObject* iIdle;
	CFlhShObjUtils& iUtils;
	CDir* iList1;
    CDir* iList2;
	TInt iStage;
	CPersistentStorageNode* iCurrentDirNode; 
	TInt iFileIndex;
	TInt iDirIndex;
};


//static 
CFlhShObjNodeLister* CFlhShObjNodeLister::NewL(const char* aNameSpace, TSglQue<CPersistentStorageNode>*& aNodeList,MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils)
{
	CFlhShObjNodeLister* self=new(ELeave) CFlhShObjNodeLister(aNodeList,aObs,aUtils);
	CleanupStack::PushL(self);
	self->ConstructL(aNameSpace);
	CleanupStack::Pop(self);
	return self;
}


CFlhShObjNodeLister::~CFlhShObjNodeLister()
{
	delete[] iNameSpace;
	delete iIdle;
	delete iList1;
	delete iList2;
	DestroyList(iFileNodeList);
	DestroyList(iDirNodeList);
}

CFlhShObjNodeLister::CFlhShObjNodeLister(TSglQue<CPersistentStorageNode>*& aNodeList,MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils)
:iClientNodeList(aNodeList), iObs(aObs), iUtils(aUtils)
{}

void CFlhShObjNodeLister::ConstructL(const char* aNameSpace)
{
	if(aNameSpace)
	{
		TPtrC8 des((const TUint8*)aNameSpace);
		TInt strln=des.Length()+1;
		iNameSpace = new char[des.Length()+1];
		Mem::Copy(iNameSpace,aNameSpace,strln);
	}
	iIdle=CFlhAsyncObject::NewL(CActive::EPriorityIdle,*this);
	iIdle->Start();
}

void CFlhShObjNodeLister::DoRunL(const CActive* /*aActive*/)
{	
	iIdle->Start(); // requeue
	if(iStage==0)
	{
		// init stuff
		iDirNodeList=new(ELeave)TSglQue<CPersistentStorageNode>(_FOFF(CPersistentStorageNode, iLinkItem));
		iFileNodeList=new(ELeave)TSglQue<CPersistentStorageNode>(_FOFF(CPersistentStorageNode, iLinkItem));
		CPersistentStorageNode* node = CPersistentStorageNode::NewL(iNameSpace); // nameSpace may be NULL
		iDirNodeList->AddLast(*node);
		iStage++;
	}

	if(iStage==1)
	{
		// working on a current level of the tree
		if(iCurrentDirNode)
		{
			const char* currentNameSpace = iCurrentDirNode->GetNameSpace();

			TInt yieldcounter=0;
			// Add all subdirectories to dirList.

			const TInt dirCount=iList2->Count();
			for (; iDirIndex < dirCount; iDirIndex++)
			{
				if(yieldcounter==10)
					break;
				yieldcounter++;
				TBuf8<KMaxFileName> newNameSpace;
				if(currentNameSpace)
				{
					newNameSpace.Append((const TUint8*)currentNameSpace, FlashStrLen(currentNameSpace));
					newNameSpace.Append(KPathDelimiter);
				}
				newNameSpace.Append((*iList2)[iDirIndex].iName);
				newNameSpace.ZeroTerminate();
				CPersistentStorageNode* node = CPersistentStorageNode::NewL((const char*)newNameSpace.Ptr());
				iDirNodeList->AddLast(*node);
			}
			
			// Insert any files to the fileList
			const TInt fileCount=iList1->Count();
			for (; iFileIndex < fileCount; iFileIndex++)
			{
				if(yieldcounter++==10)
					break;
				// Add all files to fileList
				HBufC* name = HBufC::NewLC((*iList1)[iFileIndex].iName.Length()); //2

				name->Des().Append((*iList1)[iFileIndex].iName);
				CPersistentStorageNode* newNode = CPersistentStorageNode::NewL(currentNameSpace, name);
 				iFileNodeList->AddLast(*newNode);
				CleanupStack::PopAndDestroy(name); //1
			}
			
			if(iDirIndex==iList2->Count() && iFileIndex==iList1->Count())
			{
			
				// Done, Remove the current directory
				iDirNodeList->Remove(*iCurrentDirNode);
				delete iCurrentDirNode;
				iCurrentDirNode=NULL;
				delete iList1;
				delete iList2;
				iList2=iList1=NULL;
				iFileIndex=iDirIndex=0;
			}
		}
		//done - success
		else if(iDirNodeList->IsEmpty())
		{	
			//transfer ownership
			iClientNodeList=iFileNodeList;
			iFileNodeList=NULL;
			// done
			iStage++;
			iObs.NotifyOperationComplete(this,KErrNone);
		}
		// get next level of tree
		else
		{
			iCurrentDirNode = (CPersistentStorageNode*)iDirNodeList->Last();

			const char* currentNameSpace = iCurrentDirNode->GetNameSpace();

			TFileName* dirName = NULL;
			if(currentNameSpace)
			{
				dirName = iUtils.GetPersistenceDir(currentNameSpace);
			}
			else
			{
				dirName = new(ELeave) TFileName;
                dirName->Copy(iUtils.PersistentDirectory());
			}

			TInt err = iUtils.FsSession().GetDir(*dirName, KEntryAttNormal, ESortByName, iList1, iList2);
			delete dirName;
			
			if(err)
			{
				// Done, Remove the current directory
				iDirNodeList->Remove(*iCurrentDirNode);
				delete iCurrentDirNode;
				iCurrentDirNode=NULL;
				delete iList1;
				delete iList2;
				iList2=iList1=NULL;
			}	
		}
	}
	else
	{
		__ASSERT_DEBUG(EFalse,User::Invariant()); // should have been deleted by owner before we get here
	}
}

TInt CFlhShObjNodeLister::DoRunError(const CActive* /*aActive*/, TInt aError)
{
	iObs.NotifyOperationComplete(this,aError);
	return KErrNone;
}


//
// class CFlhShObjNodeListUserBase
//
class CFlhShObjNodeListUserBase : public CBase, public MFlhAsyncHandler, public MFlhShObjNodeListObserver
{
public:
	~CFlhShObjNodeListUserBase();
protected:
	CFlhShObjNodeListUserBase(MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils);
	void BaseConstructL(const char* aNameSpace);
public:
	
	//from MFlhAsyncHandler
	void DoRunL(const CActive* aActive);
	TInt DoRunError(const CActive* aActive, TInt aError);

	//from MFlhShObjNodeListObserver
	void NotifyOperationComplete(void* aServicingObject, TInt aError);

protected:
	virtual void DoNextStepL()=0;
private:
	CFlhAsyncObject* iIdle;
protected:
	char* iNameSpace;
	TSglQue<CPersistentStorageNode>* iNodeList;
	MFlhShObjNodeListObserver& iObs;
	CBase* iNodeLister;
	CFlhShObjUtils& iUtils;
	TSglQueIter<CPersistentStorageNode>* iIter;
};


CFlhShObjNodeListUserBase::~CFlhShObjNodeListUserBase()
{
	delete [] iNameSpace;
	iNameSpace = NULL;
	delete iIdle;
	delete iNodeLister;
	DestroyList(iNodeList);
	delete iIter;
}


CFlhShObjNodeListUserBase::CFlhShObjNodeListUserBase(MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils)
:iObs(aObs), iUtils(aUtils)
{}

void CFlhShObjNodeListUserBase::BaseConstructL(const char* aNameSpace)
{
	if(aNameSpace)
	{
		TPtrC8 des((const TUint8*)aNameSpace);
		TInt strln=des.Length()+1;
		iNameSpace = new char[des.Length()+1];
		Mem::Copy(iNameSpace,aNameSpace,strln);
	}
	iIdle=CFlhAsyncObject::NewL(CActive::EPriorityIdle,*this);
	iIdle->Start();	
}


void CFlhShObjNodeListUserBase::DoRunL(const CActive* /*aActive*/)
{
	if(iNodeList==NULL)
	{	
		// Extract all shared object files in systems
		iNodeLister=iUtils.ListAllNodesL(iNameSpace, iNodeList, *this);
	}
	else
	{
		if(iIter==NULL)
		{
			iIter = new(ELeave) TSglQueIter<CPersistentStorageNode>(*iNodeList);
		}
		iIdle->Start(); //requeue
		DoNextStepL();
	}
}

TInt CFlhShObjNodeListUserBase::DoRunError(const CActive* /*aActive*/, TInt aError)
{
	iObs.NotifyOperationComplete(this,aError);
	return KErrNone;
}
#ifdef _DEBUG
void CFlhShObjNodeListUserBase::NotifyOperationComplete(void* aServicingObject,TInt aError)
#else
void CFlhShObjNodeListUserBase::NotifyOperationComplete(void* /*aServicingObject*/,TInt aError)
#endif
{
	__ASSERT_DEBUG(iNodeLister==aServicingObject,User::Invariant());
	if(KErrNone!=aError)
	{
		iObs.NotifyOperationComplete(this,aError);
	}
	else
	{
		iIdle->Start();
	}
	delete iNodeLister;
	iNodeLister=NULL;
}

// class CFlhShObjNameSpaceSizer

class CFlhShObjNameSpaceSizer : public CFlhShObjNodeListUserBase
{
public:
	static CFlhShObjNameSpaceSizer* NewL(const char* aNameSpace, TInt& aSize, MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils);
private:
	CFlhShObjNameSpaceSizer(TInt& aSize, MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils);
protected:
	//from CFlhShObjNodeListUserBase
	void DoNextStepL();
private:
	TInt& iSize;
};

//static 
CFlhShObjNameSpaceSizer* CFlhShObjNameSpaceSizer::NewL(const char* aNameSpace, TInt& aSize, MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils)
{
	CFlhShObjNameSpaceSizer* self=new(ELeave) CFlhShObjNameSpaceSizer(aSize,aObs,aUtils);
	CleanupStack::PushL(self);
	self->BaseConstructL(aNameSpace);
	CleanupStack::Pop(self);
	return self;
}

CFlhShObjNameSpaceSizer::CFlhShObjNameSpaceSizer(TInt& aSize, MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils)
:CFlhShObjNodeListUserBase( aObs, aUtils), iSize(aSize)
{
}

void CFlhShObjNameSpaceSizer::DoNextStepL()
{
	#ifdef BEFORE
			CPersistentStorageNode* node = (*iIter);
	#else
			CPersistentStorageNode* node = (*iIter)++;
	#endif // AFTER

	TInt yieldcounter=0;
	while(node && yieldcounter<10)
	{
		yieldcounter++;
		TFileName* fileName = iUtils.GetSharedObjectFileName(node->GetNameSpace(), *(node->GetName()));

		if(fileName)
		{
			// Open each file
			RFile hFile;
			TInt err = hFile.Open(iUtils.FsSession(), *fileName, EFileWrite | EFileShareExclusive);
		
			if (err == KErrNone)
			{
				TInt fileSize;
				hFile.Size(fileSize);
				iSize += fileSize;
			}
			
			hFile.Close();
			
			delete fileName;
		}
		node=(*iIter)++;
	}
	if(!node)
	{
		iObs.NotifyOperationComplete(this,KErrNone);
	}
}

//
// class CFlhShObjDirectoryRemover
//

class CFlhShObjDirectoryRemover: public CFlhShObjNodeListUserBase
{
public:
	static CFlhShObjDirectoryRemover* NewL(const char* aNameSpace, MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils);
private:
	CFlhShObjDirectoryRemover(MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils);
protected:
	//from CFlhShObjNodeListUserBase
	void DoNextStepL();
};

//static 
CFlhShObjDirectoryRemover* CFlhShObjDirectoryRemover::NewL(const char* aNameSpace, MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils)
{
	CFlhShObjDirectoryRemover* self=new(ELeave) CFlhShObjDirectoryRemover(aObs,aUtils);
	CleanupStack::PushL(self);
	self->BaseConstructL(aNameSpace);
	CleanupStack::Pop(self);
	return self;
}

CFlhShObjDirectoryRemover::CFlhShObjDirectoryRemover(MFlhShObjNodeListObserver& aObs, CFlhShObjUtils& aUtils)
:CFlhShObjNodeListUserBase( aObs, aUtils)
{
}

void CFlhShObjDirectoryRemover::DoNextStepL()
{
	TInt error=KErrNone;
	CPersistentStorageNode* node = (*iIter);

	TInt yieldcounter=0;
	while(node && yieldcounter<10)
	{
		yieldcounter++;
		TFileName* fileName = iUtils.GetSharedObjectFileName(node->GetNameSpace(), *(node->GetName()));
		if (fileName)
		{
			CleanupStack::PushL(fileName);
			error=iUtils.Utils().DeleteFile(*fileName);

			if(error != KErrNotFound)
			{
				User::LeaveIfError(error);
			}
			CleanupStack::PopAndDestroy(fileName);
		}
		node=(*iIter)++;
	}
	if(!node)
	{
		TFileName* dirName = iUtils.GetPersistenceDir(iNameSpace);
		CleanupStack::PushL(dirName);
		if(dirName)
		{
			error = iUtils.FsSession().RmDir(*dirName);
			if(error != KErrNotFound)
			{
				User::LeaveIfError(error);
			}
		}
		CleanupStack::PopAndDestroy(dirName);

		iObs.NotifyOperationComplete(this,KErrNone);
	}
}

//
// class CShObjUtils
//

CFlhShObjUtils::~CFlhShObjUtils()
{	
	delete iCleanupRequest;
	iCleanupRequest = NULL;
	
	if(iPersistentDirectory)
	{
        delete iPersistentDirectory;
        iPersistentDirectory = NULL;
    }
}

CFlhShObjUtils::CFlhShObjUtils(RFs& aFs, CFlhFileUtils& aFileUtils, CPlatformFlashEngine& aEngine)
:iFs(aFs), iFileUtils(aFileUtils), iPersistentDirectory(NULL), iEngine(aEngine)
{}

//static
CFlhShObjUtils* CFlhShObjUtils::NewL(RFs& aFs, CFlhFileUtils& aFileUtils, const TDesC& aPersistentDirectory, CPlatformFlashEngine& aEngine)
{
    CFlhShObjUtils* self = new (ELeave) CFlhShObjUtils(aFs, aFileUtils, aEngine);
    if(self)
    {
        self->SetPersistentDirectoryL(aPersistentDirectory);
    }
    return self;
}

void CFlhShObjUtils::SetPersistentDirectoryL(const TDesC& aPersistentDirectory)
{
    if(iPersistentDirectory)
    {
        delete iPersistentDirectory;
        iPersistentDirectory = NULL;
    }
    iPersistentDirectory = HBufC::NewL(aPersistentDirectory.Length());
    TPtr des = iPersistentDirectory->Des();
    des.Copy(aPersistentDirectory);
}

TFileName* CFlhShObjUtils::GetPersistenceDir(const char* aNameSpace) 
{
	// nameSpace is nowadays actually a ascii MD5 hash, not a directory
	// so we don't have to parse it in any way.

	TFileName* directory = new TFileName;

	if(directory && aNameSpace)
	{
		//directory->Copy(iBaseDirectory);
		if(iPersistentDirectory)
		{
            directory->Append(*iPersistentDirectory);
            directory->Append(KPathDelimiter);
		}
		TPtrC8 namePtr((const TUint8*)aNameSpace);
		TBuf16<KMaxFileName> url;
		if (CnvUtfConverter::ConvertToUnicodeFromUtf8(url, namePtr) >= 0)
		{
			directory->Append(url);
            directory->Append(KPathDelimiter);
		}
	}

	return directory;
}

TFileName* CFlhShObjUtils::GetSharedObjectFileName(const char* aNameSpace, const char* aName)
{
    TPtrC8 namePtr((const TUint8*)aName);
    TBuf16<KMaxFileName> unicodeName;
    TFileName* file = NULL;
	if (CnvUtfConverter::ConvertToUnicodeFromUtf8(unicodeName, namePtr) >= 0)
		{
		file = GetSharedObjectFileName(aNameSpace, unicodeName);
		}
	return file;
}

TFileName* CFlhShObjUtils::GetSharedObjectFileName(const char* aNameSpace, const TDesC& aName) 
{
	TFileName* directory = GetPersistenceDir(aNameSpace);
	if(directory)
	{
		directory->Append(aName);
	}
	return directory;
}

// Creates given directory and its subdirectories, if they do not exist 
// already. If a FILE with the same name exists, returns false.
// This method copied from Flash 7's win32filesystem.cpp w/ modifications.
TBool CFlhShObjUtils::EnsureDirectory(const TDesC& aDirectory) 
{
	TUint lAttValue = 0;

	// Check to see if the directory already exists
	TInt err = iFs.Att(aDirectory, lAttValue);

	if (err != KErrNone || !(lAttValue & KEntryAttDir) )
	{
		// Create all sub-directory hierarchy
		err = iFs.MkDirAll(aDirectory);
	}
	return (err == KErrNone) ? ETrue : EFalse;
}

TInt CFlhShObjUtils::GetObjectSizeL(const TDesC& aFilename)
{
	TInt fileSize = 0;
    
	// Open the file
	RFile hFile;
	TInt err = hFile.Open(iFs, aFilename, EFileWrite | EFileShareExclusive);
			
	if (err == KErrNone)
	{
		hFile.Size(fileSize);
		hFile.Close();		
	}
	else if ((err != KErrPathNotFound) && (err != KErrNotFound))
	{
		User::Leave(err);
	}
			
	return fileSize;
}

CBase* CFlhShObjUtils::GetNameSpaceSizeL(const char *aNameSpace, TInt& aSize, MFlhShObjNodeListObserver& aObs)
{
	return CFlhShObjNameSpaceSizer::NewL(aNameSpace, aSize, aObs, *this);
}


// Get all files nested in sub-dir in non-recursive fashion starting from the top.
CBase* CFlhShObjUtils::ListAllNodesL(const char* aNameSpace, TSglQue<CPersistentStorageNode>*& aNodeList,MFlhShObjNodeListObserver& aObs)
{
	return CFlhShObjNodeLister::NewL(aNameSpace, aNodeList, aObs, *this);
}


// Get the last modified time of a file
TInt CFlhShObjUtils::GetAccessTime(CPersistentStorageNode* aNode, TTime& aTimeStamp)
{
	TInt err = KErrNoMemory;

	if(!aNode)
	{
		return 0;
	}

	TFileName* fileName = GetSharedObjectFileName(aNode->GetNameSpace(), *(aNode->GetName()));

	if (fileName)
	{
		err = iFs.Modified(*fileName, aTimeStamp);
	}
	delete fileName;
	return err;
}

// Compare the modified times of two file, return 1 if node1 is older than node2
TBool CFlhShObjUtils::IsOlder(CPersistentStorageNode* aNode1, CPersistentStorageNode* aNode2)
{
	TBool retVal = EFalse;
//	TInt err;
	TTime modTime1, modTime2;

	/*err = */GetAccessTime(aNode1, modTime1);
	/*err = */GetAccessTime(aNode2, modTime2);

	retVal = (modTime1 < modTime2) ? ETrue : EFalse;

	return retVal;
}

CBase* CFlhShObjUtils::RemoveDirectoryL(const char* aNameSpace, MFlhShObjNodeListObserver& aObs)
{
	return CFlhShObjDirectoryRemover::NewL(aNameSpace, aObs, *this);
}


RFs& CFlhShObjUtils::FsSession()
{
	return iFs;
}

CFlhFileUtils& CFlhShObjUtils::Utils()
{
	return iFileUtils;
}


void CFlhShObjUtils::CleanupSharedObjectsL(struct MM_Object* aMMObj, 
                                                  TUint32 aNumBytesToClean,
                                                  TInt aId,
												  MFlhShObjCleanupObserver& aObs)
{
	if(!iCleanupRequest)
	{
		iCleanupRequest = CPersistentStorageCleanupRequest::NewL(aMMObj, 
                                             aNumBytesToClean, 
                                               *this, 
                                              aId, &iCleanupRequest, aObs);
	}
}

void CFlhShObjUtils::CancelCleanupSharedObjects()
{
	delete iCleanupRequest;
	iCleanupRequest = NULL;
}

void CFlhShObjUtils::MigrateMSODirectoryL(const TDesC& aMSOFileName, const TDesC& aNewMSODirectory)
{
    // old folder : C:\private\UID\SwfGUID\MSOFileName
    // new folder : 
    // widget mode : drive:\private\UID\WidgetName\SwfGUID\MSOFileName
    // other modes : C:\private\UID\SwfGUID\MSOFileName
    
    
    // got to find out where the mso files were stored in fl 3.0
    
    // get the directory
    HBufC* oldMSOFile = NULL;
    oldMSOFile = GetFlash8MSOFileLC(aMSOFileName);
    MoveMSOFileToFlash9LocationL(*oldMSOFile,aNewMSODirectory);
    CleanupStack::PopAndDestroy(oldMSOFile);
}

HBufC* CFlhShObjUtils::GetFlash8PersistentDirectoryLC()
{
    
    HBufC* swfGUID = NULL;
    swfGUID = iEngine.GetSwfGuidLC();
    
    HBufC* oldPersistentDirectory = NULL;
    oldPersistentDirectory=HBufC::NewL(KMaxFileName);
    
    TPtr odDes = oldPersistentDirectory->Des();
    iFs.PrivatePath(odDes);
    TBuf<2> driveLetterAndDelimiter;
    TChar drive = iFs.GetSystemDriveChar();       
    driveLetterAndDelimiter.Append(drive);
    driveLetterAndDelimiter.Append(KDriveDelimiter); 
    odDes.Insert(0,driveLetterAndDelimiter); // C:
    odDes.Append(*swfGUID);

    CleanupStack::PopAndDestroy(swfGUID);
    CleanupStack::PushL(oldPersistentDirectory);
    return oldPersistentDirectory;
}

HBufC* CFlhShObjUtils::GetFlash8MSOFileLC(const TDesC& aMSOFileName)
{
    HBufC* oldMSOFilePath = NULL;
    oldMSOFilePath = GetFlash8PersistentDirectoryLC();
    
    // append the mso name to construct the path
    oldMSOFilePath->Des().Append(KPathDelimiter);
    oldMSOFilePath->Des().Append(aMSOFileName);
    
    TBool fileExists = BaflUtils::FileExists(iFs,*oldMSOFilePath);
    if(!fileExists)
    {
        User::Leave(KErrNotFound);
    }
    
    return oldMSOFilePath;
}

void CFlhShObjUtils::MoveMSOFileToFlash9LocationL(const TDesC& aOldFile, const TDesC& aNewMSODirectory)
{
    CFileMan* fm = CFileMan::NewL(iFs);
    TInt retVal = iFs.MkDirAll(aNewMSODirectory);
    if(retVal == KErrNone || retVal == KErrAlreadyExists)
    {
        retVal = fm->Move(aOldFile, aNewMSODirectory);
    }
    
    if(fm)
    {
        delete fm;
        fm = NULL;
    }
}

#endif //USE_SYNC_MSO
