/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class which is a 
*				 container class for all of the SI functions.
*
*/

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "saplayer_build.h"

#include "fi_interface.h"
#include "si_interface.h"
#include "symbian_engine.h"
#include "saflash_consts.h"
#include "symbian_urlrequestinfo.h"
#include "symbian_file.h"
#include "symbian_videoengine.h"
#include "symbian_image.h"
#include "symbian_sound.h"
#include "symbian_ui_interface.h"
#include "symbian_plugin_interface.h"
#include "symbian_stringutils.h"
#include "symbian_sharedobjects.h"
#include "symbian_localhttp.h"
#include <sacls.h>
#include <hwrmpowerstatesdkpskeys.h>
#include <PSVariables.h>
#include "amp_textnavigation.h"

#ifdef FEATURE_VIDEO_DLL
#include "platform_dynamiclibrary_videodecompressor.h"
#endif
#include <hal.h>
//#include <plpvariant.h>
#include <coemain.h>
#include <commdb.h>
#include <cmmanager.h>
#include <cmdestination.h>
#include <cmconnectionmethoddef.h>
#include <es_enum.h>
#include <es_sock.h>
//#include <apselect.h>
#include <charconv.h>
#include <aknconsts.h>
#include <etelmm.h>
#include "symbian_mso.h"
#include "sharedmemorymanager.h"
const short int KVibraFeatureNotSupported =-1;

#ifdef FEATURE_AMCEXTENSIONS

#include "IAMSecurityContext.h"
#include "IAMAccelerometer.h"
#include "symbian_accelerometer.h"
#include <bautils.h>
#include "IAMGPS.h"
#include "StringLoader.h"
#include "symbian_location.h"
#include "pluginbase.h"
#include <flash2engine.rsg>
#endif

#ifndef NULL
	#define NULL	(0)
#endif

#ifdef __LOG_SI_CALLS__

#ifdef __WINS__
#ifndef __FUNCTION__
#define __FUNCTION__ "undefined by compiler"
#endif // __FUNCTION__
#endif // __WINS__

#define SI_ENTER if(1) { SI_Interface::SI_Debug(NULL,__FUNCTION__,User::TickCount()); }
#define SI_EXIT  if(1) { SI_Interface::SI_Debug(NULL,__FUNCTION__,User::TickCount()); }

#define SI_OVG_ENTER ;
#define SI_OVG_EXIT ;

#else // __LOG_SI_CALLS__

#define SI_ENTER_SP if(aMMObj) { SI_Interface::SI_Debug(aMMObj,"+ Entering SI_Interface.cpp, line ",__LINE__); SI_Interface::SI_Debug(aMMObj,__FUNCTION__,User::TickCount()); }
#define SI_EXIT_SP  if(aMMObj) { SI_Interface::SI_Debug(aMMObj,"- Exiting SI_Interface.cpp, line ",__LINE__); SI_Interface::SI_Debug(aMMObj,__FUNCTION__,User::TickCount()); }

#define SI_ENTER ;
#define SI_EXIT ;

//#define SI_OVG_ENTER RDebug::Printf("SI_Interface::%s ++",__FUNCTION__);
//#define SI_OVG_EXIT  RDebug::Printf("SI_Interface::%s --",__FUNCTION__);

#define SI_OVG_ENTER ;
#define SI_OVG_EXIT ;

#endif

#ifdef FEATURE_OPENVG
#include <VG/openvg.h>
#include <EGL/egltypes.h>
#include <EGL/egl.h>
#endif

#ifdef FEATURE_TESTAUTOMATION
#include "automationhelper.h"
#endif
//const char* const KExtendedRenderingProtocols[] = { "mydata://", "system://", NULL };

//const TInt KBytesInKilobyte = 1024;
const TInt KDateStringBufferLength = 40;
const TInt KNetworkInfoStringLength = 128;
const TInt KSubscriberInfoStringLength = 128;
//const TInt KCapabilitiesScreensaver = 0;//defined in flashdefs.h

/*const TInt16 KCleanupSharedObjectsCallbackSuccess = 1;
const TInt16 KCleanupSharedObjectsCallbackFailure = 0;
//RK: CodecTypes

const TInt16 KJPEGCodec = 1;
const TInt16 KH263Codec = 2;
const TInt16 KScreenShareCodec = 3;
const TInt16 KVP6Codec = 4;
const TInt16 KVP6AlphaCodec = 5;
*/
// -----------------------------------------------------------------------------

/**
* Called by the player to get the amount of memory that is available, not including 
* memory already in use.
*
* @static
*/
long SI_Interface::SI_GetMaxAvailablePlayerMemory(struct MM_Object* aMMObj)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	long retVal=0;
	if(pEngine)
		retVal = pEngine->GetAvailableDynamicMemory();
	
	SI_EXIT
	return retVal;
}

	
/**
* Called by the player to allocate more memory.
*
* @param aNumBlocks the number of blocks of memory to allocate in this call.
* @static
*/
void * SI_Interface::SI_Malloc(struct MM_Object* aMMObj, unsigned short aNumBlocks)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
//	SI_Debug(aMMObj,"SI_MallocCount=",pEngine->iHackCounter);
	pEngine->iHackCounter++;
	void* retVal=pEngine->AllocateDynamicMemory(aNumBlocks);
//	SI_Debug(aMMObj,"SI_MallocAddr=",(TInt)retVal);
	SI_EXIT
	return retVal;
}

	
/**
* Called by the player to free memory.
* @param aMemory the address of the memory to free.
* @static
*/	
void SI_Interface::SI_Free(struct MM_Object* aMMObj, void * aMemory)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	pEngine->iHackCounter2++;
//	SI_Debug(aMMObj,"SI_FreeCount=",pEngine->iHackCounter2);
//	SI_Debug(aMMObj,"SI_FreeAddr=",(TInt)aMemory);
	pEngine->FreeDynamicMemory(aMemory);
	SI_EXIT
}

	
/**
*
* @static
*/	
unsigned char * SI_Interface::SI_GetLockedFrameBuffer (struct MM_Object* aMMObj)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	unsigned char* retVal=0;

	if (pEngine)
	{
		retVal= (unsigned char*)pEngine->LockFrameBufferBits();
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/	
void SI_Interface::SI_UnlockFrameBuffer (struct MM_Object* aMMObj, unsigned char * aBits)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine)
	{
		pEngine->UnlockFrameBufferBits ( aBits );
	}
	SI_EXIT
}

#ifdef FEATURE_OPENVG

#ifdef ACHW
short SI_Interface::SI_GetHardwareInterfaces(struct MM_Object* /* aMMObj */, void** ppEgl, void** ppApi)
{
	return 1;
}

short SI_Interface::SI_eglInitialize(struct MM_Object* /* aMMObj */, FI_Display display, short* pMajver, short* pMinver)
{
	return 1;
}

short SI_Interface::SI_eglMakeCurrent(struct MM_Object* /* aMMObj */, FI_Display display, const FI_Surface* draw, const FI_Surface* read, FI_Context context)
{
    return 1;
}

FI_Display SI_Interface::SI_eglGetDisplay(struct MM_Object* /*aMMObj*/)
{
	return NULL;	
}

short SI_Interface::SI_eglTerminate(struct MM_Object* /*aMMObj*/, FI_Display display)
{
	return 1; 
}

FI_Context SI_Interface::SI_eglCreateContext(struct MM_Object* /*aMMObj*/, FI_Display display, FI_Config config, FI_Context shareContext, const long* pAttribList)
{
    return NULL;
}

short SI_Interface::SI_eglChooseConfig(struct MM_Object* /*aMMObj*/, FI_Display display, const long* pAttribList, FI_Config* pConfigs, unsigned short configSize, unsigned short* pNumConfig)
{
	return 1;
}

struct FI_Surface* SI_Interface::SI_eglCreatePbufferSurface(struct MM_Object* /*aMMObj*/, FI_Display display, FI_Config config, const long* pAttribList)
{
	return NULL;
}

struct FI_Surface* SI_Interface::SI_eglCreateNativePixmapSurface(struct MM_Object* aMMObj, FI_Display display, FI_Config config, unsigned short width, unsigned short height, const long* pAttribList, struct FI_PixmapInfo* pPixmapInfo)
{
	return NULL;
}

struct FI_Surface* SI_Interface::SI_eglCreatePbufferFromClientBuffer(struct MM_Object* /*aMMObj*/, FI_Display /*display*/, unsigned long /*buftype*/, FI_ClientBuffer /*buffer*/, FI_Config /*config*/, const long* /*pAttribList*/)
{
	return NULL;
}

short SI_Interface::SI_eglDestroyContext(struct MM_Object* /*aMMObj*/, FI_Display display, FI_Context context)
{
	return 1;
}

short SI_Interface::SI_eglDestroySurface(struct MM_Object* /*aMMObj*/, FI_Display display, struct FI_Surface* pSurface)
{
	return 1;	
}

short SI_Interface::SI_eglGetConfigAttrib(struct MM_Object* /*aMMObj*/, FI_Display display, FI_Config config, long attrib, long* pValue)
{
	return 1;
}

short SI_Interface::SI_eglCopySurfaceToFrameBuffer(struct MM_Object* pMMObj, FI_Display display, const struct FI_Surface* pSurface, unsigned char* pDst, unsigned short width, unsigned short height, unsigned short pitch, struct FI_Rect* dirtyRect)
{
	return 1;
}
#endif

void SI_Interface::SI_ovgAppendPath(void* clientData, void* dstPath, void* srcPath)
{
    SI_OVG_ENTER;    
    VGPath pDstPath = reinterpret_cast<VGPath>(dstPath);
    VGPath pSrcPath = reinterpret_cast<VGPath>(srcPath);
    vgAppendPath(pDstPath,pSrcPath);
    SI_OVG_EXIT;      
}

void SI_Interface::SI_ovgAppendPathData(void* clientData, void* dstPath, int numSegments, const unsigned char * pathSegments, const void * pathData)
{
    SI_OVG_ENTER;     
    VGPath dPath = reinterpret_cast<VGPath> (dstPath);
    vgAppendPathData(dPath,
               static_cast<VGint>(numSegments),
               static_cast<const VGubyte*>(pathSegments),
               pathData
               );
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgClear(void* clientData, int x, int y, int width, int height)
{
    SI_OVG_ENTER;     
    vgClear(static_cast<VGint>(x),
                static_cast<VGint>(y),
                static_cast<VGint>(width),
                static_cast<VGint>(height)
                );
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgClearPath(void* clientData, void* path, unsigned int capabilities)
{
    SI_OVG_ENTER;    
    vgClearPath(reinterpret_cast<VGPath> (path), (VGbitfield) capabilities);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgClearImage(void* clientData, void* image, int x, int y, int width, int height)
{
    SI_OVG_ENTER;    
    vgClearImage((VGImage)(image),
                static_cast<VGint>(x),            
                static_cast<VGint>(y),
                static_cast<VGint>(width),
                static_cast<VGint>(height)
                );
    SI_OVG_EXIT;    
}

void* SI_Interface::SI_ovgCreateImage(void* clientData, int format, int width, int height, unsigned int allowedQuality)
{
    SI_OVG_ENTER;     
    VGImage pImage = vgCreateImage((VGImageFormat)format,
            (VGint)width, (VGint) height,
            (VGbitfield) allowedQuality);
    return reinterpret_cast<void*>(pImage);
    SI_OVG_EXIT;    
}

void* SI_Interface::SI_ovgCreateMaskLayer(void* clientData, int width, int height)
{
    SI_OVG_ENTER;     
    VGMaskLayer mlayer = vgCreateMaskLayer(width,height);
    return reinterpret_cast<void*>(mlayer);
    SI_OVG_EXIT;    
}

void* SI_Interface::SI_ovgCreatePaint(void* clientData)
{
    SI_OVG_ENTER;     
    VGPaint paint = vgCreatePaint();
    return reinterpret_cast<void*>(paint);
    SI_OVG_EXIT;    
}

void* SI_Interface::SI_ovgCreatePath(void* clientData, int pathFormat, unsigned int datatype, float scale, float bias, int segmentCapacityHint, int coordCapacityHint, unsigned int capabilities)
{
    SI_OVG_ENTER;     
    VGPath path = vgCreatePath((VGint)pathFormat,(VGPathDatatype)datatype,(VGfloat)scale,(VGfloat)bias,(VGint)segmentCapacityHint,(VGint)coordCapacityHint,(VGbitfield)capabilities);
    SI_OVG_EXIT;    
    return  reinterpret_cast<void*>(path);    
}

void SI_Interface::SI_ovgCopyMask(void* clientData, void* maskLayer, int sx, int sy, int dx, int dy, int width, int height)
{
    SI_OVG_ENTER;     
    vgCopyMask(reinterpret_cast<VGMaskLayer>(maskLayer),
            (VGint)sx, (VGint) sy,
            (VGint) dx, (VGint) dy,
            (VGint) width, (VGint) height);
    SI_OVG_EXIT;
}

void SI_Interface::SI_ovgDestroyImage(void* clientData, void* image)
{
    SI_OVG_ENTER;     
    vgDestroyImage(reinterpret_cast<VGImage>(image));
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgDestroyMaskLayer(void* clientData, void* maskLayer)
{
    SI_OVG_ENTER;     
    vgDestroyMaskLayer(reinterpret_cast<VGMaskLayer>(maskLayer));
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgDestroyPaint(void* clientData, void* paint)
{
    SI_OVG_ENTER;     
    vgDestroyPaint(reinterpret_cast<VGPaint>(paint));
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgDestroyPath(void* clientData, void* path)
{
    SI_OVG_ENTER;     
    vgDestroyPath(reinterpret_cast<VGPath>(path));
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgDrawImage(void* clientData, void* image)
{
    SI_OVG_ENTER;     
    vgDrawImage(reinterpret_cast<VGImage>(image));
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgDrawPath(void* clientData, void* path, unsigned int paintModes)
{
    SI_OVG_ENTER;     
    vgDrawPath(reinterpret_cast<VGPath>(path),(VGbitfield)paintModes);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgFinish(void* clientData)
{
    SI_OVG_ENTER;     
    vgFinish();
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgFlush(void* clientData)
{
    SI_OVG_ENTER;     
    vgFlush();
    SI_OVG_EXIT;    
}

unsigned int SI_Interface::SI_ovgGetError(void* clientData)
{
    SI_OVG_ENTER;     
    return vgGetError();
}

void SI_Interface::SI_ovgGetMatrix(void* clientData, float * m)
{
    SI_OVG_ENTER;     
    vgGetMatrix((VGfloat*)m);
    SI_OVG_EXIT;    
}

int SI_Interface::SI_ovgGetParameteri(void* clientData, void* object, int paramType)
{
    SI_OVG_ENTER;     
    return vgGetParameteri(reinterpret_cast<VGHandle>(object),
            (VGint)paramType);
}

int SI_Interface::SI_ovgGeti(void* clientData, int type)
{    
    SI_OVG_ENTER;     
    return vgGeti((VGParamType)type);
}

void SI_Interface::SI_ovgImageSubData(void* clientData, void* image, const void * data, int dataStride, unsigned int dataFormat, int x, int y, int width, int height)
{
     SI_OVG_ENTER;     
     vgImageSubData(reinterpret_cast<VGImage>(image),data,(VGint)dataStride,(VGImageFormat)dataFormat,(VGint)x,(VGint)y,(VGint)width,(VGint)height);
     SI_OVG_EXIT;     
}

int SI_Interface::SI_ovgInterpolatePath(void* clientData, void* dstPath, void* startPath, void* endPath, float amount)
{
    SI_OVG_ENTER;     
    VGboolean VGBool = vgInterpolatePath(reinterpret_cast<VGPath>(dstPath),reinterpret_cast<VGPath>(startPath),reinterpret_cast<VGPath>(endPath),(VGfloat)amount);
    SI_OVG_EXIT;    
    return VGBool;
}

void SI_Interface::SI_ovgLoadIdentity(void* clientData)
{
    SI_OVG_ENTER;     
    vgLoadIdentity();
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgLoadMatrix(void* clientData, const float * m)
{
    SI_OVG_ENTER;     
    vgLoadMatrix((const VGfloat*)m);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgMask(void* clientData, void* mask,  unsigned int operation, int x, int y, int width, int height)
{
    SI_OVG_ENTER;     
    vgMask(reinterpret_cast<VGHandle>(mask),(VGMaskOperation) operation,
            (VGint) x, (VGint) y,
            (VGint) width, (VGint) height);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgModifyPathCoords(void* clientData, void* dstPath, int startIndex, int numSegments, const void * pathData)
{
    SI_OVG_ENTER;     
    vgModifyPathCoords(reinterpret_cast<VGPath> (dstPath),
            (VGint) startIndex,
            (VGint) numSegments,
            pathData);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgMultMatrix(void* clientData, const float * m)
{
    SI_OVG_ENTER;     
    vgMultMatrix(m);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgPaintPattern(void* clientData, void* paint, void* pattern)
{
    SI_OVG_ENTER;     
    vgPaintPattern(reinterpret_cast<VGPaint>(paint), reinterpret_cast<VGImage> (pattern));
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgPathBounds(void* clientData, void* path, float * minX, float * minY, float * width, float * height)
{
    SI_OVG_ENTER;     
    vgPathBounds(reinterpret_cast<VGPath> (path),
            (VGfloat*) minX,
            (VGfloat*) minY,
            (VGfloat*) width,
            (VGfloat*) height);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgRenderToMask(void* clientData, void* path, unsigned int paintModes, unsigned int operation)
{
    SI_OVG_ENTER;     
    vgRenderToMask(reinterpret_cast<VGPath>(path),
            VGbitfield(paintModes),
            VGMaskOperation(operation));
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgScale(void* clientData, float sx, float sy)
{
    SI_OVG_ENTER;     
    vgScale(VGfloat(sx), VGfloat(sy));
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgSetColor(void* clientData, void* paint, unsigned int rgba)
{
    SI_OVG_ENTER;     
    vgSetColor(reinterpret_cast<VGPaint> (paint), (VGuint) rgba);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgSetPaint(void* clientData, void* paint, unsigned int paintModes)
{
    SI_OVG_ENTER;     
    vgSetPaint(reinterpret_cast<VGPaint> (paint), (VGbitfield) paintModes);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgSetParameterfv(void* clientData, void* object, int paramType, int count, const float * values)
{
    SI_OVG_ENTER;     
    vgSetParameterfv(reinterpret_cast<VGHandle> (object),
            (VGint) paramType,
            (VGint) count,
            (const VGfloat*) values);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgSetParameteri(void* clientData, void* object, int paramType, int value)
{
    SI_OVG_ENTER;     
    vgSetParameteri(reinterpret_cast<VGHandle> (object),
            (VGint) paramType,
            (VGint) value);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgSetf(void* clientData, int type, float value)
{
    SI_OVG_ENTER;     
    vgSetf ( (VGParamType) type, (VGfloat) value);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgSetfv(void* clientData, int type, int count, const float * values)
{
    SI_OVG_ENTER;     
    vgSetfv((VGParamType) type, (VGint) count,
            (const VGfloat*) values);
    SI_OVG_EXIT;    
}
    
void SI_Interface::SI_ovgSetiv(void* clientData, int type, int count, const int * values)
{
    SI_OVG_ENTER;     
    vgSetiv((VGParamType) type, (VGint) count,
            (const VGint*) values);
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgTransformPath(void* clientData, void* dstPath, void* srcPath)
{
    SI_OVG_ENTER;     
    vgTransformPath(reinterpret_cast<VGPath> (dstPath), reinterpret_cast<VGPath> (srcPath) );
    SI_OVG_EXIT;    
}

void SI_Interface::SI_ovgTranslate(void* clientData, float tx, float ty)
{
    SI_OVG_ENTER;     
    vgTranslate((VGfloat) tx, (VGfloat) ty);
    SI_OVG_EXIT;    
}

void* SI_Interface::SI_ovgChildImage(void* clientData, void* parent, int x, int y, int width, int height)
{
    SI_OVG_ENTER;
    VGImage pImage = vgChildImage( reinterpret_cast<VGImage>(parent), 
                                   (VGint)x, 
                                   (VGint)y, 
                                   (VGint)width, 
                                   (VGint)height );
    SI_OVG_EXIT;
    return reinterpret_cast<void*>(pImage);
}
        
void SI_Interface::SI_ovgSeti(void* clientData, int type, int value)
{
    SI_OVG_ENTER;     
    vgSeti (VGParamType(type), VGint(value));
    SI_OVG_EXIT;    
}

#endif // FEATURE_OPENVG
	
/**
*
* @static
*/	
void SI_Interface::SI_BeginTextDraw (struct MM_Object* aMMObj)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if ( pEngine )
	{
		pEngine->BeginTextDraw();
	}
	SI_EXIT
}

	
/**
*
* @static
*/
void SI_Interface::SI_EndTextDraw (struct MM_Object* aMMObj)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if ( pEngine )
	{
		pEngine->EndTextDraw();
	}
	SI_EXIT
}

	
/**
*
* @static
*/	
unsigned short SI_Interface::SI_DeviceTextOut (struct MM_Object* aMMObj, unsigned short /*fontSize*/, 
											   const struct FI_Text * aText, struct FI_TextInfo* aTextInfo, void* bits, int width, int height)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	TBool result = EFalse;
	if (pEngine)
	{
        TRAPD_WRAPPER(result = pEngine->DrawTextL( aText, aTextInfo, bits, width, height ));
	}

	SI_EXIT
	if (result)
		return 1;
	//Text has not been rendered on screen:
		return 0;

}

	
/**
*
* @static
*/		
unsigned short SI_Interface::SI_ComplexLineBreak (struct MM_Object* aMMObj, 
                                                  unsigned short /*fontSize*/, 
                                                  const struct FI_Text *aText, 
                                                  struct FI_DeviceTextInfo *aDeviceTextInfo)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	BOOL result = FALSE;

	if (pEngine)
	{
		TRAPD_WRAPPER(result = pEngine->LinebreakL(aText, aDeviceTextInfo));
	}
	
	SI_EXIT
    return result;
}

#ifdef __LOG_SI_CALLS__
unsigned short SI_Interface::SI_GetBitmapInfo (struct MM_Object* aMMObj, unsigned short /*fontSize*/,
											   const struct FI_Text* /*aText*/, struct FI_BitmapInfo* /*pInfo*/)
#else
unsigned short SI_Interface::SI_GetBitmapInfo (struct MM_Object* /*aMMObj*/, unsigned short /*fontSize*/,
											   const struct FI_Text* /*aText*/, struct FI_BitmapInfo* /*pInfo*/)
#endif
{
	SI_ENTER
	// This is only called if FI_PlayerRendersText is passed in as a config flag to
	// FI_CreatePlayer. This mode is not supported on Symbian.
	SI_EXIT
	return 0;
}

	
/**
*
* @static
*/	
unsigned short SI_Interface::SI_GetCharacterInfo (struct MM_Object* aMMObj, unsigned short /*fontSize*/,
												  const struct FI_Text * aText, struct FI_CharacterInfo* aInfo)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	CFont * pFont = NULL;
	
	unsigned short retVal=0;

	if ( pEngine && pEngine->GetCurrentFont(pFont) )
	{
		TOpenFontCharMetrics	metrics;
		TUint 		code = (unsigned int)(aText->textEncoding);
		const TUint8 *	bitmap = 0;
		TSize		size;
		CFont::TCharacterDataAvailability cda = pFont->GetCharacterData(code, metrics, bitmap, size);

		if ( cda != CFont::ENoCharacterData )
		{
			aInfo->bitDepth	= 1;
			aInfo->width	= (unsigned short)metrics.Width();
			aInfo->height	= (unsigned short)metrics.Height();
			aInfo->pitch	= (unsigned short)size.iWidth;
			aInfo->xOrigin	= (short)metrics.HorizBearingX();
			aInfo->yOrigin	= (short)metrics.HorizBearingY();
			aInfo->step		= (short)metrics.HorizAdvance();

			retVal=1;
		}
	}

	SI_EXIT
	return retVal;
}
//SecuritySandbox
void SI_Interface::SI_ShowLocalSecurityDialog(struct MM_Object* aMMObj,const char* pPathToDisplay, const char* pAccessedDomain)
    {
        SI_ENTER
        TRAPD_WRAPPER(DOSI_ShowLocalSecurityDialogL(aMMObj,pPathToDisplay,pAccessedDomain));
        SI_EXIT
    }
//SecuritySandbox
void SI_Interface::DOSI_ShowLocalSecurityDialogL(struct MM_Object* aMMObj,const char* pPathToDisplay, const char* pAccessedDomain)
{

    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
    char* strippedUrl = StripPrefix(pPathToDisplay,"file://");
    HBufC *pathToDisplay = pEngine->CreateEpocStringLC(strippedUrl, FlashStrLen(strippedUrl));
    HBufC *accessedDomain = pEngine->CreateEpocStringLC(pAccessedDomain, FlashStrLen(pAccessedDomain));
    
    TPtrC pathToDisplayPtr = pathToDisplay->Des();
    TPtrC accessedDomainPtr= accessedDomain->Des();
    //Ask user if we can trust this content
    
    if(pEngine->GetClientIf()->CanContentBeTrustedL(accessedDomainPtr))
    //if(pEngine->GetFlashUI()->CanContentBeTrustedL(accessedDomainPtr))
        {
            TRAPD_WRAPPER(pEngine->TrustContentL(pathToDisplayPtr));
            //pEngine->GetFlashUI()->StartRestartTimerL();
            pEngine->GetClientIf()->KickoffRestartTimerL();
        }
        
    CleanupStack::PopAndDestroy(accessedDomain);
    CleanupStack::PopAndDestroy(pathToDisplay);

}	
/**
*
* @static
*/	
void SI_Interface::SI_GetFontMetrics (struct MM_Object* aMMObj, unsigned short /*fontSize*/,
									  short* aAscent, short* aDescent, unsigned short* aSingleByteSpaceWidth)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	CFont * font = NULL;
	
	if ( pEngine && pEngine->GetCurrentFont(font) )
	{
		TOpenFontMetrics	metrics;
		CFbsFont * fbsFont = NULL;
		if ( font->TypeUid() == KCFbsFontUid )
			fbsFont = (CFbsFont*)font;
		if ( fbsFont && fbsFont->GetFontMetrics( metrics ) )
		{
		#ifdef EKA2
			//*aAscent = (short)metrics.MaxHeight();
			//*aDescent = (short)metrics.MaxDepth();
			TInt ascent = metrics.MaxHeight();
			TInt descent = metrics.MaxDepth();
			if(ascent<=13)
			{
				ascent = ascent+2;
			}
			if(descent<3)
			{
				descent = descent+2;
			}
			*aAscent = (short)ascent;
			*aDescent = (short)descent;
		#else //EKA2
			*aAscent = (short)metrics.Ascent();
			*aDescent = (short)metrics.Descent();
		#endif //EKA2
			if (*aDescent < 0)
			{
				*aDescent = (short) - *aDescent;
			}
			int spaceWidth = font->CharWidthInPixels(' ');
			*aSingleByteSpaceWidth = (short)spaceWidth;
		}
		else 
		{
			*aAscent = 0;
			*aDescent = 0;
			*aSingleByteSpaceWidth = 0;
		}
	}
	SI_EXIT
}

	
/**
*
* @static
*/	
unsigned short SI_Interface::SI_GetTextWidth (struct MM_Object* aMMObj, unsigned short /*fontSize*/,
											  const struct FI_Text * aText, unsigned long* aWidth)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	CFont * font = NULL;
	
	unsigned short retVal=0;

	if ( pEngine && pEngine->GetCurrentFont(font) )
	{
		
		if(aText->textEncoding==FI_UTF16EncodingForm)
		{
			TPtrC ptr(aText->text.utf16String,aText->textLen);
            CFont::TMeasureTextInput input;
            input.iFlags = CFont::TMeasureTextInput::EFVisualOrder;
            *aWidth = font->MeasureText(ptr, &input );
			
			//*aWidth = font->TextWidthInPixels( ptr );
			retVal=1;	
		}
	}
	SI_EXIT
	return retVal;
}

	
unsigned short	SI_Interface::SI_CreateFont (struct MM_Object* aMMObj, const struct FI_FontInfo* aFontInfo, void** aFontId)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	unsigned short retval = 0;
	TInt pFontId = 0;
	if ( pEngine )
	{
		retval = pEngine->GetDeviceFont( aFontInfo, &pFontId);
			*aFontId = (void*)pFontId;
	}

	SI_EXIT
	return retval;
}


unsigned short SI_Interface::SI_DestroyFont (struct MM_Object* aMMObj, void* aFontId)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	unsigned short retval = 0;
	if ( pEngine )
	{
		retval = pEngine->DestroyFont(aFontId );
	}
	
	SI_EXIT
	return retval;
}


unsigned short SI_Interface::SI_SelectFont (struct MM_Object* aMMObj, void* aFontId)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	unsigned short retval = 0;
	
	if ( pEngine )
	{
		retval = pEngine->SetCurrentFont(aFontId );
	}

	SI_EXIT
	return retval;
}

void SI_Interface::SI_NewFontSource( struct MM_Object* aMMObj, 
                                     struct FI_FontInfo* aFontInfo, 
                                     struct FI_FontSource** aFontSource )
{
    SI_ENTER    
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
    unsigned char* font = NULL;
    TInt size = 0;
    
    if ( pEngine )
    {        
        size = pEngine->GetOTFont( &font, aFontInfo->bold );
    }

    // Create the font source and return to core player
    FI_FontSource* fontSource = new FI_FontSource;
    memset(fontSource, 0, sizeof(FI_FontSource));
    fontSource->isPlatformRendered = false;
    fontSource->shouldCopyData = false;
    
    if( size > 0 )
    {
        fontSource->pFontData = font;
        fontSource->fontDataLen = size;
        (*aFontSource) = fontSource;
    }
    
    
    SI_EXIT
}

void SI_Interface::SI_FreeFontSource(struct MM_Object* /*pMMObj*/, struct FI_FontSource* aSource)
{
    // Font data is created by the platform code. Delete it since it is not required by core player anymore.
    delete aSource->pFontData;
    delete aSource;
}

	
#ifdef FEATURE_OPENVG
/**
*
* @static
*/  
unsigned char* SI_Interface::SI_GetLockedTextBuffer (struct MM_Object* aMMObj,
                                                     unsigned int* width,
                                                     unsigned int* height,
                                                     unsigned int* pitch, 
                                                     unsigned int* depth)
{
    SI_ENTER
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

    unsigned char* retVal=0;
    if (pEngine)
    {
        retVal= (unsigned char*)pEngine->LockAuxTextBits(width,height,pitch,depth);
    }
    SI_EXIT
    return retVal;
}


/**
*
* @static
*/  
void SI_Interface::SI_UnlockTextBuffer (struct MM_Object* aMMObj, unsigned char* aBits)
{
    SI_ENTER
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

    if (pEngine)
    {
        pEngine->UnlockAuxTextBits(aBits);
    }
    SI_EXIT
}
#endif
/**
*
* @static
*/	
short SI_Interface::SI_CacheSound (struct MM_Object* aMMObj, void* aSoundBuffer,
										   unsigned long aSoundBufferSize, unsigned long * aSoundID)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	short retVal=0;

	if (pEngine && pEngine->SoundHandler())
	{
		TRAPD_WRAPPER(retVal=pEngine->SoundHandler()->CacheSoundL(aSoundBuffer, aSoundBufferSize, aSoundID));
	}
	
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/	
unsigned short SI_Interface::SI_PlayCachedSound (struct MM_Object* aMMObj, unsigned long aSoundID, unsigned long count)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->SoundHandler())
	{
		return pEngine->SoundHandler()->PlayCachedSound(aSoundID, count);
	}
	
	SI_EXIT
	return 0;
}

	
/**
*
* @static
*/	
void SI_Interface::SI_FreeCachedSound (struct MM_Object* aMMObj, unsigned long aSoundID)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->SoundHandler())
	{
		pEngine->SoundHandler()->FreeCachedSound(aSoundID);
	}
	SI_EXIT
}

	
/**
*
* @static
*/	
short SI_Interface::SI_PlaySound (struct MM_Object* aMMObj, void* aSoundBuffer, unsigned long aSoundBufferSize, unsigned long count)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	short retVal=0;
	if (pEngine && pEngine->SoundHandler())
	{
		TRAPD_WRAPPER(retVal = pEngine->SoundHandler()->PlaySoundL(aSoundBuffer, aSoundBufferSize, count));
	}

	SI_EXIT
	return retVal;
}
	
/**
*
* @static
*/	
void SI_Interface::SI_StopSound (struct MM_Object* aMMObj)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->SoundHandler())
	{
		pEngine->SoundHandler()->StopSound();
	}
	SI_EXIT
	return;
}

	
/**
*
* @static
*/
unsigned short SI_Interface::SI_PauseSound (struct MM_Object* aMMObj)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->SoundHandler())
	{
		return pEngine->SoundHandler()->PauseSound();
	}
	SI_EXIT
	return 0;
}

	
/**
*
* @static
*/
unsigned short SI_Interface::SI_ResumeSound (struct MM_Object* aMMObj)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->SoundHandler())
	{
		return pEngine->SoundHandler()->ResumeSound();
	}
	SI_EXIT
	return 0;
}

	
//FL4.0 : TODO
void SI_Interface::SI_FLVStatus(struct MM_Object* aMMObj, signed short /*streamId*/, const char* code)
{

	SI_ENTER
 
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
    if (pEngine)
	{
		pEngine->SetFLVStatus(code);			
	}
	
	SI_EXIT
	
}
/**
*
* @static
*/
short SI_Interface::SI_CreateVideo(struct MM_Object* aMMObj, const FI_Rect* aLocation, unsigned long* aRetVideoID)
{
	SI_ENTER
	short retVal = 0;

	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->GetVideoEngine())
	{
		TRAPD_WRAPPER(retVal = pEngine->GetVideoEngine()->CreateVideoL(aLocation, aRetVideoID));
		pEngine->SetSwfHasVideo(ETrue);
	}
	SI_EXIT
	
	return retVal;
}

	
/**
*
* @static
*/
short SI_Interface::SI_PlayVideo(struct MM_Object* aMMObj, const FI_Video* aVideoInfo)
{
	SI_ENTER
	short retVal = 0;

	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->GetVideoEngine())
	{
		TRAPD_WRAPPER(retVal = pEngine->GetVideoEngine()->PlayVideoL(aVideoInfo));
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/
void SI_Interface::SI_CloseVideo(struct MM_Object* aMMObj, unsigned long aVideoID)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->GetVideoEngine())
	{
		pEngine->GetVideoEngine()->CloseVideo(aVideoID);
        pEngine->SetSwfHasVideo(EFalse);
	}
	SI_EXIT
}

	
/**
*
* @static
*/
short SI_Interface::SI_StopVideo(struct MM_Object* aMMObj, unsigned long aVideoID)
{
	SI_ENTER
	short retVal = 0;

	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->GetVideoEngine())
	{
		TRAPD_WRAPPER(retVal = pEngine->GetVideoEngine()->StopVideoL(aVideoID));
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/
short SI_Interface::SI_PauseVideo(struct MM_Object* aMMObj, unsigned long aVideoID)
{
	SI_ENTER
	short retVal = 0;

	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->GetVideoEngine())
	{
		TRAPD_WRAPPER(retVal = pEngine->GetVideoEngine()->PauseVideoL(aVideoID));
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/
short SI_Interface::SI_ResumeVideo(struct MM_Object* aMMObj, unsigned long aVideoID)
{
	SI_ENTER
	short retVal = 0;

	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->GetVideoEngine())
	{
		TRAPD_WRAPPER(retVal = pEngine->GetVideoEngine()->ResumeVideoL(aVideoID));
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/
short SI_Interface::SI_GetVideoMethods(struct MM_Object* aMMObj, FI_ReturnString* aMethods)
{
	SI_ENTER
	short retVal = (short)SI_GenericError;

	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->GetVideoEngine())
	{
		retVal = pEngine->GetVideoEngine()->SupportedVideoMethods(aMethods);
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/
short SI_Interface::SI_ProcessVideoMethod(struct MM_Object* aMMObj, unsigned long aVideoID, const FI_Method* aMethod, FI_ReturnString* aRetString)
{	
	SI_ENTER
	short retVal = 0;

	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->GetVideoEngine())
	{
		TRAPD_WRAPPER(retVal = pEngine->GetVideoEngine()->ProcessVideoMethodL(aVideoID, aMethod, aRetString));
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/
void SI_Interface::SI_DestroyVideo(struct MM_Object* aMMObj, unsigned long aVideoID)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->GetVideoEngine())
	{
		TRAPD_WRAPPER(pEngine->GetVideoEngine()->DestroyVideoL(aVideoID));
	}
	SI_EXIT
}

	
/**
*
* @static
*/
void SI_Interface::SI_SetVideoRect(struct MM_Object* aMMObj, unsigned long aVideoID, const FI_Rect* aLocation)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->GetVideoEngine())
	{
		TRAPD_WRAPPER(pEngine->GetVideoEngine()->SetVideoRectL( aVideoID, aLocation) );
	}
	SI_EXIT
}


/**
*
* @static
*/
void SI_Interface::SI_GetVideoSize(struct MM_Object* aMMObj, unsigned long aVideoID, unsigned long* retWidth, unsigned long* retHeight)
{
    SI_ENTER
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

    if (pEngine && pEngine->GetVideoEngine())
    {
        TSize size = pEngine->GetVideoEngine()->GetVideoSize(aVideoID);
        *retWidth = size.iWidth;
        *retHeight= size.iHeight;
    }
    SI_EXIT
}


/**
*
* @static
*/
short SI_Interface::SI_GetVideoBits(struct MM_Object* aMMObj, unsigned long aVideoID, unsigned char** bits)
{
    SI_ENTER
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

    if (pEngine && pEngine->GetVideoEngine())
    {
        unsigned char* pDest = *bits;
        
        TSize size = pEngine->GetVideoEngine()->GetVideoSize(aVideoID);
        TInt dataSize = size.iWidth * size.iHeight * 4;
        memset(pDest, 0xFF, dataSize);
    }
    SI_EXIT
    
    return 1;
}

/**
*
* @static
*/
unsigned short SI_Interface::SI_CloseStreamSoundDevice (struct MM_Object* aMMObj)
{
	SI_ENTER
	short retVal = 0;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->SoundHandler())
	{
		pEngine->SoundHandler()->CloseStreamSoundDevice();
		retVal = 1;
	}
	SI_EXIT
	
	return retVal;
}

	
/**
*
* @static
*/	
void * SI_Interface::SI_CreateSoundDecompressor (struct MM_Object* aMMObj, struct FI_StreamSoundInfo* aSoundInfo, unsigned long aFormatInfo)
{
	SI_ENTER
	void* soundDecomp = NULL;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->SoundHandler())
	{
	   //RK : Load sound library only if sound is present in the swf file
	   // ::AddSound from core player will trigger this
	   	if(pEngine->SoundHandler()->LoadSoundLibrary())
	   	{
	   		soundDecomp = pEngine->SoundHandler()->CreateSoundDecompressor(aSoundInfo, aFormatInfo);	
	   	}
		
	}
	if(pEngine)
	    pEngine->SetSwfHasAudio(ETrue);
	SI_EXIT
	return soundDecomp;
}

	
/**
*
* @static
*/	
long SI_Interface::SI_DecompressSound (struct MM_Object* aMMObj, void* aDecompressor, void* aDestBuffer, unsigned long aNumSamples)
{
	SI_ENTER
	long retVal = -1;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	
	if (pEngine && pEngine->SoundHandler())
	{
		retVal = pEngine->SoundHandler()->DecompressSound(aDecompressor, aDestBuffer, aNumSamples);
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/	
void SI_Interface::SI_DestroySoundDecompressor (struct MM_Object* aMMObj, void* aDecompressor)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	
	if (pEngine && pEngine->SoundHandler())
	{
		pEngine->SoundHandler()->DestroySoundDecompressor(aDecompressor);
		
		pEngine->SoundHandler()->UnloadSoundLibrary();
	}
	if(pEngine)
	    pEngine->SetSwfHasAudio(EFalse);
	SI_EXIT
}
	
/**
*
* @static
*/	
unsigned short SI_Interface::SI_OpenStreamSoundDevice (struct MM_Object* aMMObj)
{
	SI_ENTER
	unsigned short retVal = 0;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	
	if (pEngine && pEngine->SoundHandler())
	{
		TRAPD_WRAPPER(retVal = pEngine->SoundHandler()->OpenStreamSoundDeviceL());
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/	
unsigned short SI_Interface::SI_PlayStreamSoundBuffer (struct MM_Object* aMMObj, void* aData)
{
	SI_ENTER
	unsigned short retVal = 0;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->SoundHandler())
	{
		TRAPD_WRAPPER(retVal = pEngine->SoundHandler()->PlayStreamSoundBufferL(aData));
	}
	SI_EXIT
	return retVal;
}
	
/**
*
* @static
*/	
void SI_Interface::SI_SetupSoundDecompressor (struct MM_Object* aMMObj, void* aDecompressor, struct FI_StreamSoundDecompressorInfo* pDecompressorInfo)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine && pEngine->SoundHandler())
	{
		pEngine->SoundHandler()->SetupSoundDecompressor(aDecompressor, pDecompressorInfo);
	}
	SI_EXIT
}

	
/**
*
* @static
*/
void SI_Interface::SI_GetInputText (struct MM_Object* aMMObj,
												const struct FI_Text* initialText,
												const struct FI_Text* /*dialogTitleText*/,
												unsigned short maxNumChars,
												unsigned short aTextFlags)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (pEngine)
	{
		TRAPD_WRAPPER(pEngine->GetInputTextL(initialText->text.utf16String, initialText->textLen, maxNumChars, aTextFlags));
	}
	SI_EXIT
}

#ifdef FEATURE_INLINE_TEXT_ENTRY

void SI_Interface::SI_EnableIME (struct MM_Object* aMMObj, struct FI_TextField* aTextField)
{
	SI_ENTER
	
	FL_TRACE(("SI_Interface::SI_EnableIME..."));

	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if(pEngine)
	{
		pEngine->GetTextNavigation( )->DidSelect( aTextField );
		TRAPD_WRAPPER(pEngine->EnableIMEL());
	}

	FL_TRACE(("...SI_Interface::SI_EnableIME"));

	SI_EXIT
}

void SI_Interface::SI_DisableIME (struct MM_Object* aMMObj)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if(pEngine)
	{
		pEngine->DisableIME();
		pEngine->GetTextNavigation( )->WillDeselect( );
	}

	SI_EXIT
}
void SI_Interface::SI_DrawFullScreenInputModeIndicator(struct MM_Object* aMMObj, struct FI_Rect* editingTextFieldBounds, struct FI_Rect* indicatorBounds)
//void SI_Interface::SI_DrawFullScreenInputModeIndicator(struct MM_Object* aMMObj, struct FI_Rect* editingTextFieldBounds)
{
 	SI_ENTER
 	// At this point the host application has the option to draw the input mode indicator.
 	// The parameter 'editingTextFieldBounds' should be used to avoid drawing the indicator on top
 	// of the currently active text field.
 
 	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
 
 	if(pEngine)
 	{
 		TRect editingTextFieldBoundsTRect(editingTextFieldBounds->xmin,editingTextFieldBounds->ymin,editingTextFieldBounds->xmax,editingTextFieldBounds->ymax);
		TRect indicatorBoundsTRect(indicatorBounds->xmin,indicatorBounds->ymin,indicatorBounds->xmax,indicatorBounds->ymax);
 		pEngine->DrawFullScreenInputModeIndicator(editingTextFieldBoundsTRect, indicatorBoundsTRect);

		indicatorBounds->xmin = (short)indicatorBoundsTRect.iTl.iX;
		indicatorBounds->ymin = (short)indicatorBoundsTRect.iTl.iY;
		indicatorBounds->xmax = (short)indicatorBoundsTRect.iBr.iX;
		indicatorBounds->ymax = (short)indicatorBoundsTRect.iBr.iY;

 	}
 
 	SI_EXIT
}


#endif // FEATURE_INLINE_TEXT_ENTRY

/**
*
* @static
*/
unsigned short SI_Interface::SI_GetURL (struct MM_Object* aMMObj, struct FI_GetURLCallInfo* aUrlInfo)
{
	SI_ENTER
	short retVal=0;
	
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
#ifdef FF_FLASHLITE_SCREENSAVER
   if(pEngine && pEngine->IsBrowserPluginActive())
    {
        if(pEngine->NoGetURLSupport()) 
        {
            retVal = SI_UnsupportedError;
            SI_EXIT
            return retVal;
        }        
    }
#endif //FF_FLASHLITE_SCREENSAVER	

	if (pEngine)
	{
		TRAP_IGNORE(retVal=pEngine->HandleURLL(aMMObj, aUrlInfo));
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/
//FL4.0 :TODO
SI_Error SI_Interface::SI_ConvertCStringToUTF16String(struct MM_Object* aMMObj,
														  const char* src, 
														  unsigned int /*codePage*/, FI_ReturnString* dst)
{
	SI_ENTER
	SI_Error error = SI_UnsupportedError;

	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine)
	{	
		TRAPD(result, pEngine->ConvertCStringToUTF16L(src, dst));

		if (result == KErrNone)
		{
			error = SI_NoError;
		}
		else
		{
			error = SI_GenericError;
		}
	}
	SI_EXIT
	return error;
}

	
/**
*
* @static
*/
SI_Error SI_Interface::SI_ConvertUTF16StringToCString(struct MM_Object* aMMObj,
														  const FI_UTF16Char* src,
														  unsigned int /*codePage*/, FI_ReturnString* dst)
{
	SI_ENTER
	SI_Error error = SI_UnsupportedError;

	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine)
	{	
		TRAPD(result, pEngine->ConvertUTF16ToCStringL(src, dst));

		if (result == KErrNone)
		{
			error = SI_NoError;
		}
		else
		{
			error = SI_GenericError;
		}
	}
	SI_EXIT
	return error;
}


TBool isFiletypeSWZ(const char* aDocumentPath)
{
	_LIT8(KFileExtensionSWZ, ".swz");
	const TInt KFileExtensionSWZLength = 4;
	TPtrC8 documentPathPtr((TUint8*)aDocumentPath);

	if (documentPathPtr.Right(KFileExtensionSWZLength).CompareF(KFileExtensionSWZ()) == 0)
		return ETrue;
	
	return EFalse;
}

_LIT8(KLitFile,"file://");
_LIT8(KLitLocalhostName, "localhost");
_LIT8(KLitLocalhostNumber, "127.0.0.1");

/**
 * The DoSI_OpenURLStreamL provides the implementation for the SI_OpenURLStream method, but is allowed to Leave.
 *
 * @see SI_OpenURLStream
 */
short DoSI_OpenURLStreamL (struct MM_Object* aMMObj, struct FI_URLStreamInfo* pStreamInfo, unsigned short streamId)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (!pEngine)
	{
		return 0;
	}	

	/**
	 * coreplayer does not support filetype of .SWZ
	 * if we get a request for this filetype, raise a critical error
	 * so that this situation is handled by displaying a "Broken Flash"
	 */	 
	if (pStreamInfo->documentPath && isFiletypeSWZ(pStreamInfo->documentPath))
	{
		pEngine->GetUserNotifierIf()->NotifyClientError(FI_CorruptSWFDataError,ETrue);
		return 0;
	}
	
	unsigned short result = SI_GenericError;

	TPtrC8 wrapper;
	if (pStreamInfo->protocolSchema)
	{
		wrapper.Set((TUint8*)pStreamInfo->protocolSchema);
	}

	if ( 0==wrapper.Compare(KLitFile))
	{
		if (pEngine->GetFileEngine())
		{
			CFileEngineRequest* feReq=CFileEngineRequest::NewL(aMMObj, pStreamInfo, streamId);
			result = pEngine->GetFileEngine()->StartFileRequest(feReq);
		}
	}
	else
	{
		// the CUrlRequestInfo validates the transport protocol, and will leave if it is not a supported type
		TPtrC8 host;
		if (pStreamInfo->hostName)
		{
			host.Set((TUint8*)pStreamInfo->hostName);
		}
#ifdef __USE_BROWSER_STREAM__
		if(pEngine->IsBrowserPluginActive())
		{
			result = pEngine->CreateBrowserStreamL(streamId, pStreamInfo);
			return result;
		}
#endif
		if ((pEngine->GetLocalHttpEngine()) && (( 0==host.Compare(KLitLocalhostName)) || (0==host.Compare(KLitLocalhostNumber))))
		{
			CUrlRequestInfo* info=CLocalUrlRequestInfo::NewL(pEngine->GetMMObject(), aMMObj,pStreamInfo,streamId);
			result = pEngine->GetLocalHttpEngine()->StartLocalRequestL(info);
		}
		else if (pEngine->GetHttpEngine())
		{
			CUrlRequestInfo* info=CUrlRequestInfo::NewL(pEngine->GetMMObject(), aMMObj,pStreamInfo,streamId);
			result = pEngine->GetHttpEngine()->StartHttpRequestL(info);
		}
	}
			return result;
}

/**
*
* @static
*/
short SI_Interface::SI_OpenURLStream(struct MM_Object* aMMObj, struct FI_URLStreamInfo* pStreamInfo, unsigned short streamId)
{
	SI_ENTER
	short retVal=0;
	TRAPD_WRAPPER(retVal= DoSI_OpenURLStreamL(aMMObj,pStreamInfo, streamId));
	SI_EXIT
	return retVal;
}

void DoSI_LoadURLStreamBufferL(struct MM_Object* aMMObj, unsigned short streamId,  struct FI_NetworkBufferInfo* pNetBuf)
{
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (!pEngine)
	{
		return;
	}
	
	// see if its a file object...
	CFileEngineRequest* info = pEngine->GetFileEngine() ? pEngine->GetFileEngine()->GetRequestObjectOrNull(streamId) : NULL;
	
	if (info) 
	{
		info->SetNetBuf(pNetBuf);
	}
	else
	{
		CUrlRequestInfo* urlInfo=(CUrlRequestInfo*) (pEngine->GetHttpEngine() ? pEngine->GetHttpEngine()->GetRequestObject(pEngine->GetMMObject(),streamId) : NULL);
			
		if(urlInfo)
		{
			urlInfo->SetNetworkBuffer(pNetBuf);
		}
		else if (pEngine->IsBrowserPluginActive())
			{
			//need to use the struct given by this func argument, to wrtie the stream data for MM Core Player
			pEngine->SetNetworkBufferL(streamId, pNetBuf);
			}
	}
}
/**
*
* @static
*/
void SI_Interface::SI_LoadURLStreamBuffer (struct MM_Object* aMMObj, unsigned short streamId,  struct FI_NetworkBufferInfo* pNetBuf)
{
	SI_ENTER
	TRAPD_WRAPPER(DoSI_LoadURLStreamBufferL(aMMObj,streamId, pNetBuf));
	SI_EXIT
}
	
/**
*
* @static
*/
void SI_Interface::SI_CancelURLStream (struct MM_Object* aMMObj, unsigned short streamId)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (!pEngine)
	{
		return;
	}
	
	TRAPD_WRAPPER(pEngine->CancelURLStream(streamId));
	
	SI_EXIT
}

	
/**
*
* @static
*/
void SI_Interface::SI_NotifyError (struct MM_Object* aMMObj, short errorCode)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	
	if(!pEngine)
		return;
	
	TBool isCriticalError = EFalse;

	switch (errorCode)
	{ 
	// only stop the movie for these errors;
	case FI_OutOfMemoryError:
	case  FI_StreamURLLimitReachedError:
	case FI_StackLimitReachedError:
	case FI_CorruptSWFDataError:
	case FI_RootMovieUnloadedError:
	case FI_CorruptSVGDataError:
		pEngine->StopPlaying();
        isCriticalError = ETrue;
		break;
    case FI_ActionScriptStuckError:
        isCriticalError = EFalse;
        break;
	}
/*    if(!(pEngine->IsBrowserPluginActive()))
        {
        //pEngine->GetUI()->NotifyFlashError(errorCode, isCriticalError); // this can't be allowed to leave
        pEngine->GetUserNotifierIf()->NotifyFlashError(errorCode, isCriticalError); // this can't be allowed to leave
        }*/
    //TODO : Revisit
    /*
    else if(pEngine->GetPlugin())
        {
            
#ifdef FF_FLASHLITE_SCREENSAVER
        //if( pEngine->GetPlugin()->IsScreenSaverActive() )
        if( pEngine->NoSoundSupport())//no screensaver specific checks here
            {
            if( errorCode == FI_BadSoundDataError )
                {
                SI_EXIT    
                return;
                }
                
            pEngine->StopPlaying();
            }
#endif // FF_FLASHLITE_SCREENSAVER            
        TBool bitsLocked = pEngine->IsBitmapLocked();
        if(bitsLocked)
	        {
        	TAny* dummyArgs = NULL;
	        pEngine->UnlockBits(dummyArgs);	        
	        }                    
        pEngine->NotifyClientError(errorCode,isCriticalError);
        if(bitsLocked)
        	{
        	pEngine->LockBits();
        	}
        } 
        */   
        if(pEngine->GetExecutionMode() == eStandAlone)
        {
            pEngine->GetUserNotifierIf()->NotifyFlashError(errorCode, isCriticalError);  
        }
        else
        {
#ifdef FF_FLASHLITE_SCREENSAVER
        //if( pEngine->GetPlugin()->IsScreenSaverActive() )
            if( pEngine->NoSoundSupport())//no screensaver specific checks here
            {
                if( errorCode == FI_BadSoundDataError )
                {
                    SI_EXIT    
                    return;
                }
                    
                pEngine->StopPlaying();
            }
#endif // FF_FLASHLITE_SCREENSAVER            
			TBool bitsLocked = pEngine->IsBitmapLocked();
			if(bitsLocked)
			{
			    TAny* dummyArgs = NULL;
			    pEngine->UnlockFrameBufferBits(dummyArgs);         
			}                    
			pEngine->GetUserNotifierIf()->NotifyClientError(errorCode,isCriticalError);
			if(bitsLocked)
			{
			    pEngine->LockFrameBufferBits();
	        }    
        }
	SI_EXIT
	return;
}

	
/**
*
* @static
*/	
void SI_Interface::SI_UpdateFrameInterval (struct MM_Object* aMMObj, unsigned long newFrameInterval)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if(pEngine)
		pEngine->UpdateFrameInterval(newFrameInterval);
	SI_EXIT
}
/*Local Helper Function for DoSI_DoFsCommandL() to Decide if the FsCommand Passed in 
 *is Device FsCommand from CorePlayer or AS-JS Communication. 
 */
TInt GetFsCommandIndexL(TDesC& commandbuf)
	{
	
	//Create an Array for the 6 FSCommands Possible:quit, fullscreen, allowscale, showmenu, exec and trapallkeys
	CDesCArrayFlat* fsCommandsList = new (ELeave) CDesCArrayFlat(6);
	{
	//Appending within Block Scope so that all _LITs will get destroyed on exiting this block.
	//List of Supported FSCommands:
	_LIT(KFSCommandQuit,"quit");
	_LIT(KFSCommandFullscreen,"fullscreen");
	_LIT(KFSCommandAllowscale,"allowscale");
	_LIT(KFSCommandShowmenu,"showmenu");
	_LIT(KFSCommandExec,"exec");
	_LIT(KFSCommandTrapallkeys,"trapallkeys");

	//Now Append to Array
	fsCommandsList->AppendL(KFSCommandQuit);		//0
	fsCommandsList->AppendL(KFSCommandFullscreen);	//1
	fsCommandsList->AppendL(KFSCommandAllowscale);	//2
	fsCommandsList->AppendL(KFSCommandShowmenu);	//3
	fsCommandsList->AppendL(KFSCommandExec);		//4
	fsCommandsList->AppendL(KFSCommandTrapallkeys);	//5
	}
	//Now Check if command passed in exists in Array:
	TInt count = 0;
	for(count = 0;count<fsCommandsList->Count();count++)
		{
		if (!commandbuf.CompareF( (*fsCommandsList)[count]) )
			{
			break;//out of For, and handle based on count value.
			}
		}
	//Done with the FsCommandList Array,Free it.
	fsCommandsList->Reset();
	delete fsCommandsList;

	//Here, if (0 <=count <=5), it is device FsCommand, else(count == 6), AS-JS communication.
	return count;	
	}
/**
*
* @static
*/
#ifdef __LOG_SI_CALLS__	
unsigned short DoSI_DoFsCommandL(struct MM_Object* aMMObj, const struct FI_Text* cmd, const struct FI_Text* args, struct FI_PlayerEventInfo* pEventInfo)
#else
unsigned short DoSI_DoFsCommandL(struct MM_Object* aMMObj, const struct FI_Text* cmd, const struct FI_Text* args, struct FI_PlayerEventInfo* pEventInfo)
#endif
{
	int retVal =0;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if(pEngine)	
	{
		HBufC * commandbuf = HBufC::NewL(cmd->textLen);
		commandbuf->Des().Copy(cmd->text.utf16String);
		
		HBufC * argumentsbuf = HBufC::NewL(args->textLen);
		argumentsbuf->Des().Copy(args->text.utf16String);
		
		TInt commandIndex = GetFsCommandIndexL(*commandbuf);

		// If commandIndex is in the range 0-5, 
		// It corresponds to one of the six device FsCommands,find out which and handle:
		// Else it is AS-JS Communcation: Default Case. 
		switch(commandIndex)
			{
			//Call specific SI_Command for the FSCommand which we already have.
			case 0://Quit
				{
				retVal = SI_Interface::SI_QuitFlashLite(aMMObj,pEventInfo);
				}
				break;
			case 1://Fullscreen: TRUE and 1 are treated as Fullscreen mode and any
                   //other values are treated as Normal mode
				{				
                _LIT(KTrue,"true");
                _LIT(KOne,"1"); 
                if (!argumentsbuf->CompareF(KTrue) || !argumentsbuf->CompareF(KOne))
                    {
                    retVal = SI_Interface::SI_SetFullScreen(aMMObj,1);                        
                    }
                else
                    {
                    retVal = SI_Interface::SI_SetFullScreen(aMMObj,0);
                    }	
				}				
				break;
			case 2://AllowScale-Not Implemented
				break;
			case 3://ShowMenu-Not Implemented
				break;
			case 4://Exec-Not Implemented
				break;
			case 5://TrapAllkeys-Not Implemented
				break;
			default://this is AS-JS communication, need to send to Browser
				{
#ifdef FEATURE_WEB_BROWSABILITY		
				char* command = NULL;
				char* arguments = NULL;
				if( cmd->textEncoding == FI_NativeEncodingForm )
					{
					FlashMemCpy(command, cmd->text.cString , cmd->textLen);
					}
				else
					{
					command = pEngine->ConvertUTF16ToCStringL(cmd->text.utf16String);
					}
				CleanupStack::PushL(command);
				if( args->textEncoding == FI_NativeEncodingForm )
					{
					FlashMemCpy(arguments, args->text.cString , args->textLen);
					}
				else
					{
					arguments = pEngine->ConvertUTF16ToCStringL(args->text.utf16String);
					}
		
				CleanupStack::PushL(arguments);
				if( pEngine->GetScriptHandler() )//this is true only in BrowserPlugin Mode
					{
					retVal = pEngine->GetScriptHandler()->DoFsCommandL( command, arguments);
					}		
				CleanupStack::PopAndDestroy(arguments);
				CleanupStack::PopAndDestroy(command);
#endif //FEATURE_WEB_BROWSABILITY
				}
				break;
			}//End Switch			
		delete commandbuf;
		commandbuf = NULL;
		delete argumentsbuf;
		argumentsbuf = NULL;
		
		
	}

	return retVal;

}
	
/**
*
* @static
*/
#ifdef __LOG_SI_CALLS__	
unsigned short SI_Interface::SI_DoFsCommand (struct MM_Object* aMMObj, const struct FI_Text* cmd, const struct FI_Text* args, struct FI_PlayerEventInfo* pEventInfo)
#else
unsigned short SI_Interface::SI_DoFsCommand (struct MM_Object* aMMObj, const struct FI_Text* cmd, const struct FI_Text* args, struct FI_PlayerEventInfo* pEventInfo)
#endif
{
	SI_ENTER
	TRAPD(err, DoSI_DoFsCommandL(aMMObj, cmd, args, pEventInfo ));
	if(err)
	{
		return SI_GenericError;
	}
	else
	{
		return SI_NoError;
	}
	SI_EXIT
}

/**
 * The DoSI_FormatTimeDate provides the implementation for the SI_FormatTimeDate method, but is allowed to Leave.
 *
 * @see SI_FormatTimeDate 
 */
short DoSI_FormatTimeDateL (struct MM_Object* aMMObj, const struct FI_LargeInteger* pTimeStamp, unsigned long modifiers, FI_ReturnString* pString, bool bIsDate)
{
	short result = 0;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine)
	{
		// Time pass in is already a localtime
		TInt64 localTimeStamp = pTimeStamp->highPart;
		localTimeStamp = localTimeStamp << 32;
		localTimeStamp = localTimeStamp + pTimeStamp->lowPart;
		localTimeStamp *= TInt64(KMillisecondsInSecond);
		localTimeStamp += pEngine->GetUTCReferenceTime().Int64();
		TTime localTime(localTimeStamp);
				
		TBuf16<KDateStringBufferLength> dateString16;

		// Format the date using locale
		if (bIsDate){
			
			if ((modifiers & FI_FormatDateLong) == FI_FormatDateLong)
				localTime.FormatL(dateString16, TLongDateFormatSpec());
			else
				localTime.FormatL(dateString16, TShortDateFormatSpec());
		} else {
			
			//  Get the current time format spec (descriptor)			
			TTimeFormatSpec tSpec;
			tSpec.Set();

			//  The current time format spec does not account for the current 12/24 hr locale setting.  We need to manually
			//  change the descriptor to force it to pay attention to the locale setting "%I" -> "%J"
			_LIT(iChar, "I");
			_LIT(jChar, "J");
			TInt loc = tSpec.Find(iChar);
			if (loc != KErrNotFound){ 
				tSpec.Replace(loc, 1, jChar);
			}
			//  Format the time using locale
			localTime.FormatL(dateString16, tSpec);

		}
		
		dateString16.ZeroTerminate();
		pString->SetUTF16String( pString, dateString16.Ptr() );
		

		result = 1;
	}

	return result;
}

	
/**
*
* @static
*/
short SI_Interface::SI_FormatDate (struct MM_Object* aMMObj, const struct FI_LargeInteger* pTimeStamp, unsigned long modifiers, FI_ReturnString* pString)
{
	SI_ENTER
	short retVal=0;
	TRAPD_WRAPPER(retVal=DoSI_FormatTimeDateL(aMMObj, pTimeStamp, modifiers, pString, true));
	SI_EXIT	
	return retVal;
}


	
/**
*
* @static
*/
short SI_Interface::SI_FormatTimeOfDay (struct MM_Object* aMMObj, const struct FI_LargeInteger* pTimeStamp, unsigned long modifiers, FI_ReturnString* pString)
{
	SI_ENTER
	short retVal=0;
	TRAPD_WRAPPER(retVal=DoSI_FormatTimeDateL(aMMObj, pTimeStamp, modifiers, pString, false));
	SI_EXIT	
	return retVal;
}

/*
*
* @static
*/
short SI_Interface::SI_GetBatteryLevel (struct MM_Object* aMMObj)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	short bval = 0;

	if (pEngine)
	{				
		TRAP_IGNORE( bval = pEngine->GetBatteryLevelL() );
    }
    SI_EXIT
	return bval;
}

const TInt KGetDeviceBufferSize = 128;	// from Flash Lite 2.0 API spec -- buffer is 128 bytes
/**
*
* @static
*/
short SI_Interface::SI_GetDevice (struct MM_Object* aMMObj, unsigned char * buffer)
{
	SI_ENTER
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	short retVal=-1;
	if (pEngine)
	{
		retVal=(short)pEngine->GetDeviceName(buffer, KGetDeviceBufferSize);
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/

unsigned short SI_Interface::SI_GetDeviceCapabilities (struct MM_Object* aMMObj)
{
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	
	SI_ENTER
	
	unsigned int caps=0;
	
    if(pEngine && pEngine->IsBrowserPluginActive())
    {
        caps = pEngine->GetClientIf()->GetFeatureCapability();//capability should come from plugin
    }
    else
    { //else viewer model
        caps = FI_DeviceCap_Email | FI_DeviceCap_SMS | 
               FI_DeviceCap_MMS | FI_DeviceCap_LoadData  | 
               FI_DeviceCap_XMLSocket | FI_DeviceCap_MappableSoftKeys;
        
        if(pEngine && pEngine->IsTouchSupported())
        {
            caps |= FI_DeviceCap_Stylus;
        }
    }
	
	if(pEngine && pEngine->HasQuertyKBSuppport())
			  caps |= FI_DeviceCap_QwertyKB;
	SI_EXIT	

	//Camp : changed from unsigned int to unsigned short
	return (unsigned short)caps;	
}


short SI_Interface::SI_GetDeviceID (struct MM_Object* aMMObj, unsigned char * buffer)
{
	SI_ENTER
	SI_Interface::SI_Debug(aMMObj,"SI_GetDeviceID",0);
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
		
	short retVal=0;
	TInt err=KErrNone;
	if(pEngine)
	{
		TRAP(err,retVal = pEngine->GetDeviceID(buffer,KGetDeviceBufferSize));
	}
		
    if(err)
    {
    	SI_EXIT                    
        return SI_GenericError;
    }
    
	SI_EXIT
	
	return retVal;
}


struct TFlashLang
{
	TInt langId;
	const char* langStr;
};

const TFlashLang KFlashLanguageList[] =
{
	{ ELangAmerican, "en-US" },
	{ ELangArabic, "ar" },
	{ ELangBasque, "eu" },
	{ ELangBulgarian, "bg" },
	{ ELangCroatian, "hr" },
	{ ELangCzech, "cz" },
	{ ELangDanish, "da" },
	{ ELangDutch, "nl" },
	{ ELangEnglish, "en-UK" },
	{ ELangEstonian, "et" },
	{ ELangFinnish, "fi" },
	{ ELangFrench, "fr" },
	{ ELangGalician, "gl" },
	{ ELangGerman, "de" },
	{ ELangGreek, "el" },
	{ ELangHebrew, "he" },
	{ ELangHungarian, "hu" },
	{ ELangIcelandic, "is" },
	{ ELangIndonesian, "id" },
	{ ELangItalian, "it" },
	{ ELangJapanese, "ja" },
	{ ELangLatvian, "lv" },
	{ ELangLithuanian, "lt" },
	{ ELangMalay, "ms" },
	{ ELangNorwegian, "no" },
	{ ELangPolish, "pl" },
	{ ELangPortuguese, "pt" },
	{ ELangRomanian, "ro" },
	{ ELangRussian, "ru" },
	{ ELangSerbian, "sr" },
	{ ELangSlovak, "sk" },
	{ ELangSlovenian, "sl" },
	{ ELangSpanish, "es" },
	{ ELangSwedish, "sv" },
	{ ELangThai, "th" },
	{ ELangUrdu, "ur" }, 
	{ ELangTurkish, "tr" },
	{ ELangUkrainian, "uk" },
	{ ELangVietnamese, "vi" },
//	{ ELangPhilipino, "??" },

	// variations on English
	{ ELangApacEnglish, "en" },
	{ ELangAustralian, "en" },
	{ ELangCanadianEnglish, "en" },
	{ ELangHongKongEnglish, "en" },
	{ ELangHongKongEnglish, "en" },
	{ ELangInternationalEnglish, "en" },
	{ ELangJapaneseEnglish, "en" },
	{ ELangNewZealand, "en" },
	{ ELangPrcEnglish, "en" },
	{ ELangPrcEnglish, "en" },
	{ ELangSouthAfricanEnglish, "en" },
	{ ELangTaiwanEnglish, "en" },
	{ ELangTaiwanEnglish, "en" },
	{ ELangThaiEnglish, "en" },
	{ ELangThaiEnglish, "en" },
	
	// variations on Chinese
	{ ELangHongKongChinese, "zh-TW" },
	{ ELangPrcChinese, "zh-CN" },
	{ ELangTaiwanChinese, "zh-TW" },

	// variations on French
	{ ELangBelgianFrench, "fr" },
	{ ELangCanadianFrench, "fr" },
	{ ELangInternationalFrench, "fr" },
	{ ELangSwissFrench, "fr" },
	
	// variations on other languages
	{ ELangApacIndonesian, "id" },
	{ ELangApacMalay, "ms" },
	{ ELangAustrian, "de" },
	{ ELangBrazilianPortuguese, "pt" },
	{ ELangCyprusGreek, "el" },
	{ ELangCyprusTurkish, "tr" },
	{ ELangFinlandSwedish, "sv" },
	{ ELangInternationalSpanish, "es" },
	{ ELangKorean, "ko" },
	{ ELangLatinAmericanSpanish, "es" },
	{ ELangSwissGerman, "de" },
	{ ELangSwissItalian, "it" },

	{ -1, NULL }
};

	
/**
*
* @static
*/
#ifdef __LOG_SI_CALLS__
short SI_Interface::SI_GetLanguage (struct MM_Object* aMMObj, unsigned char * buffer)
#else
short SI_Interface::SI_GetLanguage (struct MM_Object* /*aMMObj*/, unsigned char * buffer)
#endif
{
	SI_ENTER
	short result = 0;

	TInt lang = (TInt)User::Language();
	const char* lstr = "xu"; // default -- undefined language

	for ( TInt i = 0; KFlashLanguageList[i].langId != -1; i++ )
	{
		if ( lang == KFlashLanguageList[i].langId )
		{
			lstr = KFlashLanguageList[i].langStr;
			break;
		}
	}

	if ( lstr )
	{
		FlashStrCpy((char*)buffer, lstr);
	}

	result = (short)(lstr ? 1 : 0);
	SI_EXIT
	return result;
}

	
/**
*
* @static
*/	
#ifdef __LOG_SI_CALLS__
short SI_Interface::SI_GetMaxBatteryLevel (struct MM_Object* aMMObj)
#else
short SI_Interface::SI_GetMaxBatteryLevel (struct MM_Object* /*aMMObj*/)
#endif	
{
	SI_ENTER
	SI_EXIT
	return EBatteryLevelLevel7;
}
	
/**
 * Returns the maximum volume level.  It must return -1 if the
 * function is not support * (i.e. the information is not
 * accessible.)
 *
 * @static
 */
#ifdef __LOG_SI_CALLS__
short SI_Interface::SI_GetMaxVolumeLevel (struct MM_Object* aMMObj)
#else
short SI_Interface::SI_GetMaxVolumeLevel (struct MM_Object* /*aMMObj*/)
#endif
{
	SI_ENTER
	SI_EXIT
	return KMaxUserVolume;
}

const TInt KGetPlatformBufferSize = 128;	// from Flash Lite 2.0 API spec -- buffer is 128 bytes
/**
*
* @static
*/
short SI_Interface::SI_GetPlatform (struct MM_Object* aMMObj, unsigned char * buffer)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	short retVal=-1;

	if (pEngine)
	{
		retVal=(short)pEngine->GetPlatformName(buffer, KGetPlatformBufferSize);
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/	
short SI_Interface::SI_GetPowerSource (struct MM_Object* aMMObj)
{
	SI_ENTER
	short result = 0;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine)
	{		
		TRAP_IGNORE ( result = pEngine->GetBatterychargeStatusL() );
	}
	SI_EXIT
	return result;
}

	
/**
*
* @static
*/	
short SI_Interface::SI_GetUTCTimeStamp (struct MM_Object* aMMObj, struct FI_LargeInteger* pResult)
{
	SI_ENTER
	short result = 0;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine)
	{
		// In this function, we are calculating our own UTC time based on the current time
		// and the reference time of 1/1/1970 Midnight.  By definition, UTC time is the
		// number of micro-seconds from 1/1/1970 without any offset and DST adjustment.

		// Current time
		TTime now;
		now.UniversalTime();
		
		// Get the number of micro-seconds from 1/1/1970 until now.
		TTimeIntervalMicroSeconds utcTime = now.MicroSecondsFrom(pEngine->GetUTCReferenceTime());

		// don't forget to convert to milli-seconds for Flash
		TInt64 li = (utcTime.Int64() / TInt64(KMillisecondsInSecond));

		pResult->lowPart = li & 0xffffffff;
		pResult->highPart = (li >> 32) & 0xffffffff;

		result = 1;
	}
	SI_EXIT
	return result;
}

//--------------------------------------------------------------------------------------------------------------------------
//  Defines are specific to the _ConvertTime function
enum TConvertType {

	kConvertToUTC		= 0,
	kConvertToLocal		= 1

};

const TInt KOneHour_InMilliSeconds = 3600000;

//--------------------------------------------------------------------------------------------------------------------------
/**
*
* @static
*/
short _ConvertTime(const struct FI_LargeInteger *source, struct FI_LargeInteger *destination, TConvertType convertType)
{


	//  Get the Offset information from the current locale
	TLocale locale;
	locale.Refresh();

	//  Get the UTC offset in Milliseconds.  Offset includes signing from GTC perspective (PST = -6 hrs)
	TInt64 utcOffset = locale.UniversalTimeOffset().Int() * KMillisecondsInSecond;
	
	//  Get the daylightsavings offset in Milliseconds
	TInt64 daylightSavingsOffset = 0;
	if(locale.QueryHomeHasDaylightSavingOn()){
        daylightSavingsOffset = KOneHour_InMilliSeconds ;
    }
 

	//  Create a TInt64 from the source integer for easy calculations
	  TInt64 source64 = source->highPart;
      source64 = source64 << 32;
      source64 = source64 + source->lowPart;

	//  If converting TO UTC then SUBTRACT offset
	if (convertType == kConvertToUTC) {

		source64 -= (daylightSavingsOffset + utcOffset);
	} 
	
	//  If convderting TO LOCAL then ADD offset
	else if (convertType == kConvertToLocal) {

		source64 += (daylightSavingsOffset + utcOffset);
	}

	//  Set the destination FI_Integer
	destination->lowPart = source64;
	destination->highPart = source64 >> 32;

	return 1;
}

	
/**
*
* @static
*/
#ifdef __LOG_SI_CALLS__
short SI_Interface::SI_ConvertUTCToLocalTime(struct MM_Object* aMMObj, 
#else
short SI_Interface::SI_ConvertUTCToLocalTime(struct MM_Object* /*aMMObj*/, 
#endif
										  const struct FI_LargeInteger* aUTCTimeStamp, 
										  struct FI_LargeInteger* aLocalTimeStamp)
{
	SI_ENTER
	SI_EXIT
	return _ConvertTime(aUTCTimeStamp, aLocalTimeStamp, kConvertToLocal);
}


	
/**
*
* @static
*/
#ifdef __LOG_SI_CALLS__
short SI_Interface::SI_ConvertLocalToUTCTime(struct MM_Object* aMMObj, 
#else
short SI_Interface::SI_ConvertLocalToUTCTime(struct MM_Object* /*aMMObj*/, 
#endif
										  const struct FI_LargeInteger* aLocalTimeStamp, 
										  struct FI_LargeInteger* aUTCTimeStamp)
{
	SI_ENTER
	SI_EXIT
	return _ConvertTime(aLocalTimeStamp, aUTCTimeStamp, kConvertToUTC);
}


	
/**
*
* @static
*/
short SI_Interface::SI_GetVolumeLevel (struct MM_Object* aMMObj)
{
	SI_ENTER
	short result = 0;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	if (pEngine)
	{
		result = (short)pEngine->GetVolume();
		if (result & KMuteMask)
		{
			result = 0;
		}
	}
	SI_EXIT
	return result;
}


	
/**
*
* @static
*/
short SI_Interface::SI_QuitFlashLite (struct MM_Object* aMMObj, struct FI_PlayerEventInfo* pEventInfo)
{
	SI_ENTER
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	short retVal=0;
    if(pEngine && pEngine->IsBrowserPluginActive())
        {
        SI_EXIT
        return -1;
        }
	// only process Quit if during a key or mouse event
	if (pEventInfo->eventCode != FI_EventNoEvent)
	{
		if(!( pEngine-> IsBrowserPluginActive() ))
		{
			//pEngine->GetFlashUI()->Quit(); 
            pEngine->GetClientIf()->Quit();
		} //TODO : do what to with plug-in
		
		retVal=1;
	}
	SI_EXIT
	return retVal;
}


	
/**
 * The DoSI_LaunchL provides the implementation for the SI_Launch method, but is allowed to Leave.
 *
 * @see SI_Launch
 */
void DoSI_LaunchL(struct MM_Object* aMMObj, unsigned short argc, const FI_Text** aArgv, struct FI_PlayerEventInfo* pEventInfo, short isEmbedded)
{
	const TDesC** argv=new(ELeave) const TDesC*[argc];
	CleanupStack::PushL(argv);

	TInt len;
	const FI_Text* txt;
	TPtrC* arg;
	for(TInt i=0;i<argc;i++)
	{
		txt=aArgv[i];
		len=txt->textLen;
		arg=new(ELeave) TPtrC(txt->text.utf16String,len);
		CleanupStack::PushL(arg);
		ASSERT(txt->textEncoding==FI_UTF16EncodingForm);
		argv[i]=arg;
	}
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	pEngine->QueueSILaunchL(aMMObj,argc,argv,pEventInfo,isEmbedded);
	CleanupStack::PopAndDestroy(argc+1,argv);
}


	
/**
* The SI_Launch function is called by the player to launch an external application.
*
* @return 0 failed to launch, -1 not supported, 1 launched successfully 
* @static 
*/
short SI_Interface::SI_Launch (struct MM_Object* aMMObj, unsigned short argc, const FI_Text** argv, struct FI_PlayerEventInfo* pEventInfo, short isEmbedded)
{
	SI_ENTER
	short res=1;
	CPlatformFlashEngine* pHEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if(pHEngine && pHEngine->IsBrowserPluginActive())
    	{
		return -1;
	    }	
	TRAPD(error,DoSI_LaunchL(aMMObj,argc,argv,pEventInfo,isEmbedded));
	if(error)
		res=0;

	SI_EXIT
	return res;
}

	
/**
*
* @static
*/
short SI_Interface::SI_ResetSoftkeys (struct MM_Object* aMMObj)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if(pEngine && pEngine->IsBrowserPluginActive())
    	{
		return -1;
	    }	
	if(pEngine)
		pEngine->ResetSoftKeys();
	
	SI_EXIT
	return 1;
}

/**
*
* @static
*/
short SI_Interface::SI_GetSoftKeyLocation (struct MM_Object* aMMObj)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	
	SI_EXIT
	
	if(!pEngine)
		return -1;
	
	return pEngine->GetSoftKeyLocation();
}

#ifdef __FEATURE_OSK__
/**
*
* @static
* This function enables or diables compatibility mode onscreen keypad
*/  
short SI_Interface::SI_DisableKeypadCompatibilityMode (struct MM_Object* aMMObj)
{
    SI_ENTER
    CPlatformFlashEngine* pHEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
    if(pHEngine && pHEngine->IsBrowserPluginActive())
    {
        return -1;
    }   
    pHEngine->SetCompatibilityMode(EFalse);
    SI_EXIT
    return 1;
}
#endif // __FEATURE_OSK__
	
/**
*
* @static
*/	
short SI_Interface::SI_SetFullScreen (struct MM_Object* aMMObj, unsigned short onOrOff)
{
    SI_ENTER
    CPlatformFlashEngine* pHEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
    if ( NULL == pHEngine)
    {
        SI_EXIT
        return 0;
    }
    /*RC ou1cimx1#599150 
    The swf on the webpage without any video element shouldn't be allowed to switch to full screen mode
    as there is no option to come back to normal screen.
    The exception for above case is plugin running in widget mode, where the swf is allowed to switch to
    fullscreen mode regardless of whether the swf has video element or not. 
    Though the Action script has fscommand to switch to full screen mode, this should be rejected in the above case. hence below check is added.*/
    if ( ePlugin == pHEngine->GetExecutionMode() && EDefaultMode == pHEngine->GetBrPluginMode() && EFalse == pHEngine->GetFlvPlayState())
    {
        // Return 1 = success , -1 = Not applicable , 0 = Failure.
        SI_EXIT
        return -1 ;
    }
	//pHEngine->
	
	//MFlashUI* flashUI = pHEngine->GetFlashUI();
	
	if ( onOrOff )
	{
		pHEngine->ChangeScreen(EChangeScreenFullScreen);
		//if(flashUI)
		if(eStandAlone==pHEngine->GetExecutionMode())
		    pHEngine->SetFullScreenViaFSCommand(ETrue);
	}
	else
	{
		pHEngine->ChangeScreen(EChangeScreenAppScreen);
        //if(flashUI)
		if(eStandAlone==pHEngine->GetExecutionMode())
            pHEngine->SetFullScreenViaFSCommand(EFalse);
	}
	
	
	SI_EXIT
	return 1;
}

	
/**
*
* @static
*/
short SI_Interface::SI_GetNumberOfSoftkeys(struct MM_Object* aMMObj)
{
	SI_ENTER
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	short retVal = 0;
	if(pEngine)
		retVal = (short)pEngine->GetSoftkeyCount();
	
	SI_EXIT
	return retVal;
}

	
/**
 * The DoSI_SetSoftkeysL provides the implementation for the SI_SetSoftkeys method, but is allowed to Leave.
 *
 * @see SI_SetSoftkeys
 */	
short DoSI_SetSoftkeysL (struct MM_Object* aMMObj, unsigned short argc, const FI_Text** argv)
{
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	HBufC** bufs;
	bufs = new(ELeave) HBufC*[argc];
	CleanupStack::PushL(bufs);

    int pushed = 0;
    const FI_Text* text;
    for(TInt i=0; i<argc; i++)
    {
		text = argv[i];
        if(text != NULL)
        {
            TPtr ptr((TUint16*)text->text.utf16String, text->textLen, text->textLen);
            bufs[i] = ptr.AllocLC();
            pushed++;
        }
        else
        {
            bufs[i] = NULL;
        }
    }

	if(pEngine)
    	pEngine->SetSoftKeys(bufs, argc);
	
	CleanupStack::Pop(pushed+1, bufs);
	delete bufs;

    return 1;
}

	
/**
*
* @static
*/
short SI_Interface::SI_SetSoftkeys (struct MM_Object* aMMObj, unsigned short argc, const FI_Text** argv)
{
	SI_ENTER
	short retVal=0;
	CPlatformFlashEngine* pHEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if(pHEngine && pHEngine->IsBrowserPluginActive())
    	{
		return -1;
	    }	
	TRAPD_WRAPPER(retVal=DoSI_SetSoftkeysL (aMMObj, argc, argv));
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/
short SI_Interface::SI_StartVibrate (struct MM_Object* aMMObj, unsigned short timeOn, unsigned short timeOff, unsigned short repeatCount)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	short retVal = KVibraFeatureNotSupported;

    if(pEngine && pEngine->IsBrowserPluginActive())
    {
       	 if(pEngine->NoVibraSupport())//no screensaver ref here
        {
            return retVal;
        }
        
    }

	if (pEngine)
	{
		TBool featureVibrationSupported = pEngine->IsVibrationSupported();
		if(featureVibrationSupported)
			{
			retVal = pEngine->StartVibrate(timeOn, timeOff, repeatCount);
			}
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/
short SI_Interface::SI_StopVibrate (struct MM_Object* aMMObj)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	short retVal = KVibraFeatureNotSupported;

    if(pEngine && pEngine->IsBrowserPluginActive())
    {
        if(pEngine->NoVibraSupport()) //no screensaver support here
        {
            return retVal;
        }
    }

	if (pEngine)
	{
		TInt featureVibrationSupported = pEngine->IsVibrationSupported();
		if(featureVibrationSupported)
			{
			retVal = pEngine->StopVibrate();
			}
	}
	SI_EXIT
	return retVal;
}


void SI_Interface::SI_Debug (struct MM_Object* /*aMMObj*/, const char* message, long number)
{
	SYMBIANDEBUG(256, "SI_Debug: %s, %x\r\n", message, number);
	

#ifdef __FILE_MODE_LOGGING
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	
	RFile file;
    TInt err = file.Open(pEngine->GetRFs(), _L("c:\\fl2_debug.txt"), EFileWrite | EFileShareExclusive | EFileStream);
	if (err == KErrNotFound)
	{
        err = file.Create(pEngine->GetRFs(), _L("c:\\fl2_debug.txt"), EFileWrite | EFileShareExclusive | EFileStream);
	}
	if (err == KErrNone)
	{
		TInt pos = 0;
		file.Seek(ESeekEnd, pos);
		TBuf8<256> msgbuf;
		TPtrC8 msg((const unsigned char*)message, FlashStrLen(message));
		msgbuf.Copy(msg);
		msgbuf.AppendNum((TInt)number);
		msgbuf.Append('\r');
		msgbuf.Append('\n');
		err = file.Write(msgbuf);
		file.Close();
	}
	
#else
    
    TBuf8<256> msgbuf;
	TPtrC8 msg((const unsigned char*)message, FlashStrLen(message));
	msgbuf.Copy(msg);
	msgbuf.AppendNum((TInt)number);
	msgbuf.Append('\r');
	msgbuf.Append('\n');
    RDebug::RawPrint(msgbuf);
    
#endif
}

/**
*
* @static
*/
short SI_Interface::SI_GetImageInfo (struct MM_Object* aMMObj, unsigned long pImageId, const struct FI_ImageData* pImageData)
{
	SI_ENTER
    TInt retValue = 1;

	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	
	if (pEngine && pEngine->GetImageConverter())
	{
		TRAPD(ret, pEngine->GetImageConverter()->HandleImageInfoRequestL(pImageId, pImageData));
		if(ret != 0)
		    retValue = ret;		    
	}

	SI_EXIT
	return retValue;
}

/**
*
* @static
*/
short SI_Interface::SI_CancelImageRequest (struct MM_Object* aMMObj, unsigned long pImageId)
{
	SI_ENTER
    short retValue = 0;

	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	
	if (pEngine && pEngine->GetImageConverter())
	{
		TRAPD_WRAPPER( retValue = pEngine->GetImageConverter()->CancelImageRequest(pImageId));
	}

	SI_EXIT
	return retValue;
}

	
/**
*
* @static
*/
short SI_Interface::SI_ExtendBacklightDuration(struct MM_Object* aMMObj, unsigned short duration)
{
	SI_ENTER
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

    if(pEngine && pEngine->IsBrowserPluginActive())
    {
        short success;
        
        if(pEngine->NoBackLightProcessing()) //no screesaver specific handling here
        {
            success = -1;
            return success;
        }
    }

	short retVal = (short)pEngine->ExtendBacklight(duration);
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/
#ifdef __LOG_SI_CALLS__
short SI_Interface::SI_GetScreenInfo(struct MM_Object* aMMObj, unsigned short* x, unsigned short* y, unsigned short* rotation)
#else
short SI_Interface::SI_GetScreenInfo(struct MM_Object* /*aMMObj*/, unsigned short* x, unsigned short* y, unsigned short* rotation)
#endif
{
	SI_ENTER
	CWsScreenDevice* pScreenDevice = CCoeEnv::Static()->ScreenDevice();
	TInt mode = pScreenDevice->CurrentScreenMode();
	TPixelsAndRotation sizeAndRotation;

	short retVal=-1;

	if (pScreenDevice)
	{
		pScreenDevice->GetScreenModeSizeAndRotation(mode, sizeAndRotation);
		*x = static_cast<unsigned short>(sizeAndRotation.iPixelSize.iWidth);
		*y = static_cast<unsigned short>(sizeAndRotation.iPixelSize.iHeight);
		*rotation = 0; // default to zero
		switch(sizeAndRotation.iRotation)
		{
			case CFbsBitGc::EGraphicsOrientationNormal:
			*rotation = 0;
			break;

			case CFbsBitGc::EGraphicsOrientationRotated90:
			*rotation = 90;
			break;

			case CFbsBitGc::EGraphicsOrientationRotated180:
			*rotation = 180;
			break;

			case CFbsBitGc::EGraphicsOrientationRotated270:
			*rotation = 270;
			break;
		}
		retVal=1;
	}
	SI_EXIT
	return retVal;
}

	
/**
*
* @static
*/
short DoSI_GetNetworkInfoL(struct MM_Object* aMMObj, unsigned short infoType, long* retVal)
{

    short success = 1;
    /* In case of error, garbage value would be returned to core player, if not initialized */
    *retVal = -1;
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
    
    switch(infoType)
    {
    case FI_NetworkInfo_MaxSignalLevel:
    {
        *retVal = KSignalLevelMax - KSignalLevelMin;//(EPSNBars_7 - EPSNBars_0);
    }
    break;

    case FI_NetworkInfo_SignalLevel:
    {        
        if (!pEngine)
        {
            success = 0;
            break;
        }
        long sVal=0;

        TRAP_IGNORE ( sVal = pEngine->GetSignalStrengthL() );
        
        *retVal = sVal;

    }
    break;

#ifdef FEATURE_PACK_3

    case FI_NetworkInfo_NetworkGeneration:
    {
        long sVal;
        if(pEngine && pEngine->GetNetworkInfoL(infoType, sVal))
        {
            *retVal = sVal;
        }
        else
        {
            success = -1;
        }
    }
    break;

    case FI_NetworkInfo_NetworkStatus:
    /* As per API spec, NetworkConnect status should return
     * the status of GSM (2.5G, 3G or 3.5G) connection status, 
     * not data connection status. 
     */
    case FI_NetworkInfo_NetworkConnectStatus:
    {
        long sVal;
        if(pEngine && pEngine->GetRegistrationStatusL(infoType, sVal))
        {
            *retVal = sVal;
        }
        else
        {
            success = -1;
        }
    }
    break;

#endif // FEATURE_PACK_3
   
    case FI_NetworkInfo_NetworkRequestStatus:
    {
        if (!pEngine || !pEngine->GetHttpEngine())
        {
            success = 0;
            break;
        }
        *retVal = pEngine->GetHttpEngine()->GetStatusForPlayer();

    }
    break;

    default:
    {
        success = -1; // Function is not supported or the specific infoType
                      // parameter is not supported by this function
    }
    break;

    } // end switch

	

    return success;
}

short SI_Interface::SI_GetNetworkInfo(struct MM_Object* aMMObj, unsigned short infoType, long* retVal)
	{
	SI_ENTER
	TInt result = -1;
	TRAP_IGNORE ( result = DoSI_GetNetworkInfoL( aMMObj, infoType, retVal ) );
	SI_EXIT
	return result;
	}
	
/**
 * The DoSI_GetNetworkInfoStringL provides the implementation for the SI_GetNetworkInfoString method, but is allowed to Leave.
 *
 * @see SI_GetNetworkInfoString
 */
short DoSI_GetNetworkInfoStringL(struct MM_Object* aMMObj, 
                                 unsigned short infoType, 
                                 FI_ReturnString* retVal)
{
    short success = 0;
    
	CPlatformFlashEngine* pEngine=aMMObj->pHostObj->iFIInterface->GetEngine();

    switch(infoType)
    {
#ifdef FEATURE_PACK_3
    case FI_NetworkInfo_MobileNetworkCode:
    case FI_NetworkInfo_NetworkName:
    {
        TBuf8<KNetworkInfoStringLength> retString;
        if(pEngine && pEngine->GetNetworkInfoL(infoType, retString))
        {
            retString.ZeroTerminate();
            pEngine->ConvertCStringToUTF16L((const char*)(retString.Ptr()), retVal);
            success = 1;
        }
    }
    break;
    case FI_NetworkInfo_MobileCarrierCode:
    {
        TBuf8<KSubscriberInfoStringLength> retString;
        if(pEngine && pEngine->GetSubscriberInfoL(infoType, retString))
        {
            retString.ZeroTerminate();
            pEngine->ConvertCStringToUTF16L((const char*)(retString.Ptr()), retVal);
            success = 1;
        }
    }
    break;
#endif // FEATURE_PACK_3
        
    case FI_NetworkInfo_NetworkConnectionName:
    	{
    	TUint KInternetDestinationID = 4097;//Default Destination ID for Protected Destination "Internet"

    	TBuf16<KCommsDbSvrMaxColumnNameLength> accessPointName;     	
    	HBufC* defaultConnectionName = NULL;

    	RCmManager cmManager;
    	cmManager.OpenL();//Open Connection Manager Session

    	RCmDestination destination;  	
    	//Open Destination: "Internet", DestinationID 4097
    	destination = cmManager.DestinationL(KInternetDestinationID);       

    	/* Iterate through list of all Connection Methods for Internet. If Active Connection is found,
    	 * Break out of For Loop and return Method Name.
    	 * Store Name of Method with Highest Priority(Default) in case we don't find an Active
    	 * Connection. Check "success" to see which Method we have(Default or Active).
    	 */
    	
    	if (destination.ConnectionMethodCount() != 0)
    		{ 		
    		//Look for Active and Highest Priority Methods in Internet
    		RCmConnectionMethod connectionMethod;
    		for(int j = 0;j<destination.ConnectionMethodCount();j++)
    			{
    			connectionMethod = destination.ConnectionMethodL(j);
    			if (connectionMethod.GetBoolAttributeL(ECmConnected))
    				{
    				HBufC* activeConnectionName = connectionMethod.GetStringAttributeL(ECmName);//Name of Method(AP)    			  
    				accessPointName.Copy(*activeConnectionName);
    				accessPointName.ZeroTerminate();
    				success = 1;//Active Connection Obtained

    				delete activeConnectionName;//HBufC* ownership from GetStringAttributeL is passed back to us
    				activeConnectionName=NULL;
    				connectionMethod.Close();//We must close RCmConnectionMethod
    				break;//We got the Active Connection Method Name already.
    				}        			

    			if (destination.PriorityL(connectionMethod) == 1)//Get the Highest Priority Method
    				{
    				//Store Default Connection Name in case we do not find an Active Connection
    				defaultConnectionName = connectionMethod.GetStringAttributeL(ECmName);//Name of Method(AP)
    				}
    			connectionMethod.Close();//We must close RCmConnectionMethod
    			}    		
    		}
    	else //No Connection Methods Defined!
    		{
    		accessPointName.Zero();
    		accessPointName.ZeroTerminate();
    		success = 0;
    		}
    	destination.Close(); //Close RCmDestination

    	cmManager.Close();//Close Connection Manager Session

    	if ( (success != 1) && (defaultConnectionName) )//No Active Connection Found
    		{
    		accessPointName.Copy(*defaultConnectionName);
    		accessPointName.ZeroTerminate();
    		success = 2;//Default Connection Obtained
    		}

    	if (defaultConnectionName)
    		{
    		delete defaultConnectionName;//HBufC* ownership from GetStringAttributeL is passed back to us
    		defaultConnectionName = NULL;
    		}

    	retVal->SetUTF16String(retVal, accessPointName.Ptr());
    	
    	}
    break;
        
    default:
        success = -1;
        break;
    }
    
    return success;
}


	
/**
*
* @static
*/
short SI_Interface::SI_GetNetworkInfoString(struct MM_Object* aMMObj, 
                                            unsigned short infoType, 
                                            FI_ReturnString* retVal)
{
	SI_ENTER
    short retValue = 0;
    TRAPD_WRAPPER(retValue = DoSI_GetNetworkInfoStringL(aMMObj, infoType, retVal));
	SI_EXIT
    return retValue;
}

short DoSI_GetCallerInfoL(struct MM_Object* aMMObj, 
						 struct FI_ReturnString* aName, 
						 struct FI_ReturnString* aNumber)
{
short retValue = 4;//Default Value indicating No Call is in Progress

CPlatformFlashEngine* pEngine=aMMObj->pHostObj->iFIInterface->GetEngine();
if(pEngine)
	retValue = pEngine->GetCallerInfoL(aName,aNumber);

return retValue;
}

short SI_Interface::SI_GetCallerInfo(struct MM_Object* aMMObj, struct FI_ReturnString* aName, struct FI_ReturnString* aNumber)
{
	SI_ENTER
	
    short retValue = 4;//Default Value indicating No Call is in Progress
    TRAPD_WRAPPER(retValue = DoSI_GetCallerInfoL(aMMObj, aName, aNumber));
    
	SI_EXIT
    return retValue;
}


#ifndef USE_SYNC_MSO
/**
* The SI_SetPersistenData function is called by the Core Player to write data to a shared object.
* The request is serviced asynchronously.
*
* @return 0 failed to queue, -1 not supported, 1 queued successfully 
* @static 
*/
short SI_Interface::SI_SetPersistentData(struct MM_Object* aMMObj,
                           const char* aRootMovieGUID,
                           const char* aName,
                           const unsigned char* aData,
                           unsigned long aDataSize,
                           unsigned long aRequestId)
{
	SI_ENTER
	short retValue = -1;

 	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (pEngine && pEngine->GetShObjEngine())
	{
		TRAPD_WRAPPER(retValue=pEngine->GetShObjEngine()->AsyncSetPersistentDataL(aMMObj, aRootMovieGUID, aName,  aRequestId, aData, aDataSize, ETrue));
	}
	SI_EXIT
	return retValue;
}

/**
* The SI_GetPersistenDataSize function is called by the Core Player to find out the current size of a shared object.
* The request is serviced asynchronously.
*
* @return 0 failed to queue, -1 not supported, 1 queued successfully 
* @static 
*/
short SI_Interface::SI_GetPersistentDataSize(struct MM_Object* aMMObj,
                               const char* aRootMovieGUID,
                               const char* aName,
                               unsigned long aRequestId)
{
	SI_ENTER
	short retValue = -1;

 	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (pEngine && pEngine->GetShObjEngine())
	{
		TRAPD_WRAPPER(retValue=pEngine->GetShObjEngine()->AsyncGetPersistentDataSizeL(aMMObj, aRootMovieGUID, aName, aRequestId));
	}
	SI_EXIT
	return retValue;
}


/**
* The SI_GetPersistenData function is called by the Core Player to retrieve data from a shared object.
* The request is serviced asynchronously.
*
* @return 0 failed to queue, -1 not supported, 1 queued successfully 
* @static 
*/
short SI_Interface::SI_GetPersistentData(struct MM_Object* aMMObj,
                           const char* aRootMovieGUID,
                           const char* aName,
                           unsigned char* aData,
                           unsigned long aDataSize,
                           unsigned long aRequestId)
{
	SI_ENTER
	short retValue = -1;

 	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (pEngine && pEngine->GetShObjEngine())
	{
		TRAPD_WRAPPER(retValue=pEngine->GetShObjEngine()->AsyncGetPersistentDataL(aMMObj, aRootMovieGUID, aName, aRequestId, aData, aDataSize));
	}
	SI_EXIT
	return retValue;
}


/**
* The SI_GetMaxPersistenStorage function returns the maximum amount of space that shared objects can occupy.
* If this is exceeded, the oldest shared objects will be removed to free up space.
*
* @return 0 failed to determine max size, -1 not supported, 1 returned successfully 
* @static 
*/
short SI_Interface::SI_GetMaxPersistentStorage(struct MM_Object* aMMObj,
                                 const char* /*aRootMovieGUID*/,
                                 unsigned long* aSize)
{
	SI_ENTER
 	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();

	short retVal=SI_GenericError;
	if ((pEngine && pEngine->GetShObjEngine()))
	{
		*aSize = KMaxPersistentStoragePerMovie;	// 4k persistent_storage_per_movie
		retVal=SI_NoError;
	}
	SI_EXIT
	return retVal;
		
}

/**
* The SI_CancelSharedObjectRequest function is called by the player to cancel a
* previously requested shared object request that has not yet completed.
*
* @static 
*/
void SI_Interface::SI_CancelSharedObjectRequest(struct MM_Object* aMMObj, unsigned long aRequestId)
{
	SI_ENTER
	
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (pEngine && pEngine->GetShObjEngine())
	{
		pEngine->GetShObjEngine()->CancelPersistentDataRequest(aMMObj, aRequestId);
	}
	SI_EXIT
}

#else

short SI_Interface::SI_SetPersistentDataSync(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, const unsigned char* aData, unsigned long aDataSize)
    {
    SI_ENTER
    short err(0);
    
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
    if (pEngine)
        {
        CSharedObjectHandler* shObjHandler = pEngine->GetShObjHandler();
        if (shObjHandler)
            {
            if(aData)
                {
                err = shObjHandler->StoreData(aRootMovieGUID, aName, aData, aDataSize);
                }
            else
                {
                err = shObjHandler->RemoveData(aRootMovieGUID, aName);
                }                
            }
        }
    SI_EXIT
    if (KErrNone == err)
    	return SI_NoError;
    else
    	return SI_GenericError;
   }

short SI_Interface::SI_GetPersistentDataSizeSync(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, unsigned long* dataSize)
    {
    SI_ENTER
    short err(KErrNone);
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
    if (pEngine)
        {
        CSharedObjectHandler* shObjHandler = pEngine->GetShObjHandler();
        if (shObjHandler)
            {
            
            err = shObjHandler->StorageDataSize(aRootMovieGUID, aName, *dataSize);
            }
        }    
    SI_EXIT
    /* NOTE: When we return SI_GenericError in case of KErrNotFound,
     * core player does not proceed with other MSO Write/Read operations.
     */
    if (KErrNone == err || KErrNotFound == err)
    	return SI_NoError;
    else
    	return SI_GenericError;
   }

short SI_Interface::SI_GetPersistentDataSync(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, unsigned char* aData, unsigned long* aDataSize)
    {
    SI_ENTER
    short err(0);

    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
    if (pEngine)
        {
        CSharedObjectHandler* shObjHandler = pEngine->GetShObjHandler();
        if (shObjHandler)
            err = shObjHandler->RetrieveData(aRootMovieGUID, aName, aData, *aDataSize);
        }
    SI_EXIT
    if (KErrNone == err)
    	return SI_NoError;
    else
    	return SI_GenericError;
   }

short SI_Interface::SI_GetMaxPersistentStorage(struct MM_Object* aMMObj, const char* /*aRootMovieGUID*/, unsigned long* aSize)
    {
    SI_ENTER
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
    if (pEngine)
        {
        CSharedObjectHandler* shObjHandler = pEngine->GetShObjHandler();
        if (shObjHandler)
            shObjHandler->GetMaxStorageSize(*aSize);
        }
    SI_EXIT
    return SI_NoError;
    }
#endif //USE_SYNC_MSO

#ifdef FEATURE_XML_SOCKET

short SI_Interface::SI_OpenSocket(struct MM_Object* aMMObj, struct FI_SocketConnectionInfo* aConnInfo, const struct FI_PlayerEventInfo* /*aEventInfo*/)
{
	SI_ENTER
	
	short retVal = 0;
//	if (aEventInfo->eventCode != FI_EventNoEvent)
	{
		CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
		if (pEngine && pEngine->GetSocketEngine())
		{
			TRAPD_WRAPPER(retVal = pEngine->GetSocketEngine()->OpenNewSocketL(aConnInfo));
		}
	}
	
	SI_EXIT

	return retVal;
}

short SI_Interface::SI_CloseSocket(struct MM_Object* aMMObj, unsigned long aSocketId)
{
	SI_ENTER
	
	short retVal = 0;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (pEngine && pEngine->GetSocketEngine())
	{
		retVal = pEngine->GetSocketEngine()->CloseSocket(aSocketId);
		
	}

	SI_EXIT

	return retVal;
}

short SI_Interface::SI_SocketSend(struct MM_Object* aMMObj, unsigned long aSocketId, struct FI_NetworkBufferInfo* aDataBuffer)
{
	SI_ENTER
	
	short retVal = 0;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (pEngine && pEngine->GetSocketEngine())
	{
		TRAPD_WRAPPER(retVal=pEngine->GetSocketEngine()->SendL(aSocketId, aDataBuffer));
	}

	SI_EXIT

	return retVal;
}
#endif //FEATURE_XML_SOCKET

#ifdef FEATURE_BG_TRANSPARENCY
short SI_Interface::SI_UpdateBackgroundRegion(struct MM_Object* aMMObj, struct FI_Rect *rects, unsigned short numRects)
{
	SI_ENTER
	
	short retVal = 0;
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (pEngine)
	{
		retVal = pEngine->RedrawBackground(rects, numRects);
	}

	SI_EXIT

	return retVal;
}
#endif //FEATURE_BG_TRANSPARENCY

///file access SI functions///////////////////
unsigned short SI_Interface::SI_OpenFile(MM_Object* aMMObj, const char * fileName, FI_FileMode /*mode*/, void ** fileHandle)
{
	SI_ENTER
	
	unsigned short retVal = 0;
	
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (pEngine)
	{
		TRAP_IGNORE (*fileHandle = CFileRequestObject::OpenCreateFileL(aMMObj, fileName));
		retVal = *fileHandle != NULL;
	}

	SI_EXIT

	return retVal;
}

#ifdef __LOG_SI_CALLS__
void SI_Interface::SI_CloseFile (MM_Object* aMMObj, void * fileHandle)
#else
void SI_Interface::SI_CloseFile (MM_Object* /*aMMObj*/, void * fileHandle)
#endif
{
	SI_ENTER
	CFileRequestObject* pObj = static_cast<CFileRequestObject*> (fileHandle);
	
	if(pObj)
	  delete pObj ;
	
	SI_EXIT
}
#ifdef __LOG_SI_CALLS__
unsigned long SI_Interface::SI_GetFileSize(MM_Object* aMMObj, void * fileHandle)
#else
unsigned long SI_Interface::SI_GetFileSize(MM_Object* /*aMMObj*/, void * fileHandle)
#endif
{
	SI_ENTER
	int size = -1;
	
	CFileRequestObject* pObj = static_cast<CFileRequestObject*> (fileHandle);
	
	if(pObj) size = pObj->FileSize();
	
	if (size < 0)
		size = 0;
	
	SI_EXIT
	
	return size;
}


unsigned long SI_Interface::SI_ReadFile (MM_Object* aMMObj, void * fileHandle, unsigned char * buffer, unsigned long bufferSize)
{
	SI_ENTER
	unsigned long read = 0;
	
	CFileRequestObject* pObj = static_cast<CFileRequestObject*> (fileHandle);
	
	if(pObj) read= pObj->ReadFile(buffer, bufferSize);
	
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	pEngine->ExtendBackLightWhileSeekingLocalFlv();
	
	SI_EXIT
	return read;
}


unsigned short SI_Interface::SI_SeekFile (MM_Object* aMMObj, void * fileHandle, long offset, unsigned short origin)
{
	SI_ENTER

	unsigned short ok = 1;

	TSeek mode = ESeekStart;
	if ( origin == FI_FileSeekStart )
		mode = ESeekStart;
	else if ( origin == FI_FileSeekCurrent )
		mode = ESeekCurrent;
	else if ( origin == FI_FileSeekEnd )
		mode = ESeekEnd;
	else
		ok = 0;

	if (ok)
	{
		TInt error = KErrGeneral;
		
		CFileRequestObject* pObj = static_cast<CFileRequestObject*> (fileHandle);
		
		if(pObj) error = pObj->SeekFile(mode, offset);
		
		ok = (error == KErrNone);
	}
	
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	pEngine->ExtendBackLightWhileSeekingLocalFlv();

	SI_EXIT
	return ok;
}


#ifdef FEATURE_EXTERNAL_API
/*
 * External API functions
 */
 int SI_Interface::SI_EvaluateJScript (struct MM_Object* aMMObj, 
									   const char* expression)
 {
 	SI_ENTER
	
	int retVal = 0;
	
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (pEngine)
	{
		if( pEngine->GetScriptHandler() && pEngine->GetScriptHandler()->GetPluginScriptHandler())
		{
			retVal = pEngine->GetScriptHandler()->GetPluginScriptHandler()->EvaluateJavaScript(expression);
		}
	}
	
	SI_EXIT

	return retVal;
 }

 int SI_Interface::SI_DoFlashCall (struct MM_Object *aMMObj, 
											const char* request)
 {
 	SI_ENTER
	
	int retVal = 0;
	
	CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if (pEngine)
	{
		if( pEngine->GetScriptHandler() && pEngine->GetScriptHandler()->GetPluginScriptHandler())
		{
			TRAP_IGNORE (retVal = pEngine->GetScriptHandler()->GetPluginScriptHandler()->CallMethodL(request));
		}
			
	}
	
	SI_EXIT

	return retVal;
	
 }
 
void FillBufferToSendToCore( void** useMemFromCore, TDesC8& aMovie, struct FI_AllocatorWrapper* hostAllocWrapper)
{
	TInt len = aMovie.Length() + 1;
	*useMemFromCore = hostAllocWrapper->pMemAllocator(hostAllocWrapper->pContext,len);	
	if(*useMemFromCore)
	{
		memcpy(*useMemFromCore,(void*)aMovie.Ptr(),len);
	}
		
}
_LIT8( KDocument, "document.");

void* DoSI_GetDOMObjectIDL (struct MM_Object *aMMObj, 
										struct FI_AllocatorWrapper * allocWrapper)
    {
    
    void* retVal = NULL;
    CPlatformFlashEngine* pEngine = aMMObj->pHostObj->iFIInterface->GetEngine();
	
	if (pEngine)
	{
		if( pEngine->GetScriptHandler() && pEngine->GetScriptHandler()->GetPluginScriptHandler())
		{
		    TDesC8& movieId = pEngine->GetMovieId();
// prepending "document." to the DOM id as is required by the S60 browser from S605.0 onwards.		    
		    HBufC8* domObjectId = HBufC8::NewLC(movieId.Length()+ 1 + KDocument.iTypeLength);
		    TPtr8 domPtr = domObjectId->Des();
		    //domPtr.Copy(KDocument);
		    domPtr.Append(movieId);
		    domPtr.ZeroTerminate();
			FillBufferToSendToCore(&retVal,domPtr, allocWrapper);
			CleanupStack::PopAndDestroy(domObjectId);
		}
		
	}
	return retVal;
    
    }
 void*	SI_Interface::SI_GetDOMObjectID (struct MM_Object *aMMObj, 
										struct FI_AllocatorWrapper * allocWrapper)
 {
 	SI_ENTER
	
	void* retVal = NULL;
	
	TRAPD(err, retVal = DoSI_GetDOMObjectIDL(aMMObj, allocWrapper));
	 

	SI_EXIT
	
	if (err!=KErrNone)
	{ 
			return NULL;
	}

	return retVal;
 }

#endif //FEATURE_EXTERNAL_API
#ifdef FEATURE_VIDEO_DLL

IVideoDecompressorInstance *
SI_Interface::SI_NativeVideoCreateDecompressor( MM_Object* aMMObj,void * aAllocator,  U32 codecType )
{
	SI_ENTER
	CPlatformFlashEngine* engine = aMMObj->pHostObj->iFIInterface->GetEngine();
	
    if(!engine)
    {
        return NULL;
    }
	IVideoDecompressorInstance* instance = NULL;

	engine->ExtendBacklight(10);//Extend backlight before the first time Playstep is called

    instance = engine->GetNativeVideoDecompressor(aMMObj, aAllocator, codecType);
	
	#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
	//FLV case so UI should be able to go full screen
	
	engine->FLVideoStarted(codecType); 
	#endif

//	engine->SetSwfHasVideo(ETrue);
	
	SI_EXIT
	
	return instance;
}

void 
SI_Interface::SI_NativeVideoDestroyDecompressor( MM_Object * aMMObj, IVideoDecompressorInstance * decompressorInstance )
{
	SI_ENTER
	CPlatformFlashEngine *					engine = aMMObj->pHostObj->iFIInterface->GetEngine();

    if(!engine)
    {
        return;
    }

    engine->DestroyVideoDecompressor(aMMObj, decompressorInstance);
#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
    engine->FLVideoFinished(); 
#endif
//    engine->SetSwfHasVideo(EFalse);

	SI_EXIT
}

#endif // FEATURE_VIDEO_DLL

#ifdef FEATURE_FLV_MMC

int SI_Interface::SI_SeekURLStreamFLV(struct MM_Object* aMMObj,
							    	  unsigned short streamId,
									  unsigned int seekTime,
									  struct FI_NetworkBufferInfo* pNetBuf)
{
	
	CPlatformFlashEngine * engine = aMMObj->pHostObj->iFIInterface->GetEngine();
	if(engine && engine->IsBrowserPluginActive())
	{
		return engine->SeekURLStreamFLV(streamId, seekTime, pNetBuf);
	}
	else
	{
		CUrlRequestInfo* urlInfo=(CUrlRequestInfo*) (engine->GetHttpEngine() ? engine->GetHttpEngine()->GetRequestObject(engine->GetMMObject(),streamId) : NULL);
			
		if(urlInfo)
		{
			return urlInfo->SeekURLStreamFLV(seekTime, pNetBuf);
		}
	}	
	
	return EFalse;	
}

unsigned int SI_Interface::SI_FLVGetBytesLoaded(struct MM_Object* aMMObj,unsigned short streamId)
{
	SI_ENTER
	
	unsigned int byteval=0;
	
	CPlatformFlashEngine * engine = aMMObj->pHostObj->iFIInterface->GetEngine();
	
	if(engine && engine->IsBrowserPluginActive())
	{
		byteval = engine->GetBytesLoaded(streamId);
	}
	
	else
	{
		CUrlRequestInfo* urlInfo=(CUrlRequestInfo*) (engine->GetHttpEngine() ? engine->GetHttpEngine()->GetRequestObject(engine->GetMMObject(),streamId) : NULL);
			
		if(urlInfo)
			{
			byteval = urlInfo->GetBytes();
			}	
	}
	
	return byteval;
	
	SI_EXIT
}

unsigned int SI_Interface::SI_FLVGetTotalBytes(struct MM_Object* aMMObj, unsigned short streamId)
{
    SI_ENTER
    
    unsigned int byteval=0;
    
    CPlatformFlashEngine * engine = aMMObj->pHostObj->iFIInterface->GetEngine();
    if( engine )
    {
        if(engine->IsBrowserPluginActive())
        {
            byteval = engine->GetTotalBytes(streamId);    
        }
        else
        {
            CUrlRequestInfo* urlInfo=(CUrlRequestInfo*) (engine->GetHttpEngine() ? engine->GetHttpEngine()->GetRequestObject(engine->GetMMObject(),streamId) : NULL);
            
            if(urlInfo)
            {
                byteval = urlInfo->GetTotalBytes();
            }    
        }
    }    
    
    return byteval;
    
    SI_EXIT    
}

#endif //FEATURE_FLV_MMC


// Local connection SI calls
//shashank
#ifdef FEATURE_LOCAL_CONNECTION

/*
 * SI_Interface::SI_CreateSharedMemory(struct MM_Object* aMMObj,unsigned long aSharedMemSize )
 * creates a shared memory which are used by two sfw files to communicate with each other.
 * 
 * params : 
 *          struct MM_Object* aMMObj - PlatformPlayer
 *          unsigned long aSharedMemSize - Size of the memory that must be created.
 */
#ifdef __LOG_SI_CALLS__
void* SI_Interface::SI_CreateSharedMemory(struct MM_Object* aMMObj, const char* aName, const char* aLockname,
                                            unsigned long aSharedMemSize,unsigned long* aHandle)
#else
void* SI_Interface::SI_CreateSharedMemory(struct MM_Object* aMMObj,
											const char* aName,
											const char* aLockname,
											unsigned long aSharedMemSize,
											unsigned long* aHandle)
#endif
{
    SI_ENTER

#ifndef SHARED_MEMORY_INTER_PROCESS
    CHostEngineGlobals* globals = static_cast<CHostEngineGlobals*>(Dll::Tls());
    CSharedMemory* iSharedMemory = globals->SharedMemoryObject();
    TAny* sharedMemory = NULL;
    aHandle = NULL;
    if( iSharedMemory )
    {
        TRAP_IGNORE( sharedMemory = iSharedMemory->GetSharedMemoryL(aSharedMemSize) );
        /*
         * This check is not required as the correct values are returned
         * 
         * if( err == KErrNone )
         *      return sharedMemory;
         * else
         *      return NULL;
         */
        
    }
    SI_EXIT
    return sharedMemory;
#else //ifdef SHARED_MEMORY_INTER_PROCESS
    CPlatformFlashEngine* engine = aMMObj->pHostObj->iFIInterface->GetEngine();
    //ask the engine to create shared memory with the passed name
    //and the lock name.
    //return null if the name or lockname is null
    CSharedMemoryManager* sharedMemMgr = NULL;
    TRAPD( err, sharedMemMgr = engine->GetSharedMemoryManagerL());
    if( err == KErrNone )
    {
	    CSharedMemoryClient* shMemory = NULL;
	    TRAPD(shMemErr, shMemory = sharedMemMgr->CreateSharedMemoryL(aName,aLockname,aSharedMemSize));
	    if( shMemErr == KErrNone )
	    {
		    *aHandle = shMemory->LockHandle();
		    SI_EXIT
		    return shMemory->BaseAddress();
		  }
	  }
	  return NULL;
#endif //ifndef SHARED_MEMORY_INTER_PROCESS
	  //return NULL;
}

/*
 * SI_Interface::SI_DestroySharedMemory(struct MM_Object* aMMObj, void* aPtrSharedMemory )
 * Frees up the shared memory allocated.
 * 
 * params : 
 *          struct MM_Object* aMMObj - PlatformPlayer
 *          void* aPtrSharedMemory - Pointer to the shared memory that will be freed.
 */
 //FL4.0 : TODO
//void SI_Interface::SI_DestroySharedMemory(struct MM_Object* /* aMMObj */, void* /*aPtrSharedMemory*/ )
void SI_Interface::SI_DestroySharedMemory(struct MM_Object*  aMMObj , unsigned long aHandle)
{
#ifndef SHARED_MEMORY_INTER_PROCESS
    CHostEngineGlobals* globals = static_cast<CHostEngineGlobals*>(Dll::Tls());
    CSharedMemory* iSharedMemory = globals->SharedMemoryObject();
    if( iSharedMemory )
    {
        iSharedMemory->FreeSharedMemory();
    }
#else //ifdef SHARED_MEMORY_INTER_PROCESS
    CPlatformFlashEngine *engine = aMMObj->pHostObj->iFIInterface->GetEngine();
    CSharedMemoryManager* sharedMemMgr = NULL;
    TRAPD( err, sharedMemMgr = engine->GetSharedMemoryManagerL());
    if( err == KErrNone )
    	sharedMemMgr->DeleteSharedMemory(aHandle);
#endif //ifndef SHARED_MEMORY_INTER_PROCESS
}

short SI_Interface::SI_LockSharedMemory(struct MM_Object* pMMObj,
        unsigned long aHandle)
    {
    CPlatformFlashEngine *engine = pMMObj->pHostObj->iFIInterface->GetEngine();
    CSharedMemoryManager* sharedMemMgr = NULL;
    TRAPD( err, sharedMemMgr = engine->GetSharedMemoryManagerL());
    if( err == KErrNone )
    	return sharedMemMgr->Lock(aHandle);
    else
    	return SI_GenericError;
    }

short SI_Interface::SI_UnLockSharedMemory(struct MM_Object* pMMObj,
        unsigned long aHandle)
    {
    CPlatformFlashEngine *engine = pMMObj->pHostObj->iFIInterface->GetEngine();
    CSharedMemoryManager* sharedMemMgr = NULL;
    TRAPD( err, sharedMemMgr = engine->GetSharedMemoryManagerL());
    if( err == KErrNone )
    	return sharedMemMgr->Unlock(aHandle);
    else
    	return SI_GenericError;
}

#endif // FEATURE_LOCAL_CONNECTION
#ifdef FEATURE_EXTENSION

PluginInfo* SI_Interface::SI_GetPluginInfo(struct MM_Object* aMMObj,struct ExtensionInterface* pEI)
{

    SI_ENTER
    CPlatformFlashEngine *engine = aMMObj->pHostObj->iFIInterface->GetEngine();
    PluginInfo* pInfo = NULL;
    if(engine)
    {
        TRAPD(err, pInfo = engine->GetPluginInfoL(aMMObj,pEI)); 
        if(err != KErrNone)
            {
            return NULL;
    }
        }
    
    return pInfo;
    
    SI_EXIT
}
#endif

#ifdef FEATURE_TESTAUTOMATION
signed long SI_Interface::SI_WriteFile (MM_Object* pMMObj, const char* filePath, 
        char* platformFileURL, int /*platformFileURLSize*/, 
        const char* pBuf, FI_FileMode mode, int noOfChars)
{
	CPlatformFlashEngine* pEngine = pMMObj->pHostObj->iFIInterface->GetEngine();
    //create/open the file specified in the filepath argument [check the FileMode value too]
    //pBuf contains the data to be written and noOfChars denotes the count of characters
    //to be written to the file..
    
    TFileMode fileMode(EFileRead);
        
    if(mode!=FI_ReadFileMode) //meaning FI_ReadWriteFileMode, FI_AppendFileMode, FI_WriteFileMode
        fileMode = EFileWrite;

    HBufC *filePathBuf = pEngine->CreateEpocString(filePath, FlashStrLen(filePath));
    if (!filePathBuf)
    	return SI_GenericError;

    TPtr filePathPtr = filePathBuf->Des();
    FileInfo fileInfo(filePathPtr,TPtr8((unsigned char *)pBuf,noOfChars,noOfChars),fileMode);
    
    
    TInt err = AutomationHelper::WriteToFile(fileInfo);
    
    FlashStrCpy(platformFileURL, filePath); //static_cast to be used instead of c-style cast ??
    FlashStrLen(filePath);

    if(KErrNone==err)
        return SI_NoError;
    else
        return SI_GenericError;
    
}



signed long SI_Interface::SI_GenerateMouseEvent(MM_Object* /*pMMObj*/, FI_TestAutomationMouseEventType action, int screenX, int screenY, int /*delta*/)
{
    TRawEvent event;
    
    switch(action)
        {
        case FI_TestAutomationMouseDown:
            event.Set(TRawEvent::EButton1Down,screenX, screenY);
            break;
        case FI_TestAutomationMouseUp:
            event.Set(TRawEvent::EButton1Up,screenX, screenY);
            break;
        /*case FI_TestAutomationRightMouseDown:
            event.Set(TRawEvent::EButton2Down,screenX, screenY);
            break;
        case FI_TestAutomationRightMouseUp:
            event.Set(TRawEvent::EButton2Up,screenX, screenY);
            break;*/        
        case FI_TestAutomationMouseWheel:
        case FI_TestAutomationMouseMove:
            event.Set(TRawEvent::EPointerMove,screenX, screenY);
            break;
        default:
            return SI_GenericError;
        }
    
    //CCoeEnv::Static()->RootWin().SimulatePointerEvent(event);
    AutomationHelper::GenerateMouseEvent(event);
    
    
    return SI_NoError;
}
signed long SI_Interface::SI_GenerateKeyEvent(MM_Object* /*aMMObj*/, FI_TestAutomationKeyEventType action, unsigned int keyCode)
{
	SI_ENTER    
    TEventCode code;
    
    switch(action)
        {
        case FI_TestAutomationKeyDown:
            code = EEventKeyDown;
            break;
        case FI_TestAutomationKeyUp:
            code = EEventKeyUp;
            break;
        default:
            return SI_GenericError;
        }
    TKeyEvent event = {keyCode};
        //TODO : should we fill scancode too??
    
    //TRAPD(err,CCoeEnv::Static()->SimulateKeyEventL(event, code));
    TInt err = AutomationHelper::GenerateKeyEvent(event, code);

    if(action==FI_TestAutomationKeyDown)
        {
        //symbian specific....
        code = EEventKey;
        AutomationHelper::GenerateKeyEvent(event, code);
        }
    
    SI_EXIT
    if(KErrNone==err)
        return SI_NoError;
    
    return SI_GenericError;
}
signed long SI_Interface::SI_GetAutomationConfigurationContent( MM_Object* /*aMMObj*/, char** buffer, int* bufferSize, FI_ConfigurationFileType /*type*/)
{
	SI_ENTER
	if (buffer == NULL)
	{
		*bufferSize = 255;
		return SI_NoError;
	}
	else
	{
		FlashStrCpy(*buffer, "AutomationEnable = 1\r\nTraceOutputFileEnable = 1\r\nStageCaptureDirectory = C:\\data\\Others\\images \r\nTraceOutputFileName = C:\\data\\Others\\dctslog.txt\r\n");
		return SI_NoError;
	}
	SI_EXIT
	
}
#endif//FEATURE_TESTAUTOMATION

void SI_Interface::SI_GetDisplayRectCoord(struct MM_Object */*aMMObj*/, struct FI_Rect *rect)
{
	SI_ENTER
	rect->xmin = 0;
	rect->ymin = 0;
	rect->xmax = 320;
	rect->ymax = 640;
	SI_EXIT
}


#ifdef FEATURE_AMCEXTENSIONS


TBool Promptmsg(AMCLSID);

TBool Promptmsg(AMCLSID /*classID*/)
{
    TBufC<128> KResFile(_L("flash2engine.rss") );
    TInt resOffset = CPlatformFlashEngine::LoadPluginResourceL(KResFile);
    HBufC *accessMsg;
    
    accessMsg = StringLoader::LoadLC(R_FLASH_GEOLOC_ACCESS);
     
    TBool res=CPlatformFlashEngine::QueryAllowDenyL(*accessMsg);
    CleanupStack::PopAndDestroy(accessMsg);
    CEikonEnv::Static()->DeleteResourceFile(resOffset);
    
    return res;
}
AMEXTERN AMErr PlatformExtensionCreateInstance(IAMAllocator* pAllocator, AMCLSID classID, AMIID iid, void** ppi)
{
    AMErr err = kAMErrNone;
    if (!ppi)
    {
        err = kAMErrBadParam;
        goto exit_gracefully;
    }
    *ppi = NULL;
    switch ( classID )
    {
        case AMCLSID_ACCELEROMETER:
            if (AMIID_IAMACCELEROMETER != iid)
            {
                err = kAMErrInterfaceNotSupported;
                goto exit_gracefully;
            }
    
            IAMAccelerometer * pAccel = NULL;

            err = AccelerometerImpl::Create(pAllocator, &pAccel);
                    if (err != kAMErrNone)
                        goto exit_gracefully;
            
            IAMACCELEROMETER_AddRef(pAccel);
            *ppi = pAccel;
            break;
        
        case AMCLSID_GPS:
                       CHostEngineGlobals* globals = CHostEngineGlobals::CreateHostEngineGlobalsL();
                        TBool result=TRUE;
                       
                       if (AMIID_IAMGPS!= iid)
                       {
                           err = kAMErrInterfaceNotSupported;
                           goto exit_gracefully;
                       }

                      GeolocationAccess geolocationAccess = globals->GeolocationAccessStatus();

                      IAMGPS * pGps = NULL;
                       if (geolocationAccess == ENotChecked)
                           {
                           result=Promptmsg(classID);
                           }

                       if (!result || geolocationAccess == EDeny )
                         {
                           globals->GeolocationAccessStatus(EDeny);
                            err = kAMErrPermissionDenied;
                            goto exit_gracefully;
                         }
                   globals->GeolocationAccessStatus(EAllow);

                       err = LocationImpl::Create(pAllocator, &pGps);
                       if (err != kAMErrNone)
                       {
                        goto exit_gracefully;
                       }
                       IAMGPS_AddRef(pGps);
                       *ppi = pGps;
                       break;
       default:
       {
            err = kAMErrComponentNotSupported;
            goto exit_gracefully;
       }
    }

exit_gracefully:
     return err;
}

extern "C" AMErr StaticPlatformCreateInstance(IAMAllocator* piAllocator, AMCLSID classID, AMIID iid, void** ppi)
{
    AMErr err = kAMErrNone;
    
    
    
     if ( ! ppi )
    {
        err = kAMErrBadParam;
        goto exit_gracefully;
    }

    *ppi = NULL;

    switch ( classID )
    {
    case AMCLSID_ACTIONSCRIPTEXTENSIONBROKER:
        {
         err = kAMErrInterfaceNotSupported;
        }
         break;
    default:
        {
            err = PlatformExtensionCreateInstance(piAllocator, classID, iid, ppi);
            goto exit_gracefully;
        }
       // break;
    }

exit_gracefully:
    return err;
}



int SI_Interface::SI_GetExtensionPlatformInfo(struct MM_Object */*aMMObj*/, struct ExtensionPlatformInfo* info)
    {
    SI_ENTER
    
    info->pfnCreateInstance = (PlatformCreateInstance*)StaticPlatformCreateInstance;
    info->pPlatformGlobal = NULL;
    return TRUE;
    
    SI_EXIT
    }

int SI_Interface::SI_GetSecurityContextPermissions(struct MM_Object *aMMObj, ExtensionSecurityPermissions *permissions)
    {
    SI_ENTER
    CPlatformFlashEngine *engine = aMMObj->pHostObj->iFIInterface->GetEngine();
    if(!engine->AllowAMCExtension())
        permissions->extensionPermissions = 0;
    else
        // Set permissions for GPS and Acclerometer read and write.
        // Check IAMSecurityContext.h for details
        permissions->extensionPermissions = 0x00001000 |
                                            0x00002000 |
                                            0x00004000 |
                                            0x00008000;
    

    permissions->oemPermissions = 0;
    return TRUE;
    SI_EXIT
    }

#endif // FEATURE_AMCEXTENSIONS

//#ifdef FEATURE_MULTITOUCH
void SI_Interface::SI_GetSupportedGestures(struct MM_Object* /*pMMObj*/, FI_GestureID /*glist*/[] , int */*len*/)
    {
    return;
    }
//#endif

#ifdef FEATURE_COMPUTE_STACK
/**
*
* @static SI_ComputeStackBase(struct MM_Object* aMMObj)
* Provide stack base address to player for checking stack overflow.
* If not implemented then return 0 else return the base value
* In symbian ot is observed that stack is growing bottom down
*/

    void*  SI_Interface::SI_ComputeStackBase(struct MM_Object* /*aMMObj*/)
            {
                SI_ENTER
                RThread curThread;// object will initialize to current active thread.
                TThreadStackInfo thrdInfo; 
                
                if( KErrNone == curThread.StackInfo(thrdInfo))//get stack info for thread
                    {
                        //RDebug::Printf("Stack base=%d, stack limit=%d",thrdInfo.iBase,thrdInfo.iLimit);
                        SI_EXIT
                        return reinterpret_cast<void*>(thrdInfo.iLimit); /* return stack limit address the name
                        is misleading because of the way the negative growing stack is handled */
                    }
                else
                    {
                        //RDebug::Printf("StackInfo returned error: return 0");
                        SI_EXIT
                        return reinterpret_cast<void*>(0x0000); //return 0
                    }
            }
#endif
    
