/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that is used to 
*				 access phone battery information.
*
*/


#ifdef FEATURE_PACK_3

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "symbian_batteryinfo.h"


const TInt8 KBatterySignalBarMaxLevel = 7; //assuming max 7 bars for battery signal level


CPhoneBatteryInfo* CPhoneBatteryInfo::NewL()
{
	CPhoneBatteryInfo* self=new(ELeave) CPhoneBatteryInfo();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

CPhoneBatteryInfo::CPhoneBatteryInfo(): 
	CActive(CActive::EPriorityLow)
   ,iTelephony(NULL)
   ,iBatteryInfoV1Pckg( iBatteryInfoV1 )
{

}

void CPhoneBatteryInfo::ListenL()
{
	//Check this condition before calling. 
	//if (IsActive())
	//	return;
	
	
	if(!iTelephony)
	{
		iTelephony = CTelephony::NewL();	
	}
	
	if(iTelephony)	
	{
		iTelephony->GetBatteryInfo(iStatus, iBatteryInfoV1Pckg);
		
		if (!IsActive())
		{
			SetActive();
			//Wait for Async CTelephony Request to Complete.
			iWait->Start();
		}		
		
	}
	
}

void CPhoneBatteryInfo::ConstructL()
{
	CActiveScheduler::Add(this);
	
	//ListenL();//Not Required while creating the Object.
				//Will Listen once actual request comes.
	
	iWait = new (ELeave) CActiveSchedulerWait;
}


CPhoneBatteryInfo::~CPhoneBatteryInfo()
{
	Cancel();
	
	if(iTelephony)
	{
		delete iTelephony;
		iTelephony = NULL;
	}
	if(iWait)
		{	
		if (iWait->IsStarted())
			{
			iWait->AsyncStop();
			}
		delete iWait;
		iWait = NULL;
		}
}
					

void CPhoneBatteryInfo::DoCancel()
{
	iTelephony->CancelAsync(CTelephony::EGetBatteryInfoCancel);	
	//Cancel the CActiveSchedulerWait Loop also.
	if (iWait->IsStarted())
		iWait->AsyncStop();
}
	

void CPhoneBatteryInfo::RunL()
{
	if(iStatus==KErrNone)
       {
       		if(iTelephony)
			{
				delete iTelephony;
				iTelephony = NULL;
			}
       }
		//Async Request Done.If Error Occurred return value is 0.
		if (iWait->IsStarted())
			{
			iWait->AsyncStop();
			}
}
					

TUint CPhoneBatteryInfo::GetBatteryChargeLevel()
{
	TUint retVal = 0;
	
	TRAPD( err, retVal = GetBatteryChargeLevelL());
  	err= (err == KErrNone)? 0:1; //remove warning
	return retVal;
}

TUint CPhoneBatteryInfo::GetBatteryChargeLevelL()
{
	TUint retVal= 0;
	//ListenL() is required Here because it might have been a long
	//time since Values have been updated!( if CPhoneBatteryInfo Instance 
	//was created long back, it might no longer even be Active!)
	if(!IsActive())
		{
		ListenL() ;
		}
	
	
	if(iStatus==KErrNone)
       {
			retVal =	iBatteryInfoV1.iChargeLevel; //this value in %
			
			retVal *= KBatterySignalBarMaxLevel;
			
			//ats ID: 77971 -> batt level was not coming correct
			//due to values being INT, the roinding was not being done.
			TInt roundingneeded=retVal % 100;
			if (roundingneeded>=50)
			    roundingneeded=1;
			else
			    roundingneeded=0;
			        
			
			retVal = retVal /100;//devide by percentage to get actual bar val
			retVal += roundingneeded;
       }
       
    //ListenL()  ; //No Use of it Here. Moved to Top of function.
     
    return retVal;
}

TBool CPhoneBatteryInfo::GetBatteryChargeStatus()
{
	TBool retVal = 0;
	
	TRAPD( err, retVal = GetBatteryChargeLevelL());
	if ( err == KErrNone)
	{
		//avoid warning
	}
	return retVal;	
}

TBool CPhoneBatteryInfo::GetBatteryChargeStatusL()
{
	TBool retVal= EFalse;
	
	if(iStatus==KErrNone)
       {
			if(CTelephony::EBatteryConnectedButExternallyPowered == 
					iBatteryInfoV1.iStatus)
				retVal = ETrue;	
       }
       
    ListenL()  ;
     
    return retVal;
}


#endif //FEATURE_PACK_3

