/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that is used to 
*				 access phone battery charging information.
*
*/


#ifdef FEATURE_PACK_3

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "symbian_charginginfo.h"

CPhoneChargingInfo* CPhoneChargingInfo::NewL()
{
	CPhoneChargingInfo* self=new(ELeave) CPhoneChargingInfo();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

CPhoneChargingInfo::CPhoneChargingInfo(): 
	CActive(CActive::EPriorityLow)
   ,iTelephony(NULL)
   ,iIndicatorV1Pckg( iIndicatorV1 )
{

}

void CPhoneChargingInfo::ListenL()
{
	//check this condition before calling
	//if (IsActive())
	//	return;
	
	if(!iTelephony)
	{
		iTelephony = CTelephony::NewL();	
	}
	
	if(iTelephony)	
	{
		iTelephony->GetIndicator(iStatus, iIndicatorV1Pckg);
		
		if (!IsActive())
		{
			SetActive();
			//Wait for Async CTelephony Request to Complete.
			iWait->Start();
		}
	}
	
}

void CPhoneChargingInfo::ConstructL()
{
	CActiveScheduler::Add(this);

	//ListenL();//Not Required while creating the Object.
					//Will Listen once actual request comes.
		
	iWait = new (ELeave) CActiveSchedulerWait;
}


CPhoneChargingInfo::~CPhoneChargingInfo()
{
	Cancel();
	
	if(iTelephony)
	{
		delete iTelephony;
		iTelephony = NULL;
	}
	if(iWait)
	{
		if(iWait->IsStarted())
			iWait->AsyncStop();
		delete iWait;
	}
	
}
					

void CPhoneChargingInfo::DoCancel()
{
	iTelephony->CancelAsync(CTelephony::EGetIndicatorCancel);
	//Cancel the Async Wait Loop also
	if (iWait->IsStarted())
	{
		iWait->AsyncStop();
	}
}
	

void CPhoneChargingInfo::RunL()
{
	if(iStatus==KErrNone)
       {	
       		if(iTelephony)
			{
				delete iTelephony;
				iTelephony = NULL;
			}
       }
	//Async Request is done .
	if (iWait->IsStarted())
	{
		iWait->AsyncStop();
	}
}
					

TBool  CPhoneChargingInfo::GetDeviceChargingInfo()
{
	TBool retValue=EFalse;
	
	TRAPD( err, retValue = GetDeviceChargingInfoL());
	
	if ( err == KErrNone)
	{
		//avoid warning
	}
	
	return retValue;
	
}

TBool CPhoneChargingInfo::GetDeviceChargingInfoL()
{
	TBool retValue=EFalse;
	//ListenL() is required Here because it might have been a long
	//time since Values have been updated!( if CNetworkInfo Instance 
	//was created long back, it might no longer even be Active!)
	if(!IsActive())
	{
	ListenL() ;
	}
	if(iStatus==KErrNone)
       {
	       if(iIndicatorV1.iCapabilities & CTelephony::KIndChargerConnected)
	          {
	          // We can detect when a charger is connected
	          if(iIndicatorV1.iIndicator & CTelephony::KIndChargerConnected)
	             {
	             	retValue= ETrue;
	             } // Charger is connected
	          else
	             {	
	             	retValue= EFalse;
	             } // Charger is not connected
	          }
	       else
	          {
	          	retValue= EFalse;
	          } // We do not know whether a charger is connected
       }

	//ListenL(); Called at the begining of the function
	
	return retValue;	
}
 	  				
#endif //FEATURE_PACK_3



