/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of class that manages all of 
*				 the file requests.
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "saplayer_build.h"
#include "platformdebug.h"
#include "flashliteinterfaces.h"

#include "symbian_stringutils.h"
#include "saflash_consts.h"
#include "symbian_engine.h"
#include "symbian_file.h"
#include "symbian_ui_interface.h"
#include "engine_data_caging_literals.hrh"
#include <charconv.h>
#include <pathinfo.h>
#include <driveinfo.h>

// size of '\' in characters
//static const int KPathDelimiterSize = 1;

_LIT8(KLitImageDownScaling,"_IMAGEPROXY_");
// 
// class CFileEngineRequest
//

//static
CFileEngineRequest* CFileEngineRequest::NewL(struct MM_Object* aMMObj, struct FI_URLStreamInfo* aStreamInfo, TUint16 aStreamId)
{
	CFileEngineRequest* self=new(ELeave) CFileEngineRequest(aMMObj, aStreamId);
	CleanupStack::PushL(self);
	self->ConstructL(aStreamInfo);
	CleanupStack::Pop(self);
	return self;
}

CFileEngineRequest::~CFileEngineRequest()
{
	Cancel();
	delete iFileName;
	iOwner = NULL;
	if(iFile && iFile->SubSessionHandle())
	{
		iFile->Close();
	}
	CPlatformFlashEngine* engine= iMMObj->pHostObj->iFIInterface->GetEngine();
	//if this is a mp3stream notify engine
	if(iIsMp3Stream && engine )
    {
        TRAP_IGNORE(engine->MP3StreamClosedL());
    }
	delete iFile;

}

TBool CFileEngineRequest::SetCanceled()
{
	// if inside FI call set the cancel flag.
	// if the cancel flag is set delete is pending, so cannot delete now
	if(iStateFlags&EFlhFileReqInsideFICall)
	{
		iStateFlags|=EFlhFileReqCanceled;
	}
	return !(iStateFlags&EFlhFileReqCanceled);
}

void CFileEngineRequest::ConstructL(struct FI_URLStreamInfo* aStreamInfo)
{
	if(aStreamInfo->documentPath!=NULL)
	{
		TPtrC8 fnWrap((TUint8*)aStreamInfo->documentPath);
		iFileName=fnWrap.AllocL();
	}
}

CFileEngineRequest::CFileEngineRequest(struct MM_Object* aMMObj, TUint16 aStreamId)
:CActive(CActive::EPriorityIdle), iMMObj(aMMObj), iId(aStreamId), iOwner(NULL), iDesWrapper(0,0,0),
iImgDownScaleRequired(EFalse)
{
    iIsMp3Stream = EFalse;
}


TUint16 CFileEngineRequest::Id() const
{
	return iId;
}

_LIT8(KLitHeaderContLen,"Content-Length");
void CFileEngineRequest::StartFileRequest(CFileEngine& aOwner)
{
	iOwner=&aOwner;
	// queue the request up to run
	CActiveScheduler::Add(this);
	SetActive();
	iStatus=KRequestPending;
	TRequestStatus* temp=&iStatus;
	User::RequestComplete(temp,KErrNone);
}


TBool CFileEngineRequest::CanContentBeTrusted(const TDesC8& aSwfURL)
{
	CPlatformFlashEngine* engine = iMMObj->pHostObj->iFIInterface->GetEngine();    
    if(engine->InstanceInTrustedSandBox())
    {
        return ETrue;
    }
    
    TInt pos;
    TBuf8<KMaxFileName> unixStyleSwfUrl = aSwfURL;
    pos = unixStyleSwfUrl.Find(KBackwardSlash);
    while(pos >= 0)
    {
        unixStyleSwfUrl.Replace(pos,1,KForwardSlash);
        pos = unixStyleSwfUrl.Find(KBackwardSlash);
    }    
    RFs fsSession = CEikonEnv::Static()->FsSession();
    DriveInfo::TDriveArray driveArray;
    TInt err = ETrue;//This is used to find if we get any errors in getting drive info and pathinfo
    if(KErrNone == DriveInfo::GetUserVisibleDrives( fsSession, driveArray ))
    {
        err = EFalse;
        TInt i=0;  
        TInt driveCount=driveArray.Count();
        for(i=0; i < driveCount; i++)
        {
            TDriveNumber drive( driveArray[ i ] ); 
            TFileName path;
            if(KErrNone==PathInfo::GetRootPath( path, drive ))
            {
                TBuf8 <KMaxFileName> trustedPath;
                trustedPath.Copy(path);
                trustedPath.Append(PathInfo::OthersPath());
                TInt pos;
                pos = trustedPath.Find(KBackwardSlash);
                while(pos >= 0)
                {
                    trustedPath.Replace(pos,1,KForwardSlash);
                    pos =trustedPath.Find(KBackwardSlash);
                }
                trustedPath.Append(KTrustedDir);
                if(unixStyleSwfUrl.FindF(trustedPath) >= 0)
                {
                    return ETrue;
                }
            }
            else
            {
                err = ETrue;
                break;
            }
        }
    }   
    if(err)//If there is a error in getting user visible drives/rootpath  we check the hardcoded paths
    {
        if(unixStyleSwfUrl.FindF(KOthersTrustedDir) >= 0 || 
           unixStyleSwfUrl.FindF(KTrustedPhoneDirPath) >= 0)
        {
            return ETrue;   
        }
    }
	//Some application send file:/// and hence offset returned would not be 0,
	// hence checking for KErrNotFound
    if(unixStyleSwfUrl.FindF(KTrustedZDirPrivate) >= 0 || 
       unixStyleSwfUrl.FindF(KShowroomDirPrivate) >= 0)
    {
        return ETrue;   
    }
    
	   return EFalse;
}


void CFileEngineRequest::CreateFileL()
{
	CPlatformFlashEngine* engine= iMMObj->pHostObj->iFIInterface->GetEngine();
	if(iFile)
		{
		delete iFile;
		iFile = NULL;
		}
	iFile = OpenLocalFileL();
	if(!iFile)
	{
		User::Leave(KErrNotFound);
	}

	User::LeaveIfError(iFile->Size(iFileLen));
	
	FI_URLStreamHeaderInfo headerInfo;
	headerInfo.numHeaders = 1;
	headerInfo.headerNames  = new(ELeave) char*[headerInfo.numHeaders];
	CleanupStack::PushL(headerInfo.headerNames);
	headerInfo.headerValues = new(ELeave) char*[headerInfo.numHeaders];
	CleanupStack::PushL(headerInfo.headerValues);

	// Only need size of file
	const TDesC8* temp=&(KLitHeaderContLen);
	headerInfo.headerNames[0]	= (char*) temp->Ptr();
	
	TBuf8<50> fileSize;
    /* In Case of Image Down Scaling, the length is the legth of the string _IMAGEPROXY_<FilePath>
     * Send the contentLen in the header accordingly. 
     * Also iFile Can be closed as we dont open it and read it. 
     */	
	if(iImgDownScaleRequired)
	{
		TInt cntLen = iImgDsPath->Length() + KLitImageDownScaling.iTypeLength; //cntLen + len(_IMAGEPROXY_);
		fileSize.AppendNum(cntLen); //fileSize name is used to avoid change in the existing code base.
		iFile->Close();
	}
	else
	{
	    fileSize.AppendNum(iFileLen);
	}
	headerInfo.headerValues[0]	= (char*)fileSize.PtrZ();
	
	// Tell core that stream is opened and go ahead allocating the buffer
	short errorCode = 0;

	// this call may return the first buffer
	iNetBuf = engine->GetFIObject().FI_URLStreamOpened( iId, &headerInfo, &errorCode );

	if(errorCode<=0)
	    {
		User::Leave(KErrNoMemory);	    
	    }
	//This is a mp3 stream notify engine
    if(iNetBuf->streamMediaType == FI_StreamMediaType_Sound)
        {
            engine->NewMP3StreamStarted();
            iIsMp3Stream = ETrue;
        }

  	HBufC8* pFinalUrl = HBufC8::NewL(iFileName->Length()+KUrlProtocolFileLength+1);//1 for null terminator
	TPtr8 d(pFinalUrl->Des());
	d.Append(KLitUrlProtocolFile);
	d.Append(*iFileName);
	d.Append(TChar('\0')); //null terminate
	FI_SecuritySandbox sSandbox = FI_UseDefault;
	//check if the swf is in default trusted folders
	if(CanContentBeTrusted(d))
	{
		sSandbox = FI_LocalTrusted;
	}
//In automation mode, security sandbox prompts should be disabled               
  #if defined(FEATURE_PERFORMANCE_AUTOMATION) || defined(FEATURE_ATS_AUTOMATION) || defined(FEATURE_TESTAUTOMATION)
	 	FlashEngineConfig lConfig = engine->GetConfig();
		if((lConfig.iMode & KPlayerInATSMode) || (lConfig.iMode & KPerformanceTestMode))	
	 			{
                if(KErrNotFound == d.FindF(KNoAutoTrust))
                {
                    sSandbox = FI_LocalTrusted;
                }
   			}

   #endif
	HBufC *file = HBufC::NewLC(iFileName->Length());
	TPtr filePtr(file->Des());
	filePtr.Copy(iFileName->Des());
	if(engine->IsContentTrustedL(filePtr))
	    {
	    sSandbox = FI_LocalTrusted;
	    }
	CleanupStack::PopAndDestroy(file);

	engine->GetFIObject().FI_URLStreamSetFinalURL(iId,(const char*)pFinalUrl->Ptr(), sSandbox);
	delete pFinalUrl;
    CleanupStack::PopAndDestroy(headerInfo.headerValues);
	CleanupStack::PopAndDestroy(headerInfo.headerNames);
	

	if(iNetBuf && iNetBuf->pBuffer)
	{
		if (!iImgDownScaleRequired)
			ReadNextFileSegment();
		else
			UpdateImageDownScale();
	}
}

void CFileEngineRequest::SetNetBuf(FI_NetworkBufferInfo* aNetBuf)
{
	iNetBuf=aNetBuf;
	if(iNetBuf && iNetBuf->pBuffer)
	{
		ReadNextFileSegment();
	}
}

void CFileEngineRequest::HandleRequestFailed(TInt aError)
{
	CPlatformFlashEngine* engine= iMMObj->pHostObj->iFIInterface->GetEngine();
	//as broswer donesn't support http status ocde to be propogated to flash plugin, hence we are 
	// sending "200" for success transaction and '0' for all other cases.
	engine->GetFIObject().FI_URLStreamClosed(iId, FI_ResultError, 0);
	if(aError!=KErrCancel)
	{
		// dk: should not leave
	    iMMObj->pHostObj->iFIInterface->GetEngine()->NotifyDataAccessError((const char*)&(*iFileName)[0],ECompleteFailedFileNotFound);	    
	}
}

void CFileEngineRequest::ReadNextFileSegment()
{
	SetActive();
	iStatus=KRequestPending;
	iDesWrapper.Set((TUint8*)iNetBuf->pBuffer,0,iNetBuf->maxBufSize);
	iFile->Read(iReadSoFar,iDesWrapper,iStatus);
}

void CFileEngineRequest::UpdateImageDownScale()
{
	SetActive();
	iStatus=KRequestPending;
	iDesWrapper.Set((TUint8*)iNetBuf->pBuffer,0,iNetBuf->maxBufSize);
	iDesWrapper.Copy(KLitImageDownScaling);
	iDesWrapper.Append(iImgDsPath->Des());
	TRequestStatus* tempStatus=&iStatus;
	User::RequestComplete(tempStatus,KErrNone);
	
}
void CFileEngineRequest::RunL()
{
	if(iStatus==KErrNone)
	{
		if(!iFile)
		{
			CreateFileL();
		}
		else
		{
			
			CPlatformFlashEngine* engine= iMMObj->pHostObj->iFIInterface->GetEngine();
				// write the data back to the player
			iNetBuf->dataSize = iDesWrapper.Length();
			iReadSoFar+=iNetBuf->dataSize;
			iStateFlags=EFlhFileReqInsideFICall;
			iNetBuf = engine->GetFIObject().FI_URLStreamDataLoaded(iId, iNetBuf);
			iStateFlags&=(~EFlhFileReqInsideFICall);
			
			if((iReadSoFar<iFileLen) && !iImgDownScaleRequired)
			{
				/*Isource 15980 -> CTimerThread Crash is Observed.
				As per the changes in FL 4.03 it required to check the validity of pNetBut as well as 
				the Pointer to the data buffer into which URL data should be read-in.*/
				if(iNetBuf && iNetBuf->pBuffer)
				{
					ReadNextFileSegment();
				}
				else
				{
					return;
				}
			}
			else
			{
			//as broswer donesn't support http status ocde to be propogated to flash plugin, hence we are 
			// sending "200" for success transaction and '0' for all other cases.
				engine->GetFIObject().FI_URLStreamClosed(iId, FI_ResultOK, 200);
				iOwner->Done();
			}
		}

	}
	else
	{
		HandleRequestFailed(iStatus.Int());
		iOwner->Done();
	}
}

TInt CFileEngineRequest::RunError(TInt aError)
{
	HandleRequestFailed(aError);
	iOwner->Done();
	return KErrNone;
}

void CFileEngineRequest::DoCancel()
{
	// no cancel for RFile methods
}

HBufC* CFileEngineRequest::AllocFullPathLC(const TDesC& aBase, const TDesC& aRest)
{
	const TChar KTCharPathDelimiter(KPathDelimiter);
	HBufC* ret=HBufC::NewLC(aBase.Length()+aRest.Length()+1); // 1 for possible path delim
	TPtr ptr2(ret->Des());
	ptr2.Append(aBase);
	if (aRest[0] != (unsigned char) KTCharPathDelimiter)
	{
		ptr2.Append(KTCharPathDelimiter);
	}
	ptr2.Append(aRest);
	return ret;
}

// This function uses lots of different formats - should be refactored...
RFile* CFileEngineRequest::OpenLocalFileL()
{

	CPlatformFlashEngine* engine= iMMObj->pHostObj->iFIInterface->GetEngine();
	RFs& fs=engine->GetRFs();

	const char* url = (const char*)&((*iFileName)[0]);
	unsigned long strLen = iFileName->Length() + 1;
	char* tempURL = new(ELeave) char[strLen];
	CleanupStack::PushL(tempURL);
	char* tURL = tempURL;

	unsigned short i;

	// Replace "/" with "\\"
	for (i=0;i<strLen;i++)
	{
		if (url[i] == KCharForwardSlash)
		{
			tURL[i] = KPathDelimiter;
		}
		else if (url[i] == '|')
		{
			tURL[i] = KDriveDelimiter;
		}
		else
		{
			tURL[i] = url[i];
		}
	}
	tURL[strLen-1]='\0';

	// search for '?' and remove it and everything after it. We can't handle GET style arguments in a file URL.
	strLen = FlashStrLen(tURL);
	for (i=0;i<strLen;i++)
	{
		if (tURL[i] == '?')
		{
			tURL[i] = '\0';
		}
	}

	TInt err;
	RFile* file = new(ELeave) RFile;
	CleanupStack::PushL(file);

	char* extraDot = StripPrefix(tURL, ".");
	if (extraDot)
	{
		if ( *extraDot == KPathDelimiter ) 
		{
			extraDot++;
		}
		tURL = extraDot;
	}

	char* extraSplash = StripPrefix(tURL, "\\");
	if (extraSplash)
	{
		if ( *extraSplash == KPathDelimiter ) 
		{
			extraSplash++;
		}
		tURL = extraSplash;
	}

	extraSplash = StripPrefix(tURL, "/");
	if (extraSplash)
	{
		if ( *extraSplash == KPathDelimiter ) 
		{
			extraSplash++;
		}
		tURL = extraSplash;
	}

	// first try the path as is.
    // We need to interpret the path as utf-8. The Japanese and Korean code
    // pages will interpret backslash as YEN or WON.
    TInt oldCodepage = engine->CurrentCodePage();
    engine->ChangeCodePageL(KCharacterSetIdentifierUtf8);
	HBufC* hFilename = engine->CreateEpocStringLC(tURL, FlashStrLen(tURL));
    engine->ChangeCodePageL(oldCodepage);
	
	// Symbian does not support .\ ..\ etc. Resolve them here...
	TPtr tidyPath(hFilename->Des());
	engine->GetFileUtils()->ResolvePathDotOperators(tidyPath);
/* IMGDS:
 * Here we pass the filename to engine to understand if the file requested is a .Jpeg file.
 * If .jpg file, check the resolution and decide if Image Down Scaling is required or not
 */	
	if(engine->ImageDownScalingRequiredL(*hFilename))
	{
		iImgDsPath = hFilename->Alloc();
		iImgDownScaleRequired = ETrue;
	}
	err = file->Open(fs, *hFilename, EFileRead | EFileShareExclusive | EFileStream);
	
	#ifdef FEATURE_MULTIPLE_DRIVE 
	HBufC* driveRootPath =HBufC::NewLC(KMaxFileName);
	#endif
	
	if (err != KErrNone)
	{
		// if that file isn't valid, check to see if it's in one of the special Flash dirs
    #ifndef FEATURE_MULTIPLE_DRIVE                

		TFileName phonePath;
		phonePath.Copy(PathInfo::PhoneMemoryRootPath());
		phonePath.Append(PathInfo::OthersPath());

		TFileName memCardPath;
		memCardPath.Copy(PathInfo::MemoryCardRootPath());
		memCardPath.Append(PathInfo::OthersPath());

		HBufC* hFileName2=AllocFullPathLC(phonePath,tidyPath);
		err = file->Open(fs, *hFileName2, EFileRead | EFileShareExclusive | EFileStream);
		CleanupStack::PopAndDestroy(hFileName2);
		
		if (err != KErrNone)
		{
			HBufC* hFileName3=AllocFullPathLC(memCardPath,tidyPath);
			err = file->Open(fs, *hFileName3, EFileRead | EFileShareExclusive | EFileStream);
			CleanupStack::PopAndDestroy(hFileName3);
		}				
		
		
		
	#else ///FEATURE_MULTIPLE_DRIVE
	
		err=KErrNone;
		TInt pos = tidyPath.LocateReverse('\\');
    	if(pos!=KErrNotFound)
	 		{
	 		TInt length=tidyPath.Length();
	 		tidyPath= tidyPath.Right(length-pos - 1);
	 		}
       	
	   	TPtr ptrRootPath=driveRootPath->Des();	
	  	TInt intDrive;
	  	DriveInfo::TDriveArray driveArray;
		User::LeaveIfError( DriveInfo::GetUserVisibleDrives( fs, driveArray ) );

		for(TInt i=0; i < driveArray.Count(); i++)
	    	{
	    	TChar driveLetter = driveArray.LetterAt(i);
	    	User::LeaveIfError( RFs::CharToDrive(driveLetter, intDrive) );
	    	User::LeaveIfError( PathInfo::GetRootPath(ptrRootPath, intDrive) );
	    	ptrRootPath.Append(PathInfo::OthersPath()); 
	        ptrRootPath.Append(tidyPath);
	        
	        err = file->Open(fs, ptrRootPath, EFileRead | EFileShareExclusive | EFileStream);
	  			
	        if(err==KErrNone)//as soon as file is found, break the loop as there is no need to search in other dirs.
	        break;                
	    	}

	              
    #endif ///FEATURE_MULTIPLE_DRIVE
  	
    }
    
    #ifdef FEATURE_MULTIPLE_DRIVE
    CleanupStack::PopAndDestroy(driveRootPath);
    #endif

    CleanupStack::PopAndDestroy(hFilename);
    CleanupStack::Pop(file);
	
	if (err != KErrNone)
	{
		delete file;
		file = NULL;
	}

	CleanupStack::PopAndDestroy(tempURL); // clean up tempURL now that file is open or not found.

	return file;
}


//
// Implementation of class CFileEngine
//
CFileEngine::CFileEngine()
:iRequestQueue(_FOFF(CFileEngineRequest,iLinkItem)), iCurrentRequest(NULL)
{
}


CFileEngine::~CFileEngine()
{
	CancelAllRequests();
}


CFileEngine* CFileEngine::NewL()
{
	CFileEngine* me = new(ELeave)CFileEngine();
	return me;
}

unsigned short CFileEngine::StartFileRequest(CFileEngineRequest* aRequest)
{
	iRequestQueue.AddLast(*aRequest);
	
	TryQueueNextRequest();
	
	return 1;
	
}

void CFileEngine::TryQueueNextRequest()
{
	if(!iCurrentRequest)
	{
		// keep firing off requests until one successfully starts.
		while(!iRequestQueue.IsEmpty() && !iCurrentRequest)
		{
			// get first in line for running
			CFileEngineRequest* head=iRequestQueue.First();
			iRequestQueue.Remove(*head);
			DoIssueNextRequest(*head);
			iCurrentRequest=head;
		}
	}
}

void CFileEngine::DoIssueNextRequest(CFileEngineRequest& aInfo)
{
	aInfo.StartFileRequest(*this);
}

void CFileEngine::Done()
{
	delete iCurrentRequest;
	iCurrentRequest=NULL;
	TryQueueNextRequest();
}

// gets a pointer to the requested object, or null if not found
CFileEngineRequest* CFileEngine::GetRequestObjectOrNull(TUint16 aId)
{
	TSglQueIter<CFileEngineRequest> iter(iRequestQueue);
	
	//  See if the current request is the object we are looking for
	if (iCurrentRequest && iCurrentRequest->Id()==aId){
		return iCurrentRequest;
	} 
	
	//  Otherwise, iterate through the request queue
	CFileEngineRequest* item=iter;
	while(item)
	{
		if(item->Id()==aId)
		{
			return item;
		}
		item=iter++;
	}

	//  No matching request ID found, return NULL
	return NULL;
}

// cancels the requested object, if found. If not found does nothing
TUint16 CFileEngine::CancelRequestObject(TUint16 aId)
{
	if (iCurrentRequest && iCurrentRequest->Id()==aId)
	{
		if(iCurrentRequest->SetCanceled())
		{
			Done();
		}
	
	}
	else // maybe its still queue'd ?
	{
		CFileEngineRequest* item=GetRequestObjectOrNull(aId);

		if(item)
		{
			item->HandleRequestFailed(KErrCancel);
			iRequestQueue.Remove(*item);
			delete item;
			item = NULL;
		}
		else // item not found - cannot cancel
			return 0;
	}
	return 1;
}

// cancels all requests, including the one being processed and any queued.
void CFileEngine::CancelAllRequests()
{
	CancelActiveTransaction();

	CFileEngineRequest* info;
	while(!iRequestQueue.IsEmpty())
	{
		info=iRequestQueue.First();
		iRequestQueue.Remove(*info);
		info->HandleRequestFailed(KErrCancel);
		delete info;
	}
}

void CFileEngine::CancelActiveTransaction()
{
	if(iCurrentRequest)
	{
		iCurrentRequest->HandleRequestFailed(KErrCancel);
		delete iCurrentRequest;
		iCurrentRequest = NULL;	
	}
}


//
// CFlhFileUtils
//

TInt CFlhFileUtils::DeleteFile(const TDesC& aFile)
{
    return iFileMan->Delete(aFile);
}




CFlhFileUtils::CFlhFileUtils()
{}


CFlhFileUtils::~CFlhFileUtils()
{
    delete iFileMan;
}


CFlhFileUtils* CFlhFileUtils::NewL(RFs& aFs)
{
    CFlhFileUtils* utils = new(ELeave) CFlhFileUtils();
	CleanupStack::PushL(utils);
	CFileMan* fileMan = CFileMan::NewL(aFs);
	CleanupStack::Pop(utils);
	utils->iFileMan = fileMan;
    return utils;
}


_LIT(KLitCurrentDir,"\\.\\");
_LIT(KLitBackOneDir,"\\..\\");
void CFlhFileUtils::ResolvePathDotOperators(TDes& aPathToTidy)
{
	TInt locate;
	TInt remainingLen;
	FOREVER
	{
		locate=aPathToTidy.Find(KLitCurrentDir);
		if(locate==KErrNotFound)
			break;
		
		remainingLen=aPathToTidy.Length()-(locate+2);
		
		if(remainingLen<0) // bad filename - quit now or the descriptor will panic us
			break;

		TPtrC newEnd(aPathToTidy.Right(remainingLen)); // throw 2 chars away...
		aPathToTidy.SetLength(locate);
		aPathToTidy.Append(newEnd);
	}

	TInt locateLastDir;
	FOREVER
	{
		locate=aPathToTidy.Find(KLitBackOneDir);
		if(locate==KErrNotFound)
			break;
		
		remainingLen=aPathToTidy.Length()-(locate+3);
		
		if(remainingLen<0) // bad filename - quit now or the descriptor will panic us
			break;

		TPtrC newEnd(aPathToTidy.Right(remainingLen)); // throw 3 chars away...
		aPathToTidy.SetLength(locate-1); // remove the slash from the starting path so we can locate the directory to eat

		// now remove the previous directory...
		locateLastDir=aPathToTidy.LocateReverse(KPathDelimiter);
		if(locateLastDir!=KErrNotFound)
			aPathToTidy.SetLength(locateLastDir);

		aPathToTidy.Append(newEnd);
	}
}

RFile* CFileEngineRequest::OpenCreateFileL(struct MM_Object* iMMObj, const char* iFileName)
{
	CPlatformFlashEngine* engine = iMMObj->pHostObj->iFIInterface->GetEngine();
	RFs& fs = engine->GetRFs();

	HBufC* hFilename = engine->CreateEpocStringLC(iFileName, FlashStrLen(iFileName));

	RFile* file = new(ELeave) RFile;
	TInt err = file->Open(fs, *hFilename, EFileRead | EFileShareReadersOnly);
	if (err != KErrNone)
	{
		delete file;
		file = NULL;
	}

#ifdef __FEATURE_FLV_RECOG_PLAY

    const char* KVideoRFileStr = "@RFILE@" ;
    
    if(!file)
    {  
      //possibly this is FLV / F4V file request
      if(hFilename && (FlashStrStr(iFileName,KVideoRFileStr)))
       { 
            //request is for video playback , let us request from ui/plugin
            file  = engine->GetRFileHandleForPlay();
	   }
	}

#endif
	
	CleanupStack::PopAndDestroy(hFilename);
	return file;
}

#ifdef __FEATURE_FLV_RECOG_PLAY
void CFileEngineRequest::CloseDestroyFile(struct MM_Object* iMMObj,RFile *pFile)
#else
void CFileEngineRequest::CloseDestroyFile(RFile *pFile)
#endif
{
#ifdef __FEATURE_FLV_RECOG_PLAY
	CPlatformFlashEngine* engine = iMMObj->pHostObj->iFIInterface->GetEngine();
	RFile* fl = engine->GetRFileHandleForPlay();
	
	if (pFile && ( fl != pFile))
#else
	if (pFile)
#endif	
	{
		pFile->Close();
		delete pFile;
	}
}


TInt CFileEngineRequest::ReadFile(RFile* pFile, unsigned char* buffer, unsigned int bufferSize)
{
	TPtr8 pDest(buffer, bufferSize);

	int read = 0;
	TInt error = pFile->Read(pDest, bufferSize);
	if (error == KErrNone)
		read = pDest.Length();
	return read;
}
////////////////////////////////////////////////////////////////////////////////
///CFileRequestObject

CFileRequestObject* CFileRequestObject::OpenCreateFileL(struct MM_Object* iMMObj
									, const char* iFileName)
{
	CPlatformFlashEngine* engine = iMMObj->pHostObj->iFIInterface->GetEngine();
	RFs& fs = engine->GetRFs();

	HBufC* hFilename = engine->CreateEpocStringLC(iFileName, FlashStrLen(iFileName));
    TPtr ptr(hFilename->Des());
	ReplaceAll(ptr, KCharForwardSlash, KCharBackslash);
	
	RFile* file = new(ELeave) RFile;
	TBool owndHandle = ETrue;
	TInt err = file->Open(fs, *hFilename, EFileRead | EFileShareReadersOnly);
	if (err != KErrNone)
	{
		delete file;
		file = NULL;
	}

#ifdef __FEATURE_FLV_RECOG_PLAY

	//const char* KFlvRFileStr = "@RFILE@" ;
	
   const char* KVideoRFileStr = "@RFILE@" ;
	if(!file)
    {  //possibly this is Video file request
      if(hFilename && (FlashStrStr(iFileName,KVideoRFileStr)))
       { //request is for video playback , let us request from ui/plugin
	   		file  = engine->GetRFileHandleForPlay();
	   		owndHandle = EFalse;
	   }
	}

#endif
	
	CleanupStack::PopAndDestroy(hFilename);
	
	if(file)
	{
		return new(ELeave) CFileRequestObject(owndHandle,file);
	}
	else
	{
		return NULL;
	}
	
}
	

CFileRequestObject::~CFileRequestObject()
{
	CloseHandle();
}

CFileRequestObject::CFileRequestObject(TBool aOwnedHandle,RFile* aFile):
										iFile(aFile)									
										,iOwnedHandle(aOwnedHandle)
{	
	
}

void CFileRequestObject::CloseHandle()
{
	if(iOwnedHandle && iFile)
	{
		iFile->Close();
		delete iFile;
		iFile=NULL;
	
	}
}

void CFileRequestObject::CloseDestroyFile()
{
	CloseHandle();	
}


TInt CFileRequestObject::ReadFile( unsigned char* buffer
									, unsigned int bufferSize)
{
	TPtr8 pDest(buffer, bufferSize);

	int read = 0;
	TInt error = iFile->Read(pDest, bufferSize);
	if (error == KErrNone)
		read = pDest.Length();
	return read;
}

TInt CFileRequestObject::FileSize()
{
	int size = -1; 
	
	if (iFile) iFile->Size(size);
	 
	return size;
}
			
			
TInt CFileRequestObject::SeekFile( TSeek mode
									, TInt offset)
{
	TInt error = KErrGeneral; 
	if (iFile) 
	{
		error = iFile->Seek(mode, offset);	
	}
	 
	return error;
}
									

///////////////////////////////////////////////////////////////
//  
