/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash Methods Handler class 
*
*/
 

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.							//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "version.h"
#include "saplayer_build.h"
#include <string.h>
#include "commondef.h"
#include "symbian_hostscript_handler.h"
#include "symbian_engine.h"
#include "fi_interface.h"
#include "NPNFlashDefs.h"
#include "async_object.h"

_LIT(KValTrue,"1");
_LIT(KValFalse,"0");
_LIT(KValNaN, "NaN");
_LIT(KNullTerminate, "\0");
_LIT(KTrueStr, "true");
_LIT(KFalseStr, "false");
// -------------------------------------------------------------------------
// CHostScriptHandler* CHostScriptHandler::NewL(CPlatformFlashEngine* aEngine)
//
// CHostScriptHandler object creator
// --------------------------------------------------------------------------
//
CHostScriptHandler* CHostScriptHandler::NewL(CPlatformFlashEngine* aEngine)
{
	CHostScriptHandler* self = new (ELeave) CHostScriptHandler(aEngine);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

// -------------------------------------------------------------------------
// CHostScriptHandler::~CHostScriptHandler()
// destructor
// --------------------------------------------------------------------------
//		   
CHostScriptHandler::~CHostScriptHandler()
{
	//delete iReqArray;
	//iReqArray = NULL;
	
	delete iAsyncObject;
	iAsyncObject = NULL;
}
      
// -------------------------------------------------------------------------
// CHostScriptHandler::CHostScriptHandler(CPlatformFlashEngine* aEngine)
//
// First phase constructor
// --------------------------------------------------------------------------
//      
CHostScriptHandler::CHostScriptHandler(CPlatformFlashEngine* aEngine):
							iPluginScriptHandler(NULL)
							, iEngine(aEngine) 
{
}

// -------------------------------------------------------------------------
// void CHostScriptHandler::ConstructL()

// Second phase constructor
// --------------------------------------------------------------------------
//	
void CHostScriptHandler::ConstructL()
{
	iAsyncObject = CFlhAsyncObject::NewL(CActive::EPriorityUserInput+1,*this); 
	
	//iReqArray = new (ELeave) CArrayPtrSeg<CJSRequest>(1);
}
    
    
/////////////////////////////////////////////////////////////////////
//MHostScriptHandler methods

// -------------------------------------------------------------------------
// MFlashMethods* CHostScriptHandler::GetFlashMethods()

// Return MFlashMethods handler from host engine
// --------------------------------------------------------------------------
//	
MFlashMethods* CHostScriptHandler::GetFlashMethods()
{
  return STATIC_CAST(MFlashMethods*,this);;
}

#ifdef FEATURE_EXTERNAL_API	

// -------------------------------------------------------------------------
// MExternalInterface* CHostScriptHandler::GetExternalInterface()

// Return MExternalInterface handler from host engine
// --------------------------------------------------------------------------
//
MExternalInterface* CHostScriptHandler::GetExternalInterface()
{
  return STATIC_CAST(MExternalInterface*,this);
}

#endif

// -------------------------------------------------------------------------
// MPluginScriptHandler* CHostScriptHandler::GetPluginScriptHandler() 
//
// Used by Host engine and core player to talk to Javascript
// --------------------------------------------------------------------------
//					
//
MPluginScriptHandler* CHostScriptHandler::GetPluginScriptHandler() 
{ 
	 return iPluginScriptHandler; 
}

// -------------------------------------------------------------------------
// void CHostScriptHandler::SetPluginScriptHandler(MPluginScriptHandler* 
//													aPluginScriptHandler)
//
// This function should be called by plugin before calling
// GetFlashMethods,GetExternalInterface
// --------------------------------------------------------------------------
//					
				

void CHostScriptHandler::SetPluginScriptHandler(MPluginScriptHandler* 
													aPluginScriptHandler) 
{
	 iPluginScriptHandler = aPluginScriptHandler;
}


////////////////////////////////////////////////////////////////
//

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::GetVariable(const TDesC& aVariableNameIn 
//    					, TDes& /*aVariableValueOut*/
//    					, const TDesC& aURLIn 
//    					, TPageAccessType aPageAccessType
//    					, TBool aIsTrusted ) 
//
// GetVariable( varName )    
// Description:  Returns the value of the Flash variable specified by varName.
// Returns: null if the variable does not exist. The argument type is string.    
// Example:  var firstName = movie.GetVariable("FirstName");  
// --------------------------------------------------------------------------
//
TBool CHostScriptHandler::GetVariableL(const TDesC& aVariableNameIn 
    					, HBufC** aVariableValueOut
    					, const TDesC& aURLIn 
    					, TPageAccessType aPageAccessType
    					, TBool aIsTrusted ) 
{
		
	if( iEngine )
	{
		const char *  variableNameIn = iEngine->CreateFlashString(aVariableNameIn);
		char *  varValueOut=NULL;

		FI_BrowserURLInfo browserInfoIn;		
		
		browserInfoIn.pURLIn         = iEngine->CreateFlashString(aURLIn);
		browserInfoIn.pageAccessType = aPageAccessType;
		browserInfoIn.bIsTrusted     = aIsTrusted;
		
		CHostAllocatorWrapper *allocatorWrapper = new (ELeave) CHostAllocatorWrapper;

		
		varValueOut = STATIC_CAST(char*,(iEngine->GetFIObject()).FI_GetVariable2(variableNameIn,&browserInfoIn, allocatorWrapper ));
		
		TInt len = 0;
		
		if(varValueOut != NULL)
		{
			len  = strlen(varValueOut);	
		}
		
	 	HBufC* tempStr = iEngine->CreateEpocString(varValueOut,len);
		
		//DONE : copy to be handled
		//aVariableValueOut->Copy( *tempStr );	
		(*aVariableValueOut) = tempStr;	
		//delete tempStr;
		allocatorWrapper->FreeWrapper(allocatorWrapper->pContext,varValueOut);
		delete allocatorWrapper;		
		return ETrue;
	}
	else
	{
		return EFalse;
	}
	
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::GotoFrame(TInt /*aFrameNumber*/)
//
// Activates the frame number specified by frameNumber in the current movie.
// If the data for a requested frame is not yet available,
// the player goes to the last frame available and stops,
// causing unexpected results during playback.
// Use the PercentLoaded() method to determine if enough of the movie is 
// available to execute the GotoFrame() method.
// The argument frameNumber is zero-based;
// that is, frameNumber is 0 in the first frame of the movie,
// 1 for the second frame, and so on. This differs from the 
// Goto action within Flash, which begins at 1. The argument type is integer.
//
//Example:  movie.GotoFrame(24); 
// --------------------------------------------------------------------------
//
TBool CHostScriptHandler::GotoFrame(TInt /*aFrameNumber*/)
{
	return EFalse;
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::IsPlaying() const 
//
// Returns true if the movie is currently playing.    
// Example  if (movie.IsPlaying()) { alert("movie is playing"); }    
//
// --------------------------------------------------------------------------
//

TBool CHostScriptHandler::IsPlaying() const 
{
	if(iEngine)
	{
		return (iEngine->GetFIObject()).FI_IsPlaying();	
	}
	else
	{
		return EFalse;
	}
}
	
// -------------------------------------------------------------------------
// TBool CHostScriptHandler::LoadMovieL( TInt /*aLayerNo*/
//							,const TDesC& /*aURLIn*/)
//
//  Loads the movie identified by url to the layer specified by layerNumber. 
//  The argument type is integer for layerNumber and string for url. 
//     
// Example  movie.LoadMovieL(0, "mymovie.swf"); 
//
// --------------------------------------------------------------------------
//						
TBool CHostScriptHandler::LoadMovieL( const TDesC& aSwfURL, TInt aLayerNo
							, const TDesC& aURLIn
							, TPageAccessType aPageAccessType
    						, TBool aIsTrusted)
{
	
	TBool result = EFalse;
	if( iEngine )
	{
		FI_BrowserURLInfo browserInfoIn;

		browserInfoIn.pURLIn         = iEngine->CreateFlashString(aURLIn);
		browserInfoIn.pageAccessType = aPageAccessType;
		browserInfoIn.bIsTrusted     = aIsTrusted;
		
		char* swfURL = iEngine->CreateFlashString(aSwfURL);
		
		(iEngine->GetFIObject()).FI_LoadLayer( swfURL,
									aLayerNo, &browserInfoIn );
									
		delete 	browserInfoIn.pURLIn;					
		delete  swfURL;
		result = ETrue;
	}
	
	return result;
	
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::Pan(TInt32 aXCoordinate
//						,TInt32 aYCoordinate
//						, TInt32 aMode) 
//
// Pans a zoomed-in movie to the coordinates specified by x and y.
// Use mode to specify whether the values for x and y are pixels
// or a percent of the window. When mode is 0, the coordinates are pixels;
// when mode is 1, the coordinates are percent of the window.
// Pan does not pan beyond the boundaries of the zoomed-in movie. 
//The argument type for all arguments is integer.    
// Example  This example pans 50% right and 50% down: movie.Pan(50, 50, 1) 
//This example pans -25 pixels left and -25 pixels up:movie.Pan(-25, -25, 0) 
//
// --------------------------------------------------------------------------
//
TBool CHostScriptHandler::Pan(TInt32 /*aXCoordinate*/
						,TInt32 /*aYCoordinate*/
						, TInt32 /*aMode*/) 
{
	return EFalse;
}

	
// -------------------------------------------------------------------------
// TBool CHostScriptHandler::PercentLoaded(TInt& aLoadPercent) 
//
// Returns the percent of the Flash Player movie that has streamed
// into the browser so far; possible values are from 0 to 100.    
// Example:  if (movie.PercentLoaded() == 100) { loaded = true; } 
//
// --------------------------------------------------------------------------
//

TBool CHostScriptHandler::PercentLoaded(TInt& aLoadPercent) 
{
	aLoadPercent = 100;
	
	return ETrue;
}
// -------------------------------------------------------------------------
// TBool CHostScriptHandler::FlashVersion(HBufC** aVersion)
//
// Returns the version of the flash player.
//
// --------------------------------------------------------------------------
//
TBool CHostScriptHandler::FlashVersion(HBufC** aVersion) 
{
	if(iEngine)
	{
		HBufC* version = iEngine->GetFlashVersion();
		(*aVersion) = version;
		return ETrue;
	}
	return EFalse;
}
// -------------------------------------------------------------------------
// TBool CHostScriptHandler::Play() 
//
// Starts playing the movie.    
// Example  movie.Play(); 
//
// --------------------------------------------------------------------------
//
TBool CHostScriptHandler::Play() 
{
	if(iEngine)
	{
		TRAPD(err,iEngine->ResumeL(EReasonUser, ETrue));	
		if(err == KErrNone)
		{
			return ETrue;			
		}		
	}
	return EFalse;
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::Rewind()
//
// Goes to the first frame.    
// Example  movie.Rewind(); 
//
// --------------------------------------------------------------------------
//	
TBool CHostScriptHandler::Rewind()
{
	if(iEngine)
	{
		TRAPD(err,iEngine->RewindL());
		
		if(err == KErrNone)
		{
			return ETrue;			
		}
		
	}

	return EFalse;
	
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::SetVariable(	const TDesC& aVariableNameIn,
//	    				const TDesC& aVariableValueIn, const TDesC& aURLIn, 
//    					TPageAccessType aPageAccessType, TBool aIsTrusted )
//
// Sets the value of the Flash variable specified by variableName 
// to the value specified by value. The argument type for both arguments
// is string.    
// Example  movie.SetVariable("/Form:UserName", "John Smith"); 
// --------------------------------------------------------------------------
//
TBool CHostScriptHandler::SetVariableL(	const TDesC& aVariableNameIn
	    				, const TDesC& aVariableValueIn
	    				, const TDesC& aURLIn
    					, TPageAccessType aPageAccessType
    					, TBool aIsTrusted )
{
	if(iEngine)
	{
		const char *  variableNameIn = iEngine->CreateFlashString(aVariableNameIn);
		const char *  variableNameOut = iEngine->CreateFlashString(aVariableValueIn);
		FI_BrowserURLInfo browserInfoIn;

		browserInfoIn.pURLIn         = iEngine->CreateFlashString(aURLIn);
		browserInfoIn.pageAccessType = aPageAccessType;
		browserInfoIn.bIsTrusted     = aIsTrusted;
		
		TBool retVal =  ((iEngine->GetFIObject()).FI_SetVariable2(variableNameIn
										,variableNameOut
										,&browserInfoIn ));
										
		// in case the play attribute is false we need to refresh the screen 
		// to make sure that the change is reflected								
		
		iEngine->PlayStep(ETrue);		
									
		delete variableNameIn;									
		delete variableNameOut;	
		return 	retVal;							
	}
	else
	{
		return EFalse;
	}
}
	
// -------------------------------------------------------------------------
// TBool CHostScriptHandler::SetZoomRect(TInt /*aLeft*/, 
//							TInt /*aRight*/,
//							TInt /*aBottom*/,
//							TInt /*aTop*/
//							)
//
// Zooms in on a rectangular area of the movie. 
// The units of the coordinates are in twips (1440 units per inch). 
// To calculate a rectangle in Flash, set the ruler units to Points
// and multiply the coordinates by 20 to get twips. (There are 72 points per inch.)
// The argument type for all arguments is integer.    
// Example  This example zooms in on a 200 x 200 pixel rectangle in the 
// upper left corner of the movie: 
//
// var pointsToTwips = 20;
// movie.SetZoomRect(0, 0, 200 * pointsToTwips, 200 * pointsToTwips);
//
// --------------------------------------------------------------------------
//

TBool CHostScriptHandler::SetZoomRect(TInt /*aLeft*/, 
							TInt /*aRight*/,
							TInt /*aBottom*/,
							TInt /*aTop*/
							)
{
	return EFalse;
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::StopPlay()
//
// Stops playing the movie.    
// Example movie.StopPlay() 
//
// --------------------------------------------------------------------------
//

	
TBool CHostScriptHandler::StopPlay()
{
	if(iEngine)
	{
		iEngine->Pause(EReasonUser);	
		
		return ETrue;
	}
	else
	{
		return EFalse;
	}
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::TotalFrames(TInt& aNumberOfFrames)
//
// Returns the total number of frames in the movie.    
// Example  var totalFrames = movie.TotalFrames(); 
//
// --------------------------------------------------------------------------
//
	
TBool CHostScriptHandler::TotalFrames(TInt& aNumberOfFrames)
{
	if(iEngine)
	{
		aNumberOfFrames = iEngine->GetTotalFrameNumber();
		
		return ETrue;
	}
	
	return EFalse;
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::Zoom(TInt /*aZoomPercent*/)
//
// Zooms the view by a relative scale factor specified by percent.
// Zoom(50) doubles the size of the objects in the view. 
// Zoom(200) reduces the size of objects in the view by one half. 
// Zoom(0) resets the view to 100%. 
// You cannot specify a reduction in the size of objects in the view 
// when the current view is already 100%. The argument type is integer.
//    
// Example movie.Zoom(50);  
//
// --------------------------------------------------------------------------
//	
TBool CHostScriptHandler::Zoom(TInt /*aZoomPercent*/)
{
	return EFalse;
}
	
//////////////////////////////////////////////////////////////////////////
//TellTarget Methods


// -------------------------------------------------------------------------
// TBool CHostScriptHandler::TCallFrame( const TDesC& /*aTarget*/, 
//						TInt /*aFrameNumber*/)

//  In the timeline specified by target, executes the action in 
//  the frame specified by frameNumber. 
//     
// Example  This example runs the actions in the fifth frame of the main timeline: 
// 				movie.TCallFrame("/", 4);
// --------------------------------------------------------------------------
//
TBool CHostScriptHandler::TCallFrameL( const TDesC& /*aTarget*/, 
						TInt /*aFrameNumber*/)
{
	return EFalse;
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::TCallLabel( const TDesC& /*aTarget*/,
//										const TDesC& /*aLabel*/)
//
//  In the Timeline indicated by target, executes the action in the frame
// specified by the label frame label. The argument type for both arguments
// is string.
//     
// Example  This example runs the actions in the frame labeled 
// 			HandleScriptNotify in the main timeline: 
//			movie.TCallLabel("/", "HandleScriptNotify");
//
//
// --------------------------------------------------------------------------
//
	
TBool CHostScriptHandler::TCallLabelL( const TDesC& /*aTarget*/,
										const TDesC& /*aLabel*/)
{
	return EFalse;
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::TCurrentFrame( const TDesC& aTarget
//								 , TInt& /*aCurrentFrame*/
//								 , const TDesC& aURLIn
//								 , TPageAccessType aPageAccessType
//								 , TBool aIsTrusted) 
//
//  TCurrentFrame( target ) 
//	Returns the number of the current frame for the timeline specified by target.
//  The frame number returned is zero-based, meaning frame 1 of the Flash movie
//  would be 0, frame 2 would be 1, and so on. The argument type is string.
//    
// Example  var currentFrame = movie.TCurrentFrame("/MovieClip"); 
//
// --------------------------------------------------------------------------
//	
TBool CHostScriptHandler::TCurrentFrameL( const TDesC& aTarget
								 , TInt& aCurrentFrame
								 , const TDesC& aURLIn
								 , TPageAccessType aPageAccessType
								 , TBool aIsTrusted) 
{
	if(iEngine)
	{
		const char *  target = iEngine->CreateFlashString(aTarget);

		FI_BrowserURLInfo browserInfoIn;		
		
		browserInfoIn.pURLIn         = iEngine->CreateFlashString(aURLIn);
		browserInfoIn.pageAccessType = aPageAccessType;
		browserInfoIn.bIsTrusted     = aIsTrusted;
				
		aCurrentFrame = (iEngine->GetFIObject()).FI_TCurrentFrame(target, &browserInfoIn);
		
		return ETrue;
	}
	else
	{
		return EFalse;
	}	
		


}

// -------------------------------------------------------------------------
// bool CFlashMethods::CurrentFrame( TInt& aCurrentFrame)
//
//  CurrentFrame() 
//  Returns the frame number the root object. 
// Example  var currentFrame = movie.CurrentFrame(); 
// --------------------------------------------------------------------------
TBool CHostScriptHandler::CurrentFrame(TInt& aCurrentFrame )
 {
    
    if(iEngine)
    {
        // FI_FrameNumber returns currentFrame+1 so need to substract 1
        aCurrentFrame = (iEngine->GetFIObject()).FI_FrameNumber()-1;
        return ETrue;
    }
    else
    {
      return EFalse;
    }   
 }

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::TCurrentLabel( const TDesC& aTarget
//									, TDes& /*aLabel*/
//									, const TDesC16& aURLIn
//									, TPageAccessType aPageAccessType
//									, TBool aIsTrusted)
//
// Returns the label of the current frame of the timeline specified by target.
// If there is no current frame label, an empty string is returned. 
// The argument type is string.    
// Example  var currentLabel = movie.TCurrentLabel("/MovieClip");    
//
// --------------------------------------------------------------------------
//
	
TBool CHostScriptHandler::TCurrentLabelL( const TDesC& aTarget
									, HBufC** aLabel
									, const TDesC16& aURLIn
									, TPageAccessType aPageAccessType
									, TBool aIsTrusted) 
{
	if(iEngine)
	{
		const char *  target = iEngine->CreateFlashString(aTarget);
		char *  label=NULL;

		FI_BrowserURLInfo browserInfoIn;		

		browserInfoIn.pURLIn         = iEngine->CreateFlashString(aURLIn);
		browserInfoIn.pageAccessType = aPageAccessType;
		browserInfoIn.bIsTrusted     = aIsTrusted;
		
		CHostAllocatorWrapper *allocatorWrapper = new (ELeave) CHostAllocatorWrapper;
		
		label = STATIC_CAST(char*,(iEngine->GetFIObject()).FI_TCurrentLabel(target, &browserInfoIn, allocatorWrapper ));
		TInt length = 0;
		if(label != NULL)
		{
			length = strlen(label);	
		}
		
		HBufC* tempStr = iEngine->CreateEpocString(label,length);
		
		(*aLabel) = tempStr;
				
		allocatorWrapper->FreeWrapper(allocatorWrapper->pContext,label);
		delete allocatorWrapper;
		return ETrue;
	}
	else
	{
		return EFalse;
	}	
		

}


// -------------------------------------------------------------------------
// TBool CHostScriptHandler::TGetProperty( const TDesC& aTarget
//								, TInt aPropertyID
//								, HBufC** aPropertyValue
//								, const TDesC& aURLIn 
//	    				 		, TPageAccessType aPageAccessType
//	    				 		, TBool aIsTrusted)
//
//  For the timeline indicated by target, returns a string indicating 
//  the value of the property specified by property. For property, enter the
//  integer corresponding to the desired property. For a list of all properties
//  and their corresponding integers, see Getting and setting properties. 
//     
//  Example  var nameIndex = 13;  
//			var name = movie.TGetProperty("/", nameIndex);
//
// --------------------------------------------------------------------------
//	
const TInt KPropertyIDURL = 15;

TBool CHostScriptHandler::TGetPropertyL( const TDesC& aTarget
								, TInt aPropertyID
								, HBufC** aPropertyValue
								, const TDesC& aURLIn 
	    				 		, TPageAccessType aPageAccessType
	    				 		, TBool aIsTrusted)
{

	if(iEngine)
	{					
		struct FI_TargetPropertyInfo targetPropertyInfo;
		
		targetPropertyInfo.pTargetIn = iEngine->CreateFlashString(aTarget);
		targetPropertyInfo.propertyId = aPropertyID;
		
		
		struct FI_BrowserURLInfo browserInfoIn;		

		browserInfoIn.pURLIn         = iEngine->CreateFlashString(aURLIn);
		browserInfoIn.pageAccessType = aPageAccessType;
		browserInfoIn.bIsTrusted     = aIsTrusted;
		
		CHostAllocatorWrapper *allocatorWrapper = new (ELeave) CHostAllocatorWrapper;
		

		char* property = STATIC_CAST(char*,(iEngine->GetFIObject()).FI_TGetProperty(&targetPropertyInfo, 
													&browserInfoIn, 
													allocatorWrapper ));
		
		if( property )
		{
			TInt length = 0;
			
			if(property != NULL) 
			{
				length= strlen(property);	
			}
			if(length)
				{
					TPtrC8 ptr( (TUint8*)property, length );
				
					_LIT8(KLitPercent,"%");
					if( aPropertyID == KPropertyIDURL && ptr.Find(KLitPercent) != KErrNotFound)
					{
						(*aPropertyValue) = iEngine->ConvertEncodingLC(ptr, ETrue, ETrue);
						CleanupStack::Pop();
					}
					else
					{
						HBufC* tempPtr;
						tempPtr = iEngine->CreateEpocString(property,length);
						(*aPropertyValue) = tempPtr;
					}
					
				}
				else
					{					
					allocatorWrapper->FreeWrapper(allocatorWrapper->pContext,property);
					delete allocatorWrapper;
					/* 11173: In case of 'DROP_TARGET' FI_TGetProperty returns 'property' with  
					 * zero length, we have to provide a Null terminated string to browser. 
					 * Else Browser displays 'undefined' string instead of a blank string.
					 */					 
	                *aPropertyValue = HBufC::NewL(1);	                
	                TPtr newPtr = (*aPropertyValue)->Des();
	                newPtr.Copy(KNullTerminate);	                
					return EFalse;
					}
		
		}
		else
		{
			(*aPropertyValue) = NULL;
		}
		allocatorWrapper->FreeWrapper(allocatorWrapper->pContext,property);
		delete allocatorWrapper;
		return ETrue;
	}
	else
	{
		return EFalse;
	}
	
	
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::TGetPropertyAsNumber( const TDesC& aTarget
//							,TInt aPropertyID
//						 	, TDes& /*aPropertyValue*/
//						 	, const TDesC& aURLIn
//						 	, TPageAccessType aPageAccessType
//						 	, TBool aIsTrusted)
//
// For the timeline indicated by target, returns a number indicating 
// the value of the property specified by property. For property, 
// enter the integer corresponding to the desired property. 
// For a list of all properties and their corresponding integers, 
// see Getting and setting properties. 
//
// var framesLoadedIndex = 12; 
//
// var framesLoaded = movie.TGetPropertyAsNumber("/", framesLoadedIndex);
//
//
// --------------------------------------------------------------------------
//

TBool CHostScriptHandler::TGetPropertyAsNumberL( const TDesC& aTarget
							,TInt aPropertyID
						 	, HBufC** aPropertyValue
						 	, const TDesC& aURLIn
						 	, TPageAccessType aPageAccessType
						 	, TBool aIsTrusted)
{
    TBool stat = TGetPropertyL(aTarget,aPropertyID,aPropertyValue
                                ,aURLIn,aPageAccessType,aIsTrusted);
    
    //need to convert here to numbers
    //true to 1 , false to 0, nan to nan
    if(stat && *aPropertyValue)
    {
        TPtr propVal((*aPropertyValue)->Des());
        TLex propertyValue( propVal );
        TInt value;
        TInt isNo = propertyValue.Val(value);
        if( isNo != KErrNone)
        {
            if( KErrNone == propVal.CompareF(KTrueStr) )
                {
                propVal.Copy(KValTrue());
                return stat;
                }
            else if( KErrNone == propVal.CompareF(KFalseStr) )
                {
                propVal.Copy(KValFalse());  
                return stat;
                }
                
            if( propVal.MaxLength() < 3)            
                {
                delete (*aPropertyValue);               
                *aPropertyValue = HBufC::NewL(3);
                }
            TPtr newPtr = (*aPropertyValue)->Des();
            newPtr.Copy(KValNaN);

            return stat;            
        }
        return stat;
    }
    else
    {
        delete (*aPropertyValue);
        *aPropertyValue = HBufC::NewL(3);
        TPtr lNanPtr((*aPropertyValue)->Des());
        lNanPtr.Copy(KValNaN);
        return stat;
    }
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::TGotoFrame(const TDesC& /*aTarget*/
//									, TInt /*aTargetFrame*/
//									)
//
// For the timeline indicated by target, goes to the frame number
// specified by frameNumber. The argument type for target is string. 
// The argument type for frameNumber is integer. 
//     
//  Example : movie.TGotoFrame("/MovieClip", 2);  
//
// --------------------------------------------------------------------------
//
TBool CHostScriptHandler::TGotoFrameL(const TDesC& /*aTarget*/
									, TInt /*aTargetFrame*/
									)
{
	return EFalse;
}


// -------------------------------------------------------------------------
// TBool CHostScriptHandler::TGotoLabel( const TDesC& /*aTarget*/
//									,const TDesC& /*aLabel*/
//									)
//
// For the timeline indicated by target, goes to the frame label 
// specified by label. The argument type for both arguments is string. 
//     
// Example  movie.TGotoLabel("/MovieClip", "MyLabel");  
//
// --------------------------------------------------------------------------
//	
TBool CHostScriptHandler::TGotoLabelL( const TDesC& /*aTarget*/
									,const TDesC& /*aLabel*/
									)
{
	return EFalse;
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::TPlay( const TDesC& /*aTarget*/)
//
//  Plays the timeline specified by target. The argument type is string. 
//     
//  Example  movie.TPlay("/MovieClip");  
//
// --------------------------------------------------------------------------
//	
TBool CHostScriptHandler::TPlayL( const TDesC& /*aTarget*/)
{
	return EFalse;
}
	
// -------------------------------------------------------------------------
// bool CFlashMethods::TSetProperty(const NPVariant *args, 
//						uint32_t argCount, 
//						NPVariant */*result*/)
//
// For the timeline indicated by target, sets the value of the property 
// specified by property to the value specified by value, 
// which can be a string or a number. For property, enter the integer 
// corresponding to the desired property. For a list of all properties
// and their corresponding integers, see Getting and setting properties.
//    
// Example  var visibilityIndex = 7;var nameIndex = 13;
// 			movie.TSetProperty("/MovieClip", visibilityIndex, 1);
//			movie.TSetProperty("/MovieClip", nameIndex, "NewName");    
//
// --------------------------------------------------------------------------
//
	
TBool CHostScriptHandler::TSetPropertyL(	const TDesC& aTarget
							, TInt aPropertyID
						 	, const TDesC& aPropertyValue
						 	, const TDesC& aURLIn 
    						, TPageAccessType aPageAccessType
    						, TBool aIsTrusted )
{
	if(iEngine)
	{
		struct FI_TargetPropertyInfo targetPropertyInfo;
		
		targetPropertyInfo.pTargetIn = iEngine->CreateFlashString(aTarget);
		targetPropertyInfo.propertyId = aPropertyID;
		
		FI_BrowserURLInfo browserInfoIn;

		browserInfoIn.pURLIn         = iEngine->CreateFlashString(aURLIn);
		browserInfoIn.pageAccessType = aPageAccessType;
		browserInfoIn.bIsTrusted     = aIsTrusted;
		
		const char *  propertyValue = iEngine->CreateFlashString(aPropertyValue);
		
		return (iEngine->GetFIObject()).FI_TSetProperty(&targetPropertyInfo, &browserInfoIn, propertyValue);
			
	}
	else
	{
		return EFalse;
	}
		
	
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::TStopPlay( const TDesC& /*aTarget*/ )
//
// Stops the timeline specified by target. The argument type is string. 
//     
// Example  movie.TStopPlay("/MovieClipToStop"); 
//
// --------------------------------------------------------------------------
//	
TBool CHostScriptHandler::TStopPlayL( const TDesC& /*aTarget*/ )
{
	return EFalse;
}



#ifdef FEATURE_EXTERNAL_API

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::CallMethod( const TDesC& aMethod,
//							TDes& aRetVal)
//
// call Actionscript
//     
//
// --------------------------------------------------------------------------
//		
TBool CHostScriptHandler::CallMethodL( const TDesC& aMethod,
							HBufC** aRetVal)
{
	if(iEngine)
	{
		const char* method = iEngine->CreateFlashString(aMethod);
		
		(iEngine->GetFIObject()).FI_CallFunction(method, NULL);
		
		(iEngine->GetFIObject()).FI_DoPlay(EFalse);

		/*Fix for iSource 14518:Draw the updated frame buffer forcefully.
		In some cases ( for ex: Set speed equals to 1 in test case mentioned
		in iSource 14518.) Frame buffer gets updated in single FI_DoPlay call
		and any subsequent call to FI_DoPlay returns FI_FrameBufferNotUpdated.
		In those cases frame buffer updation is done by FI_DoPlay call
		inside FI_CallFunction function and afterwards whenever host calls
		FI_DoPlay (after calling FI_CallFuncion) it returns	FI_FrameBufferNotUpdated
		hence host has to forcefully redraw the frame buffer to resolve this issue.
		
		Note: This is just a workaround and needs to be analysed in detail.*/
		iEngine->ForceRefresh();

		CHostAllocatorWrapper *allocatorWrapper = new (ELeave) CHostAllocatorWrapper;
		
		char* retVal = STATIC_CAST(char*,(iEngine->GetFIObject()).FI_CallFunction(NULL, allocatorWrapper));
		
		TInt length = 0;
		
		if(retVal != NULL)
		{
			length = strlen(retVal);	
		}
		
		HBufC* tempPtr = iEngine->CreateEpocString(retVal,length);
		
		(*aRetVal) = tempPtr;
		
		allocatorWrapper->FreeWrapper(allocatorWrapper->pContext,retVal);
		
		delete allocatorWrapper;
		
		return ETrue;
	}
	else
	{
		return EFalse;
	}
}

TBool  CHostScriptHandler:: SetReturnValueL(const TDesC& aRetVal)
	{
	if(iEngine)
		{
		char* retVal = CreateFlashString(aRetVal);
		(iEngine->GetFIObject()).FI_SetReturnValueFromContainer(retVal);
		delete retVal;
		}
	return ETrue;
	}



#endif


//////////////////////////////////////////////////////////////////////////////
//Flash Properties

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::GetPropAllowScriptAccess(TDes& aAccessLevel)
//
// call Actionscript
//     
//
// --------------------------------------------------------------------------
//
TBool	CHostScriptHandler::GetPropAllowScriptAccess(TInt& aAccessLevel)
{
	if(iEngine)
	{
		aAccessLevel = iEngine->GetFIObject().FI_GetScriptAccess();
	
		return ETrue;
	}
	
	return EFalse;
}
 
// -------------------------------------------------------------------------
// TBool CHostScriptHandler::GetPropBGColor(TInt& aBGColor)
//
// call Actionscript
//     
//
// --------------------------------------------------------------------------
//   
TBool 	CHostScriptHandler::GetPropBGColorL(FI_Color& aBGColor)
{
	if(iEngine)
	{
		iEngine->GetFIObject().FI_GetBackgroundColor(&aBGColor);
		
		return ETrue;
	}
	
	return EFalse;
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::GetPropQuality(TInt& aQuality)
//
// call Actionscript
//     
//
// --------------------------------------------------------------------------
//
TBool	CHostScriptHandler::GetPropQuality(TInt& aQuality)
{
	if(iEngine)
	{
		aQuality = iEngine->GetFIObject().FI_GetQuality();
		
		return ETrue;
	}
	
	return EFalse;
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::SetPropBaseURL(const TDesC& aBaseURL)
//
// call Actionscript
//     
//
// --------------------------------------------------------------------------
//

TBool	CHostScriptHandler::SetPropBaseURL(const TDesC& aBaseURL)
{
	if(iEngine)
	{
		const char* baseURL = iEngine->CreateFlashString(aBaseURL);
			
		iEngine->GetFIObject().FI_SetBaseURL(baseURL);
		
		delete[] (char*)(baseURL);
		
		return ETrue;
	}
	
	return EFalse;
}
// -------------------------------------------------------------------------
// TBool CHostScriptHandler::SetPropAllowScriptAccessL(const TInt& aAccessLevel)
//
// call Actionscript
//     
//
// --------------------------------------------------------------------------
//
TBool	CHostScriptHandler::SetPropAllowScriptAccessL(const TInt& aAccessLevel)
{
	if(iEngine)
	{
		if((aAccessLevel >= 0 ) && (aAccessLevel <= 2 ))
		{
			//iEngine->GetFIObject().FI_SetScriptAccess(aAccessLevel);
		
			iEngine->SetAllowScriptAccess(aAccessLevel);
			return ETrue;	
		}
		
	}
	
	return EFalse;
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::SetPropBGColorL(const FI_Color& aBGColor)
//
// call Actionscript
//     
//
// --------------------------------------------------------------------------
//
TBool 	CHostScriptHandler::SetPropBGColorL(const FI_Color& aBGColor)
{
	if(iEngine)
	{
		iEngine->GetFIObject().FI_SetBackgroundColor(aBGColor.red
													,aBGColor.green
													,aBGColor.blue
													);
		
		return ETrue;
	}
	
	return EFalse;
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::SetPropQuality(const TInt& aQuality)
//
// call Actionscript
//     
//
// --------------------------------------------------------------------------
//
TBool	CHostScriptHandler::SetPropQualityL(const TInt& aQuality)
{
	if(iEngine)
	{
		iEngine->GetFIObject().FI_SetQuality(aQuality);
		
		return ETrue;
	}
	
	return EFalse;
}

// -------------------------------------------------------------------------
// TBool CHostScriptHandler::SetPropLoop(const TInt& aLoop)
//
// call Actionscript
//     
//
// --------------------------------------------------------------------------
//
TBool	CHostScriptHandler::SetPropLoopL(const TInt& aLoop)
{
	if(iEngine)
	{
		iEngine->GetFIObject().FI_SetLooping(aLoop);
		
		return ETrue;
	}
	
	return EFalse;
}


void CHostScriptHandler::DoRunL(const CActive* /*aActive*/)
	{
	
	TInt count = iReqArray.Count();
	if (count >0)
	    {
        CJSRequest*  lthisRequest = iReqArray[0];
        iReqArray.Remove(0);
        ServiceRequestL(lthisRequest);
        delete lthisRequest;
	    }
	count = iReqArray.Count();
	if (count)
	    {
        if(!iAsyncObject->IsActive())
	        {
	        iAsyncObject->Start();  
	        }
	    }    
	}


TInt CHostScriptHandler::DoRunError(const CActive* /*aActive*/, TInt aError)
	{
	return aError;
	}

#ifdef FEATURE_EXTERNAL_API
TBool CHostScriptHandler::CallJSMethodL( const char* aRequest)
	{
	
	CJSRequest* newRequest = CJSRequest::NewL( aRequest , NULL, 
															ECallJSRequest);
	//ServiceRequestL(newRequest);
	
	iReqArray.Append( newRequest );
	if(!iAsyncObject->IsActive())
		{
		iAsyncObject->Start();	
		}
		
	return ETrue;
	}

TBool CHostScriptHandler::EvaluateJavaScriptL( const char* aExpression)
	{

	CJSRequest* newRequest = CJSRequest::NewL( aExpression , NULL, EEvalJSRequest);
	//ServiceRequestL(newRequest);

	iReqArray.Append( newRequest );	
	if(!iAsyncObject->IsActive())
		{
		iAsyncObject->Start();	
		}
	return ETrue;
	}

TBool CHostScriptHandler::DoFsCommandL(const char* aCommand, const char* aArgs)
	{
	
	CJSRequest* newRequest = CJSRequest::NewL( aCommand,aArgs,EFsCommandRequest);
	
//	ServiceRequestL(newRequest);
	
	iReqArray.Append( newRequest );
	if(!iAsyncObject->IsActive())
		{
		iAsyncObject->Start();	
		}
	return ETrue;
	}

void CHostScriptHandler::ServiceRequestL( CJSRequest* aRequest )
	{
	if ( aRequest->GetRequestType() == EFsCommandRequest )
		{
		GetPluginScriptHandler()->DoFsCommandL(iEngine->GetMovieId(), aRequest->GetCommand(), aRequest->GetArgs());
		}
	else if( aRequest->GetRequestType()== EEvalJSRequest )	
		{
		GetPluginScriptHandler()->EvaluateJavaScript( aRequest->GetCommand() );
		}
	else if( aRequest->GetRequestType() == ECallJSRequest )
		{
		GetPluginScriptHandler()->CallMethodL( aRequest->GetCommand() );
		}
	}
#endif	//FEATURE_EXTERNAL_API
//------------------------------------------------------------------------------	
//------------------------------------------------------------------------------	


CJSRequest* CJSRequest::NewL(const char* aCommand, const char* aArgs, 
										TJSRequestType aJSRequestType)
	{
	
	CJSRequest* self = new (ELeave) CJSRequest();
	CleanupStack::PushL(self);
	self->ConstructL(aCommand,aArgs,aJSRequestType);
	CleanupStack::Pop(self);
	return self;
	
	}
	
CJSRequest::~CJSRequest()
	{
	if(iArgs)
		{
		delete [] iArgs;
		iArgs = NULL;
		}
	if(iCommand)	
		{
		delete [] iCommand;
		iCommand = NULL;
		}
	}
		
		
TJSRequestType CJSRequest::GetRequestType()
	{
	return iJSRequestType;
	}
		
const char* CJSRequest::GetCommand()
	{
	return iCommand;
	}
		
const char* CJSRequest::GetArgs()
	{
	return iArgs;
	}	
	
CJSRequest::CJSRequest():iCommand(NULL), iArgs(NULL)
	{
	
	}
void CJSRequest::ConstructL(const char* aCommand, const char* aArgs, 
												TJSRequestType aJSRequestType)
	{
	if( aCommand )
		{
		iCommand = new (ELeave) char[strlen(aCommand)+ 1];
		FlashStrCpy(iCommand, aCommand);
		}
	if( aArgs )
		{
		iArgs = new (ELeave) char[strlen(aArgs) + 1];
		FlashStrCpy(iArgs, aArgs);
		}
	iJSRequestType = aJSRequestType;
	}
//end of file
