/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that provides a 
*				 simple interface to the HTTP Client API
*
*/


#include "platformbuild.h"
#include "symbian_clib.h"
#include "platformtypes.h"
#include "saplayer_build.h"

#include "saflash_consts.h"
#include "symbian_engine.h"
#include "symbian_http.h"
#include "symbian_file.h"
#include "symbian_ui_interface.h"
#include "symbian_urlrequestinfo.h"

#include <avkon.hrh>
#include <aknnotewrappers.h>
#include <uri8.h>

#include <http.h>
#include <chttpformencoder.h>
#include <httpstringconstants.h>
#include <http/rhttptransaction.h>
#include <http/rhttpsession.h>
#include <http/rhttpheaders.h>
#include <browserdialogsprovider.h>
#include <utf.h>
#include <uaproffilter_interface.h>

#include "symbian_useragent.h"
#include <escapeutils.h>

const TUint KMaxNumHeaders = 20;

_LIT8(KLitHttp,"http://");
_LIT8(KLitHttps,"https://");
_LIT8(KLitFtp,"ftp://");
void CHttpRequestInfo::CompleteConstructL()
{
	const TChar KPortSeparator(':');
	const TChar KForwardSlash(KCharForwardSlash);
	
	/// now construct the URL...

	TPtrC8 hostNamePtr;
	if (iStreamInfo->hostName)
		hostNamePtr.Set((TUint8*)iStreamInfo->hostName);

	TPtrC8 docPathPtr;
	if (iStreamInfo->documentPath)
		docPathPtr.Set((TUint8*)iStreamInfo->documentPath);

	const TInt lenCalculated=hostNamePtr.Length()+docPathPtr.Length()+20; // {8 - https://} + { 11 - :portnumber } + { 1- / }
	iUrl=HBufC8::NewL(lenCalculated);
	TPtr8 urlPtr=iUrl->Des();

	TPtrC8 protocol;
	if (iStreamInfo->protocolSchema)
		protocol.Set((TUint8*)iStreamInfo->protocolSchema);

	if( ! (	(0==protocol.Compare(KLitHttp)) ||
			(0==protocol.Compare(KLitHttps)) ||
			(0==protocol.Compare(KLitFtp)) ) )
	{
		User::Leave(KErrNotSupported);
	}

	urlPtr.Append(protocol);
	urlPtr.Append(hostNamePtr);
	
	if (iStreamInfo->port != kDEFAULT_PORT_HTTP)
	{
		urlPtr.Append(KPortSeparator);
		TUint32 port32=iStreamInfo->port; // fixes a warning
		urlPtr.AppendNum(port32,EDecimal);
	}
	
	if ( iStreamInfo->documentPath && iStreamInfo->documentPath[0] != KCharForwardSlash )
	{
		urlPtr.Append(KForwardSlash);
	}
	
	urlPtr.Append(docPathPtr);
	
	// done constructing URL...

	if(iStreamInfo->postData)
	{
		//contentLength contains the length of the postData
		const TPtrC8 postPtr((TUint8*)iStreamInfo->postData, iStreamInfo->contentLength);
		iPostData=postPtr.AllocL();
	}

	if(iStreamInfo->additionalHeaders)
	{
		const TPtrC8 addtnPtr((TUint8*)iStreamInfo->additionalHeaders);
		iAdditionalHdrs=addtnPtr.AllocL();
	}
}



// Constants
_LIT8(KFieldSeparator, "\r\n");
_LIT8(KContentTypeNormal, "application/x-www-form-urlencoded");
_LIT8(KNoCache, "no-cache");
_LIT8(KNoStore, "no-store");
#if defined(FEATURE_PERFORMANCE_AUTOMATION) || defined(FEATURE_TESTAUTOMATION)
const TInt KConnectionTimeoutForPerfTest=36000000;//6 mins 
#endif //FEATURE_PERFORMANCE_AUTOMATION
const TInt KConnectionTimeout=30000000; //30 secs
const TInt KRTMPTCloseTimeout=1*1000*1000;  // 1 second

// enum describing current state of the HTTP engine.
// last transaction is remembered until a new one starts
// and is or'd with the current network connection status.

enum TNetworkAccessConnCode
{
	EFlhNReqNotConnectedToNetwork	=0x0,	// there is no current network connection
	EFlhNReqConnectingToNetwork		=0x1,	// connecting
	EFlhNReqConnectedToNetwork		=0x2,	// connected
};

enum TNetworkAccessStatusCode
{
	EFlhNReqNoTransactionsMade		=0x0,
	EFlhNReqNoTransactionInfo		=0x1,
	EFlhNReqNoDataSentRecvd			=0x2,	// resolving, getting first data etc...
	EFlhNReqReceivedHeaders			=0x3,	// headers are here
	EFlhNReqProcessingHeaders		=0x4,	// headers being processed
	EFlhNReqFetchingBody			=0x5,	// body being fetched
	EFlhNReqPostingData				=0x6,  // post data being supplied
	EFlhNReqCanceled				=0x7,	// request was canceled before completion
	EFlhNReqCompletedNormally		=0x8,	// request completed normally
	EFlhNReqFailedConnect			=0x9, // a network connection could not be established or dropped
	EFlhNReqFailedError				=0x10, // there was a non network related error fetching the body or headers
	EFlhNReqFailedHTTP				=0x11	// there was an HTTP error processing the request	
};

// ----------------------------------------------------------------------------
// CHttpEngine::NewL()
//
// Creates instance of CHttpEngine.
// ----------------------------------------------------------------------------
CHttpEngine* CHttpEngine::NewL(CPlatformFlashEngine* aEngine)
{
	CHttpEngine* self = new (ELeave) CHttpEngine(aEngine);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
    return self;
}

// ----------------------------------------------------------------------------
// CHttpEngine::CHttpEngine()
//
// First phase constructor.
// ----------------------------------------------------------------------------
CHttpEngine::CHttpEngine(CPlatformFlashEngine* aEngine)	
:iEngine(aEngine)
,iRequestQueue( _FOFF(CHttpRequestInfo,iLinkItem))
,iRTMPTCloseWaitTimer(NULL)
,iRTMPTCloseWait(NULL)
,iPreparingToExit(EInitial)
{
}


// ----------------------------------------------------------------------------
// CHttpEngine::~CHttpEngine()
//
// Destructor.
// ----------------------------------------------------------------------------
CHttpEngine::~CHttpEngine()
{
	// Cancel any pending transaction and clean up items related to current transaction
	CloseAllTransactions();
	delete iDlgProvider;
	delete iTimer;
	delete iUserName;
	
	if(iUserAgentInfo)
	{
		delete iUserAgentInfo;
		
		iUserAgentInfo = NULL;
		
	}
	iIPConnectionMgr = NULL;
	
	if(iConnHandler)
	{
		delete iConnHandler;
		iConnHandler = NULL;
	}
	RemoveAllCompletedRequests();

	delete iRTMPTCloseWait;
	delete iRTMPTCloseWaitTimer;

}

void CHttpEngine::ResetEngine()
{
	CloseAllTransactions();
	iTransStat=EFlhNReqNoTransactionsMade;
	
    CloseConnectionNow();
	
	iPreparingToExit = EInitial;
}


// ----------------------------------------------------------------------------
// CHttpEngine::ConstructL()
//
// Second phase construction.
// ----------------------------------------------------------------------------
void CHttpEngine::ConstructL()
{
	iDlgProvider = CBrowserDialogsProvider::NewL( this );
	iIPConnectionMgr = iEngine->GetIPConnectionMgr();
	iTimer=CFlhTimerObject::NewL(CActive::EPriorityIdle,*this);

	
	iRetryAuthentication = EFalse;
	iUserName = NULL;
	iUserAgentInfo = CUserAgentInfo::NewL();
	
}

// ----------------------------------------------------------------------------
// CHttpEngine::AddCustomHeaderL()
//
// Used to set a user-defined header value to HTTP request
// ----------------------------------------------------------------------------
void CHttpEngine::AddCustomHeaderL(RHTTPHeaders& aHeaders,
								   const TDesC8& aHdrValue)
{
	TInt colon = aHdrValue.Locate(':');
	
	if (colon != KErrNotFound)
	{
		TPtrC8 name(aHdrValue.Left(colon++));

		// Skip any white spaces		
		while (aHdrValue[colon++] == ' ')
		{
		// do nothing		
		}

		TPtrC8 value(aHdrValue.Right(aHdrValue.Length() - (colon-1)));

		// Add the custom header
		RStringF valStr = iSession.StringPool().OpenFStringL(value);
		CleanupClosePushL(valStr);

		RStringF nameStr = iSession.StringPool().OpenFStringL(name);
		CleanupClosePushL(nameStr);

		aHeaders.RemoveField(nameStr);

		aHeaders.SetRawFieldL(nameStr, value, KFieldSeparator);
		
		CleanupStack::PopAndDestroy(&nameStr);
		CleanupStack::PopAndDestroy(&valStr);
	}
	else
	{
		aHeaders.RemoveField(iSession.StringPool().StringF(HTTP::EContentType, RHTTPSession::GetTable()));
		
		SetHeaderL(aHeaders, HTTP::EContentType, aHdrValue);
	}
}

// ----------------------------------------------------------------------------
// CHttpEngine::SetHeaderL()
//
// Used to set header value to HTTP request
// ----------------------------------------------------------------------------
void CHttpEngine::SetHeaderL(RHTTPHeaders aHeaders, 
							 TInt aHdrField, 
							 const TDesC8& aHdrValue)
{
	RStringF valStr = iSession.StringPool().OpenFStringL(aHdrValue);
	CleanupClosePushL(valStr);

	THTTPHdrVal val(valStr);
	aHeaders.SetFieldL(iSession.StringPool().StringF(aHdrField, RHTTPSession::GetTable()), val);
	CleanupStack::PopAndDestroy(&valStr);
}


// ----------------------------------------------------------------------------
// CHttpEngine::IssueHTTPGetL()
//
// Start a new HTTP GET transaction.
// ----------------------------------------------------------------------------
void CHttpEngine::IssueHTTPGetL(const CHttpRequestInfo& aInfo)
{
	// Parse string to URI (as defined in RFC2396)
	TUriParser8 uri;
	uri.Parse(aInfo.Url());

	// Get request method string for HTTP GET
	RStringF method = iSession.StringPool().StringF(HTTP::EGET, RHTTPSession::GetTable());

	// Open transaction with previous method and parsed uri. This class will
	// receive transaction events in MHFRunL and MHFRunError.
	iTransaction = iSession.OpenTransactionL(uri, *this, method);

	// Set headers for request; user agent and accepted content type
	RHTTPHeaders hdr = iTransaction.Request().GetHeaderCollection();
	SetHeaderL(hdr, HTTP::EUserAgent, iUserAgentInfo->GetUserAgentL());
	SetHeaderL(hdr, HTTP::EAccept, KAccept);
	SetHeaderL(hdr, HTTP::ECacheControl, KNoCache);
	SetHeaderL(hdr, HTTP::ECacheControl, KNoStore);

	// Submit the transaction. After this the framework will give transaction
	// events via MHFRunL and MHFRunError.
	iTransaction.SubmitL();

}


// ----------------------------------------------------------------------------
// CHttpEngine::IssueHTTPPostL()
//
// Start a new HTTP POST transaction.
// ----------------------------------------------------------------------------
void CHttpEngine::IssueHTTPPostL(const CHttpRequestInfo& aInfo) 
{
	// Parse string to URI
	TUriParser8 uri; 
	uri.Parse(aInfo.Url());

	// Get request method string for HTTP POST
	RStringF method = iSession.StringPool().StringF(HTTP::EPOST, RHTTPSession::GetTable());

	// Open transaction with previous method and parsed uri. This class will 
	// receive transaction events in MHFRunL and MHFRunError.
	iTransaction = iSession.OpenTransactionL(uri, *this, method);
	
	// Set headers for request; user agent, accepted content type and body's 
	// content type.
	RHTTPHeaders hdr = iTransaction.Request().GetHeaderCollection();
	SetHeaderL(hdr, HTTP::EUserAgent, iUserAgentInfo->GetUserAgentL());
	SetHeaderL(hdr, HTTP::EAccept, KAccept);

	const TDesC8* additionalHeaders=aInfo.AdditionalHeaders();
	// If user specifies custom user-defined headers.. process them individually
	if (additionalHeaders && (additionalHeaders->Length() > 0))
	{
		TPtrC8 tmpHeaders(*additionalHeaders);
		TInt length = additionalHeaders->Length();
		TInt endPos = 0;
		TInt lenRemaining;

		FOREVER
		{
			length = tmpHeaders.Length();
			endPos = tmpHeaders.Find(KFieldSeparator);
			
			if (endPos == KErrNotFound)
			{
				// this is the last one...
				endPos = length;
			}

			TPtrC8 token(tmpHeaders.Left(endPos));

			AddCustomHeaderL(hdr, token);

			lenRemaining = length - (endPos+1);
			if (lenRemaining > 0)
			{
				tmpHeaders.Set(tmpHeaders.Right(lenRemaining-1));
			}
			else
			{
				break;
			}
		}
	}
	else
	{
		SetHeaderL(hdr, HTTP::EContentType, KContentTypeNormal);
	}

	
	const TDesC8* postData=aInfo.PostData();

	if (postData && postData->Length())
	{
		TBuf8<16> contentLength;
		contentLength.AppendNum(postData->Length());
		contentLength.ZeroTerminate();
		SetHeaderL(hdr, HTTP::EContentLength, contentLength);
	}
	
	// Set this class as an data supplier. Inherited MHTTPDataSupplier methods
	// are called when framework needs to send body data.
	MHTTPDataSupplier* dataSupplier = this;
	iTransaction.Request().SetBody(*dataSupplier);

	// Submit the transaction. After this the framework will give transaction
	// events via MHFRunL and MHFRunError.
	iTransaction.SubmitL();
	
}

// ----------------------------------------------------------------------------
// CHttpEngine::CancelActiveTransaction()
//
// Cancels currently running transaction and frees resources related to it.
// ----------------------------------------------------------------------------
void CHttpEngine::CancelActiveTransaction()
{
	if(!iCurrentRequest)
		return;
	
	if(iCurrentRequest->IsRTMPTClose() && (iPreparingToExit == EPreparingToExit))
    {
        TRAP_IGNORE(DoSynchronousRTMPTCloseL());
        //At this point, iCurrentRequest would already be deleted.
    }
    else
    {
		iCurrentRequest->HandleRequestFailed(KErrCancel,KFlashLiteHttpClientSideCancelErr); 
		delete iCurrentRequest;
		iCurrentRequest = NULL;
	
		if (iHandlingEventCallback)
		{
			iCurrentRequestDeleted = ETrue;
		}
	
		// Close() also cancels transaction (Cancel() can also be used but 
		// resources allocated by transaction must be still freed with Close())
		iTransaction.Close();
	
		iTransStat=EFlhNReqCanceled;
    }
}

char* CreateFlashStringLC(const TDesC8& aSource)
{
	char* retVal=CreateFlashString(aSource);
	User::LeaveIfNull(retVal);
	CleanupStack::PushL(retVal);
	return retVal;
}

TInt CHttpEngine::ParseHttpHeadersL(RHTTPTransaction& aTrans, FI_URLStreamHeaderInfo& aHeaderInfo)
{
	TInt numItemsOnCleanupStack=0; // not ideal, but at least it works

	RHTTPResponse resp = aTrans.Response();
	RStringPool strP = aTrans.Session().StringPool();
	RHTTPHeaders hdr = resp.GetHeaderCollection();
	THTTPHdrFieldIter it = hdr.Fields();
	
	TBuf<KMaxHeaderNameLen>  fieldName16;
	
	aHeaderInfo.numHeaders = 0;
	

	while (it.AtEnd() == EFalse)
	{
		RStringTokenF fieldName = it();
		RStringF fieldNameStr = strP.StringF(fieldName);
		THTTPHdrVal fieldVal;
		
		if (hdr.GetField(fieldNameStr,0,fieldVal) == KErrNone)
		{
			const TDesC8& fieldNameDesC = fieldNameStr.DesC();
			fieldName16.Copy(fieldNameDesC.Left(KMaxHeaderNameLen));
			_LIT8( KContentLength,"Content-Length" );
			if( fieldNameDesC.CompareF(KContentLength) == 0)
			{
				iCurrentRequest->SetContentLength(fieldVal.Int());
			}
			// Copy the field name
			aHeaderInfo.headerNames[aHeaderInfo.numHeaders] = CreateFlashStringLC(fieldNameDesC);
			numItemsOnCleanupStack++;

			// Copy the field values
			switch (fieldVal.Type())
			{
			case THTTPHdrVal::KTIntVal:
				{
					// Is there an easier way to convert fieldVal.Int() to "allocated" char*
					TBuf8<30> contentLength;
					contentLength.AppendNum(fieldVal.Int());
					aHeaderInfo.headerValues[aHeaderInfo.numHeaders] = CreateFlashStringLC(contentLength);
					numItemsOnCleanupStack++;
				}
				break;

			case THTTPHdrVal::KStrFVal:
				{
					RStringF fieldValStr = strP.StringF(fieldVal.StrF());
					const TDesC8& fieldValDesC = fieldValStr.DesC();
					aHeaderInfo.headerValues[aHeaderInfo.numHeaders]=CreateFlashStringLC(fieldValDesC);
					numItemsOnCleanupStack++;
				}
				break;

			case THTTPHdrVal::KStrVal:
				{
					RString fieldValStr = strP.String(fieldVal.Str());
					const TDesC8& fieldValDesC = fieldValStr.DesC();
					aHeaderInfo.headerValues[aHeaderInfo.numHeaders]=CreateFlashStringLC(fieldValDesC);
					numItemsOnCleanupStack++;
				}
				break;

			case THTTPHdrVal::KDateVal:
				{
					TDateTime date = fieldVal.DateTime();
					TBuf<KMaxLongDateFormatSpec + KMaxTimeFormatSpec> dateTimeString;
					TTime t(date);
					t.FormatL(dateTimeString,KDateFormat);
					
					aHeaderInfo.headerValues[aHeaderInfo.numHeaders]=CreateFlashString(dateTimeString);
					User::LeaveIfNull(aHeaderInfo.headerValues[aHeaderInfo.numHeaders]);
					CleanupStack::PushL(aHeaderInfo.headerValues[aHeaderInfo.numHeaders]);
					numItemsOnCleanupStack++;
				} 
				break;
				
			default:
				break;
			}
		}

		if (aHeaderInfo.numHeaders++ >= KMaxNumHeaders)
		{
			break;
		}

		++it;
	}
	return numItemsOnCleanupStack;
}

void CHttpEngine::HandleHeaderResponseL(RHTTPTransaction& aTransaction)
{
	// HTTP response headers have been received. We can determine now if there is
	// going to be a response body to save.
	RHTTPResponse resp = aTransaction.Response();
	TInt status = resp.StatusCode();
			
	if (resp.HasBody() && (status >= HTTPStatus::EOk)
			&& (status < HTTPStatus::EMultipleChoices) 
				&& (status != HTTPStatus::ENoContent))
	{

		// retrieve the final URI that handled the response...
		const TDesC8& finalURI=aTransaction.Request().URI().UriDes();

		//copy it so we can append a null terminator
		HBufC8* ztDes=HBufC8::NewLC(finalURI.Length()+1); // 1 for zero terminator
		TPtr8 ztPtr(ztDes->Des());
		ztPtr.Copy(finalURI);
		ztPtr.ZeroTerminate();

		unsigned int i;
		FI_URLStreamHeaderInfo headerInfo;
		headerInfo.numHeaders   = KMaxNumHeaders;
		headerInfo.headerNames	= new(ELeave) char*[KMaxNumHeaders];
		CleanupStack::PushL(headerInfo.headerNames);
		headerInfo.headerValues = new(ELeave) char*[KMaxNumHeaders];
		CleanupStack::PushL(headerInfo.headerValues);
					
		for (i = 0 ; i < headerInfo.numHeaders; i ++)
		{
			headerInfo.headerNames[i] = NULL;
			headerInfo.headerValues[i] = NULL;
		}

		// Fill the headers
		TInt itemsRequiringCleanup=ParseHttpHeadersL( aTransaction, headerInfo);

		// let the request do the right thing with it
		iTransStat=EFlhNReqProcessingHeaders;
		iCurrentRequest->HandleHeaderResponseL(headerInfo, ztPtr);

		CleanupStack::PopAndDestroy(3+itemsRequiringCleanup,ztDes);
	}
	// If client error then ignore it.
	else if (!HTTPStatus::IsClientError(status))
	{
	    iDlgProvider->DialogNotifyHttpErrorL(status,KNullDesC);
	}
	else
	{
		User::Leave(KErrGeneral);
	}
}

void CHttpEngine::HandleResponseDataL(RHTTPTransaction& aTransaction)
{
	// Get the body data supplier
	RHTTPResponse resp = aTransaction.Response();
	MHTTPDataSupplier* body = resp.Body();
			
	// Write data
	TPtrC8 dataPart;
	body->GetNextDataPart(dataPart);

	TBool consumed=iCurrentRequest->HandleResponseDataL(dataPart);
	
	// Done with that bit of body data
	// Need to check request is still valid, since if player has been aborted the stream will
	// already have been deleted!
	if (iCurrentRequestDeleted)
	{
		iCurrentRequestDeleted = EFalse;
	}
	else
	{
		if ( consumed )
		{
			body->ReleaseData();
		}
		else if ( iCurrentRequest )
		{
			//Fix
			//iCurrentRequest->SetPaused(body);
		}
	}
}

void CHttpEngine::HandleCompleteResponseL(RHTTPTransaction& aTransaction)
{
	iCurrentRequest->HandleResponseCompleteL(aTransaction.Response().StatusCode());
}


// ----------------------------------------------------------------------------
// CHttpEngine::MHFRunL()
//
// Inherited from MHTTPTransactionCallback
// Called by framework to pass transaction events.
// ----------------------------------------------------------------------------
void CHttpEngine::MHFRunL(RHTTPTransaction aTransaction, 
						  const THTTPEvent& aEvent)
{
	CPlatformFlashEngine* pEngine = iCurrentRequest->MM_Object()->pHostObj->iFIInterface->GetEngine();

	TBool rtmptCloseCompleted	= EFalse;
	iHandlingEventCallback = ETrue;
	iRetryAuthentication = EFalse;

	switch (aEvent.iStatus) 
	{
	case THTTPEvent::EGotResponseHeaders:
		{
			iTransStat=EFlhNReqReceivedHeaders;
			// Got the headers
			HandleHeaderResponseL(aTransaction);
			iTransStat=EFlhNReqFetchingBody;
		}
		break;

	case THTTPEvent::EGotResponseBodyData:
		{ 
			HandleResponseDataL(aTransaction);
		}
		break;

	case THTTPEvent::EResponseComplete:
		{
			iTransStat=EFlhNReqCompletedNormally;
			HandleCompleteResponseL(aTransaction);
		}
		break;

	case THTTPEvent::ESucceeded:
		{
			iTransStat=EFlhNReqCompletedNormally;
			if(iCurrentRequest->IsRTMPTClose())
			{
				rtmptCloseCompleted = ETrue;
			}
			TransactionOverQueueNextL(aTransaction);
		}
		break;
    case THTTPEvent::ERedirectedPermanently:
    case THTTPEvent::ERedirectedTemporarily:
                {
                TUriParser8 decodeUri;
                HBufC8* tmp=EscapeUtils::EscapeDecodeL(aTransaction.Request().URI().UriDes());
                CleanupStack::PushL(tmp);
                decodeUri.Parse(tmp->Des());
                
                aTransaction.Request().SetURIL(decodeUri);
                CleanupStack::PopAndDestroy(tmp);                        

                break;
                }

	case THTTPEvent::EFailed:
		{
			// in this case it is usually a communications error of some form
			iCurrentRequest->HandleRequestFailed(KErrGeneral, aTransaction.Response().StatusCode());
			if(!(pEngine->IsBrowserPluginActive()))
			{
			    //pEngine->GetUI()->NotifyDataAccessError(NULL, ECompleteFailedHTTP);
			    pEngine->NotifyDataAccessError(NULL, ECompleteFailedHTTP);
			}			
			iTransStat=EFlhNReqFailedHTTP;
			if(iCurrentRequest->IsRTMPTClose())
			{
				rtmptCloseCompleted = ETrue;
			}
			TransactionOverQueueNextL(aTransaction);
		}
		break;
		
	case THTTPEvent::EUnrecoverableError:
	case THTTPEvent::ETooMuchRequestData:
		{
			// these others are effectively OOM errors since there is not enough space, either in or outbound
			if(!(pEngine->IsBrowserPluginActive()))
			{
				pEngine->NotifyClientError(EInsufficientMemoryError,ETrue);	
			}			
			iTransStat=EFlhNReqFailedError;
			if(iCurrentRequest->IsRTMPTClose())
			{
				rtmptCloseCompleted = ETrue;
			}
			TransactionOverQueueNextL(aTransaction);
		}
		break;

	default:
		// Ignore other HTTP events
		break;
	}

	iHandlingEventCallback = EFalse;

    if(rtmptCloseCompleted)
    {
    	RTMPTCloseCompleted();
    }
}


// ----------------------------------------------------------------------------
// CHttpEngine::MHFRunError()
//
// Inherited from MHTTPTransactionCallback
// Called by framework when *leave* occurs in handling of transaction event.
// These errors must be handled, or otherwise HTTP-CORE 6 panic is thrown.
// ----------------------------------------------------------------------------
TInt CHttpEngine::MHFRunError(TInt aError, 
							  RHTTPTransaction aTransaction, 
							  const THTTPEvent& /*aEvent*/)
{
	// note this function will only be called on a leave from MHLRunL() if the trasaction is not already closed.
	// if the transaction is closed (i.e. the player closed it because of an error during an FI_ call)
	// this will not get called - so there is no risk of closing it twice...

	TBool rtmptCloseCompleted = iCurrentRequest->IsRTMPTClose();

	iCurrentRequest->HandleRequestFailed(aError, aTransaction.Response().StatusCode());
	iTransStat=EFlhNReqFailedError;

	CPlatformFlashEngine* pEngine = iCurrentRequest->MM_Object()->pHostObj->iFIInterface->GetEngine();

#ifdef LOG_MESSAGE
	_LIT8(KLitlogMsg,"CHttpEngine::MHFRunError:");
	TRAP_IGNORE(pEngine->WriteLogMsgWithIntL((const char*)KLitlogMsg().Ptr(), aError));
#endif

	if(!(pEngine->IsBrowserPluginActive()))
	{
		if(aError==KErrNoMemory)
		{
		    pEngine->NotifyClientError(EInsufficientMemoryError,ETrue);
		    //pEngine->GetUI()->NotifyClientError(EInsufficientMemoryError,ETrue);
		}
		else if (aError!= KErrNone && aError!= KErrGeneral)
		{
		    //pEngine->GetUI()->NotifyDataAccessError(NULL, ECompleteFailedError); // cannot leave here
		    pEngine->NotifyDataAccessError(NULL, ECompleteFailedError); // cannot leave here
		}
	}
	
	TRAPD( err, TransactionOverQueueNextL(aTransaction) );
	
	if(rtmptCloseCompleted)
	{
		RTMPTCloseCompleted();
	}
	return err;
}

void CHttpEngine::TransactionOverQueueNextL(RHTTPTransaction& aTransaction)
{
    // Transaction can be closed now, if not already closed via CancelActiveTransaction()
    if(iTransStat != EFlhNReqCanceled)
    {
        aTransaction.Close();
        // May be required for further seeking
        AddToCompletedRequestsIfFlvL( iCurrentRequest );
    }
    iCurrentRequest=NULL;
    
    // if there are any more waiting in line, service them now...
    TryQueueNextRequest();
}

// ----------------------------------------------------------------------------
// CHttpEngine::GetNextDataPart()
//
// Inherited from MHTTPDataSupplier
// Called by framework when next part of the body is needed. In this example 
// this provides data for HTTP post.
// ----------------------------------------------------------------------------
TBool CHttpEngine::GetNextDataPart(TPtrC8& aDataPart)
{
	iTransStat=EFlhNReqPostingData;
	// presently we support only the notion of passing all post data at once...

	const TDesC8* postData=iCurrentRequest->PostData();
	
	if(!postData)
		postData=&KNullDesC8;
	
	
	aDataPart.Set(*postData);
	
	return ETrue;
}


// ----------------------------------------------------------------------------
// CHttpEngine::ReleaseData()
//
// Inherited from MHTTPDataSupplier
// Called by framework. Allows us to release resources needed for previous
// chunk. (e.g. free buffers)
// ----------------------------------------------------------------------------
void CHttpEngine::ReleaseData()
{
	// don't release the post data because we might need to send it again if a
	// Reset() happens!
}


// ----------------------------------------------------------------------------
// CHttpEngine::Reset()
//
// Inherited from MHTTPDataSupplier
// Called by framework to reset the data supplier. Indicates to the data 
// supplier that it should return to the first part of the data.	
// In practise an error has occured while sending data, and framework needs to 
// resend data.
// ----------------------------------------------------------------------------
TInt CHttpEngine::Reset()
{
	// we still have the post data so we are able to supply it again with no problem
	return KErrNone;
}


// ----------------------------------------------------------------------------
// CHttpEngine::OverallDataSize()
//
// Inherited from MHTTPDataSupplier
// Called by framework. We should return the expected size of data to be sent.
// If it's not know we can return KErrNotFound (it's allowed and does not cause
// problems, since HTTP protocol allows multi-part bodys without exact content 
// length in header).
// ----------------------------------------------------------------------------
TInt CHttpEngine::OverallDataSize()
{
	// we always know the length of our data...
	TPtrC8 data;
	GetNextDataPart(data);
	return data.Length();
}


void CHttpEngine::TryQueueNextRequest()
{
	if(!iCurrentRequest)
	{
		// keep firing off requests until one successfully starts.
		// In CloseAllTransaction(), the rtmpt close calls are copied to a local queue. Hence iRequestQueue *is* empty
		while(!iRequestQueue.IsEmpty() && !iCurrentRequest)
		{
			// get first in line for running
			CHttpRequestInfo* head=iRequestQueue.First();
			#if defined(FEATURE_ATS_AUTOMATION) || defined(FEATURE_PERFORMANCE_AUTOMATION) || defined(FEATURE_TESTAUTOMATION)
				iPerfTestMode = head->MM_Object()->pHostObj->iFIInterface->GetEngine()->iPerfTestMode; //check for test mode
			#endif
			iRequestQueue.Remove(*head);
			TRAPD(error,DoIssueNextRequestL(*head));

			if(error)
			{
				head->HandleRequestFailed(error,KFlashLiteHttpClientSideCancelErr);
				iTransStat=EFlhNReqFailedError; // could not even start the request...
				head->MM_Object()->pHostObj->iFIInterface->GetEngine();
				delete head;
			}
			else
			{
				iCurrentRequest=head;
				iTransStat=EFlhNReqNoDataSentRecvd; // HTTP stack is now resolving the URL etc...
				
				if(iCurrentRequest->IsRTMPTClose() && iPreparingToExit == EPreparingToExit)
				{
					TRAP_IGNORE(DoSynchronousRTMPTCloseL());
				}
			}
		}
		
		/*
		if(pEngine && !(pEngine->IsBrowserPluginActive())) // will only be set if something fails...
		{
			pEngine->GetUI()->NotifyDataAccessError(NULL, ECompleteFailedHTTP); 
		}
		
		if(pEngine)
		{
			pEngine->NotifyDataAccessError(NULL, ECompleteFailedHTTP);
		}
		*/
		if(!iCurrentRequest && iRequestQueue.IsEmpty() && (!iTimer->IsActive()))
		{
			#if defined(FEATURE_PERFORMANCE_AUTOMATION) || defined(FEATURE_TESTAUTOMATION)
				if(iPerfTestMode)	
					iTimer->After(KConnectionTimeoutForPerfTest);
				else 
			iTimer->After(KConnectionTimeout);
			#else
					iTimer->After(KConnectionTimeout);
			#endif
		}
	}
}

void CHttpEngine::DoRunL(const CActive* /*aActive*/)
{
	CloseConnectionNow();
}

TInt CHttpEngine::DoRunError(const CActive* /*aActive*/, TInt aError)
{
	return aError; // never should happen in this case as our RunL() cannot leave...
}


void CHttpEngine::CloseConnectionNow()
{
	if(iTimer->IsActive()) // we were already shutting down the connection...
	{
		iTimer->Cancel();
	}

	iSession.Close();
	if(iConnHandler)
	{
		delete iConnHandler;
		iConnHandler = NULL;
	}
	
	iConnStat=EFlhNReqNotConnectedToNetwork;
}

void CHttpEngine::DoIssueNextRequestL(CHttpRequestInfo& aInfo)
{
	aInfo.PrepareToBeServicedL();

	const TDesC8* postData=aInfo.PostData();
	
	if(postData /*|| headers*/)
	{
		IssueHTTPPostL(aInfo);
	}
	else
	{
		IssueHTTPGetL(aInfo);
	}
}


unsigned short CHttpEngine::CancelHttpRequest(void* aOwner, unsigned long aId)
{

	if (iCurrentRequest && iCurrentRequest->IsMe(aOwner,aId))
	{
		// terminate the current transaction
		CancelActiveTransaction();
		// queue up the next request...
		TryQueueNextRequest();

	}
	else // maybe its still queue'd ?
	{
		CHttpRequestInfo* item=GetItemFromQueue(aOwner,aId);

		if(item)
		{
			item->HandleRequestFailed(KErrCancel,KFlashLiteHttpClientSideCancelErr); 
			iRequestQueue.Remove(*item);
			delete item;
		}
		else
		{
			//changed as "this" won't be correct parameter for IsMe(void* owner, streamID) used inside function
			//and expects iMMObject as Owner
			DeleteObjectFromCompletedRequestsQueue(aOwner, aId);
		} 
			
	}
	return 1;
}


CHttpRequestInfo* CHttpEngine::GetItemFromQueue(void* aOwner, TUint32 aId)
{
	TSglQueIter<CHttpRequestInfo> iter(iRequestQueue);
		
	CHttpRequestInfo* item=iter;

	while(item)
	{
		if(item->IsMe(aOwner, aId))
		{
			return item;
		}
		item=iter++;
	}
	return NULL;
}


CHttpRequestInfo* CHttpEngine::GetRequestObject(void* aOwner,TUint32 aId)
{
	if (iCurrentRequest && iCurrentRequest->IsMe(aOwner,aId))
	{
		return iCurrentRequest;
	}
	else // maybe its still queue'd ?
	{				
		CHttpRequestInfo* retObj = GetItemFromQueue(aOwner,aId);
		// the object may be in the completed requests queue.
		// try and retreive from there
		if(!retObj)
		{
			retObj = GetObjectFromCompletedRequestsQueue(aOwner, aId);
		}
		return retObj;	
	}
	
}

unsigned short CHttpEngine::StartHttpRequestL(CHttpRequestInfo* aInfo)
{
	CleanupStack::PushL(aInfo);
	aInfo->CompleteConstructL();
	
	if(iConnStat==EFlhNReqNotConnectedToNetwork)
	{
		if(iPreparingToExit == EPreparingToExit)
		{
			// do not try to connect or issue the request if we are in the process of resetting
			// just notify the core that the request has failed
			// only incase of non rtmpt close request
			// handle request failed
			if(!aInfo->IsRTMPTClose())
			{
				// handle request failed
				aInfo->HandleRequestFailed(KErrCancel,KFlashLiteHttpClientSideCancelErr); 
			}
			CleanupStack::Pop(aInfo);
			delete aInfo;
			aInfo = NULL;
		}
		else if(iPreparingToExit == EInitial)
		{
			ConnectToNetworkL(aInfo);
			iRequestQueue.AddLast(*aInfo);
			CleanupStack::Pop(aInfo);
		}
	}
	else
	{
		// timeout timer may still be going, or we could still be connecting
		if(iTimer->IsActive())
		{
			iTimer->Cancel();
		}

		iRequestQueue.AddLast(*aInfo);
		CleanupStack::Pop(aInfo);

		if(iConnStat!=EFlhNReqConnectingToNetwork)
		{
			TryQueueNextRequest();
		}
	}
	
	return 1;
}


void CHttpEngine::HandleNetworkAccessDenied()
{
	// the user declined to pick an access point, or a Leave happened setting up the connection
	CloseAllTransactions();
	iTransStat=EFlhNReqFailedConnect;
}

TBool CHttpEngine::OKToPause()
{
	return iCurrentRequest!=NULL;
}

void CHttpEngine::ConnectionEventL(TFlhIPEvent aEvent)
{
	if(aEvent== EEventIsConnectedOK)
	{
		if(iConnHandler)
		{
            iSession.OpenL();
            InstallAuthenticationL(iSession);
            //Install Prof Filter on session.
            CHttpUAProfFilterInterface::InstallFilterL( iSession );
            // set the connection parameters...
            RHTTPConnectionInfo info=iSession.ConnectionInfo();
            RStringPool sp(iSession.StringPool());
            const TStringTable& st=RHTTPSession::GetTable();
                info.SetPropertyL(sp.StringF(HTTP::EHttpSocketServ,st),THTTPHdrVal(iConnHandler->SocketSessionHandle()));
                info.SetPropertyL(sp.StringF(HTTP::EHttpSocketConnection,st),THTTPHdrVal(iConnHandler->ConnectionHandle()));
    
            iConnStat=EFlhNReqConnectedToNetwork;
            // proceed with processing transactions...
            TryQueueNextRequest();
		}
	}
	else
	{
		if(iCurrentRequest)
		{
			CPlatformFlashEngine* pEngine = iCurrentRequest->MM_Object()->pHostObj->iFIInterface->GetEngine();
			/*
			if (!(pEngine->IsBrowserPluginActive()))
			{
				pEngine->GetUI()->NotifyDataAccessError(NULL, ECompleteFailedConnect);	
			}
			*/
			pEngine->NotifyDataAccessError(NULL, ECompleteFailedConnect);
		 }
		
		CloseAllTransactions();
		iTransStat=EFlhNReqFailedConnect;
		
	}
}

enum 
	{
	ECurrentlyProcessing = 0,
	EConnectingPendingRequests = 1,
	EDisConnectedPendingRequests = 2,
	EResolvingConnection = 3,
	ENetworkError = 4,
	EGeneralFailure = 5,
	EHttpError = 6,
	ERequestCompletedNormally = 8,
	ENoTransaction = 10,
	};

TInt CHttpEngine::GetStatusForPlayer() const
{
	if(iTransStat==EFlhNReqNoTransactionsMade)
	{
		return ENoTransaction; // no transaction made - ever since reset
	}

	else if(iTransStat==EFlhNReqReceivedHeaders || iTransStat==EFlhNReqProcessingHeaders ||
		iTransStat==EFlhNReqFetchingBody ||	iTransStat==EFlhNReqPostingData	)
	{
		return ECurrentlyProcessing; // currently processing
	}

	else if(iTransStat==EFlhNReqFailedConnect)
	{
		return ENetworkError; // network error
	}
	
	else if (iTransStat==EFlhNReqFailedHTTP)
	{
		return EHttpError; // HTTP error returned from server...
	}
	
	else if (iTransStat==EFlhNReqFailedError)
	{
		return EGeneralFailure; // General (non HTTP reason) failure to get data from the server - not an exact match
	}

	else if(iTransStat==EFlhNReqCompletedNormally)
	{
		return ERequestCompletedNormally; // request completed 'normally'
	}

	else if(iTransStat==EFlhNReqNoDataSentRecvd || iTransStat==EFlhNReqNoTransactionInfo)
	{
		return EResolvingConnection; // resolving connection to server
	}

	else if(iConnStat==EFlhNReqConnectingToNetwork)
	{
		return EConnectingPendingRequests; // connecting with pending requests
	}

	else if(iConnStat==EFlhNReqNotConnectedToNetwork)
	{	
		return EDisConnectedPendingRequests; // no connection but pending requests - should never happen we should always have returned
				 // ENoTransaction or the last completion code by now...
	}
	
	return 0; // note cancel is not handled - no suitable comparison code EFlhNReqCanceled 
}

TBool CHttpEngine::GetCredentialsL(const TUriC8& aURI, RString aRealm, 
								  RStringF /*aAuthenticationType*/,
								  RString& aUsername, 
								  RString& aPassword)
{
	HBufC* returnUsername = NULL;
    HBufC* returnPassword = NULL;
    TBool basicAuthentication = ETrue;
    
    HBufC* uri = CnvUtfConverter::ConvertToUnicodeFromUtf8L(*aURI.UriDes().AllocLC());
	HBufC* realm = CnvUtfConverter::ConvertToUnicodeFromUtf8L(*aRealm.DesC().AllocLC());
		
	TBool ok;
	
	if ( iRetryAuthentication )
	{
	    HBufC* remeberedUserName = CnvUtfConverter::ConvertToUnicodeFromUtf8L(*iUserName);
		ok = iDlgProvider->DialogUserAuthenticationLC(
														*uri,
	                                                    *realm,
	                                                    *remeberedUserName,
	                                                    returnUsername,
	                                                    returnPassword,
	                                                    basicAuthentication
	                                                    );
		delete remeberedUserName;
	}
	else
	{
		ok = iDlgProvider->DialogUserAuthenticationLC(
														*uri,
	                                                    *realm,
	                                                    KNullDesC16,
	                                                    returnUsername,
	                                                    returnPassword,
	                                                    basicAuthentication
	                                                    );
	    iRetryAuthentication = ETrue;     
	}    
		
    TBool ret = EFalse;
    
	if(ok)
	{
	    //delete if already present
	    if(iUserName)
	    {
	        delete iUserName;
	        iUserName = NULL;
	    }
	    
		iUserName = CnvUtfConverter::ConvertFromUnicodeToUtf8L(*returnUsername);
		HBufC8* password = CnvUtfConverter::ConvertFromUnicodeToUtf8L(*returnPassword);
		aUsername = iSession.StringPool().OpenStringL( *iUserName );
		aPassword = iSession.StringPool().OpenStringL( *password );
   	
		delete password;
    	ret = ETrue;
	}	
	
    CleanupStack::PopAndDestroy( returnPassword );
    CleanupStack::PopAndDestroy( returnUsername );
    CleanupStack::PopAndDestroy(2); //aURI.UriDes().AllocLC() & aRealm.DesC().AllocLC()
    
    delete uri;
    delete realm;
    
    return ret;
}

void CHttpEngine::ReportDialogEventL( TInt aType, TInt aFlags )
{
	switch( aType )
	{
	case MBrowserDialogsProviderObserver::EUserAuthentication:
		if (aFlags) //True, if cancelled.
		{
		}
		break;
	default:
	break;
	}
}

void CHttpEngine::AddToCompletedRequestsIfFlvL(CHttpRequestInfo* aInfo)
{
	if(aInfo->GetStreamMediaType() != FI_StreamMediaType_FLV &&  aInfo->GetStreamMediaType() != FI_StreamMediaType_Sound )
	{
		// non-flv stream objects will not be required once completed
		delete aInfo;
		return;
	}
	if(iCompletedRequestsQueue)
	{
		iCompletedRequestsQueue->AppendL(aInfo);
	}
	else
	{
		iCompletedRequestsQueue = new(ELeave) CArrayPtrSeg<CHttpRequestInfo>(1);
		iCompletedRequestsQueue->AppendL(aInfo);
	}		
}
	
void CHttpEngine::RemoveAllCompletedRequests()	
{
	if(!iCompletedRequestsQueue)
		return;	
	
	CHttpRequestInfo* httpRequestInfo=NULL;
	
	while(iCompletedRequestsQueue->Count())
	{
		httpRequestInfo = iCompletedRequestsQueue->At(0);			
		delete httpRequestInfo;	
		iCompletedRequestsQueue->Delete(0);
	}
	
	delete iCompletedRequestsQueue;
	iCompletedRequestsQueue = NULL;
}	

CHttpRequestInfo* CHttpEngine::GetObjectFromCompletedRequestsQueue(void *aOwner, TInt aStreamId)
{
	CHttpRequestInfo* httpRequestInfo=NULL;
	if(!iCompletedRequestsQueue)
	{
		return NULL;
	}
	for (TInt i=0;i<iCompletedRequestsQueue->Count(); i++)
	{
        httpRequestInfo = iCompletedRequestsQueue->At(i);
    
        if( httpRequestInfo && ( httpRequestInfo )->IsMe(aOwner, aStreamId ) )
        {
            // found the object with the given stream ID															
            break;
        }			
	}	
	return 	httpRequestInfo;
}
void CHttpEngine::DeleteObjectFromCompletedRequestsQueue(void *aOwner, TInt aStreamId)
{
	CHttpRequestInfo* httpRequestInfo=NULL;
	if( !iCompletedRequestsQueue )
	{
		return;
	}
	for (TInt i=0; i < iCompletedRequestsQueue->Count(); i++)
    {
        httpRequestInfo = iCompletedRequestsQueue->At(i);
    
        if( httpRequestInfo && ( httpRequestInfo )->IsMe(aOwner, aStreamId ) )
        {
            // found the object with the given stream ID															
            delete httpRequestInfo;
            iCompletedRequestsQueue->Delete(i);
            break;
        }
        
    }		
}

void CHttpEngine::PrepareToExit()
{
	ASSERT(iPreparingToExit == EInitial);
	
	iPreparingToExit = EPreparingToExit;
}

void CHttpEngine::DoSynchronousRTMPTCloseL()
{
    // The RTMPT "/close" command is synchronous *only* if the player is being destroyed
	// in this case there only one request in the queue which is deleted soon after it is issued.
    // So, when this point is reached, the queue *must* be empty.
	ASSERT(iRequestQueue.IsEmpty());
	
	if(!iRTMPTCloseWait)
	{
		iRTMPTCloseWait = new (ELeave) CActiveSchedulerWait();
	}
	
/*	ASSERT(!iRTMPTCloseWait->IsStarted());	// there is no way this flow is reached if iRTMPTCloseWait is running */
    if(iRTMPTCloseWait->IsStarted())
    {
        // we are synchronously closing the rtmpt connections
        // this might get called from async connection event.
        // hence the check.
        return;
    }
	
	if(!iRTMPTCloseWaitTimer)
	{
		TCallBack timerCallback(RTMPTTimerCallback,static_cast<TAny*>(this));
		iRTMPTCloseWaitTimer = CFlhTimerObject2::NewL(CActive::EPriorityStandard,timerCallback);
	}

	ASSERT(!iRTMPTCloseWaitTimer->IsActive());	// the timer *must* be cancelled
	
	iRTMPTCloseWaitTimer->After(KRTMPTCloseTimeout);
	iRTMPTCloseWait->Start(); // blocks this AO untill its cancelled in MHFRunL() or it times out and is cancelled in DoRunL()

}

void CHttpEngine::CloseAllTransactions()
{
    TSglQue<CHttpRequestInfo> rtmptRequestQueue;
	
    //Move rtmp reqs to separate local array and close non-rtmpt requests in the queue.
    //close active transaction. if rtmpt close synchronously with ack
    //for each rtmp req, close transaction synchronously.
    
    TInt nRequestQueue = 0;
    TInt nRtmptRequestQueue = 0;
    
    CHttpRequestInfo* info;
	while(!iRequestQueue.IsEmpty())
	{
		nRequestQueue++;
		
		info=iRequestQueue.First();
		iRequestQueue.Remove(*info);

		if(info->IsRTMPTClose())
		{
			rtmptRequestQueue.AddLast(*info);
			nRtmptRequestQueue++;
		}
		else    // if its not an RTMTP close, inform the player that the request failed.
		{
			info->HandleRequestFailed(KErrCancel,KFlashLiteHttpClientSideCancelErr);
			delete info;
			info = NULL;		
		}
	}

   	CancelActiveTransaction();
    
    // synchronously send the rtmpt close requests
    while(!rtmptRequestQueue.IsEmpty())
    {
        info=rtmptRequestQueue.First();
        rtmptRequestQueue.Remove(*info);
        
        iRequestQueue.AddLast(*info);
        if(iConnStat == EFlhNReqConnectedToNetwork)
        {
        	TryQueueNextRequest();
        }
    }
    
}

void CHttpEngine::CancelSynchronousRTMPTClose()
{
	ASSERT(iCurrentRequest->IsRTMPTClose());
	
	iTransStat=EFlhNReqCanceled;
	
	TRAP_IGNORE(TransactionOverQueueNextL(iTransaction));
	
	// TransactionOverQueueNextL() closes the transaction and starts the new request
	// incase of a synchronous rtmpt close request (during shutdown), 
	// the queue would be empty and hence no request will be issued
	// Once the CActiveSchedulerWait is stopped and the control goes back to the core player
	// via CHttpEngine::StartHttpRequestL(), more "/close" requests may be issued
	
	RTMPTCloseCompleted();
}

/*
 * CHttpEngine::RTMPTTimerCallback is a callback from the timer
 */

TInt CHttpEngine::RTMPTTimerCallback(TAny* aArg)
{
	CHttpEngine* self = static_cast<CHttpEngine*>(aArg);
	if(self)
		self->CancelSynchronousRTMPTClose();

	return KErrNone;
}

void CHttpEngine::RTMPTCloseCompleted()
{
    if(iRTMPTCloseWait && iRTMPTCloseWait->IsStarted())
        iRTMPTCloseWait->AsyncStop();
    
    if(iRTMPTCloseWaitTimer && iRTMPTCloseWaitTimer->IsActive())
        iRTMPTCloseWaitTimer->Cancel();
}

void CHttpEngine::ConnectToNetworkL(CHttpRequestInfo* /*aInfo*/)
{
	iConnHandler = iIPConnectionMgr->GetConnectionL(this);
	
	iConnStat=EFlhNReqConnectingToNetwork;
	iTransStat=EFlhNReqNoTransactionInfo;

}
// end of file
