/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
#include "symbian_location.h"
#include "extension_mem_mgmt.h"
#include "locationservice.h"
#include "extension_mem_mgmt.h"
#include <e32debug.h>
#include <math.h>
const TInt KDefaultTimeOut(30000000);
AMErr LocationImpl::Create(IAMAllocator* pAllocator, IAMGPS** ppGPS)
{
    
    LocationImpl* pImpl = NULL;

    AMErr err = kAMErrNone;

    if ( ! ppGPS )
    {
        err = kAMErrBadParam;
        goto exit_gracefully; 
    }

    *ppGPS = NULL;
	  //LocationImpl::NewL();
    pImpl = AMC_NEW(pAllocator, LocationImpl);
    if ( !pImpl )
    {
        err = kAMErrOutOfMemory;
        goto exit_gracefully; 
    }
   
    pImpl->m_pAllocator = pAllocator;
    err = pImpl->Init();
    if ( kAMErrNone != err )
    {
        goto exit_gracefully; 
    }

    // transfer ownership to caller 
    *ppGPS =  pImpl;
    LocationImpl::AddRef(*ppGPS);
    pImpl = NULL;

exit_gracefully:

    if ( pImpl )
    {
        AMC_DELETE(pImpl);
        pImpl = NULL;
    }
    
    return err;
}

//  -----------------------------------------------------------------------------------------------
AMErr LocationImpl::Init()
{
   
    AMErr err = kAMErrNone;
    //! create the vtbl
    m_pFncs = (AMFNCS(IAMGPS)*)AMC_MALLOC(m_pAllocator, sizeof(AMFNCS(IAMGPS)));
    if( ! m_pFncs )
    {
        err = kAMErrOutOfMemory;
        goto exit_gracefully;
    }
 
    //Set up function pointers
    m_pFncs->AddRef    = LocationImpl::AddRef;
    m_pFncs->Release   = LocationImpl::Release;
    m_pFncs->StartTracking = LocationImpl::StaticStartTracking;
   
    m_pFncs->QueryInterface = NULL;
   
exit_gracefully:
    return err;
}

LocationImpl::LocationImpl():m_refCount(0)
    {
   
    // iGenericPosInfo= HPositionGenericInfo::NewL();
    //SetSupportedFields();
   
    }




//  -----------------------------------------------------------------------------------------------
LocationImpl::~LocationImpl()
{
    if(m_pFncs)
    {
        //AMC_DELETE(m_pGPSEngine);
        AMC_FREE(m_pFncs);
        m_pFncs = NULL;
    }
    
    delete iGenericPosInfo ;
}


//  -----------------------------------------------------------------------------------------------
/*static*/ 
U32 LocationImpl::AddRef(IAMGPS* pGPS)
{
    LocationImpl* pImpl = (LocationImpl*) pGPS;
    return ++( pImpl->m_refCount );
}

//  -----------------------------------------------------------------------------------------------

U32 LocationImpl::Release(IAMGPS* pGPS)
{
    LocationImpl* pImpl = (LocationImpl*) pGPS;
    if ( 1 == pImpl->m_refCount )
    {
        AMC_DELETE(pImpl);
        return 0;
    }
    return --( pImpl->m_refCount );
}

TInt LocationImpl :: SetSupportedFields()
    {
    //Extract all the information Accuracy, timestamp, satellite and course info and append 
    // to out parmlist
    
    
    TUint fieldIter = 0 ;
    
    iFieldList[fieldIter++] = EPositionFieldHorizontalSpeed ;
    iFieldList[fieldIter++] = EPositionFieldHeading ;
 
    
    iFieldList[fieldIter] = EPositionFieldNone  ;   //Field Termination 
   //iGenericPosInfo->ClearRequestedFields() ;
   //iGenericPosInfo->SetRequestedFields(iFieldList) ;
  
  return KErrNone ;
    }


AMErr 
LocationImpl::StaticStartTracking(IAMGPS* pGPS, AMStartTrackingCallback* pCallback, void *pUserData, S32 time, IAMGPSTracker** pInstance)
{
    
    LocationImpl* pImpl = NULL;
    AMErr status = kAMErrNone;
    
    if(!pGPS ||
       !pUserData ||
       !pCallback ||
       !pInstance)
    {
        status = kAMErrBadParam;
        goto exit_gracefully;
    }
    pImpl = (LocationImpl*)pGPS;
    //! Create the GPS Engine if not created
    //status = pImpl->CreateGPSEngine();
    if (status != kAMErrNone)
        goto exit_gracefully;
    LocationTracker::CreateInstance(pImpl->m_pAllocator, pInstance);
    if (!*pInstance) 
    {
        status = kAMErrOutOfMemory;
        goto exit_gracefully;;
    }
    status = ((LocationTracker*)(*pInstance))->StartTracking(pCallback, pUserData, pImpl, time);
   
exit_gracefully:
    return status;
}



//static functions
AMErr LocationTracker::CreateInstance(IAMAllocator* pAllocator, IAMGPSTracker** ppGPSTracker)
{
    
    AMErr err = kAMErrNone;

    if ( ! ppGPSTracker )
    {
        err = kAMErrBadParam;
        goto exit_gracefully; 
    }

    *ppGPSTracker = AMC_NEW(pAllocator, LocationTracker(pAllocator));
    if ( !*ppGPSTracker )
    {
        err = kAMErrOutOfMemory;
        goto exit_gracefully; 
    }

    err = ((LocationTracker *)*ppGPSTracker)->Init();
    if ( kAMErrNone != err )
    {
        goto exit_gracefully; 
    }

    // transfer ownership to caller 
    IAMGPSTRACKER_AddRef(*ppGPSTracker);
    
exit_gracefully:

    if (kAMErrNone != err && *ppGPSTracker )
    {
        LocationTracker::Release(*ppGPSTracker);
        *ppGPSTracker = NULL;
    }
    
    return err;
}

//  -----------------------------------------------------------------------------------------------
U32 LocationTracker::AddRef(IAMGPSTracker* pGPSTracker)
{
    LocationTracker* pImpl = (LocationTracker*) pGPSTracker;
    return ++( pImpl->m_refCount );
}

//  -----------------------------------------------------------------------------------------------
U32 LocationTracker::Release(IAMGPSTracker* pGPSTracker)
{
    LocationTracker* pImpl = (LocationTracker*) pGPSTracker;
    if ( 1 == pImpl->m_refCount )
    {
        pImpl->StopTracking();
        AMC_DELETE(pImpl);
        return 0;
    }
    return --( pImpl->m_refCount );
}

AMErr 
LocationTracker::SetUpdateInterval(IAMGPSTracker* pGPSTracker, S32 timer)
{
    
    //! Set the update interval of the RPositioner to the new time
    LocationTracker* pImpl = (LocationTracker*)pGPSTracker;
    pImpl->SetUpdateInterval(timer);
    return kAMErrNone;
}

void LocationTracker::SetUpdateInterval(S32 timer)
    {
    
   
    
  
        
     TPositionUpdateOptions upateOptions; 
     timer=timer*1000;
         upateOptions.SetUpdateInterval(TTimeIntervalMicroSeconds(timer));
         upateOptions.SetUpdateTimeOut(TTimeIntervalMicroSeconds(KDefaultTimeOut));
       
     iLocationService->SetUpdateInterval(this,&upateOptions);
     }

//  -----------------------------------------------------------------------------------------------
//non-static
AMErr LocationTracker::Init()
{
    
    AMErr err = kAMErrNone;
    m_pFncs = (AMFNCS(IAMGPSTracker)*)AMC_MALLOC(m_pAllocator, sizeof(AMFNCS(IAMGPSTracker)));

    if( !m_pFncs )
    {
        err = kAMErrOutOfMemory;
        goto exit_gracefully;
    }

    //Set up function pointers
    m_pFncs->AddRef     = LocationTracker::AddRef;
    m_pFncs->Release    = LocationTracker::Release;
    m_pFncs->SetUpdateInterval  = LocationTracker::SetUpdateInterval;
    
exit_gracefully:
    return err;
}


LocationTracker::LocationTracker(IAMAllocator* pAllocator)
{
    ipStartTrackCallBack = 0;
    ipUserData = 0;
    m_refCount = 0;
    m_pAllocator = pAllocator;
    m_timeFlag = EFalse;
}
//  -----------------------------------------------------------------------------------------------
LocationTracker::~LocationTracker()
{
    if (m_pFncs)
    {
        AMC_FREE(m_pFncs);
        m_pFncs = 0;
    }
}
long GetCurrentTimeInMilliseconds();
AMErr 
LocationTracker::StartTracking(AMStartTrackingCallback* pCallback, void *pUserData, LocationImpl* pLocationImpl, S32 timer)
{
    
    AMErr status = kAMErrNone;
    ipStartTrackCallBack = pCallback;
    ipUserData = pUserData;
    m_pLocationImpl = pLocationImpl;
  
    TRAPD(err,CreateLocationServiceL());
    
    if(KErrNone!=err)
        status = kAMErrCouldNotBeCreated;
    
    const int BASIC_LOCATIONINFO(1);
    TInt locationInfoCategory(BASIC_LOCATIONINFO);
    
    TPositionUpdateOptions upateOptions;    
    upateOptions.SetUpdateInterval(TTimeIntervalMicroSeconds(timer));
    
    TRAP_IGNORE(
    				iLocationService->TraceL(this, 
            locationInfoCategory,
            m_pLocationImpl->iFieldList,
            &upateOptions)
            );
    return status;
}

void LocationTracker::CreateLocationServiceL()
{
    iLocationService = CLocationService :: NewL();
}

AMErr
LocationTracker::StopTracking()
{
    if(iLocationService)
        iLocationService->CancelOnGoingService();
    
    return kAMErrNone;
}

TInt LocationTracker::HandleNotifyL(TPositionCourseInfo aGenericInfo , TInt aError)
    {
    
    AMGPSLocationInfo* locInfo = (AMGPSLocationInfo*)AMC_MALLOC(m_pAllocator, sizeof(AMGPSLocationInfo));
            //void* ipUserData = NULL;
            
            
            
    //construct location info..
        //Now insert the collected position informaiton(latitude, longitude , and altitude into a default map 
        TPosition pos ;
        aGenericInfo.GetPosition(pos) ;
         
        if(Math::IsNaN((TReal)pos.Longitude()))
            {
               locInfo->longitude=0;
            }
        else
               locInfo->longitude = (TReal)pos.Longitude();
        
        if(Math::IsNaN((TReal)pos.Latitude()))
            {
               locInfo->latitude =0;
            }
        else
               locInfo->latitude  = (TReal)pos.Latitude();
                               
        if(Math::IsNaN((S32)pos.Altitude()))
            {
                locInfo->altitude =0;
            }
        else
                locInfo->altitude  = (S32)pos.Altitude();
           
        if(Math::IsNaN((S32)pos.HorizontalAccuracy()))
             {
                locInfo->hAccuracy=0;
             }
        else
               locInfo->hAccuracy = (S32)pos.HorizontalAccuracy();

        TInt32 temp;
        if(Math::IsNaN((S32)pos.VerticalAccuracy()))
                    {
                temp=0;
                    }
               else
                   {
                   TInt error=Math::Int(temp,pos.VerticalAccuracy()); 
                   locInfo->vAccuracy=temp;
                   }

        locInfo->speed =-1;
        locInfo->heading =-1;
        TTime curTime = pos.Time();
        
        
        if( !m_timeFlag )
            {
            m_timeFlag = ETrue;
            m_firstTime = curTime;
            locInfo->time = 0;
            }
        else
            {
            TTimeIntervalSeconds intSec;
            if( KErrNone == curTime.SecondsFrom(m_firstTime, intSec) )
                {
                locInfo->time = intSec.Int()*1000;
                }
            else
                {
                locInfo->time = 0;
                }
            
            
            }
        
            
            
            TCourse course;
            aGenericInfo.GetCourse(course);
                
            // DO THE CONVERSION HERE
             TInt32 temp1;
        if(Math::IsNaN(course.Speed()))
                    {
                temp1=0;
                    }
               else
                   {
                   TInt error=Math::Int(temp1,course.Speed()); 
                  locInfo->speed=temp1;
                   }
                    TInt32 temp2;
        if(Math::IsNaN(course.Heading()))
                    {
                temp2=0;
                    }
               else
                   {
                   TInt error=Math::Int(temp2,course.Heading()); 
                   locInfo->heading=temp2;
                   }
           
            
           
        if(locInfo)
        (*ipStartTrackCallBack)( ipUserData, kAMErrNone, locInfo);
       
        return aError;
    }


