/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Contains the utility functions used for recognizings audio and video data
*               and creating temporary files
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "saplayer_build.h"

#include "mmf/common/mmfcontrollerpluginresolver.h" 
#include "symbian_mediarecognizer.h"
#include "symbian_engine.h"
#include <imageconversion.h>


// Audio
_LIT(KTempFileExtMXMF, ".mxmf");
_LIT(KTempFileExtRMF, ".rmf");
_LIT(KTempFileExtAMR, ".amr");
_LIT(KTempFileExtMIDI, ".mid");
_LIT(KTempFileExtMP3, ".mp3");
//_LIT(KTempFileExtRealAudio, ".ra");
_LIT(KTempFileExtAAC, ".aac");
_LIT(KTempFileExtWAV, ".wav");
_LIT(KTempFileExtWMA, ".wma");

// Audio & Video
_LIT(KTempFileExt3G2, ".3g2");
_LIT(KTempFileExt3GP, ".3gp");
_LIT(KTempFileExtMP4, ".mp4");

// Video
_LIT(KTempFileExtRealVideo, ".rm");
_LIT(KTempFileExtAVI, ".avi");
_LIT(KTempFileExtWMV, ".wmv");
_LIT(KTempFileExtMPG, ".mpg");
_LIT(KTempFileExtMOV, ".mov");

const TInt KMediaHeaderSize =30;

_LIT8( KAudioMimeTypeStr,"audio" );
_LIT8(KStar, "*");


extern void NPPDebug(TUint8 aLogLevel,TBool aIsPlugin,TRefByValue<const TDesC> /*aText*/, ... );

/*static*/
////////////////////////////////////////////////////////////////////////////////////////////
//MMF use


TBool SupportsHeaderDataL(const CDesC8Array& aHeaderDataArray
						,const TDesC8& aHeaderData
						,TInt& idx)
{
	idx = -1; 
	
	TBool matchFound = EFalse;
	// Search aHeaderData for each chunk of header data in aHeaderDataArray
	for (TInt i=0; i<aHeaderDataArray.Count(); i++)
		{
		TPtrC8 headerData(aHeaderDataArray[i]);
		// search for match section in the main buffer.
		// ...by putting it in wildcards!
		HBufC8* wildMatch = HBufC8::NewLC( headerData.Length() + 1) ;
		TPtr8 wildMatchPtr = wildMatch->Des() ;
		wildMatchPtr.Copy( headerData ) ;
	
		wildMatchPtr.Append( KStar ) ;
		if (aHeaderData.MatchF(*wildMatch) != KErrNotFound)
			{
			idx = i;
			matchFound = ETrue;
			}
		CleanupStack::PopAndDestroy(wildMatch);//wildMatch
		if (matchFound)
			break;
		}
	return matchFound;
		
}

TBool MediaRecognizer::CreateFilenameFromMMFDataL(CPlatformFlashEngine* aEngine
									,TFileName& aFileName
									, void* aData
									, unsigned long aDataLen)
{
	TBool isPlugin = aEngine->IsBrowserPluginActive();
	NPPDebug(LOG_DEBUG,isPlugin,_L("MediaRecognizer::CreateFilenameFromData ++ "));
	
	CMMFFormatSelectionParameters* formatPrms =
        CMMFFormatSelectionParameters::NewLC();
	CMMFControllerPluginSelectionParameters* controllerPrms =
        CMMFControllerPluginSelectionParameters::NewLC();

    // Empty format parameters means: "get all the supported formats"
    controllerPrms->SetRequiredPlayFormatSupportL( *formatPrms );
    RMMFControllerImplInfoArray cntrlArray;
    controllerPrms->ListImplementationsL( cntrlArray );
		
	//CleanupStack::PushL(AudioArray);
	TInt bufsize = (aDataLen > KMediaHeaderSize ) ? KMediaHeaderSize:aDataLen;
	
	TPtr8 bufptr((TUint8*)aData, bufsize, bufsize);
	
	TInt j=0,idx=0;
    
    for( TInt i( cntrlArray.Count() - 1 ); i >= 0 ; --i )
        {
	        const RMMFFormatImplInfoArray&
	            infoArray( cntrlArray[i]->PlayFormats() );
	           
	        for (j=0 ; j < infoArray.Count(); j++) 
	        {
	        	if(SupportsHeaderDataL(infoArray[j]->SupportedHeaderData(),bufptr,idx))
	        	{
	    			NPPDebug(LOG_INFO,isPlugin,_L("header is supported by MMF"));  
	    			const CDesC8Array& extnArray = infoArray[j]->SupportedFileExtensions();
	    			
	    			if(idx < extnArray.Count())
	    			{
	    				TPtrC8 mimeExtn(extnArray[idx]);
	    				HBufC* extn = aEngine->CreateEpocString((const char*)mimeExtn.Ptr(),mimeExtn.Length());
	    				aFileName.Append(extn->Des());
	    				delete extn;
	    				
	    				
    					NPPDebug(LOG_INFO,isPlugin,_L("file name [%S] for MMF "),&aFileName);
	    			}
	    				
	    			cntrlArray.ResetAndDestroy();
					cntrlArray.Close();
    				CleanupStack::PopAndDestroy( 2, formatPrms );
	    			return ETrue;  		
	        	}
	        }
         
        }
        
    cntrlArray.ResetAndDestroy();
	cntrlArray.Close();
    CleanupStack::PopAndDestroy( 2, formatPrms );
   
   	NPPDebug(LOG_DEBUG,isPlugin,_L("MediaRecognizer::CreateFilenameFromData -- "));

   return EFalse; 
}


char** MediaRecognizer::GetImageDecoderSupportedFormatsL(CPlatformFlashEngine* aEngine
											,TInt& aCount)
{
	RFileExtensionMIMETypeArray mimeArray;
	CleanupResetAndDestroyPushL(mimeArray);
	CImageDecoder::GetFileTypesL(mimeArray);
	
	aCount = mimeArray.Count();
	
	char** charArray=new(ELeave) char*[aCount];
    
    NPPDebug(LOG_INFO,aEngine->IsBrowserPluginActive()
    			,_L("Mimes %d supported by Image decoder "),aCount);
    			
	CleanupStack::PushL(charArray);
	
	TInt idx=0;
	
	for (idx=0; idx < aCount; idx++)
	{
		const TDesC8& Mime = mimeArray[idx]->MIMEType();
		
		charArray[idx]= new (ELeave) char[Mime.Length() +1];
		
		char* buf = 	charArray[idx];
		CleanupStack::PushL(buf);
		
		memcpy(buf,Mime.Ptr(),Mime.Length());
		buf[Mime.Length()] = '\0';	
	}
	
	CleanupStack::Pop(idx+1,charArray);
	CleanupStack::PopAndDestroy(); //destroy the mimeArray
	
	return charArray;
}


char** MediaRecognizer::GetMMFsupportedFormatsL(CPlatformFlashEngine* aEngine
											,TInt& aCount, TBool aSelectAudio
											, TInt aResourceId)
{
	CMMFFormatSelectionParameters* formatPrms =
        CMMFFormatSelectionParameters::NewLC();
	CMMFControllerPluginSelectionParameters* controllerPrms =
        CMMFControllerPluginSelectionParameters::NewLC();

    // Empty format parameters means: "get all the supported formats"
    controllerPrms->SetRequiredPlayFormatSupportL( *formatPrms );
    RMMFControllerImplInfoArray cntrlArray;
    controllerPrms->ListImplementationsL( cntrlArray );
	CDesC8ArrayFlat* MimeTypeArray = new (ELeave) CDesC8ArrayFlat(15);
	
	CleanupStack::PushL(MimeTypeArray);
	
	TInt j=0,k=0;
    for( TInt i( cntrlArray.Count() - 1 ); i >= 0 ; --i )
        {
        const RMMFFormatImplInfoArray&
            infoArray( cntrlArray[i]->PlayFormats() );
          
         for ( k=0; k < infoArray.Count() ; k++)  
         {
          	 const CDesC8Array& mimeArray(infoArray[k]->SupportedMimeTypes());
            
	         for ( j= 0; j < mimeArray.Count() ; j++)
	         {
	         	TPtrC8 mimeType(mimeArray[j]);
	         	
	         	if(mimeType.Find(KAudioMimeTypeStr) == 0)
	         	{  //if audio match found and audio needs to be selected
	         		if(aSelectAudio)
	         		{
	         			MimeTypeArray->AppendL(mimeType);	
	         		}
	         		
	         	}
	         	else
	         	{
	         	 //if non audio to be selected then add it
	         	 	if(!aSelectAudio)
	         		{
		        		MimeTypeArray->AppendL(mimeType);
	         		}
		        }
	         	
	         		
	         }
          	
         }
        
        }
     
    const CDesC8Array& mimeArray(*MimeTypeArray);
    
    aCount = mimeArray.Count();
    
    //need to add mime types from our RSS file
    //experience is that some mime types are not returned by MMF ,for eg. midi
    //or i dont know how to get these mime types
    
    CDesCArray* array = CEikonEnv::Static()->ReadDesC16ArrayResourceL(aResourceId);
	CleanupStack::PushL(array);
	aCount +=array->Count();
	
    
    char** charArray=new(ELeave) char*[aCount];
    
    NPPDebug(LOG_INFO,aEngine->IsBrowserPluginActive()
    			,_L("Mimes %d supported by MMF "),aCount);
	CleanupStack::PushL(charArray);
	
	//TInt i = 0;
	for(j=0;j<mimeArray.Count();j++)
	{ //make char array  string from mime array
		TPtrC8 Mime (mimeArray[j])	;
		
		charArray[j]= new (ELeave) char[Mime.Length() +1];
		
		char* buf = 	charArray[j];
		CleanupStack::PushL(buf);
		
		memcpy(buf,Mime.Ptr(),Mime.Length());
		buf[Mime.Length()] = '\0';
		
	}
	
    //now read string from RSS
	for(k=0; k<array->Count();k++)
	{
		charArray[j++]=aEngine->CreateFlashStringLC(array->MdcaPoint(k));
	}
	
	CleanupStack::Pop(j+1,charArray); //pop all strings
	CleanupStack::PopAndDestroy(array);
	
	CleanupStack::PopAndDestroy(MimeTypeArray); //destroy our mime arrya of MMF
	
	cntrlArray.ResetAndDestroy();
	cntrlArray.Close();
    CleanupStack::PopAndDestroy( 2, formatPrms );
    
    return charArray;
    
}

////////////////////////////////////////////////////////////////////////////////////////////
//
//
TBool MediaRecognizer::CreateFilenameFromData(TFileName& aFileName,
													void* aData,
													unsigned long aDataLen,
													TDataType aDataType)
{
	char* c = (char*)aData;
	TUint8* p = (TUint8 *)aData;

	// Audio or video
	//
	// Since these are among the most usual formats on devices, they are moved in the beginning

	// 3GP, 3G2, MP4
	if ((aDataLen > 8) && ((c[4] == 'f') && (c[5] == 't') && (c[6] == 'y') && (c[7] == 'p')))
	{
		if ((aDataLen > 12) && ((c[8] == '3') && (c[9] == 'g') && (c[10] == '2') && (c[11] == 'a')))
		{
			aFileName.Append(KTempFileExt3G2);
			return ETrue;
		}
		else if ((aDataLen > 12) && ((c[8] == '3') && (c[9] == 'g') && (c[10] == 'p') && (c[11] == '5')))
		{
			aFileName.Append(KTempFileExt3GP);
			return ETrue;
		}
		// I modified the mp4 recognizer to be less strict. Now it recognizes videos too. -sk
		else if ((aDataLen > 20) && ((c[16] == 'm') && (c[17] == 'p') && (c[18] == '4')))
		{
			aFileName.Append(KTempFileExtMP4);
			return ETrue;
		}
		/*
			else if ((aDataLen > 12) && ((c[8] == 'i') && (c[9] == 's') && (c[10] == 'o') && (c[11] == 'm')))
			{
				if ((aDataLen > 20) && ((c[16] == 'm') && (c[17] == 'p') && (c[18] == '4') && (c[19] == '1')))
				{
					aFileName.Append(KTempSoundFileExtMP4);
					return ETrue;
				}
			}
		*/
	}


	// Audio formats

	if(aDataType == EAudio)
	{
		
		if ((aDataLen > 5) && (c[0] == 'I') && (c[1] == 'R') && (c[2] == 'E') && (c[3] == 'Z'))
		{ // .RMF
			aFileName.Append(KTempFileExtRMF);
			return ETrue;
		}
		if ((aDataLen > 9) && (c[0] == 'X') && (c[1] == 'M') && (c[2] == 'F') && (c[3] == '_') && (c[4] == '2') && (c[5] == '.') && (c[6] == '0') && (c[7] == '0'))
		{ // .MXMF
			aFileName.Append(KTempFileExtMXMF);
			return ETrue;
		}
		if ((aDataLen > 5) && (c[0] == '#') && (c[1] == '!') && (c[2] == 'A') && (c[3] == 'M') && (c[4] == 'R'))
		{ // AMR
			aFileName.Append(KTempFileExtAMR);
			return ETrue;
		}

		if ((aDataLen > 4) && ((c[0] == 'M') && (c[1] == 'T') && (c[2] == 'h') && (c[3] == 'd')))
		{ // MIDI
			aFileName.Append(KTempFileExtMIDI);
			return ETrue;
		}
		else if ((aDataLen > 12) && ((c[0] == 'R') && (c[1] == 'I') && (c[2] == 'F') && (c[3] == 'F') && (c[8] == 'W') && (c[9] == 'A') && (c[10] == 'V') && (c[11] == 'E')))
		{ // WAVE
			aFileName.Append(KTempFileExtWAV);
			return ETrue;
		}
		else if (IsAACData(aData, aDataLen))
		{ // MP3
			aFileName.Append(KTempFileExtAAC);
			return ETrue;
		}
		else if (IsMP3Data(aData, aDataLen))
		{ // MP3
			aFileName.Append(KTempFileExtMP3);
			return ETrue;
		}
		else if(p[0] == 0x30 && p[2] == 0xB2 && p[4] == 0x8E && p[6] == 0xCF && p[8] == 0xA6 && p[15] == 0x6C)
		{ // WMA
			aFileName.Append(KTempFileExtWMA);
			return ETrue;			
		}
		/* rborcic 2005/08/03 -- It appears that phones are unable to play this format. Need to investiage
			which version of RealAudio is supported.
			else if ((aDataLen > 4) && ((c[0] == '.') && (c[1] == 'R') && (c[2] == 'M') && (c[3] == 'F')))
			{
				aFileName.Append(KTempSoundFileExtReal);
				return ETrue;
			}
		*/
	}

	// Video formats

	if(aDataType == EVideo)
	{

		if(aDataLen > 15)
		{
			if(c[1] == 'R' && c[2] == 'M' && c[3] == 'F')
			{
				aFileName.Append(KTempFileExtRealVideo);
				return ETrue;

			} else if(c[0] == 'R' && c[1] == 'I' && c[2] == 'F' && c[3] == 'F' && c[8] == 'A' && c[9] == 'V' && c[10] == 'I')
			{
				aFileName.Append(KTempFileExtAVI);
				return ETrue;
			} else if(p[0] == 0 && p[1] == 0 && p[2] == 1)
			{
				TUint8 test = p[3];
				test &= 0xF0;
				if (test == 0xB0)
				{
					aFileName.Append(KTempFileExtMPG);
					return ETrue;			
				}
			} else if( (c[4] == 'm' && c[5] == 'o' && c[6] == 'o' && c[7] == 'v')
					|| (c[4] == 'f' && c[5] == 'r' && c[6] == 'e' && c[7] == 'e')
					|| (c[4] == 'm' && c[5] == 'd' && c[6] == 'a' && c[7] == 't')
					|| (c[4] == 'w' && c[5] == 'i' && c[6] == 'd' && c[7] == 'e')
					|| (c[4] == 'p' && c[5] == 'n' && c[6] == 'o' && c[7] == 't')
					|| (c[4] == 's' && c[5] == 'k' && c[6] == 'i' && c[7] == 'p') )
			{
				aFileName.Append(KTempFileExtMOV);
				return ETrue;
			} else if(p[0] == 0x30 && p[2] == 0xB2 && p[4] == 0x8E && p[6] == 0xCF && p[8] == 0xA6 && p[15] == 0x6C)
			{
				aFileName.Append(KTempFileExtWMV);
				return ETrue;			
			}
		}
	}

	return EFalse;
}

static TUint32 const bitrate_lookup[5][15] = 
{
	// MPEG-1
	// Layer I
	{ 0,  32000,  64000,  96000, 128000, 160000, 192000, 224000, 256000, 288000, 320000, 352000, 384000, 416000, 448000 },
	// Layer II
	{ 0,  32000,  48000,  56000,  64000,  80000,  96000, 112000, 128000, 160000, 192000, 224000, 256000, 320000, 384000 },
	// Layer III
	{ 0,  32000,  40000,  48000,  56000,  64000,  80000,  96000, 112000, 128000, 160000, 192000, 224000, 256000, 320000 },

	// MPEG-2, MPEG-2.5
	// Layer I
	{ 0,  32000,  48000,  56000,  64000,  80000,  96000, 112000, 128000, 144000, 160000, 176000, 192000, 224000, 256000 },
	// Layer II & III
	{ 0,   8000,  16000,  24000,  32000,  40000,  48000,  56000, 64000,  80000,  96000, 112000, 128000, 144000, 160000 }
};

/*static*/
TBool MediaRecognizer::IsMP3Data(const TAny* aData, TUint32 aSize )
{
	U8* hdr = (U8*)aData;

    if (aSize >= 4)
	{
        // WAV formats were erroneously being detected as MP3 files.  Do a quick check
        // for any RIFF files and ignore them.
        if ((hdr[0] == 'R') && (hdr[1] == 'I') && (hdr[2] == 'F') && (hdr[3] == 'F'))
		{
			return EFalse;
		}
    }
    
	// Search for an MP3 header
	for ( S32 len = aSize; len >= 4; hdr++, len-- )
	{
		//  Check to see if we're at an ID3 header
		if ((hdr[0] == 'I') && (hdr[1] == 'D') && (hdr[2] == '3')){
		
			//  Skip 2 bytes for the Major/Minor Version
			//  Skip 1 byte for the flags
			//  See : http://id3lib.sourceforge.net/id3/id3v2com-00.html for more information on ID3 headers.
			const TInt totalSizeOfID3Header = 10;
			if (len > totalSizeOfID3Header){
				TInt size = (hdr[6] * 2097152) + (hdr[7] * 16384) + (hdr[8]*128) + hdr[9] + totalSizeOfID3Header;

				if (len >= size)
				{
					len -= size;
					hdr += size;
				}
			}
		}
		
		// Quick rejection check - first byte of an MP3 header is always FF
		if ( hdr[0] != 0xff ) continue;

		U8 byte0 = *(hdr+0);	//	bit 32-25 of the header
		U8 byte1 = *(hdr+1);	//	bit 24-17 of the header
		U8 byte2 = *(hdr+2);	//	bit 16-9 of the header
		//U8 byte3 = *(hdr+3);	//	bit 8-1 of the header
		if (byte0 == 0xff && ((byte1 & 0xe0) == 0xe0))
		{
			TBool isMPEG2_5	= !(byte1 & 0x10);			// 21st bit of the header
			TBool isLSFext	= !(byte1 & 0x08);			// 20th bit of the header
			TInt layer		= 4 - ((byte1 & 0x06)>>1);	// 18-17 bits of the header

			if (layer == 4)
			{
				return EFalse; //error
			}

			if (isMPEG2_5 && !isLSFext )
			{
				return EFalse;
			}

			// Lookup the bitrate
			TInt bit_rate = (byte2 & 0xf0)>>4;		// 15-12 bits of the header

			//	Header validity condition
			if ( bit_rate == 15 || bit_rate == 0 )
			{
				return EFalse;
			}

			TInt sampleRateNdx	= (byte2 >> 2) & 0x03;	// bit 11-20 of the header

			//	Header validity condition
			if (sampleRateNdx == 3)
			{
				return EFalse;
			}

			if (isLSFext)
			{
				bit_rate = bitrate_lookup[3 + (layer >> 1)][bit_rate];
			}
			else
			{
				bit_rate = bitrate_lookup[layer - 1][bit_rate];
			}

			TInt sfreq = (byte2 & 0x0c)>>2;			// 11-10 bits of the header
			switch (sfreq)
			{
				case 0:
				sfreq = 44100;
				break;

				case 1:
				sfreq = 48000;
				break;

				case 2:
				sfreq = 32000;
				break;

				default:
				return EFalse;	// error
			}

			if ( isLSFext )
			{
				sfreq >>= 1;
				if ( isMPEG2_5 )
				{
					sfreq >>= 1;
				}
			}

			TInt frameLen;
			// Calculate Frame Length
			//
			TInt pad_slot = (byte2 & 0x02)>>1;		// 9th bit of the header
			if ( layer == 1 )
			{
				frameLen = ((12 * bit_rate / sfreq) + pad_slot ) << 5;
			}
			else
			{
				TUint slots_per_frame;
				slots_per_frame = ((layer == 3) && isLSFext) ? 72 : 144;
				frameLen = ((slots_per_frame * bit_rate / sfreq) + pad_slot) << 3;
			}

			// Make sure that this frame is follwed by another valid frame.
			if ((frameLen > len) || (frameLen %8 != 0))
			{
				return EFalse;
			}
			else
			{
				frameLen = frameLen / 8;
				hdr += frameLen;
				len -= frameLen;
				if (len > 4)
				{
					byte0 = *(hdr+0);	//	bit 32-25 of the header
					byte1 = *(hdr+1);	//	bit 24-17 of the header

					if (byte0 == 0xff && ((byte1 & 0xe0) == 0xe0))
					{
						return ETrue;
					}
				}
				return EFalse;
			}
		}
	}

	// No valid MP3 header in the entire stream - give up
    return EFalse;
}

const TInt32 AACSampleRateTab[16] = {
    96000, 88200, 64000, 48000, 44100, 32000, 
	24000, 22050, 16000, 12000, 11025,  8000,
	0, 0 ,0, 0
};

// AAC Profile table
// Bits         ID = 1 (MPEG-2)					ID = 0 (MPEG-4 Object Type)
// 00			Main Profile						AAC Main
// 01			Low Complexity profile(LC)			AAC LC
// 10			Scalable Sample Rate Profile(SSR)	AAC SSR
// 11			(reserved)							AAC LTP

/*static*/
TBool MediaRecognizer::IsAACData(const TAny* aData, TInt aSize)
{
	if ( ( aData == NULL ) || ( aSize < 4 ) )
		return FALSE;

	U8*  uData = (U8*)aData;
	// check if it is ADIF format
	if (uData[0] == 'A' && uData[1] == 'D' && uData[2] == 'I' && uData[3] == 'F')
	{
		return TRUE;
	}

	// ADTS Header ?
	U32 header = 0;

	header = uData[0] << 24;
	header += uData[1] << 16;
	header += uData[2] << 8;
	header += uData[3];

	// AAC header format explained:
	// http://www.audiocoding.com/modules/wiki/?page=AAC
	//int test = header >> 18;
	
	int syncword      =     (header >> 20) & 0x00000fff;	// 12 bits: always 0xfff
	//int id	          =     (header >> 19) & 0x00000001;	// 1 bit: 0: MPEG-4; 1: MPEG-2
	int layer         =		(header >> 17) & 0x00000003;	// 2 bits: always '00'
	//int protection    =		(header >> 16) & 0x00000001;	// 1 bit:
	//int profile		  =     (header >> 14) & 0x00000003;	// 2 bits: 
	int sampleRateNdx =     (header >> 10) & 0x0000000f;	// 4 bits:
	//int privatebit    =     (header >>  9) & 0x00000001;	// 1 bit:
	//int channelconfig =     (header >>  6) & 0x00000007;	// 3 bit:
	//int org_copy      =     (header >>  5) & 0x00000001;	// 1 bit:
	//int home	      =     (header >>  4) & 0x00000001;	// 1 bit:
	//int emphasis      =     (header      ) & 0x00000003;	// 2 bits:  only if ID == 0 (ie MPEG-4)

	// check if header is valid
	if ( 
		   ( syncword      != 0xfff		   ) ||
		   ( layer         !=  0           ) ||
		   ( AACSampleRateTab[sampleRateNdx] == 0 )
     )
	{
		return	FALSE;
	}
	else
    {
		return TRUE;
    }
}

