/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class which manages read and write 
*				 requests to the MMC 
*
*/

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "saplayer_build.h"
#include "platformconfig.h"

#include<pathinfo.h>
#include<eikenv.h>
#include "symbian_mmc_reservoir.h"
#include "mmcactivityobserver.h"


 #ifdef FEATURE_MULTIPLE_DRIVE
 #include<driveinfo.h>
 #endif
 
_LIT(KCacheDirectory, "101FD693");
_LIT(KFileSeperator, "\\");
_LIT(KStreamFilePattern, "*.fsc");
_LIT(KCacheFileExtension, ".fsc");
	   
CMMCReservoir::~CMMCReservoir()
	{
	if(iPlatformConfig)
		{
		delete iPlatformConfig;
		iPlatformConfig=NULL;
		}
	iFileToDestroy = ETrue;
	if(IsActive())
		{
		Cancel();
		}
    TFileName filename;
    if(iFileCreated)
        {
        iFileHandle.FullName(filename);
        iFileHandle.Close();
        CEikonEnv::Static()->FsSession().Delete(filename);  
        }
	if(iTempDataBuffer)	
		{
		delete iTempDataBuffer;
		iTempDataBuffer = NULL;
		}
	iRequestArray.ResetAndDestroy();		
	}
      
// -------------------------------------------------------------------------
// CMMCReservoir::CMMCReservoir(CPlatformFlashEngine* aEngine)
//
// First phase constructor
// --------------------------------------------------------------------------
//      
CMMCReservoir::CMMCReservoir(CPlatformFlashEngine* aEngine
						, MMMCActivityObserver* aObserver)
						: CActive(CActive::EPriorityLow), iEngine(aEngine)
						, iWriteOffset(0),iReadOffset(0), iCycles(0)
						, iDrvNum(0),iObserver(aObserver)
						, iTempDataBufferPtr(0,0)
						
						

	{	
	CActiveScheduler::Add(this);
	}

// -------------------------------------------------------------------------
// void CMMCReservoir::ConstructL()

// Second phase constructor
// --------------------------------------------------------------------------
//	
TInt CMMCReservoir::InitializeL(TInt aStreamId, TInt aContentLength)
	{
	
	iPlatformConfig = new (ELeave) CPlatformConfig;
	ClearResidualTempFilesL();
	TInt retVal = CheckMemoryCardL(aContentLength);
	if(retVal == KErrNone)
		{
        HBufC* filePath = HBufC::NewLC(KMaxFileName);
	    TPtr dirPathPtr = filePath->Des();
	 
	
		///get the drive index 
	    	
	 	#ifdef FEATURE_MULTIPLE_DRIVE
	 

		User::LeaveIfError( PathInfo::GetRootPath(dirPathPtr, iDrvNum) );
       	dirPathPtr= dirPathPtr.Left(2);
                
        	 	
	 	#else
	    dirPathPtr.Append(PathInfo::MemoryCardRootPath().Left(2));
	    #endif
	    
        // build the path to the cache directory
        TFileName privatePath;
        RFs& fs =  CEikonEnv::Static()->FsSession();    
        fs.PrivatePath(privatePath);
        dirPathPtr.Append(privatePath);		
        
        dirPathPtr.Append(KCacheDirectory);
        dirPathPtr.Append(KFileSeperator);
        
        // create the cache directory if it doesn't exist
        retVal = fs.MkDir(dirPathPtr);
        if( retVal == KErrNone || retVal == KErrAlreadyExists )
		{
            // directory created!
            // build the cache file filename
            TFileName fileName;             
            GetFileName(fileName, aStreamId);
            dirPathPtr.Append(fileName);
            
            // Create the file if it does not exist.
            retVal  = iFileHandle.Open(CEikonEnv::Static()->FsSession(), dirPathPtr, EFileWrite);
            if( retVal == KErrNotFound )
            {
                retVal = iFileHandle.Create(CEikonEnv::Static()->FsSession(), dirPathPtr, EFileWrite);
                iFileCreated = ETrue;iFileToDestroy = EFalse;
            }
        }
/*
		else
		{
            // if(retVal == KErrNone) -> Directory was created. fine
            // if(retVal == KErrAlreadyExists) -> Already exists. fine
        
            // if error code is none of the above, then we have a problem here
			// Leaving so that it is correctly handled.
		
            // Leave is propogated all the way up to CHttpEngine::MHFRunL()
            // Since the caller checks for return value and if the return value != KErrNone,
            // the MMC Reservoir object is deleted.
            // So, if the cache directory was not created, just return the error value so that
            // MMC caching is not used.
			// User::Leave(retVal);
		
            // fall through and return 'retVal'
        }
*/
        CleanupStack::PopAndDestroy();  // remove filePath from cleanup stack and delete
	}
	
	return retVal;	
}
//utility functions
TInt CMMCReservoir::MemoryCardDriveL()
	{
	#ifndef FEATURE_MULTIPLE_DRIVE
    TInt err = RFs::CharToDrive( PathInfo::MemoryCardRootPath()[0], iDrvNum );
	return iDrvNum;
	
	#else ///FEATURE_MULTIPLE_DRIVE
	
	///the idea here is that if the cenrep bit is set,then select the drive which has maximum space
	//// and return the index.
		
	if(iPlatformConfig->GetMMCDriveOverridePolicy()== KMMCReservoirOverridePolicyVariable)
		{
		RFs fsSession = CEikonEnv::Static()->FsSession();
		DriveInfo::TDriveArray driveArray;
    	User::LeaveIfError( DriveInfo::GetUserVisibleDrives( fsSession, driveArray ) );
		
		TInt i=0;
        TInt driveIndex=0;	
       	TInt driveCount=driveArray.Count();
    	TInt64 maxFreespace=FreeSpaceL(driveArray[0]);
    	for(i=0; i < driveCount; i++)
    		{
    		TInt64 temp=FreeSpaceL(driveArray[i]);
    		
    		if(temp>maxFreespace)
	    		{
	    	
	    		maxFreespace=temp;
	    		driveIndex=i;	
	    		}
	    	   		       		        			
    		}
    
    	return driveArray[driveIndex];	
             
      }
      
	
	else
	{
	
	///if the cenrep override bit not set then take the default drive as E://.
    if(iPlatformConfig->GetVideoResDrive()== KVideoDrive)
        {
        TInt err = RFs::CharToDrive( PathInfo::MemoryCardRootPath()[0], iDrvNum );
        }
	    return iDrvNum;
	}
    	
    	
	#endif ///FEATURE_MULTIPLE_DRIVE
	
	}

_LIT(KTempFileName, "%d[%d]" );

void CMMCReservoir::GetFileName( TDes& aFilename, TInt aStreamID)
	{
	aFilename.Zero();
    aFilename.AppendFormat(KTempFileName, this, aStreamID);
    aFilename.Append(KCacheFileExtension);
 	}
 	
void CMMCReservoir::ClearResidualTempFilesL()
	{
	RFs& fs =  CEikonEnv::Static()->FsSession();
	
	TFileName privateDir;
	fs.PrivatePath(privateDir);
	
	HBufC* dirPath = HBufC::NewLC(KMaxFileName);
	TPtr dirPathPtr = dirPath->Des();
    dirPathPtr.Zero();
	
	#ifdef FEATURE_MULTIPLE_DRIVE
	// Residual files may be present on any drive, depending on the freespace available during the previous run.
	// So search in all the drives.
	DriveInfo::TDriveArray driveArray;
	User::LeaveIfError( DriveInfo::GetUserVisibleDrives( fs, driveArray ) );
	TInt intDrive;
	TChar driveLetter;
	for(TInt i=0; i < driveArray.Count(); i++)
    {
        driveLetter = driveArray.LetterAt(i);
        User::LeaveIfError( RFs::CharToDrive(driveLetter, intDrive) );
        if(intDrive != EDriveC)    // Lets not mess with C drive
        {
            User::LeaveIfError( PathInfo::GetRootPath(dirPathPtr, intDrive) );
            
            dirPathPtr= dirPathPtr.Left(2);
            dirPathPtr.Append(privateDir);
            dirPathPtr.Append(KCacheDirectory);
            dirPathPtr.Append(KFileSeperator);
            
            // Delete all the cache files in the tmp directory
            DeleteTempFilesL(dirPathPtr);
        }
    }
				
	#else //FEATURE_MULTIPLE_DRIVE
	
		dirPathPtr.Append(PathInfo::MemoryCardRootPath().Left(2));
		dirPathPtr.Append(privateDir);
        dirPathPtr.Append(KCacheDirectory);
        dirPathPtr.Append(KFileSeperator);
        DeleteTempFilesL(dirPathPtr);
		
	#endif //FEATURE_MULTIPLE_DRIVE
		
	CleanupStack::PopAndDestroy(dirPath);
}

void CMMCReservoir::DeleteTempFilesL(TPtr& aCacheDirectory)
{
    RFs& fs =  CEikonEnv::Static()->FsSession();
    CFileMan *fileMan = CFileMan::NewL(fs); //create file mgr for dir deletion
    CleanupStack::PushL(fileMan);
    
    // build the search pattern
    TPtr cacheFilePattern = aCacheDirectory;
    cacheFilePattern.Append(KStreamFilePattern);

    // Get all the files in the temp directory with the extension .fsc
    CDir* dirList = NULL;
    TInt err = fs.GetDir(cacheFilePattern,KEntryAttMaskSupported,ESortByName,dirList);
    if(err == KErrNone && dirList)
    {
        TInt fileCount = dirList->Count();
        for(TInt i=0; i<fileCount; i++)
        {
            TFileName fileToBeDeleted;
            fileToBeDeleted.Append(aCacheDirectory);
            fileToBeDeleted.Append((*dirList)[i].iName);    // (*dirList)[i].iName gives us just the file name in the directory

             
            TInt error = fileMan->Delete(fileToBeDeleted);
/*			
            if(error != KErrNone)
            {
                // file in use... Hence not deleted. Its ok.
				// Or the file may not be deleted due to some other reason. Ignore it...
            }
*/
        }
        
        delete dirList;
    }

    CleanupStack::PopAndDestroy(fileMan); //cleaup fileman object       
}

TInt CMMCReservoir::CheckMemoryCardL(TInt aContentLength)
	{
	

	iDrvNum = MemoryCardDriveL();
    TInt err = 
    	CEikonEnv::Static()->FsSession().CreatePrivatePath(iDrvNum);
    if(err!=KErrNone)
    	{
    	return err;
    	}
    TInt64 freespace = FreeSpaceL();
	// todo remove always on flag
  
	if( iPlatformConfig->GetMMCResUsagePolicy() == KMMCReservoirUsagePolicyVariable)
		{
		
		if( freespace > iPlatformConfig->GetMMCResHighWatermark()+ iPlatformConfig->GetMMCResvUnUtilisedSpace() )
			{
				iRecycleThreshold = iPlatformConfig->GetMMCResHighWatermark();
				return KErrNone;
			}
		else if( freespace > iPlatformConfig->GetMMCResLowWatermark()+iPlatformConfig->GetMMCResvUnUtilisedSpace() )	
			{
				iRecycleThreshold = freespace - iPlatformConfig->GetMMCResvUnUtilisedSpace();
				return KErrNone;
			}
		else if( freespace > aContentLength + iPlatformConfig->GetMMCResvUnUtilisedSpace() )
			{
			// Added to fix the bug : SRAA-78WD6L.
			if ( aContentLength == -1)
				{
				return KErrNoMemory;	
				}
			iRecycleThreshold = aContentLength;
			return KErrNone;
			}
			
#ifdef _DEBUG
		else if(aContentLength < freespace)
			{
				iRecycleThreshold = aContentLength;
				return KErrNone;
			//	return KErrNoMemory;			
			}
#endif
		else 
			{
				return KErrNoMemory;
			}	
		}
	// non-variable policy	
	else if( freespace > iPlatformConfig->GetMMCResLowWatermark() )
		{
		iRecycleThreshold = iPlatformConfig->GetMMCResLowWatermark();
		return KErrNone;
		}	
		

	return KErrNoMemory;
	}
	
TInt64 CMMCReservoir::FreeSpaceL()
	{
	TVolumeInfo vinfo;
    TInt errorCode = CEikonEnv::Static()->FsSession().Volume( vinfo, iDrvNum );
    User::LeaveIfError( errorCode );
	return TInt64( vinfo.iFree );
	}

///this function is added with the drive number as parameter to findout the available free space .
#ifdef FEATURE_MULTIPLE_DRIVE
TInt64 CMMCReservoir::FreeSpaceL(TInt aDrive)
{
	TChar res;
    TInt err = RFs::DriveToChar( aDrive, res );
	
	///our intention is to use the drive with maximum free space among E,F,G,H.....
	///so ignore the space of C drive.
	if(res=='C')
		{
			return 0;
		}
    else
	    {    	
		TVolumeInfo vinfo;
	    TInt errorCode = CEikonEnv::Static()->FsSession().Volume( vinfo, aDrive);
	    // return size as zero  for error codes like KErrNotReady.
	    if(errorCode != KErrNone)
	        {
                return 0;
	        }
	   // User::LeaveIfError( errorCode );
		return TInt64( vinfo.iFree );
	    }	
}
#endif //FEATURE_MULTIPLE_DRIVE
	
void CMMCReservoir::WriteToMMCL( const TDesC8& aData, 
                                       TInt aDataLength)
	{
    if( !aDataLength )
    	{
    	return;
    	}    	    

    iFileHandle.Seek( ESeekStart, iWriteOffset );
    
    if( (iWriteOffset + aDataLength) >= iRecycleThreshold)
    	{
    	User::LeaveIfError (iFileHandle.Write( aData.Left(iRecycleThreshold - iWriteOffset)));	    	 
    	RecycleL(aData.Right(iWriteOffset + aDataLength - iRecycleThreshold));
    	return;
    	}
    	
    User::LeaveIfError (iFileHandle.Write( aData ));
    iWriteOffset +=aDataLength;
	}
	
	
void CMMCReservoir::RecycleL(const TDesC8& aData)
	{
//	CEikonEnv::Static()->AlertWin(_L("recycled"));	
	iCycles++;
	
	User::LeaveIfError (iFileHandle.Write(0, aData ));
	iWriteOffset = aData.Length();
	}

TInt CMMCReservoir::ReadFromMMCL( 	TInt aOffset,
									TInt aDataLength)
	{
	if( !aDataLength )
    	{
    	return 0;
    	}
    TInt bytesToBeRead = aDataLength;
    //check if we are tryin to read more than we have written
    if(aOffset+aDataLength>(iCycles*iRecycleThreshold)+iWriteOffset)
    	{
    	bytesToBeRead = ((iCycles*iRecycleThreshold)+(iWriteOffset)) - aOffset;
    	}
    // get the actual offset/seek point from the absolute one which is 
    // passed as an argument	
    TInt seekPoint = GetSeekPoint(aOffset);
    TInt fileSize;
	iFileHandle.Size(fileSize);
	// make sure that we are not trying to seek 
    if(seekPoint < 0 || seekPoint > fileSize)
    	{
    	User::Leave(KErrNotSupported);
    	}
	iFileHandle.Seek( ESeekStart, seekPoint );
	if( seekPoint+bytesToBeRead >= fileSize )
		{
		bytesToBeRead = ( fileSize - seekPoint);
		}		
	if( IsActive())
		{
		// a read request is already pending
		// so queue this one
		AddToQueueL(aOffset,aDataLength);
		return bytesToBeRead;
		}
	iStatus = KRequestPending;	
	SetActive();	
	if( iTempDataBuffer )
		{
		delete iTempDataBuffer;
		iTempDataBuffer = NULL;
		}
	iTempDataBuffer = HBufC8::NewL( bytesToBeRead );
	TPtr8 tempDataBufferPtr = iTempDataBuffer->Des();
	iTempDataBufferPtr.Set(tempDataBufferPtr);	
	iFileHandle.Read( iTempDataBufferPtr, bytesToBeRead, iStatus);
	return bytesToBeRead;
	}
	
void CMMCReservoir::AddToQueueL(TInt aOffset, TInt aDataLength)	
{
	TReadRequest* readRequest = new(ELeave) TReadRequest;
	readRequest->iOffset = aOffset;
	readRequest->iDataLength = aDataLength;
	iRequestArray.Append(readRequest);
}
	
void CMMCReservoir::RunL()	
	{
	
	if(iStatus == KErrNone)
		{
		// data sent to the buffer		
		iObserver->ReadCompleteL( iStatus.Int(), iTempDataBufferPtr);
		
		delete iTempDataBuffer;
		iTempDataBuffer = NULL;
		
		if(iRequestArray.Count()>0)
			{
			TReadRequest* readRequest = iRequestArray[0];
			iRequestArray.Remove(0);			
			ReadFromMMCL(readRequest->iOffset, readRequest->iDataLength);			
			delete readRequest;
			}
		}
	else
		{
		User::Leave(iStatus.Int());
		}	
	}
	
TInt64 CMMCReservoir::GetSeekPoint(TInt aOffset)
	{
	TInt seekPosition;	
	
	if((aOffset <= ((iCycles)*iRecycleThreshold+iWriteOffset)) && (aOffset >= ((iCycles)*iRecycleThreshold)))
		{
		seekPosition = aOffset - (iCycles)*iRecycleThreshold;
		}
	else if(( aOffset >= ((iCycles-1)*iRecycleThreshold+iWriteOffset)) && (aOffset <= ((iCycles)*iRecycleThreshold)))
		{
		seekPosition = aOffset - (iCycles-1)*iRecycleThreshold;
		}
	else
		{
		return KErrNotFound;
		}
	return seekPosition;	
	}
	
void CMMCReservoir::CancelAllPendingRequests()
	{
	iRequestArray.ResetAndDestroy();
	Cancel();
	}	
	
TInt64 CMMCReservoir::GetMaxSize()
	{
	return iRecycleThreshold;
	}
	
TInt CMMCReservoir::RunError(TInt aStatus)
	{
	iObserver->NotifyError(aStatus);
	return KErrNone;
	}

void CMMCReservoir::DoCancel()
{	
	if(!iFileToDestroy)
	{
		RFile tempHandle;
		tempHandle.Duplicate(iFileHandle);
		iFileHandle.Close();
		iFileHandle = tempHandle;
	}
}
TInt64 CMMCReservoir::BytesWritten()
    {
    return iCycles*iRecycleThreshold + iWriteOffset;
    }
//end of file
