/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class used to support 
*				 the SI_GetNetworkInfo() and SI_GetNetworkInfoString() 
*				 Flash Lite API functions.
*
*/


#ifdef FEATURE_PACK_3

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "saplayer_build.h"

#include "symbian_networkinfo.h"
#include <etel3rdparty.h>
#include "commondef.h"


CNetworkInfo* CNetworkInfo::NewL()
{
	CNetworkInfo* self=new(ELeave) CNetworkInfo();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

CNetworkInfo::CNetworkInfo():	
	CActive(CActive::EPriorityLow)
   ,iTelephony(NULL)
   ,iNetworkInfoV1Pckg( iNetworkInfoV1 )
{
	
}


CNetworkInfo::~CNetworkInfo()
{
	Cancel();
	
	if(iTelephony)
	{
		delete iTelephony;
		iTelephony = NULL;
	}
	if(iWait)
	{
		if(iWait->IsStarted())
			iWait->AsyncStop();
		delete iWait;
	}

	
}

void CNetworkInfo::ListenL()
{
	//check this condition before calling
	//if (IsActive())
	//	return;
	
	if(!iTelephony)
	{
		iTelephony = CTelephony::NewL();	
	}	
	
	if(iTelephony)
	{
		iTelephony->GetCurrentNetworkInfo(iStatus, iNetworkInfoV1Pckg);
		if (!IsActive())
		{
			SetActive();
			//Wait for Async CTelephony Request to Complete.
			iWait->Start();
		}
}
	
}


void CNetworkInfo::ConstructL()
{

	CActiveScheduler::Add(this);
	
	//ListenL();//Not Required while creating the Object.
				//Will Listen once actual request comes.
	
	iWait = new (ELeave) CActiveSchedulerWait;

	
}

void CNetworkInfo::DoCancel()
{
	iTelephony->CancelAsync(CTelephony::EGetCurrentNetworkInfoCancel);
	//Cancel the Async Wait Loop also
	if (iWait->IsStarted())
	{
	iWait->AsyncStop();
	}

}
	

void CNetworkInfo::RunL()
{
	if(iStatus==KErrNone)
       {
       		if(iTelephony)
       		{
				delete iTelephony;
				iTelephony = NULL;
			}
       }
	//Async Request is done .
	if (iWait->IsStarted())
	{
		iWait->AsyncStop();
	}
}


TBool CNetworkInfo::GetNetworkInfo(unsigned short aInfoType, TDes8& aRetString)
{	
	TBool retVal = EFalse;
	TRAPD( err, retVal = GetNetworkInfoL(aInfoType, aRetString) );
	if ( err == KErrNone)
	{
		
	}
	return retVal;
}

TBool CNetworkInfo::GetNetworkInfoL(unsigned short aInfoType, TDes8& aRetString)
{
    TBool retValue = EFalse;
    //ListenL() is required Here because it might have been a long
	//time since Values have been updated!( if CNetworkInfo Instance 
	//was created long back, it might no longer even be Active!)
	if(!IsActive())
	{
	ListenL() ;
	}
	
	if(iStatus == KErrNone)
	{
		switch(aInfoType)
			{
				case FI_NetworkInfo_MobileNetworkCode:
				{
					aRetString.Copy(iNetworkInfoV1.iNetworkId);
					retValue = ETrue;
					break;
				}
				
				case FI_NetworkInfo_NetworkName:
				{
					
					aRetString.Copy(iNetworkInfoV1.iLongName);
					
					retValue = ETrue; 
					break;	
				}
				
				default:
					break;	
			} //switch
	}  

	//ListenL(); Called at the begining of the function
	
	return retValue;
}

enum 
	{
	EFlashNetworkModeUnknown = 0,
	EFlashNetworkMode2G,
	EFlashNetworkMode25G,
	EFlashNetworkmode3G
	};

TBool CNetworkInfo::GetNetworkInfo(unsigned short aInfoType, long& aRetVal)
{
	TBool retVal = EFalse;
	TRAPD( err, retVal = GetNetworkInfoL(aInfoType, aRetVal) );
	if ( err == KErrNone)
	{
		
	}
	return retVal;
} 

TBool CNetworkInfo::GetNetworkInfoL(unsigned short aInfoType, long& aRetVal)
{
    TBool retValue = ETrue;
	//ListenL() is required Here because it might have been a long
	//time since Values have been updated!( if CNetworkInfo Instance 
	//was created long back, it might no longer even be Active!)
	if(!IsActive())
	{
	ListenL() ;
	}
	

   	if(iStatus == KErrNone)
	{
		switch(aInfoType)
		{
			
			case FI_NetworkInfo_NetworkGeneration:
		    {
			        switch(iNetworkInfoV1.iMode)
			        {
			        
			        case CTelephony::ENetworkModeCdma95: // 1G
			        case CTelephony::ENetworkModeAmps: // 1G
			        case CTelephony::ENetworkModeUnregistered:
			        case CTelephony::ENetworkModeUnknown:
			        {
			            aRetVal = EFlashNetworkModeUnknown; // Unknown
			        }
			        break;
			        
			        case CTelephony::ENetworkModeGsm:
			        {
			            aRetVal = EFlashNetworkMode2G; // 2G
			        }
			        break;
			        
			        case CTelephony::ENetworkModeCdma2000: // 2.75G
			        {
			            aRetVal = EFlashNetworkMode25G; // 2.5G
			        }
			        break;
			        
			        case CTelephony::ENetworkModeWcdma:
			        {
			            aRetVal = EFlashNetworkmode3G; // 3G
			        }
			        break;

			        default:
			            retValue = EFalse;
			        break;

			       } // end switch(NetworkInfoV1.iMode)
	    	}//case FI_NetworkInfo_NetworkGeneration
	    	break;

	    	default:
			    {
			        retValue = EFalse;
			    }
			    break;
		}//switch(aInfoType)
			
	}//if(status == KErrNone)
	else
    {
        retValue = EFalse;
    }

   //ListenL() ;// Called at the begining of the function.
   	
   	
    return retValue;
}

#endif // FEATURE_PACK_3
