/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class used to handle 
*				 tel: URLs and make phone calls.
*
*/

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "saplayer_build.h"
#include "symbian_phonemonitor.h"
#include "symbian_phonecall.h"

class CPhoneMonitor;


CPhoneCall :: CPhoneCall ( CPhoneMonitor* apPhoneMonitor ) 
   : CActive ( EPriorityUserInput )
   ,iLineStatusPckg( iLineStatus )
   ,iPhoneMonitor(apPhoneMonitor)
   ,iCallState(kCallStateNone)

{
	iLineStatus.iStatus = CTelephony::EStatusUnknown;

	CActiveScheduler::Add ( this );
}

CPhoneCall :: ~CPhoneCall ( )
{
	Cancel ( );
	
	if(iTelephony)
	{
		delete iTelephony;
		iTelephony = NULL;
	}
	
}

void CPhoneCall :: InitialiseL ( )
{
	iTelephony = CTelephony::NewL();

}

TBool CPhoneCall::IsCallActive() 
{ 
	return iCallState == kCallStateDialling || iCallState == kCallStateTalking;
}

void CPhoneCall::ResetCallState() 
{ 
	iCallState = kCallStateNone;
}

void CPhoneCall :: CallTerminated ( )
{
	iCallState = kCallStateTerminated;
	iPhoneMonitor->CallTerminated ( this );
}

#if defined ( __WINS__ )
void CPhoneCall :: StartCallL ( const TDesC& /*phonenumber*/ )
{
#else	
void CPhoneCall :: StartCallL ( const TDesC& phonenumber )
{
		iCallState = kCallStateDialling;
		
		CTelephony::TTelNumber telNumber(phonenumber);

	    CTelephony::TCallParamsV1 callParams;
	    
	    callParams.iIdRestrict = CTelephony::ESendMyId;
	    
	    CTelephony::TCallParamsV1Pckg callParamsPckg(callParams);

	    iTelephony->DialNewCall(iStatus, callParamsPckg, telNumber, iCallId);
	    
		SetActive ( );

	#endif
}


CTelephony::TCallStatus CPhoneCall::GetCallStatus()
{
	if(iTelephony)
	{
		CTelephony::TCallStatusV1 callStatusV1;
		CTelephony::TCallStatusV1Pckg callStatusV1Pckg( callStatusV1 );

		CTelephony::TPhoneLine line = CTelephony::EVoiceLine;
		
		iTelephony->GetLineStatus( line, callStatusV1Pckg );	
		
		return callStatusV1.iStatus;
	}
	
	return (CTelephony::EStatusUnknown);
	
}

void CPhoneCall :: RunL ( )
{
	if ( iStatus == KErrNone )
	       {	// The call has been dialled successfully;
	          	// iCallId contains the call's ID, needed when controlling the call.
		if ( iCallState == kCallStateDialling )
		{
			iCallState = kCallStateTalking;
		}
				
				CTelephony::TCallStatus voiceLineStatus = GetCallStatus();
				
				if ( voiceLineStatus == CTelephony::EStatusUnknown ||
					 voiceLineStatus == CTelephony::EStatusIdle )
				{
					CallTerminated ( );
					return;
				}
				
				/* Request the next notification */
       			iTelephony->NotifyChange( iStatus,
                              CTelephony::EVoiceLineStatusChange,
                              iLineStatusPckg );

				SetActive ( );

			} 
	        else
			{
				// delayed destruction
				CallTerminated ( );
			}
		
    }

void CPhoneCall::DoCancel()
    {
    	iTelephony->CancelAsync(CTelephony::EDialNewCallCancel);
    }
    
//End of file
