/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that is used to 
*				 access phone id and name information.
*
*/


#ifdef FEATURE_PACK_3

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "symbian_phoneinfo.h"

CPhoneInfo* CPhoneInfo::NewL()
{
	CPhoneInfo* self=new(ELeave) CPhoneInfo();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

CPhoneInfo::CPhoneInfo(): 
	CActive(CActive::EPriorityLow)
   ,iTelephony(NULL)
   ,iPhoneIdV1Pckg( iPhoneIdV1 )
   ,iValidPhoneInfo(EFalse)
{

}

void CPhoneInfo::ListenL()
{
	if (IsActive())
		return;
	
	
	if(!iTelephony)
	{
		iTelephony = CTelephony::NewL();	
	}
	
	if(iTelephony)	
	{
		iTelephony->GetPhoneId(iStatus, iPhoneIdV1Pckg);
		
		if (!IsActive())
		{
			SetActive();
		}
	}
	
}

void CPhoneInfo::ConstructL()
{
	CActiveScheduler::Add(this);

	ListenL();
}


CPhoneInfo::~CPhoneInfo()
{
	Cancel();
	
	if(iTelephony)
	{
		delete iTelephony;
		iTelephony = NULL;
	}
	
}
					

void CPhoneInfo::DoCancel()
{
	iTelephony->CancelAsync(CTelephony::EGetPhoneIdCancel);
}
	

void CPhoneInfo::RunL()
{
	if(iStatus==KErrNone)
       {
       		iValidPhoneInfo = ETrue;
       		
       		if(iTelephony)
			{
				delete iTelephony;
				iTelephony = NULL;
			}
       }
}
					

TDesC&  CPhoneInfo::GetDeviceName()
{
	return iPhoneIdV1.iModel;
	
}

TDesC&  CPhoneInfo::GetDeviceID()
{
	return iPhoneIdV1.iSerialNumber;
	
}
 	  				
#endif //FEATURE_PACK_3

