/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class used by 
*				 CPlatformFlashEngine to keep track of the phone's call status.
*
*/

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "saplayer_build.h"
#include "symbian_ui_interface.h"
#include "symbian_phonemonitor.h"
#include "symbian_phonecall.h"
#include "symbian_engine.h"
#include <ctsydomainpskeys.h>
////////////////////////////////////////////////////////////////////////////////////


CPhoneMonitor::CPhoneMonitor() :
    CActive(CActive::EPriorityStandard), iNoMonitor(EFalse)

{
	//default constructor
	iIsCallActive = EFalse;
	iPhoneCall = NULL;
	iTelephony = NULL;
}

CPhoneMonitor::~CPhoneMonitor()
{
	Cancel();
    iProperty.Close();
	
	if(iPhoneCall)
		{
		delete iPhoneCall;
		iPhoneCall = NULL;	
		}
	if(iTelephony)
		{
		delete iTelephony;
		iTelephony = NULL;
		}
	
	}

void CPhoneMonitor::ConstructL(CPlatformFlashEngine* aEngine)
{
	iEngine = aEngine;
	
	if(iEngine)
		iNoMonitor = iEngine->NoIncomingCallMonitor();
	
	if(iNoMonitor)
		return;

#ifndef __WINS__
	CActiveScheduler::Add(this);

    User::LeaveIfError(iProperty.Attach(KPSUidCtsyCallInformation, KCTsyCallState));

    if (!IsActive())
		SetActive();
	
	iStatus = KRequestPending;
	
	TRequestStatus* temp=&iStatus;
	
    User::RequestComplete(temp,KErrNone);

#endif // __WINS__ 
}

TInt CPhoneMonitor::RunError(TInt /*aError*/)
    {
    //in case of error we can use use it.    
    return KErrNone;
    }

void CPhoneMonitor::RunL()
{

	if(iNoMonitor)
		return;
	
#ifndef __WINS__

    TInt state = EPSCTsyCallStateUninitialized;
    TInt error = iProperty.Get(state);

    if (!error)
        {

        switch (state)
            {
            case EPSCTsyCallStateNone:
                {
                Cancel();
                iIsCallActive = EFalse;
                iEngine->NotifyPhoneActiveL(iIsCallActive);
                break;
                }
            case EPSCTsyCallStateDisconnecting:
            case EPSCTsyCallStateAnswering:
            case EPSCTsyCallStateAlerting:
            case EPSCTsyCallStateRinging:
            case EPSCTsyCallStateConnected:
            case EPSCTsyCallStateDialling:
                {
                iIsCallActive = ETrue;
                iEngine->NotifyPhoneActiveL(iIsCallActive);
                break;
                }

            default:
            break;
            }
        }
    // After processing RUNL we will issue a next request.
		if (!IsActive())
		{
        iProperty.Subscribe(iStatus);
			SetActive();
	}

#endif
}

void CPhoneMonitor::DoCancel()
{
#ifndef __WINS__
    iProperty.Cancel();
#endif
}



TBool CPhoneMonitor::IsCallActive()
{
	return iIsCallActive;
}

void CPhoneMonitor::MakeCallL( const TDesC& aPhoneNumber )
{
	if ( iPhoneCall )
	{
		if ( iPhoneCall->IsCallActive() ) 
		{
			return;
		}

		delete iPhoneCall;
		iPhoneCall = NULL;
	}

	if ( !iPhoneCall )
	{
		CPhoneCall* pCallObject = new(ELeave) CPhoneCall(this);
		CleanupStack::PushL( pCallObject );
			
		pCallObject->InitialiseL();

		// For some reason, this delay is necessary on all FP1 phones. If the delay
		// is not present, the call to StartCallL() will fail. Even though this
		// delay is not needed on newer phones, it does no harm.
		User::After(1000);

		CleanupStack::Pop(pCallObject);
		iPhoneCall = pCallObject;
	}

	if ( !iPhoneCall->IsCallActive() )
	{
		iPhoneCall->StartCallL(aPhoneNumber);
	}

}

void CPhoneMonitor::CallTerminated ( CPhoneCall* /*pPhoneCall*/ )
{
}

/*
 * CPhoneMonitor::GetCallerInfoL(): Return Values for the Status:
 * 0-success,both name and number of caller available
 * 1-only number available
 * 2-only name available
 * 3-neither name nor number available
 * 4-Phone not in Active Call
 * -1- Not Supported: this is not returned by us as we are supporting this functionality now.
 */
TUint CPhoneMonitor::GetCallerInfoL(FI_ReturnString* aName,FI_ReturnString* aNumber)
	{
		{
		if(!iTelephony)
			{
			iTelephony = CTelephony::NewL();	
			}
	
		//Now Get the caller info.
		CTelephony::TCallInfoV1 callInfoV1;
		CTelephony::TCallInfoV1Pckg callInfoV1Pckg( callInfoV1 );

		CTelephony::TCallSelectionV1 callSelectionV1;
		CTelephony::TCallSelectionV1Pckg callSelectionV1Pckg( callSelectionV1 );

		CTelephony::TRemotePartyInfoV1 remotePartyInfoV1;
		CTelephony::TRemotePartyInfoV1Pckg remotePartyInfoV1Pckg( remotePartyInfoV1  );

		callSelectionV1.iLine = CTelephony::EVoiceLine;
		callSelectionV1.iSelect = CTelephony::EActiveCall;
		
		//Synchronous Call
		if(iTelephony)
			{
			iTelephony->GetCallInfo(callSelectionV1Pckg, callInfoV1Pckg, remotePartyInfoV1Pckg );
			}

		//Here we will check the call status from getcallinfo() itself
		//if we cant find any ACTIVE call, we return 4.
		if (callInfoV1.iStatus != CTelephony::EStatusConnected)
			{
			return 4;
			}
		
		//Now we got the caller info. Extract required info.
		//we need name and number of caller whichever is available.
		//Check if we have the info
		if (remotePartyInfoV1.iRemoteIdStatus == CTelephony::ERemoteIdentityAvailable)
			{
			
			if (remotePartyInfoV1.iCallingName.Length() != 0)
				aName->SetUTF16String(aName, (&remotePartyInfoV1.iCallingName)->PtrZ());
			
			if (remotePartyInfoV1.iRemoteNumber.iTelNumber.Length() != 0)				
				aNumber->SetUTF16String(aNumber, (&remotePartyInfoV1.iRemoteNumber.iTelNumber)->PtrZ());
				
			}
		
		if ( (remotePartyInfoV1.iCallingName.Length() != 0) && (remotePartyInfoV1.iRemoteNumber.iTelNumber.Length() != 0) )
			return 0;
		
		else if (remotePartyInfoV1.iRemoteNumber.iTelNumber.Length() != 0)			
			return 1;
			
		else if (remotePartyInfoV1.iCallingName.Length() != 0)
			return 2;
		
		else 
			return 3;
			
		
		}
	}




