/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that is used to 
*				 access phone version(OS , device etc) information.
*
*/


#ifdef FEATURE_PACK_3

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"

#include "symbian_platforminfo.h"
#include "symbian_engine.h"
#include "sysversioninfo.h"
#include "versioninfo.h"

_LIT(KLitOSName, "Symbian "); // name of symbian OS
_LIT(KLitS60Name,"-S60"); //name of S60 platform
_LIT(KLitVersionDot,".");
_LIT(KLitVersion,"v");
_LIT(KLitFP,"FP");
_LIT(KLitDash,"-");

const TInt KPlatformNameSize=100;

CPlatformInfo::CPlatformInfo(CPlatformFlashEngine* aEngine):
							 iEngine(aEngine)
							,iPlatformName(NULL)
							,iPlatNamePtr(NULL,0)
{
	
}
    
    
CPlatformInfo* CPlatformInfo::NewL(CPlatformFlashEngine* aEngine)
{
	CPlatformInfo* self=new(ELeave) CPlatformInfo(aEngine);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}


CPlatformInfo::~CPlatformInfo()
{
	if(iPlatformName)	
	{
		delete iPlatformName;
		iPlatformName = NULL;
	}
}
	

void CPlatformInfo::ConstructL()
{
		
}

const TDesC& CPlatformInfo::GetOSInfo()
{
	if(iPlatformName)
	{
		return iPlatNamePtr;	
	}
	
	return KNullDesC();
}

	
const TDesC& CPlatformInfo::GetOSInfoL()
{
	if(!iPlatformName)	
	{
		FindOSInfoL();
	}
	
	return iPlatNamePtr;
}
				
void CPlatformInfo::FindOSInfoL()
{
	
	//get S60 platform version

	VersionInfo::TPlatformVersion platformVersion;
	
	platformVersion.iMajorVersion = 0;
	platformVersion.iMinorVersion = 0;
	
	VersionInfo::GetVersion(platformVersion);
	
	SysVersionInfo::TSymbianOSVersion osVersion;
	
	osVersion.iMajorVersion = 0;
	osVersion.iMinorVersion = 0;
			
	TInt err = SysVersionInfo::GetVersionInfo( osVersion);
	
	if(err != KErrNone )
	{
		//we have error , so read from our resource file
		//this function impl in engine
		if(iEngine)
		{
		
			iPlatformName = HBufC::NewL(KPlatformNameSize);
		
			HBufC* platName=NULL;
			
			iEngine->GetPlatformNameL(platName);
			iPlatNamePtr.Set(iPlatformName->Des());
			if(platName)
			{
				iPlatNamePtr.Copy(platName->Des());
			
				//append S60 version info
				iPlatNamePtr.Append(KLitVersion());
				iPlatNamePtr.AppendNum(platformVersion.iMajorVersion);
				
				if(platformVersion.iMinorVersion)
				{
					iPlatNamePtr.Append(KLitDash());
					iPlatNamePtr.Append(KLitFP());
					iPlatNamePtr.AppendNum(platformVersion.iMinorVersion);
				}
				
				delete platName;	
	
			}
			
			
		}
	}
	else
	{	
		iPlatformName = HBufC::NewL(KPlatformNameSize);
		
		iPlatNamePtr.Set(iPlatformName->Des());
		
		iPlatNamePtr.Copy(KLitOSName());
		
		iPlatNamePtr.AppendNum(osVersion.iMajorVersion);
		
		iPlatNamePtr.Append(KLitVersionDot());
		
		iPlatNamePtr.AppendNum(osVersion.iMinorVersion);
		
		//we got version info, so let us form string		
		iPlatNamePtr.Append(KLitS60Name());
		iPlatNamePtr.Append(KLitVersion());
		iPlatNamePtr.AppendNum(platformVersion.iMajorVersion);
		
		if(platformVersion.iMinorVersion)
		{
			iPlatNamePtr.Append(KLitDash());
			iPlatNamePtr.Append(KLitFP());
			iPlatNamePtr.AppendNum(platformVersion.iMinorVersion);
		}
		
	}
	
}
#endif //FEATURE_PACK_3
