/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class used to access 
*				 network registration status .
*
*/


#ifdef FEATURE_PACK_3

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "saplayer_build.h"

#include "symbian_registrationstatus.h"
#include "commondef.h"

#include "engine_defs.h"
#include <etel3rdparty.h>


CRegistrationStatus* CRegistrationStatus::NewL()
{
	CRegistrationStatus* self=new(ELeave) CRegistrationStatus();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}


CRegistrationStatus::CRegistrationStatus():
	CActive(CActive::EPriorityLow)
   ,iTelephony(NULL)
   ,iNetworkRegistrationV1Pckg( iNetworkRegistrationV1 )
{
}

CRegistrationStatus::~CRegistrationStatus()
{

	Cancel();
	
	if(iTelephony)
	{
		delete iTelephony;
		iTelephony = NULL;
	}
	if(iWait)
	{
		if(iWait->IsStarted())
			iWait->AsyncStop();
		delete iWait;
	}
	
}

void CRegistrationStatus::ListenL()
{
	//check this condition before calling
	//if (IsActive())
	//	return;
		
	
	if(!iTelephony)
	{
		iTelephony = CTelephony::NewL();	
	}
	
	if(iTelephony)
	{
		iTelephony->GetNetworkRegistrationStatus(iStatus, iNetworkRegistrationV1Pckg);
	}
	
	if (!IsActive())
	{
		SetActive();
		//Wait for Async CTelephony Request to Complete.
		iWait->Start();
	}	
		
}

void CRegistrationStatus::ConstructL()
{
	
	
	CActiveScheduler::Add(this);
	//ListenL();//Not Required while creating the Object.
					//Will Listen once actual request comes.
		
	iWait = new (ELeave) CActiveSchedulerWait;
}

void CRegistrationStatus::DoCancel()
{
	iTelephony->CancelAsync(CTelephony::EGetNetworkRegistrationStatusCancel);
	//Cancel the Async Wait Loop also
	if (iWait->IsStarted())
	{
		iWait->AsyncStop();
	}
}
	

void CRegistrationStatus::RunL()
{
	if(iStatus==KErrNone)
       {
       		if(iTelephony)
			{
				delete iTelephony;
				iTelephony = NULL;
			}
       }
	//Async Request is done .
	if (iWait->IsStarted())
	{
		iWait->AsyncStop();
	}
}


TBool CRegistrationStatus::GetRegistrationStatus(unsigned short aInfoType, long& aRetValue)
{
	TBool retVal = EFalse;
	TRAPD( err, retVal = GetRegistrationStatusL(aInfoType, aRetValue) );
	if ( err == KErrNone)
	{
		//avoid warning
	} 
	return retVal;
}

TBool CRegistrationStatus::GetRegistrationStatusL(unsigned short aInfoType, long& aRetValue)
{
 
    TBool retStatus = ETrue;
    aRetValue = -1;
    //ListenL() is required Here because it might have been a long
	//time since Values have been updated!( if CRegistrationStatus Instance 
	//was created long back, it might no longer even be Active!)
	if(!IsActive())
		{
		ListenL() ;
		}
	
    if(iStatus != KErrNone)
    {
        retStatus = EFalse;
        return retStatus;
    }
    
    switch(aInfoType)
    {
        case FI_NetworkInfo_NetworkStatus:
        {
            switch(iNetworkRegistrationV1.iRegStatus)
            {
                case CTelephony::ENotRegisteredNoService:
                case CTelephony::ENotRegisteredEmergencyOnly:
                case CTelephony::ENotRegisteredSearching:
                case CTelephony::ERegistrationDenied:
                case CTelephony::ERegistrationUnknown:
                {
                    aRetValue = 0;
                }
                break;
    
                case CTelephony::ERegisteredOnHomeNetwork:
                {
                    aRetValue = 1;
                }
                break;
                
                case CTelephony::ERegisteredRoaming:
                {
                    aRetValue = 3;
                }
                break;
    
                case CTelephony::ERegisteredBusy:
                default:
                {
                    retStatus = EFalse;
                }
                break;	
            } //switch(iNetworkRegistrationV1.iRegStatus)
        } // case
        break;
        
        case FI_NetworkInfo_NetworkConnectStatus:
        {
            /* Network Registration status to corresponding NetworkConnect status 
             * value of Core player
             */
             switch(iNetworkRegistrationV1.iRegStatus)
             {
                 case CTelephony::ERegisteredOnHomeNetwork:
                 case CTelephony::ERegisteredRoaming:
                 {
                      aRetValue = ENetStatConnected;
                 }
                 break;
                 
                 case CTelephony::ENotRegisteredSearching:
                 {
                     aRetValue = ENetStatConnecting;
                 }
                 break;
                 
                 case CTelephony::ENotRegisteredEmergencyOnly:    
                 case CTelephony::ENotRegisteredNoService:
                 case CTelephony::ERegistrationUnknown:
                 {
                     aRetValue = ENetStatDisConnected;
                 }
                 break;
                 
                 case CTelephony::ERegistrationDenied:
                 case CTelephony::ERegisteredBusy:
                 {
                     aRetValue = ENetStatSuspended;
                 }
                 break;  
    
                 default:
                 {
                     retStatus = EFalse;                 
                 }
                 break;
             } //switch
        } // case
        break;
        
        default:
        {
            retStatus = EFalse;
            break;
        }
    } //switch

	//ListenL();// Called at the begining of the function.
	
    return retStatus;
}

#endif // FEATURE_PACK_3
