/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class which implements the
*				 main hostengine functionality.
*
*/

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "saplayer_build.h"
#include "flashliteinterfaces.h"

#include "flashliteinterfaces.h"
#include "symbian_plugin_interface.h"

#include "fi_interface.h"

#include "saflash_coredefs.h"
#include "symbian_engine.h"
#include "symbian_ui_interface.h"
#include "symbian_vibra.h"
#include "async_event.h"
#include "saflash_consts.h"
#include "symbian_stringutils.h"
#include "engine_data_caging_literals.hrh"

#include "amp_textnavigation.h"

#include "flash2ui.hrh"

#include <flash2engine.rsg>

#include <aknnotewrappers.h>
#include <AknGlobalNote.h>
#include <f32file.h>
#include <apgcli.h>
#include <sacls.h> // for RSystemAgent::GetState()
#include <smut.h> // for SMS
#include <sendui.h>
#include <SendUiConsts.h>
#include <miutset.h> // for SMTP
//#include <mmsconst.h> // for MMS
#include <hal.h>
//#include <plpvariant.h>
#include <AknBidiTextUtils.h>
#include <charconv.h>
#include <barsread.h>

#include "flashengineresourcehandler.h"

//#include "flash2engine.hrh"

#include <eikon.hrh>
_LIT(KCcTag, "cc=");
const TInt KCcTagLen = 3;

_LIT(KBccTag, "bcc=");
const TInt KBccTagLen = 4;

_LIT(KBodyTag, "body=");
const TInt KBodyTagLen = 5;

_LIT(KSubjectTag, "subject=");
const TInt KSubjectTagLen = 8;

const TInt KMaxUrlLength = 4096;
const TInt KTextWrapArrayGran=5;
const short int KVibraFeatureNotSupported =-1;
const TInt  kFlashCmdCustom1 = 400;
const TInt kFlashCmdCustom2 = 401;
BOOL CPlatformFlashEngine::LinebreakL(const struct FI_Text *text,
                                    struct FI_DeviceTextInfo *pDeviceTextInfo)
{
  	CFont *pFont = NULL;

	if (GetCurrentFont(pFont))
	{
		TDesC* des   = NULL;
        HBufC* pBufC = NULL;
		TPtrC ptr(text->text.utf16String, text->textLen);

        // only Unicode strings are supported
		ASSERT(text->textEncoding==FI_UTF16EncodingForm);
		des=&ptr;

        CArrayFixSeg<TInt>* lineWidthArray = new(ELeave) CArrayFixSeg<TInt>(KTextWrapArrayGran);
        CleanupStack::PushL(lineWidthArray);
        CArrayFixSeg<TPtrC>* wrappedArray  = new(ELeave) CArrayFixSeg<TPtrC>(KTextWrapArrayGran);
        CleanupStack::PushL(wrappedArray);

        // using the max TInt for the width if no wrapping is to be applied.
        // still need to convert from Logical to Visual string.
        if (!pDeviceTextInfo->wordWrap)
        {
			lineWidthArray->AppendL(KMaxTInt);
        }
        else
        {
            // there is indent on the 1st line
            if (pDeviceTextInfo->indent != 0)
            {
                lineWidthArray->AppendL(pDeviceTextInfo->width - pDeviceTextInfo->indent);
            }
            lineWidthArray->AppendL(pDeviceTextInfo->width);
        }

		const TDesC& desC=*des;
		const CFont& fontC=*pFont;

        pBufC = AknBidiTextUtils::ConvertToVisualAndWrapToArrayWholeTextL(
                    desC,
					*lineWidthArray,
					fontC,
					*wrappedArray);
        CleanupStack::PushL(pBufC);

        HBufC* pBufC2=HBufC::NewLC(pBufC->Length()+1);
        TPtr ptrBufC2=pBufC2->Des();
        ptrBufC2.Copy(*pBufC);

        pDeviceTextInfo->numOfVisualLines = wrappedArray->Count();

	    // Copy it back to pVisualText - null terminated
	    pDeviceTextInfo->pVisualText->SetUTF16String(pDeviceTextInfo->pVisualText, ptrBufC2.PtrZ());

        FI_ReturnUShortArray_Init(pDeviceTextInfo->visualTextLens, pDeviceTextInfo->numOfVisualLines);
        for (unsigned int i=0; i < pDeviceTextInfo->numOfVisualLines; i++)
        {
             FI_ReturnUShortArray_Append(pDeviceTextInfo->visualTextLens,
                                         (unsigned short) wrappedArray->At(i).Length());
        }

		CleanupStack::PopAndDestroy(pBufC2);
		CleanupStack::PopAndDestroy(pBufC);
		CleanupStack::PopAndDestroy(wrappedArray);
        CleanupStack::PopAndDestroy(lineWidthArray);

        return TRUE;
    }

    return FALSE;
}


unsigned short FI_OfferKeyEventWrapper(FI_Interface* fiInterface, unsigned short keyCode, FI_EventCode eventCode)
{
    FI_KeyId keyId;
    keyId.keyCode = keyCode;
    keyId.keyName = NULL;
    return fiInterface->FI_OfferKeyEvent(&keyId, (unsigned short)eventCode);
}

unsigned short FI_OfferKeyPressEventWrapper(FI_Interface* fiInterface, unsigned short keyCode)
{
    FI_KeyId keyId;
    keyId.keyCode = keyCode;
    keyId.keyName = NULL;
    return fiInterface->FI_OfferKeyPressEvent(&keyId);
}

#ifdef FEATURE_OPENVG
CFbsDevice* CPlatformFlashEngine::GetAuxTextFontDevice() const
{
    return iAuxTextBitmapDevice;
}
#endif

unsigned short CPlatformFlashEngine::GetDeviceFont(const struct FI_FontInfo * aFontInfo, TInt *aFontId)
{
	TBool retValue = EFalse;

	CFont * pFont = NULL;
	retValue = CreateFont(aFontInfo, pFont );

	*aFontId = 0;

	if(retValue)
	{
		if(!iFontMapTable)
		{
			TRAPD(err, iFontMapTable = new (ELeave) CFontMapTable(this));
			if(err != KErrNone)
				return 0;
		}

		TRAPD(err, *aFontId = iFontMapTable->AddNewFontEntryL(pFont, aFontInfo));
		if(err != KErrNone)
			return 0;

		return 1;
	}

	return (0);
}

TBool CPlatformFlashEngine::CreateFont(const struct FI_FontInfo * aFontInfo, CFont *& aFont )
{
	TBool rval = FALSE;

	// Check the logical title font. It is usually a scalable font on devices
	// that support scalable fonts.
	const CFont* font = AknLayoutUtils::FontFromId(EAknLogicalFontPrimarySmallFont);
	if (font->TypeUid() == KCFbsFontUid)
	{
		CFbsFont* fbsFont = (CFbsFont*)font;
		if (fbsFont->IsOpenFont())
		{		  
			// If this is an open font (i.e. scalable), then we want to use it!
			TFontSpec sys_spec = font->FontSpecInTwips();
			CFbsDevice * pFontDevice = NULL;
#ifdef FEATURE_OPENVG
			if(iOpenVgMode)
			{
                pFontDevice = GetAuxTextFontDevice();
			}
			else
			{
                pFontDevice = GetFontDevice();
			}
#else
            pFontDevice = GetFontDevice();
#endif
            
            /* Fix for TSW bug ESNA-85MERK: If swf goes to background on opening a new browser tab, 
               player will not be in openVGmode and iAuxTextBitmapDevice/iBitmapDevice would not have been initialized*/
            if(!pFontDevice)
            {
                aFont = NULL;
                return EFalse;
            }      
            
            TFontSpec spec(sys_spec.iTypeface.iName,aFontInfo->fontSize); 
            spec.iFontStyle.SetPosture(aFontInfo->italic ? EPostureItalic : EPostureUpright);
            spec.iFontStyle.SetStrokeWeight(aFontInfo->bold ? EStrokeWeightBold : EStrokeWeightNormal);
            spec.iFontStyle.SetPrintPosition(EPrintPosNormal);
            
            if (pFontDevice->GetNearestFontInPixels(aFont, spec) != KErrNone)
            {
            	aFont = NULL; 
            }
            return aFont != 0;
		}

	}

	TLanguage lang = User::Language();
	switch (lang)
	{
		case ELangTaiwanChinese:
		case ELangHongKongChinese:
		case ELangPrcChinese:
			rval = CreateChineseFont(aFontInfo, aFont);
			break;

        case ELangJapanese:
        case ELangThai:
			rval = CreateLocaleFont(aFontInfo, aFont);
			break;

		default:
			rval = CreateLatinFont(aFontInfo, aFont);
	}

	return rval;
}

const TInt KFontSize12 = 12;
const TInt KFontSize13 = 13;
const TInt KFontSize16 = 16;
const TInt KFontSize17 = 17;
const TInt KFontSize19 = 19;

TBool CPlatformFlashEngine::CreateLatinFont(const struct FI_FontInfo * aFontInfo, CFont *& aFont )
{
	TBool isAPAC = EFalse;

	EVariantFlag flag = AknLayoutUtils::Variant();
	if (flag == EApacVariant)
	{
		isAPAC = ETrue;
	}

	CFbsDevice * pFontDevice = GetFontDevice();
	if ( pFontDevice )
	{
		_LIT(KFontBoldItalic12, "Albi12" );
		_LIT(KFontBold12,		"LatinBold12" );
		_LIT(KFontItalic12,		"Alpi12" );
		_LIT(KFont12,			"LatinPlain12" );
		_LIT(KFontBoldItalic13, "Albi13" );
		_LIT(KFontBold13,		"LatinBold13" );
		_LIT(KFontItalic13,		"Alpi13" );
		_LIT(KFont13,			"Alp13" );
		_LIT(KFontBoldItalic17, "albi17b" );
		_LIT(KFontBold17,		"Alb17b" );
		_LIT(KFontItalic17,		"alpi17" );
		_LIT(KFont17,			"alp17" );
		_LIT(KFont19,			"LatinBold19" );

		TBuf<KMaxTypefaceNameLength> fontface;
		int fontSize;
		if ( aFontInfo->fontSize <= KFontSize12 )
		{
			if ( aFontInfo->bold )
			{
				if ( aFontInfo->italic )
					fontface.Copy ( KFontBoldItalic12 );
				else
					fontface.Copy ( KFontBold12 );
			}
			else
			{
				if ( aFontInfo->italic )
					fontface.Copy ( KFontItalic12 );
				else
					fontface.Copy ( KFont12 );
			}
			fontSize = KFontSize12;
		}
		else if ( aFontInfo->fontSize < KFontSize17 )
		{
			if ( aFontInfo->bold )
			{
				if ( aFontInfo->italic )
					fontface.Copy ( KFontBoldItalic13 );
				else
					fontface.Copy ( KFontBold13 );
			}
			else
			{
				if ( aFontInfo->italic )
					fontface.Copy ( KFontItalic13 );
				else
					fontface.Copy ( KFont13 );
			}
			fontSize = KFontSize13;
		}
		else if ( ( aFontInfo->fontSize < KFontSize19 ) || ( isAPAC ) )
		{
			if ( aFontInfo->bold )
			{
				if ( aFontInfo->italic )
					fontface.Copy ( KFontBoldItalic17 );
				else
					fontface.Copy ( KFontBold17 );
			}
			else
			{
				if ( aFontInfo->italic )
					fontface.Copy ( KFontItalic17 );
				else
					fontface.Copy ( KFont17 );
			}
			fontSize = KFontSize17;
		}
		else {
			fontface = KFont19;
			fontSize = KFontSize19;
		}

		TFontSpec spec ( fontface, fontSize );
		if ( pFontDevice->GetNearestFontInPixels(aFont, spec) != KErrNone )
		{
			aFont = NULL;
		}
	}
	return aFont != 0;
}

// This function creates a device based on the locale language. This is
// necessary for some locales because we do not have the device font name.
TBool CPlatformFlashEngine::CreateLocaleFont(const struct FI_FontInfo * aFontInfo, CFont *& aFont )
{
	// Create a device font for the locale specific language based on
	// the logical Secondary font.
	const CFont* font = AknLayoutUtils::FontFromId(EAknLogicalFontSecondaryFont);
	if (font->TypeUid() == KCFbsFontUid)
	{
		TFontSpec sys_spec = font->FontSpecInTwips();
		CFbsDevice * pFontDevice = GetFontDevice();
		TFontSpec spec(sys_spec.iTypeface.iName, aFontInfo->fontSize);
		spec.iFontStyle.SetPosture(aFontInfo->italic ? EPostureItalic : EPostureUpright);
		spec.iFontStyle.SetStrokeWeight(aFontInfo->bold ? EStrokeWeightBold : EStrokeWeightNormal);
		spec.iFontStyle.SetPrintPosition(EPrintPosNormal);
		if ( pFontDevice->GetNearestFontInPixels(aFont, spec)  != KErrNone )
		{
			aFont = NULL;
		}
	}

	return aFont != 0;
}

TBool CPlatformFlashEngine::CreateChineseFont(const struct FI_FontInfo * aFontInfo, CFont *& aFont )
{
	CFbsDevice * pFontDevice = GetFontDevice();

	if ( pFontDevice )
	{
		_LIT(KFontPlain12, "CombinedChinesePlain12" );
		_LIT(KFontPlain16, "CombinedChinesePlain16" );

		TBuf<KMaxTypefaceNameLength> fontface;
		int fontSize;
		if ( aFontInfo->fontSize <= KFontSize12 )
		{
			fontface = KFontPlain12;
			fontSize = KFontSize12;
		}
		else
		{
			fontface = KFontPlain16;
			fontSize = KFontSize16;
		}

		TFontSpec spec ( fontface, fontSize );
		if ( pFontDevice->GetNearestFontInPixels(aFont, spec) != KErrNone )
		{
			aFont = NULL;
		}
	}
	return aFont != 0;
}

const TInt32 KFixedPointPosTen = 0x000A0000;
const TInt32 KFixedPointNegTen = 0xFFF60000;

TKeyResponse CPlatformFlashEngine::OfferKeyEventL ( const TKeyEvent& aKeyEvent, TEventCode aType )
{

	if (iPlayerState == EPlayerStoppedError || !iIsInitialized )
	{
		return EKeyWasNotConsumed;
	}

	TKeyResponse rval = EKeyWasNotConsumed;
//Fix for Taika rc: ou1cimx1#608631, since we receive the appropriate keycode of the pressed key from platform 
// we need not check Function modifier.
/*
	//Fix for Ivalo ATS:QWERTY keyboard devices.'*' or '#' should be pressed
  //along with func key to get the icode.
	if( aKeyEvent.iModifiers & EModifierRightFunc )
	{
		unsigned short code;
		code = (unsigned short)aKeyEvent.iCode;
		if(aType == EEventKey)
	  	{
	     	FI_OfferKeyPressEventWrapper(&iFiObject,code);
	     	PlayStep(ETrue);
	     	return EKeyWasConsumed;
	  	}
	}*/

	FI_EventCode eventCode;

	// In case character received from  non standard keyboard e.g VKB
	// we need to change the buffer held by the core player


	if( aKeyEvent.iModifiers == EModifierSpecial )
	{
		unsigned short code;
		code = (unsigned short)aKeyEvent.iCode;
		if(aType == EEventKey)
	        {
	        //in case of secrettext  mode EKeyF20  is sent to delete the previouse character
	        if( aKeyEvent.iCode == EKeyF20 )
	           FI_OfferKeyPressEventWrapper(&iFiObject,FI_KEY_BACKSPACE );
	        else
	            FI_OfferKeyPressEventWrapper(&iFiObject,code);

	        PlayStep(ETrue);
	        return EKeyWasConsumed;
	        }
	}

    CArrayPtr<CFlashExtension>* exts=iFiObject.Extensions();

	// We only allow key press events to get processed by MMI
	// since controls higher up on the stack, e.g. IME's like T-9/Zi
	// will consume the press events but not down/up events --
	// this wreaks havoc with behaviors associated with soft keys
	#ifdef FEATURE_INLINE_TEXT_ENTRY
	if ( exts && iIMEControlState == EIMENotInitialized )
	#else
	if(exts)
	#endif
	{
		// first offer key press to any extensions to see if they know how to deal with this key
		// note this might not work well for more than 1 extension because there is no
		// notion of 'priority' for key handling
		const TInt numExts=exts->Count();


		CFlashExtension* ext=NULL;

		for(TInt i=0;i<numExts;i++)
		{
			ext=(*exts)[i];
			if(ext->HandleExtendedKeyEventL(iFiObject.GetMMObject(),aKeyEvent,aType))
			{
				// return once the extended key is processed. Now soft1 and pageup are the same. So
                // no need to continue to offer pageup/pagedown key events for soft keys.
                // otherwise content will get 2 key events.
                return EKeyWasConsumed;
			}
		}
	}

	// otherwise do the normal key handling stuff...

//	FI_EventCode eventCode;

	if ( aType == EEventKeyDown )
		eventCode = FI_EventKeyDown;
	else if ( aType == EEventKeyUp )
		eventCode = FI_EventKeyUp;
	else if ( aType == EEventKey )
		eventCode = FI_EventKey;
	else
		return EKeyWasNotConsumed;

	if (IsPanning())
	{
		// In Panning mode, translate the Flash content by ten units up/down or left/right
		// on directional key events
		if (eventCode == FI_EventKey)
		{
			switch (aKeyEvent.iScanCode)
			{
				case EStdKeyLeftArrow:
				iFiObject.FI_Translate(KFixedPointNegTen, 0);
				rval = EKeyWasConsumed;
				break;

				case EStdKeyRightArrow:
				iFiObject.FI_Translate(KFixedPointPosTen, 0);
				rval = EKeyWasConsumed;
				break;

				case EStdKeyUpArrow:
				iFiObject.FI_Translate(0, KFixedPointNegTen);
				rval = EKeyWasConsumed;
				break;

				case EStdKeyDownArrow:
				iFiObject.FI_Translate(0, KFixedPointPosTen);
				rval = EKeyWasConsumed;
				break;
#ifdef ENABLE_OFN_SUPPORT
                case EStdKeyLeftUpArrow:
                    iFiObject.FI_Translate(KFixedPointNegTen,
                            KFixedPointNegTen);
                    rval = EKeyWasConsumed;
                    break;
                case EStdKeyRightUpArrow:
                    iFiObject.FI_Translate(KFixedPointPosTen,
                            KFixedPointNegTen);
                    rval = EKeyWasConsumed;
                    break;
                case EStdKeyRightDownArrow:
                    iFiObject.FI_Translate(KFixedPointPosTen,
                            KFixedPointPosTen);
                    rval = EKeyWasConsumed;
                    break;
                case EStdKeyLeftDownArrow:
                    iFiObject.FI_Translate(KFixedPointNegTen,
                            KFixedPointPosTen);
                    rval = EKeyWasConsumed;
                    break;
#endif
			}
			PlayStep(ETrue);
		}
		return rval;
	}

	switch ( aKeyEvent.iScanCode )
	{
	case EStdKeyLeftArrow:
		FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_LEFT, eventCode );
		rval = EKeyWasConsumed;
		break;

	case EStdKeyRightArrow:
		FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_RIGHT, eventCode );
		rval = EKeyWasConsumed;
		break;

	case EStdKeyUpArrow:
		FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_UP, eventCode );
		rval = EKeyWasConsumed;
		break;

	case EStdKeyDownArrow:
		FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_DOWN, eventCode );
		rval = EKeyWasConsumed;
		break;

#ifdef ENABLE_OFN_SUPPORT
        case EStdKeyLeftUpArrow:
            FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_LEFT, eventCode);
            FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_UP, eventCode);
            rval = EKeyWasConsumed;
            break;
        case EStdKeyRightUpArrow:
            FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_RIGHT, eventCode);
            FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_UP, eventCode);
            rval = EKeyWasConsumed;
            break;
        case EStdKeyRightDownArrow:
            FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_RIGHT, eventCode);
            FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_DOWN, eventCode);
            rval = EKeyWasConsumed;
            break;
        case EStdKeyLeftDownArrow:
            FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_LEFT, eventCode);
            FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_DOWN, eventCode);
            rval = EKeyWasConsumed;
            break;
#endif
	case EStdKeyDevice3:
		/*
		if (eventCode == FI_EventKey)
		{
			iFiObject.FI_OfferKeyPressEvent(FI_KEY_SELECT);
		}
		*/
		FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_SELECT, eventCode );
		rval = EKeyWasConsumed;
		break;

	case EStdKeyDevice1:
		// this key only sends down and up event, so when we see down, send all three to Flash
            if ((iSoftkeyArraySize && iSoftkeyArray[1]
                 && (eventCode == FI_EventKey))
                 || ((iExecutionMode == ePlugin)
            		&& (eventCode == FI_EventKeyDown)))
		{
			FI_OfferKeyPressEventWrapper(&iFiObject, FI_KEY_PAGEDOWN);
			rval = EKeyWasConsumed;
		}

		break;

	case EStdKeyDevice0:
		// this key only sends down and up event, so when we see down, send all three to Flash
            if (iSoftkeyArraySize && iSoftkeyArray[0] && (eventCode
                    == FI_EventKey))
		{
			FI_OfferKeyPressEventWrapper(&iFiObject, FI_KEY_PAGEUP);
			rval = EKeyWasConsumed;
		}

		break;

	case EStdKeyEnter:
		if (eventCode == FI_EventKey)
		{
			#ifdef FEATURE_INLINE_TEXT_ENTRY
			if(iIMEControlState == EIMEEnabled )
			FI_OfferKeyPressEventWrapper(&iFiObject, FI_KEY_ENTER);
			else
			#endif
			FI_OfferKeyPressEventWrapper(&iFiObject, FI_KEY_SELECT);
		}
		rval = EKeyWasConsumed;
		break;


	case EStdKeyNkpAsterisk:
	case '*':
		FI_OfferKeyEventWrapper(&iFiObject, '*', eventCode);
		rval = EKeyWasConsumed;
		break;

	case EStdKeyHash:
	case '#':
		FI_OfferKeyEventWrapper(&iFiObject, '#', eventCode);
		rval = EKeyWasConsumed;
		break;

	case EStdKeyBackspace:
		FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_BACKSPACE, eventCode );
		rval = EKeyWasConsumed;
		break;

	case EStdKeyTab:
		FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_TAB, eventCode );
		rval = EKeyWasConsumed;
		break;

	case EStdKeyEscape:
		FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_ESCAPE, eventCode );
		rval = EKeyWasConsumed;
		break;

	case EStdKeyCapsLock:
		FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_CAPS, eventCode );
		rval = EKeyWasConsumed;
		break;

	case EStdKeyLeftShift:
	case EStdKeyRightShift:
		FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_SHIFT, eventCode );
		rval = EKeyWasConsumed;
		break;

	case EStdKeyLeftCtrl:
	case EStdKeyRightCtrl:
		FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_CTRL, eventCode );
		rval = EKeyWasConsumed;
		break;

	case EStdKeySpace:
		FI_OfferKeyEventWrapper(&iFiObject, ' ', eventCode );

		rval = EKeyWasConsumed;
		break;

	case EStdKeyDelete:
		FI_OfferKeyEventWrapper(&iFiObject, FI_KEY_DELETE, eventCode );
		rval = EKeyWasConsumed;
		break;

	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
		{
			if (aKeyEvent.iModifiers & EModifierShift)
			{
				if(EEventKey==aType)
				{
					// key is a shifted number - only get actual char code in EEventKey - must fake down/up
					FI_OfferKeyPressEventWrapper(&iFiObject, (unsigned short)aKeyEvent.iCode);
				}
			}
			else
			{
				// leave key as number.
				FI_OfferKeyEventWrapper(&iFiObject, (unsigned short)aKeyEvent.iScanCode, eventCode);
			}

			rval = EKeyWasConsumed;
		}
		break;

	case EStdKeyXXX:
	case EStdKeyComma:
	case EStdKeyFullStop:
	case EStdKeyForwardSlash:
	case EStdKeyBackSlash:
	case EStdKeySemiColon:
	case EStdKeySingleQuote:
	//case EStdKeyHash:
	case EStdKeySquareBracketLeft:
	case EStdKeySquareBracketRight:
	case EStdKeyMinus:
	case EStdKeyEquals:
		{
			if(EEventKey==aType)
			{
				// these keys will be potentially remapped on different keyboards - only rely on the real code
				// in EEventKey, and fake Up/down for the player
				FI_OfferKeyPressEventWrapper(&iFiObject, (unsigned short)aKeyEvent.iCode);
			}
			rval = EKeyWasConsumed;
		}
		break;

	case '+':
		{
			unsigned short code;
			if (aKeyEvent.iModifiers & EModifierShift)
			{
				// look up the code in the shifted other key table
				code = (unsigned short)'=';
			}
			else
			{
				// look up the code in the unshifted other key table
				code = (unsigned short)'+';
			}
			FI_OfferKeyEventWrapper(&iFiObject, code, eventCode);
			rval = EKeyWasConsumed;
		}
		break;


	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
		{
			unsigned short code;
			TUint isShifted = aKeyEvent.iModifiers & EModifierShift;
			TUint isCapsLocked = aKeyEvent.iModifiers & EModifierCapsLock;

			if ((isShifted || isCapsLocked)	&& !(isShifted && isCapsLocked))
			{
				// leave key as upper case.
				code = (unsigned short)aKeyEvent.iScanCode;
			}
			else
			{
				// convert key code to lower case.
				code = (unsigned short)(aKeyEvent.iScanCode - 'A' + 'a');
			}
			FI_OfferKeyEventWrapper(&iFiObject, code, eventCode);
			rval = EKeyWasConsumed;
		}
		break;


	default:
		return EKeyWasNotConsumed;
	}

	if ( iIsStopped == EFalse)
	{
		PlayStep(ETrue);
	}
	return rval;
}



void CPlatformFlashEngine::SendPointertoSoftKey(TInt aSoftKeyID)
{
	if (iSoftkeyArraySize && iSoftkeyArray[1]  && (aSoftKeyID == kFlashCmdCustom2))
	    	FI_OfferKeyPressEventWrapper(&iFiObject, FI_KEY_PAGEDOWN);


	if (iSoftkeyArraySize && iSoftkeyArray[0]  && (aSoftKeyID == kFlashCmdCustom1))
	    	FI_OfferKeyPressEventWrapper(&iFiObject, FI_KEY_PAGEUP);

}


_LIT(KLitWebBrowserArgs,"4 ");

void CPlatformFlashEngine::HandleBrowserURLL(const TDesC& aUrl)
{
	if(iBrowserUid.iUid==KNullUidValue)
	{
		TInt resoffset = FlashEngineRscHandler::LoadEngineResourceFileL();
		TResourceReader reader;
		iEnv->CreateResourceReaderLC(reader,R_FLASH_BROWSER_UID_VALUE);
		iBrowserUid.iUid=reader.ReadInt32();
		CleanupStack::PopAndDestroy(); //reader
		FlashEngineRscHandler::DeleteEngineResourceFile(iEnv,resoffset);
	}

	TPtrC ptrUrl(aUrl);

    if (ptrUrl.Length() > KMaxUrlLength - KLitWebBrowserArgs().Length())
        {
        //if(!iBrowserPluginActive)
    	if(iExecutionMode == eStandAlone)
	     {
            //iFlashUI->NotifyDataAccessError(NULL, ECompleteFailedURLTooLong); // cannot be allowed to leave
            iFlashClientIf->GetUserNotifierIf()->NotifyDataAccessError(NULL, ECompleteFailedURLTooLong); // cannot be allowed to leave
            User::Leave(KErrBadName);
	     }
    	}

    RApaLsSession lsSession;
    CleanupClosePushL(lsSession); // 1
    User::LeaveIfError(lsSession.Connect());

    TBuf16<KMaxUrlLength> urlBuf;

    TBool browserCanHandle = ETrue;
    TBool isLocalFile = EFalse;
    TPtrC ptrUrlSchemeStripped;
    isLocalFile = StripPrefix(ptrUrl, KProtocolPrefixFileUnixStyle, ptrUrlSchemeStripped);
    if (!isLocalFile)
    {
        isLocalFile = StripPrefix(ptrUrl, KProtocolPrefixFile, ptrUrlSchemeStripped);
    }

    if (isLocalFile)
	{
        HBufC16* pathBuf = ptrUrlSchemeStripped.AllocLC(); // 2
        TPtr16 ptrPath = pathBuf->Des();
        ReplaceAll(ptrPath, KCharForwardSlash, KCharBackslash);

        // Check that file exists.
        RFile file;
		TInt errValue = file.Open(iFs, ptrPath, EFileRead);
		if (errValue != KErrNone)
		{
			User::Leave (KErrNotFound);
		}
		file.Close();

        // Check that browser can handle document.
        TDataType dataType;
        TUid uid;
		User::LeaveIfError(lsSession.AppForDocument(ptrPath, uid, dataType));
		if (uid != iBrowserUid)
		{
            browserCanHandle = EFalse;
            TApaTaskList taskList (iEnv->WsSession());
             if(uid.iUid!=KNullUidValue)
            {
                TApaTask task = taskList.FindApp(uid);
                if (task.Exists())
                {
                    User::LeaveIfError(task.SwitchOpenFile(ptrPath));
                    task.BringToForeground();
                }
                else
                {
                    TThreadId threadId;
                    User::LeaveIfError(lsSession.StartDocument(ptrPath, threadId));
                }
            }
            else
            {
                TThreadId threadId;
                User::LeaveIfError(lsSession.StartDocument(ptrPath, threadId));
            }
		}
        else
        {
            // Construct URL of form "file://e:/Others/subdir/document.html".
            urlBuf.Copy(aUrl);
            ReplaceAll(urlBuf, KCharBackslash, KCharForwardSlash);
        }
        CleanupStack::PopAndDestroy(pathBuf); // 1
	}
	else
	{
        urlBuf.Copy(ptrUrl);

    }

    if(browserCanHandle)
    {
        TApaTaskList taskList(iEnv->WsSession());
	    TApaTask task = taskList.FindApp(iBrowserUid);

        if (task.Exists()) // Browser already running.
	    {
		    if (ptrUrl.Length() > 0)
		    {
			    HBufC8* param = HBufC8::NewLC(urlBuf.Length()); // 2
                TPtr8 ptr = param->Des();
                TUint oldCodePage = iCurrentCodePage;
			    ChangeCodePageL(KCharacterSetIdentifierUtf8);
			    TInt state = CCnvCharacterSetConverter::KStateDefault;
			    iCharacterConverter->ConvertFromUnicode(ptr, urlBuf, state);
                ChangeCodePageL(oldCodePage);
                task.SendMessage(TUid::Uid(KUidApaMessageSwitchOpenFileValue), *param);
			    CleanupStack::PopAndDestroy(param); // 1
		    }
	    }
	    else // Start browser.
	    {
            urlBuf.Insert(0, KLitWebBrowserArgs);

            RApaLsSession ls;
            User::LeaveIfError(ls.Connect());
            TThreadId id;
            ls.StartDocument(urlBuf, iBrowserUid, id);
            ls.Close();
	    }
    }

    CleanupStack::PopAndDestroy(&lsSession); // 0
}

unsigned short CPlatformFlashEngine::HandleMailToL(TInt msgType, const TDesC& aMsgString, TBool aIsEmbedded)
//
// extract body and subject from unencoded msg string
//
{
	TPtrC16 addrData;
    TPtrC16 ccData;
    TPtrC16 bccData;
	TPtrC16 bodyData;
	TPtrC16 subjData;

    HBufC16* msg = aMsgString.AllocLC();

    // Extract address.
    TInt qMarkIndex = msg->Locate('?');
    if(qMarkIndex == KErrNotFound)
    {
        addrData.Set(msg->Des());
    }
    else
    {
        addrData.Set(msg->Mid(0, qMarkIndex));
        TPtrC16 args;
        args.Set(msg->Mid(qMarkIndex+1));

        TBool lastAmpersandFound = EFalse;
        while(args.Length() > 0 && !lastAmpersandFound)
        {
            TInt ampIndex = args.Locate('&');
            if(ampIndex == KErrNotFound)
            {
                lastAmpersandFound = ETrue;
                ampIndex = args.Length();
            }
            if(args.FindF(KCcTag) == 0)
            {
                ccData.Set(args.Mid(KCcTagLen, ampIndex - KCcTagLen));
            }
            if(args.FindF(KBccTag) == 0)
            {
                bccData.Set(args.Mid(KBccTagLen, ampIndex - KBccTagLen));
            }
            else
            if(args.FindF(KBodyTag) == 0)
            {
                bodyData.Set(args.Mid(KBodyTagLen, ampIndex - KBodyTagLen));
            }
            else
            if(args.FindF(KSubjectTag) == 0)
            {
                subjData.Set(args.Mid(KSubjectTagLen, ampIndex - KSubjectTagLen));
            }
            if( !lastAmpersandFound && ampIndex != args.Length())
            {
                args.Set(args.Mid(ampIndex + 1));
            }
        }
    }

	TUid uid(KNullUid);

	switch (msgType)
	{
	case ESMTP:
		uid=KUidMsgTypeSMTP;
		break;

	case EMMS:
			// commenting out the code written for 3.1 product family as FL 3.1 code for 5.0 will not be used for 3.1 products.
			/*#ifdef __SERIES60_31__
				uid = KUidMsgTypeMultimedia; //unified message editor
			#else*/
        		uid = KSenduiMtmUniMessageUid;
        	//#endif
		break;

	case ESMS:
			// commenting out the code written for 3.1 product family as FL 3.1 code for 5.0 will not be used for 3.1 products.
			/*#ifdef __SERIES60_31__
				uid = KUidMsgTypeSMS;   //unified message editor
			#else*/
				uid = KSenduiMtmUniMessageUid;
			//#endif
		break;

	default:
		ASSERT(EFalse); // crash here in debug mode
	}

	TMailInfo mailInfo;
	mailInfo.iAddress.Set(addrData);
	mailInfo.iCc.Set(ccData);
	mailInfo.iBcc.Set(bccData);
	mailInfo.iSubject.Set(subjData);
	mailInfo.iBodyText.Set(bodyData);

	MDeviceExtnIf* deviceIf = iFlashClientIf->GetDeviceIf();
	if (deviceIf)
        {
        //if (!iBrowserPluginActive)
		if(iExecutionMode == eStandAlone)
            {
            deviceIf->SendMessageL(uid, mailInfo, aIsEmbedded); //addrData,ccData,bccData, subjData, bodyData, aIsEmbedded);
            //iFlashUI->SendMessageL(uid, mailInfo, aIsEmbedded); //addrData,ccData,bccData, subjData, bodyData, aIsEmbedded);
            }
        else
            {
            deviceIf->SendMessageL(uid, mailInfo); //addrData,ccData,bccData,subjData, bodyData);
            //iControl->SendMessage(uid, mailInfo); //addrData,ccData,bccData,subjData, bodyData);
            }
        }


    CleanupStack::PopAndDestroy(msg);
	iRedraw = ETrue;

	return 0;
}
/*
TBool CPlatformFlashEngine::QueryAllowDeviceIDL()
{
	TBool rval = EFalse;
	switch (iAllowDeviceId)
	{
	case EAllow_Allow:
		rval = ETrue;
		break;

	case EAllow_Deny:
		rval = EFalse;
		break;

	case EAllow_NotSet:
		{
			// save request to change screen size for later, since
			// it can screw up if there's a dialog blocking the screen
			TInt saveScreenSizeRequest = iChangeScreenSizeRequest;
			TInt saveKeysRequest = iChangeKeysRequest;
			iChangeScreenSizeRequest = EChangeScreenNull;
			iChangeKeysRequest = EChangeScreenNull;

			if(!iBrowserPluginActive)
				{
				rval = iFlashUI->QueryAllowDeviceIDL();
				}
			else
				{
				//for browser plugin need to implement, presently it is EFalse i.e. deny
				rval = iControl->QueryAllowDeviceIDL();
				}

			if (rval)
			{
				iAllowDeviceId = EAllow_Allow;
			}
			else
			{
				iAllowDeviceId = EAllow_Deny;
			}

			// restore screen size change request
			iChangeScreenSizeRequest = saveScreenSizeRequest;
			iChangeKeysRequest = saveKeysRequest;

			if ((iChangeScreenSizeRequest != EChangeScreenNull) || (iChangeKeysRequest != EChangeScreenNull))
			{
				iAsyncEventHandler->TriggerEvent();
			}
		}
		break;
	}

	return rval;
}
*/
TBool CPlatformFlashEngine::QueryAllowPhoneCallL(const TDesC& aNumberString)
{
	TBool rval = EFalse;
	switch (iAllowPhoneCall)
	{
	case EAllow_Allow:
		rval = ETrue;
		break;

	case EAllow_Deny:
		rval = EFalse;
		break;

	case EAllow_NotSet:
		{
			// save request to change screen size for later, since
			// it can screw up if there's a dialog blocking the screen
			TInt saveScreenSizeRequest = iChangeScreenSizeRequest;
			TInt saveKeysRequest = iChangeKeysRequest;
			iChangeScreenSizeRequest = EChangeScreenNull;
			iChangeKeysRequest = EChangeScreenNull;

			/*
			if(!iBrowserPluginActive)
			{
				rval = iFlashUI->QueryAllowPhoneCallL(aNumberString);
			}
			else
			{
				rval = iControl->QueryAllowPhoneCallL(aNumberString);
			}
			*/
			rval = iFlashClientIf->GetUserNotifierIf()->QueryAllowPhoneCallL(aNumberString);

//			rval = iFlashUI->QueryAllowPhoneCallL(aNumberString);
			// rborcic 2005/12/13 -- to make behavior consistent
			// with Flash Lite 1.1, do not set iAllowPhoneCall.
			// 1.1 queries the user on each attempt to dial, so
			// 2.0 should do the same.
			/*
			if (rval)
			{
				iAllowPhoneCall = EAllow_Allow;
			}
			else
			{
				iAllowPhoneCall = EAllow_Deny;
			}
			*/

			// restore screen size change request
			iChangeScreenSizeRequest = saveScreenSizeRequest;
			iChangeKeysRequest = saveKeysRequest;

			if ((iChangeScreenSizeRequest != EChangeScreenNull) || (iChangeKeysRequest != EChangeScreenNull))
			{
				iAsyncEventHandler->TriggerEvent();
			}
		}
		break;
	}
	return rval;
}


TInt16 CPlatformFlashEngine::StartVibrate(TUint16 aTimeOn, TUint16 aTimeOff, TUint16 aRepeat)
{
	if (iVibrationHandler == NULL)
	{
		iVibrationHandler = new CVibrationHandler();
	}
	if (iVibrationHandler)
	{
		return iVibrationHandler->StartVibrate(aTimeOn, aTimeOff, aRepeat);
	}
    return KVibraFeatureNotSupported;
}

TInt16 CPlatformFlashEngine::StopVibrate()
{
	if (iVibrationHandler)
	{
		return iVibrationHandler->StopVibrate();
	}
	return KVibraFeatureNotSupported;
}

#ifdef FEATURE_INLINE_TEXT_ENTRY

AMP_MTextNavigation* CPlatformFlashEngine::GetTextNavigation( )
{
	return iTextNavigation;
}

void CPlatformFlashEngine::DisableIME()
{
#if 0
	//if(iFlashUI && !iBrowserPluginActive)
if(eStandAlone==iExecutionMode)
	{
        if( iIMEControlState == EIMEEnabled )
            {
                iIMEControlState = EIMEDisable;
                //iFlashUI->DisableIME();
                iFlashClientIf->GetInputTextIf()->DisableIME();
                // might be required if we decide to disable the IME asynchronously
                // iIMEControlState = EIMEDisabled;

                // going to state 0
                iIMEControlState = EIMENotInitialized;
            }
        else
            {
                DEBUGPRINT("IME State error! Expected state : %d. Current State : %d", EIMEEnabled, iIMEControlState );
            }
	}
	else if(iFlashClientIf->GetInputTextIf() && iBrowserPluginActive)
    {
        if( iIMEControlState == EIMEEnabled )
            {
            iIMEControlState = EIMEDisable;
            iFlashClientIf->GetInputTextIf()->DisableIME();
            // might be required if we decide to disable the IME asynchronously
            // iIMEControlState = EIMEDisabled;

            // going to state 0
            iIMEControlState = EIMENotInitialized;
            }
        else
            {
                DEBUGPRINT("IME State error! Expected state : %d. Current State : %d", EIMEEnabled, iIMEControlState );
            }
    }
#endif

	if( iFlashClientIf && iIMEControlState == EIMEEnabled )
	{
		iIMEControlState = EIMEDisable;
		MInputTxtIf* inputtxtIf = iFlashClientIf->GetInputTextIf();
		if(inputtxtIf)
			inputtxtIf->DisableIME();
		// might be required if we decide to disable the IME asynchronously
		// iIMEControlState = EIMEDisabled;

		// going to state 0
		iIMEControlState = EIMENotInitialized;
	}

	FI_Rect dirtyRect2;
	dirtyRect2.xmin = (short)iTIIndicatorDirtyRect.iTl.iX;
	dirtyRect2.ymin = (short)iTIIndicatorDirtyRect.iTl.iY;
	dirtyRect2.xmax = (short)iTIIndicatorDirtyRect.iBr.iX;
	dirtyRect2.ymax = (short)iTIIndicatorDirtyRect.iBr.iY;

	iFiObject.FI_SetDirtyRect(&dirtyRect2,false);

//	iImeActive = EFalse;
}

void CPlatformFlashEngine::EnableIMEL()
{
    // if we are already in the process of handling an IMEI request, ignore the others
    if( iIMEControlState == EIMENotInitialized )
    {
        iIMEControlState = EIMEEnable;
        // the ime control has not been initialized AND we get a request to enable it
        TriggerAsyncEvent();
    }

}

_LIT(KLitAlpha,"A");
_LIT(KLitNumeric,"#");
 void CPlatformFlashEngine::DrawFullScreenInputModeIndicator(const TRect& editingTextFieldBounds, TRect& indicatorBounds)
 {
	iTIIndicatorDirtyRect = TRect();

 	//if(iFlashUI)
	if(eStandAlone==iExecutionMode)
 	{
 		if(iFlashClientIf->GetDisplay()->IsFullScreenMode())
 		{
     		TInt intCurrentInputMode = iFlashClientIf->GetInputTextIf()->GetCurrentInputMode();

     		if(intCurrentInputMode != KErrNotFound)
     		{
    			// Default values
     			//TRect clientRect = GetUI()->ClientRect();
    			//TRect appRect = GetUI()->ApplicationRect();
                TRect clientRect = iFlashClientIf->GetDisplay()->ClientRect();
     		    TRect appRect = iFlashClientIf->GetDisplay()->ApplicationRect();

    			// Default values
    			// Calculate width and height of indicator based on size of device screen.
    			// We round up by adding .5 (88/176 or 104/208) after multiplying by a known good size (12).
    			// The decimal remainder will be truncated during the assignment to the variable.
    			TInt tiIndicatorDirtyRectWidth		= (12 * appRect.iBr.iX + 88) / 176;
    			TInt tiIndicatorDirtyRectHeight	= (12 * appRect.iBr.iY + 104) / 208;

    			TInt indicatorXmin = -tiIndicatorDirtyRectWidth;
    			TInt indicatorYmin = -tiIndicatorDirtyRectHeight;
    			TInt indicatorXmax = 0;
    			TInt indicatorYmax = 0;

    			// check to see if we can attach to top right of active text field
    			if(editingTextFieldBounds.iBr.iX < clientRect.iBr.iX &&
    			   editingTextFieldBounds.iTl.iY - tiIndicatorDirtyRectHeight > clientRect.iTl.iY)
    			{
    				indicatorXmin = editingTextFieldBounds.iBr.iX - tiIndicatorDirtyRectWidth;
    				indicatorYmin = editingTextFieldBounds.iTl.iY - tiIndicatorDirtyRectHeight;
    				indicatorXmax = editingTextFieldBounds.iBr.iX;
    				indicatorYmax = editingTextFieldBounds.iTl.iY;
    			}

    			// check to see if we can attach to top left of active text field
    			else if(editingTextFieldBounds.iTl.iX > clientRect.iTl.iX &&
    					editingTextFieldBounds.iTl.iY - tiIndicatorDirtyRectHeight > clientRect.iTl.iY)
    			{
    				indicatorXmin = editingTextFieldBounds.iTl.iX;
    				indicatorYmin = editingTextFieldBounds.iTl.iY - tiIndicatorDirtyRectHeight;
    				indicatorXmax = editingTextFieldBounds.iTl.iX + tiIndicatorDirtyRectWidth;
    				indicatorYmax = editingTextFieldBounds.iTl.iY;
    			}

    			// check to see if we can attach to bottom right of active text field
    			else if(editingTextFieldBounds.iBr.iX < clientRect.iBr.iX &&
    					editingTextFieldBounds.iBr.iY + tiIndicatorDirtyRectHeight < clientRect.iBr.iY)
    			{
    				indicatorXmin = editingTextFieldBounds.iBr.iX - tiIndicatorDirtyRectWidth;
    				indicatorYmin = editingTextFieldBounds.iBr.iY;
    				indicatorXmax = editingTextFieldBounds.iBr.iX;
    				indicatorYmax = editingTextFieldBounds.iBr.iY + tiIndicatorDirtyRectHeight;
    			}

    			// check to see if we can attach to bottom left of active text field
    			else if(editingTextFieldBounds.iTl.iX > clientRect.iTl.iX &&
    					editingTextFieldBounds.iBr.iY + tiIndicatorDirtyRectHeight < clientRect.iBr.iY)
    			{
    				indicatorXmin = editingTextFieldBounds.iTl.iX;
    				indicatorYmin = editingTextFieldBounds.iBr.iY;
    				indicatorXmax = editingTextFieldBounds.iTl.iX + tiIndicatorDirtyRectWidth;
    				indicatorYmax = editingTextFieldBounds.iBr.iY + tiIndicatorDirtyRectHeight;
    			}

    			// all 4 corners of text field must be off screen. check for this and place input mode indicator on top left hand side of screen
    			else if(editingTextFieldBounds.iTl.iX < clientRect.iTl.iX &&
    					editingTextFieldBounds.iBr.iX > clientRect.iBr.iX)
    			{
    				indicatorXmin = clientRect.iBr.iX - tiIndicatorDirtyRectWidth;
    				indicatorYmin = editingTextFieldBounds.iTl.iY - tiIndicatorDirtyRectHeight;
    				indicatorXmax = clientRect.iBr.iX;
    				indicatorYmax = editingTextFieldBounds.iTl.iY;
    			}

    			iTIIndicatorDirtyRect=TRect(indicatorXmin, indicatorYmin, indicatorXmax, indicatorYmax);
    			indicatorBounds = iTIIndicatorDirtyRect;

      			CFbsBitGc* pGc = NULL;
      			                
#ifdef FEATURE_OPENVG //check if the rendering is in OpenVG
                if (iOpenVgMode)
                {
                    if(!iIMIndicatorBmp)
                    {
                        FreeInputIndicatorBits();
                    
                        TInt error = KErrNone;
                        
                        iIMIndicatorBmp = new CFbsBitmap();
                        if(iIMIndicatorBmp == NULL) {
                            error = KErrNoMemory;
                        }
                        
                        if(!error) {
                            error = iIMIndicatorBmp->Create(indicatorBounds.Size(), EColor16MU);                        
                        }
                        
                        if(!error) {
                            TRAP(error,iIMIndicatorBmpDevice = CFbsBitmapDevice::NewL(iIMIndicatorBmp));
                        }
                        
                        if(!error) {
                            iIMIndicatorBmpDevice->CreateContext(iIMIndicatorBmpGc);
                            pGc = iIMIndicatorBmpGc;
                        }
                            
                        if(error) {
                            FreeInputIndicatorBits();
                        }
                    }

                    pGc = iIMIndicatorBmpGc;
                    //if(iInputModeIndicatorImage != VG_INVALID_HANDLE && 
                    //   iCurrentIMIndicatorBounds == iTIIndicatorDirtyRect && 
                    //   iCurrentTextInputMode     == intCurrentInputMode)
                    //{
                    //    SetVgImagePosition(iTIIndicatorDirtyRect.iTl.iX,iTIIndicatorDirtyRect.iTl.iY);
                    //    vgDrawImage(iInputModeIndicatorImage);    
                    //    return;
                    //}
                }
                else
#endif                 
                {
                    pGc = GetBitmapContext();
                }
                
                if(pGc==NULL)
                    return; //in case if any of the above calls fail, return...
                
                TRect imIndicatorBounds(iTIIndicatorDirtyRect);
                
#ifdef FEATURE_OPENVG                
                if (iOpenVgMode && iIMIndicatorBmp) {
                    imIndicatorBounds.Move(-iTIIndicatorDirtyRect.iTl.iX, -iTIIndicatorDirtyRect.iTl.iY);
                    iIMIndicatorBmp->BeginDataAccess();                    
                }
#endif                
                
    			pGc->Clear(imIndicatorBounds);
    			CFont* font = (CFont*)iEnv->AnnotationFont();
    			    
    			pGc->UseFont(font);
    			TRgb pen(KRgbWhite);
    			pGc->SetBrushStyle(CGraphicsContext::ESolidBrush);
    			TRgb brush(KRgbDarkBlue);
    			pGc->SetPenStyle(CGraphicsContext::ESolidPen);

    			const TDesC* des=&KLitAlpha;
    			if(intCurrentInputMode == 2)
    			{
    				des=&KLitNumeric;
    			}
    			pGc->SetBrushColor(brush);
    			pGc->SetPenColor(pen);
    			TPoint point(imIndicatorBounds.iTl.iX,imIndicatorBounds.iBr.iY);
    			point.iX+=(imIndicatorBounds.Width()-font->TextWidthInPixels(*des))/2;
    			point.iY-=(imIndicatorBounds.Height()-font->AscentInPixels())/2;
    			pGc->DrawText(*des,point);
     			pGc->DiscardFont();
     			pGc->CancelClippingRect();
     			
#ifdef FEATURE_OPENVG
                if (iOpenVgMode && iIMIndicatorBmp) {    
                    iIMIndicatorBmp->EndDataAccess();
                }     			
     			
     			if(iOpenVgMode)
     			{
                    vgLoadIdentity();
                    
                    if(iInputModeIndicatorImage != VG_INVALID_HANDLE && 
                       (iCurrentIMIndicatorBounds != iTIIndicatorDirtyRect || 
                       iCurrentTextInputMode     != intCurrentInputMode))
                    {
                        vgDestroyImage(iInputModeIndicatorImage);
                        iInputModeIndicatorImage = VG_INVALID_HANDLE;
                    }
                    
                    VGint curImageMode = vgGeti(VG_IMAGE_MODE);                    
                    vgSeti( (VG_IMAGE_MODE), (VG_DRAW_IMAGE_NORMAL) );
                    
                    if(iInputModeIndicatorImage == VG_INVALID_HANDLE)
                        iInputModeIndicatorImage = vgCreateImage(VG_sXRGB_8888, imIndicatorBounds.Width(), imIndicatorBounds.Height(), VG_IMAGE_QUALITY_FASTER);
    
                    vgImageSubData(iInputModeIndicatorImage, 
                            iIMIndicatorBmp->DataAddress(), 
                            iIMIndicatorBmp->DataStride(), 
                            VG_sXRGB_8888, 
                            0,0, 
                            imIndicatorBounds.Width(), 
                            imIndicatorBounds.Height());
                    
                    SetVgImagePosition(iTIIndicatorDirtyRect.iTl.iX,iTIIndicatorDirtyRect.iTl.iY);
                    vgDrawImage(iInputModeIndicatorImage);
                    
                    vgSeti( (VG_IMAGE_MODE), curImageMode );
                     
                    iCurrentIMIndicatorBounds = iTIIndicatorDirtyRect;
                    iCurrentTextInputMode = intCurrentInputMode;
     			}
#endif
     		}
 		}

	}
}


#ifdef FEATURE_OPENVG
void CPlatformFlashEngine::FreeInputIndicatorBits() 
{
    delete iIMIndicatorBmp;
    iIMIndicatorBmp = NULL;
    
    delete iIMIndicatorBmpGc;
    iIMIndicatorBmpGc = NULL;
    
    delete iIMIndicatorBmpDevice;
    iIMIndicatorBmpDevice = NULL;
    
    if(iInputModeIndicatorImage != VG_INVALID_HANDLE) 
    {
        vgDestroyImage(iInputModeIndicatorImage);
        iInputModeIndicatorImage = VG_INVALID_HANDLE;
    }    
}


void CPlatformFlashEngine::SetVgImagePosition(TInt x, TInt y)
{
    vgSeti( VG_MATRIX_MODE, VG_MATRIX_IMAGE_USER_TO_SURFACE );  

    VGfloat m[9];
    m[0] = 1.0f;
    m[1] = 0.0f;
    m[2] = 0.0f;
    m[3] = 0.0f;
    m[4] = -1.0f;
    m[5] = 0.0f;
    m[6] = (VGfloat)x;
    m[7] = (VGfloat)(iFrameBufferSize.iHeight - y);
    m[8] = 1.0f;
    vgLoadMatrix( m );

    vgSeti( VG_MATRIX_MODE, VG_MATRIX_PATH_USER_TO_SURFACE );
}
#endif    

#endif // FEATURE_INLINE_TEXT_ENTRY


CFontMapTable::CFontMapTable(CPlatformFlashEngine* aEngine)
 				:iEngine(aEngine),
				 iFontId(0)
{

}

CFontMapTable::~CFontMapTable()
{
	iEngine = NULL;
	iFontId = 0;
	iFontIdFontInfoMapTable.Reset();
}

TInt CFontMapTable::AddNewFontEntryL(CFont* aFont, const struct FI_FontInfo* aFontInfo)
{
 	FI_FontInfo* fontInfo = new (ELeave) FI_FontInfo;
 	fontInfo->bold = aFontInfo->bold;
 	fontInfo->fontLanguage = aFontInfo->fontLanguage;
 	fontInfo->fontSize = aFontInfo->fontSize;
 	fontInfo->italic = aFontInfo->italic;
 	fontInfo->fontName = NULL;

 	iFontId++;

 	iFontIdFontInfoMapTable.Append(TFontIdFontInfo(iFontId, aFont, fontInfo));
 	return(iFontId);
}

CFont* CFontMapTable::GetFont(TInt aFontId)
{
 	CFont* font = NULL;

 	for(TInt i=0; i< iFontIdFontInfoMapTable.Count(); i++)
 	{
 		if(iFontIdFontInfoMapTable[i].iFontId == aFontId)
 		{
 			if(!iFontIdFontInfoMapTable[i].iFont)
 			{
 				iEngine->CreateFont(iFontIdFontInfoMapTable[i].iFontInfo, iFontIdFontInfoMapTable[i].iFont);
 			}

 			font = iFontIdFontInfoMapTable[i].iFont;
 		}
 	}

 	return(font);
}

CFont* CFontMapTable::GetFontEntryFromFontId(TInt aFontId)
{
 	CFont* font = NULL;

 	for(TInt i=0; i< iFontIdFontInfoMapTable.Count(); i++)
 	{
 		if(iFontIdFontInfoMapTable[i].iFontId == aFontId)
 		{
 			font = iFontIdFontInfoMapTable[i].iFont;
 		}
 	}

 	return(font);
}

void CFontMapTable::RemoveFontEntry(TInt aFontId)
{
 	if(iFontIdFontInfoMapTable.Count() == 0)
 		return;

 	for(TInt i=0; i< iFontIdFontInfoMapTable.Count(); i++)
 	{
 		if(iFontIdFontInfoMapTable[i].iFontId == aFontId)
 		{
 			delete iFontIdFontInfoMapTable[i].iFontInfo;
 			iFontIdFontInfoMapTable.Remove(i);
 		}
 	}
}

void CFontMapTable::ReleaseAllFonts(CFbsBitmapDevice* aBitmapDevice)
{
 	if(!aBitmapDevice)
 		return;

 	if(iFontIdFontInfoMapTable.Count()> 0)
 	{
 		for(TInt i =0; i<iFontIdFontInfoMapTable.Count(); i++)
 		{
 			if(iFontIdFontInfoMapTable[i].iFont)
 			{
 				aBitmapDevice->ReleaseFont(iFontIdFontInfoMapTable[i].iFont);
 				iFontIdFontInfoMapTable[i].iFont = NULL;
 			}
 		}
 	}
}

