/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the classes used for 
*				 shared object operations.
*
*/


#ifndef USE_SYNC_MSO

#include "platformbuild.h"
#include "saplayer_build.h"
#include "symbian_sharedobjects.h"
#include "si_interface.h"
#include "symbian_engine.h"
#include "symbian_file.h"
#include "symbian_clib.h"
#include "engine_data_caging_literals.hrh"
#include "shobjutils.h"
#include <utf.h>

// 
// class CPersistentStorageCleanupRequest
//

//static
CPersistentStorageCleanupRequest* CPersistentStorageCleanupRequest::NewL(struct MM_Object* aMMObj, 
                                                                         TUint32 aNumBytesToClean,
                                                                         CFlhShObjUtils& aShObjUtils,
                                                                         TInt aId, CPersistentStorageCleanupRequest** aSelf,
																		 MFlhShObjCleanupObserver& aObs)
{
    CPersistentStorageCleanupRequest* self = 
        new(ELeave) CPersistentStorageCleanupRequest(aMMObj, 
                                                     aNumBytesToClean,
                                                     aShObjUtils,
                                                     aId,aSelf,aObs);
    CActiveScheduler::Add(self);
    self->DoNextStage();
    return self;
}


CPersistentStorageCleanupRequest::~CPersistentStorageCleanupRequest()
{
	if (iNodeList)
	{
		while (!iNodeList->IsEmpty())
		{
			CPersistentStorageNode* node = iNodeList->First();
			iNodeList->Remove(*node);
			delete node;
		}
		delete iNodeList;
	}
    Cancel();
	delete iOutstandingReq;
	*iSelf=NULL;
}


CPersistentStorageCleanupRequest::CPersistentStorageCleanupRequest(struct MM_Object* aMMObj, 
                                                                   TUint32 aNumBytesToClean,
                                                                   CFlhShObjUtils& aShObjUtils,
                                                                   TInt aId,
																   CPersistentStorageCleanupRequest** aSelf,
																   MFlhShObjCleanupObserver& aObs)
    :CActive(CActive::EPriorityUserInput), 
     iMMObj(aMMObj),
	 iId(aId),
     iNumBytesToClean(aNumBytesToClean), 
     iShObjUtils(aShObjUtils),
	 iSelf(aSelf),
	 iObs(aObs)
{
}

void CPersistentStorageCleanupRequest::DoNextStage()
{
    SetActive();
    iStatus = KRequestPending;
    TRequestStatus* temp=&iStatus;
    User::RequestComplete(temp, KErrNone);
}


void CPersistentStorageCleanupRequest::RunL()
{
    if(iStatus == KErrNone)
    {
        if(!iDone)
        {
            if(CleanupPersistentDirectoryL())
			{	
				DoNextStage();
			}
        }
        else
        {
            SendCallbackAndDeleteThis(iRequestGranted ? 
                                      KCleanupSharedObjectsCallbackSuccess : 
                                      KCleanupSharedObjectsCallbackFailure);
        }
    }
    else
    {
        SendCallbackAndDeleteThis(KCleanupSharedObjectsCallbackFailure);
    }
}


TInt CPersistentStorageCleanupRequest::RunError(TInt /*aError*/)
{
    SendCallbackAndDeleteThis(KCleanupSharedObjectsCallbackFailure);
    return KErrNone;
}


void CPersistentStorageCleanupRequest::DoCancel()
{
    // no cancel for RFile methods
}


void CPersistentStorageCleanupRequest::SendCallbackAndDeleteThis(TInt16 aStatus)
{
	MFlhShObjCleanupObserver& temp=iObs;
	delete this;
	temp.CleanupResult(aStatus);
}
#ifdef _DEBUG
void CPersistentStorageCleanupRequest::NotifyOperationComplete(void* aServicingObject,TInt aError)
#else
void CPersistentStorageCleanupRequest::NotifyOperationComplete(void* /*aServicingObject*/,TInt aError)
#endif
{	
	__ASSERT_DEBUG(iOutstandingReq==aServicingObject,User::Invariant());
	
	//done with this
	delete iOutstandingReq;
	iOutstandingReq=NULL;
	
	//success - move on to next state
	if(KErrNone==aError)
	{
		if(iStage==EExtractObjects || iStage==ESizeObject)
		{
			iStage++;
		}
		else
		{
			__ASSERT_DEBUG(iStage==ECleanupObject,User::Invariant());
			iStage=ESelectObject;
		}
	}
	else
	{
		//failed - need to report back to player in next event...
		iDone=ETrue;
		iRequestGranted=EFalse;
	}
	
	// requeue ourselves to handle next state
	DoNextStage();
}

// Ask user if they want cleanup to happen.
// Clean up the persistent directory until the number of bytes to clean has reached.
// This function has been divided to stages that will be run one by one by
// the active scheduler.
TBool CPersistentStorageCleanupRequest::CleanupPersistentDirectoryL()
{
    switch(iStage)
    {
    case EPrepareCleanup:
    {
        CPlatformFlashEngine* pEngine = iMMObj->pHostObj->iFIInterface->GetEngine();
        User::LeaveIfNull(pEngine);

        // User is not queried if the MSO needs to be cleanedup anymore.
        // Hence commenting the below piece of code.
        //TBool response = pEngine->ShowSharedObjectConfirmationQueryL();
        //if(response)
        {
            iRequestGranted = ETrue;
            iStage++;
            return ETrue;
        }
        /*else
        {
            iRequestGranted = EFalse;
            iDone = ETrue;
            return ETrue;
        }
        */
    }
    case EExtractObjects:
    {
        // Extract all shared object files in systems
        iOutstandingReq=iShObjUtils.ListAllNodesL(NULL, iNodeList, *this);
		return EFalse;
    }
    
    case ESelectObject:
    {
        if( (iDeletedBytes < iNumBytesToClean) && (!iNodeList->IsEmpty()) )
        {
			TSglQueIter<CPersistentStorageNode> iter(*iNodeList);
			CPersistentStorageNode* node = iter;
            User::LeaveIfNull(node);
            
            iOldest = node;
			for(; iter; iter++)
			{
				node = iter;
				if (iShObjUtils.IsOlder(node, iOldest))
				{
					iOldest = node;
				}
			}
            
            iNameSpace = iOldest->GetNameSpace();
            iStage++;
        }
        else
        {
            iDone = ETrue;
        }
		return ETrue;
    }
    case ESizeObject:
    {
        iOutstandingReq=iShObjUtils.GetNameSpaceSizeL(iNameSpace, iDirSize,*this);
		return EFalse;
    }
	case ECleanupObject:
	{
		iOutstandingReq=iShObjUtils.RemoveDirectoryL(iNameSpace,*this);
		iNodeList->Remove(*iOldest);
		delete iOldest;
		iOldest = NULL;
		iDeletedBytes += iDirSize;
		return EFalse;
	}
	default:
	{
		User::Leave(KErrGeneral); // will result in a failure being reported back
	}
	}
	return ETrue;
}


//
// CFlhShObjEngine
//

//static
CFlhShObjEngine* CFlhShObjEngine::NewL(RFs& aFs, CFlhFileUtils& aUtils, const TDesC& aPersistentDirectory, CPlatformFlashEngine& aEngine)
{
	CFlhShObjEngine* self = new(ELeave) CFlhShObjEngine();
	CleanupStack::PushL(self);
	self->ConstructL(aFs,aUtils, aPersistentDirectory, aEngine);
	CleanupStack::Pop(self);
	return self;
}

CFlhShObjEngine::CFlhShObjEngine()
:iRequestQueue(_FOFF(CFlhShObjRequestBase,iLinkItem))
{}

void CFlhShObjEngine::ConstructL(RFs& aFs, CFlhFileUtils& aUtils, const TDesC& aPersistentDirectory, CPlatformFlashEngine& aEngine)
{
	iShObjUtils=CFlhShObjUtils::NewL(aFs,aUtils, aPersistentDirectory, aEngine);
}

CFlhShObjEngine::~CFlhShObjEngine()
{
	CloseAllRequests();
	delete iShObjUtils;
}

short CFlhShObjEngine::AsyncGetPersistentDataSizeL(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, unsigned long aRequestId)
{
	CFlhShObjRequestFileSize* getSize=CFlhShObjRequestFileSize::NewL(*this,*iShObjUtils,aMMObj,aRequestId,aRootMovieGUID,aName);
	return StartRequest(getSize);
}

short CFlhShObjEngine::AsyncGetPersistentDataL(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, unsigned long aRequestId, unsigned char* aData,
										unsigned long aDataSize)
{
	CFlhShObjRequestFileRead* getData=CFlhShObjRequestFileRead::NewL(*this,*iShObjUtils,aMMObj,aRequestId,aRootMovieGUID,aData,aDataSize,aName);
	return StartRequest(getData);
}

short CFlhShObjEngine::AsyncSetPersistentDataL(struct MM_Object* aMMObj, const char* aRootMovieGUID, const char* aName, unsigned long aRequestId, const unsigned char* aData,
										unsigned long aDataSize, TBool aEnforceMaxSize)
{
	CFlhShObjRequestFileWrite* setData=CFlhShObjRequestFileWrite::NewL(*this,*iShObjUtils,aMMObj,aRequestId,aRootMovieGUID,aData,aDataSize,aName,aEnforceMaxSize);
	return StartRequest(setData);
}

short CFlhShObjEngine::CancelPersistentDataRequest(struct MM_Object* /*aMMObj*/, unsigned long aRequestId)
{
	if (iCurrentRequest && iCurrentRequest->Id()==aRequestId)
	{
		delete iCurrentRequest;
		iCurrentRequest=NULL;
		// queue up the next request...
		TryQueueNextRequest();

	}
	else // maybe its still queue'd ?
	{
		TSglQueIter<CFlhShObjRequestBase> iter(iRequestQueue);
	
		CFlhShObjRequestBase* item=iter;
		while(item && item->Id()!=aRequestId)
		{
			item=iter++;
		}

		if(item)
		{
			iRequestQueue.Remove(*item);
			delete item;
		}
		else // item not found - cannot cancel
			return (short)0;
	}
	return (short)1;	
}

short CFlhShObjEngine::StartRequest(CFlhShObjRequestBase* aRequest)
{
	iRequestQueue.AddLast(*aRequest);
	
	TryQueueNextRequest();	
	return (short)1;
}

void CFlhShObjEngine::TryQueueNextRequest()
{
	if(!iCurrentRequest)
	{
		// keep firing off requests until one successfully starts.
		while(!iRequestQueue.IsEmpty() && !iCurrentRequest)
		{
			// get first in line for running
			CFlhShObjRequestBase* head=iRequestQueue.First();
			iRequestQueue.Remove(*head);
			TRAPD(error,DoIssueNextRequestL(*head));

			if(error)
			{
				head->HandleTransactionFailed();
				delete head;
			}
			else
			{
				iCurrentRequest=head;
			}
		}
	}
	CountCurrentItemsAndNotify();
}

void CFlhShObjEngine::DoIssueNextRequestL(CFlhShObjRequestBase& aInfo)
{
	aInfo.QueueRequestNowL();
}
#ifdef _DEBUG
void CFlhShObjEngine::HandleTransactionComplete(TUint32 aId, TInt /*aStatus*/)
#else
void CFlhShObjEngine::HandleTransactionComplete(TUint32 /*aId*/, TInt /*aStatus*/)
#endif
{
	ASSERT(iCurrentRequest && iCurrentRequest->Id()==aId); 
	delete iCurrentRequest;
	iCurrentRequest=NULL;
	TryQueueNextRequest();
}

void CFlhShObjEngine::MovieClosed()
{
	// first go through the queue weeding any out, then remove the current request...
	// if any are queued after removing the current then they will be request that could not be deleted
	// and so can be queued...

	TSglQueIter<CFlhShObjRequestBase> iter(iRequestQueue);

	CFlhShObjRequestBase* item=iter;

	while(item)
	{
		iter++; // iterate now in case we delete it!

		if(item->MovieClosedRequestDeletion())
		{
			iRequestQueue.Remove(*item);
			delete item;
		}

		item=iter;
	}

	if(iCurrentRequest && iCurrentRequest->MovieClosedRequestDeletion())
	{
		delete iCurrentRequest;
		iCurrentRequest=NULL;
	}
	TryQueueNextRequest();
}

TInt CFlhShObjEngine::CountCurrentItems()
{
	TInt count=0;
	if(iCurrentRequest)
	{
		count++;
	}
	
	TSglQueIter<CFlhShObjRequestBase> iter(iRequestQueue);
	while(iter++)
	{
		count++;
	}
	
	return count;

}

void CFlhShObjEngine::CountCurrentItemsAndNotify()
{
	if(iNotifiee)
	{
		iNotifiee->NotifyNumberOfItemsInQueue(CountCurrentItems());
	}
}

TInt CFlhShObjEngine::RegisterForNotificationsReturnNumToGo(MFlhShObjEngQueueNofifiee* aNotifiee)
{
	iNotifiee=aNotifiee;
	return CountCurrentItems();
}

const MFlhShObjEngQueueNofifiee* CFlhShObjEngine::Notifiee() const
{
	return iNotifiee;
}

// cancels all requests, including the one being processed and any queued.
void CFlhShObjEngine::CloseAllRequests()
{
	if(iCurrentRequest)
	{
		delete iCurrentRequest;
		iCurrentRequest=NULL;
	}

	CFlhShObjRequestBase* info;
	while(!iRequestQueue.IsEmpty())
	{
		info=iRequestQueue.First();
		iRequestQueue.Remove(*info);
		delete info;
	}
}


//
// class CPersistentStorageNode
//

CPersistentStorageNode* CPersistentStorageNode::NewL(const char* aNameSpace, const TDesC* aName /* = NULL */)
{
	CPersistentStorageNode* node = new(ELeave) CPersistentStorageNode();
	CleanupStack::PushL(node);
	node->ConstructL(aNameSpace, aName);
	CleanupStack::Pop(node);

	return node;
}


CPersistentStorageNode::CPersistentStorageNode()
{
}


void CPersistentStorageNode::ConstructL(const char* aNameSpace, const TDesC* aName)
{
	if(aNameSpace)
	{
		TInt newBufLen = FlashStrLen(aNameSpace) + 1;
		iNameSpace = new(ELeave) char[newBufLen];
		FlashMemCpy(iNameSpace, aNameSpace, newBufLen);
	}
    
	if(aName)
	{
		iName = aName->AllocL();
	}
}


CPersistentStorageNode::~CPersistentStorageNode()
{
	delete iNameSpace;
	iNameSpace = NULL;
	delete iName;
}

//
// class CFlhShObjRequestBase 
//

CFlhShObjRequestBase::CFlhShObjRequestBase(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId, TInt aBaseFlags)
:CActive(CActive::EPriorityIdle),iObs(aObs),iShObjUtils(aShObjUtils), iMMObj(aMMObj), iRequestId(aRequestId), iBaseFlags(aBaseFlags)
{
	CActiveScheduler::Add(this);
//	iStatus=KRequestPending;
}

void CFlhShObjRequestBase::BaseConstructL(const char* aRootMovieGUID, const char* aName)
{
	iName=iShObjUtils.GetSharedObjectFileName(aRootMovieGUID, aName);
	User::LeaveIfNull(iName);
	iMSOName       = HBufC::NewL(KMaxFileName);

    if(iMSOName)
    {
        TPtrC8 ptrToMSOName = (const TUint8*)aName;
        iMSOName->Des().Copy(ptrToMSOName);
    }
}

TBool CFlhShObjRequestBase::MovieClosedRequestDeletion()
{
	iBaseFlags|=EMovieHasEnded;
	return !(iBaseFlags&EOfUseAfterMovieEnded);
}

void CFlhShObjRequestBase::FI_Call(TUint32 aSizeInfo, TInt aStatus)
{
	if(!(iBaseFlags&EMovieHasEnded))
	{ // don't notify the player if the movie has ended
		iMMObj->pHostObj->iFIInterface->FI_NotifySharedObjectStatus(iRequestId,aSizeInfo,(short)aStatus);
	}
	iObs.HandleTransactionComplete(iRequestId, aStatus);
}

CFlhShObjRequestBase::~CFlhShObjRequestBase()
{
	if(iFile.SubSessionHandle())
	{
		iFile.Close();
	}

	delete iName;
}

TUint32 CFlhShObjRequestBase::Id()
{
	return iRequestId;
}

//
//	CFlhShObjRequestFileSize
//


//static 
CFlhShObjRequestFileSize* CFlhShObjRequestFileSize::NewL(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId,
		const char* aRootMovieGUID, const char* aName)
{
	CFlhShObjRequestFileSize* self=new(ELeave) CFlhShObjRequestFileSize(aObs, aShObjUtils, aMMObj, aRequestId);
	CleanupStack::PushL(self);
	self->BaseConstructL(aRootMovieGUID, aName);
	CleanupStack::Pop(self);
	return self;
}


CFlhShObjRequestFileSize::CFlhShObjRequestFileSize(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId)
:CFlhShObjRequestBase(aObs, aShObjUtils, aMMObj, aRequestId, ENotOfUseAfterMovieEnded) 
{
}

CFlhShObjRequestFileSize::~CFlhShObjRequestFileSize()
{
	Cancel();
}
	
void CFlhShObjRequestFileSize::QueueRequestNowL()
{	
    iStatus = KRequestPending;
	SetActive();
	TRequestStatus* pointer=&iStatus;
	User::RequestComplete(pointer,KErrNone);
}

void CFlhShObjRequestFileSize::RunL()
{
	TInt size=0;
	TInt file_open_error=iFile.Open(iShObjUtils.FsSession(),*iName,EFileRead|EFileShareExclusive);

    if( file_open_error == KErrNotFound || file_open_error == KErrPathNotFound )
    {
        // the new file does not exist
        // The directory where MSO files are stored are different in FL 3.0 and FL 3.1
        // If the mso file was created using FL 3.0, then we have to migrate the file to FL 3.1 location

        // the complete filepath of the mso is stored in iName, which is initialized during object creation
        // extract the new directory from iName
        TPtrC newMSODir = iName->Left(iName->LocateReverse(KPathDelimiter)+1);

        // check if the MSO file exists in the old location.
        // if it exists, then move to the new location
        //error = iShObjUtils.CheckAndMigrateToNewLocation(iMMObj->pHostObj->iFIInterface,*iMSOName,newMSODir);
        TRAPD(migrate_mso_error,iShObjUtils.MigrateMSODirectoryL(*iMSOName,newMSODir));
        if(migrate_mso_error == KErrNone)
        {
            // successfully migrated to the new location
            // try to open the file again...
        	file_open_error=iFile.Open(iShObjUtils.FsSession(),*iName,EFileRead|EFileShareExclusive);
        }
        else if(migrate_mso_error == KErrNotFound || migrate_mso_error == KErrPathNotFound)
        {
            FI_Call(size,FI_SharedObject_SizeInfoAvailable);
            return;
        }
        else //Some Other problem So we have to leave Error handling is done in RunError
        {
            User::Leave(migrate_mso_error);
        }
    }

    if(file_open_error == KErrNone)
    {
        file_open_error=iFile.Size(size);
        FI_Call(size,FI_SharedObject_SizeInfoAvailable);
    }
    else
    {
        User::Leave(file_open_error);
    }
}

void CFlhShObjRequestFileSize::DoCancel()
{
	// nothing to do
}

TInt CFlhShObjRequestFileSize::RunError(TInt /*aError*/)
{
    HandleTransactionFailed();
    return KErrNone;
}

void CFlhShObjRequestFileSize::HandleTransactionFailed()
{
	FI_Call(0,FI_SharedObject_SizeRequestFailed);
}

//
//	CFlhShObjRequestFileRead
//


//	static 
CFlhShObjRequestFileRead* CFlhShObjRequestFileRead::NewL(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId,
		const char* aRootMovieGUID, unsigned char* aData, unsigned long aDataSize, const char* aName)
{
	CFlhShObjRequestFileRead* self=new(ELeave) CFlhShObjRequestFileRead(aObs, aShObjUtils, aMMObj, aRequestId, aData, aDataSize);
	CleanupStack::PushL(self);
	self->ConstructL(aRootMovieGUID, aName);
	CleanupStack::Pop(self);
	return self;
}

CFlhShObjRequestFileRead::CFlhShObjRequestFileRead(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId, unsigned char* aData, unsigned long aDataSize)
:CFlhShObjRequestBase(aObs, aShObjUtils, aMMObj, aRequestId,ENotOfUseAfterMovieEnded),iData(aData,0,aDataSize)
{
}

CFlhShObjRequestFileRead::~CFlhShObjRequestFileRead()
{
	Cancel();
}

void CFlhShObjRequestFileRead::ConstructL(const char* aRootMovieGUID, const char* aName)
{
	BaseConstructL(aRootMovieGUID,aName);
}

void CFlhShObjRequestFileRead::QueueRequestNowL()
{
	User::LeaveIfError(iFile.Open(iShObjUtils.FsSession(),*iName,EFileRead|EFileWrite|EFileShareExclusive));
	iStatus = KRequestPending;
	SetActive();
	iFile.Read(iData,iStatus);
}

void CFlhShObjRequestFileRead::RunL()
{
	TInt len=0;
	TInt status=FI_SharedObject_ReadFailed;

	if(iStatus==KErrNone)
	{
		len=iData.Length();
		status=FI_SharedObject_ReadCompleted;
		
		// update the time stamp on the file. This ensures it will not be seen as old by cleanup operations...
		// Reset modified time as we are using this as last access time
		TTime now;
		now.HomeTime();		// Question: Could it be UTC time??? Answer: DK - Don't think so it is important we use the time zone the OS uses so that Read/Write timestamp updates are consistent
		User::LeaveIfError(iFile.SetModified(now));

	}
	FI_Call(len,status);
}

TInt CFlhShObjRequestFileRead::RunError(TInt /*aError*/)
{
	HandleTransactionFailed();
	return KErrNone; // should never happen in this class - RunL() cannot leave...
}

void CFlhShObjRequestFileRead::DoCancel()
{
	iFile.Close(); // otherwise if the read completes we will get a stray event
}

void CFlhShObjRequestFileRead::HandleTransactionFailed()
{
	FI_Call(0,FI_SharedObject_ReadFailed);
}

//
// CFlhShObjRequestFileWrite 
//

//	static
CFlhShObjRequestFileWrite* CFlhShObjRequestFileWrite::NewL(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId,
		const char* aRootMovieGUID, const unsigned char* aData, unsigned long aDataSize, const char* aName, TBool aEnforceMaxSize)
{
	CFlhShObjRequestFileWrite* self=new(ELeave) CFlhShObjRequestFileWrite(aObs, aShObjUtils,aMMObj, aRequestId, aEnforceMaxSize);
	CleanupStack::PushL(self);
	self->ConstructL(aRootMovieGUID,aName,aData,aDataSize);
	CleanupStack::Pop(self);
	return self;
}

CFlhShObjRequestFileWrite::CFlhShObjRequestFileWrite(MFlhShObjReqObs& aObs, CFlhShObjUtils& aShObjUtils, struct MM_Object* aMMObj, unsigned long aRequestId, TBool aEnforceMaxSize)
:CFlhShObjRequestBase(aObs, aShObjUtils,aMMObj, aRequestId, EOfUseAfterMovieEnded)
{
	iEnforceMaxSize = aEnforceMaxSize;
}

CFlhShObjRequestFileWrite::~CFlhShObjRequestFileWrite()
{
	Cancel();
	if(iState==ECleaningUp)
	{
		iShObjUtils.CancelCleanupSharedObjects();
	}

	delete iData;
	delete iRootMovieGUID;
	delete iOutstandingReq;
}

void CFlhShObjRequestFileWrite::ConstructL(const char* aRootMovieGUID, const char* aName, const unsigned char* aData, 
										   unsigned long aDataSize)
{
	BaseConstructL(aRootMovieGUID,aName);

	TPtrC8 ptrC((const TUint8*)aRootMovieGUID);
	iRootMovieGUID=HBufC8::NewL(ptrC.Length()+1); //for ZT
	TPtr8 ptr8(iRootMovieGUID->Des());
	ptr8.Append(ptrC);
	ptr8.ZeroTerminate();

	if(!iEnforceMaxSize)
	{
		if (aDataSize)
		{
			iState = EEnsuringDirectory; // will default to undecided
		}
		else
		{
			iState = EZeroDataDelete;
		}
	}
	
	if(aData && aDataSize)
	{
		TPtrC8 dataPtr((const TUint8*)aData,aDataSize);
		iData=dataPtr.AllocL();
	}
}



void CFlhShObjRequestFileWrite::QueueRequestNowL()
{
	DoNextStepL();
}

void CFlhShObjRequestFileWrite::Reque()
{
	SetActive();
	iStatus=KRequestPending;
	TRequestStatus* pointer=&iStatus;
	User::RequestComplete(pointer,KErrNone);
}
#ifdef _DEBUG
void CFlhShObjRequestFileWrite::NotifyOperationComplete(void* aServicingObject,TInt aError)
#else
void CFlhShObjRequestFileWrite::NotifyOperationComplete(void* /*aServicingObject*/,TInt aError)
#endif
{
	__ASSERT_DEBUG(aServicingObject==iOutstandingReq,User::Invariant());
	delete iOutstandingReq;
	iOutstandingReq=NULL;
	if(aError)
	{
		HandleTransactionFailed();
	}
	else
	{
		iState++;
		TRAP_IGNORE(DoNextStepL());
	}
}

void CFlhShObjRequestFileWrite::DoNextStepL()
{
	if(iState==ECleanupUndecided || iState==EZeroDataDelete || iState==EEnsuringDirectory || iState==EGotLocalSpace || iState==EGotTotalSpace)
	{
		Reque();
	}
	else if(iState==ECleaningUp)
	{
		TInt minCleanup=KMaxPersistentStoragePerMovie;
		if(iData && iData->Length()>minCleanup)
		{
			minCleanup=iData->Length();
		}
		// clean up the maximum size per movie to avoid repeated clean up requests for each small bit...
		iShObjUtils.CleanupSharedObjectsL(iMMObj,minCleanup,iRequestId,*this); // real figure passed in to clean...
	}
	else if(iState==ESavingFile)
	{
		User::LeaveIfError(iFile.Replace(iShObjUtils.FsSession(),*iName,EFileWrite | EFileShareExclusive));	
		iStatus=KRequestPending;
		SetActive();
		iFile.Write(*iData,iStatus);
	}
	else if(iState==EGettingLocalSpace)
	{	
		iOutstandingReq=iShObjUtils.GetNameSpaceSizeL((const char*)iRootMovieGUID->Ptr(), iCurrentDirSize, *this);
	}
	else if(iState==EGettingTotalSpace)
	{
		iOutstandingReq=iShObjUtils.GetNameSpaceSizeL(NULL, iTotalPersistentDirSize, *this);
	}
}

void CFlhShObjRequestFileWrite::RunL()
{
	if(iState==EZeroDataDelete)
	{
		User::LeaveIfError(iShObjUtils.Utils().DeleteFile(*iName)); // if this fails it is as though the write fails so need to leave
		FI_Call(0,FI_SharedObject_WriteCompleted);
		return;//done
	}
	else if(iState==ESavingFile)
	{
		TInt len=0;
		TInt status=FI_SharedObject_WriteFailed;

		if(iStatus==KErrNone && KErrNone==iFile.Size(len))
		{
			status=FI_SharedObject_WriteCompleted;
		}
		else
		{
			iShObjUtils.Utils().DeleteFile(*iName); // error ocurred writing, try to delete file but ignore if this fails - we are hosed anyway...
		}
		FI_Call(len,status);
		return; // done
	}
	else if(iState==ECleanupUndecided)
	{
		iState++;
	}
	else if(iState==EGotLocalSpace)
	{
		TUint32 currentMovieSize = iShObjUtils.GetObjectSizeL(*iName);

		if (iData && (iCurrentDirSize - currentMovieSize + iData->Length()) > KMaxPersistentStoragePerMovie)
		{
			HandleTransactionFailed();
			return;
		}
	
		iState++;
	}
	else if(iState==EGotTotalSpace)
	{
		// see if we have enough space if we replace the new shared object data
		if(iData && ((iTotalPersistentDirSize - iCurrentDirSize + (TUint32)iData->Length()) > KMaxPersistentStorage))
		{
			iState=ECleaningUp;
		}
	    else
		{
		    iState=EEnsuringDirectory;
		}		
	}
	else if(iState==EEnsuringDirectory)
	{
		TFileName* dirName = iShObjUtils.GetPersistenceDir((const char*)iRootMovieGUID->Ptr());
		User::LeaveIfNull(dirName);
        CleanupStack::PushL(dirName);
        TBool res = iShObjUtils.EnsureDirectory(*dirName);
		CleanupStack::PopAndDestroy(dirName);

		if(res)
		{
			if(iData)
			{
				iState=ESavingFile;
			}
			else
			{
				iState=EZeroDataDelete;
			}
		}
		else
		{
			HandleTransactionFailed();
			return;
		}
	}

	DoNextStepL();
}

TInt CFlhShObjRequestFileWrite::RunError(TInt /*aError*/)
{
	HandleTransactionFailed();
	return KErrNone;
}

void CFlhShObjRequestFileWrite::HandleTransactionFailed()
{
	FI_Call(0,FI_SharedObject_WriteFailed);
}

void CFlhShObjRequestFileWrite::DoCancel()
{
	if(iFile.SubSessionHandle())
		iFile.Close(); // otherwise if the write completes we will get a stray event
}
	

void CFlhShObjRequestFileWrite::CleanupResult(TBool aEnoughSpaceCleanedUp)
{
	if(aEnoughSpaceCleanedUp)
	{
		iState=EEnsuringDirectory;
		Reque();
	}
	else
	{
		HandleTransactionFailed();
	}
}

#endif //USE_SYNC_MSO
