/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that is used to 
*				 access singal strength information.
*
*/


#ifdef FEATURE_PACK_3

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "symbian_signalinfo.h"

CPhoneSignalInfo* CPhoneSignalInfo::NewL()
{
	CPhoneSignalInfo* self=new(ELeave) CPhoneSignalInfo();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

CPhoneSignalInfo::CPhoneSignalInfo(): 
	CActive(CActive::EPriorityLow)
   ,iTelephony(NULL)
   ,iSigStrengthV1Pckg( iSigStrengthV1 )
{

}

void CPhoneSignalInfo::ListenL()
{
	//Check this condition before calling. 
	//if (IsActive())
	//	return;
	
	if(!iTelephony)
	{
		iTelephony = CTelephony::NewL();	
	}
	
	if(iTelephony)	
	{
		iTelephony->GetSignalStrength(iStatus, iSigStrengthV1Pckg);
		
		if (!IsActive())
		{
			SetActive();
			//Wait for Async CTelephony Request to Complete.
			iWait->Start();
		}
	}
	
}

void CPhoneSignalInfo::ConstructL()
{
	CActiveScheduler::Add(this);

	//ListenL();//Not Required while creating the Object.
				//Will Listen once actual request comes.

	iWait = new (ELeave) CActiveSchedulerWait;
}


CPhoneSignalInfo::~CPhoneSignalInfo()
{
	Cancel();
	
	if(iTelephony)
	{
		delete iTelephony;
		iTelephony = NULL;
	}
	if(iWait)
		{	
		if (iWait->IsStarted())
			{
			iWait->AsyncStop();
			}
		delete iWait;
		iWait = NULL;
		}
	
}
					

void CPhoneSignalInfo::DoCancel()
{
	iTelephony->CancelAsync(CTelephony::EGetSignalStrengthCancel);
	//Cancel the CActiveSchedulerWait Loop also.
	if (iWait->IsStarted())
		iWait->AsyncStop();
}
	

void CPhoneSignalInfo::RunL()
{
	if(iStatus==KErrNone)
       {     
       		if(iTelephony)
			{
				delete iTelephony;
				iTelephony = NULL;
			}
       }
		//Async Request Done. If Error Occured, return value is 0.
		if (iWait->IsStarted())
			{
			iWait->AsyncStop();
			}
}
					

TInt  CPhoneSignalInfo::GetSignalBarValue()
{
	TInt retVal = 0;
	
	TRAPD( err, retVal = GetSignalBarValueL());
	if ( err == KErrNone)
	{
		//avoid warning
	}
	return retVal;
	
}

TInt  CPhoneSignalInfo::GetSignalBarValueL()
{
	TInt retVal= 0;
	//ListenL() is required Here because it might have been a long
	//time since Values have been updated!( if CPhoneBatteryInfo Instance 
	//was created long back, it might no longer even be Active!)
	if(!IsActive())
		{
		ListenL() ;
		}
	
	if(iStatus==KErrNone)
       {
			retVal = iSigStrengthV1.iBar;	
       }
       
    //ListenL()  ;
     
    return retVal;
	
}
 	  				
#endif //FEATURE_PACK_3



