/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that provides 
*				 implementation for all Flash sound playback.
*
*/

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								// It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "saplayer_build.h"
#include "flashliteinterfaces.h"
#include "fi_interface.h"

#include "symbian_sound.h"
#include "symbian_clib.h"
#include "symbian_engine.h"
#include "symbian_ui_interface.h"
#include "engine_data_caging_literals.hrh"
#include "symbian_videoengine.h"
#include "symbian_mediarecognizer.h"
#include <AudioPreference.h>

// These are for CAknKeySoundSystem
#include <aknsoundsystem.h>
#include <aknappui.h>
#include <avkon.rsg>

#include <flash2engine.rsg>
#include "flashengineresourcehandler.h"
//#define DEBUG_SOUND
//#define DEBUG_SOUND_ERROR_ONLY
//#define DEBUG_SOUND_OUTPUT
//#define DEBUG_SOUND_DECOMPRESSOR
//#define DEBUG_DECOMPRESSOR_MEMORY

//#define USE_POSITION_INFO
_LIT(KBrowserApp, "BrowserApp");

extern void NPPDebug(TUint8 aLogLevel,TBool aIsPlugin,TRefByValue<const TDesC> /*aText*/, ... );

CSoundHandler::CSoundHandler()
{
	iMdaAudioPlayerUtility = NULL;
	iVolume = KMaxUserVolume;
	iOldVolume = iVolume;
	iMidi_num = FIRST_MIDI_NUM; // just an ID - start at 1000
	iMdaEventState = EMdaNone;
	iSoundQueue[0] = 0;
	iCachedSounds = NULL;
	iEngine = NULL;
	iDeviceAudioSet = EFalse;
	iMdaAudioStream = NULL;
	iMdaStreamState = EMdaNone;
	iSndLibLoaded = EFalse;
	iMdaStreamReady = EFalse;
	iAllocdInitNo = 0;
	iAllocdBlockNo= 0;
	for (TInt i = 0; i < STREAM_SOUND_TEMP_QUEUE_SIZE; i++)
	{
		iTmpQueue[i] = 0;
		iOutQueue[i] = 0;
		iAllocQueue[i] = NULL;
	}
	
	iFncGetInterface = NULL;
	
	iFncCreateSoundDecompressor = NULL;
	iFncDecompressSound = NULL;
	iFncSetupSoundDecompressor = NULL;
	iFncDestroySoundDecompressor = NULL;
	iFncGetSupportedFormats = NULL;
	iFncSetupDecoderLibrary = NULL;	
	iHandlerMode =ESwfMP3Audio;
}

CSoundHandler::~CSoundHandler()
{
	FreeSoundBuf();	

	
	if(iSndDecDll.Handle()) 
        iSndDecDll.Close();
		
	if(iSetupInfo.formatInfo)
	{
		delete iSetupInfo.formatInfo;		
		iSetupInfo.formatInfo = NULL;
	}

	if(iMdaAudioStream)
	{
		delete iMdaAudioStream;
		iMdaAudioStream = NULL;
	}
}

static void SoundAllocHandler(void* aData)
{
	CPlatformFlashEngine* pEngine = (CPlatformFlashEngine*)aData;
	if (pEngine)
	{
		pEngine->NotifyClientError(EInsufficientMemoryError, ETrue, ETrue);
	}
}

#define KNumStreamingSoundRates 2
const int KSampleRates[KNumStreamingSoundRates]=
{
	FI_SoundSampleRate_8kHz,FI_SoundSampleRate_11kHz
	//,FI_SoundSampleRate_22kHz,FI_SoundSampleRate_44kHz
};

const int KPlatformRates[KNumStreamingSoundRates]=
{
	TMdaAudioDataSettings::ESampleRate8000Hz,
	TMdaAudioDataSettings::ESampleRate11025Hz
	/*TMdaAudioDataSettings::ESampleRate22050Hz,
	TMdaAudioDataSettings::ESampleRate44100Hz*/
};

TInt PlatformRateForFI_Rate(TInt aFI_Rate)
{
	TInt retVal=-1;
	for(TInt i=0;i<KNumStreamingSoundRates;i++)
	{
		if(aFI_Rate==KSampleRates[i])
		{
			retVal=KPlatformRates[i];
			break;
		}
	}
	return retVal;
}

TInt NumSamplesForFI_Rate(TInt aFI_Rate)
{
	// Adjusting the sound buffer size using the sample rate and we
	// can only make the buffer size same or smaller than STREAM_SOUND_BUFFER_SIZE,
	// otherwise the memory allocated will not be enough and could crash.
	TInt shiftBy = 0;
	switch(aFI_Rate)
	{
		case FI_SoundSampleRate_5kHz:
		case FI_SoundSampleRate_8kHz:
			shiftBy = 2;
			break;
		case FI_SoundSampleRate_11kHz:
		case FI_SoundSampleRate_16kHz:
			shiftBy = 1;
			break;
	/*	case FI_SoundSampleRate_22kHz:
			shiftBy = 0;
			break;
		case FI_SoundSampleRate_44kHz:
			shiftBy = 0;*/
		default:
			break;
	}

	return (STREAM_SOUND_BUFFER_SIZE>>shiftBy);
}

//do alloc using host alloc instead of symbian new
void CSoundHandler::AllocSoundBuff()
{
	if(iAllocdInitNo >=STREAM_SOUND_TEMP_QUEUE_SIZE )
	{
		return; //init is complete exit
	}
	
	TUint32 numBuf=iAllocdInitNo;

	numBuf = STREAM_SOUND_TEMP_QUEUE_SIZE;

	
	TUint32 Bufsize = STREAM_SOUND_BUFFER_SIZE * (numBuf );
	
	TUint32 blkSize = 	iEngine->MemBlockSize();
	
	TUint32 numBlocks = Bufsize / blkSize ; 
	
	TAny* mem=NULL;
	
	if(numBlocks)
	{ 
		mem = iEngine->AllocateDynamicMemory(numBlocks);
	}
	else
	{
		numBlocks = 1;
		mem = iEngine->AllocateDynamicMemory(numBlocks);	
	}
	
	if(mem) //got mem block from host
	{
		iAllocQueue[iAllocdBlockNo] = mem;
		iAllocdBlockNo++;
		TUint32 allocSize = numBlocks*blkSize;	 //allocd size for now
		TUint32 bufOffset =0;//buffer offset
		while (bufOffset < allocSize)
		{
			if(iAllocdInitNo >= STREAM_SOUND_TEMP_QUEUE_SIZE)
				break; //exit when we are done with queue
		
			if (iTmpQueue[iAllocdInitNo] == NULL)
			{
				iTmpQueue[iAllocdInitNo] =  (unsigned char*)((unsigned char*)(mem) + bufOffset);	
			}

			iAllocdInitNo++;
			
			bufOffset += STREAM_SOUND_BUFFER_SIZE;

			if( (bufOffset + STREAM_SOUND_BUFFER_SIZE ) > allocSize )
				break;
			
		}
		
	}
	else
	{ //send insuff mem notification
		SoundAllocHandler((void*)iEngine);
	}
	
}

void CSoundHandler::FreeSoundBuf()
{
	TInt i=0;
	for (i=0;i<STREAM_SOUND_TEMP_QUEUE_SIZE;i++)
	{
		if (iOutQueue[i])
		{
			delete iOutQueue[i];
		}
	}
	
	for(i =0 ; i < iAllocdBlockNo ; i++)
	{
		iEngine->FreeDynamicMemory(iAllocQueue[i]);
	}
	
}

void CSoundHandler::InitializeSoundL(CPlatformFlashEngine* aEngine)
{
	iEngine = aEngine;

	TBool isPlugin = iEngine->IsBrowserPluginActive();
	NPPDebug(LOG_DEBUG,isPlugin,_L("CSoundHandler::InitializeSoundL ++ "));
	// We prefer to set the priority to EMdaPriorityNormal because setting it
	// to max prevents the alarm from vibrating (bug #145299); however, this means that keypress tones
	// interrupt Flash sound playback, so we need to use the CAknKeySoundSystem to turn off keypress tones
	// while playing back sound.
	CAknAppUi* appUi = (CAknAppUi*)iEngine->GetEikonEnv()->EikAppUi();
	iKeySoundSystem = appUi->KeySounds();
	iSoundPriority = EMdaPriorityNormal;

	TInt count=0;
	TInt resoffset = FlashEngineRscHandler::LoadEngineResourceFileL();
	//char** array=iEngine->AllocReadResourceArrayIntoFlashStringArrayL(count, R_FLASH_SUPPORTED_AUDIO_MIME_TYPES_ARRAY);
	char** array= MediaRecognizer::GetMMFsupportedFormatsL(iEngine,count,ETrue
									,R_FLASH_SUPPORTED_AUDIO_MIME_TYPES_ARRAY);
		
	iEngine->GetFIObject().FI_SetupDeviceSound((unsigned short)count, (const char**) array, true);
	for(TInt i=0; i<count; i++)
	{
		delete[] array[i];
	}
	delete[] array;
	FlashEngineRscHandler::DeleteEngineResourceFile(iEngine->GetEikonEnv(), resoffset);
	
	iStereoSupported = iEngine->IsStereoSupported();
	
    //RK : We used to Load snddecoder library here, that part is removed from here
    
    //RK : Its not sufficient to set sound format only for mp3 
    //snddecoder supports ram pcm and adpcm samples
    
	
		
	iSoundInfo.sampleSize = FI_SoundSampleSize_16bit;
	
	if(iStereoSupported)
	{
		iSoundInfo.channels = FI_SoundChannels_stereo;	
	}
	else
	{
		iSoundInfo.channels = FI_SoundChannels_mono;
	}

	iSetupInfo.numBuffers = STREAM_SOUND_NUM_BUFFERS;
	iSetupInfo.numFormats = 4;
	if(!iSetupInfo.formatInfo)
	{
	    iSetupInfo.formatInfo = new unsigned long[4];
	}
	iSetupInfo.formatInfo[0] = FI_StreamSound_PCM;
	iSetupInfo.formatInfo[1] = FI_StreamSound_PCMI;
	iSetupInfo.formatInfo[2] = FI_StreamSound_ADPCM;
	iSetupInfo.formatInfo[3] = FI_StreamSound_MP3;
	
	
	for(TInt ii=0; ii<KNumStreamingSoundRates; ii++)
	{
		iSoundInfo.sampleRate = KSampleRates[ii]; //FI_SoundSampleRate_8kHz;

		iSetupInfo.bufferSize = NumSamplesForFI_Rate(iSoundInfo.sampleRate);

		if(1!=iEngine->GetFIObject().FI_SetupStreamSound(&iSoundInfo, &iSetupInfo))
		{
			break; // something went wrong with one of the formats being init'd - give in	
		}
	}
        
    NPPDebug(LOG_DEBUG,isPlugin,_L("CSoundHandler::InitializeSoundL -- "));
}

void CSoundHandler::SetVolume(TInt aVolume)
{
	// if volume has high bit set, then sound is muted.
	if (((aVolume & (int)~KMuteMask) >= 0) && ((aVolume & ~KMuteMask) <= KMaxUserVolume))
	{
		iVolume = aVolume;
		// If flv playback is happening, just dispatch the setvolume to engine. 
		
		if( iEngine && iEngine->IsHelixFlvMode() )
		{
            THXVDCmdArg arg;
            
            if (iVolume & KMuteMask)
            {
                arg.iVolume = 0;
            }
            else
            {
                arg.iVolume = iVolume;
            }			
				
		    iEngine->EngineDispatchHXCommand(EHXVDCmdSetVolume, arg);
		}
		
		if ((iMdaAudioPlayerUtility) && ((iMdaEventState == EMdaPlaying) || (iMdaEventState == EMdaPaused)) )
		{
			if (iVolume & KMuteMask)
			{
				iMdaAudioPlayerUtility->SetVolume(0);
			}
			else
			{
				// convert the Flash-scale volume (0-9) to device-scale
				TInt max = iMdaAudioPlayerUtility->MaxVolume();
				TInt vol = max * iVolume / KMaxUserVolume;
				iMdaAudioPlayerUtility->SetVolume(vol);
			}
		}

		if ( iMdaAudioStream )
		{
			if (iVolume & KMuteMask)
			{
				iMdaAudioStream->SetVolume(0);
			}
			else
			{
				// convert the Flash-scale volume (0-9) to device-scale
				TInt max = iMdaAudioStream->MaxVolume();
				TInt vol = max * iVolume / KMaxUserVolume;
				iMdaAudioStream->SetVolume(vol);
			}
		}
	}
}

TInt CSoundHandler::GetVolume()
{
	return iVolume;
}

void CSoundHandler::CleanupSoundPlayer()
{
	delete iMdaAudioPlayerUtility;    
	iMdaAudioPlayerUtility = NULL;
	iMdaEventState = EMdaNone;
}

void CSoundHandler::DoSoundL()
{
    TInt audioPriority = KAudioPriorityRealOnePlayer;
	TMdaPriorityPreference flashLiteAudioPriorityPreference = ( TMdaPriorityPreference ) KAudioPrefRealOneLocalPlayback;
	if (iMdaEventState != EMdaNone)
	{
		return;
	}
	
	TUint32 file_num = iSoundQueue[0];
	iSoundQueue[0] = 0;

	if (file_num >= FIRST_MIDI_NUM)
	{
		// workaround to 2.1 phones to avoid having separate sound and video
		// files playing at the same time.
		TUint32 activeVideoId = iEngine->GetVideoEngine()->VideoNowPlaying();
		// zero indicates that no video is playing, anything >0 means a video is playing.
		if (activeVideoId > 0)
		{
			iEngine->GetVideoEngine()->StopVideoL(activeVideoId);
		}
		CachedSound* pSound = iCachedSounds;
		while (pSound)
		{
			if (pSound->iId == file_num)
			{
				//iMdaAudioPlayerUtility = CMdaAudioPlayerUtility::NewFilePlayerL(pSound->iFname, *this);
				iMdaAudioPlayerUtility = CMdaAudioPlayerUtility::NewFilePlayerL(pSound->iFname, *this, audioPriority, flashLiteAudioPriorityPreference );
				iMdaEventState = EMdaInitializing;
			}
			pSound = pSound->iNext;
		}
	}
}


short CSoundHandler::CacheSoundL(TAny* aSoundBuffer, TUint32 aSoundBufferSize, TUint32* aSoundID)
{
	TBool isPlugin = iEngine->IsBrowserPluginActive();
	NPPDebug(LOG_DEBUG,isPlugin,_L("CSoundHandler::CacheSoundL ++ "));
	
	CachedSound* pSound = new(ELeave)CachedSound(EFalse);
	if (pSound)
	{
	    TPtrC name;
		if (iEngine->IsBrowserPluginActive())
		{
		    name.Set(KBrowserApp);
		}
		else
		{
    		//TFileName fn = iEngine->GetUI()->GetAppPath();
		    TFileName fn = iEngine->GetAppInfoIf()->GetAppPath();
    		TParsePtrC parse(fn);
    		name.Set(parse.Name());		        
		}
        CPlatformFlashEngine::CreateFilenameEnsurePath(iEngine->GetRFs(), pSound->iFname, ETrue);
		(pSound->iFname).Append(name);
	    (pSound->iFname).AppendNum((TInt)iMidi_num);
	
		if (!MediaRecognizer::CreateFilenameFromData(pSound->iFname
												, aSoundBuffer
												, aSoundBufferSize
												,MediaRecognizer::EAudio ))
		{
			NPPDebug(LOG_INFO,isPlugin,_L("CSoundHandler::CacheSoundL Could not find Sound Format from our recog "));
			//if we dont find from our media recognizer , let us try mmf headers for data
			if(!(MediaRecognizer::CreateFilenameFromMMFDataL(iEngine,
    						pSound->iFname
    						, aSoundBuffer
    						, aSoundBufferSize)))
			{
				NPPDebug(LOG_ERROR,isPlugin,_L("CSoundHandler::CacheSoundL Sound Format Not supported "));
		
				delete pSound;
				return SI_InvalidFormatError;
			}
			
		}
		
		NPPDebug(LOG_INFO,isPlugin,_L("CSoundHandler::CacheSoundL [%s] "),&(pSound->iFname));
				
		pSound->iNext = iCachedSounds;
		iCachedSounds = pSound;
		pSound->iId = iMidi_num;
	
		RFile file;
		TInt err = file.Replace(iEngine->GetRFs(), pSound->iFname, EFileWrite | EFileShareExclusive | EFileStream);
		if (err == KErrNone)
		{
			TPtr8 bufptr((TUint8*)aSoundBuffer, aSoundBufferSize, aSoundBufferSize);
			err = file.Write(bufptr);
			file.Close();
		}
	}
	*aSoundID = iMidi_num++;
	return SI_NoError;
}

unsigned short CSoundHandler::PlayCachedSound(TUint32 aSoundID, TUint32 aCount)
{
	StopSound();
	iSoundQueue[0] = aSoundID;
	iSoundQueuePlayCount[0] = aCount;
	iEngine->TriggerAsyncEvent();
	return SI_NoError;
}

void CSoundHandler::FreeCachedSound(TUint32 aSoundID)
{
	// rborcic 05/31/2005 -- optimization suggestion, clean up deleted CachedSound objects
	CachedSound* pSound = iCachedSounds;
	RFs fs = iEngine->GetRFs();
	while (pSound)
	{
		if (pSound->iId == aSoundID)
		{
			fs.Delete(pSound->iFname);
		}
		pSound = pSound->iNext;
	}
}

TInt16 CSoundHandler::PlaySoundL(TAny* aSoundBuffer, TUint32 aSoundBufferSize, TUint32 aCount)
{
	// This function will only be called to play local reference sounds.
	// The soundBuffer points to a string naming the reference, e.g. <incoming_mms>.
	// For now, all reference sounds are mapped to the same sound.
	char* c = (char*)aSoundBuffer;
	if ((aSoundBufferSize > 4) && ((c[0] == 'F') && (c[1] == 'L') && (c[2] == 'S') && (c[3] == 'R')))
	{
		CachedSound* pSound = new CachedSound(ETrue);
		if (!pSound)
			return SI_GenericError;
		
		// first 4 characters are thrown away, rest is an 8 bit string identifying the sound
		HBufC* refSndId16=iEngine->CreateEpocStringLC(c+4,aSoundBufferSize-4);
		pSound->iNext = iCachedSounds;
		iCachedSounds = pSound;
		pSound->iId = iMidi_num;
		TInt resoffset = FlashEngineRscHandler::LoadEngineResourceFileL();
		pSound->iFname.Copy(iEngine->MatchedStringFromArrayL(R_FLASH_REFERENCE_SOUND_ARRAY,*refSndId16));
		CleanupStack::PopAndDestroy(refSndId16);
		PlayCachedSound(iMidi_num, aCount);
		iMidi_num++;
		FlashEngineRscHandler::DeleteEngineResourceFile(iEngine->GetEikonEnv(), resoffset);
		return SI_NoError;
	}
	else
	{
		return SI_GenericError;
	}
}

unsigned short CSoundHandler::PauseSound()
{
	short retVal=SI_GenericError;
    
    if(iMdaAudioStream && (iMdaStreamState == EMdaDestroy))
    {
        iMdaAudioStream->Stop();
    }
	
	if ((iMdaEventState == EMdaPlaying) && iMdaAudioPlayerUtility && (KErrNone==iMdaAudioPlayerUtility->Pause()))
	{
		iMdaEventState = EMdaPaused;
		retVal=SI_NoError;
	}

	return retVal;
}

unsigned short CSoundHandler::ResumeSound()
{
	if (iMdaAudioPlayerUtility)
	{
		if (iMdaEventState == EMdaPaused)
		{
			iMdaAudioPlayerUtility->Play();
			iMdaEventState = EMdaPlaying;
			return SI_NoError;
		}
		else if (iMdaEventState == EMdaPausedAtStart)
		{
			iMdaAudioPlayerUtility->Play();
			iMdaEventState = EMdaPlaying;
			iEngine->GetFIObject().FI_NotifyDeviceSoundStatus(FI_DeviceSoundStarted);
			return SI_NoError;
		}
	}
	
	
	return SI_GenericError;
}

void CSoundHandler::MapcInitComplete(TInt aError, const TTimeIntervalMicroSeconds& /*aDuration*/)
{
	if (aError == KErrNone)
	{
		switch(iMdaEventState)
		{
		case EMdaNone:
			break;
			
		case EMdaInitializing:
			{
				// make sure that keysounds don't interfere with Flash sound playback
				SuppressKeySounds();

				if (iVolume & KMuteMask)
				{
					iMdaAudioPlayerUtility->SetVolume(0);
				}
				else
				{
					TInt max = iMdaAudioPlayerUtility->MaxVolume();
					TInt vol = max * iVolume / KMaxUserVolume;
					iMdaAudioPlayerUtility->SetVolume(vol);
				}
				if (iSoundQueuePlayCount[0] > 1)
				{
					iMdaAudioPlayerUtility->SetRepeats(iSoundQueuePlayCount[0] - 1, TTimeIntervalMicroSeconds(0));										
				}
				TInt audioPriority = KAudioPriorityRealOnePlayer;		
				TMdaPriorityPreference flashLiteAudioPriorityPreference = ( TMdaPriorityPreference ) KAudioPrefRealOneLocalPlayback;
				iMdaAudioPlayerUtility->SetPriority(audioPriority, flashLiteAudioPriorityPreference);

				//iMdaAudioPlayerUtility->SetPriority(iSoundPriority, EMdaPriorityPreferenceTime);
				if (iEngine->IsPaused())
				{
					iMdaEventState = EMdaPausedAtStart;
				}
				else
				{
					iMdaAudioPlayerUtility->Play();
					iMdaEventState = EMdaPlaying;
					iEngine->GetFIObject().FI_NotifyDeviceSoundStatus(FI_DeviceSoundStarted);
				}
				iMdaStreamReady = ETrue;
			}
			break;
			
		case EMdaPlaying:
			iEngine->GetFIObject().FI_NotifyDeviceSoundStatus(FI_DeviceSoundStarted);
			break;

		case EMdaDestroy:
			CleanupSoundPlayer();
			break;
		}
	}
	else
	{
		iEngine->GetFIObject().FI_NotifyDeviceSoundStatus(FI_DeviceSoundFailed);
		CleanupSoundPlayer();
	}
}

void CSoundHandler::MapcPlayComplete(TInt aError)
{
    /* RC: ou1cimx1#519098: For embedded MIDI file, when we remove the  
     * headset, KErrInUse is returned from DevSound.  To resume the 
     * playback on speaker we pause & resume the player.     
     */
    if (aError == KErrInUse)
    {
        iEngine->Pause(EReasonUser);
        TRAP_IGNORE(iEngine->ResumeL(EReasonUser, EFalse));
        return;
    }
    
    // allow keysounds to play again.
	RestoreKeySounds();
	
	if(aError==KErrNone)
	{
		iEngine->GetFIObject().FI_NotifyDeviceSoundStatus(FI_DeviceSoundComplete);
	}
    else
    {
    	//Do not include KErrCancel
    	if(aError==KErrCorrupt || aError==KErrInUse || aError==KErrNoMemory)
    	{
   	 		iEngine->GetFIObject().FI_NotifyDeviceSoundStatus(FI_DeviceSoundFailed);
    	}
    }
    
    CleanupSoundPlayer();
}


void CSoundHandler::CleanupCachedFiles(RFs& aFs)
{
	CachedSound* pSound = iCachedSounds;
	CachedSound* tpSound;

	while (pSound)
	{
		tpSound = pSound;
		if (!pSound->iBReference)
		{
			aFs.Delete(pSound->iFname);
		}
		pSound = pSound->iNext;
		delete tpSound;
	}
	iCachedSounds = NULL;
	iMidi_num = FIRST_MIDI_NUM;
}

void CSoundHandler::StopSound()
{
	if (iMdaAudioPlayerUtility)
	{
		switch(iMdaEventState)
		{
		case EMdaNone:
			CleanupSoundPlayer();
			break;
			
		case EMdaInitializing:
			iMdaEventState = EMdaDestroy;
			break;
		
		case EMdaPaused:
		case EMdaPausedAtStart:
		case EMdaPlaying:
			iMdaAudioPlayerUtility->Stop();
			CleanupSoundPlayer();
			break;
			
		case EMdaDestroy:
			CleanupSoundPlayer();
			break;
		}
	}
	
	// clear out the cache.
	iSoundQueue[0] = 0;
	
	if (iMdaAudioStream)
	{
		switch(iMdaStreamState)
		{
		case EMdaInitializing:
			//iMdaAudioStream->Stop();
			iMdaStreamState = EMdaDestroy;
			break;
			
		case EMdaPlaying:
			iMdaAudioStream->Stop();
			iMdaStreamState = EMdaDestroy;
			break;
			
		case EMdaNone:
		case EMdaDestroy:
			delete iMdaAudioStream;
			iMdaAudioStream = NULL;
			iMdaStreamState = EMdaNone;
			iMdaStreamReady = EFalse;
			break;
		}
	}
}

TInt CSoundHandler::GetChannelType()
{
	if(iStereoSupported)
	{
		return TMdaAudioDataSettings::EChannelsStereo;
	}
	else
	{
		return TMdaAudioDataSettings::EChannelsMono;
	}
}

// from MMdaAudioOutputStreamCallback
void CSoundHandler::MaoscOpenComplete(TInt aError)
{
	TRAP_IGNORE(DoMaoscOpenCompleteL(aError));
}

void CSoundHandler::DoMaoscOpenCompleteL(TInt aError)
{	
//RK : Fix for OEM's audio mixer bug. 
   if(aError!=KErrNone && aError!=KErrCancel )
	{
		iEngine->GetFIObject().FI_NotifyDeviceSoundStatus(FI_DeviceSoundFailed);
		CleanupSoundPlayer();
		return;		
	}

	if(EMdaDestroy==iMdaStreamState)
	{
		//close device message came while we were still opening
		// we don't need to call stop (never started) just make sure it gets deleted and state reset.
		MaoscPlayComplete(KErrNone);
		return;
	}
	
	// make sure that keysounds don't interfere with Flash sound playback
	SuppressKeySounds();
//#ifndef __WINS__
	if (iVolume & KMuteMask)
	{
		iMdaAudioStream->SetVolume(0);
	}
	else
	{
		TInt max = iMdaAudioStream->MaxVolume();
		TInt vol = max * iVolume / KMaxUserVolume;
		iMdaAudioStream->SetVolume(vol);
	}
//#endif
	
	if ( iMdaStreamState == EMdaInitializing )
	{
		iMdaStreamState = EMdaPlaying;
	}
	
	iMdaStreamReady = ETrue;
	iBuffersQueued = 0;

	for (TInt i = 0; i < iTmpOutBuf; i++)
	{
		PlayStreamSoundBufferL(iTmpQueue[i]);
	}
	
	iTmpOutBuf = 0;
	iSndPos = 0;
}


void CSoundHandler::MaoscPlayComplete(TInt aError)
{
    
    // If we receive KErrInUse it means that some external event has pre-empted
    // us and acquired the sound channel
    if( KErrInUse == aError &&  iMdaStreamState == EMdaPlaying )
    {
        return;
    }
    
//RK: Fix for OEM licensee bug
	if((aError!=KErrNone && aError!=KErrCancel )&& !(aError == KErrUnderflow && iMdaStreamState == EMdaDestroy))
	{
		iEngine->GetFIObject().FI_NotifyDeviceSoundStatus(FI_DeviceSoundFailed);
		CleanupSoundPlayer();
		return;		
	}
	
	if(iMdaAudioStream)
	{
		delete iMdaAudioStream;
		iMdaAudioStream = NULL;
	}
	iMdaStreamReady = EFalse;

	if (iMdaStreamState != EMdaDestroy)
	{
		iMdaStreamState = EMdaNone;
		TRAP_IGNORE(OpenStreamSoundDeviceL());
	}
	else
	{
		iMdaStreamState = EMdaNone;
	}

	iEngine->GetFIObject().FI_StreamSoundReset();
}

#ifdef USE_POSITION_INFO

void CSoundHandler::MaoscBufferCopied(TInt aError, const TDesC8& aBuffer)
{
	switch (aError)
	{
	case KErrNone:
		PollStreamSound();
		break;
		
	case KErrUnderflow:
		//iMdaStreamState = EMdaUnderflow;
		sndPos = 0;
		break;
		
	case KErrCancel:
	case KErrAbort:
	default:
		break;
	}
}

#else /* ! USE_POSITION_INFO */

void CSoundHandler::MaoscBufferCopied(TInt aError, const TDesC8& /*aBuffer*/)
{
	iEngine->GetFIObject().FI_StreamSoundBufferComplete();
	iBuffersQueued--;

	switch (aError)
	{
	case KErrNone:
	case KErrUnderflow:
		PollStreamSound();
		break;
	}
}

#endif /* USE_POSITION_INFO */

TBool CSoundHandler::OKToPause()
{
#ifdef USE_POSITION_INFO
	if (iMdaAudioStream && (iMdaStreamState == EMdaPlaying))
	{
		return EFalse;
	}
	else
	{
		return ETrue;
	}
#else
	return ETrue;
#endif // USE_POSITION_INFO
}


void CSoundHandler::PollStreamSound()
{
#ifdef USE_POSITION_INFO
	
	if (iMdaAudioStream && (iMdaStreamState == EMdaPlaying))
	{
		TTimeIntervalMicroSeconds time = iMdaAudioStream->Position();
		TUint pos = time.Int64().Low();

		while ((pos > sndPos) && (pos - sndPos >= STREAM_SOUND_BUFFER_TIME))
		{
			//lastBufferTime.HomeTime(); // reset last buffer time
			
			FI_StreamSoundBufferComplete(pMMObject);
			sndPos += STREAM_SOUND_BUFFER_TIME;
			iBuffersQueued--;
		}
	}
	
#endif /* USE_POSITION_INFO */
}

unsigned short CSoundHandler::OpenStreamSoundDeviceL()
{
	if (iEngine->IsPhoneCallActive())
	{
		return SI_GenericError;
	}

	// workaround to 2.1 phones to avoid having separate sound and video
	// files playing at the same time.
	TUint32 activeVideoId = iEngine->GetVideoEngine()->VideoNowPlaying();

	// zero indicates that no video is playing, anything >0 means a video is playing.
	if (activeVideoId > 0)
	{
		iEngine->GetVideoEngine()->CancelVideoL(activeVideoId);
		iEngine->GetVideoEngine()->DoRunL(NULL);
	}

	if(iMdaAudioStream && (iMdaStreamState == EMdaDestroy))
	{
		iMdaAudioStream->Stop();
		delete iMdaAudioStream;
		iMdaAudioStream = NULL;
		iMdaStreamState = EMdaNone;
	}

	if (iMdaAudioStream == NULL)
	{
		TInt fiRate = iEngine->GetFIObject().FI_SetupStreamSound(NULL,NULL);
		TInt platRate=PlatformRateForFI_Rate(fiRate);
		if(-1==platRate)
		{
			return SI_GenericError;
		}
		//If it is a mp3 stream we set max priority.This will allow  mixing when music player is started 
		//TSW_ID:ASHA-7QNA49 :We set the same priority and preference for playback using helix
		if(iEngine->IsMp3Stream())
        {    
            iMdaAudioStream = CMdaAudioOutputStream::NewL(*this,EMdaPriorityMax,EMdaPriorityPreferenceTimeAndQuality);
        }
		else
        {    
            iMdaAudioStream = CMdaAudioOutputStream::NewL(*this,KAudioPriorityRealOnePlayer,( TMdaPriorityPreference ) KAudioPrefRealOneLocalPlayback);
        }
        iSettings.iChannels=TMdaAudioDataSettings::EChannelsStereo;
		iSettings.iSampleRate=platRate;
		iMdaAudioStream->Open(&iSettings);
		iOutQueuePos = 0;
		iTmpOutBuf = 0;
		iMdaStreamState = EMdaInitializing;
		iMdaStreamReady = EFalse;

		// Get the actual number of samples in the buffer
		iBufSamples = NumSamplesForFI_Rate(fiRate);

	}
	else switch (iMdaStreamState)
	{
	case EMdaNone: // strange state, should not be like this.
		ASSERT(1);
		break;
		
	case EMdaPlaying: // OK - leave it alone.
		break;
		
	case EMdaInitializing: // OK - wait for OpenComplete, leave state as Initializing
		break;
		
	case EMdaDestroy: // Was being destroyed, but now reopened, change state to Initializing so it will re-open.
					  // Must allow it to close and then re-open to get new sample rate
		iMdaStreamState = EMdaInitializing;
		break;
	}
	return SI_NoError;
}

unsigned short CSoundHandler::CloseStreamSoundDevice()
{
	if (iMdaAudioStream != NULL)
	{
		if (iMdaStreamState == EMdaPlaying)
		{
			// we need to set the state to EMdaDestroy here because the Stop() triggers immediate execution
			// of MaoscPlayComplete(), which will attempt to reopen the sound interface unless the state
			// is EMdaDestroy.
			iMdaStreamState = EMdaDestroy;
			iMdaAudioStream->Stop();
		}
		else
		{
			iMdaStreamState = EMdaDestroy;
		}
	}

	// allow keysounds to play again.
	RestoreKeySounds();
	

	return SI_NoError;
}

unsigned short CSoundHandler::HandlePlayStreamUnderflowL(TAny* aData)
{
	AllocSoundBuff();

	if(iTmpQueue[iTmpOutBuf])
	{
	
		FlashMemCpy(iTmpQueue[iTmpOutBuf++], aData, iBufSamples);
	
		if (iTmpOutBuf >= STREAM_SOUND_TEMP_QUEUE_SIZE)
		{ // uh oh, ran out of buffers, keep reusing last one.
			iTmpOutBuf = STREAM_SOUND_TEMP_QUEUE_SIZE - 1;
		}
		
	}
	return SI_GenericError;	
}

unsigned short CSoundHandler::HandlePlayStreamPlayL(TAny* aData)
{
	if(iOutQueue[iOutQueuePos])
	{
		iOutQueue[iOutQueuePos]->Set((TUint8*)aData, iBufSamples);	
	
		iMdaAudioStream->WriteL(*iOutQueue[iOutQueuePos]);
	
		iBuffersQueued++;
		iOutQueuePos++;
		if (iOutQueuePos >= STREAM_SOUND_TEMP_QUEUE_SIZE) iOutQueuePos = 0;
	}

	return SI_NoError;
}

unsigned short CSoundHandler::PlayStreamSoundBufferL(TAny* aData)
{
	// workaround to 2.1 phones to avoid having separate sound and video
	// files playing at the same time.
	TUint32 activeVideoId = iEngine->GetVideoEngine()->VideoNowPlaying();
	
	// zero indicates that no video is playing, anything >0 means a video is playing.
	if (activeVideoId > 0)
	{
		iEngine->GetVideoEngine()->StopVideoL(activeVideoId);
	}
	
	if (iMdaAudioStream)
	{
		switch (iMdaStreamState)
		{
		case EMdaInitializing:
		case EMdaUnderflow:
			
			return HandlePlayStreamUnderflowL(aData);
			
		case EMdaPlaying:
			return HandlePlayStreamPlayL(aData);
		}
	}
	else 
	{
		AllocSoundBuff();

		if(iTmpQueue[iTmpOutBuf])
		{
			FlashMemCpy(iTmpQueue[iTmpOutBuf++], aData, STREAM_SOUND_BUFFER_SIZE);
			if (iTmpOutBuf >= STREAM_SOUND_TEMP_QUEUE_SIZE)
			{ // uh oh, ran out of buffers, keep reusing last one.
				iTmpOutBuf = STREAM_SOUND_TEMP_QUEUE_SIZE - 1;
			}
			
			OpenStreamSoundDeviceL();
		}

		return SI_GenericError;
	}
	
	return SI_NoError;
}

void* CSoundHandler::CreateSoundDecompressor(FI_StreamSoundInfo* aSoundInfo, TUint32 aFormatInfo)
{
	void* d = NULL;
	
	if( iFncCreateSoundDecompressor )
		d = iFncCreateSoundDecompressor(aSoundInfo, aFormatInfo);
	
	return d;
}

void CSoundHandler::SetupSoundDecompressor(TAny* aDecompressor, FI_StreamSoundDecompressorInfo* aDecompressorInfo)
{
	if( iFncSetupSoundDecompressor )
		iFncSetupSoundDecompressor(aDecompressor, aDecompressorInfo);
}

long CSoundHandler::DecompressSound(TAny* aDecompressor, TAny* aDestBuffer, TUint32 aNumSamples)
{
	if( iFncDecompressSound )
		return iFncDecompressSound(aDecompressor, aDestBuffer, aNumSamples);
	else
		return 0;
}

void CSoundHandler::DestroySoundDecompressor(TAny* aDecompressor)
{
	if( iFncDestroySoundDecompressor )
		iFncDestroySoundDecompressor(aDecompressor);
}

TBool CSoundHandler::LoadSoundLibrary()
{
	if (!iSndLibLoaded)
	{
		if (iSndDecDll.Load(KSndDecDllName) != KErrNone)
		{
			return FALSE;
		}
		
		iFncGetInterface = (PFNC_DI_FLASHSNDDEC_GETINTERFACE)iSndDecDll.Lookup(1);
		
		if (iFncGetInterface == NULL)
		{
			return FALSE;
		}
		
		iFncCreateSoundDecompressor	= (PFNC_DI_CREATESOUNDDECOMPRESSOR)iFncGetInterface(EIndex_CreateSoundDecompressor);
		iFncDecompressSound			= (PFNC_DI_DECOMPRESSSOUND)iFncGetInterface(EIndex_DecompressSound);
		iFncSetupSoundDecompressor	= (PFNC_DI_SETUPSOUNDDECOMPRESSOR)iFncGetInterface(EIndex_SetupSoundDecompressor);
		iFncDestroySoundDecompressor= (PFNC_DI_DESTROYSOUNDDECOMPRESSOR)iFncGetInterface(EIndex_DestroySoundDecompressor);
		iFncGetSupportedFormats		= (PFNC_DI_GETSUPPORTEDFORMATS)iFncGetInterface(EIndex_GetSupportedFormats);
		iFncSetupDecoderLibrary		= (PFNC_DI_SETUPDECODERLIBRARY)iFncGetInterface(EIndex_SetupDecoderLibrary);
		
		iSndLibLoaded = ETrue;
	
		for (TInt i=0;i<STREAM_SOUND_TEMP_QUEUE_SIZE;i++)
		{
			if (iOutQueue[i] == NULL)
			{
				iOutQueue[i] = new TPtrC8();
			}
			if ((iOutQueue[i] == NULL))
			{
				SoundAllocHandler((void*)iEngine);
			}
		}
		
		AllocSoundBuff()	;
	}
	return (unsigned char)iSndLibLoaded;

}

void CSoundHandler::UnloadSoundLibrary()
{
	if( !iSndLibLoaded )
	{
		return;
	}

}

TBool CSoundHandler::IsPlaying()
{
	if (EMdaPlaying == iMdaStreamState)
	{
		return ETrue;
	}
	if (EMdaPlaying == iMdaEventState)
	{
		return ETrue;
	}
	return EFalse;
}


void CSoundHandler::SuppressKeySounds()
{
	if ((!iKeySoundsOff) && iKeySoundSystem)
	{
		TRAP_IGNORE (iKeySoundSystem->PushContextL(R_AVKON_SILENT_SKEY_LIST));
		iKeySoundsOff = ETrue;
	}
}

void CSoundHandler::RestoreKeySounds()
{
	if (iKeySoundsOff && iKeySoundSystem)
	{
		iKeySoundSystem->PopContext();
		iKeySoundsOff = EFalse;
	}
}

void CSoundHandler::SetSoundHandlerMode(ESoundHandlerMode aMode)
{
	iHandlerMode = aMode;
	//if any mode change handle below
}

//RK: Buffering mechanism needs to be tuned based on 
//codec type
void CSoundHandler::HandleFlvSetup(TInt32 aCodecType)
{
	switch(aCodecType)
	{
		case KH263Codec:
						iHandlerMode = EFlvSorensonAudio;
						break;
		case KVP6Codec:
		case KVP6AlphaCodec: //now handle together
						iHandlerMode = EFlvOn2Vp6Audio;
						break;
	}		
}

CachedSound::CachedSound(TBool aIsReference)
{
	iBReference = aIsReference;
	iNext = NULL;
}

CachedSound::~CachedSound()
{
}

