/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that is used to 
*				 access subscriber information.
*
*/


#ifdef FEATURE_PACK_3

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "saplayer_build.h"


#include "symbian_subscriberinfo.h"
#include "commondef.h"

#include <etel3rdparty.h>


CSubscriberInfo* CSubscriberInfo::NewL()
{
	CSubscriberInfo* self=new(ELeave) CSubscriberInfo();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

CSubscriberInfo::CSubscriberInfo():	
	CActive(CActive::EPriorityLow)
   ,iTelephony(NULL)
   ,iSubscriberIdV1Pckg( iSubscriberIdV1 )
{
}


CSubscriberInfo::~CSubscriberInfo()
{
	Cancel();
	
	if(iTelephony)
	{
		delete iTelephony;
		iTelephony = NULL;
	}
	
	
}

void CSubscriberInfo::ListenL()
{
	if (IsActive())
		return;
	
	
	if(!iTelephony)
	{
		iTelephony = CTelephony::NewL();	
	}
	
	if(iTelephony)
	{
		iTelephony->GetSubscriberId(iStatus, iSubscriberIdV1Pckg);
	}
		
	SetActive();
	
}
void CSubscriberInfo::ConstructL()
{

	CActiveScheduler::Add(this);
	
	ListenL();
}

void CSubscriberInfo::DoCancel()
{
	iTelephony->CancelAsync(CTelephony::EGetSubscriberIdCancel);
}
	

void CSubscriberInfo::RunL()
{
	if(iStatus==KErrNone)
       {
       		if(iTelephony)
			{
				delete iTelephony;
				iTelephony = NULL;
			}
       }
}


TBool CSubscriberInfo::GetSubscriberInfo(unsigned short aInfoType, TDes8& aRetString)
{
	TBool retVal = EFalse;
	TRAPD( err, retVal = GetSubscriberInfoL(aInfoType, aRetString) );
	if ( err == KErrNone)
	{
		
	}
	return retVal;
}

TBool CSubscriberInfo::GetSubscriberInfoL(unsigned short aInfoType, TDes8& aRetString)
{
    TBool retValue = EFalse;
    if ( aInfoType== FI_NetworkInfo_MobileCarrierCode )
    {
  		if(iStatus == KErrNone)
		{
			aRetString.Copy(iSubscriberIdV1.iSubscriberId);
			retValue = ETrue;
		}   

    }
	
	ListenL();
		
	return retValue;
}

#endif // FEATURE_PACK_3
