/*
* Copyright (c) 2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  implements the the Trust store for list of trusted files.
*
*/


//SecuritySandbox
#include <bautils.h>
#include <hash.h>
#include <eikdef.h>
#include <eikenv.h>
#include "symbian_truststore.h"

//Maximun number of trusted entries allowed 
const TInt KMaxEntries = 100 ;
_LIT(KFileExtension,".dat");
_LIT(KConfigStoreDirName,"TrustConfig\\");
_LIT(KTrustEntryCount, "TrustEntryCount.dat");
const char Base64[] ="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-";
// ---------------------------------------------------------------------------
// Two phase construction
//
// ---------------------------------------------------------------------------
CFlashTrustStore* CFlashTrustStore::NewL()
    {
        CFlashTrustStore* self=new(ELeave) CFlashTrustStore();
        CleanupStack::PushL(self);
        self->ConstructL();
        CleanupStack::Pop(self);
        return self;
    }
// ---------------------------------------------------------------------------
// Two phase constructor
//
// ---------------------------------------------------------------------------
void CFlashTrustStore::ConstructL()
    {

        TChar KCharColon(':');
        TInt zero = 0;
        TInt pathLength =100;
        HBufC *privatePath = HBufC::NewLC(pathLength); 
        TPtr privatePathPtr(privatePath->Des());
        ifsSession= CEikonEnv::Static()->FsSession();
        //Get the private path without drive
        ifsSession.PrivatePath(privatePathPtr);
        //Get the drive letter
        TChar systemDrive=ifsSession.GetSystemDriveChar();
        iPrivateDirFullPath.SetLength(zero);
        iPrivateDirFullPath.Append(systemDrive);
        iPrivateDirFullPath.Append(KCharColon);
        iPrivateDirFullPath.Append(privatePathPtr);
        CleanupStack::PopAndDestroy(privatePath);
             
    }
//---------------------------------------------------------------------------
 // Check if the entry in the private directory is already presen for current file
 //
 // @param TPtrC& aSourceFileName file name for which entry is to be checked
 //---------------------------------------------------------------------------
TBool CFlashTrustStore::IsContentTrustedL(const TPtrC& aSourceFileName)
{
        TBool res=EFalse;
        HBufC* hashedFilePath=NULL ;
        TRAPD(err, hashedFilePath=GetHashedFilePathL(aSourceFileName));
        if(err==KErrNone && hashedFilePath)
            {
            CleanupStack::PushL(hashedFilePath);
            if(BaflUtils::FileExists(ifsSession,hashedFilePath->Des()))
                {
                  //Change the last modified time to current time
                   TouchFile(hashedFilePath->Des());
                   res =ETrue;
                 }
            
            CleanupStack::PopAndDestroy(1);//Destroy  hasedFilePath
            }
         return res;
}

// ---------------------------------------------------------------------------
//Get the Hashed file path for given source path
//
//This path contains the full path including the drive and private folder of flash
// ---------------------------------------------------------------------------


HBufC* CFlashTrustStore::GetHashedFilePathL(const TDesC & aSourceFileName)const
    {
        HBufC8* lHash = GetHashLC(aSourceFileName);
        TPtr8 lHashData = lHash->Des();
        TPtr8 hashedFile = ConvertHashToBase64LC(lHash);
        
        //Convert TPtr8 to TPtr
        HBufC *hashedFileName = HBufC::NewLC(hashedFile.Length());
        //length of converted hash is 16 so total length will be within  100
        TInt pathLength = 100;
        HBufC *hashedFilePath=HBufC::NewL(pathLength);
        TPtr hashedFilePathPtr = hashedFilePath->Des();
        hashedFileName->Des().Copy(hashedFile);
        
        //Convert the hashed file to full path including drive , private director and sub folder
        hashedFilePathPtr.Copy(iPrivateDirFullPath);
        hashedFilePathPtr.Append(KConfigStoreDirName);
        hashedFilePathPtr.Append(hashedFileName->Des());
        hashedFilePathPtr.Append(KFileExtension);
    
        CleanupStack::PopAndDestroy(3);//lhash,hashedFile,hashedFileName
        //CleanupStack::PushL(hashedFilePath);
        return hashedFilePath;
    }
// ---------------------------------------------------------------------------
// Convert the hash value to the base64 format
//@param  const HBufC8* lHash we change the value to base64
// ---------------------------------------------------------------------------



TPtr8 CFlashTrustStore::ConvertHashToBase64LC(const HBufC8* lHash)const
    {
            TUint8* outbuf = new(ELeave) TUint8[lHash->Length()*8];
            CleanupStack::PushL(outbuf);
            
            TInt lTempOutLen = BinTo64(lHash->Ptr(),lHash->Length(),(char*)outbuf);
            TPtr8 base64Hash(outbuf,lTempOutLen,lTempOutLen);
            return base64Hash;

    }
// ---------------------------------------------------------------------------
// Change the last modified date of the file to the current time
//@param const TFileName  aFileName for which last modified time is changed to current time
// ---------------------------------------------------------------------------


void CFlashTrustStore::TouchFile(const TFileName  aFileName)
    {

            RFile lFile;
            if(lFile.Open(ifsSession,aFileName,EFileShareAny|EFileWrite)==KErrNone)
                {
                    TTime currentTime;
                    currentTime.HomeTime();
                    lFile.SetModified(currentTime);
                    lFile.Close();
                }
    }
//---------------------------------------------------------------------------
// Add a new  entry into to the private path
//
//@param const TPtrC& aSourceFileName is the file entry to be added
//--------------------------------------------------------------------------- 
void CFlashTrustStore::TrustContentL(const TPtrC& aSourceFileName)
    {
       
        RFile aFile;
        HBufC* hashedFilePath = GetHashedFilePathL(aSourceFileName);
        CleanupStack::PushL(hashedFilePath);
        TInt  numOfEntries = NumOfEntriesL();
        TInt pathLength =100;
        
        //Cleanup if the count is more that KMaxEntries
        if(numOfEntries >= KMaxEntries)
           {
               TRAPD(err,CleanupTrustEntriesL());
               if(err==KErrNone)
                   numOfEntries--;
               else
                   return;
           }
        //Ensure that private directory and TrustConfig directory are created
        BaflUtils::EnsurePathExistsL(ifsSession,hashedFilePath->Des());
        if(aFile.Create(ifsSession,hashedFilePath->Des(),EFileShareAny|EFileWrite)==KErrNone)
            {
                      HBufC *fileCount = HBufC::NewL(pathLength);
                      TPtr trustCountFile(fileCount->Des());
                      
                      trustCountFile.Copy(iPrivateDirFullPath);
                      trustCountFile.Append(KTrustEntryCount);
                      WriteFileStream(trustCountFile,numOfEntries+1);
            }
        aFile.Close();
        CleanupStack::PopAndDestroy(1);// hashedFIlePath

        
    }
//---------------------------------------------------------------------------
// Function returns the number of entries in the trustcount file
//
//--------------------------------------------------------------------------- 

TInt CFlashTrustStore::NumOfEntriesL()
    {
          TInt pathLength = 100;
          HBufC *fileCount = HBufC::NewL(pathLength);
          TInt32 numOfValues=0;
          RFile bFile;
          TPtr trustCountFile(fileCount->Des());
          
          trustCountFile.Copy(iPrivateDirFullPath);
          trustCountFile.Append(KTrustEntryCount);

          //Read the entries in TrustEntryCount.dat
          if(BaflUtils::FileExists(ifsSession,trustCountFile))
             {
                 numOfValues= ReadFileStream(trustCountFile);
             }
          else
             {
                  bFile.Create(ifsSession,trustCountFile,EFileShareAny|EFileWrite);
                  bFile.Close();
             }
          delete fileCount;
          return numOfValues;
                       
    }

// ---------------------------------------------------------------------------
// Cleanup the trust entry
//Select the LRU(based on last modified time) used entry for deleting
//
// ---------------------------------------------------------------------------


void CFlashTrustStore::CleanupTrustEntriesL()
    {
          TInt zero =0;  TInt pathLength =100;
         CDir *dirList;
         _LIT(KCharStar,"*");
         HBufC * privateDirPath = HBufC::NewLC(pathLength);
         HBufC * trustPath = HBufC::NewLC(pathLength);
         TPtr privateDirPathPtr(privateDirPath->Des());
         TPtr trustPathPtr(trustPath->Des());
          privateDirPathPtr.Copy(iPrivateDirFullPath);
          trustPathPtr.Copy(iPrivateDirFullPath);
          trustPathPtr.Append(KConfigStoreDirName);
                    
          privateDirPathPtr.Append(KConfigStoreDirName);
          privateDirPathPtr.Append(KCharStar);
          User::LeaveIfError(ifsSession.GetDir(privateDirPathPtr,
                    KEntryAttMaskSupported,ESortByDate,dirList));
                   // if(dirList->Count()>KMaxEntries)
          if(dirList->Count())
              {
                  TFileName filetobeDeleted( (*dirList)[zero].iName);
                  trustPathPtr.Append(filetobeDeleted);
                  User::LeaveIfError(BaflUtils::DeleteFile(ifsSession,trustPathPtr));
              }
          CleanupStack::PopAndDestroy(2);//privateDirPath,trustPath
             
    }
// ---------------------------------------------------------------------------
// Read the integer value from the file
//
// ---------------------------------------------------------------------------


TInt CFlashTrustStore::ReadFileStream(const TFileName  aFileName)
    {
        RFile bFile;
        //numOfvalues is returned as Max entires if the function leaves,We dont want to add more entries if this fails.
        TInt numOfValues = KMaxEntries;
        TInt version=1;
        if(bFile.Open(ifsSession,aFileName,EFileShareAny|EFileRead)==KErrNone)
        {
            RFileReadStream fileStream;
            fileStream.Attach(bFile);
            //version for future usage in case we need to add anything more
            TRAPD(err,version = fileStream.ReadInt32L());
            if(err==KErrNone)
                {
            
                    if(version==1)
                        {
                            TRAP_IGNORE(numOfValues = fileStream.ReadInt32L());
                        }
                }    
                fileStream.Close();
                bFile.Close();
        }

        return numOfValues;
    }
// ---------------------------------------------------------------------------
// Write the integer value into the file
//
// ---------------------------------------------------------------------------



void CFlashTrustStore::WriteFileStream(const TFileName  aFileName,TInt numOfENtries)
    {
            RFile bFile;
            //Version is needed in case we make any changes in future
            TInt version=1;
            if(bFile.Open(ifsSession,aFileName,EFileShareAny|EFileWrite)==KErrNone)
                {
                    RFileWriteStream fileWriteStream;
                    fileWriteStream.Attach(bFile);
                    TRAPD(err,fileWriteStream.WriteInt32L(version));
                    if(err==KErrNone)
                        {
                            if(version==1)
                                {
                                    TRAP_IGNORE(fileWriteStream.WriteInt32L(numOfENtries));
                                }
                        }
                    fileWriteStream.Close();
                    bFile.Close();
                }

    }
// ---------------------------------------------------------------------------
// Destructor. Frees in-memory trust store entries.
//
// ---------------------------------------------------------------------------
CFlashTrustStore::~CFlashTrustStore()
    {
        
    }
// ---------------------------------------------------------------------------
// Get the hash value for the given file path
//
// ---------------------------------------------------------------------------


HBufC8* CFlashTrustStore::GetHashLC(const TDesC& aFileName) const
    {

        CMD5* lHasher = CMD5::NewL();
        CleanupStack::PushL(lHasher);
        
        HBufC8* fileName = HBufC8::NewLC(aFileName.Length());
        fileName->Des().Copy(aFileName);
        //Convert the filename to lower case and take a hash.File system in symbain is case insensitive
        fileName->Des().LowerCase();
 
        TPtrC8 lHashPtr = lHasher->Hash(fileName->Des());
        HBufC8* lHash = lHashPtr.AllocL();

        CleanupStack::PopAndDestroy(2); //lHasher, fileName
        CleanupStack::PushL(lHash);
        return lHash;
        
        
    
    }
// ---------------------------------------------------------------------------
// Convert binary to base 64
//
// ---------------------------------------------------------------------------



int CFlashTrustStore::BinTo64(const TUint8* pInBuf, int len, char* pOutBuf)
{
    int c1, c2, c3;
    //INT32 ct = 0;
    //INT32 written = 0;
    int inOffset = 0;
    int outOffset = 0;

    while(inOffset < len)
    {
    c1 = pInBuf[inOffset++];
    if(inOffset == len)
    {
        Output64Chunk(c1, 0, 0, 2, pOutBuf, outOffset);
        outOffset += 4;
    }
    else
    {
        c2 = pInBuf[inOffset++];
        if(inOffset == len)
        {
        Output64Chunk(c1, c2, 0, 1, pOutBuf, outOffset);
        outOffset += 4;
        }
        else
        {
        c3 = pInBuf[inOffset++];
        Output64Chunk(c1, c2, c3, 0, pOutBuf, outOffset);
        outOffset += 4;
        }
    }
#if XXXBAB
    ct += 4;
    if(ct > 44) // EOL after 45 chars
    {
        pOutBuf[outOffset++] = '\n';
        written += 46;
        ct = 0;
    }
#endif
    }
#if XXXBAB
    if(ct)
    {
    pOutBuf[outOffset++] = '\n';
    ct++;
    }
#endif
    //This is not necessary as we will be apppending the file extension
   // pOutBuf[outOffset++] = '\0';
    return outOffset;
}


void CFlashTrustStore::Output64Chunk(int c1, int c2, int c3, int pads, char* pBuf,
                                        int bufOffset)
{
    pBuf[bufOffset++] = Base64[c1>>2];
    pBuf[bufOffset++] = Base64[((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4)];
    if (pads == 2) 
    {
        pBuf[bufOffset++] = '=';
        pBuf[bufOffset++] = '=';
    }
    else if (pads) 
    {
        pBuf[bufOffset++] = Base64[((c2 & 0xF) << 2) | ((c3 & 0xC0) >>6)];
        pBuf[bufOffset++] = '=';
    } 
    else 
    {
        pBuf[bufOffset++] = Base64[((c2 & 0xF) << 2) | ((c3 & 0xC0) >>6)];
        pBuf[bufOffset++] = Base64[c3 & 0x3F];
    }
}

    
//EOF
