/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class used to wrap up a 
*				 URL request call from the core player.
*
*/

#include <hal.h>
#include <AknGlobalNote.h>
#include <StringLoader.h>
#include <flash2engine.rsg>
#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "saplayer_build.h"

#include "symbian_urlrequestinfo.h"
#include "fi_interface.h"
#include "symbian_engine.h"
#include "symbian_ui_interface.h"
#include "symbian_clib.h"
#include "symbian_mmc_reservoir.h"
#include "platformconfig.h"
#include "flashengineresourcehandler.h"
TInt GetErrorNote(TInt aErrorCode);
//	static
CUrlRequestInfo* CUrlRequestInfo::NewL(void* aOwner, struct MM_Object* aMMObj, 
									   struct FI_URLStreamInfo* aStreamInfo, TUint aId)
{
	CUrlRequestInfo* self = new(ELeave) CUrlRequestInfo(aOwner, aMMObj, aId, aStreamInfo);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}


CUrlRequestInfo::CUrlRequestInfo(void* aOwner, struct MM_Object* aMMObj, TUint32 aId,
								 struct FI_URLStreamInfo* aStreamInfo)
:CHttpRequestInfo(aOwner, aMMObj, aStreamInfo, aId), iDownloadComplete(EFalse)
{
	//pPausedData = NULL;
	iUseRAMReservoir=EFalse;
	iStreamCancelled=EFalse;
	//numBytesPaused = 0;
	iLastErrorNote = 0;
	iIsRTMPTClose = aStreamInfo->isRTMPTClose;
}

CUrlRequestInfo::~CUrlRequestInfo()
{
	delete pAsyncObject;
#ifdef _TIMER_MSG
	if(iTimer)
		{
		iTimer->Cancel();
		delete iTimer;
		}
#endif //_TIMER_MSG

#ifdef FEATURE_FLV_MMC
	if(iMMCManager)
		{
		delete iMMCManager;
		iMMCManager = NULL;
		}
	if(iRAMReservoir)
		{
		delete iRAMReservoir;
		iRAMReservoir = NULL;
		}
		
	if(iPlatformConfig)
	{
		delete iPlatformConfig;
		iPlatformConfig=NULL;
	}
	
	if(iMMCAsyncObject)
	{
		delete iMMCAsyncObject;
		iMMCAsyncObject = NULL;
	}
	
#endif // FEATURE_FLV_MMC		
}

void CUrlRequestInfo::ConstructL()
{
	pAsyncObject = CFlhAsyncObject::NewL(CActive::EPriorityStandard, *this);
#ifdef FEATURE_FLV_MMC	
	iMMCAsyncObject = CFlhTimerObject::NewL(CActive::EPriorityLow, *this);
	iPlatformConfig = new (ELeave) CPlatformConfig;
#endif	
#ifdef _TIMER_MSG	
	iTimer = CFlhTimerObject::NewL(CActive::EPriorityStandard, *this);
#endif //_TIMER_MSG	
}


void CUrlRequestInfo::PrepareToBeServicedL()
{
	
}


void CUrlRequestInfo::HandleRequestFailed(TInt /*aError*/, TInt /*aHttpStatus*/)
{
	if(IsRTMPTClose())
	{
		return;
	}
	
	// Player doesn't care about our error codes (in fact they are opposite) so just tell it we failed.
	CPlatformFlashEngine* pEngine = iMMObj->pHostObj->iFIInterface->GetEngine();
	//as broswer donesn't support http status ocde to be propogated to flash plugin, hence we are 
	// sending "200" for success transaction and '0' for all other cases.
	pEngine->GetFIObject().FI_URLStreamClosed((TUint16)Id(), 0, 0 );
}
FI_StreamMediaType CUrlRequestInfo:: GetStreamMediaType()
		{
		// should be implemented in the derived classes
		return iStreamMediaType;
		}	
// once being serviced handle the fact that there is a header response...
void CUrlRequestInfo::HandleHeaderResponseL(struct FI_URLStreamHeaderInfo& aHeaders, const TDesC8& aFinalUrl)
{
	if(IsRTMPTClose())
	{
		return;
	}

	CPlatformFlashEngine* pEngine = iMMObj->pHostObj->iFIInterface->GetEngine();

	short errorCode = 0;
	FI_Interface& fiInterface=pEngine->GetFIObject();
	iNetBuf = fiInterface.FI_URLStreamOpened( (TUint16)Id(), &aHeaders, &errorCode );

	// if errorCode is set we may well have been deleted by now so we cannot rely on any member data...
	if(errorCode<=0)
	{
		User::Leave(KErrGeneral); // could map to flash error codes, HTTP queue does not care at present
	}
#ifdef _TIMER_MSG	
	iTimer->After(10000000);
#endif //_TIMER_MSG	
	TUint16 id=(TUint16)Id();
	fiInterface.FI_URLStreamSetFinalURL(id,(const char*)aFinalUrl.Ptr(),FI_UseDefault);
	
	iStreamMediaType = iNetBuf->streamMediaType;
	// create MMC object & RAM reserve based on the total length of buffer required
	//Mp3Streaming
	    if(iNetBuf->streamMediaType == FI_StreamMediaType_Sound)
	        {
	            pEngine->NewMP3StreamStarted();
	        }
#ifdef FEATURE_FLV_MMC	
	    //for sound streaming  numbers of buffers insterted in coreplayer =  ((SOUNDSTREAM_CYCLING_BUFFER_COUNT/2)-1) 
	    if((iNetBuf->streamMediaType == FI_StreamMediaType_FLV && iContentLength > TELESTREAM_CYCLING_BUFFER_SIZE * TELESTREAM_CYCLING_BUFFER_COUNT)||
	            (iNetBuf->streamMediaType == FI_StreamMediaType_Sound&& iContentLength > SOUNDSTREAM_CYCLING_BUFFER_SIZE * ((SOUNDSTREAM_CYCLING_BUFFER_COUNT/2)-1))) 
	        
		{

		iUseRAMReservoir = ETrue;
		            
		iMMCManager = new CMMCReservoir(pEngine, this);
		            
		            if(iMMCManager)
		                {
		                TInt error = iMMCManager->InitializeL(id,iContentLength);
		                    if( error != KErrNone )
		                    {
		                    //problem initializing MMC reservoir, delete it.
		                    HBufC* errorNote = NULL;
		                    TInt resoffset = FlashEngineRscHandler::LoadEngineResourceFileL();
		                    TInt errorResID = GetErrorNote(error);
		                    errorNote = StringLoader::LoadLC( errorResID );
		                    ShowErrorNoteL( *errorNote );
		                    CleanupStack::PopAndDestroy( errorNote );
		                    delete iMMCManager; 
		                    iMMCManager = NULL;
		                    FlashEngineRscHandler::DeleteEngineResourceFile(pEngine->GetEikonEnv(), resoffset);
		                    
		                    }
		                
		                }
		                                
		                TInt freeRAM;
		                HAL::Get(HAL::EMemoryRAMFree, freeRAM);

		                TInt32 ramHighWatermark = pEngine->GetPlatformConfig()->GetRAMResHighWatermark();
		                TInt32 ramLowWatermark = pEngine->GetPlatformConfig()->GetRAMResLowWatermark();
		                TInt32 ramUnUtilSpace = pEngine->GetPlatformConfig()->GetFlashRAMResvUnUtilisedSpace();



		                if( iMMCManager && 
		                                ( freeRAM > ( ramLowWatermark + ramUnUtilSpace ) ) )
		                    {
		                    iRAMResSize = ramLowWatermark;
		                    }
		                else if( freeRAM > ( ramHighWatermark + ramUnUtilSpace  ) )
		                    {
		                    iRAMResSize =ramHighWatermark;
		                    }
		                else if( freeRAM > ramUnUtilSpace )
		                    {
		                    iRAMResSize = freeRAM - ramUnUtilSpace;
		                    }
		                else
		                    {
		                    User::Leave(KErrNoMemory);
		                    }
		                // This is the fix for the bug : SRAA-78WD6L -  User 30 crash is 
		                // seen during forward seek of flv video.
		                if( (iRAMResSize>iContentLength) && (iContentLength != -1) )
		                    {
		                    iRAMResSize = iContentLength;
		                    }
		iRAMReservoir = HBufC8::NewL(iRAMResSize);			
		}
		
#endif //FEATURE_FLV_MMC
}
#ifdef FEATURE_FLV_MMC
TInt CUrlRequestInfo::GetRAMResSize()
	{
	
	return iPlatformConfig->GetRAMResLowWatermark();	
    //return (1024*1024);
	}
#endif //FEATURE_FLV_MMC
void CUrlRequestInfo::SetNetworkBuffer(struct FI_NetworkBufferInfo* aNetBuf)
{
	/*iNetBuf=aNetBuf;

	if (pPausedData != NULL)
	{
		if (numBytesPaused != 0)
		{
			pAsyncObject->Start();			
		}
		else
		{
			pPausedData->ReleaseData();
			pPausedData = NULL;
			TriggerAsyncReadWrite();
		}
	}
	else
		{
		TriggerAsyncReadWrite();
		}*/
	iNetBuf=aNetBuf;
	TriggerAsyncReadWrite(); 
}
void CUrlRequestInfo::ShowErrorNoteL(const TDesC& aErrMsg)
    {
    CAknGlobalNote* globalNote = CAknGlobalNote::NewLC();        
    globalNote->ShowNoteL( EAknGlobalErrorNote, aErrMsg );
    CleanupStack::PopAndDestroy(globalNote);  
    }
void CUrlRequestInfo::DoRunL(const CActive* /*aActive*/)
{

CPlatformFlashEngine* pEngine = iMMObj->pHostObj->iFIInterface->GetEngine();
    TBool dataReadComplete = EFalse;
    if(iStreamCancelled)
        {
        if(iMMCManager)
            {
            dataReadComplete = (iReadOffset == iMMCManager->BytesWritten()) ? ETrue : EFalse;        
            }
        else
            {
			dataReadComplete = (iReadOffset == ( (iRAMCycles*iRAMResSize)+ iRamOffset) ) ? ETrue : EFalse;
			}
        }
    if( iStreamCancelled && dataReadComplete && iLastErrorNote )
        {
        TInt resoffset = FlashEngineRscHandler::LoadEngineResourceFileL();
        HBufC* errorNote = NULL;
        errorNote = StringLoader::LoadLC( iLastErrorNote  );
        ShowErrorNoteL( *errorNote );
        CleanupStack::PopAndDestroy( errorNote );
        pEngine->NotifyClientError( KErrNoMemory, ETrue, ETrue );
        FlashEngineRscHandler::DeleteEngineResourceFile(pEngine->GetEikonEnv(), resoffset);

        iLastErrorNote = 0;
        return;
        }
    if( iReadOffset < iNumBytesDwlded )
        {
        TInt bytesRead = 0;
        // check read and write are in the same RAM  cycle
        if( iReadOffset >= iRAMCycles*iRAMResSize )
            {
            HBufC8* dataBuffer = HBufC8::NewLC( iNetBuf->maxBufSize );
            TPtr8 dataPtr = dataBuffer->Des();      
            //data is definitely in the RAM storage
            bytesRead = ReadFromRAMResL( dataPtr, 
                                        iReadOffset - ( iRAMCycles*iRAMResSize ), 
                                        iNetBuf->maxBufSize );
            WriteStreamDataToCoreL( dataPtr );                                      
            CleanupStack::PopAndDestroy( dataBuffer );                                      
            if( iNetBuf && ( iNetBuf->pBuffer ) 
                                        && ( iReadOffset < iNumBytesDwlded ) )
                {
                TriggerAsyncReadWrite();
                }
            }
        else
            {
            if( iReadOffset < ( iRAMCycles*iRAMResSize ) )
                {
                // we know that the data is definitely not in the 
                // current RAM cycle
                // try to read from the MMC 
                if( iMMCManager ) 
                    {
                    // since the MMC read now is asynchronous
                    iMMCManager->ReadFromMMCL( iReadOffset, iNetBuf->maxBufSize);          
                    }
                // no MMC.. should be in previous ram cycle 
                else
                    {
                
                    HBufC8* tempData = HBufC8::NewLC( iNetBuf->maxBufSize - bytesRead);
                    TPtr8 tempDataPtr = tempData->Des();
                    
                // offset in RAM 
                    TInt ramReadOffset = iRAMResSize - ( (iRAMCycles*iRAMResSize) - ( iReadOffset + bytesRead ) );
                    ReadFromRAMResL( tempDataPtr, ramReadOffset,( iNetBuf->maxBufSize) 
                                                                                        - bytesRead );
                    WriteStreamDataToCoreL( tempDataPtr );                  
                    CleanupStack::PopAndDestroy(tempData);
                    if( iNetBuf && ( iNetBuf->pBuffer ) 
                                                && ( iReadOffset < iNumBytesDwlded ) )
                        {
                        TriggerAsyncReadWrite();
                        }

                    }
                 
                }
            }
            
        
        }  
  


}
TInt CUrlRequestInfo::NotifyError(TInt aStatus)  
    {
    TRAPD(err, NotifyErrorL(aStatus));
    return err;
    }
// the MMC read has failed and we have been notified of the error   
void CUrlRequestInfo::NotifyErrorL(TInt aStatus) 
    {
    CPlatformFlashEngine* pEngine = iMMObj->pHostObj->iFIInterface->GetEngine();
    if( aStatus!=KErrNone )
        {
        iStreamCancelled = ETrue; 
        TInt resoffset = FlashEngineRscHandler::LoadEngineResourceFileL();
        iLastErrorNote = R_FLASH_MMC_ACCESS_ERROR;      
        HBufC* errorNote = StringLoader::LoadLC( iLastErrorNote );      
        ShowErrorNoteL( *errorNote );
        CleanupStack::PopAndDestroy( errorNote );   
        pEngine->NotifyFlashError(aStatus, ETrue, ETrue);
        FlashEngineRscHandler::DeleteEngineResourceFile(pEngine->GetEikonEnv(),resoffset);
        }           
    }
//end of file

#ifdef __BUFFER_BEFORE_1ST_WRITE
void CUrlRequestInfo::WritePrivateDataL(const TDesC8& aDataPart)
	{		
	if(!iPrivateData)
		{
		iPrivateData = aDataPart.AllocL();		
		}
	else
		{
		TInt lengthToBeReAlloced = iPrivateData->Length() + (unsigned long)aDataPart.Length();		
		HBufC8* oldData = iPrivateData->AllocLC();		
		delete iPrivateData;
		iPrivateData = NULL;
		iPrivateData = HBufC8::NewL(lengthToBeReAlloced);
		TPtr8 dataPtr = iPrivateData->Des();
		dataPtr.Copy(*oldData);
		dataPtr.Append(aDataPart);
		CleanupStack::PopAndDestroy(oldData);
		}
	}
#endif //__BUFFER_BEFORE_1ST_WRITE	
TBool CUrlRequestInfo::HandleResponseDataL(const TDesC8& aDataPart)
{
	
	if(IsRTMPTClose())
	{
		return ETrue;
	}


	if( iReadOffset == iNumBytesDwlded )
    {       
        WriteStreamDataToCoreL(aDataPart);          
    }

	unsigned long Length = (unsigned long)aDataPart.Length();
	//if the stream has not been cancelled, push the write offset forward
    if( !iStreamCancelled )
        {
        iNumBytesDwlded += Length;    
        }
    
    // if the RAM storage is not being used, return without writing to it
    // RAM storage is currently used for flv content only

    if(!iUseRAMReservoir)   
        {
        return ETrue;
        }
#ifdef __DUMP_FLV_TO_FILE
    _LIT(KDebugFileName1, "C:\\Data\\orig.mp3");    
    RFile fileHandle;
    TInt error = fileHandle.Open( CEikonEnv::Static()->FsSession(), KDebugFileName1, EFileWrite );
    if( error == KErrNotFound )
        {
        error = fileHandle.Create( CEikonEnv::Static()->FsSession(), KDebugFileName1, EFileWrite );
        }
    
    TInt pos = 0;
    fileHandle.Seek(ESeekEnd, pos);
    fileHandle.Write( aDataPart );
    fileHandle.Close();
#endif
        
    // write to the RAM storage 
    WriteToRAMResL(aDataPart);

    /*if ( iNetBuf && ( iNetBuf->pBuffer == NULL ) )
            {           
                if(iReadOffset == iNumBytesDwlded)
                    {
                    //SetPausedBytes(Length);
                    }
                    
                return EFalse;
            }*/
        
        return ETrue;
        


}

void CUrlRequestInfo::HandleResponseCompleteL(TInt /*aHttpStatusCode*/) // request has completed normally
{
	if(IsRTMPTClose())
	{
		return;
	}
	CPlatformFlashEngine* pEngine = iMMObj->pHostObj->iFIInterface->GetEngine();
#ifdef _TIMER_MSG
	iTimer->Cancel();
#endif //_TIMER_MSG
	if(iReadOffset == iNumBytesDwlded)
	    {    
		//as broswer donesn't support http status ocde to be propogated to flash plugin, hence we are 
		// sending "200" for success transaction and '0' for all other cases.
	    if(pEngine)
			{
			pEngine->GetFIObject().FI_URLStreamClosed((TUint16)Id(), FI_ResultOK, 200);
			
			//Mp3Streaming
			if(iNetBuf && (iNetBuf->streamMediaType == FI_StreamMediaType_Sound))
	             TRAP_IGNORE(pEngine->MP3StreamClosedL());
			}
	    }

	iDownloadComplete = ETrue;
	
}
#ifdef FEATURE_FLV_MMC
void CUrlRequestInfo::WriteToRAMResL(const TDesC8& aData)
	{

    DEBUGPRINT("iRamoffset= %d readOffset = %d iNumbytesDownloaded =%d iRamcycles =%d",iRamOffset,iReadOffset,iNumBytesDwlded,iRAMCycles);
	if( ( !iRAMReservoir ) || ( iStreamCancelled ) )
	        {
	        return;
	        }
	        
	    TPtr8 dataPtr = iRAMReservoir->Des();
	    
	    // when there is no MMC we should make sure that the difference 
	    // between the read offset and the new write offset does not exceed 
	    // the size of the RAM storage
	    
	    if( ( !iMMCManager ) &&  (  ( iNumBytesDwlded - iReadOffset ) > iRAMResSize )  )
	        {
            DEBUGPRINT("error no space in memory");
	        iStreamCancelled = ETrue;
	        //We should not have keyword video in case of mp3Streaming
	        //Mp3Streaming 
	        
	        if(iNetBuf->streamMediaType == FI_StreamMediaType_Sound)
	            iLastErrorNote = R_FLASH_INSUFFICIENT_MEMORY_MMC_NEEDED;
	        else
	        
	        iLastErrorNote = R_FLASH_ERROR_INSERT_MEMORY_CARD;
	        return;
	        }
	        
	    if( iRAMCycles == 0)
	        {
	        // this is the first time data is written to the RAM storage
	        // so we can simply append the data
	        if( iRamOffset + aData.Length() <= iRAMResSize)
	            {                       
	            dataPtr.Append(aData);          
	            iRamOffset += aData.Length();
	            }
	        // the data length is such that it will overshoot the reservoir capacity
	        // so a turn around is required
	        else
	            {
	            // append the data till you reach reservoir size            
	            dataPtr.Append(aData.Left(iRAMResSize-iRamOffset));
	            //TInt dataToBeWritten = iRamOffset+aData.Length()-iRAMResSize;
	            TInt dataToBeWritten = aData.Length() - (iRAMResSize -iRamOffset);
	            //the RAM reservoir is full. commit to MMC storage if there is an MMC       
	            //check if this will overwrite unread data 
	            if(iMMCManager && iMMCManager->CheckMMCStatusL(iReadOffset, iRAMResSize))
	                {
	                TRAPD( err,iMMCManager->WriteToMMCL( *iRAMReservoir, iRAMResSize ) ); 
	                if( err )   // mmc access error, abort 
	                    {
	                    iStreamCancelled = ETrue; 
	                    iLastErrorNote = R_FLASH_MMC_ACCESS_ERROR ;
	                    return;
	                    }
	                }               
	            else if( iMMCManager )
	                {
	                iStreamCancelled = ETrue;//further writes should be disabled
	                //We should not have keyword video in case of mp3Streaming
	                //Mp3Streaming
	                if(iNetBuf->streamMediaType == FI_StreamMediaType_Sound)
	                    iLastErrorNote = R_FLASH_MMC_MEMORY_LOW;
	                else
	                iLastErrorNote = R_FLASH_ERROR_NOT_ENOUGH_MEMORY_VIDEO_TERMINATES ;
	                return; //break off 
	                }

	            //do a turn around and start writing from the beginning
	            
	            ClearRAMReservoir();
	            dataPtr.Replace(iRamOffset,dataToBeWritten,aData.Right(dataToBeWritten));
	            iRamOffset = dataToBeWritten;
	            }       
	        }
	    else
	        {
	        // the reservoir already contains data. 
	        // we try to replace data that is already read to the core player
	        
	        // make sure that we do not go beyond the reservoir capacity
	        if( iRamOffset + aData.Length() <= iRAMResSize)
	            {
	            
	            // a turn around in write is not required as we are not going  
	            // beyond the reservoir size. so simply replace with the new data
	            
	            dataPtr.Replace(iRamOffset, aData.Length(), aData);
	            iRamOffset += aData.Length();
	            
	            }   
	        else
	            {
	            // you can replace the data with out resetting the RAM offset till 
	            // we reach the reservoir size          
	            dataPtr.Replace( iRamOffset, iRAMResSize - iRamOffset, aData.Left( iRAMResSize - iRamOffset ) );
	            TInt dataToBeWritten = iRamOffset + aData.Length() - iRAMResSize;
	            
	            //check if this will overwrite unread data
	            if(iMMCManager && iMMCManager->CheckMMCStatusL(iReadOffset, iRAMResSize))
	                {
	                TRAPD( err,iMMCManager->WriteToMMCL( *iRAMReservoir, iRAMResSize ));  
	                if( err )   // mmc access error, abort 
	                    {
	                    iStreamCancelled = ETrue; 
	                    iLastErrorNote = R_FLASH_MMC_ACCESS_ERROR ;
	                    return;
	                    }

	                }
	            else if( iMMCManager )
	                {
	                iStreamCancelled = ETrue;//further writes should be disabled                
	                //We should not have keyword video in case of mp3Streaming
	                //Mp3Streaming
	                if(iNetBuf->streamMediaType == FI_StreamMediaType_Sound)
	                    iLastErrorNote = R_FLASH_MMC_MEMORY_LOW;
	                else
	                iLastErrorNote = R_FLASH_ERROR_NOT_ENOUGH_MEMORY_VIDEO_TERMINATES;
	                return; //break off 
	                }
	            // since the reservoir is full we need to clear it  
	            ClearRAMReservoir();
	            //do a turn around and start writing from the beginning
	            
	            dataPtr.Replace( iRamOffset, dataToBeWritten, aData.Right( dataToBeWritten ) );
	            iRamOffset = dataToBeWritten;
	            }       
	        
	        }

	}
void CUrlRequestInfo::ClearRAMReservoir()
    {
    // the reservoir is full
    // another write cycle is complete
    iRAMCycles++;
    // offset is reset as we want to write from the beginning
    iRamOffset = 0;
    }


TBool CUrlRequestInfo::IsRAMResFull()
	{
	if(!iRAMReservoir)
		{
		return EFalse;
		}
	return (iRAMReservoir->Length() == iRAMResSize);
	}

void CUrlRequestInfo::ClearReservoir()
	{
	iRAMCycles++;
	iRamOffset =0;
	}
TBool CUrlRequestInfo::IsDataInRAMRes(TInt aOffset)
	{
	if( aOffset > ( iRAMCycles*iRAMResSize ) && aOffset < iNumBytesDwlded )
		{
		return ETrue;
		}
	else
		{
		return EFalse;
		}	
	}
TBool CUrlRequestInfo::SeekURLStreamFLV(unsigned int seekTime, FI_NetworkBufferInfo* aNetworkBufferInfo)
	{
	iNetBuf = aNetworkBufferInfo;
	if(seekTime > iNumBytesDwlded)
		{
		return EFalse;
		}
	if( iMMCManager && iMMCManager->IsDataInMMC(seekTime) == EFalse || !iMMCManager)
		{
		if(!IsDataInRAMRes(seekTime))
			return EFalse;
		}
		
	iReadOffset = seekTime;
	
	TriggerAsyncReadWrite();
	if(iMMCManager)
		{
		iMMCManager->CancelAllPendingRequests();
		}
	
	return ETrue;
	}
	
void CUrlRequestInfo::TriggerAsyncReadWrite()
	{
	if(!iMMCAsyncObject->IsActive())
		iMMCAsyncObject->After(2);
	}
	
TInt CUrlRequestInfo::ReadFromRAMResL(TDes8& aData,TInt aOffset, TInt aLength)
	{
	/*if(!iRAMReservoir)
		{
		return 0;
		}
	if( aOffset+aLength < iRamOffset )
		{
		aData.Copy( iRAMReservoir->Mid(aOffset,aLength) );
		return aLength;
		}
	else if( aOffset < iRamOffset )
		{
		aData.Copy(iRAMReservoir->Mid( aOffset,( iRAMReservoir->Length() - aOffset )));
		return iRAMReservoir->Length();
		}
	else
		{
		return 0;
		}	*/
	   if(!iRAMReservoir)
	        {       
	        return 0;
	        }
	    if( aOffset > iRamOffset && iRAMCycles )
	        {
	        //check if length of the data to be read is such that a turn around is not required
	        //iRAMResSize - iRamOffset is the part of the data that is not yet over-written
	   	        if( aLength <= iRAMResSize - aOffset )
	            {
	           
	            aData.Copy( iRAMReservoir->Mid(aOffset, aLength));
	             
	            return aLength;
	            }
	        else
	            {
	           
	            // read the data that can be read from aOffset directly
	            aData.Copy( iRAMReservoir->Right( iRAMResSize - aOffset ));
	            
	            //turn around and read from the beginning and append
	            
	            aData.Append( iRAMReservoir->Left(Min((aLength -(iRAMResSize - aOffset)), iRamOffset )) );
	           
	            return aData.Length();
	            }   
	        }
	    else
	        {
	        // read and write are in the same cycle
	            if(iRamOffset > aOffset)
	                {
	                         
	                aData.Copy( iRAMReservoir->Mid( aOffset, Min( aLength, iRamOffset - aOffset ) ) );
	           
	                return aData.Length();              
	                }
	            else
	                {
	                     return 0;
	                }   

	        }   
	}
#endif //FEATURE_FLV_MMC

void CUrlRequestInfo::WriteStreamDataToCoreL(const TDesC8& aData)
	{
	    if (iNetBuf == NULL || iNetBuf->pBuffer == NULL)
	        {
	        return;
	        }   
			
        CPlatformFlashEngine* pEngine = iMMObj->pHostObj->iFIInterface->GetEngine();
	    // the number of bytes we are attemping to write to the core player 
	    unsigned long bytesToWrite = aData.Length();
	   
	    
	    
	    const unsigned char* dataToWrite;

	    dataToWrite = aData.Ptr();
	    

	// Continue writing until there's no more data
	    while (bytesToWrite > 0)
	        {
	        if ( ( iNetBuf == NULL ) || ( iNetBuf->pBuffer == NULL ) )
	            {
	            //break off now as the has not provided a core buffer
	          
	            return;
	            }           
	        // write only as much data as the core can take at once
	        iNetBuf->dataSize = 
	                    (bytesToWrite > iNetBuf->maxBufSize) ? 
	                        iNetBuf->maxBufSize : bytesToWrite;
	        // copy data to the core's buffer
	        FlashMemCpy(iNetBuf->pBuffer, dataToWrite, iNetBuf->dataSize);
	        
	        //advance the read offset 
	        iReadOffset +=iNetBuf->dataSize;
	        // need to advance the dataToWrite pointer
	        if (bytesToWrite > iNetBuf->maxBufSize)
	            {
	            dataToWrite = dataToWrite + iNetBuf->maxBufSize;
	            bytesToWrite -= iNetBuf->maxBufSize;
	            }
	        else
	            {
	            dataToWrite = NULL;
	            bytesToWrite = 0;
	            }       
	        //Camp : Changed
	        //iNetworkBufferInfo = pEngine->GetFIObject().FI_URLStreamDataLoaded(iNetworkBufferInfo->streamId, iNetworkBufferInfo );
	        iNetBuf = pEngine->GetFIObject().FI_URLStreamDataLoaded((TUint16)Id(), iNetBuf );
	        // if read offset is equal to or exceeds the write offset, and if all the data has been downloaded,
	        // indicate to the core that the stream can be closed
	        if( ( iDownloadComplete  ) && iReadOffset >= iNumBytesDwlded)
	            {
	            //if(!iCancelState)
	                {
	        		//as broswer donesn't support http status ocde to be propogated to flash plugin, hence we are 
	        		// sending "200" for success transaction and '0' for all other cases.
	                pEngine->GetFIObject().FI_URLStreamClosed((TUint16)Id(), FI_ResultOK,200);
	                if(iNetBuf)
	                    {
                        if(iNetBuf->streamMediaType == FI_StreamMediaType_Sound)
	                                 TRAP_IGNORE(pEngine->MP3StreamClosedL());
	                    }
	                }
	            }
	           
	        }
	
	}
//Sathya
#ifdef FEATURE_FLV_MMC
unsigned int CUrlRequestInfo::GetBytes()
{	
	return iNumBytesDwlded;
}

unsigned int CUrlRequestInfo::GetTotalBytes()
{   
  if (iDownloadComplete)  
      return iNumBytesDwlded;
  else
      return 0;
}
#endif //FEATURE_FLV_MMC

void CUrlRequestInfo::ReadCompleteL( TInt aStatus, const TDesC8& aData )
	{
	if( aStatus == KErrNone )
		{
		WriteStreamDataToCoreL( aData );
		TriggerAsyncReadWrite();
		}
	
	}
	
