/*
* Copyright (c) 2007 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that is used to 
*				get user agent for the device
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "symbian_useragent.h"
#include "engine_defs.h"
#include "version.h"

//browser headers
#include <cuseragent.h>

_LIT8(KDefaultUserAgent, "Mozilla/4.78 (Symbian OS; U) ");

_LIT8(KLitFlashPlayer," Flash Player ");
_LIT8(KLitVersionDot,".");

CUserAgentInfo* CUserAgentInfo::NewL()
{
	CUserAgentInfo* self=new(ELeave) CUserAgentInfo();
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

CUserAgentInfo::CUserAgentInfo(): 
			iUserAgentBuf(NULL)
 			,iUsrAgent(NULL,0)
{

}


void CUserAgentInfo::ConstructL()
{

}


CUserAgentInfo::~CUserAgentInfo()
{
	if(iUserAgentBuf)
	{
		delete iUserAgentBuf;
		iUserAgentBuf = NULL;
	}
	
}
					
void CUserAgentInfo::MakeUserAgentInfoL()
{

	CUserAgent* uagnt = CUserAgent::NewL();

	CleanupStack::PushL(uagnt);
		
	HBufC8* uagntBuf = uagnt->UserAgentL();
	
	if(uagntBuf)
	{ //we have user agent filled
		CleanupStack::PushL(uagntBuf);	
		
		iUserAgentBuf = HBufC8::NewL(uagntBuf->Length() + 80);//add another 80
		
		iUsrAgent.Set(iUserAgentBuf->Des()); //set the buf ptr
			
		iUsrAgent.Copy(uagntBuf->Des());
		
		CleanupStack::PopAndDestroy(uagntBuf);//destroy uagntBuf
	
	}
	else
	{
		iUserAgentBuf = HBufC8::NewL(300);//have size of 300
		
		iUsrAgent.Set(iUserAgentBuf->Des()); //set the buf ptr
		
		iUsrAgent.Copy(KDefaultUserAgent());//have default filled
	}
	
	iUsrAgent.Append(KLitFlashPlayer());	
	iUsrAgent.AppendNum(MAJOR_VERSION);
	iUsrAgent.Append(KLitVersionDot());
	iUsrAgent.AppendNum(MINOR_VERSION);
	
	CleanupStack::PopAndDestroy(uagnt);//destroy uagnt

}


const TDesC8& CUserAgentInfo::GetUserAgentL()
{
	if(!iUserAgentBuf)
	{
		//form user agent string
		MakeUserAgentInfoL();			
	}

	return iUsrAgent;
}

//End of file

