/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class used to enhance 
*				 the interface of the Symbian CVibraControl class.
*
*/

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "saplayer_build.h"
#ifdef FEATURE_VIBRATION

#include "symbian_vibra.h"
#include "commondef.h"
#include "engine_defs.h"

CVibrationHandler::CVibrationHandler()
{
	iOnTime = 0;
	iRepeats = 0;
	iTimer = NULL;
	iVibraCtrl = NULL;
}

CVibrationHandler::~CVibrationHandler()
{
	if (iTimer)
	{
		iTimer->Cancel();
		delete iTimer;
	}
	delete iVibraCtrl;
}

TInt16 CVibrationHandler::StartVibrate(TUint16 aTimeOn, TUint16 aTimeOff, TUint16 aRepeat)
{
	if (!iVibraCtrl)
	{
		TRAPD(err, iVibraCtrl = CHWRMVibra::NewL());
		if (err != KErrNone)
			return SI_GenericError;
	}



	CHWRMVibra::TVibraModeState state = iVibraCtrl->VibraSettings();
	if (state != CHWRMVibra::EVibraModeON)
	{
		return SI_GenericError;
	}
	StopVibrate();
	TUint16 duration = aTimeOn;
	if ((aTimeOff > 0) && (aRepeat > 0))
	{
		// we only need a timer if we have to deal with pulsing vibration
		iRepeats = aRepeat;
		iOnTime = aTimeOn;
		TTimeIntervalMicroSeconds32 delay = (aTimeOn + aTimeOff) * KMicrosecondsInMillisecond;
		TRAPD(err, iTimer = CPeriodic::NewL(CActive::EPriorityUserInput));
		if (err!=KErrNone)
			return SI_GenericError;
		iTimer->Start(delay, delay, TCallBack(StaticTimerTick, (TAny*)this));	
	}
	else if (aRepeat > 0)
	{
		// if we have no pulsing vibration, we can simply multiply the on-time by the repeat count
		duration = static_cast<TUint16>(aTimeOn * aRepeat);
	}

	if (iVibraCtrl)
	{
		TRAPD(err, iVibraCtrl->StartVibraL(duration));
		iRepeats--;
		if (err != KErrNone)
		{
			return SI_UnsupportedError;
		}
	}
	return SI_NoError;
}

TInt16 CVibrationHandler::StopVibrate()
{
	if (iVibraCtrl)
	{
		TRAPD(err, iVibraCtrl->StopVibraL());
		if (err != KErrNone)
		{
			return SI_UnsupportedError;
		}
	}
	if (iTimer)
	{
		iTimer->Cancel();
		delete iTimer;
		iTimer = NULL;
	}
	return SI_NoError;
}

void CVibrationHandler::Tick()
{
	if (iRepeats--)
	{
		if (iVibraCtrl)
		{
			TRAP_IGNORE(iVibraCtrl->StartVibraL(iOnTime));
		}
	}
	else
	{
		iTimer->Cancel();
		delete iTimer;
		iTimer = NULL;
	}
}

TBool CVibrationHandler::StaticTimerTick(TAny* aVibrationHandler)
{
	CVibrationHandler* self = (CVibrationHandler*)aVibrationHandler;
	if (self)
	{
		self->Tick();
	}
	return EFalse;
}


#endif // FEATURE_VIBRATION
