/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class that works with 
*				 CVideoEngine to support the Flash device video API.
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformtypes.h"
#include "platformclib.h"
#include "platformdebug.h"
#include "saplayer_build.h"

#ifdef FEATURE_DEVICE_VIDEO

#include "symbian_engine.h"
#include "symbian_videoobject.h"
#include "symbian_videoengine.h"
#include "symbian_file.h"
#include "saflash_consts.h"
#include "symbian_ui_interface.h"
#include "symbian_stringutils.h"
#include "engine_data_caging_literals.hrh"
#include "symbian_sound.h"
#include "symbian_plugin_interface.h"
#include "symbian_mediarecognizer.h"
#include <flash2engine.rsg>

#include <eikenv.h>
#include <videoplayer.h>
#ifdef USE_VPU2
#include <videoplayer2.h>
#endif
#include <mmferrors.h>
#include <DocumentHandler.h>
#include <f32file.h>
#include <collate.h>

//#include <apsettingshandlerui.h>
#include <hal.h>
#include <AknUtils.h> // for AknTextUtils
#include "flashengineresourcehandler.h"

/*#ifdef //Log_MESSAGE
_LIT8(KLitCr,"\n");
void WriteLogL(CVideoEngine& aVidEng,const TDesC8& aMsg, TInt aNum)
{
	TBuf8<256> msg;
	msg.Append(aMsg);
	msg.AppendNum(aNum);
	msg.Append(KLitCr);
	aVidEng.GetFlashEngine().WriteLogMsgL((const char*)msg.PtrZ());
}
#endif // //Log_MESSAGE
*/

// class CVideoObject

CVideoObject::CVideoObject(CVideoEngine& aVideoEngine) :
#ifdef ASYNC_FILE_WRITE
	CActive(CActive::EPriorityStandard)
#endif //ASYNC_FILE_WRITE
	, iVideoEngine(aVideoEngine)
	, iId(0xFFFFFFFF)
	, iState(ENotInitialized)
    , iPlayer(NULL)
	, iLoop(EFalse)
#ifdef USE_VPU2	
	, iCurrentDisplayWindow(NULL)
#endif	
{
	#ifdef ASYNC_FILE_WRITE
	CActiveScheduler::Add(this);
	#endif // ASYNC_FILE_WRITE
	
	iCurrentLocation.xmin = 0;
    iCurrentLocation.ymin = 0;
    iCurrentLocation.xmax = 0;
    iCurrentLocation.ymax = 0;    
}


CVideoObject::~CVideoObject()
{
    Reset();
	
    /* Changes to fix the issue with caling 
     * video.Close() and video.Play() again and again */
#ifdef USE_VPU2        
    iVideoEngine.GetFlashEngine().GetDisplay()->CloseVideoWindow((void*)this);
    iParentWindow = NULL;
    iCurrentDisplayWindow = NULL;
#endif 

    if(iTimer)
    	{
    	iTimer->Cancel();
    	}
	if (iTempFile)
	{
		iTempFile->Close();
		delete iTempFile;
	}

	delete iUrlToOpen;
	iIPConnectionMgr = NULL;
	if(iConnHandler)  //delete iconnectionhandler so that IAP can be released
		{
		delete iConnHandler;
		iConnHandler = NULL;
		}
}

void CVideoObject::Reset()
{

	#ifdef ASYNC_FILE_WRITE
    Cancel();
	#endif // ASYNC_FILE_WRITE

	#ifdef USING_TEMP_FILE_FOR_OPENDESL
    if (iTempFileName.Length() > 0)
    {
        iVideoEngine.GetFlashEngine().GetFileUtils()->DeleteFile(iTempFileName);
    }
	#endif	// USING_TEMP_FILE_FOR_OPENDESL
    
    if (iPlayer)
    {
        TRAP_IGNORE(StopL()); // cannot leave here	
        iPlayer->Close();
        delete iPlayer;
        iPlayer = NULL;
        
        iState = EClosed;
    }

	iNeedsToStop = EFalse;
	iIsStreaming = EFalse;
	if(iParentWindow)
        iParentWindow->Invalidate();
}

TBool CVideoObject::InitPlayerL(CPlatformFlashEngine* aEngine, const FI_Rect& aLocation,TInt aInitialVolume)
{
    TRect clientRect;
    clientRect = aEngine->GetDisplay()->ClientRect();
    
    /*
    if((iVideoEngine.GetFlashEngine().IsBrowserPluginActive()))
    {
        clientRect = aEngine->GetPlugin()->GetRect();
    }
    else
    {
        clientRect = aEngine->GetUI()->ClientRect();
    }
    */
    iIPConnectionMgr = aEngine->GetIPConnectionMgr();

	TPoint clipRectTopLeft( aLocation.xmin + clientRect.iTl.iX, aLocation.ymin + clientRect.iTl.iY );
	TSize clipRectSize( aLocation.xmax-aLocation.xmin, aLocation.ymax-aLocation.ymin );
	TRect newLocation( clipRectTopLeft, clipRectSize );

	TBool result = newLocation.Intersects( clientRect );

	// Create the player
	iCoeEnv = CCoeEnv::Static(); // Getting this is expensive.
#ifdef USE_VPU2
	iParentWindow = aEngine->GetDisplay()->GetNewVideoWindow((void *)this);
#else	
	iParentWindow = aEngine->GetDisplay()->GetWindow();
#endif	
	/*
	if(!(iVideoEngine.GetFlashEngine().IsBrowserPluginActive()))
	{
	    iParentWindow = aEngine->GetUI()->GetWindow();    
	}
	else
	{
	    iParentWindow = &iVideoEngine.GetFlashEngine().GetPlugin()->GetWindow();
	}
	 */
	
	// construct the video rect based on the current screen rect
	UpdateRectL(aLocation);

	iState = EInitializing;
	iUserVolume=aInitialVolume; // remember now, and set track volume when we start playing


	return ETrue;  // TODO : Check this. 
}

void CVideoObject::SetRectL(const FI_Rect& aLocation, TBool aForceUpdate)
{
    PLAYERASSERT(iState != ENotInitialized);
 
    if(!aForceUpdate &&
       iCurrentLocation.xmin == aLocation.xmin &&
       iCurrentLocation.ymin == aLocation.ymin &&
       iCurrentLocation.xmax == aLocation.xmax &&
       iCurrentLocation.ymax == aLocation.ymax)
    {
        return;
    }
    
    RWsSession&			wsSession		= iCoeEnv->WsSession();
    CWsScreenDevice*	screenDevice	= iCoeEnv->ScreenDevice();
    
    // Unlock global bitmap heap before calling SetDisplayWindow( ) 
    TBool lockLater = EFalse;            
    if(iVideoEngine.GetFlashEngine().IsBitmapLocked())
    {
        TAny* dummyArgs = NULL;
        iVideoEngine.GetFlashEngine().UnlockFrameBufferBits(dummyArgs);
        lockLater = ETrue;
    }
    
    // construct the video rect based on the current screen rect
	UpdateRectL(aLocation);

    if(iPlayer)
    {  
        // The region where the video is drawn is the intersection between the iRect and the clipRect
        TRect clipRect = iRect;
        if((iVideoEngine.GetFlashEngine().IsBrowserPluginActive()))
        {
            clipRect = iVideoEngine.GetFlashEngine().GetDisplay()->GetClipRect();
        }
#ifndef USE_VPU2
        TRAP_IGNORE(iPlayer->SetDisplayWindowL(wsSession, *screenDevice, *iParentWindow, iRect, clipRect)); 
#else
        TPoint winAbsPosition(iParentWindow->AbsPosition());
        
        TRect relativeWinRect(iRect);
        relativeWinRect.Move(-winAbsPosition.iX,-winAbsPosition.iY);
        TRect relativeClipRect(clipRect);
        relativeClipRect.Move(-winAbsPosition.iX,-winAbsPosition.iY);            
        
        if(iCurrentDisplayWindow != iParentWindow)
        {
            if(iCurrentDisplayWindow)
                iPlayer->RemoveDisplayWindow(*iCurrentDisplayWindow);
        
            TRAP_IGNORE(iPlayer->AddDisplayWindowL(wsSession,*screenDevice,*iParentWindow,
                 relativeWinRect,relativeClipRect));
            
            iCurrentDisplayWindow = iParentWindow;
        }
        else
        {
            TRAP_IGNORE(iPlayer->SetVideoExtentL(*iParentWindow,relativeWinRect));    
            
            TRAP_IGNORE(iPlayer->SetWindowClipRectL(*iParentWindow,relativeClipRect));              
        } 
        
        TRAP_IGNORE(iPlayer->SetAutoScaleL(*iParentWindow, EAutoScaleBestFit));
#endif        
                
        // Refresh only if playing, paused or finished playing 
        // (last frame is displayed when we are in EDone state. That needs to scroll)
        if ( iState == EPlaying || iState == EPaused || iState == EDone )
            TRAP_IGNORE(iPlayer->RefreshFrameL());         
    }
        
    if(lockLater)
    {
    	iVideoEngine.GetFlashEngine().LockFrameBufferBits();
    } 
    
    iCurrentLocation = aLocation;
}
    

TSize CVideoObject::GetVideoSize()
{
    TSize size(0,0);
    if(iPlayer) {    
        TRAP_IGNORE(iPlayer->VideoFrameSizeL(size));
    }
    /*
    if(size.iWidth == 0 && size.iHeight == 0)
    {    
        size.iWidth = iRect.Width();
        size.iHeight = iRect.Height();
    }*/
        
    return size;
}


void CVideoObject::Resume()
{
    if(iPlayer && (iState == EPaused || iState == EStopped))
    {
        //iPlayer->Play();
        //iState = EPlaying;
    
		if(!IsStreamSoundPlaying())
			{
				iVideoEngine.SetActiveVideoID(Id());
		        iPlayer->Play();
	        	iState = EPlaying;
	        	iVideoEngine.GetFlashEngine().ExtendBacklightForVideoPlayback(EOn);
			}
    }
}

// This function can be used/modified to map video to pre-installed reference video files
// To use, change the pattern matching code to check for known references 
// (e.g. <AlertVideo>, <StartVideo> etc.)
// Also change the file path to be one that maps into the given reference name

HBufC* CVideoObject::GetReferenceFilePathLC(const TDesC& aUrl)
{
	HBufC* filePath = NULL;

	const TInt lenOfName=KSchemeReference().Length()-aUrl.Length();

	if (ComparePrefix(aUrl, KSchemeReference) && lenOfName>0)
	{
		//TPtrC videoId = aUrl.Mid(KSchemeReference().Length());
		// For now, just return a reference to a hardcoded video file
		// Eventually, the mapping between videoId and filename could
		// be stored in the engine resource file.
		const TPtrC refVidId(aUrl.Right(lenOfName));
		TInt resoffset = FlashEngineRscHandler::LoadEngineResourceFileL();
		filePath = iVideoEngine.GetFlashEngine().MatchedStringFromArrayL(R_FLASH_REFERENCE_VIDEO_ARRAY,refVidId).AllocLC();
		FlashEngineRscHandler::DeleteEngineResourceFile(CEikonEnv::Static(), resoffset);
	}
	return filePath;
}

_LIT(KCharFwdSlash, "/");
_LIT(KCharBar, "|");

HBufC* CVideoObject::GetFilePathLC(const TDesC& aUrl)
{
	// check to see if url contains file://
	TPtrC tempURL;
	TInt pos = aUrl.FindF(KSchemeFile);
	if (pos >= 0)
	{
		tempURL.Set(aUrl.Mid(pos + KSchemeFile().Length()));
	}
	else
	{
		tempURL.Set(aUrl);
	}

	// Get rid of the extra /
	while(tempURL[0] == KCharForwardSlash)
	{
		tempURL.Set(tempURL.Mid(1));
	}

	TFileName fileName;
	fileName.Copy(tempURL);

	// Replace "/" with "\\"
	AknTextUtils::ReplaceCharacters(fileName, KCharFwdSlash, TChar(KPathDelimiter));
	AknTextUtils::ReplaceCharacters(fileName, KCharBar, TChar(KDriveDelimiter));

	HBufC* filePath = fileName.AllocLC();

	return filePath;
}

TBool CVideoObject::EnsureDirectory(const TDesC& aDirectory) 
{
	TUint lAttValue = 0;

	// Check to see if the directory already exists
	TInt err = iVideoEngine.GetRFs().Att(aDirectory, lAttValue);

	if (err != KErrNone || !(lAttValue & KEntryAttDir) )
	{
		// Create all sub-directory hierarchy
		err = iVideoEngine.GetRFs().MkDirAll(aDirectory);
	}
	return (err == KErrNone) ? ETrue : EFalse;
}
#ifdef USING_TEMP_FILE_FOR_OPENDESL

// This is a work-around solution until Nokia can support playing back video buffer
// using OpenDesL() API.  Basically, we save the video buffer to a file, then use
// OpenFileL() to perform the playback.

#ifdef ASYNC_FILE_WRITE

void CVideoObject::PlayBufferFromFileL(const CVideoRequest& aVideoRequest)
{
	// We want to write the file asynchronously
	// Currently Nokia doesn't support OpenDesL for buffers >300k so we need to save it
	// to a file then call OpenFileL instead.
	RThread currentThread;
	_LIT(KDoubleSlash,"\\");
	
	//Returns the Uid of Application.
	TUid Identi = User::Identity();

	TUidName uidName = Identi.Name();
	//Deleting '[' from the Uid name.
	uidName.Delete(0,1);
	//Deleting ']' from the Uid name.
	uidName.Delete(uidName.Length()-1,1);
	
	iTempFileName.Copy(KPersistentSubdirName);
	iTempFileName.Append(uidName);
	iTempFileName.Append(KDoubleSlash);
	
	EnsureDirectory(iTempFileName);
	
	TFileName tempFileName;
	tempFileName.Format(KTempVideoFileNameT, (TUint)currentThread.Id(), aVideoRequest.iVideoID);
	iTempFileName.Append(tempFileName);
	
	void* data = (void*) aVideoRequest.iVideoData.Ptr();
	unsigned long dataLen = aVideoRequest.iVideoData.Length();

	/*TBool recognized = *///MediaRecognizer::CreateFilenameFromData(iTempFileName, data, dataLen, MediaRecognizer::EVideo);

	TBool recognized= MediaRecognizer::CreateFilenameFromData(iTempFileName
												, data
												, dataLen
												, MediaRecognizer::EVideo);
												
	 
	if(!recognized)
	{ 
		//if our media recog does not recognize then try MMF headers
		
		recognized = MediaRecognizer::CreateFilenameFromMMFDataL(&(iVideoEngine.GetFlashEngine())
												,iTempFileName
												,data
												,dataLen);
	}
	
	// Here recognizers are not able to find the format then also send the file to MMf
	// let us see how MMF behaves
	
	if (iTempFile)
	{
		iTempFile->Close();
		delete iTempFile;
		iTempFile = NULL;
	}

	iTempFile = new(ELeave) RFile;
	User::LeaveIfError(iTempFile->Replace(iVideoEngine.GetRFs(), iTempFileName, EFileWrite | EFileShareExclusive | EFileStream));

	// Write the data to the file
	SetActive();
	iTempFile->Write(aVideoRequest.iVideoData, iStatus);
}

void CVideoObject::RunL()
{
	if (iStatus.Int() == KErrNone)
	{
    iTempFile->Close();
    User::LeaveIfError(iTempFile->Open(iVideoEngine.GetRFs(), iTempFileName, EFileRead | EFileShareReadersOnly | EFileStream));
    	
	//iPlayer->OpenFileL(iTempFileName,controllerUid);
	iPlayer->OpenFileL(*iTempFile);
	
#ifdef FF_FLASHLITE_SCREENSAVER 
		if (iVideoEngine.GetFlashEngine().IsBrowserPluginActive())
		{
		   // if(iVideoEngine.GetFlashEngine().GetPlugin()->IsScreenSaverActive())
		   	 if(iVideoEngine.GetFlashEngine().NoSoundSupport()) //no screensaver specifc
		    {
		        iPlayer->SetVolumeL(0);       
		    }
		}				
#endif //FF_FLASHLITE_SCREENSAVER		
		iState = ELoading;
		iIsStreaming = EFalse;
		
		if (iTempFile)
			{
			iTempFile->Close();
			delete iTempFile;
			iTempFile = NULL;
			}
	}
	else
	{
		Reset();	        		
		iVideoEngine.GetFlashEngine().SetVideoRectL(NULL, EVideoNone);
	}
	
}

void CVideoObject::DoCancel()
{
	if (iTempFile)
	{
		iTempFile->Close();
		delete iTempFile;
		iTempFile = NULL;
	}
}

TInt CVideoObject::RunError(TInt aError)
{
	return aError;
}

#else	// ASYNC_FILE_WRITE

// This function write the current video buffer to a temp file and play back the video from the file
void CVideoObject::PlayBufferFromFileL(const FI_Video& aVideoInfo)
{
	// Currently Nokia doesn't support OpenDesL... so we need to save it to a file
	// Then call OpenFileL insteads
	RFile tempFile;
	RThread currentThread;

	_LIT(KDoubleSlash,"\\");
	
	//Returns the Uid of Application.
	TUid Identi = User::Identity();

	TUidName uidName = Identi.Name();
	//Deleting '[' from the Uid name.
	uidName.Delete(0,1);
	//Deleting ']' from the Uid name.
	uidName.Delete(uidName.Length()-1,1);
	
	iTempFileName.Copy(KPersistentSubdirName);
	iTempFileName.Append(uidName);
	iTempFileName.Append(KDoubleSlash);
	
	EnsureDirectory(iTempFileName);
	
	TFileName tempFileName;
	tempFileName.Format(KTempVideoFileNameT, currentThread.Id(), aVideoInfo.videoID);
	iTempFileName.Append(tempFileName);
	
	void* data = (void*) aVideoRequest.iVideoData.Ptr();
	unsigned long dataLen = aVideoRequest.iVideoData.Length();

	/*TBool recognized = */MediaRecognizer::CreateFilenameFromData(iTempFileName, data, dataLen, EVideo)

	TInt err = tempFile.Replace(iVideoEngine.GetRFs(), iTempFileName, EFileWrite | EFileShareExclusive | EFileStream);

	if (err == KErrNone)
	{
		// Write the data to the file
		TPtrC8 videoData((const TUint8*)aVideoInfo.videoData, aVideoInfo.videoDataSize);
		err = tempFile.Write(videoData);
		
		if (err == KErrNone)
		{
			// Close the current file
			tempFile.Flush();
			tempFile.Close();
				
#ifdef LOG_MESSAGE
			CString msg("CVideoObject::PlayBufferFromFileL> OpenFileL.. file=");
			char* cFileName = iVideoEngine.GetFlashEngine().CreateFlashString(iTempFileName);
			msg.AppendString(cFileName);
			msg.AppendString("\n");
			TRAPD(err, iVideoEngine.GetFlashEngine().WriteLogMsgL(msg.CStr()));
			delete[] cFileName;
#endif	// LOG_MESSAGE

			// Now tell the player to use this temp file
			TRAPD(unsupported, iPlayer->OpenFileL(iTempFileName));
#ifdef FF_FLASHLITE_SCREENSAVER 
    		if (iVideoEngine.GetFlashEngine().IsBrowserPluginActive())
	    	{
		        if(iVideoEngine.GetFlashEngine().NoSoundSupport())
		        {
		            iPlayer->SetVolumeL(0);       
		        }
		    }				
#endif //FF_FLASHLITE_SCREENSAVER		

			if(unsupported != KErrNone)
			{
				iPlayer->Close();
                iState = EClosed;
			}
            else
            {
			iState = ELoading;
            }
		}
	}
}
#endif	// ASYNC_FILE_WRITE

#endif	// USING_TEMP_FILE_FOR_OPENDESL
TBool CVideoObject::IsStreamSoundPlaying()
{
	CSoundHandler* soundHandler = iVideoEngine.GetFlashEngine().SoundHandler();
	if (soundHandler && soundHandler->IsPlaying())
	{
		return ETrue;
	}

	return EFalse;
}

#ifdef USING_TEMP_FILE_FOR_OPENDESL
//const TUint32 KMaxVideoDescriptorSize = 300 << 10; // 300 kBytes
// for 2.8 and later devices, we can play up to 300kB from a descriptor.
// for 2.6 and earlier, it doesn't work at all.
const TInt32 KMaxVideoDescriptorSize = 0; // zero bytes
#endif


void CVideoObject::PlayL(const CVideoRequest& aVideoRequest)
{
	// Sanity check.. better match...
	PLAYERASSERT( aVideoRequest.iVideoID == iId );
    PLAYERASSERT(iParentWindow);

	if (IsStreamSoundPlaying())
	{
		// cannot play a separate sound file and video file at the same time
		User::Leave(KErrNotReady);
	}

    if(iState == ENotInitialized)
    {
        return;
    }
    else
    {
        Reset();
    }

    if(!iPlayer)
    {
        
#ifndef USE_VPU2        
        iPlayer = CVideoPlayerUtility::NewL(*this,
                                             EMdaPriorityNormal, 
                                             EMdaPriorityPreferenceNone,
                                             iCoeEnv->WsSession(), 
                                             *screenDevice, 
                                             *iParentWindow,
                                             iRect, 
                                             iRect);
#else
        iPlayer = CVideoPlayerUtility2::NewL(*this,
                                                static_cast<enum TMdaPriority>(EMdaPriorityNormal),
                                                static_cast<enum TMdaPriorityPreference>(EMdaPriorityPreferenceNone)); 
#endif
    }

	// inform engine of video rect, so we can avoid drawing in that area.
	iVideoEngine.GetFlashEngine().SetVideoRectL(&iLocation, EDeviceVideo);

    TInt protocol = PROTOCOL_SYMBOL;

	if (aVideoRequest.iPlaySrc)
	{
		protocol = GetProtocolFromURL(*aVideoRequest.iPlaySrc);
	}
	
	if ( (protocol == PROTOCOL_SYMBOL) || (aVideoRequest.iVideoData.Size() > 0) )
	{
		#ifdef USING_TEMP_FILE_FOR_OPENDESL
		if (aVideoRequest.iVideoData.Size() > KMaxVideoDescriptorSize)
		{
			PlayBufferFromFileL(aVideoRequest);
		}
		else
		#endif //USING_TEMP_FILE_FOR_OPENDESL
		{
			TRAPD(unsupported, iPlayer->OpenDesL(aVideoRequest.iVideoData));
            
			if(unsupported != KErrNone)
			{
				iPlayer->Close();
			}
			
			#ifdef FF_FLASHLITE_SCREENSAVER 
    		if (iVideoEngine.GetFlashEngine().IsBrowserPluginActive())
	    	{
		        if(iVideoEngine.GetFlashEngine().NoSoundSupport())
		        {
		            iPlayer->SetVolumeL(0);       
		        }
		    }				
			#endif //FF_FLASHLITE_SCREENSAVER					
		}
	}
	else	// Either file:// or http://
	{
		switch (protocol)
		{
			case PROTOCOL_REFERENCE:
			{
				HBufC* filePath = GetReferenceFilePathLC(*aVideoRequest.iPlaySrc);

				if (filePath)
				{
					iPlayer->OpenFileL(*filePath);
					
					#ifdef FF_FLASHLITE_SCREENSAVER 
            		if (iVideoEngine.GetFlashEngine().IsBrowserPluginActive())
            		{
            		    if(iVideoEngine.GetFlashEngine().NoSoundSupport())
            		    {
            		        iPlayer->SetVolumeL(0);       
            		    }
            		}				
					#endif //FF_FLASHLITE_SCREENSAVER		
					
					iState = ELoading;

					CleanupStack::PopAndDestroy(filePath);
				}
			}
			break;

			case PROTOCOL_FILE:
			{
				// Check for reference to a local video file
				HBufC* filePath = GetFilePathLC(*aVideoRequest.iPlaySrc);

				if (filePath)
				{
					TRAPD(unsupported, iPlayer->OpenFileL(*filePath));

					#ifdef FF_FLASHLITE_SCREENSAVER 
            		if (iVideoEngine.GetFlashEngine().IsBrowserPluginActive())
            		{
            		    if(iVideoEngine.GetFlashEngine().NoSoundSupport())
            		    {
            		        iPlayer->SetVolumeL(0);       
            		    }
            		}				
					#endif //FF_FLASHLITE_SCREENSAVER		

					if(unsupported != KErrNone)
					{
						iPlayer->Close();
					}
                    else
                    {
						iState = ELoading;
                    }

					CleanupStack::PopAndDestroy(filePath);
				}
			}
			break;
			
			case PROTOCOL_RTSP:
			case PROTOCOL_HTTP:
			{

				    delete iUrlToOpen;
				    iUrlToOpen=NULL;
				    iUrlToOpen=aVideoRequest.iPlaySrc->AllocL();
				    StartNetworkConnectionL();				    
                    iState = ELoading;
			}
			break;
		}
	}
}

BOOL CVideoObject::PauseL()
{
    if(iState != EError)
    {
        if(iState == EPlaying)
    	{
            iPlayer->PauseL();
            iState = EPaused;
            iVideoEngine.GetFlashEngine().ExtendBacklightForVideoPlayback(EOff);            
            return TRUE;
    	}
        else if ((iState == EInitializing) || (iState == ELoading))
        {
    		iNeedsToPause = ETrue;
    		return TRUE;
        }
        
    }
	return FALSE;
}


void CVideoObject::CancelL()
{
	StopL();
	if(iState == EStopped)
	{
		iState = EDone;
	}
}

void CVideoObject::StopL()
{
	if((iState == EPlaying) || (iState == EPaused))
	{
		iPlayer->Stop();
		iState = EStopped;
	}
	else if((iState == EInitializing) || (iState == ELoading))
	{
		iNeedsToStop = ETrue;
	}
	
}

void CVideoObject::Close()
{
	//Close must return if video is not playing or in paused state
	if( !(iState == EPlaying || iState == EPaused) )
		return;

    Reset();
    
//#ifdef USE_VPU2        
        //iVideoEngine.GetFlashEngine().GetDisplay()->CloseVideoWindow((void*)this);
        //iParentWindow = NULL;
        //iCurrentDisplayWindow = NULL;
//#endif    
    
#ifdef REDUCE_VIDEO_FLICKER
	TRAP_IGNORE (iVideoEngine.GetFlashEngine().SetVideoRectL(NULL, EVideoNone));
#endif // REDUCE_VIDEO_FLICKER
    iState = EClosed;
}

// Get current position in Second.. check specs
TInt CVideoObject::GetPositionL()
{
    TInt64 ret = 0;

    if((iState == EPlaying || iState == EStopped || iState == EPaused || iState == EDone) )
    {
        if(iPlayer)
        {
            ret = iPlayer->PositionL().Int64() / KMPOneSecond;
        }
    }

	#ifdef EKA2
	return (TInt)ret;
	#else // EKA2
	return ret.GetTInt();
	#endif // EKA2
}

void CVideoObject::SeekL(TInt aPosition)
{
    if(! (iState == EPlaying || iState == EStopped || iState == EPaused || iState == EDone) )
    {
        return;
    }

    // Fix for bug 146071.
    // Discarding seek requests to within 100ms of the current position in 
    // the video. Setting the position to the current frame seems to make the
    // Symbian video player utility behave badly.
    
    TInt64 posInMicroSecs = TInt64(aPosition * KMicrosecondsInSecond);
    TInt64 currentPos = iPlayer->PositionL().Int64();
    TInt64 tenthOfASecondInMicroSeconds = 100000;
    TInt64 delta = Abs(posInMicroSecs - currentPos);
    
    if(delta > tenthOfASecondInMicroSeconds)
    {
        TPlayerState oldState = iState;

        PauseL();
	    iPlayer->SetPositionL(posInMicroSecs);

        if(oldState == EPlaying)
        {
            iPlayer->Play();
            iState = EPlaying;
        }
    }
}

void CVideoObject::MuteL(TBool aMute)
{
	iIsMutedByAS=aMute;
	
	if(iState==EPlaying || iState==EPaused)
		CalculateAndSetVolumeL();
}

TBool CVideoObject::IsMuted()
{
	return iIsMutedByAS;
}

_LIT(KLitVideoDecodeFailed,"decodeFailure");

/*
-----------------------------------------------------------------------------

  void CVideoEngine::MvpuoOpenComplete(TInt aError)
  
	Description: This method defines required client behaviour when an attempt to open 
	a video sample has completed, successfully or otherwise.
	Comments   :
	
	Return values: N/A
	  
-----------------------------------------------------------------------------
*/ 
void CVideoObject::MvpuoOpenComplete(TInt aError)
{
	if(aError == KErrNone)
	{
		// Prepare for the loading the file
				
		if(IsStreamSoundPlaying())
		{
			iState = EStoppedBecauseOfError;
			return;			
		}
		
		if(iNeedsToStop)
		{
			iNeedsToStop = EFalse;
			iState = EStoppedBecauseOfError;
			return;
		}
		else
		{
			iPlayer->Prepare();
			iState = ELoading;
		}
	}
	else 
	{
		// Stop the heart beat and close the player

		//TRAPD(ignore,StopL()); // DK: don't think this is needed because if open failed we cannot be playing!
		FI_Text status;
		status.textEncoding = FI_UTF16EncodingForm;
		status.text.utf16String = (&KLitVideoDecodeFailed)->Ptr();		
		status.textLen = (&KLitVideoDecodeFailed)->Length();
		iVideoEngine.GetFlashEngine().GetFIObject().FI_NotifyVideoStatus(iId, 0, &status);
		iState = EError;
		
		if(iVideoEngine.GetFlashEngine().IsBrowserPluginActive())
	    {
			#ifdef FF_FLASHLITE_SCREENSAVER	
            iVideoEngine.GetFlashEngine().NotifyFlashError(EInvalidVideoFileError,EFalse);		            
			#endif  //FF_FLASHLITE_SCREENSAVER		            
		}
	    else
	    {
	        //iVideoEngine.GetFlashEngine().GetUI()->NotifyDataAccessError(NULL, EInvalidVideoFileError);
	        iVideoEngine.GetFlashEngine().GetUserNotifierIf()->NotifyDataAccessError(NULL, EInvalidVideoFileError);    
	    }		
   	    
		
   	    Reset();
	    
	    TRAP_IGNORE(iVideoEngine.GetFlashEngine().SetVideoRectL(NULL, EVideoNone));    	        	
	}
	
}

/*
-----------------------------------------------------------------------------

  void CVideoEngine::MvpuoPrepareComplete(TInt aError)
  
	Description:  Notification that video player is ready to begin playback. 
	Comments   :  This callback is generated in response to a call to Prepare
	in CVideoPlayerUtility
	
	Return values: N/A
	  
-----------------------------------------------------------------------------
*/  
void CVideoObject::MvpuoPrepareComplete(TInt aError)
{
    TRAPD(err,MvpuoPrepareCompleteL(aError));

    if(err)
    {
        /*
        if(!(iVideoEngine.GetFlashEngine().IsBrowserPluginActive()))
        {
            iVideoEngine.GetFlashEngine().GetUI()->NotifyClientError(NULL, EFailedPreparingVideoError);
        }            
		*/
        iVideoEngine.GetFlashEngine().GetUserNotifierIf()->NotifyClientError(NULL, EFailedPreparingVideoError);
    
		#ifdef FF_FLASHLITE_SCREENSAVER 
        else if(iVideoEngine.GetFlashEngine().GetPlugin())
        {
	        iVideoEngine.GetFlashEngine().NotifyFlashError(EInvalidVideoFileError,EFalse);		            
        }
		#endif //FF_FLASHLITE_SCREENSAVER
    }
    
}

void CVideoObject::MvpuoPrepareCompleteL(TInt aError)
{
	
	if(aError == KErrNone || aError == KErrMMPartialPlayback)
	{
		if(IsStreamSoundPlaying())
		{
			iState = EStoppedBecauseOfError;
			return;			
		}

		if(iNeedsToStop)
		{
			iNeedsToStop = EFalse;
			iState = EStoppedBecauseOfError;
			return;
		}

		if (iNeedsToPause && iIsStreaming && iVideoEngine.GetFlashEngine().IsForegroundTask())
		{
			iNeedsToPause = EFalse;
		}

		if (iNeedsToPause)
		{
	        iPlayer->PauseL();
		    iState = EPaused;
			iNeedsToPause = EFalse;
		}
		else
		{
			iPlayer->Play();
			iState = EPlaying;
			iVideoEngine.GetFlashEngine().ExtendBacklightForVideoPlayback(EOn);			
			CalculateAndSetVolumeL();
		}

#ifdef USE_VPU2
        TPoint winAbsPosition(iParentWindow->AbsPosition());
        
        TRect relativeWinRect(iRect);
        relativeWinRect.Move(-winAbsPosition.iX,-winAbsPosition.iY);
        TRect relativeClipRect(iRect);
        relativeClipRect.Move(-winAbsPosition.iX,-winAbsPosition.iY);    		
		
		TRAP_IGNORE(iPlayer->AddDisplayWindowL(iCoeEnv->WsSession(),
                                                *(iCoeEnv->ScreenDevice()),
                                                *iParentWindow, relativeWinRect, relativeClipRect));

	    iCurrentDisplayWindow = iParentWindow;
	    TRAP_IGNORE(iPlayer->SetAutoScaleL(*iParentWindow, EAutoScaleBestFit));
#endif	    
	}
	else
	{
		FI_Text status;
		status.textEncoding = FI_UTF16EncodingForm;
		status.text.utf16String = (&KLitVideoDecodeFailed)->Ptr();		
		status.textLen = (&KLitVideoDecodeFailed)->Length();
		iVideoEngine.GetFlashEngine().GetFIObject().FI_NotifyVideoStatus(iId, 0, &status);
		
		/*
		if(!(iVideoEngine.GetFlashEngine().IsBrowserPluginActive()))
		{
		    iVideoEngine.GetFlashEngine().GetUI()->NotifyDataAccessError(NULL, EFailedPreparingVideoError);
		}		
	    */
		iVideoEngine.GetFlashEngine().GetUserNotifierIf()->NotifyDataAccessError(NULL, EFailedPreparingVideoError);
	    
	    Reset();
	    
	    iVideoEngine.GetFlashEngine().SetVideoRectL(NULL, EVideoNone);    	        	
	}
}
_LIT(KLitVideoPlayComplete,"completed");
_LIT(KLitVideoPlayFailed,"failed");

/*
-----------------------------------------------------------------------------

  void CVideoEngine::MvpuoPlayComplete(TInt aError)
  
	Description:  This method defines required client behaviour when an attempt to 
	play a video sample has completed, successfully or otherwise. 
	Comments   :  This method is not called if playback is explicitly stopped by 
	calling Stop in CVideoPlayerUtility.
	
	Return values: N/A
	  
-----------------------------------------------------------------------------
*/   

void CVideoObject::MvpuoPlayComplete(TInt aError)
{
	
	if (aError == KErrNone && iLoop)
	{    
		// Reset position to beginning and play the clip again
		TRAP_IGNORE(iPlayer->SetPositionL(TTimeIntervalMicroSeconds(TInt64(0))));
		iPlayer->Play();
		iState = EPlaying;
	}
	else
	{
		iVideoEngine.GetFlashEngine().ExtendBacklightForVideoPlayback(EOff);
		iState = EDone;
		FI_Text status;
		status.textEncoding = FI_UTF16EncodingForm;
		if (aError == KErrNone)
		{
			status.text.utf16String = (&KLitVideoPlayComplete)->Ptr();		
			status.textLen = (&KLitVideoPlayComplete)->Length();
			iVideoEngine.GetFlashEngine().GetFIObject().FI_NotifyVideoStatus(iId, 1, &status);
		}
		else
		{
			status.text.utf16String = (&KLitVideoPlayFailed)->Ptr();		
			status.textLen = (&KLitVideoPlayFailed)->Length();
			iVideoEngine.GetFlashEngine().GetFIObject().FI_NotifyVideoStatus(iId, 0, &status);
		
			if(iVideoEngine.GetFlashEngine().IsBrowserPluginActive())
		    {
				#ifdef FF_FLASHLITE_SCREENSAVER 
		        iVideoEngine.GetFlashEngine().NotifyFlashError(EInvalidVideoFileError,EFalse);		            
		        #endif //FF_FLASHLITE_SCREENSAVER
		    }
	    	else
	        {
	            //iVideoEngine.GetFlashEngine().GetUI()->NotifyDataAccessError(NULL, EInvalidVideoFileError);   
                iVideoEngine.GetFlashEngine().GetUserNotifierIf()->NotifyDataAccessError(NULL, EInvalidVideoFileError); 
	        }		
	    	
	    	Reset();
	    	TRAP_IGNORE(iVideoEngine.GetFlashEngine().SetVideoRectL(NULL, EVideoNone));    	        	
		}
	if(iConnHandler)  //After the play complete we should release existing iConnHandler 
						//object to avoid battery usage
		{
		delete iConnHandler;
		iConnHandler = NULL;
		}
	}
}

/*
-----------------------------------------------------------------------------

  void CVideoEngine::MvpuoFrameReady(CFbsBitmap& aFrame,TInt aError)
  
	Description: Defines required client behaviour when a request to capture a 
	frame is made using CVideoPlayerUtility::GetFrame(). 
	Comments   :  
	
	Return values: N/A
	  
-----------------------------------------------------------------------------
*/   

void CVideoObject::MvpuoFrameReady(CFbsBitmap& /*aFrame*/,TInt /*aError*/)
{
}

/*
-----------------------------------------------------------------------------

  void CVideoEngine::MvpuoEvent(const TMMFEvent& aEvent)
  
	Description: Mvpuo Event comes in.
	Comments   : 
	
	Return values: N/A
	  
-----------------------------------------------------------------------------
*/
void CVideoObject::MvpuoEvent(const TMMFEvent& aEvent)
{
    // Higher priority application has taken over the audio device --> Do pause.
    if (aEvent.iEventType == KMMFEventCategoryVideoPlayerGeneralError &&
        aEvent.iErrorCode == KErrHardwareNotAvailable)
	{
	}
}

void CVideoObject::NotifyNewVolumeLevelL(TInt aNewVolLevel)
{
	iUserVolume=aNewVolLevel;

	if(iState==EPlaying || iState==EPaused)
		CalculateAndSetVolumeL();

}

// Note, we can only set the volume now if the track is loaded.
// If not playing otherwise it will Leave with KErrNotReady,
// because the volume is associated with the current track.
// Also the volume level may be codec dependent - so we cannot compute that till it is playing either!
void CVideoObject::CalculateAndSetVolumeL()
{
	TInt volumeToPlay=0;

	if((!(iUserVolume&KMuteMask)) && (!iIsMutedByAS))
	{
		// not muted - calculate our volume level...
		const TInt multiplier=iUserVolume&(~KMuteMask);
		volumeToPlay=(2*multiplier*(iPlayer->MaxVolume()))+(KMaxUserVolume); // multiplying this all by 2 keeps precision better when KMaxUserVolume is not an even number
		volumeToPlay/=(KMaxUserVolume*2);
	}
	
	iPlayer->SetVolumeL(volumeToPlay);
}

//need to check 
void CVideoObject::HandleNetworkAccessDenied()
{
	FI_Text status;
	status.textEncoding = FI_UTF16EncodingForm;
	status.text.utf16String = (&KLitVideoDecodeFailed)->Ptr();
	status.textLen = (&KLitVideoDecodeFailed)->Length();
	iVideoEngine.GetFlashEngine().GetFIObject().FI_NotifyVideoStatus(iId, 0, &status);
	iState=ENotInitialized;
}

void CVideoObject::UpdateRectL(const FI_Rect& aLocation)
{	
	if((iVideoEngine.GetFlashEngine().IsBrowserPluginActive()))
    {
        //iRect = iVideoEngine.GetFlashEngine().GetPlugin()->GetRect();
        iRect = iVideoEngine.GetFlashEngine().GetDisplay()->ClientRect();
    }
    //else if(iVideoEngine.GetFlashEngine().GetUI())
	else if(iVideoEngine.GetFlashEngine().GetDisplay())
    {
        //if(iVideoEngine.GetFlashEngine().GetUI()->IsFullScreenMode())
        if(iVideoEngine.GetFlashEngine().GetDisplay()->IsFullScreenMode())
        {
            iRect = iVideoEngine.GetFlashEngine().GetDisplay()->ApplicationRect();            
        }
        else
        {
            iRect = iVideoEngine.GetFlashEngine().GetDisplay()->ClientRect();                
        }
    }
   
   

	TRect clientRect = iRect;
	TRect videoRect = iRect;
    iLocation = aLocation;
	videoRect.Move(aLocation.xmin, aLocation.ymin);
	videoRect.SetSize(TSize(aLocation.xmax-aLocation.xmin, aLocation.ymax-aLocation.ymin));

	//resize rect to within bounds
	iRect.Intersection(videoRect);

	// If the size (both x and y) is not divisible by 4,
	// the DSP code may crash the phone.
	TInt width = iRect.Width();
	TInt height = iRect.Height();
	if (height > width)
	{
		// 2.8 devices crash or hang if the video rect's height is > width.
		TInt dy = (height - width) / 2;
		height = width;
		iRect.Move(0, dy);
	}

	iRect.SetSize(TSize(width - width % 4, height - height % 4));

	// Ensure bounds do not exceed clientRect
	// Some codecs will crash the entire phone if you try to
	// render offscreen
	TPoint& tl = iRect.iTl;
	TPoint& br = iRect.iBr;
	
	const TPoint& minTl = clientRect.iTl;
	const TPoint& maxBr = clientRect.iBr;

	if ( tl.iX < minTl.iX ) { tl.iX = minTl.iX; }
	if ( tl.iY < minTl.iY ) { tl.iY = minTl.iY; }
	if ( br.iX > maxBr.iX ) { br.iX = maxBr.iX; }
	if ( br.iY > maxBr.iY ) { br.iY = maxBr.iY; }

	iRect.Normalize();
	
	// inform engine of video rect, so we can avoid drawing in that area.
	if(
       /*TSWID: ESDS-7QMELB, Video display is not proper by changing the paused video to landscape mode;
       Excluding video rect propogation to engine if device video is paused, is incorrect. Engine must
       always have the updated video rect, irrespective of the video playback state
       iState != EPaused && */
	   iState != ENotInitialized &&
	   iState != EInitializing && 
	   iState != EClosed )
	{
	    iVideoEngine.GetFlashEngine().SetVideoRectL(&aLocation, EDeviceVideo);  
	}	
}

// StartDirectScreenAccessL() and StopDirectScreenAccessL is handled
// by the windows server itself, so we don't need to do this
/*void CVideoObject::HandleFocusChangedL(TBool aForeGround)
{
    
    if(iPlayer && ((iState == EStopped) || (iState == EPlaying) || (iState == EPaused) ))
    {
        if(aForeGround && iDSAStopped)
        {
            iPlayer->StartDirectScreenAccessL();
            iDSAStopped = EFalse;      
        }
        else if(!aForeGround && !iDSAStopped)
        {
            iPlayer->StopDirectScreenAccessL();
            iDSAStopped = ETrue;
        }
    }
}*/
    
void CVideoObject::HandleSizeChangedL()
{
    SetRectL(iLocation, ETrue);
}   

/*
 * Fix for the 3gp issue : ERoom # 513
 */

void CVideoObject::HandleScrollL()
{

    DEBUGPRINT("CVideoObject::HandlePositionChangedL() ++ ");
 
    if( iState == EPlaying || iState == EPaused || iState == EDone )
        SetRectL(iLocation, ETrue);
    
    DEBUGPRINT("CVideoObject::HandlePositionChangedL() -- ");
}   

void CVideoObject::SetVideoWindow(RWindow* aWindow)
{
    iParentWindow = aWindow;
#ifdef USE_VPU2    
    if(aWindow == NULL)
    {
        if(iCurrentDisplayWindow)
        {    
            iPlayer->RemoveDisplayWindow(*iCurrentDisplayWindow);
            iCurrentDisplayWindow = NULL;
        }
    }
#endif
}


void CVideoObject::StartNetworkConnectionL()
{
	if (iIPConnectionMgr)
		{
		TUint32 lIAPId = 0;
		TInt lIAPIdStatus = iIPConnectionMgr->GetIAP(lIAPId);
		if(lIAPIdStatus == EFLIAPNotSet)
			{
			iConnHandler = iIPConnectionMgr->GetConnectionL(this);
			}
		else
			{
			HandleConnectionL();
			}
		}
}

void CVideoObject::ConnectionEventL(TFlhIPEvent aEvent)
{
	TInt KAsyncTimeOut = 1000000; //this is only to start video payer utility request with dealy so that Runl of Connhanlder RunL()
									// exits properly
	if(aEvent == EEventIsConnectedOK)
	{
		//handle connection here
		iTimer = CFlhTimerObject::NewL(CActive::EPriorityUserInput, *this);		
		iTimer->After(KAsyncTimeOut);			
	}
	else
	{
	HandleNetworkAccessDenied();
	}
}

void CVideoObject::HandleConnectionL()
{
	HBufC* temp=iUrlToOpen;
	iUrlToOpen=NULL;
	CleanupStack::PushL(temp);
	iIsStreaming = ETrue;
		
	TUint32 lIAPId = 0;
	TInt lIAPIdStatus = iIPConnectionMgr->GetIAP(lIAPId);
	
	DEBUGPRINT("CVideoObject::HandleConnectionL << lIAPIdStatus %d, lIAPId %d", lIAPIdStatus, lIAPId );
	if(lIAPIdStatus == EFLIAPAlreadySet)
		{
		iPlayer->OpenUrlL(*temp,lIAPId); // the player will connect it if needed...
		}
	CleanupStack::PopAndDestroy(temp);	
}
void CVideoObject::DoRunL(const CActive* /*aActive*/)
	{
	HandleConnectionL();
	}

TInt CVideoObject::DoRunError(const CActive* /*aActive*/, TInt aError)
	{
	return aError;
	}
#endif	// FEATURE_DEVICE_VIDEO
