/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash player common defs 
*
*/

#ifndef __COMMONDEF__H__
#define __COMMONDEF__H__

#include "amp_interface.h"
#include "flashlitefontscriptdef.h"
/*
 * Use the following to catch any place where structures are CORE_NEWED or CORE_DELETED @ compile time
 */

#if defined( FEATURE_NO_GLOBALNEWDELETE )
	class Allocator;

	/*
	 *WARNING: **ALWAYS** put this at the end of the struct definition
	 */ 
	#if defined(COMPILER_USES_CPP_EXCEPTIONS)
	#define NO_OPERATOR_NEW_DELETE												\
		private: 																\
			static void* operator new ( size_t ); 								\
			static void* operator new[] ( size_t ); 							\
			static void* operator new ( size_t, Allocator*, unsigned int );		\
			static void* operator new[] ( size_t, Allocator *, unsigned int ); 	\
			static void operator delete ( void* ); 								\
			static void operator delete[] ( void* ); 							\
			static void operator delete ( void*, Allocator*, unsigned int ); 	\
			static void operator delete[] ( void*, Allocator*, unsigned int );
	#else
	#define NO_OPERATOR_NEW_DELETE												\
		private: 																\
			static void* operator new ( size_t ); 								\
			static void* operator new[] ( size_t ); 							\
			static void* operator new ( size_t, Allocator*, unsigned int );		\
			static void* operator new[] ( size_t, Allocator *, unsigned int );	\
			static void operator delete ( void* );								\
			static void operator delete[] ( void* );
	#endif
#else 
	#define NO_OPERATOR_NEW_DELETE
#endif /*defined(DEBUG) && !defined(FLASHLITE_CFRONT) && defined(FLASH_WIN32)*/
			

/*  This is an undefined type of HostObject data-type.
 *  It's up to the host application to define this type
 *  and pass it to the player when creating it.
 */
struct SI_HostObject;

/* Error codes returned by SI_ functions
 * 
 * Note: These should never exceed the bounds of short, i.e. (SHRT_MIN, SHRT_MAX)!
 */
enum _SI_Error
{
    SI_UnsupportedError = -1,
    SI_GenericError     = 0,
    SI_NoError          = 1
};

typedef short SI_Error;  /* typedef as a short for portability reasons */

struct FI_Rect
{
    short xmin;
    short xmax;
    short ymin;
    short ymax;
	NO_OPERATOR_NEW_DELETE
};

struct FI_Point
{
	short x;
	short y;
	NO_OPERATOR_NEW_DELETE
};

// ACHW

/* Rendering profile, machine depend data. */
typedef void* FI_Display;
typedef void* FI_Config;
typedef void* FI_Context;
typedef void* FI_ClientBuffer;
typedef int   FI_NativeDisplayType;
typedef void* FI_NativePixmapType;

#define FI_EGL_RED_SIZE        0x3024
#define FI_EGL_GREEN_SIZE      0x3023
#define FI_EGL_BLUE_SIZE       0x3022
#define FI_EGL_ALPHA_SIZE      0x3021
#define FI_EGL_DEPTH_SIZE      0x3025
#define FI_EGL_SAMPLES         0x3031
#define FI_EGL_SAMPLE_BUFFERS  0x3032
#define FI_EGL_SURFACE_TYPE    0x3033
#define FI_EGL_NONE            0x3038
#define FI_EGL_RENDERABLE_TYPE 0x3040
#define FI_EGL_HEIGHT          0x3056
#define FI_EGL_WIDTH           0x3057
#define FI_EGL_OPENVG_IMAGE    0x3096

#define FI_EGL_PBUFFER_BIT     0x01
#define FI_EGL_PIXMAP_BIT      0x02

#define FI_EGL_OPENGL_ES_BIT   0x01
#define FI_EGL_OPENVG_BIT      0x02

#define FI_EGL_DEFAULT_DISPLAY ((FI_NativeDisplayType)0)
#define FI_EGL_DONT_CARE       (-1)

enum _FI_SurfaceType
{
	FI_SurfaceType_Window  = 0,
	FI_SurfaceType_Pbuffer = 1,
	FI_SurfaceType_Pixmap  = 2
};

struct FI_Surface
{
	void* surface; /* handle to surface (Ex.EGLSurface) */
	void* hPixmap; /* handle to native pixmap */
	short type;    /* surface type */
	NO_OPERATOR_NEW_DELETE
};

struct FI_PixmapInfo
{
	unsigned short width;   /* width */
	unsigned short height;  /* height */
	short pitch;            /* pitch */
	unsigned char* pPixmap; /* pointer to pixmap memory */
	NO_OPERATOR_NEW_DELETE
};

#define FI_EGL_NO_CONTEXT ((FI_Context)0)
#define FI_EGL_NO_SURFACE ((struct FI_Surface*)0)

/*---------------------------------------------------------------------------*/

enum FI_HardwareConfigFlag {
	FI_HardwareConfig_UseAuxTextBufferForHWRendering	= 0x1,	/* Host cannot render device text to HW surface so aux text buffer is used */
	FI_HardwareConfig_HardwareSurfaceIsDoubleBuffered 	= 0x2,	/* A double buffered surface requires full screen updates */
	FI_HardwareConfig_PreserveHWResourcesWhenPaused		= 0x4	/* Do not delete hardware rendering resouces when pausing the player */	
};

enum FI_HardwareQuality {
	FI_HardwareQuality_Default,					/* follow software rendering quality settings */
	FI_HardwareQuality_NonAntialiased,
	FI_HardwareQuality_Faster,
	FI_HardwareQuality_Better
};
 
struct FI_HardwareConfig {
	unsigned int configFlags;
	enum FI_HardwareQuality hardwareQuality;
};


/* Rendering hint to use hardware or software */
enum FI_RenderHint
{
	FI_RenderHint_Software = 1,			/* force software */
	FI_RenderHint_Hardware = 2			/* force hardware */
};

//typedef unsigned short FI_RenderHint;

enum FI_RenderMode {
	FI_RenderMode_Error = 0,
	FI_RenderMode_Software = 1,
	FI_RenderMode_Hardware = 2
};

enum FI_HardwareDeviceLimitIndex {
	FI_HardwareDeviceLimitIndex_imageHandle = 0,
	FI_HardwareDeviceLimitIndex_imageMemory,
	FI_HardwareDeviceLimitIndex_num
};

/* Possible frame buffer pixel formats supported by player */
enum FI_DisplayFormat
{
	// Note: Do not change order, add new enums at end
	FI_DisplayFormat_None,          /* api error value */
    FI_DisplayFormat_RGB555_16,     /* 16 bit RGB 0555 format */
    FI_DisplayFormat_RGB565_16,     /* 16 bit RGB 0565 format */
    FI_DisplayFormat_RGB444_16,     /* 16 bit RGB 0444 format */
    FI_DisplayFormat_RGB5515_16,    /* 16 bit RGB 5515 format */
	FI_DisplayFormat_RGB666_24,     /* 24 bit RGB 666 format */ 
    FI_DisplayFormat_ARGB8888_32,   /* 32 bit ARGB 8888 format */
    FI_DisplayFormat_RGB666_32,      /* 32 bit RGB 0666 format */
	FI_DisplayFormat_RGB888_24,      /* 24 bit RGB 888 format*/
	FI_DisplayFormat_BRGB2664_32,    /* 32 bit RGB 2664 format */
	FI_DisplayFormat_ARGB4444_16,     /* 16 bit RGB 4444 format */
	FI_DisplayFormat_Multiple          /* Used when multiple formats compiled in but none is set runtime */
	/*
	FI_DisplayFormat_ARGB4444_16 was added in the middle in FL3.1 but that causes problems in DC.
	Since other versions of FL will be using different values for different image formats.
	For e.g., FI_DisplayFormat_RGB666_24 is 5 in older FL but 6 in FL3.1.
	The solution to this was either to change DC to take care of FL3.1 which was messy or add the new enum values
	at the end for DC and FL3.1. Chose the second option.
	*/


};



/* Error codes passed to SI_NotifyError */
enum FI_ErrorCode
{
	/* FlashLite 1.1 error codes */
    FI_NoError                      = 0,
    FI_OutOfMemoryError             = 1,
    FI_StackLimitReachedError       = 2,
    FI_CorruptSWFDataError          = 3,
    FI_ActionScriptStuckError       = 4,

	/* FlashLite 2.0 error codes */
    FI_InvalidKeyError              = 5,
    FI_BadImageDataError            = 6,
    FI_BadSoundDataError            = 7,
    FI_RootMovieUnloadedError       = 8,
    FI_ImageDecodeMemoryError       = 9,
    FI_CorruptSVGDataError          = 10,
	FI_NetworkMemoryError			= 11,

	/* FlashLite 2.1 error codes */
	FI_InvalidFrameBufferError		= 12,
	FI_InvalidFrameNumberError		= 13,

 	/* FlashLite 3.0 error codes */
	FI_StreamURLLimitReachedError	= 14,
     
	//introduced in FL3.1, in case of AS3 content
	FI_AS3Error						= 15,

	FI_RenderingStuckError			= 16,

	FI_InvaildInterfaceFunctionError= 17, //FEFATURE_AMCEXTENSIONS
    /* This should be last */
    Next_FI_ErrorCode
};


/* Use to indicate the buffer mode for FI_SetSWFDataBuffer */
enum
{
	FI_SwfBufferNoCopy			= 0,
	FI_SwfBufferCopy			= 1,
	FI_HttpBufferNew			= 2,
	FI_HttpBufferContinue		= 3
};

enum FI_Result
{
    FI_ResultError      = 0,
    FI_ResultOK         = 1
};


/* Return codes from FI_DoPlay */
enum FI_DoPlayReturnCode
{
	//Composite Flags
	//! USE ONLY WITH FI_DoPlayCompositeFlagsMask
	FI_FrameBufferNotUpdatedWithError                   = 2, // 00010
    FI_FrameBufferUpdatedWithError                      = 3, // 00011
    FI_SystemInterruptWithError                         = 6, // 00110
    FI_RenderingInterruptedByTimeoutWithError           = 10,// 01010
    FI_ActionScriptInterruptedByTimeoutWithError        = 14,// 01110
	FI_DoPlayCompositeFlagsMask							= 15,// 01111

	//Flags
    FI_FrameBufferNotUpdated                            = 0, // 000000
    FI_FrameBufferUpdated		                        = 1, // 000001
    FI_DoPlayErrorFlag									= 2, // 000010
	FI_SystemInterrupt                                  = 4, // 000100
    FI_RenderingInterruptedByTimeout                    = 8, // 001000
	FI_ActionScriptInterruptedByTimeout                 = 12,// 001100

	/*  Indicates that some actions (for example URL Requests, DelayedActions,
		Video buffering) need to be performed and DoPlay should be called faster
		than normal to process these actions quickly.
		This flag is only a performance optimization and the host can choose to 
		ignore this flag.
	*/
	FI_DoPlayActionsPendingFlag							= 16,// 010000
	/*  Indicates that frame rate of the swf is running slower than intended and DoPlay
		calls should be speeded up to for optimum performance. It is recommended to speed 
		up DoPlay call intervals by 10% to catch up with the normal timing.
		If the host wants to speed up faster than 10%, DoPlay should be called with 
		the forceRender flag on, otherwise FlashLite would ignore calls faster than 10% 
		of the normal timing.
		This flag is only a performance optimization and the host can choose to 
		ignore this flag.
	*/
	FI_DoPlayFrameRateLagging							= 32 // 100000
};

/* Return codes from FI_OfferKeyEvent, FI_OfferKeyPressEvent, FI_OfferMouseEvent */
enum FI_OfferEventReturnCode
{
    FI_EventError                       = 0,
    FI_EventSuccess                     = 1,
    FI_EventInterrupted                 = 2,
    FI_EventTimeout                     = 3,
    FI_EventInterruptedWithError        = 4,
    FI_EventTimeoutWithError            = 5
};

/* Enums for render quality */
enum FI_QualitySetting
{
    FI_QualityLow,
    FI_QualityMedium,
    FI_QualityHigh
};


/* codes to identify some of the special keys accepted by Flash Player */
enum FI_KeyCode
{
    FI_KEY_LEFT     = 1,    /* Four-way directional navigation */
    FI_KEY_RIGHT    = 2,    /* Four-way directional navigation */
    FI_KEY_HOME     = 3,    /* Send Key */
    FI_KEY_END      = 4,    /* End Key */
	FI_KEY_INSERT   = 5,	/* Insert Key */
    FI_KEY_DELETE   = 6,	/* Delete Key */
	FI_KEY_BACKSPACE = 8,	/* Backspace */
    FI_KEY_SELECT   = 13,   /* Select key */
    FI_KEY_UP       = 14,   /* Four-way directional navigation */
    FI_KEY_DOWN     = 15,   /* Four-way directional navigation */
    FI_KEY_PAGEUP   = 16,   /* Left soft key */
    FI_KEY_PAGEDOWN = 17,   /* Right soft key */
    FI_KEY_FORWARD  = 18,   /* Two-way directional navigation */
    FI_KEY_BACKWARD = 26,   /* Two-way directional navigation */
	FI_KEY_ESCAPE	= 19,	/* Escape Key */
    FI_KEY_ENTER	= 31,   /* Enter key */
	FI_KEY_TAB		= 300,	/* Tab Key */
	FI_KEY_CAPS		= 302,	/* Capslock Key */
	FI_KEY_SHIFT    = 303,	/* Shift Key */
	FI_KEY_CTRL	    = 304,	/* Ctrl Key */
	FI_KEY_NUMLOCK  = 305	/* Numlock Key */
	,FI_KEY_FAKE	= 32766 /* Fake Key for CLR Key Support */
      ,FI_KEY_HASH     = 35,   /* '#" */
      FI_KEY_ASTERISK = 42,   /* '*' */
      FI_KEY_0        = 48,   /* '0' */
      FI_KEY_1        = 49,   /* '1' */
      FI_KEY_2        = 50,   /* '2' */
      FI_KEY_3        = 51,   /* '3' */
      FI_KEY_4        = 52,   /* '4' */
      FI_KEY_5        = 53,   /* '5' */
      FI_KEY_6        = 54,   /* '6' */
      FI_KEY_7        = 55,   /* '7' */
      FI_KEY_8        = 56,   /* '8' */
      FI_KEY_9        = 57,   /* '9' */
    
      FI_KEY_PLUS     = 43,   /* Volume Up Key */
      FI_KEY_MINUS    = 45,   /* Volume Down Key */
      FI_KEY_TILDE    = 126,  /* Voice Memo Key */
      FI_KEY_EXCLAMATION  = 33,   /* Camera Key */
      FI_KEY_AT       = 64,   /* WAP Key */
      FI_KEY_COMMA    = 44,   /* MP3 1 Key (Rewind) */
      FI_KEY_DOT      = 46,   /* MP3 2 Key (Play) */
      FI_KEY_SLASH    = 47    /* MP3 3 Key (Fast Fwd) */
};


/* code to identify type of event */
enum FI_EventCode
{
    /* No-Op */
    FI_EventNoEvent         = 0x0000,

    /* Key Events */
    FI_EventKeyDown         = 0x0010,
    FI_EventKeyUp           = 0x0011,
    FI_EventKey             = 0x0012,     

    /* Mouse Events */
    FI_EventMouseDown       = 0x0020,
    FI_EventMouseUp         = 0x0021,
    FI_EventMouseMove       = 0x0022,

    /* Host Event */
    FI_EventHost            = 0x0040,

    /* Streaming events */
    FI_EventCreateInitialStream   = 0x0100
};



/* struct to contain information returned from SI_GetCharacterInfo */
struct FI_CharacterInfo
{
    unsigned short  bitDepth;   /* valid values: 1,16 */
    unsigned short  width;      /* width of bitmap in pixels */
    unsigned short  height;     /* height of bitmap in pixels */
    unsigned short  pitch;      /* number of bytes per row of bitmap */
    short           xOrigin;    /* horizontal offset for bitmap */
    short           yOrigin;    /* vertical offset to baseline for bitmap */
    short           step;       /* horizontal spacing of character */
	NO_OPERATOR_NEW_DELETE
};


/* struct to contain information passed to and returned from SI_GetBitmapInfo
 */
struct FI_BitmapInfo
{
    char *          buffer;     /* memory buffer to copy bitmap to */
    unsigned short  size;       /* size of memory buffer in bytes */
    unsigned long   backgroundColor; /* returned background color of bitmap. */
                                /* (ignored and assumed to be 0 for 1 bit bitmaps) */
    unsigned long   textColor;  /* color of text to be applied to bitmap */
                                /* (ignored for 1 bit bitmaps) */
	NO_OPERATOR_NEW_DELETE
};


enum _FI_CodePage
{
    FI_UninitializedCodePage            = 0,
    FI_Latin1_CodePage                  = 1,    /* ISO 8859-1*/
    FI_Latin2_CodePage                  = 2,    /* ISO 8859-2*/
    FI_Latin4_CodePage                  = 3,    /* ISO 8859-4*/
    FI_Cyrillic_CodePage                = 4,    /* ISO 8859-5*/
    FI_Arabic_CodePage                  = 5,    /* ISO 8859-6*/
    FI_Greek_CodePage                   = 6,    /* ISO 8859-7*/
    FI_Hebrew_CodePage                  = 7,    /* ISO 8859-8*/
    FI_Turkish_CodePage                 = 8,    /* ISO 8859-9*/
    FI_Thai_CodePage                    = 9,    /* ISO 8859-11 */
    FI_TraditionalChinese_Big5_CodePage = 10,   /* Big5*/
    FI_SimplifiedChinese_GBK_CodePage   = 11,   /* GBK*/
    FI_Japanese_ShiftJIS_CodePage       = 12,   /* Shift-JIS*/
    FI_Korean_UHC_CodePage              = 13,   /* Unified Hangul Code*/

    /* This should always be last */
    Next_FI_CodePage
};

typedef unsigned short FI_CodePage;


enum _FI_EncodingForm
{
    FI_UninitializedEncodingForm    = 0,
    FI_NativeEncodingForm           = 1,    /* a.k.a. "MBCS" encoding */
    FI_UTF16EncodingForm            = 2,

    /* This should always be last */
    Next_FI_EncodingForm
};

typedef unsigned short FI_EncodingForm;


/* Character encoding typedef's
 */
typedef char                FI_UTF8Char;
typedef unsigned short      FI_UTF16Char;


/* Text strings will be passed in the following struct.
 */
struct FI_Text
{
    union {
        const char*             cString;
        const FI_UTF16Char*     utf16String;
    } text;
    FI_EncodingForm             textEncoding;
    unsigned long               textLen;        /* length of textString */
	NO_OPERATOR_NEW_DELETE
};

/* 
 * GestureEvent and TouchEvent information provided by host
 */
enum FI_GestureID
{
	FI_Gesture_Zoom
	,FI_Gesture_Pan
	,FI_Gesture_Rotate
	,FI_Gesture_TwoFingerTap
	,FI_Gesture_PressAndTap
	,FI_Gesture_Swipe
};

enum FI_GestureStage
{
	FI_GestureStage_Begin	= 1<<0
	,FI_GestureStage_Update	= 1<<1
	,FI_GestureStage_End	= 1<<2
};

struct FI_GestureTransform
{
	unsigned long		scaleX;
	unsigned long		scaleY;
	signed long			offsetX;
	signed long			offsetY;
	signed long			rotation;
};

struct FI_GestureEventInfo
{
	FI_GestureID			gid;
	int						stage; //This is an OR of FI_GestureStage flags
	FI_Point				location;
	int						isTransform;
	FI_GestureTransform		transform;
};

enum FI_TouchEventStage
{
	FI_TouchStageBegin	= 1<<0
	,FI_TouchStageMove	= 1<<1
	,FI_TouchStageEnd	= 1<<2
};

struct FI_TouchEventInfo
{
	unsigned long	x;
	unsigned long	y;
	unsigned long	contactW;
	unsigned long	contactH;
	unsigned long	pressure;
	unsigned long	tid;
	int				stage; //This is an OR of FI_TouchEventStage flags
	unsigned short	isPrimaryPoint;
};

// SI_InitMultitouch should return OR-ed value of the following bitfields
enum FI_MultitouchCapabilities
{
	FI_MultitouchNone = 0
	, FI_MultitouchGesture = (1<<0)
	, FI_MultitouchRaw = (1<<1)
};

typedef enum FI_MultitouchCapabilities FI_MultitouchInputMode;

/* Used by FI_OfferKeyEvent and FI_OfferKeyPressEvent. When keyName is NULL,
 * keyCode is used. */
struct FI_KeyId
{
    struct FI_Text*      keyName;
    /* Stagecraft platform enables dispatching key codes with values above Unicode max. */
    unsigned long        keyCode;

    short 				 doNotify;
 
	NO_OPERATOR_NEW_DELETE
};

struct FI_TextInfo {
    short           xPos;           /* x and y coordinates where */
    short           yPos;           /*   text should be drawn */
    short           xMin;           /* clipRect bounds */
    short           xMax;
    short           yMin;
    short           yMax;
	unsigned char   alpha;            /* alpha component of color */
    unsigned char   red;            /* R,G,B components of color */
    unsigned char   green;          /*  (each component has value */
    unsigned char   blue;           /*   in range [0-255]) */
    long            rotation;
	unsigned short	useDefaultColor;	/* by default it is false*/
// ACHW
    void*           pixmap;         /* added: target pixmap */
    const struct FI_PixmapInfo* pPixmapInfo; /* pixmap information */
// ACHW
	NO_OPERATOR_NEW_DELETE
};

struct FI_FontGlyphInfo{
	unsigned short* gids;			/* pointer to an array of glyph indices. Each entry represents glyph index in font table */	
	int*			gxydisp;		/* pointer to an array of x-y displacement of each glyph mentioned in glyph indices */
	int				len;			/* length of the glyph inices array */	
	short			fontSize;		/* font size of glyphs */

	/* rotation of glyph in range [0-4] */
	/* 0 - 0 degree */
	/* 1 - 90 degree */
	/* 2 - 180 degree */
	/* 3 - 270 degree */
	short           rotation;		
	FI_Point		start;			/* starting point*/			
	FI_Rect			rect;			/* clipping rect*/
	unsigned char   alpha;          /* alpha component of color */
    unsigned char   red;            /* R,G,B components of color */
    unsigned char   green;          /* (each component has value */
    unsigned char   blue;           /* in range [0-255]) */
	NO_OPERATOR_NEW_DELETE
};

/* Used by SI_ComplexLineBreak to pass paragraph information to the device,
 * get back the processed lines from the device */
struct FI_DeviceTextInfo {
    unsigned short indent;          /* [in] the indentation (in pixels) from 
                                       the left margin to the first 
                                       character in the paragraph. */
    unsigned short width;           /* [in] max width (in pixels) of a line */
    unsigned char  wordWrap;        /* [in] 0 = no word wrap, 1 = word wrap */
    
    struct FI_ReturnString *pVisualText;    /* [out] */
    struct FI_ReturnUShortArray  *visualTextLens; /* [out] */
    unsigned long numOfVisualLines;         /* [out] */
	NO_OPERATOR_NEW_DELETE
};

/* use const data instead of char pointers to reduce the data segment of the player */
static const char kFont_Sans[] = "_sans";
static const char kFont_Serif[] = "_serif";
static const char kFont_Typewriter[] = "_typewriter";

#define kFI_SANS_FONTNAME           kFont_Sans
#define kFI_SERIF_FONTNAME          kFont_Serif
#define kFI_TYPEWRITER_FONTNAME     kFont_TypeWriter

// NOTE - this is copied from Language enum in stags.h
// so these values must be same as what are there stags.h
// Language enumeration. (The high byte of the text flags.)
enum FI_Language
{
	FI_LANGUAGE_MIXED				= 0,	// Flash 5 format: either Shift-J or latin-1 depending on other factors.
											// Should never occur in a Flash 6 file, and is never legal in code.
	FI_LANGUAGE_LATIN				= 1,	// The western languages covered by latin-1: English, French, German, etc.
	FI_LANGUAGE_JAPANESE			= 2,
	FI_LANGUAGE_KOREAN				= 3,
	FI_LANGUAGE_SIMPLIFIED_CHINESE	= 4,
	FI_LANGUAGE_TRADITIONAL_CHINESE = 5
};

struct FI_FontInfo
{
    struct FI_Text* fontName;
    unsigned short  fontSize;       /* size of font in pixels */
    unsigned char   bold;           /* 0 = normal, 1 = bold */
    unsigned char   italic;         /* 0 = normal, 1 = italic */
    unsigned short  fontLanguage;   /* not used */
	NO_OPERATOR_NEW_DELETE
};

/*	return codes - from stream-related FI_ functions (and maybe others) */
enum FI_StreamReturnCode
{
	FI_STREAM_OK			= 0,
	FI_STREAM_ERROR			= -1,
	FI_STREAM_INVALID_ID	= -2,
	FI_STREAM_OUT_OF_MEMORY	= -3
};

struct FI_ReturnString;

typedef unsigned short (*FI_ReturnString_SetCString_Func) (struct FI_ReturnString* pReturnString, const char* cstr);
typedef unsigned short (*FI_ReturnString_SetUTF16String_Func) (struct FI_ReturnString* pReturnString, const unsigned short* utf16str);

struct FI_ReturnString
{
    FI_ReturnString_SetCString_Func SetCString;
    FI_ReturnString_SetUTF16String_Func SetUTF16String;
	NO_OPERATOR_NEW_DELETE
};

#define FI_ReturnString_SetCString(p,s) p->SetCString(p,s)
#define FI_ReturnString_SetUTF16String(p,s) p->SetUTF16String(p,s)

struct FI_ReturnUShortArray;

typedef unsigned short (*FI_ReturnUShortArray_Init_Func) (struct FI_ReturnUShortArray* pReturnArray, const unsigned long numElements);

typedef unsigned short (*FI_ReturnUshortArray_Append_Func) (struct FI_ReturnUShortArray* pReturnArray, const unsigned short element);

struct FI_ReturnUShortArray
{	
    FI_ReturnUShortArray_Init_Func   Init;
    FI_ReturnUshortArray_Append_Func Append;
	NO_OPERATOR_NEW_DELETE
};

#define FI_ReturnUShortArray_Init(p,n) p->Init(p,n)
#define FI_ReturnUShortArray_Append(p,e) p->Append(p,e)

struct FI_PlayerEventInfo
{
    unsigned long    eventId;
    unsigned short   eventCode;
    struct FI_KeyId* eventKeyId;
	NO_OPERATOR_NEW_DELETE
};


struct FI_GetURLCallInfo
{
	const char*                 url;
	unsigned short              urlLength;
	const unsigned char*        postData; 
	unsigned long               postDataLength;
	struct FI_PlayerEventInfo*  pEventInfo;
	unsigned short				isURLEncoded;
	unsigned short				isUTF8;
	const char*					target;
	const char*					hdrData;
	NO_OPERATOR_NEW_DELETE
};


enum Default_Port {
    kDEFAULT_PORT_NONE  = 0,
    kDEFAULT_PORT_FTP   = 21,
    kDEFAULT_PORT_HTTP  = 80,
    kDEFAULT_PORT_HTTPS = 443
};

enum FI_SocketStatus
{
	FI_SocketStatus_Connected,
	FI_SocketStatus_Closed,
	FI_SocketStatus_Failed
};

enum FI_SocketType
{
	FI_SocketType_TCPIP
};

struct FI_URLStreamInfo
{
    /*  URL info */
    const char*                 protocolSchema;
    const char*                 hostName;
    unsigned short              port;
    const char*                 documentPath;
    
    /*  HTTP post info (optional) */
    unsigned short              mustUsePost;
    const unsigned char*        postData;
    unsigned long               contentLength;
    const char*                 contentType;
    unsigned short				isURLEncoded;
    unsigned short				isUTF8;

    const char*                 additionalHeaders;

    /* Reload/Cache option */
    unsigned short               mustReload;

    struct FI_PlayerEventInfo*  pEventInfo;
	unsigned short				isPolicyFileRequest;
	unsigned short				isAssetRequest;
	unsigned short				isFap;
	unsigned short				isRTMPTClose;
	unsigned short				isFLVFileRequest;
	
    /*  Proxy info (optional - Macromedia Internal Use only) */
    const char*                 proxyProtocolSchema;
    const char*                 proxyHostName;
    unsigned short              proxyPort;
    
    /*  Cookie (optional - Macromedia Internal Use only) */
    const char*                 cookie;
	const char*					version;
	NO_OPERATOR_NEW_DELETE
};

enum FI_StreamMediaType
{
	FI_StreamMediaType_Unknown		= 0, /*/ unknown stream media type */
	FI_StreamMediaType_SWF			= 1, /*/ something recognized by flash (XML, SWF, loadVars, etc.) */
	FI_StreamMediaType_Image		= 2, /* image file */
	FI_StreamMediaType_Sound		= 3, /* sound file */
	FI_StreamMediaType_FLV			= 4  /* FLV file */
};

struct FI_NetworkBufferInfo
{
    unsigned char*  pBuffer;
    unsigned long   maxBufSize;
    unsigned long   dataSize;
	enum FI_StreamMediaType streamMediaType;
	NO_OPERATOR_NEW_DELETE
};

struct FI_URLStreamHeaderInfo
{
    unsigned int    numHeaders;
    char**          headerNames;
    char**          headerValues;
	NO_OPERATOR_NEW_DELETE
};

struct FI_SocketConnectionInfo
{
	unsigned long	socketId;
	unsigned short	socketType;
	const char*		hostName;
	unsigned short	port;
	unsigned short	isPolicyFileSocket;	
	NO_OPERATOR_NEW_DELETE
};

/*  This enumeration defines core configuration flags
 *  for flash lite dll
 *
 *  Our 32 bit configuration variable in the FI_Fncs will
 *  inform the player about the configuration information. Order
 *  starts from low-order bit 1.
 */
enum FI_ConfigFlag
{
    FI_4WayNavigation               = 0x00000001,       /* If bit 1 is set then use 4way navigation, otherwise 2way */
    FI_PlayerRendersText            = 0x00000002,       /* If bit 2 is set then the player will request character bitmaps to render text */
    FI_4WayWrap                     = 0x00000004,       /* If bits 1 and 3 are set then use a 4-way wrapping navigation mode */
    FI_FlashLiteSVG                 = 0x00000008,       /* If bit 4 is set, it indicates support for SVG content */
    /* Bits 5 through 7 in use */
	FI_JPEGDecode					= 0x00000080,		/* Attempt to decode JPEGS internally */
    FI_DynamicMemAvailable          = 0x00000100,       /* If bit 9 is set, it indicates that dynamic memory is available */
    FI_SharedObjects                = 0x00000200,       /* If bit 10 is set, it indicates that shared objects are enabled */
    FI_UseUTF16EncodingForm         = 0x00000400,       /* If bit 11 is set, then FI_Text stores UTF-16 chars */
	FI_UseHostDecoderForEmbeddedJPEGs = 0x00001000,		/* If bit 13 is set, use external decoder for embedded JPEGs */
    FI_UsePlayerEmbeddedVectorFont  = 0x00800000,       /* If bit 24 is set, support for player embedded vector fonts is enabled */
	FI_GcDemand				        = 0x01000000,       /* If bit 25 is set, then FSCommand2("gc") is supported */
	FI_EnableVideo					= 0x02000000,		/* If bit 26 is set, then video is supported - specific to DoCoMo request*/
	FI_ExternalVideoBuffering		= 0x04000000,
	FI_ExternalVideoRendering		= 0x08000000,
	FI_UseOpenVG 					= 0x10000000,		/* If bit 29 is set, Use OpenVG HW rendering instead of software rendering */
	FI_PluginMode					= 0x20000000,		/* If bit 30 is set, it indicates that the player is created to run in the context of a browser plug-in */
    FI_FingerTouch                  = 0x40000000        /* If bit 32 is set, Finger touch feature can be enabled. */

};


/* Error codes passed to SI_NotifySVGContentError */
enum FI_SvgContentErrorCode
{
    FI_SvgContentError_UnsupportedTag,
    FI_SvgContentError_UnsupportedAttribute,
    FI_SvgContentError_InvalidValue,
    FI_SvgContentError_NonMatchingEndTag,
    FI_SvgContentError_NonConformingVersionOrBaseProfile,
	FI_SvgContentError_HrefNotFound
};


/* Possible return values from FI_DoHitTest that provide information
 * about what is under cursor when
 */
enum FI_HitTestResult
{
    FI_NullHitPoint             = 0x0001,
    FI_ButtonHitPoint           = 0x0002,
    FI_EditTextHitPoint         = 0x0003
};


/* structs used for streaming sound support */

enum FI_SoundSampleRate
{
    FI_SoundSampleRate_5kHz     = 0x0001,
    FI_SoundSampleRate_11kHz    = 0x0002,
    FI_SoundSampleRate_22kHz    = 0x0003,
    FI_SoundSampleRate_44kHz    = 0x0004,
    FI_SoundSampleRate_8kHz     = 0x0005,
    FI_SoundSampleRate_16kHz    = 0x0006,
    FI_SoundSampleRate_32kHz    = 0x0007
};


enum FI_SoundSampleSize
{
    FI_SoundSampleSize_8bit     = 0x0001,
    FI_SoundSampleSize_16bit    = 0x0002
};


enum FI_SoundChannels
{
    FI_SoundChannels_mono       = 0x0001,
    FI_SoundChannels_stereo     = 0x0002
};

enum FI_StreamSoundFormats
{
    FI_StreamSound_Unsupported  = 0x00000000,
    FI_StreamSound_MP3          = 0x01010001,
    FI_StreamSound_ADPCM        = 0x01020000,
    FI_StreamSound_PCM          = 0x01030000,
    FI_StreamSound_PCMI         = 0x01030001,
	FI_StreamSound_AAC			= 0x01040000
};

        
struct FI_StreamSoundInfo
{
    unsigned short sampleRate;
    unsigned short sampleSize;
    unsigned short channels;
	NO_OPERATOR_NEW_DELETE
};


struct FI_StreamSoundSetupInfo
{	
    unsigned short bufferSize;
    unsigned short numBuffers;
    unsigned short numFormats;
    unsigned long* formatInfo;
	NO_OPERATOR_NEW_DELETE
};


struct FI_StreamSoundDecompressorInfo
{
    void*           data;
    unsigned long   dataLength;
    unsigned long   delay;
    unsigned short  reset;
	NO_OPERATOR_NEW_DELETE
};

enum FI_DeviceSoundStatus
{
	FI_DeviceSoundStarted,
	FI_DeviceSoundComplete,
	FI_DeviceSoundFailed	
};

/* typedefs/structs used for input text */

enum FI_InputTextFlags
{
    FI_InputTextPassword        = 0x0001,
    FI_InputTextMultiline       = 0x0002,
    FI_InputTextNumeric         = 0x0004,
    FI_InputTextAlpha           = 0x0008,
    FI_InputTextAlphaNumeric    = 0x0010,
    FI_InputTextLatin           = 0x0020,
    FI_InputTextNonLatin        = 0x0040,
    FI_InputTextNoRestriction   = 0x0000
};


enum FI_DeviceCapabilitesFlags
{
    FI_DeviceCap_Email          = 0x0001,
    FI_DeviceCap_SMS            = 0x0002,
    FI_DeviceCap_MMS            = 0x0004,
    FI_DeviceCap_LoadData       = 0x0008,
    FI_DeviceCap_Mouse          = 0x0010,
    FI_DeviceCap_Stylus         = 0x0020,
    FI_DeviceCap_MappableSoftKeys=0x0040,
	FI_DeviceCap_XMLSocket		= 0x0080,
	FI_DeviceCap_QwertyKB		= 0x0100 
};

/* indices of FlashLite interface functions */
typedef enum EFlashLiteInterfaceIndex
{
    EIndex_FICreatePlayer = 0,
    EIndex_FIGetDisplayFormat = 1,
    EIndex_FIGetVersion = 2,
    EIndex_FICheckSwfDataBufferValidity = 3,
    EIndex_EFlashLiteInterfaceIndexCount = 4
} EFlashLiteInterfaceIndex;


/* indices of FlashLite sound decoder library interface functions */
enum ESoundDllInterfaceIndex
{
    EIndex_CreateSoundDecompressor = 0, 
    EIndex_DecompressSound = 1,
    EIndex_SetupSoundDecompressor = 2,
    EIndex_DestroySoundDecompressor = 3,
    EIndex_GetSupportedFormats = 4,
    EIndex_SetupDecoderLibrary = 5
};


//#ifdef USE_ATS_REFERENCE
enum EATSTestInterfaceIndex
{
	EIndex_ATSSetupSIFunctions = 0,
	EIndex_ATSSetupTestLibrary = 1,
	EIndex_TIParseSpec = 2

};
//#endif //USE_ATS_REFERENCE

/*
 *  Structure to define LargeInteger
 */
struct FI_LargeInteger
{
    unsigned long lowPart;
    unsigned long highPart;
	NO_OPERATOR_NEW_DELETE
};


/*
 * Structs used for device video
 */
struct FI_Video
{
	unsigned long videoID;
	struct FI_Text* playSrc;
	void* videoData;
	unsigned long videoDataSize;
	NO_OPERATOR_NEW_DELETE
};

struct FI_Method
{	
	unsigned long method;
	unsigned short argc;
	struct FI_Text* argv;
	NO_OPERATOR_NEW_DELETE
};


/*
 * Structs used for device image decoding
 */
enum FI_ImageFormats
{
	// Note: Order needs to be maintained
    FI_ImageFormat_RGB_444,     /* 16 bit 0000RRRRGGGGBBBB */
    FI_ImageFormat_RGB_555,     /* 16 bit 0RRRRRGGGGGBBBBB */
    FI_ImageFormat_RGB_565,     /* 16 bit RRRRRGGGGGGBBBBB */
    FI_ImageFormat_ARGB_8888,    /* 32 bit A8R8G8B8         */
	FI_ImageFormat_ARGB_4444     /* 16 bit AAAARRRRGGGGBBBB */
	/*
	FI_ImageFormat_ARGB_4444 was added in the middle in FL3.1 but that causes problems in DC.
	Since other versions of FL will be using different values for different image formats.
	For e.g., FI_ImageFormat_RGB_565 is 2 in older FL but 3 in FL3.1.
	The solution to this was either to change DC to take care of FL3.1 which was messy or add the new enum values
	at the end. Chose the second option.
	*/
};

struct FI_ImageData
{
    void*           imageData;
    unsigned long   imageDataLength;
	NO_OPERATOR_NEW_DELETE
};

struct FI_ImageInfo
{
    unsigned short  width;
    unsigned short  height;
    unsigned short  hasTransparency;
	unsigned short upscaleFactor; // this is the factor, by which the core should upscale the image, before displaying it.
	NO_OPERATOR_NEW_DELETE
};

struct FI_DecodeImageInfo
{
    unsigned long   scanlineBytes;
    unsigned char*  imageBuffer;
    unsigned short  requestedFormat; /* one of FI_ImageFormats */
	void *          handle;			// Handle to the bitmap, if the bitmap memory is allocated by SI_BmMalloc.
									// otherwise NULL.
	NO_OPERATOR_NEW_DELETE
};

struct FI_ARGB8
{
	unsigned char blue;
	unsigned char green;
	unsigned char red;
	unsigned char alpha;
	NO_OPERATOR_NEW_DELETE
};

struct FI_ARGB16
{
	signed short blue;
	signed short green;
	signed short red;
	signed short alpha;
	NO_OPERATOR_NEW_DELETE
};

enum FI_BitmapPixelFormat
{
	FI_BitmapPixelFormat_bpp1_clutRGB,
	FI_BitmapPixelFormat_bpp2_clutRGB,
	FI_BitmapPixelFormat_bpp4_clutRGB,
	FI_BitmapPixelFormat_bpp8_clutRGB,
	FI_BitmapPixelFormat_bpp16_rgb565, 
	FI_BitmapPixelFormat_bpp32_argb8888, 
	FI_BitmapPixelFormat_bpp24_rgb666, // <MSB> RRRRRRGG GGGGBBBB 000000BB <LSB>
	FI_BitmapPixelFormat_bpp24_rgb888,
	FI_BitmapPixelFormat_bpp16_argb4444
};

enum FI_FillStyle
{
	FI_FillStyle_fillBitsClip		= 0x01,	// flag that a bitmap should be clipped to its edges, otherwise a repeating texture
	FI_FillStyle_fillBitsNoSmooth	= 0x02	// set if a bitmap should never be smoothed
};

enum FI_BlitTransformFlags
{
	FI_Blit_hasTransparency = 1,		// Per-pixel transparency in the source.
	FI_Blit_hasColorTransform = 2,		// Color Transform is applied.
	FI_Blit_hasAlphaOnly = 4			// The color transform has alpha multiplier only.
};

/*
 * Data used for 2d Hardware blitter acceleration.
 */
struct FI_BlitTransformation
{
	// If rotationDx <= 0 or rotationDy != 0 then the bitmap is being
	// rotated.
	// rotationDx and rotationDy are fixed point, 16.16.
	int rotationDx, rotationDy;
	// Transformation matrix. These four ints are fixed point 16.16 values.
	// if a <= 0 or d <= 0 then the bitmap is reflected.
	// if b != 0 or c != 0 then the bitmap is being sheared.
	struct Matrix
	{
		int a;
		int b;
		int c;
		int d;
	} matrix;
	enum FI_BitmapPixelFormat srcFormat;	// Pixel format of the source bitmap. 
	enum FI_DisplayFormat destFormat;		// Pixel format of the framebuffer. 
	int	 flags;								// A combination of the enum FI_BlitTransformFlags

	struct FI_ARGB16	colorMultiplier;	// Fixed Point 8.8 RGBA value specifying the color multiplier
	struct FI_ARGB16	colorOffset;		// RGBA value specifying the color offset.
	enum FI_FillStyle	fillStyle;			// One of enum FI_FillStyle

   NO_OPERATOR_NEW_DELETE
};

struct FI_FillOp
{
	struct FI_Rect	rect;		// Fill this rectangle... 
	struct FI_ARGB8	color;		// Alpha-Multipled color value.
	char*			destHandle;	// The handle to the destination returned by SI_BMMalloc, or NULL if allocated
								// by FlashLite memory pool.
	char*			destData;	// Pointer to the destination bitmap, if allocated out of FlashLite memory pool, 
								// otherwise NULL;
	int				rowBytes;	// Stride of the destination FrameBuffer
	enum FI_DisplayFormat 
					format;		// Pixel format of the framebuffer. 
	int				blend;		// Boolean value:Indicates whether the color is to be blitted or blended.
	NO_OPERATOR_NEW_DELETE
};

struct FI_BlitOp
{
	struct SourceImage
	{
		// Source information
		struct FI_ARGB8* clut;		// CLUT information if transformation.srcFormat is clutRGB type. 
		void*		handle;			// The handle to the source returned by SI_BMMalloc, or NULL if allocated
									// by FlashLite memory pool.
		const char *data;			// The source image pointer if allocated by FlashLite memory pool or NULL.
		int			rowBytes;       // Stride of the source image.
		int			x, y;			// Offset into the source to start rendering from, in Fixed Point 16.16 format
		int			dx, dy;         // Inverse Scaling factor in Fixed Point 16.16 format
		short		width, height;	// Size of the source image (in pixels).
	} src;
	// Destination information
	struct DestImage
	{
		char*	handle;			// The handle to the destination returned by SI_BMMalloc, or NULL if allocated
								// by FlashLite memory pool.
		char*	data;			// Pointer to the destination bitmap, if allocated out of FlashLite memory pool, 
								// otherwise NULL;
		int		x, y;           // Offset into the FrameBuffer (in pixels).
		int		rowBytes;       // Stride of the destination FrameBuffer
		short	width, height;  // Dimensions of the destination FrameBuffer (in pixels).    
	} dest;
	struct FI_BlitTransformation *transformation;   // Transformation information.
	NO_OPERATOR_NEW_DELETE
};

enum BLIT_CAPS { BLITCAPS_ALPHA = 1, BLITCAPS_COLORTRANSFORM = 2, BLITCAPS_SCALE = 4, BLITCAPS_TRANSLATE = 8};

/*
 *  This enum is passed as a parameter to SI_GetNetworkInfo 
 *  and SI_GetNetworkInfoString
 */
enum FI_NetworkInfoType
{
    FI_NetworkInfo_MobileNetworkCode = 0,
    FI_NetworkInfo_MobileCarrierCode,
    FI_NetworkInfo_NetworkName,
    FI_NetworkInfo_NetworkConnectionName,
    FI_NetworkInfo_MaxSignalLevel,
    FI_NetworkInfo_SignalLevel,
    FI_NetworkInfo_NetworkGeneration,
    FI_NetworkInfo_NetworkStatus,
    FI_NetworkInfo_NetworkConnectStatus,
    FI_NetworkInfo_NetworkRequestStatus
};

/*
 * This enum is passed as a parameter to FI_NotifySharedObjectStatus.
 */
enum FI_SharedObjectStatus
{
	FI_SharedObject_SizeRequestFailed,
	FI_SharedObject_SizeInfoAvailable,
	FI_SharedObject_ReadCompleted,
	FI_SharedObject_ReadFailed,
	FI_SharedObject_WriteCompleted,
	FI_SharedObject_WriteFailed
};


typedef enum _FI_TextFieldAction
{
	FI_TextFieldFirstAction = 0,

	FI_TextFieldCopy = FI_TextFieldFirstAction,
	FI_TextFieldCut,
	FI_TextFieldPaste,

	FI_TextFieldNumActions
}
FI_TextFieldAction;


/* 
 * FI_TextField constants
 */

enum FI_TextFieldBoundaryBehavior
{
	FI_TextFieldExitsFirstAndLastLineBoundary = 0x1,
	FI_TextFieldExitsStartAndEndBoundary = 0x2,
	FI_TextFieldCursorCyclesStartAndEndBoundary = 0x4
};

struct FI_Color
{
	unsigned char red;
	unsigned char green;
	unsigned char blue;
	unsigned char alpha;	
};

enum FI_TextField_StyleBit
{
	FI_TextField_StyleBit_Bold            = 0x1,
	FI_TextField_StyleBit_Italic          = 0x2,
	FI_TextField_StyleBit_Underline       = 0x4,
	FI_TextField_StyleBit_UnderlineStrong = 0x8,
	FI_TextField_StyleBit_UnderlineDashed = 0x10
};

struct FI_TextField_Style
{
	struct FI_Color textColor;
	struct FI_Color fieldColor;

	unsigned long mask;
	unsigned long bits;
};

struct FI_TextField_Run
{
	long start;
	long length;
};

struct FI_TextField_StyleRun
{
	struct FI_TextField_Style style;
	struct FI_TextField_Run run;
};

/*
 * SI_InvalidateIME constants
 */
enum _SI_InvalidateIME_Type
{
	SI_InvalidateIME_Type_Text = 0,
	SI_InvalidateIME_Type_Selection,
	SI_InvalidateIME_Type_MaxChars,
	SI_InvalidateIME_Type_Flags,
	SI_InvalidateIME_Type_Rect
};
typedef short SI_InvalidateIME_Type;

/*
 * FI_TextField methods
 */

struct FI_TextField;

typedef void (*FI_TextField_ShowCursor)(
	struct FI_TextField* pThis,
	unsigned int isCursorShown );

typedef void (*FI_TextField_GetSel)(
	const struct FI_TextField* pThis, 
	long* pSelStart,
	long* pSelEnd );

typedef void (*FI_TextField_SetSel)(
	struct FI_TextField* pThis,
	long selStart,
	long selEnd  );

typedef long (*FI_TextField_SetConversionString)(struct FI_TextField *textField,
												 const struct FI_Text *conversionString,
												 struct FI_TextField_StyleRun *styleRuns,
												 unsigned short styleRunCount,
												 long cursorPosition,
												 unsigned short isFinal);

typedef long (*FI_TextField_GetTextLength)(
	const struct FI_TextField* pThis );

typedef long (*FI_TextField_GetMaxChars)(
	const struct FI_TextField* pThis );

typedef enum FI_Result (*FI_TextField_GetText)(
	const struct FI_TextField* pThis,
	long startIndex,
	long endIndex,
	struct FI_ReturnString* pText );

typedef unsigned long (*FI_TextField_GetFlags)(
	const struct FI_TextField* pThis );

typedef void (*FI_TextField_GetRect)(
	const struct FI_TextField* pThis,
	struct FI_Rect* rect );

typedef void (*FI_TextField_GetCursorScreenCoordinates)( const struct FI_TextField* pThis,
													struct FI_Point* leftSideOfBaseLine,
													int* ascent,
													int* descent);
/*
 * FI_TextField interface
 */

struct FI_TextField
{
	FI_TextField_ShowCursor				ShowCursor;
	FI_TextField_GetSel					GetSel;
	FI_TextField_SetSel					SetSel;
	FI_TextField_SetConversionString	SetConversionString;
	FI_TextField_GetTextLength			GetTextLength;
	FI_TextField_GetMaxChars			GetMaxChars;
	FI_TextField_GetText				GetText;
	FI_TextField_GetFlags				GetFlags;
	FI_TextField_GetRect				GetRect;
	FI_TextField_GetCursorScreenCoordinates GetCursorScreenCoordinates;

	NO_OPERATOR_NEW_DELETE
};


/*
 * FI_TextField convenience macros
 */

#define FI_TextField_ShowCursor( pThis, a0 )			pThis->ShowCursor( pThis, a0 )
#define FI_TextField_GetSel( pThis, a0, a1 )			pThis->GetSel( pThis, a0, a1 )
#define FI_TextField_SetSel( pThis, a0, a1 )			pThis->SetSel( pThis, a0, a1 )
#define FI_TextField_SetConversionString( pThis, a0, a1, a2, a3 )	pThis->SetConversionString( pThis, a0, a1, a2, a3 )
#define FI_TextField_GetTextLength( pThis )				pThis->GetTextLength( pThis )
#define FI_TextField_GetMaxChars( pThis )				pThis->GetMaxChars( pThis )
#define FI_TextField_GetText( pThis, a0, a1 )			pThis->GetText( pThis, a0, a1, a2 )
#define FI_TextField_GetFlags( pThis )					pThis->GetFlags( pThis )
#define FI_TextField_GetRect( pThis, a0 )				pThis->GetRect( pThis, a0 )
#define FI_TextField_GetCursorScreenCoordinates( pThis, a0, a1, a2 ) pThis->GetCursorScreenCoordinates( pThis, a0,a1,a2 )


/*
 * FI_TextFieldCondition
 */

struct FI_TextFieldCondition
{
	unsigned long query;
	unsigned long negate;

	NO_OPERATOR_NEW_DELETE
};

/*
*Used in FI_Tsetproperty,FI_tgetproperty.
*/
struct FI_TargetPropertyInfo
{
	const char* pTargetIn;//UTF8 Encoding
	int propertyId;
	//As defined in the property table http://www.adobe.com/support/flash/publishexport/scriptingwithflash/scriptingwithflash_04.html
};
/*
* Used in FI_Methods that are called from the ActiveX Container to check if access is permitted. This is done by the core.
This structure is used because instead of sending these two values as separate parameters, they are wrapped into a single structure. Needs to be reviewed by Kiran,Shyam and Suudhan.
*/
struct FI_BrowserURLInfo
{
	const char* pURLIn;//UTF8 encoding.
	int pageAccessType;
	int bIsTrusted;
};
/*
 * Security context access types
 */
enum _eAccessRightType
{
	eNone = -1,
	eBasic = 0,
	ePremium,
	ePPV,
	ePrivileged,
	eSystem,
	eReserved
};

typedef int eAccessRightType; 

enum FI_FileMode 
{
	FI_ReadFileMode
	, FI_WriteFileMode
	, FI_ReadWriteFileMode
	, FI_AppendFileMode
};

enum FI_TestAutomationMouseEventType
{
	FI_TestAutomationMouseEventUnknown,
	FI_TestAutomationMouseDown,
	FI_TestAutomationMouseUp,
	FI_TestAutomationMouseMove,
	FI_TestAutomationMiddleMouseDown,
	FI_TestAutomationMiddleMouseUp,
	FI_TestAutomationRightMouseDown,
	FI_TestAutomationRightMouseUp,
	FI_TestAutomationMouseWheel,
};

enum FI_TestAutomationKeyEventType
{
	FI_TestAutomationKeyEventUnknown,
	FI_TestAutomationKeyUp,
	FI_TestAutomationKeyDown,
};

enum FI_ConfigurationFileType
{
	FI_SignedConfigurationFile,
	FI_ConfigurationFile,
};



enum FI_FileSeekOrigin
{
	FI_FileSeekStart,
	FI_FileSeekCurrent,
	FI_FileSeekEnd
};

enum FI_SoftkeyLocation
{ 
	FI_SoftkeyLocation_Top		= 0, 
	FI_SoftkeyLocation_Left		= 1,
	FI_SoftkeyLocation_Bottom	= 2,
	FI_SoftkeyLocation_Right	= 3
};

struct FI_AllocatorWrapper
{
	AMP_AllocatorContext*			pContext;
	AMP_Allocator_MallocProc		pMemAllocator;
	AMP_Allocator_FreeProc			pMemDeallocator;
};

struct FI_MetaDataInfo
{
	char* metaData;
	unsigned long metaDataLength;
	struct FI_AllocatorWrapper *pAllocatorWrapper;
};

enum FI_SecuritySandbox
{
	FI_LocalTrusted			= 1,
	FI_UseDefault			= 2
};

/*
 * This struct is used to pass information back from FI_TimeStampToDateTimeInfo()
 */
struct FI_DateTimeInfo
{
	unsigned short day;
	unsigned short month;
	unsigned short year;
	unsigned short weekDay;
	unsigned short hours;
	unsigned short minutes;
	unsigned short seconds;
	unsigned short milliseconds;
	NO_OPERATOR_NEW_DELETE
};

/*
 * This struct is used to return information about a SWF
 */
struct FI_SWFInfo
{
    long xmin;
    long xmax;
    long ymin;
    long ymax;
    unsigned short version;
    unsigned short numFrames;
	NO_OPERATOR_NEW_DELETE
};

#ifdef FEATURE_IMAGE_DOWNSCALE
const int KPROXY_IDENTIFICATION_STRLEN= 12; //length of "_IMAGEPROXY_"
const char KPROXY_IDENTIFICATION_STRING[] = "_IMAGEPROXY_"; 
const int MAX_SIZE_OF_IMAGE_PROXY=512;
#endif


enum FI_IpVersion
{
	IPversion4 = 0,
	IPversion6 = 1
};

struct FI_IpInfo
{
	union
	{
		unsigned char	v4[4];	//	For IPv4
		unsigned char	v6[16];	//	For IPv6
	}m_addr;
	
	enum FI_IpVersion m_type;
	unsigned short m_port;
};

#define NumberOfDoPlayCallsPerFrameInterval 	4

struct FI_FontSource
{
	int		isPlatformRendered;	/*(boolean) Whether it's a font that the device's font APIs know about. */
	int		shouldCopyData;	/* Whether player needs to copy the buffer or can just hold the pointer to the buffer*/
	const unsigned char *pFontData;	/* This is ROM based pointer to Font data*/
  	unsigned long fontDataLen;				/* length of font data pointed by pFontData pointer*/
	NO_OPERATOR_NEW_DELETE
};

struct FI_FallbackFont
{
	unsigned short	isScript;				/* if the value is zero it means it refer to enum FI_Script 
												else it refers to FI_Group defined in flashlitefontscriptdef.h*/
	unsigned short	id;						/* the font script or font group value either from FI_Script or FI_Group*/
	unsigned char	localeName[128];		/* locale Name */
	unsigned short	fontIdentifier[128];	/* font name - null terminated string*/
	NO_OPERATOR_NEW_DELETE
};

//Allow networking values
enum FI_AllowNetworking
{ 
	kAllowNetworkingAll, 
	kAllowNetworkingInternal, 
	kAllowNetworkingNone, 
	kAllowNetworkingMax 
};

#endif /* __COMMONDEF__H__ */

