/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash adaptation engine API Interface class 
*
*/

#ifndef __FLASH_ENGINE_H__
#define __FLASH_ENGINE_H__

#include <coedef.h>
#include <w32std.h>
#include "saplayer_build.h"
#include <caf/content.h>

// FORWARD DECLARATIONS

using namespace ContentAccess;

struct FI_Rect;
//class MFlashUI;
//class MFlashPlugin;
class MMp3StateObserver;

class MPluginMemHandlerIf;
class MUserNotiferIf;
class MPluginStreamIf;
class MDeviceExtnIf;
class MFlashClientIf;
class MDisplay;
class MFlashClientAppInfo;

#ifdef SELF_IAD_UPDATE
class MIadUpdateCallback;
#endif

#ifdef FEATURE_WEB_BROWSABILITY
	class MHostScriptHandler;
#endif
/** Specifies if SVG is supported. */
const TUint32 KSupportSVG				= 1 << 0;
/** Specifies if shared object is supported.*/
const TUint32 KSupportSharedObject		= 1 << 1;
/** Specifies if sound is supported.*/
const TUint32 KSupportSound				= 1 << 2;
/** Specifies if video is supported.*/
const TUint32 KSupportVideo				= 1 << 3;
/** Specifies if network connections are supported.*/
const TUint32 KSupportNetwork			= 1 << 4;
/** Specifies if local file is supported.*/
const TUint32 KSupportLocalFile			= 1 << 5;
/** Specifies if MMI supported.*/
const TUint32 KSupportMMI				= 1 << 6;


//Nokia S60 added flags
const TUint32 KSupportStereo			= 1 << 16;

const TUint32 KSupportScriptInteraction	= 1 << 17;

//used by flash viewer, no importance to host engine
//but defined here so that developers dont forget that
//this bit is used by upper layer
const TUint32 KDisableMenuRotateScreen	= 1 << 18;

const TUint32 KPlayerInATSMode = 1 << 19;
const TUint32 KPerformanceTestMode = 1 << 21;

// ES devices to set this bit on, In flash capabiities
// hasQuerty will be set based on this bit

const TUint32 KSupportQuertyKB		= 1 << 22;

//Added by Nokia S60
//Flags negated , by default these allowed.
//screensaver dont need this
/** Specifies no sound supported.*/
const TUint32 KSupportNoSound			= 1 << 30;

/** Specifies no vibration */
const TUint32 KSupportNoVibra			= 1 << 29;

/** Specifies no GETURL to be suported */
const TUint32 KSupportNoGetURL			= 1 << 28;

/** Specifies no backlight specific processing by host engine,
 client will handle */
const TUint32 KSupportNoBackLightProcessing	= 1 << 27;

/** Specifies no incoming call monitor needed
 client will handle */
const TUint32 KSupportNoCallMonitor			= 1 << 26;

/** Specifies no phone information like device name, id, subscriber info, network info requeired
	This will make flash engine to run faster. client app can set this flag if they dont need 
	etel related information
*/
const TUint32 KSupportNoETelInfo			= 1 << 25;

const TUint32 KSupportNoMakeCall		= 1 << 24;

/*
Specifies the instance to be run in trusted sandbox
*/
const TUint32 KSupportTrustedSandBox    = 1 << 23;

/**
 Structure containing configuration information for hostengine.
*/
struct FlashEngineConfig
{
	/** Size of static heap for the core player. */
	TUint32			iStaticHeapSize;
	
	/** Size of dynamic heap for the core player. */
	TUint32 		iDynamicHeapMaxSize;
	
	/** This field controls whether certain functionality is enabled/disabled.*/
	TUint32 		iMode;
	
	/** Number of soft keys supported.*/
	TUint16 		iSoftkeyCount;
	
	/** Name of the ECOM plug-in*/
	const TDesC8*	iExtensions;
	
	TUint16			iPlayerBlendsBackground;
	
	TUint16			iBackgroundAlpha;
	
	TBool			iHelixFlvMode;
	
#ifdef __FEATURE_PRE_COMPILED_SWF
	/** Name of the Pre-installed AS classes*/
	const TDesC8* iActScriptExtensions;
#endif //__FEATURE_PRE_COMPILED_SWF


TBool iSMCheckNeeded;
#ifdef SELF_IAD_UPDATE
	TBool          iCheckIADUpdate;
#endif

};

enum
{
	EZoomIn,
	EZoomOut,
	EShowAll
};

// Pause/Resume reason flags
enum {
	ENotPaused			= 0,
	EReasonFocus		= 1 << 0,
	EReasonUser			= 1 << 1,
	EReasonPhone		= 1 << 2,
	EReasonBackground	= 1 << 3,	
	EReasonTextInput	= 1 << 4,
	EReasonNetworkPrompt= 1 << 5,
	EReasonScrolling    = 1 << 6 //This can used to pause the plugin when browser is panning
};
//This is used to know the Execution Mode(Client Mode)
enum EBrPluginExecutionMode
{
    EDefaultMode,
    EWidgetMode
};

enum EBrPluginRenderingMode
{
    EBrowserRenderingMode,
    EFlashRenderingMode
};

enum EEngineExecutionMode
    {
    eUnknown =0,
    eStandAlone,
    ePlugin
    };

enum EMediaStreamType
{
    ENotMediaStream,
    EVideoStream,
    EAudioStream,
    ERTMPTStream,
    EOctetStream
};

class MFlashEngine
{
public:
	/**
	 Second phase constructor. 
	 @param aFlashUI Instance of the App UI.
	 @param aConfig Configuration information for hostengine.
	*/
    
    /*virtual void ConstructL(MFlashUI* aFlashUI, FlashEngineConfig& aConfig
#ifdef SELF_IAD_UPDATE
            , MIadUpdateCallback* aIADUpdateCallback
#endif
            ) = 0;*/
	/**
	 Destroys the hostengine instance.
	*/
	virtual void Destroy() = 0;
	
	virtual void ConstructL(FlashEngineConfig& aConfig,
	        MFlashClientIf* aClientInterfaces,
	        EEngineExecutionMode eExecutionMode,
	        EBrPluginExecutionMode aBrPluginMode =  EDefaultMode 
            ) =0;
	
#ifdef FEATURE_WINDOWLESS	
	virtual void SetWlessSurface(CFbsBitmap* aBitmap) =0;
#endif //FEATURE_WINDOWLESS	
	
	virtual void InitializeL(const TRect& aRect, TBool aResize, TBool aLandscape = EFalse) = 0;
	
	/**
	 Initializes the hostengine in standalone mode.
	 @param aRect Rectangle that defines the size of the display region.
	 @param aResize ETrue if resize has to be done.
	 @param aLandscape ETrue if landscape mode.
	*/
	//virtual void InitializeStandaloneL(const TRect& aRect, TBool aResize, TBool aLandscape = EFalse) = 0;
	
	// These are placeholders for future plug-in support.
	/**
	 Initializes the plugin
	 @param aControl Instance of the plugin control.
	 @param aRect Rectangle that defines the size of the display region.
	 @param aResize ETrue if resize has to be done.
	 @param aLandscape ETrue if landscape mode.
	*/
	//virtual void InitializePluginL(MFlashPlugin* aControl,const TRect& aRect, TBool aResize, TBool aLandscape = EFalse) = 0;
	/* Register as Mp3 state Observer
	 * @param MMp3StateObserver* aObserver observer for mp3 state
	 */
	//virtual void AddMp3StateObserver(MMp3StateObserver* aObserver)=0;
	/**
	 Processes the parameters passed to the HTML tag
	 @param aArgnames Name of the argument.
	 @param aArgvalues Value of the argument.
	*/
	virtual void ProcessParameters(CDesCArray* aArgnames, CDesCArray* aArgvalues) = 0;
	/**
	 Destroys the http engine, file engine, shared object engine and the core player.
	*/
	virtual void DestroyPlayer() = 0;
	
	virtual void PlayStep(TBool aCanInterrupt) = 0;
	/**
	 This function is called to play the swf file, when the file has been opened in the
	 embedded mode i.e. the swf file has been opened from some other application.
	 @param aFile Handle to the swf file.
	 @param resetBaseURL ETrue if base URL to be set.
	*/
	virtual void OpenFileAndStartMovieL(const TDesC& aFilename, TBool aResetBaseURL = EFalse, 
	                                    const TBool& aFirstMovie = ETrue, CContent* aContent = NULL ) = 0;
	virtual void OpenFileHandleAndStartMovieL( RFile& aFile, 
	                                           TBool aResetBaseURL = EFalse,
	                                           TBool aResetIAPAccess = ETrue, 
	                                           const TBool& aFirstMovie = ETrue,
	                                           CContent* aContent = NULL ) = 0;
	/**
	 Sets the buffer size for a particular stream
	 @param aStreamId Id of the stream.
	 @param aNetBuf Network buffer information.
	*/
	virtual void SetNetworkBufferL(unsigned short aStreamId, struct FI_NetworkBufferInfo* aNetBuf) = 0;
	/**
	 This function is used to write data to a stream.
	 @param streamId Id of the stream.
	 @param data Data to be written.
	 @param dataLength Length of the data.
	 @return ETrue if write is successful.
	*/
	virtual TInt WriteStreamDataL(unsigned long streamId, const char* data, unsigned long dataLength) = 0;
	/**
	 Closes the stream
	 @param aStreamId Id of the stream.
	 @param aClosedStatusCode Reason for closing the stream.
	*/
	virtual void CloseStream(TUint16 aStreamId, TInt16 aClosedStatusCode
								,EMediaStreamType aMediaStreamType=ENotMediaStream) = 0;
	/**
	     Closes the stream
	     @param aStreamId Id of the stream.
	     @param aClosedStatusCode Reason for closing the stream.	     
	    */
	virtual void StreamClosed(TUint16 aStreamId, TInt16 aClosedStatusCode) = 0;
	/**
	 Notifies whether phone is active
	 @param isActive ETrue if phone is active.
	*/
	virtual void NotifyFocusChangeL(TBool isActive) = 0;
	/**
	 This function is called by the browser plugin to play the swf file.
	 @param aFilename name of the swf file.
	 @param aResetBaseURL ETrue if base URL has to be set.
	 @param aBaseUrl Base URL.
	*/
	//TODO : Revisit this API and mgerge with OpenFileAndStartMovieL
	virtual void OpenBrowserFileAndStartMovieL(	const TDesC& aFilename,
												TBool aResetBaseURL,
												const TDesC* aBaseUrl,
												const TBool& aFirstMovie = ETrue,
												CContent* aContent = NULL) = 0;
	/**
	 Specifies if vibration is supported
	 @return ETrue if vibration is supported.
	*/
	virtual TBool IsVibrationSupported() = 0;
	/**
	 Specifies if browser plugin is active.
	 @return ETrue if browser plugin is active.
	*/
	virtual TBool IsBrowserPluginActive() = 0;

	/**
	 Recreats the bitmap. Sets the frame buffer and the display rectangle.
	 @param aRect Rectangle for the bitmap.
	*/
	virtual void RecreateBitmapL(const TRect& aRect) = 0;
	/**
	 The host application calls this function after initiating a successful URL data stream.
	 @param aStreamId id of the stream.
	*/
	virtual void NewURLStreamL(const char* aFinalURL, TUint aStreamId, TInt aContentLength, TDesC8& aContentType) = 0;
	/**
	 Stops the Flash Player.
	*/
	virtual void Stop() = 0;
	virtual void Pause(TInt aReason) = 0;
	virtual void ResumeL(TInt aReason, TBool aIgnoreActiveView) = 0;
	virtual void RewindL() = 0;
	virtual void SetInputTextL(const TDesC& aNewStr) = 0;
	virtual void CancelInputText() = 0;
	virtual void AbortPlayer() = 0;
	virtual void Draw(const TRect& aDirtyRect, CWindowGc& aWGc) = 0;
	virtual void RefreshScreen() = 0;
	virtual TKeyResponse OfferKeyEventL( const TKeyEvent& aKeyEvent, TEventCode aType ) = 0;
	virtual void OfferPointerEventL(TPoint& aPoint, TInt aType) = 0;
	virtual void SetForeground(TBool aForeground) = 0;
	virtual TBool PlayerStoppedWithError() = 0;
	virtual TBool IsInitialized() = 0;
	virtual void SetQuality(TInt aQuality) = 0;
	virtual TInt GetQuality() = 0;
	virtual TBool IsRunning() = 0;
	virtual TBool IsPaused() = 0;
	virtual HBufC* CreateEpocStringL( const char* aCString, TInt aLength ) = 0;
	virtual void SetVolume(TInt aVolume) = 0;
	virtual TInt GetVolume() = 0;
	virtual HBufC* GetFlashVersion() = 0;
	virtual TBool Zoom(TInt aMode, TInt& aLimitReached) = 0;
	virtual void SetPanMode(TBool aPanMode) = 0;
	virtual TBool IsZoomable() = 0;
	virtual TBool IsPanning() = 0;
	virtual TBool IsZoomed() = 0;
	//virtual TInt  StartShutdownSequenceReturnStepsToGo() =0;
	virtual void HandleSoftKeyLocationChangedL() =0;
	virtual void  ClearBitmap() =0;
	virtual CWsBitmap*     GetBitmap()=0;
    //Returns  the background colour if present
	virtual TRgb GetBgcolour()=0;
	virtual EEngineExecutionMode GetExecutionMode() const =0;
	virtual TBool GetFlvPlayState() const =0;
	//virtual void  SetExecutionMode(EInteractionMode)=0;
		
#ifdef FF_FLASHLITE_SCREENSAVER 
    /**
    Checks if the given file is valid or not
    @param aFilename The full path of the file whose validity is to be checked.    
    @return KErrNone if the file is valid. or one of the system wide error codes 
    */
    virtual TInt CheckFileValidityL(const TDesC& aFilename) = 0;
#endif    //FF_FLASHLITE_SCREENSAVER

#ifdef FEATURE_INLINE_TEXT_ENTRY
	virtual class AMP_MTextNavigation* GetTextNavigation( ) = 0;
	virtual void RestoreSoftKeys( ) = 0;
#endif // FEATURE_INLINE_TEXT_ENTRY
	
	/**
    Checks if the sound is playing
    @return ETrue if playing
    */
    virtual TBool IsSoundPlaying() const = 0;
	
	/**
    Returns the current frame number 
    @return the current frame number
    */
	virtual TInt GetCurrentFrameNumber() = 0;
	
	/**
    Returns the current frame number 
    @return the current frame number
    */
	virtual TInt GetTotalFrameNumber()=0;
	
	virtual CFbsBitGc* GetBitmapContext() = 0;
	
	virtual void SetBaseUrlL(const TDesC8* aBaseUrl) = 0;
	virtual void SetSecurityContextUrl(const TDesC8* iSecurityContextUrl) = 0;
	virtual TRect GetViewRectangle() = 0;

#ifdef FEATURE_WEB_BROWSABILITY
	
	virtual MHostScriptHandler* GetScriptHandler() = 0;
	
#endif


#ifdef ENABLE_CURSOR
	
	virtual void DrawCursor() = 0;
	
	virtual void ClearCursor(const TRect& aRect) = 0;
		
#endif

	virtual TBool IsControlPresent(const TPoint& aPoint) = 0;

	virtual TBool ControlPresent() = 0;
	
	virtual TBool IsHelixFlvMode() = 0;

	virtual void SetVideoDisplayRectL(const TRect& aVideoRect, const TRect& aClipRect) = 0;
	
	virtual void SetVideoWindow(RWindow* aWindow) = 0;
	
	virtual void SetEglContainerWindow(RWindow* aWindow) = 0;
	
	virtual void UpdateVideoScreen() = 0;
	
	virtual void PauseVideo() = 0;
	
	virtual void ResumeVideo() = 0;
	
	virtual void PlayVideo() = 0;
	
	virtual TRect GetVideoRectL()=0;

	virtual TInt32 GetFLVMaxDynHeapSize() =  0;
	
	virtual void OffsetPointToPan(TPoint& aPoint, TInt aType) = 0;
	
	virtual void SendPointertoSoftKey(TInt ) = 0;
	
	virtual void NotifyFullScreen(TBool aFullScreen) = 0;
	
    virtual TInt GetSwfHasVideo() = 0;
    
    virtual TInt GetSwfHasAudio() = 0;
    
	virtual TBool GetFullScreenViaFSCommand() = 0;
	
	virtual TBool IsOpenVgRenderingMode() = 0;

    virtual TBool IsRenderingToBitmap() = 0;	
    virtual TBool IsSwitchingToSWRenderingInPrgress() = 0;
    
#ifdef FEATURE_COLLECT_BITMAP
    virtual CFbsBitmap* GetLastRenderedBitmap() = 0;
    virtual TInt ToggleBrowserRendering(EBrPluginRenderingMode aMode) = 0;
    virtual void HandleWidgetError(TBool) = 0;
#endif
};

typedef MFlashEngine* (*PFNC_CreateFlashEngine)();
/*#ifdef __WINS__
#ifdef EKA2
_LIT(KFlashEngineLibraryName, "\\epoc32\\RELEASE\\winscw\\UDEB\\flash2engine_vibe.dll");
#else  //EKA2
#ifdef FEATURE_PACK_3
_LIT(KFlashEngineLibraryName, "flash2engine_fp3.dll");
#else // FEATURE_PACK_3
_LIT(KFlashEngineLibraryName, "flash2engine_vibe.dll");
#endif // FEATURE_PACK_3
#endif //EKA2
#else // __WINS__
_LIT(KFlashEngineLibraryName, "flash2engine.dll");
#endif // __WINS__
*/
_LIT(KFlashEngineLibraryName, "flash2engine.dll");
const TInt KCreateFlashEngineOrdinal = 1;

inline MFlashEngine* CreateFlashEngineL(RLibrary& aLibrary)
{
	MFlashEngine* pEngine = NULL;
	User::LeaveIfError(aLibrary.Load(KFlashEngineLibraryName));
	PFNC_CreateFlashEngine pFncCreateFlashEngine = (PFNC_CreateFlashEngine)aLibrary.Lookup(KCreateFlashEngineOrdinal);
	if (pFncCreateFlashEngine == NULL)
	{
		User::Leave(KErrGeneral);
	}
	else
	{
		pEngine = (MFlashEngine*)pFncCreateFlashEngine();
	}
	return pEngine;
}

#endif // __FLASH_ENGINE_H__
