/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash engine extensions Interface class 
*
*/

#ifndef __FLASH_EXTENSION_H__
#define __FLASH_EXTENSION_H__

#include <e32base.h>
#include <w32std.h>
#include <ecom/ecomresolverparams.h>
#include <ecom/ecom.h>

class CHttpRequestInfo;
class CFlashExtension;
class CPlatformFlashEngine;
class FI_Interface;
class MHostFncs;

struct SI_HostObject
{
	FI_Interface* iFIInterface;
	MHostFncs*	  iHostFncs;
};

/**
 * The MHostFncs class is provided to each extension to 
 * allow extensions to access some functionality 
 */
class MHostFncs
{
public:
	virtual short StartHttpRequestL(struct MM_Object* aPMMObj, CHttpRequestInfo* aInfo)=0;
	virtual short CancelHttpRequest(struct MM_Object* aPMMObj, TAny* aOwner, TUint aId)=0;
	virtual CFlashExtension* GetExtension(struct MM_Object* aPMMObj, TUid aUidOfExt)=0;
	virtual short SavePersistentDataL(struct MM_Object* aPMMObj,
                             const char* nameSpace, 
                             const char* name,
							 unsigned long aRequestId,
                             const unsigned char* data, 
                             const unsigned long dataSize,
                             TBool enforceMaxSize)=0;
    virtual CEikonEnv* GetEikonEnv()=0;
	virtual void Release()=0;
};

/**
 * The CFlashExtension class is used to store extensions to the main CPlatformFlashEngine functionality.
 * It is used to provide MMI functionality.
 */
class CFlashExtension : public CBase
{
public:
	// returns NULL on failure...
	static CFlashExtension* NewL(const TDesC8& aExtensionName);
	virtual ~CFlashExtension() { REComSession::DestroyedImplementation (iDtor_ID_Key);	}
public:
	TUid ExtensionUid() {return iExtensionUid;}
	virtual TInt Flags() = 0;
	virtual void InitExtensionSIFunctions(struct SI_Fncs*& aSI_Fns) = 0;
	virtual void InitExtensionL(MM_Object* aMMObj) = 0;
	virtual TBool HandleExtendedKeyEventL(MM_Object* /*pMMObj*/, const TKeyEvent& /*aKeyEvent*/, TEventCode /*aType*/) { return EFalse; }
	virtual void PlayerDestroyed() =0; // clean up any items that might do async callbacks to the player or player owned data
	virtual void SetAllowedSubExtensionsL(const TDesC8* aExtensions) = 0;
	virtual void NotifyPauseState(TInt aPauseFlags) = 0;
	virtual void NotifyIncomingCall(TBool aStatus) =0;
#ifndef __S60_32__
	virtual TAny* ExecuteFunctionL(TInt aOpCode, TAny* aParam1, TAny* aParam2, TAny* aParam3, TAny* aParam4)=0;
#endif
private:
	TUid iDtor_ID_Key;
protected:
	TUid iExtensionUid;
};

#endif // __FLASH_EXTENSION_H__
