/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
*
*/


#ifndef __HELIX_DEF_H
#define __HELIX_DEF_H

#include <e32base.h>
#include <w32std.h>
#include "flash_video_codec.h"
#include "saplayer_build.h"

typedef void (*DI_AllocHandler) (void* data);

// Define this to log A/V packets sent to XPS.
//#define __PACKET_LOGGING

// Define this to log rect values set to helix display window. 
// #define PRINT_RECT

// Log Macros
#define LOG(aString) 					DebugLog(_L(#aString));
#define LOG1(aString, a1) 				DebugLog(_L(#aString), a1);
#define LOG2(aString, a1, a2) 			DebugLog(_L(#aString), a1, a2);
#define LOG3(aString, a1, a2, a3) 		DebugLog(_L(#aString), a1, a2, a3);
#define LOG4(aString, a1, a2, a3, a4) 	DebugLog(_L(#aString), a1, a2, a3, a4);

#define IsOK(error) (error == KErrNone)

// Flag to switch on / off the XPS SINK of Helix.
#define _USE_SINK 


enum TUserMode
{
	EUnknownMode,
	EApplicationMode,
	EPluginMode
};

enum TCodecType
{
	ENoCodec			= 0,
	EJPEGCodec 			= VIDEO_CODEC_JPEG,
	ESorensonCodec 		= VIDEO_CODEC_SORENSON,
	EOn2Codec 			= VIDEO_CODEC_VP6,
	EAudioOnlyCodec		= AUDIO_ONLY_CODEC,
	EMp3Codec 			= 0x08,				// Chosen 0x08 so that it should not overlap.
	EADPCMCodec			= 0x09,			
	EH264Codec			= VIDEO_CODEC_H264,
	EAacCodec			= 0x0A
};

class TXPSRtpFlSrcConfig
{
public:
	TInt 	iPktDropPercentage;
	TBool	iEnableAuth;
	TBool	iEnablePktReorder;
};

enum TScreenMode
{
	ENormalScreenMode,
	EFullScreenMode
};

enum THXVideoDecCommand
{
	EHXVDCmdPlay,
	EHXVDCmdASPause,
	EHXVDCmdPause,
    EHXVDCmdResume,	
	EHXVDCmdStop,
	EHXVDCmdSetScreenMode,
	EHXVDCmdSetDisplayRect,
	EHXVDCmdSetVolume,
	EHXVDCmdSeek,
	EHXVDCmdReset,
	EHXVDCmdRestart,
	EHXVDCmdSetWindow,
	EHXVDCmdRefreshScreen,
	EHXVDCmdStartDSA,
	EHXVDCmdStopDSA,
	EHXVDCmdStreamEnd,
	EHXVDCmdEOF
};

enum TFrameType { EIFrame, EPFrame };

struct THXVDCmdArg
{
	TRect 		iDisplayRect;	// Used only in SetDisplayRect command.
	TRect		iClipRect;
	TInt  		iVolume;
	TScreenMode iScreenMode; 
	RWindow*	iWindow;
};

const TUint KNormalVideoClockRate = 1000;
const TUint KPartialPlaybackError = -12017;
const TUint KMillisecondsForASec  = 1000;

const TUint KCIFWidth 		= 352;
const TUint KCIFHeight 		= 288;
const TUint KQCIFWidth 		= 176;
const TUint KQCIFHeight 	= 144;
const TUint KSQCIFWidth 	= 128;
const TUint KSQCIFHeight 	= 96;
const TUint KQVGAWidth 		= 320;
const TUint KQVGAHeight 	= 240;
const TUint KQQVGAWidth 	= 160;
const TUint KQQVGAHeight 	= 120;

#endif // __HELIX_DEF_H
