/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash player HTTP adaptation defines 
*
*/

#ifndef __HTTP_REQUEST_INFO__
#define __HTTP_REQUEST_INFO__

#include <e32def.h>

#include <http/mhttpdatasupplier.h>
#include "commondef.h"
/**
 * The CHttpRequestInfo class is used to wrap up an HTTP request call from the core player.
 * The request can be queued until needed, or used right away depending on the policy
 * of the CHttpEngine and what other requests are currently being serviced.
 * 
 */
class CHttpRequestInfo : public CBase
{
public:
	~CHttpRequestInfo() 
	{
		delete iUrl;
		delete iPostData;
		delete iAdditionalHdrs;
	}

	// CompleteConstructL is ONLY to be called by the HttpEngine in all cases
	// even derived classes should not call this function...
	void CompleteConstructL();
protected:
	CHttpRequestInfo(void* aOwner, struct MM_Object* aMMObj, struct FI_URLStreamInfo* aStreamInfo,TUint32 aId)
	:iMMObj(aMMObj), iOwner(aOwner), iStreamInfo(aStreamInfo), iId(aId)
	{}

public: // accessor functions

	// getters
	inline const TDesC8& Url() const				{ return *iUrl; }
	inline const TDesC8* PostData() const			{ return iPostData; }
	inline const TDesC8* AdditionalHeaders() const	{ return iAdditionalHdrs; }
	// non const getters
	inline struct MM_Object* MM_Object()						{ return iMMObj; }

	virtual void SetContentLength(TInt /*aContentLength*/) 
	{		
	}
	TBool IsMe(void* aOwner, TInt aId) { return (aOwner==iOwner && aId==Id());}
	virtual void SetPaused(MHTTPDataSupplier*) {}
	virtual FI_StreamMediaType GetStreamMediaType()
		{
		// should be implemented in the derived classes
		return FI_StreamMediaType_Unknown;
		}
protected:
	// get the unique Id
	TInt Id() { return iId; }

public:
	// prepare any files/buffers etc that will need to be used for recording downloaded data
	// prior to the Http request being made...
	virtual void PrepareToBeServicedL()=0;

	// The request has failed - tell the client and prepare to be deleted.
	virtual void HandleRequestFailed(TInt aError, TInt aHttpStatus)=0;
	
	// once being serviced handle the fact that there is a header response...
	virtual void HandleHeaderResponseL(struct FI_URLStreamHeaderInfo& aHeaders, const TDesC8& aFinalUrl)=0;
	virtual TBool HandleResponseDataL(const TDesC8& aDataPart)=0;
	virtual void HandleResponseCompleteL(TInt aHttpStatusCode)=0; // request has completed normally

	TBool IsRTMPTClose() 
	{
		return iIsRTMPTClose;
	}
public: // link data
	TSglQueLink iLinkItem;
	
protected: // other data...
	struct MM_Object* iMMObj; // not owned
	HBufC8* iPostData;
	TBool		 iIsRTMPTClose;	// is the request RTMPT "/close"?
private:
	HBufC8* iUrl;
	HBufC8* iAdditionalHdrs;
	
	void* iOwner;
	struct FI_URLStreamInfo* iStreamInfo;
	TUint32 iId;
};

#endif // __HTTP_REQUEST_INFO__

