/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash Methods Handler class 
*
*/
 


#ifndef __MFLASH_METHODS_H__
#define __MFLASH_METHODS_H__

/*
enum TPageAccessType
{
	EPageUrlNormal,
	EPageUrlUnknown_AllowOutboundScriptAccess,
	EPageUrlUnknown_ProhibitOutboundScriptAccess
};
*/
#include "commontypes.h"

//Forward declarations
struct FI_Color;

//--------------------------------------------------------------------
//* Flash Methods
//* A Flash method is JavaScript function that is specific to Flash movies.
//* Use Flash methods to send JavaScript calls to Flash movies from 
//* a scripting environment. Each method has a name and most methods take arguments.
//* An argument specifies a value that the method operates upon.
//* The calculation performed by some methods returns a value that
//* can be used by the scripting environment
//-------------------------------------------------------------------------------

//Refer:http://www.adobe.com/support/flash/publishexport/scriptingwithflash/scriptingwithflash_04.html
// Methods return EFalse if not implemented/supported

class MFlashMethods 
{
    public: //methods
    
    // GetVariable( varName )    
	// Description:  Returns the value of the Flash variable specified by varName.
	// Returns: null if the variable does not exist. The argument type is string.    
	// Example:  var firstName = movie.GetVariable("FirstName");
	//  
    virtual TBool GetVariableL(const TDesC& aVariableNameIn 
    					, HBufC** aVariableValueOut 
    					, const TDesC& aURLIn
    					, TPageAccessType aPageAccessType
    					, TBool aIsTrusted) = 0;
	
	// Activates the frame number specified by frameNumber in the current movie.
	// If the data for a requested frame is not yet available,
	// the player goes to the last frame available and stops,
	// causing unexpected results during playback.
	// Use the PercentLoaded() method to determine if enough of the movie is 
	// available to execute the GotoFrame() method.
	// The argument frameNumber is zero-based;
	// that is, frameNumber is 0 in the first frame of the movie,
	// 1 for the second frame, and so on. This differs from the 
	// Goto action within Flash, which begins at 1. The argument type is integer.
	//
	//Example:  movie.GotoFrame(24); 					
	
	virtual TBool GotoFrame(TInt aFrameNumber) =0;
	
	// Returns true if the movie is currently playing.    
	// Example  if (movie.IsPlaying()) { alert("movie is playing"); }    
	//					
	virtual TBool IsPlaying()const =0;
	
	//  Loads the movie identified by url to the layer specified by layerNumber. 
	//  The argument type is integer for layerNumber and string for url. 
	//     
	// Example  movie.LoadMovie(0, "mymovie.swf"); 
	//				
		virtual TBool LoadMovieL( const TDesC& aSwfURL, TInt aLayerNo
							,const TDesC& aURLIn
							,TPageAccessType aPageAccessType
    						,TBool aIsTrusted) = 0;

	
	// Pans a zoomed-in movie to the coordinates specified by x and y.
	// Use mode to specify whether the values for x and y are pixels
	// or a percent of the window. When mode is 0, the coordinates are pixels;
	// when mode is 1, the coordinates are percent of the window.
	// Pan does not pan beyond the boundaries of the zoomed-in movie. 
	// The argument type for all arguments is integer.    
	// Example  This example pans 50% right and 50% down: movie.Pan(50, 50, 1) 
	//This example pans -25 pixels left and -25 pixels up:movie.Pan(-25, -25, 0) 
	//					
	virtual TBool Pan(TInt32 aXCoordinate,
					  TInt32 aYCoordinate
					  , TInt32 aMode) =0;
	
	// Returns the percent of the Flash Player movie that has streamed
	// into the browser so far; possible values are from 0 to 100.    
	// Example:  if (movie.PercentLoaded() == 100) { loaded = true; } 
	//
	virtual TBool PercentLoaded(TInt& aLoadPercent) =0;
	
	//Returns version of the flash.
	
	virtual TBool FlashVersion(HBufC** aVersion) =0; 
	
	// Starts playing the movie.    
	// Example  movie.Play(); 
	//
	virtual TBool Play() =0;
	
	// Goes to the first frame.    
	// Example  movie.Rewind(); 
	//
	virtual TBool Rewind() =0;
	
	// Sets the value of the Flash variable specified by variableName 
	// to the value specified by value. The argument type for both arguments
	// is string.    
	// Example  movie.SetVariable("/Form:UserName", "John Smith");
	// 
	virtual TBool SetVariableL( const TDesC& aVariableNameIn
	    				, const TDesC& aVariableValueIn
	    				, const TDesC& aURLIn
	    				, TPageAccessType aPageAccessType
	    				, TBool aIsTrusted) =0;
	
	// Zooms in on a rectangular area of the movie. 
	// The units of the coordinates are in twips (1440 units per inch). 
	// To calculate a rectangle in Flash, set the ruler units to Points
	// and multiply the coordinates by 20 to get twips. (There are 72 points per inch.)
	// The argument type for all arguments is integer.    
	// Example  This example zooms in on a 200 x 200 pixel rectangle in the 
	// upper left corner of the movie: 
	//
	// var pointsToTwips = 20;
	// movie.SetZoomRect(0, 0, 200 * pointsToTwips, 200 * pointsToTwips);
	//
	virtual TBool SetZoomRect(TInt aLeft
							, TInt aRight
							, TInt aBottom
							, TInt aTop) =0;
	
	// Stops playing the movie.    
	// Example movie.StopPlay() 
	//
	virtual TBool StopPlay() =0;
	
	// Returns the total number of frames in the movie.    
	// Example  var totalFrames = movie.TotalFrames(); 
	//
	virtual TBool TotalFrames(TInt& aNumberOfFrames) =0;
	
	// Zooms the view by a relative scale factor specified by percent.
	// Zoom(50) doubles the size of the objects in the view. 
	// Zoom(200) reduces the size of objects in the view by one half. 
	// Zoom(0) resets the view to 100%. 
	// You cannot specify a reduction in the size of objects in the view 
	// when the current view is already 100%. The argument type is integer.
	//    
	// Example movie.Zoom(50);  
	//
	virtual TBool Zoom(TInt aZoomPercent) =0;
	
	//TellTarget Methods
	//  the frame specified by frameNumber. 
	//     
	// Example  This example runs the actions in the fifth frame of the main timeline: 
	// 				movie.TCallFrame("/", 4);
	// 
	virtual TBool TCallFrameL( const TDesC& aTarget, 
						TInt aFrameNumber) =0;
	
	//  In the Timeline indicated by target, executes the action in the frame
	// specified by the label frame label. The argument type for both arguments
	// is string.
	//     
	// Example  This example runs the actions in the frame labeled 
	// 			HandleScriptNotify in the main timeline: 
	//			movie.TCallLabel("/", "HandleScriptNotify");
	//
	//
	virtual TBool TCallLabelL( const TDesC& aTarget
							,const TDesC& aLabel) =0;
	
	//  TCurrentFrame( target ) 
	//	Returns the number of the current frame for the timeline specified by target.
	//  The frame number returned is zero-based, meaning frame 1 of the Flash movie
	//  would be 0, frame 2 would be 1, and so on. The argument type is string.
	//    
	// Example  var currentFrame = movie.TCurrentFrame("/MovieClip"); 
	//
	virtual TBool TCurrentFrameL(  const TDesC& aTarget
								 , TInt& aCurrentFrame
								 , const TDesC& aURLIn
								 , TPageAccessType aPageAccessType
								 , TBool aIsTrusted) =0;
	
	//  CurrentFrame( ) 
	//  Returns the number of the current frame of the root     
	// Example  var currentFrame = movie.TCurrentFrame(); 
	  
	    virtual TBool CurrentFrame(TInt &frameNumber) =0;
	

	// Returns the label of the current frame of the timeline specified by target.
	// If there is no current frame label, an empty string is returned. 
	// The argument type is string.    
	// Example  var currentLabel = movie.TCurrentLabel("/MovieClip");    
	//
	virtual TBool TCurrentLabelL( const TDesC& aTarget
									, HBufC** aLabel
									, const TDesC16& aURLIn 
									, TPageAccessType aPageAccessType
									, TBool aIsTrusted) =0;
	
	//  For the timeline indicated by target, returns a string indicating 
	//  the value of the property specified by property. For property, enter the
	//  integer corresponding to the desired property. For a list of all properties
	//  and their corresponding integers, see Getting and setting properties. 
	//     
	//  Example  var nameIndex = 13;  
	//			var name = movie.TGetProperty("/", nameIndex);
	//
	virtual TBool TGetPropertyL( const TDesC& aTarget
								, TInt aPropertyID
								, HBufC** aPropertyValue
								, const TDesC& aURLIn
	    				 		, TPageAccessType aPageAccessType
	    				 		, TBool aIsTrusted) =0;
	// For the timeline indicated by target, returns a number indicating 
	// the value of the property specified by property. For property, 
	// enter the integer corresponding to the desired property. 
	// For a list of all properties and their corresponding integers, 
	// see Getting and setting properties. 
	//
	// var framesLoadedIndex = 12; 
	//
	// var framesLoaded = movie.TGetPropertyAsNumber("/", framesLoadedIndex);
	//
	//
	virtual TBool TGetPropertyAsNumberL( const TDesC& aTarget
										, TInt aPropertyID
						 				, HBufC** aPropertyValue
						 				, const TDesC& aURLIn 
	    				 				, TPageAccessType aPageAccessType
	    				 				, TBool aIsTrusted) =0;
	
	// For the timeline indicated by target, goes to the frame number
	// specified by frameNumber. The argument type for target is string. 
	// The argument type for frameNumber is integer. 
	//     
	//  Example : movie.TGotoFrame("/MovieClip", 2);  
	//
	virtual TBool TGotoFrameL(const TDesC& aTarget,
								 TInt aTargetFrame) =0;
	
	// For the timeline indicated by target, goes to the frame label 
	// specified by label. The argument type for both arguments is string. 
	//     
	// Example  movie.TGotoLabel("/MovieClip", "MyLabel");  
	//
	virtual TBool TGotoLabelL( const TDesC& aTarget
								,const TDesC& aLabel) =0;
	
	//  Plays the timeline specified by target. The argument type is string. 
	//     
	//  Example  movie.TPlay("/MovieClip");  
	//
	virtual TBool TPlayL( const TDesC& aTarget) =0;
	
	// For the timeline indicated by target, sets the value of the property 
	// specified by property to the value specified by value, 
	// which can be a string or a number. For property, enter the integer 
	// corresponding to the desired property. For a list of all properties
	// and their corresponding integers, see Getting and setting properties.
	//    
	// Example  var visibilityIndex = 7;var nameIndex = 13;
	// 			movie.TSetProperty("/MovieClip", visibilityIndex, 1);
	//			movie.TSetProperty("/MovieClip", nameIndex, "NewName");    
	//
	virtual TBool TSetPropertyL(	const TDesC& aTarget
							,TInt aPropertyID
						 	, const TDesC& aPropertyValue
						 	, const TDesC& aURLIn 
    						, TPageAccessType aPageAccessType
    						, TBool aIsTrusted) =0;
	
	// Stops the timeline specified by target. The argument type is string. 
	//     
	// Example  movie.TStopPlay("/MovieClipToStop"); 
	//
	virtual TBool TStopPlayL( const TDesC& aTarget ) =0;
    
    public: //Flash Properties
    
    virtual TBool	GetPropAllowScriptAccess(TInt& aAccessLevel)=0;
    
    virtual TBool 	GetPropBGColorL(FI_Color& aBGColor)=0;
    
    virtual TBool	GetPropQuality(TInt& aQuality)=0;
    
    virtual TBool	SetPropBaseURL(const TDesC& aBaseURL)=0;
   
	// AllowScriptAccess( value )    
	// Description:  Sets outbound script access level to perform outbound
	// scripting from withing the SWF. Outbound scripting is acheived through fscommnd
	// getURL or through External API interface
	// values taken are 0 for Always, 1 for Never,2 for sameDomain
	// Example movie.AllowScriptAccess = "Never"
	
    virtual TBool	SetPropAllowScriptAccessL(const TInt& aAccessLevel)=0;
    
    virtual TBool 	SetPropBGColorL(const FI_Color& aBGColor)=0;
    
    virtual TBool	SetPropQualityL(const TInt& aQuality)=0;
    
    virtual TBool	SetPropLoopL(const TInt& aLoop)=0;
   	
};

#endif //__MFLASH_METHODS_H__