/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
/*
 * pluginstreamImpl.h
 *
 *  Created on: Apr 9, 2009
 */

#ifndef PLUGINSTREAMIMPL_H_
#define PLUGINSTREAMIMPL_H_

class MPluginStreamIf
    {
public:
    /** This function is used to send a HTPP GET request to a URL without streaming
     @param aStreamId Id of the Stream.
     @param aUrl The URL to which the HTTP request has to be sent.
     @param aPostData HTTP Post method data, if any
     @return ETrue if successful and EFalse otherwise
     */
    virtual TBool OpenUrlL(const TDesC& aUrl, const TDesC& aPostData) = 0;
    
    /** This function is used to send a HTPP GET request to a URL without streaming
         @param aUrl The URL to which the HTTP request has to be sent.
         @param aTarget Window in which the result will be laoded.
         @param aPostData HTTP Post method data, if any
         @return ETrue if successful and EFalse otherwise
         */
    virtual TBool OpenUrlL(const TDesC& aUrl, const TDesC& aTarget, const TDesC& aPostData) = 0;

    /** This function is used to open a new stream
     @param aStreamId Id of the Stream.
     @param aUrl The URL to which the HTTP request has to be sent.
     @param aPostData HTTP Post method data, if any
     @return ETrue if successful and EFalse otherwise     
     */
    virtual TBool OpenStreamL(unsigned short aStreamId, const TDesC& aUrl,
            const TDesC& /*aPostDataContentType*/,
            TInt /*aPostDataContentLength*/, const TDesC& aPostData) = 0;

    /** This function is used to set the buffer size for a particular stream
     @param aStreamId Id of the Stream.
     @param aBufferSizeBuffer Size for this stream.
     @return ETrue if stream has been opened and EFalse otherwise
     */
    virtual void SetStreamBufferSizeL(unsigned short streamId,
            unsigned long aBufferSize) = 0;

    /** This function is used to cancel the stream from player engine
     @param aStreamId Id of the Stream.
     */
    virtual void CancelStream(unsigned short streamId) = 0;

    };

#endif /* PLUGINSTREAMIMPL_H_ */
