/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash player sound decompresser defines 
*
*/

#ifndef __SNDDEC__INCLUDE__
#define __SNDDEC__INCLUDE__

#include "platformtypes.h"
#include <e32base.h>

class CSoundDecomp: public CBase
{ 
public: 
	CSoundDecomp(FI_StreamSoundInfo* pSoundInfo);

    virtual void Setup(void* data, U32 dataLength, S32 delay, bool reset);
    virtual U32 Decompress(S16* /*dst*/, U32 /*n*/ ) { return 0; }       // must decompress into the format specified by snd
    virtual ~CSoundDecomp() {}
	U16 NumBytesPerSample() { return m_numBytesPerSample; };
	
protected:
    virtual void SetupSelf(bool /*reset*/) {}

	U16 m_sndSampleRate;
	U16 m_sndSampleSize;
	U16 m_sndChannels;
	U16 m_numBytesPerSample;
	U8*	m_sourceSamples;
	U32	m_dataLen;
	S32	m_delay;

};


//////////////////////////////////////////////////////////////////////////////////
//
// pcm decompressor class - this is raw uncompressed sound
//
class CPCMDecomp : public CSoundDecomp
{
public:
	CPCMDecomp(FI_StreamSoundInfo* pSoundInfo);

private:
	unsigned long	pos;
	unsigned long	bytesPerBlock;
    bool			swapBytes;

public:
    virtual unsigned long Decompress(S16* dst, U32 n);	// if dst == 0, just skip the requested samples

protected:
    virtual void SetupSelf(bool reset);
};


//////////////////////////////////////////////////////////////////////////////////
//
// adpcm decompressor class
//


class CAdpcmDecomp : public CSoundDecomp
{
public:
	CAdpcmDecomp(FI_StreamSoundInfo* pSoundInfo);
private:
	// Destination format - note we always decompress to 16 bit
	bool stereo;
	int nBits;		// number of bits in each sample

	long valpred[2];	// Current state
	int index[2];

	long nSamples;	// number of samples decompressed so far

	// Parsing Info
	unsigned long bitBuf;	// this should always contain at least 24 bits of data
	int bitPos;

	void FillBuffer();
	void SkipBits(unsigned long n);
	long GetBits(int n);
	long GetSBits(int n);

public:
	virtual unsigned long Decompress(S16* dst, U32 n);	// return number of good samples

protected:
	virtual void SetupSelf(bool reset);
};

#endif // __SNDDEC__INCLUDE__
