/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the MFlashPluginUI interface, implemented by the FlashPlugincontrol class.
*
*/


#ifndef __SYMBIAN_PLUGIN_INTERFACE_H__
#define __SYMBIAN_PLUGIN_INTERFACE_H__

#include "saplayer_build.h"
#include "symbian_ui_interface.h" // For TGetInputTextData and TMailInfo
#include "flashcursor.h"

/**
 MFlashPlugin interface is implememted by the FlashPluginControl class. This provides a set
 of functions, for the plugin interface, to the host engine.
*/
class MFlashPlugin
    {
    
/** This function is used to send a message
     @param aMtmUid Id of the Mtm.
     @param aAddress Address to which the message is to be sent.
     @param aBodyText Body Text
    */    
    //virtual void SendMessage(const TUid aMtmUid, char* aAddress, char* aBodyText) = 0;
    //virtual void SendMessage(const TUid /*aMtmUid*/,const TDesC& /*aAddress*/,const TDesC& /*aBodyText*/) = 0;
    // virtual void SendMessage(const TUid /*aMtmUid*/,const TDesC& /*aAddress*/,const TDesC& /*aCc*/,const TDesC& /*aBcc*/,const TDesC& /*aSubject*/,const TDesC& /*aBodyText*/) = 0;
    //virtual void SendMessage(const TUid /*aMtmUid*/,const TMailInfo& /*aMailInfo*/) = 0;

    /** This function is used to send a HTPP GET request to a URL without streaming
     @param aStreamId Id of the Stream.
     @param aUrl The URL to which the HTTP request has to be sent.
     @param aPostData HTTP Post method data, if any
     @return ETrue if successful and EFalse otherwise
    */
    //Moved to MPluginStreamIf
    //virtual TBool OpenUrlL(const TDesC& aUrl, const TDesC& aPostData) = 0; 
    /** This function is used to open a new stream
      @param aStreamId Id of the Stream.
      @param aUrl The URL to which the HTTP request has to be sent.
      @param aPostData HTTP Post method data, if any
      @return ETrue if successful and EFalse otherwise     
      */
        //Moved to MPluginStreamIf        
    //virtual TBool OpenStreamL(unsigned short aStreamId, const TDesC& aUrl,
     //                           const TDesC& /*aPostDataContentType*/,
       //                               TInt   /*aPostDataContentLength*/,                
         //                       const TDesC& aPostData) = 0;
    
    /** This function is used to set the buffer size for a particular stream
      @param aStreamId Id of the Stream.
      @param aBufferSizeBuffer Size for this stream.
      @return ETrue if stream has been opened and EFalse otherwise
      */
    //Moved to MPluginStreamIf
    //virtual void SetStreamBufferSizeL(unsigned short streamId, unsigned long aBufferSize) = 0;
    
    /** This function is used to cancel the stream from player engine
      @param aStreamId Id of the Stream.
      */
    //Moved to MPluginStreamIf
    //virtual void CancelStream(unsigned short streamId) = 0;
    
    /**  Gets the player control's window, if the player view has been created.
      @return handle to the player control's window.
     */
    //Moved to MScreenFurnitureIf
    //virtual RWindow& GetWindow() = 0;
    
    /**  
      @return 
     */
    virtual RWindow* GetNewVideoWindow(void* aClient) = 0;    

    /**  
      @return 
     */
    virtual void CloseVideoWindow(void* aClient) = 0;     
    
    // from textInput
    /**
     Specifies if the text is being input.
     @return TBool ETrue if the user has not yet dismissed the text input
     dialog.
    */
    //Moved to MInputTxtIf
    //virtual TBool IsTextInputBusy() = 0;    
    /**
     This function initiates a modal input text dialog, pausing the 
     movie until the user dismisses the dialog
     @param aData structure for the input text data.
     */
    //Moved to MInputTxtIf
    //virtual void GetInputTextL(TGetInputTextData& aData) = 0;
    /**
      Displays a confirmation query that the flash content is
    attempting to access the phone's serial number.
      @return ETrue if the user allows the access.
    */
   // virtual TBool QueryAllowDeviceIDL() = 0;
    /**
      Displays a confirmation query that the flash content is
    attempting to access the network.
    */
    //virtual TBool QueryAllowNetAccessL() = 0;
    
    /**
      Displays a confirmation query that the flash content is attempting
      to dial a phone number.
     @return ETrue if the user allows to make the call.
    */
    //virtual TBool QueryAllowPhoneCallL(const TDesC& aNumberString) = 0;
    
    
    /**
        Gets the plugin control's extent 
        @return The rect of the plugin control
    */
     //TODO : Move to MScreenFurnitureIf
    //virtual TRect GetRect() = 0;
    /*
#ifdef FEATURE_INLINE_TEXT_ENTRY

    /**
     Enables the IME Control for inline text editing with FEP
    */
        //Moved to MInputTxtIf
        //virtual void EnableIMEL() = 0;
    
    /**
     Disables the IME Control. 
    */
        //Moved to MInputTxtIf
        //virtual void DisableIME() = 0;
    
    /**
     Gets the current text input mode. 
    */
        //Moved to MInputTxtIf
        //virtual TInt GetCurrentInputMode() = 0;

//#endif  // FEATURE_INLINE_TEXT_ENTRY

/**
	 This function is called by the host engine when there is an error in
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/
	//virtual void NotifyFlashError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue) = 0;

	/**
	 This function is called by the host engine when there is an error in
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/
	//virtual void NotifyClientError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue) = 0;
	
	/**
	 This function is called by the host engine flash status
	 processing the flash content.
	 @param aErrorCode The error code.
	 @param aIsCritical ETrue if error is critical.
	 @param aPersistent ETrue is error is persistent.
	*/
	
	//virtual void NotifyFlashStatus(TInt aFlashStatus, TAny* aParam) = 0;
	
	/**
	 This function is called by the host engine to get feature capability
	 This  will be used to support fscommand for device capability
	 @return The capability values
	*/
	//Moved to MDeviceExtnIf
    //virtual TUint GetFeatureCapability()=0;
/*	
    virtual TUint GetMaxHeapSize() = 0;   
    
    virtual void  AddDynHeapAlloced(TUint size) = 0;
    
    virtual void  DynHeapFreed(TUint size) = 0; 
#ifdef PLUG_IN_MEM_ALLOC    
    
    virtual TBool IsMemAllocSupported() = 0;
    
    virtual TAny* AllocMem(TUint16 aNumBlocks) =0;
    
    virtual void FreeMem(TAny* aMemory)=0;
    
#endif
*/
        /*
#ifdef ENABLE_CURSOR
	//Moved to MFlashClientIf
	virtual MFlashCursor* Cursor() = 0;
	
#endif    
	*/
        //Moved to MFlashClientIf
	//virtual void DrawDirtyRect(const TRect& aDirtyRect) = 0;

    //Moved to MScreenFurnitureIf
	//virtual void SetFullScreenL(TBool aSetFullScreen) =0;

/**
	 This function is called by the host engine to inform FLV start
	 host can go full screen/ increase required heap memory
	 @return none
	*/
//public:
	//virtual void FLVideoStarted() = 0;

	//Moved to MScreenFurnitureIf
	//virtual TBool IsFullScreenMode() = 0;
	
	//Moved to MScreenFurnitureIf
	//virtual TRect GetClipRect() = 0;
	
	//virtual void  HandleVideoError(const TInt& aError) = 0;
	
	//Moved to MFlashClientIf
	//virtual void AddWsObserverToControl()= 0;	
	//virtual void RemoveWsObserverFromControl() = 0;
	
	//Moved to MDeviceExtnIf
	/*
#ifdef FEATURE_EXTENSION
	virtual void ResumeOnEventL(TBool aForeGround) = 0;
	virtual void PauseOnRequest(TBool aForeGround) = 0;
#endif	
	/*
	 * Engine queries the plugin to get the persistent directory
	 */
    virtual TInt GetPersistentDirectory(const TDesC& aSwfPath, TFileName& aPersistentDir) = 0;
    };

#endif //__SYMBIAN_PLUGIN_INTERFACE_H__
