/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash player client build related defines 
*
*/
#ifndef __SYMBIAN_SA_BUILD_H__
#define __SYMBIAN_SA_BUILD_H__

#include <e32std.h>

// This define switches on the menu option and the insert key press to
// simulate screen layout changes. It also turns on code to move the 
// softkeys clockwise on successive 'screen orientation' event.
// It should never be switched on for shipping code and is for testing purposes
// only when testing is done on hardware that doesn't support real screen 
// orienation/softkey location changes.
//#define FEATURE_DEBUG_SOFTKEY_LOCATION
#ifdef FEATURE_DEBUG_SOFTKEY_LOCATION
	#undef FEATURE_DEBUG_SOFTKEY_LOCATION
#endif //FEATURE_DEBUG_SOFTKEY_LOCATION

#ifdef FEATURE_ACHW
	#define ACHW
	#define ACHW_DRAWING_LIB ACHW_DRAWING_LIB_IS_OPENGL_ES
	#define ACHW_USE_SHAPE
	#define FLASHLITE_DISABLE_SVG
#endif // FEATURE_ACHW

//#define true 1
//#define false 0

#ifdef SYMBIAN_RGB888
#define PIXELFORMAT_ARGB8888
#endif

/*******************************************************************************/
//							IME: Dynamic Input Text
/*******************************************************************************/
// the definition of FEATURE_INLINE_TEXT_ENTRY is controlled by the project settings
#ifdef FEATURE_COMPLEX_FEP
#if !defined(FEATURE_INLINE_TEXT_ENTRY)
#define FEATURE_INLINE_TEXT_ENTRY
#endif 
#endif
/*******************************************************************************/

#ifdef FEATURE_INLINE_TEXT_ENTRY
	// The IME interface requires that the focus be set correctly
	// when the text editor is in focus
#	define FEATURE_EDITTEXT_FOCUS
#else
#	define FEATURE_FEP
#endif // FEATURE_INLINE_TEXT_ENTRY

#ifndef DLL_SINGLE_ENTRYPOINT
	#define DLL_SINGLE_ENTRYPOINT // this should always be defined in MMPs now.
#endif 
// The Symbian player should always be built with MMI on now, and the host will null
// out the SI function pointers if MMI is not dynamically loaded.
#define FEATURE_MMI

#define USE_SYSFONT

//Temporarily turned off support for Player Embedded Vector fonts
#undef USE_EMBEDDEDFONT

#define FEATURE_SHAREDOBJECT

#define FEATURE_EXTENDED_KEYS

#define FEATURE_SOUND
#define FLASHLITE_DEVICE_SOUND
//#define FLASHLITE_DEVICE_SOUND_ONLY

// cthilgen: device video
#define FEATURE_DEVICE_VIDEO

#define FEATURE_ACTIONSCRIPT_SLICING

// Flashlite new URL streaming API
#define FLASHLITE_URL_STREAM

#define FEATURE_AS_API

//#define FEATURE_WEB_BROWSABILITY
//
// SVG feature from flashlite-1.1-nokia
//
#ifndef EKA2 // for now EKA2 not supporting SVG. Not required by customer at this time.
#define FEATURE_SVG
#endif //EKA2
#ifdef FEATURE_SVG
    // Define this to enable true PNG decoding - used by SVG
    //#define FEATURE_SVG_PNG

    // Define this to enable gradient and opacity support
    #define FEATURE_SVG_GRADIENTS_AND_OPACITY

	//
	#define FEATURE_SVG_OBJECT_OPACITY

	#define FEATURE_ENHANCEDGRADIENTS
#endif // FEATURE_SVG


#define FLASHLITE_STYLUS
#define FLASHLITE_MOUSE

// Support Character encoding
//#define FLASHLITE_CHARACTER_ENCODING

// Support Character encoding
//#define FLASHLITE_UNICODE_ENCODING

#ifdef FEATURE_MMI
	#ifndef FEATURE_CLIENT_IMAGE_DECODING
		#define FEATURE_CLIENT_IMAGE_DECODING
	#endif // FEATURE_CLIENT_IMAGE_DECODING
	#define FEATURE_FILE_MANAGER
	#define FEATURE_APPLICATION_MANAGER
	#define FEATURE_MMI_REQUEST_DATA
#endif // FEATURE_MMI


#if defined(FLASHLITE_DEVICE_SOUND) && defined(FLASHLITE_DEVICE_SOUND_ONLY)
		#error "FLASHLITE_DEVICE_SOUND and FLASHLITE_DEVICE_SOUND_ONLY cannot be defined together"
#endif

#if defined ( SYMBIANFILEOUTPUT )
	void SYMBIANDEBUG ( int maxlength, const char* format, ... );
	void SYMBIANDEBUG ( int maxlength, const TUint8& format, ... );
	void SYMBIANDEBUG ( int maxlength, const TUint16& format, ... );

/*	#if defined ( DEFINE_FLASHCASTOUTPUT )
		void FLASHCASTOUTPUT ( const char* format, ... );
	#else
		#define FLASHCASTOUTPUT
	#endif*/
#else
	inline void SYMBIANDEBUG (...) { }
	inline void SYMBIANDEBUG ( int /*maxlength*/, const char* /*format*/, ... ) {}
	inline void SYMBIANDEBUG ( int /*maxlength*/, const TDesC8& /*format*/, ... ) {}
	inline void SYMBIANDEBUG ( int /*maxlength*/, const TDesC16& /*format*/, ... ) {}
//	#define FLASHCASTOUTPUT
#endif

#if !defined(TRAP_IGNORE)
	#define TRAP_IGNORE(_s) TRAPD(ignore,_s);
#endif //EKA2

// define this flag to turn on log message to a temp file
// This feature does not seem to work on devices at the moment.

//#define LOG_MESSAGE
//#define DEBUG_SOUND

//#define FL_TRACE_SUPPORT
//#define FL_TRACE_COUNTER

//Enable XML socket
#define FEATURE_XML_SOCKET

#define FEATURE_MP4_CONTAINER

#if !defined(SI_DEBUG_FILE_PATH_L)

#if defined(FL_TRACE_SUPPORT)
    #define SI_DEBUG_FILE_PATH_L _L("c:\\fltrace.txt")
#else
    #define SI_DEBUG_FILE_PATH_L _L("c:\\fl2_debug.txt")
#endif

#endif

#define SI_DEBUG_FILE_FLAGS (EFileWrite | EFileShareExclusive | EFileStream)
#define DEBUG_SUPPORT_KUID 0x2000DDDD

#include <eikenv.h>
#include "platformclib.h"

struct MM_Object;

#if defined(FL_TRACE_SUPPORT)

#define FL_TRACE_MAX_LENGTH 256

#define FL_TRACE(expression) CSymbianDebugSupport::Singleton()->SIDebugf expression
#define FL_TRACE_MM_OBJECT NULL
#define FL_TRACE_PREFIX NULL
#define FL_TRACE_BOOL(boolean) ((boolean) ? "YES" : "NO")

#else

#define FL_TRACE(expression) {}

#endif

#define C_SYMBIAN_DEBUG_SUPPORT_MAX_FORMAT_LENGTH 132

class CSymbianDebugSupport : public CCoeStatic, public TDes8Overflow
{
    public:
		static CSymbianDebugSupport *Singleton() 
        {
			CSymbianDebugSupport *result = KErrNone;
			TRAPD(errorCode, result = SingletonL());
			return (errorCode == KErrNone) ? result : NULL;
		}
		
		static CSymbianDebugSupport *SingletonL() 
		{
			CSymbianDebugSupport *result = STATIC_CAST(CSymbianDebugSupport *, CCoeEnv::Static(TUid::Uid(DEBUG_SUPPORT_KUID)));

			if (!result) 
			{
				result = new(ELeave)CSymbianDebugSupport();
				CleanupStack::PushL(result);
				result->ConstructL();
				CleanupStack::Pop();				
			}
			
			return result;
		}

	    CSymbianDebugSupport() : CCoeStatic(TUid::Uid(DEBUG_SUPPORT_KUID))
        {
			iIsFileOpen = FALSE;
			iTraceCounter = 0;
		}

		void ConstructL()
		{
		}

		void Overflow(TDes8 &) 
		{
			SIDebug("CSymbianDebugSupport::_OverflowHandler::Overflow()", NULL);
		}

		void SIDebug(const char *message, long number)
        {
			TPtrC8 msg((const unsigned char *)message, FlashStrLen(message));
			TBuf8<256> lineBuffer;

            #if defined(FL_TRACE_COUNTER)
				lineBuffer.AppendFormat(_L8("%04d "), iTraceCounter);
            #endif

            lineBuffer.Append(msg);

            #if defined(FL_TRACE_SUPPORT)
                number;
            #else
			    lineBuffer.AppendNum((TInt)number);
            #endif

            lineBuffer.Append('\r');
			lineBuffer.Append('\n');

			SpoolLine(lineBuffer);

			iTraceCounter++;
		}

		void SIDebugf(const char *format, ...)
        {
			TPtrC8 tFormat(REINTERPRET_CAST(const TUint8 *, format));
			const char *prefix = NULL;
			const char *c = prefix;
			VA_LIST vaList;
	
			VA_START(vaList, format);

			iFormatBuffer.Zero();

			#if defined(FL_TRACE_PREFIX)
			    prefix = FL_TRACE_PREFIX;
			#endif

			if ((prefix) && (*prefix))
			{
				while (*c) c++;
				iFormatBuffer.Append(REINTERPRET_CAST(const TUint8 *, prefix), c - prefix);
				iFormatBuffer.Append(_L8(" "));
			}
	
			iFormatBuffer.AppendFormatList(tFormat, vaList, this);
			iFormatBuffer.ZeroTerminate();

			VA_END(vaList);
	
			SIDebug(REINTERPRET_CAST(const char *, iFormatBuffer.Ptr()), NULL);
		}
		
		void SpoolLine(const TDesC8 &line)
        {
			if ((iIsFileOpen) || (_Open()))
			{
				iFile.Write(line);
				iFile.Close();
				iIsFileOpen = FALSE;
			}
		}

    protected:

		TInt _Reopen()
		{
			TInt errorCode = KErrNone;
			RFs &fsSession = CEikonEnv::Static()->FsSession();
			TInt position = 0;
			
			errorCode = iFile.Open(fsSession, SI_DEBUG_FILE_PATH_L, SI_DEBUG_FILE_FLAGS);
			
			if (errorCode == KErrNone) 
			{
				iFile.Seek(ESeekEnd, position);
			}
			else
			{
				errorCode = iFile.Create(fsSession, SI_DEBUG_FILE_PATH_L, SI_DEBUG_FILE_FLAGS);
			}

			return errorCode;
		}
		
		bool _Open()
		{
			TInt errorCode = KErrNone;
			RFs &fsSession = CEikonEnv::Static()->FsSession();
			
			if (iTraceCounter)
			{
				errorCode = _Reopen();
			}
			else 
			{
                #if defined(SI_DEBUG_FILE_APPEND)
			        errorCode = _Reopen();
                #else
			        fsSession.Delete(SI_DEBUG_FILE_PATH_L);
				    errorCode = iFile.Create(fsSession, SI_DEBUG_FILE_PATH_L, SI_DEBUG_FILE_FLAGS);
                #endif
			}
			
			iIsFileOpen = (errorCode == KErrNone);			
			
			return iIsFileOpen;
		}
		
        TBuf8<C_SYMBIAN_DEBUG_SUPPORT_MAX_FORMAT_LENGTH + 1> iFormatBuffer;
		int iTraceCounter;
		bool iIsFileOpen;
	    RFile iFile;
};

#endif	// __SYMBIAN_SA_BUILD_H__
#if defined(FEATURE_GCDEMAND) && !defined(FEATURE_MMI)
   #undef FEATURE_GCDEMAND
#endif /*defined(FEATURE_GCDEMAND) && !defined(FEATURE_MMI)*/

