/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Flash adaptation engine interface API for Viewer Fw 
*
*/

#ifndef __SYMBIAN_UI_INTERFACE_H__
#define __SYMBIAN_UI_INTERFACE_H__


#include <w32std.h>
#include <AknWaitDialog.h>


class RWindow;


enum TEngineState
{
    EEnginePlaying,
    EEnginePaused,
    EEngineResumed,
    EEngineStopped
};

enum TDataAccessErrorCode
{
	ECompleteFailedConnect = 0x1000,
	ECompleteFailedError,
	ECompleteFailedHTTP,
	ECompleteFailedFileNotFound,
	ECompleteFailedURLTooLong,
	EInvalidVideoFileError,
	EFailedPreparingVideoError
};


enum TClientErrorCode
{
	EInsufficientMemoryError			= 0x2000,
	EInvalidTelURLError,
	EInvalidFsCommandLaunchError,
	EGeneralError,
	EInvalidConfigError,
	EInvalidFileVersionError,
	EInsufficientGPUMemoryError
};


class TGetInputTextData
{
public:
	HBufC* iInitialString;
	TUint iMaxNumChars;
	TUint iTextFlags;
};

class TMailInfo
{
public:
 TPtrC16 iAddress;
 TPtrC16 iCc;
 TPtrC16 iBcc;
 TPtrC16 iSubject;
 TPtrC16 iBodyText;
};


union ErrorInfo
	{
	TInt iInfo;    
	};
/*
class MFlashUI
{
public:

	// from AppUI
	virtual TBool IsPlayerViewActive() = 0;
	// virtual void SendMessageL(const TUid aMtmUid, const TDesC& aAddress, const TDesC& aCc,const TDesC& aBcc, const TDesC& aSubject, const TDesC& aBodyText, TBool aLaunchEmbedded) = 0;
	virtual void SendMessageL(const TUid aMtmUid, const TMailInfo& aMailInfo,TBool aLaunchEmbedded)=0;
	virtual const TFileName GetAppPath() = 0;
	virtual void Quit() = 0;

	// from view
	virtual void SetFullScreenL() = 0;
#ifdef __FEATURE_OSK__
	virtual void SetOSKIsActive(TBool aShow) = 0;//enable or disable compatibility mode onscreen keypad
#endif
	
#ifdef FEATURE_EXTENSION
    virtual void SetAutoRotationMode(TBool aMode) = 0; //enable or disable Sensor autoroatation mode
#endif
	virtual void SetAppScreenL() = 0;
	virtual void ResetScreenInCurrentModeL() = 0;
	virtual void SetSoftkeysL(HBufC** aSoftkeyArray, const TInt aSoftkeyArraySize) = 0;
	virtual void ResetSoftkeysL() = 0;
	virtual void ClearErrors() = 0;
	virtual void NotifyFlashError(TInt aErrorCode, TBool aIsCritical, 
								  TBool aPersistent = ETrue,
								  ErrorInfo* aErroInfo = NULL) = 0;
	virtual void NotifyClientError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue) = 0;
	virtual void NotifyDataAccessError(const char* aUrl, TInt aErrorCode) = 0;
	virtual void NotifyPhoneActiveL(TBool aIsActive) = 0;
	virtual TRect ClientRect() = 0;
	virtual TRect ApplicationRect() = 0;
	virtual RWindow* GetWindow() = 0;
    virtual RWindow* GetNewVideoWindow(void* aClient) = 0;	
    virtual void CloseVideoWindow(void* aClient) = 0;

	// from textInput
	virtual TBool IsTextInputBusy() = 0;
	virtual void GetInputText(TGetInputTextData& aData) = 0;

	// misc
	virtual TBool ShowSharedObjectConfirmationQueryL() = 0;
	//virtual TBool QueryAllowDeviceIDL() = 0;
	virtual TBool QueryAllowPhoneCallL(const TDesC& aNumberString) = 0;
	virtual TBool QueryAllowNetAccessL() = 0;
	virtual void SaveIAPId(TUint32 aIAPId) = 0;
	virtual TUint32 IAPId() = 0;
	virtual void NotifyShutdownCountdown(TInt aItemsRemaining) = 0;
    virtual TBool GetClippingRect(TRect& aRect) = 0;
    virtual void  DrawDirtyRect(const TRect& aRect) = 0;
    
//   
//	 Displays a wait note that the user is connecting through the specified IAP
//	 @param aSelectedIAP The IAP selected by the user.
//	 @param aCallback MProgressDialogCallback sructure pointer
//	 @param aWaitDialog a Pointer to pointer to the newly created 
//	 CAknWaitDialog
//	
    virtual void CreateConnectingWaitNoteL(const TDesC& 
    aSelectedIAP, MProgressDialogCallback*  aCallback,
     CAknWaitDialog** aWaitDialog) = 0;
   	virtual TBool IsFullScreenMode() = 0;

#ifdef FEATURE_INLINE_TEXT_ENTRY
	virtual void EnableIMEL() = 0;
	virtual void DisableIME() = 0;
	virtual TInt GetCurrentInputMode() = 0;
#endif // FEATURE_INLINE_TEXT_ENTRY

#ifdef FEATURE_EXTENSION
	virtual void ResumeOnEventL(TBool aForeGround) = 0;
	virtual void PauseOnRequest(TBool aForeGround) = 0;
#endif
public:
// for relative path i.e. to get base url
	virtual TBool  IsUrlDescSet() = 0;
	virtual TPtr GetUrlDesc() = 0;
	virtual TBool CanContentBeTrustedL(const TPtrC pAccessedDomain)= 0;
	virtual void RestartPlayerL()=0;
	virtual void StartRestartTimerL()=0;

//	
//	 This function is called by the host engine to inform FLV start
//	 host can go full screen/ increase required heap memory
//	 @return none
//	
	virtual void FLVideoStarted() = 0;
	
//	
//     This function is called by the host engine to inform FLV end
//     @return none
//   
    virtual void FLVideoFinished() = 0;
	
    
//     This function is called by host engine to Zoom out the display
//     when any video content is present
//    
    virtual void ZoomShowAll() = 0;

//    
//	This Function is called by the host engine to Inform Swf Start host to set 
//	Landscape/portrait mode 
//	@return none
	
	
	virtual void SetLandScapeModeL(TBool aLandScapeMode) = 0;
#ifdef __FEATURE_FLV_RECOG_PLAY
public:	
		virtual RFile* GetRFileHandleForPlay() = 0;
#endif	

	
//	This Function is called by the host engine to inform the viewer that an 
//	error has occured in videoplayback
//	@return none
//	

	virtual void  HandleVideoError(const TInt& aError) = 0;

	
//	 * Need to notify any engine changes to the viewer so that it can take some action
	 
	
	virtual void NotifyEngineState(TEngineState aState) = 0;

	
	//virtual TInt GetPersistentDirectory(const TDesC& aSwfPath, TFileName& aPersistentDir) = 0;
};
*/


#endif //__SYMBIAN_UI_INTERFACE_H__
