/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/


#include <apmrec.h>
#include <apmstd.h>
#include <eikenv.h>
#include "recswf.h"
#include <implementationproxy.h>
#include "saplayer_build.h"

const TInt KUidMimeSwfRecognizerValue = 0x101F70EA;
const TUid KUidMimeSwfRecognizer = { KUidMimeSwfRecognizerValue };

_LIT8(KDataTypeSwf,"application/x-shockwave-flash");
_LIT8(KSwfFileFirstFewBytes1, "FWS");
_LIT8(KSwfFileFirstFewBytes2, "CWS");
// The file extension that shall be used by data we are recognizing
_LIT(KSwfFileExtension,".swf");

// The size of the data buffer that will be passed to the recognizer
// so that it performs the recognition
// 8 bytes required for f4v - the second word contains the id "ftybf4v"
#define KSwfRecognizerBufferSize 12


#ifdef __FEATURE_FLV_RECOG_PLAY
	_LIT8(KDataTypeFLV,"video/x-flv");
	_LIT8(KFLVFileFirstFewBytes, "FLV");
	_LIT(KFLVFileExtension,".flv");

/*
	#define TOTAL_RECOG_MIME_TYPE	2 //we recognize 2 mime types ( swf ,flv)
#else

	#define TOTAL_RECOG_MIME_TYPE	1 //we recognize only one mime type (swf)
*/
#endif

#ifdef __FEATURE_F4V_RECOG_PLAY
    _LIT8(KDataTypeF4VX,"video/x-f4v");
    _LIT8(KF4VXFileFirstFewBytes, "ftypf4v");
    _LIT(KF4VXFileExtension,".f4v");
    
    const TInt KF4V_FTYP_POS = 4;

/*
    #define TOTAL_RECOG_MIME_TYPE   3 //we recognize 2 mime types ( swf ,flv, f4v )
#else

    #define TOTAL_RECOG_MIME_TYPE   1 //we recognize only one mime type (swf)
*/
#endif

#define INDEX_SWF_TYPE  0
#define INDEX_FLV_TYPE  1
#define INDEX_F4V_TYPE  2

/*
#ifdef __FEATURE_FLV_RECOG_PLAY
    #ifdef __FEATURE_F4V_RECOG_PLAY 
        #define TOTAL_RECOG_MIME_TYPE 3
    #else
        #define TOTAL_RECOG_MIME_TYPE 2
    #endif
#elif __FEATURE_F4V_RECOG_PLAY
    #ifdef __FEATURE_FLV_RECOG_PLAY 
        #define TOTAL_RECOG_MIME_TYPE 3
    #else
        #define TOTAL_RECOG_MIME_TYPE 2
    #endif
#else
    #define TOTAL_RECOG_MIME_TYPE 1
#endif
*/
    
#if defined(__FEATURE_FLV_RECOG_PLAY) && defined(__FEATURE_F4V_RECOG_PLAY) 
        #define TOTAL_RECOG_MIME_TYPE 3
#endif
    
#if defined(__FEATURE_FLV_RECOG_PLAY) && !defined(__FEATURE_F4V_RECOG_PLAY) 
        #define TOTAL_RECOG_MIME_TYPE 2
#endif

#if defined(__FEATURE_F4V_RECOG_PLAY) && !defined(__FEATURE_FLV_RECOG_PLAY) 
        #define TOTAL_RECOG_MIME_TYPE 2
#endif

#if !defined(__FEATURE_F4V_RECOG_PLAY) && !defined(__FEATURE_FLV_RECOG_PLAY) 
        #define TOTAL_RECOG_MIME_TYPE 1
#endif
CSwfRecognizer::CSwfRecognizer()
	:CApaDataRecognizerType(KUidMimeSwfRecognizer,CApaDataRecognizerType::ENormal)
{
	iCountDataTypes=TOTAL_RECOG_MIME_TYPE; 
}

CSwfRecognizer::~CSwfRecognizer()
{
}

TUint CSwfRecognizer::PreferredBufSize()
{
	return KSwfRecognizerBufferSize;
}


TDataType CSwfRecognizer::SupportedDataTypeL(TInt aIndex) const
{
	switch(aIndex)
	{
		default:
					{
						return TDataType(KDataTypeSwf);
					}
		#ifdef __FEATURE_FLV_RECOG_PLAY					
		case INDEX_FLV_TYPE:
					{
						return TDataType(KDataTypeFLV);
					}
		#endif
        #ifdef __FEATURE_F4V_RECOG_PLAY                 
        case INDEX_F4V_TYPE:
                    {
                        // tell the symbian framework that we support f4v (datatype video/x-f4v)
                        return TDataType(KDataTypeF4VX);
                    }
        #endif
	}

}

#define TYPE_UNKNOWN 0
#define TYPE_SWF 1

#ifdef __FEATURE_FLV_RECOG_PLAY
#define TYPE_FLV 2
#endif

#ifdef __FEATURE_F4V_RECOG_PLAY
#define TYPE_F4V 3
#endif
void CSwfRecognizer::DoRecognizeL(const TDesC& aName, const TDesC8& aBuffer)
	{
    /* TBool nameRecognized = EFalse;
     * removing warning "Warning:  #550-D: variable "nameRecognized" was set but never used"
     */
     
    iConfidence = ENotRecognized;
    
    // Check that we got the required amount of data
   if(aBuffer.Length()<KSwfRecognizerBufferSize)
           return;
   
	TInt type = NameRecognizedL(aName);
	
	if( type != TYPE_UNKNOWN)
		iConfidence = EProbable;
	
	TInt dataType = HeaderRecognized(aBuffer);
	
	switch(dataType)
	{
		case TYPE_SWF:
				{
					iConfidence = ECertain;
					iDataType=TDataType(KDataTypeSwf);	
				}
				break;
		#ifdef __FEATURE_FLV_RECOG_PLAY	
		case TYPE_FLV:
				{
					iConfidence = ECertain;
					iDataType=TDataType(KDataTypeFLV);	
				}
				break;
		#endif
        #ifdef __FEATURE_F4V_RECOG_PLAY 
        case TYPE_F4V:
                {
                    iConfidence = ECertain;
                    iDataType=TDataType(KDataTypeF4VX);  
                }
                break;
        #endif
		
		default:
				{
					iConfidence = ENotRecognized;
				}
				break;
	}
	
}

//
// The file name recognition function. This functions checks whether the 
// provided filename matches our criteria (here we want it to have the .boot
// extension)
// \param aName: the name to check
// \return Mime Type index 0 for unknow, 1 for  swf , 2 for flv
//
TInt CSwfRecognizer::NameRecognizedL(const TDesC& aName)
{ 
  TInt res=TYPE_UNKNOWN;
  if(aName.Length()>4)
  {
     TInt dotPos = aName.LocateReverse( '.' );
     if (dotPos != KErrNotFound)
     {
        TInt extLength = aName.Length() - dotPos;
        HBufC* ext = aName.Right( extLength ).AllocL();
        CleanupStack::PushL( ext );
        if ( ext->CompareF(KSwfFileExtension) == 0 )
        {
          res = TYPE_SWF;
        }
        else
        {
            #ifdef __FEATURE_FLV_RECOG_PLAY
        	if ( ext->CompareF(KFLVFileExtension) == 0 )
        	{
        		res = TYPE_FLV;
        	}
            #endif
            #ifdef __FEATURE_F4V_RECOG_PLAY
            if ( ext->CompareF(KF4VXFileExtension) == 0 )
            {
                res = TYPE_F4V;
            }
            #endif
        }
        
        CleanupStack::PopAndDestroy(); // ext
     }
   }
   return(res);
}

//
// The data recognition function. This functions checks whether the 
// provided data starts with our data header
// extension
// \param aBuf: the data buffer to check
// \return TYPE_UNKNOWN not found, TYPE_SWF if swf data type,TYPE_FLV
//
TInt CSwfRecognizer::HeaderRecognized(const TDesC8& aBuf)
{ 
	if(aBuf.Find(KSwfFileFirstFewBytes1)==0)
		return TYPE_SWF;
	
	if(aBuf.Find(KSwfFileFirstFewBytes2)==0)
		return TYPE_SWF;
	
	#ifdef __FEATURE_FLV_RECOG_PLAY
	if(aBuf.Find(KFLVFileFirstFewBytes)==0)
		return TYPE_FLV;
	#endif
	
    #ifdef __FEATURE_F4V_RECOG_PLAY
     if(aBuf.Find(KF4VXFileFirstFewBytes)==KF4V_FTYP_POS)
         return TYPE_F4V;
     #endif
	return TYPE_UNKNOWN;

}


CApaDataRecognizerType* CSwfRecognizer::CreateRecognizerL()
    {
    return new (ELeave) CSwfRecognizer();
    }

const TImplementationProxy ImplementationTable[] =
    {
    IMPLEMENTATION_PROXY_ENTRY(KUidMimeSwfRecognizerValue, CSwfRecognizer::CreateRecognizerL)
    };

EXPORT_C const TImplementationProxy* ImplementationGroupProxy(TInt& aTableCount)
    {
    aTableCount = sizeof(ImplementationTable) / sizeof(TImplementationProxy);
    return ImplementationTable;
    }
    
//End of file
