/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
#ifndef OnScreenKeypad_H_
#define OnScreenKeypad_H_

#include <e32base.h>
#include <e32keys.h>
#include <w32std.h>
#include <flogger.h>
#include "OverlayKey.h"
#include <keypad_fl_t.mbg>
#include <akniconutils.h>
#include <bautils.h>

#include "../../inc/flash_engine.h"

const TInt KEY_INTERVAL=150;
_LIT( KMyFile, "_:\\resource\\apps\\keypad_fl_t.mif");

/*initial keypad sizes - not really used but recalculated using percentage values*/
const TInt KKeyVSpace               = 5;
const TInt KKeyHSpace               = 15;
const TInt KNavKeyH                 = 32;
const TInt KNavKeyW                 = 70;
const TInt KSoftkeyReductionFactor  = 25;
const TInt KInterKeySpace           = 0;

/*percentage values for keypad, as it is scalable*/
const TInt KPercentNavKeyH          = 29;
const TInt KPercentNavKeyW          = 30;
const TInt KPercentInterKeySpace    = 3;
const TInt KPercentKeyVSpacing      = 2;
const TInt KPercentKeyHSpacing      = 5;

/*backgroung color*/
const TInt KColorBlack              = 0;

enum TOSKKeyCodes
{
    EKeyPressNone                   = -1,
    EKeyPressLSK                    = -2,
    EKeyPressRSK                    = -3,
    EKeyPressBack                   = -4
};

class COnScreenKeypad : public CActive{
private:
    //startup position
    TInt                            iX;
    TInt                            iY;
    TInt                            iKeybWidth;
    TInt                            iKeybHeight;
    TInt                            iKeyVSpace;
    TInt                            iKeyHSpace;
    TInt                            iNavKeyH; 
    TInt                            iNavKeyW; //*MUST* be even
    TInt                            iSoftKeyH;
    TInt                            iSoftKeyW;
    TInt                            iSoftkeyReductionFactor;
    TInt                            iInterKeySpace;
    RPointerArray <COverlayKey>     iTheKeys;
    TInt                            iPressed; //index of the key that is pressed
    //softkey override details
    TBool                           iLSKOverride;
    TBool                           iRSKOverride;
    //back button
    TBool                           iBackButton;
    TBool                           iActive; //stores if the keypad is activated or not
    TBool                           iUpdateRequired; //is there a change in the drawing of the keypad   
    //for recurring keypresses
    RTimer                          iTimer;
    TInt                            iDuration;
    MFlashEngine*                   iTempEngine;
    
    TBool                           iQwertyMode;
public:
    //new
    static COnScreenKeypad* NewL();
    
    //From CActive
    void DoCancel(); //abort
    void RunL(); //RunL keeps sending KEYEVENT
    
    void SendPressedKey(MFlashEngine* aEngine = NULL); //this is the async function called in KEYDOWN    
    void AbortPressedKey(MFlashEngine* aEngine); //called on KEYUP
    
protected:
    void ConstructL();
    COnScreenKeypad();
    
public:    
    ~COnScreenKeypad();
    
    TRect GetDimensions(); //gets the dimensions of the keypad
    
    void SetKeyboardSizePortrait(TRect aDimensions);
    void SetKeyboardSizeLandscape(TRect aDimensions);
    void SetKeyboardSizeLandscapeWithQwerty(TRect aDimensions);
    
    void Activate(); //set iActive=true
    void Deactivate(); //set iActive=flase
    TBool IsActive(); //returns if the keypad is active or now   
    
    void SetDrawUpdate(TBool aVal); //set that an update is needed or not
    TBool GetDrawUpdate(); //is an update in the keypad needed
    
    void DrawKeypad( CWindowGc& mygc/*, TInt aX, TInt aY*/); //draw it
    TInt CheckKey(TInt aX, TInt aY); //returns index of pressed key, -1 if nothing
    TInt GetKeyCode(); //key code of pressed key
    TInt GetScanCode(); //scancode of pressed key
    
    void SetCustomSoftKeys(TBool aLsk, TBool aRsk);
    void SetBackButton(TBool aShow);
    
    TBool IsLSKOverride();
    TBool IsRSKOverride();
    
    TBool IsPressInKeypad(TInt aX, TInt aY);
    TInt GetiPressed(); //the index of the key that is pressed
    void ResetiPressed(); //sets iPressed to -1
    
    
    static void OSKDebug(TRefByValue<const TDesC> aText, ... ) {
        VA_LIST args;
        VA_START( args, aText );
        
        
        TBuf<512> buf;
        buf.FormatList( aText, args );

        RFileLogger logger;
        TInt ret=logger.Connect();
        if (ret==KErrNone)
            {
            logger.SetDateAndTime( EFalse,EFalse );
            logger.CreateLog( _L("FlashLogs"), _L("OSK_LogFile"), EFileLoggingModeAppend );       
            logger.Write(buf);
            }
        logger.Close();
        RDebug::Print(buf);
        VA_END( args );   
    }

};

#endif /*OnScreenKeypad_H_*/
