/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
#ifndef OVERLAYKEY_H_
#define OVERLAYKEY_H_

#include <e32base.h>
#include <e32keys.h>
#include <w32std.h>
#include <eikenv.h>
#include <akniconutils.h>
//#include <keypad_fl_t.mbg>

#include <flogger.h>

const TInt KColorDarkGrey       = 127;
const TInt KColorLightGrey      = 196;
const TInt KColorText           = 0;
const TInt KThickBorderWidth    = 3;
const TInt KBorderWidth         = 1;

class COverlayKey : public CBase {
private:
    //left value of rect
    TInt                iLeftVal;   
    //top value of rect
    TInt                iTopVal;  
    //right vlaue of rect
    TInt                iRightVal;  
    //bottom val of rect
    TInt                iBottomVal;
    //scan code
    TInt                iScanCode;
    //key code
    TInt                iKeyCode;
    //icon id from mif
    TInt                iIconID;
    //filename
    TFileName           iIconsFile;
    //bitmap
    CFbsBitmap*         iBitmap;
    //bitmap mask
    CFbsBitmap*         iBitmapMask;
    
public:
    //constructors
    //basic construction
    static COverlayKey* NewL (TInt aKeyCode, TInt aScanCode, TInt aIconID, TFileName aIconsFile);    
    //destructor
    ~COverlayKey ();
    //set new position for key
    void ResetPosition(TInt aLeft, TInt aTop, TInt aRight, TInt aBottom);
    //this function checks to see if the passed cordinates lie on the button
    TBool IsClicked(TInt aXval,TInt aYval);
    //draw the key with svg icon
    void DrawSVGKey(CWindowGc& gc, TBool aThickBorder);
    void DrawSVGKeyPressed(CWindowGc& gc/*, TBool aThickBorder*/);
    
    TRect GetKeyRect();
    TInt GetKeyCode();
    TInt GetScanCode();

private:
    void ConstructL();
    COverlayKey (TInt aKeyCode, TInt aScanCode, TInt aIconID, TFileName aIconsFile);
    
    static void OSKDebug2(TRefByValue<const TDesC> aText, ... ) {
    
        VA_LIST args;
        VA_START( args, aText );
        
        TBuf<512> buf;
        buf.FormatList( aText, args );
    
        RFileLogger logger;
        TInt ret=logger.Connect();
        if (ret==KErrNone)
            {
            logger.SetDateAndTime( EFalse,EFalse );
            logger.CreateLog( _L("FlashLogs"), _L("OSK_LogFile"), EFileLoggingModeOverwrite );       
            logger.Write(buf);
            }
        logger.Close();
        RDebug::Print(buf);
        VA_END( args );
    }
};


#endif /*OVERLAYKEY_H_*/
