/*
* Copyright (c) 2006-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/


#ifndef OVERLAYKEYPAD_H_
#define OVERLAYKEYPAD_H_

#ifdef __FEATURE_OSK__

#include <e32base.h>
#include <e32std.h>
#include <coecntrl.h>

#include "on_screen_key_observer.h"
#include "on_screen_keypad_constants.h"

class COnScreenKey;
class CButtonGrid;


enum TOSKMode
{
    EPortrait,
    EPortraitWithoutToggleScreenIcon,
    EPortraitWithOnlyToggleScreenIcon,
    ELandscape,
    ELandscapeWithoutToggleScreenIcon,
    EQwertyLandscape,
    EQwertyNoKeypad
};
enum TOSKKeys 
{
    EKeyPressDummy        = -1,
    EKeyPressLSK          = -2,
    EKeyPressRSK          = -3,
    EKeyPressToggleScreen = -4
};
class COnScreenKeypad : public CCoeControl, public MCoeControlObserver
{
public:
	static COnScreenKeypad* NewL(TPoint aLocation, TSize aSize, TInt aRows, TInt aCols, TOSKMode aMode,
                                MOnScreenKeyObserver* aOverlayKeyObserver, const CCoeControl& aContainer);
	~COnScreenKeypad();
	
	void UpdateL(TPoint aLocation, TSize aSize, TOSKMode aMode, TInt aRows=0, TInt aCols=0);
	void HandleForegroud();
	void DisableToggleScreen();
	void HideKeypad(TPoint aLocation, TSize aSize, TOSKMode aMode);

protected:
	COnScreenKeypad(TPoint aLocation, TSize aSize, TInt aRows, TInt aCols, TOSKMode aMode,
                   MOnScreenKeyObserver* aOverlayKeyObserver, const CCoeControl& aContainer);
	void ConstructL(const CCoeControl& aContainer);

private:	// from base classes
	void Draw(const TRect& aRect) const;
	void OfferKeyEvent(const TPointerEvent& aPointerEvent);
	void HandlePointerEventL(const TPointerEvent& aPointer);
	void PrepareForFocusLossL();
	void PrepareForFocusGainL();
	TInt CountComponentControls() const;
	CCoeControl* ComponentControl(TInt aIndex) const;
	
	// from MCoeControlObserver
	void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);

private:	// methods
	void PopulateKeysL();
	void PopulateKeysForPortraitModeL();
	void PopulateKeysForLandscapeModeL();
	void PopulateKeysForQwertyLandscapeModeL();
	void PopulateKeysForPortraitToggleScreenIconModeL();	
	
	void ClearAllKeys(); 
	void RemoveAllKeys();
	
	COnScreenKey* SearchKey(TInt aKeyCode);
	COnScreenKey* GetKeyL(TInt aKeyCode, TInt aScanCode, TInt aResourceId=0);
	void AddKeyL(TInt aKeyCode, TInt aScanCode, TInt aResourceId=0);
	void HideButton(TInt aKeyCode);
	
	COnScreenKey* GetUpButton();
	COnScreenKey* GetDownButton();
	COnScreenKey* GetLeftButton();
	COnScreenKey* GetRightButton();
	COnScreenKey* GetLSKButton();
	COnScreenKey* GetRSKButton();
	COnScreenKey* GetToggleScreenButton();
	
private: // members
	TPoint	iLocation;
	TSize	iSize;
	TInt	iRows;
	TInt	iCols;
	TOSKMode iMode;
	
	const CCoeControl&		iContainer;
    MOnScreenKeyObserver*   iOverlayObserver;

	RPointerArray<COnScreenKey> iKeys;
	CButtonGrid*				iButtonGrid;
};

#endif // __FEATURE_OSK__
#endif /* OVERLAYKEYPAD_H_ */
