/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description: 
*       Provides the CPlayerViewMenuManager defines.
*
*
*/


#ifndef __PLAYER_VIEW_MENU_MANAGER_H__
#define __PLAYER_VIEW_MENU_MANAGER_H__

//Forward declaration
class CSaFlashPlayerView;
class CSaFlashPlayerControl;
class CSaFlashAppUi;

#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
class CAknStylusPopUpMenu;
class MEikMenuObserver;
#endif

class CPlayerViewMenuManager : public CBase
{
public:
    /**
     * Symbian constructor.
     *
     * @since S60 v3.1
     * @return Pointer to created CPlayerViewMenuManager object
     */
    static CPlayerViewMenuManager* NewL(CSaFlashPlayerView* aView
    									,CSaFlashPlayerControl* aCtrl
    									,CSaFlashAppUi* aAppUI);
   
    /**
     * Destructor.
     */
    ~CPlayerViewMenuManager();
    
private:
    /**
     * C++ default constructor.
     *
     * @param aView CSaFlashPlayerView object.
     */
    CPlayerViewMenuManager(CSaFlashPlayerView* aView
    						,CSaFlashPlayerControl* aCtrl
    						,CSaFlashAppUi* aAppUI);
    
    /**
        * By default Symbian 2nd phase constructor is private.
        */
    void ConstructL();
    	
    	
public:
    /**
     * Init Menu items.
     * @since S60 v3.1
     * @param aResourceId   Id of the resource
     * @param aMenuPane   	MenuPane Object
     * @return None.
     */
    void DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane);
    
    /**
     * Init Menu bar.
     * @since S60 v3.1
     * @param aResourceId   Id of the resource
     * @param aMenuBar   	MenuBar Object
     * @return None.
     */
    void DynInitMenuBarL(TInt aResourceId,CEikMenuBar* aMenuBar);
    
//    void HandleCommandL( TInt aCommand );

#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
     void DynInitContextSensitiveMenu(MEikMenuObserver* aObserver, const TPoint& aPoint);
#endif

private: //methods
	
     void DynInitMenuPaneforVideoL(CEikMenuPane* aMenuPane );
     
     void DynInitMenuPaneforNoEngineModeL(CEikMenuPane* aMenuPane);

     void DynInitMenuPaneforRunModeL(CEikMenuPane* aMenuPane );

     void DynInitMenuPaneforNoRunModeL(CEikMenuPane* aMenuPane );

     void DynInitControlMenuPaneL(CEikMenuPane* aMenuPane );

//	 void RotateAppL();
	 
private: //atributes
	
    CSaFlashPlayerView* iView;
    
    CSaFlashPlayerControl* iCtrl;
    
    CSaFlashAppUi* iAppUI;
    
    TBool iIsTouchFeatureSupported;
    
#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
    CAknStylusPopUpMenu* iStylusPopupMenu;
#endif
			
};

#endif //__PLAYER_VIEW_MENU_MANAGER_H__


