/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the App Ui class.
*
*/


#ifndef __SYMBIAN_APPUI_H__
#define __SYMBIAN_APPUI_H__
#include "symbian_restarttimer.h"
// SELF_IAD_UPDATE
#include "saplayer_build.h"
#include <aknapp.h>
#include <aknViewAppUi.h>
#include <AknProgressDialog.h>
#include <sendui.h> // for CSendAppUi.
#include <content.h>
#include <AvkonInternalCRKeys.h> // KAknQwertyInputModeActive
#include <e32property.h> // for RProperty
#include "keypadhandler.h" // for CKeypadHandler
#include "symbian_ui_interface.h"
#include <flash_ui.h>
#include "flite.hlp.hrh"
#include "flash2ui.hrh"
#include "mp3streamstateobserver.h"
#include <ConeResLoader.h>

// FORWARD DECLARATIONS
using namespace ContentAccess;

class MFlashEngine;
class CSaFlashPlayerView;
class CSaFlashFileView;
class CSaFlashVolumeView;
class IniData;
class TextInput;

class MPluginMemHandlerIf;
class MPluginStreamIf;
class MFlashCursor;
class CAsyncDeleter;

#ifdef SELF_IAD_UPDATE
#include "flash_iad_update_callback.h"
#endif

#include "flashengineclientif.h"
#include "screenfurnitureif.h"
#include "uinotifierif.h"
#include "deviceextnif.h"
#include "inputtexthandlerif.h"
#include "flashvideoif.h"
#include "appinfoif.h"

struct FlashEngineConfig;


//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------
//  Class (TExitHandler) to allow execution stack to unwind before quitting.  This allows the debug
//  emulator complete without memory leak errors.
//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------
class CSaFlashAppUi;

class TExitHandler
{
private:
	TExitHandler() : iInsideEventHandler(EFalse), iExitEvent(EFalse) {}
	~TExitHandler() {}

	void StartEventHandler();
	void EndEventHandler(CSaFlashAppUi* aUi);
	void DoExit(CSaFlashAppUi* aUi);
	
	TBool iInsideEventHandler;
	TBool iExitEvent;

	friend class CSaFlashAppUi;
};
//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------



/*
 * class : CSaFlashAppUi 
 * description : <Need some desc>
 * derived from : 
 *              : MFlashUI (Desc needed)
 *              : CAknViewAppUi (Desc needed)
 *              : MProgressDialogCallback (Desc needed)
 *              : MIadUpdateCallback (Provides a callback which is called after the IAD update is complete)
 */

class CSaFlashAppUi : public CAknViewAppUi, /*public MFlashUI,*/ public MProgressDialogCallback, public MQwertyKeypadObserver, public MMp3StateObserver
#ifdef SELF_IAD_UPDATE
, public MIadUpdateCallback
#endif
, public MFlashClientIf
, public MUserNotiferIf
, public MDeviceExtnIf
, public MInputTxtIf
, public MFlashVideoClientIf
, public MDisplay
, public MFlashClientAppInfo
, public MAccessPointIf

{
public:
	void ConstructL();
	CSaFlashAppUi(const FlashUIConfig& aConfig);
	~CSaFlashAppUi();

public: // from MFlashClientIf
     virtual MFlashClientAppInfo* GetClientAppInfoIf() { return this; }
     virtual MDisplay* GetDisplay() { return this; }
     virtual MPluginMemHandlerIf* GetPluginMemHdlrIf() { return NULL; }
     virtual MUserNotiferIf* GetUserNotifierIf() { return this; }
     virtual MPluginStreamIf* GetStreamIf() { return NULL; }
     virtual MDeviceExtnIf* GetDeviceIf() { return this; }
     virtual MMp3StateObserver* GetMp3StateObserver() { return this; }
     virtual MFlashCursor* GetCursor() { return NULL; }
     virtual MInputTxtIf* GetInputTextIf() { return this; }
     virtual MFlashVideoClientIf* GetFlvClientIf() { return this; }
     TBool AllowAMCExtension() {return ETrue;}
     TBool IsWidgetMode() {return EFalse;}
     
#ifdef SELF_IAD_UPDATE     
     virtual MIadUpdateCallback* GetIADUpdateCallback() { return this; }
#endif

     virtual MAccessPointIf* GetIAPIf() { return this; }
     virtual TWindowMode GetWMode() { return EWindow; }
     //virtual EEngineExecutionMode GetExecutionMode() { return eStandAlone; }    
     void  DrawDirtyRect(const TRect& aRect);
     virtual void AddEventObserverToControl() {} //Do Nothing 
     virtual void RemoveEventObserverFromControl() {} //Do Nothing   
     virtual TInt GetPersistentDirectory(const TDesC& aSwfPath, TFileName& aPersistentDir);
     
     virtual void StartShutDownProcess();
     
public: // from MFlashUI
    
   // TBool CanContentBeTrustedL(const TPtrC pAccessedDomain);
   TBool CanContentBeTrustedL(const TDesC& pAccessedDomain);
    void RestartPlayerL();
    void KickoffRestartTimerL();
    void MP3PlayStateChangedL(EMP3StreamState aMP3StreamState);

	TBool IsEmbeddedModeOn() const;
	TBool IsPlayerViewActive();
	TBool IsVolumeViewActive();
	//void SendMessageL(const TUid aMtmUid, const TDesC& aAddress, const TDesC& aCc,const TDesC& aBcc, const TDesC& aSubject, const TDesC& aBodyText, TBool aLaunchEmbedded);
	void SendMessageL(const TUid aMtmUid, const TMailInfo& aMailInfo,TBool aLaunchEmbedded);//DesC& aAddress, const TDesC& aCc,const TDesC& aBcc, const TDesC& aSubject, const TDesC& aBodyText, TBool aLaunchEmbedded);
	void SendMessage(const TUid /*aMtmUid*/, char* /*aAddress*/, char* /*aBodyText*/) {}
	void SendMessage(const TUid /*aMtmUid*/, const TDesC& /*aAddress*/, const TDesC& /*aBodyText*/) {}
	TUint GetFeatureCapability() { return 0x00; }
	
	const TFileName GetAppPath();
	void CloseApplication();
	void Quit();
	TBool GetClippingRect(TRect& aRect);
#ifdef FEATURE_EXTENSION
    void ResumeOnEventL(TBool aForeGround);    
    void PauseOnRequest(TBool aForeGround);
#endif
#ifdef FEATURE_INLINE_TEXT_ENTRY
	void EnableIMEL();
	void DisableIME();
	TInt GetCurrentInputMode();
#endif // FEATURE_INLINE_TEXT_ENTRY

	TBool IsFullScreenMode();
	void SetLandScapeModeL(TBool aLandScapeMode);
#ifdef __FEATURE_OSK__
	void SetOSKIsActive(TBool aShow);
	TBool UserInitiatedFullScreen();
#endif // __FEATURE_OSK__

#ifdef FEATURE_EXTENSION 
    void SetAutoRotationModeL(TBool aMode);
#endif  	
	void SetScreenL(EScreenMode aMode);
	void SetFullScreenL(TBool aMode=EFalse);
	void SetAppScreenL();
	void ResetScreenInCurrentModeL();
	void SetSoftkeysL(HBufC** aSoftkeyArray, const TInt aSoftkeyArraySize);
	void ResetSoftkeysL();
	void ClearErrors();
	
	void NotifyError(TInt /*aErrorCode*/, TBool /*aIsCritical*/) {}
	void NotifyFlashStatus(TInt /*aFlashStatus*/, TAny* /*aParam*/) {}
	
	void NotifyFlashError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue,
							ErrorInfo* aErroInfo = NULL);
	void NotifyClientError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue);
	void NotifyDataAccessError(const char* aUrl, TInt aErrorCode);
	TRect ClientRect();
	TRect ApplicationRect();
	
	TRect GetClipRect()
	    {
	    return ApplicationRect();	    
	    }
	
	RWindow* GetWindow();

    RWindow* GetNewVideoWindow(void* aClient);
    void CloseVideoWindow(void* aClient);	
	// from textInput
	TBool IsTextInputBusy();
	void GetInputTextL(TGetInputTextData& aData);

	// Not Used
	//TBool QuerySharedObject();
	
	TBool QueryAllowDeviceIDL() {return ETrue; }
	
	TBool QueryAllowPhoneCallL(const TDesC& aNumberString);
	TBool QueryAllowNetAccessL();
    void CreateConnectingWaitNoteL(const TDesC& 
    aSelectedIAP, MProgressDialogCallback*  aCallback,
     CAknWaitDialog** aWaitDialog);
	TBool IsFullScreen();
	void SaveIAPId(TUint32 aIAPId);
	TUint32 IAPId();
	void NotifyShutdownCountdown(TInt aItemsRemaining);
//#ifdef __FLASHLITE2.0_REL3.1
	HBufC* GetFilename();
	void HandleResourceChangeL(TInt aType);
	void HandleResourceChangeInVolumeView(TInt aType);
	void NotifyPhoneActiveL(TBool aIsActive);
//#endif //__FLASHLITE2.0_REL3.1
		
public:
	void OpenFileL(const TDesC& aFileName);
	void OpenFileL(RFile& aFile);
	void SaveFileL(const TDesC& aDirectory);
	void ShowMuteIconL(TBool aMutedIconVisible);
	TBool CheckRightsL(const TDesC& aFilename);
	
	// Implements MEikMenuObserver
	void DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane);
	
	// new functions for send functionality
	void DoSendMenuL(CEikMenuPane* aMenuPane, TInt aAttachSize, TInt aItemBefore);
	void DoSendSubMenuL(CEikMenuPane* aMenuPane);
	void HandleSendCommandL(TInt aCommand, const TDesC& aFilename);
	
	// new function to launch Help app.
	void LaunchHelpL(TSAFlashHelpID aHelpID);
	TBool HelpAppExists();
	
	void AboutL();
	
	IniData* GetIniData();
	void SaveVolumeL(TInt aVolume);
	
	MFlashEngine* GetEngine();
	//TBool CheckPlayRightsL(const TDesC& aFilename);
	#if defined (__FEATURE_FLV_RECOG_PLAY) || defined(__FEATURE_F4V_RECOG_PLAY)
	TBool CheckPlayRightsL(CContent* aContent,TBool aIsVideo=EFalse);
	#else
	TBool CheckPlayRightsL(CContent* aContent);
	#endif
	TBool IsDRMRightsObjectMissing(const TDesC& aFileName);
	TBool  IsUrlDescSet();
	TBool IsStubApp();
	const TPtr GetUrlDesc();
	void SetUrlDescL(TPtrC& aDes);
	void SetAllowMove(TBool aMove = ETrue);
    TBool CanAllowMove();
	
	// This function is to get the present status(ON/OFF) of sliding keypad.
	TInt GetQwertyKeyPadStatus();
	
	#ifdef __FEATURE_FLV_RECOG_PLAY
public:	
	virtual RFile* GetRFileHandleForPlay();
#endif
	void  HandleVideoError(const TInt& aError);

public:
#ifdef FEATURE_TESTAUTOMATION
    virtual TRect PositionRelativeToDeviceScreen();
#endif

    
public: 
		HBufC* iurlDesc; 
		
		TBool isUrlDesc;
		
public: 
	TWsEvent iWsEvent;
#ifdef FEATURE_EXTENSION 
    TAppUiOrientation iFirstOrientation;
#endif
private:
	// Inherited from class CEikAppUi
	void HandleCommandL(TInt aCommand );
	TBool ProcessCommandParametersL(TApaCommand aCommand, TFileName& aDocumentName, const TDesC8& aTail);
	
	// from CAknAppUI
	void HandleForegroundEventL(TBool aForeground);
	void HandleWsEventL(const TWsEvent& aEvent, CCoeControl* aDestination);

	// from MAknProgressDialogObserver
	void DialogDismissedL(TInt aButtonId);

	// TCleanupItem function to re-show control
	static void RestorePlayerView(TAny* aPlayerView);
	
//	TBool QueryAllowDenyL(const TDesC& aQueryText);
	TBool QueryAllowDenyWithFormatResourceL(const TDesC& aQueryText,TInt aResId);

//	void StartShutdownProgressDialogL(TInt aNumItemsRemaining);

	static TInt ShutdownCallBack(TAny* aThis);
	TBool CheckAndProcessDRMFileL();
	TBool EnoughSpaceL(const TDesC &aTarget);

private: //methods
	void EnableSkinL();
	void LoadResourceL();
	TBool IsFlashViewerFeatureSupportedL();
	void FillFlashConfigL(FlashEngineConfig& aConfig);
	void CreateFlashViewsL(FlashEngineConfig& aConfig);
	TBool IsMemorySufficient();
	void OpenFlashContentL();
	void HandleMp3PlayingForegroundEventL(TBool aForeground);
private:// from MQwertyKeypadObserver
	//This function is notified whenever sliding keypad status is changed to (ON/OFF)
    void HandleQwertyModeChangeNotification(TInt aQwertyMode);
#ifdef FEATURE_EXTENSION 	
	TBool ScreenState();
#endif
	void DestroyEngine();
	void PrepareToExitL();
private:
    CRestartTimer* iRestartTimer;
	MFlashEngine* iEngine;
	HBufC16* iSavedFilename;
	CSaFlashPlayerView* iPlayerView;
	CSaFlashFileView* iFileView;
	CSaFlashVolumeView* iVolumeView;
	CSendUi* iSendAppUi;
	IniData* iIniData;
	TBool iHelpAppExists;
	TextInput* iTextInput;
	//TInt iResourceFileOffset;
	RArray<TInt> *iResourceFileOffset;
	FlashUIConfig iUIConfig;
	CAknProgressDialog* iProgressDialog;
	CEikProgressInfo* iProgressInfo; // not owned
	CIdle* iIdle;

	TExitHandler iExitHandler;
	friend class TExitHandler;
	RFile iFile;
	TBool iMMIEnabled;
	RConeResourceLoader iResourceLoader;
	TBool iMoveAllowed;
	TBool iHwKeyObserverRemoved;
	TBool iIsFlashStubApp;
	TBool trustedQueryAsked;	
	CKeypadHandler* iQwertyModeStatusSubscriber;
    RProperty iQwertyModeStatusProperty;
    TInt iQwertyMode;
    EMP3StreamState iMP3StreamState;
    TBool iForeground;
    CAsyncDeleter* iAsyncDeleter;
    TBool iQuitApp;
    
#ifdef SELF_IAD_UPDATE 
    TBool iIsIADFeatureEnabled;
#endif
    TInt iVideoContentCount;   //To keep track of no. of video content
	public: 
		TPtrC urlDesc; 
		
//		TBool isUrlDesc;

public:
	virtual void FLVideoStarted();
    virtual void FLVideoFinished();
    virtual void ZoomShowAll();  //Added to Zoom out in case any Video content is present
    virtual void VideoIsEmbedded(TBool aEmbeddedVideo);

#ifdef SELF_IAD_UPDATE
    virtual void IADUpdateCallback( TInt aErrorCode ); // from MIadUpdateCallback
#endif
    
    void NotifyEngineState(TEngineState aState);
};

#endif //__SYMBIAN_APPUI_H__
