/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Defines the file view class.
*
*/


#ifndef __SYMBIAN_FILEFIEW_H__
#define __SYMBIAN_FILEFIEW_H__

#include <coeccntx.h>
#include <aknview.h>
#include <aknenv.h>
#include <aknViewAppUi.h>
#include <aknapp.h>
#include <AknDoc.h>
#include <eikmenup.h>
#include <eikon.hrh>
#include <AknGlobalNote.h>
//#include <eikfrlb.h>
#include <sendui.h> // for CSendAppUi.
#include <DRMHelper.h>
#include <fbs.h>
#include <AknUtils.h>
#include <ConeResLoader.h>
#include <AknTabObserver.h>

class CDirectoryLocalizer;
//#include "f32file.h"
//#include "saflash.hrh"
#include "saflash_consts.h"

#include "symbian_view.h"


//#ifdef __FLASHLITE2.0_REL3.1
////////////////////////////////////////////////////////////////////////////////
// Constants
////////////////////////////////////////////////////////////////////////////////
const TInt KFileCountOne = 1;
const TInt KBufSize = 100;
const TInt KDeleteLength = 2;
const TInt KZero = 0;
const TInt KMaxFileNameLength = 255;
_LIT(KNullString, "");
//#endif //__FLASHLITE2.0_REL3.1
// forward declarations
class CSaFlashFileControl;
class CAknNavigationControlContainer;
class CAknNavigationDecorator;
class CAknTabGroup;
class MFlashEngine;
class CFlashErrorDialog;

NONSHARABLE_CLASS(TOverflowHandler) : public TDes16Overflow
	{
public:
    TOverflowHandler();
	void Overflow(TDes16& aDes);
	TInt IsOverflowed();
	
private:	
	int iError;
	};
class CSaFlashFileView : public CSaFlashView, public MAknTabObserver
{
//#ifdef __FLASHLITE2.0_REL3.1
public:
	enum TNewNameType
	{
		ENewFolder,
		ERenameFileOrFolder
	};
//#endif //__FLASHLITE2.0_REL3.1
public:
	CSaFlashFileView(MFlashEngine* aEngine, CFlashErrorDialog& aLowMemDialog);
	void ConstructL(TBool aIncludeSVG);
	~CSaFlashFileView();

public: // Methods for external interface

	TBool	IsShowDirsOnly() const { return iShowDirsOnly; }
	void	SetShowDirsOnly(TBool);

	void	GetCurrentDir(TDes &);
	void	GetSelectedPath(TDes &);

public: // from CAknView
	TUid Id() const;
	void DoActivateL(const TVwsViewId& aPrevViewId, TUid aCustomessageId, const TDesC8& aCustomMessage);
	void DoDeactivate();
	void HandleCommandL(TInt aCommand);

	// Implements MEikMenuObserver
	void DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane);
	void DynInitMenuBarL(TInt aResourceId, CEikMenuBar* aMenuBar);
    
    // from MAknTabObserver
    /**
    * called when tab index has changed.
    * default does nothing.
    *
    * @since 1.1
    * @param aIndex Selected tab index.
    */
    void TabChangedL(TInt aIndex);
public: // from CSaFlashView
	CCoeControl* GetContainer();

public: // Methods accessed by CSaFlashFileControl

	CAknTabGroup* TabGroup() { return iTabGroup; }
	
	// change tabs
	void SetTabL( TInt index );

	// notifies view that selection was changed
	void UpdateSelectionL( TInt index );
	void UpdateSelectionL( const TDesC& itemName );

	// notifies view that item was selected
	void ItemSelectedL();

	TBool AtTopLevel() { return (iNaviBmp == NULL); };
//#ifdef __FLASHLITE2.0_REL3.1
	void NotifyErrorL( TAknGlobalNoteType aType, 
	    const TDesC& aNoteText = KNullDesC, 
	    TInt aErrCode = 0 );
	void HandleResourceChange(TInt aType);
	void HandleResourceChangeL(TInt aType);
	MFlashEngine* GetEngine();
//#endif //__FLASHLITE2.0_REL3.1	
private: // implementation support methods
//#ifdef __FLASHLITE2.0_REL3.1
    	void IconConstructLC();
	void SizeChanged();
	void CreateFolderNaviL( TInt aFolderDepth );
//#endif //__FLASHLITE2.0_REL3.1	
	void SetNavigationPaneL();
	void SetNavigationTextFromPathL(TInt aMedia, TDes& aPath);
	void SetTitlePaneTextL(const TDesC& text);
	TBool BuildListBoxStringL(TEntry aEntry, TDes& aItem, TBool aIsDir,TBool aSwfExtn=EFalse);
	//void BuildListBoxStringL(TEntry aEntry, TDes& aItem, TBool aIsDir);
	void ExtractListBoxString(const TDesC& aItem, TDes& aText);
	TBool FindListBoxItemL(CDesCArray& aArray, const TDesC& aName, TBool aIsDir, TInt& aIndex);
	CDesCArray* CreateListBoxStringsForDirL(const TDesC& aDirPath);

	void UpdateListL(TBool aUpdateSelection);
	void ReplaceList();

	HBufC* PopPath(TDes& aStr);
	TBool TraverseUpL();
	void TraverseDownL();
	void DoHandleCommandL(TInt aCommand);
	void HandleDeleteCmdL();
	void HandleRenameCmdL();
	void HandleNewFolderCmdL();
	void HandlePasteCmdL();
	void HandleMoveCmdL();

	TInt AddItemToListL(const TDesC& aNewItem, CDesCArray &aArray);

	TBool GetNewNameL( const TDesC& aInitialName, TDes& aNewName, TNewNameType aType );
	void HandleCopyCmdL();
	void DisplayFileDetailsL();
	void CheckMemoryCard();
	TInt MemoryCardDrive();
	TBool FindItemExistsL();

	TBool ViewingPhoneDir() const { return ( iCurrentTab == 0 ); }
	TBool CurrentItemIsValid() const { return ( iCurrentItem.Length() > 0 ); }
	TBool CurrentItemIsDir() const { return iCurrentItemIsDir; }

	void ClearCurrentItem();
	void HandleError(TInt aErr);	
	void AddCountsL(TUint32 aCounter, CDesCArrayFlat* aItems);
	void AddStringPairL(TUint32 aTitleId, TUint32 aBodyId, 
		CDesCArrayFlat* aItems, TBool displayPlayText);
		
	void AddTimeLeftL(TUint32 aTitleId, TTimeIntervalSeconds aInterval, CDesCArrayFlat* aItems, TBool aDisplayPlayText);
	void AddDateAndTimeL(TUint32 aTitleId, TTime aDateTime, CDesCArrayFlat* aItems);
	void AddSingleValueL(TUint32 aTitleId, TUint32 aOne, TUint32 aTwoFour, 
		TUint32 aFiveZero, TUint32 aOneFinal, CDesCArrayFlat* aItems, 
		TUint8 aValue, TBool aDisplayPlayText);
		
	void AddDoubleValueL(TUint32 aTitleId,
	    TUint32 aFirstOne, TUint32 aFirstTwoFour, TUint32 aFirstFiveZero, TUint32 aFirstOneFinal,
	    TUint32 aSecondOne, TUint32 aSecondTwoFour, TUint32 aSecondFiveZero, TUint32 aSecondOneFinal,
	    CDesCArrayFlat* aItems, TUint8 aFirstValue, TUint8 aSecondValue, TBool aDisplayPlayText);
	    
	void AddSingleOrDoubleValueL(TUint32 aTitleId,
	    TUint32 aFirstOne, TUint32 aFirstTwoFour, TUint32 aFirstFiveZero, TUint32 aFirstOneFinal,
	    TUint32 aSecondOne, TUint32 aSecondTwoFour, TUint32 aSecondFiveZero, TUint32 aSecondOneFinal,
	    CDesCArrayFlat* aItems, TUint8 aFirstValue, TUint8 aSecondValue, TBool aDisplayPlayText);
	HBufC* FormatValueLC(TUint32 aOne, TUint32 aTwoFour, TUint32 aFiveZero, TUint32 aOneFinal, TUint8 aValue);

    CDir* GetFileDirSearchL(const TDesC& aDirPath
						   ,const TDesC& aSearchStr
						   ,TUint aSearchFilter);
	TBool BuildItemsL(CDir* aDir_Item
								 ,CDesCArrayFlat*& aItems
								 ,TBool aIsDir
								 ,TBool aIsSwfExtn );
	TBool DoLocalizeDirL(const TEntry& aEntry
						, const TDesC& aBasePath
						, TDes& aItem);
						
	TBool BuildListBoxStringforDirL(const TEntry& aEntry,
								TDes& aItem);
								
	TBool BuildListBoxStringforSwfL(const TEntry& aEntry,
									TDes& aItem,
									TBool aSwfExtn	);													  
	TPtrC StripFinalDot( const TDesC& aName );
	TBool IsValidName(const TDesC& aName);
//#endif //__FLASHLITE2.0_REL3.1

private: // Data
	MFlashEngine*	iEngine;
	CFlashErrorDialog& iLowMemDialog;
	TBool iHasMMC;
	TBool iMMCLocked;
	TBool iMMCCorrupted;
	HBufC* iSeparator;
	HBufC* iEndChar;
	CFbsBitmap* iPhoneIcon;
	CFbsBitmap* iPhoneIconMask;
	CFbsBitmap* iMMCIcon;
	CFbsBitmap* iMMCIconMask;
	CFbsBitmap* iFolderIcon;
	CFbsBitmap* iFolderIconMask;
	TInt iLevel;
	RConeResourceLoader iResourceLoader;
	CAknNavigationDecorator* iFolderNavi;
	
	// container for this view
	CSaFlashFileControl*	iContainer;
	
	// tab navigation pointers
	CAknNavigationControlContainer* iNaviPane;
	CAknNavigationDecorator* iDecoratedTabGroup;
	CAknNavigationDecorator* iNaviBmp;
	CAknTabGroup* iTabGroup;
	
	// working directories
	TBuf<KMaxFileName>	iPhonePath;
	TBuf<KMaxFileName>	iCardPath;

    // Rectangles where bitmaps are drawn ( 0 = rootbitmap, 1..4 = defaultbitmaps )
        TAknLayoutRect iBitmapLayout[5];
    // Rectangles where slashs are drawn ( 0..4 = \, 5 = ... )
       TAknLayoutText iTextLayout[6];
    
//#endif //__FLASHLITE2.0_REL3.1

	TBuf<KMaxFileName>	iCurrentItem;

	TBuf<KMaxFileName>	iSavedPath;	// saved by cut operation

	// list items in directory
	CDesCArray *	iPhoneDirItems;
	CDesCArray *	iCardDirItems;


	TInt	iCurrentTab;
	TInt	iCurrentItemIndex;
	TInt	iCurrentMMCItemIndex;
	TBool	iShowDirsOnly;
	TBool	iCurrentItemIsDir;
	TBool	iIsSavedPathValid;
	TBool	iIncludeSVG;
	TFileName iRootPhonePath;
	TFileName iRootMemCardPath;
	TBuf<KMaxFileNameLength> iNewFolderName;
	
	
	CDRMHelper* iDrmHelper;
	TBuf<KMaxFileName> iOrigName;
    TBuf<KMaxFileName> iNewName;
    TBuf<KMaxFileName> iNewPath;
	TBuf<KMaxFileName> iOrigPath;
    TBuf<KMaxFileName> iOldDir;
	TBuf<KMaxFileName> iNewDir;

	CDirectoryLocalizer*        iLocalizer;
};


#endif	// __SYMBIAN_FILEFIEW_H__
