/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
                                 
*
*/


#ifndef __SYMBIAN_PLAYERCONTROL_H__
#define __SYMBIAN_PLAYERCONTROL_H__

#include "symbian_sa_build.h"
// for __FEATURE_FLV_RECOG_PLAY
// and __FEATURE_F4V_RECOG_PLAY flags
#include "saplayer_build.h"
#include "hwkeywatchobserver.h"
#include "flashvolumecontrol.h"
#include <coecntrl.h>
#include <aknnavi.h>
#include <aknnavide.h>
#include <AknUtils.h>
#include <touchfeedback.h>

#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR 
#include <aknpointereventsuppressor.h>
#include "flashlongtapdetector.h"
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR

#include "commondef.h"

#ifdef FEATURE_INLINE_TEXT_ENTRY
#include <aknedsts.h>
#include <coeinput.h>
#include <fepbase.h>
#include <akntoolbar.h>
#ifdef __FEATURE_OSK__
#include "on_screen_key_observer.h"
#endif // __FEATURE_OSK__


//#	include <frmtlay.h>		// For TCursorSelection


/**
 * Zi FEP calls GetText with the whole text field as parameter
 * (T9 only for a portion).
 * This will cause a lot of copying if the text field is large.
 * On some devices this seems to cause the player slow down.
 * Virtual document shows the FEP (any FEP) only a portion of
 * the text field. It maps the cursor location. Hopefully
 * everything goes well and the FEP doesn't get confused.
 *
 * NOTE: Any navigation bugs or weird behavior, please undefine
 * this feature and try again.
 */
#ifndef VIRTUAL_DOCUMENT_FOR_FEP
// Spell dialog freezes on some phones.
// Keep on working on this.
//	#define VIRTUAL_DOCUMENT_FOR_FEP
#endif // VIRTUAL_DOCUMENT_FOR_FEP

class CSaFlashPlayerBaseIMEControl;
class AMP_MTextNavigation;
class CSaFlashPlayerIMEControl;
#endif // FEATURE_INLINE_TEXT_ENTRY

class CSaFlashPlayerView;
class CHwKeyPropertyWatch;
class CRepository;
class CVolumeDisplayTimer;
//class MFlashUI;
class MFlashEngine;
struct FlashEngineConfig;

#ifdef SELF_IAD_UPDATE
class MIadUpdateCallback;
#endif

class CSurfaceContainerControl;

class CSaFlashPlayerControl : public CCoeControl, public MCoeControlObserver, public MHwKeyWatchObserver
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR 
, public MFlashLongTapDetectorCallBack
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
#ifdef __FEATURE_OSK__
, MOnScreenKeyObserver
#endif // __FEATURE_OSK__
{
public:
	CSaFlashPlayerControl(CSaFlashPlayerView* aView);
	~CSaFlashPlayerControl();
    void ConstructL(/*MFlashUI* aFlashUI,*/ const TRect& aRect, FlashEngineConfig& aConfig
/*
#ifdef SELF_IAD_UPDATE
            , MIadUpdateCallback* aIADUpdateCallbackInterface
#endif
*/
            );
	
	void OpenFileL(const TDesC& aFilename);

	void OpenFileL(RFile& aFile);
	void HandleResourceChangeL(TInt aType);
	void HandleResourceChange(TInt aType);

	
	void UpdateScreenSizeL(const TRect& aRect);
	
	void FocusChanged(TDrawNow aDrawNow);
	
	void ResizeOtherControls();
	
	TBool IsEngineDead() { return (iEngine==NULL) ? ETrue : EFalse; }
	
	
public: //Virtual functions for MHwKeyWatchObserver
	  	virtual void MPausePlayKeyPressed() ;
	  
	  	virtual void MStopKeyPressed();
	  
		virtual void MRewindKeyPressed();
	
		virtual void MForwardKeyPressed();
			
		virtual void MBackwardKeyPressed();
			
		virtual void MFastForwardKeyPressed();

	    virtual void MHandleVolumeCmdL(TInt );
	    
	    virtual void MVolumeChanged(TInt );
	    virtual void MHandleVolumeRectRedraw();

public:
	RWindow* GetWindow() { return &Window(); } 
	RWindow* GetNewVideoWindow(void* aClient);
	void CloseVideoWindow(void* aClient);
	CSaFlashPlayerView* GetView() { return iView; };
	MFlashEngine* GetEngine() { return iEngine; }
	void EngineDestroyed() { iEngine = NULL; }
	void CancelFocusHandler();
    void ShowPauseIconL(TBool aPauseIconVisible);
	void ShowMutedIconL(TBool aMutedIconVisible);
	void HandleResourceChangeInVolumeView(TInt aType);
	void HandleResourceChangeInVolumeViewL(TInt aType);
	void IconConstructL();
	void ShowVolumeNaviPane(const TInt& aVolume);
	void ShowVolumeIconL(TBool aVolumeIconVisible);
	
	void ShowVolumePopupL(TInt );

	// Set and Get methods for current input language.
	TLanguage CurrentInputLanguage() 	{	return iCurrentInputLanguage ; }
	void SetCurrentInputLanguage(TLanguage aCurrentInputLanguage) 	{	iCurrentInputLanguage = aCurrentInputLanguage; }	
	void SimulateIMEEventL();

	void CancelPauseBlinker();
	void ResumePauseBlinker();
	void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);
	void ProvideFeedbackL(const TPointerEvent& aPointerEvent);

	#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
	void ToggleScreenL();
#endif // __FEATURE_UI_LANDSCAPE_FULLSCREEN
	TBool IsVolumeMuted(TInt aVolume);

	void VideoInPlay();
	void VideoNotInPlay();
	
	void CancelHwKeyObserver();
	void AddHwKeyObserverL();
	void CloseVolPopUp();

#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN       
    void HandleFixedToolbarVisibility(TBool ); 
#endif // __FEATURE_UI_LANDSCAPE_FULLSCREEN 
    void SetErrorOccured(TInt aError);
	void  HandleVideoError(const TInt& aError);

protected:
	
	void SizeChanged();
	
private:
    
    #ifdef __USE_EMPTY_NAVI__
    /**
    * Empty navi pane costructor. 
    * @since 2.8m
    */
    void CreateEmptyNaviPaneL( );
    #endif
    
    /**
    * Muted navi pane costructor. 
    * @since 2.8m
    */
    void CreateMutedNaviPaneL( );
    
    /**
    * Muted and paused navi pane costructor.
    * @since 2.8m
    */
    void CreatePausedMutedNaviPaneL( );
    
    /**
    * Paused navi pane costructor. 
    * @since 2.8m
    */
    void CreatePausedNaviPaneL( );
//#ifdef __SIDE_VOLUME_KEYS
    void ShowVolumeDisplayL(const TInt& aVolume);
    

//#endif //__SIDE_VOLUME_KEYS    
    /**
    * Timer callback for blinking paused icon.
    * @since
    * @param aPtr Pointer to timers callback 
    * @return KErrNone
    */
    static TInt BlinkPauseIconL(TAny* aPtr);

    /**
    * Blinks paused icon.
    * @since
    * @return void
    */
    void DoBlinkPauseIconL();
//#endif //__FLASHLITE2.0_REL3.1
	
    void LoadIconsL();

	
private:
	// Inherited from CCoeControl
	void Draw( const TRect& /*aRect*/ )  const;
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
	void HandlePointerEventL(const TPointerEvent& aPointerEvent);
	void ProcessFileL(const TDesC& aFilename);
	void ProcessFileL(RFile& aFile);
	
#ifdef __FEATURE_OSK__
	// from MOnScreenKeyObserver
public:
	TKeyResponse OfferOverlayKeyEvent(const TKeyEvent& aKeyEvent, TEventCode aType);
#endif // __FEATURE_OSK__

	TInt CountComponentControls() const;
	CCoeControl* ComponentControl(TInt ) const;
#if defined(__FEATURE_FLV_RECOG_PLAY) || defined(__FEATURE_F4V_RECOG_PLAY)
	TBool GetVideoPlayerPath(TDes& aFileName);
#endif
	
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
	
    //Inherited from MFlashLongTapDetectorCallBack
    virtual void HandleFlashLongTapEventL( const TPoint& aPenEventLocation, 
                                      const TPoint& aPenEventScreenLocation );
	
	
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR

	
#ifdef FEATURE_INLINE_TEXT_ENTRY
private:
	// only a reference
	CSaFlashPlayerBaseIMEControl* GetIMEControl();
#endif // FEATURE_INLINE_TEXT_ENTRY

private:
	MFlashEngine* iEngine;
	CSaFlashPlayerView* iView;
	RLibrary iFELib;
	CIdle* iFocusHandler;
	
	CHwKeyPropertyWatch*    	iHwKeyProperty;

#ifdef USE_VPU2	
	//A coecontrol to create a dummy RWindow establish the z-order 
	//of egl and mmf surfaces.
	CSurfaceContainerControl*    iVideoSurfaceContainer;
#endif
    
public:
#ifdef FEATURE_INLINE_TEXT_ENTRY
	CSaFlashPlayerBaseIMEControl* iImeControl;
	TBool iIsActivatingIME;
#endif // FEATURE_INLINE_TEXT_ENTRY
	//#ifdef __FLASHLITE2.0_REL3.1
	//#ifdef SCALABLE_MUTE
private:// data members
	CFbsBitmap* iPauseIcon;
	CFbsBitmap* iPauseIconMask;
	CFbsBitmap* iMuteIcon;
	CFbsBitmap* iMuteIconMask;
    CFbsBitmap* iErrorImageBitmap;
    CFbsBitmap* iErrorImageBitmapMask;	
	CAknNavigationControlContainer* iNaviPane;
	CAknNavigationDecorator* iMutedNavi;
	CAknNavigationDecorator* iPausedNavi;
    #ifdef __USE_EMPTY_NAVI__	
	CAknNavigationDecorator* iEmptyNavi;
    #endif
	CAknNavigationDecorator* iPausedMutedNavi;
	//#ifdef __SIDE_VOLUME_KEYS
	CAknNavigationDecorator* iVolumeNavi; 
	CVolumeDisplayTimer* iVolumeDisplay;
	TBool iVolumeIconVisible;
	//#endif
	TAknLayoutRect iBitmapLayout;
	TAknLayoutRect iIconRect;
	TDisplayMode iDispMode;
	// Timer for blinking pause icon
	CPeriodic* iBlinker;
	TBool iPauseIconVisible;
	TBool iMutedIconVisible;
	TBool iPaused;
	TBool iSideVolKeysSupported;
	// Id of current input language.
	TLanguage iCurrentInputLanguage;
    TInt	iLastError;
    TBool   iErrorOccured;
	
#ifdef __FEATURE_FLV_RECOG_PLAY		
    private:
	TBool	iFLVFile;
	public:
			TBool IsFlvFilePlaying();
#endif //__FEATURE_FLV_RECOG_PLAY

#ifdef __FEATURE_F4V_RECOG_PLAY
    private:
    TBool   iF4VFile;
    public:
            TBool IsF4vFilePlaying();
#endif //__FEATURE_F4V_RECOG_PLAY
            
	//#endif //SCALABLE_MUTE
	//#endif //__FLASHLITE2.0_REL3.1
			
	public:
	
	TPoint iPreviousPoint;
	
	TPoint iDownPoint;
	
	TBool ihaveDragged;
	
	CFlashVolumeControl* 	iVolPopup;
	
	CRepository* iCenRepos;
	
#ifdef	FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR	
	CFlashLongTapDetector* iFlashLongTapDetector;
	
    enum 
    { 
    ETouchStarted, 
    ETouchTap, 
    ETouchLongTap, 
    ETouchDrag 
    } iTouchType;
    
    TPoint iCorePosition;
    TBool iControlPresent;
    
    CAknPointerEventSuppressor* iSuppressor;
    

    
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
	
    MTouchFeedback* iFeedback;
    /*
    * Flag which determines whether tactile feedback is supported or not.
    */
    TBool iTactileFeedbackSupported;

#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
	    CAknToolbar* iToolbar;
#endif // __FEATURE_UI_LANDSCAPE_FULLSCREEN
	
	/*
    * Flag which determines whether touch feature is supported or not.
    */
    TBool iTouchFeatureSupported;
    	    
}; 


//This class decides which input mode the text field should be in. 
//The decision is based on initial string and current input language if avaiable. 
 #ifdef FEATURE_INLINE_TEXT_ENTRY
class CInputModeSelector : public CBase 
{
public:
	static CInputModeSelector* NewL(AMP_MTextNavigation* pNav, CSaFlashPlayerControl* aPlayerControl);
	TBool IsFEP();
	~CInputModeSelector(); 
	
private:
	CInputModeSelector(CSaFlashPlayerControl* aPlayerControl);
	void ConstructL(AMP_MTextNavigation* pNav);
private:
	FI_UTF16Char *iInitialString;
	int          iLen;
	//Returns true if the initial string is in complex language.
	TBool ContainsComplexLangCharacters(FI_UTF16Char * string, long length);

	//Returns true if the current input language is complex. 	
	TBool IsInComplexLanguageMode();
	CSaFlashPlayerControl* iPlayerControl;
};
// Based on decision taken from CInputModeSelector,this class is responsible for creating
// IME or FEP.

class CSaFlashPlayerBaseIMEControl : public CCoeControl 
{
public:
	virtual ~CSaFlashPlayerBaseIMEControl() 
	{	
	}
	virtual void ConstructL()=0;
	virtual TBool IsInLine()=0;	
	virtual void CommitConversion(TBool aConversionCanceled)=0;
	virtual void ReportStateEventL() =0;

};

class CSaFlashPlayerFEPIMEControl : public CSaFlashPlayerBaseIMEControl
{
public:
	CSaFlashPlayerFEPIMEControl(MFlashEngine* aEngine,
								CSaFlashPlayerView* aView,
								TPoint aPoint);
	void ConstructL();
	virtual ~CSaFlashPlayerFEPIMEControl()
	{
	}
	
	void CommitConversion(TBool /*aConversionCanceled*/) 
	{
	}
	void ReportStateEventL()
	{
		
	}
	TBool IsInLine();
	MFlashEngine* iEngine;
	CSaFlashPlayerView* iPlayerView;
};
#endif //#ifdef FEATURE_INLINE_TEXT_ENTRY

#ifdef FEATURE_INLINE_TEXT_ENTRY
#ifdef VIRTUAL_DOCUMENT_FOR_FEP
class CVirtualDocumentForFEP;
#endif //VIRTUAL_DOCUMENT_FOR_FEP

class CSaFlashPlayerIMEControl : public CSaFlashPlayerBaseIMEControl, public MCoeFepAwareTextEditor, private MCoeFepAwareTextEditor_Extension1
{
public:
	CSaFlashPlayerIMEControl(MFlashEngine* aEngine,
								CSaFlashPlayerView* aView,
								TPoint aPoint);

	virtual ~CSaFlashPlayerIMEControl();

	void ConstructL();

	void CommitConversion(TBool aConversionCanceled);

	MCoeFepAwareTextEditor_Extension1* Extension1();
	
	CAknEdwinState* FepState() { return iFepState; }
	
	// Returns true, as it is IME.
	TBool IsInLine();

public:
	// Set and Get methods for current input language.
	TLanguage CurrentInputLanguage() 	{	return iCurrentInputLanguage ; }
	void SetCurrentInputLanguage(TLanguage aCurrentInputLanguage) 	{	iCurrentInputLanguage = aCurrentInputLanguage; }
	void HandlePointerEventL(const TPointerEvent& aPointerEvent);
	
	void ReportStateEventL();
protected:
	    void SetFepStateInputModesToTextFieldFlags(CAknEdwinState *state, unsigned long flags);
	
private:

	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);

	virtual MCoeFepAwareTextEditor_Extension1* Extension1(TBool& aSetToTrue);

	// from CCoeControl
	// Implement the InputCapabilities interface of CCoeControl
	// so that we can override the FEP interface
	virtual TCoeInputCapabilities InputCapabilities() const;

private:	// from MCoeFepAwareTextEditor
	void StartFepInlineEditL(const TDesC& aInitialInlineText, TInt aPositionOfInsertionPointInInlineText, TBool aCursorVisibility, const MFormCustomDraw* aCustomDraw, MFepInlineTextFormatRetriever& aInlineTextFormatRetriever, MFepPointerEventHandlerDuringInlineEdit& aPointerEventHandlerDuringInlineEdit);
	void StartFepInlineEditL(TBool& aSetToTrue, const TCursorSelection& aCursorSelection, const TDesC& aInitialInlineText, TInt aPositionOfInsertionPointInInlineText, TBool aCursorVisibility, const MFormCustomDraw* aCustomDraw, MFepInlineTextFormatRetriever& aInlineTextFormatRetriever, MFepPointerEventHandlerDuringInlineEdit& aPointerEventHandlerDuringInlineEdit);
	void UpdateFepInlineTextL(const TDesC& aNewInlineText, TInt aPositionOfInsertionPointInInlineText);
	void SetInlineEditingCursorVisibilityL(TBool aCursorVisibility);
	void CancelFepInlineEdit();
	TInt DocumentLengthForFep() const;
	TInt DocumentMaximumLengthForFep() const;
	void SetCursorSelectionForFepL(const TCursorSelection& aCursorSelection);
	void GetCursorSelectionForFep(TCursorSelection& aCursorSelection) const;
	void GetEditorContentForFep(TDes& aEditorContent, TInt aDocumentPosition, TInt aLengthToRetrieve) const;
	void GetFormatForFep(TCharFormat& aFormat, TInt aDocumentPosition) const;
	void GetScreenCoordinatesForFepL(TPoint& aLeftSideOfBaseLine, TInt& aHeight, TInt& aAscent, TInt aDocumentPosition) const;
	void DoCommitFepInlineEditL();
	void MCoeFepAwareTextEditor_Reserved_2();


private:	// from MCoeFepAwareTextEditor_Extension1
	void SetStateTransferingOwnershipL(CState* aState, TUid aTypeSafetyUid);
	CState* State(TUid aTypeSafetyUid);
	void SetCursorType(TBool& aSetToTrue, const TTextCursor& aTextCursor);
	void MCoeFepAwareTextEditor_Extension1_Reserved_3();
	void MCoeFepAwareTextEditor_Extension1_Reserved_4();

private:
	TBool IsLangChanged();
	void CreateStateL();
	
protected:	// specific
	TUid iStateFepUid;
	CAknEdwinState* iFepState;

	TBool iIsSpelling;
	TBool iIsCommitting;
	TBool iIsCbaShownForT9Spell;

	// State for an inline editing session
	HBufC*		iCompositionStringBuf;			// A buffer to hold the current composition string
	TInt		iCompositionStringBufSize;		// The size of the composition string buffer, in bytes
	TInt		iCompositionStringLen;			// The number of characters in the composition string

	TInt		iCompositionStringCursorPos;		// Character offset in the composition string of the cursor
	TBool		iCursorVisibility;					// ETrue for visible text cursor, EFalse for invisible
													// text cursor in the text editor.
private:
	MFlashEngine* iEngine;
	CSaFlashPlayerView* iPlayerView;
	TPoint iPoint;



	// When calling SetConversionString with embedded font
	// the call sometimes fail to insert all the chars because of missing glyphs.
	// T9 crashes on this. The following variable is made to make T9 to believe
	// we inserted the chars we didn't. This is unfortunately hacky but seems to work.
   	TInt iCursorPosAdd;

	// This is related to the same issue as above.
	TBool iMonitorConversionStringLengthForZi;
	
	// Some FEPs may set too large a selection when embedded font is used and glyphs are missing.
	// Please see above.
	TInt iLastConversionStringCharactersInserted;


	// Store the retriever referece
	MFepInlineTextFormatRetriever* iTextFormatRetriever;

	// Fix for Chinese phone's multitap mode. They seem to use
	// StartFepInlineEditL function's version that has the initial selection.
	// Other T9 devices use that only after spell. To fix some weird issues
	// with Chinese phones' multitap mode disable that function if no spelling occurs.
	TBool iLetCallSecondTime;
	
	TBool iIsPencilKeyPressed;
	TLanguage   iCurrentInputLanguage;
	//This  is required for VKB  In multitap mode(Secret text editor mode)  Fep manager sends EKeyF20 to delete previous key we need to know if we have consumed previous character
	TInt iCharsConsumed;

#ifdef VIRTUAL_DOCUMENT_FOR_FEP
public:
	void ResetIfEditorChanged(const TKeyEvent& aKeyEvent, TEventCode aType);
private:
	CVirtualDocumentForFEP* iVirtualDocument;
#endif // VIRTUAL_DOCUMENT_FOR_FEP
};

#ifdef VIRTUAL_DOCUMENT_FOR_FEP

/**
 * Zi FEP calls GetText with the whole text field as parameter
 * (T9 only for a portion).
 * This will cause a lot of copying if the text field is large.
 * On some devices this seems to cause the player slow down.
 * Virtual document shows the FEP (any FEP) only a portion of
 * the text field. It maps the cursor location. Hopefully
 * everything goes well and the FEP doesn't get confused.
 *
 * NOTE: Any navigation bugs or weird behavior, please undefine
 * this feature and try again.
 */
class CVirtualDocumentForFEP: public CBase
{
public:

	CVirtualDocumentForFEP(MFlashEngine* aEngine, CSaFlashPlayerIMEControl* aIMEControl);
	~CVirtualDocumentForFEP();

	void Reset();
	void ResetIfEditorChanged(const TKeyEvent& aKeyEvent, TEventCode aType);

	TInt DocumentLengthForFep();
	void SetCursorSelectionForFepL(const TCursorSelection& aCursorSelection);
	void GetCursorSelectionForFep(TCursorSelection& aCursorSelection);
	void GetEditorContentForFep(TDes& aEditorContent, TInt aDocumentPosition, TInt aLengthToRetrieve);

private:

	MFlashEngine* iEngine;

	CSaFlashPlayerIMEControl* iIMEControl;

	TInt iActualStartIndex;
	TInt iOriginalLength;
	TInt iVirtualLengthInBeginning;

	TInt iPreviousEventKeyCode;

	TBool iReseted;
};

#endif // VIRTUAL_DOCUMENT_FOR_FEP


#endif // FEATURE_INLINE_TEXT_ENTRY
// ----------------------------------------------------
// CVolumeDisplayTimer
// Description: Display a Volume control in the menu 
// pane for 2 seconds.
// ----------------------------------------------------
class CVolumeDisplayTimer : public CTimer 
{
public:
	static CVolumeDisplayTimer* NewL(CSaFlashPlayerControl* aNaviHandler);
    virtual ~CVolumeDisplayTimer();
    
	void StartVolumeTimer();
    
public: //from CActive
	void DoCancel();
	void RunL();	
                                   
private:
	CVolumeDisplayTimer(CSaFlashPlayerControl* aNaviHandler);
	void ConstructL();
	
	CSaFlashPlayerControl* iControl;
};

#endif // __SYMBIAN_PLAYERCONTROL_H__
