/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
                                 
*
*/


#ifndef __SYMBIAN_FLASH_PLAYER_VIEW_H__
#define __SYMBIAN_FLASH_PLAYER_VIEW_H__

#include "symbian_view.h"
#include "symbian_uiutil.h"
#include "symbian_playercontrol.h"
#include "symbian_ui_interface.h"

#include "flash2ui.hrh"
#ifdef __FEATURE_TOOLBAR__
#include <akntoolbar.h>
#include <akntoolbarobserver.h>
#include <aknbutton.h>
#endif

#include <aknappui.h>

#include "screenfurnitureif.h"

#ifdef __FEATURE_OSK__
#include "on_screen_key_observer.h"
#include "on_screen_keypad_constants.h"
#include "on_screen_keypad.h"
#include "timer_object.h"
#endif // __FEATURE_OSK__

const TInt KMaxErrorMsgSize = 256;
const TInt KMute = 0;
const TInt KUnmute = 1;
const TInt KVolumeMuteValue = 0;
#include <DRMHelper.h>
//class MFlashUI;
class MFlashEngine;
struct FlashEngineConfig;
class CFlashErrorDialog;


union ErrorInfo;
enum CAknAppUiBase::TAppUiOrientation;
class CAsyncUIChanger;
class CPlayerViewMenuManager;

class CFlashVolumeControl;

#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
class CAknStylusPopUpMenu;
#endif

#ifdef __FEATURE_OSK__
class COnScreenKeypadControl;
#endif // __FEATURE_OSK__

class CSaFlashPlayerView : public CSaFlashView
#ifdef __FEATURE_TOOLBAR__
                       ,   public MAknToolbarObserver
#endif
#ifdef __FEATURE_OSK__
                       ,public MOnScreenKeyObserver
#endif // __FEATURE_OSK__
{
public:
	CSaFlashPlayerView(TUint32 aWarningFlags, TBool aIsStubApp);
	~CSaFlashPlayerView();
	
    /*
     * If the top level class () wishes to receive IAD Update complete notification, 
     * then it must implement MIadUpdateCallback and let the HostEngine know
     * that it wants to receive the notification by passing the "this" pointer.
     * If it does not want toreceive notification, then it need not bother with the third parameter.
     * Hence making it NULL by default.
     */ 
    void ConstructL(/*MFlashUI* aFlashUI,*/ FlashEngineConfig& aConfig
/*
#ifdef SELF_IAD_UPDATE
            , MIadUpdateCallback* aIADUpdateCallbackInterface=NULL
#endif
*/
            );
	
	// from CAknView
	TUid Id() const;
	void DoActivateL(const TVwsViewId& aPrevViewId, TUid aCustomMessageId, const TDesC8& aCustomMessage);
	void DoDeactivate();
	void HandleCommandL(TInt aCommand);
	
public: // from CSaFlashView
	CCoeControl* GetContainer();
	RWindow* GetWindow();
	RWindow* GetNewVideoWindow(void* aClient);
	void CloseVideoWindow(void* aClient);
	
	TBool iFullScreen;

public:	    
	    
#ifdef FEATURE_INLINE_TEXT_ENTRY
	void DisableIME();
	void EnableIMEL();
	TInt GetCurrentInputMode();
	CSaFlashPlayerBaseIMEControl* GetIMEControl();
	//If complex language is the case, then this function delegates control to Avkon dialog.
	void GetInputTextL(const TUint16* aInitialString, TUint aStrLen, TUint aMaxNumChars, TUint aTextFlags);
	void ResetIMEL();
	void RemoveCba(TUint aButtonPos);
#endif // FEATURE_INLINE_TEXT_ENTRY

	void HandleBackgroundEvent();
 	void HandleForegroundEvent();
 	void CancelHwKeyObserver();
 	void AddHwKeyObserverL();
	void  CloseVolPopUp();

	TBool QueryAllowDenyL(const TDesC& aQueryText);
	// Implements MEikMenuObserver
	void DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane);
	void DynInitMenuBarL(TInt aResourceId, CEikMenuBar* aMenuBar);
	
	// new functions
	CEikButtonGroupContainer* GetCba();
	void SetFullScreenL();
	void SetAppScreenL();
	
#ifdef __FEATURE_OSK__
	void SetOSKIsActive(TBool aShow); //enable or disable the compatiblity mode keypad
    
	// from MOnScreenKeyObserver
	TKeyResponse OfferOverlayKeyEvent(const TKeyEvent& aKeyEvent, TEventCode aType);

	TBool HandleSpecialKeyL(const TKeyEvent& aKeyEvent, TEventCode aType);
#endif
    
#ifdef __FEATURE_TOOLBAR__  
    //From MAknToolbarObserver
    void OfferToolbarEventL(TInt aCommand);
    void ChangeToolbarPauseIcon (TBool aIsPaused);
    void DynInitToolbarL(TInt aResourceId, CAknToolbar* aToolbar);
     TBool iShowToolBar;
#endif    
	
	void ResetScreenInCurrentModeL();
	void NotifyFlashError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue,
							ErrorInfo* aErrorInfo = NULL);
	void NotifyDataAccessError(const char* aUrl, TInt aErrorCode);
	void NotifyClientError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent = ETrue);
	void NotifyClientErrorL(TInt aErrorCode, TBool aIsCritical, TBool aPersistent);
	void NotifyFlashErrorL(TInt aErrorCode, TBool aIsCritical, TBool aPersistent,
							ErrorInfo* aErrorInfo = NULL);
	MFlashEngine* GetEngine();
	void EngineDestroyed();
	void OpenFileL(const TDesC& aFilename);
	void OpenFileL(RFile& aFile);
	void SetTitleL(const TDesC& aFilename);
//    TBool IsFullScreen();
	void ClearErrors();
	TBool DisplayingError() { return iErrorHandler->DisplayingError(); }
	TRect ApplicationRect();
	
	void SetSoftkeysL(HBufC** aSoftkeyArray, TInt aSoftkeyArraySize);
	void ResetSoftkeysL();
	void HandleEmbeddedAppLaunch(TBool aAppTakesForeground);
	
 	void Mute(TBool aMute);
 	void HandleMuteL (TBool aSave=ETrue);
 	void DisplayFileDetailsL();
 	void SetVolumeL(TInt aVolume);
 	TInt GetVolume();
 	CDRMHelper* GetDRMHelper() {return iDrmHelper;};
 	void HandleResourceChangeL(TInt aType);
	enum TLaunchType {
		EStartedFromMenu = 0,
		EStartedFromApp,
		EEmbedded
	};
	
	void SetLaunchType(TLaunchType aLaunchType) { iLaunchType = aLaunchType; }
	TLaunchType GetLaunchType();
	
	void  SetStubApp(TBool aStubApp) { iIsStubApp = aStubApp; }
	TBool IsStubApp() { return iIsStubApp; }
	
	SAFlashErrorHandler* GetErrorHandler ( ) { return iErrorHandler; }
	CFlashErrorDialog& GetLowMemDialog( ) { return *iLowMemErrorDialog; }

	void CancelFocusHandler() { iContainer->CancelFocusHandler(); }
	TBool IsFullScreenMode() const;
//use of playerviewmenumgr
	TBool IsDisableMenuRotateScreen() const;
	TBool IsVideoContent() const;
	TBool IsVideoEmbedded();
	TBool IsVolumeKeySupported()const;
	TBool IsVolumeMuted() const;
	TBool IsFullScreen() const;
	TBool IsLandScape() const;
	TBool CanZoomInfurther() const;
	TBool CanZoomOutfurther() const;
	TBool FileExistsInDefaultFolder();
	#ifdef FEATURE_MULTIPLE_DRIVE
	TBool FileExistsInDefaultDriveL();
	#endif
	TInt  GetQuality()const;
	void  HandleVideoError(const TInt& aError);
	
private:
	void NotifyDataAccessErrorL(const char* aUrl, TInt aResId);
	void ResetCbaL(CEikButtonGroupContainer* aCba);
	void SetCbaL(CEikButtonGroupContainer* aCba, TInt aRsrcId, const TDesC* aKey1, TInt aKeyId1, const TDesC* aKey2, TInt aKeyId2);
private:
	void SetPlayerViewTitleL();
	void AddCountsL(TUint32 aCounter, CDesCArrayFlat* aItems);
	void AddStringPairL(TUint32 aTitleId, TUint32 aBodyId, 
		CDesCArrayFlat* aItems, TBool aDisplayPlayText);
	
	void AddTimeLeftL(TUint32 aTitleId, TTimeIntervalSeconds aInterval, CDesCArrayFlat* aItems, TBool aDisplayPlayText);
	void AddDateAndTimeL(TUint32 aTitleId, TTime aDateTime, CDesCArrayFlat* aItems);
	void AddSingleValueL(TUint32 aTitleId, TUint32 aOne, TUint32 aTwoFour, 
		TUint32 aFiveZero, TUint32 aOneFinal, CDesCArrayFlat* aItems, 
		TUint8 aValue, TBool aDisplayPlayText);
	
	void AddDoubleValueL(TUint32 aTitleId,
		TUint32 aFirstOne, TUint32 aFirstTwoFour, TUint32 aFirstFiveZero, TUint32 aFirstOneFinal,
		TUint32 aSecondOne, TUint32 aSecondTwoFour, TUint32 aSecondFiveZero, TUint32 aSecondOneFinal,
		CDesCArrayFlat* aItems, TUint8 aFirstValue, TUint8 aSecondValue, TBool aDisplayPlayText);
    
	void AddSingleOrDoubleValueL(TUint32 aTitleId,
		TUint32 aFirstOne, TUint32 aFirstTwoFour, TUint32 aFirstFiveZero, TUint32 aFirstOneFinal,
		TUint32 aSecondOne, TUint32 aSecondTwoFour, TUint32 aSecondFiveZero, TUint32 aSecondOneFinal,
		CDesCArrayFlat* aItems, TUint8 aFirstValue, TUint8 aSecondValue, TBool aDisplayPlayText);
	HBufC* FormatValueLC(TUint32 aOne, TUint32 aTwoFour, TUint32 aFiveZero, TUint32 aOneFinal, TUint8 aValue);
	
	TBool CheckMemoryCard();
	TInt MemoryCardDrive();
	
		/**
        * This function is called when the user presses back or Exit(only in embedded mode).
        * prompts a confirmation query to the user and then calls save of AppUi
        *       
        */                          
     void DoSaveL();
     void HandleSaveCommandL();
     
     TBool LaunchSaveQueryDialogL(const TFileName& aFileName) const;
     void DynInitMenuPaneforVideoL(CEikMenuPane* aMenuPane );
     void DynInitMenuPaneforRunModeL(CEikMenuPane* aMenuPane );
     void DynInitMenuPaneforNoRunModeL(CEikMenuPane* aMenuPane );
     void DynInitControlMenuPaneL(CEikMenuPane* aMenuPane );
//#endif //__FLASHLITE2.0_REL3.1	
     
#ifdef __FEATURE_OSK__
    void ShowOnScreenKeypadL(const TRect& aAppRect, TBool aUserInitiatedFullScreen); 
	void InitializeOnScreenKeypadL(const TRect& aAppRect, TBool aUserInitiatedFullScreen);
	void UpdateOnScreenKeypadL(const TRect& aAppRect, TBool aUserInitiatedFullScreen);
	TRect CreateOnScreenKeypadControl(const TRect& aAppRect);
	void HandleSoftKeyL(TInt aKey);
	void SimulateSoftKey(TInt aKey);
	TOSKMode GetOnScreenKeypadMode(TBool aFullScreenViaFSCommand);
	TRect GetOnScreenKeypadRect(const TRect& aAppRect, TOSKMode aMode);
#endif // __FEATURE_OSK__

public:
    inline CSaFlashPlayerControl* GetPlayerControl() const 
        {return iContainer;}
    
#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
public:
    void ShowContextSensitiveMenu(const TPoint& aPoint);
private:
    CAknStylusPopUpMenu* iStylusPopupMenu;
#endif
    
    
private:	
	CSaFlashPlayerControl*	iContainer;
	SAFlashErrorHandler*	iErrorHandler;
	CFlashErrorDialog*		iLowMemErrorDialog;

#ifdef __FEATURE_OSK__	
	COnScreenKeypadControl* iOnScreenKeypadControl;
	TBool					iCompatabilityMode;
	EFlashPlayerCommands	iLSK;
	EFlashPlayerCommands	iRSK;
	CFlhTimerObject2*		iASyncOnScreenKeypadDestroyer;
	
	TRect iOnScreenKeypadRect;

	static TInt DestroyOnScreenKeypadControl(TAny* aArgs);
	
	void  SetOnScreenKeypadRect(const TRect& aRect) { iOnScreenKeypadRect = aRect; }
	TRect GetOnScreenKeypadRect() { return iOnScreenKeypadRect; }
#endif // __FEATURE_OSK__

    TRect iFlashDisplayRect;	
	
    void  SetFlashDisplayAreaRect(const TRect& aRect) { iFlashDisplayRect = aRect; }
	TRect GetFlashDisplayAreaRect() { return iFlashDisplayRect; }


	TInt iQuality;

	TBool iFirstHit;
	TBool iIMEFirstHit;
	HBufC* iFlashErrorBuffer;

	TUint32 iReportedErrorsFlag;
	
	TLaunchType iLaunchType;
	TBool		iIsStubApp;
	
	TUint32 iDebugMode;
//#ifdef __FLASHLITE2.0_REL3.1
	HBufC16* iSavedFilename;
	TInt iVolume;
	TInt iVolumeLevel;
	TBool iBackButtonActive;
	CDRMHelper* iDrmHelper;
	TBuf<KMaxFileName> iOrigName;
	RFile iFile;
	TBool iSideVolKeysSupported;
	TBool iActivatingVolumeView;
	TBool iZoomSuccess;
	TInt  iZoomedTimes;
	TInt  iFileInitialized;
	TBool iEngineInitiatedPause;
	
	TEngineState iEngineState;

	CPlayerViewMenuManager* iMenuMgr;
	TBool 		iDisableMenuRotateScreen;
	
	#ifdef FEATURE_INLINE_TEXT_ENTRY
		CSaFlashPlayerBaseIMEControl* iIMEControl;
		TBool iIMEWasDeactivatedForPausing;
	#endif // FEATURE_INLINE_TEXT_ENTRY

#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
	private:
		CAknAppUiBase::TAppUiOrientation iAppOrtn;
		TBool iVideoContent;
		CAsyncUIChanger* iUiChanger;
		TBool iLandScape;
		TBool iEmbeddedVideo;
		
	private://methods
		void RestoreOrientationL();
		void HandleVideoErrorsL();
	
		
	public :
			void VideoInPlay();
            void VideoNotInPlay();
            void DisableZoomToolBarButton();
            void EnableZoomToolBarButton();
            void DoZoomShowAll();
			void GoLandscapefullScreenL();
			void RotateAppL();
			void VideoIsEmbedded(TBool aEmbeddedVideo);
			
#endif
	TBool iErrorOcurred;

#ifdef __FEATURE_FLV_RECOG_PLAY		
	public:
			TBool IsFlvFilePlaying();
	#endif	
	
	TBool iIsTouchFeatureSupported;

#ifdef __FEATURE_F4V_RECOG_PLAY     
    public:
            TBool IsF4vFilePlaying();
#endif
            
    public:
        void HandleEngineStateChangeL(TEngineState aState);
        TBool ShowPlayOption();

            
//#endif __FLASHLITE2.0_REL3.1
};

#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN

class CAsyncUIChanger : public CActive
{
public:
    CAsyncUIChanger( CSaFlashPlayerView* aPlayerView);
                  
    ~CAsyncUIChanger();
    
public:
    void RunL();
    void DoCancel();
    void IssueRequest();
    
private:
    CSaFlashPlayerView* iPlayerView;
};

#endif

#endif
