/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
                                 
*
*/



#ifndef __SYMBIAN_UIUTIL_H__
#define __SYMBIAN_UIUTIL_H__

#include "symbian_s60_notedlg.h"

#include <eikenv.h>

class CDictionaryFileStore;
class CSaFlashPlayerView;
class MFlashEngine;

class CFlashErrorDialog : public CFlhQueryDialog
	{
public:
	static CFlashErrorDialog* NewL(MFlashEngine& aFlashEngine, TInt aDialogResId, TInt aMsgResId, CEikonEnv& aEikonEnv);
	~CFlashErrorDialog();
	TInt RouseSleepingDialog();
private:
	void ConstructSleepingDialogL(TInt aResourceId);
	CFlashErrorDialog(MFlashEngine& aFlashEngine);
	//from CAknDialog
	TBool OkToExitL(TInt aButtonId);
private:
	MFlashEngine& iFlashEngine;
	};

class FlashUiUtil
	{
public:
	static CGulIcon* CreateIconL( const TDesC& aIconFile, TInt aIconIndex,
                                     TInt aMaskIndex );
	static TInt CheckSubdirCountL(const TDesC& aBaseDir, const TDesC& aSubDir,
		TBool aDirsOnly, TBool aSwfDcfFileCount);
	static void DebugShowNoteL(const TDesC& aMsg);
	static void ShowNoteL(TAknGlobalNoteType aNoteType, const TDesC& aMsg);
	static HBufC * Format1L(CEikonEnv& aEnv, TInt aFormatResource, const TDesC& aArg1);
	static void ShowNoteFormat1L(CEikonEnv& aEnv, TAknGlobalNoteType aNoteType, TInt aFormatResource, const TDesC& aArg1);
	static TInt ShowNoteWithoutLeaving(TAknGlobalNoteType aNoteType, const TDesC& aMsg, TInt aFormatResource, CEikonEnv* aEnv, CFlashErrorDialog* aLowMemDialog=NULL);
	static TBool DisplayQueryL(const TInt aResourceId, const TDesC& aQueryText);
	static TInt GetFileTypeL(const TDesC& aFilename);
	static TInt GetFileTypeL(RFile aRFile);
	static TInt  CheckSwfCountL(const TDesC& aBaseDir
								, const TDesC& aSubDir
								, const TDesC& aSearchFilter
								, TBool aSwfExtn=EFalse
								);
	static TInt CheckDirCountL(const TDesC& aBaseDir
							, const TDesC& aSubDir);
private:
	static void TryShowNoteL(TAknGlobalNoteType aNoteType, const TDesC & aMsg, TInt aFormatResource, CEikonEnv* aEnv);
	static TInt DoSwfCountL(CDir* drmSwf,const TDesC& aBaseDir );
	};


class SAFlashErrorHandler
{
public:
	enum {
		EFlagContinue = 1,
		EFlagAbort = 2,
		EFlagAllFlags = EFlagContinue | EFlagAbort
	};
	
private:
	struct TErrorMessage 
	{
	public:
		TErrorMessage (HBufC* aErrorMessage, TInt aFlags)
			: iFlags(aFlags), pErrorMessage(aErrorMessage), pNextError(NULL) { }
		
	public:
		TInt iFlags;
		HBufC* pErrorMessage;
		TErrorMessage* pNextError;
	};
	
	public:
		SAFlashErrorHandler(CSaFlashPlayerView* apPlayerView, CFlashErrorDialog& aLowMemDialog);
		void ClearErrors();
		void ReportError(HBufC* aErrorMessage, TInt aFlags);
		TInt ShowErrorsL();
		~SAFlashErrorHandler();
		void ShowLowMemoryDialog();
		TBool DisplayingError() { return iDisplayingError; }
		
		TBool QueryAllowDenyL(const TDesC& aQueryText);
	private:
		static TBool StaticTimerTick(TAny* aAny);
		void TimerTickL();
		void ReportErrorL(HBufC* aErrorMessage, TInt aFlags);
	private:
		CPeriodic* iTimer;
		TErrorMessage* iFirstError;
		CSaFlashPlayerView* iPlayerView;
		TBool iDisplayingError;
		CFlashErrorDialog& iLowMemDialog;
		
		TBool iNetworkPromptShown;

};


class IniData
{
public:
	IniData(RFs& aFs);
	~IniData();
	void SaveUserDataL();
	void LoadL();

public:
	TUint32 iStaticHeapSize;
	TUint32 iDynamicHeapMaxSize;
	TUint32 iDebugMode;
	TUint32 iVolume;
	TUint32 iEngineMode;
	TUint32 iIAPId;
	TUint16 iBackgroundAlpha;
	TBool	iPlayerBlendsBackground;
private:
	CDictionaryFileStore* OpenIniFileLC();
	
private:
	RFs& iFs;
};

class FlhUiUtils
{
public:
	static void GetFileInAppDir(CEikonEnv& aEnv, TDes& aFullPathFileName, const TDesC& aFileName, const TDesC& aDriveLetter);
	static TInt SearchResource( TBuf16<128>& aResourcePath, TBuf16<128>& aFileName, TBuf16<512>& aVariantFile );
};


#endif //__SYMBIAN_UIUTIL_H__
