/*
* Copyright (c) 2002-2004 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  AO that completes on a change in the volume property.  Used for 
*                the new Central Repository Server.  Used in EKA2 builds only
*
*/


#ifndef CVOLUME_PROPERTY_WATCH
#define CVOLUME_PROPERTY_WATCH

#include <e32std.h>
#include <e32base.h>
#include <e32property.h>

#include <centralrepository.h>
#include<RemConCoreApiTargetObserver.h>    //for MRemConCoreApiTargetObserver
#include<RemConInterfaceSelector.h>     //for CRemConInterfaceSelector
#include<remconcoreapitarget.h>         //for CRemConCoreApiTarget

//
// for emulator max volume = 65535
#if defined(__WINS__)
    #define WINS_DEFAULT_VOLUME 43690 //two-third of the maximum volume
#endif        

//
// Forward declarations
class MVolumeObserver;

// DATA TYPES
// Used for volume up/down events
enum TVolumeControlCommands
    {
    EVolumeControlCmdVolumeUpByOne,
    EVolumeControlCmdVolumeDownByOne
    };

class CVolumePropertyWatch : public CBase, public MRemConCoreApiTargetObserver
    {
public:
    
    static CVolumePropertyWatch* NewL(const TInt& aMinVolume, const TInt& aMaxVolume,MVolumeObserver* aObserver);
    virtual ~CVolumePropertyWatch();
    TInt GetCurrentVolume();
                                   
private:
    CVolumePropertyWatch(const TInt& aMinVolume, const TInt& aMaxVolume);
    void ConstructL(MVolumeObserver* aObserver);

    //From MRemConCoreApiTargetObserver (remconcoreapitargetobserver.h)
    void MrccatoCommand( TRemConCoreApiOperationId aOperationId, 
                         TRemConCoreApiButtonAction aButtonAct );
    /**
      * Handle repeat event
      */
    void HandleRepeatEvent();

    /**
    * Send play command to app UI or view.
    * @since 3.1
    */
    void FilterAndSendCommand(TInt aCommandId);
    /**
    * Callback for timer
    */
    static TInt TimerCallback(TAny* aPtr);


    const TInt iMinVolume; 
    const TInt iMaxVolume;
    MVolumeObserver* iObserver;
    CRemConCoreApiTarget* iInterfaceSelector;
    CRemConInterfaceSelector* iSelector;
    CRepository* iCenRepos;
    CPeriodic* iTimer;                                   // owned
    TInt iCommandId;

    };
    
#endif
