/*
* Copyright (c) 2006-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/

#include "saplayer_build.h"

#ifdef __FEATURE_OSK__
#include "button_grid.h"
#include "on_screen_key.h"

CButtonGrid::CButtonGrid(TRect aLocation, TInt aRows, TInt aCols, TInt aHMargin, TInt aVMargin)
{
    ChangeLayout(aLocation,aRows,aCols,aHMargin,aVMargin);
}

CButtonGrid::~CButtonGrid()
{
	iButtonArray.Close();
	iButtonLocationArray.Close();
}

TInt CButtonGrid::AddButton(COnScreenKey* aOverlayKey)
{
    TInt locationCount = iButtonLocationArray.Count();
	TInt index = iButtonArray.Count();
	TRect buttonRect(iButtonLocationArray[index], iActualCellSize);
	buttonRect.Move(iRect.iTl);
	aOverlayKey->SetRect(buttonRect);
	iButtonArray.Append(aOverlayKey);
	
	return KErrNone;
}

TInt CButtonGrid::ChangeLayout(TRect aLocation, TInt aRows, TInt aCols, TInt aHMargin, TInt aVMargin)
{
    iRect    = aLocation;
    iRows    = aRows;
    iCols    = aCols;
    iHMargin = aHMargin;
    iVMargin = aVMargin;
    
    if(iRows < 1 )
    {
        iRows = 1;
    }
    if( iCols < 1 )
    {
        iCols = 1;
    }
    
    iCellSize.iWidth    = iRect.Width() / iCols;
    iCellSize.iHeight   = iRect.Height() / iRows;
    
    iActualCellSize = iCellSize;
    iActualCellSize.iHeight -= (2*iVMargin);
    iActualCellSize.iWidth -= (2*iHMargin);

    // empty the button position array
    iButtonLocationArray.Reset();
    iButtonArray.Reset();
    
    // change the button position here
    // calculate the location where the buttons are to be added.
    for(TInt i=0; i<iRows; i++)
    {
        for(TInt j=0; j<iCols; j++)
        {
            TPoint location;
            location.iX = j*iCellSize.iWidth + iHMargin;
            location.iY = i*iCellSize.iHeight + iVMargin;
            iButtonLocationArray.Append(location);
        }
    }
    
	return KErrNone;
}

void CButtonGrid::RemoveAllButtons()
{
    // empty the button position array
    iButtonArray.Reset();
}
#endif //__FEATURE_OSK__

