/*
* Copyright (c) 2006-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/


#include "saplayer_build.h"

#ifdef __FEATURE_OSK__
#include <coecntrl.h>
#include <aknbutton.h>

#include "on_screen_key.h"

const TInt KRepeatDelay = 300;
const TInt KRepeatInterval = 300;

COnScreenKey* COnScreenKey::NewL(TKeyEvent aKeyEvent, TInt aResourceId, TRect aLocation, MCoeControlObserver* aObserver, const CCoeControl& aContainer)
{
	COnScreenKey* self = new (ELeave) COnScreenKey(aKeyEvent,aResourceId,aLocation);
	self->ConstructL(aObserver,aContainer);
	return self;
}

COnScreenKey* COnScreenKey::NewL(TKeyEvent aKeyEvent, TDesC& aCaption, TRect aLocation, MCoeControlObserver* aObserver, const CCoeControl& aContainer)
{
    COnScreenKey* self = new (ELeave) COnScreenKey(aKeyEvent,aCaption,aLocation);
    self->ConstructL(aObserver,aContainer);
    return self;
}

COnScreenKey::COnScreenKey(TKeyEvent aKeyEvent, TInt aResourceId, TRect aLocation)
: CAknButton(0)
{
	iKeyEvent	= aKeyEvent;
	iKeyEvent.iModifiers = EModifierKeypad;
	iKeyEvent.iRepeats   = 0;
	
	iResourceId = aResourceId;
	iLocation	= aLocation;

	iKeyUpsReceived = 0;
}

COnScreenKey::COnScreenKey(TKeyEvent aKeyEvent, TDesC& /*aCaption*/, TRect aLocation)
: CAknButton(0)
{
    iKeyEvent   = aKeyEvent;
    iKeyEvent.iModifiers = EModifierKeypad;
    iKeyEvent.iRepeats   = 0;
    
    iLocation   = aLocation;
    
    iResourceId = 0;
    iKeyUpsReceived = 0;
}

void COnScreenKey::ConstructL(MCoeControlObserver* aObserver, const CCoeControl& aContainer)
{
	if(iResourceId != 0)
	{
		CAknButton::ConstructL();
		ConstructFromResourceL(iResourceId);
	}
	else
	{
		CAknButton::ConstructL(NULL,NULL,NULL,NULL,_L(""),_L(""),0);
	}

	SetContainerWindowL(aContainer);
	SetRect(iLocation);
	SetObserver(aObserver);
	MakeVisible(ETrue);
	ActivateL();
	//SetButtonFlags(KAknButtonKeyRepeat); // handled in the resource files
	SetKeyRepeatInterval(KRepeatDelay,KRepeatInterval);
}

COnScreenKey::~COnScreenKey()
{
	
}

TKeyEvent COnScreenKey::GetKeyEvent() const
{
	return iKeyEvent;
}

void COnScreenKey::SetButtonState(TEventCode aEventCode)
{
	if(aEventCode == EEventKeyDown)
	{
        iKeyUpsReceived = 0;
	}
	else if(aEventCode == EEventKeyUp)
	{
		iKeyUpsReceived++;
	}
}

TBool COnScreenKey::IsRepeat()
{
	if(iKeyUpsReceived>1)
	{
		return ETrue;
	}
	else
	{
		return EFalse;
	}
}
#endif // __FEATURE_OSK__
