/*
* Copyright (c) 2006-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/


#include "saplayer_build.h"

#ifdef __FEATURE_OSK__

#include <flash2ui.rsg>
#include <keypad_fl_t.mbg>
#include <bautils.h>

#include "on_screen_keypad.h"
#include "on_screen_key.h"
#include "on_screen_keypad_constants.h"
#include "on_screen_key_observer.h"
#include "button_grid.h"

COnScreenKeypad* COnScreenKeypad::NewL(TPoint aLocation, TSize aSize, TInt aRows, TInt aCols, TOSKMode aMode,
									 MOnScreenKeyObserver* aOverlayKeyObserver, const CCoeControl& aContainer)
{
	COnScreenKeypad* self = new COnScreenKeypad(aLocation,aSize,aRows,aCols,aMode, aOverlayKeyObserver,aContainer);
	self->ConstructL(aContainer);
	return self;
}

COnScreenKeypad::COnScreenKeypad(TPoint aLocation, TSize aSize, TInt aRows, TInt aCols, TOSKMode aMode,
							   MOnScreenKeyObserver* aOverlayKeyObserver, const CCoeControl& aContainer)
:	iContainer(aContainer),
	iOverlayObserver(aOverlayKeyObserver)
{
	iLocation = aLocation;
	iRows	  = aRows;
	iCols	  = aCols;
	iSize	  = aSize;
	iMode	  = aMode;
}

void COnScreenKeypad::ConstructL(const CCoeControl& aContainer)
{
	SetContainerWindowL(aContainer);
	TRect overlayKeyPadRect(iLocation,iSize);
	SetRect(overlayKeyPadRect);
	
	TRect location = overlayKeyPadRect;

	iButtonGrid = new (ELeave) CButtonGrid(location,iRows,iCols,KOSKGridHorizontalMargin,KOSKGridVerticalMargin);
	PopulateKeysL();
	
	ActivateL();
}

COnScreenKeypad::~COnScreenKeypad()
{
	TInt count = iKeys.Count();
	for(TInt i=0; i<count; i++)
	{
		COnScreenKey* key = iKeys[i];
		if(key)
		{
			delete key;
			key = NULL;
		}
	}
	
	iKeys.Close();
	
	if(iButtonGrid)
	{
		delete iButtonGrid;
		iButtonGrid = NULL;
	}
	
}

void COnScreenKeypad::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();
	gc.SetBrushColor(TRgb(0,0,0));
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	gc.DrawRect(aRect);
}

void COnScreenKeypad::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
	CCoeControl::HandlePointerEventL(aPointerEvent);
}

void COnScreenKeypad::HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType)
{
    TEventCode eventCode = EEventNull;
    switch(aEventType)
    {
    case EEventPrepareFocusTransition:
        eventCode = EEventKeyDown;
        break;
    case EEventRequestFocus:
        eventCode = EEventKey;
        break;
    case EEventStateChanged:
        eventCode = EEventKeyUp;
        break;
    }
    
    COnScreenKey* key = static_cast<COnScreenKey*>(aControl);
    
    key->SetButtonState(eventCode);
    TKeyEvent keyEvent = key->GetKeyEvent();
    
    if(iOverlayObserver)
    {
    	if(key->IsRepeat())
    	{
    		iOverlayObserver->OfferOverlayKeyEvent(keyEvent,EEventKeyDown);
    		iOverlayObserver->OfferOverlayKeyEvent(keyEvent,EEventKey);
    	}
    	iOverlayObserver->OfferOverlayKeyEvent(keyEvent,eventCode);
    }
}

void COnScreenKeypad::PrepareForFocusGainL()
{
	
}

void COnScreenKeypad::PrepareForFocusLossL()
{
	
}

TInt COnScreenKeypad::CountComponentControls() const
{
	return iKeys.Count();
}

CCoeControl* COnScreenKeypad::ComponentControl(TInt aIndex) const
{
	if(aIndex < iKeys.Count())
	{
		return (CCoeControl*) iKeys[aIndex];
	}
	else
	{
		return NULL;	// we have no other controls other than the keys
	}
}

void COnScreenKeypad::PopulateKeysL()
{
	ClearAllKeys();
	
    switch(iMode)
    {
    case EPortrait:
    case EPortraitWithoutToggleScreenIcon:
		PopulateKeysForPortraitModeL();
		break;
    case ELandscape:
    case ELandscapeWithoutToggleScreenIcon:
		PopulateKeysForLandscapeModeL();
		break;
    case EQwertyLandscape:
		PopulateKeysForQwertyLandscapeModeL();
		break;
    case EPortraitWithOnlyToggleScreenIcon:
		PopulateKeysForPortraitToggleScreenIconModeL();
		break;
    }
}

void COnScreenKeypad::PopulateKeysForPortraitModeL()
{
    /*
     * Portrait layout
     * 
     * lsk | ^ | rsk
     *   < | * | >
     *   - | v | ts
     * 
     */
	
    AddKeyL(EKeyPressLSK,EKeyPressLSK,R_KEYPAD_BUTTON_LSK);
    AddKeyL(EKeyUpArrow,EStdKeyUpArrow,R_KEYPAD_BUTTON_UP);
    AddKeyL(EKeyPressRSK,EKeyPressRSK,R_KEYPAD_BUTTON_RSK);
    AddKeyL(EKeyLeftArrow,EStdKeyLeftArrow,R_KEYPAD_BUTTON_LEFT);
    AddKeyL(EKeyEnter,EStdKeyDevice3,R_KEYPAD_BUTTON_SELECT);
    AddKeyL(EKeyRightArrow,EStdKeyRightArrow,R_KEYPAD_BUTTON_RIGHT);
    AddKeyL(EKeyPressDummy,EKeyPressDummy);
    AddKeyL(EKeyDownArrow,EStdKeyDownArrow,R_KEYPAD_BUTTON_DOWN);
    AddKeyL(EKeyPressToggleScreen,EKeyPressToggleScreen, R_KEYPAD_BUTTON_TOGGLESCREEN);
    
    if(iMode ==EPortraitWithoutToggleScreenIcon)
    {
    	HideButton(EKeyPressToggleScreen);
    }
}

void COnScreenKeypad::PopulateKeysForPortraitToggleScreenIconModeL()
{
    /*
     * Portrait layout
     * 
     *   - | - | ts
     * 
     */
    AddKeyL(EKeyPressDummy,EKeyPressDummy);
    AddKeyL(EKeyPressDummy,EKeyPressDummy);	
    AddKeyL(EKeyPressToggleScreen,EKeyPressToggleScreen, R_KEYPAD_BUTTON_TOGGLESCREEN);
    
}

void COnScreenKeypad::PopulateKeysForLandscapeModeL()
{
	/*
	 * Landscape layout
	 * 
	 *   - | ^ | ts
	 *   < | * | >
	 * lsk | v | rsk
	 * 
	 */
	
    AddKeyL(EKeyPressDummy,EKeyPressDummy);
    AddKeyL(EKeyUpArrow,EStdKeyUpArrow,R_KEYPAD_BUTTON_UP);
    AddKeyL(EKeyPressToggleScreen,EKeyPressToggleScreen, R_KEYPAD_BUTTON_TOGGLESCREEN);
    AddKeyL(EKeyLeftArrow,EStdKeyLeftArrow,R_KEYPAD_BUTTON_LEFT);
    AddKeyL(EKeyEnter,EStdKeyDevice3,R_KEYPAD_BUTTON_SELECT);
    AddKeyL(EKeyRightArrow,EStdKeyRightArrow,R_KEYPAD_BUTTON_RIGHT);
    AddKeyL(EKeyPressLSK,EKeyPressLSK,R_KEYPAD_BUTTON_LSK);
    AddKeyL(EKeyDownArrow,EStdKeyDownArrow,R_KEYPAD_BUTTON_DOWN);
    AddKeyL(EKeyPressRSK,EKeyPressRSK,R_KEYPAD_BUTTON_RSK);
    
    if(iMode ==ELandscapeWithoutToggleScreenIcon)
    {
    	HideButton(EKeyPressToggleScreen);
    }

}

void COnScreenKeypad::PopulateKeysForQwertyLandscapeModeL()
{
	/*
	 * -
	 * -
	 * ts
	 * 
	 */
	
    AddKeyL(EKeyPressDummy,EKeyPressDummy);
    AddKeyL(EKeyPressDummy,EKeyPressDummy);
    AddKeyL(EKeyPressToggleScreen,EKeyPressToggleScreen, R_KEYPAD_BUTTON_TOGGLESCREEN);

}

void COnScreenKeypad::UpdateL(TPoint aLocation, TSize aSize, TOSKMode aMode, TInt aRows, TInt aCols)
{
	iLocation = aLocation;
    iSize     = aSize;
    iMode	  = aMode;
    
    if(aRows>0)
    {
        iRows = aRows;
    }
    if(aCols>0)
    {
        iCols = aCols;
    }
    
	TRect overlayKeyPadRect(iLocation,iSize);
	SetRect(overlayKeyPadRect);
	
	TRect newLocation = overlayKeyPadRect;
	
    iButtonGrid->ChangeLayout(newLocation,iRows,iCols,KOSKGridHorizontalMargin,KOSKGridVerticalMargin);
    PopulateKeysL();
}

void COnScreenKeypad::DisableToggleScreen()
{
	TInt count = iKeys.Count();
	for(TInt i=0; i<count; i++)
	{
		TKeyEvent keyEvent = iKeys[i]->GetKeyEvent();
		TInt code = (TInt)keyEvent.iCode;
		if(code == EKeyPressToggleScreen)
		{
			iKeys[i]->MakeVisible(EFalse);
			break;
		}
	}
}

void COnScreenKeypad::HandleForegroud()
{
	FocusChanged(EDrawNow);
}

COnScreenKey* COnScreenKeypad::SearchKey(TInt aKeyCode)
{
	TInt count = iKeys.Count();
	for(TInt i=0; i<count; i++)
	{
		if(iKeys[i]->GetKeyEvent().iCode == aKeyCode)
		{
			return iKeys[i]; 
		}
	}
	
	return NULL;
}

COnScreenKey* COnScreenKeypad::GetKeyL(TInt aKeyCode, TInt aScanCode, TInt aResourceId)
{
	COnScreenKey* key = SearchKey(aKeyCode);

	if(key == NULL)
	{
		TRect location(10,10,50,50);
		TKeyEvent keyEvent;
		keyEvent.iCode 	  = aKeyCode;
		keyEvent.iScanCode = aScanCode;
        keyEvent.iModifiers = 0;
        keyEvent.iRepeats = 0;        
		key = COnScreenKey::NewL(keyEvent,aResourceId,location,this,iContainer);
		iKeys.Append(key);
	}
	
	return key;
}

void COnScreenKeypad::AddKeyL(TInt aKeyCode, TInt aScanCode, TInt aResourceId)
{
	COnScreenKey* key = GetKeyL(aKeyCode,aScanCode,aResourceId);
	iButtonGrid->AddButton(key);
	
	if(aResourceId != 0)
	{
		key->MakeVisible(ETrue);
	}
	else
	{
		key->MakeVisible(EFalse);
	}
	
}

void COnScreenKeypad::HideButton(TInt aKeyCode)
{
	COnScreenKey* key = SearchKey(aKeyCode);
	if(key)
	{
		key->MakeVisible(EFalse);
	}
}

void COnScreenKeypad::RemoveAllKeys()
{
	// delete all the keys
    TInt count = iKeys.Count();
    for(TInt i=0; i<count; i++)
    {
        COnScreenKey* key = iKeys[i];
        delete key;
        key = NULL;
    }
    
    iKeys.Reset();

}

void COnScreenKeypad::ClearAllKeys()
{
	iButtonGrid->RemoveAllButtons();
	
	TInt count = iKeys.Count();
	for(TInt i=0; i<count; i++)
	{
		iKeys[i]->MakeVisible(EFalse);
	}
}

#endif // __FEATURE_OSK__
