/*
* Copyright (c) 2006-2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/


#include "saplayer_build.h"

#ifdef __FEATURE_OSK__
#include "on_screen_keypad_control.h"
#include "on_screen_keypad.h"


COnScreenKeypadControl* COnScreenKeypadControl::NewL(MOnScreenKeyObserver* aObserver, const TRect& aRect, TOSKMode aMode)
{
	COnScreenKeypadControl* self = new (ELeave) COnScreenKeypadControl();
	self->ConstructL(aObserver,aRect,aMode);
	return self;
}

COnScreenKeypadControl::COnScreenKeypadControl()
{
	iOnScreenKeypad = NULL;
}

COnScreenKeypadControl::~COnScreenKeypadControl()
{
	if(iOnScreenKeypad)
	{
		delete iOnScreenKeypad;
		iOnScreenKeypad = NULL;
	}
}

void COnScreenKeypadControl::ConstructL(MOnScreenKeyObserver* aObserver, const TRect& aRect, TOSKMode aMode)
{
	CreateWindowL();
	SetRect(aRect);
	
	TPoint	location(KOSKOnScreenKeypadPadding,KOSKOnScreenKeypadPadding);
	TSize	size(aRect.Width()-2*KOSKOnScreenKeypadPadding,aRect.Height()-2*KOSKOnScreenKeypadPadding);
	
	TInt	rows = KOSKDefaultRows;
	TInt	cols = KOSKDefaultCols;

    if(aMode == EQwertyLandscape)
    {
    	rows = KOSKQwertyLandscapeRows;
    	cols = KOSKQwertyLandscapeCols;
    }
    
    if(aMode == EPortraitWithOnlyToggleScreenIcon)
    {
    	rows = KOSKPortraitWithOneRow;
    	cols = KOSKPortraitWithThreeColumns;
    }
	
	iOnScreenKeypad = COnScreenKeypad::NewL(location,size,rows,cols,aMode,aObserver,*this);
	EnableDragEvents();
	ActivateL();
}

TInt COnScreenKeypadControl::CountComponentControls() const
{
	if(iOnScreenKeypad)
	{
		return 1;
	}
	
	return 0;
}

CCoeControl* COnScreenKeypadControl::ComponentControl(TInt aIndex) const
{
	if(aIndex == 0)
	{
		return iOnScreenKeypad;
	}
	
	return NULL;
}

void COnScreenKeypadControl::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();
	gc.SetBrushColor(KRgbBlack);
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	gc.DrawRect(aRect);
}

void COnScreenKeypadControl::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
	CCoeControl::HandlePointerEventL(aPointerEvent);
}

void COnScreenKeypadControl::UpdateL(TRect& aRect, TOSKMode aMode)
{
	SetRect(aRect);

	TPoint	location(KOSKOnScreenKeypadPadding,KOSKOnScreenKeypadPadding);
	TSize	size(aRect.Width()-2*KOSKOnScreenKeypadPadding,aRect.Height()-2*KOSKOnScreenKeypadPadding);
	
    TInt    rows = KOSKDefaultRows;
    TInt    cols = KOSKDefaultCols;
    
    if(aMode == EQwertyLandscape)
    {
    	rows = KOSKQwertyLandscapeRows;
    	cols = KOSKQwertyLandscapeCols;
    }
    
    if(aMode == EPortraitWithOnlyToggleScreenIcon)
    {
    	rows = KOSKPortraitWithOneRow;
     	cols = KOSKPortraitWithThreeColumns;
    }    

    iOnScreenKeypad->UpdateL(location,size,aMode,rows,cols);
    DrawDeferred();
}

void COnScreenKeypadControl::DisableToggleScreen()
{
	iOnScreenKeypad->DisableToggleScreen();
}

void COnScreenKeypadControl::HandleForeground()
{
	iOnScreenKeypad->HandleForegroud();
}

#endif // __FEATURE_OSK__

