/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
#include "../../inc/saplayer_build.h"

#ifdef __FEATURE_OSK__
#include "OnScreenKeypad.h"
//new
COnScreenKeypad* COnScreenKeypad::NewL() {
    COnScreenKeypad* self = new (ELeave) COnScreenKeypad(); //builds the key pointers iThekeys
    CleanupStack::PushL(self);
    self->ConstructL(); //adds to active scheduler and creates timer
    CleanupStack::Pop();
    return self;
}

COnScreenKeypad::COnScreenKeypad() : CActive(CActive::EPriorityUserInput) {
    iKeyVSpace = KKeyVSpace; 
    iKeyHSpace = KKeyHSpace; 
    iNavKeyH = KNavKeyH; 
    iNavKeyW = KNavKeyW; //*MUST* be even
    iSoftkeyReductionFactor=KSoftkeyReductionFactor;
    iInterKeySpace=KInterKeySpace;
    
    iBackButton=EFalse;
    iDuration=KEY_INTERVAL*1000; //required in micro seconds
    
    iPressed =-1;
    iActive = EFalse;
    iUpdateRequired=ETrue;
    
    iQwertyMode=EFalse;
}



TBool COnScreenKeypad::IsPressInKeypad(TInt aX, TInt aY) {
if (aX >= iX && aX <=iX+iKeybWidth) {
    if (aY >= iY && aY <= iY+iKeybHeight) {
    return ETrue;
    }
}
return EFalse;
}
void COnScreenKeypad::SetKeyboardSizePortrait(TRect aDimensions) {
    iQwertyMode=EFalse;
    
    //calculate dimensions
    iKeybHeight=aDimensions.Height(); //190
    iKeybWidth=aDimensions.Width(); //360
    iNavKeyH = (KPercentNavKeyH*iKeybHeight)/100; //55
    iNavKeyW = (KPercentNavKeyW*iKeybWidth)/100; //108
    iInterKeySpace = (KPercentInterKeySpace*iKeybHeight)/100; //6
    iSoftKeyH = iNavKeyH-((iSoftkeyReductionFactor*iNavKeyH)/100);
    iSoftKeyW = iNavKeyW-((iSoftkeyReductionFactor*iNavKeyW)/100);
    iKeyVSpace = (KPercentKeyVSpacing*iKeybHeight)/100; //4
    iKeyHSpace = (KPercentKeyHSpacing*iKeybWidth)/100;
    
    //configure keys
    TInt left, top ,right, bottom;
    
    iX=aDimensions.iTl.iX;
    iY=aDimensions.iTl.iY;
    
    
    //
    //MISC keys
    //
    //LSK
    left=iX;
    top = iY+iKeyVSpace;
    bottom = top+iSoftKeyH;
    right= left+iSoftKeyW;
    iTheKeys[5]->ResetPosition(left, top, right, bottom);
    
    //RSK
    left=iX+iKeybWidth-iSoftKeyW;
    right=iX+iKeybWidth;
    iTheKeys[6]->ResetPosition(left, top, right, bottom);
    
    //back button is included at the bottom
    
     //
     //navipad
     //
     //up
     left = iX+(iKeybWidth/2)-(iNavKeyW/2) + iInterKeySpace - iInterKeySpace; //the '/2' is used to calculate the mid point of the keypad 
     top = iY+iKeyVSpace;
     right = left+iNavKeyW;
     bottom = top+iNavKeyH;
     iTheKeys[0]->ResetPosition(left, top, right, bottom);
     
     //left
     left = iX+iKeyHSpace;
     top=bottom+iInterKeySpace;
     right=left+iNavKeyW-iInterKeySpace;
     bottom=top+(iNavKeyH);
     iTheKeys[1]->ResetPosition(left, top, right, bottom);

     //select key (enter)
     left=iX+(iKeybWidth/2)-(iNavKeyW/2) + iInterKeySpace - iInterKeySpace; //the '/2' is used to calculate the mid point of the keypad
     right=left+(iNavKeyW);
     iTheKeys[2]->ResetPosition(left, top, right, bottom);
     
     //right
     left = right+iInterKeySpace;
     right = iX+iKeybWidth-iKeyHSpace;
     iTheKeys[3]->ResetPosition(left, top, right, bottom);

     //down
     left = iX+(iKeybWidth/2)-(iNavKeyW/2) + iInterKeySpace - iInterKeySpace; //the '/2' is used to calculate the mid point of the keypad
     top = bottom+iInterKeySpace;
     right = left+iNavKeyW;
     bottom = top+iNavKeyH;
     iTheKeys[4]->ResetPosition(left, top, right, bottom);
     
     //back button
     top = iY+iKeybHeight-iNavKeyH;
     bottom = top+iNavKeyH;
     left=iX+iKeybWidth-iNavKeyW;
     right=iX+iKeybWidth;
     iTheKeys[7]->ResetPosition(left, top, right, bottom);
     
     iActive=ETrue;
}

void COnScreenKeypad::SetKeyboardSizeLandscape(TRect aDimensions) {
    iQwertyMode=EFalse;
    
    //calculate dimensions
    iKeybHeight=aDimensions.Height(); //190
    iKeybWidth=aDimensions.Width(); //360
    iNavKeyH = (KPercentNavKeyH*iKeybHeight)/100; //55
    iNavKeyW = (KPercentNavKeyW*iKeybWidth)/100; //108
    iInterKeySpace = (KPercentInterKeySpace*iKeybHeight)/100; //6
    iSoftKeyH = iNavKeyH-((iSoftkeyReductionFactor*iNavKeyH)/100);
    iSoftKeyW = iNavKeyW-((iSoftkeyReductionFactor*iNavKeyW)/100);
    iKeyVSpace = (KPercentKeyVSpacing*iKeybHeight)/100; //4
    iKeyHSpace = (KPercentKeyHSpacing*iKeybWidth)/100;
    
    //configure keys
    TInt left, top ,right, bottom;
    
    iX=aDimensions.iTl.iX;
    iY=aDimensions.iTl.iY;
    
    //back button
    top = iY+iKeyVSpace;
    bottom = top+iSoftKeyH;
    left=iX+iKeybWidth-iSoftKeyW;
    right=iX+iKeybWidth;
    iTheKeys[7]->ResetPosition(left, top, right, bottom);
    
    /*
     * The NUMBERS [#] below are the index of the key in the RPointerArray and are needed to track each key
     */
    
    //
    //navipad
    //
    //up
    left = iX+(iKeybWidth/2)-(iNavKeyW/2) + iInterKeySpace - iInterKeySpace; //the '/2' is used to calculate the mid point of the keypad
    top = iY+iKeyVSpace;
    right = left+iNavKeyW;
    bottom = top+iNavKeyH;
    iTheKeys[0]->ResetPosition(left, top, right, bottom);
    
    //left
    left = iX+iKeyHSpace;
    top=bottom+iInterKeySpace;
    right=left+iNavKeyW-iInterKeySpace;
    bottom=top+(iNavKeyH);
    iTheKeys[1]->ResetPosition(left, top, right, bottom);

    //select key (enter)
    left=iX+(iKeybWidth/2)-(iNavKeyW/2) + iInterKeySpace - iInterKeySpace; //the '/2' is used to calculate the mid point of the keypad
    right=left+(iNavKeyW);
    iTheKeys[2]->ResetPosition(left, top, right, bottom);
    
    //right
    left = right+iInterKeySpace;
    right = iX+iKeybWidth-iKeyHSpace;
    iTheKeys[3]->ResetPosition(left, top, right, bottom);

    //down
    left = iX+(iKeybWidth/2)-(iNavKeyW/2) + iInterKeySpace - iInterKeySpace; //the '/2' is used to calculate the mid point of the keypad
    top = bottom+iInterKeySpace;
    right = left+iNavKeyW;
    bottom = top+iNavKeyH;
    iTheKeys[4]->ResetPosition(left, top, right, bottom);
    
    //back button is included at the bottom

     //
     //MISC keys
     //
     //LSK
     left=iX+(iKeybWidth/2)-(iNavKeyW/2)-iNavKeyW; //the '/2' is used to calculate the mid point of the keypad
     top = iY+iKeybHeight-iSoftKeyH;
     bottom = top+iSoftKeyH;
     right= left+iSoftKeyW;
     iTheKeys[5]->ResetPosition(left, top, right, bottom);
     
     //RSK
     left=iX+iKeybWidth-iSoftKeyW;
     right=iX+iKeybWidth;
     iTheKeys[6]->ResetPosition(left, top, right, bottom);
     
     iActive=ETrue;
}

void COnScreenKeypad::SetKeyboardSizeLandscapeWithQwerty(TRect aDimensions) {
    iQwertyMode=ETrue;
    
    //calculate dimensions
    iKeybHeight=aDimensions.Height(); //190
    iKeybWidth=aDimensions.Width(); //360
    iNavKeyH = (KPercentNavKeyH*iKeybHeight)/100; //55
    iNavKeyW = (KPercentNavKeyW*iKeybWidth)/100; //108
    iInterKeySpace = (KPercentInterKeySpace*iKeybHeight)/100; //6
    iSoftKeyH = iNavKeyH-((iSoftkeyReductionFactor*iNavKeyH)/100);
    iSoftKeyW = iNavKeyW-((iSoftkeyReductionFactor*iNavKeyW)/100);
    iKeyVSpace = (KPercentKeyVSpacing*iKeybHeight)/100; //4
    iKeyHSpace = (KPercentKeyHSpacing*iKeybWidth)/100;
    
    //configure keys
    TInt left, top ,right, bottom;
    
    iX=aDimensions.iTl.iX;
    iY=aDimensions.iTl.iY;
    
    //back button - top right
    /*
    top = iY+iKeyVSpace;
    bottom = top+iSoftKeyH;
    left=iX+iKeyHSpace;
    right=iX+iKeybWidth;
    iTheKeys[7]->ResetPosition(left, top, right, bottom);
    */
    //back button - bottom right
    
    top = iY+iKeybHeight/*-iKeyVSpace*/-iSoftKeyH;
    bottom = top+iSoftKeyH;
    left=iX+iKeyHSpace;
    right=iX+iKeybWidth;
    iTheKeys[7]->ResetPosition(left, top, right, bottom);
    
    //
    // The remaining keys will not be drawn in QWERTY mode
    //
    
    /*
     * The NUMBERS [#] below are the index of the key in the RPointerArray and are needed to track each key
     */
    
    //
    //navipad
    //
    //up
    left = iX+(iKeybWidth/2)-(iNavKeyW/2) + iInterKeySpace - iInterKeySpace; //the '/2' is used to calculate the mid point of the keypad
    top = iY+iKeyVSpace;
    right = left+iNavKeyW;
    bottom = top+iNavKeyH;
    iTheKeys[0]->ResetPosition(left, top, right, bottom);
    
    //left
    left = iX+iKeyHSpace;
    top=bottom+iInterKeySpace;
    right=left+iNavKeyW-iInterKeySpace;
    bottom=top+(iNavKeyH);
    iTheKeys[1]->ResetPosition(left, top, right, bottom);

    //select key (enter)
    left=iX+(iKeybWidth/2)-(iNavKeyW/2) + iInterKeySpace - iInterKeySpace; //the '/2' is used to calculate the mid point of the keypad
    right=left+(iNavKeyW);
    iTheKeys[2]->ResetPosition(left, top, right, bottom);
    
    //right
    left = right+iInterKeySpace;
    right = iX+iKeybWidth-iKeyHSpace;
    iTheKeys[3]->ResetPosition(left, top, right, bottom);

    //down
    left = iX+(iKeybWidth/2)-(iNavKeyW/2) + iInterKeySpace - iInterKeySpace; //the '/2' is used to calculate the mid point of the keypad
    top = bottom+iInterKeySpace;
    right = left+iNavKeyW;
    bottom = top+iNavKeyH;
    iTheKeys[4]->ResetPosition(left, top, right, bottom);
    
    //back button is included at the bottom

     //
     //MISC keys
     //
     //LSK
     left=iX+(iKeybWidth/2)-(iNavKeyW/2)-iNavKeyW; //the '/2' is used to calculate the mid point of the keypad
     top = iY+iKeybHeight-iSoftKeyH;
     bottom = top+iSoftKeyH;
     right= left+iSoftKeyW;
     iTheKeys[5]->ResetPosition(left, top, right, bottom);
     
     //RSK
     left=iX+iKeybWidth-iSoftKeyW;
     right=iX+iKeybWidth;
     iTheKeys[6]->ResetPosition(left, top, right, bottom);
     
     iActive=ETrue;
}

void COnScreenKeypad::ConstructL() {

    //find the icons file! - this is needed because of avkon not picking the right icons file 
    TChar     driveChar;
    TFileName fileName;
    TBool     found ( EFalse );
    TDriveList driveList;
    RFs fs;
    User::LeaveIfError( fs.Connect() );
    CleanupClosePushL( fs );
    fs.DriveList( driveList );
    fileName.Append( KMyFile );
    for( TInt drive = EDriveY; drive >= EDriveA; drive-- )
        {
        if ( driveList[ drive ] != 0 )
            {
            // Drive exists.
            // You can also check drive attributes and search files in
            // internal drives only for example (KDriveAttInternal)
            TDriveInfo info;
            User::LeaveIfError( fs.Drive( info, drive ) );
            RFs::DriveToChar( drive, driveChar );
            fileName[0] = driveChar;
            // Check if file exists in that drive
            if ( BaflUtils::FileExists( fs, fileName ) )
                {
                // Found file. you can  take it into use
                found = ETrue;
                break;
                }
            }
        }
    if ( ! found )
        {
        // File not found in drives from Y to A. Search drive Z
        RFs::DriveToChar( EDriveZ, driveChar );
        fileName[0] = driveChar;
        if ( BaflUtils::FileExists( fs, fileName ) )
            {
            // found in Z - last resort
            found = ETrue;
            }
        }
    CleanupStack::PopAndDestroy( &fs );
    
    if (!found) fileName=NULL;
    
    //add the keys with the scancode, keycode, icon, and caption
    //up
    iTheKeys.Append(COverlayKey::NewL(EKeyUpArrow, EStdKeyUpArrow, EMbmKeypad_fl_tQgn_indi_flash_keypad_up, fileName));
    //left
    iTheKeys.Append(COverlayKey::NewL(EKeyLeftArrow, EStdKeyLeftArrow, EMbmKeypad_fl_tQgn_indi_flash_keypad_left, fileName));
    //select key (enter)
    iTheKeys.Append(COverlayKey::NewL(EKeyEnter , EStdKeyDevice3, EMbmKeypad_fl_tQgn_indi_flash_keypad_select, fileName));
    //right
    iTheKeys.Append(COverlayKey::NewL(EKeyRightArrow, EStdKeyRightArrow, EMbmKeypad_fl_tQgn_indi_flash_keypad_right, fileName));
    //down
    iTheKeys.Append(COverlayKey::NewL(EKeyDownArrow, EStdKeyDownArrow, EMbmKeypad_fl_tQgn_indi_flash_keypad_down, fileName));
    //LSK
    iTheKeys.Append(COverlayKey::NewL(EKeyPressLSK, EKeyPressLSK, EMbmKeypad_fl_tQgn_indi_flash_keypad_lsk, fileName));
    //RSK
    iTheKeys.Append(COverlayKey::NewL(EKeyPressRSK, EKeyPressRSK, EMbmKeypad_fl_tQgn_indi_flash_keypad_rsk, fileName));
    //back button
    iTheKeys.Append(COverlayKey::NewL(EKeyPressBack, EKeyPressBack, EMbmKeypad_fl_tQgn_indi_flash_keypad_exit_fs, fileName));
    iTimer.CreateLocal();
    CActiveScheduler::Add(this);
}

void COnScreenKeypad::DoCancel() {
    iTimer.Cancel();
}

void COnScreenKeypad::SetCustomSoftKeys (TBool aLsk, TBool aRsk) {
    iLSKOverride=aLsk;
    iRSKOverride=aRsk;
}

void COnScreenKeypad::SetBackButton(TBool aShow) {
    iBackButton=aShow;
}

TBool COnScreenKeypad::IsLSKOverride() {
    return iLSKOverride;
}
TBool COnScreenKeypad::IsRSKOverride(){
    return iRSKOverride;
}


/*
void COnScreenKeypad::IsLSKactive() {
    return iLSKOverride;
}

void COnScreenKeypad::IsLSKactive() {
    return iRSKOverride;
}
*/


void COnScreenKeypad::SendPressedKey(MFlashEngine* aEngine) {
    TKeyEvent keyEvent;
    TUint delay;
    
    if (aEngine) {
    iTempEngine=aEngine;
    delay=KEY_INTERVAL*3;
    }
    else {
    delay=KEY_INTERVAL;
    }

    keyEvent.iCode      = GetKeyCode();
    keyEvent.iScanCode  = GetScanCode();
    keyEvent.iModifiers = EModifierKeypad;
    keyEvent.iRepeats   = 0;
    TRAPD(err, iTempEngine->OfferKeyEventL(keyEvent, EEventKey));
    if (err!=KErrNone) {
        // unable to send the key, theres no need to crash
    }
    //iTempEngine->OfferKeyEventL(keyEvent, EEventKey);
    //fix for AKAR-7N4DRS:FL3.1, Tube : ATS content for Inline_Text crashed the player.
    Cancel();
    iTimer.After(iStatus,delay*1000); //1000microsesonds
    SetActive();
}
void COnScreenKeypad::RunL() {
    SendPressedKey(NULL);
}
void COnScreenKeypad::AbortPressedKey(MFlashEngine* aEngine) {
    iTimer.Cancel();
    TKeyEvent keyEvent;
    keyEvent.iCode      = GetKeyCode();
    keyEvent.iScanCode  = GetScanCode();
    keyEvent.iModifiers = EModifierKeypad;
    keyEvent.iRepeats   = 0;                   
    if( !aEngine )
        return;
    //aEngine->OfferKeyEventL(keyEvent, EEventKeyUp);
    TRAPD(err, aEngine->OfferKeyEventL(keyEvent, EEventKeyUp));
    if (err!=KErrNone) {
        // unable to send key, theres no need to crash
    }
    Cancel();
    ResetiPressed();
}


COnScreenKeypad::~COnScreenKeypad() {
    if (iTimer.Handle()!=0) {
    iTimer.Cancel();
    iTimer.Close();
    }
    iTheKeys.ResetAndDestroy();
}

void COnScreenKeypad::SetDrawUpdate(TBool aVal) {
    iUpdateRequired=aVal;
}
TBool COnScreenKeypad::GetDrawUpdate(){
    return iUpdateRequired;
}


void COnScreenKeypad::DrawKeypad(CWindowGc& mygc/*, TInt aX, TInt aY*/) {     
    TInt i=0;
    TInt keys=0;
    keys=iTheKeys.Count();
    
    //background
    mygc.SetBrushStyle((CGraphicsContext::TBrushStyle) 1 );
    mygc.SetBrushColor(TRgb(KColorBlack, KColorBlack, KColorBlack)); //??
    mygc.SetPenColor(TRgb(KColorBlack, KColorBlack, KColorBlack)); //draws in grey
    mygc.DrawRect(TRect(iX, iY, iX+iKeybWidth, iY+iKeybHeight));
    
    //this draws svgs
    
    //in QWERTY Mode - only draw the back key
    if (iQwertyMode) {
        iTheKeys[7]->DrawSVGKey(mygc, EFalse);
        return;
    }

    for (i=0; i<keys; i++) {
        if (i==5 || i==6 || i==7) { //the numbers here are the index of the key in the RPointerArray
            //LSK, RSK
            if (i==5 || i==6 ) {
                iTheKeys[i]->DrawSVGKey(mygc, EFalse);
            }
            //back button
            if (i==7 && iBackButton) {
                iTheKeys[i]->DrawSVGKey(mygc, EFalse);
            }
        }
        else {
                if (iPressed==i) {
                    iTheKeys[i]->DrawSVGKeyPressed(mygc);
            }
            else {
                iTheKeys[i]->DrawSVGKey(mygc, ETrue);
            }
        }
    } //end for 
}

TInt COnScreenKeypad::GetiPressed() {
return iPressed;
}
void COnScreenKeypad::ResetiPressed() {
    Cancel();
    iPressed=-1;
}

//remove this
/*
TRect COnScreenKeypad::GetiPressedRect() {
    if (iPressed!=-1)
        return iTheKeys[iPressed]->GetKeyRect();
}
*/

TRect COnScreenKeypad::GetDimensions() {
    return TRect(iX, iY, iX+iKeybWidth, iY+iKeybHeight);
}


void COnScreenKeypad::Activate() {
    iActive=ETrue;
}
void COnScreenKeypad::Deactivate() {
    Cancel();
    iPressed=-1;
    iActive=EFalse;
}

TInt COnScreenKeypad::CheckKey(TInt aX, TInt aY) {
    TInt i=0;
    TInt pressed=-1;
    TInt keys=0;
    
    //in QWERTY Mode - only check for the back button
    if (iQwertyMode) {
        if (iTheKeys[7]->IsClicked(aX, aY))
         {
            pressed=iTheKeys[7]->GetKeyCode();
            iPressed=7;
            if (iBackButton)
                return pressed;
        }
        return -1;
    }
    
    
    keys = iTheKeys.Count();
    for (i=0; i<keys; i++) {
        if (iTheKeys[i]->IsClicked(aX, aY)) {
            pressed=iTheKeys[i]->GetKeyCode();
            iPressed=i;
            break;
        }
        iPressed=-1;
    }
    
    //back button activated or not?
    if (iPressed==7) { //the numbers here are the index of the key in the RPointerArray
        if (iBackButton) return pressed;
        else return -1;
    }
    return pressed;
}
TInt COnScreenKeypad::GetKeyCode() {
if (iPressed!=-1)
    return iTheKeys[iPressed]->GetKeyCode();
return -1;
}
TInt COnScreenKeypad::GetScanCode(){
if (iPressed!=-1)
    return iTheKeys[iPressed]->GetScanCode();
return -1;
}


TBool COnScreenKeypad::IsActive() {
    return iActive;
}

#endif

