/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
#include "../../inc/saplayer_build.h"

#ifdef __FEATURE_OSK__
#include "OverlayKey.h"

COverlayKey* COverlayKey::NewL(TInt aKeyCode, TInt aScanCode, TInt aIconID, TFileName aIconsFile)
{
    COverlayKey* self = new (ELeave) COverlayKey(aKeyCode, aScanCode, aIconID, aIconsFile);
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop();
    return self;
}

COverlayKey::COverlayKey(TInt aKeyCode, TInt aScanCode, TInt aIconID, TFileName aIconsFile) {
    iKeyCode=aKeyCode;
    iScanCode=aScanCode;
    iIconID=aIconID;
    
    iIconsFile=aIconsFile;
}

void COverlayKey::ConstructL()
{
    TRAPD(err, AknIconUtils::CreateIconL(iBitmap, iBitmapMask, iIconsFile, iIconID, iIconID+1));
    if (err!=KErrNone) {
        // we dont have the icons..
        User::Leave(err);
    }
    if (iBitmap)
        AknIconUtils::ExcludeFromCache(iBitmap);
}


void COverlayKey::ResetPosition(TInt aLeft, TInt aTop, TInt aRight, TInt aBottom) {
    iLeftVal = aLeft;
    iTopVal = aTop;
    iRightVal = aRight;
    iBottomVal = aBottom;
}

TInt COverlayKey::GetKeyCode() {
    //return keycode;
    return iKeyCode;
}

TInt COverlayKey::GetScanCode() {
    //return keycode;
    return iScanCode;
}


TBool COverlayKey::IsClicked(TInt aXval, TInt aYval) {
    if (aXval >= iLeftVal && aXval <=iRightVal) {
        if (aYval >= iTopVal && aYval <= iBottomVal) {
        return ETrue;
        }
    }
    return EFalse;
}

void COverlayKey::DrawSVGKey(CWindowGc& gc, TBool aThickBorder) {
    if (aThickBorder) {
        //outer border
        gc.SetBrushStyle((CGraphicsContext::TBrushStyle) 1 );
        gc.SetBrushColor(TRgb(KColorDarkGrey, KColorDarkGrey, KColorDarkGrey));
        gc.SetPenColor(TRgb(KColorDarkGrey, KColorDarkGrey, KColorDarkGrey)); 
        gc.DrawRect(TRect(iLeftVal, iTopVal, iRightVal, iBottomVal));
        //inner fill
        gc.SetBrushStyle((CGraphicsContext::TBrushStyle) 1 );
        gc.SetBrushColor(TRgb(KColorLightGrey, KColorLightGrey, KColorLightGrey)); 
        gc.SetPenColor(TRgb(KColorLightGrey, KColorLightGrey, KColorLightGrey)); 
        gc.DrawRect(TRect(iLeftVal+KThickBorderWidth, iTopVal+KThickBorderWidth, iRightVal-KThickBorderWidth, iBottomVal-KThickBorderWidth));
    }
    else {
        //border
        gc.SetBrushStyle((CGraphicsContext::TBrushStyle) 0 );
        gc.SetBrushColor(TRgb(KColorDarkGrey, KColorDarkGrey, KColorDarkGrey)); 
        gc.SetPenColor(TRgb(KColorDarkGrey, KColorDarkGrey, KColorDarkGrey)); 
        gc.DrawRect(TRect(iLeftVal, iTopVal, iRightVal, iBottomVal));
        //inner fill
        gc.SetBrushStyle((CGraphicsContext::TBrushStyle) 1 );
        gc.SetBrushColor(TRgb(KColorLightGrey, KColorLightGrey, KColorLightGrey));
        gc.SetPenColor(TRgb(KColorLightGrey, KColorLightGrey, KColorLightGrey)); 
        gc.DrawRect(TRect(iLeftVal+KBorderWidth, iTopVal+KBorderWidth, iRightVal-KBorderWidth, iBottomVal-KBorderWidth));
    }

    if (iBitmap && iBitmapMask) {
    AknIconUtils::SetSize(iBitmap, TSize((iRightVal-iLeftVal)-1, (iBottomVal-iTopVal)-1));
    gc.BitBltMasked(TPoint(iLeftVal, iTopVal), iBitmap, TRect(TPoint(0,0), TSize((iRightVal-iLeftVal)-1, (iBottomVal-iTopVal)-1)), iBitmapMask, EFalse);
    }
    else {
    // alternative to drawing the icon
    }
    
}

void COverlayKey::DrawSVGKeyPressed(CWindowGc& gc/*, TBool aThickBorder*/) {
        //border
    
        gc.SetBrushStyle((CGraphicsContext::TBrushStyle) 1 );
        gc.SetBrushColor(TRgb(KColorDarkGrey, KColorDarkGrey, KColorDarkGrey));
        gc.SetPenColor(TRgb(KColorDarkGrey, KColorDarkGrey, KColorDarkGrey)); //draws in grey
        gc.DrawRect(TRect(iLeftVal, iTopVal, iRightVal, iBottomVal));

    if (iBitmap && iBitmapMask) {
    AknIconUtils::SetSize(iBitmap, TSize((iRightVal-iLeftVal)-1, (iBottomVal-iTopVal)-1));
    gc.BitBltMasked(TPoint(iLeftVal, iTopVal), iBitmap, TRect(TPoint(0,0), TSize((iRightVal-iLeftVal)-1, (iBottomVal-iTopVal)-1)), iBitmapMask, EFalse);
    }
    else {
    // alternative to drawing the icon
    }
}



TRect COverlayKey::GetKeyRect() {
return TRect(iLeftVal, iTopVal, iRightVal, iBottomVal);
}

COverlayKey::~COverlayKey() {
    if (iBitmap) {
    delete iBitmap;
    iBitmap=NULL;
    }
    if (iBitmapMask) {
    delete iBitmapMask;
    iBitmapMask=NULL;
    }
}
#endif
