/*
* Copyright (c) 2006-2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description: 
*       Provides the CSaFlashPlayerView Menu Manager implementation.
*
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "saplayer_build.h"

#include "playerviewmenumgr.h"

#include "symbian_playerview.h"
#include "symbian_playercontrol.h"
#include "flash_engine.h"
#include "symbian_appui.h"
#include "saflash_consts.h"

//UI includes
#include <flash2ui.rsg>
#include "flash2ui.hrh"
#include "ui_defs.h"

#include <pathinfo.h> //PathInfo 
#include <coemain.h>
#include <featmgr.h>

#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
#include <bautils.h>
#include <aknstyluspopupmenu.h>
#endif

//constants

/**
 * Symbian constructor.
 *
 * @since S60 v3.1
 * @return Pointer to created CPlayerViewMenuManager object
 */
CPlayerViewMenuManager* CPlayerViewMenuManager::NewL(CSaFlashPlayerView* aView
													,CSaFlashPlayerControl* aCtrl
													,CSaFlashAppUi* aAppUI)
{

	CPlayerViewMenuManager* self = new(ELeave) CPlayerViewMenuManager(aView,aCtrl,aAppUI);

	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);

	return self;	
}

/**
 * Destructor.
 */
CPlayerViewMenuManager::~CPlayerViewMenuManager()
{
#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
    if(iStylusPopupMenu)
    {
        delete iStylusPopupMenu;
        iStylusPopupMenu = NULL;
    }
#endif
    
	FeatureManager::UnInitializeLib(); // Uninitialise Feature manager
}
    
/**
 * C++ default constructor.
 *
 * @param aView CSaFlashPlayerView object.
 * @param aCtrl CSaFlashPlayerControl object.
 * @param aAppUI CSaFlashAppUi object.
 */
CPlayerViewMenuManager::CPlayerViewMenuManager(CSaFlashPlayerView* aView
												,CSaFlashPlayerControl* aCtrl
												,CSaFlashAppUi* aAppUI):
												 iView(aView)
												,iCtrl(aCtrl)
												,iAppUI(aAppUI)
												,iIsTouchFeatureSupported(EFalse)
#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
												,iStylusPopupMenu(NULL)
#endif
{
	
}

/**
    * By default Symbian 2nd phase constructor is private.
    */
void CPlayerViewMenuManager::ConstructL()
{
    FeatureManager::InitializeLibL(); // initialise feature manager
    iIsTouchFeatureSupported = FeatureManager::FeatureSupported(KFeatureIdPenSupport);
    FeatureManager::UnInitializeLib(); // Uninitialise Feature manager
}
	
	

/**
 * Init Menu items.
 * @since S60 v3.1
 * @param aResourceId   Id of the resource
 * @param aMenuPane   	MenuPane Object
 * @return None.
 */
void CPlayerViewMenuManager::DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane)
{
	if (aResourceId == R_FLASHPLAYER_QUALITY_MENU)
	{
		aMenuPane->SetItemButtonState(iView->GetQuality(), EEikMenuItemSymbolOn);
	}
	else if (aResourceId == R_FLASHPLAYER_CONTROL_MENU)
	{
		DynInitControlMenuPaneL(aMenuPane );
	}
	
}



/**
 * Init Menu bar.
 * @since S60 v3.1
 * @param aResourceId   Id of the resource
 * @param aMenuBar   	MenuBar Object
 * @return None.
 */
void CPlayerViewMenuManager::DynInitMenuBarL(TInt /*aResourceId*/,CEikMenuBar* /*aMenuBar*/)
{
	
}

void CPlayerViewMenuManager::DynInitMenuPaneforVideoL(CEikMenuPane* aMenuPane )
{
	MFlashEngine* engine = iCtrl->GetEngine();
	
	aMenuPane->SetItemDimmed(EEikFlashCmdZoomIn, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdZoomOut, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdPanMode, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdNonPanMode, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdPlay, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdShowAll, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdRewind, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdStop, ETrue);
	
	//Part of Fix for EHLI-7YXE63. Save Removed from Options Menu as it is
	//No Longer Required.
	/*
	TBool showSaveOption = iView->FileExistsInDefaultFolder();
	if (showSaveOption == EFalse)
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdSave,
			(engine->PlayerStoppedWithError() ? ETrue : EFalse) );
	}
	*/
	//else
	//{
	aMenuPane->SetItemDimmed(EEikFlashCmdSave, ETrue);//Always Dimmed
	//}

	if (engine && engine->IsPaused())
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdPause, ETrue);
	}
	else
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdResume, ETrue);
	}
}

void CPlayerViewMenuManager::DynInitMenuPaneforRunModeL(CEikMenuPane* aMenuPane )
{
	MFlashEngine* pEngine = iCtrl->GetEngine();
	
	if (pEngine->IsPaused())
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdZoomIn, ETrue);
		aMenuPane->SetItemDimmed(EEikFlashCmdZoomOut, ETrue);
		aMenuPane->SetItemDimmed(EEikFlashCmdPause, ETrue);
		aMenuPane->SetItemDimmed(EEikFlashCmdPlay, ETrue);
		aMenuPane->SetItemDimmed(EEikFlashCmdShowAll, ETrue);
		aMenuPane->SetItemDimmed(EEikFlashCmdPanMode, ETrue);
		aMenuPane->SetItemDimmed(EEikFlashCmdNonPanMode, ETrue);
	}
	else
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdPlay, ETrue);
		aMenuPane->SetItemDimmed(EEikFlashCmdResume, ETrue);
	}
	
    // lets query the viewer to find out if we should show the following menu items 
	if( !iView->ShowPlayOption() )
	{
        aMenuPane->SetItemDimmed(EEikFlashCmdPause, ETrue);
        aMenuPane->SetItemDimmed(EEikFlashCmdPlay, ETrue);
        aMenuPane->SetItemDimmed(EEikFlashCmdResume, ETrue);
	}
	
	if (!pEngine->IsZoomable())
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdZoomIn, ETrue);
		aMenuPane->SetItemDimmed(EEikFlashCmdZoomOut, ETrue);
		aMenuPane->SetItemDimmed(EEikFlashCmdShowAll, ETrue);
		aMenuPane->SetItemDimmed(EEikFlashCmdPanMode, ETrue);
		aMenuPane->SetItemDimmed(EEikFlashCmdNonPanMode, ETrue);
	}
	else
	{
		if ( pEngine->IsZoomed() && (!pEngine->IsPaused()) )
		{
			aMenuPane->SetItemDimmed(EEikFlashCmdZoomOut, EFalse);
			aMenuPane->SetItemDimmed(EEikFlashCmdShowAll, EFalse);					
			if (pEngine->IsPanning())
			{
				aMenuPane->SetItemDimmed(EEikFlashCmdPanMode, ETrue);
				aMenuPane->SetItemDimmed(EEikFlashCmdNonPanMode, EFalse);
			}
			else
			{
				aMenuPane->SetItemDimmed(EEikFlashCmdPanMode, EFalse);
				aMenuPane->SetItemDimmed(EEikFlashCmdNonPanMode, ETrue);
			}
		}
		else
		{
			aMenuPane->SetItemDimmed(EEikFlashCmdZoomOut, ETrue);
			aMenuPane->SetItemDimmed(EEikFlashCmdShowAll, ETrue);
			aMenuPane->SetItemDimmed(EEikFlashCmdPanMode, ETrue);
			aMenuPane->SetItemDimmed(EEikFlashCmdNonPanMode, ETrue);
		}				
	}

	if(!iView->CanZoomInfurther())
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdZoomIn, ETrue);
	}
	
	if(!iView->CanZoomOutfurther())
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdZoomOut, ETrue);
	}
	
	
}

void CPlayerViewMenuManager::DynInitMenuPaneforNoEngineModeL(CEikMenuPane* aMenuPane)
{
    DynInitMenuPaneforNoRunModeL(aMenuPane);
    aMenuPane->SetItemDimmed(EEikFlashCmdRotate, ETrue);
    aMenuPane->SetItemDimmed(EEikFlashCmdQuality, ETrue);
    aMenuPane->SetItemDimmed(EEikFlashCmdSave, ETrue);
    aMenuPane->SetItemDimmed(EEikFlashCmdVolume, ETrue);
    aMenuPane->SetItemDimmed(EEikFlashCmdQuality, ETrue);
    aMenuPane->SetItemDimmed(EEikFlashCmdMute, ETrue);
    aMenuPane->SetItemDimmed(EEikFlashCmdUnmute, ETrue);
    aMenuPane->SetItemDimmed(EEikFlashCmdUnmute, ETrue);
    aMenuPane->SetItemDimmed(EEikFlashCmdFullScreen, ETrue);
    aMenuPane->SetItemDimmed(EEikFlashCmdPartScreen, ETrue);
}

void CPlayerViewMenuManager::DynInitMenuPaneforNoRunModeL(CEikMenuPane* aMenuPane )
{
	aMenuPane->SetItemDimmed(EEikFlashCmdZoomIn, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdZoomOut, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdResume, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdStop, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdShowAll, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdPanMode, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdNonPanMode, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdPause, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdRewind, ETrue);
	aMenuPane->SetItemDimmed(EEikFlashCmdViewDetails, ETrue);

    if(iIsTouchFeatureSupported)
    {
        // play is there in the toolbar 
        aMenuPane->SetItemDimmed(EEikFlashCmdPlay, ETrue);
    }

	iAppUI->DoSendMenuL(aMenuPane, KMaximumMessageSize, EEikFlashCmdAbout);
}

void CPlayerViewMenuManager::DynInitControlMenuPaneL(CEikMenuPane* aMenuPane )
{
	MFlashEngine* pEngine = iCtrl->GetEngine();

	if(pEngine == NULL)
	{
        DynInitMenuPaneforNoEngineModeL(aMenuPane);
		return;
	}

	if(! (pEngine->IsInitialized()))
	{
		return;
	}
	
	if (iView->IsVolumeKeySupported())
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdVolume, ETrue);	
	}
	
	aMenuPane->SetItemDimmed(EEikFlashCmdRotate
							, ETrue/*iView->IsDisableMenuRotateScreen()*/);//removing the roate screen command as it has become obsolete now.
							
						
							
	#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
	if( iView->IsVideoContent()
	/* Show  Video Specific Menu for any kind of video */ 
	        /*
        #ifdef __FEATURE_FLV_RECOG_PLAY
            && !(iView->IsFlvFilePlaying())
        #endif
        #ifdef __FEATURE_F4V_RECOG_PLAY
            && !(iView->IsF4vFilePlaying())
        #endif // __FEATURE_FLV_RECOG_PLAY
        */
	        )
	{
		DynInitMenuPaneforVideoL(aMenuPane);
	}
	else
	#endif
	{
		if (pEngine->IsRunning())
		{
			DynInitMenuPaneforRunModeL(aMenuPane);
		}
		else
		{
			DynInitMenuPaneforNoRunModeL(aMenuPane);
		}
	
	//Part of Fix for EHLI-7YXE63. Save Removed from Options Menu as it is
	//No Longer Required.	
		/*
		TBool showSaveOption = iView->FileExistsInDefaultFolder();
		
		if(iAppUI->IsStubApp())
		    {
		        showSaveOption = ETrue;
		    }
		    
		    
		
		//if ( (iLaunchType == EEmbedded) || (showSaveOption == EFalse))
		if (showSaveOption == EFalse)
		{
			aMenuPane->SetItemDimmed(EEikFlashCmdSave,
				(pEngine->PlayerStoppedWithError() ? ETrue : EFalse) );
		}
		*/
		//else
		//{
	aMenuPane->SetItemDimmed(EEikFlashCmdSave, ETrue);//Always Dimmed
		//}
		
		
	}
	
	//common menu options here
	if(iView->IsVolumeMuted() )
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdMute, ETrue);
	}
	else
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdUnmute, ETrue);
	}

    if (!iView->IsFullScreen() && iIsTouchFeatureSupported)
    {
        // if its a touch device, we have the toolbar to pause/resume
        aMenuPane->SetItemDimmed(EEikFlashCmdPause, ETrue);
        aMenuPane->SetItemDimmed(EEikFlashCmdResume, ETrue);
    }	

    if(iIsTouchFeatureSupported)
    {
        // if its a touch device, we have the toolbar to go to full screen 
        // and the osk to come back to normal screen. Hence no need for these menu options
        aMenuPane->SetItemDimmed(EEikFlashCmdFullScreen, ETrue);
        aMenuPane->SetItemDimmed(EEikFlashCmdPartScreen, ETrue);
    }
    else
    {
        // non touch
        if (iView->IsFullScreen())
        {
            aMenuPane->SetItemDimmed(EEikFlashCmdFullScreen, ETrue);
        }
        else
        {
            aMenuPane->SetItemDimmed(EEikFlashCmdPartScreen, ETrue);
        }
    }
    
	if (!iAppUI->HelpAppExists())
	{
		aMenuPane->SetItemDimmed(EEikFlashCmdHelp, ETrue);
	}

}

#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
void CPlayerViewMenuManager::DynInitContextSensitiveMenu(MEikMenuObserver* aObserver, const TPoint& aPoint)
{
    if(!iStylusPopupMenu)
    {
        TRAPD(err, iStylusPopupMenu = CAknStylusPopUpMenu::NewL(aObserver,aPoint));
		if(err != KErrNone)
			return;
			
        TResourceReader reader;
        CEikonEnv::Static()->CreateResourceReaderLC(reader,R_STYLUS_POPUP_MENU);
        iStylusPopupMenu->ConstructFromResourceL(reader);
        CleanupStack::PopAndDestroy();
    }
    
    // decide what to show and what to hide
    MFlashEngine* pEngine = iCtrl->GetEngine();
    if(pEngine->IsPaused())
    {
        iStylusPopupMenu->SetItemDimmed(EEikFlashCmdPause,ETrue);
        iStylusPopupMenu->SetItemDimmed(EEikFlashCmdResume,EFalse);
    }
    else
    {
        iStylusPopupMenu->SetItemDimmed(EEikFlashCmdPause,EFalse);
        iStylusPopupMenu->SetItemDimmed(EEikFlashCmdResume,ETrue);
    }
    
    if(iView->IsFullScreen() && !pEngine->GetFullScreenViaFSCommand())
    {
        iStylusPopupMenu->SetItemDimmed(EEikFlashCmdPartScreen,EFalse);
        iStylusPopupMenu->SetItemDimmed(EEikFlashCmdFullScreen,ETrue);
    }
    else
    {
        iStylusPopupMenu->SetItemDimmed(EEikFlashCmdPartScreen,ETrue);
        iStylusPopupMenu->SetItemDimmed(EEikFlashCmdFullScreen,EFalse);
    }

    iStylusPopupMenu->SetItemDimmed(EEikFlashCmdMute,ETrue);
    iStylusPopupMenu->SetItemDimmed(EEikFlashCmdUnmute,ETrue);

    if(pEngine->GetSwfHasAudio())
    {
        /* When Muted and SWF is reopened, stylus popup menu 
         * includes 'Mute', instead of 'Unmute' as displayed 
         * in 'Options' menu. */
		if(iView->IsVolumeMuted())
        {
            // we have muted
            iStylusPopupMenu->SetItemDimmed(EEikFlashCmdUnmute,EFalse);
        }
        else
        {
            iStylusPopupMenu->SetItemDimmed(EEikFlashCmdMute,EFalse);
        }
    }

    iStylusPopupMenu->ShowMenu();
    iStylusPopupMenu->SetPosition(aPoint, CAknStylusPopUpMenu::EPositionTypeRightBottom);
}
#endif
