/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
                                 
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "saplayer_build.h"
#include "symbian_appui.h"
#include "symbian_uiutil.h"
#include "symbian_document.h"
#include "symbian_playerview.h"
#include "symbian_textinput.h"
#include "symbian_fileview.h"
#include "symbian_volumeview.h"
#include "flash2ui.hrh"
#include "flash2ui.hlp.hrh"
#include "flash_engine.h"
#include "hlplch.h"
#include "symbian_clib.h"
#include "saflash_coredefs.h"
#include "symbian_ui_interface.h"
#include "ui_defs.h"
#include "ui_data_caging_literals.hrh"
#include "symbian_s60_notedlg.h"
#include "async_object.h"
#ifdef FEATURE_MULTIPLE_DRIVE
#include <pathinfo.h>
#include <driveinfo.h>
#endif
#include <StringLoader.h>
#include <flash2ui.rsg>
#include <flash2uiiad.rsg>
//#include <mmsconst.h> // for CSendAs MMS
#include <txtrich.h> // for CRichText
#include <AknQueryDialog.h>
#include <aknnotedialog.h> 
#include <bautils.h>
#include <eikprogi.h>

//#include <MGXFileManagerFactory.h> Media gallery not required
//#include <CMGXFileManager.h>

#include <featmgr.h>
#include <textresolver.h>
#include <SendUiConsts.h>
#include <aknmessagequerydialog.h>
#include <sysutil.h> 
#include <hal.h> 
#ifdef EKA2
#include <CMessageData.h>
#endif //EKA2

#ifdef _VIEWER_DEBUG__

#include <flogger.h>
const TUint KMaxLogLength   = 512;

void VwrDebug(TUint8 aLogLevel,TRefByValue<const TDesC> aText, ... )
    {
    if(aLogLevel < CUR_LOG_LEVEL)  
    	return;
    
    VA_LIST args;
    VA_START( args, aText );
    
    TBuf<KMaxLogLength> buf;
    buf.FormatList( aText, args );

#ifdef __FILE_MODE_LOGGING
    RFileLogger logger;
    
    TInt ret=logger.Connect();
    
    if (ret==KErrNone)
        {
        logger.SetDateAndTime( EFalse,EFalse );
        logger.CreateLog( _L("FlashLogs"), _L("ViewrLogFle"), EFileLoggingModeAppend );       
        logger.Write(buf);
        }

    logger.Close();
#else
    RDebug::Print(buf);

#endif
    VA_END( args );
    }

#else

void VwrDebug( TUint8 /*aLogLevel*/,TRefByValue<const TDesC> /*aText*/, ... )
    {  
    }

#endif //_VIEWER_DEBUG__


//const TUint KDriveLetterLength = 2;
#ifdef __FEATURE_STUB_ENHANCED_HEAP
	const TUint32 KStubMaxHeapSize = 48 << 10;  //48 MB max heap allowed this value taken if not able to read from cen rep
	const TUint32 KStubHeapSizeBits = 0x0000FFFF; //this is used to obtain the lower 2 bytes from the config parameter
#endif // __FEATURE_STUB_ENHANCED_HEAP
////////////////////////////////////////////////////////////////////////////////
// CSaFlashAppUi
////////////////////////////////////////////////////////////////////////////////
TBool CSaFlashAppUi::IsEmbeddedModeOn() const
{
	return iEikonEnv->StartedAsServerApp();
}

//SecuritySandbox
void CSaFlashAppUi::KickoffRestartTimerL()
    {
            //start the timer to restart the player 
            iRestartTimer = CRestartTimer::NewL(CActive::EPriorityHigh,this);
             iRestartTimer->Start();
    }
//SecuritySandbox
void CSaFlashAppUi::RestartPlayerL()
    {
         (((CSaFlashPlayerControl*)iPlayerView->GetContainer())->GetEngine())->DestroyPlayer();
         if(IsEmbeddedModeOn())
          ((CSaFlashPlayerControl*)iPlayerView->GetContainer())->OpenFileL(iFile);
          else
          ((CSaFlashPlayerControl*)iPlayerView->GetContainer())->OpenFileL(*iSavedFilename);
    }

void CSaFlashAppUi::SaveVolumeL(TInt aVolume)
{
	iIniData->iVolume = aVolume;
		TRAP_IGNORE( iIniData->SaveUserDataL());	// don't bail out if the ini file can;t be written to
}

void CSaFlashAppUi::EnableSkinL()
{
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::EnableSkinL() ++"));
	if (AknLayoutUtils::MSKEnabled())
	{
    	BaseConstructL(EAknEnableSkin | EAknEnableMSK | EAknSingleClickCompatible);
	}
	else
	{
		BaseConstructL(EAknEnableSkin | EAknSingleClickCompatible);
	}
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::EnableSkinL() --"));
}

void CSaFlashAppUi::LoadResourceL()
{
    VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::LoadResourceL() ++"));
    
    iResourceFileOffset = new (ELeave) RArray<TInt>;
    
    _LIT( KFlash2ui, "flash2ui.rsc" );
    TBuf16<128> flash2ui( KFlash2ui );
    _LIT( KFlash2uiIAD, "flash2uiiad.rsc" );
    TBuf16<128> flash2uiIAD( KFlash2uiIAD );
    _LIT( KDRMRightsManager, "drmrightsmanager.rsc" );
    TBuf16<128> DRMRightsManager( KDRMRightsManager );
    
    _LIT16( KResourcePath, "\\resource\\apps" );
    TBuf16<128> resourcePath( KResourcePath );

	TInt resOffset = 0;
    // flash2ui.rsc
    TBuf16<512> resFlash2ui;
    TInt err = FlhUiUtils::SearchResource( flash2ui, resourcePath, resFlash2ui );
    VwrDebug(LOG_DEBUG,_L("flash2ui : %S"), &resFlash2ui );
    if( err == KErrNone )
    {
        resOffset = CCoeEnv::Static()->AddResourceFileL(resFlash2ui);
        iResourceFileOffset->AppendL(resOffset);
    }
    else
    {
        VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::LoadResourceL():fileExists is false"));
        // If we didn't find the resource file, then we should leave    
        User::Leave(KErrNotFound);
    }

    // flash2uiIAD.rsc
    TBuf16<512> resFlash2uiIAD;
    err = FlhUiUtils::SearchResource( flash2uiIAD, resourcePath, resFlash2uiIAD );
    VwrDebug(LOG_DEBUG,_L("flash2ui : %S"), &resFlash2uiIAD );
    if( err == KErrNone )
    {
        resOffset = CCoeEnv::Static()->AddResourceFileL(resFlash2uiIAD);
        iResourceFileOffset->AppendL(resOffset);
    }
    else
    {
        VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::LoadResourceL():fileExists is false"));
        // If we didn't find the resource file, then we should leave    
        User::Leave(KErrNotFound);
    }

    // DRMRightsManager.rsc
    TBuf16<512> resDRMRightsManager;
    err = FlhUiUtils::SearchResource( DRMRightsManager, resourcePath, resDRMRightsManager );
    VwrDebug(LOG_DEBUG,_L("flash2ui : %S"), &resDRMRightsManager );
    if( err == KErrNone )
    {
        resOffset = CCoeEnv::Static()->AddResourceFileL(resDRMRightsManager);
        iResourceFileOffset->AppendL(resOffset);
    }
    else
    {
        VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::LoadResourceL():fileExists is false"));
        // If we didn't find the resource file, then we should leave    
        // User::Leave(KErrNotFound);
    }

    VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::LoadResourceL() --"));

}

TBool CSaFlashAppUi::IsFlashViewerFeatureSupportedL()
{
    
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::IsFlashViewerFeatureSupportedL() ++"));
#ifdef FEATURE_MANAGER_CHECK
    // Check if starting Flash Lite is allowed for this device
    FeatureManager::InitializeLibL();
    if ( !( FeatureManager::FeatureSupported( KFeatureIdFlashLiteViewer ) ) )
    {
    	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::IsFlashViewerFeatureSupportedL(): FLViewer Not Supported."));
        CAknGlobalNote* globalNote = CAknGlobalNote::NewLC();
        HBufC* errorMsg = StringLoader::LoadLC( R_STR_NOT_AUTHORIZED );
        globalNote->ShowNoteL( EAknGlobalErrorNote, errorMsg->Des() );
        CleanupStack::PopAndDestroy(2);  // globalNote, errorMsg

        if ( iDoorObserver )
        {
           	CloseApplication();
        }
        else
        {
            Exit();
        }
    }
    
    #ifdef __SERIES60_HELP 
	if (FeatureManager::FeatureSupported(KFeatureIdHelp))
    {
        iHelpAppExists = ETrue;        
    }
	#endif
    
    //else
    {
        FeatureManager::UnInitializeLib();
    }

#endif // FEATURE_MANAGER_CHECK
	
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::IsFlashViewerFeatureSupportedL() --"));
	return ETrue;
}

void CSaFlashAppUi::FillFlashConfigL(FlashEngineConfig& aConfig)
{
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::FillFlashConfigL() ++"));
	iIniData = new(ELeave) IniData(iEikonEnv->FsSession());
	TRAP_IGNORE(iIniData->LoadL()); // if the iniData fails to load then don't bail out...

	aConfig.iStaticHeapSize = iIniData->iStaticHeapSize;
	aConfig.iDynamicHeapMaxSize = iIniData->iDynamicHeapMaxSize;	
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::FillFlashConfigL(): iDynamicHeapMaxSize = %d"), iIniData->iDynamicHeapMaxSize);
	aConfig.iMode = iIniData->iEngineMode;
	aConfig.iSoftkeyCount = KSoftkeyCount;
	aConfig.iExtensions = iUIConfig.iExtensions;
	aConfig.iBackgroundAlpha = iIniData->iBackgroundAlpha;
	aConfig.iPlayerBlendsBackground = iIniData->iPlayerBlendsBackground;
	
#ifdef FEATURE_PERFORMANCE_AUTOMATION
//check whether debug mode bit for Performance Automation is On 
	if(iIniData->iDebugMode & KPerformanceTestMode)
	{
		//if enabled then enable the bit in engine config also.
		aConfig.iMode = (aConfig.iMode | KPerformanceTestMode);
	}
#endif 	

	aConfig.iSMCheckNeeded = ETrue;

	if ( iIniData->iEngineMode & KSupportMMI ) 
		{
		// MMI support hardcoded until the new cenrep
	  // check procedure is available.
		if (iUIConfig.iIsStubApp)
		    {
              iIsFlashStubApp = ETrue;
			  aConfig.iSMCheckNeeded = EFalse;
			  iMMIEnabled = ETrue;
		    }
		}


	if ( iUIConfig.iIsStubApp )
	{
        iIsFlashStubApp = ETrue;
	#ifdef __FEATURE_PRE_COMPILED_SWF
		aConfig.iActScriptExtensions = iUIConfig.iReserved1;
	#endif //__FEATURE_PRE_COMPILED_SWF
		
	#ifdef __FEATURE_STUB_ENHANCED_HEAP
		VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::FillFlashConfigL(): Default stub iDynamicHeapMaxSize,  = %d"), iIniData->iDynamicHeapMaxSize);
		//get custom sise from iReserved2, only first 16 bits are used for this
		TUint16 customSize = iUIConfig.iReserved2 & KStubHeapSizeBits;
		
		//Max size is defined in cen rep, it can change device RAM support
		customSize = (customSize>KStubMaxHeapSize)?KStubMaxHeapSize:customSize;
		VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::FillFlashConfigL(): customSize = %d"), customSize);
		
		if (customSize)
		{
			aConfig.iDynamicHeapMaxSize = customSize << 10;	// allocate customSize *1 Kb memory.
		}
		VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::FillFlashConfigL(): Stub iDynamicHeapMaxSize,  = %d"), iIniData->iDynamicHeapMaxSize);
	#endif //__FEATURE_STUB_ENHANCED_HEAP
		//Bring flash stub apps in trusted sandbox
		aConfig.iMode |= KSupportTrustedSandBox;

	}
#ifdef __FEATURE_PRE_COMPILED_SWF
	else // Not a stub app
	{
		aConfig.iActScriptExtensions = 0;
	}		
#endif //__FEATURE_PRE_COMPILED_SWF

#ifndef FEATURE_HIDE_SAFLASH	
	if(IsEmbeddedModeOn())
	{
		//aConfig.iMode &= ~KSupportSharedObject; // disable support for shared objects when embedded.
		aConfig.iMode &= ~KSupportLocalFile; // disable support for accessing local files when embedded.
	}
#endif //FEATURE_HIDE_SAFLASH
#ifdef SELF_IAD_UPDATE
	aConfig.iCheckIADUpdate = ETrue;   // viewer initiates updates
#endif
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::FillFlashConfigL() --"));
}

TBool CSaFlashAppUi::IsStubApp()
    {
    return iIsFlashStubApp;
    }
	
	
void CSaFlashAppUi::CreateFlashViewsL(FlashEngineConfig& aConfig)
{

	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::CreateFlashViewsL() ++"));
	//create Flash player view
	iPlayerView = new(ELeave) CSaFlashPlayerView(iIniData->iDebugMode, iUIConfig.iIsStubApp);
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::CreateFlashViewsL():iPlayerView = new(ELeave) CSaFlashPlayerView(iIniData->iDebugMode)"));
	iPlayerView->ConstructL(aConfig);
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::CreateFlashViewsL():iPlayerView->ConstructL(this, aConfig)"));
	AddViewL(iPlayerView);
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::CreateFlashViewsL():AddViewL(iPlayerView)"));
	iEngine = iPlayerView->GetEngine();
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::CreateFlashViewsL():iEngine = iPlayerView->GetEngine()"));
	iTextInput = new(ELeave) TextInput(iEngine);
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::CreateFlashViewsL():iTextInput = new(ELeave) TextInput(iEngine)"));
	iEngine->SetVolume(iIniData->iVolume);
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::CreateFlashViewsL():iEngine->SetVolume(iIniData->iVolume)"));
	
	//create file view
	if ((!iUIConfig.iIsStubApp) && (!IsEmbeddedModeOn()))
	{ 	//stub app will not have file view
		//Review : need to check why file view required in embedded mode?
		
		iFileView = new(ELeave) CSaFlashFileView(iEngine, iPlayerView->GetLowMemDialog());
		iFileView->ConstructL(iIniData->iEngineMode & KSupportSVG ? ETrue : EFalse);
		AddViewL(iFileView);
		VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::CreateFlashViewsL():FileView Created !!"));
	}
	
	//create volume view
	iVolumeView = new (ELeave) CSaFlashVolumeView(iEngine);
	iVolumeView->ConstructL();
	AddViewL(iVolumeView);
	
	//Review : this if condition has to be refactored again	
	if (!IsEmbeddedModeOn())
	{
		if (iUIConfig.iIsStubApp)
		{
			//SetDefaultViewL(*iPlayerView);			
			iPlayerView->SetLaunchType(CSaFlashPlayerView::EStartedFromApp);
		}
		else
		{
		    //SetDefaultViewL(*iFileView);			
			iPlayerView->SetLaunchType(CSaFlashPlayerView::EStartedFromMenu);
		}
	}
	else
	{	    
		//SetDefaultViewL(*iPlayerView);		
		iPlayerView->SetLaunchType(CSaFlashPlayerView::EEmbedded);
	}
	/* EPLU-84GAA5: Moved additon of view to AppUi stack from 
     * CSaFlashPlayerView::DoActivateL function. iFileView is not
     * supported any more, hence iPlayerView is set as default.  
     */
	SetDefaultViewL(*iPlayerView);
    AddToStackL(*iPlayerView, iPlayerView->GetContainer());
	
	/* Change done for TSWID: EHVI-7RAEYG:
	 * SendUI will not be used in case of stubapp.
	 * This will ensure that Minimum capability required for stub app is None. Otherwise it is ReaduserData
	 * iSendAppUi is checked in other places. 
	 */
	if (!iUIConfig.iIsStubApp)
    {
        iSendAppUi = CSendUi::NewL();
    }
	
	iPlayerView->SetStubApp(iUIConfig.iIsStubApp);
	if(iUIConfig.iIsStubApp)
	{
		iPlayerView->SetFullScreenL();
	}
	
	
	//iEngine->AddMp3StateObserver(this);
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::CreateFlashViewsL() --"));
}

void CSaFlashAppUi::OpenFlashContentL()
{
	if (iUIConfig.iContentFileName.Length() > 0)
	{
		TParse p;//localvar guideleins dont allow
		p.SetNoWild(iUIConfig.iContentFileName, NULL, NULL);
		if (p.PathPresent())
		{
		    TFileName fileName;
		    if(!p.DrivePresent())
		        {	        
		        TParse p1;
		        p1.SetNoWild(iEikonEnv->EikAppUi()->Application()->AppFullName(), NULL,NULL); //to get drive letter if not present		        
		        fileName.Append(p1.Drive());
		        }
		    
		    fileName.Append(p.FullName());
			OpenFileL(fileName);
		}
		else
		{
		    TParsePtrC p(iUIConfig.iContentFileName);
	    	if (p.PathPresent())
    		{
    			OpenFileL(iUIConfig.iContentFileName);
    		}
    		else
    		{
    			TFileName fn;//local var guidelines dont allow
    			FlhUiUtils::GetFileInAppDir(*iEikonEnv,fn,iUIConfig.iContentFileName,KNullDesC);
    			OpenFileL(fn);
    		}
		}
	}
}

TBool CSaFlashAppUi::IsMemorySufficient()
	{
	TInt freeRAM;
	//TBD: Remove hardcoded 4MB. CenRep code to be added by Rajeshwari. Key KFlashRAMResvUnUtilisedSpace will be used.
	const TUint KFlashRAMResvUnUtilisedSpace = 4*1024*1024;
	TInt r = HAL::Get(HAL::EMemoryRAMFree, freeRAM);
	if ((r != KErrNone)  || freeRAM < KFlashRAMResvUnUtilisedSpace)
		{
		return EFalse;
		}
	return ETrue;
	}
void CSaFlashAppUi::ConstructL( )
{

	VwrDebug(LOG_DEBUG,_L("==========================================================="));
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::ConstructL() ++"));
   
	EnableSkinL();
	
	if (!IsMemorySufficient())
		{
		User::Leave(KErrNoMemory);
		}

	LoadResourceL();
	
	if(IsFlashViewerFeatureSupportedL())	
	{
		
		FlashEngineConfig config;
	
		FillFlashConfigL(config);
	
	// iDoorObserver is non-NULL when the app is running embedded within another app (i.e. invoked from
	// the web browser or messaging app.
		
		CreateFlashViewsL(config);
		
		OpenFlashContentL();
		
		iIdle=CIdle::NewL(CActive::EPriorityIdle-10);	
	}
	
	// Start listening qwerty mode status.
    User::LeaveIfError(iQwertyModeStatusProperty.Attach(KCRUidAvkon,KAknQwertyInputModeActive));
    iQwertyModeStatusSubscriber = new (ELeave) CKeypadHandler(*this, iQwertyModeStatusProperty);
    iQwertyMode = iQwertyModeStatusSubscriber->GetQwertMode();
    iQwertyModeStatusSubscriber->SubscribeL();
#ifdef FEATURE_EXTENSION     
	if(ScreenState())
        {
        iFirstOrientation = CAknAppUi::EAppUiOrientationPortrait;
        }
    else 
        {
        iFirstOrientation = CAknAppUi::EAppUiOrientationLandscape;
        }
#endif	
    
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::ConstructL() --"));
}
//SecuritySandbox
 //TBool CSaFlashAppUi::CanContentBeTrustedL(const TPtrC pAccessedDomain)
 TBool CSaFlashAppUi::CanContentBeTrustedL(const TDesC& pAccessedDomain)
     {
     //Ask user if the content accesing the domain pAccessedDomain can be trusted
     HBufC * errorMsg = StringLoader::LoadLC( R_STR_SECURITY_TRUST,
             pAccessedDomain, iEikonEnv );

     TBool res = iPlayerView->QueryAllowDenyL(*errorMsg);
     if(res == 1)
         trustedQueryAsked=ETrue;
     CleanupStack::PopAndDestroy(errorMsg);
     return res;
         
     }
void CSaFlashAppUi::DoSendMenuL(CEikMenuPane* aMenuPane, TInt /*aAttachSize*/, TInt aItemBefore)
{
	TInt pos;
	if (iSendAppUi)
	{
	// get position of passed in item, so we can insert the send menu at the requested location.
	aMenuPane->ItemAndPos(aItemBefore, pos);
	    iSendAppUi->AddSendMenuItemL(*aMenuPane, pos, EEikFlashCmdSend,
			TSendingCapabilities ( 0, 100000, 0 ) );
	}
}

void CSaFlashAppUi::DoSendSubMenuL(CEikMenuPane* /*aMenuPane*/)
{
    if (iSendAppUi)
    {
    CArrayFix<TUid> * array = new(ELeave) CArrayFixFlat<TUid>(1);
	CleanupStack::PushL(array);
//#ifdef __FLASHLITE_BETA3	
    array->AppendL(KSenduiMtmSmsUid);
    array->AppendL(KSenduiMtmPostcardUid);
//#endif //__FLASHLITE_BETA3    
		iSendAppUi->ShowSendQueryL(NULL, KCapabilitiesForAllServices, array);
	CleanupStack::Pop(array);
    }
}

void CSaFlashAppUi::DynInitMenuPaneL(TInt aResourceId, CEikMenuPane* aMenuPane )
{
	if (iView)
	{
		iView->DynInitMenuPaneL(aResourceId, aMenuPane);
	}
}

void CSaFlashAppUi::HandleSendCommandL(TInt /*aCommand*/, const TDesC& aFilename)
{
	TInt attach_size = 0;
	RFile file;
	const TDesC* filename;
	TInt err = KErrNone;
	CSaFlashPlayerView::TLaunchType launchType = iPlayerView->GetLaunchType();
	
	if(iSendAppUi == NULL)
    {
        return;
    }
	if (aFilename.Length() > 0)
	{
		filename = &aFilename;
	}
	else if (iSavedFilename)
	{
		filename = iSavedFilename;
	}
	else
	{
		return;
	}
	
	if ( launchType == CSaFlashPlayerView::EEmbedded)
	{
		file = iFile;
	}
	else
	{
		err = file.Open(iEikonEnv->FsSession(), *filename, EFileRead|EFileShareReadersOnly);
	}
	if (err == KErrNone && launchType != CSaFlashPlayerView::EEmbedded)
	{
		file.Size(attach_size);
		file.Close();
	}
	else if(err != KErrNone)
	{
		return;
	}


    CArrayFix<TUid> * filter = new(ELeave) CArrayFixFlat<TUid>(1);
//#ifdef __FLASHLITE_BETA3
	CleanupStack::PushL(filter);
    filter->AppendL(KSenduiMtmSmsUid);
    filter->AppendL(KSenduiMtmPostcardUid);
//#endif //__FLASHLITE_BETA3

    CMessageData* messageBody = CMessageData::NewL();        
    if (messageBody != NULL)
    	{
		CleanupStack::PushL(messageBody);           
		if ( launchType == CSaFlashPlayerView::EEmbedded )   
			{
			messageBody->AppendAttachmentHandleL(file);
			}
		else
			{
			messageBody->AppendAttachmentL(*filename );            	
			}
			
            iSendAppUi->ShowQueryAndSendL(messageBody, KCapabilitiesForAllServices, filter);
            CleanupStack::PopAndDestroy(messageBody);
    	}
	CleanupStack::PopAndDestroy(filter);

}

/*void CSaFlashAppUi::SendMessageL(const TUid aMtmUid, 
								 const TDesC& aAddress, 
                                 const TDesC& aCc,
                                 const TDesC& aBcc,
								 const TDesC& aSubject, 
								 const TDesC& aBodyText, */
								 //TBool /*aLaunchEmbedded*/)
void CSaFlashAppUi::SendMessageL(const TUid aMtmUid, const TMailInfo& aMailInfo,TBool/* aLaunchEmbedded*/)
{
    if(iSendAppUi == NULL)
    {
        return;
    }
	
	CMessageData* messageBody;
	
	messageBody = CMessageData::NewLC();
    		
    CParaFormatLayer* paraFormatLayer = CParaFormatLayer::NewL();
    CleanupStack::PushL(paraFormatLayer);

    CCharFormatLayer* charFormatLayer = CCharFormatLayer::NewL();
	CleanupStack::PushL(charFormatLayer);

    CRichText* messageBodyContent = CRichText::NewL(paraFormatLayer, charFormatLayer); 
    CleanupStack::PushL(messageBodyContent);
  
	if (aMailInfo.iAddress.Length() != 0)
	{
		messageBody->AppendToAddressL(aMailInfo.iAddress);			
	}
    if (aMailInfo.iSubject.Length() != 0)
    {
        messageBody->SetSubjectL(&(aMailInfo.iSubject));
    }
  	
    if (aMailInfo.iCc.Length() != 0)
    {
        messageBody->AppendCcAddressL(aMailInfo.iCc);
    }
    if (aMailInfo.iBcc.Length() != 0)
    {
        messageBody->AppendBccAddressL(aMailInfo.iBcc);
    }

  	            
    if (aMailInfo.iBodyText.Length() != 0)
    {
    	messageBodyContent->InsertL(0, aMailInfo.iBodyText);
    	messageBody->SetBodyTextL(messageBodyContent);
    }

	iSendAppUi->CreateAndSendMessageL(aMtmUid, messageBody);

	CleanupStack::PopAndDestroy(4,messageBody); // messageBody, messageBodyContent, charFormatLayer, paraFormatLayer

}

void CSaFlashAppUi::HandleCommandL( TInt aCommand )
{
	// In full screen, treat EAknSoftkeyExit as EAknSoftkeyBack
	if ( IsFullScreenMode() && EAknSoftkeyExit == aCommand )
	{
		aCommand = EAknSoftkeyBack;
	}

	switch ( aCommand )
	{
		// Handles exit command
	case EAknSoftkeyExit:
	case EEikCmdExit:
	    //closes the volume popup window  when end key is pressed
	    iPlayerView->CloseVolPopUp();
		CloseApplication();
		break;
	default:
		if (iView)
		{
			iView->HandleCommandL(aCommand);
		}
		break;
	}
}


CSaFlashAppUi::CSaFlashAppUi(const FlashUIConfig& aConfig):
	 iRestartTimer(NULL)
	,iResourceFileOffset(0)
	,iResourceLoader(*iCoeEnv)
{
	iUIConfig = aConfig;
	iSavedFilename = NULL;
	iSendAppUi = NULL;
	iHelpAppExists = EFalse;
	iMMIEnabled = EFalse;
	isUrlDesc = EFalse;
	iurlDesc =NULL;
	iMoveAllowed= EFalse;
	iHwKeyObserverRemoved = EFalse;
	iIsFlashStubApp = EFalse;

	trustedQueryAsked =EFalse;
	iForeground = ETrue;
	iMP3StreamState=EFlashNoMP3Stream;
    iVideoContentCount = 0;
    iResourceFileOffset = NULL;
	iTextInput = NULL;
	iSavedFilename = NULL;
	iIniData = NULL;
	iProgressDialog	= NULL;
	iIdle = NULL;
	iQwertyModeStatusSubscriber= NULL;
	iAsyncDeleter = NULL;
}

void CSaFlashAppUi::CloseApplication()
{
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::CloseApplication() ++"));
	if(iEngine)
    {
        iEngine->DestroyPlayer();
    }

	PrepareToExit();
	//fix for MLIO-79ZE9U open to viewer crash
	iExitHandler.DoExit(this);

/*	if (iDoorObserver)
	{
		// rborcic 2005/09/20 -- moved the call to NotifyExit to the destructor because it
		// seems to make it less likely to crash on exit if called from the destructor.
		// iDoorObserver->NotifyExit(MApaEmbeddedDocObserver::ENoChanges);

		iExitHandler.DoExit(this);
	}
	else
	{
		TInt error=KErrNone;
		TInt toGo=iEngine->StartShutdownSequenceReturnStepsToGo();
		if(toGo)
		{
			TRAP(error,StartShutdownProgressDialogL(toGo));
		}
		if(0==toGo || error)
		{
			iExitHandler.DoExit(this);
		}
	}*/	
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::CloseApplication() --"));
}

/**
 Quit is called by the FlashEngine module when SI_QuitFlashLite is called. This will go back to the fileview
 if Flash is running in standalone mode, or exit if launched embedded or launched from an app.
*/
void CSaFlashAppUi::Quit()
{
	iQuitApp = ETrue;
	TRAP_IGNORE(PrepareToExitL());
}

CSaFlashAppUi::~CSaFlashAppUi( )
{
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::~CSaFlashAppUi() ++"));
	if (iDoorObserver)
	{
		iDoorObserver->NotifyExit(MApaEmbeddedDocObserver::ENoChanges);
	}

	if ( iPlayerView )
	{
		iPlayerView->fBeingDestroyed = ETrue;
	}
	
	if (iFileView)
	{
		iFileView->fBeingDestroyed = ETrue;
	}
	
	if(iTextInput)
	{
		delete iTextInput;
		iTextInput = NULL;
	}
	
	if(iSavedFilename)
	{
		delete iSavedFilename;
		iSavedFilename = NULL;
	}
/*
	if(iResourceFileOffset)
	{
		iCoeEnv->DeleteResourceFile(iResourceFileOffset);
	}
*/	
	if(iResourceFileOffset)
	{
		TInt offsetFileIndex = 0;
		TInt offsetFileCount = iResourceFileOffset->Count();
		while( offsetFileIndex < offsetFileCount )
		{
			TInt resOffset = (*iResourceFileOffset)[offsetFileIndex];
			if( resOffset )
				iCoeEnv->DeleteResourceFile(resOffset);
				
			offsetFileIndex++;
		}
		
		iResourceFileOffset->Close();
		delete iResourceFileOffset;
		iResourceFileOffset = NULL;		
	}

	if(iRestartTimer)
	{
		delete iRestartTimer;
		iRestartTimer=NULL;
	        
	}
	if(iSendAppUi)
	{
		delete iSendAppUi;
		iSendAppUi = NULL;
	}
	
	if(iIniData)
	{
		delete iIniData;
		iIniData = NULL;
	}
	
	if(iProgressDialog)
	{
		delete iProgressDialog;
		iProgressDialog = NULL;
	}
	
	if(iIdle)
	{
		delete iIdle;
		iIdle = NULL;
	}
	
	iFile.Close();
	
	if(iurlDesc)
	{
		delete iurlDesc;
		iurlDesc = NULL;
	}

	iResourceLoader.Close();
	
	// stop listening qwerty mode status
	if (iQwertyModeStatusSubscriber)
    {
        iQwertyModeStatusSubscriber->UnSubscribe();
        delete iQwertyModeStatusSubscriber;
        iQwertyModeStatusSubscriber = NULL;
    }
    
	iQwertyModeStatusProperty.Close();
	
	if(iAsyncDeleter)
	{
	    delete iAsyncDeleter;
	    iAsyncDeleter = NULL;
	}

	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::~CSaFlashAppUi() --"));
	
}

void CSaFlashAppUi::MP3PlayStateChangedL(EMP3StreamState aMP3StreamState)
{
    //if we are already in nomp3state dont do anything
    if(aMP3StreamState == EFlashNoMP3Stream && iMP3StreamState != EFlashNoMP3Stream)
    {
          if(iForeground == EFalse) 
          {

              iPlayerView->CancelFocusHandler();
              iEngine->SetForeground(EFalse);
              iEngine->Pause(EReasonBackground);
               // If in File View, no need to draw pause icon.
              if (iPlayerView->IsForeground() || iVolumeView->IsForeground())
                  {
                  //If Engine is Stop, then no need to draw pause icon.
                  if(iEngine->IsRunning())
                      {
                      ((CSaFlashPlayerControl*)iPlayerView->GetContainer())->ShowPauseIconL(ETrue);
                      }
                  }
                  
              if (IsPlayerViewActive())
              {
                  iPlayerView->HandleBackgroundEvent();
                  
              }
              if(!iHwKeyObserverRemoved)
              {
                  iPlayerView->CancelHwKeyObserver();
                  iHwKeyObserverRemoved = ETrue;              
              }
          }
    }
    iMP3StreamState = aMP3StreamState;
}
void CSaFlashAppUi::HandleMp3PlayingForegroundEventL(TBool aForeground)
{
    if (iEngine)
    {
        if (aForeground)
        {
            //if we come to foreground we change the state to no mp3stream
            if(iMP3StreamState == EFlashMP3StreamClosed)
            {
                iMP3StreamState = EFlashNoMP3Stream;
            }
        }
        else
        {
            //Close the volume popup when going to background
            iPlayerView->CloseVolPopUp();
        }
    } 
}
void CSaFlashAppUi::HandleForegroundEventL(TBool aForeground)
{
    iForeground = aForeground;
    if (iEngine)
	{
         //if mp3 stream is in progress we need not pause
        if(iMP3StreamState != EFlashNoMP3Stream)
	    {
	        HandleMp3PlayingForegroundEventL(aForeground);
	    }
        else
        {
            if (aForeground)
            {
                VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::HandleForegroundEventL(): foreground "));
                if (IsPlayerViewActive())
				{
                    iEngine->SetForeground(ETrue);
                    iEngine->ResumeL(EReasonBackground, EFalse);
                    if (iEngine->IsPaused())
                    	{
                        ((CSaFlashPlayerControl*)iPlayerView->GetContainer())->ShowPauseIconL(ETrue);
                    	}
                    else//Engine has resumed so cancel Pause Blinker
                    	{
                        ((CSaFlashPlayerControl*)iPlayerView->GetContainer())->ShowPauseIconL(EFalse);
                    	}

                    iEngine->RefreshScreen();
                }
                else if (IsVolumeViewActive())
				{
                    ActivateLocalViewL(KSAFlashVolumeViewId);
                    iEngine->SetForeground(ETrue);
                    iEngine->ResumeL(EReasonBackground, ETrue);
                    
				}
                if(iHwKeyObserverRemoved)
                {
                    iPlayerView->AddHwKeyObserverL();
                    iHwKeyObserverRemoved = EFalse;
                }
            }
            else
            {
                VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::HandleForegroundEventL(): background %d"),aForeground);
                //Close the volume popup when going to background
                iPlayerView->CloseVolPopUp();
                            
                iPlayerView->CancelFocusHandler();
                iEngine->SetForeground(EFalse);
                iEngine->Pause(EReasonBackground);
            
                    
                if (IsPlayerViewActive())
                {
                    ((CSaFlashPlayerControl*)iPlayerView->GetContainer())->CancelPauseBlinker();                                  
                }
                if(!iHwKeyObserverRemoved)
                {
                    iPlayerView->CancelHwKeyObserver();
                    iHwKeyObserverRemoved = ETrue;				
                }
            }
        }   
    }
    CAknAppUi::HandleForegroundEventL(aForeground);
}
#ifdef FEATURE_EXTENSION
//To Resume & Pause when recieves SAPI events.
void CSaFlashAppUi::ResumeOnEventL(TBool aForeGround) 
    {
    
    if(aForeGround)// If at the time of registering for the event, bringToForeground option was enabled, then bring to foreground
        {        
        TApaTask task(iEikonEnv->WsSession( ));
        task.SetWgId(CEikonEnv::Static()->RootWin().Identifier());
        task.BringToForeground();// BringToForeground takes care of resuming viewer by notifying to HandleForegroundEventL(aForeground)   
        }
    else
        {
        iEngine->ResumeL(EReasonBackground, EFalse); //If bringToForeground oprions was disabled then resume player only no change needed in UI.
        }
    }

//This function is a stub for Actionscript API which will enable Flash developer to Pause the flash player.
//function takes TBool as input i.e. information about Application status (foreground/background).
void CSaFlashAppUi::PauseOnRequest(TBool aForeGround)
    {
    if(aForeGround) 
        {
        TApaTask task(iEikonEnv->WsSession( ));        
        task.SetWgId(CEikonEnv::Static()->RootWin().Identifier());        
        task.SendToBackground();
        }
    else
        {
        iEngine->Pause(EReasonBackground);
        }
    }

#endif

void CSaFlashAppUi::HandleWsEventL(const TWsEvent& aEvent,CCoeControl* aDestination)
{
	iExitHandler.StartEventHandler();
	switch (aEvent.Type())
	{	// if we have been sent a message, and we are embedded, we should exit,
		// since otherwise, the browser will not deactivate our view and it
		// will end up in a strange state.
 #ifdef MMI_FL2_0 
		case KAknUidValueEndKeyCloseEvent:
		{
			//	TVwsViewId phoneViewId;
    		//	User::LeaveIfError(AknDef::GetPhoneIdleViewId(phoneViewId));
		    //	ActivateViewL(phoneViewId);

		    // Move Flash Player 
		    // magic: 0 = force wg to foreground
		    //do not allow it to go into the default case, else it will kill the application
			if ( iMMIEnabled ) 
			{
	    		CEikonEnv::Static()->RootWin().SetOrdinalPosition( 0 );		       
			}
			else
			{
				CAknViewAppUi::HandleWsEventL(aEvent, aDestination);
			}
		
			iExitHandler.EndEventHandler(this);
			return;
		}
		//break;

  		#endif// MMI_FL2.0
  		
		case EEventMessageReady:
		{
			CAknViewAppUi::HandleWsEventL(aEvent, aDestination);
			if (iDoorObserver) // test for embedded
			{
				CloseApplication();
			}
			
		}
		break;


			
		default:
		{
			CAknViewAppUi::HandleWsEventL(aEvent, aDestination);
			iWsEvent = aEvent;
		}
	}
	
	iExitHandler.EndEventHandler(this);
}


TBool CSaFlashAppUi::ProcessCommandParametersL(TApaCommand aCommand,
											   TFileName& aDocumentName,
											   const TDesC8& /*aTail*/)
{
	TBool retVal = ETrue;
	switch(aCommand)
	{
	case EApaCommandOpen:
		if (aDocumentName.Length() > 0)
		{
			iPlayerView->SetLaunchType(CSaFlashPlayerView::EStartedFromApp);
		}
		retVal =  ETrue;
		break;
		
	case EApaCommandCreate:
		retVal =  EFalse;
		break;
		
	case EApaCommandRun:
		retVal =  ETrue;
		break;
		
	case EApaCommandBackground:
		retVal =  EFalse;
		break;
		
	case EApaCommandViewActivate:
		retVal = EFalse;
		break;
		
	case EApaCommandRunWithoutViews:
		retVal = EFalse;
		break;
	default:
		retVal = ETrue;
	}
	
	return retVal;
}


void CSaFlashAppUi::OpenFileL(const TDesC& aFilename)
{
	if (iSavedFilename)
	{
		delete iSavedFilename;
		iSavedFilename = NULL;
	}
	iSavedFilename = aFilename.AllocL();
	
	//iQuality = EEikFlashCmdQualityMed;
	TBool openFile = CheckAndProcessDRMFileL( );
	
	if (openFile)
	{
		SetDefaultViewL(*iPlayerView);
		ActivateLocalViewL(KSAFlashPlayerViewId);
		TRAPD(err, iPlayerView->OpenFileL(aFilename));
		if (err == KErrNoMemory)
			{
			iPlayerView->NotifyClientError(EInsufficientMemoryError, ETrue);
			iEngine->ClearBitmap();
			}
	}
//#endif //__FLASHLITE2.0_REL3.1	
}

void CSaFlashAppUi::OpenFileL(RFile& aFile)
{
	iFile = aFile;
	if (iSavedFilename)
	{
		delete iSavedFilename;
		iSavedFilename = NULL;
	}

    // Get file path from RFile
    TPath* filepath = new(ELeave) TPath;
    aFile.FullName(*filepath);
    iSavedFilename = filepath->Alloc();
    delete filepath;
    TBool openFile = CheckAndProcessDRMFileL( );
    if(openFile)
    {
    	SetDefaultViewL(*iPlayerView);
    	ActivateLocalViewL(KSAFlashPlayerViewId);

    	TRAPD(err, iPlayerView->OpenFileL(aFile));
    	if (err == KErrNoMemory)
    	{
			iPlayerView->NotifyClientError(EInsufficientMemoryError, ETrue);
			iEngine->ClearBitmap();
    	}
    }
    else
    {
    	CloseApplication();
    }
}


void CSaFlashAppUi::SaveFileL(const TDesC& aFileName)
	{
	TInt err = KErrNotFound;
	if (!EnoughSpaceL(aFileName))
		{
		//Not enough space, Disk is full.
		err = KErrDiskFull; 	
		}
	else	
		{		
		if (iSavedFilename)
			{
			HBufC* pBufC = HBufC::NewL(KCommonBufSize);
			CleanupStack::PushL(pBufC);
			TPtr filename(pBufC->Des());
			filename.Copy(aFileName);

			RFs& fs = iEikonEnv->FsSession();
			CFileMan* fileMan = CFileMan::NewL(fs);
			CSaFlashPlayerView::TLaunchType launchType = iPlayerView->GetLaunchType();
			if ( launchType == CSaFlashPlayerView::EEmbedded)
				{
				err = fileMan->Copy(iFile, filename, CFileMan::EOverWrite);    
				}
			else
				{
				err = fileMan->Copy(*iSavedFilename, filename, CFileMan::EOverWrite);
				}
			
			/* // Media Gallery not required
			CMGXFileManager* iMgxFileMgr = MGXFileManagerFactory::NewFileManagerL(CEikonEnv::Static()->FsSession());
			iMgxFileMgr->UpdateL(filename);
			delete iMgxFileMgr;
			*/

			delete fileMan;
			CleanupStack::PopAndDestroy(pBufC);
			}
		}
	
	HBufC* message = NULL;
	TAknGlobalNoteType noteType = EAknGlobalInformationNote;
	
	if (err == KErrNone)
		{		
		noteType = EAknGlobalConfirmationNote;
		HBufC* appName = StringLoader::LoadLC(R_STR_FLITE_APPLICATION_NAME, iEikonEnv);
		message = StringLoader::LoadLC( R_STR_MOVIE_SAVED,*appName, iEikonEnv );        
		FlashUiUtil::ShowNoteL( noteType, *message );
		CleanupStack::PopAndDestroy();    // message 
		CleanupStack::PopAndDestroy();    // appName
		}
	else
		{
		noteType = EAknGlobalErrorNote;
		CTextResolver* textResolver = CTextResolver::NewLC(*CEikonEnv::Static()); 
		message = ( textResolver->ResolveErrorString( err ) ).AllocLC();
		FlashUiUtil::ShowNoteL( noteType, *message );
		CleanupStack::PopAndDestroy();    //  message
		CleanupStack::PopAndDestroy();    //  textResolver
		}
	}

TBool CSaFlashAppUi::EnoughSpaceL(const TDesC &aTarget)
	{
	TBool result( ETrue );

	RFs& fs = iEikonEnv->FsSession();;
		
	TEntry entry;

	TInt fileSize = 0;
	CSaFlashPlayerView::TLaunchType launchType = iPlayerView->GetLaunchType();
	
	if ( launchType == CSaFlashPlayerView::EEmbedded)
		{
		iFile.Size(fileSize);
		}
	else
		{
		fs.Entry( *iSavedFilename, entry );
		fileSize = entry.iSize;
		}
		
	#ifdef FEATURE_MULTIPLE_DRIVE

	TInt drv = TDriveUnit( aTarget );//to get the drive number
	   
    if ( SysUtil::DiskSpaceBelowCriticalLevelL( NULL, fileSize , drv ) )
		{
		result = EFalse;
		}
	else
		{
		result = ETrue;
		}

	#else ///FEATURE_MULTIPLE_DRIVE
	
	TParse parse;
	parse.Set( aTarget, NULL, NULL );
	TInt drive;
	fs.CharToDrive( parse.Drive()[0], drive);
	
	if (drive == EDriveC )	
		{
		if ( SysUtil::DiskSpaceBelowCriticalLevelL( NULL, fileSize , drive ) )
			{
			result = EFalse;
			}
		else
			{
			result = ETrue;
			}
		} 
	else if (drive == EDriveE )	
		{
		if ( SysUtil::MMCSpaceBelowCriticalLevelL( NULL, fileSize) )
			{
			result = EFalse;
			}
		else
			{
			result = ETrue;
			}
		}
#endif  ////FEATURE_MULTIPLE_DRIVE

	return result;
	}
void CSaFlashAppUi::AboutL()
{	
	// show about dialog
    HBufC* aboutText = StringLoader::LoadLC( R_SAFLASH_ABOUT_CONTEXT, iEikonEnv ); 
    CAknMessageQueryDialog* dlg = CAknMessageQueryDialog::NewL( *aboutText );
    dlg->ExecuteLD( R_SAFLASH_ABOUT_DIALOG );
    CleanupStack::PopAndDestroy(aboutText);
}

void CSaFlashAppUi::LaunchHelpL(TSAFlashHelpID aHelpID )
{
    CArrayFix<TCoeHelpContext>* myContextList;
    myContextList = new (ELeave) CArrayFixFlat<TCoeHelpContext>(1);
    TCoeHelpContext myContext;
    myContext.iMajor = KUidSAFlash;

    switch ( aHelpID )
    {
    /* This has been removed as we dont have the file view in Flashlite
        case EEikFlashHelpPhoneMemory:
            {
            	myContext.iContext = KFLITE_HLP_PHONE_MEMORY;
            	break;
            }
	    case EEikFlashHelpMMC:
	        {
	        	myContext.iContext = KFLITE_HLP_MEMORY_CARD;
            	break;
            }
	    case EEikFlashHelpFolderView:
	        {
	        	myContext.iContext = KFLITE_HLP_FOLDER_VIEW;
            	break;
            }
     */
	    case EEikFlashHelpPlay:
            {
            	myContext.iContext = KFLITE_HLP_VIEWER_PLAY;
            	break;
            }
        case EEikFlashHelpPause:
            {
            	myContext.iContext = KFLITE_HLP_VIEWER_PAUSED;
            	break;
            }
        case EEikFlashHelpStop:
            {
            	myContext.iContext = KFLITE_HLP_STOPPED;
            	break;
            }
        default:
            {
            	break;
            }
     }

    myContextList->AppendL(myContext);
    // launches Help at specific location
    RWsSession& wsSession=iCoeEnv->WsSession();
    HlpLauncher::LaunchHelpApplicationL(wsSession, myContextList);
}

TBool CSaFlashAppUi::IsPlayerViewActive()
{
	return (iView == iPlayerView);
}

TBool CSaFlashAppUi::IsVolumeViewActive()
{
	return (iView == iVolumeView);
}

// Not Used
#if 0
TBool CSaFlashAppUi::QuerySharedObject()
{
    /*TInt retCode = EFalse;
	HBufC* prompt = iEikonEnv->AllocReadResourceLC(R_STR_SHARED_OBJ_CONFIRMATION);
    retCode = QueryAllowDenyL(*prompt);
    CleanupStack::PopAndDestroy(prompt);
    return retCode;*/
    return ETrue;
}
#endif

 /*
TBool CSaFlashAppUi::QueryAllowDeviceIDL()
{
	HBufC* pMessage = iEikonEnv->AllocReadResourceLC(R_STR_ALLOW_DEVICE_ID);
	TBool rval=QueryAllowDenyL(*pMessage);
	CleanupStack::PopAndDestroy(pMessage);

	return rval;
}*/

/*TBool CSaFlashAppUi::QueryAllowDenyL(const TDesC& aQueryText)
{
	TBool rval = EFalse;

    CAknQueryDialog* dlg = CAknQueryDialog::NewL();
    HBufC *queryText = aQueryText.AllocLC();
    TPtr queryPtr = queryText->Des();
    AknTextUtils::LanguageSpecificNumberConversion(queryPtr);
    TInt result(dlg->ExecuteLD(R_SAFLASH_DIALOG_ALLOW_DENY, queryPtr));

    if (result)
        {
	    rval = ETrue;
        }
    CleanupStack::PopAndDestroy(queryText);
	return rval;
}*/

TBool CSaFlashAppUi::QueryAllowDenyWithFormatResourceL(const TDesC& aQueryText, TInt aResId)
{
	HBufC* pMsg = FlashUiUtil::Format1L(*iEikonEnv, aResId, aQueryText);
	CleanupStack::PushL(pMsg);

	TBool allow=iPlayerView->QueryAllowDenyL(*pMsg);
	CleanupStack::PopAndDestroy(pMsg);
	
	return allow;
}

TBool CSaFlashAppUi::QueryAllowPhoneCallL(const TDesC& aNumberString)
{
	return QueryAllowDenyWithFormatResourceL(aNumberString, R_STR_ALLOW_PHONE_CALL);
}

TBool CSaFlashAppUi::QueryAllowNetAccessL()
{
	
//This is for not showing the network prompt when trust query is asked 
    if(!trustedQueryAsked)
	    {
	HBufC* pMessage = iEikonEnv->AllocReadResourceLC(R_STR_ALLOW_NWORK_ACCESS);
	TBool rval=iPlayerView->QueryAllowDenyL(*pMessage);
	CleanupStack::PopAndDestroy(pMessage);
	return rval;
	    }
	    //Always return true no need for separate prompt
	return ETrue;
}

void CSaFlashAppUi::CreateConnectingWaitNoteL(const TDesC& 
    aSelectedIAP, MProgressDialogCallback*  aCallback, CAknWaitDialog** aWaitDialog)
    {                    
    HBufC* waitNoteText = StringLoader::LoadLC(
    R_QTN_FLITE_WAIT_NOTE_CONNECTING, aSelectedIAP);
    
    *aWaitDialog = new ( ELeave )CAknWaitDialog
        ( REINTERPRET_CAST(CEikDialog**,aWaitDialog), ETrue );
    
    (*aWaitDialog)->PrepareLC( R_SAFLASH_CONN_WAIT_NOTE_GPRS );
    (*aWaitDialog)->SetCallback( aCallback );    
    (*aWaitDialog)->SetTextL(waitNoteText->Des());    
    (*aWaitDialog)->RunLD();
    CleanupStack::PopAndDestroy(waitNoteText);
    }

void CSaFlashAppUi::RestorePlayerView(TAny* aPlayerView)
{
	static_cast<CSaFlashPlayerView*>(aPlayerView)->HandleEmbeddedAppLaunch(EFalse);
}

const TFileName CSaFlashAppUi::GetAppPath()
{
	return Application()->AppFullName();
}

TBool CSaFlashAppUi::IsTextInputBusy()
{
	return iTextInput->IsBusy();
}

void CSaFlashAppUi::GetInputTextL(TGetInputTextData& aData)
{
	iTextInput->GetInputText(aData);
}

#ifdef __FEATURE_OSK__
void CSaFlashAppUi::SetOSKIsActive(TBool aShow)
{
    if (iPlayerView)
    {
        iPlayerView->SetOSKIsActive(aShow);
    }
}

TBool CSaFlashAppUi::UserInitiatedFullScreen()
{
	TBool userInitiated = ETrue;
	if(iEngine && iEngine->GetFullScreenViaFSCommand())
	{
		userInitiated = EFalse;
	}
	return userInitiated;
}

#endif // __FEATURE_OSK__

#ifdef FEATURE_EXTENSION
void CSaFlashAppUi::SetAutoRotationModeL(TBool aMode)
{
    if(aMode)
        iAvkonAppUi->SetOrientationL(iFirstOrientation);
    else
        iAvkonAppUi->SetOrientationL(EAppUiOrientationAutomatic);
}
#endif

void CSaFlashAppUi::SetScreenL(EScreenMode aMode)
{
	if(aMode == eFullScreen)
		SetFullScreenL(TRUE);
	if(aMode == eAppScreen)
		SetAppScreenL();
}
void CSaFlashAppUi::SetFullScreenL(TBool /*aMode*/)
{
	if (iPlayerView)
	{
		iPlayerView->SetFullScreenL();
	}
}

void CSaFlashAppUi::SetAppScreenL()
{
	if (iPlayerView)
	{
		iPlayerView->SetAppScreenL();
	}
}

void CSaFlashAppUi::SetLandScapeModeL(TBool aLandScapeMode)
{
	if (iPlayerView)
	{
		if( aLandScapeMode
#ifdef __FEATURE_FLV_RECOG_PLAY
		        || iPlayerView->IsFlvFilePlaying()
#endif
#ifdef __FEATURE_F4V_RECOG_PLAY
                || iPlayerView->IsF4vFilePlaying()
#endif
		        )
		{	
		    iPlayerView->iFullScreen = ETrue;
		    iAvkonViewAppUi->SetOrientationL(EAppUiOrientationLandscape);
		}
		else
		{
			iPlayerView->SetAppScreenL();
		}
			
	}	
}

void CSaFlashAppUi::ResetScreenInCurrentModeL()
{
	if(IsPlayerViewActive()||IsVolumeViewActive())
	{
		iPlayerView->ResetScreenInCurrentModeL();
	}
}
TBool CSaFlashAppUi::IsFullScreen()
{
	if(iPlayerView)
	{
		return iPlayerView->IsFullScreenMode();
	}
	return EFalse;
}

void CSaFlashAppUi::SetSoftkeysL(HBufC** aSoftkeyArray, TInt aSoftkeyArraySize)
{
	if (iPlayerView)
	{
		iPlayerView->SetSoftkeysL(aSoftkeyArray, aSoftkeyArraySize);
	}
}

void CSaFlashAppUi::ResetSoftkeysL()
{
	if (iPlayerView)
	{
		iPlayerView->ResetSoftkeysL();
	}
}

void CSaFlashAppUi::ClearErrors()
{
	if (iPlayerView)
	{
		iPlayerView->ClearErrors();
	}
}

void CSaFlashAppUi::NotifyFlashError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent,
										ErrorInfo* aErrorInfo)
{
	if (iPlayerView)
	{
		iPlayerView->NotifyFlashError(aErrorCode, aIsCritical, aPersistent, aErrorInfo);
	}
}

void CSaFlashAppUi::NotifyClientError(TInt aErrorCode, TBool aIsCritical, TBool aPersistent)
{
    
	if (iPlayerView)
	{
		iPlayerView->NotifyClientError(aErrorCode, aIsCritical, aPersistent);
	}
    
	if(aIsCritical)
	{
	    TRAP_IGNORE(PrepareToExitL());
	}
    
}

void CSaFlashAppUi::NotifyDataAccessError(const char* aUrl, TInt aErrorCode)
{
	if (iPlayerView)
	{
		iPlayerView->NotifyDataAccessError(aUrl, aErrorCode);
	}
}

TRect CSaFlashAppUi::ClientRect()
{
	return iPlayerView ? iPlayerView->ClientRect() : TRect();
}

TRect CSaFlashAppUi::ApplicationRect()
{
	return iPlayerView ? iPlayerView->ApplicationRect() : TRect();
}

RWindow* CSaFlashAppUi::GetWindow()
{
//	return iPlayerView ? iPlayerView->GetWindow() : (RWindow*)NULL;
    if(iPlayerView)
        {
        if(IsVolumeViewActive())
            {
            return iVolumeView->GetWindow();
            }
        else
            {
            return iPlayerView->GetWindow();
            }
        }
        return NULL;
}

RWindow* CSaFlashAppUi::GetNewVideoWindow(void* aClient)
{
    if(iPlayerView)
    {
        return iPlayerView->GetNewVideoWindow(aClient);
    }
    return NULL;
}

void CSaFlashAppUi::CloseVideoWindow(void* aClient)
{
    if(iPlayerView)
    {
        iPlayerView->CloseVideoWindow(aClient);
    }
}

TBool CSaFlashAppUi::HelpAppExists()
{
	return iHelpAppExists;
}

IniData* CSaFlashAppUi::GetIniData() 
{
	return iIniData;
}

MFlashEngine* CSaFlashAppUi::GetEngine()
{
	return iEngine;
}

void CSaFlashAppUi::SaveIAPId(TUint32 aIAPId)
{
	TUint32 old=iIniData->iIAPId;
	
	iIniData->iIAPId = aIAPId;
	
	if(old!=aIAPId)
	{
		TRAPD(ignore, iIniData->SaveUserDataL());	// don't bail out if the ini file can;t be written to
		if (ignore == KErrNone)
		{
			
		}
	}
}

TUint32 CSaFlashAppUi::IAPId()
{
	return iIniData->iIAPId;
}

void CSaFlashAppUi::NotifyShutdownCountdown(TInt aNumStepsRemaining)
{
	if(aNumStepsRemaining==0)
	{
		if (iProgressDialog)
		{
			TRAP_IGNORE(iProgressDialog->ProcessFinishedL());
		}
	}
	else
	{
		if (iProgressInfo)
		{
			iProgressInfo->IncrementAndDraw(aNumStepsRemaining-(iProgressInfo->CurrentValue()));
		}
	}
}

/*void CSaFlashAppUi::StartShutdownProgressDialogL(TInt aNumItemsToGo)
{
	iProgressDialog = new ( ELeave ) CAknProgressDialog( reinterpret_cast<CEikDialog**>(iProgressDialog) );
    
    iProgressDialog->SetCallback( this );
    iProgressDialog->PrepareLC( R_SAFLASH_SHUTDOWN_PROGRESS_DIALOG );
    iProgressInfo = iProgressDialog->GetProgressInfoL();
    iProgressInfo->SetFinalValue( aNumItemsToGo );
    iProgressDialog->RunLD();
}*/

void CSaFlashAppUi::DialogDismissedL(TInt /*aButtonId*/)
{
	iProgressDialog=NULL; // it will kindly delete itself, but unkindly not set itself to null!
	iIdle->Cancel();
	iIdle->Start(TCallBack(ShutdownCallBack,this));
}

//static
TInt CSaFlashAppUi::ShutdownCallBack(TAny* aThis)
{
	// rborcic 2005/09/20 -- for some reason, the reinterpret_cast made it
	// more likely to crash when exiting from an embedded app. The c-style
	// cast has proven to be more reliable.
	//reinterpret_cast<CSaFlashAppUi*>(aThis)->Exit();
	CSaFlashAppUi* ui = (CSaFlashAppUi*)aThis;
	ui->Exit();
	return 0;
}


#ifdef FEATURE_INLINE_TEXT_ENTRY

void CSaFlashAppUi::DisableIME()
{
	iPlayerView->DisableIME();
}

void CSaFlashAppUi::EnableIMEL()
{
	iPlayerView->EnableIMEL();
}

TInt CSaFlashAppUi::GetCurrentInputMode()
{
	if (iPlayerView)
	{
		return iPlayerView->GetCurrentInputMode();
	}

	return KErrNotFound;
}

#endif // FEATURE_INLINE_TEXT_ENTRY

TBool CSaFlashAppUi::IsFullScreenMode()
{
	if (iPlayerView)
	{
		return iPlayerView->IsFullScreenMode();
	}

	return EFalse;
}


//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------
//  Forces execution stack to unwind before quitting.  This allows the debug
//  emulator complete without memory leak errors.
//-------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------
void TExitHandler::StartEventHandler()
{
	iInsideEventHandler = ETrue;
}

//-------------------------------------------------------------------------------------------------------------
void TExitHandler::EndEventHandler(CSaFlashAppUi* aUi)
{
	iInsideEventHandler = EFalse;
	if (iExitEvent)
	{
		CSaFlashAppUi::ShutdownCallBack(aUi);
	}
}

//-------------------------------------------------------------------------------------------------------------
void TExitHandler::DoExit(CSaFlashAppUi* aUi)
{	
	//  If we're inside an event handler, let the stack unwind to the origination of the event
	if (iInsideEventHandler)
	{
		iExitEvent = ETrue;
	}
	else
	{
		//  Otherwise just exit now
		// CSaFlashAppUi::ShutdownCallBack(aUi);
		// rborcic 2005/09/18 -- use the AppUi's idle object to trigger the shutdown, otherwise
		// we can run into problems, since this can be called from the error handlers timer tick.
		if (aUi->iDoorObserver)
		{
			// rborcic 2005/09/19
			// we can't trigger an exit when we're embedded because it will crash for some unknown reason.
			// So, we set iExitEvent, which will cause the player to exit after the next WsEvent.
			// We also send a NULL event to ensure that the exit happens sooner rather than later.
			iExitEvent = ETrue;
			TWsEvent event;
			event.SetType(EEventNull);
			CCoeEnv* env = CCoeEnv::Static();
			RWsSession ws = env->WsSession();
			//ws.SendEventToAllWindowGroups(event);
			ws.SendEventToWindowGroup(env->RootWin().Identifier(), event);
			ws.Flush();
			//aUi->Exit();
		}
		else
		{
			if (!(aUi->iIdle->IsActive()))
			{
			// iExitEvent=ETrue;
				aUi->iIdle->Start(TCallBack(CSaFlashAppUi::ShutdownCallBack, aUi));	
				//fix for the bug ESJY-7FCDLJ app hangs on exit
			 //commenting this to correct corrupted swf bug  
			//aUi->Exit();	
			 
			
				
			}
		}
	}
}
HBufC* CSaFlashAppUi::GetFilename()
{
    return iSavedFilename;
}

void CSaFlashAppUi::HandleResourceChangeL(TInt aType)
{
 	CAknAppUi::HandleResourceChangeL(aType);
}

void CSaFlashAppUi::HandleResourceChangeInVolumeView(TInt aType)
{
	if(aType == KEikDynamicLayoutVariantSwitch)
	{
		if(iPlayerView)
		{
			((CSaFlashPlayerControl*)iPlayerView->GetContainer())->HandleResourceChangeInVolumeView(aType);	
		}
	}
}
/*
TBool CSaFlashAppUi::CheckPlayRightsL(const TDesC& aFilename)
{
	TBool ret = EFalse;
	ContentAccess::RAttributeSet attributeSet;
	CleanupClosePushL(attributeSet);

	attributeSet.AddL(ECanExecute);

	CManager *manager = CManager::NewLC();
	// Retrieve the attribute values from the agent
	User::LeaveIfError(manager->GetAttributeSet(attributeSet, aFilename));

	TInt attributeValue;

	TInt err = attributeSet.GetValue(ECanExecute, attributeValue);
	
	//If the file has play rights, return ETrue.
	if( ( err == KErrNone && attributeValue ) )
    {
        // content object has rights that allow it to be displayed on screen
        ret = ETrue;
    }
	CleanupStack::PopAndDestroy(2);
	return ret;	    	
}
*/

#if defined (__FEATURE_FLV_RECOG_PLAY) || defined (__FEATURE_F4V_RECOG_PLAY)
TBool CSaFlashAppUi::CheckPlayRightsL(CContent* aContent, TBool aIsVideo)
#else
TBool CSaFlashAppUi::CheckPlayRightsL(CContent* aContent)
#endif
{

	if( !aContent )
	    return EFalse;

	// Retrieve the attribute values from the agent
	TInt attributeValue = 0;
	User::LeaveIfError(aContent->GetAttribute(ECanExecute, attributeValue));

	if(attributeValue)
	    {
	    // content object has rights that allow it to be displayed on screen
	    return ETrue;
	    }
#ifdef __FEATURE_FLV_RECOG_PLAY
	    else
	    {
	    	if(aIsVideo)
	    	{
	    		User::LeaveIfError(aContent->GetAttribute(ECanPlay, attributeValue));	
	    		
	    		if(attributeValue)
	    		{// content object has rights that allow it to be displayed on screen
	    			return ETrue;
	    		}
	    			
	    	}
	    }
#endif

	return EFalse;
}
    

TBool CSaFlashAppUi::CheckAndProcessDRMFileL()
{	
	TBool isProtected;
	TBool hasPlayRights;
	CContent *content; 
	CSaFlashPlayerView::TLaunchType launchType = iPlayerView->GetLaunchType();
	if ( launchType == CSaFlashPlayerView::EEmbedded)
	{
		content = CContent::NewLC(iFile); 	
	}
	else
	{
		content = CContent::NewLC(*iSavedFilename); 	
	}
    content->GetAttribute( EIsProtected, isProtected );
    
	if (isProtected)
	{
	
		HBufC* tmpMIME = HBufC::NewLC( KMaxDataTypeLength );
    	
    	TPtr mimePtr( tmpMIME->Des() );  
    	          
		content->GetStringAttribute(EMimeType, mimePtr );
		
    	TBool isFlash =  (0 != mimePtr.Compare(KFlashMimeType))? EFalse : ETrue ;
    	
    	TBool isVideo = EFalse;
    	
    	#ifdef __FEATURE_FLV_RECOG_PLAY
    	
    	isVideo = (0 != mimePtr.Compare(KFLVMimeType))? EFalse : ETrue ;
    	
    	#endif
		
        #ifdef __FEATURE_F4V_RECOG_PLAY
         
        isVideo = (0 != mimePtr.Compare(KF4VMimeType))? EFalse : ETrue ;
         
        #endif
        
		CleanupStack::PopAndDestroy(tmpMIME);
		
		if( !isFlash && !isVideo )
		{
			HBufC* errorMessage = StringLoader::LoadLC( R_STR_RLS_CORRUPTSWFDATA_ERROR,
            	iEikonEnv );
            FlashUiUtil::ShowNoteL( EAknGlobalErrorNote, *errorMessage );
            CleanupStack::PopAndDestroy(errorMessage); 
            CleanupStack::PopAndDestroy(content);    
			return EFalse;
		}	
			
		// Check for DRM rights
        if ( launchType == CSaFlashPlayerView::EEmbedded)
        {
            iPlayerView->GetDRMHelper()->CheckRightsAmountL(iFile);
        }
        else
        {
            iPlayerView->GetDRMHelper()->CheckRightsAmountL(*iSavedFilename);	
        }

		#if defined (__FEATURE_FLV_RECOG_PLAY) || defined(__FEATURE_F4V_RECOG_PLAY ) 
			hasPlayRights = CheckPlayRightsL(content,isVideo);	
		#else
			hasPlayRights = CheckPlayRightsL(content);	
		#endif
		
		CleanupStack::PopAndDestroy(); // content
		//Check if play rights present	
		if (hasPlayRights)
		{
    		return ETrue;	
		}
		else
		{
			return EFalse;
		}
	}
	else
	{
		CleanupStack::PopAndDestroy(); // content
	}

    return ETrue;
}
void CSaFlashAppUi::ShowMuteIconL(TBool aMutedIconVisible)
{
	if(iPlayerView)
	{
		((CSaFlashPlayerControl*)(iPlayerView->GetContainer()))->ShowMutedIconL( aMutedIconVisible );
		iPlayerView->Mute( aMutedIconVisible );
	}
	
}
TBool CSaFlashAppUi::CheckRightsL(const TDesC& aFilename)
{
	TInt isProtected;
	CContent *content = CContent::NewLC( aFilename );
	content->GetAttribute( EIsProtected, isProtected );
	
	TBool ret = ETrue;
	// create the CManager object
	if(isProtected)
	{
		if(IsDRMRightsObjectMissing(aFilename))
		{
			CleanupStack::PopAndDestroy(content);
			return EFalse;
		}
		
		CManager* manager = CManager::NewLC();
		
		//Create RPointerArray to store pointers to the agents
		RArray<TAgent> agentArray;

		// Get the list of agents
		manager->ListAgentsL( agentArray );
		TInt canExecute;
		content->GetAttribute( ECanExecute, canExecute  );
		// Create a CRightsManager for the first agent		
		CRightsManager *rightsManager = manager->CreateRightsManagerL(agentArray[0]);
		CleanupStack::PushL(rightsManager);
		RStreamablePtrArray<CRightsInfo> myArray;
		CleanupClosePushL(myArray);
				
		if( canExecute )
		{
			rightsManager->ListRightsL(myArray, aFilename);
		}
		//rightsManager->ListAllRightsL(myArray);		
		TInt numRights = myArray.Count();

		if( numRights )
		{
			CRightsInfo* rightsInfo = myArray[0];
			if( rightsInfo->RightsType() == ERightsTypeStateless )
			{
				ret = EFalse;
			}		
		}
		CleanupStack::PopAndDestroy( &myArray );
		CleanupStack::PopAndDestroy( rightsManager );
		CleanupStack::PopAndDestroy( manager );
		agentArray.Close();
	}
	else
	{
		ret = EFalse;
	}
	CleanupStack::PopAndDestroy(content);
	return ret;
}

TBool CSaFlashAppUi::IsDRMRightsObjectMissing(const TDesC& aFileName)
{
	TBool roMissing(EFalse);

    TUint32 intent(EExecute);
    TBool isExpired(EFalse);
    TBool isSendingAllowed(EFalse);
    CDRMHelperRightsConstraints* rightsPlay = NULL;
    CDRMHelperRightsConstraints* rightsDisplay = NULL; 
	CDRMHelperRightsConstraints* rightsExecute = NULL; 
    CDRMHelperRightsConstraints* rightsPrint = NULL;
    
    TRAPD(err2, iPlayerView->GetDRMHelper()->GetRightsDetailsL( aFileName, intent, isExpired, isSendingAllowed, 
		    rightsPlay, rightsDisplay, rightsExecute, rightsPrint ));
    
    if( rightsPlay )
    {
    	delete rightsPlay;	
    }
    if( rightsDisplay )
    {
    	delete rightsDisplay;	
    }
    if( rightsExecute )
    {
    	delete rightsExecute;	
    }
    if( rightsPrint )
    {
    	delete rightsPrint;
    }
    
    if((err2 == KErrCANoRights))
   	{
    	roMissing = ETrue;
   	}
    
    return roMissing;
}


TBool CSaFlashAppUi::GetClippingRect(TRect& aRect)
    {
    if(IsVolumeViewActive())    
        {
        aRect = iVolumeView->GetClippingRect();
        return ETrue;
        }        
    else if(IsPlayerViewActive())    
        {
        aRect = ApplicationRect();
        return ETrue;
        }
    return EFalse;    
    }
    
void CSaFlashAppUi::NotifyPhoneActiveL(TBool aIsActive)
	{
	//Engine Paused Condition is checked in Engine->NotifyPhoneActiveL
	if(iEngine->IsRunning())
		{
		((CSaFlashPlayerControl*)iPlayerView->GetContainer())->ShowPauseIconL(aIsActive);
		}
	}

TBool CSaFlashAppUi::IsUrlDescSet()
		{
			return 	isUrlDesc;
		}
		
const TPtr CSaFlashAppUi::GetUrlDesc()
		{
		
			return iurlDesc->Des();
		}


void CSaFlashAppUi::SetUrlDescL(TPtrC& aDes)
		{
	           if(iurlDesc)
	           {
	           		delete iurlDesc;
	           		iurlDesc = NULL; 
	           }
	           iurlDesc = aDes.AllocL();
	           isUrlDesc = ETrue;
		}

void CSaFlashAppUi::HandleQwertyModeChangeNotification(TInt aQwertyMode)
{
	 iQwertyMode = aQwertyMode;
 	 //Hide/Show Onscreen keypad if swf/flv is playing in FullScreen and Landscape mode
 	 // provided sliding keypad is ON/OFF.
     if(IsFullScreenMode() && (ApplicationRect().Width() > ApplicationRect().Height()))
     {
    	TRAPD(error, iPlayerView->HandleCommandL(EEikFlashCmdFullScreen));
    	if (KErrNone != error)
    	{
    	    VwrDebug(LOG_DEBUG,
    	      _L("CSaFlashAppUi::HandleQwertyModeChangeNotification() HandleCommandL leave: %d\n"),
    	      error);    	    
    	}
     }
}

TInt CSaFlashAppUi::GetQwertyKeyPadStatus()
{
 	return iQwertyMode;
}

// -----------------------------------------------------------------------------
// CSaFlashAppUi::SetAllowMove
// -----------------------------------------------------------------------------
//
void CSaFlashAppUi::SetAllowMove(TBool aMove)
{
	iMoveAllowed = aMove;
}

TBool CSaFlashAppUi::CanAllowMove()
{
	return iMoveAllowed;
}



void CSaFlashAppUi::FLVideoStarted()
{

#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
    iVideoContentCount ++;
    if (1 == iVideoContentCount)
    {	
        iPlayerView->VideoInPlay();	
#ifdef __FEATURE_TOOLBAR__
        iPlayerView->DisableZoomToolBarButton();
#endif
    }
#endif	
}

void CSaFlashAppUi::FLVideoFinished()
{
#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
    iVideoContentCount --;
    /* Enable the Zoom button only when all Video Content
     * is stopped/closed */
    if (0 == iVideoContentCount)
    {
        iPlayerView->VideoNotInPlay(); 
#ifdef __FEATURE_TOOLBAR__
        iPlayerView->EnableZoomToolBarButton();
#endif
    }
#endif      
}

void CSaFlashAppUi::ZoomShowAll()
{
#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
    iPlayerView->DoZoomShowAll();
#endif	
}

#ifdef __FEATURE_FLV_RECOG_PLAY
RFile* CSaFlashAppUi::GetRFileHandleForPlay()
{
	return &iFile;			
}
#endif

void CSaFlashAppUi::VideoIsEmbedded(TBool aEmbeddedVideo)
{
	if(iPlayerView)
		iPlayerView->VideoIsEmbedded(aEmbeddedVideo);
}

void CSaFlashAppUi::DrawDirtyRect(const TRect& aRect)
{
	if( iPlayerView && IsPlayerViewActive() )
	{
		iPlayerView->GetContainer()->DrawNow(aRect);
	}
	if (iVolumeView && IsVolumeViewActive() )
	{
		iVolumeView->GetContainer()->DrawNow();
	}
}
#ifdef SELF_IAD_UPDATE

void CSaFlashAppUi::IADUpdateCallback( TInt /*aErrorCode*/ )
{
    // this will never be called!
}

#endif

void CSaFlashAppUi::HandleVideoError(const TInt& aError)
{
    VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::HandleVideoError ++"));
	if( iPlayerView)
	{
		iPlayerView->HandleVideoError(aError);
	}
	TRAP_IGNORE(PrepareToExitL());
	
	VwrDebug(LOG_DEBUG,_L("CSaFlashAppUi::HandleVideoError --"));
}


void CSaFlashAppUi::NotifyEngineState(TEngineState aState)
{    
    TRAPD(error, iPlayerView->HandleEngineStateChangeL(aState));
    if (KErrNone != error)
    {
        VwrDebug(LOG_DEBUG,
            _L("CSaFlashAppUi::NotifyEngineState() HandleEngineStateChangeL leave: %d \n"), 
            error);
    }
}

TInt CSaFlashAppUi::GetPersistentDirectory(const TDesC& /*aSwfPath*/, TFileName& aPersistentDir)
{
    //construct default persistent path
    RFs fs = CEikonEnv::Static()->FsSession();
    aPersistentDir.SetLength(0);
    fs.PrivatePath(aPersistentDir);
    
    TBuf<2> driveLetterAndDelimiter;
    TChar drive = fs.GetSystemDriveChar();       
    driveLetterAndDelimiter.Append(drive);
    driveLetterAndDelimiter.Append(KDriveDelimiter); 
    aPersistentDir.Insert(0,driveLetterAndDelimiter);
    
    // strip the trailing '\'
    aPersistentDir.SetLength(aPersistentDir.Length()-1);

    return KErrNone;
}

#ifdef FEATURE_EXTENSION  
TBool CSaFlashAppUi::ScreenState()
    {
    TRect rect( this->ClientRect());       
    if( rect.iBr.iX < rect.iBr.iY )  // Portrait
        {
        return ETrue;
        }
    else
        {           
        return EFalse;              
        }
    }
#endif

#ifdef FEATURE_TESTAUTOMATION
TRect CSaFlashAppUi::PositionRelativeToDeviceScreen()
	{ 
	TPoint relativePosition = this->iPlayerView->GetContainer()->PositionRelativeToScreen();
	return TRect(TPoint(0,0),relativePosition);
	}
#endif
void CSaFlashAppUi::StartShutDownProcess()
{
	if(iEngine)
    {
        iEngine->Stop();  
    }
    
    DestroyEngine();
    
    if(iQuitApp)
    {
    	iPlayerView->HandleCommandL(EAknSoftkeyBack);
    }    
}

void CSaFlashAppUi::DestroyEngine()    
{
    if (iEngine)
    {
        iEngine->DestroyPlayer();
        iEngine->Destroy();
        iEngine = NULL;
    }    
}

void CSaFlashAppUi::PrepareToExitL()
{
    //TRAP_IGNORE(iPluginView->PrepareToExitL());
    
    
    if(iPlayerView)
    {
        iPlayerView->EngineDestroyed();
        //if (iPlayerView->GetLastError() != KErrNone)
        {
            if(iAsyncDeleter == NULL)
            {
                iAsyncDeleter = new (ELeave) CAsyncDeleter(this);
            }
            iAsyncDeleter->IssueRequest();
        }
    }
    
}
