/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
                                 
*
*/

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "saplayer_build.h"
#include "symbian_document.h"
#include "symbian_appui.h"
#include <flash_ui.h>

#include <AiwGenericParam.h>

// defined originally in AvKon
const TUid KUidEmbedInputStream  = { 0x1000 };
////////////////////////////////////////////////////////////////////////////////
// CSaFlashDocument
////////////////////////////////////////////////////////////////////////////////


CSaFlashDocument::CSaFlashDocument( CEikApplication& aApp, const FlashUIConfig& aConfig )
: CAknDocument( aApp )
{
	iConfig = aConfig;
}

CFileStore* CSaFlashDocument::OpenFileL(TBool /*aDoOpen*/, const TDesC& aFilename, RFs& /*aFs*/)
{
	((CSaFlashAppUi*)iAppUi)->OpenFileL(aFilename);
	return NULL;
}


void CSaFlashDocument::OpenFileL(CFileStore*& /*aFileStore*/,  RFile& aFile)
{
	((CSaFlashAppUi*)iAppUi)->OpenFileL(aFile);
}


CEikAppUi* CSaFlashDocument::CreateAppUiL( )
{
    return new(ELeave) CSaFlashAppUi(iConfig);
}


void CSaFlashDocument::CheckAllowMove(CAiwGenericParamList*& aParam)
{
	TInt findIndexMove( 0 ),findIndexSave(0);
	TInt32 value=0;
	
	const TAiwGenericParam* paramMove = aParam->FindFirst( findIndexMove, EGenericParamAllowMove );
	
	const TAiwGenericParam* paramSave = aParam->FindFirst( findIndexSave, EGenericParamAllowSave );
	// Use only filename string, handles are ignored
	if( (findIndexMove != KErrNotFound) && paramMove && 
		(paramMove->Value().TypeId() == EVariantTypeTInt32 ))
	{
	   paramMove->Value().Get(value);
       
	}
	else
	{
		if((findIndexSave != KErrNotFound) && paramSave && 
		(paramSave->Value().TypeId() == EVariantTypeTInt32 ))
		{
			paramSave->Value().Get(value);
		}
	}	
	
	if(value)
   	{
		static_cast<CSaFlashAppUi*>(iAppUi)->SetAllowMove();
	}

}

void CSaFlashDocument::RestoreL  (  const CStreamStore &  aStore,const CStreamDictionary &  aStreamDic )
    {
    // Clear previous generic param values
	ResetUrlSetFlag();
	
    // create new param list
    CAiwGenericParamList* paramList = CAiwGenericParamList::NewLC();
    RStoreReadStream stream;
    stream.OpenLC( aStore, aStreamDic.At( KUidEmbedInputStream ) );
    // read params from the store
    paramList->InternalizeL( stream );
    
   	if( paramList )
	{
	    if( paramList && paramList->Count() )
	    {
			// First find if any files are listed
			TInt findIndex( 0 );
			const TAiwGenericParam* param = paramList->FindFirst( findIndex, EGenericParamURL );
				
			// Use only filename string, handles are ignored
			if( findIndex != KErrNotFound && param && param->Value().TypeId() == EVariantTypeDesC )
			{
				TPtrC ref = param->Value().AsDes();
	           static_cast<CSaFlashAppUi*>(iAppUi)->SetUrlDescL(ref);
			}
			CheckAllowMove(paramList);
		
			
	    }
	}
	paramList->Reset();
    CleanupStack::PopAndDestroy(&stream);  // stream
    CleanupStack::PopAndDestroy(paramList);  // stream
    
    }
    
void CSaFlashDocument::ResetUrlSetFlag()
{
	((CSaFlashAppUi*)iAppUi)->isUrlDesc = EFalse;
}


/**
 Creates the document class.
 
 @param pApp Instance of the application class.
 @param aConfig Instance of the class that contains configuration for UI.
 @return The new document object.
*/
EXPORT_C void* CreateDocumentL(CEikApplication* pApp, const FlashUIConfig& aConfig)
{
	CApaDocument* nd = new(ELeave)CSaFlashDocument(*pApp, aConfig);
	return (void*)nd;
}

#ifndef EKA2
GLDEF_C TInt E32Dll( TDllReason )
{
	return KErrNone;
}
#endif //EKA2
