/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the file control class.
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "saplayer_build.h"

#include <flash2ui.rsg>


#include <AknIconArray.h>
#include <akntabgrp.h>
#include <SAFlash.MBG>
#include <eikfrlb.h>
#include <aknlists.h> 
//#ifdef __FLASHLITE2.0_REL3.1
#include <AknWaitDialog.h> //for wait dialog
#include <gulicon.h>

#include "ui_defs.h"
#include "flash_engine.h"
//#endif //__FLASHLITE2.0_REL3.1
#include "flash2ui.hrh"
#include "symbian_fileview.h"
#include "symbian_filecontrol.h"
#include "ui_data_caging_literals.hrh"
#include "symbian_uiutil.h"
#include <featmgr.h>
////////////////////////////////////////////////////////////////////////////////
// CSaFlashFileControl
////////////////////////////////////////////////////////////////////////////////


CSaFlashFileControl::CSaFlashFileControl(CSaFlashFileView* aView)
	: iListBox (NULL)
	, iView (aView)
{
}

CSaFlashFileControl::~CSaFlashFileControl()
{
	delete iListBox;
	iListBox = NULL;
	if (iWaitNoteTimer)
	{
		delete iWaitNoteTimer;
		iWaitNoteTimer = NULL;
	}
}

void CSaFlashFileControl::ConstructL(const TRect &aRect)
{
	CreateWindowL();
	FeatureManager::InitializeLibL();  // initialise feature manager
	iTactileFeedbackSupported = FeatureManager::FeatureSupported(KFeatureIdTactileFeedback);
    FeatureManager::UnInitializeLib(); // Uninitialise Feature manager
	if(iTactileFeedbackSupported)      //adding feature manager to check the feature at run time.
	{
	    //Create an instance of Touch feedback for this class instance, otherwise it
	    //will be time consuming to retreive pointer every time from Thread local storage
	    //when the events occur on touch events.
	    	
		iFeedback = MTouchFeedback::Instance();
	}

	SetRect(aRect);
	BuildListBoxL();
	ActivateL();
}


TInt CSaFlashFileControl::CountComponentControls() const
{
	return (iListBox) ? 1 : 0;
}


CCoeControl* CSaFlashFileControl::ComponentControl(TInt /*aIndex*/) const
{
	return iListBox;
}


void CSaFlashFileControl::BuildListBoxL()
{
	//DebugShowNoteL(_L("+CSaFlashFileControl::BuildListBoxL"));
	iListBox = new( ELeave ) CAknDoubleLargeStyleListBox();
	iListBox->SetContainerWindowL(*this);
	//iListBox->SetMopParent( this );
	iListBox->SetListBoxObserver( this );

	iListBox->ConstructL( this, EAknListBoxSelectionList );

	// Creates scrollbar.
	iListBox->CreateScrollBarFrameL( ETrue );
	iListBox->ScrollBarFrame()->SetScrollBarVisibilityL(
		CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto );
	iListBox->ScrollBarFrame()->SetScrollBarFrameObserver(this);

	SizeChanged();

	// add icons to list
	SetGraphicIconsL();

	iListBox->ActivateL();
	//DebugShowNoteL(_L("-CSaFlashFileControl::BuildListBoxL"));
//#ifdef __FLASHLITE2.0_REL3.1	
	iWaitNoteTimer = CPeriodic::NewL(KWaitNoteCallBackPriority);
//#endif //__FLASHLITE2.0_REL3.1
}


void CSaFlashFileControl::UpdateTextArrayL( CDesCArray* aTextArray, const TDesC& aEmptyMsg )
{
	if ( iListBox )
	{
		// Sets list item from resource.
		iListBox->Model()->SetItemTextArray( aTextArray );
		iListBox->Model()->SetOwnershipType( ELbmDoesNotOwnItemArray );

		if ( iListBox->ScrollBarFrame() ) iListBox->ScrollBarFrame()->MoveVertThumbTo( 0 );
		SetCurrentItemL( 0 );

		iListBox->View()->SetListEmptyTextL(aEmptyMsg);

	}
}


void CSaFlashFileControl::SetCurrentItemL( TInt aIndex )
{
	iListBox->SetCurrentItemIndex( aIndex );
	SizeChanged();
	iListBox->UpdateScrollBarsL();
	iListBox->DrawDeferred();
}

void CSaFlashFileControl::HandleScrollEventL(CEikScrollBar* aScrollBar,
                                                        TEikScrollEvent aEventType)
{
	iListBox->HandleScrollEventL(aScrollBar,aEventType);
    if(iTactileFeedbackSupported) //adding feature manager to check the feature at run time.
    {
		//Tactile Feedback for events like scroll up, scroll down with thumb and
		//tapping on the empty areas in the file view with an intent to move the thumb
		
		if(iFeedback->TouchFeedbackSupported())
		{		    
			iFeedback->InstantFeedback(iListBox, ETouchFeedbackSensitive );	
		}
    }
}

void CSaFlashFileControl::SetGraphicIconsL()
{
	if ( iListBox )
	{
		// Creates gul icon.
		CArrayPtr<CGulIcon>* icons = new (ELeave) CAknIconArray( 2 );
		CleanupStack::PushL( icons );
		    
    TFileName resFile(KFlash2UIMbmFilePath);
    TFileName dllName;
    Dll::FileName(dllName);
    TBuf<2> drive = dllName.Left(2);
    resFile.Insert(0, drive);

		{
//#ifdef __FLASHLITE_BETA3

			// add directory icon
			icons->AppendL( FlashUiUtil::CreateIconL(KAvkonMbmFilePath1,
												   EMbmAvkonQgn_prop_folder_large,
												   EMbmAvkonQgn_prop_folder_large_mask) );

			// add file icon
			icons->AppendL(FlashUiUtil::CreateIconL(resFile,
												  EMbmSaflashQgn_prop_file_swf,
												  EMbmSaflashQgn_prop_file_swf_mask) );

			icons->AppendL( FlashUiUtil::CreateIconL(resFile,
												   EMbmSaflashQgn_prop_folder_sub_large,
												   EMbmSaflashQgn_prop_folder_sub_large_mask) );	
			
		    icons->AppendL(FlashUiUtil::CreateIconL(resFile,
		                                           EMbmSaflashQgn_prop_drm_rights_valid,
		                                           EMbmSaflashQgn_prop_drm_rights_valid_mask) );

			// Add rights invalid - sendable icon
		    icons->AppendL(FlashUiUtil::CreateIconL(resFile,
		                                           EMbmSaflashQgn_prop_drm_rights_exp,
		                                           EMbmSaflashQgn_prop_drm_rights_exp_mask) );

//#endif //__FLASHLITE_BETA3

			// Sets graphics as ListBox icon.
			iListBox->ItemDrawer()->ColumnData()->SetIconArray( icons );
		}
		CleanupStack::Pop( icons );
	}
}


// ----------------------------------------------------------------------------
// void CSaFlashFileControl::HandleListBoxEventL( CEikListBox*,
//  TListBoxEvent )
// Handles listbox event.
// ----------------------------------------------------------------------------
void CSaFlashFileControl::HandleListBoxEventL(
	CEikListBox* /*aListBox*/,
	TListBoxEvent aEventType )
{
	 switch ( aEventType )
        {
        case MEikListBoxObserver::EEventEnterKeyPressed:
        case MEikListBoxObserver::EEventItemDoubleClicked:
        {
      
          	if(iTactileFeedbackSupported) //adding feature manager to check the feature at run time.
          		{
				if(iFeedback->TouchFeedbackSupported())
					{		    
					iFeedback->InstantFeedback(iListBox, ETouchFeedbackBasic );	
					}		
          		}
    		if ( iView )
    		    {
    		    iView->ItemSelectedL();
    		    }    			
        }
            break;
            
        case MEikListBoxObserver::EEventItemClicked:
        {
        
        	if(iTactileFeedbackSupported) //adding feature manager to check the feature at run time.
	        	{
				if(iFeedback->TouchFeedbackSupported())
					{		    
					iFeedback->InstantFeedback(iListBox, ETouchFeedbackSensitive );	
					}
    	    	}
			if ( iView )
				{
			iView->UpdateSelectionL( iListBox->CurrentItemIndex() );
				}
				}
            break;
            
        default:
            break;
	}
}


// ----------------------------------------------------------------------------
// TKeyResponse CSaFlashFileControl::OfferKeyEventL( const TKeyEvent&,
//  TEventCode )
// Handles the key events.
// ----------------------------------------------------------------------------
//
TKeyResponse CSaFlashFileControl::OfferKeyEventL(
	const TKeyEvent& aKeyEvent,
	TEventCode aType )
{
	if ( aType != EEventKey ) // Is not key event?
		return EKeyWasNotConsumed;

	if ( !iView || !iListBox )
		return EKeyWasNotConsumed;

	CAknTabGroup* tabGroup = iView->TabGroup();
	TChar charCode(aKeyEvent.iCode);

	TBool consumed = EFalse;
	TKeyResponse response;
	switch ( charCode ) // The code of key event is...
	{
		// Switches tab.
	case EKeyLeftArrow: // Left key.
	case EKeyRightArrow: // Right Key.
		if (tabGroup) // Is tab group created?
		{
			if (iView->AtTopLevel())
			{
				if ( tabGroup->OfferKeyEventL( aKeyEvent, aType ) == EKeyWasConsumed )
				{
					iView->SetTabL( tabGroup->ActiveTabIndex() );
					response =  EKeyWasConsumed;
					consumed = ETrue;
				}
			}
		}
		break;

	case EKeyUpArrow: // Left key.
	case EKeyDownArrow: // Right Key.
		{
			// save current index
			TInt currentItem = iListBox->CurrentItemIndex();

			// Let list box handle event
			response = iListBox->OfferKeyEventL( aKeyEvent, aType );
			consumed = ETrue;

			TInt newItem = iListBox->CurrentItemIndex();
			if ( currentItem != newItem )
			{
				iView->UpdateSelectionL( newItem );
			}
		}
		break;

	case EKeyDevice3:
		{
			// Gets current selected item index.
			if ( iView )
				iView->ItemSelectedL();
			response =  EKeyWasConsumed;
			consumed = ETrue;
		}
		break;

	case EKeyBackspace:
		{ // act like "delete" was selected from menu.
			if (iView)
				iView->HandleCommandL(EEikFlashCmdDelete);
		}
		break;

	default:
		break;
	}

	if ( consumed )
	{
		return response;
	}
	else
	{
		return iListBox->OfferKeyEventL( aKeyEvent, aType );
	}
}


void CSaFlashFileControl::SizeChanged()
{
	if ( iListBox )
	{
		iListBox->SetRect( Rect() ); // Sets rectangle of listbox.
	}
}
//#ifdef __FLASHLITE2.0_REL3.1
void CSaFlashFileControl::ShowWaitNoteL(TInt aResourceId)
{
	if (iWaitNoteTimer->IsActive())
    {
        iWaitNoteTimer->Cancel();
    }
    
    // Timer for deleting note is set if user cannot delete the note
    TCallBack callback(WaitNoteTimeoutL, this);
    iWaitNoteTimer->Start(
        (TTimeIntervalMicroSeconds32) KMaxWaitNoteShowTime,
        (TTimeIntervalMicroSeconds32) KMaxWaitNoteShowTime, 
        callback);
            
    CAknWaitDialog* waitDialog = 
        new (ELeave) CAknWaitDialog(NULL, ETrue);
    waitDialog->ExecuteLD(aResourceId); 
}

// ----------------------------------------------------------------------------
// SAFlashFileControl::WaitNoteTimeoutL
// Callback function for deleting wait note.
// ----------------------------------------------------------------------------
TInt CSaFlashFileControl::WaitNoteTimeoutL(TAny* aThis )
{
	CSaFlashFileControl* container = 
        STATIC_CAST(CSaFlashFileControl*, aThis);
    
    container->iWaitNoteTimer->Cancel();
    CCoeAppUi* ui = STATIC_CAST(CCoeAppUi*, container->iCoeEnv->AppUi()); 
    
    if (ui->IsDisplayingMenuOrDialog()) // Check whether wait note is displayed.
    {
        TKeyEvent key;
        key.iCode = EKeyEscape;
        key.iModifiers = 0;
        container->iCoeEnv->SimulateKeyEventL(key, EEventKey);
    }

    return 0; 
}
void CSaFlashFileControl::HandleResourceChange(TInt aType)
{
	CCoeControl::HandleResourceChange(aType);
	if ( aType ==KEikDynamicLayoutVariantSwitch )
	{
		TRect rect;
		AknLayoutUtils::LayoutMetricsRect(AknLayoutUtils::EMainPane,rect);
		SetRect(rect);
        iView->HandleResourceChange(aType);
	}
}
