/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the file view class.
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "saplayer_build.h"
#include <flash2ui.rsg>

#include "symbian_appui.h"
#include "flash_engine.h"
#include "saflash_coredefs.h"
#include "flash2ui.hrh"
#include "ui_defs.h"
#include "ui_data_caging_literals.hrh"

#include <AknIconArray.h>
#include <AknQueryDialog.h>
#include <aknnavide.h>
#include <akntabgrp.h>
#include <akntitle.h> // for title pane
#include <favouritesdb.h>
#include <sendnorm.rsg> // for CSendAppUi reference to R_SENDUI_MENU
#include <SAFlash.MBG>
#include <AknIconUtils.h>
#include <StringLoader.h>
#include <pathinfo.h>
#include <AknCommonDialogs.h>
#include <e32math.h>
#include <content.h> 
#include <DRMRightsManager.rsg>
#include <textresolver.h>
#include <ConeResLoader.h>
#include <barsread.h>
#include <badesca.h>
#include <bautils.h>
#include <aknlayoutscalable_avkon.cdl.h>
#include "ui_defs.h"
#include <tz.h>

#include <AknsUtils.h>
#include <aknlists.h>
//#include <MGXFileManagerFactory.h> //MG not required
//#include <CMGXFileManager.h>

#include <CDirectoryLocalizer.h>
#include <featmgr.h>

#include "saflash_consts.h"
#include "symbian_fileview.h"
#include "symbian_filecontrol.h"
//#include "Symbian_DirList.h"

#include "symbian_uiutil.h"

using namespace ContentAccess;
// CONSTANTS
// If number is 10 or more 2 digits are needed to display it
const TInt KTwoDigitNeeded = 10;
//const TChar KTabChar = '\t';
#define KTabChar '\t'
//_LIT( KTabChar,"\t");
_LIT( KDirectionalChars, "\x202A\x202B\x202C\x202D\x200E\x200F" );
//_LIT(KLitDot,".");
/////////////////////////////////////////////////////////////////////////////////
// TOverflowHandler
////////////////////////////////////////////////////////////////////////////////

extern void VwrDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );
TOverflowHandler::TOverflowHandler()
    {
    iError = KErrNone;       
    }

TInt TOverflowHandler::IsOverflowed()
    {
    return iError;
    }

void TOverflowHandler::Overflow(TDes16& /*aDes*/)
    {
    iError = KErrOverflow;    
    }
////////////////////////////////////////////////////////////////////////////////
// CSaFlashFileView
////////////////////////////////////////////////////////////////////////////////


CSaFlashFileView::CSaFlashFileView(MFlashEngine* aEngine, CFlashErrorDialog& aLowMemDialog)
	: iEngine(aEngine)
	, iLowMemDialog(aLowMemDialog)
//#ifdef __FLASHLITE2.0_REL3.1
	,iHasMMC( ETrue )
	,iMMCLocked( EFalse )
	,iMMCCorrupted( EFalse )
	,iSeparator(NULL)
    ,iEndChar(NULL)
    ,iPhoneIcon(NULL)
    ,iPhoneIconMask(NULL)
    ,iMMCIcon(NULL)
    ,iMMCIconMask(NULL)
    ,iFolderIcon(NULL)
    ,iFolderIconMask(NULL)
    ,iLevel(0)
	,iResourceLoader(*iCoeEnv)
	,iFolderNavi(NULL)

//#endif //__FLASHLITE2.0_REL3.1
{
}

CSaFlashFileView::~CSaFlashFileView()
{
	FeatureManager::UnInitializeLib(); // Uninitialise Feature manager
	delete iDecoratedTabGroup;
	iDecoratedTabGroup = NULL;
//#ifdef __FLASHLITE2.0_REL3.1
	if (iDrmHelper)
    {
        delete iDrmHelper;
		iDrmHelper = NULL;
    }
    if(iSeparator)
    {
        delete iSeparator;
        iSeparator = NULL;
    }
    if(iEndChar)
    {
        delete iEndChar;
        iEndChar = NULL;
    }
    iResourceLoader.Close();
    delete iLocalizer;

}

void CSaFlashFileView::ConstructL(TBool aIncludeSVG)
{
	FeatureManager::InitializeLibL();  // initialise feature manager
	VwrDebug(LOG_DEBUG,_L("CSaFlashFileView::ConstructL() ++"));
	iIncludeSVG	= aIncludeSVG;

	BaseConstructL(R_SAFLASH_FILEVIEW);

	iRootPhonePath.Copy(PathInfo::PhoneMemoryRootPath());
	iRootPhonePath.Append(PathInfo::OthersPath());
	iRootMemCardPath.Copy(PathInfo::MemoryCardRootPath());
	iRootMemCardPath.Append(PathInfo::OthersPath());

	CheckMemoryCard();
	// test for presence of KFlashPhonePath and KFlashCardPath directories and create them if necessary.
	iEikonEnv->FsSession().MkDirAll(iRootPhonePath);
//#ifdef __FLASHLITE2.0_REL3.1
	iDrmHelper = CDRMHelper::NewL( *CCoeEnv::Static() );
	
//#endif //__FLASHLITE2.0_REL3.1	
	iLocalizer = CDirectoryLocalizer::NewL();
	VwrDebug(LOG_DEBUG,_L("CSaFlashFileView::ConstructL() --"));
}


TUid CSaFlashFileView::Id() const
{
	return KSAFlashFileViewId;
}

void CSaFlashFileView::DoActivateL(const TVwsViewId& /*aPrevViewId*/, TUid /*aCustomMessageId*/, const TDesC8& /*aCustomMessage*/)
{
	//DebugShowNoteL(_L("+CSaFlashFileView::DoActivateL"));
	#ifdef FEATURE_HIDE_SAFLASH

   	User::Leave(KErrNotSupported);
       
    #endif
    
	ASSERT(!iContainer);

	// make sure player is paused.
	((CSaFlashAppUi*)AppUi())->GetEngine()->Pause(EReasonFocus);

	StatusPane()->MakeVisible(ETrue);
	Cba()->MakeVisible(ETrue);

	if (!iContainer)
	{
		iContainer = new(ELeave) CSaFlashFileControl(this) ;
		iContainer->ConstructL(AppUi()->ClientRect());
		iContainer->SetMopParent(this);
	}
	else
	{
	    iContainer->BuildListBoxL();    
	}

	// Adds container to view control stack.
	AppUi()->AddToStackL( *this, iContainer );
	iContainer->MakeVisible( ETrue );

	// Build the list box
	

	SetNavigationPaneL();

	if (iPhonePath.Length() == 0)
	{
		iPhonePath = iRootPhonePath;
		iCardPath  = iRootMemCardPath;
		HBufC* hFlash = iEikonEnv->AllocReadResourceL(R_STR_FLASH_TITLE);
		if (hFlash)
		{
			SetTitlePaneTextL(*hFlash);
		}
		delete hFlash;
	}
	else
	{
		if (ViewingPhoneDir())
		{
			SetNavigationTextFromPathL(0, iPhonePath);
		}
		else
		{
			SetNavigationTextFromPathL(0, iCardPath);
		}
	}

	//iContainer->UpdateScreenSize(AppUi()->ClientRect());

	TInt err;
	TRAP(err, UpdateListL(EFalse));
	if (err != KErrNone)
	{
		iCurrentItemIndex = 0;
		iCurrentMMCItemIndex = 0;
		TraverseUpL();
	}
	//DebugShowNoteL(_L("-CSaFlashFileView::DoActivateL"));
}
MFlashEngine* CSaFlashFileView::GetEngine()
{
	return iEngine;
}
void CSaFlashFileView::DoDeactivate()
{
	if (iContainer)
	{
		AppUi()->RemoveFromStack(iContainer);
		iContainer->MakeVisible(EFalse);
		delete iContainer;
		iContainer = NULL;
	}

	if (iNaviBmp)
	{
		iNaviPane->Pop(iNaviBmp);
		delete iNaviBmp;
		iNaviBmp = NULL;
	}

	if ( iDecoratedTabGroup )
	{
		iNaviPane->Pop(iDecoratedTabGroup);
		delete iDecoratedTabGroup;
		iDecoratedTabGroup = NULL;
	}
/*
	if (!fBeingDestroyed)
	{
		HBufC* hFlash = NULL;
		TInt err;
		TRAP(err, hFlash = iEikonEnv->AllocReadResourceL(R_STR_FLASH_TITLE););
		if (hFlash)
		{
			TRAP_IGNORE(SetTitlePaneTextL(*hFlash));
			delete hFlash;
		}
	}
*/
	delete iPhoneDirItems;
	iPhoneDirItems = NULL;

	delete iCardDirItems;
	iCardDirItems = NULL;
}

void CSaFlashFileView::HandleCommandL( TInt aCommand )
{
	TRAPD(err, DoHandleCommandL(aCommand));
	if (err ==KErrNotSupported)
	{
		User::Leave(err);
	}
	else if (err)
	{
		HandleError(err);
	}

}

void CSaFlashFileView::DoHandleCommandL(TInt aCommand)
{
	switch  ( aCommand )
	{
		// Handles exit command
		case EAknSoftkeyExit:
		case EEikCmdExit:
			((CSaFlashAppUi*)AppUi())->CloseApplication();
			break;

		case EAknSoftkeyCancel:
			AppUi()->ActivateLocalViewL(KSAFlashPlayerViewId);
			break;

		case EAknSoftkeyBack:
			TraverseUpL();
			break;

		case EEikFlashCmdOpen:
			ItemSelectedL();
			break;

		case EEikFlashCmdRename:
			HandleRenameCmdL();
			break;

		case EEikFlashCmdOrganiseNewFolder:
			HandleNewFolderCmdL();
			break;

		case EEikFlashCmdDelete:
			HandleDeleteCmdL();
			break;

		case EEikFlashCmdCut:
			if ( CurrentItemIsValid() //&& !CurrentItemIsDir()
				 )
			{
				GetSelectedPath( iSavedPath );
				iIsSavedPathValid = (iSavedPath.Length() > 0);
			}
			break;

		case EEikFlashCmdPaste:
			HandlePasteCmdL();
			break;

		case EEikFlashCmdOrganiseMoveToFolder:
//#endif //__FLASHLITE2.0_REL3.1
			HandleMoveCmdL();
			break;
//#ifdef __FLASHLITE2.0_REL3.1
		case EEikFlashCmdOrganiseCopyTo:
			{
				HandleCopyCmdL();
			}
			break;
			
		case EEikFlashCmdViewDetails:
			{
				DisplayFileDetailsL();
			}
			break;
		
//#endif //__FLASHLITE2.0_REL3.1

		case EEikFlashCmdClose:
			AppUi()->ActivateLocalViewL(KSAFlashPlayerViewId);
			break;

		case EEikFlashCmdHelp:
			{
				if ( ViewingPhoneDir() && AtTopLevel())
                {
				((CSaFlashAppUi*)AppUi())->LaunchHelpL( EEikFlashHelpPhoneMemory );    
                }
				else if (ViewingPhoneDir() && !AtTopLevel())
                {
               	    ((CSaFlashAppUi*)AppUi())->LaunchHelpL( EEikFlashHelpFolderView );    
                }
            	else
                {
                    ((CSaFlashAppUi*)AppUi())->LaunchHelpL( EEikFlashHelpMMC );    
                }	
			}
			break;
//#ifdef __FLASHLITE2.0_REL3.1
		case EEikFlashCmdAbout:
			{
				((CSaFlashAppUi*)AppUi())->AboutL();
			}
			break;
//#endif //__FLASHLITE2.0_REL3.1

		case EEikFlashCmdSend:
			{
				TBuf<KMaxFileName> aFilename;
				GetSelectedPath(aFilename);
//#ifdef __FLASHLITE2.0_REL3.1
//#endif //__FLASHLITE2.0_REL3.1				
					((CSaFlashAppUi*)AppUi())->HandleSendCommandL(aCommand, aFilename);
			}
		default:
			break;
	}
}

void CSaFlashFileView::DynInitMenuPaneL( TInt aResourceId, CEikMenuPane* aMenuPane )
{
//#ifdef __FLASHLITE2.0_REL3.1
	TBool currentFolderEmpty( ETrue );
    CheckMemoryCard();
    UpdateListL(EFalse);
    
    TInt itemCount = ViewingPhoneDir() ? iPhoneDirItems->Count() : iCardDirItems->Count();
    if ( itemCount )
    {
        currentFolderEmpty = EFalse;
    }

	if (aResourceId == R_FILEVIEW_ORGANISE_MENU)
	{
		CEikMenuPane::CItemArray* itemArray = 
			new(ELeave) CEikMenuPane::CItemArray();
		CleanupStack::PushL(itemArray);
		
		CEikMenuPaneItem* item_NewFolder;
		CEikMenuPaneItem* item_CopyTo;
		CEikMenuPaneItem* item_MoveToFolder;
		
		item_NewFolder = new(ELeave) CEikMenuPaneItem();
		CleanupStack::PushL(item_NewFolder);
		item_NewFolder->iData.iCommandId = EEikFlashCmdOrganiseNewFolder;
		iCoeEnv->ReadResourceL(item_NewFolder->iData.iText, R_STR_FILEVIEW_ORGANISE_NEWFOLDER);
		
		item_MoveToFolder = new(ELeave) CEikMenuPaneItem();
		CleanupStack::PushL(item_MoveToFolder);
		item_MoveToFolder->iData.iCommandId = EEikFlashCmdOrganiseMoveToFolder;
		iCoeEnv->ReadResourceL(item_MoveToFolder->iData.iText, R_STR_FILEVIEW_ORGANISE_MOVETOFOLDER);
		
		item_CopyTo = new(ELeave) CEikMenuPaneItem();
		CleanupStack::PushL(item_CopyTo);
		item_CopyTo->iData.iCommandId = EEikFlashCmdOrganiseCopyTo;
		iCoeEnv->ReadResourceL(item_CopyTo->iData.iText, R_STR_FILEVIEW_ORGANISE_COPYTO);

		TBool showMoveToFolder = EFalse;
		if ( !currentFolderEmpty )
		{
			itemArray->AddItemL(item_CopyTo);
			showMoveToFolder = ETrue;
		}
		
		//If the PWD is phone root dir, then "move to folder" option 
		//is shown if it has any sub dirs or if MMC is present.		
		GetCurrentDir(iOldDir);
		if (!iOldDir.Compare(iRootPhonePath))
		{
			TInt numOfSubDirs = FlashUiUtil::CheckSubdirCountL(KNullDesC, iOldDir, ETrue, EFalse);
			if (!iHasMMC && !numOfSubDirs)
			{	
				showMoveToFolder = EFalse;
			}					
		}
		
		if (showMoveToFolder)
		{
			itemArray->AddItemL(item_MoveToFolder);
		}
		
		itemArray->AddItemL(item_NewFolder);
		// itemArray will own these objects, so pop them off
		// the cleanup stack after adding them.
		if( currentFolderEmpty )
		{
			CleanupStack::PopAndDestroy(item_CopyTo);		
		}
		else
		{
			CleanupStack::Pop(item_CopyTo);			
		}
		
		if (!showMoveToFolder)
		{
			CleanupStack::PopAndDestroy(item_MoveToFolder);
		}
		else
		{
			CleanupStack::Pop(item_MoveToFolder);
		}
		CleanupStack::Pop(item_NewFolder);
		
		// menu pane will own the item array, so we can pop
		// it off the cleanup stack.
		aMenuPane->SetItemArray(itemArray);
		CleanupStack::Pop(itemArray);
	}
	else if (aResourceId == R_FILEVIEW_CONTROL_MENU)
	{
//#endif //__FLASHLITE2.0_REL3.1

		if( !ViewingPhoneDir() && (!iHasMMC || iMMCLocked || iMMCCorrupted))
		{
			aMenuPane->SetItemDimmed( EEikFlashCmdOrganise, ETrue );
		}
		if ( currentFolderEmpty ) // remove items from the options menu if folder is empty
            {
//#ifdef __FLASHLITE2.0_REL3.1
				aMenuPane->SetItemDimmed( EEikFlashCmdViewDetails, ETrue );
//#endif //__FLASHLITE2.0_REL3.1            
                aMenuPane->SetItemDimmed( EEikFlashCmdOpen, ETrue );
                aMenuPane->SetItemDimmed( EEikFlashCmdDelete, ETrue );
                aMenuPane->SetItemDimmed( EEikFlashCmdRename, ETrue );
            }
		if (!CurrentItemIsDir() && !currentFolderEmpty)
		{
			((CSaFlashAppUi*)AppUi())->DoSendMenuL(aMenuPane, KMaximumMessageSize, EEikFlashCmdDelete);
		}
		else
		{
			aMenuPane->SetItemDimmed(EEikFlashCmdViewDetails, ETrue);			
		}
//#endif //__FLASHLITE2.0_REL3.1
		if (!((CSaFlashAppUi*)AppUi())->HelpAppExists())
		{
			aMenuPane->SetItemDimmed(EEikFlashCmdHelp, ETrue);
		}
	}
}

void CSaFlashFileView::DynInitMenuBarL( TInt /*aResourceId*/, CEikMenuBar* /*aMenuBar*/ )
{
}

void CSaFlashFileView::SetTabL( TInt newTab )
{
	if ( newTab != iCurrentTab )
	{
		CheckMemoryCard();
		iCurrentTab = newTab;
		//UpdateListL();
		ReplaceList();
	}
}

void CSaFlashFileView::ClearCurrentItem()
{
	iCurrentItem.SetLength( 0 );
	iCurrentItemIsDir = EFalse;
	iCurrentItemIndex = 0;
	iCurrentMMCItemIndex = 0;
}

void CSaFlashFileView::ItemSelectedL()
{
	if ( CurrentItemIsValid() )
	{
		if ( CurrentItemIsDir() )
		{
			// path is a dir.
			TraverseDownL();
		}
		else 
		{
			// open file
			TBuf<KMaxFileName>	path;
			GetSelectedPath( path );
			((CSaFlashAppUi*)AppUi())->OpenFileL(path);
		}
	}
}

void CSaFlashFileView::HandleDeleteCmdL()
{
	TBool showNote = EFalse;
	TBool showNoteForNonSWFFileDeletion = ETrue;
	if ( CurrentItemIsValid() )
	{
		GetSelectedPath( iOrigPath );

		// confirmation msg: "Do you want to delete this file?"
		TBool isDir = CurrentItemIsDir();
				
		_LIT(KNULL, "");
		
		iNewPath.Copy(iOrigPath);
		iNewPath.Delete( iOrigPath.Length()-1, KDeleteLength);
		
		TInt numOfSubDirs = FlashUiUtil::CheckSubdirCountL(KNULL, iNewPath, ETrue, EFalse);
		TInt numOfSWFFiles = FlashUiUtil::CheckSubdirCountL(KNULL, iNewPath, EFalse, ETrue);
		
		TBuf<KBufSize> formatResource;
		HBufC* errorMessage;
		
		//If deleting an empty directory or a file then show 
		//"Delete filename". A directory is empty if it has zero files and 
		//zero sub directories.
		if (( numOfSubDirs == KZero && numOfSWFFiles == KZero ) || (!isDir))
		{
			if (isDir)
			{
			//Use length-1 for not to display character "\".
			TBuf<KMaxFileName> temp(iCurrentItem);
			errorMessage = StringLoader::LoadLC( R_STR_DELETE_FILE_QUERY_MSG,
            temp.Left(iCurrentItem.Length()-1), iEikonEnv );	
			}
			else
			{
			//But for file, it isn't required.
			errorMessage = StringLoader::LoadLC( R_STR_DELETE_FILE_QUERY_MSG,
            iCurrentItem, iEikonEnv );
            }
			
		}
		else
		{
       		errorMessage = StringLoader::LoadLC( R_STR_DELETE_FOLDER_QUERY_MSG,
            	iEikonEnv );
		}
				
		CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::EConfirmationTone );
		if ( dlg->ExecuteLD(R_SAFLASH_CONFIRMATION_QUERY, *errorMessage ) )
		{
        	CleanupStack::PopAndDestroy();    // errorMessage 	
        	
        	if (!isDir)
        	{
        		TBool fileExists( BaflUtils::FileExists(CCoeEnv::Static()->FsSession(), iOrigPath) );
        		if ( fileExists == EFalse)
        		{
        			HBufC* errorMessage = StringLoader::LoadLC( R_STR_DELETE_FOLDER_FAIL_FORMAT1_MSG,
            			iCurrentItem, iEikonEnv );
        			FlashUiUtil::ShowNoteL( EAknGlobalInformationNote, *errorMessage );
        			CleanupStack::PopAndDestroy();    // errorMessage 
        			//Refreshing the list. For detail: ECPI-6SXF8Z.
        			ReplaceList();
        			return;	
        		}
        	}

			//iContainer->ShowWaitNoteL(R_SAFLASH_DELETE_WAITDIALOG);
	
			TInt err;
			if ( isDir )
			{
				RFs fs = iEikonEnv->FsSession();
				CFileMan* fileMan = CFileMan::NewL(fs);
				CleanupStack::PushL(fileMan);
				
				TInt allFilesCount = FlashUiUtil::CheckSwfCountL(KNULL,iNewPath,KFileSearchStr,ETrue);
				//If a folder is not empty, then recursive delete 
				//using CFileMan is done. If a directory is empty 
				//non-recursive delete using RFs is done
				if ( (numOfSubDirs != KZero) || (numOfSWFFiles!= KZero) )
					{
					//if (( allFilesCount > KZero) && numOfSWFFiles == KZero)					
					if (allFilesCount > numOfSWFFiles)
						{
						// Handling the deletion of files with extension other than .SWF
						HBufC*  errorMessage;
						errorMessage = StringLoader::LoadLC( R_STR_DEL_ALL_HIDDEN_QUERY_MSG,
						iEikonEnv );
						CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::EConfirmationTone );
						if ( dlg->ExecuteLD(R_SAFLASH_CONFIRMATION_QUERY, *errorMessage ) )
							{
							iContainer->ShowWaitNoteL(R_SAFLASH_DELETE_WAITDIALOG);
							err = fileMan->RmDir(iOrigPath);
							}
						else
							{
							err = !KErrNone;
							showNoteForNonSWFFileDeletion = EFalse;
							}
						CleanupStack::PopAndDestroy();    // errorMessage 	
						}
					else
						{
						iContainer->ShowWaitNoteL(R_SAFLASH_DELETE_WAITDIALOG);
						err = fileMan->RmDir(iOrigPath);	
						}
					}
				else
					{
					//if (( allFilesCount > KZero) && numOfSWFFiles == KZero)
					if (allFilesCount > numOfSWFFiles)
						{
						// Handling the deletion of files with extension other than .SWF
						HBufC*  errorMessage;
						errorMessage = StringLoader::LoadLC( R_STR_DEL_ALL_HIDDEN_QUERY_MSG,
	            		iEikonEnv );
	            		CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::EConfirmationTone );
						if ( dlg->ExecuteLD(R_SAFLASH_CONFIRMATION_QUERY, *errorMessage ) )
							{
							iContainer->ShowWaitNoteL(R_SAFLASH_DELETE_WAITDIALOG);
							//err = iEikonEnv->FsSession().RmDir( iOrigPath );
							err = fileMan->RmDir(iOrigPath);
							}
						else
							{
							err = !KErrNone;
							showNoteForNonSWFFileDeletion = EFalse;
							}
						CleanupStack::PopAndDestroy();    // errorMessage 	
						}
					else
						{
						iContainer->ShowWaitNoteL(R_SAFLASH_DELETE_WAITDIALOG);
						err = iEikonEnv->FsSession().RmDir( iOrigPath );	
						}
					}
				
				CleanupStack::PopAndDestroy(fileMan);
			}
			else
			{
				showNote = ((CSaFlashAppUi*)AppUi())->CheckRightsL(iOrigPath);
				err = iEikonEnv->FsSession().Delete( iOrigPath );
			}
			
			if ( KErrNone != err )
				{
				//If user tries to delete an open file or a
				//folder that contains open items, then error message
				//is displayed.
				if (KErrInUse == err)
					{
					TInt isDirectory = isDir
					? R_STR_DIRECTORY_OPEN_ERROR
					: R_STR_FILE_OPEN_ERROR;
					
					if (isDir)
						{
						TInt numOfSWFFiles = FlashUiUtil::CheckSubdirCountL(KNULL, iNewPath, EFalse, EFalse);
						TInt numOfSubDirs = FlashUiUtil::CheckSubdirCountL(KNULL, iNewPath, ETrue, EFalse);

						HBufC* errorMessage = StringLoader::LoadLC( isDirectory, (numOfSWFFiles + numOfSubDirs), iEikonEnv );
	                	FlashUiUtil::ShowNoteL( EAknGlobalInformationNote, *errorMessage );
						CleanupStack::PopAndDestroy();    // errorMessage
						ReplaceList();
						}
					else
						{
						HBufC* errorMessage = StringLoader::LoadLC( isDirectory,
	                	iCurrentItem, iEikonEnv );	
						FlashUiUtil::ShowNoteL( EAknGlobalInformationNote, *errorMessage );         		
						CleanupStack::PopAndDestroy();    // errorMessage 
						}
					}
				else
					{
					// If the selected folder cannot be deleted for
					//any other reason, then error message is shown.
					if (showNoteForNonSWFFileDeletion)
						{
						if (isDir)
							{
							//Use length-1 for not to display character "\".
							TBuf<KMaxFileName> temp(iCurrentItem);
							HBufC* errorMessage = StringLoader::LoadLC( R_STR_DELETE_FOLDER_FAIL_FORMAT1_MSG,
	                			temp.Left(iCurrentItem.Length()-1), iEikonEnv );
	            			FlashUiUtil::ShowNoteL( EAknGlobalInformationNote, *errorMessage );
	            			CleanupStack::PopAndDestroy();    // errorMessage 	
							}
						else
							{
							HBufC* errorMessage = StringLoader::LoadLC( R_STR_DELETE_FOLDER_FAIL_FORMAT1_MSG,
	                			iCurrentItem, iEikonEnv );
	                		FlashUiUtil::ShowNoteL( EAknGlobalInformationNote, *errorMessage );
	            			CleanupStack::PopAndDestroy();    // errorMessage 	
							}
						//Refreshing the list. For detail: ECPI-6SXF8Z.
						ReplaceList();
						}
					}
				}
			else 
			{
				// Update list since item deleted
				if(showNote && !(CurrentItemIsDir()))
				{
					CAknGlobalNote* globalNote = CAknGlobalNote::NewLC();
        			HBufC* msg = StringLoader::LoadLC( R_QTN_DRM_MO_DELETED,iOrigPath );
        			globalNote->ShowNoteL( EAknGlobalErrorNote, msg->Des() );
        			CleanupStack::PopAndDestroy(2);  // globalNote, errorMsg
				}
				ReplaceList();
				
				/*  // Media Gallery not required
				CMGXFileManager* mgxFileMgr = MGXFileManagerFactory::NewFileManagerL(iEikonEnv->FsSession());
				mgxFileMgr->UpdateL();        				
	        	delete mgxFileMgr;
	        	*/

			}
		}
		else
		{
	    	CleanupStack::PopAndDestroy();    // errorMessage 	
		}
	}
}

static TBool HasSwfExtension( const TDesC& aStr )
{
	TInt len = aStr.Length();
	return ( len >= 4 &&
			 (aStr[len-4] == '.' &&
			  (aStr[len-3] == 's' || aStr[len-3] == 'S') &&
			  (aStr[len-2] == 'w' || aStr[len-2] == 'W') &&
			  (aStr[len-1] == 'f' || aStr[len-1] == 'F') ) );
}

#ifdef __FEATURE_FLV_RECOG_PLAY

static TBool HasFlvExtension( const TDesC& aStr )
{
	TInt len = aStr.Length();
	return ( len >= 4 &&
			 (aStr[len-4] == '.' &&
			  (aStr[len-3] == 'f' || aStr[len-3] == 'F') &&
			  (aStr[len-2] == 'l' || aStr[len-2] == 'L') &&
			  (aStr[len-1] == 'v' || aStr[len-1] == 'V') ) );
}

#endif

#ifdef __FEATURE_F4V_RECOG_PLAY

static TBool HasF4vExtension( const TDesC& aStr )
{
    TInt len = aStr.Length();
    return ( len >= 5 &&
             (aStr[len-5] == '.' &&
              (aStr[len-4] == 'f' || aStr[len-4] == 'F') &&
              (aStr[len-3] == '4') &&
              (aStr[len-2] == 'v' || aStr[len-2] == 'V') 
              )
           );
}

#endif
void CSaFlashFileView::HandleRenameCmdL()
	{
    TOverflowHandler overflow;
//#ifdef __FLASHLITE2.0_REL3.1
	// do nothing unless the current selection is valid
	TBool getNewName = ETrue;
	if ( CurrentItemIsValid() )
		{
        iOrigName = iCurrentItem;
		TBool isDir = CurrentItemIsDir();

		// strip off trailing '\\' if dir
		TInt len = iOrigName.Length();
		if ( len > 0 && iOrigName[len-1] == KCharBackslash )
			{
			iOrigName.SetLength( len-1 );
			isDir = ETrue;
			}
        while (getNewName)
        {
		    if ( GetNewNameL( iOrigName, iNewName, ERenameFileOrFolder ) )
		    	{
		    	if (iOrigName != iNewName )
			    	{
			    	TBool hasSwfExtension = HasSwfExtension( iNewName );
		    		#ifdef __FEATURE_FLV_RECOG_PLAY
			    	TBool hasFlvExtension = HasFlvExtension( iNewName);
			    	#endif
                    #ifdef __FEATURE_F4V_RECOG_PLAY
                    TBool hasF4vExtension = HasF4vExtension( iNewName);
                    #endif
				    len = iNewName.Length();
				    TBool validName = IsValidName( iNewName );

				    // check if new name is valid name
				    if ( isDir )
				    	{
					    // the new name is a directory name
					    // so make sure it doesn't have the
					    // "swf" extension
	//				    if ( hasSwfExtension )
	//				    	{
	//				    	iNewName.SetLength( len-4 );
	//			    		}
				    	}
				    else
				    	{
					    // the new name is a file name so
					    // make sure it ends in ".swf".
				    	if( !hasSwfExtension
                            #ifdef __FEATURE_FLV_RECOG_PLAY
				    	        && !hasFlvExtension
                            #endif
                            #ifdef __FEATURE_F4V_RECOG_PLAY
                                && !hasF4vExtension
                            #endif
				    	        )
					    	{//by defualt we append swf extnesions as flv extensions are not common
					    	iNewName.AppendFormat( KSwfExtension, &overflow );
					    	}
				    	}
			    	len = iNewName.Length();
				
				    if ( (len == 0 && isDir) ||
					    (len <= 4 && !isDir) ||
					    !validName)
				    	{

					    // invalid file name
					    // Report error
					    HBufC* errorMessage = StringLoader::LoadLC( R_STR_INVALID_FILE_NAME_FORMAT1_MSG,
	                	    iEikonEnv );
	                    FlashUiUtil::ShowNoteL( EAknGlobalWarningNote, *errorMessage );
	                    CleanupStack::PopAndDestroy();    // errorMessage 
	                    getNewName = EFalse;
				    	}
				    else 
				    	{
				        CDesCArray *array = ViewingPhoneDir() ? iPhoneDirItems : iCardDirItems;
	                    TInt index;
	                    TBool isDir = CurrentItemIsDir();

		                if (array && FindListBoxItemL(*array, iNewName, isDir, index) )
		                	{
		                    // folder already exists
			                // Report error
							HBufC* pMessage = StringLoader::LoadLC( R_STR_RENAME_FOLDER_EXISTS_FORMAT1_MSG,
							iNewName, iEikonEnv );
                                     
							CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::EConfirmationTone );
							if ( dlg->ExecuteLD(R_SAFLASH_CONFIRMATION_QUERY, *pMessage ) )
								{
								getNewName = ETrue;
								}
							else
								{
								getNewName = EFalse;
								}
							CleanupStack::PopAndDestroy(1);	// pMessage	
		                	}
		                else
		                	{
		                    // build path to orig item and path for new item
		                    GetCurrentDir( iOrigPath );
			                iNewPath = iOrigPath;
		                TInt err = KErrNone;
		                
		           //   iOrigPath.AppendFormat( iOrigName, &overflow );		                
		           //   iNewPath.AppendFormat( iNewName , &overflow);
      		            TInt newLen = iOrigPath.Length () + iOrigName.Length ();
					    if (newLen > KMaxFileName)
					    {
					    	err = KErrOverflow;
					    }
					    else 
					    {
					    	iOrigPath.Append (iOrigName);	
					    }
					    
      		            newLen = iNewPath.Length () + iNewName.Length ();
					    if (newLen > KMaxFileName)
					    {
					    	err = KErrOverflow;
					    }
					    else 
					    {
					    	iNewPath.Append (iNewName);	
					    }					    

	      		            
	      		            if (overflow.IsOverflowed() == KErrOverflow)
	      		                {
	      		                err =  KErrOverflow;   
	      		                }
      		            else if (KErrNone == err)
	      		                {
	          		            // perform the rename
	        		            err = iEikonEnv->FsSession().Rename(iOrigPath, iNewPath);        		                    
	      		                }
			                if ( err == KErrNone )
			                	{
			                    ReplaceList();
				                UpdateSelectionL(iNewName);

								/* ??Media Gallery not required
	               				CMGXFileManager* mgxFileMgr = MGXFileManagerFactory::NewFileManagerL(iEikonEnv->FsSession());
	            				mgxFileMgr->UpdateL();        				
		                    	delete mgxFileMgr;
		                    	*/
		                    	
				                getNewName = EFalse;
			                	}
			                else if( err == KErrAlreadyExists)
			                	{
			                	HBufC* pMessage = StringLoader::LoadLC( R_STR_RENAME_FOLDER_EXISTS_FORMAT1_MSG,
								iNewName, iEikonEnv );

								CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::EConfirmationTone );
								if ( dlg->ExecuteLD(R_SAFLASH_CONFIRMATION_QUERY, *pMessage ) )
									{
									getNewName = ETrue;
									}
								else
									{
									getNewName = EFalse;
									}
								CleanupStack::PopAndDestroy(1);	// pMessage
			                	
			                	}
			                else if( err == KErrOverflow)
			                    {
			                    // invalid file name
	        				    // Report error
	        				    HBufC* errorMessage = StringLoader::LoadLC( R_STR_INVALID_FILE_NAME_FORMAT1_MSG,
	                        	    iEikonEnv );
	                            FlashUiUtil::ShowNoteL( EAknGlobalWarningNote, *errorMessage );
	                            CleanupStack::PopAndDestroy();    // errorMessage 
	                            getNewName = EFalse;
			                    }
			                else 
	    	                	{
		    	                TInt len = iCurrentItem.Length();
			                    TFileName tempFile = iCurrentItem;
			                    if ( len > 0 && tempFile[len-1] == KCharBackslash)
				                    {	
				                    tempFile.SetLength( len-1 );					
				                    }
		    	                
		    	                // Unable to rename file/dir
				                // Report error
				                HBufC* errorMessage = StringLoader::LoadLC( R_STR_RENAME_FILE_FAIL_FORMAT1_MSG,
	            	                tempFile, iEikonEnv );
	                            FlashUiUtil::ShowNoteL( EAknGlobalWarningNote, *errorMessage );
	                            getNewName = EFalse;
	                            CleanupStack::PopAndDestroy();    // errorMessage 															
			                	}
			                //getNewName = EFalse;
		                	}
				    	}
			    	}
		    	else
			    	{
			       	getNewName = EFalse; 
			    	}
		    	}
		    else
		    	{
		    	getNewName = EFalse; 
		    	}
        	} //while
		}
	}


void CSaFlashFileView::HandleNewFolderCmdL()
{
	CDesCArray *array = ViewingPhoneDir() ? iPhoneDirItems : iCardDirItems;
    TOverflowHandler overflow;
	TBool getNewName = ETrue;
    TInt index;
    if ( array )
        {
        if(iNewFolderName.Length() == 0)
            {
            iNewFolderName.Append( 
                    StringLoader::LoadLC( R_STR_DEFAULT_FOLDER_NAME_MSG, iEikonEnv )->Des() );
            TInt i( 1 );
            while( FindListBoxItemL(*array, iNewFolderName, ETrue, index) )
                {
                CleanupStack::PopAndDestroy();
                if( i < KTwoDigitNeeded ) 
                    {
                    iNewFolderName.Delete( 0, iNewFolderName.Length() );
                    iNewFolderName.Append( 
                           StringLoader::LoadLC( R_STR_DEFAULT_FOLDER_NAME_N_MSG, i, iEikonEnv )->Des() );
                    }
                else
                    {
                    iNewFolderName.Delete( 0, iNewFolderName.Length() );
                    iNewFolderName.Append(
                           StringLoader::LoadLC( R_STR_DEFAULT_FOLDER_NAME_NN_MSG, i, iEikonEnv )->Des() );
                    }
                ++i;
                }
            CleanupStack::PopAndDestroy();
            }
//#ifdef __FLASHLITE2.0_REL3.1
        while( getNewName )
        	{
		    if ( GetNewNameL( iNewFolderName, iNewFolderName, ENewFolder ) )
//#endif //__FLASHLITE2.0_REL3.1		
		    	{
			// check if valid name for folder
//#ifdef __FLASHLITE2.0_REL3.1
			    if ( iNewFolderName.Length() == 0 ||
		//		    HasSwfExtension( iNewFolderName ) ||
				    !IsValidName( iNewFolderName ) )
			    	{
				    // invalid name
				    // report error
				    FlashUiUtil::ShowNoteWithoutLeaving(EAknGlobalWarningNote, 
														iNewFolderName,
														R_STR_INVALID_FOLDER_NAME_FORMAT1_MSG,
														iEikonEnv,
														&iLowMemDialog);
			        getNewName = EFalse;														
			    	}
			   	else 
			    	{
				    
				    // create the folder
					//
					GetCurrentDir( iNewDir );
//#ifdef __FLASHLITE2.0_REL3.1
						TInt err = KErrNone;
				//		iNewDir.AppendFormat( iNewFolderName, &overflow);
					    TInt newLen = iNewDir.Length () + iNewFolderName.Length ();
					    if (newLen > KMaxFileName)
					    {
					    	err = KErrOverflow;
					    }
					    else 
					    {
					    	iNewDir.Append (iNewFolderName);	
					    }					    
				
					iNewDir.AppendFormat( KDirSeparator, &overflow );
					    
					if (overflow.IsOverflowed() == KErrOverflow)
						{
						err = KErrOverflow;  
						}
					    else if (KErrNone == err)
						{
						err = iEikonEnv->FsSession().MkDir( iNewDir);
						}
					if ( KErrNone == err )
					    {
					    ReplaceList();
					    UpdateSelectionL(iNewFolderName);
						getNewName = EFalse;    
//#endif //__FLASHLITE2.0_REL3.1
					    }
					else if ( KErrAlreadyExists == err )
					    {
						HBufC* errorMessage = StringLoader::LoadLC( R_STR_NEW_FOLDER_EXISTS_FORMAT1_MSG,
						iNewFolderName, iEikonEnv );
						FlashUiUtil::ShowNoteL( EAknGlobalInformationNote, *errorMessage );
						CleanupStack::PopAndDestroy();    // errorMessage 
						getNewName = ETrue;					    	
					    }
					else if (KErrOverflow == err)
						{
						// invalid name
						// report error
						FlashUiUtil::ShowNoteWithoutLeaving(EAknGlobalWarningNote, 
															iNewFolderName,
															R_STR_INVALID_FOLDER_NAME_FORMAT1_MSG,
															iEikonEnv,
															&iLowMemDialog);
						getNewName = EFalse;														  
						}
					    else
						    {
					    	NotifyErrorL( EAknGlobalInformationNote, KNullDesC, err );
					    	getNewName = EFalse;
						    }					    
				    //}
				   	}				   	
		    	}
		    else
		    	{
		        getNewName = EFalse;
		    	}
        	}//while
    	iNewFolderName = KNullString;
		}
}
//#endif //__FLASHLITE2.0_REL3.1

void CSaFlashFileView::HandleMoveCmdL()
{
    TOverflowHandler overflow;
	if (CurrentItemIsValid())
	{
		GetCurrentDir(iOldDir);
		GetSelectedPath(iOrigPath);
		
		TBool getNewName = ETrue;
		TFileName srcDirName = BaflUtils::FolderNameFromFullName(iOrigPath);
		while ( getNewName )
		{
			TBool moveItem = ETrue;
			//if (CSaFlashDirList::GetMoveDirL(iNewDir))
			if(AknCommonDialogs::RunMoveDlgLD(
    								iNewDir, 
                                    R_SAFLASH_MEMORY_SELECTIONDIALOG ))
			{
				//iNewDir.AppendFormat(KDirSeparator, &overflow);
				iNewPath.Copy(iNewDir);
	//			iNewPath.AppendFormat(iCurrentItem, &overflow);
								
				TInt err = KErrNone;
				TInt newLen = iNewPath.Length () + iCurrentItem.Length ();
				if (newLen > KMaxFileName)
				{
				   	err = KErrOverflow;
				}
				else 
				{
				   	iNewPath.Append (iCurrentItem);	
				}
				
				
				if (KErrNone != err || (overflow.IsOverflowed() == KErrOverflow) || (iNewPath.Compare(iOrigPath) == 0))
					{
					// Failed to move file
					// report error
					TBool isFolder;					
					BaflUtils::IsFolder(CCoeEnv::Static()->FsSession(), iOrigPath, isFolder);
					HBufC* errorMessage;
					
					if (isFolder)
					{
						errorMessage = StringLoader::LoadLC (R_STR_MOVE_FAIL_FORMAT2_MSG,
														BaflUtils::FolderNameFromFullName(iOrigPath),iEikonEnv);
					}
					else
					{
						errorMessage = StringLoader::LoadLC (R_STR_MOVE_FAIL_FORMAT1_MSG,iCurrentItem,iEikonEnv);
					}
	            	FlashUiUtil::ShowNoteL( EAknGlobalWarningNote, *errorMessage );
	            	CleanupStack::PopAndDestroy();    // errorMessage 	
						
					return;
					}
				
				TBuf<2 * KMaxFileName> tempCmpTarget(iNewDir);
				tempCmpTarget.Append(srcDirName);
            	tempCmpTarget.Append(KDirSeparator); 
            	
            	if ( iOrigPath[iOrigPath.Length()-1] == '\\')
	            	{
	            	if ( ( iOrigPath.Length() < tempCmpTarget.Length() &&
				         0 == iOrigPath.Compare(tempCmpTarget.Left(iOrigPath.Length())) ) )
						{
						TInt err = KErrGeneral;				    	
						TBool isFolder;					
						BaflUtils::IsFolder(CCoeEnv::Static()->FsSession(), iOrigPath, isFolder);
						HBufC* errorMessage;
						if (isFolder)
						{
							errorMessage = StringLoader::LoadLC (R_STR_MOVE_FAIL_FORMAT2_MSG,
													BaflUtils::FolderNameFromFullName(iOrigPath), iEikonEnv );	
						}
						else
						{
							errorMessage = StringLoader::LoadLC (R_STR_MOVE_FAIL_FORMAT1_MSG,
													BaflUtils::FolderNameFromFullName(iOrigPath), iEikonEnv );
						}					
						NotifyErrorL( EAknGlobalInformationNote, *errorMessage, err );
						CleanupStack::PopAndDestroy();    // errorMessage		    				
						return;
						}
	            	}
				TBool found = FindItemExistsL();
				
				if ( found )
	            {
	             	// folder already exists
		            // Report error
		            TInt len = iCurrentItem.Length();
		            TFileName tempFile = iCurrentItem;
		            if ( len > 0 && tempFile[len-1] == KCharBackslash)
			            {	
			            tempFile.SetLength( len-1 );					
			            }
		            HBufC* pMessage = StringLoader::LoadLC( R_FLDR_OVERWRITE_QUERY,
                    	tempFile, iEikonEnv );
                                     
		            CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::EConfirmationTone );
		            if ( dlg->ExecuteLD(R_SAFLASH_CONFIRMATION_QUERY, *pMessage ) )
		            {
		            	getNewName = EFalse;
		            	moveItem = ETrue;
		            }
		            else
		            {
		            	//Ask for new name.
		            	TBool nameExists = ETrue;
		            	while(nameExists)
			            	{
			            	TInt found;
			            	if ( GetNewNameL( iOrigName, iNewName, ERenameFileOrFolder))
			            		{
								TBool validName = IsValidName( iNewName );

								if (!validName)
									{
									// Invalid file name
									// Report error
									HBufC* errorMessage = StringLoader::LoadLC( R_STR_INVALID_FILE_NAME_FORMAT1_MSG,
									iEikonEnv );
									FlashUiUtil::ShowNoteL( EAknGlobalWarningNote, *errorMessage );
									CleanupStack::PopAndDestroy();    // errorMessage
									CleanupStack::PopAndDestroy();    // pMessage
									return;
									}

			            		if (CurrentItemIsDir())
									{
									iNewName.Append(KCharBackslash);
									}
								else
									{
									TBool hasSwfExtension = HasSwfExtension( iNewName );
									#ifdef __FEATURE_FLV_RECOG_PLAY
									TBool hasFlvExtension = HasFlvExtension( iNewName );
									#endif
                                    #ifdef __FEATURE_F4V_RECOG_PLAY
									TBool hasF4vExtension = HasF4vExtension( iNewName );
                                    #endif
		                       if( !hasSwfExtension
		                            #ifdef __FEATURE_FLV_RECOG_PLAY
		                                && !hasFlvExtension
		                            #endif
		                            #ifdef __FEATURE_F4V_RECOG_PLAY
		                                && !hasF4vExtension
		                            #endif
		                                )
										{//by defualt we append swf extension
				    					iNewName.Append( KSwfExtension);				    					
				    					}	
									}
									
								iNewPath.Copy(iNewDir);
								iNewPath.Append(iNewName);								
								
								iCurrentItem = iNewName;
				            	// check whether item exists or not.
				            	found = FindItemExistsL();
				            	//TInt attVal = 0;
				            	//found = iEikonEnv->FsSession().Att(iNewPath,attVal);
				            	if (found)
					            	{
					            	// Item exists, ask for Rename.
					            	TInt len = 0;
									iNewName = iCurrentItem;
									len = iNewName.Length();
									if ( len > 0 && iNewName[len-1] == KCharBackslash)
										{
										iNewName.SetLength( len-1 );
										}

									HBufC* pMsg = StringLoader::LoadLC( R_STR_RENAME_FOLDER_EXISTS_FORMAT1_MSG,
									iNewName, iEikonEnv );

									CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::EConfirmationTone );
									if ( dlg->ExecuteLD(R_SAFLASH_CONFIRMATION_QUERY, *pMsg ) )
										{
										iOrigName = iNewName;
										nameExists = ETrue;
										}
									else
										{
										nameExists = EFalse;
					            		getNewName = EFalse;
					            		moveItem = EFalse;	
										}
									CleanupStack::PopAndDestroy(); // pMsg
					            	}
					            else
					            	{
									//Name entered doesn't exist, rename it.
					            	nameExists = EFalse;
					            	getNewName = EFalse;
			            			moveItem = ETrue;
					            	}
				            	}
				            else
				            	{
				            	// Don't want to move with new name.Discard whole transaction.
				            	nameExists = EFalse;
					            getNewName = EFalse;
			            		moveItem = EFalse;	
				            	}			            	
			            	}
		            }
		            CleanupStack::PopAndDestroy(1);	// pMessage
	            }
				
				// If item is to be moved.
				if ( moveItem )
				{
					getNewName = EFalse;
					
					TInt err(0);
					CFileMan* fm = CFileMan::NewL(iEikonEnv->FsSession());
					
					_LIT(KNULL, "");
		            TInt numOfSubDirs = FlashUiUtil::CheckSubdirCountL(KNULL, iNewPath, ETrue, EFalse);
					TInt numOfFiles = FlashUiUtil::CheckSubdirCountL(KNULL, iNewPath, EFalse, EFalse);
					
					//If the folder already exists in the destination, delete it.
					TBool isDir = CurrentItemIsDir();	
					if ( isDir )
					{
						if ( (numOfSubDirs != KZero) || (numOfFiles!= KZero) )
						{
							err = fm->RmDir( iNewPath );
						}
						else
						{
							err = iEikonEnv->FsSession().RmDir( iNewPath );
						}
					}
					
					if (iOrigPath[0] != iNewPath[0])
					{ 
					    // files are not on the same drive, so use CFileMan, since it can move files across drives.
						TInt len = iNewPath.Length();
						_LIT(KNULL, "");
		            	TInt numOfSubDirs = FlashUiUtil::CheckSubdirCountL(KNULL, iOrigPath, ETrue, EFalse);
						TInt numOfFiles = FlashUiUtil::CheckSubdirCountL(KNULL, iOrigPath, EFalse, EFalse);
						
						if (iCurrentItem[iCurrentItem.Length()-1] == KCharBackslash)
						{ 
						    // if dest is a folder, make the dir first, since CFileMan::Move() does not
						    // create the dest folder.
							
							if((numOfSubDirs)||(numOfFiles))
								{
								//TInt lenCurrentItem = iCurrentItem.Length();
								//iNewPath.Delete(len-lenCurrentItem, lenCurrentItem);
								
								//Keep track of right most Folder from iOrigPath & iNewPath.
								TFileName  origFolder = BaflUtils::FolderNameFromFullName(iOrigPath);
								TFileName  newFolder = BaflUtils::FolderNameFromFullName(iNewPath);
								
								origFolder.Append(KDirSeparator);
								newFolder.Append(KDirSeparator);
								
								TFileName origPath(iOrigPath);
								
								//The absence of a trailing backslash ("\") character on the end of the 
								//source path:For example fm->Move(_L("C:\a\b"), _L("C:\x\y\"), CFileMan::ERecurse);
								//there is no longer a directory "b" under C:\\a. Instead there is a new directory structure 
								//C:\\x\\y\\b.Also if "b" contains subdirectories, then these are also moved along with "b". 
								//That's why last 2 characters ("\\") are deleted.
								origPath.Delete(origPath.Length() - 1,2);
								
								err = fm->Move(origPath,iNewDir, CFileMan::ERecurse);
								if (err == KErrAlreadyExists)
									{
									//1. Rename OrigPath using random number.
									//2. Move item to destination path.
									//3. Rename destination path, to actual path name.
									
									TBool pathExists = ETrue;
									TFileName renamedPath(origPath);
									TInt8 random;									
									while (pathExists)
										{
										random = Math::Random();
										renamedPath.AppendNum(random);
										
										TFileName renamedFolder(renamedPath);
										
										renamedFolder.Append(KDirSeparator);
										renamedFolder = BaflUtils::FolderNameFromFullName(renamedFolder);										
										renamedFolder.Append(KDirSeparator);
										
										err = fm->Rename(origPath,renamedPath);
										if (err == KErrAlreadyExists || err != KErrNone )
											{
											pathExists = ETrue;	
											}
										else
											{
											err = fm->Move(renamedPath,iNewDir, CFileMan::ERecurse);
											iNewDir.Append(renamedFolder);
											err = fm->Rename(iNewDir, iNewPath);
											err = fm->RmDir(iNewDir);
											pathExists = EFalse;
											}
										}
									}
								}
							else
							{
								//empty dirs
								err = iEikonEnv->FsSession().MkDir( iNewPath);
								err = iEikonEnv->FsSession().RmDir( iOrigPath);
							}
						}
						else
						{
							//iNewPath.Append(iCurrentItem);
							err = fm->Move(iOrigPath, iNewPath, CFileMan::EOverWrite);	
						}
						
					}
					else
					{
						//Move files or folders on the same drive.
										
						TBool isDir = CurrentItemIsDir();
						
						_LIT(KNULL, "");
		            	TInt numOfSubDirs = FlashUiUtil::CheckSubdirCountL(KNULL, iNewPath, ETrue, EFalse);
						TInt numOfFiles = FlashUiUtil::CheckSubdirCountL(KNULL, iNewPath, EFalse, EFalse);
						
						//If the file already exists in the destination, delete it.
						if (!isDir)
						{
							err = iEikonEnv->FsSession().Delete( iNewPath );
						}
						
						err = iEikonEnv->FsSession().Rename(iOrigPath, iNewPath);
						/*
						Rename of folder will not happen if any of item inside the folder is in use.
						so we need to call move for that case.Two types of move operation available CFileman::move().If we remove 
						the trailing backslash in the foldername CFileman::move will move the Entire folder itself,if we don't remove trailing backslash
						CFileman::move will move the contents only. 
						
						*/
						if(BaflUtils::PathExists(CCoeEnv::Static()->FsSession(),iOrigPath)) //If some files in use
						{
							TInt error = fm->Move(iOrigPath,iNewPath, CFileMan::ERecurse);
							fm->RmDir(iOrigPath);
								
						}
					}
					
					if ( err == KErrNone )
					{
						ReplaceList();
					}
					else if (err == KErrInUse )
						{
						// Failed to move file
						// report error
						HBufC* errorMessage;
						if (isDir)
							{
							TInt numOfFiles = FlashUiUtil::CheckSubdirCountL(KNULL, iOrigPath, EFalse, EFalse);
							TInt numOfSubDirs = FlashUiUtil::CheckSubdirCountL(KNULL, iOrigPath, ETrue, EFalse);

							errorMessage = StringLoader::LoadLC( R_STR_FLDR_MOVE_FAIL_FORMAT1_MSG,
							(numOfFiles + numOfSubDirs), iEikonEnv );	
							ReplaceList();
							}
						else
							{
							//But for file, it isn't required.
							errorMessage = StringLoader::LoadLC( R_STR_MOVE_FAIL_FORMAT1_MSG,
							iCurrentItem, iEikonEnv );
							}
		            	FlashUiUtil::ShowNoteL( EAknGlobalWarningNote, *errorMessage );
		            	CleanupStack::PopAndDestroy();    // errorMessage
						}
					delete fm;	
				}
			}
			else
			{
				getNewName = EFalse;
			}	
		} //while
	}
}

void CSaFlashFileView::HandlePasteCmdL()
{
    TOverflowHandler overflow;
	if ( iIsSavedPathValid )
	{
		// set this flag to false
		iIsSavedPathValid = EFalse;

		// validate object still exists
		// ????

		// Get the dirt to paste the file/dir into
		GetCurrentDir( iNewPath );

		// get start position of last item in path
		TInt len = iSavedPath.Length();
		TInt index = len-1;
		TInt isDir = ( iSavedPath[index] == KCharBackslash ) ? 1 : 0;
		if (isDir)
		{
			 --index;
		}
		while( index >= 0 && iSavedPath[index] != KCharBackslash )
		{
			--index;
		}
		if ( iSavedPath[index] == KCharBackslash )
		{
			++index;
		}

		// append name to new dir
		if (len-index + iNewPath.Length() <= KMaxFileName)
		    {
		iNewPath.Append( iSavedPath.Ptr() + index, len-index);
		if ( iSavedPath != iNewPath )
		{
			// move the object
			TInt err = iEikonEnv->FsSession().Rename(iSavedPath, iNewPath);
			if ( err == KErrNone )
			{
				ReplaceList();
			}
			else
			{
				// Failed to move file
				// report error
				FlashUiUtil::ShowNoteWithoutLeaving(EAknGlobalWarningNote, 
														iSavedPath,
														R_STR_MOVE_FAIL_FORMAT1_MSG,
														iEikonEnv,
														&iLowMemDialog);
			}
		}
		    }
        else
            {
			// Failed to move file
				// report error
			FlashUiUtil::ShowNoteWithoutLeaving(EAknGlobalWarningNote, 
													iSavedPath,
													R_STR_MOVE_FAIL_FORMAT1_MSG,
													iEikonEnv,
													&iLowMemDialog);               
            }
	}
}

TInt CSaFlashFileView::AddItemToListL( const TDesC& aNewItem, CDesCArray& aArray )
{
	aArray.AppendL( aNewItem );
	aArray.Sort();
	TInt len = aArray.Count();
	TInt currentItem;
	for( currentItem = 0; currentItem < len; ++currentItem )
	{
		if ( aArray[currentItem] == aNewItem )
			break;
	}

	return currentItem;
}

void CSaFlashFileView::UpdateSelectionL( const TDesC& aItemName )
{
	CDesCArray **dirItems = ViewingPhoneDir() ? &iPhoneDirItems : &iCardDirItems;
	ClearCurrentItem();
	TInt index;
	if ( *dirItems )
	{
		TInt count =  (*dirItems)->Count();
		for (index = 0; index < count; index++)
		{
			TPtrC item = (**dirItems)[index];
			TInt len = item.Length() - 2;
			TPtrC right = item.Right(len);
			TInt tab = right.Locate(0x0009);
			if (tab != KErrNotFound)
			{
				len -= (len - tab);
			}
			TPtrC dir = right.Left(len);
			if (dir.Compare(aItemName) == 0)
			{
				ExtractListBoxString( item, iCurrentItem );
				if ( item[0] == KDirIconId || item[0] == KDirSubIconId )
				{
					iCurrentItem.Append( KDirSeparator );
					iCurrentItemIsDir = ETrue;
				}
			if (ViewingPhoneDir())
				{
				iCurrentItemIndex = index;
				}
			else
				{
				iCurrentMMCItemIndex = index;	
				}
				iContainer->SetCurrentItemL(index);
			}
		}
	}
}

void CSaFlashFileView::UpdateSelectionL( TInt aIndex )
{
	iContainer->SetCurrentItemL(aIndex);
	CDesCArray **dirItems = ViewingPhoneDir() ? &iPhoneDirItems : &iCardDirItems;
	ClearCurrentItem();
	if ( *dirItems )
	{
		if ( aIndex < (*dirItems)->Count() )
		{
			TPtrC item = (**dirItems)[aIndex];
			ExtractListBoxString( item, iCurrentItem );
			if ( item[0] == KDirIconId || item[0] == KDirSubIconId )
			{
				iCurrentItem.Append( KDirSeparator );
				iCurrentItemIsDir = ETrue;
			}
			if (ViewingPhoneDir())
				{
			iCurrentItemIndex = aIndex;
				}
			else
				{
				iCurrentMMCItemIndex = aIndex;	
				}
		}
	}
}

void CSaFlashFileView::GetCurrentDir( TDes& aDir )
{
	if ( ViewingPhoneDir() )
	{
		aDir = iPhonePath;
	}
	else
	{
		aDir = iCardPath;
	}
}

void CSaFlashFileView::GetSelectedPath( TDes&  aPath )
{
	GetCurrentDir( aPath );
	aPath.Append( iCurrentItem );
}


// ---------------------------------------------------------
// CSaFlashFileView::TabChangedL(TInt aIndex)
// ---------------------------------------------------------
void CSaFlashFileView::TabChangedL(TInt aIndex)
    {
    //If Pen is enabled than only handle otherwise it is handled by file control.
    if(FeatureManager::FeatureSupported(KFeatureIdPenSupport)) //adding feature manager to check the feature at run time.
    	{
	    if (AknLayoutUtils::PenEnabled())
			{
	    	SetTabL(aIndex);
			}
    	}
    //SetNavigationPaneL();
    }

void CSaFlashFileView::SetNavigationPaneL()
{
	if ( !iNaviPane )
	{
		iNaviPane = STATIC_CAST(CAknNavigationControlContainer*, StatusPane()->ControlL( TUid::Uid( EEikStatusPaneUidNavi ) ) );
	}

	if (!iDecoratedTabGroup)
	{
		TResourceReader reader;
		iCoeEnv->CreateResourceReaderLC( reader, R_SAFLASH_FILEVIEW_TAB );
		iDecoratedTabGroup = iNaviPane->CreateTabGroupL( reader, this);
		CleanupStack::PopAndDestroy();  // resource reader

		iTabGroup = STATIC_CAST( CAknTabGroup*, iDecoratedTabGroup->DecoratedControl() );
		
		iTabGroup->SetActiveTabByIndex( iCurrentTab );
	}

	iNaviPane->PushL( *iDecoratedTabGroup );
}

void CSaFlashFileView::SetTitlePaneTextL(const TDesC& aText)
{
    // Fetches pointer to the default title pane control.
    CAknTitlePane* titlePane = STATIC_CAST(CAknTitlePane*, StatusPane()->ControlL( TUid::Uid(EEikStatusPaneUidTitle)));
	if (!aText.Length())
	{
		titlePane->SetTextToDefaultL();
	}
	else
	{
		titlePane->SetTextL(aText);
	}
}


void CSaFlashFileView::SetShowDirsOnly( TBool aState )
{
	iShowDirsOnly = aState;
}

TPtrC CSaFlashFileView::StripFinalDot( const TDesC& aName )
        {
        _LIT( KDot, "." );
        TInt len( aName.Length() );
        if( len && aName[ len - 1 ] == KDot()[0] )
            {
            return aName.Left( len - 1 );
            }
        return TPtrC( aName );
        }

TBool CSaFlashFileView::IsValidName(const TDesC& aName)
	{
	// Check name for bad chars
	const TUint16 KMinAllowedChar = 0x0020;
	const TUint16 KParagraphSeparator = 0x2029;
	const TUint16 KDot = '.';
	TInt nameLen( aName.Length() );
	if ( !nameLen )
		{
		return EFalse;
		}
		
	for ( TInt i( 0 ); i < nameLen; i++ )
		{
		TUint16 ch( aName[ i ] );
		if ( ch < KMinAllowedChar || ch == KParagraphSeparator )
			{
			return EFalse;
			}
		}
		
	// File system ignores totally dot in the end of name, so 
	// we set here as not valid name, so that user gets correctly informed
	if ( aName[ nameLen - 1 ] == KDot )
		{
		return EFalse;
		}
	TInt ret = EFalse;
	ret = iEikonEnv->FsSession().IsValidName( aName );
	return ret;	 
	}


CDir* CSaFlashFileView::GetFileDirSearchL(const TDesC& aDirPath
										,const TDesC& aSearchStr
										,TUint aSearchFilter)
{

	TInt searchBufLen = aSearchStr.Length();
	searchBufLen += aDirPath.Length() + 1;	

	CDir* cdir_files = 0;

		// create search string
	HBufC* searchBuf = HBufC::NewL( searchBufLen );
	CleanupStack::PushL( searchBuf );
	{
		TPtr searchStr = searchBuf->Des();

		// Set directory search string
		searchStr.Copy( aDirPath );
		searchStr.Append( aSearchStr );

		// create a list of directory entries ( swf file names )
		// create a object that contains the found dirs
		TInt err = iEikonEnv->FsSession().GetDir(searchStr, 
								 	  aSearchFilter
									, ESortByName | EAscending
									, cdir_files);
		
		if (err == KErrNoMemory)
		{
			User::Leave(err);
		}
	}
	
	CleanupStack::PopAndDestroy(searchBuf);	
	
	return cdir_files;
}



TBool CSaFlashFileView::BuildItemsL(CDir* aDir_Item
								 ,CDesCArrayFlat*& aItems
								 ,TBool aIsDir
								 ,TBool aIsSwfExtn )
{
	TBool buildStat = EFalse;
	if(aDir_Item && aItems)
	{
		HBufC* pBufC = HBufC::NewLC( KMaxFileName );
		{
			TPtr ptr = pBufC->Des();
			TEntry entry;
			TInt	i, n=0;
			TBool stat=EFalse;
			
			n = aDir_Item->Count();
			TInt bldListErr = KErrNone;
			for ( i = 0; i < n; ++i )
			{
				entry = (*aDir_Item)[i];
				bldListErr = KErrNone;
				stat=EFalse;
				TRAP (bldListErr, stat = BuildListBoxStringL( entry
												, ptr
												, aIsDir
												,aIsSwfExtn ));
			
				if (KErrNone != bldListErr)
				{
					continue;
				}
			
				if(stat)
				{
					buildStat = ETrue;	//ok added item..
					aItems->AppendL( ptr );	
				}
			
			}//for 
		}
		CleanupStack::PopAndDestroy(pBufC);
	}
					
	return 	buildStat;			
}


CDesCArray* CSaFlashFileView::CreateListBoxStringsForDirL( const TDesC& aDirPath )
{
	CDesCArrayFlat* items = 0;
	
	CDir* cdir_dirs = GetFileDirSearchL(aDirPath
									,KDirSearchStr
									,KEntryAttDir | KEntryAttMatchExclusive);
	CDir* cdir_Swf=NULL;
	CDir* cdir_dcf=NULL;
	CDir* cdir_odf=NULL;
	
	#ifdef __FEATURE_FLV_RECOG_PLAY
	CDir* cdir_flv=NULL;	
	#endif
    #ifdef __FEATURE_F4V_RECOG_PLAY
    CDir* cdir_f4v=NULL;    
    #endif
	
	TInt count = 0;
	
	if(cdir_dirs)	
	{	//start counting dirs
		count += cdir_dirs->Count();
		CleanupStack::PushL( cdir_dirs );
	}
									
	if ( ! iShowDirsOnly )
	{
		//start counting .swf
		cdir_Swf = GetFileDirSearchL(aDirPath
									,KSwfSearchStr //*.swf
									,KEntryAttNormal);
		//start counting .dcf
		cdir_dcf = GetFileDirSearchL(aDirPath
									,KDcfSearchStr //*.dcf
									,KEntryAttNormal);
		//start counting .odf
		cdir_odf = GetFileDirSearchL(aDirPath
									,KOdfSearchStr //*.odf
									,KEntryAttNormal);
		
		#ifdef __FEATURE_FLV_RECOG_PLAY
		cdir_flv = GetFileDirSearchL(aDirPath
									,KFlvSearchStr //*.flv
									,KEntryAttNormal);
		#endif
		#ifdef __FEATURE_F4V_RECOG_PLAY
        cdir_f4v = GetFileDirSearchL(aDirPath
                                    ,KF4vSearchStr //*.f4v
                                    ,KEntryAttNormal);
        #endif
		
									
		if(cdir_Swf)	
		{
			count += cdir_Swf->Count();
			CleanupStack::PushL( cdir_Swf );
		}
		
		if(cdir_dcf)	
		{
			count += cdir_dcf->Count();
			CleanupStack::PushL( cdir_dcf );
		}
		
		if(cdir_odf)	
		{
			count += cdir_odf->Count();
			CleanupStack::PushL( cdir_odf );
		}
		
		#ifdef __FEATURE_FLV_RECOG_PLAY
		if(cdir_flv)	
		{
			count += cdir_flv->Count();
			CleanupStack::PushL( cdir_flv );
		}
		#endif
        #ifdef __FEATURE_F4V_RECOG_PLAY
        if(cdir_f4v)    
        {
            count += cdir_f4v->Count();
            CleanupStack::PushL( cdir_f4v );
		}
		#endif
	}
	
	TBool anItemExists = EFalse;

	if(count > 0)
	{	//build items now
		items = new (ELeave) CDesCArrayFlat(count);
		CleanupStack::PushL(items);
		{
			if(cdir_dirs)
			{ //build directories first
				if(BuildItemsL(cdir_dirs,items,ETrue,EFalse))
				{
					anItemExists = ETrue;
				}
			}
			
			if ( ! iShowDirsOnly )
			{	//start building .swf
				if(cdir_Swf)
				{
					if(BuildItemsL(cdir_Swf,items,EFalse,ETrue))
						anItemExists = ETrue;
				}
				//start building .dcf
				if(cdir_dcf)
				{
					if(BuildItemsL(cdir_dcf,items,EFalse,EFalse))
						anItemExists = ETrue;
				}
				//start building .odf
				if(cdir_odf)
				{
					if(BuildItemsL(cdir_odf,items,EFalse,EFalse))
						anItemExists = ETrue;
				}
				
				#ifdef __FEATURE_FLV_RECOG_PLAY
				//start building .flv
				if(cdir_flv)
				{
					if(BuildItemsL(cdir_flv,items,EFalse,EFalse))
						anItemExists = ETrue;
				}
				
				#endif
                #ifdef __FEATURE_F4V_RECOG_PLAY
                //start building .f4v
                if(cdir_f4v)
                {
                    if(BuildItemsL(cdir_f4v,items,EFalse,EFalse))
                        anItemExists = ETrue;
                }
                
                #endif
			}
		}
		
		// remove items pointers from cleanup stack
		CleanupStack::Pop(items);
		//MSK related change .. to show MSK or no
		if (anItemExists)
		{
			if(!Cba()->IsCommandVisible(EEikFlashCmdOpen))
			{
			//Special case when there are no items in a view and if, a new folder is created.
			Cba()->MakeCommandVisible(EEikFlashCmdOpen,ETrue);	
			}
		}
		else
		{ 	//no items found
			if(Cba()->IsCommandVisible(EEikFlashCmdOpen))
			{
			Cba()->MakeCommandVisible(EEikFlashCmdOpen,EFalse);
			}
			
			//remove allocated memory, possibly 
			//this can happen no swf but dcf, odf present 
			//but they are not swfs
			delete items;
			//make one item
			items = new (ELeave) CDesCArrayFlat(1);	
		}		
	}
	else
	{ //no files found
		items = new (ELeave) CDesCArrayFlat(1);	
		//MSK related change .. to show MSK or no
		Cba()->MakeCommandVisible(EEikFlashCmdOpen,EFalse);
	}
	
	//cleanup now	
	if ( ! iShowDirsOnly )
	{
		#ifdef __FEATURE_FLV_RECOG_PLAY
		if ( cdir_flv )
			CleanupStack::PopAndDestroy(cdir_flv);
		#endif
        #ifdef __FEATURE_F4V_RECOG_PLAY
        if ( cdir_f4v )
            CleanupStack::PopAndDestroy(cdir_f4v);
		#endif
		
		if ( cdir_odf )
			CleanupStack::PopAndDestroy(cdir_odf);
	
		if ( cdir_dcf )
			CleanupStack::PopAndDestroy(cdir_dcf);
	
		if ( cdir_Swf )
			CleanupStack::PopAndDestroy(cdir_Swf);
	}
	
	//cleanup cdirs now
	if ( cdir_dirs )
		CleanupStack::PopAndDestroy(cdir_dirs);	
	
	return items;
	
}

//localize the directory string
TBool CSaFlashFileView::DoLocalizeDirL(const TEntry& aEntry
									, const TDesC& aBasePath
									, TDes& aItem)
{
	TBool isLocalized = EFalse;
	HBufC* fullPath = HBufC::NewLC( aBasePath.Length() + aEntry.iName.Length() + 1 );
	                                
    TPtr temp = fullPath->Des();
                            	
    temp.Append(aBasePath);
    temp.Append(aEntry.iName);
    temp.Append(KCharBackslash);
       	
    iLocalizer->SetFullPath( temp );
						
	isLocalized = iLocalizer->IsLocalized();	
	TPtrC localizedPtr;
	if ( isLocalized )
	{
		localizedPtr.Set(StripFinalDot(iLocalizer->LocalizedName()))  ;
		aItem.Append(localizedPtr);
	}	
	
	CleanupStack::PopAndDestroy(fullPath);
	
	return isLocalized;
}

TBool CSaFlashFileView::BuildListBoxStringforDirL(const TEntry& aEntry,
								TDes& aItem)
{
	//#ifdef __FLASHLITE2.0_REL3.1
	GetCurrentDir(iOldDir);

    TInt subDirCount = FlashUiUtil::CheckSubdirCountL( iOldDir, aEntry.iName, ETrue, EFalse);
        
	if( subDirCount > 0 )
       {
        	aItem.Copy( KDirSubItemPrefix );	
       }
    else
       {
       	aItem.Copy( KDirItemPrefix );
       }
//#endif //__FLASHLITE2.0_REL3.1
	aItem.Append(KTabChar);
		
		// To fix the bug IPOP-6V6G8J.
	TBool isLocalized = EFalse;
if ( !ViewingPhoneDir() )
	{
		isLocalized = DoLocalizeDirL(aEntry,iCardPath,aItem);
	}
	
	else
	
	   isLocalized = DoLocalizeDirL(aEntry,iPhonePath,aItem);
	if ( isLocalized == EFalse)
	{
	aItem.Append( aEntry.iName );	
	}
		
	TInt subdirs = FlashUiUtil::CheckSubdirCountL(iOldDir, aEntry.iName, EFalse, ETrue);
	aItem.Append(KTabChar);
//#ifdef __FLASHLITE2.0_REL3.1
	TBuf<KBufSize> fileCountBuffer;
	if ( subdirs == KFileCountOne)
	{
		CCoeEnv::Static()->ReadResourceL(fileCountBuffer,
       		R_STR_VIEW_COUNT_FILE_ONE);
		aItem.Append(fileCountBuffer);	
	}
	else
	{
		HBufC* numFiles = StringLoader::LoadLC( R_STR_VIEW_COUNT_FILES, subdirs, iEikonEnv );	
			TPtr itemPointer = numFiles->Des();
			AknTextUtils::LanguageSpecificNumberConversion(itemPointer);
		if (numFiles)
            {
	            aItem.Append(*numFiles);
            }
        CleanupStack::PopAndDestroy( numFiles );
	}
//#endif //__FLASHLITE2.0_REL3.1
	aItem.AppendFill(KTabChar, 2);
	
	return ETrue;
}

TBool CSaFlashFileView::BuildListBoxStringforSwfL(const TEntry& aEntry,
												TDes& aItem,
												TBool aSwfExtn	)
{
	if(!aSwfExtn) //not a swf extension ,then verify MIME type etc..
    {
	    GetCurrentDir(iOldDir);

    	iOldDir.Append(aEntry.iName);

    	TBool isProtected=EFalse;

    	CContent *content = CContent::NewLC( iOldDir, EContentShareReadWrite ); 
    	content->GetAttribute( EIsProtected, isProtected );
    	
    	HBufC* tmpMIME = HBufC::NewLC( KMaxDataTypeLength );
    	
    	TPtr mimePtr( tmpMIME->Des() );  
    	          
		content->GetStringAttribute(EMimeType, mimePtr );
		
    	TBool isFlash =  (0 != mimePtr.Compare(KFlashMimeType))? EFalse : ETrue ;
    	
    	TBool isVideo = EFalse;
    	#ifdef __FEATURE_FLV_RECOG_PLAY
    	
    	isVideo = (0 != mimePtr.Compare(KFLVMimeType))? EFalse : ETrue ;
    	
    	#endif
    	
        #ifdef __FEATURE_F4V_RECOG_PLAY
        
    	isVideo = (0 != mimePtr.Compare(KF4VMimeType))? EFalse : ETrue ;
        
        #endif
    	CleanupStack::PopAndDestroy(tmpMIME);

	    if( !isFlash && !isVideo )
		{
			CleanupStack::PopAndDestroy( content );
			return EFalse;//not swf or video file return error
		}
		
    	if (isProtected)
		{
			TBool hasRights = ((CSaFlashAppUi*)AppUi())->CheckPlayRightsL(content);
			
			if(hasRights)
			{
				aItem.Copy(KSwfRightsValidPrefix);	
			}
			else
			{
				aItem.Copy(KSwfItemRightsExpPrefix);
			}
			
		}
    	else // Not protected so display normal icon
		{
			aItem.Copy( KSwfItemPrefix );
		}
		
		CleanupStack::PopAndDestroy( content );
    }
   else // Not protected so display normal icon
   {
		aItem.Copy( KSwfItemPrefix );
   }
//#endif //__FLASHLITE2.0_REL3.1	
	aItem.Append(KTabChar);
	aItem.Append(aEntry.iName);
	aItem.Append(KTabChar);
	TInt sizeInKB = (aEntry.iSize + 512) / 1024;
	if (sizeInKB < 1) sizeInKB = 1;
	HBufC* hKBytes = StringLoader::LoadLC( R_STR_FLASH_KBYTES, sizeInKB, iEikonEnv );
	TPtr itemptr = hKBytes->Des();
	AknTextUtils::LanguageSpecificNumberConversion(itemptr);
	if (hKBytes)
	{
		aItem.Append(*hKBytes);
	}
	aItem.AppendFill(KTabChar, 2);
	CleanupStack::PopAndDestroy( hKBytes );	
	
	return ETrue;

}
TBool CSaFlashFileView::BuildListBoxStringL(TEntry aEntry,
										  TDes& aItem,
										  TBool aIsDirItem
										  ,TBool aSwfExtn)
{
	
	if ( aIsDirItem )
	{
		return BuildListBoxStringforDirL(aEntry,aItem);
	}
	
	
	return BuildListBoxStringforSwfL(aEntry,aItem,aSwfExtn);
	

}

void CSaFlashFileView::ExtractListBoxString( const TDesC& aStr, TDes& aText )
{
	if ( aStr.Length() > 4 )
	{
		const TUint16* src = aStr.Ptr();
		TInt len = aStr.Length() - 2;
		TInt newLen = len - 2;
		TInt i;
		for (i=2;i<len;i++)
		{
			if (src[i] == '\t')
			{
				newLen = i - 2;
				break;
			}
		}
		aText.Copy( aStr.Ptr() + 2, newLen );
	}
	else
	{
		aText.SetLength( 0 );
	}
}


TBool CSaFlashFileView::FindListBoxItemL(
	CDesCArray& aArray,
	const TDesC& aName,
	TBool aIsDir,
	TInt& aIndex )
{
	TBool found = EFalse;
	TBuf<KMaxFileName> itemName;
	TInt n = aArray.Count();
	TInt i;
//#ifdef __FLASHLITE2.0_REL3.1
	TChar ch;
    if ( aIsDir )
    	{
    		TBuf<KMaxFileName> dir;
    		GetCurrentDir(dir);
    		TInt subDirCount = FlashUiUtil::CheckSubdirCountL( dir, aName, ETrue, EFalse );
        	if( subDirCount > 0 )
        	{
        		ch = KDirSubIconId;
        	}	
    		else
    		{
    			ch = KDirIconId; 
    		}
    	}
    else
    	{
    		ch = KSwfIconId;
    	}
//#endif //__FLASHLITE2.0_REL3.1
	for( i = 0; i < n; ++i )
		{
		if ( (TChar)(aArray[i])[0] == ch )
			{
			ExtractListBoxString( aArray[i], itemName );
			if ( itemName==aName )
				{
				aIndex = i;
				found = ETrue;				
				break;	
				}		
			}
		}
	return found;
}


// Create the list box strings for the appropriate dir
// and then update the list
//
void CSaFlashFileView::UpdateListL(TBool /*aUpdateSelection*/)
{
	CDesCArray* array = 0;
	HBufC* hEmptyMsg;


	if ( ViewingPhoneDir() )
	{
		if ( !iPhoneDirItems )
		{
			iPhoneDirItems = CreateListBoxStringsForDirL( iPhonePath );
		}					
		array = iPhoneDirItems;
		hEmptyMsg = iEikonEnv->AllocReadResourceLC(R_STR_NO_MOVIES);
	}
	else
	{
		if (iHasMMC)
		{
//#ifdef __FLASHLITE2.0_REL3.1
			if (iMMCCorrupted)
			{
				hEmptyMsg = iEikonEnv->AllocReadResourceLC(R_STR_MMCCORRUPTED);	
			}
			else if (iMMCLocked)
			{
				hEmptyMsg = iEikonEnv->AllocReadResourceLC(R_STR_MMCLOCKED);	
			}
			else
			{
				hEmptyMsg = iEikonEnv->AllocReadResourceLC(R_STR_NO_MOVIES);
			}
//#endif //__FLASHLITE2.0_REL3.1
		}
		else
		{
			hEmptyMsg = iEikonEnv->AllocReadResourceLC(R_STR_NO_MEMCARD);
		}
		if ( !iCardDirItems )
			iCardDirItems = CreateListBoxStringsForDirL( iCardPath );
		array = iCardDirItems;
	}

	if ( array )
	{
		iContainer->UpdateTextArrayL( array, *hEmptyMsg );
		TInt newIndex = 0;
		//if (!aUpdateSelection)
			{
			if (ViewingPhoneDir())
				{
				CDesCArray **dirItems = &iPhoneDirItems;
				//Less than or equal comparision because of immediate call after Delete,
				//which doesn't alter iCurrentItemIndex but CDesCArray::Count() is altered.
				if (*dirItems && iCurrentItemIndex <= (*dirItems)->Count())
					{
					newIndex = iCurrentItemIndex;
					//If selected item is last in the list and if it is deleted, then after deletion 
					//last item in the list should be selected instead of first item.
					if((newIndex !=0) && iCurrentItemIndex == (*dirItems)->Count())
						{
						newIndex--;	// To avoid crash in EikListBox, if only one item in the list.
						}
					}
				}				
			else
				{
				CDesCArray **dirItems = &iCardDirItems;
				if (*dirItems && iCurrentMMCItemIndex <= (*dirItems)->Count() )
					{
					newIndex = iCurrentMMCItemIndex;
					if((newIndex !=0) && iCurrentMMCItemIndex == (*dirItems)->Count())
						{
						newIndex--;	// To avoid crash in EikListBox. if only one item in the list
						}
					}
				}
			}
		UpdateSelectionL(newIndex);
	}
	CleanupStack::PopAndDestroy(hEmptyMsg);
}


// helper function for creating a new list and deleting the old
void CSaFlashFileView::ReplaceList()
{
	CDesCArray *tmp;
	if ( ViewingPhoneDir() )
	{
		tmp = iPhoneDirItems;
		iPhoneDirItems = NULL;
		TRAPD(err,UpdateListL(ETrue));
		if (iPhoneDirItems)
			delete tmp;
		else
			iPhoneDirItems = tmp;

		if (err!=KErrNone)
			HandleError(err);
	}
	else
	{
		tmp = iCardDirItems;
		iCardDirItems = NULL;
		TRAPD(err,UpdateListL(ETrue));
		if (iCardDirItems)
			delete tmp;
		else
			iCardDirItems = tmp;

		if (err!=KErrNone)
			HandleError(err);
	}
}



// helper function for removing a directory from the path
HBufC* CSaFlashFileView::PopPath( TDes& aStr )
{
	HBufC* dir = NULL;

	int len = aStr.Length();
	if ( aStr[len-1] == KCharBackslash )
	{
		aStr.SetLength( len-1 );
	}
	int pos = aStr.LocateReverse( KCharBackslash );
	if ( pos != KErrNotFound )
	{
		TPtrC right = aStr.Right(len-pos-2);
		dir = right.Alloc();
		aStr.SetLength( pos+1 );
	}
	return dir;
}

void CSaFlashFileView::SetNavigationTextFromPathL(TInt /*aMedia*/, TDes& aPath)
{
	//DebugShowNoteL(_L("+CSaFlashFileView::SetNavigationTextFromPathL"));

	if (iNaviBmp)
	{
		iNaviPane->Pop(iNaviBmp);
		delete iNaviBmp;
		iNaviBmp = NULL;
	}

	if ((aPath == iRootPhonePath) || (aPath == iRootMemCardPath))
	{
		SetNavigationPaneL();
		HBufC* hFlash = iEikonEnv->AllocReadResourceL(R_STR_FLASH_TITLE);
		if (hFlash)
		{
			SetTitlePaneTextL(*hFlash);
		}
		delete hFlash;

		//Cba()->SetCommandSetL(R_AVKON_SOFTKEYS_OPTIONS_EXIT);
		Cba()->SetCommandSetL(R_SAFLASH_FILEVIEW_SOFTKEYS_OPTIONS_EXIT_OPEN);
		Cba()->DrawDeferred();
		iLevel = 0;
	}
	else
	{
		TBuf<KMaxFileName> dirName;

		const TUint16* str = aPath.Ptr();
		TInt len = aPath.Length();
		if (str[len-1] == KCharBackslash)
		{
			len--;
		}
		TInt i;
		TInt start = 0;
		for (i=len-1;i>=0;i--)
		{
			if (str[i] == KCharBackslash)
			{
				start = i+1;
				break;
			}
		}
		TInt slashCount = 0;
		for(i=0;i<len;i++)
		{
			if (str[i] == KCharBackslash)
			{
				slashCount++;
			}
		}
 
        IconConstructLC();
        if ( ViewingPhoneDir() )
        {
            switch (slashCount)
	        {
                case 0:
                case 1:
                case 2:
                	break;
                	
                case 3:
                	{
                		iLevel=1;
            	    	CreateFolderNaviL( KSaFlashFirstLevel );	
                	}
               	 	break;
               	 	
                case 4:
                	{
                		iLevel=2;
                		CreateFolderNaviL( KSaFlashSecondLevel );
                	}
                	break;
                	
                case 5:
                	{
                		iLevel=3;
                		CreateFolderNaviL( KSaFlashThirdLevel );	
                	}
                	break;
                	
                case 6:
                	{
                		iLevel=4;
                		CreateFolderNaviL( KSaFlashFourthLevel );
                	}
                	break;
                	
                default:
                	{
                		iLevel=5;
                		CreateFolderNaviL( KSaFlashLastLevel );
                	}
            	    break;
	        }
        }
        else
        {
            switch (slashCount)
	        {
                case 0:
                case 1:
                	break;
                case 2:
            	    iLevel=1;
            	    CreateFolderNaviL( KSaFlashFirstLevel );
               	 	break;
                case 3:
                    iLevel=2;
                	CreateFolderNaviL( KSaFlashSecondLevel );
                	break;
                case 4:
                    iLevel=3;
                	CreateFolderNaviL( KSaFlashThirdLevel );
                	break;
                case 5:
                    iLevel=4;
                	CreateFolderNaviL( KSaFlashFourthLevel );
                	break;
                default:
                    iLevel=5;
                	CreateFolderNaviL( KSaFlashLastLevel );
            	    break;
	         }
	     }
   		
        dirName.Copy(&str[start], len - start);
        SetTitlePaneTextL(dirName);
        iNaviBmp = iFolderNavi;
        iNaviPane->PushL(*iNaviBmp);
		
        //Cba()->SetCommandSetL(R_AVKON_SOFTKEYS_OPTIONS_BACK);
        Cba()->SetCommandSetL(R_SAFLASH_FILEVIEW_SOFTKEYS_OPTIONS_BACK);
        Cba()->DrawDeferred();
        
        CleanupStack::PopAndDestroy( 6 ); //remove the six objects of icon construct
    }
}

void CSaFlashFileView::CreateFolderNaviL( TInt aFolderDepth )
{
    TSize size( TSize::EUninitialized );
    AknLayoutUtils::LayoutMetricsSize( AknLayoutUtils::ENaviPane, size );
        
    TDisplayMode dispMode = CCoeEnv::Static()->ScreenDevice()->DisplayMode();
    // Create bitmaps size of navi pane
    CFbsBitmap* navibmp = new (ELeave) CFbsBitmap();
    CleanupStack::PushL( navibmp );
    navibmp->Create( size, dispMode ); 
    
    CFbsBitmap* navibmpmask = new (ELeave) CFbsBitmap();
    CleanupStack::PushL( navibmpmask );
    navibmpmask->Create( size, dispMode ); 
    
    // Add color to navi pane bitmap    
    CFbsBitGc* fbsBitGc = CFbsBitGc::NewL(); //graphic context
    CleanupStack::PushL( fbsBitGc );
    
    CFbsBitmapDevice* bmpDevice = CFbsBitmapDevice::NewL( navibmp );
    fbsBitGc->Activate( bmpDevice );
    TRgb color( KRgbWhite );
    MAknsSkinInstance* skin = AknsUtils::SkinInstance();

    AknsUtils::GetCachedColor( skin, color, KAknsIIDQsnIconColors,   
        EAknsCIQsnIconColorsCG7 ); 

    fbsBitGc->SetBrushColor( color );
    fbsBitGc->Clear(); // Fill bitmap with color
    delete bmpDevice;
    
    // Create the mask bitmap 
    bmpDevice = CFbsBitmapDevice::NewL( navibmpmask );
    fbsBitGc->Activate( bmpDevice );
    // Add phone (mask) or MMC (mask) icon to navi pane mask bitmap 
    if(ViewingPhoneDir())
    	{
		AknIconUtils::SetSize( iPhoneIcon, iBitmapLayout[ KSaFlashRoot ].Rect().Size() );
        iBitmapLayout[ KSaFlashRoot ].DrawImage( *fbsBitGc, iPhoneIcon, iPhoneIconMask );
    	}
    else
    	{
		AknIconUtils::SetSize( iMMCIcon, iBitmapLayout[ KSaFlashRoot ].Rect().Size() );
        iBitmapLayout[ KSaFlashRoot ].DrawImage( *fbsBitGc, iMMCIcon, iMMCIconMask);						
    	}
    // Add depth amount of folders and separators to the bitmap
    if ( aFolderDepth <= KSaFlashMaxDepth-1 )
    	{
    	for ( TInt i( KSaFlashFirstLevel ); i <= aFolderDepth; i++ )
	    	{
			AknIconUtils::SetSize( iFolderIcon, iBitmapLayout[ i ].Rect().Size() );
			iBitmapLayout[ i ].DrawImage( *fbsBitGc, iFolderIcon, iFolderIconMask);
	    	}
	    // draw backslashes between folderbitmaps depending on current folderdepth
	    for( TInt j( KSaFlashRoot ); j < aFolderDepth; j++ )
	    	{
	        const CFont* font = iTextLayout[j].Font(); // Not owned
	        fbsBitGc->UseFont( font );
	        fbsBitGc->SetPenColor( KRgbBlack ); //mask is always black
	        fbsBitGc->DrawText( *iSeparator, iTextLayout[j].TextRect(), 
	        	font->AscentInPixels(), CGraphicsContext::ELeft, 0 );
	    	}
		}
	else
    	{
		for( TInt i( KSaFlashFirstLevel ); i <= (KSaFlashMaxDepth-1); i++ )
	    	{
			AknIconUtils::SetSize( iFolderIcon, iBitmapLayout[ i ].Rect().Size() );
			iBitmapLayout[ i ].DrawImage( *fbsBitGc, iFolderIcon, iFolderIconMask);
	    	}
	    // draw backslashes between folderbitmaps depending on current folderdepth
	    for( TInt j( KSaFlashRoot ); j < (KSaFlashMaxDepth); j++ )
	    	{
	        const CFont* font = iTextLayout[j].Font(); // Not owned
	        fbsBitGc->UseFont( font );
	        fbsBitGc->SetPenColor( KRgbBlack ); //mask is always black
	        fbsBitGc->DrawText( *iSeparator, iTextLayout[j].TextRect(), 
	        	font->AscentInPixels(), CGraphicsContext::ELeft, 0 );
	    	}
	    // draw three dots at the end
	    const CFont* font = iTextLayout[KSaFlashLastLevel].Font(); // Not owned
        fbsBitGc->UseFont( font );
        fbsBitGc->SetPenColor( KRgbBlack ); //mask is always black
        fbsBitGc->DrawText( *iEndChar, iTextLayout[KSaFlashLastLevel].TextRect(),
        	font->AscentInPixels(), CGraphicsContext::ELeft, 0 );   
		}
    delete bmpDevice;
        
    CleanupStack::PopAndDestroy();// fbsBitGc

	// Transfers ownership of bitmaps
    iFolderNavi = iNaviPane->CreateNavigationImageL( navibmp, navibmpmask );
    CleanupStack::Pop( 2 ); // navibmp, navibmpmask
}
    
// -----------------------------------------------------------------------------
// CSaFlashFileView::IconConstructLC
// -----------------------------------------------------------------------------
//
void CSaFlashFileView::IconConstructLC()
{
    CFbsBitmap* bitmap = NULL; // To be pushed into CleanupStack
    CFbsBitmap* mask = NULL;
    HBufC* string = NULL;

    if ( !iSeparator )
   	{
    	string = CCoeEnv::Static()->AllocReadResourceL(R_QTN_MEMC_MAIN_SEPARATOR);
    	iSeparator = string;
   	}
    if ( !iEndChar )
    {
	    string = CCoeEnv::Static()->AllocReadResourceL(R_QTN_MEMC_MAIN_END);	
	    iEndChar = string;
    }
     
	// Init layout
    SizeChanged(); 

    // Phone icon
    
    TFileName resFile(KFlash2UIMifFilePath);
    TFileName dllName;
    Dll::FileName(dllName);
    TBuf<2> drive = dllName.Left(2);
    resFile.Insert(0, drive);


   AknIconUtils::CreateIconLC(
        bitmap,
        mask,
        resFile,	
        EMbmSaflashQgn_prop_memc_phone_tab, 
        EMbmSaflashQgn_prop_memc_phone_tab_mask );
    if ( AknIconUtils::IsMifIcon( bitmap ) )
    {
        SizeChanged(); // Init layout
        // Sets the sizes of both bitmap and mask
        AknIconUtils::SetSize( bitmap, iBitmapLayout[KSaFlashRoot].Rect().Size() );
    }
    iPhoneIconMask = mask;
    iPhoneIcon = bitmap;
   
    // MMC icon    
    AknIconUtils::CreateIconLC(
            bitmap,
            mask,
            resFile,	
            EMbmSaflashQgn_prop_memc_mmc_tab, 
            EMbmSaflashQgn_prop_memc_mmc_tab_mask );
    if ( AknIconUtils::IsMifIcon( bitmap ) )
    {
        SizeChanged(); // Init layout
        // Sets the sizes of both bitmap and mask
        AknIconUtils::SetSize( bitmap, iBitmapLayout[KSaFlashRoot].Rect().Size() ); 
    }
    iMMCIconMask = mask;
    iMMCIcon = bitmap;
       
    // Folder icon
    AknIconUtils::CreateIconLC(
        bitmap,
        mask,
        resFile,	
        EMbmSaflashQgn_prop_folder_tab, 
        EMbmSaflashQgn_prop_folder_tab_mask );
        
    if ( AknIconUtils::IsMifIcon( bitmap ) )
    {
        SizeChanged(); // Init layout
        // Sets the sizes of both bitmap and mask
        AknIconUtils::SetSize( bitmap, iBitmapLayout[KSaFlashFirstLevel].Rect().Size() );
    }

    iFolderIconMask = mask;
    iFolderIcon = bitmap;
}

// ---------------------------------------------------------
// CSaFlashFileView::SizeChanged
// ---------------------------------------------------------
//
void CSaFlashFileView::SizeChanged() 
{ 
	TSize size(  TSize::EUninitialized );
    AknLayoutUtils::LayoutMetricsSize( AknLayoutUtils::ENaviPane, size );
    TRect parentRect( size );  
    // Get pane icon and text layouts
    TAknWindowLineLayout paneIconLayout(
        AknLayoutScalable_Avkon::navi_icon_text_pane_g1().LayoutLine() );
    TAknTextLineLayout paneTextLayout(
        AknLayoutScalable_Avkon::navi_icon_text_pane_t1().LayoutLine() );
    TAknLayoutRect layoutRect;
    TAknLayoutText layoutText;
    layoutRect.LayoutRect( parentRect, paneIconLayout );
    layoutText.LayoutText( parentRect, paneTextLayout );
    // Setup pane items
    TRect itemsRect( layoutRect.Rect() );
    const CFont* font( layoutText.Font() ); // Not owned
    TInt bsWidth( font->MeasureText( *iSeparator ) );
    TInt dotsWidth( font->MeasureText( *iEndChar ) );
    TInt textWidth( KSaFlashMaxDepth * bsWidth + dotsWidth );
    TInt iconWidth( ( itemsRect.Width() - textWidth )
        / KSaFlashMaxDepth );
    TInt i( 0 );

    if ( AknLayoutUtils::LayoutMirrored() ) //For Languages like Hebrew/Arabic
        {
        TInt offs( itemsRect.Width() );
        for ( i = 0 ; i < KSaFlashMaxDepth; i++ )
            {
            // Root and Folder Icons
            offs -= iconWidth;
            iBitmapLayout[ i ].LayoutRect(
                parentRect,
                paneIconLayout.iC,
                paneIconLayout.il + offs,
                paneIconLayout.it,
                paneIconLayout.ir,
                paneIconLayout.ib,
                iconWidth,
                paneIconLayout.iH
                );
            // Backslashes
            offs -= bsWidth;
            iTextLayout[ i ].LayoutText(
                parentRect,
                paneTextLayout.iFont,
                paneTextLayout.iC,
                paneTextLayout.il + offs,
                paneTextLayout.ir,
                paneTextLayout.iB,
                paneTextLayout.iW,
                paneTextLayout.iJ
                );
            }
        // Dots
        offs -= dotsWidth;
        iTextLayout[ i ].LayoutText(
            parentRect,
            paneTextLayout.iFont,
            paneTextLayout.iC,
            paneTextLayout.il + offs,
            paneTextLayout.ir,
            paneTextLayout.iB,
            paneTextLayout.iW,
            paneTextLayout.iJ
            );
        }
    else
        {
        TInt offs( 0 );
        for ( i = 0 ; i < KSaFlashMaxDepth; i++ )
            {
            // Root and Folder Icons
            iBitmapLayout[ i ].LayoutRect(
                parentRect,
                paneIconLayout.iC,
                paneIconLayout.il + offs,
                paneIconLayout.it,
                paneIconLayout.ir,
                paneIconLayout.ib,
                iconWidth,
                paneIconLayout.iH
                );
            offs += iconWidth;
            // Backslashes
            iTextLayout[ i ].LayoutText(
                parentRect,
                paneTextLayout.iFont,
                paneTextLayout.iC,
                paneTextLayout.il + offs,
                paneTextLayout.ir,
                paneTextLayout.iB,
                paneTextLayout.iW,
                paneTextLayout.iJ
                );
            offs += bsWidth;
            }
        // Dots
        iTextLayout[ i ].LayoutText(
            parentRect,
            paneTextLayout.iFont,
            paneTextLayout.iC,
            paneTextLayout.il + offs,
            paneTextLayout.ir,
            paneTextLayout.iB,
            paneTextLayout.iW,
            paneTextLayout.iJ
            );
        }
}

TBool CSaFlashFileView::TraverseUpL()
{
	HBufC* dir = NULL;

	if ( ViewingPhoneDir() )
	{
		if ( iPhonePath == iRootPhonePath )
			return EFalse;
		dir = PopPath( iPhonePath );
		if (dir)
			CleanupStack::PushL(dir);
		SetNavigationTextFromPathL(0, iPhonePath);
	}
	else 
	{
		if ( iCardPath == iRootMemCardPath )
			return EFalse;
		dir = PopPath( iCardPath );
		if (dir)
			CleanupStack::PushL(dir);
		SetNavigationTextFromPathL(1, iCardPath);
	}

	ReplaceList();

	if (dir)
	{
		UpdateSelectionL(*dir)	;
		CleanupStack::PopAndDestroy(dir);
	}
	return ETrue;
}


void CSaFlashFileView::TraverseDownL()
{
	if ( CurrentItemIsValid() )
	{
		if ( ViewingPhoneDir() )
		{
			iPhonePath.Append( iCurrentItem );
			TRAPD(err,SetNavigationTextFromPathL(0, iPhonePath));
			if (err!=KErrNone)
				HandleError(err);
		}
		else
		{
			iCardPath.Append( iCurrentItem );
			TRAPD(err,SetNavigationTextFromPathL(1, iCardPath));
			if (err!=KErrNone)
				HandleError(err);
		}

		ClearCurrentItem();

		ReplaceList();
	}
}


TBool	CSaFlashFileView::GetNewNameL( const TDesC& aInitialName, TDes& aNewName, TNewNameType aType )
{
	TBool res = EFalse;

	aNewName = aInitialName;
	AknTextUtils::StripCharacters( aNewName, KDirectionalChars );
	TInt len = aNewName.Length();
	if ( len > 0 && aNewName[len-1] == KCharBackslash )
		{		
		aNewName.SetLength( len-1 );
		}
    CAknTextQueryDialog* dlg = CAknTextQueryDialog::NewL( aNewName, CAknQueryDialog::ENoTone );
    if ( aType == ENewFolder)
    {
    	res = dlg->ExecuteLD( R_SAFLASH_NEW_NAME_QUERY );	
    }
    else
    {
    	res = dlg->ExecuteLD( R_SAFLASH_RENAME_QUERY );
    	if ( res )
		{
			aNewName.Trim(); // removing leading and trailing spaces
			//res = (aNewName != aInitialName);
		}
	
    }
	return res;
}

//#ifdef FLASHLITE_BETA3
void CSaFlashFileView::HandleCopyCmdL()
{    
    TOverflowHandler overflow;
	if (CurrentItemIsValid())
	{

		GetCurrentDir(iOldDir);
		GetSelectedPath(iOrigPath);
		
		TFileName srcDirName = BaflUtils::FolderNameFromFullName(iOrigPath);
		TBool getNewName = ETrue;
		while ( getNewName )
		{
			TBool copyItem = ETrue;
			//if (CSaFlashDirList::GetCopyDirL(iNewDir))
			if (AknCommonDialogs::RunCopyDlgLD(
        							iNewDir, 
        							R_SAFLASH_MEMORY_SELECTIONDIALOG,
        							R_SAFLASH_FILE_SELECTIONDIALOG ))
			{
				//iNewDir.AppendFormat(KDirSeparator, &overflow);
				iNewPath.Copy(iNewDir);
		//		iNewPath.AppendFormat(iCurrentItem, &overflow);	
				
				
				TInt newLen = iNewPath.Length () + iCurrentItem.Length ();
				TInt err = KErrNone;
				if (newLen > KMaxFileName)
				{
				   	err = KErrOverflow;
				}
				else 
				{
				   	iNewPath.Append (iCurrentItem);	
				}
				
				if (KErrOverflow == err || overflow.IsOverflowed() == KErrOverflow)
				{
					// Failed to copy file
					// report error
					HBufC* errorMessage = StringLoader::LoadLC( R_STR_COPY_FAIL,
	            	iOrigPath, iEikonEnv );
	            	FlashUiUtil::ShowNoteL( EAknGlobalWarningNote, *errorMessage );
	                CleanupStack::PopAndDestroy();    // errorMessage
	                return; 	
				}
				
				TBuf<2 * KMaxFileName> tempCmpTarget(iNewDir);
				tempCmpTarget.Append(srcDirName);
            	tempCmpTarget.Append(KDirSeparator); 
            	if ( iOrigPath[iOrigPath.Length()-1] == '\\')
	            	{
	            	if ( ( iOrigPath.Length() < tempCmpTarget.Length() &&
				         0 == iOrigPath.Compare(tempCmpTarget.Left(iOrigPath.Length())) ) ) 	
					    	{
					    	TInt err = KErrGeneral;				    	
		    				
		    				HBufC* errorMessage = StringLoader::LoadLC( 
		                			R_STR_COPY_FAIL,
									BaflUtils::FolderNameFromFullName(iOrigPath), iEikonEnv );
				        	NotifyErrorL( EAknGlobalInformationNote, *errorMessage, err );
		                	CleanupStack::PopAndDestroy();    // errorMessage		    				
		    				return;
						    }
	            	}
				if (iNewPath.Compare(iOrigPath) == 0)
					{
					TBool nameExists = ETrue;
					TInt found;
					TFileName tempFile = iCurrentItem;
		            while(nameExists)
			           	{
			           	TInt len = tempFile.Length();
		            	if ( len > 0 && tempFile[len-1] == KCharBackslash)
			            	{	
			            	tempFile.SetLength( len-1 );					
			            	}
			           	HBufC* pMsg = StringLoader::LoadLC( R_STR_RENAME_FOLDER_EXISTS_FORMAT1_MSG,
									tempFile, iEikonEnv );

						CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::EConfirmationTone );
						if ( dlg->ExecuteLD(R_SAFLASH_CONFIRMATION_QUERY, *pMsg ) )
							{
							TUint attVal = 0;
							if ( GetNewNameL( tempFile, iNewName, ERenameFileOrFolder))
			           			{
			           			TBool validName = IsValidName( iNewName );
			           			
			           			if (!validName)
									{
									// Invalid file name
									// Report error
									HBufC* errorMessage = StringLoader::LoadLC( R_STR_INVALID_FILE_NAME_FORMAT1_MSG,
									iEikonEnv );
									FlashUiUtil::ShowNoteL( EAknGlobalWarningNote, *errorMessage );
									CleanupStack::PopAndDestroy();    // errorMessage
									CleanupStack::PopAndDestroy();    // pMsg
									return;
									}
			           			if (CurrentItemIsDir())
									{
									iNewName.Append(KCharBackslash);
									}
								else
									{
									TBool hasSwfExtension = HasSwfExtension( iNewName );
									#ifdef __FEATURE_FLV_RECOG_PLAY
									TBool hasFlvExtension = HasFlvExtension( iNewName );
									#endif
                                    #ifdef __FEATURE_F4V_RECOG_PLAY
                                    TBool hasF4vExtension = HasF4vExtension( iNewName );
                                    #endif
									
									    if( !hasSwfExtension
                                            #ifdef __FEATURE_FLV_RECOG_PLAY
                                                && !hasFlvExtension
                                            #endif
                                            #ifdef __FEATURE_F4V_RECOG_PLAY
                                                && !hasF4vExtension
                                            #endif
									            )
										{//by default we append swf extensions
										iNewName.Append( KSwfExtension);				    					
										}	
									}
								iNewPath.Copy(iNewDir);
								iNewPath.Append(iNewName);
								
			           			found = iEikonEnv->FsSession().Att(iNewPath,attVal);
				            	if (found == KErrNone)	
				            		{
				            		//Already Exists.
				            		nameExists = ETrue;
				            		}
				            	else
					            	{
					            	//Doesn't exist.
					            	nameExists = EFalse;	
					            	}
					            tempFile = iNewName;
			           			}
			           		else
			           			{
			           			CleanupStack::PopAndDestroy();    // pMsg
			           			return;
			           			}
							}
						else
							{
							CleanupStack::PopAndDestroy();    // pMsg
			           		return;	
							}
						CleanupStack::PopAndDestroy();    // pMsg
						}
					iCurrentItem = tempFile;	
					}
				
				TBool found = FindItemExistsL();
					            
	            if ( found )
	            	{
	             	// folder already exists
		            // Report error
					TInt len = iCurrentItem.Length();
		            TFileName tempFile = iCurrentItem;
		            if ( len > 0 && tempFile[len-1] == KCharBackslash)
			            {	
			            tempFile.SetLength( len-1 );					
			            }
		            
		            HBufC* pMessage = StringLoader::LoadLC( R_FLDR_OVERWRITE_QUERY,tempFile, iEikonEnv );
                                     
		            CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::EConfirmationTone );
		            if ( dlg->ExecuteLD(R_SAFLASH_CONFIRMATION_QUERY, *pMessage ) )
		            	{
		            	getNewName = EFalse;
		            	copyItem = ETrue;
		            	}
		            else
		            	{
		            	//Ask for new name.
		            	TBool nameExists = ETrue;
		            	while(nameExists)
			            	{
			            	TInt found;
			            	if ( GetNewNameL( iOrigName, iNewName, ERenameFileOrFolder))
			            		{
			            		TBool validName = IsValidName( iNewName );
			            		
			            		if (!validName)
									{
									// Invalid file name
									// Report error
									HBufC* errorMessage = StringLoader::LoadLC( R_STR_INVALID_FILE_NAME_FORMAT1_MSG,
									iEikonEnv );
									FlashUiUtil::ShowNoteL( EAknGlobalWarningNote, *errorMessage );
									CleanupStack::PopAndDestroy();    // errorMessage 
									CleanupStack::PopAndDestroy(); // pMsg
									return;
									}
								
								if (CurrentItemIsDir())
									{
									iNewName.Append(KCharBackslash);
									}
								else
									{
									TBool hasSwfExtension = HasSwfExtension( iNewName );
									#ifdef __FEATURE_FLV_RECOG_PLAY
									TBool hasFlvExtension = HasFlvExtension( iNewName );
									#endif
                                    #ifdef __FEATURE_F4V_RECOG_PLAY
                                    TBool hasF4vExtension = HasF4vExtension( iNewName );
                                    #endif
									
                                    if( !hasSwfExtension
                                         #ifdef __FEATURE_FLV_RECOG_PLAY
                                             && !hasFlvExtension
                                         #endif
                                         #ifdef __FEATURE_F4V_RECOG_PLAY
                                             && !hasF4vExtension
                                         #endif
                                             )
										{
				    					iNewName.Append( KSwfExtension);				    					
				    					}	
									}
									
								iNewPath.Copy(iNewDir);
								iNewPath.Append(iNewName);								
								
								iCurrentItem = iNewName;
				            	// check whether item exists or not.
				            	found = FindItemExistsL();
				            	//TInt attVal = 0;
				            	//found = iEikonEnv->FsSession().Att(iNewPath,attVal);
				            	if (found)
					            	{
					            	// Item exists, ask for Rename.
					            	TInt len = 0;
									iNewName = iCurrentItem;
									len = iNewName.Length();
									if ( len > 0 && iNewName[len-1] == KCharBackslash)
										{
										iNewName.SetLength( len-1 );
										}

									HBufC* pMsg = StringLoader::LoadLC( R_STR_RENAME_FOLDER_EXISTS_FORMAT1_MSG,
									iNewName, iEikonEnv );

									CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::EConfirmationTone );
									if ( dlg->ExecuteLD(R_SAFLASH_CONFIRMATION_QUERY, *pMsg ) )
										{
										iOrigName = iNewName;
										nameExists = ETrue;
										}
									else
										{
										nameExists = EFalse;
					            		getNewName = EFalse;
					            		copyItem = EFalse;	
										}
									CleanupStack::PopAndDestroy(); // pMsg
					            	}
					            else
					            	{
									//Name entered doesn't exist, rename it.
					            	nameExists = EFalse;
					            	getNewName = EFalse;
			            			copyItem = ETrue;
					            	}
				            	}
				            else
				            	{
				            	// Don't want to move with new name.Discard whole transaction.
				            	nameExists = EFalse;
					            getNewName = EFalse;
			            		copyItem = EFalse;	
				            	}			            	
			            	}
			            }
		            CleanupStack::PopAndDestroy(1);	// pMessage
	            }
	            
	            if ( copyItem )
	            {
	            	getNewName = EFalse;
	             	
				    TInt err = KErrNone;
					CFileMan* fm = CFileMan::NewL(iEikonEnv->FsSession());
					if ( iOrigPath[iOrigPath.Length()-1] == '\\')
			        { 
			         	_LIT(KNULL, "");
			            TInt numOfSubDirs = FlashUiUtil::CheckSubdirCountL(KNULL, iNewPath, ETrue, EFalse);
						TInt numOfFiles = FlashUiUtil::CheckSubdirCountL(KNULL, iNewPath, EFalse, EFalse);
						
						if ( (numOfSubDirs != KZero) || (numOfFiles!= KZero) )
						{
							err = fm->RmDir( iNewPath );
						}
						else
						{
							err = iEikonEnv->FsSession().RmDir( iNewPath );
						}
                        
                        numOfSubDirs = FlashUiUtil::CheckSubdirCountL(KNULL, iOrigPath, ETrue, EFalse);
					    numOfFiles = FlashUiUtil::CheckSubdirCountL(KNULL, iOrigPath, EFalse, EFalse);
					    
			            err = iEikonEnv->FsSession().MkDir( iNewPath);
			            if ( ( numOfSubDirs != KZero || numOfFiles != KZero ) && ( err == KErrNone ) )
			            {
			            	err = fm->Copy( iOrigPath, iNewPath, CFileMan::ERecurse | CFileMan::EOverWrite);
			            }
			         }
			         else
			         {
			 	     	err = fm->Copy( iOrigPath, iNewPath, CFileMan::EOverWrite);
			         }
			         
					 delete fm;
					 
					 if ( err == KErrNone )
					 {
						ReplaceList();

						/* // Media Gallery not required
						CMGXFileManager* mgxFileMgr = MGXFileManagerFactory::NewFileManagerL(iEikonEnv->FsSession());
            			mgxFileMgr->UpdateL();        				
	                	delete mgxFileMgr;
	                	*/
					 }
					 //else
					 //{
						// Failed to copy file
						// report error
					//	HBufC* errorMessage = StringLoader::LoadLC( R_STR_COPY_FAIL,
		            //	iOrigPath, iEikonEnv );
		            //	FlashUiUtil::ShowNoteL( EAknGlobalWarningNote, *errorMessage );
		            //    CleanupStack::PopAndDestroy();    // errorMessage 
					// }
	        	}
			}
			else
			{
				getNewName = EFalse;
			}
			
		} //while
	}
}

void CSaFlashFileView::DisplayFileDetailsL()
{
    // Create listbox
    CEikFormattedCellListBox* list = new(ELeave) CAknDoublePopupMenuStyleListBox;
    CleanupStack::PushL(list);

    // Create popup list
    CAknPopupList* popupList = CAknPopupList::NewL(list, R_SAFLASH_SOFTKEYS_OK_EMPTY_SELECTICON, AknPopupLayouts::EMenuDoubleWindow);
    CleanupStack::PushL(popupList);

    // initialize listbox.
    
    // Clear flag.
    TInt flags = 0; 
    list->ConstructL(popupList, flags);
    list->CreateScrollBarFrameL(ETrue);
    list->ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto );

    HBufC* popupTitle = CCoeEnv::Static()->AllocReadResourceLC(R_STR_HEADING_PRES_DETAILS);
    popupList->SetTitleL(*popupTitle);
    CleanupStack::PopAndDestroy(); // popupTitle

    // Make listitems.
    CDesCArrayFlat* items = new(ELeave) CDesCArrayFlat(1);
    CleanupStack::PushL(items);

    _LIT(KListBoxItemFormat, "%S\t%S");

    // Get the directory entry
    TEntry entry;
        
    TInt length = iCurrentItem.Length();
	if(ViewingPhoneDir())
	{
		length+=iPhonePath.Length();
	}
	else
	{
		length+=iCardPath.Length();
	}
	
	HBufC* fileNameBuf = HBufC::NewLC(length);
	TPtr fileName  = fileNameBuf->Des();
    GetSelectedPath(fileName);
    User::LeaveIfError(iCoeEnv->FsSession().Entry(fileName, entry));	
    // Display filename
    {
        HBufC* fnameTitle = iCoeEnv->AllocReadResourceLC(R_STR_DETAILS_FILENAME);
        HBufC* fnameItem = HBufC::NewLC(entry.iName.Length() + fnameTitle->Length() + 1);
        TPtr itemptr = fnameItem->Des();

        itemptr.Format(KListBoxItemFormat, fnameTitle, &entry.iName);
        items->AppendL(*fnameItem);
        CleanupStack::PopAndDestroy(2); // fnameItem, filenameTitle
    }

    // Display format
    {
        TInt file_type = FlashUiUtil::GetFileTypeL(fileName);
        if(file_type)
        {
            TInt dot = entry.iName.LocateReverse('.');
            //if (dot != KErrNotFound)
            {
            	TPtrC ext;
            	if(file_type == 1) //swf type
            	{
            		ext.Set((KSwfExtension().Right(KSwfExtension().Length() - 1)));	
            	}
				#ifdef __FEATURE_FLV_RECOG_PLAY
            	else if ( file_type == 2 ) //KFLVFileType) //flv type
            	{
            		ext.Set((KFlvExtension().Right(KFlvExtension().Length() - 1)));	
            	}
				#endif                
                #ifdef __FEATURE_F4V_RECOG_PLAY
                else if ( file_type == 3 ) // KF4VFileType) //f4v type
                {
                    ext.Set((KF4vExtension().Right(KF4vExtension().Length() - 1))); 
            	}
				#endif                

                HBufC* formatTitle = iCoeEnv->AllocReadResourceLC(R_STR_DETAILS_FORMAT);
                HBufC* formatItem = HBufC::NewLC(formatTitle->Length() + entry.iName.Length() - dot);
                TPtr itemptr = formatItem->Des();
                itemptr.Append(*formatTitle);
                itemptr.Append('\t');
                itemptr.Append(ext);

                items->AppendL(*formatItem);
                CleanupStack::PopAndDestroy(2); // formatItem, formatTitle
            }
        }
       	CleanupStack::PopAndDestroy(fileNameBuf);
    }
    TTime localTime = entry.iModified; // convert from universal time
    RTz tz;
    User::LeaveIfError( tz.Connect() );
    User::LeaveIfError( tz.ConvertToLocalTime( localTime ) );
    tz.Close();
    // Display date
    {
        HBufC* dateTitle = iCoeEnv->AllocReadResourceLC(R_STR_DETAILS_DATE);
        HBufC* dateFormat = iCoeEnv->AllocReadResourceLC(R_QTN_DATE_USUAL_WITH_ZERO);
        TBuf<KMaxShortDateFormatSpec*2> dateString;
        localTime.FormatL(dateString, *dateFormat);
        HBufC* dateItem = HBufC::NewLC(dateTitle->Length() + dateString.Length() + 1);
        TPtr itemptr = dateItem->Des();

        itemptr.Format(KListBoxItemFormat, dateTitle, &dateString);
        AknTextUtils::LanguageSpecificNumberConversion(itemptr);
        items->AppendL(*dateItem);
        CleanupStack::PopAndDestroy(3); // dateItem, dateFormat, dateTitle
    }

    // Display time
    {
        HBufC* timeTitle = iCoeEnv->AllocReadResourceLC(R_STR_DETAILS_TIME);
        HBufC* timeFormat = iCoeEnv->AllocReadResourceLC(R_QTN_TIME_USUAL_WITH_ZERO);
        TBuf<KMaxTimeFormatSpec*2> timeString;
        localTime.FormatL(timeString, *timeFormat);
        HBufC* timeItem = HBufC::NewLC(timeTitle->Length() + timeString.Length() + 1);
        TPtr itemptr = timeItem->Des();

        itemptr.Format(KListBoxItemFormat, timeTitle, &timeString);
        AknTextUtils::LanguageSpecificNumberConversion(itemptr);
        items->AppendL(*timeItem);
        CleanupStack::PopAndDestroy(3); // timeItem, timeFormat, timeTitle
    }

    // Display size
    {
        HBufC* sizeTitle = iCoeEnv->AllocReadResourceLC(R_STR_DETAILS_SIZE);
        HBufC* sizeString = NULL;
        if (entry.iSize >= 1024)
        {
            sizeString = StringLoader::LoadLC(R_QTN_SIZE_KB, ((entry.iSize + 512) >> 10));
        }
        else
        {
            sizeString = StringLoader::LoadLC(R_QTN_SIZE_B, entry.iSize);
        }
        HBufC* sizeItem = HBufC::NewLC(sizeTitle->Length() + sizeString->Length() + 1);
        TPtr itemptr = sizeItem->Des();
        AknTextUtils::LanguageSpecificNumberConversion(itemptr);
        itemptr.Format(KListBoxItemFormat, sizeTitle, sizeString);

        items->AppendL(*sizeItem);
        CleanupStack::PopAndDestroy(3); // sizeItem, sizeString, sizeTitle
    }

//#ifdef __FLASHLITE2.0_REL3.1
	    // Display the DRM rights
    {
        TInt result;
        GetSelectedPath(iOrigName);
        TBool isProtected;

        CContent *content = CContent::NewLC(iOrigName); 
        content->GetAttribute( EIsProtected, isProtected );
        CleanupStack::PopAndDestroy(); // content

        if (isProtected)
        {
            CDRMHelperRightsConstraints* aExecuteRights = NULL;
            CDRMHelperRightsConstraints* aIgnore1 = NULL;
            CDRMHelperRightsConstraints* aIgnore2 = NULL;
            CDRMHelperRightsConstraints* aIgnore3 = NULL;

            TBool aExpired;
            TBool aSendingAllowed;
            TBool notYetValid = FALSE;

			TInt err;
            TRAP(err,iDrmHelper->GetRightsDetailsL(iOrigName, 
               	ContentAccess::EExecute,
               	aExpired,
               	aSendingAllowed,               	
               	aIgnore1,               	 
               	aIgnore2,
               	aExecuteRights, 
               	aIgnore3)); 
            // Delete ignored constraints
            if( aIgnore1)
            {
				delete aIgnore1;
			}
            if( aIgnore2 )
            {
            	delete aIgnore2;
            }
            if( aIgnore3)
            {
            	delete aIgnore3;
            }
            
            if( err == KErrCANoRights)
            {
          	      AddStringPairL(R_QTN_DRM_MGR_DET_STAT, R_QTN_DRM_MGR_DET_NOT_VALID_YET, items, EFalse);
            }
			if (aExecuteRights)
			{
	            CleanupStack::PushL(aExecuteRights);
	            
	            if (aExecuteRights->FullRights())
	            {
	                AddStringPairL(R_QTN_DRM_MGR_DET_FULL_X, R_QTN_DRM_MGR_DET_UNLIMITED, items, ETrue);
	            }
	            else
	            {
	                // Check for count based rights
	                TUint32 aCounter;
	                TUint32 aOriginalCounter;
	                TTimeIntervalSeconds aTimer;
	                TTimeIntervalSeconds aInterval;
	                TTime aStartTime;
	                TTime aEndTime;
	                
	                TRAP(result, aExecuteRights->GetCountersL(aCounter, aOriginalCounter));
	                if ((result != KErrNotFound) && (result != KErrNone))
	                {
	                    User::Leave(result);
	                }
	                if (result != KErrNotFound)
	                {
	                    AddCountsL(aCounter, items);
	                }

#ifdef PHASE_TWO
	                // Check for timed count based rights
	                TRAP(result, aExecuteRights->GetTimedCountL(aCounter, aOriginalCounter, aTimer));
	                if ((result != KErrNotFound) && (result != KErrNone))
	                {
	                    User::Leave(result);
	                }
	                if (result != KErrNotFound)
	                {
	                    AddCountsL(aCounter, items);
	                }
#endif
	                
	                // Check for interval based rights
	                TRAP(result, aExecuteRights->GetIntervalL(aInterval));
	                if ((result != KErrNotFound) && (result != KErrNone))
	                {
	                    User::Leave(result);
	                }
	                if (result != KErrNotFound)
	                {
	                    TRAP(result, aExecuteRights->GetIntervalStartL(aStartTime));
	                    if ((result != KErrNotFound) && (result != KErrNone))
	                    {
	                        User::Leave(result);
	                    }
	                    if (result == KErrNotFound)  // Interval has not yet been started
	                    {
	                        AddStringPairL(R_QTN_DRM_MGR_DET_UTS_X, R_QTN_DRM_MGR_DET_NOT_ACT, items, ETrue);
	                        AddTimeLeftL(R_QTN_DRM_MGR_DET_UDL_X, aInterval, items, ETrue);
	                        notYetValid = TRUE;
	                    }
	                }
	                
	                // Check for date/time based rights
	                TRAP(result, aExecuteRights->GetStartTimeL(aStartTime));
	                if ((result != KErrNotFound) && (result != KErrNone))
	                {
	                    User::Leave(result);
	                }
	                
	                if (result != KErrNotFound)
	                {
	                    TRAP(result, aExecuteRights->GetEndTimeL(aEndTime));
	                    if ((result != KErrNotFound) && (result != KErrNone))
	                    {
	                        User::Leave(result);
	                    }
						
						if (result != KErrNotFound)
						{
							AddDateAndTimeL(R_QTN_DRM_MGR_DET_RVF_X, aStartTime, items);
	                    	AddDateAndTimeL(R_QTN_DRM_MGR_DET_RVT_X, aEndTime, items);	
						}
	                }
	                
#ifdef PHASE_TWO
	                // Check for accumulated based rights
	                TRAP(result, aExecuteRights->GetAccumulatedTimeL(aInterval));
	                if ((result != KErrNotFound) && (result != KErrNone))
	                {
	                    User::Leave(result);
	                }
	                if (result != KErrNotFound)
	                {
	                   AddTimeLeftL(R_QTN_DRM_MGR_DET_ACCUM_TIME_LEFT, aInterval, items, EFalse);
	                }
#endif

	                // Add status
	                if (aExpired)
	                {
	                    AddStringPairL(R_QTN_DRM_MGR_DET_STAT, R_QTN_DRM_MGR_DET_EXP, items, EFalse);
	                }
	                else
	                {
	                    if (notYetValid)
	                    {
	                        AddStringPairL(R_QTN_DRM_MGR_DET_STAT, R_QTN_DRM_MGR_DET_NOT_VALID_YET, items, EFalse);
	                    }
	                    else
	                    {
	                        AddStringPairL(R_QTN_DRM_MGR_DET_STAT, R_QTN_DRM_MGR_DET_VALID, items, EFalse);
	                    }
	                }

	                // Add content sending attribute
	                if (aSendingAllowed)
	                {
	                    AddStringPairL(R_QTN_DRM_MGR_DET_CS, R_QTN_DRM_MGR_DET_ALLOWED, items, EFalse);
	                }
	                else
	                {
	                    AddStringPairL(R_QTN_DRM_MGR_DET_CS, R_QTN_DRM_MGR_DET_FORBID, items, EFalse);
	                }
	            }
	            CleanupStack::PopAndDestroy(); //playRights
			}
        }
    }
//#endif //__FLASHLITE2.0_REL3.1
    // Set listitems.
    CTextListBoxModel* model = list->Model();
    model->SetItemTextArray(items);
    model->SetOwnershipType(ELbmOwnsItemArray);
    CleanupStack::Pop(items); // items

    // Show popup list
    popupList->ExecuteLD();

    CleanupStack::Pop(popupList); // popupList
    CleanupStack::PopAndDestroy(list); // list
}
//#endif //FLASHLITE_BETA3

//#ifdef __FLASHLITE2.0_REL3.1
void CSaFlashFileView::CheckMemoryCard()
{
	TDriveInfo driveInfo;
	// Get drive info
    TInt error = iEikonEnv->FsSession().Drive( driveInfo,
        MemoryCardDrive() ); 
        
	if ( error || driveInfo.iType == EMediaNotPresent )
    {
        iHasMMC = EFalse;
    }
        
    else if (driveInfo.iType == EMediaHardDisk)
	    {
	    iHasMMC = ETrue;	
	    }
    TInt err = KErrNotReady;    
    if(iHasMMC)
    	{
    	// Check that volume is valid (ie. MMC is not corrupted)
		TVolumeInfo volumeInfo;
		error = iEikonEnv->FsSession().Volume( volumeInfo,MemoryCardDrive());
		switch (error)
			{
			case KErrNone:
				iMMCCorrupted = EFalse;
				iMMCLocked = EFalse;
				break;
			case KErrCorrupt:
				iMMCCorrupted = ETrue;
				break;
			case KErrLocked:
				iMMCLocked = ETrue;
				break;
			default:
				break;
			}		
		err = iEikonEnv->FsSession().MkDirAll(iRootMemCardPath);
    	}
	
	switch(err)
		{
		case KErrNone:
		case KErrAlreadyExists:
		case KErrCorrupt:
		case KErrLocked:
		iHasMMC = ETrue;
		break;

		case KErrNotReady:
		iHasMMC = EFalse;
		break;

		default:
		iHasMMC = EFalse;
		}
}

TInt CSaFlashFileView::MemoryCardDrive()
{
	TInt res = 0;
    TInt err = RFs::CharToDrive( PathInfo::MemoryCardRootPath()[0], res );
	return res;
}

CCoeControl* CSaFlashFileView::GetContainer()
{
	return iContainer;
}


void CSaFlashFileView::HandleError(TInt aErr)
{
	switch(aErr)
	{
		case KErrNoMemory:
			((CSaFlashAppUi*)AppUi())->NotifyClientError(EInsufficientMemoryError, ETrue, EFalse);
			break;
		default:
			((CSaFlashAppUi*)AppUi())->NotifyClientError(EGeneralError, EFalse, EFalse);
			break;
	}
}
//#ifdef __FLASHLITE2.0_REL3.1
void CSaFlashFileView::AddDateAndTimeL(TUint32 aTitleId, TTime aDateTime, CDesCArrayFlat* aItems)
{
    HBufC* dateFormat = StringLoader::LoadLC( R_QTN_DATE_USUAL_WITH_ZERO, iEikonEnv );
    TBuf<KMaxShortDateFormatSpec*2> dateString;
    aDateTime.FormatL(dateString, *dateFormat);
    
    HBufC* timeFormat = StringLoader::LoadLC( R_QTN_TIME_USUAL, iEikonEnv );
    TBuf<KMaxTimeFormatSpec*2> timeString;
    aDateTime.FormatL(timeString, *timeFormat);

    // Join the two strings
    HBufC* dateTimeBuffer = HBufC::NewLC(dateString.Length() + timeString.Length() + 3);
    TPtr dateTimePointer = dateTimeBuffer->Des();
    
    _LIT(KSpace, " ");
    dateTimePointer.Append(dateString);
    dateTimePointer.Append(KSpace);
    dateTimePointer.Append(timeString);

    HBufC* dateTimeTitleText = StringLoader::LoadLC( R_QTN_DRM_MGR_DET2_EXECUTE, iEikonEnv );
    HBufC* timeTitleText = StringLoader::LoadLC( aTitleId, *dateTimeTitleText, iEikonEnv );
    
    HBufC* itemBuffer = HBufC::NewLC(dateTimeBuffer->Length() + timeTitleText->Length() + 50);
    TPtr itemPointer = itemBuffer->Des();
    
    itemPointer.Append(*timeTitleText);
    _LIT(KTab, "\t");
    itemPointer.Append(KTab);
    itemPointer.Append(*dateTimeBuffer);
    AknTextUtils::LanguageSpecificNumberConversion(itemPointer);
    aItems->AppendL(*itemBuffer);
    CleanupStack::PopAndDestroy(6); 
}

void CSaFlashFileView::AddTimeLeftL(TUint32 aTitleId, TTimeIntervalSeconds aInterval, CDesCArrayFlat* aItems, TBool aDisplayPlayText)
{
    TUint8 seconds = aInterval.Int() % 60;
    TUint8 minutes = (aInterval.Int() / 60) % 60;
    TUint8 hours = (aInterval.Int() / 60 / 60) % 24;
    TUint8 days = (aInterval.Int() / 60 / 60 / 24) % 31;
    TUint8 months = (aInterval.Int() / 60 / 60 / 24 / 31) % 12;
    TUint16 years = (aInterval.Int() / 60 / 60 / 24 / 31 / 12);

    if (years > 0)
    {
        AddSingleOrDoubleValueL(aTitleId, 
                R_QTN_DRM_NBR_OF_YEARS_ONE,
                R_QTN_DRM_NBR_OF_YEARS_TWO_FOUR,
                R_QTN_DRM_NBR_OF_YEARS_FIVE_ZERO,
                R_QTN_DRM_NBR_OF_YEARS_ONE_FINAL,
                R_QTN_DRM_NBR_OF_MONTHS_ONE,
                R_QTN_DRM_NBR_OF_MONTHS_TWO_FOUR,
                R_QTN_DRM_NBR_OF_MONTHS_FIVE_ZERO,
                R_QTN_DRM_NBR_OF_MONTHS_FIVE_ZERO, // No need for a one final
                aItems,
                years,
                months, aDisplayPlayText);
    }
    else  if (months > 0)
    {
        AddSingleOrDoubleValueL(aTitleId, 
                R_QTN_DRM_NBR_OF_MONTHS_ONE,
                R_QTN_DRM_NBR_OF_MONTHS_TWO_FOUR,
                R_QTN_DRM_NBR_OF_MONTHS_FIVE_ZERO,
                R_QTN_DRM_NBR_OF_MONTHS_FIVE_ZERO, // No need for a one final
                R_QTN_DRM_NBR_OF_DAYS_ONE,
                R_QTN_DRM_NBR_OF_DAYS_TWO_FOUR,
                R_QTN_DRM_NBR_OF_DAYS_FIVE_ZERO,
                R_QTN_DRM_NBR_OF_DAYS_ONE_FINAL,
                aItems,
                months,
                days, aDisplayPlayText);
    }
    else  if (days > 0)
    {
        AddSingleOrDoubleValueL(aTitleId, 
                R_QTN_DRM_NBR_OF_DAYS_ONE,
                R_QTN_DRM_NBR_OF_DAYS_TWO_FOUR,
                R_QTN_DRM_NBR_OF_DAYS_FIVE_ZERO,
                R_QTN_DRM_NBR_OF_DAYS_ONE_FINAL,
                R_QTN_DRM_NBR_OF_HOURS_ONE,
                R_QTN_DRM_NBR_OF_HOURS_TWO_FOUR,
                R_QTN_DRM_NBR_OF_HOURS_FIVE_ZERO,
                R_QTN_DRM_NBR_OF_HOURS_ONE_FINAL,
                aItems,
                days,
                hours, aDisplayPlayText);
    }
    else  if (hours > 0)
    {
        AddSingleOrDoubleValueL(aTitleId, 
                R_QTN_DRM_NBR_OF_HOURS_ONE,
                R_QTN_DRM_NBR_OF_HOURS_TWO_FOUR,
                R_QTN_DRM_NBR_OF_HOURS_FIVE_ZERO,
                R_QTN_DRM_NBR_OF_HOURS_ONE_FINAL,
                R_QTN_DRM_NBR_OF_MINS_ONE,
                R_QTN_DRM_NBR_OF_MINS_TWO_FOUR,
                R_QTN_DRM_NBR_OF_MINS_FIVE_ZERO,
                R_QTN_DRM_NBR_OF_MINS_ONE_FINAL,
                aItems,
                hours,
                minutes, aDisplayPlayText);
    }
    else  if (minutes > 0)
    {
        AddSingleOrDoubleValueL(aTitleId, 
                R_QTN_DRM_NBR_OF_MINS_ONE,
                R_QTN_DRM_NBR_OF_MINS_TWO_FOUR,
                R_QTN_DRM_NBR_OF_MINS_FIVE_ZERO,
                R_QTN_DRM_NBR_OF_MINS_ONE_FINAL,
                R_QTN_DRM_NBR_OF_SECS_ONE,
                R_QTN_DRM_NBR_OF_SECS_TWO_FOUR,
                R_QTN_DRM_NBR_OF_SECS_FIVE_ZERO,
                R_QTN_DRM_NBR_OF_SECS_ONE_FINAL,
                aItems,
                minutes,
                seconds, aDisplayPlayText);
    }
    else
    {
        AddSingleValueL(aTitleId, 
                R_QTN_DRM_NBR_OF_SECS_ONE,
                R_QTN_DRM_NBR_OF_SECS_TWO_FOUR,
                R_QTN_DRM_NBR_OF_SECS_FIVE_ZERO,
                R_QTN_DRM_NBR_OF_SECS_ONE_FINAL,
                aItems,
                seconds, aDisplayPlayText);
    }
}

void CSaFlashFileView::AddSingleOrDoubleValueL(TUint32 aTitleId,
    TUint32 aFirstOne, TUint32 aFirstTwoFour, TUint32 aFirstFiveZero, TUint32 aFirstOneFinal,
    TUint32 aSecondOne, TUint32 aSecondTwoFour, TUint32 aSecondFiveZero, TUint32 aSecondOneFinal,
    CDesCArrayFlat* aItems, TUint8 aFirstValue, TUint8 aSecondValue, TBool aDisplayPlayText)
{
    if (aSecondValue == 0)
    {
        AddSingleValueL(aTitleId, aFirstOne, aFirstTwoFour, aFirstFiveZero, 
        	aFirstOneFinal, aItems, aFirstValue, aDisplayPlayText);
    }
    else
    {
        AddDoubleValueL(aTitleId, 
            aFirstOne, aFirstTwoFour, aFirstFiveZero, aFirstOneFinal,
            aSecondOne, aSecondTwoFour, aSecondFiveZero, aSecondOneFinal,
            aItems, aFirstValue, aSecondValue, aDisplayPlayText);
    }
}

void CSaFlashFileView::AddDoubleValueL(TUint32 aTitleId,
    TUint32 aFirstOne, TUint32 aFirstTwoFour, TUint32 aFirstFiveZero, 
    TUint32 aFirstOneFinal, TUint32 aSecondOne, TUint32 aSecondTwoFour, 
    TUint32 aSecondFiveZero, TUint32 aSecondOneFinal, 
    CDesCArrayFlat* aItems, TUint8 aFirstValue, TUint8 aSecondValue, TBool aDisplayPlayText)
{
    _LIT(KListBoxItemFormat, "%S\t%S");
    HBufC* firstMeasurement;
    HBufC* secondMeasurement;

    // Format measurements
    firstMeasurement = FormatValueLC(aFirstOne, aFirstTwoFour, aFirstFiveZero, aFirstOneFinal, aFirstValue);
    secondMeasurement = FormatValueLC(aSecondOne, aSecondTwoFour, aSecondFiveZero, aSecondOneFinal, aSecondValue);
    CDesC16ArrayFlat* array = new(ELeave) CDesCArrayFlat(1);
    CleanupStack::PushL(array);
    array->AppendL(*firstMeasurement);
    array->AppendL(*secondMeasurement);
    HBufC*  formatString = StringLoader::LoadLC(R_QTN_DRM_MGR_DET_INTER_TWO, *array, iEikonEnv );
        
	HBufC* title;
    // Add item to list
		
	if ( aDisplayPlayText )
	{
		HBufC* titleText = StringLoader::LoadLC( R_QTN_DRM_MGR_DET2_EXECUTE, iEikonEnv );
		title = StringLoader::LoadLC( aTitleId, *titleText, iEikonEnv );
	}
	else
	{
		title = StringLoader::LoadLC( aTitleId, iEikonEnv );
	}

    HBufC* itemBuffer = HBufC::NewLC(formatString->Length() + title->Length() + 1);
    TPtr itemPointer = itemBuffer->Des();
    itemPointer.Format(KListBoxItemFormat, title, formatString);
    AknTextUtils::LanguageSpecificNumberConversion(itemPointer);
    aItems->AppendL(*itemBuffer);
	if ( aDisplayPlayText )
	{
		CleanupStack::PopAndDestroy(7); // title, itemBuffer, bodyText
	}
	else
	{
		CleanupStack::PopAndDestroy(6); 
	}
}

HBufC* CSaFlashFileView::FormatValueLC(TUint32 aOne, TUint32 aTwoFour, TUint32 aFiveZero, TUint32 aOneFinal, TUint8 aValue)
{
    HBufC* formattedValue;
    if (aValue == 1)
    {
        return iCoeEnv->AllocReadResourceLC(aOne);
    }
    
    if (((aValue % 10) == 0) || ((aValue >=11) && (aValue <= 14)) || (((aValue % 10) >= 5) && ((aValue % 10) <= 9)))
    {
        formattedValue = StringLoader::LoadLC( aFiveZero, aValue, iEikonEnv );
    }
    else if ((aValue % 10) == 1)
    {
        formattedValue = StringLoader::LoadLC( aOneFinal, aValue, iEikonEnv );
    }
    else
    {
        formattedValue = StringLoader::LoadLC( aTwoFour, aValue, iEikonEnv );
    }
    return formattedValue;
}

void CSaFlashFileView::AddSingleValueL(TUint32 aTitleId, TUint32 aOne, 
	TUint32 aTwoFour, TUint32 aFiveZero, TUint32 aOneFinal, 
	CDesCArrayFlat* aItems, TUint8 aValue, TBool aDisplayPlayText)
{
    _LIT(KListBoxItemFormat, "%S\t%S");
    
    HBufC* titleText;
    if ( aDisplayPlayText )
	{
		HBufC* titleResourceText = StringLoader::LoadLC( R_QTN_DRM_MGR_DET2_EXECUTE, iEikonEnv );
		titleText = StringLoader::LoadLC( aTitleId, *titleResourceText, iEikonEnv );
	}
	else
	{
		titleText = StringLoader::LoadLC( aTitleId, iEikonEnv );
	}
	
    HBufC* formattedValue = FormatValueLC(aOne, aTwoFour, aFiveZero, aOneFinal, aValue);
    
    HBufC* itemBuffer = HBufC::NewLC(formattedValue->Length() + titleText->Length() + 1);
    TPtr itemPointer = itemBuffer->Des();
    itemPointer.Format(KListBoxItemFormat, titleText, formattedValue);
    AknTextUtils::LanguageSpecificNumberConversion(itemPointer);
    aItems->AppendL(*itemBuffer);
    if ( aDisplayPlayText )
    {
    	CleanupStack::PopAndDestroy(4); 
    }
    else
    {
    	CleanupStack::PopAndDestroy(3);
    }
}

void CSaFlashFileView::AddStringPairL(TUint32 aTitleId, TUint32 aBodyId, 
	CDesCArrayFlat* aItems, TBool displayPlayText)
{
    _LIT(KListBoxItemFormat, "%S\t%S");
    HBufC* title;
    if ( displayPlayText )
    {
    	HBufC* titleText = StringLoader::LoadLC( R_QTN_DRM_MGR_DET2_EXECUTE, iEikonEnv );
    	title = StringLoader::LoadLC( aTitleId, *titleText, iEikonEnv );	
    }
    else
    {
    	title = StringLoader::LoadLC( aTitleId, iEikonEnv );	
    }
    
    HBufC* body = iCoeEnv->AllocReadResourceLC(aBodyId);
    HBufC* itemBuffer = HBufC::NewLC(title->Length() + body->Length() + 1);
    TPtr itemPointer = itemBuffer->Des();
    
    itemPointer.Format(KListBoxItemFormat, title, body);
    aItems->AppendL(*itemBuffer);
    if ( displayPlayText )
    {
   	    CleanupStack::PopAndDestroy(4); // titleText, title, body, itemBuffer
    }
    else
    {
    	CleanupStack::PopAndDestroy(3);
    }
}

void CSaFlashFileView::AddCountsL(TUint32 aCounter, CDesCArrayFlat* aItems)
{
    _LIT(KListBoxItemFormat, "%S\t%S");
    
    HBufC* countsTitleText = StringLoader::LoadLC( R_QTN_DRM_MGR_DET2_EXECUTE, iEikonEnv );
    HBufC* countsTitle = StringLoader::LoadLC( R_QTN_DRM_MGR_DET_UTL_X, *countsTitleText, iEikonEnv );
   
    HBufC* countsValue;
    if (aCounter == 1)
    {
       countsValue = StringLoader::LoadLC( R_QTN_DRM_MGR_DET_1_COUNT, iEikonEnv ); 
    }
    else
    {
       countsValue = StringLoader::LoadLC( R_QTN_DRM_MGR_DET_N_COUNTS, aCounter, iEikonEnv);
    }
 
    HBufC* itemBuffer = HBufC::NewLC(countsTitle->Length() + countsValue->Length() + 1);
    TPtr itemPointer = itemBuffer->Des();

    itemPointer.Format(KListBoxItemFormat, countsTitle, countsValue);
    AknTextUtils::LanguageSpecificNumberConversion(itemPointer);
    aItems->AppendL(*itemBuffer);
    CleanupStack::PopAndDestroy(4); 
}

// -----------------------------------------------------------------------------
// SAFlashFileView::NotifyError(TAknGlobalNoteType aType,const TDesC& aNoteText, TInt aErrCode )
// (other items were commented in a header).
// -----------------------------------------------------------------------------
//
void CSaFlashFileView::NotifyErrorL(TAknGlobalNoteType aType,const TDesC& aNoteText, TInt aErrCode )
{
    if ( aNoteText != KNullDesC )
    {
        CAknGlobalNote* pNote = CAknGlobalNote::NewLC();
        pNote->ShowNoteL( aType, aNoteText );
        CleanupStack::PopAndDestroy();    // pNote 
    }
    else if ( aErrCode != 0 )
    {
        HBufC* errorMessage = NULL;
        CTextResolver* textResolver = CTextResolver::NewLC(*CEikonEnv::Static()); 

		errorMessage = ( textResolver->ResolveErrorString( aErrCode ) ).Alloc();
		CleanupStack::PushL(errorMessage);

        CAknGlobalNote* pNote = CAknGlobalNote::NewLC();
        pNote->ShowNoteL( aType, *errorMessage );
        CleanupStack::PopAndDestroy(pNote); 
        CleanupStack::PopAndDestroy(errorMessage);   
		CleanupStack::PopAndDestroy(textResolver);        
    }
}

void CSaFlashFileView::HandleResourceChange(TInt aType)
{
     TRAPD(err, HandleResourceChangeL(aType));
     if (err ==KErrNone)
     {
     	
     }
}

void CSaFlashFileView::HandleResourceChangeL(TInt aType)
{
     if(iLevel > 0)
     {
        IconConstructLC();
        	if(iNaviBmp)
        	{
            	iNaviPane->Pop(iNaviBmp);
            	delete iNaviBmp;
            	iNaviBmp = NULL;
        	}
        	switch(iLevel)
        	{     
	            case 1:
    	            CreateFolderNaviL( KSaFlashFirstLevel );
        	     	break;
            	case 2:
               		CreateFolderNaviL( KSaFlashSecondLevel );
               		break;
            	case 3:
               		CreateFolderNaviL( KSaFlashThirdLevel );
               		break;
            	case 4:
               		CreateFolderNaviL( KSaFlashFourthLevel );
               		break;
            	default:
               		CreateFolderNaviL( KSaFlashLastLevel );
        	        break;
	    	}
	    	iNaviBmp = iFolderNavi;
        	iNaviPane->PushL(*iNaviBmp);
        	CleanupStack::PopAndDestroy( 6 ); //remove the six objects of icon construct
     	}
 
     ((CSaFlashAppUi*)AppUi())->HandleResourceChangeL(aType);
}

TBool CSaFlashFileView::FindItemExistsL()
{
 	iOrigName = iCurrentItem;
	TBool found = EFalse;
	if(BaflUtils::PathExists(CCoeEnv::Static()->FsSession(),iNewDir))
		{
		TFileName item(iNewDir);
		item.Append(iCurrentItem);
		TBool isFolder;					
		BaflUtils::IsFolder(CCoeEnv::Static()->FsSession(), iOrigPath, isFolder);
		if (isFolder)
			{
			found = BaflUtils::FolderExists(CCoeEnv::Static()->FsSession(),item);	
			}
		else
			{
			found = BaflUtils::FileExists(CCoeEnv::Static()->FsSession(),item);	
			}					
		}	
	return found;
}
