/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
                                 
*
*/


#include<centralrepository.h>
#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformclib.h"
#include "symbian_appui.h"
#include "symbian_playercontrol.h"
#include "symbian_playerview.h"
#include "flash_engine.h"
#include "saflash_coredefs.h"
#include "symbian_ui_interface.h"
#include "surfacecontainercontrol.h"

#include <centralrepository.h>
#include "hwkeywatch.h"
#ifndef ENABLE_SF_CHANGES
#include <Flashlite_2_0PrivateCRKeys.h> //senRep keys
#else
#include "Flashlite_2_0PrivateCRKeys.h" //senRep keys
#endif 
#include "flash2ui.hrh"

#include <applayout.cdl.h> // Scalable UI
#include <AknDef.h>
#include <AknUtils.h>
#include <AknIconUtils.h>
#include <eikspane.h>
#include <SAFlash.MBG>
#include <AknsUtils.h>
#include <featmgr.h>
#include <bautils.h>
#include <AknSettingCache.h>
#include "ui_data_caging_literals.hrh"

#include "saflash_consts.h"
#ifdef FEATURE_INLINE_TEXT_ENTRY
#include <aknedsts.h>			// For CAknEdwinState
#include <fepitfr.h>			// For MFepInlineTextFormatRetriever

#include "commondef.h"			// For FI_Text
#include "amp_textnavigation.h"
#include "amp_returnstring.h"

#ifdef __SCALABLE_ICONS
#include <FLASHPLUGIN.MBG>
#else
#include <flashplugin_bitmap.mbg>
#endif

const TInt KT9SpellSoftkeyCommandId=16385;
const TInt KDocumentMaximumLengthForFep = 0x7FFFFFFFL;
#endif // FEATURE_INLINE_TEXT_ENTRY

const TInt  KErrorImageWidth        = 20;
const TInt  KErrorImageHeight       = 20;

#include "NPNFlashDefs.h"
const TInt KMinVolume = 0;  // Minimum volume level(Mute)
const TInt KMaxVolume = 10; // Maximum volume level
extern void VwrDebug(TUint8 aLogLevel, TRefByValue<const TDesC> aText, ... );

/*
#ifdef __FEATURE_FLV_RECOG_PLAY
_LIT(KFLvPlayerSwf,"Nokia_FlVPlayer.swf");
_LIT(KFlvVideoName,"videoName=");
_LIT(KFlvVideoNameRFile,"videoName=@RFILE@");
#endif

#ifdef __FEATURE_F4V_RECOG_PLAY
_LIT(KF4vPlayerSwf,"Nokia_FlVPlayer.swf");
_LIT(KF4vVideoName,"videoName=");
_LIT(KF4vVideoNameRFile,"videoName=@RFILE@");
#endif
*/

#if defined(__FEATURE_FLV_RECOG_PLAY) || defined (__FEATURE_F4V_RECOG_PLAY)
_LIT(KVideoPlayerSwf,"Nokia_FlVPlayer.swf");
_LIT(KVideoName,"videoName=");
_LIT(KVideoNameRFile,"videoName=@RFILE@");
#endif

_LIT(KFlashPluginMifFilePath,"\\resource\\apps\\Flashplugin.mif");

////////////////////////////////////////////////////////////////////////////////
// CSaFlashPlayerControl
////////////////////////////////////////////////////////////////////////////////

CSaFlashPlayerControl::CSaFlashPlayerControl(CSaFlashPlayerView* aView)
{
	iEngine = NULL;
	iView = aView;
	iHwKeyProperty = NULL;
	
	iCenRepos = NULL;
#ifdef FEATURE_INLINE_TEXT_ENTRY
	iIsActivatingIME = EFalse;
	iImeControl=NULL;
#endif // FEATURE_INLINE_TEXT_ENTRY
	iSideVolKeysSupported = FeatureManager::FeatureSupported(KFeatureIdSideVolumeKeys);
#ifdef __FEATURE_FLV_RECOG_PLAY		
	iFLVFile = EFalse;
#endif
    #ifdef __FEATURE_F4V_RECOG_PLAY     
    iF4VFile = EFalse;
	#endif

#ifdef USE_VPU2
	iVideoSurfaceContainer = NULL;
#endif
	
	iFlashLongTapDetector = NULL;
	iSuppressor = NULL;
	
	iTouchFeatureSupported = EFalse;
	
}

CSaFlashPlayerControl::~CSaFlashPlayerControl( )
{
	delete iFocusHandler;
	
	if (iEngine)
	{
		iEngine->Destroy();
	}

#ifdef USE_VPU2 
    delete iVideoSurfaceContainer;
    iVideoSurfaceContainer = NULL;
#endif	
	
	if(iFELib.Handle())
	{
		iFELib.Close();
	}
//	#ifdef __FLASHLITE2.0_REL3.1
    if (iMutedNavi)
    {
    	delete iMutedNavi;
    	iMutedNavi = NULL;
    }
    
    if (iPausedNavi)
    {
    	delete iPausedNavi;
    	iPausedNavi = NULL;
    }
    
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR  
    if(iFlashLongTapDetector)
        {
        delete iFlashLongTapDetector;
        iFlashLongTapDetector = NULL;
        }
        
    if(iSuppressor)
        {
        delete iSuppressor;
        iSuppressor = NULL;
        }
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
#ifdef __USE_EMPTY_NAVI__
    if (iEmptyNavi)
    {
    	delete iEmptyNavi;
    	iEmptyNavi = NULL;
    }
#endif	// __USE_EMPTY_NAVI__    
    
    if (iPausedMutedNavi)
    {
    	delete iPausedMutedNavi;
    	iPausedMutedNavi = NULL;
    }
//#ifdef __SIDE_VOLUME_KEYS
if(iSideVolKeysSupported)
	{
    if( iVolumeDisplay )
    	{
    	if(iVolumeDisplay->IsActive())
    		{
    		iVolumeDisplay->Cancel();
    		}
    	delete iVolumeDisplay;
    	iVolumeDisplay = NULL; 
	    }
	    
    if(iVolumeNavi)
    	{
    	delete iVolumeNavi;
    	iVolumeNavi = NULL;
    	}
    	
    	
	if(iVolPopup)
	{
		delete iVolPopup	;
		iVolPopup = NULL;
		
	}
	
	

	if( iHwKeyProperty )
	{
		iHwKeyProperty->RemoveObserver(this);
    	delete iHwKeyProperty;
    	iHwKeyProperty = NULL;
	}	

	
	}
//#endif //__SIDE_VOLUME_KEYS

    if(iCenRepos)
    {
        delete iCenRepos;
    }

    if (iBlinker)
    {
    	if (iBlinker->IsActive())
    	{
    		iBlinker->Cancel();
    	}
    	
    	delete iBlinker;
    	iBlinker = NULL;
    }
   
// #endif //__FLASHLITE2.0_REL3.1
#ifdef FEATURE_INLINE_TEXT_ENTRY
	if(iImeControl)
	{
			iImeControl=NULL;
	}
#endif
	if( iErrorImageBitmap )
	{
	    delete iErrorImageBitmap;
	}
	    
	if( iErrorImageBitmapMask )
	{
	    delete iErrorImageBitmapMask;
	}
}


void CSaFlashPlayerControl::ConstructL(/*MFlashUI* aFlashUI,*/ const TRect& aRect, FlashEngineConfig& aConfig 
/*
#ifdef SELF_IAD_UPDATE
        ,MIadUpdateCallback* aIADUpdateCallbackInterface 
#endif
*/
        )
{
	VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::ConstructL() ++"));
	FeatureManager::InitializeLibL();

    iTouchFeatureSupported = FeatureManager::FeatureSupported(KFeatureIdPenSupport);

    iTactileFeedbackSupported = FeatureManager::FeatureSupported(KFeatureIdTactileFeedback);
    iSideVolKeysSupported = FeatureManager::FeatureSupported(KFeatureIdSideVolumeKeys);
    FeatureManager::UnInitializeLib();
	iEngine = CreateFlashEngineL(iFELib);
	VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::ConstructL() iEngine: [%x]"),iEngine);
    
//	 ConstructL(FlashEngineConfig& aConfig,
//	            MFlashClientIf* aClientInterfaces,
//	            EEngineExecutionMode eExecutionMode
//	            )
	 
    /*iEngine->ConstructL(aFlashUI, aConfig
                        #ifdef SELF_IAD_UPDATE
                        , aIADUpdateCallbackInterface
                        #endif
                        );*/
	MFlashClientIf* clientIf = static_cast<CSaFlashAppUi*>(iEikonEnv->AppUi());
	iEngine->ConstructL(aConfig, clientIf, eStandAlone);
	VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::ConstructL(): iEngine->ConstructL(aFlashUI, aConfig)"));
	
	//TRAPD(result, iEngine->InitializeStandaloneL(aRect, EFalse));
	TRAPD(result, iEngine->InitializeL(aRect,EFalse));
	if ( result == KErrNoMemory )
	{
		//pView->NotifyClientError(EInsufficientMemoryError, ETrue, EFalse);
		VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::ConstructL(): KErrNoMemory"));
		User::Leave(result);
	}
	
	if(iTactileFeedbackSupported) //adding feature manager to check the feature at run time.
		iFeedback = MTouchFeedback::Instance();
	
	ihaveDragged = EFalse;

	//Open a connection to the Central Repository...
    TRAP_IGNORE(iCenRepos = CRepository::NewL(KCRUIdFlashliteViewer));
    
    /**
    if(errCode)
    {
    	VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerView::ConstructL(MFlashUI*, const TRect&, FlashEngineConfig&),Error Code %d"), errCode);
        User::Leave(errCode);
    }
    **/
 
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR  
    
    if(iTouchFeatureSupported)
    {
        iFlashLongTapDetector = CFlashLongTapDetector::NewL(this);
        
        iSuppressor = CAknPointerEventSuppressor::NewL();
        iSuppressor->SetMaxTapDuration(KMaxTInt64);
    }
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
    
    if (iSideVolKeysSupported)
	{
		TRAP_IGNORE(iHwKeyProperty = CHwKeyPropertyWatch::NewL());	
		
		if(iHwKeyProperty)
		{
			iHwKeyProperty->AddObserverL(this);
		}
	}
    
    iErrorOccured = EFalse;
    iLastError = KErrNone;
    
    LoadIconsL();
    
	CreateWindowL();
	SetRect(aRect);
	if(iView->IsStubApp())
	{
        SetExtentToWholeScreen();
	}
	
	// We use an instance of CIdle to resume the Flash Engine because
	// of the case where there may be a video playing and the profile
	// has keypad tones turned on. If the Flash volume is high and the
	// user invokes, then cancels the menu, the keypad tone is played
	// while the video is resumed, and the keypad tone plays at a much
	// higher than normal volume (bug #137041).
			
	iFocusHandler = CIdle::NewL(CActive::EPriorityIdle);
//#ifdef __FLASHLITE2.0_REL3.1
	iNaviPane = static_cast<CAknNavigationControlContainer*>( CEikStatusPaneBase::Current()->ControlL( TUid::Uid( EEikStatusPaneUidNavi ) ) );
	iPauseIconVisible = EFalse;
	iMutedIconVisible = EFalse;
    IconConstructL();
//#ifdef __SIDE_VOLUME_KEYS
if (iSideVolKeysSupported)
	{
    iVolPopup =  CFlashVolumeControl::NewL(this,this,ETrue, iEngine);
    
	}
//#endif //__SIDE_VOLUME_KEYS
//#endif //__FLASHLITE2.0_REL3.1
#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
//Toolbar was introduced in 5.0 not available in 3.2
  iToolbar = static_cast<CAknAppUi*>( ControlEnv()->AppUi() )->CurrentFixedToolbar(); 
#endif // __FEATURE_UI_LANDSCAPE_FULLSCREEN

	// Gets the current input launguage id.
	CAknSettingCache& cache = CAknEnv::Static()->SettingCache();
	iCurrentInputLanguage = cache.InputLanguage();	
	
 	Window().PointerFilter(EPointerFilterMove|EPointerFilterDrag, 0);
	//ClaimPointerGrab(ETrue);  // Get the events also outside Press and Release events such as enter and exit
 	
 	//Window().SetPointerCapture(RWindowBase::TCaptureFlagDragDrop | RWindowBase::TCaptureFlagEnabled);
	
	ActivateL();
	
	VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::ConstructL() --"));
}

void CSaFlashPlayerControl::LoadIconsL() 
{ 
	 TFileName rscfn; //cautious symbian does not recommend auto var for TfileName
	 TFileName dllName;
	 Dll::FileName(dllName);
	 const TUint KDriveLetterLength = 2;
	 TBuf<KDriveLetterLength> drive = dllName.Left(KDriveLetterLength);
	 
	 rscfn.Copy(drive); // get the drive letter
	 rscfn.Append(KFlashPluginMifFilePath);
	 
	 TBool fileExists( BaflUtils::FileExists(CCoeEnv::Static()->FsSession(), rscfn) );
	 
	 if ( !fileExists )
	    {
	    // If we didn't find the resource file, then we should leave 
	    User::Leave(KErrNotFound);
	    }
    
	TInt iconId = 0;
	TInt iconMaskId = 0;
    
#ifdef __SCALABLE_ICONS

    const TInt KBrokenIconId     = EMbmFlashpluginQgn_indi_flash_error; 
    const TInt KBrokenIconMaskId = EMbmFlashpluginQgn_indi_flash_error; 
    iconId 		= KBrokenIconId;
    iconMaskId 	= KBrokenIconMaskId;
        
#else

    const TInt KBrokenBitmapIconId     = EMbmFlashplugin_bitmapQgn_indi_flash_error; 
    const TInt KBrokenBitmapIconMaskId = EMbmFlashplugin_bitmapQgn_indi_flash_error; 
	
	  iconId 		= KBrokenBitmapIconId;
    iconMaskId 	= KBrokenBitmapIconMaskId;
	
#endif

    AknIconUtils::CreateIconL(
        iErrorImageBitmap,
        iErrorImageBitmapMask,
        rscfn,	
        iconId, 
        iconMaskId );
        
    if ( AknIconUtils::IsMifIcon( iErrorImageBitmap ) )
    {
        CCoeControl::SizeChanged(); // Init layout
        
        TSize bitmapSize(KErrorImageWidth, KErrorImageHeight);
        
        // Sets the sizes of both bitmap and mask
        AknIconUtils::SetSize( iErrorImageBitmap, bitmapSize );
    }
}    

//-----------------------------------------------------------------------
// void CSaFlashPlayerControl::HandleVideoError(const TInt& aError)   
//
// Video Error Handler.
//-----------------------------------------------------------------------

void CSaFlashPlayerControl::HandleVideoError(const TInt& aError)
{
	//NPPDebug(LOG_ERROR,_L("CBrowserPluginControl::HandleVideoError() : ERROR = [%d]"), aError);
	
	
	iLastError = aError;
	//iErrorIsCritical = ETrue;
	
	SetErrorOccured(aError);
	
	//TRAP_IGNORE(iPlayerInstance->PrepareToExitL());	
}    

//----------------------------------------------------------------------
// void CSaFlashPlayerControl::SetErrorOccured(TInt aError)    
//
// This function is called when there is an error while opening 
// a flash file 
//-----------------------------------------------------------------------
 
void CSaFlashPlayerControl::SetErrorOccured(TInt aError) 
{
    iErrorOccured  = ETrue;  
    iLastError = aError;
    DrawNow();  
}   

void CSaFlashPlayerControl::UpdateScreenSizeL(const TRect& aRect)
	{	
	//iEngine->InitializeStandaloneL(aRect, ETrue);
    SetRect( aRect );   
	
#ifdef USE_VPU2        
        if(iVideoSurfaceContainer)
            iVideoSurfaceContainer->SetRect(aRect);
#endif

    if(iEngine)
    {
        TRAPD(err, iEngine->InitializeL(aRect, ETrue));
        if(err != KErrNone)
        {
            iView->NotifyClientError(err,ETrue);
            iLastError = err;
            SetErrorOccured(err);
            User::Leave(err);
        }

        if(!(iEngine->IsRunning()))
            {
            iEngine->PlayStep(ETrue);	
            }
        
        if(iEngine->IsPaused())
            iEngine->PlayStep(ETrue);
    }
	}


#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
void CSaFlashPlayerControl::HandleFlashLongTapEventL( const TPoint& aPoint, const TPoint& /*aParentPoint*/)
    {
    if (iTouchType == ETouchStarted) 
        { 
        iTouchType = ETouchLongTap; 
        } 

#ifdef __FEATURE_CONTEXT_SENSITIVE_MENU__
    if(!(iFLVFile || iF4VFile) && !iView->IsStubApp())
    {
    	// show the context sensitive menu *only* if the video file is not directly opened
    	// i.e., do not show for .flv and .f4v files
    	// Also, if the content goes to full screen via FS command, 
    	// then do not give an option to come back to normal screen
    	
    	TPoint point = this->PositionRelativeToScreen() + aPoint;
    	iView->ShowContextSensitiveMenu(point);
    }
#endif
    
    VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::HandleFlashLongTapEventL, Long Tap"));
    }
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR


#ifndef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
const TInt  KStylusDragThreshold	= 12;
#endif
//In touch screen supported device, this function si responsible for handling Pointer event.
void CSaFlashPlayerControl::HandlePointerEventL(const TPointerEvent &aPointerEvent)
{
//set this to true if we need to toggle to FS-NS 
    TInt togglescreen = EFalse;
	if(iTouchFeatureSupported) //adding feature manager to check the feature at run time.
	{	
		if (!AknLayoutUtils::PenEnabled() || !iEngine )
		   return;
		TPoint point = aPointerEvent.iPosition;
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
        if(iSuppressor && iSuppressor->SuppressPointerEvent(aPointerEvent))
            {
            return;
            }

        if(iFlashLongTapDetector && !(iEngine->IsControlPresent(point)))
           iFlashLongTapDetector->PointerEventL(aPointerEvent);
            
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR  
		
	    if(iEngine->IsPanning())
	    {
	        TInt offsetX = point.iX - iPreviousPoint.iX;
	        TInt offsetY = point.iY - iPreviousPoint.iY;
	           
	        TPoint offset(offsetX, offsetY);
        
	        iPreviousPoint = aPointerEvent.iPosition;
	        
	        iEngine->OffsetPointToPan( offset, aPointerEvent.iType );       
            switch(aPointerEvent.iType)
                   {
                        case TPointerEvent::EButton1Down:
                        {
                            iTouchType = ETouchStarted;
                            break;
                        }
                        case TPointerEvent::EDrag:
                        {
                            iTouchType = ETouchDrag;
                            break;
                        }
                        case TPointerEvent::EButton1Up:
                        {
                            // if we are panning, then all events except Drag are ignored in iEngine->OffsetPointToPan( offset, aPointerEvent.iType );
                            // now, if we ar eplaying a video, then we need to switch to normal screen on Mouse Up
#ifndef __FEATURE_CONTEXT_SENSITIVE_MENU__
			                if(iEngine->GetSwfHasVideo())
			                    {
			                    TRect videoRect = iEngine->GetVideoRectL();
			                    TBool onVideo = videoRect.Contains(aPointerEvent.iPosition);
			                    if( !iEngine->GetFullScreenViaFSCommand() && iTouchType == ETouchStarted )
			                            {
			                            // since controls are completely ignored 
			                            // (we do not pass events to the engine when panning), this check is not required
			                            // if(!iControlPresent)
                            			#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
			                           // ToggleScreenL(); Set the flag and call togglescreenL at the end of this function,We must toggle after calling all iEngine->OfferPointerEventL 
                                    	togglescreen = ETrue;
										#endif            
			                            }
			                    }
#endif
                            if( iTouchType == ETouchStarted )
                                iTouchType = ETouchTap;
                            
                            break;
                        }
                    }
	    }
	    else
	    {
	    	if(iTactileFeedbackSupported)
	    		ProvideFeedbackL(aPointerEvent);
	          
#ifdef  FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR
	            switch (aPointerEvent.iType) 
	            { 
	            case TPointerEvent::EButton1Down: 
	                    iTouchType = ETouchStarted; 
	                    //ClaimPointerGrab(ETrue);
	                    VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::HandlePointerEventL, TPointerEvent::EButton1Down,(%d, %d)"), aPointerEvent.iPosition.iX, aPointerEvent.iPosition.iY);
	                                     
	                    iCorePosition = aPointerEvent.iPosition;
	                    iControlPresent = iEngine->IsControlPresent(aPointerEvent.iPosition);

	                    iEngine->OfferPointerEventL(point, TPointerEvent::EButton1Down);                   
	   
	                    
	                    break; 
	                    
	            case TPointerEvent::EButton1Up: 
	                    if (iTouchType == ETouchStarted) 
	                        { 
	                        iTouchType = ETouchTap; 
	                        
#ifndef __FEATURE_CONTEXT_SENSITIVE_MENU__	                        
	                        if(!iControlPresent)
                       		{
                      		if(iEngine->GetSwfHasVideo())
                       		    {
                                TRect videoRect = iEngine->GetVideoRectL();
                                TBool onVideo = videoRect.Contains(aPointerEvent.iPosition);
                                if( !iEngine->GetFullScreenViaFSCommand() && onVideo )
                                    {
									#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
                                // ToggleScreenL(); Set the flag and call togglescreenL at the end of this function,We must toggle after calling all iEngine->OfferPointerEventL
                                    togglescreen = ETrue;
									#endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN
                                    }
                       		    }
                       		}
#endif	    
	    		    		}  
			    			 			                 
	                    
	                    VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::HandlePointerEventL, TPointerEvent::EButton1Up,(%d, %d)"), aPointerEvent.iPosition.iX, aPointerEvent.iPosition.iY);
	                    
	                    if((iTouchType == ETouchTap) || (iTouchType == ETouchLongTap))
	                        

	                        
	                        iEngine->OfferPointerEventL(iCorePosition, TPointerEvent::EButton1Up);

	                    else
	                        iEngine->OfferPointerEventL(point, TPointerEvent::EButton1Up);
	 
	                    
	                    break; 
	                    
	            case TPointerEvent::EDrag: 

	                    iTouchType = ETouchDrag;                      

	                    VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::HandlePointerEventL, TPointerEvent::EDrag,(%d, %d)"), aPointerEvent.iPosition.iX, aPointerEvent.iPosition.iY);
	                    
	                    iEngine->OfferPointerEventL(point, TPointerEvent::EDrag);
	                    
	                    break; 
	                    
	            default: 
	                    break; 
	            }            
#else
	            if(aPointerEvent.iType==TPointerEvent::EButton1Down)
	              {
	                   iDownPoint = point;
	                   VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::HandlePointerEventL, Event:Down , Point(%d, %d)"), point.iX, point.iY);
	              }
	            
	            
	            if(aPointerEvent.iType == TPointerEvent::EDrag && !controlPresent)
	            {  
	            
	                TPoint offsetdrag = iDownPoint - point;
	                  if(offsetdrag.iX > KStylusDragThreshold || offsetdrag.iX < -KStylusDragThreshold || 
	                                             offsetdrag.iY > KStylusDragThreshold || offsetdrag.iY < -KStylusDragThreshold)
	                        ihaveDragged = ETrue;
	                        
	                  VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::HandlePointerEventL, Event:Drag, Point(%d, %d)"), point.iX, point.iY);
	            }
	                
	                
	     
	            
	            
	            if(aPointerEvent.iType == TPointerEvent::EButton1Up)
	            {  
	               if(!controlPresent && !ihaveDragged) 
			    				{
							     	if(iFLVFile)	
							     		ToggleScreenL();
							     	else if(!iFLVFile && iView->IsLandScape())
							     		ToggleScreenL();
							     	else if (!iFLVFile && !iView->IsLandScape() && iView->IsFullScreenMode())
							    		ToggleScreenL();		     		
			    				}   
	               ihaveDragged = EFalse;
	               
	               VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::HandlePointerEventL, Event:Up, Point(%d, %d)"), point.iX, point.iY);
	            }
	            
	            
	            iEngine->OfferPointerEventL( point, aPointerEvent.iType );
	            
#endif  //FLASH_ENABLE_STRAY_DRAG_SUPPRESSOR    
    
   }
     	
#ifdef FEATURE_INLINE_TEXT_ENTRY    	
	iImeControl = iView->GetIMEControl();

	if(iImeControl)  // check ime session created
	{
#ifdef __FEATURE_TOOLBAR__ 
	/*
	 * this is added for toolbar as the usual way of handle forground event (options menu) is bypassed when panning with toolbar,
	 * if this line is not there the fep will pop up even while panning (when zoomed in from toolbar)
	 */                    
	    if (!GetEngine()->IsPanning())
#endif
	   iImeControl->HandlePointerEventL(aPointerEvent);
	}	
#endif // FEATURE_INLINE_TEXT_ENTRY
	} // if(iTouchFeatureSupported)
	else	// touch not supported
	{
		CCoeControl::HandlePointerEventL(aPointerEvent);
	}
	if(togglescreen)
	{
	 	ToggleScreenL();
	}
}

void CSaFlashPlayerControl::ProvideFeedbackL(const TPointerEvent& aPointerEvent)
{	
	if(iTactileFeedbackSupported) //adding feature manager to check the feature at run time.
	{
		// Provide feedback only if platform supports it. 
//		if( iFeedback && !iFeedback->TouchFeedbackSupported() )
//		{
		//	return; 
//		}
	
		TBool controlPresent = iEngine->IsControlPresent(aPointerEvent.iPosition);
			
		if( controlPresent )
		{
			if(iTactileFeedbackSupported) //adding feature manager to check the feature at run time.
			{
				switch( aPointerEvent.iType )
				{
					case TPointerEvent::EButton1Down:
					{
						iFeedback->InstantFeedback( ETouchFeedbackBasic );
						break;
					}
					case TPointerEvent::EDrag:
					case TPointerEvent::EButtonRepeat:
					{
						iFeedback->InstantFeedback( ETouchFeedbackSensitive );
						break;
					}
				}
			}
		}	
		// following code commented on 23/02/2010 as per the requirement discussed with RM ,not to provide feedback when
        // there are no controls present.		
		/*else
		{
			// When there are no controls, we just need to give sensitive feedback when we raise the stylus. 
			// Also, screen mode will be switched accordingly if it is allowed. 
			#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
			if( aPointerEvent.iType == TPointerEvent::EButton1Up)
			{
				iFeedback->InstantFeedback(ETouchFeedbackSensitive );	
	    	}
			#endif //__FEATURE_UI_LANDSCAPE_FULLSCREEN
		}*/
	}
}

#ifdef __FEATURE_UI_LANDSCAPE_FULLSCREEN
//Function to toggle from normal to full /full to normal
void CSaFlashPlayerControl::ToggleScreenL()
{
    VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::Tap Screen"));
	
	if(!iView->IsFullScreenMode())
	{
		iView->SetFullScreenL();
		HandleFixedToolbarVisibility(EFalse);

	}
	else
	{
		iView->SetAppScreenL();
        HandleFixedToolbarVisibility(ETrue);
	
	}
}

void CSaFlashPlayerControl::HandleFixedToolbarVisibility(TBool aEnable)
{
    if(!iToolbar)
        return;
    
    if(aEnable){
       if(iAvkonAppUi->Orientation()== CAknAppUiBase::EAppUiOrientationLandscape || 
               iAvkonAppUi->Orientation()== CAknAppUiBase::EAppUiOrientationUnspecified)
           iToolbar->SetToolbarVisibility(ETrue);
   }
   else
   {
       if(iAvkonAppUi->Orientation()== CAknAppUiBase::EAppUiOrientationLandscape|| 
                  iAvkonAppUi->Orientation()== CAknAppUiBase::EAppUiOrientationUnspecified)
           iToolbar->SetToolbarVisibility(EFalse, EFalse);
   }  
}

#endif // __FEATURE_UI_LANDSCAPE_FULLSCREEN

void CSaFlashPlayerControl::ResizeOtherControls()
    {
    iVolPopup->ResizeToToggle();
    }
	
//#ifdef __FLASHLITE2.0_REL3.1
void CSaFlashPlayerControl::HandleResourceChange(TInt aType)
{
	TRAPD(result, HandleResourceChangeL(aType));
	if( result != KErrNone)
	{
	}
}

void CSaFlashPlayerControl::HandleResourceChangeInVolumeView(TInt aType)
{
	TRAPD(err, HandleResourceChangeInVolumeViewL(aType));
	
	if (err == KErrNone)
	{
		
	}
}

void CSaFlashPlayerControl::HandleResourceChangeInVolumeViewL(TInt aType)
{
	if ( aType == KEikDynamicLayoutVariantSwitch )
	{
		TRect rect;
		AknLayoutUtils::LayoutMetricsRect(AknLayoutUtils::EMainPane,rect);
		SetRect(rect);		
		IconConstructL();
		//TRAPD(ignore,iEngine->InitializeStandaloneL(rect, ETrue));
		TRAPD(ignore,iEngine->InitializeL(rect, ETrue));
		
		if (ignore == KErrNone)
		{
			
		}
        iEngine->HandleSoftKeyLocationChangedL();
		iNaviPane->Pop();
		if(iMutedIconVisible && iPaused)
		{
			ShowMutedIconL(ETrue);
			ShowPauseIconL(ETrue);
		}
		else if ( iMutedIconVisible )
		{
			iNaviPane->PushL(*iMutedNavi);
		}
		else if ( iPaused )
		{
			ShowPauseIconL(ETrue);
		}
#ifdef __USE_EMPTY_NAVI__
		else
		{
			iNaviPane->PushL(*iEmptyNavi);
		}
#endif	// __USE_EMPTY_NAVI__
	//	iView->HandleResourceChangeL(aType);
	}
}

void CSaFlashPlayerControl::HandleResourceChangeL(TInt aType)
{
	CCoeControl::HandleResourceChange(aType);

	// ADDED FOR SCALABLE UI SUPPORT
	if ( aType==KEikDynamicLayoutVariantSwitch )
	{
		
		TRect rect;
		AknLayoutUtils::LayoutMetricsRect(AknLayoutUtils::EMainPane,rect);
		SetRect(rect);		
#ifdef USE_VPU2
	    if(iVideoSurfaceContainer)
	        iVideoSurfaceContainer->SetRect(rect); 
#endif
		IconConstructL();
		/* EPLU-84GAA5: In case of File browser, EGL surface is 
		 * created & RecreateBitmapL does not recreate the EGL
		 * surface, hence calling InitializeL.
		 */
		iEngine->InitializeL(rect, ETrue);
		if ( iView->IsForeground() )
		{
			//iEngine->InitializeStandaloneL(rect, ETrue);
            //iEngine->InitializeL(rect, ETrue);	
		    iNaviPane->Pop();
			if (iMutedIconVisible && iPaused)
			{
				ShowMutedIconL(ETrue);
				ShowPauseIconL(ETrue);
			}
			else if(iMutedIconVisible)
			{
				iNaviPane->PushL(*iMutedNavi);
			}
			else if(iPaused)
			{
				ShowPauseIconL(ETrue);
			}
#ifdef __USE_EMPTY_NAVI__
			else
			{
				iNaviPane->PushL(*iEmptyNavi);
			}
#endif	// __USE_EMPTY_NAVI__
			iView->HandleResourceChangeL(aType);
			
			iVolPopup->HandleResourceChange(aType);
			
		}
		else
		{
			//iEngine->RecreateBitmapL(rect);
		}

		if(iEngine)
		    {
	     	     iEngine->HandleSoftKeyLocationChangedL();
		    }
		
	}
	
	//If input language is changed by other application/s, then notification for language change
	//is received.Store new language id.
	if (aType == KEikInputLanguageChange)
		{
		CAknSettingCache& cache = CAknEnv::Static()->SettingCache();
		iCurrentInputLanguage = cache.InputLanguage();
		}
  //need to redraw if windows are faded 
     if(aType == KEikMessageWindowsFadeChange)
	    {
			 if(iEngine)
				 iEngine->RefreshScreen();
        } 
	}

void CSaFlashPlayerControl::IconConstructL()
{
    CFbsBitmap* bitmap = NULL; // To be pushed into CleanupStack
    CFbsBitmap* mask = NULL;
	iDispMode = CCoeEnv::Static()->ScreenDevice()->DisplayMode();
#ifdef __USE_EMPTY_NAVI__
	if(iEmptyNavi)
	{
		delete iEmptyNavi;
		iEmptyNavi = NULL;
	}
	// empty navi pane
	CreateEmptyNaviPaneL();
#endif	// __USE_EMPTY_NAVI__
    
    // Init layout
    SizeChanged(); 
	if(iBlinker)
	{
		delete iBlinker;
		iBlinker = NULL;
	}
    // Pause icon
    iBlinker = CPeriodic::NewL(0);

    delete iPausedNavi;
    iPausedNavi = NULL;
    
	AknIconUtils::CreateIconLC(
    	bitmap,
        mask,
        KFlash2UIMifFilePath,	//iconsPath,
        EMbmSaflashQgn_indi_paused, 
        EMbmSaflashQgn_indi_paused_mask );
    
	  if ( AknIconUtils::IsMifIcon( bitmap ) )
	  {
	  	SizeChanged(); // Init layout
	    // Sets the sizes of both bitmap and mask
	    AknIconUtils::SetSize( bitmap, iIconRect.Rect().Size() );
	  }

    iPauseIconMask = mask;
    // Get skinned bitmap
    CFbsBitmap* skinBitmap = AknsUtils::GetCachedBitmap( 
    	AknsUtils::SkinInstance(), KAknsIIDQsnComponentColorBmpCG2 );
    
    if ( skinBitmap )
    {
    	iPauseIcon = skinBitmap;
    	CreatePausedNaviPaneL( );
    }
    else
    {
    	iPauseIcon = bitmap;
    	CreatePausedNaviPaneL( );
    }    
	
	delete iMutedNavi;
    iMutedNavi = NULL;
	AknIconUtils::CreateIconLC(
        bitmap,
        mask,
        KFlash2UIMifFilePath,	
        EMbmSaflashQgn_indi_speaker_muted , 
        EMbmSaflashQgn_indi_speaker_muted_mask  );
    
    if ( AknIconUtils::IsMifIcon( bitmap ) )
    {
	    SizeChanged(); // Init layout
    	// Sets the sizes of both bitmap and mask
    	AknIconUtils::SetSize( bitmap, iBitmapLayout.Rect().Size() ); 
    }

    iMuteIconMask = mask;
	// Get skinned bitmap
    CFbsBitmap* skinBitmap2 = AknsUtils::GetCachedBitmap( 
    	AknsUtils::SkinInstance(), KAknsIIDQsnComponentColorBmpCG2 );
    
    if ( skinBitmap2 )
    {
        iMuteIcon = skinBitmap2;
        CreateMutedNaviPaneL( );
    }
    else
    {
        iMuteIcon = bitmap;
        CreateMutedNaviPaneL( );
    }

    iMuteIcon = bitmap;
    delete iPausedMutedNavi;
    iPausedMutedNavi = NULL;
    
    CreatePausedMutedNaviPaneL(); //The bitmaps have to be in Cleanupstack for this operation
    CleanupStack::PopAndDestroy( 2 ); // bitmap, mask 
    CleanupStack::PopAndDestroy( 2 ); // bitmap, mask 
}

void CSaFlashPlayerControl::SizeChanged() 
{ 
     if(iNaviPane)
     {
    // Pause icon
     	iIconRect.LayoutRect( iNaviPane->Rect(), AknLayout::First_general_event_elements( 1 ) ); 

    // Mute icon
    	iBitmapLayout.LayoutRect( iNaviPane->Rect(), AppLayout::Media_Player_Playback_view_navi_pane_elements_Line_2() ); 
     }
}

#ifdef __USE_EMPTY_NAVI__
void CSaFlashPlayerControl::CreateEmptyNaviPaneL( )
{
    // Create bitmaps size of navi pane
    CFbsBitmap* navibmp = new (ELeave) CFbsBitmap();
    CleanupStack::PushL( navibmp );
    navibmp->Create( iNaviPane->Size(), iDispMode ); 
    CFbsBitmap* navibmpmask = new (ELeave) CFbsBitmap();
    CleanupStack::PushL( navibmpmask );
    navibmpmask->Create( iNaviPane->Size(), iDispMode ); 
        
    // Transfers ownership of bitmaps
    iEmptyNavi = iNaviPane->CreateNavigationImageL( navibmp, navibmpmask );
    CleanupStack::Pop( 2 ); // navibmp, navibmpmask
}
#endif

void CSaFlashPlayerControl::CreateMutedNaviPaneL( )
{    
    // Create bitmaps size of navi pane
    CFbsBitmap* navibmp = new (ELeave) CFbsBitmap();
    CleanupStack::PushL( navibmp );
    navibmp->Create( iNaviPane->Size(), iDispMode ); 
    CFbsBitmap* navibmpmask = new (ELeave) CFbsBitmap();
    CleanupStack::PushL( navibmpmask );
    navibmpmask->Create( iNaviPane->Size(), iDispMode ); 
        
    // Add color to navi pane bitmap
    CFbsBitGc* fbsBitGc = CFbsBitGc::NewL(); //graphic context
    CleanupStack::PushL( fbsBitGc );
    CFbsBitmapDevice* bmpDevice = CFbsBitmapDevice::NewL( navibmp );
    fbsBitGc->Activate( bmpDevice );


    TRgb color( KRgbWhite );
    MAknsSkinInstance* skin = AknsUtils::SkinInstance();

    AknsUtils::GetCachedColor( skin, color, KAknsIIDQsnIconColors,   
        EAknsCIQsnIconColorsCG7 ); 
    fbsBitGc->SetBrushColor( color );
    fbsBitGc->Clear(); // Fill bitmap with color
	if (bmpDevice)
	{
		delete bmpDevice;	
	}
    // Add muted icon (mask) to navi pane bitmap mask
    bmpDevice = CFbsBitmapDevice::NewL( navibmpmask );
    fbsBitGc->Activate( bmpDevice );
    if(AknIconUtils::IsMifIcon( iPauseIcon ))
    {
    	TRect muteBitmapRect(TRect(TPoint(0,0), iMuteIcon ->SizeInPixels()));
    	//fbsBitGc->DrawBitmap( iBitmapLayout.Rect(), iMuteIcon );	
		fbsBitGc->BitBltMasked(iBitmapLayout.Rect().iTl, iMuteIcon  ,
    							  muteBitmapRect, iMuteIconMask,ETrue);
    }
    else
    {
    	fbsBitGc->DrawBitmap( iBitmapLayout.Rect(), iMuteIcon );	
    }
	
    delete bmpDevice;
    CleanupStack::PopAndDestroy();// fbsBitGc
    // Transfers ownership of bitmaps
    iMutedNavi = iNaviPane->CreateNavigationImageL( navibmp, navibmpmask );
    CleanupStack::Pop( 2 ); // navibmp, navibmpmask
 }

// -----------------------------------------------------------------------------
// CSaFlashPlayerControl::CreatePausedMutedNaviPaneL
// -----------------------------------------------------------------------------
//
void CSaFlashPlayerControl::CreatePausedMutedNaviPaneL( )
{      
    // Create bitmaps size of navi pane
    CFbsBitmap* navibmp = new (ELeave) CFbsBitmap();
    CleanupStack::PushL( navibmp );
    navibmp->Create( iNaviPane->Size(), iDispMode ); 
    CFbsBitmap* navibmpmask = new (ELeave) CFbsBitmap();
    CleanupStack::PushL( navibmpmask );
    navibmpmask->Create( iNaviPane->Size(), iDispMode ); 
        
    // Add color to navi pane bitmap
    CFbsBitGc* fbsBitGc = CFbsBitGc::NewL(); //graphic context
    CleanupStack::PushL( fbsBitGc );
    CFbsBitmapDevice* bmpDevice = CFbsBitmapDevice::NewL( navibmp );
    fbsBitGc->Activate( bmpDevice );

    TRgb color( KRgbWhite );
    MAknsSkinInstance* skin = AknsUtils::SkinInstance();

    AknsUtils::GetCachedColor( skin, color, KAknsIIDQsnIconColors,   
        EAknsCIQsnIconColorsCG7 ); 

    fbsBitGc->SetBrushColor( color );
    fbsBitGc->Clear(); // Fill bitmap with color


    delete bmpDevice;
    
    // Add paused and muted icon's (masks) to navi pane bitmap mask
    bmpDevice = CFbsBitmapDevice::NewL( navibmpmask );
    fbsBitGc->Activate( bmpDevice );
    TRect dummyRect = iIconRect.Rect();        
	if( dummyRect.iTl.iX > iBitmapLayout.Rect().iTl.iX )
	{ 
	//Special check for left-to-right read languages
	    dummyRect.iTl.iX = iBitmapLayout.Rect().iBr.iX; 
	//Adjust pause icon
   	    dummyRect.iBr.iX = dummyRect.iTl.iX + iIconRect.Rect().Width();
   	}
    if ( AknIconUtils::IsMifIcon( iPauseIcon ) ) //mif icons don't use masks
	{
		TRect pauseBitmapRect(TRect(TPoint(0,0), iPauseIcon->SizeInPixels()));
		TRect muteBitmapRect(TRect(TPoint(0,0), iMuteIcon->SizeInPixels()));
		//fbsBitGc->DrawBitmap( dummyRect, iPauseIcon );
		fbsBitGc->BitBltMasked(dummyRect.iTl, iPauseIcon ,
    							  pauseBitmapRect, iPauseIconMask,ETrue);
    							  
		fbsBitGc->BitBltMasked(iBitmapLayout.Rect().iTl, iMuteIcon  ,
    							  muteBitmapRect, iMuteIconMask,ETrue);



	}
	else
	{
		fbsBitGc->DrawBitmap( dummyRect, iPauseIconMask );
    	fbsBitGc->DrawBitmap( iBitmapLayout.Rect(), iMuteIconMask );
	}
    delete bmpDevice;
    CleanupStack::PopAndDestroy();// fbsBitGc

	// Transfers ownership of bitmaps
    iPausedMutedNavi = iNaviPane->CreateNavigationImageL( navibmp, navibmpmask );
    CleanupStack::Pop( 2 ); // navibmp, navibmpmask
}

// -----------------------------------------------------------------------------
// CSaFlashPlayerControl::CreatePausedNaviPaneL
// -----------------------------------------------------------------------------
//
void CSaFlashPlayerControl::CreatePausedNaviPaneL( )
{
    // Create bitmaps size of navi pane
    CFbsBitmap* navibmp = new (ELeave) CFbsBitmap();
    CleanupStack::PushL( navibmp );
    navibmp->Create( iNaviPane->Size(), iDispMode ); 
    CFbsBitmap* navibmpmask = new (ELeave) CFbsBitmap();
    CleanupStack::PushL( navibmpmask );
    navibmpmask->Create( iNaviPane->Size(), iDispMode ); 
        
    // Add color to navi pane bitmap
    CFbsBitGc* fbsBitGc = CFbsBitGc::NewL(); //graphic context
    CleanupStack::PushL( fbsBitGc );
    CFbsBitmapDevice* bmpDevice = CFbsBitmapDevice::NewL( navibmp );
    fbsBitGc->Activate( bmpDevice );

    TRgb color( KRgbWhite );
    MAknsSkinInstance* skin = AknsUtils::SkinInstance();

    AknsUtils::GetCachedColor( skin, color, KAknsIIDQsnIconColors,   
        EAknsCIQsnIconColorsCG7 ); 
    fbsBitGc->SetBrushColor( color );
    fbsBitGc->Clear(); // Fill bitmap with color

    delete bmpDevice;

    // Add paused icon (mask) to navi pane bitmap mask
    bmpDevice = CFbsBitmapDevice::NewL( navibmpmask );
    fbsBitGc->Activate( bmpDevice );
#ifdef SYMBIAN_BUILD_GCE 
    if(iDispMode == EColor16MA || iDispMode == EColor16MAP)
    {
        fbsBitGc->SetDrawMode(CGraphicsContext::EDrawModeWriteAlpha);      
        fbsBitGc->SetBrushColor( TRgb(0,0,0,0) );
        //fbsBitGc->SetBrushColor( KRgbTransparent );
        fbsBitGc->SetBrushStyle(CGraphicsContext::ESolidBrush);
        fbsBitGc->Clear(); // Fill bitmap with color    
    }
#endif   
	TRect dummyRect = iIconRect.Rect();        
	if( dummyRect.iTl.iX > iBitmapLayout.Rect().iTl.iX )
	{ 
	//Special check for left-to-right read languages
	    dummyRect.iTl.iX = iBitmapLayout.Rect().iBr.iX; 
	//Adjust pause icon
    	dummyRect.iBr.iX = dummyRect.iTl.iX + iIconRect.Rect().Width();
	}
	
	if ( AknIconUtils::IsMifIcon( iPauseIcon )
#ifdef SYMBIAN_BUILD_GCE
		&& !((iDispMode == EColor16MA || iDispMode == EColor16MAP))
#endif
	)
	{
		TRect bitmapRect(TRect(TPoint(0,0), iPauseIcon->SizeInPixels()));
		fbsBitGc->BitBltMasked(dummyRect.iTl, iPauseIcon ,
    							  bitmapRect, iPauseIconMask,ETrue);
	}
	else
	{
		fbsBitGc->DrawBitmap( dummyRect, iPauseIconMask );
    }
    delete bmpDevice;
    CleanupStack::PopAndDestroy();// fbsBitGc

    // Transfers ownership of bitmaps
    iPausedNavi = iNaviPane->CreateNavigationImageL( navibmp, navibmpmask );
    CleanupStack::Pop( 2 ); // navibmp, navibmpmask
}



TInt CSaFlashPlayerControl::CountComponentControls() const
{
    return 0;
}


CCoeControl* CSaFlashPlayerControl::ComponentControl(TInt /*aIndex*/) const
{
	return NULL; 	

}


// -----------------------------------------------------------------------------
// CSaFlashPlayerControl::ShowMutedIconL
// -----------------------------------------------------------------------------
//
void CSaFlashPlayerControl::ShowMutedIconL(TBool aMutedIconVisible)
{
	iMutedIconVisible = aMutedIconVisible;
	TInt err;
    if ( !aMutedIconVisible)
    {
        iNaviPane->Pop( iMutedNavi );
#ifdef __USE_EMPTY_NAVI__
        iNaviPane->Pop( iEmptyNavi );
#endif	// __USE_EMPTY_NAVI__
        iNaviPane->Pop( iPausedMutedNavi );
    }
    
    if ( iMutedIconVisible && iPaused )
   	{
    		iNaviPane->PushL( *iPausedMutedNavi );
   	}
    else if ( !iPaused && iMutedIconVisible)
   	{
#ifdef __USE_EMPTY_NAVI__
      	err = iNaviPane->ReplaceL( *iEmptyNavi, *iMutedNavi );
    	if( err != KErrNone )
#endif	// __USE_EMPTY_NAVI__
   		{
    			iNaviPane->PushL( *iMutedNavi );
   		}
	}
    else
   	{
    	if ( iPaused )
   		{
    		err = iNaviPane->ReplaceL( *iPausedMutedNavi, *iPausedNavi );
    		if ( err != KErrNone )
   			{
    			iNaviPane->PushL( *iPausedNavi );
   			}
   		}
#ifdef __USE_EMPTY_NAVI__    
    	else if (!iPaused)
   		{
    		err = iNaviPane->ReplaceL( *iMutedNavi, *iEmptyNavi );
    		if ( err != KErrNone)
    		{
    			iNaviPane->PushL( *iEmptyNavi );
    		}
   		}
#endif	// __USE_EMPTY_NAVI__    
    	else
   		{
    		iNaviPane->Pop( iMutedNavi );
   		}
   	}
}
//#ifdef __SIDE_VOLUME_KEYS
void CSaFlashPlayerControl::ShowVolumeNaviPane(const TInt& aVolume)
{
	TRAP_IGNORE(ShowVolumeDisplayL(aVolume));
	// ensure the view has the right volume for unmuting
	if(!aVolume)
	{
		
		iEngine->SetVolume(KMuteMask);//if condition added only for s605.0 for mute  
	}
	else
	iEngine->SetVolume(aVolume);
}
  
void CSaFlashPlayerControl::ShowVolumeDisplayL(const TInt& aVolume)	
{
	const TInt KMinVolume(0);  //CAknVolumeControl's min value
	const TInt KMaxVolume(10); //CAknVolumeControl's max value

	// if we're not mute:
	if ( aVolume > KMinVolume )
	{
		// do we have to hide the mute icons?
		if ( iMutedIconVisible )
		{
			ShowMutedIconL(EFalse);
		}
		if(iTouchFeatureSupported) //adding feature manager to check the feature at run time.
		{
			//order changed for 5.0 showvolumeiconl called before setvalue() 
			if ( !iVolumeIconVisible )  
			{
				ShowVolumeIconL(ETrue);
			}
			
			if ( aVolume <= KMaxVolume )
			{
				(static_cast<CAknVolumeControl*>(iVolumeNavi->DecoratedControl()))->SetValue(aVolume);
			}
		}
		else
		{
			// change the volume
			if ( aVolume <= KMaxVolume )
			{
				(static_cast<CAknVolumeControl*>(iVolumeNavi->DecoratedControl()))->SetValue(aVolume);
			}
			
			// display it
			if ( !iVolumeIconVisible )
			{
				ShowVolumeIconL(ETrue);
			}
		}
		// start the timer
		iVolumeDisplay->StartVolumeTimer();
	}
	
	else // ( aVolume <= KMinVolume )
	//else if ( !iMutedIconVisible )
	{
		if(iTouchFeatureSupported) //adding feature manager to check the feature at run time.
		{
			ShowVolumeIconL(ETrue);
			if ( aVolume <= KMaxVolume )
			{
				(static_cast<CAknVolumeControl*>(iVolumeNavi->DecoratedControl()))->SetValue(aVolume);
			}
			iVolumeDisplay->StartVolumeTimer();
		}
		else
		{
			if ( !iMutedIconVisible )
			 {
				// we are just mute: cancel the timer, hide the volume and show the mute icon
				iVolumeDisplay->Cancel();
				// special case: if the volume had been at 1 and the pause icon is visible,
				// it's possible to enter this state without the volume icon appearing at all
				if ( iVolumeIconVisible )
				{
					ShowVolumeIconL(EFalse);
				}
				ShowMutedIconL(ETrue);
			 }
		}
	} // if ( aVolume > KMinVolume )
	
}  
void CSaFlashPlayerControl::ShowVolumeIconL(TBool aVolumeIconVisible)
{
#ifdef __USE_EMPTY_NAVI__
	TInt err = KErrNone;
#endif
	
	if ( aVolumeIconVisible )
	{
		// we want to display it. The Mute icon cannot be present!
		if ( iPauseIconVisible )
		{
			// switch this for the volume
			iNaviPane->ReplaceL( *iPausedNavi,*iVolumeNavi );
		  	iPauseIconVisible = EFalse;
		}
		else
		{
#ifdef __USE_EMPTY_NAVI__
			// currently empty
			err = iNaviPane->ReplaceL( *iEmptyNavi, *iVolumeNavi );
		}
	  	if( err != KErrNone )
	  	{
#endif	// __USE_EMPTY_NAVI__
	  		iNaviPane->PushL( *iVolumeNavi );
	  	}
	}
	else
	{
		// we want to hide it: at this point, we are not showing the mute
		// icon (that will come later) or the pause icon (that will come
		// at the next timeout). So just put up the empty pane
#ifdef __USE_EMPTY_NAVI__
	  	iNaviPane->ReplaceL( *iVolumeNavi, *iEmptyNavi ) ;
	  	if( err != KErrNone )
	  	{
	  		iNaviPane->PushL( *iEmptyNavi );
	  	}
#else
	  	iNaviPane->Pop( iVolumeNavi ) ;
#endif	// __USE_EMPTY_NAVI__
	}
	iVolumeIconVisible = aVolumeIconVisible;

}
//#endif  //__SIDE_VOLUME_KEYS
// -----------------------------------------------------------------------------
// CSaFlashPlayerControl::ShowPauseIconL
// -----------------------------------------------------------------------------
//
void CSaFlashPlayerControl::ShowPauseIconL(TBool aPauseIconVisible)
{
    if (iBlinker)
    {
#ifdef __FEATURE_TOOLBAR__
    //icon on toolbar needs to be changed too
    iView->ChangeToolbarPauseIcon(aPauseIconVisible);
#endif
        if ( !aPauseIconVisible )
        {
            iNaviPane->Pop( iPausedNavi );
#ifdef __USE_EMPTY_NAVI__
            iNaviPane->Pop( iEmptyNavi );
#endif	// __USE_EMPTY_NAVI__
            iNaviPane->Pop( iPausedMutedNavi );
            if(iMutedIconVisible) 
            {
            	iNaviPane->PushL(*iMutedNavi);
            }
#ifdef __USE_EMPTY_NAVI__
            else
            {
            	iNaviPane->PushL(*iEmptyNavi);
            }
#endif	// __USE_EMPTY_NAVI__
        }
        
        iPauseIconVisible = aPauseIconVisible;
        iPaused = aPauseIconVisible;
        const TInt KSaFlashOneSecond(1000000);
        if (aPauseIconVisible && !iBlinker->IsActive())
        {
            iBlinker->Start( KSaFlashOneSecond/2, KSaFlashOneSecond/2, 
                    TCallBack( BlinkPauseIconL, this ) );
        }
        if (!aPauseIconVisible && iBlinker->IsActive())
        {
            iBlinker->Cancel();
        }
    }
}

void CSaFlashPlayerControl::CancelPauseBlinker()
{
	if(iPauseIconVisible) //&& iBlinker->IsActive() removed sometimes
	{//cancel with force
		iBlinker->Cancel();
	}
}

void CSaFlashPlayerControl::ResumePauseBlinker()
{
	if (iPauseIconVisible && !iBlinker->IsActive())
        {
        	const TInt KSaFlashOneSecond(1000000);
            iBlinker->Start( KSaFlashOneSecond/2, KSaFlashOneSecond/2, 
                    TCallBack( BlinkPauseIconL, this ) );
        }
}

void CSaFlashPlayerControl::CancelHwKeyObserver()
{
	iHwKeyProperty->RemoveObserver(this);
}

void CSaFlashPlayerControl::AddHwKeyObserverL()
{
	iHwKeyProperty->AddObserverL(this);
}

void CSaFlashPlayerControl::CloseVolPopUp()
{
	if(iVolPopup)
	{
		iVolPopup->CloseVolPopUp();
	}
}

// -----------------------------------------------------------------------------
// CSaFlashPlayerControl::BlinkPauseIconL
// -----------------------------------------------------------------------------
//
TInt CSaFlashPlayerControl::BlinkPauseIconL(TAny* aPtr)
{
    static_cast<CSaFlashPlayerControl*>(aPtr)->DoBlinkPauseIconL();
    return KErrNone;
}

// -----------------------------------------------------------------------------
// CSaFlashPlayerControl::DoBlinkPauseIconL
// -----------------------------------------------------------------------------
//
void CSaFlashPlayerControl::DoBlinkPauseIconL()
{
    TInt err;
//#ifdef __SIDE_VOLUME_KEYS
if (iSideVolKeysSupported)
	{
	if ( iVolumeIconVisible )
		{
		return;
		}
	}
//#endif //__SIDE_VOLUME_KEYS
    if( iMutedIconVisible && iPauseIconVisible )
    {
    	iNaviPane->PushL( *iPausedMutedNavi );
    }
    else if( iPauseIconVisible && !iMutedIconVisible)
    {
#ifdef __USE_EMPTY_NAVI__
      	err = iNaviPane->ReplaceL( *iEmptyNavi, *iPausedNavi );
    	if( err != KErrNone )
#endif	// __USE_EMPTY_NAVI__
    	{
    		iNaviPane->PushL( *iPausedNavi );
    	}
   	}
    else
    {
    	if( iMutedIconVisible )
    	{
    		err = iNaviPane->ReplaceL( *iPausedMutedNavi, *iMutedNavi );
    		if( err != KErrNone )
    		{
    			iNaviPane->PushL( *iMutedNavi );
    		}
    	}
#ifdef __USE_EMPTY_NAVI__
    	else if(!iMutedIconVisible)
    	{
    		err = iNaviPane->ReplaceL( *iPausedNavi, *iEmptyNavi );
    		if( err != KErrNone)
    		{
    			iNaviPane->PushL( *iEmptyNavi );
    		}
    	}
#endif	// __USE_EMPTY_NAVI__
    	else
    	{
    		iNaviPane->Pop( iPausedNavi );
    	}
    }
    iPauseIconVisible = !iPauseIconVisible;
}

//#endif //__FLASHLITE2.0_REL3.1

void CSaFlashPlayerControl::Draw( const TRect& aRect)  const
{
    CWindowGc& gc = SystemGc();
    
    if( iErrorOccured )
    {
        VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::Draw ++"));
        gc.SetBrushColor(KRgbWhite);
        gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
        gc.Clear(Rect());
        
        TSize bitmapSize = iErrorImageBitmap->SizeInPixels();
        
        /*TInt xPos = Rect().iTl.iX + (Size().iWidth / 2) - (bitmapSize.iWidth /2);
        TInt yPos = Rect().iTl.iY + (Size().iHeight / 2) - (bitmapSize.iHeight /2);
        
        TPoint centerPoint(xPos, yPos);*/
        TInt yPos = Rect().iTl.iY;
        TPoint TopLeft(0, yPos);
        TRect  bitmapRect(0, 0, bitmapSize.iWidth, bitmapSize.iHeight);        
        
        gc.BitBltMasked(TopLeft, iErrorImageBitmap, bitmapRect, iErrorImageBitmapMask, ETrue);

#ifdef SYMBIAN_BUILD_GCE
        ControlEnv()->WsSession().Finish();
#endif        
        return;
    }    

	if ( iEngine )
	{
		CWindowGc& gc = SystemGc();
		iEngine->Draw(aRect, gc);
		
#ifdef SYMBIAN_BUILD_GCE
        ControlEnv()->WsSession().Finish();
#endif        
	}
}

void CSaFlashPlayerControl::SimulateIMEEventL()
	{
	TKeyEvent keyEvent;
	keyEvent.iCode      = 0;
	keyEvent.iScanCode  = EStdKeyDevice3;
	keyEvent.iModifiers = 0;
	keyEvent.iRepeats   = 0;   

	if( !iEngine )
	return;

	iEngine->OfferKeyEventL(keyEvent, EEventKey);
	}
/*
The FCC must route keyboard navigation between the FCC GUI, Active FCA, and the FCC itself.
*/
TKeyResponse CSaFlashPlayerControl::OfferKeyEventL( const TKeyEvent& aKeyEvent, TEventCode aType )
{
	const TKeyEvent& keyEvent = aKeyEvent;

	TKeyResponse response = EKeyWasNotConsumed;

	if ( iEngine )
	{
#ifdef ENABLE_ZOOM_STRIP_SUPPORT

        switch(keyEvent.iScanCode)
        {
        case EStdKeyApplicationC: //ZoomIn
            
            if(aType==EEventKey && (!iView->IsVideoContent()) && (!iEngine->IsBrowserPluginActive()))
            {
                iView->HandleCommandL(EEikFlashCmdZoomIn);
                response = EKeyWasConsumed;
            }
            break;

        case EStdKeyApplicationD: //ZoomOut
            
            if(aType==EEventKey && (!iView->IsVideoContent()) && (!iEngine->IsBrowserPluginActive()))
            {
                iView->HandleCommandL(EEikFlashCmdZoomOut);
                response = EKeyWasConsumed;
            }
            break;

        default:
#endif
            response = iEngine->OfferKeyEventL( keyEvent, aType );

#ifdef ENABLE_ZOOM_STRIP_SUPPORT
  			break;
        }
#endif



#ifdef FEATURE_INLINE_TEXT_ENTRY
	// this causes the editor content to be fetched by the FEP
		if(GetIMEControl() && response && aType == EEventKey)
		{
			GetIMEControl()->ReportStateEventL();
		}
#ifdef VIRTUAL_DOCUMENT_FOR_FEP
		if(IMEControl)
		{
			IMEControl->ResetIfEditorChanged( keyEvent, aType );
		}
#endif // VIRTUAL_DOCUMENT_FOR_FEP
#endif // FEATURE_INLINE_TEXT_ENTRY

	}
	
	if( (keyEvent.iScanCode == EStdKeyDevice1 || keyEvent.iScanCode == EStdKeyDevice0 ) 
		&& aType == EEventKeyUp )
	{
        if(iEngine)
            iEngine->UpdateVideoScreen();
	}

	return response;
}

#if defined (__FEATURE_FLV_RECOG_PLAY) || defined (__FEATURE_F4V_RECOG_PLAY)

const TUint KDriveLetterLength = 2;

TBool CSaFlashPlayerControl::GetVideoPlayerPath(TDes& aFileName)
{
    TFileName rscfn; //cautious symbian does not recommend auto var for TFileName
    TFileName dllName;
    Dll::FileName(dllName);
    TBuf<KDriveLetterLength> drive = dllName.Left(KDriveLetterLength);
    
    CEikonEnv::Static()->FsSession().PrivatePath(aFileName);   
    aFileName.Append(KVideoPlayerSwf());
    
    rscfn.Copy(drive); // get the drive letter
    rscfn.Append(aFileName);
    
    TBool fileExists( BaflUtils::FileExists(CCoeEnv::Static()->FsSession(), rscfn) );
    
    if ( !fileExists )
    { //not found , so find it from ROM drive
        return EFalse;
    }
    
    aFileName.Copy(rscfn);
    
    return ETrue;   
}

#endif
void CSaFlashPlayerControl::ProcessFileL(const TDesC& aFilename)
{
	switch(FlashUiUtil::GetFileTypeL(aFilename))
	{
		case 1 : //swf file
				{
					#ifdef __FEATURE_FLV_RECOG_PLAY		
						iFLVFile = EFalse;
					#endif
                    #ifdef __FEATURE_F4V_RECOG_PLAY     
                        iF4VFile = EFalse;
					#endif
					
					iEngine->OpenFileAndStartMovieL(aFilename, ETrue);	
				}
				break;
		
		#ifdef __FEATURE_FLV_RECOG_PLAY		
		case 2 : //flv file
				{
					iFLVFile = ETrue;
					
					CDesCArray* ArgName = new(ELeave) CDesCArrayFlat(1);
					
					CleanupStack::PushL(ArgName);
					CDesCArray* ArgVal = new(ELeave) CDesCArrayFlat(1);
										
					CleanupStack::PushL(ArgVal);
					
					ArgName->AppendL(KFlashVars());
					
					HBufC* temp = HBufC::NewL(KMaxDataTypeLength);
					CleanupStack::PushL(temp);
					TPtr tmpPtr(temp->Des());
					
					tmpPtr.Copy(KVideoName());
					tmpPtr.Append(aFilename);
					ArgVal->AppendL(tmpPtr);
					
					HBufC* flvPlayr = HBufC::NewL(KMaxDataTypeLength);;
					CleanupStack::PushL(flvPlayr);
					TPtr flpPlayerPtr(flvPlayr->Des());
					TBool flvPlayerFound = GetVideoPlayerPath(flpPlayerPtr);
					
					if(flvPlayerFound && iEngine)
					{
						iEngine->ProcessParameters(ArgName,ArgVal);
					
						iEngine->OpenFileAndStartMovieL(flpPlayerPtr, ETrue);	
						
					}
					
					CleanupStack::PopAndDestroy(flvPlayr);
					CleanupStack::PopAndDestroy(temp);
					CleanupStack::PopAndDestroy(ArgVal);
					CleanupStack::PopAndDestroy(ArgName);
				}
				break;
		#endif
        #ifdef __FEATURE_F4V_RECOG_PLAY     
        case 3 : //f4v file
                {
                    iF4VFile = ETrue;
                    
                    CDesCArray* ArgName = new(ELeave) CDesCArrayFlat(1);
                    
                    CleanupStack::PushL(ArgName);
                    CDesCArray* ArgVal = new(ELeave) CDesCArrayFlat(1);
                                        
                    CleanupStack::PushL(ArgVal);
                    
                    ArgName->AppendL(KFlashVars());
                    
                    HBufC* temp = HBufC::NewL(KMaxDataTypeLength);
                    CleanupStack::PushL(temp);
                    TPtr tmpPtr(temp->Des());
                    
                    tmpPtr.Copy(KVideoName());
                    tmpPtr.Append(aFilename);
                    ArgVal->AppendL(tmpPtr);
                    
                    HBufC* f4vPlayr = HBufC::NewL(KMaxDataTypeLength);;
                    CleanupStack::PushL(f4vPlayr);
                    TPtr flpPlayerPtr(f4vPlayr->Des());
                    TBool f4vPlayerFound = GetVideoPlayerPath(flpPlayerPtr);
                    
                    if(f4vPlayerFound && iEngine)
                    {
                        iEngine->ProcessParameters(ArgName,ArgVal);
                    
                        iEngine->OpenFileAndStartMovieL(flpPlayerPtr, ETrue);   
                        
                    }
                    
                    CleanupStack::PopAndDestroy(f4vPlayr);
                    CleanupStack::PopAndDestroy(temp);
                    CleanupStack::PopAndDestroy(ArgVal);
                    CleanupStack::PopAndDestroy(ArgName);
                }
                break;
        #endif
						
	}
	
}

void CSaFlashPlayerControl::OpenFileL(const TDesC& aFilename)
{
	if ( !iEngine->IsInitialized() )
	{
		//TRAPD(result, iEngine->InitializeStandaloneL(Rect(), EFalse));
		
        TRAPD(result, iEngine->InitializeL(Rect(), EFalse));
        if ( result == KErrNoMemory )
		{
			User::Leave(result);
			return;
		}
	}

	//iEngine->OpenFileAndStartMovieL(aFilename, ETrue);
	ProcessFileL(aFilename);
	
	DrawDeferred ( );
}

void CSaFlashPlayerControl::ProcessFileL(RFile& aFile)
{
	switch(FlashUiUtil::GetFileTypeL(aFile))
	{
		case 1 : //swf file
				{
					#ifdef __FEATURE_FLV_RECOG_PLAY		
					iFLVFile = EFalse;
					#endif
                    #ifdef __FEATURE_F4V_RECOG_PLAY     
                    iF4VFile = EFalse;
					#endif
					iEngine->OpenFileHandleAndStartMovieL(aFile, ETrue);	
				}
				break;
		
		#ifdef __FEATURE_FLV_RECOG_PLAY		
		case 2 : //flv file
				{
		
					iFLVFile = ETrue;
			
					CDesCArray* ArgName = new(ELeave) CDesCArrayFlat(1);
					
					CleanupStack::PushL(ArgName);
					CDesCArray* ArgVal = new(ELeave) CDesCArrayFlat(1);
										
					CleanupStack::PushL(ArgVal);
					
					ArgName->AppendL(KFlashVars());
					
					HBufC* temp = HBufC::NewL(KMaxDataTypeLength);
					CleanupStack::PushL(temp);
					TPtr tmpPtr(temp->Des());
					//here append RFile
					tmpPtr.Copy(KVideoNameRFile());
					
					ArgVal->AppendL(tmpPtr);
					
					HBufC* flvPlayr = HBufC::NewL(KMaxDataTypeLength);;
					CleanupStack::PushL(flvPlayr);
					TPtr flpPlayerPtr(flvPlayr->Des());
					TBool flvPlayerFound = GetVideoPlayerPath(flpPlayerPtr);
					
					if(iEngine)
					{
						iEngine->ProcessParameters(ArgName,ArgVal);
					
						iEngine->OpenFileAndStartMovieL(flpPlayerPtr, ETrue);	
						
					}
					
					CleanupStack::PopAndDestroy(flvPlayr);
					CleanupStack::PopAndDestroy(temp);
					CleanupStack::PopAndDestroy(ArgVal);
					CleanupStack::PopAndDestroy(ArgName);
				}
				break;
		#endif
        #ifdef __FEATURE_F4V_RECOG_PLAY     
        case 3 : //f4v file
                {
        
                    iF4VFile = ETrue;
            
                    CDesCArray* ArgName = new(ELeave) CDesCArrayFlat(1);
                    
                    CleanupStack::PushL(ArgName);
                    CDesCArray* ArgVal = new(ELeave) CDesCArrayFlat(1);
                                        
                    CleanupStack::PushL(ArgVal);
                    
                    ArgName->AppendL(KFlashVars());
                    
                    HBufC* temp = HBufC::NewL(KMaxDataTypeLength);
                    CleanupStack::PushL(temp);
                    TPtr tmpPtr(temp->Des());
                    //here append RFile
                    tmpPtr.Copy(KVideoNameRFile());
                    
                    ArgVal->AppendL(tmpPtr);
                    
                    HBufC* f4vPlayr = HBufC::NewL(KMaxDataTypeLength);;
                    CleanupStack::PushL(f4vPlayr);
                    TPtr flpPlayerPtr(f4vPlayr->Des());
                    TBool f4vPlayerFound = GetVideoPlayerPath(flpPlayerPtr);
                    
                    if(iEngine)
                    {
                        iEngine->ProcessParameters(ArgName,ArgVal);
                    
                        iEngine->OpenFileAndStartMovieL(flpPlayerPtr, ETrue);   
                        
                    }
                    
                    CleanupStack::PopAndDestroy(f4vPlayr);
					CleanupStack::PopAndDestroy(temp);
					CleanupStack::PopAndDestroy(ArgVal);
					CleanupStack::PopAndDestroy(ArgName);
				}
				break;
		#endif
						
	}
	
}
void CSaFlashPlayerControl::OpenFileL(RFile& aFile)
{
	if ( !iEngine->IsInitialized() )
	{
		//TRAPD(result, iEngine->InitializeStandaloneL(Rect(), EFalse));
        TRAPD(result, iEngine->InitializeL(Rect(), EFalse));	
        if ( result == KErrNoMemory )
		{
			//iView->NotifyClientError(EInsufficientMemoryError, ETrue, EFalse);
			User::Leave(result);
			return;
		}
	}

	ProcessFileL(aFile);
	
	DrawDeferred ( );
}


TInt HandleFocusResumeL(TAny *aEngine)
{
	((MFlashEngine*)aEngine)->ResumeL(EReasonFocus, EFalse);
	return 0;
}

void CSaFlashPlayerControl::FocusChanged(TDrawNow /*aDrawNow*/)
{
	// The error handler class properly pauses and resumes the Flash Engine,
	// so we don't need to do anything here if we are displaying an error.
	// This function only pauses the Flash Engine when we lose focus
	// due to a menu display or system dialog.
	if (!iView->DisplayingError()
#ifdef FEATURE_INLINE_TEXT_ENTRY
	// Adding CSaFlashPlayerIMEControl to control stack in
	// CSaFlashPlayerView::EnableIMEL() would cause this function
	// to be called by the OS. We don't want to stop the player while inside
	// the text field so that DoPlay gets called and cursor blinks.
		&& !iIsActivatingIME
#endif // FEATURE_INLINE_TEXT_ENTRY
		)
	{
/*		TBool focus = IsFocused();
		if (focus && !iFocusHandler->IsActive())
		{
			iFocusHandler->Start(TCallBack(HandleFocusResumeL, iEngine));
		}
		else
		{
			CancelFocusHandler();
			iEngine->Pause(EReasonFocus);
		}*/
		TBool focus = IsFocused();
		
		if(iEngine)
			iEngine->SetForeground(focus);
	}
}


//update  volume  for touch 
void CSaFlashPlayerControl::HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType)
	{	
	
	VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::HandleControlEventL(CCoeControl*, TCoeEvent)"));
		
		
	if (iVolumeNavi && (iVolumeNavi == aControl) && (aEventType == MCoeControlObserver::EEventStateChanged))
		{
		
		/***
		CAknVolumeControl* volumeControl =(static_cast<CAknVolumeControl*>(iVolumeNavi->DecoratedControl()));
		TInt  newVal=0;
		
	
		newVal=volumeControl->Value();
		if(!newVal)
			{
		
			iView->HandleMuteL(ETrue);
			
			}
		else 
			{
			iView->SetVolumeL(newVal);
			iEngine->SetVolume(newVal);
			
			}
			***/
			
			
			/////////////////////////////////////////////////////////////////////////////////////////////////
			{
				VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::ShowVolumeDisplayL(const TInt& ),Make Volume Icon Visible"));
				
				//ShowVolumeIconL(ETrue);
				Window().Invalidate((static_cast<CAknVolumeControl*>(iVolumeNavi->DecoratedControl()))->Rect());
				(static_cast<CAknVolumeControl*>(iVolumeNavi->DecoratedControl()))->MakeVisible(ETrue);
				
				VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerControl::ShowVolumeDisplayL(const TInt& ),Draw Volume Icon"));
								
				(static_cast<CAknVolumeControl*>(iVolumeNavi->DecoratedControl()))->DrawNow();
				
			}
			/////////////////////////////////////////////////////////////////////////////////////////////////
			
									
		}	
	}
	
	
	
void CSaFlashPlayerControl::ShowVolumePopupL(TInt aCurrentVolume)
{
    //Show the volume popup only if control in focus
    if(IsFocused())
    {    
        TRAP_IGNORE(iVolPopup->ShowPopupL(aCurrentVolume));	
    }
}
	
	
void CSaFlashPlayerControl::MHandleVolumeCmdL(TInt aNewValue)
{

	VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerView::MHandleVolumeCmdL(TInt)"));
	
	TInt currentVolume = 0, oldVal=0;
	
	if(iCenRepos)
	{
		iCenRepos->Get(KFlashliteVwrVolume, currentVolume);
	}
	else
	{
		currentVolume = iView->GetVolume();
	}
	
	if(IsVolumeMuted(currentVolume))
	{
		currentVolume &= KUnMuteMask;
	}
	
	if(currentVolume == 0 && aNewValue == EVolumeControlCmdVolumeDownByOne)
	{
		ShowVolumePopupL(KMinVolume);
		return;
	}
	

	VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerView::MHandleVolumeCmdL(TInt),Current Volume %d"), currentVolume);
    
    oldVal =  currentVolume;//keep data in old value
    
    //make sure its in range
    currentVolume = (currentVolume > KMaxVolume)? KMaxVolume:currentVolume;	
   
    //in mute case the value goes to negative
    //this mute case is screwed up by both ( UI specifier and dev)
    //in mute case if h/w key pressed it will start from min value
    currentVolume = (currentVolume < KMinVolume)? KMinVolume:currentVolume;	
    
    switch(aNewValue)
    {	
    	case EVolumeControlCmdVolumeUpByOne:
        {
        	if((currentVolume< KMaxVolume))
        	    {
                ++currentVolume;
        	    }
        	else
        	    currentVolume = KMaxVolume;
        	VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerView::MHandleVolumeCmdL(TInt), Volume++ %d"), currentVolume);
        }
        break;
        
        case EVolumeControlCmdVolumeDownByOne:
        {
        	if((currentVolume> (KMinVolume)))
        	    {
                --currentVolume;
        	    }
        	else
        	    currentVolume = KMinVolume;
        				
         	VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerView::MHandleVolumeCmdL(TInt), Volume-- %d"), currentVolume);	
        }
    	break;
    	   	
       	default:
		if((aNewValue >= KMinVolume) && 
			(aNewValue < KMaxVolume))
		{
			currentVolume = aNewValue;

		}
        break;     
    
    }

	ShowVolumePopupL(currentVolume);

    //has anything changed at all
    if(oldVal !=  currentVolume)
    { //set it only if changed
    
        VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerView::MHandleVolumeCmdL(TInt), Change in Volume"));
 		if(currentVolume == 0)
 		{
 			currentVolume |= KMuteMask;
			iView->Mute(ETrue);
		}
    }
    
    iView->SetVolumeL(currentVolume);

}

void CSaFlashPlayerControl::MVolumeChanged(TInt aNewValue)
{
	VwrDebug(LOG_DEBUG,_L("CSaFlashPlayerView::MVolumeChanged(TInt)"));
	
	TBool wasMuted = EFalse;
	
	if(iView->IsVolumeMuted() && aNewValue == 0)
	{
		return;
	}

	TInt currentVolume = 0;
	

	if(iCenRepos)
	{
		iCenRepos->Get(KFlashliteVwrVolume, currentVolume);
	}
	else
	{
		currentVolume = iView->GetVolume();
	}
	
	if(IsVolumeMuted(currentVolume))
	{
		currentVolume &= KUnMuteMask;
		wasMuted = ETrue;
	}

	//This is to handle mute from options menu and unmutting by tapping on the 
	//volume icon on the volume bar
	if(wasMuted && aNewValue == 1)
	{
		TRAP_IGNORE(iView->SetVolumeL(currentVolume));
		TRAP_IGNORE(ShowVolumePopupL(currentVolume));	
		return;
	}
	
	if((aNewValue >= KMinVolume) && 
       (aNewValue <= KMaxVolume) && !((aNewValue == 0) && currentVolume > 1))
    {
       	 currentVolume = aNewValue;
    }
    
#if defined(__WINS__) //offset required on the emulator
    //currentVolume += WINS_DEFAULT_VOLUME;
#endif  

    if(aNewValue == 0)
    {
		currentVolume |= KMuteMask;
    }

	TRAP_IGNORE(iView->SetVolumeL(currentVolume));
	
	return;
}

TBool CSaFlashPlayerControl::IsVolumeMuted(TInt aVolume)
{
	if(aVolume & KMuteMask)
	{
		return ETrue;
	}
	else
	{
		return EFalse;	
	}
}

void CSaFlashPlayerControl::MHandleVolumeRectRedraw()
{
	return;
}

void CSaFlashPlayerControl::MPausePlayKeyPressed()
{
	if(iEngine)	
	{
		if(iEngine->IsRunning())
		{
			if(iEngine->IsPaused())
			{
				TRAP_IGNORE(iView->HandleCommandL(EEikFlashCmdResume));	
			}
			else
			{
				TRAP_IGNORE(iView->HandleCommandL(EEikFlashCmdPause));
			}	
		}
		else
		{
			TRAP_IGNORE(iView->HandleCommandL(EEikFlashCmdPlay));
		}
	}
		
}

	  
void CSaFlashPlayerControl::MStopKeyPressed()
{
	TRAP_IGNORE(iView->HandleCommandL(EEikFlashCmdStop));
}

	  
void CSaFlashPlayerControl::MRewindKeyPressed()
{
	//handle through option menu way
	TRAP_IGNORE(iView->HandleCommandL(EEikFlashCmdRewind));
}

	
void CSaFlashPlayerControl::MForwardKeyPressed()
{
	
}

			
void CSaFlashPlayerControl::MBackwardKeyPressed()
{
	
}

			
void CSaFlashPlayerControl::MFastForwardKeyPressed()
{
	
}

RWindow* CSaFlashPlayerControl::GetNewVideoWindow(void* /*aClient*/)
{
    RWindow* videoWindow = NULL;

#ifdef USE_VPU2
    if(!iVideoSurfaceContainer && iEngine && !iEngine->IsSwitchingToSWRenderingInPrgress() &&iEngine->IsOpenVgRenderingMode())
    {
        TRAP_IGNORE(
        iVideoSurfaceContainer = CSurfaceContainerControl::NewL((RWindowTreeNode*)&(iCoeEnv->RootWin()),TRect(GetWindow()->Position(),GetWindow()->Size()));
        iVideoSurfaceContainer->GetWindow().SetOrdinalPosition(-1, ECoeWinPriorityNormal-1); 
        );
    }
    
    if(iVideoSurfaceContainer)
        videoWindow = &(iVideoSurfaceContainer->GetWindow());
#endif

    if(videoWindow == NULL)
        videoWindow = GetWindow(); 
                
    return videoWindow;
}

void CSaFlashPlayerControl::CloseVideoWindow(void* /*aClient*/)
{
#ifdef USE_VPU2
    if(iVideoSurfaceContainer)
    {
        delete iVideoSurfaceContainer;
        iVideoSurfaceContainer = NULL;
    }
#endif    
}

void CSaFlashPlayerControl::CancelFocusHandler()
{
	iFocusHandler->Cancel();
}
#ifdef __FEATURE_FLV_RECOG_PLAY		

TBool CSaFlashPlayerControl::IsFlvFilePlaying()
{
	return iFLVFile;
}

#endif

#ifdef __FEATURE_F4V_RECOG_PLAY     

TBool CSaFlashPlayerControl::IsF4vFilePlaying()
{
    return iF4VFile;
}

#endif

#ifdef __FEATURE_OSK__
TKeyResponse CSaFlashPlayerControl::OfferOverlayKeyEvent(const TKeyEvent& aKeyEvent, TEventCode aType)
{
    TKeyResponse response = EKeyWasNotConsumed;
    TRAP_IGNORE(response = OfferKeyEventL(aKeyEvent,aType));
    return response;
}
#endif // __FEATURE_OSK__

#ifdef FEATURE_INLINE_TEXT_ENTRY
CSaFlashPlayerBaseIMEControl* CSaFlashPlayerControl::GetIMEControl()
{
	if(iView)
	{
		return iView->GetIMEControl();
	} else
	{
		return NULL;
	}
}
#endif // FEATURE_INLINE_TEXT_ENTRY

void CSaFlashPlayerControl::VideoInPlay()
{
}

void CSaFlashPlayerControl::VideoNotInPlay()
{
}

#ifdef FEATURE_INLINE_TEXT_ENTRY

//constructor
CInputModeSelector::CInputModeSelector(CSaFlashPlayerControl* aPlayerControl):iInitialString(0), iLen(0),iPlayerControl(aPlayerControl)
{
	
}


//Input mode selector
CInputModeSelector* CInputModeSelector::NewL(AMP_MTextNavigation* pNav, CSaFlashPlayerControl* aPlayerControl)
{
	CInputModeSelector* self = new (ELeave)	CInputModeSelector(aPlayerControl);
	CleanupStack::PushL(self);
	self->ConstructL(pNav);
	CleanupStack::Pop(self);
	return self;
}
//ConstructL
void CInputModeSelector::ConstructL(AMP_MTextNavigation* pNav)
{
	long len = pNav->GetTextLength();
	long maxLength = pNav->GetMaxChars();

	AMP_ReturnString ampString(FI_UTF16EncodingForm);
	FI_ReturnString* fiReturnString = ampString.Wrapper();
	FI_Result res =  pNav->GetText(0, len, fiReturnString);

	if(res == FI_ResultOK)
		{
		const FI_UTF16Char* chars = ampString.GetTemporaryUTF16String();
		if(chars)
			{
			iInitialString = new (ELeave)FI_UTF16Char[len+1];
			if(iInitialString)
				{
				FlashMemCpy(iInitialString, chars,(len+1) * sizeof(FI_UTF16Char));
				iLen=len;
				}
			}
		}

}

CInputModeSelector::~CInputModeSelector()
{
	if (iInitialString)
		delete [ ] iInitialString;
}


TBool CInputModeSelector::IsInComplexLanguageMode()
{
/*
	CAknEdwinState *inputState= new (ELeave) CAknEdwinState();
	inputState->SetObjectProvider(this);
	TLanguage language = inputState->LocalLanguage();
	delete inputState;
*/
	if ((iPlayerControl->CurrentInputLanguage() == ELangThai) 	||
		(iPlayerControl->CurrentInputLanguage() == ELangArabic) ||
		(iPlayerControl->CurrentInputLanguage() == ELangFarsi) 	||
		(iPlayerControl->CurrentInputLanguage() == ELangHebrew) ||
		(iPlayerControl->CurrentInputLanguage() == ELangHindi) 	||
		(iPlayerControl->CurrentInputLanguage() == ELangUrdu) 	|| 
		(iPlayerControl->CurrentInputLanguage() == ELangVietnamese)
		) 
		{
		return true;
		}
	else 
		{
		return false;		
		}
	
}

TBool CInputModeSelector::IsFEP() 
	{
	//Returns true, if either intial string values or current input language mode is Complex.
	if (ContainsComplexLangCharacters(iInitialString, iLen) ||  IsInComplexLanguageMode())
		return true;
	else 
		return false;
	}


TBool CInputModeSelector::ContainsComplexLangCharacters(FI_UTF16Char * string, long length) 
	{
	FI_UTF16Char utfchar;
	TBool retval = false;

	for ( int i = 0; i<length; i++)
		{
		utfchar = string[i];
				// Arabic 
		//http://www.unicode.org/charts/PDF/U0600.pdf
		if ((utfchar >= 0x0600) && (utfchar <= 0x06FF)) {
			retval = true;
			break;
		}
		//Arabic Supplement
		//http://www.unicode.org/charts/PDF/U0750.pdf
		if ((utfchar >= 0x0750) && (utfchar <= 0x77F)) {
			retval = true;
			break;
		}
		// Arabic Presentation Forms-A
		// http://www.unicode.org/charts/PDF/UFB50.pdf 
		if ((utfchar >= 0xFB50) && (utfchar <= 0xFDFF)) {
			retval = true;
			break;
		}
		// Arabic Presentation Forms-B
	    // http://www.unicode.org/charts/PDF/UFE70.pdf
		if ((utfchar >= 0xFE70) && (utfchar <= 0xFEFF)) {
			retval = true;
			break;
		}
		// Hebrew 
		// http://www.unicode.org/charts/PDF/U0590.pdf
		if ((utfchar >= 0x0590) && (utfchar <= 0x05FF)) {
			retval = true;
			break;
		}
		// Hebrew presentation Forms
		// http://www.unicode.org/charts/PDF/UFB00.pdf
		if ((utfchar >= 0xFB00) && (utfchar <= 0xFB4F)) {
			retval = true;
			break;
		}
		// Thai
		// http://www.unicode.org/charts/PDF/U0E00.pdf
		if ((utfchar >= 0x0E00) && (utfchar <= 0x0E7F)) {
			retval = true;
			break;
		}
		// Bengali
		// http://www.unicode.org/charts/PDF/U0980.pdf
		if ((utfchar >= 0x0980) && (utfchar <= 0x09FF)) {
			retval = true;
			break;
		}
		// Devanagari
		// http://www.unicode.org/charts/PDF/U0900.pdf
		if ((utfchar >= 0x0900) && (utfchar <= 0x097F)) {
			retval = true;
			break;
		}
		//Gujarati
		//http://www.unicode.org/charts/PDF/U0A80.pdf		
		if ((utfchar >= 0x0A80) && (utfchar <= 0x0AFF)) {
			retval = true;
			break;
		}
		// Gurmukhi
		//http://www.unicode.org/charts/PDF/U0A00.pdf		
		if ((utfchar >= 0x0A00) && (utfchar <= 0x0A7F)) {
			retval = true;
			break;
		}
		// Kannada
		//http://www.unicode.org/charts/PDF/U0C80.pdf		
		if ((utfchar >= 0x0C80) && (utfchar <= 0x0CFF)) {
			retval = true;
			break;
		}
		// Limbu
		//http://www.unicode.org/charts/PDF/U1900.pdf		
		if ((utfchar >= 0x1900) && (utfchar <= 0x194F)) {
			retval = true;
			break;
		}
		// Malayalam
		//http://www.unicode.org/charts/PDF/U0D00.pdf		
		if ((utfchar >= 0x0D00) && (utfchar <= 0x0D7F)) {
			retval = true;
			break;
		}
		// Oriya
		//http://www.unicode.org/charts/PDF/U0B00.pdf		
		if ((utfchar >= 0x0B00) && (utfchar <= 0x0B7F)) {
			retval = true;
			break;
		}
		// Sinhala
		//http://www.unicode.org/charts/PDF/U0D80.pdf		
		if ((utfchar >= 0x0D80) && (utfchar <= 0x0DFF)) {
			retval = true;
			break;
		}
		// Syloti Nagri
		//http://www.unicode.org/charts/PDF/UA800.pdf		
		if ((utfchar >= 0xA800) && (utfchar <= 0x0A82F)) {
			retval = true;
			break;
		}
		// Tamil
		//http://www.unicode.org/charts/PDF/U0B80.pdf		
		if ((utfchar >= 0x0B80) && (utfchar <= 0x0BFF)) {
			retval = true;
			break;
		}
		// Telugu
		//http://www.unicode.org/charts/PDF/U0C00.pdf		
		if ((utfchar >= 0x0C00) && (utfchar <= 0x0C7F)) {
			retval = true;
			break;
		}
#if 0
		// Special case for vietnames.  
		// The Vietnamese alphabets are listed in several noncontiguous Unicode ranges:
		// http://vietunicode.sourceforge.net/charset/vietcharset.html
		// Vietnamese in basic latin code range {U+0000..U+007F} 
		if ((utfchar >= 0x0000) && (utfchar <= 0x007F)) {
			retval = true;
			break;
		}
		// Vietnamese in Latin-1 Supplement {U+0080..U+00FF}
		if ((utfchar >= 0x0080) && (utfchar <= 0x00FF)) {
			retval = true;
			break;
		}
		// Vietnamese in Latin Extended-A LantinExtdended-B {U+0100..U+024F} 
		if ((utfchar >= 0x0100) && (utfchar <= 0x024F)) {
			retval = true;
			break;
		}
		// Vietnamese in Latin Extended Additional U+1E00..U+1EFF}
		if ((utfchar >= 0x1E00) && (utfchar <= 0x1EFF)) {
			retval = true;
			break;
		}
		// Vietnames in Combining Diacritical Marks {U+0300.. U+036F}. 
		if ((utfchar >= 0x0300) && (utfchar <= 0x36F)) {
			retval = true;
			break;
		}
		// Vietnamese dong currency symbol is (U+20AB).
		if (utfchar == 0x20AB) {
			retval = true;
			break;
		}
#endif
		
	  }
	return retval;
	}


CSaFlashPlayerFEPIMEControl::CSaFlashPlayerFEPIMEControl(MFlashEngine* aEngine,
													CSaFlashPlayerView* aView,
													TPoint /*aPoint*/): 
iEngine(aEngine),
iPlayerView(aView)
	{
	}
void CSaFlashPlayerFEPIMEControl::ConstructL()
{
 	AMP_MTextNavigation *pNav= iEngine->GetTextNavigation();
 	long length = pNav->GetTextLength();
 	
 	 		
 	AMP_ReturnString returnString(FI_UTF16EncodingForm);
 	FI_ReturnString* fiReturnString = returnString.Wrapper();

	pNav->GetText(0,length, fiReturnString);
		{
		const TUint16* text = returnString.GetTemporaryUTF16String();
	    iPlayerView->GetInputTextL(text, length, pNav->GetMaxChars(), pNav->GetTextFieldFlags());
		}
	
	
}
TBool CSaFlashPlayerFEPIMEControl::IsInLine() 
{
		return false;
}


////////////////////////////////////////////////////////////////////////////////
// CSaFlashPlayerIMEControl
////////////////////////////////////////////////////////////////////////////////


CSaFlashPlayerIMEControl::CSaFlashPlayerIMEControl(MFlashEngine* aEngine,
													CSaFlashPlayerView* aView,
													TPoint aPoint):
	iStateFepUid(TUid::Uid(-1)),
	iFepState(NULL),
	iIsSpelling(EFalse),
	iIsCommitting(EFalse),
	iIsCbaShownForT9Spell(EFalse),
	iCompositionStringBuf(NULL),
	iCompositionStringBufSize(0),
	iCompositionStringLen(0),
	iCompositionStringCursorPos(0),
	iCursorVisibility(EFalse),
	iEngine(aEngine),
	iPlayerView(aView),
	iCursorPosAdd(0),
	iMonitorConversionStringLengthForZi(EFalse),
	iLastConversionStringCharactersInserted(0),
	iTextFormatRetriever(NULL),
	iLetCallSecondTime(EFalse),
	iIsPencilKeyPressed(EFalse),
	iCurrentInputLanguage(ELangEnglish),
	iCharsConsumed(0)
{
	iPoint.SetXY(aPoint.iX, aPoint.iY);
}

// Returns true, as text input is happening inline.
TBool CSaFlashPlayerIMEControl::IsInLine() 
{
	return true;
}


void CSaFlashPlayerIMEControl::ConstructL()
{
	FeatureManager::InitializeLibL(); // Initialise Feature manager
 
	#ifdef VIRTUAL_DOCUMENT_FOR_FEP
	iVirtualDocument = new (ELeave)CVirtualDocumentForFEP(iEngine, this);
	#endif // VIRTUAL_DOCUMENT_FOR_FEP
	/*This is necessary in 50 because text wont be displayed without this.
	if createwindow is called in 3.2.3 text is not displayed in flash text box
	ideal solution would be imecontrol becoming the child control of player control*/
#ifndef __S60_32__
	CreateWindowL(); 
#endif //__S60_32__
	ActivateL();
}

CSaFlashPlayerIMEControl::~CSaFlashPlayerIMEControl()
{
    FeatureManager::UnInitializeLib(); // Uninitialise Feature Manager
    
	delete iCompositionStringBuf;
	iCompositionStringBuf = NULL;

	delete iFepState;
	iFepState = NULL;

#ifdef VIRTUAL_DOCUMENT_FOR_FEP
	delete iVirtualDocument;
	iVirtualDocument = NULL;
#endif // VIRTUAL_DOCUMENT_FOR_FEP
}


TKeyResponse CSaFlashPlayerIMEControl::OfferKeyEventL( const TKeyEvent& aKeyEvent, TEventCode aType )
{
	// Start knocking down special cases

	// On Zi, when right key is used for committing a word, cursor goes
	// in the beginning of the text field. This happens because Zi doesn't
	// consume the EEventKey for the right key.. Although it uses it for
	// committing the word.
	// Please trace of the usage of iIsCommitting variable to find out how this works.
    
	// Text field shows garbage if we send these while editing text.
	switch(aKeyEvent.iScanCode)
	{
		case EStdKeyLeftShift: // Pencil key on device, shift key on wireless keyboard
		case EStdKeyRightShift: // Pencil key on emulator, shift key on wireless keyboard
		{
		iIsPencilKeyPressed = ETrue;
		}
		case EStdKeyTab:
		case EStdKeyEscape:
		case EStdKeyCapsLock:
		case EStdKeyLeftCtrl:
		case EStdKeyRightCtrl:
		{
			return EKeyWasConsumed;
		}
		default:
		    break;
	}
	// in case of the vkb, hwr and full screen qwerty, unicode is sent
	// in as the key's icode
	//one exception to this is  in case of secrettext  mode EKeyF20  is sent to delete the previouse character
	//unsigned long inputflags = iEngine->GetTextNavigation()->GetTextFieldFlags();
	//if(!(inputflags & FI_InputTextPassword) || (inputflags & FI_InputTextPassword && aKeyEvent.iCode != EKeyF20)  )
	//If  we consume the previous only then delete the previous character
    if(aKeyEvent.iCode == EKeyF20 && aKeyEvent.iModifiers == EModifierSpecial  && iCharsConsumed==0  )
          {
              return EKeyWasConsumed;
          }
	if( aKeyEvent.iCode != EKeyF20  )
	    {    
	        if( aKeyEvent.iModifiers == EModifierSpecial && aKeyEvent.iCode>= 32  )
	            {
	     
	            if(aType == EEventKey)
	                {

	                 if(iCompositionStringBuf)
	                    {
	                    delete iCompositionStringBuf;
	                    iCompositionStringBuf = NULL;
	                    }
	                 TBuf<20> aEditorContent1;
	                 TInt contentLen=DocumentLengthForFep();
	                 iCompositionStringBuf = HBufC::NewL(2);
	                TPtr des = iCompositionStringBuf->Des();
	                des.Append(TChar(aKeyEvent.iCode));	
	                des.ZeroTerminate();
	                TInt32 selStart = 0;
	                TInt32 selEnd = 0;
	                iEngine->GetTextNavigation( )->GetSel(selStart, selEnd);
	                FI_Text txt = { { ((char*) des.Ptr()) }, FI_UTF16EncodingForm, 1 };
	               iCharsConsumed= iEngine->GetTextNavigation( )->SetConversionString(&txt, NULL, 0, selStart+1, ETrue);
	               // iEngine->PlayStep(ETrue);	
	                }
	            // this causes the editor content to be fetched by the FEP
	             ReportStateEventL();
	            return EKeyWasConsumed;
	            }
	}

	 
	if( iIsPencilKeyPressed )    
		{
		TBool isLangChanged = IsLangChanged();
		if (isLangChanged && aType == EEventKeyUp )
			{
			// NOTE : Since this will disable IME, this will be deleted.
			// So this function should be returned immediately after this!
			iPlayerView->ResetIMEL();
			return EKeyWasConsumed;
			}
		iIsPencilKeyPressed = EFalse;    
		} 

	if(aKeyEvent.iScanCode == EStdKeyRightArrow && aType == EEventKeyDown)
	{
		iIsCommitting = EFalse;
	}

	if(aKeyEvent.iCode == EKeyRightArrow && aType == EEventKey)
	{
		if(iIsCommitting)
		{
			iIsCommitting = EFalse;
			// Don't send the key event to player navigation.
			return EKeyWasConsumed;
		}
	}

	// Part of T9 question mark fix. When T9 switches to spell state
	// the conversion string doesn't include the question mark in the end.
	// The code here is for resetting the conversion string after a clear
	// key has been pressed. T9 resets the Cba key after it calls UpdateInlineEditL
	// so we can't detect the clear key press in there.
	if(aKeyEvent.iScanCode == EStdKeyBackspace && aType == EEventKeyUp && iIsSpelling)
	{
		if (iCompositionStringBuf && iCompositionStringLen)
			{
			TPtr des = iCompositionStringBuf->Des();
			FI_Text txt = { { ((char*) des.Ptr()) }, FI_UTF16EncodingForm, iCompositionStringLen };
			iEngine->GetTextNavigation( )->SetConversionString(&txt, NULL, 0, iCompositionStringLen-1, EFalse);			
			iEngine->RestoreSoftKeys();
			}

		iIsSpelling = EFalse;

		if(iIsCbaShownForT9Spell && iPlayerView->IsFullScreenMode())
		{
			iPlayerView->GetCba()->MakeVisible(EFalse);
			iIsCbaShownForT9Spell = EFalse;
		}
	}
	
	TKeyResponse result = EKeyWasNotConsumed;

	// If we're in the middle of a conversion session (predictive session)
	// then we disable the enter key since the IME is not consuming it.
	// Otherwise, the key event gets sent to the player which will 
	// cause the field to be deactivated -- this causes a crash
	// on Japanese devices since we cannot properly cancel the
	// Fep transaction, although we can on non-Japanese devices.  This is
	// likely a Japanese-Symbian-specific issue.
	if ( iCompositionStringBuf && ( EStdKeyEnter == aKeyEvent.iScanCode || EKeyEnter == aKeyEvent.iCode ) )
	{
		result = EKeyWasConsumed;
	}

	return result;
}
void CSaFlashPlayerIMEControl::HandlePointerEventL(const TPointerEvent& /*aPointerEvent*/)
{
	if(FeatureManager::FeatureSupported(KFeatureIdPenSupport)) //adding feature manager to check the feature at run time.
	{
		if (!AknLayoutUtils::PenEnabled())
		    return;
		//launch VKB if Qwerty Keypad is Off.
		if( iFepState && (!(((CSaFlashAppUi*)iEikonEnv->AppUi())->GetQwertyKeyPadStatus())))
		{
			iFepState->ReportAknEdStateEventL(MAknEdStateObserver::EAknActivatePenInputRequest);
		}
		unsigned long inputflags = iEngine->GetTextNavigation()->GetTextFieldFlags();
	 
		if((inputflags & FI_InputTextPassword))
		{
			//Place the cursor at the end of the text for the pwd field.
			//so that user can delete the text entered in the field.
				
			TInt contentLen=DocumentLengthForFep();
			if(contentLen)
			{
			iEngine->GetTextNavigation( )->SetSel(contentLen, contentLen);
			}
		    	
		}
	}
}

TBool CSaFlashPlayerIMEControl::IsLangChanged()
	{
	CAknSettingCache& cache = CAknEnv::Static()->SettingCache();
	if (CurrentInputLanguage() != cache.InputLanguage())
		{
		SetCurrentInputLanguage(cache.InputLanguage());
		return ETrue;	
		}
	return EFalse;
	}
void CSaFlashPlayerIMEControl::CommitConversion(TBool aConversionCanceled)
{
	if(aConversionCanceled)
	{
		CancelFepInlineEdit();
	} else
	{
		TRAP_IGNORE(DoCommitFepInlineEditL());
	}
}


TCoeInputCapabilities CSaFlashPlayerIMEControl::InputCapabilities() const
{
	if ( iEngine->IsPanning( ) )
	{
		return TCoeInputCapabilities::ENone;
	}

	unsigned long inputflags = iEngine->GetTextNavigation()->GetTextFieldFlags();

    if (AknLayoutUtils::PenEnabled())
    {
		if(inputflags & FI_InputTextPassword)
		{
			TCoeInputCapabilities inputCapabilities(
			TCoeInputCapabilities::ENavigation | TCoeInputCapabilities::ESecretText
			,(MCoeFepAwareTextEditor *)this
			,NULL /* (MCoeCaptionRetrieverForFep *) */
			);
			
			return inputCapabilities;		
		}
    }
	
	TCoeInputCapabilities inputCapabilities(
	TCoeInputCapabilities::ENavigation | TCoeInputCapabilities::EAllText
	,(MCoeFepAwareTextEditor *)this
	,NULL /* (MCoeCaptionRetrieverForFep *) */
	);
	
	return inputCapabilities;		
}

/** Starts a FEP inline editing transaction. 

Inserts a descriptor containing the initial inline text into the text editor. 
The inline text should normally replace any selected text.

The final three parameters are instances of abstract classes, so that the 
caller of this function must create and instantiate classes deriving from 
them. These instances must remain in existence for the entire duration of 
the inline editing transaction.

Inline editing should not already be taking place when this function is called.

@param aInitialInlineText The inline text to insert into the text editor.
@param aPositionOfInsertionPointInInlineText An insertion position within the 
inline text. This is an offset from the start of the inline text.
@param aCursorVisibility ETrue for visible text cursor, EFalse for invisible 
text cursor in the text editor.
@param aCustomDraw Pointer to a custom drawing object. May be used to do advanced 
formatting of the inline text. This parameter is optional; a NULL pointer 
may be specified.
@param aInlineTextFormatRetriever Defines a single member function, GetFormatOfFepInlineText() 
which is used by the text editor to find out the formatting to apply to the 
inline text. It is also possible to apply different formatting to different 
parts of the inline text.
@param aPointerEventHandlerDuringInlineEdit Defines a single function, HandlePointerEventInInlineTextL() 
which is called when a pointer event is received within the inline text. This 
function might update the cursor position within the inline text and do text 
selection. */
void CSaFlashPlayerIMEControl::StartFepInlineEditL(const TDesC& aInitialInlineText, TInt aPositionOfInsertionPointInInlineText, TBool aCursorVisibility, const MFormCustomDraw* /* aCustomDraw */, MFepInlineTextFormatRetriever& aInlineTextFormatRetriever, MFepPointerEventHandlerDuringInlineEdit& /* aPointerEventHandlerDuringInlineEdit */)
{
	iTextFormatRetriever = &aInlineTextFormatRetriever;
	iCursorPosAdd = 0;

#ifdef VIRTUAL_DOCUMENT_FOR_FEP
	iVirtualDocument->Reset();
#endif // VIRTUAL_DOCUMENT_FOR_FEP

	iCursorVisibility = aCursorVisibility;
	SetInlineEditingCursorVisibilityL(aCursorVisibility);

	// Since we draw the text ourselves, ignore the aCustomDraw and the aInlineTextFormatRetriever sparameter.

	// Since we do not handle pointers, ignore the aPointerEventHandlerDuringInlineEdit parameter
	// (there will never be any pointer events anywhere in the editor, let alone within the inline text)

	// Use the UpdateFepInlineTextL method to extract the text
	UpdateFepInlineTextL(aInitialInlineText, aPositionOfInsertionPointInInlineText);
}

void CSaFlashPlayerIMEControl::StartFepInlineEditL(TBool& /*aSetToTrue*/, const TCursorSelection& aCursorSelection, const TDesC& aInitialInlineText, TInt aPositionOfInsertionPointInInlineText, TBool aCursorVisibility, const MFormCustomDraw* aCustomDraw, MFepInlineTextFormatRetriever& aInlineTextFormatRetriever, MFepPointerEventHandlerDuringInlineEdit& aPointerEventHandlerDuringInlineEdit)
{
	// On Chinese Nokia phones this function is used extensively in English multitap mode.
	// For some reason, enabling this functions for the multitap in those phones causes problems,
	// namely 1 additional cursor jump when a character is committed. To prevent that,
	// restrict the usage of this function to only to two cases: after spell dialog has
	// been closed and when input mode is changed from numeric to alpha by press-holding # key.
	//
	// For future research: why does Chinese multitap go wrong if this function is enabled to it?

	// Switching from numeric mode to alpha mode by press-holding # key case
	if(aInitialInlineText.Length() == 1 && (aInitialInlineText.Ptr())[0] == 35)
	{
		SetCursorSelectionForFepL(aCursorSelection);
		StartFepInlineEditL(aInitialInlineText, aPositionOfInsertionPointInInlineText, aCursorVisibility, aCustomDraw, aInlineTextFormatRetriever, aPointerEventHandlerDuringInlineEdit);
	}

	// Canceling Spell dialog-case: two separate calls. First time iIsSpelling is true, the second time not. 
	if(iIsSpelling)
	{
		SetCursorSelectionForFepL(aCursorSelection);
		StartFepInlineEditL(aInitialInlineText, aPositionOfInsertionPointInInlineText, aCursorVisibility, aCustomDraw, aInlineTextFormatRetriever, aPointerEventHandlerDuringInlineEdit);
		iLetCallSecondTime = ETrue;
	}
	else if(iLetCallSecondTime)
	{
		SetCursorSelectionForFepL(aCursorSelection);
		StartFepInlineEditL(aInitialInlineText, aPositionOfInsertionPointInInlineText, aCursorVisibility, aCustomDraw, aInlineTextFormatRetriever, aPointerEventHandlerDuringInlineEdit);
		iLetCallSecondTime = EFalse;
	}
}


/** Updates the inline text. 

Called when a character is added to or deleted from the inline text. 

The descriptor aNewInlineText contains the entire new inline text string, 
not just the new text to be combined with the old inline text.

FEP inline editing should already be taking place, i.e. StartFepInlineEditL() 
should have been called beforehand.

@param aNewInlineText Descriptor which holds the entire new inline text string.
@param aPositionOfInsertionPointInInlineText The position of the insertion 
point (i.e. the cursor) within the inline text string aNewInlineText. This 
is an offset from the start of the string. */
void CSaFlashPlayerIMEControl::UpdateFepInlineTextL(const TDesC& aNewInlineText, TInt aPositionOfInsertionPointInInlineText)
{
	// Try to figure out the conversion string format.

	TBool underlineOn = ETrue;

	if(iTextFormatRetriever)
	{
		// Zi crashes if we call GetFormatOfFepInlineText when the conversion string is of zero length.
		// T9 doesn't care.
		int conversionStringLength = aNewInlineText.Length();
		if(conversionStringLength)
		{
			TCharFormat defaultFormat; // out param
			TInt numOfCharsWithSameFormat; // out param
			TInt posOfChars = 0;

			iTextFormatRetriever->GetFormatOfFepInlineText(defaultFormat, numOfCharsWithSameFormat, posOfChars);
			underlineOn = (defaultFormat.iFontPresentation.iUnderline == EUnderlineOn);
		}
	}


	iMonitorConversionStringLengthForZi = EFalse;

	// Delete any existing text
	if (iCompositionStringBuf) {
		delete iCompositionStringBuf;
		iCompositionStringBuf = NULL;
		iCompositionStringBufSize = 0;
		iCompositionStringLen = 0;
	}

	// Check for the spell state

	// Works only for T9
	iIsSpelling = (iFepState->CbaState().iLeftSoftkeyCommandId == KT9SpellSoftkeyCommandId);

	if(iIsSpelling)
	{
		if(iPlayerView->IsFullScreenMode())
		{
			iIsCbaShownForT9Spell = ETrue;
			iPlayerView->GetCba()->MakeVisible(ETrue);
		}
	}

	// Extract the text
	iCompositionStringLen = aNewInlineText.Length();
	if (iCompositionStringLen)
	{
		iCompositionStringBuf = HBufC::NewL(iCompositionStringLen + 1);

		TPtr des = iCompositionStringBuf->Des();

		des.Copy(aNewInlineText);

		des.ZeroTerminate();

		iCompositionStringLen = iCompositionStringBuf->Length();
		// not used
		iCompositionStringBufSize = iCompositionStringBuf->Size();
	}

	iCompositionStringCursorPos = aPositionOfInsertionPointInInlineText;

	// Write the new conversion string to the editor
	if (iCompositionStringBuf && iCompositionStringLen) {
		TPtr des = iCompositionStringBuf->Des();
		FI_Text txt = { { ((char*) des.Ptr()) }, FI_UTF16EncodingForm, iCompositionStringLen };

		FI_TextField_StyleRun styleRun;
		styleRun.style.mask = FI_TextField_StyleBit_Underline;
		styleRun.style.bits = (underlineOn) ? FI_TextField_StyleBit_Underline : 0;
		// styleRun.style.textColor is ignored by the player
		// styleRun.style.fieldColor is ignored by the player
		// The run is ignored by the player but set it anyway.
		styleRun.run.length = iCompositionStringLen;
		styleRun.run.start = 0;

		long charsInserted = iEngine->GetTextNavigation( )->SetConversionString(&txt, &styleRun, 1, aPositionOfInsertionPointInInlineText, EFalse);

		iLastConversionStringCharactersInserted = charsInserted;
		
		if(charsInserted < iCompositionStringLen)
		{
			iCursorPosAdd = iCompositionStringLen - charsInserted;
		} else
		{
			iCursorPosAdd = 0;
		}
	} else
	{
		FI_Text empty;
		empty.textEncoding = FI_UTF16EncodingForm;
		empty.textLen = 0;
		empty.text.utf16String = 0;
		iLastConversionStringCharactersInserted = 0;
		iEngine->GetTextNavigation()->SetConversionString(&empty, NULL, 0, 0, ETrue);
	}
}

/** Sets the visibility of the text cursor in the text editor.

The cursor visibility is initialised using StartFepInlineEditL(). SetInlineEditingCursorVisibilityL() 
is provided for FEPs which need to change the visibility of the cursor during 
the inline editing transaction.

@param aCursorVisibility ETrue for visible text cursor, EFalse for invisible 
text cursor. */
void CSaFlashPlayerIMEControl::SetInlineEditingCursorVisibilityL(TBool aCursorVisibility)
{
	iCursorVisibility = aCursorVisibility;
	iEngine->GetTextNavigation()->ShowCursor(aCursorVisibility);
}

/** Cancels the inline editing transaction. 

The edit window should be rolled back to the state it was in before the FEP 
transaction started, i.e. any inline text in the document which has not yet 
been committed should be removed. If the inline text has replaced existing text, 
(e.g. a selection) the replaced text should be reinstated. */
void CSaFlashPlayerIMEControl::CancelFepInlineEdit()
{
	iCursorPosAdd = 0;

	// When in T9 user presses 0-key three times T9 FEP calls
	// UpdateFepInlineEdit with a conversion string that contains 0x21b2.
	// After that, it calls CancelFepInlineEdit. For some reason it never calls
	// DoCommitFepInlineEditL(). We do it manually here for this special case.
	// DoCommitFepInlineEditL will convert 0x21b2 to a new line character 0x0A.
	if(iCompositionStringBuf)
	{
		if(iCompositionStringLen == 1 && (iCompositionStringBuf->Ptr())[0] == 0x21b2)
		{
			TRAP_IGNORE(DoCommitFepInlineEditL());
			return;
		}
	}
	else
		{
		return;
		}
	// All other cases, cancel.

	if(iIsCbaShownForT9Spell && iPlayerView->IsFullScreenMode())
	{
		iPlayerView->GetCba()->MakeVisible(EFalse);
		iIsCbaShownForT9Spell = EFalse;
	}

	FI_Text emptyText; FlashMemSet( & emptyText, 0, sizeof( FI_Text ) );
	iEngine->GetTextNavigation( )->SetConversionString(&emptyText, NULL, 0, 0, EFalse);

	// Delete any existing text
	if (iCompositionStringBuf) {
		delete iCompositionStringBuf;
		iCompositionStringBuf = NULL;
		iCompositionStringBufSize = 0;
		iCompositionStringLen = 0;
		iLastConversionStringCharactersInserted = 0;
	}
	iCompositionStringCursorPos = 0;
	
	TRAP_IGNORE(SetInlineEditingCursorVisibilityL(ETrue));
	
	iTextFormatRetriever = NULL;
}

/** Returns the total number of characters in the text editor.

@return The total number of characters in the text editor. */
TInt CSaFlashPlayerIMEControl::DocumentLengthForFep() const
{
#ifdef VIRTUAL_DOCUMENT_FOR_FEP
	return iVirtualDocument->DocumentLengthForFep();
#else
	TInt documentLength = 0;

	documentLength = iEngine->GetTextNavigation( )->GetTextLength()
						+ iCursorPosAdd;

	return documentLength;
#endif // VIRTUAL_DOCUMENT_FOR_FEP
}

/** Returns the upper limit (if any) on the length of text that the text editor 
can hold.

@return The maximum number of characters that the text editor can hold. */
TInt CSaFlashPlayerIMEControl::DocumentMaximumLengthForFep() const
{
	// Return the maximum value
	TInt maxLen = KDocumentMaximumLengthForFep;
	maxLen = iEngine->GetTextNavigation( )->GetMaxChars();

	return maxLen;
}

/** Sets the range of characters in the text editor which should be selected.

@param aCursorSelection Contains the cursor and anchor positions for the selection. */
void CSaFlashPlayerIMEControl::SetCursorSelectionForFepL(const TCursorSelection& aCursorSelection)
{
#ifdef VIRTUAL_DOCUMENT_FOR_FEP
	iVirtualDocument->SetCursorSelectionForFepL(aCursorSelection);
#else
	// Set the cursor position in the editor
	long selStart = aCursorSelection.iAnchorPos;
	long selEnd = aCursorSelection.iCursorPos;

	if(selStart > selEnd)
	{
		long temp = selStart;
		selStart = selEnd;
		selEnd = temp;
	}

	// This should happen only after a spell dialog is confirmed
	if(iMonitorConversionStringLengthForZi)
	{
		long selLength = selEnd - selStart;

		if(selLength > iLastConversionStringCharactersInserted)
		{
			selEnd = selStart + iLastConversionStringCharactersInserted;
		}
	}
	
	iEngine->GetTextNavigation( )->SetSel(selStart, selEnd);

#endif // VIRTUAL_DOCUMENT_FOR_FEP
}

/** Gets the range of characters in the text editor which are selected.

@param aCursorSelection On return, contains the cursor and anchor positions 
of the selection. */
void CSaFlashPlayerIMEControl::GetCursorSelectionForFep(TCursorSelection& aCursorSelection) const
{
#ifdef VIRTUAL_DOCUMENT_FOR_FEP
	iVirtualDocument->GetCursorSelectionForFep(aCursorSelection);
#else
	// Return the cursor position from the editor
	long selStart = 0, selEnd = 0;
	iEngine->GetTextNavigation( )->GetSel(selStart, selEnd);
	aCursorSelection.SetSelection(selStart + iCursorPosAdd,
									selEnd + iCursorPosAdd);

#endif // VIRTUAL_DOCUMENT_FOR_FEP
}

/** Copies a portion of the text editor's text content into a descriptor.

@param aEditorContent A descriptor; on return contains a copy of a portion 
of the text.
@param aDocumentPosition The document position in the text editor from which 
to copy.
@param aLengthToRetrieve The number of characters to copy. */
void CSaFlashPlayerIMEControl::GetEditorContentForFep(TDes& aEditorContent, TInt aDocumentPosition, TInt aLengthToRetrieve) const
{
#ifdef VIRTUAL_DOCUMENT_FOR_FEP
	iVirtualDocument->GetEditorContentForFep(aEditorContent, aDocumentPosition, aLengthToRetrieve);
#else
	
	AMP_ReturnString returnString(FI_UTF16EncodingForm);
	FI_ReturnString* fiReturnString = returnString.Wrapper();

	FI_Result result = FI_ResultError;

	// In low memory situations fiReturnString may be null.
	// There is a null check in the player too but this code makes
	// the possibility more clear.
	if(fiReturnString)
	{
		result = iEngine->GetTextNavigation()->GetText(aDocumentPosition, aDocumentPosition + aLengthToRetrieve, fiReturnString);
	}

	long charsRetrieved = returnString.GetLength();

	if(charsRetrieved > aLengthToRetrieve)
	{
		charsRetrieved = aLengthToRetrieve;
	}

	// When using embedded font we sometimes insert less characters than in
	// conversion string because of missing glyphs. We should still return the
	// amount of characters requested or T9 crashes.

	if(result != FI_ResultOK || charsRetrieved < aLengthToRetrieve)
	{
		aEditorContent.Delete(0, aEditorContent.Length());
		if(charsRetrieved > 0)
		{
			aEditorContent.Copy(returnString.GetTemporaryUTF16String(), charsRetrieved);
		}

		long amountOfSpaces = aLengthToRetrieve - charsRetrieved;

		if(amountOfSpaces > 0)
		{
			TChar ch(' ');
			aEditorContent.AppendFill(ch, amountOfSpaces);
		}
	} else
	{
		aEditorContent.Copy(returnString.GetTemporaryUTF16String(), aLengthToRetrieve);
	}
#endif // VIRTUAL_DOCUMENT_FOR_FEP
}

void CSaFlashPlayerIMEControl::GetFormatForFep(TCharFormat& /* aFormat */, TInt /* aDocumentPosition */) const
{
	// Ignored
}

void CSaFlashPlayerIMEControl::GetScreenCoordinatesForFepL(TPoint& aLeftSideOfBaseLine, TInt& aHeight, TInt& aAscent, TInt /*aDocumentPosition*/) const
{
	FI_Point pos;
	int ascent;
	int descent;

	// Note: currently this does not use aDocumentPosition.
	// Implement later. Though hard to test as Zi doesn't use it.

	iEngine->GetTextNavigation()->GetCursorScreenCoordinates(pos,
																ascent,
																descent);

	pos.x += iPoint.iX;
	pos.y += iPoint.iY;

	aLeftSideOfBaseLine.SetXY(pos.x, pos.y);

	aAscent = ascent;
	aHeight = ascent + descent;
}

/** Private function called by CommitFepInlineEditL(). 

Implementations should commit the inline text to the document. This 
ends the inline editing transaction and causes the inline text to 
become a part of the document. 

@publishedAll
@released */
void CSaFlashPlayerIMEControl::DoCommitFepInlineEditL()
{
	iCursorPosAdd = 0;

	iIsCommitting = ETrue;

	// Zi specific missing embedded glyphs fix - set variable	
	CEikButtonGroupContainer* cba = CEikButtonGroupContainer::Current();
	if(cba)
	{
		MEikButtonGroup* group = cba->ButtonGroup();
		if(group->CommandId(0) != EAknSoftkeyOptions)
		{
			iMonitorConversionStringLengthForZi = ETrue;
		}
	}

	if(iIsCbaShownForT9Spell && iPlayerView->IsFullScreenMode())
	{
		iPlayerView->GetCba()->MakeVisible(EFalse);
		iIsCbaShownForT9Spell = EFalse;
	}

	if (iCompositionStringBuf && iCompositionStringLen) {
		TPtr des = iCompositionStringBuf->Des();
		FI_Text txt = { { ((char*) des.Ptr()) }, FI_UTF16EncodingForm, iCompositionStringLen };

		FI_UTF16Char lineFeed[2] = { 0x0A, 0 };
		if(txt.textLen == 1)
		{

			FI_UTF16Char ch = txt.text.utf16String[0];

			// After recent change core player recognizes Unicode line breaks.
			// We still need to change 0x21b2: the character code for "Enter" key.
			// This is a T9 specific fix. Please see CancelFepInlineEdit().
			if(ch == 0x2029 || ch == 0x2028 || ch == 0x21b2)
			{
				txt.text.utf16String = lineFeed;
			}
		}
  		long charsInserted = iEngine->GetTextNavigation( )->SetConversionString(&txt, NULL, 0, iCompositionStringCursorPos, ETrue);

		if(charsInserted < iCompositionStringLen)
		{
			iCursorPosAdd = iCompositionStringLen - charsInserted;
		} else
		{
			iCursorPosAdd = 0;
		}
	}

	// Delete any existing text
	if (iCompositionStringBuf) {
		delete iCompositionStringBuf;
		iCompositionStringBuf = NULL;
		iCompositionStringBufSize = 0;
		iCompositionStringLen = 0;
	}
	iCompositionStringCursorPos = 0;

	SetInlineEditingCursorVisibilityL(ETrue);

	iTextFormatRetriever = NULL;
}

MCoeFepAwareTextEditor_Extension1* CSaFlashPlayerIMEControl::Extension1()
{
	return this;
}

MCoeFepAwareTextEditor_Extension1* CSaFlashPlayerIMEControl::Extension1(TBool& aSetToTrue)
{
	aSetToTrue = ETrue;
	return this;
}

void CSaFlashPlayerIMEControl::SetStateTransferingOwnershipL(CState* /*aState*/, TUid /*fepTUid*/)
{
// This code is not needed for T9 since this API is not used in the correct way by device.
// Please see below.
//
//	if(iFepState)
//	{
//		delete iFepState;
//		iFepState = NULL;
//		iStateFepUid.Uid(-1);
//
//	}
//	iFepState = aState;
//	iStateFepUid.Uid(aTypeSafetyUid.iUid);
}

void CSaFlashPlayerIMEControl::SetFepStateInputModesToTextFieldFlags(CAknEdwinState *state, unsigned long flags)
{
	if(flags & FI_InputTextPassword)
	{
		state->SetDefaultInputMode(EAknEditorSecretAlphaInputMode);
		state->SetCurrentInputMode(state->CurrentInputMode());
		state->SetPermittedInputModes(EAknEditorSecretAlphaInputMode | EAknEditorNumericInputMode | EAknEditorTextInputMode);
		state->SetCurrentCase(EAknEditorLowerCase);
		state->SetDefaultCase(EAknEditorLowerCase);
		state->SetPermittedCases(EAknEditorLowerCase | EAknEditorUpperCase);
		state->SetFlags(EAknEditorFlagNoT9 | EAknEditorFlagNoLRNavigation | EAknEditorFlagSelectionVisible);
	} else
	if(flags & FI_InputTextNumeric)
	{
		state->SetDefaultInputMode(EAknEditorNumericInputMode);
		state->SetCurrentInputMode(EAknEditorNumericInputMode);
		state->SetPermittedInputModes(EAknEditorNumericInputMode);
		state->SetFlags(EAknEditorFlagSelectionVisible);
	} else
	{
		state->SetPermittedInputModes(EAknEditorAllInputModes);
		state->SetPermittedCases(EAknEditorAllCaseModes);
		// In Ivalo we have to set the flag EAknEditorFlagSelectionVisible explicitily
		// in order to select text in VKB
		state->SetFlags(EAknEditorFlagSelectionVisible);
	}
}

CSaFlashPlayerIMEControl::CState* CSaFlashPlayerIMEControl::State(TUid fepUid)
{
// In documentation the State object should have been passed to us in
// SetStateTransferOwnershipL. It should be perfectly OK to return null from
// this function. However, T9 (at least) crashes if null is returned.
// So create a CState object here.
	
	if (!iFepState) 
		{
		TRAP_IGNORE(CreateStateL());
		
		if (!iFepState)
			{	
			return NULL;
			}
			
		iFepState->SetObjectProvider(this);
		iStateFepUid = fepUid;
		}
		
	SetFepStateInputModesToTextFieldFlags(iFepState, iEngine->GetTextNavigation()->GetTextFieldFlags());
	return (iStateFepUid == fepUid) ? iFepState : NULL;
}

void CSaFlashPlayerIMEControl::CreateStateL()
	{
	iFepState = new (ELeave) CAknEdwinState();
	}


void CSaFlashPlayerIMEControl::MCoeFepAwareTextEditor_Reserved_2()
{
}


void CSaFlashPlayerIMEControl::SetCursorType(TBool& /*aSetToTrue*/, const TTextCursor& /*aTextCursor*/) //Allows you to change cursor type
{
}

void CSaFlashPlayerIMEControl::MCoeFepAwareTextEditor_Extension1_Reserved_3()
{
	// Empty in CEikEdwin
}

void CSaFlashPlayerIMEControl::MCoeFepAwareTextEditor_Extension1_Reserved_4()
{
	// Empty in CEikEdwin
}

void CSaFlashPlayerIMEControl::ReportStateEventL()
{
	if( iFepState )
	{
		iFepState->ReportAknEdStateEventL(MAknEdStateObserver::EAknCursorPositionChanged);
	}
	
}

//////////////////////////////////////////////////////////////////////////
// CVirtualDocumentForFEP
//////////////////////////////////////////////////////////////////////////

#ifdef VIRTUAL_DOCUMENT_FOR_FEP

void CSaFlashPlayerIMEControl::ResetIfEditorChanged(const TKeyEvent& aKeyEvent, TEventCode aType)
{
	iVirtualDocument->ResetIfEditorChanged(aKeyEvent, aType);
}

CVirtualDocumentForFEP::CVirtualDocumentForFEP(MFlashEngine* aEngine, CSaFlashPlayerIMEControl* aIMEControl)
{
	iEngine = aEngine;
	iIMEControl = aIMEControl;
	iReseted = EFalse;
}

CVirtualDocumentForFEP::~CVirtualDocumentForFEP()
{
}

// This sets the state of the virtual document.
// All variable should be reseted here.
void CVirtualDocumentForFEP::Reset()
{
	long selStart;
	long selEnd;

	// This should be more than any imaginable conversion string.
	const long lookForwardAndBackwardDistance = 200;

	AMP_MTextNavigation* nav = iEngine->GetTextNavigation();

	long length = nav->GetTextLength();

	iOriginalLength = length;

	nav->GetSel(selStart, selEnd);

	// use selStart for the start index

	iActualStartIndex = selStart - lookForwardAndBackwardDistance;

	if(iActualStartIndex<0)
	{
		iActualStartIndex = 0;
	}

	long endIndex = selStart + lookForwardAndBackwardDistance;

	if(endIndex > length)
	{
		endIndex = length;
	}

	iVirtualLengthInBeginning = endIndex - iActualStartIndex;

	iPreviousEventKeyCode = 0;

	iReseted = ETrue;
}
TInt CVirtualDocumentForFEP::DocumentLengthForFep()
{
	if(!iReseted)
	{
		Reset();
	}

	AMP_MTextNavigation* nav = iEngine->GetTextNavigation();

	long length = nav->GetTextLength();

	long retLength = length - iOriginalLength + iVirtualLengthInBeginning
						// T9 embedded fonts crash factor
						+ iIMEControl->iCursorPosAdd;

	return retLength;
}
void CVirtualDocumentForFEP::SetCursorSelectionForFepL(const TCursorSelection& aCursorSelection)
{
	if(!iReseted)
	{
		Reset();
	}

	AMP_MTextNavigation* nav = iEngine->GetTextNavigation();
	nav->SetSel(aCursorSelection.iAnchorPos + iActualStartIndex,
				aCursorSelection.iCursorPos + iActualStartIndex);
}
void CVirtualDocumentForFEP::GetCursorSelectionForFep(TCursorSelection& aCursorSelection)
{
	if(!iReseted)
	{
		Reset();
	}

	long selStart;
	long selEnd;
	iEngine->GetTextNavigation( )->GetSel(selStart, selEnd);
	aCursorSelection.SetSelection(selStart - iActualStartIndex
										// T9 embedded fonts crash factor
										+ iIMEControl->iCursorPosAdd,
									selEnd - iActualStartIndex
										// T9 embedded fonts crash factor
										+ iIMEControl->iCursorPosAdd);
}

void CVirtualDocumentForFEP::GetEditorContentForFep(TDes& aEditorContent, TInt aDocumentPosition, TInt aLengthToRetrieve) 
{
	if(!iReseted)
	{
		Reset();
	}

	AMP_ReturnString returnString(FI_UTF16EncodingForm);
	FI_ReturnString* fiReturnString = returnString.Wrapper();

	// TODO: This needs to be updated as in the normal GetEditorContent: append spaces.
	// Preferably don't copy/paste the same code in two locations, optimize.
	if ((iEngine->GetTextNavigation()->GetText(aDocumentPosition + iActualStartIndex,
												aDocumentPosition  + iActualStartIndex + aLengthToRetrieve,
												fiReturnString) == FI_ResultOK))
	{
		aEditorContent.Copy(returnString.GetTemporaryUTF16String(), aLengthToRetrieve);
	} else
	{
		TChar ch(' ');
		aEditorContent.Delete(0, aEditorContent.Length());
		aEditorContent.AppendFill(ch, aLengthToRetrieve);
	}
}

void CVirtualDocumentForFEP::ResetIfEditorChanged(const TKeyEvent& aKeyEvent, TEventCode aType)
{
	if(aType != EEventKey)
	{
		return;
	}

	switch(aKeyEvent.iCode)
	{
		case EKeyBackspace:
		case EKeyRightArrow:
		case EKeyLeftArrow:
		case EKeyUpArrow:
		case EKeyDownArrow:
			Reset();
			break;
		default:
			break;
	};
}

#endif // VIRTUAL_DOCUMENT_FOR_FEP


#endif // FEATURE_INLINE_TEXT_ENTRY
