/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
                                 
*
*/

#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
//			It contains all the #define build parameters.
#include "platformtypes.h"
#include "platformclib.h"
#include "flashliteinterfaces.h"

#include <flash2ui.rsg>

#include "saflash_coredefs.h"
//#include "symbian_app.h"
#include "flash_engine.h"
#include "symbian_textinput.h"
#include "saflash_consts.h"
#include "symbian_ui_interface.h"
#include "ui_defs.h"

#include <aknnotewrappers.h>
#include <AknGlobalNote.h>
#include <f32file.h>
#include <apgcli.h>
#include <hal.h>


/************************************************************************/
/* This file should be shared identically between SAPlayer and FlashCast*/
/* Please be sure to keep the two files in sync until they can be shared*/
/* properly.															*/
/************************************************************************/


/*
*	Custom dialog class for handling text input in FlashLite.
*  Overrides OkToExitL to copy the user text back to the flash dll.
*/
class CGetInputTextDialog : public CAknTextQueryDialog
{
public:
	CGetInputTextDialog(TextInput& aEngine, TUint16* aTextData, TPtr16* aDataText, const TTone& aTone);
	
	TBool OkToExitL(TInt aButtonID);
	~CGetInputTextDialog();
	virtual void  UpdateLeftSoftKeyL() { }; // do nothing, so the left ok key is visible
	
private:
	TUint16* iTextData;
	TextInput& iEngine;
	TPtr* iTextEntryPtr;
};

CGetInputTextDialog::CGetInputTextDialog(TextInput& aEngine, TUint16* aTextData, TPtr16* aDataText, const TTone& aTone)
	: CAknTextQueryDialog(*aDataText, aTone)
	, iTextData(aTextData)
	, iEngine(aEngine)
	, iTextEntryPtr(aDataText)
{
}

CGetInputTextDialog::~CGetInputTextDialog()
{
	delete iTextData;
	iEngine.ResetBusyFlag();
}

TBool CGetInputTextDialog::OkToExitL(TInt aButtonID)
{
	CAknTextQueryDialog::OkToExitL ( aButtonID );

	if ( EAknSoftkeyOk == aButtonID )
	{
		iEngine.SetInputTextL(*iTextEntryPtr);
	}
	else
	{
		iEngine.CancelInputText( );
	}

	delete iTextData;
	iTextData = NULL;

	return ETrue;
}

// -------------------------------------------

TextInput::~TextInput()
{
}

void TextInput::SetInputTextL(const TDesC& aNewStr)
{
	iEngine->SetInputTextL(aNewStr);
	iEngine->ResumeL(EReasonTextInput,EFalse);
}

void TextInput::CancelInputText()
{
	iEngine->CancelInputText();
	TRAP_IGNORE(iEngine->ResumeL(EReasonTextInput,EFalse));	
}

TInt atoi(char* s);


const TInt KShortStringLen = 128;
const TInt KShortStringMaxLen = 256;

void TextInput::DoGetInputTextL(TGetInputTextData& aData)
{
	// if maxNumChars is zero, this means Flash does not enforce a limit to the maximum
	// number of characters that can be entered. However, we don't want to allow an
	// infinite amount of text, so pick a reasonable limit. If the string is less than
	// 128 characters, then set a max of 256, otherwise set the max to double the string
	// length.
	TInt strLen = aData.iInitialString->Length();
	TInt maxNumChars = aData.iMaxNumChars;
	if (maxNumChars == 0)
	{
		maxNumChars = (unsigned short)((strLen < KShortStringLen) ? KShortStringMaxLen : (strLen<<1));
	}

	TUint16* textData = new (ELeave)TUint16[maxNumChars];
	CleanupStack::PushL(textData);
		
	iTextEntryPtr.Set(textData, 0, maxNumChars );
		
	if (strLen > maxNumChars)
	{
		strLen = maxNumChars;
	}
		
	// Flash converts all newline characters to CR/LF (0x0A, 0x0D).
	// Map these to Unicode line separator (0x2029) so that the Series60 text dialog will
	// render them properly.
	for (TInt i = 0; i < strLen;i++)
	{
		switch ((*aData.iInitialString)[i])
		{
		case FLASH_LINE_FEED_CHAR:
			iTextEntryPtr.Append((TChar)KUnicodeParagraphSeparator);
			break;
			
		case FLASH_CARRIAGE_RETURN_CHAR:
			break;
			
		default:
			iTextEntryPtr.Append((TChar)((*aData.iInitialString)[i]));
		}
	}
	
	
	CGetInputTextDialog* dlg;
	dlg = new(ELeave) CGetInputTextDialog(*this, textData, &iTextEntryPtr, CAknTextQueryDialog::ENoTone);
    CleanupStack::Pop(textData);
	
	dlg->SetPredictiveTextInputPermitted(ETrue);
	
	if (aData.iTextFlags & FI_InputTextNumeric)
	{
		dlg->SetMaxLength(maxNumChars);
		dlg->ExecuteLD(R_SAFLASH_INPUTTEXT_NUMBER_QUERY);
	}
	else if (aData.iTextFlags & FI_InputTextPassword)
	{
		if (maxNumChars > KMaxPasswordInputChars)
		{
			maxNumChars = KMaxPasswordInputChars;
		}									
		dlg->SetMaxLength(maxNumChars);
		dlg->ExecuteLD(R_SAFLASH_INPUTTEXT_PASSWD_QUERY);
	}
	else if (aData.iTextFlags & FI_InputTextMultiline)
	{
		dlg->SetMaxLength(maxNumChars);
		dlg->ExecuteLD(R_SAFLASH_INPUTTEXT_MULTILINE_QUERY);
	}
	else
	{
		dlg->SetMaxLength(maxNumChars);
		dlg->ExecuteLD(R_SAFLASH_INPUTTEXT_QUERY);
	}
	iEngine->Pause(EReasonTextInput);
	iTextEntryActive = TRUE;
}

void TextInput::GetInputText(TGetInputTextData& aData)
{
	// if maxNumChars is zero, this means Flash does not enforce a limit to the maximum
	// number of characters that can be entered. However, we don't want to allow an
	// infinite amount of text, so pick a reasonable limit. If the string is less than
	// 128 characters, then set a max of 256, otherwise set the max to double the string
	// length.
	
	TRAP_IGNORE( DoGetInputTextL(aData) );
}
