/*
* Copyright (c) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  
                                 
*
*/


#include "platformbuild.h"		// Warning: this *must* the first include of every source file.
								//			It contains all the #define build parameters.

#include "platformclib.h"
#include "symbian_debug.h"

#include <flash2ui.rsg>
#include <s32file.h>
#include <f32file.h>
#include <eikapp.h>
#include <bautils.h>
#include <aknnotedialog.h>
#include <AknIconUtils.h>
#include <gulicon.h>
#include <centralrepository.h>
#include <AknQueryDialog.h>

#include "flash2ui.hrh"
#include "symbian_defs.h"
#include "symbian_playerview.h"
#include "flash_engine.h"
#include "symbian_uiutil.h"
#include "saflash_consts.h"
#include "saflash_coredefs.h"
#include "ui_defs.h"
#ifndef ENABLE_SF_CHANGES
#include <Flashlite_2_0PrivateCRKeys.h> //senRep keys
#else
#include "Flashlite_2_0PrivateCRKeys.h"
#endif

#include <aknnotedialog.h>
#include <s32file.h>
#include <f32file.h>
#include <bautils.h>


#include <gulicon.h>
#include <AknIconUtils.h>



CGulIcon* FlashUiUtil::CreateIconL( const TDesC& aIconFile, TInt aIconIndex,
                                     TInt aMaskIndex )
    {
    CFbsBitmap* bitmap = NULL;
    CFbsBitmap* mask = NULL;
    CGulIcon* icon = NULL;

    AknIconUtils::CreateIconLC( bitmap, mask, aIconFile,
                                aIconIndex, aMaskIndex );
                                
    icon = CGulIcon::NewL( bitmap, mask );
    icon->SetBitmapsOwnedExternally( EFalse );
    
    CleanupStack::Pop( 2 ); // bitmap and mask owned by aIcon

    return icon;
    }



TInt FlashUiUtil::CheckDirCountL(const TDesC& aBaseDir, const TDesC& aSubDir)
{
	TInt count = 0;

	TInt searchBufLen = KDirSearchStr().Length() + aBaseDir.Length() + 2 + aSubDir.Length();

	// create search string
	HBufC* searchBuf = HBufC::New( searchBufLen );
	if (searchBuf)
	{
		TPtr searchStr = searchBuf->Des();

		// Set directory search string
		searchStr.Copy(aBaseDir);
		if (aSubDir.Length())
		{
			//searchStr.Append(KDirSeparator);
			searchStr.Append(aSubDir);
		}
		if (searchStr.Right(1).Compare(KDirSeparator) != 0)
		{
			searchStr.Append(KDirSeparator);
		}
		searchStr.Append(KDirSearchStr);

		CDir* dir = NULL;
		RFs fs;
		TInt err = fs.Connect();
		if(err == KErrNone) 
		{
			
			err = fs.GetDir(searchStr, 
						KEntryAttDir | KEntryAttMatchExclusive
						, ESortByName | EAscending, dir);
		}
		
		fs.Close();
		
		if (err == KErrNone)
		{
			count = dir->Count();
		}
		
		if(dir)
		{
			delete dir;
			dir = NULL;
		}
		delete searchBuf;
	}
	
	return count;
		
}

TInt FlashUiUtil::DoSwfCountL(CDir* drmSwf,const TDesC& aBaseDir )
{
	TInt count = 0;
	if(drmSwf)
	{
		HBufC* pBufC = HBufC::NewLC( KMaxFileName );
		{
			TPtr ptr = pBufC->Des();
			TEntry entry;
			TInt dirCnt = drmSwf->Count();
				
			for(int i=0; i < dirCnt; i++)
			{
				entry = (*drmSwf)[i];
				ptr.Copy(aBaseDir);
				ptr.Append(entry.iName);
				if(GetFileTypeL(ptr))
				{
					count++;	
				}
			}
		}
		CleanupStack::PopAndDestroy(pBufC);
	}
	
	return count;
}


TInt FlashUiUtil::CheckSwfCountL(const TDesC& aBaseDir
								, const TDesC& aSubDir
								, const TDesC& aSearchFilter
								, TBool aSwfExtn
								)
{
	TInt count = 0;
//	_LIT(KDcfSearchStr, "*.dcf;*.odf");
	
	TInt searchBufLen = aSearchFilter.Length() + 
						aBaseDir.Length() + 2 + aSubDir.Length();

	// create search string
	HBufC* searchBuf = HBufC::New( searchBufLen );
	if (searchBuf)
	{
		TPtr searchStr = searchBuf->Des();

		// Set directory search string
		searchStr.Copy(aBaseDir);
		if (aSubDir.Length())
		{
			//searchStr.Append(KDirSeparator);
			searchStr.Append(aSubDir);
		}
		if (searchStr.Right(1).Compare(KDirSeparator) != 0)
		{
			searchStr.Append(KDirSeparator);
		}
		
		searchStr.Append(aSearchFilter);

		CDir* SwfDir = NULL;
		RFs fs;
		TInt err = fs.Connect();
		
		if(err == KErrNone) 
		{
			err = fs.GetDir(searchStr
						, KEntryAttNormal
						, ESortByName | EAscending, SwfDir);
			
		}
		
		fs.Close();
		
		if (err == KErrNone)
		{
			count =  SwfDir->Count();
			
			if((!aSwfExtn) && (SwfDir->Count() > 0))
			{
				searchStr.Copy(aBaseDir);
				searchStr.Append(aSubDir);
				if (searchStr.Right(1).Compare(KDirSeparator) != 0)
				{
					searchStr.Append(KDirSeparator);
				}
				
				count = DoSwfCountL(SwfDir,searchStr);	
			}
			
		}
		
		if(SwfDir)
		{
			delete SwfDir;
			SwfDir = NULL;
		}
		
		delete searchBuf;
	}
	
	return count;	
}

TInt FlashUiUtil::CheckSubdirCountL(const TDesC& aBaseDir
									, const TDesC& aSubDir
									, TBool aDirsOnly
									, TBool aSwfDcfFileCount
									)
{
	TInt count = 0;
	
	if(aDirsOnly)
	{
		return CheckDirCountL(aBaseDir,aSubDir);
	}
	else
	{
		count = CheckSwfCountL(aBaseDir,aSubDir,KSwfSearchStr,ETrue);
		
		if(aSwfDcfFileCount)
		{
			count += CheckSwfCountL(aBaseDir,aSubDir,KDcfSearchStr,EFalse);
			count += CheckSwfCountL(aBaseDir,aSubDir,KOdfSearchStr,EFalse);
		}
		
		#ifdef __FEATURE_FLV_RECOG_PLAY
		count += CheckSwfCountL(aBaseDir,aSubDir,KFlvSearchStr,EFalse);
		#endif
		
        #ifdef __FEATURE_F4V_RECOG_PLAY
        count += CheckSwfCountL(aBaseDir,aSubDir,KF4vSearchStr,EFalse);
        #endif
	}
	
	return count;
}

TInt FlashUiUtil::GetFileTypeL(const TDesC& aFilename)
{
    HBufC* tmpMIME = HBufC::NewLC( KMaxDataTypeLength );

	CContent* content = CContent::NewLC(aFilename, EContentShareReadWrite);
    TPtr mimePtr( tmpMIME->Des() );            
    content->GetStringAttribute(EMimeType, mimePtr );

    TBool isFlash =  (0 != mimePtr.Compare(KFlashMimeType))? EFalse : ETrue ;

	#ifdef __FEATURE_FLV_RECOG_PLAY
	TBool isFlv =  (0 != mimePtr.Compare(KFLVMimeType))? EFalse : ETrue ;
	#endif
	
    #ifdef __FEATURE_F4V_RECOG_PLAY
    TBool isF4v =  (0 != mimePtr.Compare(KF4VMimeType))? EFalse : ETrue ;
    #endif
	CleanupStack::PopAndDestroy(content);
    CleanupStack::PopAndDestroy(tmpMIME);
    
    if(isFlash)
    	return 1;
    
    #ifdef __FEATURE_FLV_RECOG_PLAY
    if(isFlv)
    	return 2;
    #endif
    
    #ifdef __FEATURE_F4V_RECOG_PLAY
    if(isF4v)
        return 3;
    #endif
    return 0;
}

TInt FlashUiUtil::GetFileTypeL(RFile aRFile)
{
	
	HBufC* tmpMIME = HBufC::NewLC( KMaxDataTypeLength );

	CContent* content = CContent::NewLC(aRFile);
    TPtr mimePtr( tmpMIME->Des() );            
    content->GetStringAttribute(EMimeType, mimePtr );

    TBool isFlash =  (0 != mimePtr.Compare(KFlashMimeType))? EFalse : ETrue ;
    
	#ifdef __FEATURE_FLV_RECOG_PLAY
	TBool isFlv =  (0 != mimePtr.Compare(KFLVMimeType))? EFalse : ETrue ;
	#endif
	
    #ifdef __FEATURE_F4V_RECOG_PLAY
    TBool isF4v =  (0 != mimePtr.Compare(KF4VMimeType))? EFalse : ETrue ;
    #endif
	CleanupStack::PopAndDestroy(content);
    CleanupStack::PopAndDestroy(tmpMIME);
    
    if(isFlash)
    	return 1;
    
    #ifdef __FEATURE_FLV_RECOG_PLAY
    if(isFlv)
    	return 2;
    #endif
    
    #ifdef __FEATURE_F4V_RECOG_PLAY
    if(isF4v)
        return 3;
    #endif
    return 0;
}

void FlashUiUtil::DebugShowNoteL(const TDesC & aMsg)
{
	CAknGlobalNote* pNote = CAknGlobalNote::NewLC();
	pNote->ShowNoteL(EAknGlobalErrorNote, aMsg);
	CleanupStack::PopAndDestroy(pNote);
}

// show note with msg
void FlashUiUtil::ShowNoteL( TAknGlobalNoteType aNoteType, const TDesC & aMsg )
{
	CAknGlobalNote* pNote = CAknGlobalNote::NewLC();
	pNote->ShowNoteL(aNoteType, aMsg);
	CleanupStack::PopAndDestroy(pNote);
}


HBufC * FlashUiUtil::Format1L(CEikonEnv& aEnv, TInt aFormatResource, const TDesC & aArg1 )
{
	HBufC* pMsg;
	HBufC* pFormat = aEnv.AllocReadResourceLC(aFormatResource);
	{
		pMsg = HBufC::NewLC( pFormat->Length() + aArg1.Length() );
		{
			TPtr ptr = pMsg->Des();
			ptr.Format( *pFormat, &aArg1 );
		}
		CleanupStack::Pop(pMsg);
	}
	CleanupStack::PopAndDestroy(pFormat);
	return pMsg;
}

void FlashUiUtil::ShowNoteFormat1L( CEikonEnv& aEnv, TAknGlobalNoteType aNoteType, TInt aFormatResource, const TDesC & aArg1 )
{
	HBufC * pMsg = Format1L( aEnv, aFormatResource, aArg1 );
	CleanupStack::PushL( pMsg );
	FlashUiUtil::ShowNoteL( aNoteType, *pMsg );
	CleanupStack::PopAndDestroy(pMsg);
}

void FlashUiUtil::TryShowNoteL(TAknGlobalNoteType aNoteType, const TDesC & aMsg, TInt aFormatResource, CEikonEnv* aEnv)
{
	if(aFormatResource && aEnv)
	{
		ShowNoteFormat1L(*aEnv, aNoteType, aFormatResource, aMsg);
	}
	else
	{
		ShowNoteL(aNoteType, aMsg);
	}
}

TInt FlashUiUtil::ShowNoteWithoutLeaving(TAknGlobalNoteType aNoteType, const TDesC & aMsg, TInt aFormatResource, CEikonEnv* aEnv, CFlashErrorDialog* aLowMemDialog)
{
	TRAPD(error,FlashUiUtil::TryShowNoteL(aNoteType,aMsg,aFormatResource, aEnv));

	if(error && aLowMemDialog)
	{
		(void)aLowMemDialog->RouseSleepingDialog();
	}
	return error;
}

 
// ----------------------------------------------------------------------
// TBool FlashUiUtil::DisplayQueryL(const TInt aResourceId, 
//    const TDesC& aQueryText)
//  
// This function is used to display a dialog to the user with the 
// specified query
// ----------------------------------------------------------------------
// 
TBool FlashUiUtil::DisplayQueryL(const TInt aResourceId, 
    const TDesC& aQueryText)
    {
	TBool rval = EFalse;

    CAknQueryDialog* dlg = CAknQueryDialog::NewL();
    TInt result(dlg->ExecuteLD(aResourceId, aQueryText));

    if (result)
        {
	    rval = ETrue;
        }

	return rval;
    }
////////////////////////////////////////////////////////////////////////////////
// SAFlashErrorHandler utility class
////////////////////////////////////////////////////////////////////////////////


// This class is required because we can't
// display a dialog that requires user interaction from inside
// a callback. This sets up a single instance timer that will
// display the dialog at a safe time.

SAFlashErrorHandler::SAFlashErrorHandler ( CSaFlashPlayerView* aPlayerView, CFlashErrorDialog& aLowMemDialog )
	: iTimer(NULL), iFirstError(NULL), iPlayerView(aPlayerView), iDisplayingError(EFalse), iLowMemDialog(aLowMemDialog), iNetworkPromptShown (EFalse)
{
}

SAFlashErrorHandler::~SAFlashErrorHandler ( )
{
	ClearErrors ( );
}

TBool SAFlashErrorHandler::QueryAllowDenyL(const TDesC& aQueryText)
{
	iNetworkPromptShown = ETrue;
	
	TBool rval = EFalse;

    CAknQueryDialog* dlg = CAknQueryDialog::NewL();
    HBufC *queryText = aQueryText.AllocLC();
    TPtr queryPtr = queryText->Des();
    AknTextUtils::LanguageSpecificNumberConversion(queryPtr);    

    TInt result(dlg->ExecuteLD(R_SAFLASH_DIALOG_ALLOW_DENY, queryPtr));

    if (result)
        {	
		rval = ETrue;		
        }
    CleanupStack::PopAndDestroy(queryText);
    
    iNetworkPromptShown = EFalse;    

	return rval;
}
void SAFlashErrorHandler::ClearErrors ( )
{
	if ( iTimer )
	{
		iTimer->Cancel ( );
		delete iTimer;
		iTimer = NULL;
	}

	TErrorMessage* pNextError = NULL;
	while ( iFirstError )
	{
		pNextError = iFirstError->pNextError;

    	delete iFirstError->pErrorMessage;
		delete iFirstError;
//		iFirstError = NULL;
		iFirstError = pNextError;
	}
	iPlayerView->ClearErrors();
}

void SAFlashErrorHandler::ReportErrorL(HBufC* aErrorMessage, TInt aFlags)
{
	
	CleanupStack::PushL ( aErrorMessage );

	TErrorMessage* pNewError = new(ELeave) TErrorMessage( aErrorMessage, aFlags );
	
	CleanupDeletePushL ( pNewError );
	
	if ( iFirstError )
	{
		TErrorMessage* pLastError = iFirstError;
		while ( pLastError->pNextError ) pLastError = pLastError->pNextError;
		pLastError->pNextError = pNewError;
	}
	else
	{
		PLAYERASSERT(!iTimer);
		iTimer = CPeriodic::NewL ( CActive::EPriorityUserInput );
		iTimer->Start ( KReportErrorStartDelay, KReportErrorInterval, TCallBack(StaticTimerTick, (TAny*)this) );
		iFirstError = pNewError;
	}
	
	CleanupStack::Pop( 2 );
}

void SAFlashErrorHandler::ReportError( HBufC* aErrorMessage, TInt aFlags )
{
	TRAPD(result, ReportErrorL(aErrorMessage,aFlags));
	
	if(result)
	{
		(void)iLowMemDialog.RouseSleepingDialog();
	}
}

TBool SAFlashErrorHandler::StaticTimerTick ( TAny* aStaticErrorHandler )
{
	SAFlashErrorHandler* pErrorHandler = (SAFlashErrorHandler*)aStaticErrorHandler;
	if(pErrorHandler->iNetworkPromptShown)
		{
		return EFalse;
		}
	
	TRAPD(error, pErrorHandler->TimerTickL());
	if(error)
	{
		pErrorHandler->iDisplayingError = EFalse;
		pErrorHandler->ClearErrors ( );
		if(error == KErrNoMemory)
		{
			pErrorHandler->ShowLowMemoryDialog();
		}
	}
	return EFalse;
}

void SAFlashErrorHandler::TimerTickL()
{
	TInt result = EEikBidOk;
	result = ShowErrorsL();
	iDisplayingError = EFalse;
	ClearErrors ( );

	if (result == EEikFlashCmdAbort)
	{
		// In an abort situation, we should behave as if the user hit the "Back" softkey.
		// This will either display the fileview or exit the app, as appropriate.
		iPlayerView->HandleCommandL(EAknSoftkeyBack);
	}
	else if (result == EEikFlashCmdContinue)
	{
		iPlayerView->GetEngine()->ResumeL(EReasonUser, EFalse);
	} 
}

TInt SAFlashErrorHandler::ShowErrorsL()
{
	iPlayerView->GetEngine()->Pause(EReasonFocus);
	iTimer->Cancel ( );

	TErrorMessage* pError = iFirstError;
	TInt result = EEikBidOk;
	while ( pError )
	{
		iDisplayingError = ETrue;
		CAknNoteDialog* dlg = new(ELeave) CFlhQueryDialog ( CAknNoteDialog::EErrorTone, CAknNoteDialog::ENoTimeout );
		switch ( pError->iFlags )
		{
		case EFlagContinue:
			{
				dlg->PrepareLC ( R_SAFLASH_ERRORDIALOG_CONTINUE );
				break;
			}

		case EFlagAbort:
			{
				dlg->PrepareLC ( R_SAFLASH_ERRORDIALOG_ABORT );
				break;
			}

		case ( EFlagAbort | EFlagContinue ):
			{
				dlg->PrepareLC ( R_SAFLASH_ERRORDIALOG_ABORT_CONTINUE );
				break;
			}

		default:
			{
				delete dlg;
				return result;
			}
		}

		dlg->SetTextL( *pError->pErrorMessage );
		result = dlg->RunLD ( );

		if ( result == EEikFlashCmdAbort )
		{
			// If the user decides to abort, don't display any more errors.
			ClearErrors();
			return result;
		}

		pError = pError->pNextError;
	}

	iPlayerView->GetEngine()->ResumeL(EReasonFocus, EFalse);
	ClearErrors();
	return result;
}

void SAFlashErrorHandler::ShowLowMemoryDialog()
{
	(void)iLowMemDialog.RouseSleepingDialog();
}
////////////////////////////////////////////////////////////////////////////////
// CFlashErrorDialog
////////////////////////////////////////////////////////////////////////////////
CFlashErrorDialog::CFlashErrorDialog(MFlashEngine& aFlashEngine)
: CFlhQueryDialog( CAknNoteDialog::EErrorTone, CAknNoteDialog::ENoTimeout ),
iFlashEngine(aFlashEngine)
{
}

CFlashErrorDialog::~CFlashErrorDialog()
{
}

CFlashErrorDialog* CFlashErrorDialog::NewL(MFlashEngine& aFlashEngine, TInt aDialogResId, TInt aMsgResId, CEikonEnv& aEikonEnv)
{
	CFlashErrorDialog* self = new(ELeave) CFlashErrorDialog(aFlashEngine);
	CleanupStack::PushL(self);
	self->ConstructSleepingDialogL(aDialogResId);
	HBufC* message = aEikonEnv.AllocReadResourceL(aMsgResId);
	CleanupStack::PushL(message);
	self->SetTextL(*message);
	CleanupStack::PopAndDestroy(message);
	CleanupStack::Pop(self);
	return self;
}

void CFlashErrorDialog::ConstructSleepingDialogL(TInt aResourceId)
{
	CAknNoteDialog::ConstructSleepingDialogL(aResourceId);
}

TBool CFlashErrorDialog::OkToExitL(TInt /*aButtonId*/)
{
	iFlashEngine.AbortPlayer();
	return ETrue;
}

TInt CFlashErrorDialog::RouseSleepingDialog()
{
	return CAknNoteDialog::RouseSleepingDialog();
}
////////////////////////////////////////////////////////////////////////////////
// IniData utility class
////////////////////////////////////////////////////////////////////////////////

_LIT(KFlashIniFile,"c:\\system\\apps\\saflash2\\saflash2.ini");
//_LIT(KFlashUserIniFile,"c:\\system\\apps\\saflash2\\saflash2_user.ini");

IniData::IniData(RFs& aFs)
:iStaticHeapSize(KSAFlashAppStaticHeapSize)
,iDynamicHeapMaxSize(KSAFlashAppDynamicHeapMaxSize)
,iDebugMode(0)
,iVolume(KDefaultVolume)
,iEngineMode(0)
,iIAPId(0)
,iBackgroundAlpha(255)
,iPlayerBlendsBackground(0)
,iFs(aFs)
{

	// set default values
	iEngineMode = KSupportSVG | KSupportSharedObject | KSupportSound | KSupportVideo | KSupportNetwork | KSupportLocalFile |KSupportStereo;
}

IniData::~IniData()
{

}


//_LIT8(KFlashBackgroundAlpha, "BackgroundAlpha");
//_LIT8(KFlashPlayerBlendsBackground, "PlayerBlendsBackground");
	  
void IniData::LoadL()
{


	TInt retVal = KErrNone;
	
	TInt staticHeapsSizeInt = KSAFlashAppStaticHeapSize;
	TInt dynamicHeapMaxSizeInt = KSAFlashAppDynamicHeapMaxSize;
	TInt debugModeInt = 0;
	TInt volumeInt = KMaxUserVolume;
	TInt engineModeInt = KSupportSharedObject | KSupportSound | KSupportVideo | KSupportNetwork | KSupportLocalFile |KSupportStereo;
	TInt aIAPIdInt;

	CRepository* repository=NULL;
	// Connecting to repository
	TRAPD( err, repository = CRepository::NewL(KCRUIdFlashliteViewer)); 
	if (err == KErrNone)
		{
		// Getting the values
		retVal = repository->Get(KFlashliteVwrStaticHeapSize, staticHeapsSizeInt);
		if ( retVal == KErrNone ) iStaticHeapSize = (TUint32)staticHeapsSizeInt;

		retVal = repository->Get(KFlashliteVwrDynamicHeapMaxSize, dynamicHeapMaxSizeInt);
		if ( retVal == KErrNone ) iDynamicHeapMaxSize = (TUint32)dynamicHeapMaxSizeInt;

		retVal = repository->Get(KFlashliteVwrIAPId, aIAPIdInt);
		if ( retVal == KErrNone ) iIAPId = (TUint32)aIAPIdInt;
		
		retVal = repository->Get(KFlashliteVwrEngineMode, engineModeInt);
		if ( retVal == KErrNone ) iEngineMode = (TUint32)engineModeInt;

		retVal = repository->Get(KFlashliteVwrDebugMode, debugModeInt);
		if ( retVal == KErrNone ) iDebugMode = (TUint32)debugModeInt;

		retVal = repository->Get(KFlashliteVwrVolume, volumeInt);
		if ( retVal == KErrNone ) iVolume = (TUint32)volumeInt;
		
		}
		
	if(repository)
	{
		delete repository;	
	}
	

}

void IniData::SaveUserDataL()
{
	TInt volumeInt = (TInt)iVolume;
	TInt IAPIdInt = (TInt)iIAPId;
	
	CRepository* repository = NULL;
	// Connecting to repository
	TRAPD( err, repository = CRepository::NewL(KCRUIdFlashliteViewer));
	if (err == KErrNone)
	{
		// Setting the values - only volume is read-write
		repository->Set(KFlashliteVwrVolume, volumeInt);
		repository->Set(KFlashliteVwrIAPId, IAPIdInt);
		// Close connection
		delete repository;
	}
}

#ifdef EKA2
_LIT(KLitCDrive,"c:\\private\\");
#endif //EKA2
CDictionaryFileStore* IniData::OpenIniFileLC()
{
#ifdef EKA2
	TFileName fn;
	fn.Append(KLitCDrive);	
	
	// now get the UID for the process we are running in so we can figure the data caged location
	TUid uid(User::Identity());
		
	//uidName consists of Uid name of application. For Ex. [101fd693]
	TPtrC uidName(uid.Name().Mid(1,8));
	
	fn.Append(uidName);
	fn.Append(KPathDelimiter);
	
	TParsePtrC iniPath(fn);
#else //EKA2
	TParsePtrC iniPath(KFlashIniFile);
#endif //EKA2
	if (!BaflUtils::FolderExists(iFs, iniPath.DriveAndPath()))
	{
		iFs.MkDirAll(KFlashIniFile);
	}
	return CDictionaryFileStore::OpenLC(iFs,KFlashIniFile,KUidSAFlash);
}

_LIT(KLitPrivateDir,":\\private\\");


//static 
void FlhUiUtils::GetFileInAppDir(CEikonEnv& aEnv, TDes& aFullPathFileName, const TDesC& aFileName, const TDesC& aDriveLetter)
{
	TFileName applicationFullName(aEnv.EikAppUi()->Application()->AppFullName());
		
	TChar chr=applicationFullName[0];
	aFullPathFileName.SetLength(1);
	
	if(aDriveLetter.Length())
	{
		chr=aDriveLetter[0];
	}
	
	aFullPathFileName[0]=chr;
	
	aFullPathFileName.Append(KLitPrivateDir);	
	
	// now get the UID for the process we are running in so we can figure the data caged location
	TUid uid(User::Identity());
		
	//uidName consists of Uid name of application. For Ex. [101fd693]
	TPtrC uidName(uid.Name().Mid(1,8));
	
	aFullPathFileName.Append(uidName);
	aFullPathFileName.Append(KPathDelimiter);
	const TPtrC driveAndPath(aFullPathFileName);

	aFullPathFileName.Append(aFileName);


	RFs& fs=aEnv.FsSession();
	if (!BaflUtils::FolderExists(fs, driveAndPath))
	{
		/*ignoreError=*/fs.MkDirAll(driveAndPath);
	}
}

TInt FlhUiUtils::SearchResource( TBuf16<128>& aFileName, TBuf16<128>& aResourcePath, TBuf16<512>& aVariantFile )
    {
        // strip the extension
        TInt fileNameLength = aFileName.Length();
        TFileName resourceFileName;
        
        RFs fileServerSession = CCoeEnv::Static()->FsSession();
 
        TBuf16<512> variantFile;
        _LIT16( KFormat, "%c:%S\\%S.r%S%d");

        if( fileNameLength > 4 )
        {
            // we want just the name 
            // ex, if aFileName is flash2ui.rsc,
            // fName will be flash2ui
            TBuf16<128> fName = aFileName.Mid( 0, fileNameLength - 4 );
        
            // search for the resource file based on the current language
            TInt languageID = User::Language();
            TBuf16<8> leadingZero;
            leadingZero.Copy( _L("") );
            if( languageID < 10 )
                leadingZero.Copy( _L("0") );
                
        
            // start from Y: go to A: and then finally search Z:
            for( TInt i=EDriveY; i>=EDriveA; i-- )
            {
                TChar driveLetter;
                fileServerSession.DriveToChar( i, driveLetter );
                variantFile.Format(KFormat, driveLetter, &aResourcePath, &fName, &leadingZero, languageID );
                
                resourceFileName.Copy( variantFile );
                TBool fileExists = BaflUtils::FileExists( CCoeEnv::Static()->FsSession(), resourceFileName );
                if( fileExists )
                {
                    aVariantFile.Copy( variantFile );
                    return KErrNone;
                }
                
            }
            // see if it is present on z drive
            TChar driveLetterZ;
            fileServerSession.DriveToChar( EDriveZ, driveLetterZ );
            variantFile.Format(KFormat, driveLetterZ, &aResourcePath, &fName, &leadingZero, languageID );
            resourceFileName.Copy( variantFile );
            TBool fileExists = BaflUtils::FileExists( CCoeEnv::Static()->FsSession(), resourceFileName );
            if( fileExists )
            {
                aVariantFile.Copy( variantFile );
                return KErrNone;
            }
            

            // if exact language is not found, search for the nearest language in the respective drive
            // start from Y: go to A: and then finally search Z:
            _LIT16( KNearestLangFormat, "%c:%S\\%S");
            for( TInt i=EDriveY; i>=EDriveA; i-- )
            {
                TChar driveLetter;
                fileServerSession.DriveToChar( i, driveLetter );
                variantFile.Format(KNearestLangFormat, driveLetter, &aResourcePath, &aFileName );    // use the filename that was passed in
            
                
                resourceFileName.Copy( variantFile );
                BaflUtils::NearestLanguageFile( CCoeEnv::Static()->FsSession(), resourceFileName );
                TBool fileExists = BaflUtils::FileExists( CCoeEnv::Static()->FsSession(), resourceFileName );
                if( fileExists )
                {
                    aVariantFile.Copy( variantFile );
                    return KErrNone;
                }
                
            }
            // see if it is present on z drive
            // TBuf16<128> KZFormat( _L("z:%S\\%S"));
            variantFile.Format(KNearestLangFormat, driveLetterZ, &aResourcePath, &aFileName );
           
            
            resourceFileName.Copy( variantFile );
            BaflUtils::NearestLanguageFile( CCoeEnv::Static()->FsSession(), resourceFileName );
            fileExists = BaflUtils::FileExists( CCoeEnv::Static()->FsSession(), resourceFileName );
            if( fileExists )
            {
                aVariantFile.Copy( variantFile );
                return KErrNone;
            }
            
        }
        
        return KErrNotFound;
    }
