/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the volume control class.
*
*/


#include <AknsSkinInstance.h> 
#include <AknsUtils.h>
#include <Aknvolumecontrol.h>
//#ifdef __FLASHLITE2.0_REL3.1
#include <AknDef.h>
#include <AknUtils.h>
#include <AknIconUtils.h>
#include <gulicon.h>
#include <SAFlash.MBG>
#include <barsread.h>
#include <featmgr.h>
//#endif //__FLASHLITE2.0_REL3.1

#include "symbian_volumecontrol.h"
#include "symbian_volumeview.h"
#include "flash_engine.h"
#include "saflash_consts.h"
#include "ui_defs.h"
#include "ui_data_caging_literals.hrh"

const TInt KTriArrayGranularity=3;
const TInt KVolumeValue = 1;

void FillPointArrayL(CArrayFix<TPoint>& aArray)
{
	TPoint empty(0,0);

	for(TInt i=0;i<KTriArrayGranularity;i++)
	{
		aArray.AppendL(empty);
	}

	__ASSERT_DEBUG(aArray.Count()==KTriArrayGranularity,User::Invariant()); // TODO - Flash panic codes
}

CSaFlashVolumeControl::CSaFlashVolumeControl(CSaFlashVolumeView* aView)
{
	iView = aView;
	iVolume = iView->iEngine->GetVolume();
	iMaxVolume = KMaxUserVolume;
	iDrawnVolume = KUnMuteMask;
	iActivatedNow = ETrue;
}

CSaFlashVolumeControl::~CSaFlashVolumeControl()
{
	FeatureManager::UnInitializeLib(); // Uninitialise Feature manager
	delete iAknVolumeControl;
	delete iVolumeBitmap;
	delete iMuteIcon;
}

void CSaFlashVolumeControl::ConstructL(const TRect& aRect)
{
	CreateWindowL();
	FeatureManager::InitializeLibL();  // initialise feature manager
	TResourceReader reader;
	iAknVolumeControl = new(ELeave) CAknVolumeControl;
	iAknVolumeControl->SetContainerWindowL( *this );
	iCoeEnv->CreateResourceReaderLC(reader, R_AVKON_POPUP_VOLUME_INDICATOR);	
	iAknVolumeControl->ConstructFromResourceL( reader );	
	CleanupStack::PopAndDestroy(); // reader
	
	// Default volume bitmap with minimum volume.
	iVolumeBitmap = iAknVolumeControl->CreateBitmapL( KVolumeValue );
	SetRect(aRect);
	ActivateL();
	
	CFbsBitmap* bitmap = NULL;
    CFbsBitmap* mask = NULL;
    SizeChanged();
    //Create mute icon.
	AknIconUtils::CreateIconLC(
        bitmap,
        mask,
        KFlash2UIMifFilePath,	
        EMbmSaflashQgn_indi_speaker_muted, 
        EMbmSaflashQgn_indi_speaker_muted_mask );
   iMuteIcon = CGulIcon::NewL( bitmap, mask );
   iMuteIcon->SetBitmapsOwnedExternally( EFalse );
   CleanupStack::Pop( 2 ); // bitmap and mask owned by aIcon
        
}

void CSaFlashVolumeControl::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();
	
    if(iActivatedNow)
    {
    	CONST_CAST(CSaFlashVolumeControl*, this)->iActivatedNow = EFalse;
    	iView->iEngine->RefreshScreen();
    }
    
    iView->iEngine->Draw(iEngRect, gc);
	
	TRAPD(err,((CSaFlashVolumeControl*)this)->DrawControlL(aRect));
	if(err != KErrNone)
	{
		iView->HandleError(err);	
	}
	
#ifdef SYMBIAN_BUILD_GCE
    ControlEnv()->WsSession().Finish();
#endif 
}


void CSaFlashVolumeControl::DrawControlL(const TRect& /*aRect*/)
{
	if (iVolume & KMuteMask)
		{
		//iDrawnVolume = iVolume & KUnMuteMask;
		iDrawnVolume = 0;
		iAknVolumeControl->SetValue(KVolumeValue);
		}
	else
		{
		iDrawnVolume = iVolume;
		iAknVolumeControl->SetValue(iVolume);
		}
	
	CWindowGc& gc = SystemGc();
		
	// clear the mute bit
	gc.SetPenStyle(CGraphicsContext::ENullPen);
	gc.SetBrushColor(KRgbWhite);
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);


	TSize volumebitmapSize = iVolumeBitmap->SizeInPixels();

	
    TPoint volTopPoint(iEngRect.iTl.iX, iEngRect.iBr.iY);
    TSize volStripSize(iEngRect.Size().iWidth ,volumebitmapSize.iHeight);
    TRect volStripRect(volTopPoint, volStripSize);
    gc.DrawRect(volStripRect);
    
    TPoint volumeBitmapTopPoint(volStripRect.iBr.iX/2 - volumebitmapSize.iWidth/2, volTopPoint.iY);
	iVolumeBitmapRect.SetRect(volumeBitmapTopPoint, volumebitmapSize);
	iAknVolumeControl->SetRect(iVolumeBitmapRect);
	
	
	if( iDrawnVolume <= 0)
    	{
    	TPoint muteIconTopPoint((volumeBitmapTopPoint.iX - volStripSize.iHeight), volStripRect.iTl.iY);
    	TSize muteIconSize(volStripSize.iHeight, volStripSize.iHeight);
    	TRect muteIconRect(muteIconTopPoint, muteIconSize);
    	
		// Create a bitmap with the correct background ( /icon) color
		CFbsBitmap* bg = new (ELeave) CFbsBitmap();
		CleanupStack::PushL( bg );
		TDisplayMode dispMode = CCoeEnv::Static()->ScreenDevice()->DisplayMode();
		bg->Create(muteIconRect.Size(), dispMode);
	    CFbsBitGc* fbsBitGc = CFbsBitGc::NewL(); //graphic context
    	CleanupStack::PushL( fbsBitGc );
    	CFbsBitmapDevice* bmpDevice = CFbsBitmapDevice::NewL( bg );
    	fbsBitGc->Activate( bmpDevice );
    	TRgb color( KRgbBlack );
    	MAknsSkinInstance* skin = AknsUtils::SkinInstance();

    	AknsUtils::GetCachedColor( skin, color, 
    							   KAknsIIDQsnIconColors,
        						   EAknsCIQsnIconColorsCG1 );
   		fbsBitGc->SetBrushColor( color );
    	fbsBitGc->Clear(); // Fill bitmap with color
    	delete bmpDevice;
    	CleanupStack::PopAndDestroy();// fbsBitGc

		// draw the mute icon
		CFbsBitmap* muteBitmap = iMuteIcon->Bitmap();   	
		gc.DrawRect(muteIconRect);
	   	AknIconUtils::SetSize( muteBitmap, muteIconRect.Size() ); 
	   	AknIconUtils::SetSize( iMuteIcon->Mask(), muteIconRect.Size() ); 	   	

	   	TRect muteBitmapRect (TRect(TPoint(0,0),muteBitmap->SizeInPixels()));
	   	//gc.DrawBitmap(muteIconRect,muteBitmap);
	   	gc.BitBltMasked(muteIconRect.iTl,muteBitmap,muteBitmapRect,iMuteIcon->Mask(),ETrue);
	    CleanupStack::PopAndDestroy();// bg
	    
#ifdef SYMBIAN_BUILD_GCE
        ControlEnv()->WsSession().Finish();
#endif 	    
    	}
}

TKeyResponse CSaFlashVolumeControl::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
{
	if (aType == EEventKey)
	{
		switch (aKeyEvent.iCode) // The code of key event is...
		{
			case EKeyLeftArrow: // Left key.
			{
				if(iVolume & KMuteMask )
				{
					//1. Options-> Mute (at level 8)
					//2. Options-> VolumeLevel (with mute icon, at level 0)
					//3. If volumelevel is decreased by pressing LeftArrow key,it jumps to 8.
					//Above sequence shouldn't occur.
					iVolume = 0;
				}
				
				iVolume--;
				
				if (iVolume < 0)
            	{
                	iVolume = 0;
            	}
            	
            	if(iVolume == 0)
            	{
            		iView->ShowMuteIconL(ETrue);
            		iVolume |= KMuteMask;
            	}
            
            	if( iVolume > 0 )
            	{
            		iView->ShowMuteIconL(EFalse);
            	}
            	
				iView->iEngine->SetVolume(iVolume);
				iAknVolumeControl->OfferKeyEventL(aKeyEvent, aType);
				break;	
			}
			
			
		case EKeyRightArrow: // Right Key.
			{
				if( iVolume & KMuteMask )
				{
					//1. Options-> Mute (at level 8)
					//2. Options-> VolumeLevel (with mute icon, at level 0)
					//3. If volumelevel is increased by pressing Rightarrow key,it jumps to 8.
					//Above sequence shouldn't occur.
					iVolume = 0;
				}
				
				iVolume++;
				
				if ( iVolume > iMaxVolume )
            	{
                	iVolume = iMaxVolume;
            	}
            
				iView->iEngine->SetVolume(iVolume);
            	iView->ShowMuteIconL(EFalse);
            	iAknVolumeControl->OfferKeyEventL(aKeyEvent, aType);
            	break;
			}
			
		default:
			{
				break;
			}		
		
		} //switch
		
		DrawNow();
		return EKeyWasConsumed;
	}
	return EKeyWasNotConsumed;
}

void CSaFlashVolumeControl::Mute(TBool aMute)
{
	if (aMute)
	{
		iVolume |= KMuteMask;
	}
	else
	{
		iVolume &= KUnMuteMask;
	}
	iView->iEngine->SetVolume(iVolume);
	DrawNow();
}

void CSaFlashVolumeControl::AddControls()
{
}
//#ifdef __FLASHLITE2.0_REL3.1
void CSaFlashVolumeControl::HandleResourceChange(TInt aType)
{
	CCoeControl::HandleResourceChange(aType);
	//ADDED FOR SCALABLE UI SUPPORT
	if ( aType==KEikDynamicLayoutVariantSwitch )
	{
		TRect rect;
        AknLayoutUtils::LayoutMetricsRect(AknLayoutUtils::EMainPane,rect);        
		SetRect(rect);
		iView->HandleResourceChange(aType);
	}
}
//#endif //__FLASHLITE2.0_REL3.1

TInt CSaFlashVolumeControl::CountComponentControls() const
	{
    if(iAknVolumeControl)
	   	{
    	return 1;  //number of contained controls
    	}
    return NULL;
   	}

CCoeControl* CSaFlashVolumeControl::ComponentControl( TInt /*aIndex */) const
	{
    if(iAknVolumeControl)
    	{
    	return iAknVolumeControl;
    	}
    return NULL;	
	}


void CSaFlashVolumeControl::SizeChanged() 
{     
    iVolumeRect = Rect();    
    TRect cEngRect(iVolumeRect.iTl, iVolumeRect.iBr);

	TSize volumebitmapSize = iVolumeBitmap->SizeInPixels();
    cEngRect.iBr.iY = cEngRect.iBr.iY - volumebitmapSize.iHeight;
    iEngRect = cEngRect;
}

void CSaFlashVolumeControl::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
	if(FeatureManager::FeatureSupported(KFeatureIdPenSupport)) //adding feature manager to check the feature at run time.
	{
	if (AknLayoutUtils::PenEnabled())
		{
		if (iVolumeBitmapRect.Contains(aPointerEvent.iPosition))
			{
			iAknVolumeControl->HandlePointerEventL(aPointerEvent);
			iVolume = iAknVolumeControl->Value();
			iView->iEngine->SetVolume(iVolume);
			if( iVolume > 0 )
				{
				iView->ShowMuteIconL(EFalse);
				DrawNow();
				}
			}
		}
	}
    CCoeControl::HandlePointerEventL(aPointerEvent);
}

