/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the volume view class.
*
*/


#include <eikbtgpc.h>
#include <flash2ui.rsg>

#include "symbian_volumeview.h"
#include "symbian_volumecontrol.h"
#include "flash_engine.h"
#include "saflash_consts.h"
#include "symbian_appui.h"
#include "flash2ui.hrh"
#include "ui_defs.h"

CSaFlashVolumeView::CSaFlashVolumeView(MFlashEngine* aEngine)
{
	iContainer = NULL;
	iEngine = aEngine;
}

void CSaFlashVolumeView::ConstructL()
{
	BaseConstructL(R_SAFLASH_VOLUMEVIEW);
}

CSaFlashVolumeView::~CSaFlashVolumeView()
{
}

TUid CSaFlashVolumeView::Id() const
{
	return KSAFlashVolumeViewId;
}

void CSaFlashVolumeView::DoActivateL(const TVwsViewId& /*aPrevViewId*/, TUid /*aCustomMessageId*/, const TDesC8& /*aCustomMessage*/ )
{
	iContainer = new(ELeave) CSaFlashVolumeControl(this) ;
	if(IsFullScreen())
	    {
	    StatusPane()->MakeVisible(ETrue);
	    }
	iContainer->ConstructL(AppUi()->ClientRect());
	iContainer->SetMopParent( this );
	
	iContainer->AddControls();

	iPrevVolume = iEngine->GetVolume();
	// Adds container to view control stack.
	AppUi()->AddToStackL( *this, iContainer );
	iContainer->MakeVisible( ETrue );
}

void CSaFlashVolumeView::DoDeactivate()
{
	iEngine->RefreshScreen();
	if (iContainer)
	{
		AppUi()->RemoveFromStack(iContainer);
		delete iContainer;
		iContainer = NULL;
	}
}
RWindow* CSaFlashVolumeView::GetWindow()
{
	return iContainer->GetWindow();
}
void CSaFlashVolumeView::HandleCommandL(TInt aCommand)
{
	switch(aCommand)
	{
	case EAknSoftkeyOk:
		((CSaFlashAppUi*)AppUi())->SaveVolumeL(iEngine->GetVolume());
		AppUi()->ActivateLocalViewL(KSAFlashPlayerViewId);
		break;
		
	case EEikFlashCmdMute:
		iContainer->Mute(ETrue);
		Cba()->SetCommandSetL(R_SOFTKEYS_OK_UNMUTE);
		Cba()->DrawNow();
		break;
		
	case EEikFlashCmdUnmute:
		iContainer->Mute(EFalse);
		Cba()->SetCommandSetL(R_SOFTKEYS_OK_MUTE);
		Cba()->DrawNow();
		break;
	case EAknSoftkeyCancel:
		iEngine->SetVolume(iPrevVolume);
		if( iPrevVolume & KMuteMask )
		{
			ShowMuteIconL(ETrue);
		}
		else
		{
			ShowMuteIconL(EFalse);
		}
		AppUi()->ActivateLocalViewL(KSAFlashPlayerViewId);
		break;
	}
}

CCoeControl* CSaFlashVolumeView::GetContainer()
	{
	return iContainer;
	}
void CSaFlashVolumeView::HandleResourceChange(TInt aType)
{
	((CSaFlashAppUi*)AppUi())->HandleResourceChangeInVolumeView(aType);
}
TBool CSaFlashVolumeView::IsFullScreen()
{
    return ((CSaFlashAppUi*)AppUi())->IsFullScreen();
}
void CSaFlashVolumeView::ShowMuteIconL(TBool aMute)
{
	((CSaFlashAppUi*)AppUi())->ShowMuteIconL(aMute);
}
void CSaFlashVolumeView::MakeStatusPaneInvisible()
{
  	StatusPane()->MakeVisible(EFalse);
}

void CSaFlashVolumeView::HandleError(TInt aErr)
{
	switch(aErr)
	{
		case KErrNoMemory:
			((CSaFlashAppUi*)AppUi())->NotifyClientError(EInsufficientMemoryError, ETrue, EFalse);
			break;
		default:
			((CSaFlashAppUi*)AppUi())->NotifyClientError(EGeneralError, EFalse, EFalse);
			break;
	}
}
TRect CSaFlashVolumeView::GetClippingRect()
    {
    return iContainer->GetClippingRect();
    }

//End of File

