/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of CVolumeDisplayTimer class.
*
*/

#include "symbian_playercontrol.h"
// ----------------------------------------------------
// CVolumeDisplayTimer::NewL
// Description: static constructor.
// Input:  	CFlashNaviHandler object
// Output: none
// Return: pointer to a CVolumeDisplayTimer object
// ----------------------------------------------------
CVolumeDisplayTimer* CVolumeDisplayTimer::NewL(CSaFlashPlayerControl* aControl)
{
	CVolumeDisplayTimer* self = new (ELeave) CVolumeDisplayTimer(aControl);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
	
}
// ----------------------------------------------------
// CVolumeDisplayTimer::~CVolumeDisplayTimer
// Description: destructor.
// Input:  none
// Output: none
// ----------------------------------------------------
CVolumeDisplayTimer::~CVolumeDisplayTimer()
{
    Cancel();	
}

// ----------------------------------------------------
// CVolumeDisplayTimer::DoCancel
// Description: Cancel outstanding requests.
// Input:  	none
// Output: none
// Return: none
// ----------------------------------------------------
void CVolumeDisplayTimer::DoCancel()
{
	CTimer::DoCancel(); 	
}

// ----------------------------------------------------
// CVolumeDisplayTimer::RunL
// Description: Asynchounous reply to timeout.
// Input:  	none
// Output: none
// Return: none
// ----------------------------------------------------
void CVolumeDisplayTimer::RunL()
{
	if(iStatus==KErrNone)
	{
		iControl->ShowVolumeIconL(EFalse);
	}	
}
                               
// ----------------------------------------------------
// CVolumeDisplayTimer::RunL
// Description: Constructor.
// Input:  CFlashNaviHandler* aNaviHandler
// Output: none
// Return: none
// ----------------------------------------------------
CVolumeDisplayTimer::CVolumeDisplayTimer(CSaFlashPlayerControl* aControl) :
											CTimer(CActive::EPriorityStandard),
											iControl(aControl)
{
	
}

// ----------------------------------------------------
// CVolumeDisplayTimer::RunL
// Description: Second phase constructor. Construct
// the Volume control and initialise the timer
// Input:  none
// Output: none
// Return: none
// ----------------------------------------------------
void CVolumeDisplayTimer::ConstructL()
{
    // Call the base class ConstructL
    CTimer::ConstructL();

    // Add this timer to the active scheduler
    CActiveScheduler::Add(this);	
}

// ----------------------------------------------------
// CVolumeDisplayTimer::StartVolumeTimer
// Description: Sets the timer going
// Input:  none
// Output: none
// Return: none
// ----------------------------------------------------
void CVolumeDisplayTimer::StartVolumeTimer()
{
	const TInt KVolumeDisplayTimeOut(2*1000*1000); //2 seconds
	
	// cancel any existing timer
	Cancel();
	After(KVolumeDisplayTimeOut);
}
