/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  A container storing a list of callback event
*
*/


#ifndef CALLBACKSCHEDULER_H
#define CALLBACKSCHEDULER_H

#include <e32base.h>


/**
 *  Simulation of a playback module
 */
class CCallbackScheduler : public CActive
    {
public:
    /**
     * Two-phased constructor
     *
     * @return object of constructed
     */
    static CCallbackScheduler* NewL();
    
    /**
     * Destructor
     */
    virtual ~CCallbackScheduler();

    /**
     * Reset the scheduler
     */
    void Reset();
    
    /**
     * Append an event callback into container
     * @param aCallback, a callback will be executed after certain interval
     * @param anInterval, an interval in millisec the callback will be executed 
     * NOTE: once the scheduler is started, no more event can be append until
     *       scheduler has been reseted
     */
    void AppendEvent(TCallBack aCallback);

    void DoCancel();
    void RunL();
    TInt RunError(TInt aError);
    
private:
    /**
     * C++ constructor
     */
    CCallbackScheduler();
    
    /**
     * Second phase constructor
     */
    void ConstructL();

    
private:    // data
    RArray<TCallBack>           iEventList;
    };


#endif  // CALLBACKSCHEDULER_H    

