/*
* Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/

#ifndef __FLPLATFORMDECODER_H
#define __FLPLATFORMDECODER_H

#include "IVideoDecompressor.h"
#include "flash_video_codec.h"
#include <e32base.h>
#include <w32std.h>
#include "flrtpsource.h"

// Composition offset cannot be more than 1 sec(30 frames ahead in time)
#define KMaxCompositionTimeOffset 1000

enum THXVDState
{
	EHXVDInitial,
	EHXVDFirstFrameReceived,
	EHXVDInitialize,
	EHXVDInitComplete,
	EHXVDStarted,
	EHXVDSeeking,
	EHXVDReseting,
	EHXVDErrorOccured
};

struct IAudioInfoStruct
{
	S32		formatID; // quick way to check has the format changed since the last package. combination of the three parameters below.
	U8		channels; // the number of audio channels (1 = mono, 2 = stereo)
	U8		bitsPerSample; // number of bits per sample, usually 8 or 16
	U8		compressFormat; // one of IAudioFormats
	U8		sampleRate; // samplerate of the audio. one of IAudioSampleRates
};

enum IAudioSampleRates
{
	IAudio5kHz = 0,
	IAudio11kHz = 1,
	IAudio22kHz = 2,
	IAudio44kHz = 3
};

enum IAudioFormats
{
	IAudioCompressNone	= 0,
	IAudioCompressADPCM = 1,
	IAudioCompressMP3 = 2,
	IAudioCompressAAC = 3
};

static	void DumpAVCConfigToFile(const TUint8* aData, TUint aDataLength);
static  int	BinTo64(const TUint8* pInBuf, int len, char* pOutBuf);
static	void Output64Chunk(int c1, int c2, int c3, int pads, char* pBuf,
   									int bufOffset);


// FORWARD DECLARATIONS
class CFLPlatformDecoder;

// -------------------------------------------------------------------------
// class CFLPlatformDecoder 
//
// Class representing the decoder interface to the actual helix side flv decoder 
// Also represents a wrapper or medium to send the media data to XPS. 
// --------------------------------------------------------------------------
// 

class CFLPlatformDecoder : public CBase, 
						 public IVideoDecompressorInstance, 
						 public MFlRtpSourceObserver
{
public:
    static CFLPlatformDecoder* NewL(void *aAllocator, TUserMode aUserMode, const TCodecType& aCodecType, 
    							  CPlatformFlashEngine& aEngine);
    virtual ~CFLPlatformDecoder();
	
	void 	Initialize();	
	void 	DispatchHXCommand(const THXVideoDecCommand& aCommand, const THXVDCmdArg& aArgInfo);
		
	inline  TInt32 	CodecType() const;
	inline  TInt 	AudioStreamId() const;
	inline  void 	SetWindow(RWindow* aWindow);
	inline  void 	SetUserMode(const TUserMode& aUserMode);	
	inline  TInt 	MaxVolume() const;
	inline  TBool 	CheckState(THXVDState aState) const;
	inline  void 	StoreInitialVolume(TInt aVolume);
	inline  void 	SetSwfVolume(TInt aSwfVolume);
	inline  TInt 	GetSwfVolume() const;
	inline 	TBool	IsComplete() const;	

	inline  CFLRtpSource* RtpSource();
	void 		SetMetadata(IVideoMetaDataStruct* aMetaData);

	TTimeIntervalMicroSeconds GetVideoPosition();
	TInt GetEngineVolume() const;

private:
    CFLPlatformDecoder(void *aAllocator, TUserMode aUserMode, const TCodecType& aCodecType, 
    				 CPlatformFlashEngine& aEngine);
    				 
    void 		ConstructL();
    void        ASPause();    
    void 		Pause();
	void 		Play();
	void        Resume();
	void 		Stop();
	void 		Restart();
	void        RestartInPausedState();
	void 		Seek();	
	void        Reset();
	
	TInt 		SeekVideo(const TStreamPacketData& aPktData);
	TInt 		ResetVideo(const TStreamPacketData& aPktData);
	TInt 		SeekAudio(const TStreamPacketData& aPktData);
	TInt 		ResetAudio(const TStreamPacketData& aPktData);
	
	TInt 		DecodeAudioData(TStreamPacketData& aPktData);
	TInt 		DecodeVideoFrame(TStreamPacketData& aPktData, const TSize& aSize); 
	TInt 		GetSampleRate(const IAudioSampleRates& aSampleRate);
	TCodecType  GetAudioCodecType(const TUint& aAudioFormat);
	void 		InitializeAudioL(const IAudioInfoStruct* aAudioInfo);
	void 		StoreAvcInfo(TInt aProfileInfo, TInt aConstraintInfo, TInt aLevelInfo);
	void 		SetVideoRectL(FI_Rect* aRectangle);	
	void 		SetSwfWindowOrigin(const TUserMode& aUserMode, const TScreenMode& aScreenMode);
	void 		UpdateSeekState();
	void 		UpdateResetState();
	void 		CalculateVideoRectangleL(const TRect& aVideoRect);
	TRect		CalculateClippingRegion(const TRect& aVideoRect, 
										const TUserMode& aUserMode);
	void 		SetPadSizeInEmbeddedVideo(const TSize& aSize);
	TInt 		ParseHeader(TStreamPacketData& aPktData, TSize& aFrameSize);
    TInt 		ParseSparkHeader(TStreamPacketData& aPktData, TSize& aFrameSize);
    TInt 		ParseVp6Header(TStreamPacketData& aPktData, TSize& aFrameSize);

	inline 		THXVDState 	State() const;
	inline      void 		SetVolume(const TInt& aVolume) const;
	inline      void 		SetScreenMode(const TScreenMode& aScreenMode) const;
	void        GetAudioFormat(const char aAudioFormatInfo, IAudioInfoStruct *aAudioInfoStruct);
	bool 		DecompressAudioData(IVideoDecompressorInstance * aDecompressor,
							IVideoExternalBufferingDataStruct *aDecompressData);
	bool 		DecompressVideoData(IVideoDecompressorInstance * aDecompressor,
							IVideoExternalBufferingDataStruct *aDecompressData);
	TInt 		ParseAndQueueAllNALUnits(IVideoDecompressorInstance * aDecompressor,IVideoExternalBufferingDataStruct *aDecompressorData);
	TInt 		DecodeH264VideoFrame(TStreamPacketData& aPktData);
	TInt 		GetNalLength(TUint8* aPayldPtr);
	TInt 		SetAvcConfigData(TUint8* aCfgData, TInt aCfgLen);
	TInt 		SetAacConfigData(TUint8* aCfgData, TInt aCfgLen);
	TBool 		CheckPlayerState(TInt aState);
	bool 		GetFrameSize(TSize& aFrameSize);
    IVideoDecompressorInstance* CreateFlashDecompressor();
    void 		DestroyFlashDecompressor();
	TInt StreamingPauseDoReset(const TInt& aStreamId, const TStreamPacketData& aPktData);
	void UpdateStreamingResetState();
	
public:
	// IVideoDecompressorInstanceFunctions
	static S32  CodecSubType( IVideoDecompressorInstance * decompressor, U32 type );

	static bool EmbeddedIsKeyFrame( IVideoDecompressorInstance * decompressor, 
									const U8 * data, U32 len, bool alphaCodec );

	static bool DecompressFrame( IVideoDecompressorInstance * decompressor, 
								 IVideoDecompressDataStruct *decompressData );

	static bool BlitTo32( IVideoDecompressorInstance * decompressor, 
						  IVideoBlitTo32Struct *blitData );

	static bool BlitTo32Single( IVideoDecompressorInstance * decompressor,
								IVideoBlitTo32SingleStruct *blitData );

	static bool BlitTo16( IVideoDecompressorInstance * decompressor, 
						  IVideoBlitTo16Struct *blitData );

	static bool BlitTo16Single( IVideoDecompressorInstance * decompressor, 
								IVideoBlitTo16SingleStruct *blitData );

	static bool GetFrameDimensions( IVideoDecompressorInstance * decompressor, 
								    IVideoDecompressDataStruct *decompressData );

	static bool SetAudioFormat( IVideoDecompressorInstance * decompressor, 
								IAudioInfoStruct *audioInfo );

	static void SetVideoRectangle( IVideoDecompressorInstance * decompressor, 
								   FI_Rect* rectangle );

	static bool IsExternalVideoRenderingEnabled( IVideoDecompressorInstance * decompressor );
	
	static S32 GetVideoPosition(IVideoDecompressorInstance * decompressor);
	
	static bool FlushAfterSeek(IVideoDecompressorInstance* aDecompressor);

	static bool MiscData( IVideoDecompressorInstance * aDecompressor, IVideoMiscDataStruct *decompressData);
	
	static bool DecompressData(IVideoDecompressorInstance * aDecompressor,
							IVideoExternalBufferingDataStruct *aDecompressData);
							
	static bool QueryVideoConfigFlags(IVideoDecompressorInstance * aDecompressor,U32 aAudioCodec, U32* aVideoConfigFlags);
	static void		SendEmbeddedAudioInfo(U8 audioHeader);

    static bool DecompressAudioEmbedded (IVideoDecompressorInstance * aDecompressor, IVideoExternalBufferingDataStruct *aDecompressData);	
	

	void ParseConfigData(const TUint8* aData, TUint aDataLength);	
public: // From MFlRtpSourceObserver
	void MfrsoInitComplete(const TInt& aError);	
	void MfrsoReadyToStart(const TInt& aError);
	void MfrsoPlayerStateChanged(TPlayerState aState);
	void MfrsoPlaybackComplete(const TInt& aError);
	void MfrsoHandleError(const TInt& aError);

private:
    void CFLPlatformDecoder::ProcessHXCommand(const THXVideoDecCommand& aCommand, 
                                             const THXVDCmdArg& aArgInfo);
    
    
private:
    class THxCommandBuf
    {
    public:
        THxCommandBuf(const THXVideoDecCommand& aCmd, const THXVDCmdArg& aArg) {
            iCommand    = aCmd;
            iCommandArg = aArg;
        }

        THXVideoDecCommand iCommand;
        THXVDCmdArg        iCommandArg;
    };
    
    RArray<THxCommandBuf>   iCommandQueue;
    
	
private:    
    CFLRtpSource*		 	iFlRtpSource;
    CHelixGlobals* 			iHelixGlobals;
    CPlatformFlashEngine&	iEngine;
    
    TSize		 		    iFrameSize;
    TSize					iPadSize;

    TBool					iInitWaiting;
    TBool					iVideoRectSet;
    TBool					iChangedScreenMode;
    TBool					iPlayComplete;
    TBool					iSeekAudio;
    TBool					iSeekVideo;  
    TBool					iAudioInitComplete;

	TBool					iDoResetVideoStream;
	TBool					iDoResetAudioStream;

    RWindow* 				iWindow;

    TInt					iVideoStreamId;
    TInt					iAudioStreamId;
    TInt					iSeekCount;
	TInt					iResetStreamCount;
    TUint					iDuration;

    THXVDState 				iState;    
    TCodecType				iVideoCodecType;
    TCodecType				iAudioCodecType;
    
    TPoint					iSwfOrigin;
    TRect   				iSwfRect;
    TRect 					iVideoRect;
    TRect					iClipRect;
    TInt32					iPrevPlayTime;
    
    TTimeIntervalMicroSeconds iVideoPosition;    

    TInt					iInitialVolume;    
    TInt					iSwfVolume;    
    TInt 					iProfileInfo;
    TInt					iLevelInfo;
    TInt					iSizeOfNalUnitLength;

    TTimeIntervalMicroSeconds               iSeekPosition;
	TBool					iFrameSizeKnown;
    TUserMode				iUserMode;
    IVideoDecompressorInstance *iEmbeddedDecompressorInstance;
    const IVideoDecompressorInstanceFunctions*	 iEmbeddedDecompressorInstanceFncs;
    void *								iAllocator;
	
	//Indicates if helix was paused from Actionscript. EFalse if it is paused because of other reason
	//like application going to background event.
	TBool                   iPausedFromAS;
    TBool					iStreamingVideoReset;
    TBool					iStreamingAudioReset;
    TInt					iStreamingPauseResetCount;
    TBool 					iSeekInStreamingPausedState;
    TBool					iSeekPositionGot;
    TBool					iAvcConfigDone;
    TBool					iAacConfigDone;
    TBool					iRtmpStreamingPaused;
    TBool					iRtmpStreamingResuming;
    //Flag used for taking debug traces
    TBool                   iDoLog;
    
protected:
    IVideoDecompressorInstanceFunctions*	 iInstanceFncs;
    IVideoMetaDataStruct* 			 iMetaData;
    TBool                            iAudioStream;
};

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::SetUserMode(TUserMode aUserMode)
// 
// Sets the use case mode of the video i.e., either through a Flash app or through web.  
// --------------------------------------------------------------------------
// 

inline void CFLPlatformDecoder::SetUserMode(const TUserMode& aUserMode)
{
    if( iFlRtpSource )
    {
        iFlRtpSource->SetUserMode(aUserMode);    
	}
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::SetWindow(RWindow& aWindow)
// 
// Sets the shared window to display the video to the video player utility.  
// --------------------------------------------------------------------------
// 

inline void CFLPlatformDecoder::SetWindow(RWindow* aWindow)
{
	iWindow = aWindow;	

	if( iFlRtpSource )
	{
		iFlRtpSource->SetWindow(aWindow);
	}	
}

// -------------------------------------------------------------------------
// TInt32 CFLPlatformDecoder::CodecType()
// 
// Gets the codec type  
// --------------------------------------------------------------------------
// 

inline TInt32 CFLPlatformDecoder::CodecType() const
{
	return iVideoCodecType;
}
// -------------------------------------------------------------------------
// TInt CFLPlatformDecoder::AudioStreamId()
// 
// Gets the audio stream id. 
// --------------------------------------------------------------------------
// 

inline TInt CFLPlatformDecoder::AudioStreamId() const
{
	return iAudioStreamId; 
}

// -------------------------------------------------------------------------
// THXVDState CFLPlatformDecoder::State()
// 
// Returns the current state of the decoder.  
// --------------------------------------------------------------------------
// 

inline THXVDState CFLPlatformDecoder::State() const
{
	return iState;
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::SetVolume(TInt aVolume)
// 
// Sets the current volume of the audio 
// --------------------------------------------------------------------------
// 

inline void CFLPlatformDecoder::SetVolume(const TInt& aVolume) const
{
	if( iFlRtpSource )
	{
		iFlRtpSource->SetVolume(aVolume);	
	}
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::SetScreenMode(TScreenMode aScreenMode)
// 
// Sets the screen modes i.e., normal screen ot fullscreen. 
// --------------------------------------------------------------------------
// 

inline void CFLPlatformDecoder::SetScreenMode(const TScreenMode& aScreenMode) const
{
	if( iFlRtpSource )
	{
		iFlRtpSource->SetScreenMode(aScreenMode);
	}
}

// -------------------------------------------------------------------------
// TInt CFLPlatformDecoder::MaxVolume()
// 
// Gets the maximum volume that can be set in video player utility. 
// --------------------------------------------------------------------------
// 

inline TInt CFLPlatformDecoder::MaxVolume() const
{
	if( !iFlRtpSource )
	{
		return 0;
	}
	
	return iFlRtpSource->MaxVolume();
}

// -------------------------------------------------------------------------
// inline void CFLPlatformDecoder::SetSwfVolume(TInt aSwfVolume)
// 
// Sets the swf Volume
// --------------------------------------------------------------------------
// 
inline void CFLPlatformDecoder::SetSwfVolume(TInt aSwfVolume)
{
	iSwfVolume = aSwfVolume;
}

// -------------------------------------------------------------------------
// inline TInt CFLPlatformDecoder::GetSwfVolume()
// 
// Gets the current swf volume
// --------------------------------------------------------------------------
// 
inline TInt CFLPlatformDecoder::GetSwfVolume() const
{
	return iSwfVolume;
}

// -------------------------------------------------------------------------
// inline TBool CFLPlatformDecoder::CheckState(THXVDState aState)
// 
// Compares the current state with aState
// --------------------------------------------------------------------------
// 
inline TBool CFLPlatformDecoder::CheckState(THXVDState aState) const
{
	return (iState == aState);
}

// -------------------------------------------------------------------------
// inline void CFLPlatformDecoder::StoreInitialVolume(TInt aVolume)
// 
// Stores the initial volume in iInitialVolume
// --------------------------------------------------------------------------
// 
inline void CFLPlatformDecoder::StoreInitialVolume(TInt aVolume)
{
	iInitialVolume = aVolume;	
}

// -------------------------------------------------------------------------
// inline TBool CFLPlatformDecoder::IsComplete() const
// 
// Tells whether Playback is completed. 
// --------------------------------------------------------------------------
// 

inline TBool CFLPlatformDecoder::IsComplete() const
{
	return iPlayComplete;
}

// -------------------------------------------------------------------------
// inline CFlRtpSource* CFLPlatformDecoder::GetRtpSource()
// 
// Returns the pointer to iFlRtpSource
// --------------------------------------------------------------------------
// 
inline CFLRtpSource* CFLPlatformDecoder::RtpSource()
{
	return (iFlRtpSource);
}

#endif // __FLPLATFORMDECODER_H
