/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Definition of the class  CFlRtpSource
*
*/


#ifndef __FLRTPSOURCE_H
#define __FLRTPSOURCE_H

#include "e32base.h"
#include "CXPSPacketSink.h"
#include <eikenv.h>
#include <videoplayer.h>
#include "helixdef.h"
#include "xpsmediastream.h"

#ifdef USE_VPU2
#include <videoplayer2.h> 
#include <surfaceeventhandler.h>
#endif

// Macros

#define ISPLAYERSTATE(aState)  (iPlayerState == aState)

class CHelixGlobals;
class CCallbackScheduler;

_LIT(KXPSServerName, "FlashXPSServer");
_LIT(KXPSUrl, "xps://www.S60.com/dummy.xps?XPSServer=");


const TUint KTimeStampIncr 	           = 120;
const TUint KMaxStreams                = 4;
//const TUint KMillisecondsForASec     = 1000;
const TUint KMaxSDPLength              = 3000;
/*
On seek, when key frames are not so frequent Player used for fall in deadlock situation due 
to packet overflow. So, value is increased to 450(previous value 200) to avoid overflow of 
packets on seek.
*/
const TUint KMaxStreamBuffers          = 450;
const TUint KAuxiliaryDuration         = 5;

/*                                                                    ___(streamend)____
                                                                     |                  |
                                                                     |                  V
    (initial)------->(initializing)------->(initcomplete)------->(started)--------->(stopped)
    <any_state>----error---->(dead)
*/


enum TRtpSrcState
{
    EInitial,
    EInitializing,
    EInitComplete,
    EStarted,
    EStreamEOF,
    EStopped,
    EDead
};

enum TPlayerState
{
    EHXPInitial,
    EHXPOpening,
    EHXPPreparing,
    EHXPPlaying,
    EHXPPaused,
    EHXPPlayCompleted,
    EHXPErrorOccured
};

class CPlatformFlashEngine;

// -------------------------------------------------------------------------
// class MFlRtpSourceObserver
//
// Represents the observer interface for RTP source object. 
// --------------------------------------------------------------------------
// 

class MFlRtpSourceObserver
{
public:
    virtual void MfrsoInitComplete(const TInt& aError) = 0;
    virtual void MfrsoReadyToStart(const TInt& aError) = 0;
    virtual void MfrsoPlaybackComplete(const TInt& aError) = 0;
    virtual void MfrsoPlayerStateChanged(TPlayerState aNewState) = 0;    
    virtual void MfrsoHandleError(const TInt& aError)  = 0;
};

// -------------------------------------------------------------------------
// class CFlRtpSource
//
// Represents the RTP source which pushes media data through RTP packets.  
// --------------------------------------------------------------------------
// 

class CFLRtpSource : public CBase
                    , public MXPSPktSinkObserver
                    , public MVideoPlayerUtilityObserver
#ifdef USE_VPU2
                    , public MMMFSurfaceEventHandler
#endif
{
public:
            static                    CFLRtpSource* NewL(CPlatformFlashEngine& aEngine);
                                      ~CFLRtpSource();

protected:
                                      CFLRtpSource(CPlatformFlashEngine& aEngine);
            void                      ConstructL();

public:
            void                      InitializeL(MFlRtpSourceObserver* aObs, const TCodecType& aCodecType);

    //stream apis
            TInt                      AddStreamL(const TXPSStreamType& aStreamType, const TCodecType& aCodecType);
            void                      RemoveStream(const TInt& aStreamId);
            void                      AddMedia();
            void                      RemoveMedia();

            void                      Start(const TInt& aStreamId);
            void                      ResetL(const TInt& aStreamId, const TStreamPacketData& aPktData);
			void                      CancelReset();
            TInt                      AddPacket(const TInt& aStreamId, const TStreamPacketData& aPktData);      
            void                      PausePktSupplyForAllStreams();
            void                      ResetAudioVideoStream();
            void                      StreamEnd();
            TBool                     IsStreamEnded() const;
    
    //playback apis
            void                      Play();
            void                      Pause();
            void                      Resume();    
	     	void 			     	  RTMPStreamResume();
            void                      Stop();
            void                      Restart();
            void                      RestartInPausedState();

            void                      SetPositionL(const TTimeIntervalMicroSeconds& aPosition);
            void                      SetVideoWindowRectL(const TRect& aWindowRect, const TRect& aClipRect, const TBool& aSetWindow);        
            void                      SetWindow(RWindow* aWindow);
            void                      SetFrameSize(const TSize& aFrameSize);
            void                      SetScreenMode(const TScreenMode& aMode);
            void                      SetVolume(const TInt& aVolume);
            void                      SetTotalDuration(const TInt& aDuration);
            void                      SetUserMode(const TUserMode& aUserMode);

            TTimeIntervalMicroSeconds GetVideoPosition();             
            TScreenMode               ScreenMode() const;            
            TInt                      MaxVolume() const;
            TInt                      TotalDuration() const;
            TUserMode                 UserMode() const;
            TRect                     GetAppRect();    
            TBool                     IsVideoStreamPresent() const;
            TBool                     IsAudioStreamPresent() const;        
            TInt                      MediaCount() const;
            TReal32                   VideoFrameRateL() const;   
            TInt                      AudioSamplingRate(TInt& aSamplingRate);
            TInt                      GetPrerollBufferSizeTime();
            
            CPlatformFlashEngine&     HostEngine() const;    
            TPlayerState              PlayerState() const;
            TRtpSrcState              RtpSrcState() const;
            void                      HandleHelixError(const TInt& aError);
            void                      HandlePrepareCompleteError(const TInt& aError);
           
            void                      SetAudioInfo(const TInt& aStreamId, const TAudioInfo& aAudioInfo);
            void                      SetAvcInfo(const TInt& aStreamId, const TAvcStreamInfo& aAvcInfo);
            TInt                      SetStreamConfig(const TInt& aStreamId, const TStreamConfigData& aAvcCfgData);
	     	void 					  StreamingPauseDoResetL(const TInt& aStreamId, const TStreamPacketData& aPktData);
            TInt                      GetStreamInfo(const TInt& aStreamId, CXPSMediaStream*& aStreamInfo);

    
public: // From MXPSPktSinkObserver
            void                      RestorePacketSupply(TUint aStreamId);

    
public: // From MVideoPlayerUtilityObserver
            void                      MvpuoOpenComplete( TInt aError );
            void                      MvpuoPrepareComplete( TInt aError );
            void                      MvpuoFrameReady( CFbsBitmap& aFrame, TInt aError );
            void                      MvpuoPlayComplete( TInt aError );
            void                      MvpuoEvent( const TMMFEvent& aEvent );
    
#ifdef USE_VPU2
public: // from MMMFSurfaceEventHandler
            void                      MmsehSurfaceCreated(TInt aDisplayId, const TSurfaceId& aId, const TRect& aCropRect, TVideoAspectRatio aAspectRatio);
            void                      MmsehSurfaceParametersChanged(const TSurfaceId& aId, const TRect& aCropRect, TVideoAspectRatio aAspectRatio);
            void                      MmsehRemoveSurface(const TSurfaceId& aId);    
#endif
            
private:
            void                      SetPlayerState(TPlayerState aState);
            TInt                      GetAudioStream(TInt& aStreamId, CXPSMediaStream*& aStreamInfo); 
            TInt                      GetVideoStream(TInt& aStreamId, CXPSMediaStream*& aStreamInfo);
            void                      InitCompleteL();
            void                      ConstructSDPInfoL();
            void                      ConstructVideoSDP(const TCodecType& aCodecType);
            void                      ConstructH264SDP();
            void                      ConstructAACSDP();    
	static  TInt 				      IssueCallback(TAny* aArgs);


public:
			RPointerArray<TInt>					    iArgList;

private:
#ifdef USE_VPU2
            CVideoPlayerUtility2*                   iVideoPlayer;            
#else            
            CVideoPlayerUtility*                    iVideoPlayer;
#endif            

            CXPSPacketSink*                         iXPSPacketSink;    

            RPointerArray<MFlRtpSourceObserver>     iObserverList;
            RPointerArray<CXPSMediaStream>          iXPSStreamList;
            RArray<TCodecType>                      iCodecTypeList;
			CCallbackScheduler*					    iCallbackScheduler;

            TRect                                   iWindowRect;
            RWindow*                                iWindow;
#ifdef USE_VPU2            
            RWindow*                                iCurrentDisplayWindow;        
#endif            
            RWsSession                              iWsSession;
            CWsScreenDevice*                        iScreenDevice;
            TScreenMode                             iScreenMode;
            TSize                                   iFrameSize;
            TRect                                   iClipRect;
            TRect                                   iAppRect;
            CEikAppUi*                              iApp;


			TTimeIntervalMicroSeconds				iSeekPosition;   
			TTimeIntervalMicroSeconds				iPlayCompleteVideoPosition;
			
            TBool                                   iVideoStreamPresent;
            TBool                                   iAudioStreamPresent;
            TInt                                    iTotalDuration;    
            TInt                                    iResetCount;
            TInt                                    iMediaCount;
            TUserMode                               iUserMode;
            TInt                                    iStreamEndCount;
            TInt                                    iRefreshCount;
            TInt                                    iInitCount;

            CPlatformFlashEngine&                   iHostEngine;
            TPlayerState                            iPlayerState;
            TRtpSrcState                            iState;

            HBufC8*                                 iSDPInfo;

            TBuf<KMaxFileName>                      iXpsServerName; 

            TBool					     			iDoLog;
            TInt                                    iStreamingPauseResetCount;
	     	TBool					     			iSeekPositionGot;
			
};

#endif // __FLRTPSOURCE_H
