/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class CFLPlatformDecoderInterface and other 
		 related utilities
*
*/


#ifndef __FLPLATFORMDECODER_INTERFACE_H
#define __FLPLATFORMDECODER_INTERFACE_H

#include <e32base.h>

#include "helixdef.h"
#include "IVideoDecompressor.h"
#include "flashliteinterfaces.h"

// -------------------------------------------------------------------------
// class CFLPlatformDecoderInterface
// 
// Class representing the decoder interface used by the Flash host engine.  
// --------------------------------------------------------------------------
// 

class CFLPlatformDecoderInterface : public CBase
{        
public :
    static CFLPlatformDecoderInterface* NewL(const TUserMode& aUserMode);
    virtual ~CFLPlatformDecoderInterface();
           
private:
    CFLPlatformDecoderInterface(const TUserMode& aUserMode);

public:
    IVideoDecompressorInstance* CreateVideoDecompressorL( 
    	MM_Object* aMMObj, void * aAllocator, U32 codecType);
    
    void DestroyVideoDecompressor( MM_Object * aMMObj, 
    	IVideoDecompressorInstance * decompressorInstance );
    
    void DispatchHXCommand(const THXVideoDecCommand& aCommand, const THXVDCmdArg& aArgInfo);
    IVideoDecompressorInstance* AddShare();    
    TInt MaxVolume() const; 
    TInt GetSwfVolume() const; 
    TBool IsComplete() const;
    IVideoDecompressorInstance* DecompresssorInstance();
	TTimeIntervalMicroSeconds GetVideoPosition();

private:
    IVideoDecompressorInstanceFunctions*     iInstanceFuncs;
    IVideoDecompressorInstance*              iInstance;
    TUserMode 								 iUserMode;									 
    TBool									 iHelixMode;
};


#endif // __FLPLATFORMDECODER_INTERFACE_H
