/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  A container storing a list of callback event
*
*/


// INCLUDES
#include "callbackscheduler.h"
#include <e32debug.h>
// CONSTANTS

// ======== MEMBER FUNCTIONS ========

// ---------------------------------------------------------------------------
// Two-phased constructor
// ---------------------------------------------------------------------------
//
CCallbackScheduler* CCallbackScheduler::NewL()
    {
    CCallbackScheduler* p = new(ELeave) CCallbackScheduler();
    CleanupStack::PushL(p);
    p->ConstructL();
    CleanupStack::Pop(p);
    return p;
    }
    
// ---------------------------------------------------------------------------
// Destructor
// ---------------------------------------------------------------------------
//
CCallbackScheduler::~CCallbackScheduler()
    {
	Cancel();

    iEventList.Reset();

    iEventList.Close();

    }


// ---------------------------------------------------------------------------
// Clears all the scheduled events
// ---------------------------------------------------------------------------
//
void CCallbackScheduler::Reset()
    {
	iEventList.Reset();
    }

// ---------------------------------------------------------------------------
// Append an event callback into container
// ---------------------------------------------------------------------------
//
void CCallbackScheduler::AppendEvent(TCallBack aCallback)
    {
    iEventList.Append(aCallback);
	if(!IsActive()) 
		{
		SetActive();
		TRequestStatus* status = &iStatus;
		User::RequestComplete(status,KErrNone);
		}
    }


// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
//
void CCallbackScheduler::RunL()
    {
	if(iEventList.Count() == 0)
		return;

	TCallBack lCbk = iEventList[0];
	iEventList.Remove(0);

	if(iEventList.Count())
	{
		SetActive();
		TRequestStatus* status = &iStatus;
		User::RequestComplete(status,KErrNone);
	}

    TRAP_IGNORE(lCbk.CallBack());
    }


// ---------------------------------------------------------------------------
// Cancel all the scheduled events
// ---------------------------------------------------------------------------
//
void CCallbackScheduler::DoCancel()
    {
	while(iEventList.Count())
		{
		TCallBack lCbk = iEventList[0];
		iEventList.Remove(0);
		TRAP_IGNORE(lCbk.CallBack());
		}
    }


// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
//
TInt CCallbackScheduler::RunError(TInt aError)
    {
	return aError;
    }



// ---------------------------------------------------------------------------
// C++ constructor
// ---------------------------------------------------------------------------
//
CCallbackScheduler::CCallbackScheduler()
	:CActive(EPriorityStandard)
    {
    }

// ---------------------------------------------------------------------------
// Second phase constructor
// ---------------------------------------------------------------------------
//
void CCallbackScheduler::ConstructL()
    {
	CActiveScheduler::Add(this);
	iEventList.Reset();
    }



