/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class 
*
*/


// -----------------------------------------------------------------
// NOTE: This MUST be the first include of every source file.
//		 It contains all the #define build parameters.

#include "platformbuild.h"		

// -----------------------------------------------------------------

#include "platformtypes.h"
#include "platformclib.h"
#include "flash_video_codec.h"
#include "flplatformdecoder.h" 
#include "symbian_engine.h"
#include "saflash_consts.h"
#include "symbian_plugin_interface.h"
#include "platform_dynamiclibrary_videodecompressor.h"

#include <string.h>
#include <coemain.h>
#include <bautils.h>

extern void DebugLog(TRefByValue<const TDesC> aText, ...);

// MACROS

#define FLPDECODERTRACE(args...) if(iDoLog) { RDebug::Print(##args); }
#define FLPDECODERTRACEF(args...) if(iDoLog) { RDebug::Printf(##args); }

#define SETSTATE(aState) 	iState = (aState)
#define CHECKSTATE(aState) (iState == (aState))

#define IsVideoSeeking() 	( CHECKSTATE(EHXVDSeeking) && !iSeekVideo)
#define IsAudioSeeking()	( CHECKSTATE(EHXVDSeeking) && !iSeekAudio)

#define IsVideoReseting()	(CHECKSTATE(EHXVDReseting) && iDoResetVideoStream)
#define IsAudioReseting()	(CHECKSTATE(EHXVDReseting) && iDoResetAudioStream)

#define IsStreamingPausedVideoResetPending() (iFlRtpSource->PlayerState() == EHXPPaused && iMetaData->isStreamed && !iStreamingVideoReset)
#define IsStreamingPausedAudioResetPending() (iFlRtpSource->PlayerState() == EHXPPaused && iMetaData->isStreamed && !iStreamingAudioReset)

#define IsAudioStreamPresent()	(iAudioStreamId >= 0)
#define IsVideoStreamPresent()	(iVideoStreamId >= 0)

#define IsAllStreamsEnded()	(iFlRtpSource->IsStreamEnded())

#define IsAudioPacket(pkt) (pkt->msg.msgID == kMessageAudio)
#define IsVideoPacket(pkt) (pkt->msg.msgID == kMessageVideo)

#define PROFILE_INFO_OFFSET	1
#define CONSTRAINT_INFO_OFFSET 2
#define LEVEL_INFO_OFFSET	3
#define	NAL_UNIT_LENGTH_SIZE_OFFSET 4

#define NAL_UNIT_LENGTH_SIZE_MASK 0x03 

#define FLASH_BUFFERING_RENDERING 0x80

const TUint KMaxVolumeFromFL     = 100;
const TUint KByteLength			 = 8;

const TUint KMaxPlaybackTime		= 18000;//5hrs

const char Base64[] ="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

#define XX 127

/*
 * Table for decoding base64
 */
#if 0
static const char Index64[256] = { /* Flawfinder: ignore */
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,62,XX,XX, XX,XX,XX,XX, XX,XX,63,62, XX,XX,XX,63,
    52,53,54,55, 56,57,58,59, 60,61,XX,XX, XX,XX,XX,XX,
    XX, 0, 1, 2,  3, 4, 5, 6,  7, 8, 9,10, 11,12,13,14,
    15,16,17,18, 19,20,21,22, 23,24,25,XX, XX,XX,XX,XX,
    XX,26,27,28, 29,30,31,32, 33,34,35,36, 37,38,39,40,
    41,42,43,44, 45,46,47,48, 49,50,51,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX
};
#define CHAR64(c)  (Index64[(unsigned char)(c)])
#endif

// -------------------------------------------------------------------------
// TUint16 Reverse16(TUint16* aPtr)
// 
//
// Reverses the bytes while reading (Used in case of big-endian architecutre)
// --------------------------------------------------------------------------
// 

TUint16 Reverse16(TUint16* aPtr)
{
	TUint8* p = (TUint8*) aPtr;
	TUint8 A = p[0];
	TUint8 B = p[1];
	TUint16 word = 0;
	word = (A << KByteLength) | B;
	return word;
}

// -------------------------------------------------------------------------
// CFLPlatformDecoder* CFLPlatformDecoder::NewL(TCodecType aCodecType, 
//										 IVideoMetaDataStruct* aMetaData,
//										 CPlatformFlashEngine& aEngine)
// 
//
// Static function to start construction. 
// --------------------------------------------------------------------------
// 

CFLPlatformDecoder* CFLPlatformDecoder::NewL(void * aAllocator, TUserMode aUserMode, const TCodecType& aCodecType, 
										 CPlatformFlashEngine& aEngine)
{	
	LOG(CFLPlatformDecoder::NewL::This is FL3.1);
	CFLPlatformDecoder* self = new (ELeave) CFLPlatformDecoder(aAllocator, aUserMode, aCodecType, aEngine);    
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop(self); 
    
    return self;
}

// -------------------------------------------------------------------------
// CFLPlatformDecoder::~CFLPlatformDecoder()
// 
//
// Destructor. 
// --------------------------------------------------------------------------
// 

CFLPlatformDecoder::~CFLPlatformDecoder()
{
    if(iMetaData->hasVideo)
    {
        // we no longer have the video
        iEngine.SetSwfHasVideo(EFalse);
    }
    if(iMetaData->hasAudio)
    {
        // we no longer have the audio
        iEngine.SetSwfHasAudio(EFalse);
    }

    if(iAudioStream)
    {
       TRAP_IGNORE(iEngine.MP3StreamClosedL());
    }
	if(iMetaData->isEmbeddedVideo)
	{
		if(iEmbeddedDecompressorInstance)
		{
			DestroyFlashDecompressor();
		}
		iEmbeddedDecompressorInstance = NULL;
		iEmbeddedDecompressorInstanceFncs = NULL;
	}
	if(iMetaData)
	{
		delete iMetaData;
		iMetaData = NULL;
	}
	if(iFlRtpSource)
	{
		Stop();
		
		if(IsVideoStreamPresent())
		{
			iFlRtpSource->RemoveStream(iVideoStreamId);	
			iFlRtpSource->RemoveMedia(); 
		}
		if(IsAudioStreamPresent())
		{
			iFlRtpSource->RemoveStream(iAudioStreamId);	
			iFlRtpSource->RemoveMedia();
		}
		
		delete iFlRtpSource;
		iFlRtpSource = NULL;
		
#ifdef USE_VPU2
        MDisplay* display = iEngine.GetDisplay();
		display->CloseVideoWindow((void*)this);
#endif
    }    
	
	iCommandQueue.Reset();
	iCommandQueue.Close();

	if(iInstanceFncs)
	{
		delete iInstanceFncs;
	}
}

// -------------------------------------------------------------------------
// CFLPlatformDecoder::CFLPlatformDecoder(TCodecType aCodecType, 
//								   IVideoMetaDataStruct* aMetaData,
//								   CPlatformFlashEngine& aEngine)
// 
//
// First Phase constructor. 
// --------------------------------------------------------------------------
// 

CFLPlatformDecoder::CFLPlatformDecoder(void *aAllocator, TUserMode aUserMode, const TCodecType& aCodecType, 
								   CPlatformFlashEngine& aEngine)
:iEngine(aEngine), iUserMode(aUserMode), iAllocator(aAllocator)  
{
	iFlRtpSource = NULL;	
	SETSTATE(EHXVDInitial);
	iInitWaiting  = EFalse;
	iVideoRectSet = EFalse;
	iChangedScreenMode = EFalse;
	iPlayComplete	= EFalse;
	iMetaData	  = NULL;
	iAudioInitComplete = EFalse;
	
	iVideoCodecType	  = aCodecType;
	iVideoStreamId 	  = -1;
	
	iAudioStreamId 	= -1;
	iAudioCodecType = ENoCodec;
	
	iFrameSize.SetSize(0, 0);	
	iPadSize.SetSize(0, 0);
	iSwfOrigin.SetXY(0, 0);

	iInitialVolume = 0;
	iSwfVolume = 0;
	iProfileInfo = 0;
	iLevelInfo = 0;
	iSizeOfNalUnitLength = 0;
	
	iCommandQueue.Reset();	
	iFrameSizeKnown = EFalse;

	iEmbeddedDecompressorInstance = NULL;
	iEmbeddedDecompressorInstanceFncs = NULL;
	
	iPausedFromAS = EFalse;
	iAudioStream= EFalse;
	
	// These flags are used for handling Pause Resume in RTMP Streaming
	iStreamingVideoReset = EFalse;
	iStreamingAudioReset = EFalse;
	iStreamingPauseResetCount = 0;
	
	// These flags are used for handling Seek in RTMP Streaming Paused state
	iSeekInStreamingPausedState = EFalse;
	iSeekPositionGot = EFalse;
	
	iAvcConfigDone = EFalse;
	iAacConfigDone = EFalse;
	
	iRtmpStreamingPaused = EFalse;
	iRtmpStreamingResuming = EFalse;
	
	iDoLog = EFalse;

	LOG1(Codec Type : [%d], aCodecType);
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::ConstructL()
// 
//
// Second Phase constructor. 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::ConstructL()
{
    FLPDECODERTRACE( _L("CXPSMediaStream.ConstructL< ++") );

    RFs lFs(CCoeEnv::Static()->FsSession());
    TFileName lLogFlagFileName(_L("C:\\data\\xpslogs\\flpdecoder.txt"));
    
    if(BaflUtils::FileExists(lFs, lLogFlagFileName))
    {
        iDoLog = ETrue;
    }
    else
    {
        iDoLog = EFalse;
    }

    iInstanceFncs = new (ELeave) IVideoDecompressorInstanceFunctions;
		
	iInstanceFncs->CodecSubType          = CodecSubType;
	iInstanceFncs->EmbeddedIsKeyFrame    = EmbeddedIsKeyFrame;
	iInstanceFncs->DecompressFrame       = DecompressFrame;
	iInstanceFncs->BlitTo32              = BlitTo32;
	iInstanceFncs->BlitTo32Single        = BlitTo32Single;
	iInstanceFncs->BlitTo16              = BlitTo16;
	iInstanceFncs->BlitTo16Single        = BlitTo16Single;
	iInstanceFncs->FlushAfterSeek		 = FlushAfterSeek;
	iInstanceFncs->GetFrameDimensions	 = GetFrameDimensions;
	iInstanceFncs->MiscData				 = MiscData;
	iInstanceFncs->DecompressData		 = DecompressData;
	//DecompressAudioEmbedded has been removed from latest IVideoDecompressor.h file
//	iInstanceFncs->DecompressAudioEmbedded = DecompressAudioEmbedded;
	iInstanceFncs->GetVideoPosition		= GetVideoPosition;
	iInstanceFncs->QueryVideoConfigFlags = QueryVideoConfigFlags;
	
	m_fncs = iInstanceFncs;
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::Initialize()
// 
//
// 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::Initialize()
{	
	if(!iFlRtpSource || CHECKSTATE(EHXVDInitComplete))
	{
		return;
	}
	
	if(	iState == EHXVDInitial)
	{
		iInitWaiting = ETrue;
	}
	
	iVideoRectSet = EFalse;
}

// -------------------------------------------------------------------------
// S32 CFLPlatformDecoder::CodecSubType(IVideoDecompressorInstance* decompressor, U32 type)
// 
//
// 
// --------------------------------------------------------------------------
// 

S32 CFLPlatformDecoder::CodecSubType(IVideoDecompressorInstance* aDecompressor, U32 aType)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);

	if(!decoder)
	{
		LOG(CFLPlatformDecoder::CodecSubType() : Unknown codec type !);
		
		return -1;
	}
	
	if(decoder->iMetaData->isEmbeddedVideo)
	{
		if(decoder->iEmbeddedDecompressorInstance && 
				decoder->iEmbeddedDecompressorInstanceFncs)
		{
			S32 ret = decoder->iEmbeddedDecompressorInstanceFncs->CodecSubType(decoder->iEmbeddedDecompressorInstance, 
																			   aType);
			return ret;
		}
		else
		{
			return (-1);
		}
	}

	if(!aType)
	{
		return decoder->CodecType();		
	}

	return -1;
}

// -------------------------------------------------------------------------
// bool CFLPlatformDecoder::EmbeddedIsKeyFrame(IVideoDecompressorInstance* decompressor,
//                                        const U8* data,
//                                        U32 len, 
//                                        bool /*alphaCodec*/)
//
// 
// --------------------------------------------------------------------------
// 

bool CFLPlatformDecoder::EmbeddedIsKeyFrame(IVideoDecompressorInstance* aDecompressor,
                                          const U8* aData,
                                          U32 aLen, 
                                          bool aAlphaCodec)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);
	
	if(!decoder || !aData)
	{
		return false;
	}

	if(decoder->iMetaData->isEmbeddedVideo)
	{
		if(decoder->iEmbeddedDecompressorInstance && 
				decoder->iEmbeddedDecompressorInstanceFncs)
		{
			bool ret = decoder->iEmbeddedDecompressorInstanceFncs->EmbeddedIsKeyFrame(decoder->iEmbeddedDecompressorInstance, 
																					  aData, aLen, aAlphaCodec);
			return ret;
		}
		else
		{
			return (false);
		}
	}
	
	TStreamPacketData pktData;
	pktData.iFrameData 	= (TUint8*) aData;
	pktData.iDataLength = aLen;
	pktData.iTimeStamp 	= 0;
	pktData.iEmbeddedVideo = ETrue;
	pktData.iPadSize.SetSize(0, 0);
	pktData.iFrameType = EIFrame;
	pktData.iPacketConsumed = ETrue;
	
	// Parse the frame headers to know the frame type. 
	
	TSize pictureSize(0, 0);
	TInt retVal = decoder->ParseHeader(pktData, pictureSize);
	if(IsOK(retVal))
	{
		if(pktData.iFrameType == EIFrame)
		{
			return true; 
		}
	}

    return false;
}

// -------------------------------------------------------------------------
// bool CFLPlatformDecoder::DecompressFrame(IVideoDecompressorInstance* decompressor,
//                                        IVideoDecompressDataStruct* decompressData)
//
// 
// --------------------------------------------------------------------------
// 

bool CFLPlatformDecoder::DecompressFrame(IVideoDecompressorInstance* aDecompressor,
                                       IVideoDecompressDataStruct* aDecompressData)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);
	
	if(!decoder || !aDecompressData)
	{
		LOG(CFLPlatformDecoder::DecompressFrame() : No decoder!);
		return false;
	}

	if(decoder->iMetaData->isEmbeddedVideo)
	{
		if(decoder->iEmbeddedDecompressorInstance && 
				decoder->iEmbeddedDecompressorInstanceFncs)
		{
			bool ret = decoder->iEmbeddedDecompressorInstanceFncs->DecompressFrame(decoder->iEmbeddedDecompressorInstance, 
																					aDecompressData);
			return ret;
		}
		else
		{
			return (false);
		}
	}
	
	TStreamPacketData pktData;
	pktData.iFrameData 	= (TUint8*) aDecompressData->msg.data;
	pktData.iDataLength = aDecompressData->msg.length;
	pktData.iTimeStamp 	= aDecompressData->msg.time;
	pktData.iEmbeddedVideo = aDecompressData->embeddedVideo;
	pktData.iPadSize.SetSize(0, 0);
	pktData.iFrameType = EIFrame;
	pktData.iPacketConsumed = ETrue;
	
	TSize size(*(aDecompressData->rOutWidth), *(aDecompressData->rOutHeight));
	
	TInt error = decoder->DecodeVideoFrame(pktData, size);
	if(!IsOK(error))
	{
		return false;
	}
	
	*(aDecompressData->rOutWidth)	 = decoder->iFrameSize.iWidth;  
	*(aDecompressData->rOutHeight) 	 = decoder->iFrameSize.iHeight; 
	
	// 	Determine whether alpha channel was processed 
	
	*(aDecompressData->rAlphaChannel) = false;
    aDecompressData->canAcceptMoreData = bool(pktData.iPacketConsumed);
	
    return true;
}

// -------------------------------------------------------------------------
// bool CFLPlatformDecoder::GetFrameDimensions(IVideoDecompressorInstance* decompressor,
//                                           IVideoDecompressDataStruct* decompressData)
//
// 
// --------------------------------------------------------------------------
// 

bool CFLPlatformDecoder::GetFrameDimensions(IVideoDecompressorInstance* aDecompressor,
                                          IVideoDecompressDataStruct* aDecompressData)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);

	if(!decoder || !aDecompressData) {
		LOG(CFLPlatformDecoder::GetFrameDimensions() : No decoder!);
		return false;
	}

	if(decoder->iMetaData->isEmbeddedVideo)
	{
		if(decoder->iEmbeddedDecompressorInstance && 
				decoder->iEmbeddedDecompressorInstanceFncs)
		{
			bool ret = decoder->iEmbeddedDecompressorInstanceFncs->GetFrameDimensions(decoder->iEmbeddedDecompressorInstance, 
																						aDecompressData);
			return ret;
		}
		else
		{
			return (false);
		}
	}

	TSize frameSize;
	
	bool ret = true;
	ret = decoder->GetFrameSize(frameSize);
	if(ret)
	{
		*(aDecompressData->rOutWidth)	 = (unsigned long)frameSize.iWidth;  
		*(aDecompressData->rOutHeight) 	 = (unsigned long)frameSize.iHeight;
	}
    return ret;
}

// -------------------------------------------------------------------------
// bool CFLPlatformDecoder::BlitTo32(IVideoDecompressorInstance* /*decompressor*/,
//                                IVideoBlitTo32Struct* /*blitData*/)
// 
//
// 
// --------------------------------------------------------------------------
// 

bool CFLPlatformDecoder::BlitTo32(IVideoDecompressorInstance* aDecompressor,
                                IVideoBlitTo32Struct* aBlitData)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);

	if(!decoder || !aBlitData)
	{
		LOG(CFLPlatformDecoder::BlitTo32() : No decoder!);
		return false;
	}

	if(decoder->iMetaData->isEmbeddedVideo)
	{
		if(decoder->iEmbeddedDecompressorInstance && 
				decoder->iEmbeddedDecompressorInstanceFncs)
		{
			bool ret = decoder->iEmbeddedDecompressorInstanceFncs->BlitTo32(decoder->iEmbeddedDecompressorInstance, 
																			aBlitData);
			return ret;
		}
		else
		{
			return (false);
		}
	}

	// Not used in Ph-2 : so just return true here
	return true;
}

// -------------------------------------------------------------------------
// bool CFLPlatformDecoder::BlitTo32Single(IVideoDecompressorInstance* decompressor,
//                                       IVideoBlitTo32SingleStruct* blitData)
//
// 
// --------------------------------------------------------------------------
// 

bool CFLPlatformDecoder::BlitTo32Single(IVideoDecompressorInstance* aDecompressor,
                                      IVideoBlitTo32SingleStruct* aBlitData)
{

	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);

	if(!decoder || !aBlitData)
	{
		LOG(CFLPlatformDecoder::BlitTo32Single() : No decoder!);
		return false;
	}

	if(decoder->iMetaData->isEmbeddedVideo)
	{
		if(decoder->iEmbeddedDecompressorInstance && 
				decoder->iEmbeddedDecompressorInstanceFncs)
		{
			bool ret = decoder->iEmbeddedDecompressorInstanceFncs->BlitTo32Single(decoder->iEmbeddedDecompressorInstance, 
																					aBlitData);
			return ret;
		}
		else
		{
			return (false);
		}
	}

	// Not used in Ph-2 : so just return true here
    return true;
}

// -------------------------------------------------------------------------
// bool CFLPlatformDecoder::BlitTo16(IVideoDecompressorInstance* decompressor, 
//                                 IVideoBlitTo16Struct* blitData)
//
// 
// --------------------------------------------------------------------------
// 

bool CFLPlatformDecoder::BlitTo16(IVideoDecompressorInstance* aDecompressor, 
                                IVideoBlitTo16Struct* aBlitData)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);

	if(!decoder || !aBlitData )
	{
		LOG(CFLPlatformDecoder::BlitTo16() : No decoder!);
		return false;
	}

	if(decoder->iMetaData->isEmbeddedVideo)
	{
		if(decoder->iEmbeddedDecompressorInstance && 
				decoder->iEmbeddedDecompressorInstanceFncs)
		{
			bool ret = decoder->iEmbeddedDecompressorInstanceFncs->BlitTo16(decoder->iEmbeddedDecompressorInstance, 
																			aBlitData);
			return ret;
		}
		else
		{
			return (false);
		}
	}

	// Not used in Ph-2 : so just return true here
    return true;
}

// -------------------------------------------------------------------------
// bool CFLPlatformDecoder::BlitTo16Single(IVideoDecompressorInstance* decompressor, 
//                                       IVideoBlitTo16SingleStruct* blitData)
// 
// --------------------------------------------------------------------------
// 

bool CFLPlatformDecoder::BlitTo16Single(IVideoDecompressorInstance* aDecompressor, 
                                      IVideoBlitTo16SingleStruct* aBlitData)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);

	if(!decoder && !aBlitData)
	{
		LOG(CFLPlatformDecoder::BlitTo16Single() : No decoder!);
		return false;
	}

	if(decoder->iMetaData->isEmbeddedVideo)
	{
		if(decoder->iEmbeddedDecompressorInstance && decoder->iEmbeddedDecompressorInstanceFncs)
		{
			bool ret = decoder->iEmbeddedDecompressorInstanceFncs->BlitTo16Single(decoder->iEmbeddedDecompressorInstance, 
																					aBlitData);	
			return ret;
		}
		else
		{
			return (false);
		}
	}

	// Not used in Ph-2 : so just return true here
    return true;
}

bool CFLPlatformDecoder::GetFrameSize(TSize& aFrameSize)
{
	if(iFrameSizeKnown)
	{
		aFrameSize.iHeight = iFrameSize.iHeight;
		aFrameSize.iWidth  = iFrameSize.iWidth;
		return true;
	}
	
	return false;
}

// -------------------------------------------------------------------------
//bool CFLPlatformDecoder::FlushAfterSeek(IVideoDecompressorInstance* aDecompressor)
//
// 
//
// 
// --------------------------------------------------------------------------
// 
bool CFLPlatformDecoder::FlushAfterSeek(IVideoDecompressorInstance* aDecompressor)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);
	
	if(!decoder) {
		LOG(CFLPlatformDecoder::FlushAfterSeek() : No decoder!);
		return false;
	}

	if(decoder->iMetaData->isEmbeddedVideo)
	{
		if(decoder->iEmbeddedDecompressorInstance && 
				decoder->iEmbeddedDecompressorInstanceFncs)
		{
			bool ret = decoder->iEmbeddedDecompressorInstanceFncs->FlushAfterSeek(decoder->iEmbeddedDecompressorInstance);
			return ret;
		}
		else
		{
			return (false);
		}
	}
	
	CFLRtpSource *rtpSource = NULL;
	rtpSource = decoder->RtpSource();
	
	if(rtpSource)
	{
		//rtpSource->PausePktSupplyForAllStreams();
	}
	
	rtpSource->ResetAudioVideoStream();

	return true;
}

// -------------------------------------------------------------------------
// bool CFLPlatformDecoder::MiscData(IVideoDecompressorInstance * aDecompressor, 
//                                        IVideoMiscDataStruct *decompressData)
// 
//
// 
// --------------------------------------------------------------------------
// 

bool CFLPlatformDecoder::MiscData(IVideoDecompressorInstance * aDecompressor, 
										IVideoMiscDataStruct *decompressData)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);
	
	ASSERT(decoder);
	
	THXVDCmdArg argInfo;
	THXVideoDecCommand command;

	switch(decompressData->miscDataType)	
	{
	    case IMiscData_StreamCommand:
	    {
    		// dataItem1 indicates the Stream command
    		switch((decompressData->dataItem1)	)
    		{    		        		    
        		case IStreamPause: 
        		{			
        			command = EHXVDCmdASPause; 
        			break;
        		}
        		
        		case IStreamResume:
        		{
        			command = EHXVDCmdPlay;	
        			break;
        		}
        		
        		case IStreamSeek:
        		{
        			command = EHXVDCmdSeek;	
        			break;
        		}
				case IStreamReset:
				{
					command = EHXVDCmdReset; break;
				}
				case IStreamEOF:
				{
					command = EHXVDCmdEOF; break;
				}        		
    	        case IStreamSetVolume:
                {
             		command = EHXVDCmdSetVolume; 
    				// dataItem3 is the stream parameter
        			if(TInt32(decompressData->dataItem3) >= 0)
        			{
						decoder->SetSwfVolume(decompressData->dataItem3);
						if(decoder->CheckState(EHXVDInitialize) || decoder->CheckState(EHXVDInitial))
						{
							decoder->StoreInitialVolume(decompressData->dataItem3);
							return true;
						}
						else
						{
							TInt currentHwKeyVolume = decoder->GetEngineVolume();
							if(currentHwKeyVolume & KMuteMask)
							{
								currentHwKeyVolume = 0; 
							}
							
							TInt maxVolume = decoder->MaxVolume();
							
							argInfo.iVolume = (decompressData->dataItem3 * currentHwKeyVolume / KMaxUserVolume) * maxVolume / KMaxVolumeFromFL ; 	
							LOG3(currVolume swfVol volbeingset [%d %d %d], currentHwKeyVolume, decompressData->dataItem3, argInfo.iVolume);
						}
					}
        			else
        			{
        				argInfo.iVolume = 0;
        			}
    			
    			break;
                }
                default:
    			{
    			    return false;
    			}    	            
    		}
	        decoder->DispatchHXCommand(command, argInfo);
	        break;
	    }

	    case IMiscData_VideoRectangle:
        {
        	//dataItem2 contains pointer to the video rect
	        decoder->SetVideoRectangle(aDecompressor,(FI_Rect*)decompressData->dataItem2);
	        break;
        }

	    case IMiscData_MetaData:
        {
	        //dataItem2 containd pointer to metadata info
	        decoder->SetMetadata((IVideoMetaDataStruct*)decompressData->dataItem2);
	        break;
        }

		default:
		{
			return false;
		}
	}
	return true;
	
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::SetMetadata(IVideoMetaDataStruct* aMetaData)
// 
//
// 
// --------------------------------------------------------------------------
// 
void CFLPlatformDecoder::SetMetadata(IVideoMetaDataStruct* aMetaData)
{
    FLPDECODERTRACEF("************ MetaData Information ************");
    FLPDECODERTRACEF("Duration 		  [%d]", aMetaData->duration);
    FLPDECODERTRACEF("HasAudio 		  [%d]", aMetaData->hasAudio);
    FLPDECODERTRACEF("HasVideo 		  [%d]", aMetaData->hasVideo);
    FLPDECODERTRACEF("Height   		  [%d]", aMetaData->height);
    FLPDECODERTRACEF("Width  		      [%d]", aMetaData->width);
    FLPDECODERTRACEF("IsStreamed 	      [%d]", aMetaData->isStreamed);
    FLPDECODERTRACEF("IsEmbedded Video  [%d]", aMetaData->isEmbeddedVideo);
    FLPDECODERTRACEF("Total bit rate  [%d]", aMetaData->totalBitRate);    
    FLPDECODERTRACEF("**********************************************");

	IVideoMetaDataStruct oldMetaData;
	TBool oldMetaDataExists = EFalse;
	
	if(!iMetaData)
	{
		TRAPD(err, iMetaData = new (ELeave) IVideoMetaDataStruct);
		if(err)
		{
			MfrsoHandleError(err);
		}
	}
	else
	{
	    oldMetaData = *iMetaData;
	    oldMetaDataExists = ETrue;
	}

	if(aMetaData->duration == -1)
	{//If duration is -1(i.e duration is unknown), we set to some value so that playback can happen till end of stream
		iMetaData->duration = KMaxPlaybackTime * KMillisecondsForASec;
	}
	else
	{
		iMetaData->duration   	   = aMetaData->duration;
	}
	
	iMetaData->hasAudio   	   = aMetaData->hasAudio;
	iMetaData->hasVideo   	   = aMetaData->hasVideo;
	iMetaData->height     	   = aMetaData->height;
	iMetaData->isStreamed 	   = aMetaData->isStreamed;
	iMetaData->isEmbeddedVideo = aMetaData->isEmbeddedVideo;
	iMetaData->width      	   = aMetaData->width;

    if(iMetaData->hasAudio)
    {
        iEngine.SetSwfHasAudio(ETrue);
    }

    if(oldMetaDataExists)
    {
        if((iMetaData->hasAudio        != oldMetaData.hasAudio) ||
           (iMetaData->hasVideo        != oldMetaData.hasVideo) ||
           (iMetaData->isStreamed      != oldMetaData.isStreamed) ||
           (iMetaData->isEmbeddedVideo != oldMetaData.isEmbeddedVideo) ||
           (iMetaData->height          != oldMetaData.height) ||
           (iMetaData->width           != oldMetaData.width))
           
           
        {
            MfrsoHandleError(KErrGeneral);
            return;
        }
        else
        {//change in bitrates are not necessarily fatal.
            return;
        }
    }	
	
	if(!aMetaData->isEmbeddedVideo)
	{
        //If it is a audio only stream notify Engine  
        if(iMetaData->hasVideo==0 && iMetaData->hasAudio==1)
	    {
	        iEngine.NewMP3StreamStarted();
	        iAudioStream = ETrue;
	    }
		TRAPD(err, iFlRtpSource = CFLRtpSource::NewL(iEngine));
		if(err)
		{
			MfrsoHandleError(err);
		}

		CFLPlatformDecoder* decoderInstance = (CFLPlatformDecoder*)( this );

		decoderInstance->SetUserMode(iUserMode);

		if( iUserMode == EPluginMode )	
		{
		//	MFlashPlugin* plgControl = iEngine.GetPlugin();
		MDisplay* screenIf = iEngine.GetDisplay();
            
				
			THXVDCmdArg arg;
			if( screenIf) 
			{
				if( screenIf->IsFullScreenMode() )
					arg.iScreenMode  = EFullScreenMode;
				else
					arg.iScreenMode  = ENormalScreenMode;
			
				//TODO : Revisit
				//arg.iDisplayRect = screenIf->GetRect();
				arg.iDisplayRect = screenIf->ClientRect();				
                
				#ifdef USE_VPU2
				arg.iWindow      = screenIf->GetNewVideoWindow((void*)this);
                #else
				arg.iWindow		 = screenIf->GetWindow();
                #endif
				
				arg.iClipRect	 = screenIf->GetClipRect();
			}
			else
			{
				MfrsoHandleError(KErrAbort);
			}
			decoderInstance->DispatchHXCommand(EHXVDCmdSetScreenMode, arg);	
			decoderInstance->DispatchHXCommand(EHXVDCmdSetWindow, arg);
			decoderInstance->DispatchHXCommand(EHXVDCmdSetDisplayRect, arg);		
		}
		else if( iUserMode == EApplicationMode )	
		{
			//MFlashUI* uiControl = iEngine.GetFlashUI();
		
            MDisplay* display = iEngine.GetDisplay();

			if( display )
			{
				THXVDCmdArg arg;
				/* ESKI-7SEF99: FL3.1 Tube : CNN video not displayed properly in Lanscape mode
				 * In case of ViewerFw also, we should check for FullScreenMode and update the 
				 * iScreenMode & DisplayRect accordingly. 
				 */
				if ( display->IsFullScreenMode() )
				{
					arg.iDisplayRect = display->ApplicationRect();
					arg.iScreenMode  = EFullScreenMode;         	
				}
				else
				{
				    arg.iDisplayRect = display->ClientRect();
				    arg.iScreenMode  = ENormalScreenMode; 
				}
				
				decoderInstance->DispatchHXCommand(EHXVDCmdSetScreenMode, arg);				
			
                #ifdef USE_VPU2
				RWindow* window = display->GetNewVideoWindow((void*)this);
                #else
				RWindow* window = display->GetWindow();
                #endif
				
				if( window )
				{
					decoderInstance->SetWindow(window);
				}
			}						
		}
		iEngine.VideoIsEmbedded(EFalse);
			decoderInstance->Initialize();
	}
	else
	{		
		iEmbeddedDecompressorInstance = CreateFlashDecompressor();
		if(iEmbeddedDecompressorInstance == NULL )
		{
			MfrsoHandleError(KErrGeneral);
			return;
		}
		iEmbeddedDecompressorInstanceFncs = iEmbeddedDecompressorInstance->m_fncs;
		if(iEmbeddedDecompressorInstanceFncs == NULL )
		{
			MfrsoHandleError(KErrGeneral);
			return;
		}		
		iEngine.VideoIsEmbedded(ETrue);
		return;
	}

	if(iMetaData->hasVideo)
	{
		iFlRtpSource->AddMedia();
		
		TRAPD(err,iVideoStreamId = iFlRtpSource->AddStreamL(EXPSVideoStream, iVideoCodecType));
		if (err)
		{
			iFlRtpSource->HandleHelixError(err);
		}
	}
	if(iMetaData->hasAudio)
	{
		iFlRtpSource->AddMedia();
	}
	
	iFlRtpSource->SetTotalDuration(iMetaData->duration);
	iDuration = iMetaData->duration;
	
	if(aMetaData->hasVideo)
	{
        iEngine.SetSwfHasVideo(ETrue);
	}
	
	LOG1(CFLPlatformDecoder::SetMetadata : Total Duration = [%d], iDuration);
}

IVideoDecompressorInstance* CFLPlatformDecoder::CreateFlashDecompressor()
{
    if(	iVideoCodecType == KH263Codec )
    {    	
    	iEngine.GetFIObject().FI_SetMaxVideoFramesToProcess(4);    
    }		
	else if ( iVideoCodecType == KVP6Codec /*|| codecType == KVP6AlphaCodec*/ )
	{	    
		iEngine.GetFIObject().FI_SetMaxVideoFramesToProcess(1);
    }	
    else
    {
		//Other type of codecs - like H.264, VIDEO_CODEC_HOMEBREW, VP6Alpha, VIDEO_CODEC_JPEG
		//We have either not tested these codec types or they are not supported by core player
		//so preserve the default ..
		
    	iEngine.GetFIObject().FI_SetMaxVideoFramesToProcess(2);
	}

    FL_ReferenceCountedObjectShare*	share = iEngine.GetShareForCodec( iVideoCodecType );
	     
	if ( share == NULL )
		return NULL;

	PlatformDynamicLibrary_VideoDecompressor *	decompressor;
	Allocator *									allocator = NULL;
		
	decompressor = (PlatformDynamicLibrary_VideoDecompressor *) share->GetObject();
	if ( decompressor == NULL )
	{
		decompressor = new PlatformDynamicLibrary_VideoDecompressor( allocator, (IAllocator *)iAllocator, iVideoCodecType );
		if ( decompressor->Initialize() != DL_Ok )
		{
			PLAYERASSERT( false );
			delete decompressor;
			return NULL;
		}

		share->SetObject( decompressor );
	}
	else
		share->RetainObject();
	
	IVideoDecompressorInstance *decompressorInstance = decompressor->GetVideoDecompressorComponentFunctions()->CreateDecompressor(
										decompressor->GetVideoDecompressorComponent(), 
										iVideoCodecType);
	
	return decompressorInstance;
}


void CFLPlatformDecoder::DestroyFlashDecompressor()
{	
	// Phase 1 codec deletion
	S32 codecType = iEmbeddedDecompressorInstanceFncs->CodecSubType( iEmbeddedDecompressorInstance, 0 /*first available sub-type*/);
	FL_ReferenceCountedObjectShare* share = iEngine.GetShareForCodec( iVideoCodecType );
		
	if ( share == NULL )
	{
		return;		
	}

	PlatformDynamicLibrary_VideoDecompressor * p = (PlatformDynamicLibrary_VideoDecompressor *) share->GetObject();

	if ( p->GetVideoDecompressorComponent() )
	{
		p->GetVideoDecompressorComponentFunctions()->DestroyDecompressor( p->GetVideoDecompressorComponent(), 
																		  iEmbeddedDecompressorInstance );
	}

	share->ReleaseObject();
}
 
// -------------------------------------------------------------------------
// void CFLPlatformDecoder::DecompressAudioEmbedded(IVideoDecompressorInstance * aDecompressor,
//							IAudioDecompressDataStruct *aDecompressData)
// 
//
// 
// --------------------------------------------------------------------------
// 
bool CFLPlatformDecoder::DecompressAudioEmbedded(IVideoDecompressorInstance * aDecompressor,
							IVideoExternalBufferingDataStruct *aDecompressData)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)( aDecompressor );
	
	if( !decoder || !aDecompressData )
	{
		LOG(CHelixFlvDecoder::DecompressFrame() : No decoder!);
		return false;
	}


	TStreamPacketData pktData;
	pktData.iFrameData 	= (TUint8*) aDecompressData->msg.data;
	pktData.iDataLength = aDecompressData->msg.length;
	pktData.iTimeStamp 	= aDecompressData->msg.time;
	pktData.iPacketConsumed = ETrue;
	
	TInt error = decoder->DecodeAudioData(pktData);
	if( !IsOK(error) )
	{
		return false;
	}
	
	return true;

}
// -------------------------------------------------------------------------
// void CFLPlatformDecoder::DecompressData(IVideoDecompressorInstance * decompressor,
//	    				       IVideoExternalBufferingDataStruct *decompressData)
// 
//
// 
// --------------------------------------------------------------------------
// 
bool CFLPlatformDecoder::DecompressData(IVideoDecompressorInstance * aDecompressor,
							IVideoExternalBufferingDataStruct *aDecompressData)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);

	if(decoder->CheckPlayerState(EHXVDErrorOccured))
	{
		LOG(CFLPlatformDecoder::DecompressData() : Error Occured returning);
		return false;
	}
	if(!decoder || !aDecompressData)
	{
		LOG(CFLPlatformDecoder::DecompressData() : No decoder!);
		return false;
	}

	if(IsAudioPacket(aDecompressData))
	{
		return (decoder->DecompressAudioData(aDecompressor, aDecompressData));
	}

	if(IsVideoPacket(aDecompressData))
	{
		return (decoder->DecompressVideoData(aDecompressor, aDecompressData));
	}

	return true;
}

TBool CFLPlatformDecoder::CheckPlayerState(TInt aState)
{
	return ((aState == iState) ? ETrue:EFalse);
}

// -------------------------------------------------------------------------
// bool CFLPlatformDecoder::DecompressAudioData(IVideoDecompressorInstance * aDecompressor,
//							IVideoExternalBufferingDataStruct *aDecompressData)
// 
//
// 
// --------------------------------------------------------------------------
// 
bool CFLPlatformDecoder::DecompressAudioData(IVideoDecompressorInstance * aDecompressor,
							IVideoExternalBufferingDataStruct *aDecompressData)
{
    CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);

    /* Core player sends packets that are streamed from the server in the paused state.
     * We need to just discard the packet and inform the coreplayer that we have
     * consumed the packet. If not he will try to send the same packet when we resume.
     * When we resume, packets are streamed from the server based on the time returned
     * by GetVideoPosition.
     */
    if(iRtmpStreamingPaused)
    {
    	aDecompressData->messageConsumed = ETrue;
    	return true;
    }    
    
	IAudioInfoStruct audioInfoStruct;

	const char audioFormatInfo = aDecompressData->msg.data[0];
	GetAudioFormat(audioFormatInfo, &audioInfoStruct);				
	
	switch(aDecompressData->msg.data[1])
	{
		case AAC_PACKET_TYPE_AUDIOSPEC:
		{
			if(!iAacConfigDone)
			{
				iAacConfigDone = ETrue;
				IAudioInfoStruct audioInfoStruct;
			
				if(!IsAudioStreamPresent())
				{
					const char audioFormatInfo = aDecompressData->msg.data[0];
					GetAudioFormat(audioFormatInfo, &audioInfoStruct);				
				
					iAudioCodecType = GetAudioCodecType(audioInfoStruct.compressFormat);
					
					if(iAudioCodecType != ENoCodec)
					{
						TRAPD(err,iAudioStreamId = iFlRtpSource->AddStreamL(EXPSAudioStream, iAudioCodecType));
						if(err)
						{
							iFlRtpSource->HandleHelixError(err);
						}
					}
				}

				TStreamPacketData pktData;
				pktData.iFrameData 	= (TUint8*) &aDecompressData->msg.data[2];
				pktData.iDataLength = aDecompressData->msg.length - 2;
				pktData.iTimeStamp 	= aDecompressData->msg.time;
				//this doesnt really indicates buffer overflow as there is a
				//second level buffering in xpsbridge, hence make it true
				pktData.iPacketConsumed = ETrue;
				
				TUint8* lTempOut = NULL;
				TUint8* lTempIn = NULL;
				
			 	TRAPD(err, 	lTempOut = new (ELeave) TUint8[pktData.iDataLength*8];
							CleanupStack::PushL((TAny*)lTempOut);
							lTempIn = new (ELeave) TUint8[pktData.iDataLength+4];
							CleanupStack::PushL((TAny*)lTempIn);
				
							memcpy((void*)(lTempIn+1),(void*)pktData.iFrameData,pktData.iDataLength);
							lTempIn[0] = 0x2;
							lTempIn[pktData.iDataLength+1] = '\0';
							
							pktData.iDataLength += 2;
							
							TInt lTempOutLen = BinTo64((const TUint8*)lTempIn,pktData.iDataLength,(char*)lTempOut);
							
							SetAacConfigData(lTempOut, lTempOutLen);

						
							CleanupStack::PopAndDestroy(2));
				if(err)
				{
					iFlRtpSource->HandleHelixError(err);
				}
		
				
				if(iAudioCodecType != ENoCodec && !iAudioInitComplete)
				{					
					TAudioInfo info;
					info.iSampleRate 	= GetSampleRate((IAudioSampleRates)audioInfoStruct.sampleRate);
					info.iSampleLength  = audioInfoStruct.bitsPerSample / KByteLength ;
					info.iChannels	  	= audioInfoStruct.channels;
					
					iFlRtpSource->SetAudioInfo(iAudioStreamId, info);
					
					TRAPD(err,iFlRtpSource->InitializeL(this, iAudioCodecType));
					if(err)
					{
						iFlRtpSource->HandleHelixError(err);
					}
					iAudioInitComplete = ETrue;
				}
				
				TInt error = 0;

				aDecompressData->messageConsumed = pktData.iPacketConsumed;
					
				if(!IsOK(error))
				{
					return false;
				}
			}
			else
			{
			    aDecompressData->messageConsumed = ETrue;
				if(IsAudioSeeking() && !IsVideoStreamPresent() && iSeekInStreamingPausedState && !iSeekPositionGot)
				{

					iSeekPositionGot = ETrue;
					iSeekPosition = aDecompressData->msg.time*1000;
				}
			}
			
		    return true;			
		}
		
		case AAC_PACKET_TYPE_RAW:
		{
			if(!IsAudioStreamPresent())
			{
				IAudioInfoStruct audioInfoStruct;
				const char audioFormatInfo = aDecompressData->msg.data[0];
				GetAudioFormat(audioFormatInfo, &audioInfoStruct);
			
				TRAPD(error, decoder->InitializeAudioL(&audioInfoStruct));	
				if(!IsOK(error))
				{
					return false;
				}
			}				
		
		
			if(IsAudioStreamPresent())
			{
				TStreamPacketData pktData;
				pktData.iFrameData 	= (TUint8*) &aDecompressData->msg.data[2];
				pktData.iDataLength = aDecompressData->msg.length - 2;
				pktData.iTimeStamp 	= aDecompressData->msg.time;
				//this doesnt really indicates buffer overflow as there is a
				//second level buffering in xpsbridge, hence make it true
				pktData.iPacketConsumed= ETrue;
				
				TInt error = decoder->DecodeAudioData(pktData);
				if(!IsOK(error))
				{
					return false;
				}

				aDecompressData->messageConsumed = pktData.iPacketConsumed;
				return true;
			}
		}
		break;
		
		default:
		{
			if(!IsAudioStreamPresent())
			{
				IAudioInfoStruct audioInfoStruct;
				const char audioFormatInfo = aDecompressData->msg.data[0];
				GetAudioFormat(audioFormatInfo, &audioInfoStruct);
			
				TRAPD(error, decoder->InitializeAudioL(&audioInfoStruct));	
				if(!IsOK(error))
				{
					return false;
				}
			}

			if(IsAudioStreamPresent())
			{
				TStreamPacketData pktData;
				pktData.iFrameData 	= (TUint8*) &aDecompressData->msg.data[1];
				pktData.iDataLength = aDecompressData->msg.length - 1;
				pktData.iTimeStamp 	= aDecompressData->msg.time;
				//this doesnt really indicates buffer overflow as there is a
				//second level buffering in xpsbridge, hence make it true
				pktData.iPacketConsumed= ETrue;
				
	            TInt error = decoder->DecodeAudioData(pktData);
				if(!IsOK(error))
				{
					return false;
				}

				aDecompressData->messageConsumed= pktData.iPacketConsumed;				
				return true;			
			}
		}
	}
	
	return false;
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::GetAudioFormat(char aAudioFormatInfo, IAudioInfoStruct *aAudioInfoStruct)
// 
//
// 
// --------------------------------------------------------------------------
// 
void CFLPlatformDecoder::GetAudioFormat(const char aAudioFormatInfo, IAudioInfoStruct *aAudioInfoStruct)
{
	aAudioInfoStruct->formatID = (S32) aAudioFormatInfo;
	
	unsigned int channel = aAudioFormatInfo & sndChannelMask;	
	//getting the number of channels
	if(channel == sndMono)
		aAudioInfoStruct->channels = 1;
	if(channel == sndStereo)
		aAudioInfoStruct->channels = 2;
	
	unsigned int bitsPerSample = aAudioFormatInfo & sndBitMask;
	//getting the number of bits per second
	if(bitsPerSample == snd8Bit)
		aAudioInfoStruct->bitsPerSample = 8;
	if(bitsPerSample == snd16Bit)
		aAudioInfoStruct->bitsPerSample = 16;
	
	unsigned int sampleRate = aAudioFormatInfo & sndRateMask;
	//getting the sample rate
	if(sampleRate == snd5K)
		aAudioInfoStruct->sampleRate = IAudio5kHz;
	if(sampleRate == snd11K)
		aAudioInfoStruct->sampleRate = IAudio11kHz;
	if(sampleRate == snd22K)
		aAudioInfoStruct->sampleRate = IAudio22kHz;
	if(sampleRate == snd44K)
		aAudioInfoStruct->sampleRate = IAudio44kHz;
	
	unsigned int compressFormat = aAudioFormatInfo & sndCompressMask;
	//getting the compression format
	if((compressFormat == sndCompressNone) || (compressFormat == sndCompressNoneI))
		aAudioInfoStruct->compressFormat = IAudioCompressNone;
	if(compressFormat == sndCompressADPCM)
		aAudioInfoStruct->compressFormat = IAudioCompressADPCM;
	if(compressFormat == sndCompressMP3)
		aAudioInfoStruct->compressFormat = IAudioCompressMP3;
#ifdef FEATURE_MP4_CONTAINER
	if(compressFormat == sndCompressAAC)
		aAudioInfoStruct->compressFormat = IAudioCompressAAC;		
#endif // FEATURE_MP4_CONTAINER
}

// -------------------------------------------------------------------------
// bool CFLPlatformDecoder::DecompressVideoData(IVideoDecompressorInstance * aDecompressor,
//							IVideoExternalBufferingDataStruct *aDecompressData)
// 
//
// 
// --------------------------------------------------------------------------
// 
bool CFLPlatformDecoder::DecompressVideoData(IVideoDecompressorInstance * aDecompressor,
							IVideoExternalBufferingDataStruct *aDecompressData)
{
    CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);
    
    if(!IsVideoStreamPresent())
    {
    	return false;
    }
   
    /* Core player sends packets that are streamed from the server in the paused state.
     * We need to just discard the packet and inform the coreplayer that we have
     * consumed the packet. If not he will try to send the same packet when we resume.
     * When we resume, packets are streamed from the server based on the time returned
     * by GetVideoPosition.
     */    
    if(iRtmpStreamingPaused)
    {
    	aDecompressData->messageConsumed = ETrue;
    	return true;
    }

	if(iVideoCodecType != EH264Codec)
	{
		TStreamPacketData pktData;
		pktData.iFrameData 	= (TUint8*) aDecompressData->msg.data;
		pktData.iDataLength = aDecompressData->msg.length;
		pktData.iTimeStamp 	= aDecompressData->msg.time;
		pktData.iEmbeddedVideo = iMetaData->isEmbeddedVideo;
		pktData.iPadSize.SetSize(0, 0);
		pktData.iFrameType = EIFrame;
		pktData.iPacketConsumed= ETrue;
		
		TSize size(0,0);
		
		TInt error = decoder->DecodeVideoFrame(pktData, size);
		if(!IsOK(error))
		{
			return false;
		}

		aDecompressData->messageConsumed = pktData.iPacketConsumed;
	    return true;
	}
	else
	if(iVideoCodecType == EH264Codec)
	{
		TInt avcPktType = aDecompressData->msg.data[AVC_PACKET_TYPE_IDX];
		switch(avcPktType)
		{
			case AVC_PACKET_TYPE_AVCC:
			{
				if(!iAvcConfigDone)
				{
					ParseConfigData(aDecompressData->msg.data+5, aDecompressData->msg.length-5);
					iAvcConfigDone = ETrue;
				}
				aDecompressData->messageConsumed = ETrue;
			    return true;
				
			}
			
			case AVC_PACKET_TYPE_SAMPLE:
			{
				TBool retVal = ParseAndQueueAllNALUnits(aDecompressor, aDecompressData);
			    return retVal;
				
			}
			
			case AVC_PACKET_TYPE_EOS:
			{
			aDecompressData->messageConsumed = ETrue;
				break;
			}
			
			default:
			{
				return false;
			}
		}  				
	} 

	return true;
}


TInt CFLPlatformDecoder::ParseAndQueueAllNALUnits(IVideoDecompressorInstance * aDecompressor,IVideoExternalBufferingDataStruct *aDecompressData)
{
    CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);

	TUint8 *payldPtr = NULL;
	
	if(aDecompressData == NULL || !decoder )
	{
		return false;
	}

	/*Fix for ESLM-85BB58.
	  If host receives video message of length <= 5 then drop the 
	  packets & inform core player that it is consumed 
	  as first 5 character in video msg contains header info
	  and actual data starts from 6th character so if video
	  video msg length <= 5 then it will contain any useful info
	  and can result in memory corruption which will lead to crash.*/
	const TInt KHeaderLength = 5;
	if(aDecompressData->msg.length <= KHeaderLength)
	{
		aDecompressData->messageConsumed = true;
		return (true);
	}

	payldPtr = (unsigned char *)&aDecompressData->msg.data[5];
	TInt payldIndex = 0;
	TUint32 accessUnitLen = 0;
	//TInt i=0;
	TInt pktLength = 0;
	TUint32 payldLength = aDecompressData->msg.length - KHeaderLength;
	TInt totalNalLength = 0;
	
	
	/* Computation of Composition time Offset from Msg data. This change is done to tackle B Frame flickering problem.
	   3 bytes from index AVC_PACKET_COMP_TIME_IDX in msgdata corresponds to compositionTimeOffset.
	*/
	TUint32 compositionTimeOffset = (aDecompressData->msg.data[AVC_PACKET_COMP_TIME_IDX]<<16) 
	                       + (aDecompressData->msg.data[AVC_PACKET_COMP_TIME_IDX+1]<<8)
	                       + (aDecompressData->msg.data[AVC_PACKET_COMP_TIME_IDX+2]);

		
	while (payldIndex < payldLength)
	{
		if(iSizeOfNalUnitLength > 0)
		{
			accessUnitLen = GetNalLength(payldPtr);
			totalNalLength+= accessUnitLen;
			if(totalNalLength > payldLength)
			{
				return false;
			}
			payldIndex += iSizeOfNalUnitLength;
			payldPtr += iSizeOfNalUnitLength;
			
			TStreamPacketData pktData;		
			{
				pktData.iFrameData 	= payldPtr;
				pktData.iDataLength = accessUnitLen;
				
				/* This is a workaround fix given to avoid situations where the composition 
				 * offset value is corrupted.
				 */ 
				if (compositionTimeOffset > KMaxCompositionTimeOffset)
				    compositionTimeOffset = 0;
				// Addition of Composition time stamp to Decode time stamp.
				pktData.iTimeStamp 	= aDecompressData->msg.time+compositionTimeOffset;
							
				pktData.iEmbeddedVideo = iMetaData->isEmbeddedVideo; 
				pktData.iPadSize.SetSize(0, 0);
				pktData.iFrameType = EIFrame;
				pktData.iPacketConsumed= ETrue;					
			}

			TInt error = decoder->DecodeH264VideoFrame(pktData);
			if(!IsOK(error))
			{
				return false;
			}

			aDecompressData->messageConsumed = pktData.iPacketConsumed;

			payldPtr += accessUnitLen;
			pktLength += accessUnitLen;		
			payldIndex += accessUnitLen;			
		}
		else
		{
			TStreamPacketData pktData;		
			{
				pktData.iFrameData 	= payldPtr;
				pktData.iDataLength = payldLength;
				
				/* This is a workaround fix given to avoid situations where the composition 
				 * offset value is corrupted.
				 */ 
				if (compositionTimeOffset > KMaxCompositionTimeOffset)
				    compositionTimeOffset = 0;
				// Addition of Composition time stamp to Decode time stamp.
				pktData.iTimeStamp 	= aDecompressData->msg.time+compositionTimeOffset;
			
				pktData.iEmbeddedVideo = iMetaData->isEmbeddedVideo; 
				pktData.iPadSize.SetSize(0, 0);
				pktData.iFrameType = EIFrame;
				pktData.iPacketConsumed= ETrue;					
			}

			TInt error = decoder->DecodeH264VideoFrame(pktData);
			if(!IsOK(error))
			{
				return false;
			}			
			
			aDecompressData->messageConsumed = pktData.iPacketConsumed;
			break;
		}

	}
	
	return (true);
}

TInt CFLPlatformDecoder::GetNalLength(TUint8* aPayldPtr)
{
	char *tmp = NULL;;
	//TUint8* payloadPtr = aPayldPtr;
	
	tmp = (char *) new char[iSizeOfNalUnitLength];
	
	TInt i =0;
	
	for(i = 0; i < iSizeOfNalUnitLength; i++)
	{
		tmp[i] = aPayldPtr[i];
	}
	
	TUint32 accessUnitLen = 0;
	
	if(iSizeOfNalUnitLength == 4)
	{
		accessUnitLen = (((tmp[0] << 24) & 0xFF000000) | 
						 ((tmp[1] << 16) & 0x00FF0000) | 
						 ((tmp[2] <<  8) & 0x0000FF00) | 
						  (tmp[3]        & 0x000000FF));		
	}
	else
	if(iSizeOfNalUnitLength == 2)
	{
		accessUnitLen = (((tmp[0] <<  8) & 0xFF00) | 
						  (tmp[1]        & 0x00FF));
		
	}
	
	delete[] tmp;
					  
	return (accessUnitLen);

}

// -------------------------------------------------------------------------
// bool CFLPlatformDecoder::SetAudioFormat(IVideoDecompressorInstance * decompressor, 
//									      IAudioInfoStruct *audioInfo)
// 
//
// 
// --------------------------------------------------------------------------
// 

bool CFLPlatformDecoder::SetAudioFormat(IVideoDecompressorInstance *aDecompressor, 
									   IAudioInfoStruct *aAudioInfo)
{	
	if(!aDecompressor || !aAudioInfo)
	{	
		return false;
	}
	
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);
		
	if(decoder->AudioStreamId() < 0)
	{
		TRAPD(error, decoder->InitializeAudioL(aAudioInfo));	
		if(!IsOK(error))
		{
			return false;
		}
	}	
	
	return true;
}


// -------------------------------------------------------------------------
// void CFLPlatformDecoder::SetVideoRectangle(IVideoDecompressorInstance * decompressor, 
//										  FI_Rect* rectangle)
// 
//
// 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::SetVideoRectangle(IVideoDecompressorInstance * aDecompressor, 
										  FI_Rect* aRectangle)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);
	
	if(!decoder || !aRectangle)
	{
		LOG(CFLPlatformDecoder::SetVideoRectangle() : Bad Arguments !);
		return;
	}
	
	if(decoder->iChangedScreenMode)
	{
	    LOG(CFLPlatformDecoder::SetVideoRectangle() : Changed scr mode);
		decoder->iChangedScreenMode = EFalse;
		decoder->iVideoRectSet	    = EFalse;
	}
	
	TRAP_IGNORE(decoder->SetVideoRectL(aRectangle));
		
	return;
}

// -------------------------------------------------------------------------
// bool CFLPlatformDecoder::IsExternalVideoRenderingEnabled(IVideoDecompressorInstance * decompressor)
// 
//
// 
// --------------------------------------------------------------------------
// 

bool CFLPlatformDecoder::IsExternalVideoRenderingEnabled(IVideoDecompressorInstance * /*decompressor*/)
{
	return true;
}

// -------------------------------------------------------------------------
// float CFLPlatformDecoder::GetVideoPosition(IVideoDecompressorInstance * decompressor)
// 
// Gets the current video playback time (position) from helix.  
// --------------------------------------------------------------------------
// 

S32 CFLPlatformDecoder::GetVideoPosition(IVideoDecompressorInstance * decompressor)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(decompressor);
	if(!decoder)
	{
		return 0;
	}

	if((decoder->iFlRtpSource->PlayerState()== EHXPPaused) 
		&& decoder->iMetaData->isStreamed
		&& decoder->iSeekInStreamingPausedState 
		&& decoder->iSeekPositionGot)
	{
		TTimeIntervalMicroSeconds pos = decoder->iSeekPosition;
		S32 time = pos.Int64()/1000;
		return (time);
	}
	
	TTimeIntervalMicroSeconds position = decoder->GetVideoPosition();
	//TInt32 playTime = I64LOW(position.Int64());

	//Adding machine epsilon (http://en.wikipedia.org/wiki/Machine_epsilon)
	//to compensate the loss in value with float division/representation.
	//this is ideally of the order of 2E-7. Using 2E-3 with a defensive approach.
	S32  time = (position.Int64() / 1000);// + (1000 * 2E-3); 

	return  time;
}

void		SendEmbeddedAudioInfo(U8 /*audioHeader*/)
{
//FIXME: Fill in the implementation
	return;
}

bool CFLPlatformDecoder::QueryVideoConfigFlags(IVideoDecompressorInstance *aDecompressor,
							U32 /*aAudioCodec*/, U32* aVideoConfigFlags)
{
	CFLPlatformDecoder* decoder = (CFLPlatformDecoder*)(aDecompressor);

	if(!decoder) {
		LOG(CFLPlatformDecoder::QueryVideoConfigFlags() : No decoder!);
		return false;
	}

	if(decoder->iMetaData->isEmbeddedVideo)
	{
		*aVideoConfigFlags = FLASH_BUFFERING_RENDERING;
		return true;
	}
	else
		return false;
}

// -------------------------------------------------------------------------
// TInt CFLPlatformDecoder::GetEngineVolume()
// 
// Gets the engine volume
// --------------------------------------------------------------------------
// 
TInt CFLPlatformDecoder::GetEngineVolume() const
{
	return iEngine.GetVolume();
}


// -------------------------------------------------------------------------
// void CFLPlatformDecoder::SetVideoRectL(FI_Rect* aRectangle)
// 
//
// 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::SetVideoRectL(FI_Rect* aRectangle)
{
	__ASSERT_ALWAYS((iFlRtpSource && aRectangle), User::Invariant());
	
	FI_Rect* rect = aRectangle;

	TInt width  = rect->xmax - rect->xmin;
	TInt height = rect->ymax - rect->ymin;
	
	// If width or height is zero, then try once with the width and height of metadata.
	
	if(iMetaData && (width == 0 || height == 0))
	{		
		width  = iMetaData->width;
		height = iMetaData->height;
		
		aRectangle->xmax = aRectangle->xmin + width;
		aRectangle->ymax = aRectangle->ymin + height;
		
		rect = aRectangle;
	}
	
	if(aRectangle->xmin < 0)
	{
		aRectangle->xmin = 0;
	}
	if(aRectangle->ymin < 0)
	{
		aRectangle->ymin = 0;
	}
	
	TRect videoRect;
	videoRect.iTl.iX = aRectangle->xmin;
	videoRect.iTl.iY = aRectangle->ymin;
	videoRect.iBr.iX = aRectangle->xmax;
	videoRect.iBr.iY = aRectangle->ymax;
	
	if(videoRect.IsEmpty())
	{
		iVideoRectSet = EFalse;
		return;
	}
	
	if(iFlRtpSource->UserMode() == EApplicationMode && iVideoRect != videoRect)
	{
		LOG(CFLPlatformDecoder::SetVideoRectL() : Rectangle Changed);
		iVideoRectSet = EFalse;
	}

	if(IsVideoStreamPresent())
	{
		iEngine.SetVideoRectL(rect, ENativeVideo);
		
		CalculateVideoRectangleL(videoRect);		
	}
}

void CFLPlatformDecoder::CalculateVideoRectangleL(const TRect& aVideoRect)
{
	/* LOG4(CFLPlatformDecoder::SetVideoRect() : Rect = (%d %d) (%d %d), aVideoRect.iTl.iX, aVideoRect.iTl.iY);
       																   aVideoRect.iBr.iX, aVideoRect.iBr.iY); */
	
	TUserMode 	userMode 	= iFlRtpSource->UserMode();
	TScreenMode screenMode 	= iFlRtpSource->ScreenMode();

	SetSwfWindowOrigin(userMode, screenMode);
	
	iVideoRect = aVideoRect;
	
	TRect videoRect = aVideoRect;
	TRect appRect = iFlRtpSource->GetAppRect();
	
	videoRect.iTl.iX = iSwfOrigin.iX + videoRect.iTl.iX ;
	videoRect.iTl.iY = iSwfOrigin.iY + videoRect.iTl.iY ;
	videoRect.iBr.iX = iSwfOrigin.iX + videoRect.iBr.iX ;
	videoRect.iBr.iY = iSwfOrigin.iY + videoRect.iBr.iY ;
	
	TRect clipRect;
	if (userMode == EPluginMode && screenMode == EFullScreenMode)
	{
	    /* RC: ou1cimx1#478531. Check the boundry values. For few YouTube contents, 
         * while switching to Fullscreen we are getting out of bound values 
         * hence restricting to App rect if exceeding
         */
        if(videoRect.Height() > appRect.Height())
        {
            videoRect.SetHeight(appRect.Height());
        }
        if (videoRect.Width() > appRect.Width())
        {
            videoRect.SetWidth(appRect.Width()); 
        }

        clipRect = videoRect;
	}
	else
	{
	    clipRect = CalculateClippingRegion(videoRect, userMode);
	}
	
	TBool shouldSetWindow = (iVideoRectSet ? EFalse : ETrue);
	iFlRtpSource->SetVideoWindowRectL(videoRect, clipRect, shouldSetWindow);
	
	iVideoRectSet = ETrue;
}

// -------------------------------------------------------------------------
// TRect CFLPlatformDecoder::CalculateClippingRegion(const TRect& aVideoRect, 
//												   const TUserMode& aUserMode)
// 
//
// 
// --------------------------------------------------------------------------
// 

TRect CFLPlatformDecoder::CalculateClippingRegion(const TRect& aVideoRect, 
												const TUserMode& aUserMode)
{
	TRect clipRect(0, 0, 0, 0);
	
	if(!iFlRtpSource)
	{
		return clipRect;
	}
	
	if(aUserMode == EApplicationMode)
	{
		clipRect = aVideoRect;
		
		TRect appRect = iFlRtpSource->GetAppRect();
		
		if(appRect.Intersects(aVideoRect))
		{		
			clipRect.Intersection(appRect);
		}
	}
	else if(aUserMode == EPluginMode)
	{
        clipRect = iClipRect;
        clipRect.Intersection(aVideoRect);
	}
	
	return clipRect;
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::SetSwfWindowOrigin()
// 
//
// 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::SetSwfWindowOrigin(const TUserMode& aUserMode,
										  const TScreenMode& aScreenMode)
{	
	TRect appRect = iFlRtpSource->GetAppRect();
	
	// In FL App, the application rect will be the swf boundary.
	// In browser, only in fullscreen the app rect becomes the swfRect 
	
	if(aUserMode == EApplicationMode || aScreenMode == EFullScreenMode)
	{
		iSwfRect = appRect;
	}

	iSwfOrigin.SetXY(iSwfRect.iTl.iX, iSwfRect.iTl.iY);
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::InitializeAudioL(IAudioInfoStruct* aAudioInfo)
// 
//
// 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::InitializeAudioL(const IAudioInfoStruct* aAudioInfo)
{
	if(!IsAudioStreamPresent())
	{	
		if(iMetaData && !iMetaData->hasAudio)
		{			 
			//iFlRtpSource->AddMedia();
			LOG(CFLPlatformDecoder::InitializeAudioL() : Adding audio media as it was missed out);
		}
		
		iAudioCodecType = GetAudioCodecType(aAudioInfo->compressFormat);
		
		if(iAudioCodecType != ENoCodec)
		{
		iAudioStreamId = iFlRtpSource->AddStreamL(EXPSAudioStream, iAudioCodecType);
		
		TAudioInfo info;
		info.iSampleRate 	= GetSampleRate((IAudioSampleRates)aAudioInfo->sampleRate);
		info.iSampleLength  = aAudioInfo->bitsPerSample / KByteLength ;
		info.iChannels	  	= aAudioInfo->channels;
		
		iFlRtpSource->SetAudioInfo(iAudioStreamId, info);
		
		iFlRtpSource->InitializeL(this, iAudioCodecType);	
		}
	}
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::StoreAvcInfo(TInt aProfileInfo, TInt aLevelInfo)
// 
//
// 
// --------------------------------------------------------------------------
// 
void CFLPlatformDecoder::StoreAvcInfo(TInt aProfileInfo, TInt aConstraintInfo, TInt aLevelInfo)
{
	TAvcStreamInfo avcInfo;
	avcInfo.iProfileInfo = aProfileInfo;
	avcInfo.iConstraintInfo = aConstraintInfo;
	avcInfo.iLevelInfo   = aLevelInfo;
	
	iFlRtpSource->SetAvcInfo(iVideoStreamId, avcInfo);
}

TInt CFLPlatformDecoder::SetAvcConfigData(TUint8* aCfgData, TInt aCfgLen)
{
	TStreamConfigData avcCfgData;
	
	avcCfgData.iConfigData = aCfgData;
	avcCfgData.iConfigLength = aCfgLen;
		
	return (iFlRtpSource->SetStreamConfig(iVideoStreamId, avcCfgData));
}

TInt CFLPlatformDecoder::SetAacConfigData(TUint8* aCfgData, TInt aCfgLen)
{
	TStreamConfigData aacCfgData;
	
	aacCfgData.iConfigData = aCfgData;
	aacCfgData.iConfigLength = aCfgLen;
		
	return (iFlRtpSource->SetStreamConfig(iAudioStreamId, aacCfgData));
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::MfrsoInitComplete(TInt /*aError*/)
// 
//
// 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::MfrsoInitComplete(const TInt& /*aError*/) 
{
	SETSTATE(EHXVDInitComplete);
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::MfrsoReadyToStart(TInt aError)
// 
//
// 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::MfrsoReadyToStart(const TInt& aError)
{
	LOG1(CFLPlatformDecoder::MfrsoReadyToStart() ++ Error = [%d], aError);
        THXVDCmdArg args;

	TInt currentHwKeyVolume = GetEngineVolume();
	if(currentHwKeyVolume & KMuteMask)
	{
		currentHwKeyVolume = 0; 
	}
	TInt maxVolume = MaxVolume();
	args.iVolume = (iInitialVolume * currentHwKeyVolume / KMaxUserVolume) * maxVolume / KMaxVolumeFromFL ; 	
	SetVolume(args.iVolume);
	
	if(IsOK(aError) && !CHECKSTATE(EHXVDStarted))
	{
		SETSTATE(EHXVDInitComplete);
		
        if(!iFlRtpSource) return;
        
        if(IsAudioStreamPresent())
        {
        	iFlRtpSource->Start(iAudioStreamId);
        }
        if(IsVideoStreamPresent())
        {
        	iFlRtpSource->Start(iVideoStreamId);	
        }        
        
        SETSTATE(EHXVDStarted);
        iSeekCount = 0;
        
        DispatchHXCommand(EHXVDCmdRefreshScreen, args);
        
	}
	else if(!IsOK(aError))
	{
		LOG(CFLPlatformDecoder::MfrsoReadyToStart() : Error Occured);
		SETSTATE(EHXVDErrorOccured);
	}
		
	LOG(CFLPlatformDecoder::MfrsoReadyToStart() --);
}



// -------------------------------------------------------------------------
// void CFLPlatformDecoder::MfrsoPlayerStateChanged(TInt /*aError*/)
// 
// Error Notification Callback. 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::MfrsoPlayerStateChanged(TPlayerState aState)
{
    FLPDECODERTRACEF("CFLPlatformDecoder::MfrsoPlayerStateChanged() : rtpsource state = [%d]", aState);
	
	TBool lDone = EFalse;

	//Extend backlight only if Video is present 
	if(IsVideoStreamPresent())
	{
		if (aState == EHXPPlaying)
			iEngine.ExtendBacklightForVideoPlayback(EOn);
		else
			iEngine.ExtendBacklightForVideoPlayback(EOff);
	}
	//if(aState == EHXPPlaying || aState == EHXPPaused)
    {
		while(iCommandQueue.Count() && !lDone)
		{
			switch(iCommandQueue[0].iCommand)
			{
				case EHXVDCmdPause:
				case EHXVDCmdASPause:
				case EHXVDCmdPlay:
				case EHXVDCmdResume:
				case EHXVDCmdStop:
				case EHXVDCmdSetVolume:
				case EHXVDCmdSeek:
				case EHXVDCmdReset:
				case EHXVDCmdEOF:
				{
					if(iFlRtpSource->PlayerState() == EHXPInitial ||
					   iFlRtpSource->PlayerState() == EHXPOpening ||        
					   iFlRtpSource->PlayerState() == EHXPPreparing)
					{
						//These commands cannot be serviced in these conditions *now*
						//Wait for state change.
						lDone = ETrue;
					}
					break;
				}
			};

			if(!lDone) {
				ProcessHXCommand(iCommandQueue[0].iCommand, iCommandQueue[0].iCommandArg);
				iCommandQueue.Remove(0);
			}
		}
    }
}


// -------------------------------------------------------------------------
// void CFLPlatformDecoder::MfrsoHandleError(TInt /*aError*/)
// 
// Error Notification Callback. 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::MfrsoHandleError(const TInt& aError)
{
	if(IsOK(aError) || CHECKSTATE(EHXVDErrorOccured))
	{
		return;
	}
	
	LOG1(CFLPlatformDecoder::MfrsoHandleError() : aError = [%d], aError);
	
	// Handle the error
	SETSTATE(EHXVDErrorOccured);
	iEngine.HandleHelixError(aError);
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::MfrsoPlaybackComplete(const TInt& aError)
// 
// Callback to notify that playback is complete.  
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::MfrsoPlaybackComplete(const TInt& aError)
{
	LOG(CFLPlatformDecoder::MfrsoPlaybackComplete() ++);
	
	if(IsVideoStreamPresent())
		iEngine.ExtendBacklightForVideoPlayback(EOff);
	
	if(!IsOK(aError))
	{
		MfrsoHandleError(aError);
		return;
	}

	if(iPlayComplete)	
	{
		return;
	}

	iPlayComplete = ETrue;
	
	iEngine.DrawVideoRect();
	
	LOG(CFLPlatformDecoder::MfrsoPlaybackComplete() --);
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::DecodeVideoFrameL(TStreamPacketData aPktData, TInt aWidth, TInt aHeight)
// 
//
// 
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoder::DecodeVideoFrame(TStreamPacketData& aPktData, const TSize& aSize)
{
	if(!iFlRtpSource)
	{
		LOG(CFLPlatformDecoder::DecodeVideoFrame() : No XPS);
		return KErrNotReady;
	}
	
	if(CHECKSTATE(EHXVDErrorOccured))
	{
		LOG(CFLPlatformDecoder::DecodeVideoFrame() : In Error State);
		return KErrNone;
	}
	
	if(CHECKSTATE(EHXVDInitial))
	{
		TInt parseError = ParseHeader(aPktData, iFrameSize);
		
		if(IsOK(parseError))
		{			
			SETSTATE(EHXVDFirstFrameReceived);
			iFrameSizeKnown = ETrue;
			
			// Note : Padding bytes are absent in Embedded Flv Vp6 video payloads.
			// 		  In this case, we use this iPadSize.
			
			if(aPktData.iEmbeddedVideo && iVideoCodecType == EOn2Codec)
			{
				SetPadSizeInEmbeddedVideo(aSize);
			}
		}
		else
		{
			SETSTATE(EHXVDErrorOccured);
			return KErrCorrupt;
		}
	}

	// skip the first byte if it is not Embedded Video. 
	
	if(!aPktData.iEmbeddedVideo)
	{
		aPktData.iFrameData  += 1; 
		aPktData.iDataLength -= 1;			
	}
	
	aPktData.iPadSize = iPadSize;
	
	TInt error = KErrNone;

	if(IsVideoSeeking() && iSeekInStreamingPausedState && !iSeekPositionGot)
	{

		iSeekPositionGot = ETrue;
		iSeekPosition = aPktData.iTimeStamp *1000;
		return ETrue;
	}
	
	if(IsVideoSeeking() && iSeekInStreamingPausedState && iSeekPositionGot)
	{
		return ETrue;
	}

	
	if(IsVideoSeeking())
	{
		error = SeekVideo(aPktData);
		if(!IsOK(error))
		{
			return error;
		}
	}

	if(IsVideoReseting())
	{
		error = ResetVideo(aPktData);
		if(!IsOK(error))
		{
			return error;
		}
	}

	if(IsStreamingPausedVideoResetPending())
	{
		error = StreamingPauseDoReset(iVideoStreamId, aPktData);
		if(!IsOK(error))
		{
			return error;
		}
	}

	else
	{
		error = iFlRtpSource->AddPacket(iVideoStreamId, aPktData);
	    if( error == KErrOverflow )
		{
	        aPktData.iPacketConsumed= EFalse;
			error = KErrNone;
		}
		if(!IsOK(error))
		{
			return error;
		}
	}


	// Previously an init call was issued but not considered because we were not ready.
	// Now we are ready, and we should issue init again. 
	// This init call is for flv video stream only. 
	
	
	if(iInitWaiting)
	{
		if(iFrameSizeKnown)
		{
			iFlRtpSource->SetFrameSize(iFrameSize);
		}
		
		iInitWaiting = EFalse;
		
		TRAP(error, iFlRtpSource->InitializeL(this, iVideoCodecType));
		if(!IsOK(error))
		{
			return error;
		}
		
		SETSTATE(EHXVDInitialize);
	}
	
	return error;
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::DecodeVideoFrameL(TStreamPacketData aPktData, TInt aWidth, TInt aHeight)
// 
//
// 
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoder::DecodeH264VideoFrame(TStreamPacketData& aPktData)//, TBool aCodedPicture)
{
	if(!iFlRtpSource)
	{
		LOG(CFLPlatformDecoder::DecodeFrame() : No XPS);
		return KErrNotReady;
	}
	
	if(CHECKSTATE(EHXVDInitial))
	{
			SETSTATE(EHXVDFirstFrameReceived);
			iFrameSizeKnown = ETrue;
			iInitWaiting = ETrue;
	}

	TInt error = KErrNone;

	if(IsVideoSeeking() && iSeekInStreamingPausedState && !iSeekPositionGot)
	{

		iSeekPositionGot = ETrue;
		iSeekPosition = aPktData.iTimeStamp *1000;
		return ETrue;
	}
	
	if(IsVideoSeeking() && iSeekInStreamingPausedState && iSeekPositionGot)
	{
		return ETrue;
	}
	
	if(IsVideoSeeking())
	{
		error = SeekVideo(aPktData);
		if(!IsOK(error))
		{
			return error;
		}
	}
	
	if(IsVideoReseting())
	{
		error = ResetVideo(aPktData);
		if(!IsOK(error))
		{
			return error;
		}
	}
	
	if(IsStreamingPausedVideoResetPending())
	{
		error = StreamingPauseDoReset(iVideoStreamId, aPktData);
		if(!IsOK(error))
		{
			return error;
		}
	}
	else
	{
		error = iFlRtpSource->AddPacket(iVideoStreamId, aPktData);
	    if( error == KErrOverflow )
		{
	        aPktData.iPacketConsumed= EFalse;
			error = 0;
		}
		if(!IsOK(error))
		{
			return error;
		}
	}


	// Previously an init call was issued but not considered because we were not ready.
	// Now we are ready, and we should issue init again. 
	// This init call is for flv video stream only. 
	
	if(iInitWaiting)
	{
		if(iFrameSizeKnown)
		{
			iFlRtpSource->SetFrameSize(iFrameSize);
		}
		
		iInitWaiting = EFalse;
		
		TRAP(error, iFlRtpSource->InitializeL(this, iVideoCodecType));
		if(!IsOK(error))
		{
			return error;
		}
		
		SETSTATE(EHXVDInitialize);
	}
	
	return error;
}


// -------------------------------------------------------------------------
// void CFLPlatformDecoder::DecodeAudioDataL(TStreamPacketData aPktData)
// 
// Sends an audio packet to XPS audio stream to send it to the audio decoder. 
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoder::DecodeAudioData(TStreamPacketData& aPktData)
{
	if(!iFlRtpSource || !IsAudioStreamPresent())
	{
		LOG(CFLPlatformDecoder::DecodeAudioData() : Bad State);
		return KErrNotReady;
	}
	
	if(CHECKSTATE(EHXVDErrorOccured))
	{
		LOG(CFLPlatformDecoder::DecodeAudioData() : Error State);
		return KErrNone;
	}

	TInt error = KErrNone;

	if(IsAudioSeeking() && !IsVideoStreamPresent() && iSeekInStreamingPausedState && !iSeekPositionGot)
	{

		iSeekPositionGot = ETrue;
		iSeekPosition = aPktData.iTimeStamp *1000;
		return ETrue;
	}
	
	if(IsAudioSeeking() && iSeekInStreamingPausedState && iSeekPositionGot)
	{
		return ETrue;
	}

	if(IsAudioSeeking())
	{
		error = SeekAudio(aPktData);
		if(!IsOK(error))
		{
			return error;
		}
	}
	
	if(IsAudioReseting())
	{
		error = ResetAudio(aPktData);
		if(!IsOK(error))
		{
			return error;
		}
	}
	
	if(IsStreamingPausedAudioResetPending())
	{
		error = StreamingPauseDoReset(iAudioStreamId, aPktData);
		if(!IsOK(error))
		{
			return error;
		}		
	}
	else	
	{
		error = iFlRtpSource->AddPacket(iAudioStreamId, aPktData);	
	    if( error == KErrOverflow )
		{
	        aPktData.iPacketConsumed= EFalse;
			error = 0;
		}
		if(!IsOK(error))
		{
			return error;
		}		
	}
	
	return error;
}

TInt CFLPlatformDecoder::StreamingPauseDoReset(const TInt& aStreamId, const TStreamPacketData& aPktData)
{
	TRAPD(error, iFlRtpSource->StreamingPauseDoResetL(aStreamId, aPktData));
	if(!IsOK(error))
	{
		return error;
	}

	CXPSMediaStream* stream;
    TInt err = iFlRtpSource->GetStreamInfo(aStreamId, stream);
    if(!IsOK(err)) 
	{
    	return err;
    }

	if(stream->StreamType() == EXPSAudioStream)
		iStreamingAudioReset = ETrue;
	else if(stream->StreamType() == EXPSVideoStream)
		iStreamingVideoReset = ETrue;
	
	UpdateStreamingResetState();
	
	return KErrNone;
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::UpdateStreamingResetState()
// 
// 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::UpdateStreamingResetState()
{
	iStreamingPauseResetCount++;
	
	if(iStreamingPauseResetCount == iFlRtpSource->MediaCount())
	{
		iStreamingPauseResetCount 	= 0;
		iStreamingVideoReset 	= EFalse;
		iStreamingAudioReset 	= EFalse;
		iRtmpStreamingResuming	= EFalse;
	}
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::UpdateSeekState()
// 
// Updates the seek state based on the number of streams seeked to a new position. 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::UpdateSeekState()
{
	iSeekCount++;
	
	if(iPlayComplete)
	{
		iPlayComplete  = EFalse;
	}
	
	if(iSeekCount == iFlRtpSource->MediaCount() && CHECKSTATE(EHXVDSeeking))
	{
		SETSTATE(EHXVDStarted);
		iSeekCount 	= 0;
		iSeekAudio 	= EFalse;
		iSeekVideo 	= EFalse;
	}
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::UpdateResetState()
// 
// Updates the reset state based on the number of streams reset to a new position. 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::UpdateResetState()
{
	iResetStreamCount++;
	
	if(iPlayComplete)
	{
		iPlayComplete  = EFalse;
	}

	if(iResetStreamCount == iFlRtpSource->MediaCount() && CHECKSTATE(EHXVDReseting))
	{
		SETSTATE(EHXVDStarted);
		iDoResetVideoStream = iDoResetAudioStream = EFalse;
		iResetStreamCount = 0;
	}
}


// -------------------------------------------------------------------------
// TInt CFLPlatformDecoder::SeekVideo(const TStreamPacketData& aPktData)
// 
// Issues seek command to video player and gives a new packet to be decoded after seek.  
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoder::SeekVideo(const TStreamPacketData& aPktData)
{
	TRAPD(error, iFlRtpSource->ResetL(iVideoStreamId, aPktData));
	if(!IsOK(error))
	{
		return error;
	}
		
	iSeekVideo = ETrue;
    iSeekPosition = aPktData.iTimeStamp * 1000;

	UpdateSeekState();
	return KErrNone;
}

// -------------------------------------------------------------------------
// TInt CFLPlatformDecoder::ResetVideo(const TStreamPacketData& aPktData)
// 
// Issues reset command to video player and gives a new packet to be decoded after reset.  
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoder::ResetVideo(const TStreamPacketData& aPktData)
{
	TRAPD(error, iFlRtpSource->ResetL(iVideoStreamId, aPktData));
	if(!IsOK(error))
	{
		return error;
	}
		
	iDoResetVideoStream = EFalse;
	UpdateResetState();
	return KErrNone;
}

// -------------------------------------------------------------------------
// TInt CFLPlatformDecoder::SeekAudio(const TStreamPacketData& aPktData)
// 
// Issues seek command to video player and gives a new packet to be decoded after seek.  
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoder::SeekAudio(const TStreamPacketData& aPktData)
{
	TRAPD(error, iFlRtpSource->ResetL(iAudioStreamId, aPktData));
	if(!IsOK(error))
	{
		return error;
	}
		
	iSeekAudio = ETrue;
	UpdateSeekState();
	return KErrNone;
}

// -------------------------------------------------------------------------
// TInt CFLPlatformDecoder::ResetAudio(const TStreamPacketData& aPktData)
// 
// Issues reset command to video player and gives a new packet to be decoded after reset.  
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoder::ResetAudio(const TStreamPacketData& aPktData)
{
	TRAPD(error, iFlRtpSource->ResetL(iAudioStreamId, aPktData));
	if(!IsOK(error))
	{
		return error;
	}
		
	iDoResetAudioStream = EFalse;
	UpdateResetState();
	return KErrNone;
}


// -------------------------------------------------------------------------
// void CFLPlatformDecoder::SetPadSizeInEmbeddedVideo(const TSize& aSize)
// 
// Sets the padding frame size in case of embedded video 
// as the padding bytes are missing in the header this needs to be done differently. 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::SetPadSizeInEmbeddedVideo(const TSize& aSize)
{
	if(iFrameSize.iWidth > aSize.iWidth)
	{
		iPadSize.iWidth    = iFrameSize.iWidth - aSize.iWidth;
		iFrameSize.iWidth -= iPadSize.iWidth;
	}
	if(iFrameSize.iHeight > aSize.iHeight)
	{
		iPadSize.iHeight    = iFrameSize.iHeight - aSize.iHeight;
		iFrameSize.iHeight -= iPadSize.iHeight;
	}
}


// -------------------------------------------------------------------------
// TInt CFLPlatformDecoder::ParseHeader(TStreamPacketData& aPktData, TSize& aFrameSize)
// 
// Parses the packet headers based on the packet format. 
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoder::ParseHeader(TStreamPacketData& aPktData, TSize& aFrameSize)
{
	switch(iVideoCodecType)
	{
		case ESorensonCodec:
		{
			return ParseSparkHeader(aPktData, aFrameSize);
		}
		case EOn2Codec:
		{
			return ParseVp6Header(aPktData, aFrameSize);
		}
		default:
		{
			return KErrNotSupported;
		}
	}
}

// -------------------------------------------------------------------------
// TInt CFLPlatformDecoder::ParseSparkHeader(TStreamPacketData& aPktData, TSize& aFrameSize)
// 
// Parses the Spark header 
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoder::ParseSparkHeader(TStreamPacketData& aPktData, TSize& aFrameSize)
{
	TUint8* framePtr = (TUint8*) aPktData.iFrameData;
	TUint   frameLen = aPktData.iDataLength;
	
	__ASSERT_ALWAYS(framePtr, User::Invariant());
	
	// Skip the first byte for normal flv. 
	// They will contain FrameType + CodecId info. 
		
	if(!aPktData.iEmbeddedVideo)
	{
		framePtr += 1;
		frameLen -= 1;	
	}

	if(!framePtr || frameLen < 5)
	{
		return KErrArgument;
	}
	
	TUint16* dataPtr = (TUint16*) framePtr;
	TUint16 word 	 = Reverse16(dataPtr);
	dataPtr++;
	
	if(word != 0)
	{
		return KErrCorrupt;
	}
	
	word = Reverse16(dataPtr);
	dataPtr++;
	
	if(!(word & 0x8000))
	{
		return KErrCorrupt;
	}
	
	TUint16 pictureSize = (word & 0x0003) << 1; 
	TUint8  pictureType = 0;
	
	TUint16 word2 = 0; 
	TUint16 word3 = 0;
	
	word = Reverse16(dataPtr);
	dataPtr++;
		
	pictureSize |= (word & 0x8000) >> 15;
	
	// Adjustments for custom width and height
	
	if(pictureSize == 0 || pictureSize == 1)
	{
		word2 = Reverse16(dataPtr);
		dataPtr++;
		
		word = (word << 1) | ((word2 & 0x8000) >> 15); 

		word3 = Reverse16(dataPtr);
		dataPtr++;
		
		word2 = (word2 << 1) | ((word3 & 0x8000) >> 15); 
	}
	else
	{
		pictureType = (word & 0x6000) >> 13;
	}	
	
	switch(pictureSize)
	{
		case 0: 
		{
			TUint16 width  = word >> KByteLength;
			TUint16 height = word & 0x00FF; 
			aFrameSize.SetSize(width, height);			
			pictureType = (word2 & 0xC000) >> 14;
			break;
		}
		case 1:
		{
			TUint16 width  = word;
			TUint16 height = word2; 
			aFrameSize.SetSize(width, height);
			pictureType = (word3 & 0x6000) >> 13;
			break;
		}
		case 2: aFrameSize.SetSize(KCIFWidth, KCIFHeight); break;
		case 3: aFrameSize.SetSize(KQCIFWidth, KQCIFHeight); break;
		case 4: aFrameSize.SetSize(KSQCIFWidth, KSQCIFHeight); break;
		case 5: aFrameSize.SetSize(KQVGAWidth, KQVGAHeight); break;
		case 6: aFrameSize.SetSize(KQQVGAWidth, KQQVGAHeight); break;
		default: aFrameSize.SetSize(0, 0); break; // Error
	}
	
	if(!pictureType)
	{
		 aPktData.iFrameType = EIFrame;
	}
	else
	{
		aPktData.iFrameType = EPFrame;
	}
	
	return KErrNone;
}

// -------------------------------------------------------------------------
// TInt CFLPlatformDecoder::ParseVp6Header(TStreamPacketData& aPktData, TSize& aFrameSize)
// 
// Parses Vp6 Headers.  
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoder::ParseVp6Header(TStreamPacketData& aPktData, TSize& aFrameSize)
{
	TUint8* framePtr = (TUint8*) aPktData.iFrameData;
	TUint   frameLen = aPktData.iDataLength;
	
	__ASSERT_ALWAYS(framePtr, User::Invariant());
	
	// Skip the first byte for normal flv. 
	// They will contain FrameType + CodecId info. 
		
	if(!aPktData.iEmbeddedVideo)
	{
		framePtr += 1;
		frameLen -= 1;	
	}
	
	if(!framePtr || frameLen < 5)
	{
		return KErrArgument;
	}
	
	TUint8* dataPtr = framePtr;
	TUint8  byte 	= *dataPtr++;
		
	TUint8 padWidth  = 0; 
	TUint8 padHeight = 0; 
	
	TFrameType frameType   = EPFrame;
	TBool multiStreamField = EFalse;
	
	if(!aPktData.iEmbeddedVideo)
	{
		padWidth  = (byte>>4) & 0x0F;
		padHeight = byte & 0x0F;
		byte 	  = *dataPtr++;
	}	
	
	if((byte >> 7) == 0)
	{
		frameType = EIFrame;		
	}
	if(byte & 0x1)
	{
		multiStreamField = ETrue;
	}
	
	if(frameType == EIFrame)
	{
		byte = *dataPtr++;
		
		TUint16 vp6Profile = (byte >> 1) & 0x11;
		
		if(multiStreamField || !vp6Profile)
		{
			dataPtr += 2;
		}
		
		TUint16 vFragments = *dataPtr++;
		TUint16 hFragments = *dataPtr++;
		
		TUint16 width  = hFragments << 4;
		TUint16 height = vFragments << 4;
		
		width  -= padWidth;
		height -= padHeight;
		
		aFrameSize.SetSize(width, height);
		aPktData.iFrameType = EIFrame;	
	}
	else if(frameType == EPFrame)
	{
		aPktData.iFrameType = EPFrame; 
		
		// Cannot determine the frame size from this. 
		// So should wait for an I-Frame. 
		
		return KErrNotReady;	
	}
		
	return KErrNone;
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::DispatchHXCommand(THXVideoDecCommand aCommand, THXVDCmdArg aArgInfo)
// 
// Dispatches Player commands to the Helix Video player.  
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::DispatchHXCommand(const THXVideoDecCommand& aCommand, 
										 const THXVDCmdArg& aArgInfo)
{
	TBool lDone = EFalse;
	/*Case: ou1cimx1#446095 , User may swtich between FS to NS multiple times before video starts
	hence we need to set RWindow given by Brplugin whether player is in preparing/opening. 
	otherwise we may get crash with wrong RWindow which is not existent.*/
	if ((aCommand == EHXVDCmdSetWindow) && (iFlRtpSource
			&& (iFlRtpSource->PlayerState() == EHXPOpening
					|| iFlRtpSource->PlayerState() == EHXPPreparing)))
		{
		ProcessHXCommand(aCommand, aArgInfo);
		}
	else
		iCommandQueue.Append(THxCommandBuf(aCommand, aArgInfo));

    while(iCommandQueue.Count() > 0 && !lDone)
    {
		switch(iCommandQueue[0].iCommand)
		{
			case EHXVDCmdPause:
			case EHXVDCmdASPause:
			case EHXVDCmdPlay:
			case EHXVDCmdResume:
			case EHXVDCmdStop:
			case EHXVDCmdSetVolume:
			case EHXVDCmdSeek:
			case EHXVDCmdReset:
			case EHXVDCmdEOF:
			{
				if(iFlRtpSource && (iFlRtpSource->PlayerState() == EHXPOpening ||        
				   iFlRtpSource->PlayerState() == EHXPPreparing))
				{
					//These commands cannot be serviced in these conditions *now*
					//Wait for state change.
					lDone = ETrue;
				}
				break;
			}
		};

		if(!lDone) {
	        ProcessHXCommand(iCommandQueue[0].iCommand, iCommandQueue[0].iCommandArg);
		    iCommandQueue.Remove(0);
		}
    }
}

    
// -------------------------------------------------------------------------
// void CFLPlatformDecoder::ProcessHXCommand(const THXVideoDecCommand& aCommand, const THXVDCmdArg& aArgInfo)
// 
// Dispatches Player commands to the Helix Video player.  
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::ProcessHXCommand(const THXVideoDecCommand& aCommand, 
                                         const THXVDCmdArg& aArgInfo)
{
    if(!iFlRtpSource)
    {
		return;
    }
	
    switch(aCommand)
	{
		case EHXVDCmdPause:
		{
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = Pause");
			Pause();
			break;
		}
		case EHXVDCmdASPause:
		{
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = ASPause");
			ASPause();
			break;
		}		
		case EHXVDCmdPlay:
		{
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = Play");
			Play();
			break;
		}
        case EHXVDCmdResume:
        {
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = Resume from engine");
            Resume();
            break;
        }
		case EHXVDCmdStop:
		{
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = Stop");
			Stop();
			break;
		}
		case EHXVDCmdSetScreenMode:
		{
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = SetScreenMode");
			iVideoRectSet = EFalse;
			iChangedScreenMode = ETrue;
			SetScreenMode(aArgInfo.iScreenMode);
			break;
		}
		case EHXVDCmdSetVolume:
		{
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = SetVolume");
			SetVolume(aArgInfo.iVolume);
			break;
		}
		case EHXVDCmdSetWindow:
		{			
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = SetWindow");
			SetWindow(aArgInfo.iWindow);
			break;
		}
		case EHXVDCmdRestart:
		{			
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = Restart");
			Restart();
			break;
		}
		case EHXVDCmdSeek:
		{
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = Seek");
			Seek();
			break;
		}
		case EHXVDCmdReset:
		{
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = Reset");
			Reset();
			break;
		}
		case EHXVDCmdSetDisplayRect:
		{
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = SetDisplayRect");
			
			// Not required in full-screen from flash plugin. 
			
			if(iFlRtpSource->UserMode() == EPluginMode &&  
			    iFlRtpSource->ScreenMode() == EFullScreenMode)
			{
				break;
			}
			
			if(aArgInfo.iDisplayRect != iSwfRect || iClipRect != aArgInfo.iClipRect)
			{
				iSwfRect  = aArgInfo.iDisplayRect;
				iClipRect = aArgInfo.iClipRect;

				iVideoRectSet = EFalse;

    			//fall through
				FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : fallthrough");
			}
			else
			{
				break;
			}
		}
		case EHXVDCmdRefreshScreen:
		{
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = RefreshScreen");
			
			// Need not refresh the video window if it is empty. 
			if(iVideoRect.IsEmpty())
			{
                FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : VideoRect is NULL");
				break;	
			}
			/* 
			//Commented: Incorrect to wait for SetVideoRectangle to initiate change in
			//display dimensions. Fixed for HWET-7M99BV
			else if(iChangedScreenMode)
			{
				FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Changing the screen mode and not ready");
				break;
			}*/
			
			iVideoRectSet = EFalse;
			
			FI_Rect rect; 
			rect.xmin = iVideoRect.iTl.iX;
			rect.xmax = iVideoRect.iBr.iX;
			rect.ymin = iVideoRect.iTl.iY;
			rect.ymax = iVideoRect.iBr.iY;
			
			TRAP_IGNORE(SetVideoRectL(&rect));
						
			break;
		}
		case EHXVDCmdEOF:
		{
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : cmd = EOF");
			iFlRtpSource->StreamEnd();
			break;
		}
		default:
		{
            FLPDECODERTRACEF("CFLPlatformDecoder::ProcessHXCommand() : Cmd = Unrecognized command");
			// Unknown Command 
			break;
		}		
	}
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::GetVideoPosition()
// 
//
// 
// --------------------------------------------------------------------------
// 

TTimeIntervalMicroSeconds CFLPlatformDecoder::GetVideoPosition()
{
	if(!iFlRtpSource)
		return 0;
	iVideoPosition = iFlRtpSource->GetVideoPosition();
	return iVideoPosition;
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::Seek()
// 
// Handles the seek command. 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::Seek()
{
	if(!(CHECKSTATE(EHXVDStarted) || 
		  CHECKSTATE(EHXVDSeeking) ||
		  CHECKSTATE(EHXVDReseting)))
	{
		return;
	}

	if(!iFlRtpSource)
	{
		return;
	}

	//We might be in the middle of a seek already.
	if(iSeekVideo || iSeekAudio)
	{
		iFlRtpSource->CancelReset();
	}	
	iSeekVideo = EFalse;
	iSeekVideo = EFalse;	
	iSeekCount = 0;

	//We might be in the middle of a reset already.
	if(iDoResetVideoStream || iDoResetAudioStream)
	{
		iFlRtpSource->CancelReset();
	}	
	iDoResetAudioStream = iDoResetVideoStream = ETrue;
	iResetStreamCount = 0;
	
	//In the case of seek in paused state in RTMP Streaming just change state to
	//Seeking and reset the states for handling seek in paused state.
	
	if(iFlRtpSource->PlayerState() == EHXPPaused && iMetaData->isStreamed)
	{
		if(iSeekInStreamingPausedState)
		{
			iSeekPositionGot = EFalse;
		}
		iSeekInStreamingPausedState = ETrue;
	   /*  RTMP stream seek relies on the packet timestamp to determine the seek position.
		*  When the stream has ended, we will not get any more packets and as a result when we 
		*  resume playback the VideoPosition time is not proper. We restart the streams so that
		*  when playback is resumed the seek position is proper. 
		*/
		if(IsAllStreamsEnded())
        {
            iFlRtpSource->Stop();
            RestartInPausedState();
        }
		SETSTATE(EHXVDSeeking);
		return;
	}
	//this if is required when we pause the video using swf controls and seek and player has sent EOF
	//We need to stop the player and restart the streams and set the video player state to paused	
	if(IsAllStreamsEnded() && iFlRtpSource->PlayerState() == EHXPPaused )
    {
        iFlRtpSource->PausePktSupplyForAllStreams();
        iFlRtpSource->Stop();
        RestartInPausedState();
        SETSTATE(EHXVDSeeking);
        return;
    }
	/* This handles seek when coreplayer has sent EOF, but playback from Helix is not
	     Complete
	*/
	if(IsAllStreamsEnded() && !iPlayComplete) 
	{
		iFlRtpSource->PausePktSupplyForAllStreams();
		iFlRtpSource->Stop();
		Restart();
	}
	/*  This handles seek when coreplayer has sent EOF and the playback is complete*/
	else if(IsAllStreamsEnded() && iPlayComplete) 
	{
		Restart();
	}
	
	SETSTATE(EHXVDSeeking);
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::Reset()
// 
// Handles the seek command. 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::Reset()
{
	if(!(CHECKSTATE(EHXVDStarted) || 
		  CHECKSTATE(EHXVDSeeking) ||
		  CHECKSTATE(EHXVDReseting)))
	{
		return;
	}	

	if(!iFlRtpSource)
	{
		return;
	}

	//We might be in the middle of a seek already.
	if(iSeekVideo || iSeekAudio)
	{
		iFlRtpSource->CancelReset();
	}
	iSeekVideo = EFalse;
	iSeekVideo = EFalse;	
	iSeekCount = 0;

	//We might be in the middle of a reset already.
	if(iDoResetVideoStream || iDoResetAudioStream)
	{
		iFlRtpSource->CancelReset();
	}
	iDoResetAudioStream = iDoResetVideoStream = ETrue;
	iResetStreamCount = 0;
	
		
	if(IsAllStreamsEnded() && !iPlayComplete)
	{
		iFlRtpSource->PausePktSupplyForAllStreams();
		iFlRtpSource->Stop();
	}
    
    /*
     * iPlayComplete is set to true in MfrsoPlaybackComplete(), which is called synchronously from CFLRtpSource::Stop()
     * 11800: Need to check for iPausedFromAS flag also, to handle various seek command sequence, 
     * like ASPause, Seek, Reset & ASPause, Reset.  In these scenario 'Play' will not called, 
     * hence we have to start RTP srouce.
     */
    if(iFlRtpSource && (iPlayComplete || iPausedFromAS))
	{
        if (iPausedFromAS)
            iPausedFromAS = EFalse;
        
		Restart();
	}

	SETSTATE(EHXVDReseting);
}


// -------------------------------------------------------------------------
// void CFLPlatformDecoder::RestartInPausedState()
// 
// Restarts the player and set the player state to paused and resumes the packet supply. 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::RestartInPausedState()
{
//	iCommandQueue.Reset();

	if(!iFlRtpSource) {
		return;
	}
	
	iPlayComplete  = EFalse;
	
    iFlRtpSource->RestartInPausedState();	
}
// -------------------------------------------------------------------------
// void CFLPlatformDecoder::Restart()
// 
// Restarts the playback and resumes the packet supply. 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::Restart()
{
//	iCommandQueue.Reset();

	if(!iFlRtpSource) {
		return;
	}
	
	iPlayComplete  = EFalse;
	
    iFlRtpSource->Restart();	
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::ASPause()
// 
// Pauses the playback of flash video. 
// --------------------------------------------------------------------------
// 


void CFLPlatformDecoder::ASPause()
{
    if(!iFlRtpSource)
    {
        return;
    }
    
    iPausedFromAS = ETrue;
    
    iFlRtpSource->Pause();
    if(iMetaData->isStreamed)
    {
    	iRtmpStreamingPaused = ETrue;
		iFlRtpSource->ResetAudioVideoStream();
    }    
}


// -------------------------------------------------------------------------
// void CFLPlatformDecoder::Pause()
// 
// Pauses the playback of flash video. Called from engine only.
// --------------------------------------------------------------------------
// 


void CFLPlatformDecoder::Pause()
{
	if(!iFlRtpSource)
	{
		return;
	}
	
	iFlRtpSource->Pause();
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::Play()
// 
// Starts the playback of flash video. 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::Play()
{
    iPausedFromAS = EFalse;
	if(!iFlRtpSource || CHECKSTATE(EHXVDErrorOccured))
	{
		return;
	}
	
	if(iFlRtpSource->PlayerState() == EHXPPaused)
	{
		if(iMetaData->isStreamed)
		{
			//To restart the playback from paused position when pause and resume is issued once EOF is reached
			if((iFlRtpSource->RtpSrcState() == EStreamEOF) && !iPlayComplete)
			{
				RestartInPausedState();
			}
			iFlRtpSource->RTMPStreamResume();
			iSeekInStreamingPausedState = EFalse;
			iSeekPositionGot = EFalse;
			iRtmpStreamingPaused = EFalse;
			iRtmpStreamingResuming = ETrue;
		}
		else
			iFlRtpSource->Resume();
	}
	
	else if(iFlRtpSource->RtpSrcState() == EStreamEOF)
	{
		iFlRtpSource->Stop();
	}

	if(iPlayComplete)
	{
		Restart();
		return;
	}   	
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::Resume()
// 
// This can be called from engine only
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::Resume()
{
    if(!iFlRtpSource || iPausedFromAS || CHECKSTATE(EHXVDErrorOccured))
    {  
        return;
    }
    
    if(iFlRtpSource->PlayerState() == EHXPPaused)
    {      
    	if(iMetaData->isStreamed && iRtmpStreamingResuming)
    	{
    		return;
    	}
        iFlRtpSource->Resume();
        return;
    }
    
    if(iFlRtpSource->RtpSrcState() == EStreamEOF && iMetaData->isStreamed)
    {
        /* ou1cimx1#386901: In case of Local & RTMP during BG to FG, we
         * receive Resume from Core & Host. In this case Helix might  
         * have buffers to be played. Hence, we should issue Stop 
         * only in case of RTMP streaming.
         * Split the if() condition also, which was a comment
         */
        iFlRtpSource->Stop();
    }     
}

// -------------------------------------------------------------------------
// void CFLPlatformDecoder::Stop()
// 
// Stops the playback of flash video. 
// --------------------------------------------------------------------------
// 

void CFLPlatformDecoder::Stop() 
{
	if(iFlRtpSource) {
	    iFlRtpSource->Stop();
	}
}

// -------------------------------------------------------------------------
// TInt CFLPlatformDecoder::GetSampleRate(IAudioSampleRates aSampleRate)
// 
//
// 
// --------------------------------------------------------------------------
// 

TInt CFLPlatformDecoder::GetSampleRate(const IAudioSampleRates& aSampleRate)
{
	switch(aSampleRate)
	{
		case IAudio5kHz:  return 5000; 
		case IAudio11kHz: return 11025; 
		case IAudio22kHz: return 22050; 
		case IAudio44kHz: return 44100; 
		default: return KErrNotSupported; 
	}
}

// -------------------------------------------------------------------------
// TCodecType CFLPlatformDecoder::GetAudioCodecType(TUint aAudioFormat)
// 
//
// 
// --------------------------------------------------------------------------
// 

TCodecType CFLPlatformDecoder::GetAudioCodecType(const TUint& aAudioFormat)
{
	switch(aAudioFormat)
	{
		case IAudioCompressMP3: return EMp3Codec;
		case IAudioCompressADPCM: return EADPCMCodec; // Support this. 
		case IAudioCompressAAC: return EAacCodec;
		default : return ENoCodec;
	}
}

void CFLPlatformDecoder::ParseConfigData(const TUint8* aConfigData, TUint aConfigLength)
{
	TUint8 nParameterSets = 0;
	TUint32 ulIndex;
	TUint16 usDataCount = 5;
	TUint nWritten = 0;
	TUint16 usParameterSetLength;
	TUint8 configData[255];
	
	DumpAVCConfigToFile((const TUint8*)aConfigData, aConfigLength);
	DumpAVCConfigToFile((const TUint8*)"\n", 1);
	
	//configData = (TUint8*) new TUint8[aConfigLength];
	
	//for H264 codec video we need to get the profile and the level information
	TInt profileInfo = aConfigData[PROFILE_INFO_OFFSET];
	TInt constraintInfo = aConfigData[CONSTRAINT_INFO_OFFSET];
	TInt levelInfo   = aConfigData[LEVEL_INFO_OFFSET];
	
	LOG3(Profile [%x] Constraint [%x] Level [%x], profileInfo, constraintInfo, levelInfo);
	
	//TODO: store the profile and level which will be reqd for forming the SDPInfo
	StoreAvcInfo(profileInfo, constraintInfo, levelInfo);
				
	// Get the size of the NAL Unit length [number of bytes]
	iSizeOfNalUnitLength = (aConfigData[NAL_UNIT_LENGTH_SIZE_OFFSET] & NAL_UNIT_LENGTH_SIZE_MASK) + 1;
	
	if((iSizeOfNalUnitLength % 2) != 0)
	{
		//TODO : Handle Invalid length
	}
	
    // Write Sequence Parameter set
    nParameterSets = aConfigData[usDataCount++] & 0x1F;

    for (ulIndex=0; ulIndex < nParameterSets;  ulIndex++) 
    {
         usParameterSetLength = (aConfigData[usDataCount]<<8) + aConfigData[usDataCount+1];
         usDataCount += 2;

         nWritten = BinTo64((aConfigData + usDataCount), 
                                        usParameterSetLength, 
                                        (char *)configData + nWritten);
         nWritten--;
         if(ulIndex != nParameterSets-1)
         {
         	configData[nWritten++] = ',';
         }

         usDataCount += usParameterSetLength;
    }

    // Write Picture Parameter set
    nParameterSets = aConfigData[usDataCount++];

    for (ulIndex=0; ulIndex < nParameterSets;  ulIndex++)
    {
         usParameterSetLength = (aConfigData[usDataCount]<<8) + aConfigData[usDataCount+1];
         usDataCount += 2;

         configData[nWritten++] = ',';

         nWritten += BinTo64((aConfigData + usDataCount), 
                                        usParameterSetLength, 
                                        (char *)configData + nWritten);
         nWritten--;
         if(ulIndex != nParameterSets-1)
         {
         	configData[nWritten++] = ',';
         }
    }
    
    *(configData + nWritten++) = '\0';
    
    //TODO: Handle OOM error
    SetAvcConfigData(configData, nWritten);
    
    DumpAVCConfigToFile((const TUint8*)configData, nWritten);
    
    //delete configData;
}

void DumpAVCConfigToFile(const TUint8* aData, 
                               TUint aDataLength)
{
    if(!aData || !aDataLength)
        return;
    
    TFileName fileName;
    fileName.Copy(_L("C:\\Logs\\HelixLogs\\"));
    fileName.AppendFormat(_L("avccfg.txt"));
    
    TPtr8 buf((TUint8*)aData, aDataLength, aDataLength);
    
    TInt error = KErrNone;
    RFs fs;
    if(fs.Connect() != KErrNone)
    {
    	return;
    }
    
    RFile dumpFile;
    error = dumpFile.Open(fs, fileName, EFileWrite);
    if(error == KErrNotFound)
        {
        error = dumpFile.Create(fs, fileName, EFileWrite);
        }
    if(IsOK(error))
        {
        TInt pos = 0;
        dumpFile.Seek(ESeekEnd, pos);
        dumpFile.Write(buf);
        }
    dumpFile.Close();
    fs.Close();
}

int BinTo64(const TUint8* pInBuf, int len, char* pOutBuf)
{
    int c1, c2, c3;
    //INT32 ct = 0;
    //INT32 written = 0;
    int inOffset = 0;
    int outOffset = 0;

    while(inOffset < len)
    {
	c1 = pInBuf[inOffset++];
	if(inOffset == len)
	{
	    Output64Chunk(c1, 0, 0, 2, pOutBuf, outOffset);
	    outOffset += 4;
	}
	else
	{
	    c2 = pInBuf[inOffset++];
	    if(inOffset == len)
	    {
		Output64Chunk(c1, c2, 0, 1, pOutBuf, outOffset);
		outOffset += 4;
	    }
	    else
	    {
		c3 = pInBuf[inOffset++];
		Output64Chunk(c1, c2, c3, 0, pOutBuf, outOffset);
		outOffset += 4;
	    }
	}
#if XXXBAB
	ct += 4;
	if(ct > 44)	// EOL after 45 chars
	{
	    pOutBuf[outOffset++] = '\n';
	    written += 46;
	    ct = 0;
	}
#endif
    }
#if XXXBAB
    if(ct)
    {
	pOutBuf[outOffset++] = '\n';
	ct++;
    }
#endif
    pOutBuf[outOffset++] = '\0';
    return outOffset;
}

void Output64Chunk(int c1, int c2, int c3, int pads, char* pBuf,
    									int bufOffset)
{
    pBuf[bufOffset++] = Base64[c1>>2];
    pBuf[bufOffset++] = Base64[((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4)];
    if (pads == 2) 
    {
        pBuf[bufOffset++] = '=';
        pBuf[bufOffset++] = '=';
    }
    else if (pads) 
    {
        pBuf[bufOffset++] = Base64[((c2 & 0xF) << 2) | ((c3 & 0xC0) >>6)];
        pBuf[bufOffset++] = '=';
    } 
    else 
    {
        pBuf[bufOffset++] = Base64[((c2 & 0xF) << 2) | ((c3 & 0xC0) >>6)];
        pBuf[bufOffset++] = Base64[c3 & 0x3F];
    }
}
