/*
* Copyright (c) 2008 - 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class
*
*/


#include "flrtpsource.h"
//#include "helixglobals.h"
#include <coemain.h>
#include <eikappui.h>
#include <gdi.h>
#include <w32std.h>
#include <e32math.h>
#include <f32file.h>
#include <bautils.h>
#include <AudioPreference.h>
#include <NokiaAudioPreference.h>
#include <platform/mmf/common/mmferrors.h>
#include "callbackscheduler.h"
#include "symbian_engine.h"

_LIT8(KBasicSDPInfo, "v=0\r\nc=IN IP4 0.0.0.0\r\nt=0 0\r\na=XpsStreamSyncOffset:integer;1000\r\n");
//KFlvVideoSDPInfor's XpsPyldBufSize has been chaged to 165800 from 55240. This change has been done to fix TSW Id: AMMA-7MGBNX
_LIT8(KFlvVideoSDPInfo, "m=video 0 RTP/AVP 96\r\na=fmtp:96 profile=0; level=20\r\na=secureidkey:string;\"DFXY5TJNHU3NDQXFWDQ3\"\r\na=XpsHdrBufSize:integer;512\r\na=XpsPyldBufSize:integer;1750000\r\n");
_LIT8(KFlvMp3SDPInfo, "m=audio 49000 RTP/AVP 97\r\na=rtpmap:97 MPEG-ELEMENTARY/");
_LIT8(KH264VideoSDPInfo, "m=video 0 RTP/AVP 96\r\n");
//KXpsHdrPyldSize, XpsPyldbufSize old value was not enough to hold bigger packets for playback of high bitrate video. e.g. H.264 main and high profile video. So buffer size has been change to 165800 from
// 55240
_LIT8(KXpsHdrPyldSize, "a=XpsHdrBufSize:integer;512\r\na=XpsPyldBufSize:integer;1750000\r\n"); //55240
_LIT8(KAacAudioSDPInfo, "m=audio 0 RTP/AVP 97\r\n");


//#define _DUMP_SDPINFO_TO_FILE_

extern void DebugLog(TRefByValue<const TDesC> aText, ...);

const TInt KRefreshCount = 5;

//#define PRINT_RECT
#define RTPTRACE(args...) if(iDoLog) { RDebug::Print(##args); }
#define RTPTRACEF(args...) if(iDoLog) { RDebug::Printf(##args); }

// -------------------------------------------------------------------------
// void CFLRtpSource::NewL()
//
// Static Function NewL to construct the RTP source.
// --------------------------------------------------------------------------
//

CFLRtpSource* CFLRtpSource::NewL(CPlatformFlashEngine& aEngine)
{
    CFLRtpSource* flRtpSource = new (ELeave) CFLRtpSource(aEngine);
    CleanupStack::PushL(flRtpSource);
    flRtpSource->ConstructL();
    CleanupStack::Pop(flRtpSource);
    return flRtpSource;
}


// -------------------------------------------------------------------------
// void CFLRtpSource::~CFLRtpSource()
//
// Destructor
// --------------------------------------------------------------------------
//

CFLRtpSource::~CFLRtpSource()
{
    RTPTRACE( _L("CFLRtpSource.~< ++") );

    RTPTRACE( _L("CFLRtpSource.~< reseting cbk scheduler ++") );

	//Dont need any more callbacks at this stage, hence reset
    if (iCallbackScheduler)
        {
        iCallbackScheduler->Reset();
        }

    RTPTRACE( _L("CFLRtpSource.~< destroying cbk scheduler ++") );

	delete iCallbackScheduler;
	iCallbackScheduler = 0;

    RTPTRACE( _L("CFLRtpSource.~< resetdestroy arglist ++") );

	//some callbacks might have been cancelled.
	//cleanup its args
	iArgList.ResetAndDestroy();

    RTPTRACE( _L("CFLRtpSource.~< closing arglist ++") );

	iArgList.Close();

	#ifdef _USE_SINK
	delete iXPSPacketSink;
	#endif

	delete iSDPInfo;

	iObserverList.Reset();
	iCodecTypeList.Reset();
	iXPSStreamList.ResetAndDestroy();

    RTPTRACE( _L("CFLRtpSource.~< deleting video player ++") );

	if(iVideoPlayer)
    {
        iVideoPlayer->Close();
        delete iVideoPlayer;
        iVideoPlayer = NULL;
    }

    RTPTRACE( _L("CFLRtpSource.~> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::CFLRtpSource()
//
// Constructor
// --------------------------------------------------------------------------
//

CFLRtpSource::CFLRtpSource(CPlatformFlashEngine& aEngine)
: iHostEngine(aEngine)
{
    RTPTRACE( _L("CFLRtpSource.CFLRtpSource< ++") );

	#ifdef _USE_SINK
	iXPSPacketSink  = NULL;
	#endif

    iVideoPlayer = NULL;

    iObserverList.Reset();
	iCodecTypeList.Reset();
	iXPSStreamList.Reset();

	iInitCount      = 0;
    iResetCount     = 0;
	iRefreshCount   = 0;

	iTotalDuration  = 0;

	iVideoStreamPresent = EFalse;
	iAudioStreamPresent = EFalse;

	iSDPInfo = NULL;

	iFrameSize.SetSize(0, 0);
	iAppRect.SetRect(TPoint(0,0), TSize(0, 0));
    iClipRect.SetRect(TPoint(0,0), TSize(0, 0));
	iWindowRect.SetRect(TPoint(0,0), TSize(0, 0));

	iArgList.Reset();

	SetPlayerState(EHXPInitial);
	iState = EInitial;
	
#ifdef USE_VPU2	
	iCurrentDisplayWindow = NULL;
#endif
	iWindow = NULL;

    RTPTRACE( _L("CFLRtpSource.CFLRtpSource> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::ConstructL()
//
// II Phase constructor.
// --------------------------------------------------------------------------
//

void CFLRtpSource::ConstructL()
{
    RTPTRACE( _L("CFLRtpSource.ConstructL< ++") );

	RFs lFs(CCoeEnv::Static()->FsSession());
	TFileName lLogFlagFileName(_L("C:\\data\\xpslogs\\rtpsource.txt"));
    if(BaflUtils::FileExists(lFs, lLogFlagFileName))
	{
		iDoLog = ETrue;
	}
	else
	{
		iDoLog = EFalse;
	}

    #ifdef _USE_SINK
	iXPSPacketSink = CXPSPacketSink::New();
	if(!iXPSPacketSink) {
		User::LeaveNoMemory();
	}
    #endif

	iScreenDevice = CCoeEnv::Static()->ScreenDevice();
	if(!iScreenDevice) {
		User::Leave(KErrNotReady);
	}

	iApp = CEikonEnv::Static()->EikAppUi();
	if(!iApp) {
		User::Leave(KErrNotReady);
	}

	iWsSession = CCoeEnv::Static()->WsSession();

	iCallbackScheduler = CCallbackScheduler::NewL();

    RTPTRACE( _L("CFLRtpSource.ConstructL> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::InitializeL(MFlRtpSourceObserver* aObs, TCodecType /*aCodecType*/)
//
// Initializes the RTP source and the XPS.
// --------------------------------------------------------------------------
//

void CFLRtpSource::InitializeL(MFlRtpSourceObserver* aObs, const TCodecType& /*aCodecType*/)
{
    RTPTRACE( _L("CFLRtpSource.InitializeL< ++") );

    if(iState != EInitial && iState != EInitializing) {
        RTPTRACE( _L("CFLRtpSource.InitializeL> --, Invalid state! [%d]"), iState );
        return;
    }

    if(iInitCount == 0)
    {
        #ifdef _USE_SINK
        iXpsServerName.Append(KXPSServerName);
        iXpsServerName.AppendNum(Math::Random());
        RTPTRACE(iXpsServerName);
        iXPSPacketSink->Init(iXpsServerName, this);
        LOG(CFLRtpSource::InitializeL() : Init Server);
        #endif
    }

    iObserverList.Append(aObs);

    iState = EInitializing;
    iInitCount++;


    if(iInitCount == MediaCount())
    {
        if(iVideoPlayer)
            User::Panic(_L("Player is not deleted"),KErrGeneral);

        RTPTRACEF("CFLRtpSource::InitializeL() : WRect (%d %d) (%d %d)",  iWindowRect.iTl.iX, iWindowRect.iTl.iY, iWindowRect.iBr.iX, iWindowRect.iBr.iY);
        RTPTRACEF("CFLRtpSource::InitializeL() : ClipRect (%d %d) (%d %d)",   iClipRect.iTl.iX, iClipRect.iTl.iY, iClipRect.iBr.iX, iClipRect.iBr.iY);

        /*
		Doesn't matter if the rect is empty or not.
        if(iWindowRect.IsEmpty() && IsVideoStreamPresent())
        {
            LOG(CFLRtpSource::InitializeL() : Empty Window rect);
            RTPTRACE( _L("CFLRtpSource.InitializeL> --") );
            return;
        }
        */

#ifndef USE_VPU2        
        iVideoPlayer = CVideoPlayerUtility::NewL(*this,EMdaPriorityMax ,
                                            EMdaPriorityPreferenceTimeAndQuality,
                                            CCoeEnv::Static()->WsSession(),
                                            *iScreenDevice,
                                            *iWindow,
                                            iWindowRect,
                                            iClipRect);
#else
        /* While creating video player the priority of playback is chosen such that the video play will be 
           given preference over audio play, priority and preference settings allow the Audio Policy Server 
           making decisions regarding play requests received from the clients.
         */

        if(IsVideoStreamPresent())
        {
            iVideoPlayer = CVideoPlayerUtility2::NewL(*this,
                                              static_cast<enum TMdaPriority>(KAudioPriorityVideoPlayback),
                                              static_cast<enum TMdaPriorityPreference> (KAudioPrefVideoPlayback));
        }
        else if(IsAudioStreamPresent())
        {
        /* RC Defect ou1cimx1#517917 : If Videostream is not present for eg. M4A streaming we set 
         * the priority and preference same as in Mp3 playback 
         */
            iVideoPlayer = CVideoPlayerUtility2::NewL(*this,
                                              static_cast<enum TMdaPriority>(EMdaPriorityMax),
                                              static_cast<enum TMdaPriorityPreference>(EMdaPriorityPreferenceTimeAndQuality));
        }
#endif

        #ifdef _USE_SINK
        TSecurityPolicy securityPolicy(ECapability_None);
        iXPSPacketSink->AuthenticateClient(securityPolicy);

        // Provide the SDP Info to the Packet Sink
        ConstructSDPInfoL();

        // Configure the number of buffers thorugh ConfigStream()
        for(TInt i = 0; i < iInitCount; i++)
        {
            TInt error = iXPSPacketSink->ConfigStream(i, KMaxStreamBuffers);
            if(error) {
                iState = EDead;
                User::Leave(error);
            }
        }
        #endif

        iState = EInitComplete;
        InitCompleteL();
    }

    RTPTRACE( _L("CFLRtpSource.InitializeL> --") );
}


// -------------------------------------------------------------------------
// TInt CFLRtpSource::AddStreamL(const TXPSStreamType& aStreamType, const TCodecType& aCodecType)
//
// Creates and adds an XPS stream to the list of streams.
// --------------------------------------------------------------------------
//

TInt CFLRtpSource::AddStreamL(const TXPSStreamType& aStreamType, const TCodecType& aCodecType)
{
    RTPTRACE( _L("CFLRtpSource.AddStreamL< ++") );

    TInt streamId = -1;

    if(iState == EInitial || iState == EInitializing)
    {
        streamId = iXPSStreamList.Count();

        #ifdef _USE_SINK
        CXPSMediaStream* stream = CXPSMediaStream::NewLC(streamId, *iXPSPacketSink, *this);
        #endif

        iCodecTypeList.Append(aCodecType);
        iXPSStreamList.AppendL(stream);
        CleanupStack::Pop(stream);

        stream->SetStreamType(aStreamType);
        stream->SetCodecType(aCodecType);

        if(aStreamType == EXPSVideoStream) {
            iVideoStreamPresent = ETrue;
        }
        else if(aStreamType == EXPSAudioStream) {
            iAudioStreamPresent = ETrue;
        }
    }
    else {
        RTPTRACEF("CFLRtpSource::AddStreamL() : Invalid state! [%d]", iState);
    }

    RTPTRACE(_L("CFLRtpSource.AddStreamL> --"));
    return streamId;
}


// -------------------------------------------------------------------------
// void CFLRtpSource::RemoveStream(TInt aStreamId)
//
// Removes the XPS stream from the list.
// --------------------------------------------------------------------------
//

void CFLRtpSource::RemoveStream(const TInt& aStreamId)
{
    RTPTRACE( _L("CFLRtpSource.RemoveStream< ++") );

    for(TInt i = 0; i < iXPSStreamList.Count(); i++)
    {
        CXPSMediaStream* stream = iXPSStreamList[i];
        if(stream->StreamId() == aStreamId)
        {
            iXPSStreamList.Remove(i);
            delete stream;
            break;
        }
    }

    RTPTRACE( _L("CFLRtpSource.RemoveStream> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::AddMedia()
//
// Adds the media reference.
// --------------------------------------------------------------------------
//

void CFLRtpSource::AddMedia()
{
    RTPTRACE( _L("CFLRtpSource.AddMedia< ++") );
    iMediaCount++;
    RTPTRACE( _L("CFLRtpSource.AddMedia> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::RemoveMedia()
//
// Decrements the reference count.
// --------------------------------------------------------------------------
//

void CFLRtpSource::RemoveMedia()
{
    RTPTRACE( _L("CFLRtpSource.RemoveMedia< ++") );
    iMediaCount--;
    RTPTRACE( _L("CFLRtpSource.RemoveMedia> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::Start(const TInt& aStreamId)
//
// Starts the pkt supply of a particular XPS stream.
// --------------------------------------------------------------------------
//

void CFLRtpSource::Start(const TInt& aStreamId)
{
    RTPTRACE( _L("CFLRtpSource.Start< ++") );

    if(iState == EInitComplete)
    {
        CXPSMediaStream* stream;
        TInt error = GetStreamInfo(aStreamId, stream);
        if(!IsOK(error)) {
            RTPTRACE( _L("CFLRtpSource.Start> --, error getting stream object [%d]"),error );
            return;
        }

        stream->Start();

        TInt lNumStreamsStarted = 0;
        for(TInt i = 0; i < iXPSStreamList.Count(); i++)
        {
            if(iXPSStreamList[i]->State() == CXPSMediaStream::EStreamStarted) {
                lNumStreamsStarted++;
            }
        }

        if(lNumStreamsStarted == iXPSStreamList.Count())
            iState = EStarted;
    }
    else {
        RTPTRACEF("CFLRtpSource::Start() : Invalid state! [%d]", iState);
    }

    RTPTRACE( _L("CFLRtpSource.Start> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::ResetL(TInt aStreamId, TStreamPacketData aPktData)
//
// Resets the playback head - resets the pkt list and sets a new position.
// Primarily used in case of seeking.
// --------------------------------------------------------------------------
//

void CFLRtpSource::ResetL(const TInt& aStreamId, const TStreamPacketData& aPktData)
{
    RTPTRACE( _L("CFLRtpSource.ResetL< ++, StreamId = [%d]"), aStreamId );

    //Cannot reset in EOF state, a streamend might have already been
    //signalled to xps and playbackcomplete might be on its way!
    if(iState == EInitComplete || iState == EStarted || ISPLAYERSTATE(EHXPPlayCompleted))
    {
		if(!iResetCount)
		{
			/* Doing pause,setposition,play atomically when
			 * all the streams are received, so that video player
			 * does not goto paused state which is inconsistent with
			 * actual playback state.
			iVideoPlayer->PauseL();

			#ifdef _USE_SINK
			iXPSPacketSink->Reset();
			#endif
			 */
		}

        for(TInt i = 0; i < iXPSStreamList.Count(); i++)
        {
            CXPSMediaStream* stream = iXPSStreamList[i];
            if(stream && stream->StreamId() == aStreamId)
            {
                stream->Pause();

				//Not reseting here any more.
				//This is redundant since we reset anyway in
				//FlushAfterSeek call
                //stream->Reset();
                if((stream->StreamType() == EXPSVideoStream) || ((stream->StreamType() == EXPSAudioStream) && !IsVideoStreamPresent()) )
				{
					iSeekPosition = aPktData.iTimeStamp * 1000;
				}
                User::LeaveIfError(AddPacket(aStreamId, aPktData));
				iResetCount++;
                break;
            }
        }

		if(iResetCount == iXPSStreamList.Count())
		{
			if(ISPLAYERSTATE(EHXPPlaying))
			{//Pause only if not already paused.
				iVideoPlayer->PauseL();
			}

			#ifdef _USE_SINK
			iXPSPacketSink->Reset();
			#endif

			SetPositionL(iSeekPosition);
			iResetCount = 0;

			if(ISPLAYERSTATE(EHXPPlaying))
			{//resume player if only in playing mode.
				for(TInt i = 0; i < iXPSStreamList.Count(); i++)
				{
					iXPSStreamList[i]->Resume();
				}

				iVideoPlayer->Play();
			}
		}
    }
    else {
		RTPTRACEF("CFLRtpSource::Reset() : Invalid state! [%d]", iState);
        User::Leave(KErrGeneral);
    }

    RTPTRACE( _L("CFLRtpSource.ResetL> --") );
}


void CFLRtpSource::StreamingPauseDoResetL(const TInt& aStreamId, const TStreamPacketData& aPktData)
{
    for(TInt i = 0; i < iXPSStreamList.Count(); i++)
    {
		CXPSMediaStream* stream = iXPSStreamList[i];
        if(stream && stream->StreamId() == aStreamId)
        {
			if((stream->StreamType() == EXPSVideoStream) || ((stream->StreamType() == EXPSAudioStream) && !IsVideoStreamPresent()))
		  	{
				iSeekPosition = aPktData.iTimeStamp * 1000;
		  	}
         	User::LeaveIfError(AddPacket(aStreamId, aPktData));
	  		iStreamingPauseResetCount++;
            break;
        }
    }
	if(iStreamingPauseResetCount == iXPSStreamList.Count())
	{
		SetPositionL(iSeekPosition);
		iStreamingPauseResetCount	= 0;
		for(TInt i = 0; i < iXPSStreamList.Count(); i++)
		{
			iXPSStreamList[i]->Resume();
		}

		iVideoPlayer->Play();
		SetPlayerState(EHXPPlaying);
	}
}

/*
 *  Revert anything done in any partial ResetL.
 */
void CFLRtpSource::CancelReset()
{
	iResetCount = 0;

	if(ISPLAYERSTATE(EHXPPlaying))
	{
		for(TInt i = 0; i < iXPSStreamList.Count(); i++)
		{
			iXPSStreamList[i]->Reset();
			iXPSStreamList[i]->Resume();
		}
	}
}


// -------------------------------------------------------------------------
// TInt CFLRtpSource::AddPacketL(TInt aStreamId, TStreamPacketData aPktData)
//
// Adds a RTP media packet into the packet list of the pertinent stream.
// --------------------------------------------------------------------------
//

TInt CFLRtpSource::AddPacket(const TInt& aStreamId, const TStreamPacketData& aPktData)
{
    RTPTRACE(_L("CFLRtpSource.AddPacket< [%d]++"), aStreamId);
   TInt err = KErrNone;

    if(!aPktData.iFrameData || aPktData.iDataLength <= 0) {
        RTPTRACE( _L("CFLRtpSource.AddPacket> --, 1") );
        //Fix for PMEO-83NDHM
        //return KErrArgument;
        return KErrNone;
    }
    if(ISPLAYERSTATE(EHXPErrorOccured)) {
        RTPTRACE( _L("CFLRtpSource.AddPacket> --, 2") );
        return KErrGeneral;
    }

    if(iState == EInitial ||
	   iState == EInitializing ||
	   iState == EInitComplete ||
	   iState == EStarted)
    {
        CXPSMediaStream* stream;
        TInt error = GetStreamInfo(aStreamId, stream);
        if(!IsOK(error)) {
            RTPTRACE( _L("CFLRtpSource.AddPacket> --, error getting stream object [%d]"), error );
            return error;
        }

        err = stream->AddPacket(aPktData);
    }
    else {
        RTPTRACEF("(CFLRtpSource::AddPacket() : Invalid state! [%d]", iState);
    }

    RTPTRACE( _L("CFLRtpSource.AddPacket> --") );
    return err;
}


// -------------------------------------------------------------------------
// void CFLRtpSource::PausePktSupplyForAllStreams()
//
// Notifies to pause the pkt supply for all streams.
// --------------------------------------------------------------------------
//

void CFLRtpSource::PausePktSupplyForAllStreams()
{
    RTPTRACE( _L("CFLRtpSource.PausePktSupplyForAllStreams< ++") );

    if(iState == EStarted || iState == EStreamEOF)
    {
        for(TInt i = 0; i < iXPSStreamList.Count(); i++) {
            iXPSStreamList[i]->Pause();
        }
    }
    else {
        RTPTRACEF("CFLRtpSource::PausePktSupplyForAllStreams() : Invalid state! [%d]", iState);
    }

    RTPTRACE( _L("CFLRtpSource.PausePktSupplyForAllStreams> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::ResetAudioVideoStream()
//
//
// --------------------------------------------------------------------------
//

void CFLRtpSource::ResetAudioVideoStream()
{
    RTPTRACE( _L("CFLRtpSource.ResetAudioVideoStream< ++") );

    if(iState == EInitializing ||
       iState == EInitComplete ||
       iState == EStarted      ||
       iState == EStreamEOF    ||
	   iState == EStopped)
    {
	    for(TInt i = 0; i < iXPSStreamList.Count(); i++) {
		    iXPSStreamList[i]->Reset();
	    }

        #ifdef _USE_SINK
	    iXPSPacketSink->Reset();
        #endif
    }
    else {
        RTPTRACEF("CFLRtpSource::ResetAudioVideoStream() : Invalid state! [%d]", iState);
    }

    RTPTRACE( _L("CFLRtpSource.ResetAudioVideoStream> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::StreamEnd()
//
//
// --------------------------------------------------------------------------
//

void CFLRtpSource::StreamEnd()
{
    RTPTRACE( _L("CFLRtpSource.StreamEnd< ++") );

    if(iState == EInitComplete ||
       iState == EStarted)
    {
	    for(TInt i = 0; i < iXPSStreamList.Count(); i++) {
		    iXPSStreamList[i]->StreamEnd();
	    }

	    iState = EStreamEOF;
    }
    else {
        RTPTRACEF("CFLRtpSource::StreamEnd() : Invalid state! [%d]", iState);
    }

    RTPTRACE( _L("CFLRtpSource.StreamEnd> --") );
}


// -------------------------------------------------------------------------
// TBool CFLRtpSource::IsStreamEnded() const
//
// Checks whether any one of the streams has notified it's end of stream.
// --------------------------------------------------------------------------
//

TBool CFLRtpSource::IsStreamEnded() const
{
    RTPTRACE( _L("CFLRtpSource.IsStreamEnded< ++") );
    RTPTRACE( _L("CFLRtpSource.IsStreamEnded> -- [%d]"),(iState == EStreamEOF || iState == EStopped) );
    return (iState == EStreamEOF || iState == EStopped);
}


// -------------------------------------------------------------------------
// void CFLRtpSource::Play()
//
// Starts / Resumes the playback.
// --------------------------------------------------------------------------
//

void CFLRtpSource::Play()
{
    RTPTRACE( _L("CFLRtpSource.Play< ++") );

    if(ISPLAYERSTATE(EHXPPaused))
    {
        iVideoPlayer->Play();
        SetPlayerState(EHXPPlaying);
    }
    else {
        RTPTRACEF("CFLRtpSource::Play() : Invalid state! [%d]", iState);
    }

    RTPTRACE( _L("CFLRtpSource.Play> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::Pause()
//
// Pauses the video player and the pkt supply.
// --------------------------------------------------------------------------
//

void CFLRtpSource::Pause()
{
    RTPTRACE( _L("CFLRtpSource.Pause< ++") );

    if(ISPLAYERSTATE(EHXPPlaying))
    {
        TRAP_IGNORE(iVideoPlayer->PauseL());
        SetPlayerState(EHXPPaused);

        for(TInt i = 0; i < iXPSStreamList.Count(); i++) {
            iXPSStreamList[i]->Pause();
        }
    }
    else {
        RTPTRACEF("CFLRtpSource::Pause() : Invalid state! [%d]", iState);
    }

    RTPTRACE( _L("CFLRtpSource.Pause> --") );
}

// -------------------------------------------------------------------------
// void CFLRtpSource::Resume()
//
// Resumes the video player.
// --------------------------------------------------------------------------
//

void CFLRtpSource::Resume()
{
    RTPTRACE( _L("CFLRtpSource.Resume< ++") );

    if(ISPLAYERSTATE(EHXPPaused))
    {
        Play();

        for(TInt i = 0; i < iXPSStreamList.Count(); i++) {
            iXPSStreamList[i]->Resume();
        }
    }
    else {
        RTPTRACEF("CFLRtpSource::Resume() : Invalid state! [%d]", iState);
    }

    RTPTRACE( _L("CFLRtpSource.Resume> --") );
}

// -------------------------------------------------------------------------
// void CFLRtpSource::RTMPStreamResume()
//
// Resumes the video player in the case of RTMP Streaming.
// --------------------------------------------------------------------------
//
void CFLRtpSource::RTMPStreamResume()
{
    RTPTRACE( _L("CFLRtpSource.RTMPStreamResume< ++") );

    if(ISPLAYERSTATE(EHXPPaused))
    {
        for(TInt i = 0; i < iXPSStreamList.Count(); i++)
		{
            iXPSStreamList[i]->Resume();
        }
    }
    else
	{
        RTPTRACEF("CFLRtpSource::RTMPStreamResume() : Invalid state! [%d]", iState);
    }

    RTPTRACE( _L("CFLRtpSource.RTMPStreamResume> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::Stop()
//
// Stops the pkt supply of all the streams.
// --------------------------------------------------------------------------
//

void CFLRtpSource::Stop()
{
    RTPTRACE( _L("CFLRtpSource.Stop< ++") );

    if( (ISPLAYERSTATE(EHXPPlaying) || ISPLAYERSTATE(EHXPPaused) || ISPLAYERSTATE(EHXPPlayCompleted)) &&
         (iState == EStarted || iState == EStreamEOF))
    {
        for(TInt i = 0; i < iXPSStreamList.Count(); i++) {
            iXPSStreamList[i]->Stop();
        }
        iState = EStopped;

        #ifdef _USE_SINK
        iXPSPacketSink->Reset();
        #endif

        iVideoPlayer->Stop();
        iPlayCompleteVideoPosition = 0;
        SetPlayerState(EHXPPlayCompleted);

        for(TInt i = 0; i < iObserverList.Count(); i++) {
            iObserverList[i]->MfrsoPlaybackComplete(KErrNone);
        }
    }
    else {
        RTPTRACEF("CFLRtpSource::Stop() : Invalid state! [%d]", iState);
    }

    RTPTRACE( _L("CFLRtpSource.Stop> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::Restart()
//
// Restarts the playback by reseting the pkt list.
// --------------------------------------------------------------------------
//

void CFLRtpSource::Restart()
{
    RTPTRACE( _L("CFLRtpSource.Restart< ++") );

    //we are expecting a playbackcomplete callback.
    if(iState == EStreamEOF && !ISPLAYERSTATE(EHXPPlayCompleted))
    {
        //TODO: In this case, should the playerutility be stopped and played?
        RTPTRACEF("CFLRtpSource::Restart() : Invalid state! [%d,%d]", iState, iPlayerState);
    }

    if( (ISPLAYERSTATE(EHXPPlaying)|| ISPLAYERSTATE(EHXPPaused) || ISPLAYERSTATE(EHXPPlayCompleted)) &&
        (iState == EStarted || iState == EStopped || iState == EStreamEOF))
    {
        #ifdef _USE_SINK
        iXPSPacketSink->Reset();
        #endif

		RTPTRACEF("Restarting media streams");

		for(TInt i = 0; i < iXPSStreamList.Count(); i++) {
            RTPTRACEF("CFLRtpSource::Restart() stream[%d]",i);
			iXPSStreamList[i]->Restart();
		}
		iState = EStarted;

        //TODO: Should the pending state change callback be cancelled?

		//iVideoPlayer->SetDisplayWindowL(iWsSession, *iScreenDevice, iWindow, iWindowRect, iClipRect);

		SetPlayerState(EHXPPlaying);

		//iVideoPlayer->RefreshFrameL();

        RTPTRACEF("VideoPlayerUtilty : SetPosition(0)");

        TRAPD(lErr, SetPositionL(0));
        if(lErr) {
            HandleHelixError(lErr);
        }

    }
    else {
        RTPTRACEF("CFLRtpSource::Restart() : Invalid state! [%d,%d]", iState, iPlayerState);
    }

    RTPTRACE( _L("CFLRtpSource.Restart> --") );
}

//this is called when we seek in paused state
void CFLRtpSource::RestartInPausedState()
{
    RTPTRACE( _L("CFLRtpSource.RestartInPausedState< ++") );

    //we are expecting a playbackcomplete callback.
    if(iState == EStreamEOF && !ISPLAYERSTATE(EHXPPlayCompleted))
    {
        //TODO: In this case, should the playerutility be stopped and played?
        RTPTRACEF("CFLRtpSource::Restart() : Invalid state! [%d,%d]", iState, iPlayerState);
    }

    if( (ISPLAYERSTATE(EHXPPlaying)|| ISPLAYERSTATE(EHXPPaused) || ISPLAYERSTATE(EHXPPlayCompleted)) &&
        (iState == EStarted || iState == EStopped || iState == EStreamEOF))
    {
        #ifdef _USE_SINK
        iXPSPacketSink->Reset();
        #endif

        RTPTRACEF("Restarting media streams");

        for(TInt i = 0; i < iXPSStreamList.Count(); i++) {
            RTPTRACEF("CFLRtpSource::Restart() stream[%d]",i);
            iXPSStreamList[i]->Restart();
        }
        iState = EStarted;

        //TODO: Should the pending state change callback be cancelled?

        //iVideoPlayer->SetDisplayWindowL(iWsSession, *iScreenDevice, iWindow, iWindowRect, iClipRect);

        SetPlayerState(EHXPPaused);

        //iVideoPlayer->RefreshFrameL();

        RTPTRACEF("VideoPlayerUtilty : SetPosition(0)");
        //commented because resetting position to zero is not required  when we seek in paused state.
        /*TRAPD(lErr, SetPositionL(0));
        if(lErr) {
            HandleHelixError(lErr);
        }*/

    }
    else {
        RTPTRACEF("CFLRtpSource::RestartInPausedState() : Invalid state! [%d,%d]", iState, iPlayerState);
    }

    RTPTRACE( _L("CFLRtpSource.Restart> --") );
}

// -------------------------------------------------------------------------
// void CFLRtpSource::SetPositionL(const TInt& aPosition)
//
// Sets the position of the playback. Position is in terms of time.
// --------------------------------------------------------------------------
//

void CFLRtpSource::SetPositionL(const TTimeIntervalMicroSeconds& aPosition)
{
    RTPTRACE( _L("CFLRtpSource.SetPositionL< ++ [%d]"), I64LOW(aPosition.Int64()) );

    if(iState == EStreamEOF && !ISPLAYERSTATE(EHXPPlayCompleted))
    {
        RTPTRACEF("CFLRtpSource::SetPositionL() : Invalid state! [%d,%d]", iState, iPlayerState);
    }

    if( (ISPLAYERSTATE(EHXPPlaying)|| ISPLAYERSTATE(EHXPPaused) || ISPLAYERSTATE(EHXPPlayCompleted)) &&
        (iState == EStarted || iState == EStopped || iState == EStreamEOF))
    {
        TRAPD(lErr,iVideoPlayer->SetPositionL(aPosition));
        if(lErr) {
            HandleHelixError(lErr);
        }
    }
    else {
        RTPTRACEF("CFLRtpSource::SetPositionL() : Invalid state! [%d,%d]", iState, iPlayerState);
    }

    RTPTRACE( _L("CFLRtpSource.SetPositionL> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::SetVideoWindowRect(const TRect& aWindowRect, const TRect& aClipRect)
//
// Sets the video window rect for the video player utility
// --------------------------------------------------------------------------
//

void CFLRtpSource::SetVideoWindowRectL(const TRect& aWindowRect, const TRect& aClipRect, const TBool& aSetWindow)
{
    RTPTRACE( _L("CFLRtpSource.SetVideoWindowRectL< ++") );

    iWindowRect = aWindowRect;
    iClipRect   = aClipRect;

    if(iVideoPlayer && iState != EDead && !(ISPLAYERSTATE(EHXPInitial) || ISPLAYERSTATE(EHXPOpening) || ISPLAYERSTATE(EHXPPreparing)))
	{
		if(aSetWindow)
		{
			RTPTRACEF("CFLRtpSource::SetVideoWindowRect() : WRect = (%d %d) (%d %d)", iWindowRect.iTl.iX, iWindowRect.iTl.iY, iWindowRect.iBr.iX, iWindowRect.iBr.iY);
			RTPTRACEF("CFLRtpSource::SetVideoWindowRect() : WRect Size = (%d %d)", iWindowRect.Width(), iWindowRect.Height());
			RTPTRACEF("CFLRtpSource::SetVideoWindowRect() : ClipRect = (%d %d) (%d %d)", iClipRect.iTl.iX, iClipRect.iTl.iY, iClipRect.iBr.iX, iClipRect.iBr.iY);
			RTPTRACEF("CFLRtpSource::SetVideoWindowRect() : ClipRect Size = (%d %d)", iClipRect.Width(), iClipRect.Height());
#ifndef USE_VPU2
			iVideoPlayer->SetDisplayWindowL(iWsSession, *iScreenDevice, *iWindow, iWindowRect, iClipRect);
#else
            TPoint winAbsPosition(iWindow->AbsPosition());
            
            TRect relativeWinRect(iWindowRect);
            relativeWinRect.Move(-winAbsPosition.iX,-winAbsPosition.iY);
            TRect relativeClipRect(iClipRect);
            relativeClipRect.Move(-winAbsPosition.iX,-winAbsPosition.iY);            
            
            if(iCurrentDisplayWindow != iWindow)
            {
                if(iCurrentDisplayWindow)
                    iVideoPlayer->RemoveDisplayWindow(*iCurrentDisplayWindow);
            
                TRAP_IGNORE(iVideoPlayer->AddDisplayWindowL(CCoeEnv::Static()->WsSession(),*iScreenDevice,*iWindow,relativeWinRect,relativeClipRect));
                iCurrentDisplayWindow = iWindow;
            }
            else
            {
                TRAPD(err1,iVideoPlayer->SetVideoExtentL(*iWindow,relativeWinRect));    
                RTPTRACEF("SetVideoExtent %d",err1);
                
                TRAPD(err2,iVideoPlayer->SetWindowClipRectL(*iWindow,relativeClipRect));
                RTPTRACEF("SetWindowClipRectL %d",err2);                
            }
            
            TRAP_IGNORE(iVideoPlayer->SetAutoScaleL(*iWindow, EAutoScaleBestFit));            
#endif

			iRefreshCount = KRefreshCount;

			//if(ISPLAYERSTATE(EHXPPaused))
            {
				LOG(CFLRtpSource::SetVideoWindowRect() : Refreshing the frame);
				iVideoPlayer->RefreshFrameL();
			}

			// This was required pre-9.2. This has been removed as black patches are seen in the 
			// swf area while scrolling/panning.
			//iHostEngine.DrawVideoRect();
		}
		else if(iRefreshCount)
		{
			iRefreshCount--;
			iVideoPlayer->RefreshFrameL();
		}
	}
    else {
        RTPTRACEF("CFLRtpSource::SetVideoWindowRectL() : Invalid state! [%d,%d]", iState, iPlayerState);
    }

    RTPTRACE( _L("CFLRtpSource.SetVideoWindowRectL> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::SetWindow(RWindow& aWindow)
//
// Sets the video window for the video player utility
// --------------------------------------------------------------------------
//

void CFLRtpSource::SetWindow(RWindow* aWindow)
{
    RTPTRACE( _L("CFLRtpSource.SetWindow< ++") );
    iWindow = aWindow;
    
#ifdef USE_VPU2    
    if(iWindow == NULL)
    {
        if(iCurrentDisplayWindow)
        {    
            iVideoPlayer->RemoveDisplayWindow(*iCurrentDisplayWindow);
            iCurrentDisplayWindow = NULL;
        }
    }
#endif
    
    RTPTRACE( _L("CFLRtpSource.SetWindow> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::SetFrameSize(const TSize& aFrameSize)
//
// Sets the frame size of the video.
// --------------------------------------------------------------------------
//

void CFLRtpSource::SetFrameSize(const TSize& aFrameSize)
{
    RTPTRACE( _L("CFLRtpSource.SetFrameSize< [%d,%d]++"),aFrameSize.iWidth,aFrameSize.iHeight );
    iFrameSize = aFrameSize;
    RTPTRACE( _L("CFLRtpSource.SetFrameSize> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::SetScreenMode(const TScreenMode& aScreenMode)
//
// Sets the screenmode like fullscreen, normal screen etc.
//
// --------------------------------------------------------------------------
//
void CFLRtpSource::SetScreenMode(const TScreenMode& aScreenMode)
{
    RTPTRACE( _L("CFLRtpSource.SetScreenMode< ++") );
    iScreenMode = aScreenMode;
    RTPTRACE( _L("CFLRtpSource.SetScreenMode> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::SetVolume(const TInt& aVolume)
//
// Sets the volume for video player utility.
//
// --------------------------------------------------------------------------
//

void CFLRtpSource::SetVolume(const TInt& aVolume)
{
    RTPTRACE( _L("CFLRtpSource.SetVolume< aVolume [%d]++"), aVolume );
    if(ISPLAYERSTATE(EHXPPlaying) || ISPLAYERSTATE(EHXPPaused) || ISPLAYERSTATE(EHXPPlayCompleted))
    {
        TRAP_IGNORE(iVideoPlayer->SetVolumeL(aVolume));
    }
    else {
        RTPTRACEF("CFLRtpSource::SetVolume() : Invalid state! [%d,%d]", iState, iPlayerState);
    }
    RTPTRACE( _L("CFLRtpSource.SetVolume> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::SetTotalDuration(TInt aDuration)
//
// Sets the total duration of the stream.
// --------------------------------------------------------------------------
//

void CFLRtpSource::SetTotalDuration(const TInt& aDuration)
{
    RTPTRACE( _L("CFLRtpSource.SetTotalDuration< ++") );
    iTotalDuration = aDuration;
    RTPTRACE( _L("CFLRtpSource.SetTotalDuration> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::SetUserMode(const TUserMode& aUserMode)
//
// Sets the user mode.
// --------------------------------------------------------------------------
//

void CFLRtpSource::SetUserMode(const TUserMode& aUserMode)
{
    RTPTRACE( _L("CFLRtpSource.SetUserMode< ++") );
    iUserMode = aUserMode;
    RTPTRACE( _L("CFLRtpSource.SetUserMode> --") );
}


// -------------------------------------------------------------------------
// TTimeIntervalMicroSeconds CFLRtpSource::GetVideoPosition()
//
// To get the current video position played by VideoPlayerUtility.
// --------------------------------------------------------------------------
//

TTimeIntervalMicroSeconds CFLRtpSource::GetVideoPosition()
{
    RTPTRACE( _L("CFLRtpSource.GetVideoPosition< ++") );

    TTimeIntervalMicroSeconds position = 0;

    if(ISPLAYERSTATE(EHXPPlaying) ||
       ISPLAYERSTATE(EHXPPaused))
    {
        TRAP_IGNORE(position = iVideoPlayer->PositionL());
        if(position.Int64() != 0) {
			iPlayCompleteVideoPosition = position;
		if(position.Int64() == 0)
			position = iPlayCompleteVideoPosition;
        }
    }
    else if(ISPLAYERSTATE(EHXPPlayCompleted)) {
		position = iPlayCompleteVideoPosition;
    }
    else {
        RTPTRACEF("CFLRtpSource::VideoPosition() : Invalid state!");
    }

	TInt lTime = position.Int64();

    RTPTRACE( _L("CFLRtpSource.GetVideoPosition> %d --"), lTime);
    return position;
}


// -------------------------------------------------------------------------
// void CFLRtpSource::ScreenMode() const
//
// Gets the screen mode.
// --------------------------------------------------------------------------
//

TScreenMode CFLRtpSource::ScreenMode() const
{
    RTPTRACE( _L("CFLRtpSource.ScreenMode< ++") );
    RTPTRACE( _L("CFLRtpSource.ScreenMode> --") );
    return iScreenMode;
}


// -------------------------------------------------------------------------
// TInt CFLRtpSource::MaxVolume() const
//
// Gets the maximum volume of the video player utility.
// --------------------------------------------------------------------------
//

TInt CFLRtpSource::MaxVolume() const
{
    RTPTRACE( _L("CFLRtpSource.MaxVolume< ++") );
    if(ISPLAYERSTATE(EHXPPlaying) ||
       ISPLAYERSTATE(EHXPPaused)  ||
       ISPLAYERSTATE(EHXPPlayCompleted))
    {
        RTPTRACE( _L("CFLRtpSource.MaxVolume> --") );
        return iVideoPlayer->MaxVolume();
    }

    RTPTRACE( _L("CFLRtpSource.MaxVolume> --, vol = 0") );
    return 0;
}


// -------------------------------------------------------------------------
// TInt CFLRtpSource::TotalDuration() const
//
// Gets the total duration of the video clip.
// --------------------------------------------------------------------------
//

TInt CFLRtpSource::TotalDuration() const
{
    RTPTRACE( _L("CFLRtpSource.TotalDuration< ++") );
    RTPTRACE( _L("CFLRtpSource.TotalDuration> --") );
    return iTotalDuration;
}


// -------------------------------------------------------------------------
// TUserMode CFLRtpSource::UserMode() const
//
// Gets the user mode.
// --------------------------------------------------------------------------
//

TUserMode CFLRtpSource::UserMode() const
{
    RTPTRACE( _L("CFLRtpSource.UserMode< ++") );
    RTPTRACE( _L("CFLRtpSource.UserMode> --") );
    return iUserMode;
}


// -------------------------------------------------------------------------
// TRect CFLRtpSource::GetAppRect()
//
// Gets the application rect extent.
// --------------------------------------------------------------------------
//

TRect CFLRtpSource::GetAppRect()
{
    RTPTRACE( _L("CFLRtpSource.GetAppRect< ++") );

	if(iScreenMode == ENormalScreenMode) {
		iAppRect = iApp->ClientRect();
	}
	else if(iScreenMode == EFullScreenMode) {
		iAppRect = iApp->ApplicationRect();
	}

    RTPTRACE( _L("CFLRtpSource.GetAppRect> -- tlwh=[%d,%d,%d,%d]"),
                        iAppRect.iTl.iX,iAppRect.iTl.iY,
                        iAppRect.Width(),iAppRect.Height());
	return iAppRect;
}


// -------------------------------------------------------------------------
// TBool CFLRtpSource::IsVideoStreamPresent() const
//
// Specifies whether a video stream is present or not.
// --------------------------------------------------------------------------
//

TBool CFLRtpSource::IsVideoStreamPresent() const
{
    RTPTRACE( _L("CFLRtpSource.IsVideoStreamPresent< ++") );
    RTPTRACE( _L("CFLRtpSource.IsVideoStreamPresent> -- [%d]"), iVideoStreamPresent);
    return iVideoStreamPresent;
}


// -------------------------------------------------------------------------
// TBool CFLRtpSource::IsAudioStreamPresent() const
//
// Specifies whether an audio stream is present or not.
// --------------------------------------------------------------------------
//

TBool CFLRtpSource::IsAudioStreamPresent() const
{
    RTPTRACE( _L("CFLRtpSource.IsAudioStreamPresent< ++") );
    RTPTRACE( _L("CFLRtpSource.IsAudioStreamPresent> -- [%d]"), iAudioStreamPresent);
    return iAudioStreamPresent;
}


// -------------------------------------------------------------------------
// TInt CFLRtpSource::MediaCount()
//
// Gets the total number of media used.
// --------------------------------------------------------------------------
//

TInt CFLRtpSource::MediaCount() const
{
    RTPTRACE( _L("CFLRtpSource.MediaCount< ++") );
    RTPTRACE( _L("CFLRtpSource.MediaCount> -- [%d]"), iMediaCount);
    return iMediaCount;
}


// -------------------------------------------------------------------------
// TInt CFLRtpSource::VideoFrameRateL() const
//
// Gets the FPS.
// --------------------------------------------------------------------------
//

TReal32 CFLRtpSource::VideoFrameRateL() const
{
    RTPTRACE( _L("CFLRtpSource.VideoFrameRateL< ++") );

    if(iVideoPlayer) {
        TReal32 frameRate = iVideoPlayer->VideoFrameRateL();
        RTPTRACE( _L("CFLRtpSource.VideoFrameRateL> -- [%d]"),(TInt)frameRate);
        return frameRate;
    }

    RTPTRACE( _L("CFLRtpSource.VideoFrameRateL> --, fr=-1") );
    return -1;
}


// -------------------------------------------------------------------------
// TInt CFLRtpSource::AudioSamplingRate(TInt& aSamplingRate)
//
// Returns the sampling rate of the audio stream.
// --------------------------------------------------------------------------
//

TInt CFLRtpSource::AudioSamplingRate(TInt& aSamplingRate)
{
    RTPTRACE( _L("CFLRtpSource.AudioSamplingRate< ++") );

	CXPSMediaStream* stream = NULL;
	TInt streamId = 0;
	aSamplingRate = -1;

	TInt error = GetAudioStream(streamId, stream);

	if(IsOK(error) && stream)
	{
		TAudioInfo audioInfo;
		stream->GetAudioInfo(audioInfo);
		aSamplingRate = audioInfo.iSampleRate;
	}

    RTPTRACE( _L("CFLRtpSource.AudioSamplingRate> -- [%d]"), aSamplingRate );
	return error;
}


// -------------------------------------------------------------------------
// TInt CFLRtpSource::GetPrerollBufferSizeTime()
//
// Gets the preroll buffering times based on total duration.
// --------------------------------------------------------------------------
//

TInt CFLRtpSource::GetPrerollBufferSizeTime()
{
    RTPTRACE( _L("CFLRtpSource.GetPrerollBufferSizeTime< ++") );
	TInt bufferSizeTime = 0;

	// 2 secs if time is between 1 - 30 secs.
	if(iTotalDuration > (10 * KMillisecondsForASec) &&
		iTotalDuration <= (30 * KMillisecondsForASec))
	{
		bufferSizeTime = 2 * KMillisecondsForASec;
	}
	// 3 secs if time is between 30 secs - 1 min.
	else if(iTotalDuration > (30 * KMillisecondsForASec) &&
			 iTotalDuration <= (60 * KMillisecondsForASec))
	{
		bufferSizeTime = 3 * KMillisecondsForASec;
	}
	// 4 secs if time is more than 1 min.
	else if(iTotalDuration > (60 * KMillisecondsForASec))
	{
		bufferSizeTime = 4 * KMillisecondsForASec;
	}

    RTPTRACE( _L("CFLRtpSource.GetPrerollBufferSizeTime> -- [%d]"), bufferSizeTime );
	return bufferSizeTime;
}


// -------------------------------------------------------------------------
// CPlatformFlashEngine& CFLRtpSource::HostEngine() const
//
// Gets the Host engine instance reference.
// --------------------------------------------------------------------------
//

CPlatformFlashEngine& CFLRtpSource::HostEngine() const
{
    RTPTRACE( _L("CFLRtpSource.HostEngine< ++") );
    RTPTRACE( _L("CFLRtpSource.HostEngine> --") );
    return iHostEngine;
}


// -------------------------------------------------------------------------
// TPlayerState CFLRtpSource::State() const
//
// Gets the current state of the video player.
// --------------------------------------------------------------------------
//

TPlayerState CFLRtpSource::PlayerState() const
{
    RTPTRACE( _L("CFLRtpSource.PlayerState< ++") );
    RTPTRACE( _L("CFLRtpSource.PlayerState> -- [%d]"), iPlayerState );
    return iPlayerState;
}


// -------------------------------------------------------------------------
// TRtpSrcState CFLRtpSource::RtpSrcState() const
//
//
// --------------------------------------------------------------------------
//

TRtpSrcState CFLRtpSource::RtpSrcState() const
{
    RTPTRACE( _L("CFLRtpSource.RtpSrcState< ++") );
    RTPTRACE( _L("CFLRtpSource.RtpSrcState> -- [%d]"), iState );
    return iState;
}


// -------------------------------------------------------------------------
// void CFLRtpSource::HandleHelixError(const TInt& aError)
//
// Error recovery handler.
// --------------------------------------------------------------------------
//

void CFLRtpSource::HandleHelixError(const TInt& aError)
{
    RTPTRACE( _L("CFLRtpSource.HandleHelixError< ++ [%d]"), aError );

	if(IsOK(aError) || ISPLAYERSTATE(EHXPErrorOccured))
	{
        RTPTRACE( _L("CFLRtpSource.HandleHelixError> --, dupe error") );
		return;
	}

	// Stop all the XPS streams as some error occurred.
	if(ISPLAYERSTATE(EHXPPlaying) || ISPLAYERSTATE(EHXPPaused))	{
		Stop();
	}

	// Set the error state.
	SetPlayerState(EHXPErrorOccured);
    iState = EDead;

	// Notify all the observers to handle the error situation.
	for(TInt i = 0; i < iObserverList.Count(); i++)	{
		iObserverList[i]->MfrsoHandleError(aError);
	}

    RTPTRACE( _L("CFLRtpSource.HandleHelixError> --") );
}

// -------------------------------------------------------------------------
// void CFLRtpSource::HandlePrepareCompleteError(const TInt& aError)
//
// Preparecomplete Error recovery handler.
// --------------------------------------------------------------------------
//

void CFLRtpSource::HandlePrepareCompleteError(const TInt& aError)
{
    RTPTRACE( _L("CFLRtpSource.HandlePrepareCompleteError< ++ "));

	//SetPlayerState(EHXPPreparingFailed);

	// Stop all the XPS streams as some error occurred.
//	Stop();
        for(TInt i = 0; i < iXPSStreamList.Count(); i++) {
            iXPSStreamList[i]->Stop();
        }
        iState = EStopped;

	iVideoPlayer->Close();

	// Set the error state.
	SetPlayerState(EHXPErrorOccured);
       iState = EDead;

	// Notify all the observers to handle the error situation.
	for(TInt i = 0; i < iObserverList.Count(); i++)	{
		iObserverList[i]->MfrsoHandleError(aError);
	}

	RTPTRACE( _L("CFLRtpSource.HandlePrepareCompleteError> --") );
}

// -------------------------------------------------------------------------
// void CFLRtpSource::SetAudioInfo(TInt aStreamId, const TAudioInfo& aAudioInfo)
//
// Gets the audio related info of the audio stream.
// --------------------------------------------------------------------------
//

void CFLRtpSource::SetAudioInfo(const TInt& aStreamId, const TAudioInfo& aAudioInfo)
{
    RTPTRACE( _L("CFLRtpSource.SetAudioInfo< ++") );

	CXPSMediaStream* stream;
	TInt error = GetStreamInfo(aStreamId, stream);

	if(stream && IsOK(error))
	{
        stream->SetAudioInfo(aAudioInfo);
    }

    RTPTRACE( _L("CFLRtpSource.SetAudioInfo> --") );
}

// -------------------------------------------------------------------------
// void CFLRtpSource::SetAvcInfo(TInt aStreamId, const TAudioInfo& aAudioInfo)
//
// Sets the profile and level info for an avc video stream.
// --------------------------------------------------------------------------
//

void CFLRtpSource::SetAvcInfo(const TInt& aStreamId, const TAvcStreamInfo& aAvcInfo)
{
    RTPTRACE( _L("CFLRtpSource.SetAvcInfo< ++") );

	CXPSMediaStream* stream;
	TInt error = GetStreamInfo(aStreamId, stream);
	if(IsOK(error)) {
		stream->SetAvcStreamInfo(aAvcInfo);
    }

    RTPTRACE( _L("CFLRtpSource.SetAvcInfo> --") );
}


// -------------------------------------------------------------------------
// TInt CFLRtpSource::SetStreamConfig(const TInt& aStreamId, const TStreamConfigData& aStreamConfig)
//
//
// --------------------------------------------------------------------------
//

TInt CFLRtpSource::SetStreamConfig(const TInt& aStreamId, const TStreamConfigData& aStreamConfig)
{
    RTPTRACE( _L("CFLRtpSource.SetStreamConfig< ++") );

	CXPSMediaStream* stream;
	TInt error = KErrGeneral;

    error = GetStreamInfo(aStreamId, stream);
	if(IsOK(error))
	{
		error = stream->SetStreamConfig(aStreamConfig);
    }

    RTPTRACE( _L("CFLRtpSource.SetStreamConfig> --") );
    return error;
}


// -------------------------------------------------------------------------
// void CFLRtpSource::RestorePacketSupply(TUint aStreamId)
//
// Notifies to restore the pkt supply. This is issued from XPS to pull the pkts.
// --------------------------------------------------------------------------
//

void CFLRtpSource::RestorePacketSupply(TUint aStreamId)
{
    RTPTRACE( _L("CFLRtpSource.RestorePacketSupply< ++ stream[%d]"), aStreamId );

    if(iState == EStarted || iState == EStreamEOF)
    {
	    CXPSMediaStream* stream;
	    TInt error = GetStreamInfo(aStreamId, stream);
	    if(IsOK(error))
	    {
    	    stream->RestorePacketSupply();
        }
    }

    RTPTRACE( _L("CFLRtpSource.RestorePacketSupply> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::MvpuoOpenComplete(TInt aError)
//
// Callback from VideoPlayerUtility when open is completed.
// --------------------------------------------------------------------------
//

void CFLRtpSource::MvpuoOpenComplete(TInt aError)
{
    RTPTRACE( _L("CFLRtpSource.MvpuoOpenComplete< [%d] ++"), aError );

	if(!IsOK(aError) || !ISPLAYERSTATE(EHXPOpening)) {
		HandleHelixError(aError);
		RTPTRACE(_L("CFLRtpSource.MvpuoOpenComplete> --, Invalid state"));
		return;
	}

    iVideoPlayer->Prepare();
	SetPlayerState(EHXPPreparing);

	RTPTRACE( _L("CFLRtpSource.MvpuoOpenComplete> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::MvpuoPrepareComplete(TInt aError)
//
// Callback from VideoPlayerUtility when Prepare is completed.
// --------------------------------------------------------------------------
//

void CFLRtpSource::MvpuoPrepareComplete(TInt aError)
{
    RTPTRACE( _L("CFLRtpSource.MvpuoPrepareComplete< [%d] ++"), aError );

	if((!IsOK(aError) && aError != KPartialPlaybackError) || !ISPLAYERSTATE(EHXPPreparing)) {
		HandleHelixError(aError);
		RTPTRACE( _L("CFLRtpSource.MvpuoPrepareComplete> --, Invalid state") );
		return;
	}

	if(iWindow == NULL)
	{
        HandleHelixError(KErrGeneral);
        RTPTRACE( _L("CFLRtpSource.MvpuoPrepareComplete> --, No RWindow") );
        return;
    }
	if(aError == KPartialPlaybackError && ISPLAYERSTATE(EHXPPreparing))
	{
		TInt mediaCount = MediaCount();

		for(TInt index = 0; index < mediaCount; index++)
		{
			if(iCodecTypeList[index] == EH264Codec)
			{
				HandlePrepareCompleteError(aError);
				// Stop all the XPS streams as some error occurred.
				Stop();

				// Set the error state.
				SetPlayerState(EHXPErrorOccured);
			    iState = EDead;

				// Notify all the observers to handle the error situation.
				for(TInt i = 0; i < iObserverList.Count(); i++)
				{
					iObserverList[i]->MfrsoHandleError(aError);
				}
                    RTPTRACE( _L("CFLRtpSource.MvpuoPrepareComplete> --, PrepareComplete Partial playback") );
		        	return;
			}
		}
	}

#ifndef USE_VPU2	
	TRAP_IGNORE(iVideoPlayer->SetDisplayWindowL(CCoeEnv::Static()->WsSession(),
										            *iScreenDevice,
										            *iWindow,
										            iWindowRect,
										            iClipRect));
#else
    TPoint winAbsPosition(iWindow->AbsPosition());
	
	TRect relativeWinRect(iWindowRect);
	relativeWinRect.Move(-winAbsPosition.iX,-winAbsPosition.iY);
    TRect relativeClipRect(iClipRect);
    relativeClipRect.Move(-winAbsPosition.iX,-winAbsPosition.iY);
	
    TRAP_IGNORE(iVideoPlayer->AddDisplayWindowL(CCoeEnv::Static()->WsSession(),
                                                    *iScreenDevice,
                                                    *iWindow,
                                                    relativeWinRect,
                                                    relativeClipRect));
    iCurrentDisplayWindow = iWindow;
    TRAP_IGNORE(iVideoPlayer->SetAutoScaleL(*iWindow, EAutoScaleBestFit));
#endif

	iVideoPlayer->Play();
	SetPlayerState(EHXPPlaying);

    // Now all the stream handlers are ready to start
    for(TInt i = 0; i < iInitCount; i++)
    {
    	if(i < iObserverList.Count()) {
    		iObserverList[i]->MfrsoReadyToStart(KErrNone);
    	}
    }

    RTPTRACE( _L("CFLRtpSource.MvpuoPrepareComplete> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::MvpuoFrameReady(CFbsBitmap& aFrame,TInt aError)
//
// Callback from VideoPlayerUtility when Frame is ready.
// --------------------------------------------------------------------------
//

void CFLRtpSource::MvpuoFrameReady(CFbsBitmap& /*aFrame*/, TInt /*aError*/)
{
    RTPTRACE( _L("CFLRtpSource.MvpuoFrameReady< ++") );
	// Not handled.
    RTPTRACE( _L("CFLRtpSource.MvpuoFrameReady> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::MvpuoPlayComplete(TInt aError)
//
// Callback from VideoPlayerUtility when Play is completed.
// --------------------------------------------------------------------------
//

void CFLRtpSource::MvpuoPlayComplete(TInt aError)
{
    RTPTRACE( _L("CFLRtpSource.MvpuoPlayComplete< [%d] ++"), aError );

    if(!IsOK(aError)) {
		HandleHelixError(aError);
		RTPTRACE( _L("CFLRtpSource.MvpuoPlayComplete> --, error") );
		return;
	}

	if(ISPLAYERSTATE(EHXPPlayCompleted)) {
        User::Panic( _L("CFLRtpSource.MvpuoPlayComplete> --, duplicate cbk error"), KErrGeneral );
	}

	SetPlayerState(EHXPPlayCompleted);

	for(TInt i = 0; i < iXPSStreamList.Count(); i++) {
		iXPSStreamList[i]->Stop();
	}
    iState = EStopped;

	#ifdef _USE_SINK
	iXPSPacketSink->Reset();
	#endif

	for(TInt i = 0; i < iObserverList.Count(); i++)	{
		iObserverList[i]->MfrsoPlaybackComplete(aError);
	}

	RTPTRACE( _L("CFLRtpSource.MvpuoPlayComplete> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::MvpuoEvent(const TMMFEvent& aEvent)
//
// Callback from VideoPlayerUtility when an Event occurs.
// --------------------------------------------------------------------------
//

void CFLRtpSource::MvpuoEvent(const TMMFEvent& aEvent)
{
    RTPTRACE( _L("CFLRtpSource.MvpuoEvent< ++") );
    /* Fix for ou1cimx1#422308: On removing the headset 
	 helix is throwing an error KErrMMAudioDevice(-12014) 
	 to host which needs to be handled at host side*/
    if(aEvent.iErrorCode == KErrMMAudioDevice )
    {
    	Pause();
    	Resume();
    }
    RTPTRACE( _L("CFLRtpSource.MvpuoEvent> --") );
}


//====

#ifdef USE_VPU2
void CFLRtpSource::MmsehSurfaceCreated(TInt /*aDisplayId*/, const TSurfaceId& /*aId*/, const TRect& /*aCropRect*/, TVideoAspectRatio /*aAspectRatio*/)
{

}

void CFLRtpSource::MmsehSurfaceParametersChanged(const TSurfaceId& /*aId*/, const TRect& /*aCropRect*/, TVideoAspectRatio /*aAspectRatio*/)
{

}

void CFLRtpSource::MmsehRemoveSurface(const TSurfaceId& /*aId*/)
{
    
}
#endif



//=====

// -------------------------------------------------------------------------
//
//
//
// --------------------------------------------------------------------------
//

void CFLRtpSource::SetPlayerState(TPlayerState aState)
{
    RTPTRACE( _L("CFLRtpSource.SetPlayerState< ++") );

    iPlayerState = aState;

    //There is a separate callback for playback complete.
    if(aState != EHXPPlayCompleted)
    {
        // Notify all the observers
        for(TInt i = 0; i < iObserverList.Count(); i++)
        {
            TInt* lArgs = new TInt[3];
		    lArgs[0] = (TInt)this;
		    lArgs[1] = (TInt)iObserverList[i];
		    lArgs[2] = iPlayerState;

		    if(lArgs) {
			    iCallbackScheduler->AppendEvent(TCallBack((CFLRtpSource::IssueCallback),(TAny*)lArgs));
		    }
		    else {//fatal
			    HandleHelixError(KErrNoMemory);
		    }

		    iArgList.Append(lArgs);
        }
    }
    RTPTRACE( _L("CFLRtpSource.SetPlayerState> --") );
}


// -------------------------------------------------------------------------
//
//
//
// --------------------------------------------------------------------------
//

TInt CFLRtpSource::IssueCallback(TAny* aArgs)
{
	TInt* lArgs = (TInt*)aArgs;

	if(aArgs == NULL)
		User::Panic(_L("Callback scheduler fatal error"),KErrArgument);

	if(((CFLRtpSource*)(lArgs[0]))->iArgList.Count() == 0)
		User::Panic(_L("CFLRtpSource fatal error"),KErrGeneral);

	if(((CFLRtpSource*)(lArgs[0]))->iArgList[0] != aArgs)
		User::Panic(_L("Callback scheduler fatal error"),KErrArgument);

	MFlRtpSourceObserver* lFlRtpSourceObserver = (MFlRtpSourceObserver*)lArgs[1];
	TPlayerState lState = (TPlayerState)lArgs[2];

	((CFLRtpSource*)(lArgs[0]))->iArgList.Remove(0);
	delete lArgs; //Callbacks are sequential. So first in arg is first out.

    lFlRtpSourceObserver->MfrsoPlayerStateChanged(lState);
	return KErrNone;
}


// -------------------------------------------------------------------------
// TInt CFLRtpSource::GetStreamInfo(TInt aStreamId, CXPSMediaStream*& aStreamInfo)
//
// Gets the info about the XPS stream.
// --------------------------------------------------------------------------
//

TInt CFLRtpSource::GetStreamInfo(const TInt& aStreamId, CXPSMediaStream*& aStreamInfo)
{
    RTPTRACE( _L("CFLRtpSource.GetStreamInfo< ++") );
	for(TInt i = 0; i < iXPSStreamList.Count(); i++)
	{
		CXPSMediaStream* streamInfo = iXPSStreamList[i];

		if(streamInfo && streamInfo->StreamId() == aStreamId)
		{
			aStreamInfo = streamInfo;
            RTPTRACE( _L("CFLRtpSource.GetStreamInfo> --") );
			return KErrNone;
		}
	}

    RTPTRACE( _L("CFLRtpSource.GetStreamInfo> --, not found") );
	return KErrNotFound;
}


// -------------------------------------------------------------------------
// TInt CFLRtpSource::GetAudioStream(TInt& aStreamId, CXPSMediaStream*& aStreamInfo)
//
// Gets the audio stream object.
// --------------------------------------------------------------------------
//

TInt CFLRtpSource::GetAudioStream(TInt& aStreamId, CXPSMediaStream*& aStreamInfo)
{
    RTPTRACE( _L("CFLRtpSource.GetAudioStream< ++") );
	aStreamInfo = NULL;
	aStreamId   = -1;

	for(TInt id = 0; id < iXPSStreamList.Count(); id++)
	{
		if(iXPSStreamList[id]->StreamType() == EXPSAudioStream)
		{
			aStreamInfo = iXPSStreamList[id];
			aStreamId   = id;
            RTPTRACE( _L("CFLRtpSource.GetAudioStream> --") );
			return KErrNone;
		}
	}

    RTPTRACE( _L("CFLRtpSource.GetAudioStream> --, not found") );
	return KErrNotFound;
}


// -------------------------------------------------------------------------
// TInt CFLRtpSource::GetVideoStream(TInt& aStreamId, CXPSMediaStream*& aStreamInfo)
//
// Gets the audio stream object.
// --------------------------------------------------------------------------
//

TInt CFLRtpSource::GetVideoStream(TInt& aStreamId, CXPSMediaStream*& aStreamInfo)
{
    RTPTRACE( _L("CFLRtpSource.GetVideoStream< ++") );
	aStreamInfo = NULL;
	aStreamId   = -1;

	for(TInt id = 0; id < iXPSStreamList.Count(); id++)
	{
		if(iXPSStreamList[id]->StreamType() == EXPSVideoStream)
		{
			aStreamInfo = iXPSStreamList[id];
			aStreamId   = id;
            RTPTRACE( _L("CFLRtpSource.GetVideoStream> --") );
			return KErrNone;
		}
	}

    RTPTRACE( _L("CFLRtpSource.GetVideoStream> --, not found") );
	return KErrNotFound;
}


// -------------------------------------------------------------------------
// void CFLRtpSource::ConstructSDPInfoL()
//
// Constructs the SDP description string used to init the XPS session.
// --------------------------------------------------------------------------
//

void CFLRtpSource::ConstructSDPInfoL()
{
    RTPTRACE( _L("CFLRtpSource.ConstructSDPInfoL< ++") );
    TInt duration = (iTotalDuration / KMillisecondsForASec) + KAuxiliaryDuration;

    iSDPInfo = HBufC8::NewL(KMaxSDPLength);

    TPtr8 sdpInfoPtr = (TPtr8) iSDPInfo->Des();

    sdpInfoPtr.Copy(KBasicSDPInfo);

    sdpInfoPtr.AppendFormat(_L8("a=range:npt=0-%d\r\n"), duration);

    TInt i = 0;

    for(; i < MediaCount(); i++)
    {
        TCodecType codecType = iCodecTypeList[i];

        switch(codecType)
        {
            case EMp3Codec:
            {
                sdpInfoPtr = (TPtr8) iSDPInfo->Des();

                sdpInfoPtr.AppendFormat(KFlvMp3SDPInfo);

                TAudioInfo info;
                if(i < iXPSStreamList.Count())
                {
                    iXPSStreamList[i]->GetAudioInfo(info);
                }

                sdpInfoPtr.AppendFormat(_L8("%d/%d\r\na=fmtp:97 octet-align=1\r\n"), info.iSampleRate, info.iChannels);
                TInt time = GetPrerollBufferSizeTime();
                sdpInfoPtr.AppendFormat(_L8("a=preroll:integer;%d\r\n"), time);

                break;
            }
            case EAacCodec:
            {
                ConstructAACSDP();
                break;
            }

            case EH264Codec:
            {
                ConstructH264SDP();
                break;
            }

            default:
            case ESorensonCodec:
            case EOn2Codec:
            {
                ConstructVideoSDP(codecType);
                break;
            }
        }
    }


    #ifdef _DUMP_SDPINFO_TO_FILE_
    RFs fs;
    if(fs.Connect() != KErrNone)
    {
        RTPTRACE( _L("CFLRtpSource.ConstructSDPInfoL> --") );
        return;
    }

    TFileName fileName;
    fileName.Copy(_L("C:\\Logs\\HelixLogs\\sdpdump.txt"));

    RFile dumpFile;
    TInt error = dumpFile.Open(fs, fileName, EFileWrite);
    if(error == KErrNotFound)
    {
        error = dumpFile.Create(fs, fileName, EFileWrite);
    }
    if(IsOK(error))
    {
        TInt pos = 0;
        dumpFile.Seek(ESeekEnd, pos);
        dumpFile.Write(iSDPInfo->Des());
    }
    dumpFile.Close();
    fs.Close();
    #endif //_DUMP_SDPINFO_TO_FILE


    #ifdef _USE_SINK
    User::LeaveIfError(iXPSPacketSink->SetSessionDescription(*iSDPInfo, i));
    #endif
    RTPTRACE( _L("CFLRtpSource.ConstructSDPInfoL> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::ConstructVideoSDP(const TCodecType& aCodecType)
//
// COnstructs the SDP description substring for the video part.
// --------------------------------------------------------------------------
//

void CFLRtpSource::ConstructVideoSDP(const TCodecType& aCodecType)
{
    RTPTRACE( _L("CFLRtpSource.ConstructVideoSDP< ++") );
    TPtr8 sdpInfoPtr = (TPtr8) iSDPInfo->Des();
    sdpInfoPtr.AppendFormat(KFlvVideoSDPInfo);

    TInt sampleRate = KNormalVideoClockRate;

    if(iAudioStreamPresent)
    {
        TAudioInfo info;
        TInt streamId = -1;
        CXPSMediaStream* stream = NULL;

        TInt error = GetAudioStream(streamId, stream);

        if(IsOK(error) && stream)
        {
            stream->GetAudioInfo(info);
            sampleRate = info.iSampleRate;
        }
    }

    if(aCodecType == EOn2Codec)
    {
        sdpInfoPtr.AppendFormat(_L8("a=rtpmap:96 x-vp6/%d\r\n"), sampleRate);
    }
    else if(aCodecType == ESorensonCodec)
    {
        sdpInfoPtr.AppendFormat(_L8("a=rtpmap:96 x-sorenson-spark/%d\r\n"), sampleRate);
    }

    TInt time = GetPrerollBufferSizeTime();

    sdpInfoPtr.AppendFormat(_L8("a=framesize:96 %d-%d\r\n"), iFrameSize.iWidth, iFrameSize.iHeight);
    sdpInfoPtr.AppendFormat(_L8("a=preroll:integer;%d\r\n"), time);
    RTPTRACE( _L("CFLRtpSource.ConstructVideoSDP> --") );
}

void CFLRtpSource::ConstructH264SDP()
{
    RTPTRACE( _L("CFLRtpSource.ConstructH264SDP< ++") );
    TPtr8 sdpInfoPtr = (TPtr8) iSDPInfo->Des();
    sdpInfoPtr.AppendFormat(KH264VideoSDPInfo);

    TInt sampleRate = KNormalVideoClockRate;

    if(iAudioStreamPresent)
    {
        TAudioInfo info;
        TInt streamId = -1;
        CXPSMediaStream* stream = NULL;

        TInt error = GetAudioStream(streamId, stream);

        if(IsOK(error) && stream)
        {
            stream->GetAudioInfo(info);
            sampleRate = info.iSampleRate;
        }
    }

    sdpInfoPtr.AppendFormat(_L8("a=rtpmap:96 H264/%d\r\n"), sampleRate);

    TInt videoStreamId = -1;
    CXPSMediaStream* videoStream = NULL;
    TAvcStreamInfo videoInfo;
    TUint8* cfgData = NULL;

    TInt error = GetVideoStream(videoStreamId, videoStream);
    if(!IsOK(error) || !videoStream)
    {
        return;
    }
    videoStream->GetAvcStreamInfo(videoInfo);
    cfgData = videoStream->GetStreamConfig();

    TInt profileLevel = 0;

    profileLevel = (((videoInfo.iProfileInfo    << 16) & 0x00FF0000) |
                    ((videoInfo.iConstraintInfo << 8)  & 0x0000FF00) |
                    (videoInfo.iLevelInfo & 0x000000FF));

	if(cfgData)
	{
	    sdpInfoPtr.AppendFormat(_L8("a=fmtp:96 profile-level-id=%x;sprop-parameter-sets=%s;packetization-mode=0\r\n"),profileLevel,cfgData);
	}

    sdpInfoPtr.AppendFormat(KXpsHdrPyldSize);
    TInt time = GetPrerollBufferSizeTime();
    sdpInfoPtr.AppendFormat(_L8("a=preroll:integer;5000\r\n"));
    RTPTRACE( _L("CFLRtpSource.ConstructH264SDP> --") );
}

void CFLRtpSource::ConstructAACSDP()
{
    RTPTRACE( _L("CFLRtpSource.ConstructAACSDP< ++") );
    TPtr8 sdpInfoPtr = (TPtr8) iSDPInfo->Des();
    sdpInfoPtr.AppendFormat(KAacAudioSDPInfo);

//  TInt sampleRate = 48000;
    TInt sampleRate = KNormalVideoClockRate;
    TUint8* cfgData = NULL;

    if(iAudioStreamPresent)
    {
        TAudioInfo info;
        TInt streamId = -1;
        CXPSMediaStream* stream = NULL;

        TInt error = GetAudioStream(streamId, stream);

        if(IsOK(error) && stream)
        {
            stream->GetAudioInfo(info);
            sampleRate = info.iSampleRate;
            cfgData = stream->GetStreamConfig();
        }
    }



    sdpInfoPtr.AppendFormat(_L8("a=control:streamid=65637\r\n"), sampleRate);
    sdpInfoPtr.AppendFormat(_L8("a=rtpmap:97 AAC/%d\r\n"), sampleRate);
    sdpInfoPtr.AppendFormat(_L8("a=fmtp:97\r\n"), sampleRate);

    if(cfgData)
    {
		sdpInfoPtr.AppendFormat(_L8("a=opaquedata:buffer;\"%s\"\r\n"), cfgData);
    }

    sdpInfoPtr.AppendFormat(_L8("a=mimetype:string;\"audio/X-HX-AAC-GENERIC\"\r\n"));

    sdpInfoPtr.AppendFormat(_L8("a=Preroll:integer;5000\r\n"));

    sdpInfoPtr.AppendFormat(_L8("a=ASMRuleBook:string;\"Marker=0,AverageBandwidth=8092,TimeStampDelivery=TRUE;Marker=1,AverageBandwidth=0,TimeStampDelivery=TRUE;\"\r\n"));
    RTPTRACE( _L("CFLRtpSource.ConstructAACSDP> --") );
}


// -------------------------------------------------------------------------
// void CFLRtpSource::InitCompleteL()
//
// Used to notify the completion of the init to the RTP src obeserver.
// --------------------------------------------------------------------------
//

void CFLRtpSource::InitCompleteL()
{
    RTPTRACE( _L("CFLRtpSource.InitCompleteL< ++") );

    TBuf<KMaxFileName> xpsUrl;
	xpsUrl.Append(KXPSUrl);
	xpsUrl.Append(iXpsServerName);

    TRAPD(error, iVideoPlayer->OpenUrlL(xpsUrl));
    if(!IsOK(error))
    {
        iState = EDead;
        User::Leave(error);
        RTPTRACE( _L("CFLRtpSource.InitCompleteL> --, openurl [%d]"), error );
    	return;
    }

    SetPlayerState(EHXPOpening);

    RTPTRACE( _L("CFLRtpSource.InitCompleteL> --") );
}




