/*
* Copyright (c) 2008 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This software, including documentation is protected by copyright controlled
* by Nokia Corporation. Copying, including reproducing, storing, adapting or
* translating, any or all of this material can only be done in accordance
* with the Nokia Symbian License version 1.0 (or any subsequent update) or
* any other license terms expressly agreed between you and Nokia.
* This material contains Nokia's confidential information which
* may not be disclosed to others without the prior written consent of Nokia.
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  Implements the member functions of the class  XPSMediaStream
*
*/



#include "xpsmediastream.h"
#include "flrtpsource.h"

//#define __DUMP_TO_FILE

#ifdef __DUMP_TO_FILE
#include <f32file.h>
#endif

#include <f32file.h>
#include <bautils.h>

extern void DebugLog(TRefByValue<const TDesC> aText, ...);


#define XPSSTREAMTRACE(args...) if(iDoLog) { RDebug::Print(##args); }
#define XPSSTREAMTRACEF(args...) if(iDoLog) { RDebug::Printf(##args); }


#define ISAUDIOSTREAM() (iStreamType == EXPSAudioStream)
#define ISVIDEOSTREAM() (iStreamType == EXPSVideoStream)



// -------------------------------------------------------------------------
// CXPSMediaStream* CXPSMediaStream::NewLC(const TInt& aStreamId, 
//										   CXPSPacketSink& aSink, 
//										   CFLRtpSource& aStreamHandler)
// 
// static constructor NewLC function for CXPSMediaStream class 
// --------------------------------------------------------------------------
// 
CXPSMediaStream* CXPSMediaStream::NewLC(const TInt&     aStreamId, 
										CXPSPacketSink& aSink, 
										CFLRtpSource&   aStreamHandler)
{
	CXPSMediaStream* stream = new (ELeave) CXPSMediaStream(aStreamId, aSink, aStreamHandler);
	CleanupStack::PushL(stream);
	stream->ConstructL();
	return stream;
}


// -------------------------------------------------------------------------
// CXPSMediaStream::~CXPSMediaStream()
// 
// Destructor for CXPSMediaStream. 
// --------------------------------------------------------------------------
// 
CXPSMediaStream::~CXPSMediaStream()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.~< ++") );
    Stop();

	iPacketArray.ResetAndDestroy();
	iPacketArray.Close();
	
	delete iStreamConfigData.iConfigData;	
    XPSSTREAMTRACE( _L("CXPSMediaStream.~> --") );
}


// -------------------------------------------------------------------------
// CXPSMediaStream::ConstructL()
// 
// 2nd phase constructor for CXPSMediaStream. 
// --------------------------------------------------------------------------
// 
void CXPSMediaStream::ConstructL()
{
   XPSSTREAMTRACE( _L("CXPSMediaStream.ConstructL< ++") );

	RFs lFs(CCoeEnv::Static()->FsSession());
	TFileName lLogFlagFileName(_L("C:\\data\\xpslogs\\xpsmediastream.txt"));
    if(BaflUtils::FileExists(lFs, lLogFlagFileName))
	{
		iDoLog = ETrue;
	}
	else
	{
		iDoLog = EFalse;
	}

   XPSSTREAMTRACE( _L("CXPSMediaStream.ConstructL< --") );
}


// -------------------------------------------------------------------------
// CXPSMediaStream::CXPSMediaStream(const TInt& aStreamId, 
//								 CXPSPacketSink& aSink, 
//								 CFLRtpSource& aStreamHandler)
// 
// Constructor for CXPSMediaStream. 
// --------------------------------------------------------------------------
// 
CXPSMediaStream::CXPSMediaStream(const TInt&     aStreamId, 
								 CXPSPacketSink& aSink, 
								 CFLRtpSource&  aStreamHandler)
	: iStreamId(aStreamId),
	  iSeqNumber(0),
	  iSink(aSink),
	  iStreamHandler(aStreamHandler)
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.CXPSMediaStream< ++") );
	iPacketArray.Reset();
	iPrevTS    = 0;
	iSumOfDiff = 0;
	iTimeDiff  = 0;
	iTimestamp = 0;
	
	iStreamConfigData.iConfigData = 0;
	iStreamConfigData.iConfigLength = 0;	

    iStreamState = EStreamInitial;
    iXpsState    = EXpsReady;
    
    iStreamEndSignalled = EFalse;
    XPSSTREAMTRACE( _L("CXPSMediaStream.CXPSMediaStream> --") );
}


// -------------------------------------------------------------------------
// void CXPSMediaStream::Start()
// 
// Starts the packet supply. This means that the stream is ready to push the packets. 
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::Start()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.Start< ++") );
    if(iStreamState == EInitial) {
        iStreamState = EStreamStarted;
    }

    if(iStreamState == EStreamStarted && 
          iXpsState == EXpsReady) {
    	PushPacket();
    }
    else {
        XPSSTREAMTRACE( _L("CXPSMediaStream.Start, Invalid state!") );
    }
    XPSSTREAMTRACE( _L("CXPSMediaStream.Start> --") );
}

// -------------------------------------------------------------------------
// void CXPSMediaStream::Pause()
// 
// Pause the packet supply to XPS.  
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::Pause()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.Pause< ++") );
    //EStreamStopped is added to handle ASPause when EOF is reached
    if(iStreamState == EStreamStarted || iStreamState == EStreamStopped) {
        iStreamState = EStreamPaused;	
    }
    else {
        XPSSTREAMTRACE( _L("CXPSMediaStream.Pause, Invalid state!") );
    }

    XPSSTREAMTRACE( _L("CXPSMediaStream.Pause> --") );
}

// -------------------------------------------------------------------------
// void CXPSMediaStream::Resume()
// 
// Resumes the packet supply to XPS.  
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::Resume()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.Resume< ++") );
	if(iStreamState == EStreamPaused) 
    {
	    iStreamState = EStreamStarted;

        if(iXpsState == EXpsReady)
            PushPacket();
	}
    else {
        XPSSTREAMTRACE( _L("CXPSMediaStream.Resume, Invalid state!") );
    }
 
    XPSSTREAMTRACE( _L("CXPSMediaStream.Resume> --") );
}


// -------------------------------------------------------------------------
// void CXPSMediaStream::Reset()
// 
// Empties the stream's Packet array. Also deletes the individual packets.  
// --------------------------------------------------------------------------
// 
void CXPSMediaStream::Reset()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.Reset< ++") );

    if(iPacketArray.Count()) {
        XPSSTREAMTRACEF("CXPSMediaStream::Reset(); pkts=%d",iPacketArray.Count());
    }

    if(iStreamState != EStreamStopped)
    {
        while(iPacketArray.Count() > 0) 
	    {		
		    CRTPPacket* packet = iPacketArray[0];
		    iPacketArray.Remove(0);
		    delete packet;
	    }

		iXpsState = EXpsReady;
    }
    else if(iPacketArray.Count()) {
        //if stopped, stream must not have any media packets
        User::Panic(_L("CXPSMediaStream::Reset(); Invalid state"),KErrGeneral);
    }
    
    XPSSTREAMTRACE( _L("CXPSMediaStream.Reset> --") );
}


// -------------------------------------------------------------------------
// void CXPSMediaStream::Stop()
// 
// Stops the packet supply to XPS. 
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::Stop()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.Stop< ++") );

	if(iStreamState == EStreamStarted ||
       iStreamState == EStreamPaused)
	{
        //loose all media packets
        Reset(); 

        //signal streamend to xps (will automatically move to stopped state, 
        //as there are no more packets
        StreamEnd(); 
        
        iStreamState = EStreamStopped; 
    }		
    else {
        XPSSTREAMTRACE( _L("CXPSMediaStream.Stop, Invalid state!") );
    }

    XPSSTREAMTRACE( _L("CXPSMediaStream.Stop> --") );
}


// -------------------------------------------------------------------------
// TStreamState CXPSMediaStream::State()
// 
// Stops the packet supply to XPS. 
// --------------------------------------------------------------------------
// 

CXPSMediaStream::TStreamState CXPSMediaStream::State()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.State< ++") );
    XPSSTREAMTRACE( _L("CXPSMediaStream.State> --") );
    return iStreamState;
}



// -------------------------------------------------------------------------
// TInt CXPSMediaStream::AddPacket(const TStreamPacketData& aPktData)
// 
// Adds the packet into the packet list. 
// It is a non-leaving version and returns the error in case of failure. 
// --------------------------------------------------------------------------
// 

TInt CXPSMediaStream::AddPacket(const TStreamPacketData& aPktData)
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.AddPacket< ++") );

    if(iXpsState == EXpsStreamEnd || iStreamState == EStreamStopped)
        User::Panic(_L("Cant push packets after stream end"),KErrOverflow);

    XPSSTREAMTRACEF("Add packet to stream[%d] type[%d] ts=[%d]",iStreamId,iStreamType,aPktData.iTimeStamp);    
    
    if( iXpsState == EXpsOverflow )
    {    
        return KErrOverflow;
    }

    if(iPacketArray.Count() >=KMaxStreamBuffers)
    {
        return KErrOverflow;
    }

	CRTPPacket* packet = new CRTPPacket(); // Note : Non-leaving construction. 
	if(!packet) {
        XPSSTREAMTRACE( _L("CXPSMediaStream.AddPacket> --, KErrNoMemory") );
		return KErrNoMemory;
	}
	
	packet->iRTPRecvHdr = InsertRtpHeader(aPktData.iTimeStamp);
	
	TUint8* payload = NULL;
	
	// Embedded Video Vp6 stream will not have padding byte 
	// So insert the padding byte. 
	
	if(aPktData.iEmbeddedVideo && iCodecType == EOn2Codec)
	{
		payload = new TUint8[aPktData.iDataLength + 1];
		if(!payload) {
			delete packet;			
            XPSSTREAMTRACE( _L("CXPSMediaStream.AddPacket> --, KErrNoMemory") );
			return KErrNoMemory;
		}
		payload[0] = ((TUint8) aPktData.iPadSize.iWidth << 4) | ((TUint8) aPktData.iPadSize.iHeight); 		
		Mem::Copy(payload + 1, aPktData.iFrameData, aPktData.iDataLength);
	}
	else
	{
		payload = new TUint8[aPktData.iDataLength];
		if(!payload) {
			delete packet;
            XPSSTREAMTRACE( _L("CXPSMediaStream.AddPacket> --, KErrNoMemory") );
			return KErrNoMemory;
		}		
		Mem::Copy(payload, aPktData.iFrameData, aPktData.iDataLength);
	}
	
	packet->iRTPPayLoadData = payload; 
	packet->iPayloadDataLength = aPktData.iDataLength;
	packet->iTSAdjusted = EFalse;	
	
	
	TInt error = iPacketArray.Append(packet);
	
	if(!IsOK(error)) {		
		delete packet;
        XPSSTREAMTRACE( _L("CXPSMediaStream.AddPacket> --, KErrNoMemory") );
		return error;
	}

	if(iXpsState == EXpsReady && 
	   iStreamState == EStreamStarted)
	{
		PushPacket();	
	}
	
    XPSSTREAMTRACE( _L("CXPSMediaStream.AddPacket> --") );
	return KErrNone;
}


// -------------------------------------------------------------------------
// void CXPSMediaStream::RestorePacketSupply()
// 
// Restore the packet supply to XPS. 
// If the stream supply is paused / stopped due to overflow/underflow, 
// then XPS can re-initiate the supply when it becomes ready. 
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::RestorePacketSupply()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.RestorePacketSupply< ++") );

    if(iXpsState == EXpsOverflow) {
       iXpsState = EXpsReady;
    }

    if(iStreamState == EStreamStarted && 
          iXpsState == EXpsReady) {
    	PushPacket();
    }

    XPSSTREAMTRACE( _L("CXPSMediaStream.RestorePacketSupply> --") );
}


// -------------------------------------------------------------------------
// void CXPSMediaStream::StreamEnd()
// 
// Marks the ending of the stream
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::StreamEnd()
{
    XPSSTREAMTRACE( _L("Stream.StreamEnd< ++") );

    if(iStreamState == EStreamStarted || 
       iStreamState == EStreamPaused)
    {
        XPSSTREAMTRACEF("CXPSMediaStream::StreamEnd() : Stream %d Ended", iStreamId);

	    iStreamEndSignalled = ETrue;
	    TryXpsStreamEnd(); 

        //If packet queue is empty already, stream state moves to 'stopped'.
    }
    
    XPSSTREAMTRACE( _L("Stream.StreamEnd> --") );
}


// -------------------------------------------------------------------------
// TBool CXPSMediaStream::IsStreamEnded()
// 
// Specifies whether the stream has ended or not. 
// --------------------------------------------------------------------------
// 

TBool CXPSMediaStream::IsStreamEnded()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.IsStreamEnded< ++") );
    XPSSTREAMTRACE( _L("CXPSMediaStream.IsStreamEnded> -- [%d]"),(iXpsState == EXpsStreamEnd));
    return (iXpsState == EXpsStreamEnd);
}


// -------------------------------------------------------------------------
// TBool CXPSMediaStream::Restart()
// 
// Special API with main use case being to restart playback.
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::Restart()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream::Restart()< ++") );

	//Get ready for new stream. 
	//Clearing xps sink should be taken care by rtp source

	Reset();//clear existing packets
	iStreamEndSignalled = EFalse;

	iStreamState = EStreamStarted;
	iXpsState = EXpsReady;

    XPSSTREAMTRACE( _L("CXPSMediaStream::Restart()> --") );
}


// -------------------------------------------------------------------------
// TInt CXPSMediaStream::StreamId() const
// 
// returns the stream Id.  
// --------------------------------------------------------------------------
// 

TInt CXPSMediaStream::StreamId() const
{
    XPSSTREAMTRACE( _L("Stream.StreamId< ++") );
    XPSSTREAMTRACE( _L("Stream.StreamId> --") );
    return iStreamId;
}


// -------------------------------------------------------------------------
// void CXPSMediaStream::SetStreamType(const TXPSStreamType& aStreamType)
// 
// Sets the type of the stream. 
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::SetStreamType(const TXPSStreamType& aStreamType)
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.SetStreamType< ++") );
    iStreamType = aStreamType;
    XPSSTREAMTRACE( _L("CXPSMediaStream.SetStreamType> --") );
}


// -------------------------------------------------------------------------
// TXPSStreamType CXPSMediaStream::StreamType() const
// 
// Returns the type of the stream. 
// --------------------------------------------------------------------------
// 

TXPSStreamType CXPSMediaStream::StreamType() const
{
    XPSSTREAMTRACE( _L("Stream.StreamType< ++") );
    XPSSTREAMTRACE( _L("Stream.StreamType> --") );
    return iStreamType;
}


// -------------------------------------------------------------------------
// void CXPSMediaStream::SetCodecType(const TCodecType& aCodecType)
// 
// Sets the codec type associated with the stream. 
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::SetCodecType(const TCodecType& aCodecType)
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.SetCodecType< ++") );
    iCodecType = aCodecType;
    XPSSTREAMTRACE( _L("CXPSMediaStream.SetCodecType> --") );
}


// -------------------------------------------------------------------------
// TXPSStreamType CXPSMediaStream::StreamType() const
// 
// Returns the type of the stream. 
// --------------------------------------------------------------------------
// 

TCodecType CXPSMediaStream::CodecType() const
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.CodecType< ++") );
    XPSSTREAMTRACE( _L("CXPSMediaStream.CodecType> --") );
    return iCodecType;
}


// -------------------------------------------------------------------------
// TInt CXPSMediaStream::SetStreamConfig(const TStreamConfigData& aAvcCfgInfo)
// 
// Sets config data of the stream.
// --------------------------------------------------------------------------
// 

TInt CXPSMediaStream::SetStreamConfig(const TStreamConfigData& aStreamConfigData)
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.SetStreamConfig< ++") );
	TUint8* configData =  NULL;
	configData = new TUint8[aStreamConfigData.iConfigLength];
	if(configData == NULL) {
        XPSSTREAMTRACE( _L("CXPSMediaStream.SetStreamConfig> --, KErrNoMemory") );
		return KErrNoMemory;
	}
	
	memcpy(configData, aStreamConfigData.iConfigData, aStreamConfigData.iConfigLength);
	
	iStreamConfigData.iConfigData = configData;
	iStreamConfigData.iConfigLength = aStreamConfigData.iConfigLength;
	
    XPSSTREAMTRACE( _L("CXPSMediaStream.SetStreamConfig> --") );
	return KErrNone;
}


// -------------------------------------------------------------------------
// TUint8* CXPSMediaStream::GetStreamConfig()
// 
// Gets config data of the stream.
// --------------------------------------------------------------------------
// 

TUint8* CXPSMediaStream::GetStreamConfig()
{	
    XPSSTREAMTRACE( _L("CXPSMediaStream.GetStreamConfig< ++") );
    XPSSTREAMTRACE( _L("CXPSMediaStream.GetStreamConfig> --") );
	return (iStreamConfigData.iConfigData);
}


// -------------------------------------------------------------------------
// void CXPSMediaStream::SetAudioInfo(const TAudioInfo& aAudioInfo)
// 
// Sets the audio info of the audio stream. 
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::SetAudioInfo(const TAudioInfo& aAudioInfo)
{    
    XPSSTREAMTRACE( _L("CXPSMediaStream.SetAudioInfo< ++") );
    iAudioInfo.iSampleRate      = aAudioInfo.iSampleRate;
    iAudioInfo.iSampleLength    = aAudioInfo.iSampleLength;
    iAudioInfo.iChannels        = aAudioInfo.iChannels;
    XPSSTREAMTRACE( _L("CXPSMediaStream.SetAudioInfo> --") );
}


// -------------------------------------------------------------------------
// void CXPSMediaStream::GetAudioInfo(TAudioInfo& aAudioInfo)
// 
// Retrieves the audio info if the pertinent stream is audio stream.  
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::GetAudioInfo(TAudioInfo& aAudioInfo)
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.GetAudioInfo< ++") );
	aAudioInfo.iSampleRate 		= iAudioInfo.iSampleRate;
	aAudioInfo.iSampleLength 	= iAudioInfo.iSampleLength;
	aAudioInfo.iChannels		= iAudioInfo.iChannels;
    XPSSTREAMTRACE( _L("CXPSMediaStream.GetAudioInfo> --") );
}


void CXPSMediaStream::SetAvcStreamInfo(const TAvcStreamInfo& aAvcInfo)
{    
    XPSSTREAMTRACE( _L("CXPSMediaStream.SetAvcStreamInfo< ++") );
    iAvcStreamInfo.iProfileInfo      = aAvcInfo.iProfileInfo;
    iAvcStreamInfo.iConstraintInfo   = aAvcInfo.iConstraintInfo;
    iAvcStreamInfo.iLevelInfo        = aAvcInfo.iLevelInfo;
    XPSSTREAMTRACE( _L("CXPSMediaStream.SetAvcStreamInfo> --") );
}


void CXPSMediaStream::GetAvcStreamInfo(TAvcStreamInfo& aVideoInfo)
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.GetAvcStreamInfo< ++") );
	aVideoInfo.iConstraintInfo 	= iAvcStreamInfo.iConstraintInfo;
	aVideoInfo.iLevelInfo		= iAvcStreamInfo.iLevelInfo;
	aVideoInfo.iProfileInfo		= iAvcStreamInfo.iProfileInfo;
    XPSSTREAMTRACE( _L("CXPSMediaStream.GetAvcStreamInfo> --") );
}


// -------------------------------------------------------------------------
// TUint CXPSMediaStream::GetFirstPacketTimeStamp()
// 
// Returns the timestamp of the first packet in the queue. (if not empty).
// Returns zero if the queue is empty. 
// --------------------------------------------------------------------------
// 

TUint CXPSMediaStream::GetFirstPacketTimeStamp()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.GetFirstPacketTimeStamp< ++") );
	TUint time = 0;
	if(iPacketArray.Count() > 0)
	{
		time = iPacketArray[0]->iRTPRecvHdr.iTimestamp;
	}
	
    XPSSTREAMTRACE( _L("CXPSMediaStream.GetFirstPacketTimeStamp> --") );
	return time;
}


// -------------------------------------------------------------------------
// TUint CXPSMediaStream::CurrentTimestamp()
// 
// Gets the latest timestamp of the pkt in the stream
// --------------------------------------------------------------------------
// 

TUint CXPSMediaStream::CurrentTimestamp()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.CurrentTimestamp< ++") );
    XPSSTREAMTRACE( _L("CXPSMediaStream.CurrentTimestamp> --") );
    return iTimestamp;
}

// -------------------------------------------------------------------------
// TInt CXPSMediaStream::PacketCount() const
// 
// Gets the count of the packets present in the stream pkt list. 
// --------------------------------------------------------------------------
// 

TInt CXPSMediaStream::PacketCount() const
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.PacketCount< ++") );
    XPSSTREAMTRACE( _L("CXPSMediaStream.PacketCount> --") );
    return iPacketArray.Count();
}


// -------------------------------------------------------------------------
// void CXPSMediaStream::PushPacket()
// 
// Pushes the packet from the packet list into XPS packet queue.  
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::PushPacket()
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.PushPacket< ++") );

    if(iXpsState != EXpsReady || iStreamState != EStreamStarted) {
        XPSSTREAMTRACE( _L("CXPSMediaStream.PushPacket> --, Xps not ready") );
        return;
    }
	if(iPacketArray.Count() == 0) {
        XPSSTREAMTRACE( _L("CXPSMediaStream.PushPacket> --, no media packets") );
		return;		
	}

	do 
	{		
		CRTPPacket* packet = iPacketArray[0];
		
		iTimestamp = packet->iRTPRecvHdr.iTimestamp;
		
		#ifdef __PACKET_LOGGING 
		if(ISVIDEOSTREAM())
		{
			LOG1(CXPSMediaStream::PushPacket() : stream = [Video] TS = [%d], iTimestamp);
		}
		else if(ISAUDIOSTREAM())
		{
			LOG1(CXPSMediaStream::PushPacket() : stream = [Audio] TS = [%d], iTimestamp);
		}
		#endif
		
		if(!packet->iTSAdjusted) {
			AdjustTimestamp(packet);
		}		
		
		TPtr8 payload(packet->iRTPPayLoadData, packet->iPayloadDataLength, packet->iPayloadDataLength);

        #ifdef _USE_SINK		
		TInt error = iSink.Enqueue(iStreamId, packet->iRTPRecvHdr, payload);
        #else
		TInt error = KErrNone;
        #endif
		
		#ifdef __DUMP_TO_FILE
		DumpToFile(iStreamId, packet->iRTPPayLoadData, packet->iPayloadDataLength);
		DumpHeaderToFile(iStreamId, packet->iPayloadDataLength, packet->iRTPRecvHdr);
		LogFps(iStreamId, packet->iRTPRecvHdr.iSeqNum, packet->iRTPRecvHdr.iTimestamp);
		#endif
		
		if(error == KErrOverflow)
		{
			LOG(CXPSMediaStream::PushPacket() : OVERFLOW);
			iPacketArray.Remove(0);
			delete packet;
			iXpsState = EXpsOverflow;
			break;
		}
		else if(IsOK(error)) {				
			iPacketArray.Remove(0);
			delete packet;
		}
		else {
			LOG1(CXPSMediaStream::PushPacket() : ERROR = [%d], error);
            XPSSTREAMTRACE( _L("CXPSMediaStream.PushPacket> --, 3") );
			return;
		}		
		
	} while(iPacketArray.Count() > 0);
	
    (iStreamEndSignalled) && (iPacketArray.Count() == 0) && TryXpsStreamEnd(); 

    XPSSTREAMTRACE( _L("CXPSMediaStream.PushPacket> --") );
}

// -------------------------------------------------------------------------
// TRtpRecvHeader CXPSMediaStream::InsertRtpHeader(const TUint& aTimeStamp)
// 
// Inserts the RTP header info into the data before adding the packet into the list. 
// This is done because XPS receives data in the RTP packet format. 
// --------------------------------------------------------------------------
// 

TRtpRecvHeader CXPSMediaStream::InsertRtpHeader(const TUint& aTimeStamp)
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.InsertRtpHeader< ++") );
	TRtpRecvHeader header;
	header.iTimestamp = 0;
	header.iMarker = 0;
	header.iSeqNum = ++iSeqNumber;
	header.iTimestamp = aTimeStamp;
	
	TUint timeDiff = (aTimeStamp - iPrevTS); 
	iTimeDiff = (timeDiff + iSumOfDiff) / iSeqNumber;
	iSumOfDiff += timeDiff;
	iPrevTS = aTimeStamp;
	
    XPSSTREAMTRACE( _L("CXPSMediaStream.InsertRtpHeader> --") );
	return header;
}


// -------------------------------------------------------------------------
// void CXPSMediaStream::AdjustTimestamp(CRTPPacket* aPacket)
// 
// Adjusts the timestamp when both audio and video streams are present. 
// This is required as the timescale for audio and video are different. 
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::AdjustTimestamp(CRTPPacket* aPacket)
{	
    XPSSTREAMTRACE( _L("CXPSMediaStream.AdjustTimestamp< ++") );
	// *********************************************************************
	// NOTE : Timestamp is not so imp if the clip has only Video / Audio. 
	// 		  But it is important when both A-V are present. In such cases, 
	// 		  Use the following formula : 
	// 
	// 		  VT = t(i) * AUDIO_Sampling_Rate / 1000;   
	// ********************************************************************* 
	
	if(ISAUDIOSTREAM())
	{
		TInt samplingRate = -1; 
		TInt error = iStreamHandler.AudioSamplingRate(samplingRate);
		if(IsOK(error))
		{
			aPacket->iRTPRecvHdr.iTimestamp *= (TReal) samplingRate / KMillisecondsForASec;
			aPacket->iTSAdjusted = ETrue;
		}
		else
		{
			LOG1(CXPSMediaStream::AdjustTimestamp() : Error = [%d], error);
		}
	}
	else if(ISVIDEOSTREAM())
	{
		if(iStreamHandler.IsAudioStreamPresent())
		{
			TInt samplingRate = -1; 
			TInt error = iStreamHandler.AudioSamplingRate(samplingRate);
			if(IsOK(error))
			{
				aPacket->iRTPRecvHdr.iTimestamp *= (TReal) samplingRate / KMillisecondsForASec;
				aPacket->iTSAdjusted = ETrue;
			}
			else
			{
				LOG1(CXPSMediaStream::AdjustTimestamp() : Error = [%d], error);
			}
		}		
	}
    XPSSTREAMTRACE( _L("CXPSMediaStream.AdjustTimestamp> --") );
}


// -------------------------------------------------------------------------
// void CXPSMediaStream::LogSummary()
// 
// Print the summary of packet supply of the stream to XPS. 
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::LogSummary() 
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.LogSummary< ++") );
    if(ISVIDEOSTREAM())
    {
        XPSSTREAMTRACE(_L("Video Stream ID = [%d] : Total Pkts Sent to XPS = [%d]"), iStreamId, iSeqNumber);   
        TReal encodedFrameRate =  ((TReal) iSeqNumber * KMillisecondsForASec) / iStreamHandler.TotalDuration(); 
        XPSSTREAMTRACE(_L("Video Stream ID = [%d] : Encoded Fps = [%f]"), iStreamId, encodedFrameRate);   
    }
    XPSSTREAMTRACE( _L("CXPSMediaStream.LogSummary> --") );
}


// -------------------------------------------------------------------------
// TBool CXPSMediaStream::TryXpsStreamEnd() 
// 
//  
// --------------------------------------------------------------------------
// 

TBool CXPSMediaStream::TryXpsStreamEnd() 
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.TryXpsStreamEnd< ++") );
    TBool xpsStreamEnd = EFalse;
    
    if(iStreamEndSignalled && iPacketArray.Count() == 0)
    {
        iXpsState = EXpsStreamEnd;
        iStreamState = EStreamStopped;

        XPSSTREAMTRACEF("CXPSMediaStream::TryXpsStreamEnd() - StreamEnd");
        
        #ifdef _USE_SINK        
        iSink.StreamEnd(iStreamId);
        #endif
        
        xpsStreamEnd = ETrue;
        
        LogSummary();        
    }

    XPSSTREAMTRACE( _L("CXPSMediaStream.TryXpsStreamEnd> --") );
    return xpsStreamEnd;
}




#ifdef __DUMP_TO_FILE

// -------------------------------------------------------------------------
// void CXPSMediaStream::DumpToFile(TInt aStreamId,
//                               const TUint8* aData, 
//                               TUint aDataLength)
// 
// Dumps the payload of RTP packets into streamXX.pyld file, where XX = stream Id. 
// This is used while debugging.
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::DumpToFile(TInt aStreamId,
                               const TUint8* aData, 
                               TUint aDataLength)
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.DumpToFile< ++") );
    if(!aData || !aDataLength)
    XPSSTREAMTRACE( _L("CXPSMediaStream.DumpToFile> --") );
        return;
    
    TFileName fileName;
    fileName.Copy(_L("C:\\Logs\\HelixLogs\\"));
    fileName.AppendFormat(_L("stream%d.pyld"), aStreamId);
    
    TPtr8 buf((TUint8*)aData, aDataLength, aDataLength);
    
    TInt error = KErrNone;
    RFs fs;
    if(fs.Connect() != KErrNone)
    {
    	return;
    }
    
    RFile dumpFile;
    error = dumpFile.Open(fs, fileName, EFileWrite);
    if(error == KErrNotFound)
        {
        error = dumpFile.Create(fs, fileName, EFileWrite);
        }
    if(IsOK(error))
        {
        TInt pos = 0;
        dumpFile.Seek(ESeekEnd, pos);
        dumpFile.Write(buf);
        }
    dumpFile.Close();
    fs.Close();
}

// -------------------------------------------------------------------------
// void CXPSMediaStream::DumpHeaderToFile(TInt aStreamId, TUint aDataLength, TRtpRecvHeader aHeader)
//
// Dumps the RTP headers into streamXX.hdr file where XX = stream Id. 
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::DumpHeaderToFile(TInt aStreamId, TUint aDataLength, TRtpRecvHeader aHeader)
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.DumpHeaderToFile< ++") );
    TFileName fileName;
    fileName.Copy(_L("C:\\Logs\\HelixLogs\\"));
    fileName.AppendFormat(_L("stream%d.hdr"), aStreamId);
    
    // StreamNo= 0 Seq= 29037 sz= 875 ts= 0 mkr= 1
    
    TBuf8<100> buf;
    buf.AppendFormat(_L8("StreamNo= %d Seq= %d sz= %d ts= %d mkr= %d\n"), aStreamId, aHeader.iSeqNum,
    									aDataLength, aHeader.iTimestamp, aHeader.iMarker);
    
    TInt error = KErrNone;
    RFs fs;
    if(fs.Connect() != KErrNone)
    {
        XPSSTREAMTRACE( _L("CXPSMediaStream.DumpHeaderToFile> --") );
    	return;
    }
    
    RFile dumpFile;
    error = dumpFile.Open(fs, fileName, EFileWrite);
    if(error == KErrNotFound)
        {
        error = dumpFile.Create(fs, fileName, EFileWrite);
        }
    if(IsOK(error))
        {
        TInt pos = 0;
        dumpFile.Seek(ESeekEnd, pos);
        dumpFile.Write(buf);
        }
    dumpFile.Close();
    fs.Close();
    XPSSTREAMTRACE( _L("CXPSMediaStream.DumpHeaderToFile> --") );
}

// -------------------------------------------------------------------------
// void CXPSMediaStream::LogFps(TInt aStreamId, TUint aFrameNum, TUint aTotalTime)
// 
// Logs the time and fps info of RTP packets into a streamXX.fps file where XX = stream Id. 
// --------------------------------------------------------------------------
// 

void CXPSMediaStream::LogFps(TInt aStreamId, TUint aFrameNum, TUint aTotalTime)
{
    XPSSTREAMTRACE( _L("CXPSMediaStream.LogFps< ++") );
    TFileName fileName;
    fileName.Copy(_L("C:\\Logs\\HelixLogs\\"));
    fileName.AppendFormat(_L("stream%d.fps"), aStreamId);

    TReal fps = (TReal) KMillisecondsForASec * aFrameNum / aTotalTime;
    
    TBuf8<100> buf;
    buf.AppendFormat(_L8("StreamNo= %d Seq= %d time= %d fps= %f\n"), aStreamId, aFrameNum, aTotalTime, fps);
    
    TInt error = KErrNone;
    RFs fs;
    if(fs.Connect() != KErrNone)
    {
        XPSSTREAMTRACE( _L("CXPSMediaStream.LogFps> --") );
    	return;
    }
    
    RFile dumpFile;
    error = dumpFile.Open(fs, fileName, EFileWrite);
    if(IsOK(error))
        {
        error = dumpFile.Create(fs, fileName, EFileWrite);
        }
    if(error == KErrNone)
        {
        TInt pos = 0;
        dumpFile.Seek(ESeekEnd, pos);
        dumpFile.Write(buf);
        }
    dumpFile.Close();
    fs.Close();
    XPSSTREAMTRACE( _L("CXPSMediaStream.LogFps> --") );
}

#endif


